import math
from pysndlib import *
import numpy as np

#==================================================================================
# The code is an attempt at translation of Bill Schottstedaet's 'analog-filter.scm' 
# file available at https://ccrma.stanford.edu/software/snd/sndlib/
#==================================================================================


# def analog2digital(n, num, den, fz):
#     g = 1.0
#     Q = 1.0
#     wc = math.tan(math.pi * fz)
#     c = np.zeros(2 * n, dtype=np.double)
    
    



