/* Generated by Cython 3.0.8 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "define_macros": [
            [
                "NPY_NO_DEPRECATED_API",
                "NPY_1_7_API_VERSION"
            ]
        ],
        "extra_compile_args": [
            "-Wno-parentheses-equality",
            "-Wno-unreachable-code-fallthrough"
        ],
        "extra_objects": [
            "./sndlib/libsndlib.a"
        ],
        "include_dirs": [
            "/private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/core/include",
            "./sndlib"
        ],
        "name": "pysndlib.instruments.lbj_piano",
        "sources": [
            "src/pysndlib/instruments/lbj_piano.py"
        ]
    },
    "module_name": "pysndlib.instruments.lbj_piano"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
#if defined(CYTHON_LIMITED_API) && 0
  #ifndef Py_LIMITED_API
    #if CYTHON_LIMITED_API+0 > 0x03030000
      #define Py_LIMITED_API CYTHON_LIMITED_API
    #else
      #define Py_LIMITED_API 0x03030000
    #endif
  #endif
#endif

#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02070000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.7+ or Python 3.3+.
#else
#if defined(CYTHON_LIMITED_API) && CYTHON_LIMITED_API
#define __PYX_EXTRA_ABI_MODULE_NAME "limited"
#else
#define __PYX_EXTRA_ABI_MODULE_NAME ""
#endif
#define CYTHON_ABI "3_0_8" __PYX_EXTRA_ABI_MODULE_NAME
#define __PYX_ABI_MODULE_NAME "_cython_" CYTHON_ABI
#define __PYX_TYPE_MODULE_PREFIX __PYX_ABI_MODULE_NAME "."
#define CYTHON_HEX_VERSION 0x030008F0
#define CYTHON_FUTURE_DIVISION 1
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(_WIN32) && !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #define HAVE_LONG_LONG
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#define __PYX_LIMITED_VERSION_HEX PY_VERSION_HEX
#if defined(GRAALVM_PYTHON)
  /* For very preliminary testing purposes. Most variables are set the same as PyPy.
     The existence of this section does not imply that anything works or is even tested */
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 1
  #define CYTHON_COMPILING_IN_NOGIL 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS (PY_MAJOR_VERSION >= 3)
  #endif
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
#elif defined(PYPY_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS (PY_MAJOR_VERSION >= 3)
  #endif
  #if PY_VERSION_HEX < 0x03090000
    #undef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #elif !defined(CYTHON_PEP489_MULTI_PHASE_INIT)
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1 && PYPY_VERSION_NUM >= 0x07030C00)
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
#elif defined(CYTHON_LIMITED_API)
  #ifdef Py_LIMITED_API
    #undef __PYX_LIMITED_VERSION_HEX
    #define __PYX_LIMITED_VERSION_HEX Py_LIMITED_API
  #endif
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 1
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #undef CYTHON_CLINE_IN_TRACEBACK
  #define CYTHON_CLINE_IN_TRACEBACK 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 1
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #endif
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 1
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 0
  #endif
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
#elif defined(Py_GIL_DISABLED) || defined(Py_NOGIL)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_NOGIL 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #ifndef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 1
  #endif
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #ifndef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #ifndef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0 || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_GIL
    #define CYTHON_FAST_GIL (PY_MAJOR_VERSION < 3 || PY_VERSION_HEX >= 0x03060000 && PY_VERSION_HEX < 0x030C00A6)
  #endif
  #ifndef CYTHON_METH_FASTCALL
    #define CYTHON_METH_FASTCALL (PY_VERSION_HEX >= 0x030700A1)
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #elif !defined(CYTHON_PEP489_MULTI_PHASE_INIT)
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_MODULE_STATE
    #define CYTHON_USE_MODULE_STATE 0
  #endif
  #if PY_VERSION_HEX < 0x030400a1
    #undef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 0
  #elif !defined(CYTHON_USE_TP_FINALIZE)
    #define CYTHON_USE_TP_FINALIZE 1
  #endif
  #if PY_VERSION_HEX < 0x030600B1
    #undef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS 0
  #elif !defined(CYTHON_USE_DICT_VERSIONS)
    #define CYTHON_USE_DICT_VERSIONS  (PY_VERSION_HEX < 0x030C00A5)
  #endif
  #if PY_VERSION_HEX < 0x030700A3
    #undef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK 0
  #elif !defined(CYTHON_USE_EXC_INFO_STACK)
    #define CYTHON_USE_EXC_INFO_STACK 1
  #endif
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 1
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if !defined(CYTHON_VECTORCALL)
#define CYTHON_VECTORCALL  (CYTHON_FAST_PYCCALL && PY_VERSION_HEX >= 0x030800B1)
#endif
#define CYTHON_BACKPORT_VECTORCALL (CYTHON_METH_FASTCALL && PY_VERSION_HEX < 0x030800B1)
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_MAJOR_VERSION < 3
    #include "longintrepr.h"
  #endif
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(maybe_unused) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(maybe_unused)
        #define CYTHON_UNUSED [[maybe_unused]]
      #endif
    #endif
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
  #define CYTHON_MAYBE_UNUSED_VAR(x) CYTHON_UNUSED_VAR(x)
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_USE_CPP_STD_MOVE
  #if defined(__cplusplus) && (\
    __cplusplus >= 201103L || (defined(_MSC_VER) && _MSC_VER >= 1600))
    #define CYTHON_USE_CPP_STD_MOVE 1
  #else
    #define CYTHON_USE_CPP_STD_MOVE 0
  #endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
            typedef unsigned char     uint8_t;
            typedef unsigned short    uint16_t;
            typedef unsigned int      uint32_t;
        #else
            typedef unsigned __int8   uint8_t;
            typedef unsigned __int16  uint16_t;
            typedef unsigned __int32  uint32_t;
        #endif
    #endif
    #if _MSC_VER < 1300
        #ifdef _WIN64
            typedef unsigned long long  __pyx_uintptr_t;
        #else
            typedef unsigned int        __pyx_uintptr_t;
        #endif
    #else
        #ifdef _WIN64
            typedef unsigned __int64    __pyx_uintptr_t;
        #else
            typedef unsigned __int32    __pyx_uintptr_t;
        #endif
    #endif
#else
    #include <stdint.h>
    typedef uintptr_t  __pyx_uintptr_t;
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(fallthrough) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(fallthrough)
        #define CYTHON_FALLTHROUGH [[fallthrough]]
      #endif
    #endif
    #ifndef CYTHON_FALLTHROUGH
      #if __has_cpp_attribute(clang::fallthrough)
        #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
      #elif __has_cpp_attribute(gnu::fallthrough)
        #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
      #endif
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif
#ifdef __cplusplus
  template <typename T>
  struct __PYX_IS_UNSIGNED_IMPL {static const bool value = T(0) < T(-1);};
  #define __PYX_IS_UNSIGNED(type) (__PYX_IS_UNSIGNED_IMPL<type>::value)
#else
  #define __PYX_IS_UNSIGNED(type) (((type)-1) > 0)
#endif
#if CYTHON_COMPILING_IN_PYPY == 1
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x030A0000)
#else
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000)
#endif
#define __PYX_REINTERPRET_FUNCION(func_pointer, other_pointer) ((func_pointer)(void(*)(void))(other_pointer))

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_DefaultClassType PyClass_Type
  #define __Pyx_PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_DefaultClassType PyType_Type
#if CYTHON_COMPILING_IN_LIMITED_API
    static CYTHON_INLINE PyObject* __Pyx_PyCode_New(int a, int p, int k, int l, int s, int f,
                                                    PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                                    PyObject *fv, PyObject *cell, PyObject* fn,
                                                    PyObject *name, int fline, PyObject *lnos) {
        PyObject *exception_table = NULL;
        PyObject *types_module=NULL, *code_type=NULL, *result=NULL;
        #if __PYX_LIMITED_VERSION_HEX < 0x030B0000
        PyObject *version_info;
        PyObject *py_minor_version = NULL;
        #endif
        long minor_version = 0;
        PyObject *type, *value, *traceback;
        PyErr_Fetch(&type, &value, &traceback);
        #if __PYX_LIMITED_VERSION_HEX >= 0x030B0000
        minor_version = 11;
        #else
        if (!(version_info = PySys_GetObject("version_info"))) goto end;
        if (!(py_minor_version = PySequence_GetItem(version_info, 1))) goto end;
        minor_version = PyLong_AsLong(py_minor_version);
        Py_DECREF(py_minor_version);
        if (minor_version == -1 && PyErr_Occurred()) goto end;
        #endif
        if (!(types_module = PyImport_ImportModule("types"))) goto end;
        if (!(code_type = PyObject_GetAttrString(types_module, "CodeType"))) goto end;
        if (minor_version <= 7) {
            (void)p;
            result = PyObject_CallFunction(code_type, "iiiiiOOOOOOiOO", a, k, l, s, f, code,
                          c, n, v, fn, name, fline, lnos, fv, cell);
        } else if (minor_version <= 10) {
            result = PyObject_CallFunction(code_type, "iiiiiiOOOOOOiOO", a,p, k, l, s, f, code,
                          c, n, v, fn, name, fline, lnos, fv, cell);
        } else {
            if (!(exception_table = PyBytes_FromStringAndSize(NULL, 0))) goto end;
            result = PyObject_CallFunction(code_type, "iiiiiiOOOOOOOiOO", a,p, k, l, s, f, code,
                          c, n, v, fn, name, name, fline, lnos, exception_table, fv, cell);
        }
    end:
        Py_XDECREF(code_type);
        Py_XDECREF(exception_table);
        Py_XDECREF(types_module);
        if (type) {
            PyErr_Restore(type, value, traceback);
        }
        return result;
    }
    #ifndef CO_OPTIMIZED
    #define CO_OPTIMIZED 0x0001
    #endif
    #ifndef CO_NEWLOCALS
    #define CO_NEWLOCALS 0x0002
    #endif
    #ifndef CO_VARARGS
    #define CO_VARARGS 0x0004
    #endif
    #ifndef CO_VARKEYWORDS
    #define CO_VARKEYWORDS 0x0008
    #endif
    #ifndef CO_ASYNC_GENERATOR
    #define CO_ASYNC_GENERATOR 0x0200
    #endif
    #ifndef CO_GENERATOR
    #define CO_GENERATOR 0x0020
    #endif
    #ifndef CO_COROUTINE
    #define CO_COROUTINE 0x0080
    #endif
#elif PY_VERSION_HEX >= 0x030B0000
  static CYTHON_INLINE PyCodeObject* __Pyx_PyCode_New(int a, int p, int k, int l, int s, int f,
                                                    PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                                    PyObject *fv, PyObject *cell, PyObject* fn,
                                                    PyObject *name, int fline, PyObject *lnos) {
    PyCodeObject *result;
    PyObject *empty_bytes = PyBytes_FromStringAndSize("", 0);
    if (!empty_bytes) return NULL;
    result =
      #if PY_VERSION_HEX >= 0x030C0000
        PyUnstable_Code_NewWithPosOnlyArgs
      #else
        PyCode_NewWithPosOnlyArgs
      #endif
        (a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, name, fline, lnos, empty_bytes);
    Py_DECREF(empty_bytes);
    return result;
  }
#elif PY_VERSION_HEX >= 0x030800B2 && !CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_NewWithPosOnlyArgs(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
#endif
#if PY_VERSION_HEX >= 0x030900A4 || defined(Py_IS_TYPE)
  #define __Pyx_IS_TYPE(ob, type) Py_IS_TYPE(ob, type)
#else
  #define __Pyx_IS_TYPE(ob, type) (((const PyObject*)ob)->ob_type == (type))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_Is)
  #define __Pyx_Py_Is(x, y)  Py_Is(x, y)
#else
  #define __Pyx_Py_Is(x, y) ((x) == (y))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsNone)
  #define __Pyx_Py_IsNone(ob) Py_IsNone(ob)
#else
  #define __Pyx_Py_IsNone(ob) __Pyx_Py_Is((ob), Py_None)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsTrue)
  #define __Pyx_Py_IsTrue(ob) Py_IsTrue(ob)
#else
  #define __Pyx_Py_IsTrue(ob) __Pyx_Py_Is((ob), Py_True)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsFalse)
  #define __Pyx_Py_IsFalse(ob) Py_IsFalse(ob)
#else
  #define __Pyx_Py_IsFalse(ob) __Pyx_Py_Is((ob), Py_False)
#endif
#define __Pyx_NoneAsNull(obj)  (__Pyx_Py_IsNone(obj) ? NULL : (obj))
#if PY_VERSION_HEX >= 0x030900F0 && !CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyObject_GC_IsFinalized(o) PyObject_GC_IsFinalized(o)
#else
  #define __Pyx_PyObject_GC_IsFinalized(o) _PyGC_FINALIZED(o)
#endif
#ifndef CO_COROUTINE
  #define CO_COROUTINE 0x80
#endif
#ifndef CO_ASYNC_GENERATOR
  #define CO_ASYNC_GENERATOR 0x200
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef Py_TPFLAGS_SEQUENCE
  #define Py_TPFLAGS_SEQUENCE 0
#endif
#ifndef Py_TPFLAGS_MAPPING
  #define Py_TPFLAGS_MAPPING 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_METH_FASTCALL
  #define __Pyx_METH_FASTCALL METH_FASTCALL
  #define __Pyx_PyCFunction_FastCall __Pyx_PyCFunctionFast
  #define __Pyx_PyCFunction_FastCallWithKeywords __Pyx_PyCFunctionFastWithKeywords
#else
  #define __Pyx_METH_FASTCALL METH_VARARGS
  #define __Pyx_PyCFunction_FastCall PyCFunction
  #define __Pyx_PyCFunction_FastCallWithKeywords PyCFunctionWithKeywords
#endif
#if CYTHON_VECTORCALL
  #define __pyx_vectorcallfunc vectorcallfunc
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  PY_VECTORCALL_ARGUMENTS_OFFSET
  #define __Pyx_PyVectorcall_NARGS(n)  PyVectorcall_NARGS((size_t)(n))
#elif CYTHON_BACKPORT_VECTORCALL
  typedef PyObject *(*__pyx_vectorcallfunc)(PyObject *callable, PyObject *const *args,
                                            size_t nargsf, PyObject *kwnames);
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  ((size_t)1 << (8 * sizeof(size_t) - 1))
  #define __Pyx_PyVectorcall_NARGS(n)  ((Py_ssize_t)(((size_t)(n)) & ~__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET))
#else
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  0
  #define __Pyx_PyVectorcall_NARGS(n)  ((Py_ssize_t)(n))
#endif
#if PY_MAJOR_VERSION >= 0x030900B1
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_CheckExact(func)
#else
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_Check(func)
#endif
#define __Pyx_CyOrPyCFunction_Check(func)  PyCFunction_Check(func)
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  (((PyCFunctionObject*)(func))->m_ml->ml_meth)
#elif !CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  PyCFunction_GET_FUNCTION(func)
#endif
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FLAGS(func)  (((PyCFunctionObject*)(func))->m_ml->ml_flags)
static CYTHON_INLINE PyObject* __Pyx_CyOrPyCFunction_GET_SELF(PyObject *func) {
    return (__Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_STATIC) ? NULL : ((PyCFunctionObject*)func)->m_self;
}
#endif
static CYTHON_INLINE int __Pyx__IsSameCFunction(PyObject *func, void *cfunc) {
#if CYTHON_COMPILING_IN_LIMITED_API
    return PyCFunction_Check(func) && PyCFunction_GetFunction(func) == (PyCFunction) cfunc;
#else
    return PyCFunction_Check(func) && PyCFunction_GET_FUNCTION(func) == (PyCFunction) cfunc;
#endif
}
#define __Pyx_IsSameCFunction(func, cfunc)   __Pyx__IsSameCFunction(func, cfunc)
#if __PYX_LIMITED_VERSION_HEX < 0x030900B1
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  ((void)m, PyType_FromSpecWithBases(s, b))
  typedef PyObject *(*__Pyx_PyCMethod)(PyObject *, PyTypeObject *, PyObject *const *, size_t, PyObject *);
#else
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  PyType_FromModuleAndSpec(m, s, b)
  #define __Pyx_PyCMethod  PyCMethod
#endif
#ifndef METH_METHOD
  #define METH_METHOD 0x200
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyThreadState_Current PyThreadState_Get()
#elif !CYTHON_FAST_THREAD_STATE
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x030d00A1
  #define __Pyx_PyThreadState_Current PyThreadState_GetUnchecked()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE void *__Pyx_PyModule_GetState(PyObject *op)
{
    void *result;
    result = PyModule_GetState(op);
    if (!result)
        Py_FatalError("Couldn't find the module state");
    return result;
}
#endif
#define __Pyx_PyObject_GetSlot(obj, name, func_ctype)  __Pyx_PyType_GetSlot(Py_TYPE(obj), name, func_ctype)
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((func_ctype) PyType_GetSlot((type), Py_##name))
#else
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((type)->name)
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if PY_MAJOR_VERSION < 3
    #if CYTHON_COMPILING_IN_PYPY
        #if PYPY_VERSION_NUM < 0x07030600
            #if defined(__cplusplus) && __cplusplus >= 201402L
                [[deprecated("`with nogil:` inside a nogil function will not release the GIL in PyPy2 < 7.3.6")]]
            #elif defined(__GNUC__) || defined(__clang__)
                __attribute__ ((__deprecated__("`with nogil:` inside a nogil function will not release the GIL in PyPy2 < 7.3.6")))
            #elif defined(_MSC_VER)
                __declspec(deprecated("`with nogil:` inside a nogil function will not release the GIL in PyPy2 < 7.3.6"))
            #endif
            static CYTHON_INLINE int PyGILState_Check(void) {
                return 0;
            }
        #else  // PYPY_VERSION_NUM < 0x07030600
        #endif  // PYPY_VERSION_NUM < 0x07030600
    #else
        static CYTHON_INLINE int PyGILState_Check(void) {
            PyThreadState * tstate = _PyThreadState_Current;
            return tstate && (tstate == PyGILState_GetThisThreadState());
        }
    #endif
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030d0000 || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX > 0x030600B4 && PY_VERSION_HEX < 0x030d0000 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStrWithError(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStr(PyObject *dict, PyObject *name) {
    PyObject *res = __Pyx_PyDict_GetItemStrWithError(dict, name);
    if (res == NULL) PyErr_Clear();
    return res;
}
#elif PY_MAJOR_VERSION >= 3 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07020000)
#define __Pyx_PyDict_GetItemStrWithError  PyDict_GetItemWithError
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#else
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStrWithError(PyObject *dict, PyObject *name) {
#if CYTHON_COMPILING_IN_PYPY
    return PyDict_GetItem(dict, name);
#else
    PyDictEntry *ep;
    PyDictObject *mp = (PyDictObject*) dict;
    long hash = ((PyStringObject *) name)->ob_shash;
    assert(hash != -1);
    ep = (mp->ma_lookup)(mp, name, hash);
    if (ep == NULL) {
        return NULL;
    }
    return ep->me_value;
#endif
}
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#endif
#if CYTHON_USE_TYPE_SLOTS
  #define __Pyx_PyType_GetFlags(tp)   (((PyTypeObject *)tp)->tp_flags)
  #define __Pyx_PyType_HasFeature(type, feature)  ((__Pyx_PyType_GetFlags(type) & (feature)) != 0)
  #define __Pyx_PyObject_GetIterNextFunc(obj)  (Py_TYPE(obj)->tp_iternext)
#else
  #define __Pyx_PyType_GetFlags(tp)   (PyType_GetFlags((PyTypeObject *)tp))
  #define __Pyx_PyType_HasFeature(type, feature)  PyType_HasFeature(type, feature)
  #define __Pyx_PyObject_GetIterNextFunc(obj)  PyIter_Next
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_SetItemOnTypeDict(tp, k, v) PyObject_GenericSetAttr((PyObject*)tp, k, v)
#else
  #define __Pyx_SetItemOnTypeDict(tp, k, v) PyDict_SetItem(tp->tp_dict, k, v)
#endif
#if CYTHON_USE_TYPE_SPECS && PY_VERSION_HEX >= 0x03080000
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  {\
    PyTypeObject *type = Py_TYPE((PyObject*)obj);\
    assert(__Pyx_PyType_HasFeature(type, Py_TPFLAGS_HEAPTYPE));\
    PyObject_GC_Del(obj);\
    Py_DECREF(type);\
}
#else
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  PyObject_GC_Del(obj)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GetLength(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_ReadChar(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((void)u, 1114111U)
  #define __Pyx_PyUnicode_KIND(u)         ((void)u, (0))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)u)
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)k, PyUnicode_ReadChar((PyObject*)(d), i))
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GetLength(u))
#elif PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_READY(op)       (0)
  #else
    #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                                0 : _PyUnicode_Ready((PyObject *)(op)))
  #endif
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         ((int)PyUnicode_KIND(u))
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, (Py_UCS4) ch)
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #else
    #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
    #else
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
    #endif
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535U : 1114111U)
  #define __Pyx_PyUnicode_KIND(u)         ((int)sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = (Py_UNICODE) ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #if !defined(PyUnicode_DecodeUnicodeEscape)
    #define PyUnicode_DecodeUnicodeEscape(s, size, errors)  PyUnicode_Decode(s, size, "unicode_escape", errors)
  #endif
  #if !defined(PyUnicode_Contains) || (PY_MAJOR_VERSION == 2 && PYPY_VERSION_NUM < 0x07030500)
    #undef PyUnicode_Contains
    #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
  #endif
  #if !defined(PyByteArray_Check)
    #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
  #endif
  #if !defined(PyObject_Format)
    #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
  #endif
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#if CYTHON_COMPILING_IN_CPYTHON
  #define __Pyx_PySequence_ListKeepNew(obj)\
    (likely(PyList_CheckExact(obj) && Py_REFCNT(obj) == 1) ? __Pyx_NewRef(obj) : PySequence_List(obj))
#else
  #define __Pyx_PySequence_ListKeepNew(obj)  PySequence_List(obj)
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        __Pyx_IS_TYPE(obj, &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_ITEM(o, i) PySequence_ITEM(o, i)
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) (PyTuple_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyList_SET_ITEM(o, i, v) (PyList_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_GET_SIZE(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_GET_SIZE(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_GET_SIZE(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_GET_SIZE(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_GET_SIZE(o)
#else
  #define __Pyx_PySequence_ITEM(o, i) PySequence_GetItem(o, i)
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) PyTuple_SetItem(o, i, v)
  #define __Pyx_PyList_SET_ITEM(o, i, v) PyList_SetItem(o, i, v)
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_Size(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_Size(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_Size(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_Size(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_Size(o)
#endif
#if PY_VERSION_HEX >= 0x030d00A1
  #define __Pyx_PyImport_AddModuleRef(name) PyImport_AddModuleRef(name)
#else
  static CYTHON_INLINE PyObject *__Pyx_PyImport_AddModuleRef(const char *name) {
      PyObject *module = PyImport_AddModule(name);
      Py_XINCREF(module);
      return module;
  }
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define __Pyx_Py3Int_Check(op)       PyLong_Check(op)
  #define __Pyx_Py3Int_CheckExact(op)  PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#else
  #define __Pyx_Py3Int_Check(op)       (PyLong_Check(op) || PyInt_Check(op))
  #define __Pyx_Py3Int_CheckExact(op)  (PyLong_CheckExact(op) || PyInt_CheckExact(op))
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsHash_t
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsSsize_t
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)
  #if !defined(_USE_MATH_DEFINES)
    #define _USE_MATH_DEFINES
  #endif
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifdef CYTHON_EXTERN_C
    #undef __PYX_EXTERN_C
    #define __PYX_EXTERN_C CYTHON_EXTERN_C
#elif defined(__PYX_EXTERN_C)
    #ifdef _MSC_VER
    #pragma message ("Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.")
    #else
    #warning Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.
    #endif
#else
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__pysndlib__instruments__lbj_piano
#define __PYX_HAVE_API__pysndlib__instruments__lbj_piano
/* Early includes */
#include <string.h>
#include <stdio.h>

    /* Using NumPy API declarations from "numpy/__init__.cython-30.pxd" */
    
#include "numpy/arrayobject.h"
#include "numpy/ndarrayobject.h"
#include "numpy/ndarraytypes.h"
#include "numpy/arrayscalars.h"
#include "numpy/ufuncobject.h"
#include "sndlib.h"
#include "clm.h"

    typedef mus_float_t (*connect_points_cb)(mus_float_t val);
    typedef mus_float_t (*input_cb)(void *arg, int direction);
    typedef int (*edit_cb)(void *closure);
    typedef bool (*analyze_cb)(void *arg, mus_float_t (*input)(void *arg1, int direction));
	typedef mus_float_t (*synthesize_cb)(void *arg);
	typedef void (*detour_cb)(mus_any *ptr, mus_long_t val);

    typedef mus_any* mus_any_ptr;
    typedef mus_any** mus_any_ptr_ptr;
    
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s);
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char*);
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const wchar_t *u)
{
    const wchar_t *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#else
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u)
{
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#endif
#define __Pyx_PyUnicode_FromOrdinal(o)       PyUnicode_FromOrdinal((int)o)
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_VERSION_HEX >= 0x030C00A7
  #ifndef _PyLong_SIGN_MASK
    #define _PyLong_SIGN_MASK 3
  #endif
  #ifndef _PyLong_NON_SIZE_BITS
    #define _PyLong_NON_SIZE_BITS 3
  #endif
  #define __Pyx_PyLong_Sign(x)  (((PyLongObject*)x)->long_value.lv_tag & _PyLong_SIGN_MASK)
  #define __Pyx_PyLong_IsNeg(x)  ((__Pyx_PyLong_Sign(x) & 2) != 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (!__Pyx_PyLong_IsNeg(x))
  #define __Pyx_PyLong_IsZero(x)  (__Pyx_PyLong_Sign(x) & 1)
  #define __Pyx_PyLong_IsPos(x)  (__Pyx_PyLong_Sign(x) == 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  (__Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  ((Py_ssize_t) (((PyLongObject*)x)->long_value.lv_tag >> _PyLong_NON_SIZE_BITS))
  #define __Pyx_PyLong_SignedDigitCount(x)\
        ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * __Pyx_PyLong_DigitCount(x))
  #if defined(PyUnstable_Long_IsCompact) && defined(PyUnstable_Long_CompactValue)
    #define __Pyx_PyLong_IsCompact(x)     PyUnstable_Long_IsCompact((PyLongObject*) x)
    #define __Pyx_PyLong_CompactValue(x)  PyUnstable_Long_CompactValue((PyLongObject*) x)
  #else
    #define __Pyx_PyLong_IsCompact(x)     (((PyLongObject*)x)->long_value.lv_tag < (2 << _PyLong_NON_SIZE_BITS))
    #define __Pyx_PyLong_CompactValue(x)  ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * (Py_ssize_t) __Pyx_PyLong_Digits(x)[0])
  #endif
  typedef Py_ssize_t  __Pyx_compact_pylong;
  typedef size_t  __Pyx_compact_upylong;
  #else
  #define __Pyx_PyLong_IsNeg(x)  (Py_SIZE(x) < 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (Py_SIZE(x) >= 0)
  #define __Pyx_PyLong_IsZero(x)  (Py_SIZE(x) == 0)
  #define __Pyx_PyLong_IsPos(x)  (Py_SIZE(x) > 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  ((Py_SIZE(x) == 0) ? 0 : __Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  __Pyx_sst_abs(Py_SIZE(x))
  #define __Pyx_PyLong_SignedDigitCount(x)  Py_SIZE(x)
  #define __Pyx_PyLong_IsCompact(x)  (Py_SIZE(x) == 0 || Py_SIZE(x) == 1 || Py_SIZE(x) == -1)
  #define __Pyx_PyLong_CompactValue(x)\
        ((Py_SIZE(x) == 0) ? (sdigit) 0 : ((Py_SIZE(x) < 0) ? -(sdigit)__Pyx_PyLong_Digits(x)[0] : (sdigit)__Pyx_PyLong_Digits(x)[0]))
  typedef sdigit  __Pyx_compact_pylong;
  typedef digit  __Pyx_compact_upylong;
  #endif
  #if PY_VERSION_HEX >= 0x030C00A5
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->long_value.ob_digit)
  #else
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->ob_digit)
  #endif
#endif
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
#include <string.h>
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = (char) c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#include <string.h>
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

#if !CYTHON_USE_MODULE_STATE
static PyObject *__pyx_m = NULL;
#endif
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm = __FILE__;
static const char *__pyx_filename;

/* Header.proto */
#if !defined(CYTHON_CCOMPLEX)
  #if defined(__cplusplus)
    #define CYTHON_CCOMPLEX 1
  #elif (defined(_Complex_I) && !defined(_MSC_VER)) || ((defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L) && !defined(__STDC_NO_COMPLEX__) && !defined(_MSC_VER))
    #define CYTHON_CCOMPLEX 1
  #else
    #define CYTHON_CCOMPLEX 0
  #endif
#endif
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #include <complex>
  #else
    #include <complex.h>
  #endif
#endif
#if CYTHON_CCOMPLEX && !defined(__cplusplus) && defined(__sun__) && defined(__GNUC__)
  #undef _Complex_I
  #define _Complex_I 1.0fj
#endif

/* #### Code section: filename_table ### */

static const char *__pyx_f[] = {
  "src/pysndlib/instruments/lbj_piano.py",
  "__init__.cython-30.pxd",
  "type.pxd",
  "src/pysndlib/clm.pxd",
};
/* #### Code section: utility_code_proto_before_types ### */
/* ForceInitThreads.proto */
#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif

/* #### Code section: numeric_typedefs ### */

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":730
 * # in Cython to enable them only on the right systems.
 * 
 * ctypedef npy_int8       int8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 */
typedef npy_int8 __pyx_t_5numpy_int8_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":731
 * 
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t
 */
typedef npy_int16 __pyx_t_5numpy_int16_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":732
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int64      int64_t
 * #ctypedef npy_int96      int96_t
 */
typedef npy_int32 __pyx_t_5numpy_int32_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":733
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_int96      int96_t
 * #ctypedef npy_int128     int128_t
 */
typedef npy_int64 __pyx_t_5numpy_int64_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":737
 * #ctypedef npy_int128     int128_t
 * 
 * ctypedef npy_uint8      uint8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 */
typedef npy_uint8 __pyx_t_5numpy_uint8_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":738
 * 
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t
 */
typedef npy_uint16 __pyx_t_5numpy_uint16_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":739
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint64     uint64_t
 * #ctypedef npy_uint96     uint96_t
 */
typedef npy_uint32 __pyx_t_5numpy_uint32_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":740
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_uint96     uint96_t
 * #ctypedef npy_uint128    uint128_t
 */
typedef npy_uint64 __pyx_t_5numpy_uint64_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":744
 * #ctypedef npy_uint128    uint128_t
 * 
 * ctypedef npy_float32    float32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_float64    float64_t
 * #ctypedef npy_float80    float80_t
 */
typedef npy_float32 __pyx_t_5numpy_float32_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":745
 * 
 * ctypedef npy_float32    float32_t
 * ctypedef npy_float64    float64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_float80    float80_t
 * #ctypedef npy_float128   float128_t
 */
typedef npy_float64 __pyx_t_5numpy_float64_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":754
 * # The int types are mapped a bit surprising --
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   longlong_t
 * 
 */
typedef npy_long __pyx_t_5numpy_int_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":755
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   longlong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_ulong      uint_t
 */
typedef npy_longlong __pyx_t_5numpy_longlong_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":757
 * ctypedef npy_longlong   longlong_t
 * 
 * ctypedef npy_ulong      uint_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 */
typedef npy_ulong __pyx_t_5numpy_uint_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":758
 * 
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulonglong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_intp       intp_t
 */
typedef npy_ulonglong __pyx_t_5numpy_ulonglong_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":760
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 * ctypedef npy_intp       intp_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uintp      uintp_t
 * 
 */
typedef npy_intp __pyx_t_5numpy_intp_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":761
 * 
 * ctypedef npy_intp       intp_t
 * ctypedef npy_uintp      uintp_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_double     float_t
 */
typedef npy_uintp __pyx_t_5numpy_uintp_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":763
 * ctypedef npy_uintp      uintp_t
 * 
 * ctypedef npy_double     float_t             # <<<<<<<<<<<<<<
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t
 */
typedef npy_double __pyx_t_5numpy_float_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":764
 * 
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longdouble longdouble_t
 * 
 */
typedef npy_double __pyx_t_5numpy_double_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":765
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cfloat      cfloat_t
 */
typedef npy_longdouble __pyx_t_5numpy_longdouble_t;
/* #### Code section: complex_type_declarations ### */
/* Declarations.proto */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
  #ifdef __cplusplus
    typedef ::std::complex< float > __pyx_t_float_complex;
  #else
    typedef float _Complex __pyx_t_float_complex;
  #endif
#else
    typedef struct { float real, imag; } __pyx_t_float_complex;
#endif
static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float, float);

/* Declarations.proto */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
  #ifdef __cplusplus
    typedef ::std::complex< double > __pyx_t_double_complex;
  #else
    typedef double _Complex __pyx_t_double_complex;
  #endif
#else
    typedef struct { double real, imag; } __pyx_t_double_complex;
#endif
static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double, double);

/* #### Code section: type_declarations ### */

/*--- Type declarations ---*/
struct __pyx_obj_8pysndlib_3clm_mus_any;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":767
 * ctypedef npy_longdouble longdouble_t
 * 
 * ctypedef npy_cfloat      cfloat_t             # <<<<<<<<<<<<<<
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t
 */
typedef npy_cfloat __pyx_t_5numpy_cfloat_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":768
 * 
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t             # <<<<<<<<<<<<<<
 * ctypedef npy_clongdouble clongdouble_t
 * 
 */
typedef npy_cdouble __pyx_t_5numpy_cdouble_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":769
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cdouble     complex_t
 */
typedef npy_clongdouble __pyx_t_5numpy_clongdouble_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":771
 * ctypedef npy_clongdouble clongdouble_t
 * 
 * ctypedef npy_cdouble     complex_t             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 */
typedef npy_cdouble __pyx_t_5numpy_complex_t;

/* "pysndlib/sndlib.pxd":11
 * csndlib.mus_sound_initialize()
 * 
 * cpdef enum Header:             # <<<<<<<<<<<<<<
 *     """
 *     file types for audio files
 */
enum __pyx_t_8pysndlib_6sndlib_Header {
  __pyx_e_8pysndlib_6sndlib_UNKNOWN_HEADER,
  __pyx_e_8pysndlib_6sndlib_NEXT,
  __pyx_e_8pysndlib_6sndlib_AIFC,
  __pyx_e_8pysndlib_6sndlib_RIFF,
  __pyx_e_8pysndlib_6sndlib_RF64,
  __pyx_e_8pysndlib_6sndlib_BICSF,
  __pyx_e_8pysndlib_6sndlib_NIST,
  __pyx_e_8pysndlib_6sndlib_INRS,
  __pyx_e_8pysndlib_6sndlib_ESPS,
  __pyx_e_8pysndlib_6sndlib_SVX,
  __pyx_e_8pysndlib_6sndlib_VOC,
  __pyx_e_8pysndlib_6sndlib_SNDT,
  __pyx_e_8pysndlib_6sndlib_RAW,
  __pyx_e_8pysndlib_6sndlib_SMP,
  __pyx_e_8pysndlib_6sndlib_AVR,
  __pyx_e_8pysndlib_6sndlib_IRCAM,
  __pyx_e_8pysndlib_6sndlib_SD1,
  __pyx_e_8pysndlib_6sndlib_SPPACK,
  __pyx_e_8pysndlib_6sndlib_MUS10,
  __pyx_e_8pysndlib_6sndlib_HCOM,
  __pyx_e_8pysndlib_6sndlib_PSION,
  __pyx_e_8pysndlib_6sndlib_MAUD,
  __pyx_e_8pysndlib_6sndlib_IEEE,
  __pyx_e_8pysndlib_6sndlib_MATLAB,
  __pyx_e_8pysndlib_6sndlib_ADC,
  __pyx_e_8pysndlib_6sndlib_MIDI,
  __pyx_e_8pysndlib_6sndlib_SOUNDFONT,
  __pyx_e_8pysndlib_6sndlib_GRAVIS,
  __pyx_e_8pysndlib_6sndlib_COMDISCO,
  __pyx_e_8pysndlib_6sndlib_GOLDWAVE,
  __pyx_e_8pysndlib_6sndlib_SRFS,
  __pyx_e_8pysndlib_6sndlib_MIDI_SAMPLE_DUMP,
  __pyx_e_8pysndlib_6sndlib_DIAMONDWARE,
  __pyx_e_8pysndlib_6sndlib_ADF,
  __pyx_e_8pysndlib_6sndlib_SBSTUDIOII,
  __pyx_e_8pysndlib_6sndlib_DELUSION,
  __pyx_e_8pysndlib_6sndlib_FARANDOLE,
  __pyx_e_8pysndlib_6sndlib_SAMPLE_DUMP,
  __pyx_e_8pysndlib_6sndlib_ULTRATRACKER,
  __pyx_e_8pysndlib_6sndlib_YAMAHA_SY85,
  __pyx_e_8pysndlib_6sndlib_YAMAHA_TX16W,
  __pyx_e_8pysndlib_6sndlib_DIGIPLAYER,
  __pyx_e_8pysndlib_6sndlib_COVOX,
  __pyx_e_8pysndlib_6sndlib_AVI,
  __pyx_e_8pysndlib_6sndlib_OMF,
  __pyx_e_8pysndlib_6sndlib_QUICKTIME,
  __pyx_e_8pysndlib_6sndlib_ASF,
  __pyx_e_8pysndlib_6sndlib_YAMAHA_SY99,
  __pyx_e_8pysndlib_6sndlib_KURZWEIL_2000,
  __pyx_e_8pysndlib_6sndlib_AIFF,
  __pyx_e_8pysndlib_6sndlib_PAF,
  __pyx_e_8pysndlib_6sndlib_CSL,
  __pyx_e_8pysndlib_6sndlib_FILE_SAMP,
  __pyx_e_8pysndlib_6sndlib_PVF,
  __pyx_e_8pysndlib_6sndlib_SOUNDFORGE,
  __pyx_e_8pysndlib_6sndlib_TWINVQ,
  __pyx_e_8pysndlib_6sndlib_AKAI4,
  __pyx_e_8pysndlib_6sndlib_IMPULSETRACKER,
  __pyx_e_8pysndlib_6sndlib_KORG,
  __pyx_e_8pysndlib_6sndlib_NVF,
  __pyx_e_8pysndlib_6sndlib_CAFF,
  __pyx_e_8pysndlib_6sndlib_MAUI,
  __pyx_e_8pysndlib_6sndlib_SDIF,
  __pyx_e_8pysndlib_6sndlib_OGG,
  __pyx_e_8pysndlib_6sndlib_FLAC,
  __pyx_e_8pysndlib_6sndlib_SPEEX,
  __pyx_e_8pysndlib_6sndlib_MPEG,
  __pyx_e_8pysndlib_6sndlib_SHORTEN,
  __pyx_e_8pysndlib_6sndlib_TTA,
  __pyx_e_8pysndlib_6sndlib_WAVPACK,
  __pyx_e_8pysndlib_6sndlib_SOX,
  __pyx_e_8pysndlib_6sndlib_NUM_HEADERS
};

/* "pysndlib/sndlib.pxd":17
 *     UNKNOWN_HEADER, NEXT, AIFC, RIFF, RF64, BICSF, NIST, INRS, ESPS, SVX, VOC, SNDT, RAW, SMP, AVR, IRCAM, SD1, SPPACK, MUS10, HCOM, PSION, MAUD, IEEE, MATLAB, ADC, MIDI, SOUNDFONT, GRAVIS, COMDISCO, GOLDWAVE, SRFS, MIDI_SAMPLE_DUMP, DIAMONDWARE, ADF, SBSTUDIOII, DELUSION, FARANDOLE, SAMPLE_DUMP, ULTRATRACKER, YAMAHA_SY85, YAMAHA_TX16W, DIGIPLAYER, COVOX, AVI, OMF, QUICKTIME, ASF, YAMAHA_SY99, KURZWEIL_2000, AIFF, PAF, CSL, FILE_SAMP, PVF, SOUNDFORGE, TWINVQ, AKAI4, IMPULSETRACKER, KORG, NVF, CAFF, MAUI, SDIF, OGG, FLAC, SPEEX, MPEG, SHORTEN, TTA, WAVPACK, SOX, NUM_HEADERS,
 * 
 * cpdef enum Sample:             # <<<<<<<<<<<<<<
 *     """
 *     numerical sample types
 */
enum __pyx_t_8pysndlib_6sndlib_Sample {
  __pyx_e_8pysndlib_6sndlib_UNKNOWN_SAMPLE,
  __pyx_e_8pysndlib_6sndlib_BSHORT,
  __pyx_e_8pysndlib_6sndlib_MULAW,
  __pyx_e_8pysndlib_6sndlib_BYTE,
  __pyx_e_8pysndlib_6sndlib_BFLOAT,
  __pyx_e_8pysndlib_6sndlib_BINT,
  __pyx_e_8pysndlib_6sndlib_ALAW,
  __pyx_e_8pysndlib_6sndlib_UBYTE,
  __pyx_e_8pysndlib_6sndlib_B24INT,
  __pyx_e_8pysndlib_6sndlib_BDOUBLE,
  __pyx_e_8pysndlib_6sndlib_LSHORT,
  __pyx_e_8pysndlib_6sndlib_LINT,
  __pyx_e_8pysndlib_6sndlib_LFLOAT,
  __pyx_e_8pysndlib_6sndlib_LDOUBLE,
  __pyx_e_8pysndlib_6sndlib_UBSHORT,
  __pyx_e_8pysndlib_6sndlib_ULSHORT,
  __pyx_e_8pysndlib_6sndlib_L24INT,
  __pyx_e_8pysndlib_6sndlib_BINTN,
  __pyx_e_8pysndlib_6sndlib_LINTN,
  __pyx_e_8pysndlib_6sndlib_BFLOAT_UNSCALED,
  __pyx_e_8pysndlib_6sndlib_LFLOAT_UNSCALED,
  __pyx_e_8pysndlib_6sndlib_BDOUBLE_UNSCALED,
  __pyx_e_8pysndlib_6sndlib_LDOUBLE_UNSCALED,
  __pyx_e_8pysndlib_6sndlib_NUM_SAMPLES
};

/* "pysndlib/sndlib.pxd":23
 *     UNKNOWN_SAMPLE, BSHORT, MULAW, BYTE, BFLOAT, BINT, ALAW, UBYTE, B24INT, BDOUBLE, LSHORT, LINT, LFLOAT, LDOUBLE, UBSHORT, ULSHORT, L24INT, BINTN, LINTN, BFLOAT_UNSCALED, LFLOAT_UNSCALED, BDOUBLE_UNSCALED, LDOUBLE_UNSCALED, NUM_SAMPLES,
 * 
 * cpdef enum Error:             # <<<<<<<<<<<<<<
 *     """
 *     sndlib and clm errors
 */
enum __pyx_t_8pysndlib_6sndlib_Error {
  __pyx_e_8pysndlib_6sndlib_NO_ERROR,
  __pyx_e_8pysndlib_6sndlib_NO_FREQUENCY,
  __pyx_e_8pysndlib_6sndlib_NO_PHASE,
  __pyx_e_8pysndlib_6sndlib_NO_GEN,
  __pyx_e_8pysndlib_6sndlib_NO_LENGTH,
  __pyx_e_8pysndlib_6sndlib_NO_DESCRIBE,
  __pyx_e_8pysndlib_6sndlib_NO_DATA,
  __pyx_e_8pysndlib_6sndlib_NO_SCALER,
  __pyx_e_8pysndlib_6sndlib_MEMORY_ALLOCATION_FAILED,
  __pyx_e_8pysndlib_6sndlib_CANT_OPEN_FILE,
  __pyx_e_8pysndlib_6sndlib_NO_SAMPLE_INPUT,
  __pyx_e_8pysndlib_6sndlib_NO_SAMPLE_OUTPUT,
  __pyx_e_8pysndlib_6sndlib_NO_SUCH_CHANNEL,
  __pyx_e_8pysndlib_6sndlib_NO_FILE_NAME_PROVIDED,
  __pyx_e_8pysndlib_6sndlib_NO_LOCATION,
  __pyx_e_8pysndlib_6sndlib_NO_CHANNEL,
  __pyx_e_8pysndlib_6sndlib_NO_SUCH_FFT_WINDOW,
  __pyx_e_8pysndlib_6sndlib_UNSUPPORTED_SAMPLE_TYPE,
  __pyx_e_8pysndlib_6sndlib_HEADER_READ_FAILED,
  __pyx_e_8pysndlib_6sndlib_UNSUPPORTED_HEADER_TYPE,
  __pyx_e_8pysndlib_6sndlib_FILE_DESCRIPTORS_NOT_INITIALIZED,
  __pyx_e_8pysndlib_6sndlib_NOT_A_SOUND_FILE,
  __pyx_e_8pysndlib_6sndlib_FILE_CLOSED,
  __pyx_e_8pysndlib_6sndlib_WRITE_ERROR,
  __pyx_e_8pysndlib_6sndlib_HEADER_WRITE_FAILED,
  __pyx_e_8pysndlib_6sndlib_CANT_OPEN_TEMP_FILE,
  __pyx_e_8pysndlib_6sndlib_INTERRUPTED,
  __pyx_e_8pysndlib_6sndlib_BAD_ENVELOPE,
  __pyx_e_8pysndlib_6sndlib_AUDIO_CHANNELS_NOT_AVAILABLE,
  __pyx_e_8pysndlib_6sndlib_AUDIO_SRATE_NOT_AVAILABLE,
  __pyx_e_8pysndlib_6sndlib_AUDIO_SAMPLE_TYPE_NOT_AVAILABLE,
  __pyx_e_8pysndlib_6sndlib_AUDIO_NO_INPUT_AVAILABLE,
  __pyx_e_8pysndlib_6sndlib_AUDIO_CONFIGURATION_NOT_AVAILABLE,
  __pyx_e_8pysndlib_6sndlib_AUDIO_WRITE_ERROR,
  __pyx_e_8pysndlib_6sndlib_AUDIO_SIZE_NOT_AVAILABLE,
  __pyx_e_8pysndlib_6sndlib_AUDIO_DEVICE_NOT_AVAILABLE,
  __pyx_e_8pysndlib_6sndlib_AUDIO_CANT_CLOSE,
  __pyx_e_8pysndlib_6sndlib_AUDIO_CANT_OPEN,
  __pyx_e_8pysndlib_6sndlib_AUDIO_READ_ERROR,
  __pyx_e_8pysndlib_6sndlib_AUDIO_CANT_WRITE,
  __pyx_e_8pysndlib_6sndlib_AUDIO_CANT_READ,
  __pyx_e_8pysndlib_6sndlib_AUDIO_NO_READ_PERMISSION,
  __pyx_e_8pysndlib_6sndlib_CANT_CLOSE_FILE,
  __pyx_e_8pysndlib_6sndlib_ARG_OUT_OF_RANGE,
  __pyx_e_8pysndlib_6sndlib_NO_CHANNELS,
  __pyx_e_8pysndlib_6sndlib_NO_HOP,
  __pyx_e_8pysndlib_6sndlib_NO_WIDTH,
  __pyx_e_8pysndlib_6sndlib_NO_FILE_NAME,
  __pyx_e_8pysndlib_6sndlib_NO_RAMP,
  __pyx_e_8pysndlib_6sndlib_NO_RUN,
  __pyx_e_8pysndlib_6sndlib_NO_INCREMENT,
  __pyx_e_8pysndlib_6sndlib_NO_OFFSET,
  __pyx_e_8pysndlib_6sndlib_NO_XCOEFF,
  __pyx_e_8pysndlib_6sndlib_NO_YCOEFF,
  __pyx_e_8pysndlib_6sndlib_NO_XCOEFFS,
  __pyx_e_8pysndlib_6sndlib_NO_YCOEFFS,
  __pyx_e_8pysndlib_6sndlib_NO_RESET,
  __pyx_e_8pysndlib_6sndlib_BAD_SIZE,
  __pyx_e_8pysndlib_6sndlib_CANT_CONVERT,
  __pyx_e_8pysndlib_6sndlib_READ_ERROR,
  __pyx_e_8pysndlib_6sndlib_NO_FEEDFORWARD,
  __pyx_e_8pysndlib_6sndlib_NO_FEEDBACK,
  __pyx_e_8pysndlib_6sndlib_NO_INTERP_TYPE,
  __pyx_e_8pysndlib_6sndlib_NO_POSITION,
  __pyx_e_8pysndlib_6sndlib_NO_ORDER,
  __pyx_e_8pysndlib_6sndlib_NO_COPY,
  __pyx_e_8pysndlib_6sndlib_CANT_TRANSLATE,
  __pyx_e_8pysndlib_6sndlib_NUM_ERRORS
};
struct __pyx_opt_args_8pysndlib_3clm_7mus_any_from_ptr;
struct __pyx_opt_args_8pysndlib_3clm_7mus_any_set_up_data;
struct __pyx_opt_args_8pysndlib_3clm_7mus_any_next;
struct __pyx_opt_args_8pysndlib_3clm_get_srate;
struct __pyx_opt_args_8pysndlib_3clm_random;
struct __pyx_opt_args_8pysndlib_3clm_contrast_enhancement;
struct __pyx_opt_args_8pysndlib_3clm_mus_interpolate;
struct __pyx_opt_args_8pysndlib_3clm_make_fft_window;
struct __pyx_opt_args_8pysndlib_3clm_file2ndarray;
struct __pyx_opt_args_8pysndlib_3clm_ndarray2file;
struct __pyx_opt_args_8pysndlib_3clm_mus_spectrum;
struct __pyx_opt_args_8pysndlib_3clm_spectrum;
struct __pyx_opt_args_8pysndlib_3clm_convolution;
struct __pyx_opt_args_8pysndlib_3clm_mus_convolution;
struct __pyx_opt_args_8pysndlib_3clm_partials2wave;
struct __pyx_opt_args_8pysndlib_3clm_phase_partials2wave;
struct __pyx_opt_args_8pysndlib_3clm_partials2polynomial;
struct __pyx_opt_args_8pysndlib_3clm_oscil;
struct __pyx_opt_args_8pysndlib_3clm_pulsed_env;
struct __pyx_opt_args_8pysndlib_3clm_table_lookup;
struct __pyx_opt_args_8pysndlib_3clm_make_table_lookup_with_env;
struct __pyx_opt_args_8pysndlib_3clm_polywave;
struct __pyx_opt_args_8pysndlib_3clm_polyshape;
struct __pyx_opt_args_8pysndlib_3clm_polyshape_unmodulated;
struct __pyx_opt_args_8pysndlib_3clm_triangle_wave;
struct __pyx_opt_args_8pysndlib_3clm_square_wave;
struct __pyx_opt_args_8pysndlib_3clm_make_sawtooth_wave;
struct __pyx_opt_args_8pysndlib_3clm_sawtooth_wave;
struct __pyx_opt_args_8pysndlib_3clm_pulse_train;
struct __pyx_opt_args_8pysndlib_3clm_ncos;
struct __pyx_opt_args_8pysndlib_3clm_nsin;
struct __pyx_opt_args_8pysndlib_3clm_nrxysin;
struct __pyx_opt_args_8pysndlib_3clm_nrxycos;
struct __pyx_opt_args_8pysndlib_3clm_rxykcos;
struct __pyx_opt_args_8pysndlib_3clm_rxyksin;
struct __pyx_opt_args_8pysndlib_3clm_ssb_am;
struct __pyx_opt_args_8pysndlib_3clm_wave_train;
struct __pyx_opt_args_8pysndlib_3clm_rand;
struct __pyx_opt_args_8pysndlib_3clm_rand_interp;
struct __pyx_opt_args_8pysndlib_3clm_formant;
struct __pyx_opt_args_8pysndlib_3clm_firmant;
struct __pyx_opt_args_8pysndlib_3clm_delay;
struct __pyx_opt_args_8pysndlib_3clm_tap;
struct __pyx_opt_args_8pysndlib_3clm_comb;
struct __pyx_opt_args_8pysndlib_3clm_filtered_comb;
struct __pyx_opt_args_8pysndlib_3clm_notch;
struct __pyx_opt_args_8pysndlib_3clm_all_pass;
struct __pyx_opt_args_8pysndlib_3clm_asymmetric_fm;
struct __pyx_opt_args_8pysndlib_3clm_file2sample;
struct __pyx_opt_args_8pysndlib_3clm_src;
struct __pyx_opt_args_8pysndlib_3clm_out_any;
struct __pyx_opt_args_8pysndlib_3clm_outa;
struct __pyx_opt_args_8pysndlib_3clm_outb;
struct __pyx_opt_args_8pysndlib_3clm_outc;
struct __pyx_opt_args_8pysndlib_3clm_outd;
struct __pyx_opt_args_8pysndlib_3clm_convolve_files;
struct __pyx_opt_args_8pysndlib_3clm_biquad_set_resonance;
struct __pyx_opt_args_8pysndlib_3clm_edot_product;

/* "clm.pxd":14
 * 
 * # --------------- clm enums ---------------- #
 * cpdef enum Interp:             # <<<<<<<<<<<<<<
 *     """
 *     various interpolation types
 */
enum __pyx_t_8pysndlib_3clm_Interp {
  __pyx_e_8pysndlib_3clm_NONE,
  __pyx_e_8pysndlib_3clm_LINEAR,
  __pyx_e_8pysndlib_3clm_SINUSOIDAL,
  __pyx_e_8pysndlib_3clm_ALL_PASS,
  __pyx_e_8pysndlib_3clm_LAGRANGE,
  __pyx_e_8pysndlib_3clm_BEZIER,
  __pyx_e_8pysndlib_3clm_HERMITE
};

/* "clm.pxd":20
 *     NONE, LINEAR,SINUSOIDAL, ALL_PASS, LAGRANGE, BEZIER, HERMITE
 * 
 * cpdef enum Window:             # <<<<<<<<<<<<<<
 *     """
 *     many useful windows
 */
enum __pyx_t_8pysndlib_3clm_Window {
  __pyx_e_8pysndlib_3clm_RECTANGULAR,
  __pyx_e_8pysndlib_3clm_HANN,
  __pyx_e_8pysndlib_3clm_WELCH,
  __pyx_e_8pysndlib_3clm_PARZEN,
  __pyx_e_8pysndlib_3clm_BARTLETT,
  __pyx_e_8pysndlib_3clm_HAMMING,
  __pyx_e_8pysndlib_3clm_BLACKMAN2,
  __pyx_e_8pysndlib_3clm_BLACKMAN3,
  __pyx_e_8pysndlib_3clm_BLACKMAN4,
  __pyx_e_8pysndlib_3clm_EXPONENTIAL,
  __pyx_e_8pysndlib_3clm_RIEMANN,
  __pyx_e_8pysndlib_3clm_KAISER,
  __pyx_e_8pysndlib_3clm_CAUCHY,
  __pyx_e_8pysndlib_3clm_POISSON,
  __pyx_e_8pysndlib_3clm_GAUSSIAN,
  __pyx_e_8pysndlib_3clm_TUKEY,
  __pyx_e_8pysndlib_3clm_DOLPH_CHEBYSHEV,
  __pyx_e_8pysndlib_3clm_HANN_POISSON,
  __pyx_e_8pysndlib_3clm_CONNES,
  __pyx_e_8pysndlib_3clm_SAMARAKI,
  __pyx_e_8pysndlib_3clm_ULTRASPHERICAL,
  __pyx_e_8pysndlib_3clm_BARTLETT_HANN,
  __pyx_e_8pysndlib_3clm_BOHMAN,
  __pyx_e_8pysndlib_3clm_FLAT_TOP,
  __pyx_e_8pysndlib_3clm_BLACKMAN5,
  __pyx_e_8pysndlib_3clm_BLACKMAN6,
  __pyx_e_8pysndlib_3clm_BLACKMAN7,
  __pyx_e_8pysndlib_3clm_BLACKMAN8,
  __pyx_e_8pysndlib_3clm_BLACKMAN9,
  __pyx_e_8pysndlib_3clm_BLACKMAN10,
  __pyx_e_8pysndlib_3clm_RV2,
  __pyx_e_8pysndlib_3clm_RV3,
  __pyx_e_8pysndlib_3clm_RV4,
  __pyx_e_8pysndlib_3clm_MLT_SINE,
  __pyx_e_8pysndlib_3clm_PAPOULIS,
  __pyx_e_8pysndlib_3clm_DPSS,
  __pyx_e_8pysndlib_3clm_SINC
};

/* "clm.pxd":26
 *     RECTANGULAR, HANN, WELCH, PARZEN, BARTLETT, HAMMING, BLACKMAN2, BLACKMAN3, BLACKMAN4, EXPONENTIAL, RIEMANN, KAISER, CAUCHY, POISSON, GAUSSIAN, TUKEY, DOLPH_CHEBYSHEV, HANN_POISSON, CONNES, SAMARAKI, ULTRASPHERICAL, BARTLETT_HANN, BOHMAN, FLAT_TOP, BLACKMAN5, BLACKMAN6, BLACKMAN7, BLACKMAN8, BLACKMAN9, BLACKMAN10, RV2, RV3, RV4, MLT_SINE, PAPOULIS, DPSS, SINC,
 * 
 * cpdef enum Spectrum:             # <<<<<<<<<<<<<<
 *     """
 *     types of normalizations when using the spectrum function. the results are in db if in_db, or linear and normalized to 1.0 normalized, or linear unnormalized raw
 */
enum __pyx_t_8pysndlib_3clm_Spectrum {
  __pyx_e_8pysndlib_3clm_IN_DB,
  __pyx_e_8pysndlib_3clm_NORMALIZED,
  __pyx_e_8pysndlib_3clm_RAW
};

/* "clm.pxd":32
 *     IN_DB, NORMALIZED, RAW
 * 
 * cpdef enum Polynomial:             # <<<<<<<<<<<<<<
 *     """
 *     used for polynomial based gens
 */
enum __pyx_t_8pysndlib_3clm_Polynomial {
  __pyx_e_8pysndlib_3clm_EITHER_KIND,
  __pyx_e_8pysndlib_3clm_FIRST_KIND,
  __pyx_e_8pysndlib_3clm_SECOND_KIND,
  __pyx_e_8pysndlib_3clm_BOTH_KINDS
};

/* "clm.pxd":77
 *     # these should all be considered private
 *     @staticmethod
 *     cdef mus_any from_ptr(cclm.mus_any *_ptr, bint owner=*, cython.int length=*)             # <<<<<<<<<<<<<<
 * 
 *     cpdef cache_append(self, obj)
 */
struct __pyx_opt_args_8pysndlib_3clm_7mus_any_from_ptr {
  int __pyx_n;
  int owner;
  int length;
};

/* "clm.pxd":81
 *     cpdef cache_append(self, obj)
 *     cpdef cache_extend(self, obj)
 *     cpdef set_up_data(self , cython.int length=*)             # <<<<<<<<<<<<<<
 *     cpdef set_up_xcoeffs(self)
 *     cpdef set_up_ycoeffs(self)
 */
struct __pyx_opt_args_8pysndlib_3clm_7mus_any_set_up_data {
  int __pyx_n;
  int length;
};

/* "clm.pxd":88
 *     # these should all be considered public
 *     cpdef mus_reset(self)
 *     cpdef next(self, cython.double arg1=*, cython.double arg2=*)             # <<<<<<<<<<<<<<
 * 
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_7mus_any_next {
  int __pyx_n;
  double arg1;
  double arg2;
};

/* "clm.pxd":97
 * cpdef mus_reset(mus_any obj)
 * cpdef cython.long get_length(obj)
 * cpdef cython.double get_srate(obj=*)             # <<<<<<<<<<<<<<
 * cpdef cython.long get_framples(obj)
 * cpdef cython.long get_channels(obj)
 */
struct __pyx_opt_args_8pysndlib_3clm_get_srate {
  int __pyx_n;
  PyObject *obj;
};

/* "clm.pxd":100
 * cpdef cython.long get_framples(obj)
 * cpdef cython.long get_channels(obj)
 * cpdef cython.double random(cython.double x=*)             # <<<<<<<<<<<<<<
 * cpdef bint is_zero( cython.double x)
 * cpdef bint is_even(n)
 */
struct __pyx_opt_args_8pysndlib_3clm_random {
  int __pyx_n;
  double x;
};

/* "clm.pxd":119
 * cpdef cython.double ring_modulate(cython.double s1, cython.double s2)
 * cpdef cython.double amplitude_modulate(cython.double s1, cython.double s2, cython.double s3)
 * cpdef cython.double contrast_enhancement(cython.double insig, cython.double fm_index=* )             # <<<<<<<<<<<<<<
 * cpdef cython.double dot_product(np.ndarray data1, np.ndarray  data2)
 * cpdef cython.double polynomial(np.ndarray coeffs, cython.double x )
 */
struct __pyx_opt_args_8pysndlib_3clm_contrast_enhancement {
  int __pyx_n;
  double fm_index;
};

/* "clm.pxd":124
 * cpdef cython.double array_interp(np.ndarray  fn, cython.double  x)
 * cpdef cython.double bessi0(cython.double x)
 * cpdef cython.double mus_interpolate(Interp interp_type, cython.double x, np.ndarray table, cython.double y1 =*)             # <<<<<<<<<<<<<<
 * cpdef np.ndarray make_fft_window(Window window_type, cython.int size, cython.double beta=* , cython.double alpha=*)
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_mus_interpolate {
  int __pyx_n;
  double y1;
};

/* "clm.pxd":125
 * cpdef cython.double bessi0(cython.double x)
 * cpdef cython.double mus_interpolate(Interp interp_type, cython.double x, np.ndarray table, cython.double y1 =*)
 * cpdef np.ndarray make_fft_window(Window window_type, cython.int size, cython.double beta=* , cython.double alpha=*)             # <<<<<<<<<<<<<<
 * 
 * cpdef file2ndarray(str filename, channel=*, beg=*, size=*)
 */
struct __pyx_opt_args_8pysndlib_3clm_make_fft_window {
  int __pyx_n;
  double beta;
  double alpha;
};

/* "clm.pxd":127
 * cpdef np.ndarray make_fft_window(Window window_type, cython.int size, cython.double beta=* , cython.double alpha=*)
 * 
 * cpdef file2ndarray(str filename, channel=*, beg=*, size=*)             # <<<<<<<<<<<<<<
 * cpdef ndarray2file(str filename, np.ndarray arr, size=*, sr=*, sample_type=*, header_type=*, comment=* )
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_file2ndarray {
  int __pyx_n;
  PyObject *channel;
  PyObject *beg;
  PyObject *size;
};

/* "clm.pxd":128
 * 
 * cpdef file2ndarray(str filename, channel=*, beg=*, size=*)
 * cpdef ndarray2file(str filename, np.ndarray arr, size=*, sr=*, sample_type=*, header_type=*, comment=* )             # <<<<<<<<<<<<<<
 * 
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_ndarray2file {
  int __pyx_n;
  PyObject *size;
  PyObject *sr;
  PyObject *sample_type;
  PyObject *header_type;
  PyObject *comment;
};

/* "clm.pxd":139
 * cpdef np.ndarray mus_polar2rectangular(np.ndarray rdat,  np.ndarray idat)
 * cpdef tuple polar2rectangular(np.ndarray rdat, np.ndarray idat)
 * cpdef np.ndarray mus_spectrum(np.ndarray rdat, np.ndarray  idat, np.ndarray  window, Spectrum norm_type=*)             # <<<<<<<<<<<<<<
 * cpdef tuple spectrum(np.ndarray rdat,np.ndarray idat,np.ndarray window, Spectrum norm_type=*)
 * cpdef np.ndarray mus_spectrum(np.ndarray rdat, np.ndarray  idat, np.ndarray  window, Spectrum norm_type=*)
 */
struct __pyx_opt_args_8pysndlib_3clm_mus_spectrum {
  int __pyx_n;
  enum __pyx_t_8pysndlib_3clm_Spectrum norm_type;
};

/* "clm.pxd":140
 * cpdef tuple polar2rectangular(np.ndarray rdat, np.ndarray idat)
 * cpdef np.ndarray mus_spectrum(np.ndarray rdat, np.ndarray  idat, np.ndarray  window, Spectrum norm_type=*)
 * cpdef tuple spectrum(np.ndarray rdat,np.ndarray idat,np.ndarray window, Spectrum norm_type=*)             # <<<<<<<<<<<<<<
 * cpdef np.ndarray mus_spectrum(np.ndarray rdat, np.ndarray  idat, np.ndarray  window, Spectrum norm_type=*)
 * cpdef np.ndarray convolution(np.ndarray rl1, np.ndarray  rl2, fft_size = *)
 */
struct __pyx_opt_args_8pysndlib_3clm_spectrum {
  int __pyx_n;
  enum __pyx_t_8pysndlib_3clm_Spectrum norm_type;
};

/* "clm.pxd":142
 * cpdef tuple spectrum(np.ndarray rdat,np.ndarray idat,np.ndarray window, Spectrum norm_type=*)
 * cpdef np.ndarray mus_spectrum(np.ndarray rdat, np.ndarray  idat, np.ndarray  window, Spectrum norm_type=*)
 * cpdef np.ndarray convolution(np.ndarray rl1, np.ndarray  rl2, fft_size = *)             # <<<<<<<<<<<<<<
 * cpdef np.ndarray mus_convolution(np.ndarray  rl1, np.ndarray  rl2, fft_size=*)
 * cpdef np.ndarray autocorrelate(np.ndarray data)
 */
struct __pyx_opt_args_8pysndlib_3clm_convolution {
  int __pyx_n;
  PyObject *fft_size;
};

/* "clm.pxd":143
 * cpdef np.ndarray mus_spectrum(np.ndarray rdat, np.ndarray  idat, np.ndarray  window, Spectrum norm_type=*)
 * cpdef np.ndarray convolution(np.ndarray rl1, np.ndarray  rl2, fft_size = *)
 * cpdef np.ndarray mus_convolution(np.ndarray  rl1, np.ndarray  rl2, fft_size=*)             # <<<<<<<<<<<<<<
 * cpdef np.ndarray autocorrelate(np.ndarray data)
 * cpdef np.ndarray mus_autocorrelate(np.ndarray data)
 */
struct __pyx_opt_args_8pysndlib_3clm_mus_convolution {
  int __pyx_n;
  PyObject *fft_size;
};

/* "clm.pxd":150
 * cpdef np.ndarray mus_cepstrum( np.ndarray data)
 * cpdef np.ndarray cepstrum(np.ndarray data)
 * cpdef np.ndarray partials2wave(partials, np.ndarray wave=*, table_size=*, bint norm=* )             # <<<<<<<<<<<<<<
 * cpdef np.ndarray phase_partials2wave(partials, np.ndarray wave=*, table_size=*, bint norm=* )
 * cpdef np.ndarray partials2polynomial(partials, Polynomial kind=*)
 */
struct __pyx_opt_args_8pysndlib_3clm_partials2wave {
  int __pyx_n;
  PyArrayObject *wave;
  PyObject *table_size;
  int norm;
};

/* "clm.pxd":151
 * cpdef np.ndarray cepstrum(np.ndarray data)
 * cpdef np.ndarray partials2wave(partials, np.ndarray wave=*, table_size=*, bint norm=* )
 * cpdef np.ndarray phase_partials2wave(partials, np.ndarray wave=*, table_size=*, bint norm=* )             # <<<<<<<<<<<<<<
 * cpdef np.ndarray partials2polynomial(partials, Polynomial kind=*)
 * cpdef np.ndarray normalize_partials(partials)
 */
struct __pyx_opt_args_8pysndlib_3clm_phase_partials2wave {
  int __pyx_n;
  PyArrayObject *wave;
  PyObject *table_size;
  int norm;
};

/* "clm.pxd":152
 * cpdef np.ndarray partials2wave(partials, np.ndarray wave=*, table_size=*, bint norm=* )
 * cpdef np.ndarray phase_partials2wave(partials, np.ndarray wave=*, table_size=*, bint norm=* )
 * cpdef np.ndarray partials2polynomial(partials, Polynomial kind=*)             # <<<<<<<<<<<<<<
 * cpdef np.ndarray normalize_partials(partials)
 * cpdef cython.double chebyshev_tu_sum(cython.double x, tcoeffs, ucoeffs)
 */
struct __pyx_opt_args_8pysndlib_3clm_partials2polynomial {
  int __pyx_n;
  enum __pyx_t_8pysndlib_3clm_Polynomial kind;
};

/* "clm.pxd":160
 * # ---------------- clm gen ---------------- #
 * 
 * cpdef cython.double oscil(mus_any gen, cython.double fm=*, cython.double pm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double oscil_unmodulated(mus_any gen)
 * cpdef bint is_oscil(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_oscil {
  int __pyx_n;
  double fm;
  double pm;
};

/* "clm.pxd":176
 * cpdef cython.double env_any(mus_any gen , connection_function)
 * 
 * cpdef cython.double pulsed_env(mus_any gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double pulsed_env_unmodulated(mus_any gen)
 * cpdef bint is_pulsed_env(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_pulsed_env {
  int __pyx_n;
  double fm;
};

/* "clm.pxd":180
 * cpdef bint is_pulsed_env(mus_any gen)
 * 
 * cpdef cython.double table_lookup(mus_any gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double table_lookup_unmodulated(mus_any gen)
 * cpdef bint is_table_lookup(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_table_lookup {
  int __pyx_n;
  double fm;
};

/* "clm.pxd":183
 * cpdef cython.double table_lookup_unmodulated(mus_any gen)
 * cpdef bint is_table_lookup(mus_any gen)
 * cpdef mus_any make_table_lookup_with_env(cython.double frequency, envelope, size= *)             # <<<<<<<<<<<<<<
 * 
 * cpdef cython.double polywave(mus_any gen, cython.double fm=*)
 */
struct __pyx_opt_args_8pysndlib_3clm_make_table_lookup_with_env {
  int __pyx_n;
  PyObject *size;
};

/* "clm.pxd":185
 * cpdef mus_any make_table_lookup_with_env(cython.double frequency, envelope, size= *)
 * 
 * cpdef cython.double polywave(mus_any gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double polywave_unmodulated(mus_any gen)
 * cpdef bint is_polywave(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_polywave {
  int __pyx_n;
  double fm;
};

/* "clm.pxd":189
 * cpdef bint is_polywave(mus_any gen)
 * 
 * cpdef cython.double polyshape(mus_any gen, cython.double index=*, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double polyshape_unmodulated(mus_any gen, cython.double index=*)
 * cpdef bint is_polyshape(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_polyshape {
  int __pyx_n;
  double index;
  double fm;
};

/* "clm.pxd":190
 * 
 * cpdef cython.double polyshape(mus_any gen, cython.double index=*, cython.double fm=*)
 * cpdef cython.double polyshape_unmodulated(mus_any gen, cython.double index=*)             # <<<<<<<<<<<<<<
 * cpdef bint is_polyshape(mus_any gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_polyshape_unmodulated {
  int __pyx_n;
  double index;
};

/* "clm.pxd":193
 * cpdef bint is_polyshape(mus_any gen)
 * 
 * cpdef cython.double triangle_wave(mus_any gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double triangle_wave_unmodulated(mus_any gen)
 * cpdef bint is_triangle_wave(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_triangle_wave {
  int __pyx_n;
  double fm;
};

/* "clm.pxd":197
 * cpdef bint is_triangle_wave(mus_any gen)
 * 
 * cpdef cython.double square_wave(mus_any gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef bint is_square_wave(mus_any gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_square_wave {
  int __pyx_n;
  double fm;
};

/* "clm.pxd":200
 * cpdef bint is_square_wave(mus_any gen)
 * 
 * cpdef mus_any make_sawtooth_wave(cython.double frequency, cython.double amplitude=*, cython.double phase=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double sawtooth_wave(mus_any gen, cython.double fm =*)
 * cpdef bint is_sawtooth_wave(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_make_sawtooth_wave {
  int __pyx_n;
  double amplitude;
  double phase;
};

/* "clm.pxd":201
 * 
 * cpdef mus_any make_sawtooth_wave(cython.double frequency, cython.double amplitude=*, cython.double phase=*)
 * cpdef cython.double sawtooth_wave(mus_any gen, cython.double fm =*)             # <<<<<<<<<<<<<<
 * cpdef bint is_sawtooth_wave(mus_any gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_sawtooth_wave {
  int __pyx_n;
  double fm;
};

/* "clm.pxd":204
 * cpdef bint is_sawtooth_wave(mus_any gen)
 * 
 * cpdef cython.double pulse_train(mus_any gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double pulse_train_unmodulated(mus_any gen)
 * cpdef bint is_pulse_train(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_pulse_train {
  int __pyx_n;
  double fm;
};

/* "clm.pxd":208
 * cpdef bint is_pulse_train(mus_any gen)
 * 
 * cpdef cython.double ncos(mus_any gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef bint is_ncos(mus_any gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_ncos {
  int __pyx_n;
  double fm;
};

/* "clm.pxd":211
 * cpdef bint is_ncos(mus_any gen)
 * 
 * cpdef cython.double nsin(mus_any gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef bint is_nsin(mus_any gen )
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_nsin {
  int __pyx_n;
  double fm;
};

/* "clm.pxd":214
 * cpdef bint is_nsin(mus_any gen )
 * 
 * cpdef cython.double nrxysin(mus_any gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef bint is_nrxysin(mus_any gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_nrxysin {
  int __pyx_n;
  double fm;
};

/* "clm.pxd":217
 * cpdef bint is_nrxysin(mus_any gen)
 * 
 * cpdef cython.double nrxycos(mus_any gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef bint is_nrxycos(mus_any gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_nrxycos {
  int __pyx_n;
  double fm;
};

/* "clm.pxd":220
 * cpdef bint is_nrxycos(mus_any gen)
 * 
 * cpdef cython.double rxykcos(mus_any gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef bint is_rxykcos(mus_any gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_rxykcos {
  int __pyx_n;
  double fm;
};

/* "clm.pxd":223
 * cpdef bint is_rxykcos(mus_any gen)
 * 
 * cpdef cython.double rxyksin(mus_any gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef bint is_rxyksin(mus_any gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_rxyksin {
  int __pyx_n;
  double fm;
};

/* "clm.pxd":226
 * cpdef bint is_rxyksin(mus_any gen)
 * 
 * cpdef cython.double ssb_am(mus_any gen, cython.double insig, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double ssb_am_unmodulated(mus_any gen, cython.double insig)
 * cpdef bint is_ssb_am(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_ssb_am {
  int __pyx_n;
  double fm;
};

/* "clm.pxd":230
 * cpdef bint is_ssb_am(mus_any gen)
 * 
 * cpdef cython.double wave_train(mus_any gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double wave_train_unmodulated(mus_any gen)
 * cpdef bint is_wave_train(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_wave_train {
  int __pyx_n;
  double fm;
};

/* "clm.pxd":234
 * cpdef bint is_wave_train(mus_any gen)
 * 
 * cpdef cython.double rand(mus_any gen, cython.double sweep=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double rand_unmodulated(mus_any gen)
 * cpdef bint is_rand(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_rand {
  int __pyx_n;
  double sweep;
};

/* "clm.pxd":238
 * cpdef bint is_rand(mus_any gen)
 * 
 * cpdef rand_interp(mus_any gen, cython.double sweep=*)             # <<<<<<<<<<<<<<
 * cpdef rand_interp_unmodulated(mus_any gen)
 * cpdef bint is_rand_interp(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_rand_interp {
  int __pyx_n;
  double sweep;
};

/* "clm.pxd":254
 * cpdef bint is_two_zero(mus_any gen)
 * 
 * cpdef cython.double formant(mus_any gen, cython.double insig, cython.double radians=*)             # <<<<<<<<<<<<<<
 * cpdef bint is_formant(mus_any gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_formant {
  int __pyx_n;
  double radians;
};

/* "clm.pxd":260
 * cpdef bint is_formant_bank(mus_any gen)
 * 
 * cpdef firmant(mus_any gen, cython.double insig, cython.double radians=*)             # <<<<<<<<<<<<<<
 * cpdef is_firmant(mus_any gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_firmant {
  int __pyx_n;
  double radians;
};

/* "clm.pxd":272
 * cpdef bint is_iir_filter(mus_any gen)
 * 
 * cpdef cython.double delay(mus_any gen, cython.double insig, cython.double pm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double delay_unmodulated(mus_any gen, cython.double insig)
 * cpdef bint is_delay(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_delay {
  int __pyx_n;
  double pm;
};

/* "clm.pxd":275
 * cpdef cython.double delay_unmodulated(mus_any gen, cython.double insig)
 * cpdef bint is_delay(mus_any gen)
 * cpdef cython.double tap(mus_any gen, offset=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double tap_unmodulated(mus_any gen)
 * cpdef bint is_tap(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_tap {
  int __pyx_n;
  PyObject *offset;
};

/* "clm.pxd":280
 * cpdef cython.double delay_tick(mus_any gen, cython.double insig )
 * 
 * cpdef cython.double comb(mus_any gen, cython.double insig, pm=*)             # <<<<<<<<<<<<<<
 * cpdef bint is_comb(mus_any gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_comb {
  int __pyx_n;
  PyObject *pm;
};

/* "clm.pxd":286
 * cpdef bint is_comb_bank(mus_any gen)
 * 
 * cpdef cython.double filtered_comb(mus_any gen, cython.double insig, cython.double pm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double filtered_comb_unmodulated(mus_any gen, cython.double insig)
 * cpdef bint is_filtered_comb(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_filtered_comb {
  int __pyx_n;
  double pm;
};

/* "clm.pxd":293
 * cpdef bint is_filtered_comb_bank(mus_any gen)
 * 
 * cpdef cython.double notch(mus_any gen, cython.double insig, cython.double pm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double notch_unmodulated(mus_any gen, cython.double insig)
 * cpdef bint is_notch(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_notch {
  int __pyx_n;
  double pm;
};

/* "clm.pxd":297
 * cpdef bint is_notch(mus_any gen)
 * 
 * cpdef cython.double all_pass(mus_any gen, cython.double insig, pm=*)             # <<<<<<<<<<<<<<
 * cpdef is_all_pass(mus_any gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_all_pass {
  int __pyx_n;
  PyObject *pm;
};

/* "clm.pxd":315
 * cpdef is_moving_norm(mus_any gen)
 * 
 * cpdef cython.double asymmetric_fm(mus_any gen, cython.double index, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double asymmetric_fm_unmodulated(mus_any gen, cython.double index)
 * cpdef bint is_asymmetric_fm(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_asymmetric_fm {
  int __pyx_n;
  double fm;
};

/* "clm.pxd":319
 * cpdef bint is_asymmetric_fm(mus_any gen)
 * 
 * cpdef cython.double file2sample(mus_any gen, cython.long loc, cython.int chan=*)             # <<<<<<<<<<<<<<
 * cpdef bint is_file2sample(mus_any gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_file2sample {
  int __pyx_n;
  int chan;
};

/* "clm.pxd":335
 * cpdef is_readin(mus_any gen)
 * 
 * cpdef cython.double src(mus_any gen, cython.double sr_change=*)             # <<<<<<<<<<<<<<
 * cpdef bint is_src(mus_any gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_src {
  int __pyx_n;
  double sr_change;
};

/* "clm.pxd":352
 * cpdef np.ndarray phase_vocoder_phase_increments(mus_any gen)
 * 
 * cpdef out_any(cython.long loc, cython.double data, cython.int channel, output=*)             # <<<<<<<<<<<<<<
 * cpdef outa(cython.long loc, cython.double data, output=*)
 * cpdef outb(cython.long loc, cython.double data, output=*)
 */
struct __pyx_opt_args_8pysndlib_3clm_out_any {
  int __pyx_n;
  PyObject *output;
};

/* "clm.pxd":353
 * 
 * cpdef out_any(cython.long loc, cython.double data, cython.int channel, output=*)
 * cpdef outa(cython.long loc, cython.double data, output=*)             # <<<<<<<<<<<<<<
 * cpdef outb(cython.long loc, cython.double data, output=*)
 * cpdef outc(cython.long loc, cython.double data, output=*)
 */
struct __pyx_opt_args_8pysndlib_3clm_outa {
  int __pyx_n;
  PyObject *output;
};

/* "clm.pxd":354
 * cpdef out_any(cython.long loc, cython.double data, cython.int channel, output=*)
 * cpdef outa(cython.long loc, cython.double data, output=*)
 * cpdef outb(cython.long loc, cython.double data, output=*)             # <<<<<<<<<<<<<<
 * cpdef outc(cython.long loc, cython.double data, output=*)
 * cpdef outd(cython.long loc, cython.double data, output=*)
 */
struct __pyx_opt_args_8pysndlib_3clm_outb {
  int __pyx_n;
  PyObject *output;
};

/* "clm.pxd":355
 * cpdef outa(cython.long loc, cython.double data, output=*)
 * cpdef outb(cython.long loc, cython.double data, output=*)
 * cpdef outc(cython.long loc, cython.double data, output=*)             # <<<<<<<<<<<<<<
 * cpdef outd(cython.long loc, cython.double data, output=*)
 * cpdef in_any(cython.long loc, cython.int channel, inp)
 */
struct __pyx_opt_args_8pysndlib_3clm_outc {
  int __pyx_n;
  PyObject *output;
};

/* "clm.pxd":356
 * cpdef outb(cython.long loc, cython.double data, output=*)
 * cpdef outc(cython.long loc, cython.double data, output=*)
 * cpdef outd(cython.long loc, cython.double data, output=*)             # <<<<<<<<<<<<<<
 * cpdef in_any(cython.long loc, cython.int channel, inp)
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_outd {
  int __pyx_n;
  PyObject *output;
};

/* "clm.pxd":372
 * cpdef move_locsig(mus_any gen,cython.double degree, cython.double distance)
 * 
 * cpdef convolve_files(str file1, str file2, cython.double maxamp=*, str outputfile=*, sample_type=*, header_type=*)             # <<<<<<<<<<<<<<
 * 
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_convolve_files {
  int __pyx_n;
  double maxamp;
  PyObject *outputfile;
  PyObject *sample_type;
  PyObject *header_type;
};

/* "clm.pxd":380
 * cpdef cython.double biquad(mus_any gen, cython.double insig)
 * cpdef bint is_biquad(mus_any gen)
 * cpdef biquad_set_resonance(mus_any gen, cython.double fc, cython.double radius, bint normalize=*)             # <<<<<<<<<<<<<<
 * cpdef biquad_set_equal_gain_zeroes(mus_any gen)
 * cpdef cython.double bes_j0(cython.double x)
 */
struct __pyx_opt_args_8pysndlib_3clm_biquad_set_resonance {
  int __pyx_n;
  int normalize;
};

/* "clm.pxd":390
 * 
 * 
 * cpdef cython.doublecomplex edot_product(cython.doublecomplex freq, np.ndarray data, size=*)             # <<<<<<<<<<<<<<
 * 
 * #cpdef mus_any make_oscil(cython.double frequency=*,cython.double  initial_phase = *)
 */
struct __pyx_opt_args_8pysndlib_3clm_edot_product {
  int __pyx_n;
  PyObject *size;
};
struct __pyx_opt_args_8pysndlib_11instruments_9lbj_piano_lbj_piano;

/* "pysndlib/instruments/lbj_piano.py":121
 *     return [0,0,(attackTime/4), 1.0, attackTime, 1.0, 100, releaseAmp]
 * 
 * @cython.ccall             # <<<<<<<<<<<<<<
 * def lbj_piano(startime, duration, frequency, amplitude, pfreq=None, degree=45, reverb_amount=0, distance=1):
 *     piano_attack_duration = .04
 */
struct __pyx_opt_args_8pysndlib_11instruments_9lbj_piano_lbj_piano {
  int __pyx_n;
  PyObject *pfreq;
  PyObject *degree;
  PyObject *reverb_amount;
  PyObject *distance;
};

/* "clm.pxd":40
 * 
 * 
 * cdef class mus_any:             # <<<<<<<<<<<<<<
 *     """
 *     a wrapper class for mus_any pointers in c
 */
struct __pyx_obj_8pysndlib_3clm_mus_any {
  PyObject_HEAD
  struct __pyx_vtabstruct_8pysndlib_3clm_mus_any *__pyx_vtab;
  mus_any *_ptr;
  int ptr_owner;
  input_cb _inputcallback;
  edit_cb _editcallback;
  analyze_cb _analyzecallback;
  synthesize_cb _synthesizecallback;
  PyObject *_cache;
  mus_float_t *_data_ptr;
  PyArrayObject *_data;
  mus_float_t *_xcoeffs_ptr;
  PyArrayObject *_xcoeffs;
  mus_float_t *_ycoeffs_ptr;
  PyArrayObject *_ycoeffs;
  mus_float_t *_pv_amp_increments_ptr;
  PyArrayObject *_pv_amp_increments;
  mus_float_t *_pv_amps_ptr;
  PyArrayObject *_pv_amps;
  mus_float_t *_pv_freqs_ptr;
  PyArrayObject *_pv_freqs;
  mus_float_t *_pv_phases_ptr;
  PyArrayObject *_pv_phases;
  mus_float_t *_pv_phase_increments_ptr;
  PyArrayObject *_pv_phase_increments;
};



struct __pyx_vtabstruct_8pysndlib_3clm_mus_any {
  struct __pyx_obj_8pysndlib_3clm_mus_any *(*from_ptr)(mus_any *, struct __pyx_opt_args_8pysndlib_3clm_7mus_any_from_ptr *__pyx_optional_args);
  PyObject *(*cache_append)(struct __pyx_obj_8pysndlib_3clm_mus_any *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*cache_extend)(struct __pyx_obj_8pysndlib_3clm_mus_any *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*set_up_data)(struct __pyx_obj_8pysndlib_3clm_mus_any *, int __pyx_skip_dispatch, struct __pyx_opt_args_8pysndlib_3clm_7mus_any_set_up_data *__pyx_optional_args);
  PyObject *(*set_up_xcoeffs)(struct __pyx_obj_8pysndlib_3clm_mus_any *, int __pyx_skip_dispatch);
  PyObject *(*set_up_ycoeffs)(struct __pyx_obj_8pysndlib_3clm_mus_any *, int __pyx_skip_dispatch);
  PyObject *(*set_up_pv_data)(struct __pyx_obj_8pysndlib_3clm_mus_any *, int __pyx_skip_dispatch);
  PyObject *(*mus_reset)(struct __pyx_obj_8pysndlib_3clm_mus_any *, int __pyx_skip_dispatch);
  PyObject *(*next)(struct __pyx_obj_8pysndlib_3clm_mus_any *, int __pyx_skip_dispatch, struct __pyx_opt_args_8pysndlib_3clm_7mus_any_next *__pyx_optional_args);
};
static struct __pyx_vtabstruct_8pysndlib_3clm_mus_any *__pyx_vtabptr_8pysndlib_3clm_mus_any;
/* #### Code section: utility_code_proto ### */

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, Py_ssize_t);
    void (*DECREF)(void*, PyObject*, Py_ssize_t);
    void (*GOTREF)(void*, PyObject*, Py_ssize_t);
    void (*GIVEREF)(void*, PyObject*, Py_ssize_t);
    void* (*SetupContext)(const char*, Py_ssize_t, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
          }
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__))
  #define __Pyx_RefNannyFinishContextNogil() __Pyx_RefNannyFinishContext()
#endif
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_XINCREF(r)  do { if((r) == NULL); else {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) == NULL); else {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) == NULL); else {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) == NULL); else {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContextNogil()
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_Py_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; Py_XDECREF(tmp);\
    } while (0)
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#if PY_VERSION_HEX >= 0x030C00A6
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->current_exception != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->current_exception ? (PyObject*) Py_TYPE(__pyx_tstate->current_exception) : (PyObject*) NULL)
#else
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->curexc_type != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->curexc_type)
#endif
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  (PyErr_Occurred() != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A6
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK && CYTHON_FAST_THREAD_STATE
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* TupleAndListFromArray.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n);
static CYTHON_INLINE PyObject* __Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n);
#endif

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* fastcall.proto */
#if CYTHON_AVOID_BORROWED_REFS
    #define __Pyx_Arg_VARARGS(args, i) PySequence_GetItem(args, i)
#elif CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_Arg_VARARGS(args, i) PyTuple_GET_ITEM(args, i)
#else
    #define __Pyx_Arg_VARARGS(args, i) PyTuple_GetItem(args, i)
#endif
#if CYTHON_AVOID_BORROWED_REFS
    #define __Pyx_Arg_NewRef_VARARGS(arg) __Pyx_NewRef(arg)
    #define __Pyx_Arg_XDECREF_VARARGS(arg) Py_XDECREF(arg)
#else
    #define __Pyx_Arg_NewRef_VARARGS(arg) arg
    #define __Pyx_Arg_XDECREF_VARARGS(arg)
#endif
#define __Pyx_NumKwargs_VARARGS(kwds) PyDict_Size(kwds)
#define __Pyx_KwValues_VARARGS(args, nargs) NULL
#define __Pyx_GetKwValue_VARARGS(kw, kwvalues, s) __Pyx_PyDict_GetItemStrWithError(kw, s)
#define __Pyx_KwargsAsDict_VARARGS(kw, kwvalues) PyDict_Copy(kw)
#if CYTHON_METH_FASTCALL
    #define __Pyx_Arg_FASTCALL(args, i) args[i]
    #define __Pyx_NumKwargs_FASTCALL(kwds) PyTuple_GET_SIZE(kwds)
    #define __Pyx_KwValues_FASTCALL(args, nargs) ((args) + (nargs))
    static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
    CYTHON_UNUSED static PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues);
  #else
    #define __Pyx_KwargsAsDict_FASTCALL(kw, kwvalues) _PyStack_AsDict(kwvalues, kw)
  #endif
    #define __Pyx_Arg_NewRef_FASTCALL(arg) arg  /* no-op, __Pyx_Arg_FASTCALL is direct and this needs
                                                   to have the same reference counting */
    #define __Pyx_Arg_XDECREF_FASTCALL(arg)
#else
    #define __Pyx_Arg_FASTCALL __Pyx_Arg_VARARGS
    #define __Pyx_NumKwargs_FASTCALL __Pyx_NumKwargs_VARARGS
    #define __Pyx_KwValues_FASTCALL __Pyx_KwValues_VARARGS
    #define __Pyx_GetKwValue_FASTCALL __Pyx_GetKwValue_VARARGS
    #define __Pyx_KwargsAsDict_FASTCALL __Pyx_KwargsAsDict_VARARGS
    #define __Pyx_Arg_NewRef_FASTCALL(arg) __Pyx_Arg_NewRef_VARARGS(arg)
    #define __Pyx_Arg_XDECREF_FASTCALL(arg) __Pyx_Arg_XDECREF_VARARGS(arg)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
#define __Pyx_ArgsSlice_VARARGS(args, start, stop) __Pyx_PyTuple_FromArray(&__Pyx_Arg_VARARGS(args, start), stop - start)
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) __Pyx_PyTuple_FromArray(&__Pyx_Arg_FASTCALL(args, start), stop - start)
#else
#define __Pyx_ArgsSlice_VARARGS(args, start, stop) PyTuple_GetSlice(args, start, stop)
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) PyTuple_GetSlice(args, start, stop)
#endif

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject *const *kwvalues,
    PyObject **argnames[],
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,
    const char* function_name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  do {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  do {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_MultiplyCObj(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_MultiplyCObj(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceMultiply(op1, op2) : PyNumber_Multiply(op1, op2))
#endif

/* PyFloatBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyFloat_TrueDivideObjC(PyObject *op1, PyObject *op2, double floatval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyFloat_TrueDivideObjC(op1, op2, floatval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceTrueDivide(op1, op2) : PyNumber_TrueDivide(op1, op2))
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#if !CYTHON_VECTORCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
#if !CYTHON_VECTORCALL
#if PY_VERSION_HEX >= 0x03080000
  #include "frameobject.h"
#if PY_VERSION_HEX >= 0x030b00a6 && !CYTHON_COMPILING_IN_LIMITED_API
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
  #define __Pxy_PyFrame_Initialize_Offsets()
  #define __Pyx_PyFrame_GetLocalsplus(frame)  ((frame)->f_localsplus)
#else
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif
#endif
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectFastCall.proto */
#define __Pyx_PyObject_FastCall(func, args, nargs)  __Pyx_PyObject_FastCallDict(func, args, (size_t)(nargs), NULL)
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject **args, size_t nargs, PyObject *kwargs);

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject *key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_MultiplyObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_MultiplyObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceMultiply(op1, op2) : PyNumber_Multiply(op1, op2))
#endif

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_TrueDivideObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_TrueDivideObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceTrueDivide(op1, op2) : PyNumber_TrueDivide(op1, op2))
#endif

/* DivInt[Py_ssize_t].proto */
static CYTHON_INLINE Py_ssize_t __Pyx_div_Py_ssize_t(Py_ssize_t, Py_ssize_t);

/* SetItemInt.proto */
#define __Pyx_SetItemInt(o, i, v, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_SetItemInt_Fast(o, (Py_ssize_t)i, v, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list assignment index out of range"), -1) :\
               __Pyx_SetItemInt_Generic(o, to_py_func(i), v)))
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v);
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v,
                                               int is_list, int wraparound, int boundscheck);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto_3_0_8
#define __PYX_HAVE_RT_ImportType_proto_3_0_8
#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L
#include <stdalign.h>
#endif
#if (defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L) || __cplusplus >= 201103L
#define __PYX_GET_STRUCT_ALIGNMENT_3_0_8(s) alignof(s)
#else
#define __PYX_GET_STRUCT_ALIGNMENT_3_0_8(s) sizeof(void*)
#endif
enum __Pyx_ImportType_CheckSize_3_0_8 {
   __Pyx_ImportType_CheckSize_Error_3_0_8 = 0,
   __Pyx_ImportType_CheckSize_Warn_3_0_8 = 1,
   __Pyx_ImportType_CheckSize_Ignore_3_0_8 = 2
};
static PyTypeObject *__Pyx_ImportType_3_0_8(PyObject* module, const char *module_name, const char *class_name, size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_3_0_8 check_size);
#endif

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyTypeObject *type);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportDottedModule.proto */
static PyObject *__Pyx_ImportDottedModule(PyObject *name, PyObject *parts_tuple);
#if PY_MAJOR_VERSION >= 3
static PyObject *__Pyx_ImportDottedModule_WalkParts(PyObject *module, PyObject *name, PyObject *parts_tuple);
#endif

/* IncludeStructmemberH.proto */
#include <structmember.h>

/* FixUpExtensionType.proto */
#if CYTHON_USE_TYPE_SPECS
static int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type);
#endif

/* FetchSharedCythonModule.proto */
static PyObject *__Pyx_FetchSharedCythonABIModule(void);

/* FetchCommonType.proto */
#if !CYTHON_USE_TYPE_SPECS
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);
#else
static PyTypeObject* __Pyx_FetchCommonTypeFromSpec(PyObject *module, PyType_Spec *spec, PyObject *bases);
#endif

/* PyMethodNew.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ) {
    PyObject *typesModule=NULL, *methodType=NULL, *result=NULL;
    CYTHON_UNUSED_VAR(typ);
    if (!self)
        return __Pyx_NewRef(func);
    typesModule = PyImport_ImportModule("types");
    if (!typesModule) return NULL;
    methodType = PyObject_GetAttrString(typesModule, "MethodType");
    Py_DECREF(typesModule);
    if (!methodType) return NULL;
    result = PyObject_CallFunctionObjArgs(methodType, func, self, NULL);
    Py_DECREF(methodType);
    return result;
}
#elif PY_MAJOR_VERSION >= 3
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ) {
    CYTHON_UNUSED_VAR(typ);
    if (!self)
        return __Pyx_NewRef(func);
    return PyMethod_New(func, self);
}
#else
    #define __Pyx_PyMethod_New PyMethod_New
#endif

/* PyVectorcallFastCallDict.proto */
#if CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject *__Pyx_PyVectorcall_FastCallDict(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw);
#endif

/* CythonFunctionShared.proto */
#define __Pyx_CyFunction_USED
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CYFUNCTION_COROUTINE     0x08
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_CyFunction_GetClassObj(f)\
      (((__pyx_CyFunctionObject *) (f))->func_classobj)
#else
  #define __Pyx_CyFunction_GetClassObj(f)\
      ((PyObject*) ((PyCMethodObject *) (f))->mm_class)
#endif
#define __Pyx_CyFunction_SetClassObj(f, classobj)\
    __Pyx__CyFunction_SetClassObj((__pyx_CyFunctionObject *) (f), (classobj))
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject_HEAD
    PyObject *func;
#elif PY_VERSION_HEX < 0x030900B1
    PyCFunctionObject func;
#else
    PyCMethodObject func;
#endif
#if CYTHON_BACKPORT_VECTORCALL
    __pyx_vectorcallfunc func_vectorcall;
#endif
#if PY_VERSION_HEX < 0x030500A0 || CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_classobj;
#endif
    void *defaults;
    int defaults_pyobjects;
    size_t defaults_size;
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
    PyObject *func_is_coroutine;
} __pyx_CyFunctionObject;
#undef __Pyx_CyOrPyCFunction_Check
#define __Pyx_CyFunction_Check(obj)  __Pyx_TypeCheck(obj, __pyx_CyFunctionType)
#define __Pyx_CyOrPyCFunction_Check(obj)  __Pyx_TypeCheck2(obj, __pyx_CyFunctionType, &PyCFunction_Type)
#define __Pyx_CyFunction_CheckExact(obj)  __Pyx_IS_TYPE(obj, __pyx_CyFunctionType)
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void *cfunc);
#undef __Pyx_IsSameCFunction
#define __Pyx_IsSameCFunction(func, cfunc)   __Pyx__IsSameCyOrCFunction(func, cfunc)
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject* op, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void __Pyx__CyFunction_SetClassObj(__pyx_CyFunctionObject* f, PyObject* classobj);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(PyObject *module);
#if CYTHON_METH_FASTCALL
static PyObject * __Pyx_CyFunction_Vectorcall_NOARGS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_O(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
#if CYTHON_BACKPORT_VECTORCALL
#define __Pyx_CyFunction_func_vectorcall(f) (((__pyx_CyFunctionObject*)f)->func_vectorcall)
#else
#define __Pyx_CyFunction_func_vectorcall(f) (((PyCFunctionObject*)f)->vectorcall)
#endif
#endif

/* CythonFunction.proto */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);

/* PyObjectLookupSpecial.proto */
#if CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_LookupSpecialNoError(obj, attr_name)  __Pyx__PyObject_LookupSpecial(obj, attr_name, 0)
#define __Pyx_PyObject_LookupSpecial(obj, attr_name)  __Pyx__PyObject_LookupSpecial(obj, attr_name, 1)
static CYTHON_INLINE PyObject* __Pyx__PyObject_LookupSpecial(PyObject* obj, PyObject* attr_name, int with_error);
#else
#define __Pyx_PyObject_LookupSpecialNoError(o,n) __Pyx_PyObject_GetAttrStrNoError(o,n)
#define __Pyx_PyObject_LookupSpecial(o,n) __Pyx_PyObject_GetAttrStr(o,n)
#endif

/* PyObjectCallNoArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
#if !CYTHON_COMPILING_IN_LIMITED_API
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);
#endif

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* GCCDiagnostics.proto */
#if !defined(__INTEL_COMPILER) && defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* RealImag.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #define __Pyx_CREAL(z) ((z).real())
    #define __Pyx_CIMAG(z) ((z).imag())
  #else
    #define __Pyx_CREAL(z) (__real__(z))
    #define __Pyx_CIMAG(z) (__imag__(z))
  #endif
#else
    #define __Pyx_CREAL(z) ((z).real)
    #define __Pyx_CIMAG(z) ((z).imag)
#endif
#if defined(__cplusplus) && CYTHON_CCOMPLEX\
        && (defined(_WIN32) || defined(__clang__) || (defined(__GNUC__) && (__GNUC__ >= 5 || __GNUC__ == 4 && __GNUC_MINOR__ >= 4 )) || __cplusplus >= 201103)
    #define __Pyx_SET_CREAL(z,x) ((z).real(x))
    #define __Pyx_SET_CIMAG(z,y) ((z).imag(y))
#else
    #define __Pyx_SET_CREAL(z,x) __Pyx_CREAL(z) = (x)
    #define __Pyx_SET_CIMAG(z,y) __Pyx_CIMAG(z) = (y)
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
    #define __Pyx_c_eq_float(a, b)   ((a)==(b))
    #define __Pyx_c_sum_float(a, b)  ((a)+(b))
    #define __Pyx_c_diff_float(a, b) ((a)-(b))
    #define __Pyx_c_prod_float(a, b) ((a)*(b))
    #define __Pyx_c_quot_float(a, b) ((a)/(b))
    #define __Pyx_c_neg_float(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_float(z) ((z)==(float)0)
    #define __Pyx_c_conj_float(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (::std::abs(z))
        #define __Pyx_c_pow_float(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_float(z) ((z)==0)
    #define __Pyx_c_conj_float(z)    (conjf(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (cabsf(z))
        #define __Pyx_c_pow_float(a, b)  (cpowf(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex);
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex);
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex, __pyx_t_float_complex);
    #endif
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
    #define __Pyx_c_eq_double(a, b)   ((a)==(b))
    #define __Pyx_c_sum_double(a, b)  ((a)+(b))
    #define __Pyx_c_diff_double(a, b) ((a)-(b))
    #define __Pyx_c_prod_double(a, b) ((a)*(b))
    #define __Pyx_c_quot_double(a, b) ((a)/(b))
    #define __Pyx_c_neg_double(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_double(z) ((z)==(double)0)
    #define __Pyx_c_conj_double(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (::std::abs(z))
        #define __Pyx_c_pow_double(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_double(z) ((z)==0)
    #define __Pyx_c_conj_double(z)    (conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (cabs(z))
        #define __Pyx_c_pow_double(a, b)  (cpow(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex);
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex);
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex, __pyx_t_double_complex);
    #endif
#endif

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* FormatTypeName.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
typedef PyObject *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%U"
static __Pyx_TypeName __Pyx_PyType_GetName(PyTypeObject* tp);
#define __Pyx_DECREF_TypeName(obj) Py_XDECREF(obj)
#else
typedef const char *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%.200s"
#define __Pyx_PyType_GetName(tp) ((tp)->tp_name)
#define __Pyx_DECREF_TypeName(obj)
#endif

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) __Pyx_IsAnySubtype2(Py_TYPE(obj), (PyTypeObject *)type1, (PyTypeObject *)type2)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) (PyObject_TypeCheck(obj, (PyTypeObject *)type1) || PyObject_TypeCheck(obj, (PyTypeObject *)type2))
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyErr_ExceptionMatches2(err1, err2)  __Pyx_PyErr_GivenExceptionMatches2(__Pyx_PyErr_CurrentExceptionType(), err1, err2)
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static unsigned long __Pyx_get_runtime_version(void);
static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer);

/* FunctionImport.proto */
static int __Pyx_ImportFunction_3_0_8(PyObject *module, const char *funcname, void (**f)(void), const char *sig);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

/* #### Code section: module_declarations ### */
static CYTHON_INLINE PyObject *__pyx_f_5numpy_7ndarray_4base_base(PyArrayObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE PyArray_Descr *__pyx_f_5numpy_7ndarray_5descr_descr(PyArrayObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE int __pyx_f_5numpy_7ndarray_4ndim_ndim(PyArrayObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE npy_intp *__pyx_f_5numpy_7ndarray_5shape_shape(PyArrayObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE npy_intp *__pyx_f_5numpy_7ndarray_7strides_strides(PyArrayObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE npy_intp __pyx_f_5numpy_7ndarray_4size_size(PyArrayObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE char *__pyx_f_5numpy_7ndarray_4data_data(PyArrayObject *__pyx_v_self); /* proto*/

/* Module declarations from "cython" */

/* Module declarations from "pysndlib.csndlib" */

/* Module declarations from "pysndlib.cclm" */

/* Module declarations from "pysndlib" */

/* Module declarations from "libc.string" */

/* Module declarations from "libc.stdio" */

/* Module declarations from "__builtin__" */

/* Module declarations from "cpython.type" */

/* Module declarations from "cpython" */

/* Module declarations from "cpython.object" */

/* Module declarations from "cpython.ref" */

/* Module declarations from "numpy" */

/* Module declarations from "numpy" */
static CYTHON_INLINE int __pyx_f_5numpy_import_array(void); /*proto*/

/* Module declarations from "pysndlib.sndlib" */

/* Module declarations from "pysndlib.clm" */
static double (*__pyx_f_8pysndlib_3clm_hz2radians)(double, int __pyx_skip_dispatch); /*proto*/
static double (*__pyx_f_8pysndlib_3clm_db2linear)(double, int __pyx_skip_dispatch); /*proto*/
static long (*__pyx_f_8pysndlib_3clm_seconds2samples)(double, int __pyx_skip_dispatch); /*proto*/
static PyArrayObject *(*__pyx_f_8pysndlib_3clm_normalize_partials)(PyObject *, int __pyx_skip_dispatch); /*proto*/
static double (*__pyx_f_8pysndlib_3clm_oscil_bank)(struct __pyx_obj_8pysndlib_3clm_mus_any *, int __pyx_skip_dispatch); /*proto*/
static double (*__pyx_f_8pysndlib_3clm_env)(struct __pyx_obj_8pysndlib_3clm_mus_any *, int __pyx_skip_dispatch); /*proto*/
static double (*__pyx_f_8pysndlib_3clm_locsig)(struct __pyx_obj_8pysndlib_3clm_mus_any *, long, double, int __pyx_skip_dispatch); /*proto*/

/* Module declarations from "pysndlib.instruments.lbj_piano" */
static PyObject *__pyx_f_8pysndlib_11instruments_9lbj_piano_lbj_piano(PyObject *, PyObject *, PyObject *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_8pysndlib_11instruments_9lbj_piano_lbj_piano *__pyx_optional_args); /*proto*/
/* #### Code section: typeinfo ### */
/* #### Code section: before_global_var ### */
#define __Pyx_MODULE_NAME "pysndlib.instruments.lbj_piano"
extern int __pyx_module_is_main_pysndlib__instruments__lbj_piano;
int __pyx_module_is_main_pysndlib__instruments__lbj_piano = 0;

/* Implementation of "pysndlib.instruments.lbj_piano" */
/* #### Code section: global_var ### */
static PyObject *__pyx_builtin_round;
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_ImportError;
/* #### Code section: string_decls ### */
static const char __pyx_k__5[] = "*";
static const char __pyx_k_np[] = "np";
static const char __pyx_k__15[] = "?";
static const char __pyx_k_clm[] = "clm";
static const char __pyx_k_dur[] = "dur";
static const char __pyx_k_log[] = "log";
static const char __pyx_k_base[] = "base";
static const char __pyx_k_exit[] = "__exit__";
static const char __pyx_k_freq[] = "freq";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_math[] = "math";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_play[] = "play";
static const char __pyx_k_spec[] = "__spec__";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_Sound[] = "Sound";
static const char __pyx_k_dtype[] = "dtype";
static const char __pyx_k_enter[] = "__enter__";
static const char __pyx_k_numpy[] = "numpy";
static const char __pyx_k_pfreq[] = "pfreq";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_round[] = "round";
static const char __pyx_k_zeros[] = "zeros";
static const char __pyx_k_degree[] = "degree";
static const char __pyx_k_double[] = "double";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_random[] = "random";
static const char __pyx_k_scaler[] = "scaler";
static const char __pyx_k_stable[] = "stable";
static const char __pyx_k_distance[] = "distance";
static const char __pyx_k_duration[] = "duration";
static const char __pyx_k_make_env[] = "make_env";
static const char __pyx_k_pysndlib[] = "pysndlib";
static const char __pyx_k_startime[] = "startime";
static const char __pyx_k_amplitude[] = "amplitude";
static const char __pyx_k_frequency[] = "frequency";
static const char __pyx_k_is_number[] = "is_number";
static const char __pyx_k_lbj_piano[] = "lbj_piano";
static const char __pyx_k_attackTime[] = "attackTime";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_releaseAmp[] = "releaseAmp";
static const char __pyx_k_statistics[] = "statistics";
static const char __pyx_k_ImportError[] = "ImportError";
static const char __pyx_k_make_locsig[] = "make_locsig";
static const char __pyx_k_initializing[] = "_initializing";
static const char __pyx_k_is_coroutine[] = "_is_coroutine";
static const char __pyx_k_pysndlib_clm[] = "pysndlib.clm";
static const char __pyx_k_PIANO_SPECTRA[] = "PIANO_SPECTRA";
static const char __pyx_k_class_getitem[] = "__class_getitem__";
static const char __pyx_k_reverb_amount[] = "reverb_amount";
static const char __pyx_k_make_oscil_bank[] = "make_oscil_bank";
static const char __pyx_k_make_piano_ampfun[] = "make_piano_ampfun";
static const char __pyx_k_asyncio_coroutines[] = "asyncio.coroutines";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_db_drop_per_second[] = "db_drop_per_second";
static const char __pyx_k_get_piano_partials[] = "get_piano_partials";
static const char __pyx_k_piano_attack_duration[] = "piano_attack_duration";
static const char __pyx_k_pysndlib_instruments_lbj_piano[] = "pysndlib.instruments.lbj_piano";
static const char __pyx_k_numpy_core_multiarray_failed_to[] = "numpy.core.multiarray failed to import";
static const char __pyx_k_numpy_core_umath_failed_to_impor[] = "numpy.core.umath failed to import";
static const char __pyx_k_src_pysndlib_instruments_lbj_pia[] = "src/pysndlib/instruments/lbj_piano.py";
/* #### Code section: decls ### */
static PyObject *__pyx_pf_8pysndlib_11instruments_9lbj_piano_get_piano_partials(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_freq); /* proto */
static PyObject *__pyx_pf_8pysndlib_11instruments_9lbj_piano_2make_piano_ampfun(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_dur, PyObject *__pyx_v_piano_attack_duration); /* proto */
static PyObject *__pyx_pf_8pysndlib_11instruments_9lbj_piano_4lbj_piano(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_startime, PyObject *__pyx_v_duration, PyObject *__pyx_v_frequency, PyObject *__pyx_v_amplitude, PyObject *__pyx_v_pfreq, PyObject *__pyx_v_degree, PyObject *__pyx_v_reverb_amount, PyObject *__pyx_v_distance); /* proto */
/* #### Code section: late_includes ### */
/* #### Code section: module_state ### */
typedef struct {
  PyObject *__pyx_d;
  PyObject *__pyx_b;
  PyObject *__pyx_cython_runtime;
  PyObject *__pyx_empty_tuple;
  PyObject *__pyx_empty_bytes;
  PyObject *__pyx_empty_unicode;
  #ifdef __Pyx_CyFunction_USED
  PyTypeObject *__pyx_CyFunctionType;
  #endif
  #ifdef __Pyx_FusedFunction_USED
  PyTypeObject *__pyx_FusedFunctionType;
  #endif
  #ifdef __Pyx_Generator_USED
  PyTypeObject *__pyx_GeneratorType;
  #endif
  #ifdef __Pyx_IterableCoroutine_USED
  PyTypeObject *__pyx_IterableCoroutineType;
  #endif
  #ifdef __Pyx_Coroutine_USED
  PyTypeObject *__pyx_CoroutineAwaitType;
  #endif
  #ifdef __Pyx_Coroutine_USED
  PyTypeObject *__pyx_CoroutineType;
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  PyTypeObject *__pyx_ptype_7cpython_4type_type;
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  PyTypeObject *__pyx_ptype_5numpy_dtype;
  PyTypeObject *__pyx_ptype_5numpy_flatiter;
  PyTypeObject *__pyx_ptype_5numpy_broadcast;
  PyTypeObject *__pyx_ptype_5numpy_ndarray;
  PyTypeObject *__pyx_ptype_5numpy_generic;
  PyTypeObject *__pyx_ptype_5numpy_number;
  PyTypeObject *__pyx_ptype_5numpy_integer;
  PyTypeObject *__pyx_ptype_5numpy_signedinteger;
  PyTypeObject *__pyx_ptype_5numpy_unsignedinteger;
  PyTypeObject *__pyx_ptype_5numpy_inexact;
  PyTypeObject *__pyx_ptype_5numpy_floating;
  PyTypeObject *__pyx_ptype_5numpy_complexfloating;
  PyTypeObject *__pyx_ptype_5numpy_flexible;
  PyTypeObject *__pyx_ptype_5numpy_character;
  PyTypeObject *__pyx_ptype_5numpy_ufunc;
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  PyTypeObject *__pyx_ptype_8pysndlib_3clm_mus_any;
  #if CYTHON_USE_MODULE_STATE
  #endif
  PyObject *__pyx_n_s_ImportError;
  PyObject *__pyx_n_s_PIANO_SPECTRA;
  PyObject *__pyx_n_s_Sound;
  PyObject *__pyx_n_s__15;
  PyObject *__pyx_n_s__5;
  PyObject *__pyx_n_s_amplitude;
  PyObject *__pyx_n_s_asyncio_coroutines;
  PyObject *__pyx_n_s_attackTime;
  PyObject *__pyx_n_s_base;
  PyObject *__pyx_n_s_class_getitem;
  PyObject *__pyx_n_s_cline_in_traceback;
  PyObject *__pyx_n_s_clm;
  PyObject *__pyx_n_s_db_drop_per_second;
  PyObject *__pyx_n_s_degree;
  PyObject *__pyx_n_s_distance;
  PyObject *__pyx_n_s_double;
  PyObject *__pyx_n_s_dtype;
  PyObject *__pyx_n_s_dur;
  PyObject *__pyx_n_s_duration;
  PyObject *__pyx_n_s_enter;
  PyObject *__pyx_n_s_exit;
  PyObject *__pyx_n_s_freq;
  PyObject *__pyx_n_s_frequency;
  PyObject *__pyx_n_s_get_piano_partials;
  PyObject *__pyx_n_s_import;
  PyObject *__pyx_n_s_initializing;
  PyObject *__pyx_n_s_is_coroutine;
  PyObject *__pyx_n_s_is_number;
  PyObject *__pyx_n_s_lbj_piano;
  PyObject *__pyx_n_s_log;
  PyObject *__pyx_n_s_main;
  PyObject *__pyx_n_u_main;
  PyObject *__pyx_n_s_make_env;
  PyObject *__pyx_n_s_make_locsig;
  PyObject *__pyx_n_s_make_oscil_bank;
  PyObject *__pyx_n_s_make_piano_ampfun;
  PyObject *__pyx_n_s_math;
  PyObject *__pyx_n_s_name;
  PyObject *__pyx_n_s_np;
  PyObject *__pyx_n_s_numpy;
  PyObject *__pyx_kp_u_numpy_core_multiarray_failed_to;
  PyObject *__pyx_kp_u_numpy_core_umath_failed_to_impor;
  PyObject *__pyx_n_s_pfreq;
  PyObject *__pyx_n_s_piano_attack_duration;
  PyObject *__pyx_n_s_play;
  PyObject *__pyx_n_s_pysndlib;
  PyObject *__pyx_n_s_pysndlib_clm;
  PyObject *__pyx_n_s_pysndlib_instruments_lbj_piano;
  PyObject *__pyx_n_s_pyx_vtable;
  PyObject *__pyx_n_s_random;
  PyObject *__pyx_n_s_range;
  PyObject *__pyx_n_s_releaseAmp;
  PyObject *__pyx_n_s_reverb_amount;
  PyObject *__pyx_n_s_round;
  PyObject *__pyx_n_s_scaler;
  PyObject *__pyx_n_s_spec;
  PyObject *__pyx_kp_s_src_pysndlib_instruments_lbj_pia;
  PyObject *__pyx_n_s_stable;
  PyObject *__pyx_n_s_startime;
  PyObject *__pyx_n_s_statistics;
  PyObject *__pyx_n_s_test;
  PyObject *__pyx_n_s_zeros;
  PyObject *__pyx_float__4;
  PyObject *__pyx_float__34;
  PyObject *__pyx_float_1_0;
  PyObject *__pyx_float_0_97;
  PyObject *__pyx_float_0_98;
  PyObject *__pyx_float_0_99;
  PyObject *__pyx_float_1_00;
  PyObject *__pyx_float_1_01;
  PyObject *__pyx_float_1_02;
  PyObject *__pyx_float_1_97;
  PyObject *__pyx_float_1_98;
  PyObject *__pyx_float_1_99;
  PyObject *__pyx_float_2_00;
  PyObject *__pyx_float_2_01;
  PyObject *__pyx_float_2_02;
  PyObject *__pyx_float_2_03;
  PyObject *__pyx_float_2_07;
  PyObject *__pyx_float_2_09;
  PyObject *__pyx_float_2_96;
  PyObject *__pyx_float_2_97;
  PyObject *__pyx_float_2_98;
  PyObject *__pyx_float_2_99;
  PyObject *__pyx_float_3_00;
  PyObject *__pyx_float_3_01;
  PyObject *__pyx_float_3_02;
  PyObject *__pyx_float_3_03;
  PyObject *__pyx_float_3_04;
  PyObject *__pyx_float_3_09;
  PyObject *__pyx_float_3_11;
  PyObject *__pyx_float_3_12;
  PyObject *__pyx_float_3_15;
  PyObject *__pyx_float_3_19;
  PyObject *__pyx_float_3_95;
  PyObject *__pyx_float_3_97;
  PyObject *__pyx_float_3_98;
  PyObject *__pyx_float_3_99;
  PyObject *__pyx_float_4_00;
  PyObject *__pyx_float_4_01;
  PyObject *__pyx_float_4_02;
  PyObject *__pyx_float_4_03;
  PyObject *__pyx_float_4_04;
  PyObject *__pyx_float_4_05;
  PyObject *__pyx_float_4_06;
  PyObject *__pyx_float_4_10;
  PyObject *__pyx_float_4_12;
  PyObject *__pyx_float_4_13;
  PyObject *__pyx_float_4_17;
  PyObject *__pyx_float_4_19;
  PyObject *__pyx_float_4_26;
  PyObject *__pyx_float_4_94;
  PyObject *__pyx_float_4_95;
  PyObject *__pyx_float_4_96;
  PyObject *__pyx_float_4_97;
  PyObject *__pyx_float_4_98;
  PyObject *__pyx_float_4_99;
  PyObject *__pyx_float_5_00;
  PyObject *__pyx_float_5_01;
  PyObject *__pyx_float_5_02;
  PyObject *__pyx_float_5_03;
  PyObject *__pyx_float_5_04;
  PyObject *__pyx_float_5_05;
  PyObject *__pyx_float_5_07;
  PyObject *__pyx_float_5_09;
  PyObject *__pyx_float_5_10;
  PyObject *__pyx_float_5_11;
  PyObject *__pyx_float_5_12;
  PyObject *__pyx_float_5_14;
  PyObject *__pyx_float_5_15;
  PyObject *__pyx_float_5_19;
  PyObject *__pyx_float_5_20;
  PyObject *__pyx_float_5_22;
  PyObject *__pyx_float_5_95;
  PyObject *__pyx_float_5_96;
  PyObject *__pyx_float_5_97;
  PyObject *__pyx_float_5_98;
  PyObject *__pyx_float_5_99;
  PyObject *__pyx_float_6_00;
  PyObject *__pyx_float_6_01;
  PyObject *__pyx_float_6_02;
  PyObject *__pyx_float_6_03;
  PyObject *__pyx_float_6_04;
  PyObject *__pyx_float_6_06;
  PyObject *__pyx_float_6_07;
  PyObject *__pyx_float_6_08;
  PyObject *__pyx_float_6_09;
  PyObject *__pyx_float_6_12;
  PyObject *__pyx_float_6_13;
  PyObject *__pyx_float_6_14;
  PyObject *__pyx_float_6_16;
  PyObject *__pyx_float_6_17;
  PyObject *__pyx_float_6_20;
  PyObject *__pyx_float_6_22;
  PyObject *__pyx_float_6_26;
  PyObject *__pyx_float_6_27;
  PyObject *__pyx_float_6_32;
  PyObject *__pyx_float_6_35;
  PyObject *__pyx_float_6_94;
  PyObject *__pyx_float_6_95;
  PyObject *__pyx_float_6_96;
  PyObject *__pyx_float_6_97;
  PyObject *__pyx_float_6_98;
  PyObject *__pyx_float_6_99;
  PyObject *__pyx_float_7_00;
  PyObject *__pyx_float_7_01;
  PyObject *__pyx_float_7_03;
  PyObject *__pyx_float_7_04;
  PyObject *__pyx_float_7_05;
  PyObject *__pyx_float_7_06;
  PyObject *__pyx_float_7_07;
  PyObject *__pyx_float_7_09;
  PyObject *__pyx_float_7_11;
  PyObject *__pyx_float_7_12;
  PyObject *__pyx_float_7_13;
  PyObject *__pyx_float_7_15;
  PyObject *__pyx_float_7_19;
  PyObject *__pyx_float_7_22;
  PyObject *__pyx_float_7_23;
  PyObject *__pyx_float_7_25;
  PyObject *__pyx_float_7_26;
  PyObject *__pyx_float_7_27;
  PyObject *__pyx_float_7_32;
  PyObject *__pyx_float_7_33;
  PyObject *__pyx_float_7_35;
  PyObject *__pyx_float_7_41;
  PyObject *__pyx_float_7_59;
  PyObject *__pyx_float_7_93;
  PyObject *__pyx_float_7_94;
  PyObject *__pyx_float_7_96;
  PyObject *__pyx_float_7_97;
  PyObject *__pyx_float_7_98;
  PyObject *__pyx_float_7_99;
  PyObject *__pyx_float_8_00;
  PyObject *__pyx_float_8_01;
  PyObject *__pyx_float_8_02;
  PyObject *__pyx_float_8_04;
  PyObject *__pyx_float_8_05;
  PyObject *__pyx_float_8_06;
  PyObject *__pyx_float_8_08;
  PyObject *__pyx_float_8_09;
  PyObject *__pyx_float_8_11;
  PyObject *__pyx_float_8_12;
  PyObject *__pyx_float_8_14;
  PyObject *__pyx_float_8_17;
  PyObject *__pyx_float_8_19;
  PyObject *__pyx_float_8_20;
  PyObject *__pyx_float_8_22;
  PyObject *__pyx_float_8_29;
  PyObject *__pyx_float_8_30;
  PyObject *__pyx_float_8_32;
  PyObject *__pyx_float_8_33;
  PyObject *__pyx_float_8_36;
  PyObject *__pyx_float_8_39;
  PyObject *__pyx_float_8_40;
  PyObject *__pyx_float_8_46;
  PyObject *__pyx_float_8_50;
  PyObject *__pyx_float_8_63;
  PyObject *__pyx_float_8_94;
  PyObject *__pyx_float_8_95;
  PyObject *__pyx_float_8_96;
  PyObject *__pyx_float_8_97;
  PyObject *__pyx_float_8_98;
  PyObject *__pyx_float_8_99;
  PyObject *__pyx_float_9_00;
  PyObject *__pyx_float_9_01;
  PyObject *__pyx_float_9_04;
  PyObject *__pyx_float_9_07;
  PyObject *__pyx_float_9_08;
  PyObject *__pyx_float_9_10;
  PyObject *__pyx_float_9_11;
  PyObject *__pyx_float_9_13;
  PyObject *__pyx_float_9_15;
  PyObject *__pyx_float_9_16;
  PyObject *__pyx_float_9_19;
  PyObject *__pyx_float_9_21;
  PyObject *__pyx_float_9_23;
  PyObject *__pyx_float_9_25;
  PyObject *__pyx_float_9_27;
  PyObject *__pyx_float_9_29;
  PyObject *__pyx_float_9_32;
  PyObject *__pyx_float_9_43;
  PyObject *__pyx_float_9_50;
  PyObject *__pyx_float_9_51;
  PyObject *__pyx_float_9_53;
  PyObject *__pyx_float_9_58;
  PyObject *__pyx_float_9_66;
  PyObject *__pyx_float_9_92;
  PyObject *__pyx_float_9_95;
  PyObject *__pyx_float_9_96;
  PyObject *__pyx_float_9_98;
  PyObject *__pyx_float_9_99;
  PyObject *__pyx_float__0001;
  PyObject *__pyx_float__0002;
  PyObject *__pyx_float__0003;
  PyObject *__pyx_float__0004;
  PyObject *__pyx_float__0005;
  PyObject *__pyx_float__0006;
  PyObject *__pyx_float__0007;
  PyObject *__pyx_float__0008;
  PyObject *__pyx_float__0009;
  PyObject *__pyx_float__0010;
  PyObject *__pyx_float__0011;
  PyObject *__pyx_float__0012;
  PyObject *__pyx_float__0013;
  PyObject *__pyx_float__0014;
  PyObject *__pyx_float__0015;
  PyObject *__pyx_float__0016;
  PyObject *__pyx_float__0017;
  PyObject *__pyx_float__0018;
  PyObject *__pyx_float__0019;
  PyObject *__pyx_float__0020;
  PyObject *__pyx_float__0021;
  PyObject *__pyx_float__0022;
  PyObject *__pyx_float__0023;
  PyObject *__pyx_float__0024;
  PyObject *__pyx_float__0025;
  PyObject *__pyx_float__0026;
  PyObject *__pyx_float__0027;
  PyObject *__pyx_float__0028;
  PyObject *__pyx_float__0029;
  PyObject *__pyx_float__0030;
  PyObject *__pyx_float__0031;
  PyObject *__pyx_float__0032;
  PyObject *__pyx_float__0033;
  PyObject *__pyx_float__0034;
  PyObject *__pyx_float__0035;
  PyObject *__pyx_float__0036;
  PyObject *__pyx_float__0037;
  PyObject *__pyx_float__0038;
  PyObject *__pyx_float__0039;
  PyObject *__pyx_float__0040;
  PyObject *__pyx_float__0041;
  PyObject *__pyx_float__0042;
  PyObject *__pyx_float__0043;
  PyObject *__pyx_float__0044;
  PyObject *__pyx_float__0045;
  PyObject *__pyx_float__0046;
  PyObject *__pyx_float__0047;
  PyObject *__pyx_float__0048;
  PyObject *__pyx_float__0049;
  PyObject *__pyx_float__0050;
  PyObject *__pyx_float__0051;
  PyObject *__pyx_float__0052;
  PyObject *__pyx_float__0053;
  PyObject *__pyx_float__0054;
  PyObject *__pyx_float__0055;
  PyObject *__pyx_float__0056;
  PyObject *__pyx_float__0057;
  PyObject *__pyx_float__0058;
  PyObject *__pyx_float__0059;
  PyObject *__pyx_float__0060;
  PyObject *__pyx_float__0061;
  PyObject *__pyx_float__0062;
  PyObject *__pyx_float__0063;
  PyObject *__pyx_float__0064;
  PyObject *__pyx_float__0065;
  PyObject *__pyx_float__0066;
  PyObject *__pyx_float__0067;
  PyObject *__pyx_float__0068;
  PyObject *__pyx_float__0069;
  PyObject *__pyx_float__0070;
  PyObject *__pyx_float__0071;
  PyObject *__pyx_float__0072;
  PyObject *__pyx_float__0073;
  PyObject *__pyx_float__0074;
  PyObject *__pyx_float__0075;
  PyObject *__pyx_float__0076;
  PyObject *__pyx_float__0077;
  PyObject *__pyx_float__0078;
  PyObject *__pyx_float__0080;
  PyObject *__pyx_float__0081;
  PyObject *__pyx_float__0082;
  PyObject *__pyx_float__0083;
  PyObject *__pyx_float__0084;
  PyObject *__pyx_float__0085;
  PyObject *__pyx_float__0086;
  PyObject *__pyx_float__0087;
  PyObject *__pyx_float__0088;
  PyObject *__pyx_float__0089;
  PyObject *__pyx_float__0090;
  PyObject *__pyx_float__0091;
  PyObject *__pyx_float__0092;
  PyObject *__pyx_float__0093;
  PyObject *__pyx_float__0094;
  PyObject *__pyx_float__0095;
  PyObject *__pyx_float__0097;
  PyObject *__pyx_float__0098;
  PyObject *__pyx_float__0099;
  PyObject *__pyx_float__0100;
  PyObject *__pyx_float__0101;
  PyObject *__pyx_float__0102;
  PyObject *__pyx_float__0103;
  PyObject *__pyx_float__0104;
  PyObject *__pyx_float__0105;
  PyObject *__pyx_float__0106;
  PyObject *__pyx_float__0107;
  PyObject *__pyx_float__0108;
  PyObject *__pyx_float__0109;
  PyObject *__pyx_float__0111;
  PyObject *__pyx_float__0112;
  PyObject *__pyx_float__0113;
  PyObject *__pyx_float__0114;
  PyObject *__pyx_float__0115;
  PyObject *__pyx_float__0116;
  PyObject *__pyx_float__0117;
  PyObject *__pyx_float__0118;
  PyObject *__pyx_float__0119;
  PyObject *__pyx_float__0120;
  PyObject *__pyx_float__0121;
  PyObject *__pyx_float__0122;
  PyObject *__pyx_float__0123;
  PyObject *__pyx_float__0124;
  PyObject *__pyx_float__0125;
  PyObject *__pyx_float__0126;
  PyObject *__pyx_float__0127;
  PyObject *__pyx_float__0128;
  PyObject *__pyx_float__0129;
  PyObject *__pyx_float__0132;
  PyObject *__pyx_float__0133;
  PyObject *__pyx_float__0134;
  PyObject *__pyx_float__0135;
  PyObject *__pyx_float__0136;
  PyObject *__pyx_float__0137;
  PyObject *__pyx_float__0138;
  PyObject *__pyx_float__0139;
  PyObject *__pyx_float__0140;
  PyObject *__pyx_float__0141;
  PyObject *__pyx_float__0142;
  PyObject *__pyx_float__0144;
  PyObject *__pyx_float__0145;
  PyObject *__pyx_float__0147;
  PyObject *__pyx_float__0148;
  PyObject *__pyx_float__0151;
  PyObject *__pyx_float__0152;
  PyObject *__pyx_float__0153;
  PyObject *__pyx_float__0154;
  PyObject *__pyx_float__0155;
  PyObject *__pyx_float__0156;
  PyObject *__pyx_float__0158;
  PyObject *__pyx_float__0159;
  PyObject *__pyx_float__0160;
  PyObject *__pyx_float__0161;
  PyObject *__pyx_float__0162;
  PyObject *__pyx_float__0163;
  PyObject *__pyx_float__0165;
  PyObject *__pyx_float__0166;
  PyObject *__pyx_float__0167;
  PyObject *__pyx_float__0170;
  PyObject *__pyx_float__0171;
  PyObject *__pyx_float__0172;
  PyObject *__pyx_float__0173;
  PyObject *__pyx_float__0174;
  PyObject *__pyx_float__0175;
  PyObject *__pyx_float__0177;
  PyObject *__pyx_float__0178;
  PyObject *__pyx_float__0179;
  PyObject *__pyx_float__0181;
  PyObject *__pyx_float__0182;
  PyObject *__pyx_float__0183;
  PyObject *__pyx_float__0184;
  PyObject *__pyx_float__0185;
  PyObject *__pyx_float__0186;
  PyObject *__pyx_float__0189;
  PyObject *__pyx_float__0190;
  PyObject *__pyx_float__0191;
  PyObject *__pyx_float__0192;
  PyObject *__pyx_float__0194;
  PyObject *__pyx_float__0196;
  PyObject *__pyx_float__0197;
  PyObject *__pyx_float__0201;
  PyObject *__pyx_float__0202;
  PyObject *__pyx_float__0203;
  PyObject *__pyx_float__0206;
  PyObject *__pyx_float__0207;
  PyObject *__pyx_float__0209;
  PyObject *__pyx_float__0210;
  PyObject *__pyx_float__0211;
  PyObject *__pyx_float__0212;
  PyObject *__pyx_float__0213;
  PyObject *__pyx_float__0215;
  PyObject *__pyx_float__0217;
  PyObject *__pyx_float__0218;
  PyObject *__pyx_float__0221;
  PyObject *__pyx_float__0222;
  PyObject *__pyx_float__0225;
  PyObject *__pyx_float__0227;
  PyObject *__pyx_float__0228;
  PyObject *__pyx_float__0229;
  PyObject *__pyx_float__0230;
  PyObject *__pyx_float__0232;
  PyObject *__pyx_float__0233;
  PyObject *__pyx_float__0234;
  PyObject *__pyx_float__0235;
  PyObject *__pyx_float__0236;
  PyObject *__pyx_float__0237;
  PyObject *__pyx_float__0238;
  PyObject *__pyx_float__0240;
  PyObject *__pyx_float__0241;
  PyObject *__pyx_float__0242;
  PyObject *__pyx_float__0245;
  PyObject *__pyx_float__0246;
  PyObject *__pyx_float__0247;
  PyObject *__pyx_float__0248;
  PyObject *__pyx_float__0249;
  PyObject *__pyx_float__0250;
  PyObject *__pyx_float__0252;
  PyObject *__pyx_float__0256;
  PyObject *__pyx_float__0257;
  PyObject *__pyx_float__0258;
  PyObject *__pyx_float__0259;
  PyObject *__pyx_float__0260;
  PyObject *__pyx_float__0264;
  PyObject *__pyx_float__0265;
  PyObject *__pyx_float__0266;
  PyObject *__pyx_float__0268;
  PyObject *__pyx_float__0272;
  PyObject *__pyx_float__0274;
  PyObject *__pyx_float__0276;
  PyObject *__pyx_float__0278;
  PyObject *__pyx_float__0279;
  PyObject *__pyx_float__0280;
  PyObject *__pyx_float__0284;
  PyObject *__pyx_float__0285;
  PyObject *__pyx_float__0287;
  PyObject *__pyx_float__0289;
  PyObject *__pyx_float__0290;
  PyObject *__pyx_float__0291;
  PyObject *__pyx_float__0292;
  PyObject *__pyx_float__0293;
  PyObject *__pyx_float__0297;
  PyObject *__pyx_float__0298;
  PyObject *__pyx_float__0299;
  PyObject *__pyx_float__0300;
  PyObject *__pyx_float__0301;
  PyObject *__pyx_float__0305;
  PyObject *__pyx_float__0306;
  PyObject *__pyx_float__0307;
  PyObject *__pyx_float__0309;
  PyObject *__pyx_float__0310;
  PyObject *__pyx_float__0311;
  PyObject *__pyx_float__0313;
  PyObject *__pyx_float__0314;
  PyObject *__pyx_float__0315;
  PyObject *__pyx_float__0319;
  PyObject *__pyx_float__0322;
  PyObject *__pyx_float__0324;
  PyObject *__pyx_float__0326;
  PyObject *__pyx_float__0328;
  PyObject *__pyx_float__0329;
  PyObject *__pyx_float__0332;
  PyObject *__pyx_float__0334;
  PyObject *__pyx_float__0336;
  PyObject *__pyx_float__0340;
  PyObject *__pyx_float__0342;
  PyObject *__pyx_float__0345;
  PyObject *__pyx_float__0349;
  PyObject *__pyx_float__0353;
  PyObject *__pyx_float__0355;
  PyObject *__pyx_float__0363;
  PyObject *__pyx_float__0364;
  PyObject *__pyx_float__0365;
  PyObject *__pyx_float__0367;
  PyObject *__pyx_float__0368;
  PyObject *__pyx_float__0369;
  PyObject *__pyx_float__0370;
  PyObject *__pyx_float__0373;
  PyObject *__pyx_float__0381;
  PyObject *__pyx_float__0387;
  PyObject *__pyx_float__0389;
  PyObject *__pyx_float__0390;
  PyObject *__pyx_float__0392;
  PyObject *__pyx_float__0396;
  PyObject *__pyx_float__0402;
  PyObject *__pyx_float__0403;
  PyObject *__pyx_float__0404;
  PyObject *__pyx_float__0410;
  PyObject *__pyx_float__0411;
  PyObject *__pyx_float__0415;
  PyObject *__pyx_float__0418;
  PyObject *__pyx_float__0420;
  PyObject *__pyx_float__0421;
  PyObject *__pyx_float__0423;
  PyObject *__pyx_float__0425;
  PyObject *__pyx_float__0437;
  PyObject *__pyx_float__0445;
  PyObject *__pyx_float__0448;
  PyObject *__pyx_float__0449;
  PyObject *__pyx_float__0465;
  PyObject *__pyx_float__0472;
  PyObject *__pyx_float__0473;
  PyObject *__pyx_float__0475;
  PyObject *__pyx_float__0482;
  PyObject *__pyx_float__0492;
  PyObject *__pyx_float__0493;
  PyObject *__pyx_float__0497;
  PyObject *__pyx_float__0498;
  PyObject *__pyx_float__0500;
  PyObject *__pyx_float__0503;
  PyObject *__pyx_float__0506;
  PyObject *__pyx_float__0512;
  PyObject *__pyx_float__0514;
  PyObject *__pyx_float__0516;
  PyObject *__pyx_float__0517;
  PyObject *__pyx_float__0522;
  PyObject *__pyx_float__0523;
  PyObject *__pyx_float__0525;
  PyObject *__pyx_float__0578;
  PyObject *__pyx_float__0593;
  PyObject *__pyx_float__0596;
  PyObject *__pyx_float__0613;
  PyObject *__pyx_float__0621;
  PyObject *__pyx_float__0629;
  PyObject *__pyx_float__0650;
  PyObject *__pyx_float__0654;
  PyObject *__pyx_float__0669;
  PyObject *__pyx_float__0674;
  PyObject *__pyx_float__0676;
  PyObject *__pyx_float__0678;
  PyObject *__pyx_float__0683;
  PyObject *__pyx_float__0692;
  PyObject *__pyx_float__0698;
  PyObject *__pyx_float__0708;
  PyObject *__pyx_float__0714;
  PyObject *__pyx_float__0719;
  PyObject *__pyx_float__0726;
  PyObject *__pyx_float__0727;
  PyObject *__pyx_float__0752;
  PyObject *__pyx_float__0763;
  PyObject *__pyx_float__0765;
  PyObject *__pyx_float__0773;
  PyObject *__pyx_float__0813;
  PyObject *__pyx_float__0820;
  PyObject *__pyx_float__0835;
  PyObject *__pyx_float__0841;
  PyObject *__pyx_float__0846;
  PyObject *__pyx_float__0877;
  PyObject *__pyx_float__0909;
  PyObject *__pyx_float__0919;
  PyObject *__pyx_float__0920;
  PyObject *__pyx_float__0921;
  PyObject *__pyx_float__0933;
  PyObject *__pyx_float__0938;
  PyObject *__pyx_float__0951;
  PyObject *__pyx_float__0963;
  PyObject *__pyx_float__0967;
  PyObject *__pyx_float__0976;
  PyObject *__pyx_float__0982;
  PyObject *__pyx_float__0985;
  PyObject *__pyx_float__1008;
  PyObject *__pyx_float__1011;
  PyObject *__pyx_float__1015;
  PyObject *__pyx_float__1018;
  PyObject *__pyx_float__1054;
  PyObject *__pyx_float__1066;
  PyObject *__pyx_float__1085;
  PyObject *__pyx_float__1126;
  PyObject *__pyx_float__1132;
  PyObject *__pyx_float__1174;
  PyObject *__pyx_float__1193;
  PyObject *__pyx_float__1238;
  PyObject *__pyx_float__1243;
  PyObject *__pyx_float__1273;
  PyObject *__pyx_float__1296;
  PyObject *__pyx_float__1304;
  PyObject *__pyx_float__1311;
  PyObject *__pyx_float__1357;
  PyObject *__pyx_float__1400;
  PyObject *__pyx_float__1440;
  PyObject *__pyx_float__1441;
  PyObject *__pyx_float__1486;
  PyObject *__pyx_float__1541;
  PyObject *__pyx_float__1555;
  PyObject *__pyx_float__1611;
  PyObject *__pyx_float__1655;
  PyObject *__pyx_float__1715;
  PyObject *__pyx_float__1821;
  PyObject *__pyx_float__1822;
  PyObject *__pyx_float__1868;
  PyObject *__pyx_float__1889;
  PyObject *__pyx_float__1919;
  PyObject *__pyx_float__1933;
  PyObject *__pyx_float__2095;
  PyObject *__pyx_float__2113;
  PyObject *__pyx_float__2185;
  PyObject *__pyx_float__2270;
  PyObject *__pyx_float__2388;
  PyObject *__pyx_float__2455;
  PyObject *__pyx_float__2598;
  PyObject *__pyx_float__2735;
  PyObject *__pyx_float__3637;
  PyObject *__pyx_float_10_00;
  PyObject *__pyx_float_10_01;
  PyObject *__pyx_float_10_02;
  PyObject *__pyx_float_10_03;
  PyObject *__pyx_float_10_04;
  PyObject *__pyx_float_10_05;
  PyObject *__pyx_float_10_08;
  PyObject *__pyx_float_10_10;
  PyObject *__pyx_float_10_11;
  PyObject *__pyx_float_10_12;
  PyObject *__pyx_float_10_15;
  PyObject *__pyx_float_10_17;
  PyObject *__pyx_float_10_18;
  PyObject *__pyx_float_10_22;
  PyObject *__pyx_float_10_23;
  PyObject *__pyx_float_10_25;
  PyObject *__pyx_float_10_30;
  PyObject *__pyx_float_10_33;
  PyObject *__pyx_float_10_36;
  PyObject *__pyx_float_10_38;
  PyObject *__pyx_float_10_40;
  PyObject *__pyx_float_10_45;
  PyObject *__pyx_float_10_59;
  PyObject *__pyx_float_10_69;
  PyObject *__pyx_float_10_93;
  PyObject *__pyx_float_10_96;
  PyObject *__pyx_float_10_98;
  PyObject *__pyx_float_10_99;
  PyObject *__pyx_float_11_00;
  PyObject *__pyx_float_11_01;
  PyObject *__pyx_float_11_02;
  PyObject *__pyx_float_11_03;
  PyObject *__pyx_float_11_04;
  PyObject *__pyx_float_11_05;
  PyObject *__pyx_float_11_07;
  PyObject *__pyx_float_11_08;
  PyObject *__pyx_float_11_11;
  PyObject *__pyx_float_11_14;
  PyObject *__pyx_float_11_15;
  PyObject *__pyx_float_11_17;
  PyObject *__pyx_float_11_19;
  PyObject *__pyx_float_11_23;
  PyObject *__pyx_float_11_25;
  PyObject *__pyx_float_11_30;
  PyObject *__pyx_float_11_31;
  PyObject *__pyx_float_11_35;
  PyObject *__pyx_float_11_40;
  PyObject *__pyx_float_11_44;
  PyObject *__pyx_float_11_48;
  PyObject *__pyx_float_11_53;
  PyObject *__pyx_float_11_58;
  PyObject *__pyx_float_11_75;
  PyObject *__pyx_float_11_78;
  PyObject *__pyx_float_11_94;
  PyObject *__pyx_float_11_97;
  PyObject *__pyx_float_11_98;
  PyObject *__pyx_float_11_99;
  PyObject *__pyx_float_12_00;
  PyObject *__pyx_float_12_01;
  PyObject *__pyx_float_12_02;
  PyObject *__pyx_float_12_03;
  PyObject *__pyx_float_12_05;
  PyObject *__pyx_float_12_06;
  PyObject *__pyx_float_12_07;
  PyObject *__pyx_float_12_08;
  PyObject *__pyx_float_12_10;
  PyObject *__pyx_float_12_11;
  PyObject *__pyx_float_12_12;
  PyObject *__pyx_float_12_15;
  PyObject *__pyx_float_12_18;
  PyObject *__pyx_float_12_20;
  PyObject *__pyx_float_12_24;
  PyObject *__pyx_float_12_26;
  PyObject *__pyx_float_12_30;
  PyObject *__pyx_float_12_32;
  PyObject *__pyx_float_12_33;
  PyObject *__pyx_float_12_40;
  PyObject *__pyx_float_12_45;
  PyObject *__pyx_float_12_50;
  PyObject *__pyx_float_12_58;
  PyObject *__pyx_float_12_63;
  PyObject *__pyx_float_12_67;
  PyObject *__pyx_float_12_70;
  PyObject *__pyx_float_12_78;
  PyObject *__pyx_float_12_95;
  PyObject *__pyx_float_12_96;
  PyObject *__pyx_float_12_98;
  PyObject *__pyx_float_12_99;
  PyObject *__pyx_float_13_00;
  PyObject *__pyx_float_13_01;
  PyObject *__pyx_float_13_02;
  PyObject *__pyx_float_13_03;
  PyObject *__pyx_float_13_04;
  PyObject *__pyx_float_13_05;
  PyObject *__pyx_float_13_06;
  PyObject *__pyx_float_13_08;
  PyObject *__pyx_float_13_11;
  PyObject *__pyx_float_13_13;
  PyObject *__pyx_float_13_14;
  PyObject *__pyx_float_13_15;
  PyObject *__pyx_float_13_16;
  PyObject *__pyx_float_13_21;
  PyObject *__pyx_float_13_24;
  PyObject *__pyx_float_13_25;
  PyObject *__pyx_float_13_26;
  PyObject *__pyx_float_13_29;
  PyObject *__pyx_float_13_34;
  PyObject *__pyx_float_13_38;
  PyObject *__pyx_float_13_40;
  PyObject *__pyx_float_13_42;
  PyObject *__pyx_float_13_49;
  PyObject *__pyx_float_13_53;
  PyObject *__pyx_float_13_58;
  PyObject *__pyx_float_13_66;
  PyObject *__pyx_float_13_75;
  PyObject *__pyx_float_13_79;
  PyObject *__pyx_float_13_89;
  PyObject *__pyx_float_13_91;
  PyObject *__pyx_float_13_97;
  PyObject *__pyx_float_13_99;
  PyObject *__pyx_float_14_00;
  PyObject *__pyx_float_14_02;
  PyObject *__pyx_float_14_03;
  PyObject *__pyx_float_14_04;
  PyObject *__pyx_float_14_05;
  PyObject *__pyx_float_14_06;
  PyObject *__pyx_float_14_07;
  PyObject *__pyx_float_14_08;
  PyObject *__pyx_float_14_10;
  PyObject *__pyx_float_14_12;
  PyObject *__pyx_float_14_13;
  PyObject *__pyx_float_14_19;
  PyObject *__pyx_float_14_25;
  PyObject *__pyx_float_14_32;
  PyObject *__pyx_float_14_33;
  PyObject *__pyx_float_14_39;
  PyObject *__pyx_float_14_45;
  PyObject *__pyx_float_14_50;
  PyObject *__pyx_float_14_53;
  PyObject *__pyx_float_14_54;
  PyObject *__pyx_float_14_62;
  PyObject *__pyx_float_14_66;
  PyObject *__pyx_float_14_74;
  PyObject *__pyx_float_14_84;
  PyObject *__pyx_float_14_93;
  PyObject *__pyx_float_14_95;
  PyObject *__pyx_float_15_00;
  PyObject *__pyx_float_15_01;
  PyObject *__pyx_float_15_03;
  PyObject *__pyx_float_15_04;
  PyObject *__pyx_float_15_05;
  PyObject *__pyx_float_15_06;
  PyObject *__pyx_float_15_07;
  PyObject *__pyx_float_15_08;
  PyObject *__pyx_float_15_09;
  PyObject *__pyx_float_15_11;
  PyObject *__pyx_float_15_12;
  PyObject *__pyx_float_15_15;
  PyObject *__pyx_float_15_16;
  PyObject *__pyx_float_15_21;
  PyObject *__pyx_float_15_22;
  PyObject *__pyx_float_15_24;
  PyObject *__pyx_float_15_27;
  PyObject *__pyx_float_15_31;
  PyObject *__pyx_float_15_41;
  PyObject *__pyx_float_15_42;
  PyObject *__pyx_float_15_48;
  PyObject *__pyx_float_15_53;
  PyObject *__pyx_float_15_62;
  PyObject *__pyx_float_15_65;
  PyObject *__pyx_float_15_77;
  PyObject *__pyx_float_15_81;
  PyObject *__pyx_float_15_88;
  PyObject *__pyx_float_15_91;
  PyObject *__pyx_float_15_93;
  PyObject *__pyx_float_15_99;
  PyObject *__pyx_float_16_00;
  PyObject *__pyx_float_16_01;
  PyObject *__pyx_float_16_03;
  PyObject *__pyx_float_16_04;
  PyObject *__pyx_float_16_05;
  PyObject *__pyx_float_16_06;
  PyObject *__pyx_float_16_07;
  PyObject *__pyx_float_16_08;
  PyObject *__pyx_float_16_09;
  PyObject *__pyx_float_16_11;
  PyObject *__pyx_float_16_12;
  PyObject *__pyx_float_16_13;
  PyObject *__pyx_float_16_14;
  PyObject *__pyx_float_16_15;
  PyObject *__pyx_float_16_16;
  PyObject *__pyx_float_16_18;
  PyObject *__pyx_float_16_20;
  PyObject *__pyx_float_16_21;
  PyObject *__pyx_float_16_24;
  PyObject *__pyx_float_16_29;
  PyObject *__pyx_float_16_30;
  PyObject *__pyx_float_16_34;
  PyObject *__pyx_float_16_40;
  PyObject *__pyx_float_16_49;
  PyObject *__pyx_float_16_50;
  PyObject *__pyx_float_16_59;
  PyObject *__pyx_float_16_66;
  PyObject *__pyx_float_16_80;
  PyObject *__pyx_float_17_01;
  PyObject *__pyx_float_17_03;
  PyObject *__pyx_float_17_04;
  PyObject *__pyx_float_17_06;
  PyObject *__pyx_float_17_07;
  PyObject *__pyx_float_17_08;
  PyObject *__pyx_float_17_09;
  PyObject *__pyx_float_17_10;
  PyObject *__pyx_float_17_11;
  PyObject *__pyx_float_17_12;
  PyObject *__pyx_float_17_14;
  PyObject *__pyx_float_17_15;
  PyObject *__pyx_float_17_17;
  PyObject *__pyx_float_17_21;
  PyObject *__pyx_float_17_22;
  PyObject *__pyx_float_17_24;
  PyObject *__pyx_float_17_25;
  PyObject *__pyx_float_17_30;
  PyObject *__pyx_float_17_36;
  PyObject *__pyx_float_17_38;
  PyObject *__pyx_float_17_42;
  PyObject *__pyx_float_17_48;
  PyObject *__pyx_float_17_59;
  PyObject *__pyx_float_17_61;
  PyObject *__pyx_float_17_62;
  PyObject *__pyx_float_17_70;
  PyObject *__pyx_float_17_79;
  PyObject *__pyx_float_17_95;
  PyObject *__pyx_float_17_97;
  PyObject *__pyx_float_18_02;
  PyObject *__pyx_float_18_03;
  PyObject *__pyx_float_18_08;
  PyObject *__pyx_float_18_09;
  PyObject *__pyx_float_18_10;
  PyObject *__pyx_float_18_11;
  PyObject *__pyx_float_18_12;
  PyObject *__pyx_float_18_13;
  PyObject *__pyx_float_18_14;
  PyObject *__pyx_float_18_15;
  PyObject *__pyx_float_18_16;
  PyObject *__pyx_float_18_18;
  PyObject *__pyx_float_18_19;
  PyObject *__pyx_float_18_20;
  PyObject *__pyx_float_18_21;
  PyObject *__pyx_float_18_23;
  PyObject *__pyx_float_18_25;
  PyObject *__pyx_float_18_27;
  PyObject *__pyx_float_18_30;
  PyObject *__pyx_float_18_35;
  PyObject *__pyx_float_18_43;
  PyObject *__pyx_float_18_45;
  PyObject *__pyx_float_18_51;
  PyObject *__pyx_float_18_59;
  PyObject *__pyx_float_18_71;
  PyObject *__pyx_float_18_75;
  PyObject *__pyx_float_18_84;
  PyObject *__pyx_float_18_94;
  PyObject *__pyx_float_18_95;
  PyObject *__pyx_float_19_04;
  PyObject *__pyx_float_19_05;
  PyObject *__pyx_float_19_06;
  PyObject *__pyx_float_19_11;
  PyObject *__pyx_float_19_12;
  PyObject *__pyx_float_19_14;
  PyObject *__pyx_float_19_15;
  PyObject *__pyx_float_19_16;
  PyObject *__pyx_float_19_17;
  PyObject *__pyx_float_19_18;
  PyObject *__pyx_float_19_21;
  PyObject *__pyx_float_19_22;
  PyObject *__pyx_float_19_23;
  PyObject *__pyx_float_19_24;
  PyObject *__pyx_float_19_26;
  PyObject *__pyx_float_19_29;
  PyObject *__pyx_float_19_34;
  PyObject *__pyx_float_19_36;
  PyObject *__pyx_float_19_39;
  PyObject *__pyx_float_19_41;
  PyObject *__pyx_float_19_51;
  PyObject *__pyx_float_19_56;
  PyObject *__pyx_float_19_57;
  PyObject *__pyx_float_19_61;
  PyObject *__pyx_float_19_71;
  PyObject *__pyx_float_19_83;
  PyObject *__pyx_float_19_84;
  PyObject *__pyx_float_19_89;
  PyObject *__pyx_float_20_00;
  PyObject *__pyx_float_20_08;
  PyObject *__pyx_float_20_11;
  PyObject *__pyx_float_20_12;
  PyObject *__pyx_float_20_13;
  PyObject *__pyx_float_20_16;
  PyObject *__pyx_float_20_17;
  PyObject *__pyx_float_20_18;
  PyObject *__pyx_float_20_19;
  PyObject *__pyx_float_20_21;
  PyObject *__pyx_float_20_23;
  PyObject *__pyx_float_20_24;
  PyObject *__pyx_float_20_25;
  PyObject *__pyx_float_20_26;
  PyObject *__pyx_float_20_27;
  PyObject *__pyx_float_20_28;
  PyObject *__pyx_float_20_29;
  PyObject *__pyx_float_20_30;
  PyObject *__pyx_float_20_34;
  PyObject *__pyx_float_20_36;
  PyObject *__pyx_float_20_40;
  PyObject *__pyx_float_20_43;
  PyObject *__pyx_float_20_44;
  PyObject *__pyx_float_20_50;
  PyObject *__pyx_float_20_60;
  PyObject *__pyx_float_20_65;
  PyObject *__pyx_float_20_72;
  PyObject *__pyx_float_20_84;
  PyObject *__pyx_float_20_98;
  PyObject *__pyx_float_20_99;
  PyObject *__pyx_float_21_04;
  PyObject *__pyx_float_21_11;
  PyObject *__pyx_float_21_13;
  PyObject *__pyx_float_21_17;
  PyObject *__pyx_float_21_18;
  PyObject *__pyx_float_21_20;
  PyObject *__pyx_float_21_21;
  PyObject *__pyx_float_21_22;
  PyObject *__pyx_float_21_23;
  PyObject *__pyx_float_21_24;
  PyObject *__pyx_float_21_25;
  PyObject *__pyx_float_21_27;
  PyObject *__pyx_float_21_29;
  PyObject *__pyx_float_21_30;
  PyObject *__pyx_float_21_31;
  PyObject *__pyx_float_21_32;
  PyObject *__pyx_float_21_33;
  PyObject *__pyx_float_21_41;
  PyObject *__pyx_float_21_47;
  PyObject *__pyx_float_21_50;
  PyObject *__pyx_float_21_51;
  PyObject *__pyx_float_21_56;
  PyObject *__pyx_float_21_57;
  PyObject *__pyx_float_21_70;
  PyObject *__pyx_float_21_74;
  PyObject *__pyx_float_21_84;
  PyObject *__pyx_float_21_98;
  PyObject *__pyx_float_22_12;
  PyObject *__pyx_float_22_13;
  PyObject *__pyx_float_22_14;
  PyObject *__pyx_float_22_16;
  PyObject *__pyx_float_22_20;
  PyObject *__pyx_float_22_21;
  PyObject *__pyx_float_22_22;
  PyObject *__pyx_float_22_25;
  PyObject *__pyx_float_22_26;
  PyObject *__pyx_float_22_27;
  PyObject *__pyx_float_22_28;
  PyObject *__pyx_float_22_29;
  PyObject *__pyx_float_22_30;
  PyObject *__pyx_float_22_32;
  PyObject *__pyx_float_22_35;
  PyObject *__pyx_float_22_36;
  PyObject *__pyx_float_22_37;
  PyObject *__pyx_float_22_38;
  PyObject *__pyx_float_22_48;
  PyObject *__pyx_float_22_49;
  PyObject *__pyx_float_22_54;
  PyObject *__pyx_float_22_59;
  PyObject *__pyx_float_22_60;
  PyObject *__pyx_float_22_68;
  PyObject *__pyx_float_22_82;
  PyObject *__pyx_float_22_87;
  PyObject *__pyx_float_22_99;
  PyObject *__pyx_float_23_11;
  PyObject *__pyx_float_23_19;
  PyObject *__pyx_float_23_20;
  PyObject *__pyx_float_23_22;
  PyObject *__pyx_float_23_24;
  PyObject *__pyx_float_23_26;
  PyObject *__pyx_float_23_28;
  PyObject *__pyx_float_23_29;
  PyObject *__pyx_float_23_30;
  PyObject *__pyx_float_23_31;
  PyObject *__pyx_float_23_32;
  PyObject *__pyx_float_23_33;
  PyObject *__pyx_float_23_34;
  PyObject *__pyx_float_23_35;
  PyObject *__pyx_float_23_38;
  PyObject *__pyx_float_23_39;
  PyObject *__pyx_float_23_41;
  PyObject *__pyx_float_23_42;
  PyObject *__pyx_float_23_46;
  PyObject *__pyx_float_23_55;
  PyObject *__pyx_float_23_56;
  PyObject *__pyx_float_23_62;
  PyObject *__pyx_float_23_69;
  PyObject *__pyx_float_23_72;
  PyObject *__pyx_float_23_73;
  PyObject *__pyx_float_23_80;
  PyObject *__pyx_float_23_90;
  PyObject *__pyx_float_23_95;
  PyObject *__pyx_float_23_99;
  PyObject *__pyx_float_24_00;
  PyObject *__pyx_float_24_05;
  PyObject *__pyx_float_24_13;
  PyObject *__pyx_float_24_14;
  PyObject *__pyx_float_24_19;
  PyObject *__pyx_float_24_26;
  PyObject *__pyx_float_24_29;
  PyObject *__pyx_float_24_31;
  PyObject *__pyx_float_24_32;
  PyObject *__pyx_float_24_33;
  PyObject *__pyx_float_24_34;
  PyObject *__pyx_float_24_35;
  PyObject *__pyx_float_24_36;
  PyObject *__pyx_float_24_37;
  PyObject *__pyx_float_24_38;
  PyObject *__pyx_float_24_40;
  PyObject *__pyx_float_24_48;
  PyObject *__pyx_float_24_50;
  PyObject *__pyx_float_24_52;
  PyObject *__pyx_float_24_64;
  PyObject *__pyx_float_24_71;
  PyObject *__pyx_float_24_80;
  PyObject *__pyx_float_24_90;
  PyObject *__pyx_float_24_97;
  PyObject *__pyx_float_25_05;
  PyObject *__pyx_float_25_15;
  PyObject *__pyx_float_25_20;
  PyObject *__pyx_float_25_24;
  PyObject *__pyx_float_25_30;
  PyObject *__pyx_float_25_31;
  PyObject *__pyx_float_25_32;
  PyObject *__pyx_float_25_33;
  PyObject *__pyx_float_25_35;
  PyObject *__pyx_float_25_38;
  PyObject *__pyx_float_25_41;
  PyObject *__pyx_float_25_42;
  PyObject *__pyx_float_25_43;
  PyObject *__pyx_float_25_45;
  PyObject *__pyx_float_25_49;
  PyObject *__pyx_float_25_54;
  PyObject *__pyx_float_25_55;
  PyObject *__pyx_float_25_56;
  PyObject *__pyx_float_25_58;
  PyObject *__pyx_float_25_59;
  PyObject *__pyx_float_25_70;
  PyObject *__pyx_float_25_74;
  PyObject *__pyx_float_25_82;
  PyObject *__pyx_float_25_87;
  PyObject *__pyx_float_25_88;
  PyObject *__pyx_float_26_02;
  PyObject *__pyx_float_26_17;
  PyObject *__pyx_float_26_22;
  PyObject *__pyx_float_26_24;
  PyObject *__pyx_float_26_36;
  PyObject *__pyx_float_26_38;
  PyObject *__pyx_float_26_39;
  PyObject *__pyx_float_26_40;
  PyObject *__pyx_float_26_41;
  PyObject *__pyx_float_26_43;
  PyObject *__pyx_float_26_46;
  PyObject *__pyx_float_26_47;
  PyObject *__pyx_float_26_49;
  PyObject *__pyx_float_26_50;
  PyObject *__pyx_float_26_54;
  PyObject *__pyx_float_26_55;
  PyObject *__pyx_float_26_61;
  PyObject *__pyx_float_26_62;
  PyObject *__pyx_float_26_64;
  PyObject *__pyx_float_26_67;
  PyObject *__pyx_float_26_68;
  PyObject *__pyx_float_26_69;
  PyObject *__pyx_float_26_70;
  PyObject *__pyx_float_26_81;
  PyObject *__pyx_float_26_91;
  PyObject *__pyx_float_27_02;
  PyObject *__pyx_float_27_03;
  PyObject *__pyx_float_27_04;
  PyObject *__pyx_float_27_16;
  PyObject *__pyx_float_27_36;
  PyObject *__pyx_float_27_41;
  PyObject *__pyx_float_27_43;
  PyObject *__pyx_float_27_44;
  PyObject *__pyx_float_27_46;
  PyObject *__pyx_float_27_47;
  PyObject *__pyx_float_27_49;
  PyObject *__pyx_float_27_50;
  PyObject *__pyx_float_27_52;
  PyObject *__pyx_float_27_53;
  PyObject *__pyx_float_27_54;
  PyObject *__pyx_float_27_55;
  PyObject *__pyx_float_27_59;
  PyObject *__pyx_float_27_60;
  PyObject *__pyx_float_27_61;
  PyObject *__pyx_float_27_70;
  PyObject *__pyx_float_27_72;
  PyObject *__pyx_float_27_76;
  PyObject *__pyx_float_27_77;
  PyObject *__pyx_float_27_78;
  PyObject *__pyx_float_27_95;
  PyObject *__pyx_float_27_97;
  PyObject *__pyx_float_28_03;
  PyObject *__pyx_float_28_09;
  PyObject *__pyx_float_28_12;
  PyObject *__pyx_float_28_16;
  PyObject *__pyx_float_28_30;
  PyObject *__pyx_float_28_39;
  PyObject *__pyx_float_28_44;
  PyObject *__pyx_float_28_45;
  PyObject *__pyx_float_28_47;
  PyObject *__pyx_float_28_50;
  PyObject *__pyx_float_28_51;
  PyObject *__pyx_float_28_52;
  PyObject *__pyx_float_28_53;
  PyObject *__pyx_float_28_55;
  PyObject *__pyx_float_28_59;
  PyObject *__pyx_float_28_60;
  PyObject *__pyx_float_28_62;
  PyObject *__pyx_float_28_63;
  PyObject *__pyx_float_28_64;
  PyObject *__pyx_float_28_65;
  PyObject *__pyx_float_28_66;
  PyObject *__pyx_float_28_70;
  PyObject *__pyx_float_28_73;
  PyObject *__pyx_float_28_78;
  PyObject *__pyx_float_28_79;
  PyObject *__pyx_float_28_80;
  PyObject *__pyx_float_28_82;
  PyObject *__pyx_float_28_87;
  PyObject *__pyx_float_28_88;
  PyObject *__pyx_float_29_03;
  PyObject *__pyx_float_29_12;
  PyObject *__pyx_float_29_17;
  PyObject *__pyx_float_29_28;
  PyObject *__pyx_float_29_31;
  PyObject *__pyx_float_29_40;
  PyObject *__pyx_float_29_48;
  PyObject *__pyx_float_29_54;
  PyObject *__pyx_float_29_57;
  PyObject *__pyx_float_29_58;
  PyObject *__pyx_float_29_60;
  PyObject *__pyx_float_29_61;
  PyObject *__pyx_float_29_64;
  PyObject *__pyx_float_29_65;
  PyObject *__pyx_float_29_66;
  PyObject *__pyx_float_29_68;
  PyObject *__pyx_float_29_71;
  PyObject *__pyx_float_29_72;
  PyObject *__pyx_float_29_76;
  PyObject *__pyx_float_29_80;
  PyObject *__pyx_float_29_81;
  PyObject *__pyx_float_29_85;
  PyObject *__pyx_float_29_89;
  PyObject *__pyx_float_29_90;
  PyObject *__pyx_float_29_91;
  PyObject *__pyx_float_29_95;
  PyObject *__pyx_float_30_03;
  PyObject *__pyx_float_30_08;
  PyObject *__pyx_float_30_09;
  PyObject *__pyx_float_30_10;
  PyObject *__pyx_float_30_22;
  PyObject *__pyx_float_30_32;
  PyObject *__pyx_float_30_37;
  PyObject *__pyx_float_30_43;
  PyObject *__pyx_float_30_46;
  PyObject *__pyx_float_30_58;
  PyObject *__pyx_float_30_60;
  PyObject *__pyx_float_30_65;
  PyObject *__pyx_float_30_70;
  PyObject *__pyx_float_30_73;
  PyObject *__pyx_float_30_75;
  PyObject *__pyx_float_30_77;
  PyObject *__pyx_float_30_78;
  PyObject *__pyx_float_30_81;
  PyObject *__pyx_float_30_85;
  PyObject *__pyx_float_30_91;
  PyObject *__pyx_float_30_97;
  PyObject *__pyx_float_30_98;
  PyObject *__pyx_float_31_01;
  PyObject *__pyx_float_31_05;
  PyObject *__pyx_float_31_06;
  PyObject *__pyx_float_31_13;
  PyObject *__pyx_float_31_14;
  PyObject *__pyx_float_31_30;
  PyObject *__pyx_float_31_45;
  PyObject *__pyx_float_31_59;
  PyObject *__pyx_float_31_61;
  PyObject *__pyx_float_31_62;
  PyObject *__pyx_float_31_66;
  PyObject *__pyx_float_31_67;
  PyObject *__pyx_float_31_71;
  PyObject *__pyx_float_31_73;
  PyObject *__pyx_float_31_74;
  PyObject *__pyx_float_31_78;
  PyObject *__pyx_float_31_81;
  PyObject *__pyx_float_31_82;
  PyObject *__pyx_float_31_84;
  PyObject *__pyx_float_31_86;
  PyObject *__pyx_float_31_88;
  PyObject *__pyx_float_31_91;
  PyObject *__pyx_float_31_95;
  PyObject *__pyx_float_31_96;
  PyObject *__pyx_float_31_97;
  PyObject *__pyx_float_32_00;
  PyObject *__pyx_float_32_03;
  PyObject *__pyx_float_32_07;
  PyObject *__pyx_float_32_08;
  PyObject *__pyx_float_32_09;
  PyObject *__pyx_float_32_12;
  PyObject *__pyx_float_32_13;
  PyObject *__pyx_float_32_14;
  PyObject *__pyx_float_32_17;
  PyObject *__pyx_float_32_24;
  PyObject *__pyx_float_32_27;
  PyObject *__pyx_float_32_48;
  PyObject *__pyx_float_32_61;
  PyObject *__pyx_float_32_65;
  PyObject *__pyx_float_32_73;
  PyObject *__pyx_float_32_75;
  PyObject *__pyx_float_32_77;
  PyObject *__pyx_float_32_78;
  PyObject *__pyx_float_32_79;
  PyObject *__pyx_float_32_83;
  PyObject *__pyx_float_32_84;
  PyObject *__pyx_float_32_89;
  PyObject *__pyx_float_32_92;
  PyObject *__pyx_float_32_93;
  PyObject *__pyx_float_32_94;
  PyObject *__pyx_float_32_96;
  PyObject *__pyx_float_32_97;
  PyObject *__pyx_float_33_00;
  PyObject *__pyx_float_33_02;
  PyObject *__pyx_float_33_14;
  PyObject *__pyx_float_33_21;
  PyObject *__pyx_float_33_28;
  PyObject *__pyx_float_33_29;
  PyObject *__pyx_float_33_32;
  PyObject *__pyx_float_33_36;
  PyObject *__pyx_float_33_37;
  PyObject *__pyx_float_33_52;
  PyObject *__pyx_float_33_59;
  PyObject *__pyx_float_33_60;
  PyObject *__pyx_float_33_77;
  PyObject *__pyx_float_33_81;
  PyObject *__pyx_float_33_82;
  PyObject *__pyx_float_33_85;
  PyObject *__pyx_float_33_95;
  PyObject *__pyx_float_34_03;
  PyObject *__pyx_float_34_04;
  PyObject *__pyx_float_34_08;
  PyObject *__pyx_float_34_09;
  PyObject *__pyx_float_34_12;
  PyObject *__pyx_float_34_13;
  PyObject *__pyx_float_34_19;
  PyObject *__pyx_float_34_21;
  PyObject *__pyx_float_34_26;
  PyObject *__pyx_float_34_30;
  PyObject *__pyx_float_34_31;
  PyObject *__pyx_float_34_37;
  PyObject *__pyx_float_34_42;
  PyObject *__pyx_float_34_52;
  PyObject *__pyx_float_34_76;
  PyObject *__pyx_float_34_79;
  PyObject *__pyx_float_34_91;
  PyObject *__pyx_float_34_95;
  PyObject *__pyx_float_34_96;
  PyObject *__pyx_float_34_98;
  PyObject *__pyx_float_35_06;
  PyObject *__pyx_float_35_07;
  PyObject *__pyx_float_35_13;
  PyObject *__pyx_float_35_14;
  PyObject *__pyx_float_35_16;
  PyObject *__pyx_float_35_17;
  PyObject *__pyx_float_35_18;
  PyObject *__pyx_float_35_25;
  PyObject *__pyx_float_35_30;
  PyObject *__pyx_float_35_38;
  PyObject *__pyx_float_35_44;
  PyObject *__pyx_float_35_46;
  PyObject *__pyx_float_35_50;
  PyObject *__pyx_float_35_59;
  PyObject *__pyx_float_35_64;
  PyObject *__pyx_float_35_67;
  PyObject *__pyx_float_35_75;
  PyObject *__pyx_float_36_00;
  PyObject *__pyx_float_36_01;
  PyObject *__pyx_float_36_02;
  PyObject *__pyx_float_36_07;
  PyObject *__pyx_float_36_09;
  PyObject *__pyx_float_36_14;
  PyObject *__pyx_float_36_16;
  PyObject *__pyx_float_36_20;
  PyObject *__pyx_float_36_25;
  PyObject *__pyx_float_36_27;
  PyObject *__pyx_float_36_28;
  PyObject *__pyx_float_36_29;
  PyObject *__pyx_float_36_30;
  PyObject *__pyx_float_36_32;
  PyObject *__pyx_float_36_34;
  PyObject *__pyx_float_36_40;
  PyObject *__pyx_float_36_43;
  PyObject *__pyx_float_36_55;
  PyObject *__pyx_float_36_56;
  PyObject *__pyx_float_36_62;
  PyObject *__pyx_float_36_74;
  PyObject *__pyx_float_36_78;
  PyObject *__pyx_float_36_88;
  PyObject *__pyx_float_37_11;
  PyObject *__pyx_float_37_16;
  PyObject *__pyx_float_37_20;
  PyObject *__pyx_float_37_23;
  PyObject *__pyx_float_37_27;
  PyObject *__pyx_float_37_32;
  PyObject *__pyx_float_37_34;
  PyObject *__pyx_float_37_35;
  PyObject *__pyx_float_37_38;
  PyObject *__pyx_float_37_39;
  PyObject *__pyx_float_37_41;
  PyObject *__pyx_float_37_51;
  PyObject *__pyx_float_37_56;
  PyObject *__pyx_float_37_57;
  PyObject *__pyx_float_37_68;
  PyObject *__pyx_float_37_69;
  PyObject *__pyx_float_37_76;
  PyObject *__pyx_float_37_77;
  PyObject *__pyx_float_37_88;
  PyObject *__pyx_float_37_89;
  PyObject *__pyx_float_37_90;
  PyObject *__pyx_float_37_93;
  PyObject *__pyx_float_37_99;
  PyObject *__pyx_float_38_10;
  PyObject *__pyx_float_38_18;
  PyObject *__pyx_float_38_20;
  PyObject *__pyx_float_38_25;
  PyObject *__pyx_float_38_26;
  PyObject *__pyx_float_38_30;
  PyObject *__pyx_float_38_33;
  PyObject *__pyx_float_38_36;
  PyObject *__pyx_float_38_46;
  PyObject *__pyx_float_38_51;
  PyObject *__pyx_float_38_53;
  PyObject *__pyx_float_38_54;
  PyObject *__pyx_float_38_55;
  PyObject *__pyx_float_38_58;
  PyObject *__pyx_float_38_64;
  PyObject *__pyx_float_38_68;
  PyObject *__pyx_float_38_80;
  PyObject *__pyx_float_38_85;
  PyObject *__pyx_float_38_86;
  PyObject *__pyx_float_38_92;
  PyObject *__pyx_float_38_93;
  PyObject *__pyx_float_39_00;
  PyObject *__pyx_float_39_03;
  PyObject *__pyx_float_39_06;
  PyObject *__pyx_float_39_11;
  PyObject *__pyx_float_39_17;
  PyObject *__pyx_float_39_23;
  PyObject *__pyx_float_39_29;
  PyObject *__pyx_float_39_35;
  PyObject *__pyx_float_39_41;
  PyObject *__pyx_float_39_47;
  PyObject *__pyx_float_39_59;
  PyObject *__pyx_float_39_60;
  PyObject *__pyx_float_39_62;
  PyObject *__pyx_float_39_64;
  PyObject *__pyx_float_39_65;
  PyObject *__pyx_float_39_66;
  PyObject *__pyx_float_39_67;
  PyObject *__pyx_float_39_75;
  PyObject *__pyx_float_39_80;
  PyObject *__pyx_float_39_88;
  PyObject *__pyx_float_40_02;
  PyObject *__pyx_float_40_03;
  PyObject *__pyx_float_40_05;
  PyObject *__pyx_float_40_07;
  PyObject *__pyx_float_40_08;
  PyObject *__pyx_float_40_09;
  PyObject *__pyx_float_40_11;
  PyObject *__pyx_float_40_19;
  PyObject *__pyx_float_40_25;
  PyObject *__pyx_float_40_32;
  PyObject *__pyx_float_40_33;
  PyObject *__pyx_float_40_35;
  PyObject *__pyx_float_40_43;
  PyObject *__pyx_float_40_53;
  PyObject *__pyx_float_40_54;
  PyObject *__pyx_float_40_55;
  PyObject *__pyx_float_40_58;
  PyObject *__pyx_float_40_69;
  PyObject *__pyx_float_40_71;
  PyObject *__pyx_float_40_77;
  PyObject *__pyx_float_40_80;
  PyObject *__pyx_float_40_85;
  PyObject *__pyx_float_40_90;
  PyObject *__pyx_float_40_95;
  PyObject *__pyx_float_40_97;
  PyObject *__pyx_float_41_00;
  PyObject *__pyx_float_41_16;
  PyObject *__pyx_float_41_20;
  PyObject *__pyx_float_41_23;
  PyObject *__pyx_float_41_28;
  PyObject *__pyx_float_41_37;
  PyObject *__pyx_float_41_40;
  PyObject *__pyx_float_41_43;
  PyObject *__pyx_float_41_47;
  PyObject *__pyx_float_41_50;
  PyObject *__pyx_float_41_51;
  PyObject *__pyx_float_41_52;
  PyObject *__pyx_float_41_53;
  PyObject *__pyx_float_41_61;
  PyObject *__pyx_float_41_77;
  PyObject *__pyx_float_41_82;
  PyObject *__pyx_float_41_86;
  PyObject *__pyx_float_41_89;
  PyObject *__pyx_float_41_90;
  PyObject *__pyx_float_41_93;
  PyObject *__pyx_float_41_97;
  PyObject *__pyx_float_42_09;
  PyObject *__pyx_float_42_23;
  PyObject *__pyx_float_42_37;
  PyObject *__pyx_float_42_42;
  PyObject *__pyx_float_42_43;
  PyObject *__pyx_float_42_50;
  PyObject *__pyx_float_42_53;
  PyObject *__pyx_float_42_66;
  PyObject *__pyx_float_42_67;
  PyObject *__pyx_float_42_80;
  PyObject *__pyx_float_42_92;
  PyObject *__pyx_float_42_95;
  PyObject *__pyx_float_43_04;
  PyObject *__pyx_float_43_06;
  PyObject *__pyx_float_43_08;
  PyObject *__pyx_float_43_24;
  PyObject *__pyx_float_43_25;
  PyObject *__pyx_float_43_27;
  PyObject *__pyx_float_43_35;
  PyObject *__pyx_float_43_38;
  PyObject *__pyx_float_43_40;
  PyObject *__pyx_float_43_41;
  PyObject *__pyx_float_43_46;
  PyObject *__pyx_float_43_48;
  PyObject *__pyx_float_43_49;
  PyObject *__pyx_float_43_54;
  PyObject *__pyx_float_43_60;
  PyObject *__pyx_float_43_65;
  PyObject *__pyx_float_43_74;
  PyObject *__pyx_float_43_75;
  PyObject *__pyx_float_43_87;
  PyObject *__pyx_float_43_89;
  PyObject *__pyx_float_43_93;
  PyObject *__pyx_float_44_02;
  PyObject *__pyx_float_44_03;
  PyObject *__pyx_float_44_10;
  PyObject *__pyx_float_44_14;
  PyObject *__pyx_float_44_17;
  PyObject *__pyx_float_44_21;
  PyObject *__pyx_float_44_24;
  PyObject *__pyx_float_44_31;
  PyObject *__pyx_float_44_48;
  PyObject *__pyx_float_44_51;
  PyObject *__pyx_float_44_52;
  PyObject *__pyx_float_44_53;
  PyObject *__pyx_float_44_68;
  PyObject *__pyx_float_44_73;
  PyObject *__pyx_float_44_74;
  PyObject *__pyx_float_44_94;
  PyObject *__pyx_float_45_03;
  PyObject *__pyx_float_45_05;
  PyObject *__pyx_float_45_12;
  PyObject *__pyx_float_45_13;
  PyObject *__pyx_float_45_24;
  PyObject *__pyx_float_45_25;
  PyObject *__pyx_float_45_29;
  PyObject *__pyx_float_45_36;
  PyObject *__pyx_float_45_38;
  PyObject *__pyx_float_45_42;
  PyObject *__pyx_float_45_49;
  PyObject *__pyx_float_45_62;
  PyObject *__pyx_float_45_65;
  PyObject *__pyx_float_45_68;
  PyObject *__pyx_float_45_85;
  PyObject *__pyx_float_45_95;
  PyObject *__pyx_float_45_99;
  PyObject *__pyx_float_46_02;
  PyObject *__pyx_float_46_11;
  PyObject *__pyx_float_46_13;
  PyObject *__pyx_float_46_18;
  PyObject *__pyx_float_46_20;
  PyObject *__pyx_float_46_33;
  PyObject *__pyx_float_46_35;
  PyObject *__pyx_float_46_36;
  PyObject *__pyx_float_46_42;
  PyObject *__pyx_float_46_43;
  PyObject *__pyx_float_46_54;
  PyObject *__pyx_float_46_56;
  PyObject *__pyx_float_46_57;
  PyObject *__pyx_float_46_62;
  PyObject *__pyx_float_46_65;
  PyObject *__pyx_float_46_68;
  PyObject *__pyx_float_46_86;
  PyObject *__pyx_float_46_93;
  PyObject *__pyx_float_47_00;
  PyObject *__pyx_float_47_07;
  PyObject *__pyx_float_47_11;
  PyObject *__pyx_float_47_16;
  PyObject *__pyx_float_47_20;
  PyObject *__pyx_float_47_21;
  PyObject *__pyx_float_47_25;
  PyObject *__pyx_float_47_40;
  PyObject *__pyx_float_47_41;
  PyObject *__pyx_float_47_46;
  PyObject *__pyx_float_47_50;
  PyObject *__pyx_float_47_60;
  PyObject *__pyx_float_47_64;
  PyObject *__pyx_float_47_65;
  PyObject *__pyx_float_47_72;
  PyObject *__pyx_float_47_78;
  PyObject *__pyx_float_47_79;
  PyObject *__pyx_float_47_81;
  PyObject *__pyx_float_47_82;
  PyObject *__pyx_float_47_87;
  PyObject *__pyx_float_48_07;
  PyObject *__pyx_float_48_11;
  PyObject *__pyx_float_48_28;
  PyObject *__pyx_float_48_36;
  PyObject *__pyx_float_48_40;
  PyObject *__pyx_float_48_42;
  PyObject *__pyx_float_48_43;
  PyObject *__pyx_float_48_44;
  PyObject *__pyx_float_48_52;
  PyObject *__pyx_float_48_57;
  PyObject *__pyx_float_48_65;
  PyObject *__pyx_float_48_68;
  PyObject *__pyx_float_48_71;
  PyObject *__pyx_float_48_76;
  PyObject *__pyx_float_48_79;
  PyObject *__pyx_float_48_83;
  PyObject *__pyx_float_48_87;
  PyObject *__pyx_float_48_88;
  PyObject *__pyx_float_48_89;
  PyObject *__pyx_float_48_95;
  PyObject *__pyx_float_48_98;
  PyObject *__pyx_float_48_99;
  PyObject *__pyx_float_49_01;
  PyObject *__pyx_float_49_14;
  PyObject *__pyx_float_49_27;
  PyObject *__pyx_float_49_29;
  PyObject *__pyx_float_49_44;
  PyObject *__pyx_float_49_48;
  PyObject *__pyx_float_49_55;
  PyObject *__pyx_float_49_65;
  PyObject *__pyx_float_49_67;
  PyObject *__pyx_float_49_68;
  PyObject *__pyx_float_49_87;
  PyObject *__pyx_float_49_93;
  PyObject *__pyx_float_49_95;
  PyObject *__pyx_float_50_01;
  PyObject *__pyx_float_50_03;
  PyObject *__pyx_float_50_10;
  PyObject *__pyx_float_50_11;
  PyObject *__pyx_float_50_13;
  PyObject *__pyx_float_50_18;
  PyObject *__pyx_float_50_21;
  PyObject *__pyx_float_50_48;
  PyObject *__pyx_float_50_49;
  PyObject *__pyx_float_50_63;
  PyObject *__pyx_float_50_72;
  PyObject *__pyx_float_50_76;
  PyObject *__pyx_float_50_83;
  PyObject *__pyx_float_50_88;
  PyObject *__pyx_float_50_92;
  PyObject *__pyx_float_50_96;
  PyObject *__pyx_float_51_09;
  PyObject *__pyx_float_51_12;
  PyObject *__pyx_float_51_19;
  PyObject *__pyx_float_51_27;
  PyObject *__pyx_float_51_29;
  PyObject *__pyx_float_51_34;
  PyObject *__pyx_float_51_37;
  PyObject *__pyx_float_51_42;
  PyObject *__pyx_float_51_54;
  PyObject *__pyx_float_51_62;
  PyObject *__pyx_float_51_81;
  PyObject *__pyx_float_51_93;
  PyObject *__pyx_float_52_00;
  PyObject *__pyx_float_52_15;
  PyObject *__pyx_float_52_17;
  PyObject *__pyx_float_52_33;
  PyObject *__pyx_float_52_43;
  PyObject *__pyx_float_52_47;
  PyObject *__pyx_float_52_70;
  PyObject *__pyx_float_52_73;
  PyObject *__pyx_float_52_95;
  PyObject *__pyx_float_53_42;
  PyObject *__pyx_float_53_46;
  PyObject *__pyx_float_53_52;
  PyObject *__pyx_float_53_56;
  PyObject *__pyx_float_53_68;
  PyObject *__pyx_float_53_79;
  PyObject *__pyx_float_53_82;
  PyObject *__pyx_float_53_90;
  PyObject *__pyx_float_53_91;
  PyObject *__pyx_float_54_00;
  PyObject *__pyx_float_54_05;
  PyObject *__pyx_float_54_15;
  PyObject *__pyx_float_54_23;
  PyObject *__pyx_float_54_39;
  PyObject *__pyx_float_54_69;
  PyObject *__pyx_float_54_74;
  PyObject *__pyx_float_54_75;
  PyObject *__pyx_float_54_76;
  PyObject *__pyx_float_55_02;
  PyObject *__pyx_float_55_06;
  PyObject *__pyx_float_55_09;
  PyObject *__pyx_float_55_14;
  PyObject *__pyx_float_55_24;
  PyObject *__pyx_float_55_30;
  PyObject *__pyx_float_55_31;
  PyObject *__pyx_float_55_34;
  PyObject *__pyx_float_55_45;
  PyObject *__pyx_float_55_50;
  PyObject *__pyx_float_55_58;
  PyObject *__pyx_float_55_75;
  PyObject *__pyx_float_55_88;
  PyObject *__pyx_float_55_90;
  PyObject *__pyx_float_55_98;
  PyObject *__pyx_float_56_04;
  PyObject *__pyx_float_56_06;
  PyObject *__pyx_float_56_17;
  PyObject *__pyx_float_56_18;
  PyObject *__pyx_float_56_20;
  PyObject *__pyx_float_56_23;
  PyObject *__pyx_float_56_31;
  PyObject *__pyx_float_56_60;
  PyObject *__pyx_float_56_62;
  PyObject *__pyx_float_56_68;
  PyObject *__pyx_float_56_73;
  PyObject *__pyx_float_56_90;
  PyObject *__pyx_float_56_98;
  PyObject *__pyx_float_57_02;
  PyObject *__pyx_float_57_10;
  PyObject *__pyx_float_57_21;
  PyObject *__pyx_float_57_26;
  PyObject *__pyx_float_57_34;
  PyObject *__pyx_float_57_40;
  PyObject *__pyx_float_57_41;
  PyObject *__pyx_float_57_53;
  PyObject *__pyx_float_57_63;
  PyObject *__pyx_float_57_69;
  PyObject *__pyx_float_57_81;
  PyObject *__pyx_float_57_90;
  PyObject *__pyx_float_57_96;
  PyObject *__pyx_float_57_97;
  PyObject *__pyx_float_57_98;
  PyObject *__pyx_float_58_03;
  PyObject *__pyx_float_58_11;
  PyObject *__pyx_float_58_17;
  PyObject *__pyx_float_58_28;
  PyObject *__pyx_float_58_34;
  PyObject *__pyx_float_58_42;
  PyObject *__pyx_float_58_53;
  PyObject *__pyx_float_58_56;
  PyObject *__pyx_float_58_67;
  PyObject *__pyx_float_58_71;
  PyObject *__pyx_float_58_75;
  PyObject *__pyx_float_58_79;
  PyObject *__pyx_float_58_86;
  PyObject *__pyx_float_59_02;
  PyObject *__pyx_float_59_13;
  PyObject *__pyx_float_59_16;
  PyObject *__pyx_float_59_21;
  PyObject *__pyx_float_59_29;
  PyObject *__pyx_float_59_30;
  PyObject *__pyx_float_59_33;
  PyObject *__pyx_float_59_40;
  PyObject *__pyx_float_59_47;
  PyObject *__pyx_float_59_70;
  PyObject *__pyx_float_59_81;
  PyObject *__pyx_float_59_83;
  PyObject *__pyx_float_59_86;
  PyObject *__pyx_float_59_95;
  PyObject *__pyx_float_59_96;
  PyObject *__pyx_float_60_00;
  PyObject *__pyx_float_60_03;
  PyObject *__pyx_float_60_09;
  PyObject *__pyx_float_60_23;
  PyObject *__pyx_float_60_33;
  PyObject *__pyx_float_60_35;
  PyObject *__pyx_float_60_42;
  PyObject *__pyx_float_60_59;
  PyObject *__pyx_float_60_62;
  PyObject *__pyx_float_60_67;
  PyObject *__pyx_float_60_75;
  PyObject *__pyx_float_61_05;
  PyObject *__pyx_float_61_13;
  PyObject *__pyx_float_61_21;
  PyObject *__pyx_float_61_29;
  PyObject *__pyx_float_61_34;
  PyObject *__pyx_float_61_37;
  PyObject *__pyx_float_61_39;
  PyObject *__pyx_float_61_40;
  PyObject *__pyx_float_61_56;
  PyObject *__pyx_float_61_59;
  PyObject *__pyx_float_61_62;
  PyObject *__pyx_float_61_66;
  PyObject *__pyx_float_61_99;
  PyObject *__pyx_float_62_04;
  PyObject *__pyx_float_62_20;
  PyObject *__pyx_float_62_22;
  PyObject *__pyx_float_62_54;
  PyObject *__pyx_float_62_56;
  PyObject *__pyx_float_62_72;
  PyObject *__pyx_float_62_84;
  PyObject *__pyx_float_62_86;
  PyObject *__pyx_float_63_08;
  PyObject *__pyx_float_63_48;
  PyObject *__pyx_float_63_55;
  PyObject *__pyx_float_63_78;
  PyObject *__pyx_float_63_97;
  PyObject *__pyx_float_64_29;
  PyObject *__pyx_float_64_36;
  PyObject *__pyx_float_64_82;
  PyObject *__pyx_float_64_83;
  PyObject *__pyx_float_64_86;
  PyObject *__pyx_float_64_97;
  PyObject *__pyx_float_65_09;
  PyObject *__pyx_float_65_19;
  PyObject *__pyx_float_65_20;
  PyObject *__pyx_float_65_42;
  PyObject *__pyx_float_65_54;
  PyObject *__pyx_float_65_65;
  PyObject *__pyx_float_65_89;
  PyObject *__pyx_float_66_12;
  PyObject *__pyx_float_66_24;
  PyObject *__pyx_float_66_26;
  PyObject *__pyx_float_66_34;
  PyObject *__pyx_float_66_44;
  PyObject *__pyx_float_66_47;
  PyObject *__pyx_float_66_48;
  PyObject *__pyx_float_66_56;
  PyObject *__pyx_float_66_60;
  PyObject *__pyx_float_66_64;
  PyObject *__pyx_float_66_72;
  PyObject *__pyx_float_66_91;
  PyObject *__pyx_float_66_92;
  PyObject *__pyx_float_66_96;
  PyObject *__pyx_float_67_18;
  PyObject *__pyx_float_67_23;
  PyObject *__pyx_float_67_37;
  PyObject *__pyx_float_67_46;
  PyObject *__pyx_float_67_63;
  PyObject *__pyx_float_67_70;
  PyObject *__pyx_float_67_83;
  PyObject *__pyx_float_67_94;
  PyObject *__pyx_float_67_97;
  PyObject *__pyx_float_67_98;
  PyObject *__pyx_float_68_18;
  PyObject *__pyx_float_68_50;
  PyObject *__pyx_float_68_61;
  PyObject *__pyx_float_68_65;
  PyObject *__pyx_float_68_81;
  PyObject *__pyx_float_68_85;
  PyObject *__pyx_float_68_90;
  PyObject *__pyx_float_68_94;
  PyObject *__pyx_float_68_99;
  PyObject *__pyx_float_69_09;
  PyObject *__pyx_float_69_13;
  PyObject *__pyx_float_69_17;
  PyObject *__pyx_float_69_33;
  PyObject *__pyx_float_69_37;
  PyObject *__pyx_float_69_60;
  PyObject *__pyx_float_69_78;
  PyObject *__pyx_float_69_91;
  PyObject *__pyx_float_69_97;
  PyObject *__pyx_float_70_10;
  PyObject *__pyx_float_70_25;
  PyObject *__pyx_float_70_42;
  PyObject *__pyx_float_70_43;
  PyObject *__pyx_float_70_52;
  PyObject *__pyx_float_70_53;
  PyObject *__pyx_float_70_58;
  PyObject *__pyx_float_70_68;
  PyObject *__pyx_float_70_73;
  PyObject *__pyx_float_70_88;
  PyObject *__pyx_float_71_17;
  PyObject *__pyx_float_71_21;
  PyObject *__pyx_float_71_25;
  PyObject *__pyx_float_71_36;
  PyObject *__pyx_float_71_59;
  PyObject *__pyx_float_71_72;
  PyObject *__pyx_float_71_84;
  PyObject *__pyx_float_72_00;
  PyObject *__pyx_float_72_01;
  PyObject *__pyx_float_72_14;
  PyObject *__pyx_float_72_24;
  PyObject *__pyx_float_72_45;
  PyObject *__pyx_float_73_05;
  PyObject *__pyx_float_73_28;
  PyObject *__pyx_float_73_50;
  PyObject *__pyx_float_73_53;
  PyObject *__pyx_float_73_57;
  PyObject *__pyx_float_73_62;
  PyObject *__pyx_float_73_68;
  PyObject *__pyx_float_74_22;
  PyObject *__pyx_float_74_40;
  PyObject *__pyx_float_74_95;
  PyObject *__pyx_float_75_00;
  PyObject *__pyx_float_75_44;
  PyObject *__pyx_float_75_49;
  PyObject *__pyx_float_75_52;
  PyObject *__pyx_float_75_76;
  PyObject *__pyx_float_75_92;
  PyObject *__pyx_float_76_12;
  PyObject *__pyx_float_76_45;
  PyObject *__pyx_float_76_52;
  PyObject *__pyx_float_76_53;
  PyObject *__pyx_float_76_96;
  PyObject *__pyx_float_77_03;
  PyObject *__pyx_float_77_08;
  PyObject *__pyx_float_77_13;
  PyObject *__pyx_float_77_93;
  PyObject *__pyx_float_77_97;
  PyObject *__pyx_float_78_00;
  PyObject *__pyx_float_78_02;
  PyObject *__pyx_float_78_31;
  PyObject *__pyx_float_78_32;
  PyObject *__pyx_float_78_45;
  PyObject *__pyx_float_78_94;
  PyObject *__pyx_float_79_27;
  PyObject *__pyx_float_79_38;
  PyObject *__pyx_float_79_48;
  PyObject *__pyx_float_79_49;
  PyObject *__pyx_float_79_57;
  PyObject *__pyx_float_79_65;
  PyObject *__pyx_float_79_73;
  PyObject *__pyx_float_79_83;
  PyObject *__pyx_float_80_16;
  PyObject *__pyx_float_80_39;
  PyObject *__pyx_float_80_44;
  PyObject *__pyx_float_80_77;
  PyObject *__pyx_float_81_00;
  PyObject *__pyx_float_81_16;
  PyObject *__pyx_float_81_33;
  PyObject *__pyx_float_81_56;
  PyObject *__pyx_float_81_69;
  PyObject *__pyx_float_81_75;
  PyObject *__pyx_float_81_87;
  PyObject *__pyx_float_82_06;
  PyObject *__pyx_float_82_39;
  PyObject *__pyx_float_82_44;
  PyObject *__pyx_float_82_47;
  PyObject *__pyx_float_82_52;
  PyObject *__pyx_float_82_59;
  PyObject *__pyx_float_82_70;
  PyObject *__pyx_float_82_99;
  PyObject *__pyx_float_83_17;
  PyObject *__pyx_float_83_53;
  PyObject *__pyx_float_83_91;
  PyObject *__pyx_float_83_97;
  PyObject *__pyx_float_84_01;
  PyObject *__pyx_float_84_10;
  PyObject *__pyx_float_84_22;
  PyObject *__pyx_float_84_61;
  PyObject *__pyx_float_84_81;
  PyObject *__pyx_float_85_28;
  PyObject *__pyx_float_85_41;
  PyObject *__pyx_float_85_42;
  PyObject *__pyx_float_85_97;
  PyObject *__pyx_float_86_88;
  PyObject *__pyx_float_87_27;
  PyObject *__pyx_float_87_46;
  PyObject *__pyx_float_87_56;
  PyObject *__pyx_float_87_65;
  PyObject *__pyx_float_88_36;
  PyObject *__pyx_float_88_39;
  PyObject *__pyx_float_89_08;
  PyObject *__pyx_float_89_85;
  PyObject *__pyx_float_90_29;
  PyObject *__pyx_float_90_30;
  PyObject *__pyx_float_90_55;
  PyObject *__pyx_float_90_70;
  PyObject *__pyx_float_91_35;
  PyObject *__pyx_float_91_75;
  PyObject *__pyx_float_92_09;
  PyObject *__pyx_float_92_30;
  PyObject *__pyx_float_92_86;
  PyObject *__pyx_float_93_28;
  PyObject *__pyx_float_93_40;
  PyObject *__pyx_float_93_52;
  PyObject *__pyx_float_94_02;
  PyObject *__pyx_float_95_26;
  PyObject *__pyx_float_95_59;
  PyObject *__pyx_float_97_07;
  PyObject *__pyx_float_97_22;
  PyObject *__pyx_float_98_86;
  PyObject *__pyx_float_105_28;
  PyObject *__pyx_float_106_22;
  PyObject *__pyx_float_107_45;
  PyObject *__pyx_float_108_37;
  PyObject *__pyx_float_108_70;
  PyObject *__pyx_float_109_39;
  PyObject *__pyx_float_122_08;
  PyObject *__pyx_float_125_54;
  PyObject *__pyx_float_32_703;
  PyObject *__pyx_float_10000_0;
  PyObject *__pyx_int_0;
  PyObject *__pyx_int_1;
  PyObject *__pyx_int_2;
  PyObject *__pyx_int_4;
  PyObject *__pyx_int_12;
  PyObject *__pyx_int_45;
  PyObject *__pyx_int_100;
  PyObject *__pyx_int_220;
  PyObject *__pyx_int_261;
  PyObject *__pyx_int_440;
  PyObject *__pyx_tuple_;
  PyObject *__pyx_slice__3;
  PyObject *__pyx_slice__4;
  PyObject *__pyx_tuple__2;
  PyObject *__pyx_tuple__6;
  PyObject *__pyx_tuple__7;
  PyObject *__pyx_tuple__9;
  PyObject *__pyx_tuple__11;
  PyObject *__pyx_tuple__13;
  PyObject *__pyx_tuple__14;
  PyObject *__pyx_codeobj__8;
  PyObject *__pyx_codeobj__10;
  PyObject *__pyx_codeobj__12;
} __pyx_mstate;

#if CYTHON_USE_MODULE_STATE
#ifdef __cplusplus
namespace {
  extern struct PyModuleDef __pyx_moduledef;
} /* anonymous namespace */
#else
static struct PyModuleDef __pyx_moduledef;
#endif

#define __pyx_mstate(o) ((__pyx_mstate *)__Pyx_PyModule_GetState(o))

#define __pyx_mstate_global (__pyx_mstate(PyState_FindModule(&__pyx_moduledef)))

#define __pyx_m (PyState_FindModule(&__pyx_moduledef))
#else
static __pyx_mstate __pyx_mstate_global_static =
#ifdef __cplusplus
    {};
#else
    {0};
#endif
static __pyx_mstate *__pyx_mstate_global = &__pyx_mstate_global_static;
#endif
/* #### Code section: module_state_clear ### */
#if CYTHON_USE_MODULE_STATE
static int __pyx_m_clear(PyObject *m) {
  __pyx_mstate *clear_module_state = __pyx_mstate(m);
  if (!clear_module_state) return 0;
  Py_CLEAR(clear_module_state->__pyx_d);
  Py_CLEAR(clear_module_state->__pyx_b);
  Py_CLEAR(clear_module_state->__pyx_cython_runtime);
  Py_CLEAR(clear_module_state->__pyx_empty_tuple);
  Py_CLEAR(clear_module_state->__pyx_empty_bytes);
  Py_CLEAR(clear_module_state->__pyx_empty_unicode);
  #ifdef __Pyx_CyFunction_USED
  Py_CLEAR(clear_module_state->__pyx_CyFunctionType);
  #endif
  #ifdef __Pyx_FusedFunction_USED
  Py_CLEAR(clear_module_state->__pyx_FusedFunctionType);
  #endif
  Py_CLEAR(clear_module_state->__pyx_ptype_7cpython_4type_type);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_dtype);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_flatiter);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_broadcast);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_ndarray);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_generic);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_number);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_integer);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_signedinteger);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_unsignedinteger);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_inexact);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_floating);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_complexfloating);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_flexible);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_character);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_ufunc);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_3clm_mus_any);
  Py_CLEAR(clear_module_state->__pyx_n_s_ImportError);
  Py_CLEAR(clear_module_state->__pyx_n_s_PIANO_SPECTRA);
  Py_CLEAR(clear_module_state->__pyx_n_s_Sound);
  Py_CLEAR(clear_module_state->__pyx_n_s__15);
  Py_CLEAR(clear_module_state->__pyx_n_s__5);
  Py_CLEAR(clear_module_state->__pyx_n_s_amplitude);
  Py_CLEAR(clear_module_state->__pyx_n_s_asyncio_coroutines);
  Py_CLEAR(clear_module_state->__pyx_n_s_attackTime);
  Py_CLEAR(clear_module_state->__pyx_n_s_base);
  Py_CLEAR(clear_module_state->__pyx_n_s_class_getitem);
  Py_CLEAR(clear_module_state->__pyx_n_s_cline_in_traceback);
  Py_CLEAR(clear_module_state->__pyx_n_s_clm);
  Py_CLEAR(clear_module_state->__pyx_n_s_db_drop_per_second);
  Py_CLEAR(clear_module_state->__pyx_n_s_degree);
  Py_CLEAR(clear_module_state->__pyx_n_s_distance);
  Py_CLEAR(clear_module_state->__pyx_n_s_double);
  Py_CLEAR(clear_module_state->__pyx_n_s_dtype);
  Py_CLEAR(clear_module_state->__pyx_n_s_dur);
  Py_CLEAR(clear_module_state->__pyx_n_s_duration);
  Py_CLEAR(clear_module_state->__pyx_n_s_enter);
  Py_CLEAR(clear_module_state->__pyx_n_s_exit);
  Py_CLEAR(clear_module_state->__pyx_n_s_freq);
  Py_CLEAR(clear_module_state->__pyx_n_s_frequency);
  Py_CLEAR(clear_module_state->__pyx_n_s_get_piano_partials);
  Py_CLEAR(clear_module_state->__pyx_n_s_import);
  Py_CLEAR(clear_module_state->__pyx_n_s_initializing);
  Py_CLEAR(clear_module_state->__pyx_n_s_is_coroutine);
  Py_CLEAR(clear_module_state->__pyx_n_s_is_number);
  Py_CLEAR(clear_module_state->__pyx_n_s_lbj_piano);
  Py_CLEAR(clear_module_state->__pyx_n_s_log);
  Py_CLEAR(clear_module_state->__pyx_n_s_main);
  Py_CLEAR(clear_module_state->__pyx_n_u_main);
  Py_CLEAR(clear_module_state->__pyx_n_s_make_env);
  Py_CLEAR(clear_module_state->__pyx_n_s_make_locsig);
  Py_CLEAR(clear_module_state->__pyx_n_s_make_oscil_bank);
  Py_CLEAR(clear_module_state->__pyx_n_s_make_piano_ampfun);
  Py_CLEAR(clear_module_state->__pyx_n_s_math);
  Py_CLEAR(clear_module_state->__pyx_n_s_name);
  Py_CLEAR(clear_module_state->__pyx_n_s_np);
  Py_CLEAR(clear_module_state->__pyx_n_s_numpy);
  Py_CLEAR(clear_module_state->__pyx_kp_u_numpy_core_multiarray_failed_to);
  Py_CLEAR(clear_module_state->__pyx_kp_u_numpy_core_umath_failed_to_impor);
  Py_CLEAR(clear_module_state->__pyx_n_s_pfreq);
  Py_CLEAR(clear_module_state->__pyx_n_s_piano_attack_duration);
  Py_CLEAR(clear_module_state->__pyx_n_s_play);
  Py_CLEAR(clear_module_state->__pyx_n_s_pysndlib);
  Py_CLEAR(clear_module_state->__pyx_n_s_pysndlib_clm);
  Py_CLEAR(clear_module_state->__pyx_n_s_pysndlib_instruments_lbj_piano);
  Py_CLEAR(clear_module_state->__pyx_n_s_pyx_vtable);
  Py_CLEAR(clear_module_state->__pyx_n_s_random);
  Py_CLEAR(clear_module_state->__pyx_n_s_range);
  Py_CLEAR(clear_module_state->__pyx_n_s_releaseAmp);
  Py_CLEAR(clear_module_state->__pyx_n_s_reverb_amount);
  Py_CLEAR(clear_module_state->__pyx_n_s_round);
  Py_CLEAR(clear_module_state->__pyx_n_s_scaler);
  Py_CLEAR(clear_module_state->__pyx_n_s_spec);
  Py_CLEAR(clear_module_state->__pyx_kp_s_src_pysndlib_instruments_lbj_pia);
  Py_CLEAR(clear_module_state->__pyx_n_s_stable);
  Py_CLEAR(clear_module_state->__pyx_n_s_startime);
  Py_CLEAR(clear_module_state->__pyx_n_s_statistics);
  Py_CLEAR(clear_module_state->__pyx_n_s_test);
  Py_CLEAR(clear_module_state->__pyx_n_s_zeros);
  Py_CLEAR(clear_module_state->__pyx_float__4);
  Py_CLEAR(clear_module_state->__pyx_float__34);
  Py_CLEAR(clear_module_state->__pyx_float_1_0);
  Py_CLEAR(clear_module_state->__pyx_float_0_97);
  Py_CLEAR(clear_module_state->__pyx_float_0_98);
  Py_CLEAR(clear_module_state->__pyx_float_0_99);
  Py_CLEAR(clear_module_state->__pyx_float_1_00);
  Py_CLEAR(clear_module_state->__pyx_float_1_01);
  Py_CLEAR(clear_module_state->__pyx_float_1_02);
  Py_CLEAR(clear_module_state->__pyx_float_1_97);
  Py_CLEAR(clear_module_state->__pyx_float_1_98);
  Py_CLEAR(clear_module_state->__pyx_float_1_99);
  Py_CLEAR(clear_module_state->__pyx_float_2_00);
  Py_CLEAR(clear_module_state->__pyx_float_2_01);
  Py_CLEAR(clear_module_state->__pyx_float_2_02);
  Py_CLEAR(clear_module_state->__pyx_float_2_03);
  Py_CLEAR(clear_module_state->__pyx_float_2_07);
  Py_CLEAR(clear_module_state->__pyx_float_2_09);
  Py_CLEAR(clear_module_state->__pyx_float_2_96);
  Py_CLEAR(clear_module_state->__pyx_float_2_97);
  Py_CLEAR(clear_module_state->__pyx_float_2_98);
  Py_CLEAR(clear_module_state->__pyx_float_2_99);
  Py_CLEAR(clear_module_state->__pyx_float_3_00);
  Py_CLEAR(clear_module_state->__pyx_float_3_01);
  Py_CLEAR(clear_module_state->__pyx_float_3_02);
  Py_CLEAR(clear_module_state->__pyx_float_3_03);
  Py_CLEAR(clear_module_state->__pyx_float_3_04);
  Py_CLEAR(clear_module_state->__pyx_float_3_09);
  Py_CLEAR(clear_module_state->__pyx_float_3_11);
  Py_CLEAR(clear_module_state->__pyx_float_3_12);
  Py_CLEAR(clear_module_state->__pyx_float_3_15);
  Py_CLEAR(clear_module_state->__pyx_float_3_19);
  Py_CLEAR(clear_module_state->__pyx_float_3_95);
  Py_CLEAR(clear_module_state->__pyx_float_3_97);
  Py_CLEAR(clear_module_state->__pyx_float_3_98);
  Py_CLEAR(clear_module_state->__pyx_float_3_99);
  Py_CLEAR(clear_module_state->__pyx_float_4_00);
  Py_CLEAR(clear_module_state->__pyx_float_4_01);
  Py_CLEAR(clear_module_state->__pyx_float_4_02);
  Py_CLEAR(clear_module_state->__pyx_float_4_03);
  Py_CLEAR(clear_module_state->__pyx_float_4_04);
  Py_CLEAR(clear_module_state->__pyx_float_4_05);
  Py_CLEAR(clear_module_state->__pyx_float_4_06);
  Py_CLEAR(clear_module_state->__pyx_float_4_10);
  Py_CLEAR(clear_module_state->__pyx_float_4_12);
  Py_CLEAR(clear_module_state->__pyx_float_4_13);
  Py_CLEAR(clear_module_state->__pyx_float_4_17);
  Py_CLEAR(clear_module_state->__pyx_float_4_19);
  Py_CLEAR(clear_module_state->__pyx_float_4_26);
  Py_CLEAR(clear_module_state->__pyx_float_4_94);
  Py_CLEAR(clear_module_state->__pyx_float_4_95);
  Py_CLEAR(clear_module_state->__pyx_float_4_96);
  Py_CLEAR(clear_module_state->__pyx_float_4_97);
  Py_CLEAR(clear_module_state->__pyx_float_4_98);
  Py_CLEAR(clear_module_state->__pyx_float_4_99);
  Py_CLEAR(clear_module_state->__pyx_float_5_00);
  Py_CLEAR(clear_module_state->__pyx_float_5_01);
  Py_CLEAR(clear_module_state->__pyx_float_5_02);
  Py_CLEAR(clear_module_state->__pyx_float_5_03);
  Py_CLEAR(clear_module_state->__pyx_float_5_04);
  Py_CLEAR(clear_module_state->__pyx_float_5_05);
  Py_CLEAR(clear_module_state->__pyx_float_5_07);
  Py_CLEAR(clear_module_state->__pyx_float_5_09);
  Py_CLEAR(clear_module_state->__pyx_float_5_10);
  Py_CLEAR(clear_module_state->__pyx_float_5_11);
  Py_CLEAR(clear_module_state->__pyx_float_5_12);
  Py_CLEAR(clear_module_state->__pyx_float_5_14);
  Py_CLEAR(clear_module_state->__pyx_float_5_15);
  Py_CLEAR(clear_module_state->__pyx_float_5_19);
  Py_CLEAR(clear_module_state->__pyx_float_5_20);
  Py_CLEAR(clear_module_state->__pyx_float_5_22);
  Py_CLEAR(clear_module_state->__pyx_float_5_95);
  Py_CLEAR(clear_module_state->__pyx_float_5_96);
  Py_CLEAR(clear_module_state->__pyx_float_5_97);
  Py_CLEAR(clear_module_state->__pyx_float_5_98);
  Py_CLEAR(clear_module_state->__pyx_float_5_99);
  Py_CLEAR(clear_module_state->__pyx_float_6_00);
  Py_CLEAR(clear_module_state->__pyx_float_6_01);
  Py_CLEAR(clear_module_state->__pyx_float_6_02);
  Py_CLEAR(clear_module_state->__pyx_float_6_03);
  Py_CLEAR(clear_module_state->__pyx_float_6_04);
  Py_CLEAR(clear_module_state->__pyx_float_6_06);
  Py_CLEAR(clear_module_state->__pyx_float_6_07);
  Py_CLEAR(clear_module_state->__pyx_float_6_08);
  Py_CLEAR(clear_module_state->__pyx_float_6_09);
  Py_CLEAR(clear_module_state->__pyx_float_6_12);
  Py_CLEAR(clear_module_state->__pyx_float_6_13);
  Py_CLEAR(clear_module_state->__pyx_float_6_14);
  Py_CLEAR(clear_module_state->__pyx_float_6_16);
  Py_CLEAR(clear_module_state->__pyx_float_6_17);
  Py_CLEAR(clear_module_state->__pyx_float_6_20);
  Py_CLEAR(clear_module_state->__pyx_float_6_22);
  Py_CLEAR(clear_module_state->__pyx_float_6_26);
  Py_CLEAR(clear_module_state->__pyx_float_6_27);
  Py_CLEAR(clear_module_state->__pyx_float_6_32);
  Py_CLEAR(clear_module_state->__pyx_float_6_35);
  Py_CLEAR(clear_module_state->__pyx_float_6_94);
  Py_CLEAR(clear_module_state->__pyx_float_6_95);
  Py_CLEAR(clear_module_state->__pyx_float_6_96);
  Py_CLEAR(clear_module_state->__pyx_float_6_97);
  Py_CLEAR(clear_module_state->__pyx_float_6_98);
  Py_CLEAR(clear_module_state->__pyx_float_6_99);
  Py_CLEAR(clear_module_state->__pyx_float_7_00);
  Py_CLEAR(clear_module_state->__pyx_float_7_01);
  Py_CLEAR(clear_module_state->__pyx_float_7_03);
  Py_CLEAR(clear_module_state->__pyx_float_7_04);
  Py_CLEAR(clear_module_state->__pyx_float_7_05);
  Py_CLEAR(clear_module_state->__pyx_float_7_06);
  Py_CLEAR(clear_module_state->__pyx_float_7_07);
  Py_CLEAR(clear_module_state->__pyx_float_7_09);
  Py_CLEAR(clear_module_state->__pyx_float_7_11);
  Py_CLEAR(clear_module_state->__pyx_float_7_12);
  Py_CLEAR(clear_module_state->__pyx_float_7_13);
  Py_CLEAR(clear_module_state->__pyx_float_7_15);
  Py_CLEAR(clear_module_state->__pyx_float_7_19);
  Py_CLEAR(clear_module_state->__pyx_float_7_22);
  Py_CLEAR(clear_module_state->__pyx_float_7_23);
  Py_CLEAR(clear_module_state->__pyx_float_7_25);
  Py_CLEAR(clear_module_state->__pyx_float_7_26);
  Py_CLEAR(clear_module_state->__pyx_float_7_27);
  Py_CLEAR(clear_module_state->__pyx_float_7_32);
  Py_CLEAR(clear_module_state->__pyx_float_7_33);
  Py_CLEAR(clear_module_state->__pyx_float_7_35);
  Py_CLEAR(clear_module_state->__pyx_float_7_41);
  Py_CLEAR(clear_module_state->__pyx_float_7_59);
  Py_CLEAR(clear_module_state->__pyx_float_7_93);
  Py_CLEAR(clear_module_state->__pyx_float_7_94);
  Py_CLEAR(clear_module_state->__pyx_float_7_96);
  Py_CLEAR(clear_module_state->__pyx_float_7_97);
  Py_CLEAR(clear_module_state->__pyx_float_7_98);
  Py_CLEAR(clear_module_state->__pyx_float_7_99);
  Py_CLEAR(clear_module_state->__pyx_float_8_00);
  Py_CLEAR(clear_module_state->__pyx_float_8_01);
  Py_CLEAR(clear_module_state->__pyx_float_8_02);
  Py_CLEAR(clear_module_state->__pyx_float_8_04);
  Py_CLEAR(clear_module_state->__pyx_float_8_05);
  Py_CLEAR(clear_module_state->__pyx_float_8_06);
  Py_CLEAR(clear_module_state->__pyx_float_8_08);
  Py_CLEAR(clear_module_state->__pyx_float_8_09);
  Py_CLEAR(clear_module_state->__pyx_float_8_11);
  Py_CLEAR(clear_module_state->__pyx_float_8_12);
  Py_CLEAR(clear_module_state->__pyx_float_8_14);
  Py_CLEAR(clear_module_state->__pyx_float_8_17);
  Py_CLEAR(clear_module_state->__pyx_float_8_19);
  Py_CLEAR(clear_module_state->__pyx_float_8_20);
  Py_CLEAR(clear_module_state->__pyx_float_8_22);
  Py_CLEAR(clear_module_state->__pyx_float_8_29);
  Py_CLEAR(clear_module_state->__pyx_float_8_30);
  Py_CLEAR(clear_module_state->__pyx_float_8_32);
  Py_CLEAR(clear_module_state->__pyx_float_8_33);
  Py_CLEAR(clear_module_state->__pyx_float_8_36);
  Py_CLEAR(clear_module_state->__pyx_float_8_39);
  Py_CLEAR(clear_module_state->__pyx_float_8_40);
  Py_CLEAR(clear_module_state->__pyx_float_8_46);
  Py_CLEAR(clear_module_state->__pyx_float_8_50);
  Py_CLEAR(clear_module_state->__pyx_float_8_63);
  Py_CLEAR(clear_module_state->__pyx_float_8_94);
  Py_CLEAR(clear_module_state->__pyx_float_8_95);
  Py_CLEAR(clear_module_state->__pyx_float_8_96);
  Py_CLEAR(clear_module_state->__pyx_float_8_97);
  Py_CLEAR(clear_module_state->__pyx_float_8_98);
  Py_CLEAR(clear_module_state->__pyx_float_8_99);
  Py_CLEAR(clear_module_state->__pyx_float_9_00);
  Py_CLEAR(clear_module_state->__pyx_float_9_01);
  Py_CLEAR(clear_module_state->__pyx_float_9_04);
  Py_CLEAR(clear_module_state->__pyx_float_9_07);
  Py_CLEAR(clear_module_state->__pyx_float_9_08);
  Py_CLEAR(clear_module_state->__pyx_float_9_10);
  Py_CLEAR(clear_module_state->__pyx_float_9_11);
  Py_CLEAR(clear_module_state->__pyx_float_9_13);
  Py_CLEAR(clear_module_state->__pyx_float_9_15);
  Py_CLEAR(clear_module_state->__pyx_float_9_16);
  Py_CLEAR(clear_module_state->__pyx_float_9_19);
  Py_CLEAR(clear_module_state->__pyx_float_9_21);
  Py_CLEAR(clear_module_state->__pyx_float_9_23);
  Py_CLEAR(clear_module_state->__pyx_float_9_25);
  Py_CLEAR(clear_module_state->__pyx_float_9_27);
  Py_CLEAR(clear_module_state->__pyx_float_9_29);
  Py_CLEAR(clear_module_state->__pyx_float_9_32);
  Py_CLEAR(clear_module_state->__pyx_float_9_43);
  Py_CLEAR(clear_module_state->__pyx_float_9_50);
  Py_CLEAR(clear_module_state->__pyx_float_9_51);
  Py_CLEAR(clear_module_state->__pyx_float_9_53);
  Py_CLEAR(clear_module_state->__pyx_float_9_58);
  Py_CLEAR(clear_module_state->__pyx_float_9_66);
  Py_CLEAR(clear_module_state->__pyx_float_9_92);
  Py_CLEAR(clear_module_state->__pyx_float_9_95);
  Py_CLEAR(clear_module_state->__pyx_float_9_96);
  Py_CLEAR(clear_module_state->__pyx_float_9_98);
  Py_CLEAR(clear_module_state->__pyx_float_9_99);
  Py_CLEAR(clear_module_state->__pyx_float__0001);
  Py_CLEAR(clear_module_state->__pyx_float__0002);
  Py_CLEAR(clear_module_state->__pyx_float__0003);
  Py_CLEAR(clear_module_state->__pyx_float__0004);
  Py_CLEAR(clear_module_state->__pyx_float__0005);
  Py_CLEAR(clear_module_state->__pyx_float__0006);
  Py_CLEAR(clear_module_state->__pyx_float__0007);
  Py_CLEAR(clear_module_state->__pyx_float__0008);
  Py_CLEAR(clear_module_state->__pyx_float__0009);
  Py_CLEAR(clear_module_state->__pyx_float__0010);
  Py_CLEAR(clear_module_state->__pyx_float__0011);
  Py_CLEAR(clear_module_state->__pyx_float__0012);
  Py_CLEAR(clear_module_state->__pyx_float__0013);
  Py_CLEAR(clear_module_state->__pyx_float__0014);
  Py_CLEAR(clear_module_state->__pyx_float__0015);
  Py_CLEAR(clear_module_state->__pyx_float__0016);
  Py_CLEAR(clear_module_state->__pyx_float__0017);
  Py_CLEAR(clear_module_state->__pyx_float__0018);
  Py_CLEAR(clear_module_state->__pyx_float__0019);
  Py_CLEAR(clear_module_state->__pyx_float__0020);
  Py_CLEAR(clear_module_state->__pyx_float__0021);
  Py_CLEAR(clear_module_state->__pyx_float__0022);
  Py_CLEAR(clear_module_state->__pyx_float__0023);
  Py_CLEAR(clear_module_state->__pyx_float__0024);
  Py_CLEAR(clear_module_state->__pyx_float__0025);
  Py_CLEAR(clear_module_state->__pyx_float__0026);
  Py_CLEAR(clear_module_state->__pyx_float__0027);
  Py_CLEAR(clear_module_state->__pyx_float__0028);
  Py_CLEAR(clear_module_state->__pyx_float__0029);
  Py_CLEAR(clear_module_state->__pyx_float__0030);
  Py_CLEAR(clear_module_state->__pyx_float__0031);
  Py_CLEAR(clear_module_state->__pyx_float__0032);
  Py_CLEAR(clear_module_state->__pyx_float__0033);
  Py_CLEAR(clear_module_state->__pyx_float__0034);
  Py_CLEAR(clear_module_state->__pyx_float__0035);
  Py_CLEAR(clear_module_state->__pyx_float__0036);
  Py_CLEAR(clear_module_state->__pyx_float__0037);
  Py_CLEAR(clear_module_state->__pyx_float__0038);
  Py_CLEAR(clear_module_state->__pyx_float__0039);
  Py_CLEAR(clear_module_state->__pyx_float__0040);
  Py_CLEAR(clear_module_state->__pyx_float__0041);
  Py_CLEAR(clear_module_state->__pyx_float__0042);
  Py_CLEAR(clear_module_state->__pyx_float__0043);
  Py_CLEAR(clear_module_state->__pyx_float__0044);
  Py_CLEAR(clear_module_state->__pyx_float__0045);
  Py_CLEAR(clear_module_state->__pyx_float__0046);
  Py_CLEAR(clear_module_state->__pyx_float__0047);
  Py_CLEAR(clear_module_state->__pyx_float__0048);
  Py_CLEAR(clear_module_state->__pyx_float__0049);
  Py_CLEAR(clear_module_state->__pyx_float__0050);
  Py_CLEAR(clear_module_state->__pyx_float__0051);
  Py_CLEAR(clear_module_state->__pyx_float__0052);
  Py_CLEAR(clear_module_state->__pyx_float__0053);
  Py_CLEAR(clear_module_state->__pyx_float__0054);
  Py_CLEAR(clear_module_state->__pyx_float__0055);
  Py_CLEAR(clear_module_state->__pyx_float__0056);
  Py_CLEAR(clear_module_state->__pyx_float__0057);
  Py_CLEAR(clear_module_state->__pyx_float__0058);
  Py_CLEAR(clear_module_state->__pyx_float__0059);
  Py_CLEAR(clear_module_state->__pyx_float__0060);
  Py_CLEAR(clear_module_state->__pyx_float__0061);
  Py_CLEAR(clear_module_state->__pyx_float__0062);
  Py_CLEAR(clear_module_state->__pyx_float__0063);
  Py_CLEAR(clear_module_state->__pyx_float__0064);
  Py_CLEAR(clear_module_state->__pyx_float__0065);
  Py_CLEAR(clear_module_state->__pyx_float__0066);
  Py_CLEAR(clear_module_state->__pyx_float__0067);
  Py_CLEAR(clear_module_state->__pyx_float__0068);
  Py_CLEAR(clear_module_state->__pyx_float__0069);
  Py_CLEAR(clear_module_state->__pyx_float__0070);
  Py_CLEAR(clear_module_state->__pyx_float__0071);
  Py_CLEAR(clear_module_state->__pyx_float__0072);
  Py_CLEAR(clear_module_state->__pyx_float__0073);
  Py_CLEAR(clear_module_state->__pyx_float__0074);
  Py_CLEAR(clear_module_state->__pyx_float__0075);
  Py_CLEAR(clear_module_state->__pyx_float__0076);
  Py_CLEAR(clear_module_state->__pyx_float__0077);
  Py_CLEAR(clear_module_state->__pyx_float__0078);
  Py_CLEAR(clear_module_state->__pyx_float__0080);
  Py_CLEAR(clear_module_state->__pyx_float__0081);
  Py_CLEAR(clear_module_state->__pyx_float__0082);
  Py_CLEAR(clear_module_state->__pyx_float__0083);
  Py_CLEAR(clear_module_state->__pyx_float__0084);
  Py_CLEAR(clear_module_state->__pyx_float__0085);
  Py_CLEAR(clear_module_state->__pyx_float__0086);
  Py_CLEAR(clear_module_state->__pyx_float__0087);
  Py_CLEAR(clear_module_state->__pyx_float__0088);
  Py_CLEAR(clear_module_state->__pyx_float__0089);
  Py_CLEAR(clear_module_state->__pyx_float__0090);
  Py_CLEAR(clear_module_state->__pyx_float__0091);
  Py_CLEAR(clear_module_state->__pyx_float__0092);
  Py_CLEAR(clear_module_state->__pyx_float__0093);
  Py_CLEAR(clear_module_state->__pyx_float__0094);
  Py_CLEAR(clear_module_state->__pyx_float__0095);
  Py_CLEAR(clear_module_state->__pyx_float__0097);
  Py_CLEAR(clear_module_state->__pyx_float__0098);
  Py_CLEAR(clear_module_state->__pyx_float__0099);
  Py_CLEAR(clear_module_state->__pyx_float__0100);
  Py_CLEAR(clear_module_state->__pyx_float__0101);
  Py_CLEAR(clear_module_state->__pyx_float__0102);
  Py_CLEAR(clear_module_state->__pyx_float__0103);
  Py_CLEAR(clear_module_state->__pyx_float__0104);
  Py_CLEAR(clear_module_state->__pyx_float__0105);
  Py_CLEAR(clear_module_state->__pyx_float__0106);
  Py_CLEAR(clear_module_state->__pyx_float__0107);
  Py_CLEAR(clear_module_state->__pyx_float__0108);
  Py_CLEAR(clear_module_state->__pyx_float__0109);
  Py_CLEAR(clear_module_state->__pyx_float__0111);
  Py_CLEAR(clear_module_state->__pyx_float__0112);
  Py_CLEAR(clear_module_state->__pyx_float__0113);
  Py_CLEAR(clear_module_state->__pyx_float__0114);
  Py_CLEAR(clear_module_state->__pyx_float__0115);
  Py_CLEAR(clear_module_state->__pyx_float__0116);
  Py_CLEAR(clear_module_state->__pyx_float__0117);
  Py_CLEAR(clear_module_state->__pyx_float__0118);
  Py_CLEAR(clear_module_state->__pyx_float__0119);
  Py_CLEAR(clear_module_state->__pyx_float__0120);
  Py_CLEAR(clear_module_state->__pyx_float__0121);
  Py_CLEAR(clear_module_state->__pyx_float__0122);
  Py_CLEAR(clear_module_state->__pyx_float__0123);
  Py_CLEAR(clear_module_state->__pyx_float__0124);
  Py_CLEAR(clear_module_state->__pyx_float__0125);
  Py_CLEAR(clear_module_state->__pyx_float__0126);
  Py_CLEAR(clear_module_state->__pyx_float__0127);
  Py_CLEAR(clear_module_state->__pyx_float__0128);
  Py_CLEAR(clear_module_state->__pyx_float__0129);
  Py_CLEAR(clear_module_state->__pyx_float__0132);
  Py_CLEAR(clear_module_state->__pyx_float__0133);
  Py_CLEAR(clear_module_state->__pyx_float__0134);
  Py_CLEAR(clear_module_state->__pyx_float__0135);
  Py_CLEAR(clear_module_state->__pyx_float__0136);
  Py_CLEAR(clear_module_state->__pyx_float__0137);
  Py_CLEAR(clear_module_state->__pyx_float__0138);
  Py_CLEAR(clear_module_state->__pyx_float__0139);
  Py_CLEAR(clear_module_state->__pyx_float__0140);
  Py_CLEAR(clear_module_state->__pyx_float__0141);
  Py_CLEAR(clear_module_state->__pyx_float__0142);
  Py_CLEAR(clear_module_state->__pyx_float__0144);
  Py_CLEAR(clear_module_state->__pyx_float__0145);
  Py_CLEAR(clear_module_state->__pyx_float__0147);
  Py_CLEAR(clear_module_state->__pyx_float__0148);
  Py_CLEAR(clear_module_state->__pyx_float__0151);
  Py_CLEAR(clear_module_state->__pyx_float__0152);
  Py_CLEAR(clear_module_state->__pyx_float__0153);
  Py_CLEAR(clear_module_state->__pyx_float__0154);
  Py_CLEAR(clear_module_state->__pyx_float__0155);
  Py_CLEAR(clear_module_state->__pyx_float__0156);
  Py_CLEAR(clear_module_state->__pyx_float__0158);
  Py_CLEAR(clear_module_state->__pyx_float__0159);
  Py_CLEAR(clear_module_state->__pyx_float__0160);
  Py_CLEAR(clear_module_state->__pyx_float__0161);
  Py_CLEAR(clear_module_state->__pyx_float__0162);
  Py_CLEAR(clear_module_state->__pyx_float__0163);
  Py_CLEAR(clear_module_state->__pyx_float__0165);
  Py_CLEAR(clear_module_state->__pyx_float__0166);
  Py_CLEAR(clear_module_state->__pyx_float__0167);
  Py_CLEAR(clear_module_state->__pyx_float__0170);
  Py_CLEAR(clear_module_state->__pyx_float__0171);
  Py_CLEAR(clear_module_state->__pyx_float__0172);
  Py_CLEAR(clear_module_state->__pyx_float__0173);
  Py_CLEAR(clear_module_state->__pyx_float__0174);
  Py_CLEAR(clear_module_state->__pyx_float__0175);
  Py_CLEAR(clear_module_state->__pyx_float__0177);
  Py_CLEAR(clear_module_state->__pyx_float__0178);
  Py_CLEAR(clear_module_state->__pyx_float__0179);
  Py_CLEAR(clear_module_state->__pyx_float__0181);
  Py_CLEAR(clear_module_state->__pyx_float__0182);
  Py_CLEAR(clear_module_state->__pyx_float__0183);
  Py_CLEAR(clear_module_state->__pyx_float__0184);
  Py_CLEAR(clear_module_state->__pyx_float__0185);
  Py_CLEAR(clear_module_state->__pyx_float__0186);
  Py_CLEAR(clear_module_state->__pyx_float__0189);
  Py_CLEAR(clear_module_state->__pyx_float__0190);
  Py_CLEAR(clear_module_state->__pyx_float__0191);
  Py_CLEAR(clear_module_state->__pyx_float__0192);
  Py_CLEAR(clear_module_state->__pyx_float__0194);
  Py_CLEAR(clear_module_state->__pyx_float__0196);
  Py_CLEAR(clear_module_state->__pyx_float__0197);
  Py_CLEAR(clear_module_state->__pyx_float__0201);
  Py_CLEAR(clear_module_state->__pyx_float__0202);
  Py_CLEAR(clear_module_state->__pyx_float__0203);
  Py_CLEAR(clear_module_state->__pyx_float__0206);
  Py_CLEAR(clear_module_state->__pyx_float__0207);
  Py_CLEAR(clear_module_state->__pyx_float__0209);
  Py_CLEAR(clear_module_state->__pyx_float__0210);
  Py_CLEAR(clear_module_state->__pyx_float__0211);
  Py_CLEAR(clear_module_state->__pyx_float__0212);
  Py_CLEAR(clear_module_state->__pyx_float__0213);
  Py_CLEAR(clear_module_state->__pyx_float__0215);
  Py_CLEAR(clear_module_state->__pyx_float__0217);
  Py_CLEAR(clear_module_state->__pyx_float__0218);
  Py_CLEAR(clear_module_state->__pyx_float__0221);
  Py_CLEAR(clear_module_state->__pyx_float__0222);
  Py_CLEAR(clear_module_state->__pyx_float__0225);
  Py_CLEAR(clear_module_state->__pyx_float__0227);
  Py_CLEAR(clear_module_state->__pyx_float__0228);
  Py_CLEAR(clear_module_state->__pyx_float__0229);
  Py_CLEAR(clear_module_state->__pyx_float__0230);
  Py_CLEAR(clear_module_state->__pyx_float__0232);
  Py_CLEAR(clear_module_state->__pyx_float__0233);
  Py_CLEAR(clear_module_state->__pyx_float__0234);
  Py_CLEAR(clear_module_state->__pyx_float__0235);
  Py_CLEAR(clear_module_state->__pyx_float__0236);
  Py_CLEAR(clear_module_state->__pyx_float__0237);
  Py_CLEAR(clear_module_state->__pyx_float__0238);
  Py_CLEAR(clear_module_state->__pyx_float__0240);
  Py_CLEAR(clear_module_state->__pyx_float__0241);
  Py_CLEAR(clear_module_state->__pyx_float__0242);
  Py_CLEAR(clear_module_state->__pyx_float__0245);
  Py_CLEAR(clear_module_state->__pyx_float__0246);
  Py_CLEAR(clear_module_state->__pyx_float__0247);
  Py_CLEAR(clear_module_state->__pyx_float__0248);
  Py_CLEAR(clear_module_state->__pyx_float__0249);
  Py_CLEAR(clear_module_state->__pyx_float__0250);
  Py_CLEAR(clear_module_state->__pyx_float__0252);
  Py_CLEAR(clear_module_state->__pyx_float__0256);
  Py_CLEAR(clear_module_state->__pyx_float__0257);
  Py_CLEAR(clear_module_state->__pyx_float__0258);
  Py_CLEAR(clear_module_state->__pyx_float__0259);
  Py_CLEAR(clear_module_state->__pyx_float__0260);
  Py_CLEAR(clear_module_state->__pyx_float__0264);
  Py_CLEAR(clear_module_state->__pyx_float__0265);
  Py_CLEAR(clear_module_state->__pyx_float__0266);
  Py_CLEAR(clear_module_state->__pyx_float__0268);
  Py_CLEAR(clear_module_state->__pyx_float__0272);
  Py_CLEAR(clear_module_state->__pyx_float__0274);
  Py_CLEAR(clear_module_state->__pyx_float__0276);
  Py_CLEAR(clear_module_state->__pyx_float__0278);
  Py_CLEAR(clear_module_state->__pyx_float__0279);
  Py_CLEAR(clear_module_state->__pyx_float__0280);
  Py_CLEAR(clear_module_state->__pyx_float__0284);
  Py_CLEAR(clear_module_state->__pyx_float__0285);
  Py_CLEAR(clear_module_state->__pyx_float__0287);
  Py_CLEAR(clear_module_state->__pyx_float__0289);
  Py_CLEAR(clear_module_state->__pyx_float__0290);
  Py_CLEAR(clear_module_state->__pyx_float__0291);
  Py_CLEAR(clear_module_state->__pyx_float__0292);
  Py_CLEAR(clear_module_state->__pyx_float__0293);
  Py_CLEAR(clear_module_state->__pyx_float__0297);
  Py_CLEAR(clear_module_state->__pyx_float__0298);
  Py_CLEAR(clear_module_state->__pyx_float__0299);
  Py_CLEAR(clear_module_state->__pyx_float__0300);
  Py_CLEAR(clear_module_state->__pyx_float__0301);
  Py_CLEAR(clear_module_state->__pyx_float__0305);
  Py_CLEAR(clear_module_state->__pyx_float__0306);
  Py_CLEAR(clear_module_state->__pyx_float__0307);
  Py_CLEAR(clear_module_state->__pyx_float__0309);
  Py_CLEAR(clear_module_state->__pyx_float__0310);
  Py_CLEAR(clear_module_state->__pyx_float__0311);
  Py_CLEAR(clear_module_state->__pyx_float__0313);
  Py_CLEAR(clear_module_state->__pyx_float__0314);
  Py_CLEAR(clear_module_state->__pyx_float__0315);
  Py_CLEAR(clear_module_state->__pyx_float__0319);
  Py_CLEAR(clear_module_state->__pyx_float__0322);
  Py_CLEAR(clear_module_state->__pyx_float__0324);
  Py_CLEAR(clear_module_state->__pyx_float__0326);
  Py_CLEAR(clear_module_state->__pyx_float__0328);
  Py_CLEAR(clear_module_state->__pyx_float__0329);
  Py_CLEAR(clear_module_state->__pyx_float__0332);
  Py_CLEAR(clear_module_state->__pyx_float__0334);
  Py_CLEAR(clear_module_state->__pyx_float__0336);
  Py_CLEAR(clear_module_state->__pyx_float__0340);
  Py_CLEAR(clear_module_state->__pyx_float__0342);
  Py_CLEAR(clear_module_state->__pyx_float__0345);
  Py_CLEAR(clear_module_state->__pyx_float__0349);
  Py_CLEAR(clear_module_state->__pyx_float__0353);
  Py_CLEAR(clear_module_state->__pyx_float__0355);
  Py_CLEAR(clear_module_state->__pyx_float__0363);
  Py_CLEAR(clear_module_state->__pyx_float__0364);
  Py_CLEAR(clear_module_state->__pyx_float__0365);
  Py_CLEAR(clear_module_state->__pyx_float__0367);
  Py_CLEAR(clear_module_state->__pyx_float__0368);
  Py_CLEAR(clear_module_state->__pyx_float__0369);
  Py_CLEAR(clear_module_state->__pyx_float__0370);
  Py_CLEAR(clear_module_state->__pyx_float__0373);
  Py_CLEAR(clear_module_state->__pyx_float__0381);
  Py_CLEAR(clear_module_state->__pyx_float__0387);
  Py_CLEAR(clear_module_state->__pyx_float__0389);
  Py_CLEAR(clear_module_state->__pyx_float__0390);
  Py_CLEAR(clear_module_state->__pyx_float__0392);
  Py_CLEAR(clear_module_state->__pyx_float__0396);
  Py_CLEAR(clear_module_state->__pyx_float__0402);
  Py_CLEAR(clear_module_state->__pyx_float__0403);
  Py_CLEAR(clear_module_state->__pyx_float__0404);
  Py_CLEAR(clear_module_state->__pyx_float__0410);
  Py_CLEAR(clear_module_state->__pyx_float__0411);
  Py_CLEAR(clear_module_state->__pyx_float__0415);
  Py_CLEAR(clear_module_state->__pyx_float__0418);
  Py_CLEAR(clear_module_state->__pyx_float__0420);
  Py_CLEAR(clear_module_state->__pyx_float__0421);
  Py_CLEAR(clear_module_state->__pyx_float__0423);
  Py_CLEAR(clear_module_state->__pyx_float__0425);
  Py_CLEAR(clear_module_state->__pyx_float__0437);
  Py_CLEAR(clear_module_state->__pyx_float__0445);
  Py_CLEAR(clear_module_state->__pyx_float__0448);
  Py_CLEAR(clear_module_state->__pyx_float__0449);
  Py_CLEAR(clear_module_state->__pyx_float__0465);
  Py_CLEAR(clear_module_state->__pyx_float__0472);
  Py_CLEAR(clear_module_state->__pyx_float__0473);
  Py_CLEAR(clear_module_state->__pyx_float__0475);
  Py_CLEAR(clear_module_state->__pyx_float__0482);
  Py_CLEAR(clear_module_state->__pyx_float__0492);
  Py_CLEAR(clear_module_state->__pyx_float__0493);
  Py_CLEAR(clear_module_state->__pyx_float__0497);
  Py_CLEAR(clear_module_state->__pyx_float__0498);
  Py_CLEAR(clear_module_state->__pyx_float__0500);
  Py_CLEAR(clear_module_state->__pyx_float__0503);
  Py_CLEAR(clear_module_state->__pyx_float__0506);
  Py_CLEAR(clear_module_state->__pyx_float__0512);
  Py_CLEAR(clear_module_state->__pyx_float__0514);
  Py_CLEAR(clear_module_state->__pyx_float__0516);
  Py_CLEAR(clear_module_state->__pyx_float__0517);
  Py_CLEAR(clear_module_state->__pyx_float__0522);
  Py_CLEAR(clear_module_state->__pyx_float__0523);
  Py_CLEAR(clear_module_state->__pyx_float__0525);
  Py_CLEAR(clear_module_state->__pyx_float__0578);
  Py_CLEAR(clear_module_state->__pyx_float__0593);
  Py_CLEAR(clear_module_state->__pyx_float__0596);
  Py_CLEAR(clear_module_state->__pyx_float__0613);
  Py_CLEAR(clear_module_state->__pyx_float__0621);
  Py_CLEAR(clear_module_state->__pyx_float__0629);
  Py_CLEAR(clear_module_state->__pyx_float__0650);
  Py_CLEAR(clear_module_state->__pyx_float__0654);
  Py_CLEAR(clear_module_state->__pyx_float__0669);
  Py_CLEAR(clear_module_state->__pyx_float__0674);
  Py_CLEAR(clear_module_state->__pyx_float__0676);
  Py_CLEAR(clear_module_state->__pyx_float__0678);
  Py_CLEAR(clear_module_state->__pyx_float__0683);
  Py_CLEAR(clear_module_state->__pyx_float__0692);
  Py_CLEAR(clear_module_state->__pyx_float__0698);
  Py_CLEAR(clear_module_state->__pyx_float__0708);
  Py_CLEAR(clear_module_state->__pyx_float__0714);
  Py_CLEAR(clear_module_state->__pyx_float__0719);
  Py_CLEAR(clear_module_state->__pyx_float__0726);
  Py_CLEAR(clear_module_state->__pyx_float__0727);
  Py_CLEAR(clear_module_state->__pyx_float__0752);
  Py_CLEAR(clear_module_state->__pyx_float__0763);
  Py_CLEAR(clear_module_state->__pyx_float__0765);
  Py_CLEAR(clear_module_state->__pyx_float__0773);
  Py_CLEAR(clear_module_state->__pyx_float__0813);
  Py_CLEAR(clear_module_state->__pyx_float__0820);
  Py_CLEAR(clear_module_state->__pyx_float__0835);
  Py_CLEAR(clear_module_state->__pyx_float__0841);
  Py_CLEAR(clear_module_state->__pyx_float__0846);
  Py_CLEAR(clear_module_state->__pyx_float__0877);
  Py_CLEAR(clear_module_state->__pyx_float__0909);
  Py_CLEAR(clear_module_state->__pyx_float__0919);
  Py_CLEAR(clear_module_state->__pyx_float__0920);
  Py_CLEAR(clear_module_state->__pyx_float__0921);
  Py_CLEAR(clear_module_state->__pyx_float__0933);
  Py_CLEAR(clear_module_state->__pyx_float__0938);
  Py_CLEAR(clear_module_state->__pyx_float__0951);
  Py_CLEAR(clear_module_state->__pyx_float__0963);
  Py_CLEAR(clear_module_state->__pyx_float__0967);
  Py_CLEAR(clear_module_state->__pyx_float__0976);
  Py_CLEAR(clear_module_state->__pyx_float__0982);
  Py_CLEAR(clear_module_state->__pyx_float__0985);
  Py_CLEAR(clear_module_state->__pyx_float__1008);
  Py_CLEAR(clear_module_state->__pyx_float__1011);
  Py_CLEAR(clear_module_state->__pyx_float__1015);
  Py_CLEAR(clear_module_state->__pyx_float__1018);
  Py_CLEAR(clear_module_state->__pyx_float__1054);
  Py_CLEAR(clear_module_state->__pyx_float__1066);
  Py_CLEAR(clear_module_state->__pyx_float__1085);
  Py_CLEAR(clear_module_state->__pyx_float__1126);
  Py_CLEAR(clear_module_state->__pyx_float__1132);
  Py_CLEAR(clear_module_state->__pyx_float__1174);
  Py_CLEAR(clear_module_state->__pyx_float__1193);
  Py_CLEAR(clear_module_state->__pyx_float__1238);
  Py_CLEAR(clear_module_state->__pyx_float__1243);
  Py_CLEAR(clear_module_state->__pyx_float__1273);
  Py_CLEAR(clear_module_state->__pyx_float__1296);
  Py_CLEAR(clear_module_state->__pyx_float__1304);
  Py_CLEAR(clear_module_state->__pyx_float__1311);
  Py_CLEAR(clear_module_state->__pyx_float__1357);
  Py_CLEAR(clear_module_state->__pyx_float__1400);
  Py_CLEAR(clear_module_state->__pyx_float__1440);
  Py_CLEAR(clear_module_state->__pyx_float__1441);
  Py_CLEAR(clear_module_state->__pyx_float__1486);
  Py_CLEAR(clear_module_state->__pyx_float__1541);
  Py_CLEAR(clear_module_state->__pyx_float__1555);
  Py_CLEAR(clear_module_state->__pyx_float__1611);
  Py_CLEAR(clear_module_state->__pyx_float__1655);
  Py_CLEAR(clear_module_state->__pyx_float__1715);
  Py_CLEAR(clear_module_state->__pyx_float__1821);
  Py_CLEAR(clear_module_state->__pyx_float__1822);
  Py_CLEAR(clear_module_state->__pyx_float__1868);
  Py_CLEAR(clear_module_state->__pyx_float__1889);
  Py_CLEAR(clear_module_state->__pyx_float__1919);
  Py_CLEAR(clear_module_state->__pyx_float__1933);
  Py_CLEAR(clear_module_state->__pyx_float__2095);
  Py_CLEAR(clear_module_state->__pyx_float__2113);
  Py_CLEAR(clear_module_state->__pyx_float__2185);
  Py_CLEAR(clear_module_state->__pyx_float__2270);
  Py_CLEAR(clear_module_state->__pyx_float__2388);
  Py_CLEAR(clear_module_state->__pyx_float__2455);
  Py_CLEAR(clear_module_state->__pyx_float__2598);
  Py_CLEAR(clear_module_state->__pyx_float__2735);
  Py_CLEAR(clear_module_state->__pyx_float__3637);
  Py_CLEAR(clear_module_state->__pyx_float_10_00);
  Py_CLEAR(clear_module_state->__pyx_float_10_01);
  Py_CLEAR(clear_module_state->__pyx_float_10_02);
  Py_CLEAR(clear_module_state->__pyx_float_10_03);
  Py_CLEAR(clear_module_state->__pyx_float_10_04);
  Py_CLEAR(clear_module_state->__pyx_float_10_05);
  Py_CLEAR(clear_module_state->__pyx_float_10_08);
  Py_CLEAR(clear_module_state->__pyx_float_10_10);
  Py_CLEAR(clear_module_state->__pyx_float_10_11);
  Py_CLEAR(clear_module_state->__pyx_float_10_12);
  Py_CLEAR(clear_module_state->__pyx_float_10_15);
  Py_CLEAR(clear_module_state->__pyx_float_10_17);
  Py_CLEAR(clear_module_state->__pyx_float_10_18);
  Py_CLEAR(clear_module_state->__pyx_float_10_22);
  Py_CLEAR(clear_module_state->__pyx_float_10_23);
  Py_CLEAR(clear_module_state->__pyx_float_10_25);
  Py_CLEAR(clear_module_state->__pyx_float_10_30);
  Py_CLEAR(clear_module_state->__pyx_float_10_33);
  Py_CLEAR(clear_module_state->__pyx_float_10_36);
  Py_CLEAR(clear_module_state->__pyx_float_10_38);
  Py_CLEAR(clear_module_state->__pyx_float_10_40);
  Py_CLEAR(clear_module_state->__pyx_float_10_45);
  Py_CLEAR(clear_module_state->__pyx_float_10_59);
  Py_CLEAR(clear_module_state->__pyx_float_10_69);
  Py_CLEAR(clear_module_state->__pyx_float_10_93);
  Py_CLEAR(clear_module_state->__pyx_float_10_96);
  Py_CLEAR(clear_module_state->__pyx_float_10_98);
  Py_CLEAR(clear_module_state->__pyx_float_10_99);
  Py_CLEAR(clear_module_state->__pyx_float_11_00);
  Py_CLEAR(clear_module_state->__pyx_float_11_01);
  Py_CLEAR(clear_module_state->__pyx_float_11_02);
  Py_CLEAR(clear_module_state->__pyx_float_11_03);
  Py_CLEAR(clear_module_state->__pyx_float_11_04);
  Py_CLEAR(clear_module_state->__pyx_float_11_05);
  Py_CLEAR(clear_module_state->__pyx_float_11_07);
  Py_CLEAR(clear_module_state->__pyx_float_11_08);
  Py_CLEAR(clear_module_state->__pyx_float_11_11);
  Py_CLEAR(clear_module_state->__pyx_float_11_14);
  Py_CLEAR(clear_module_state->__pyx_float_11_15);
  Py_CLEAR(clear_module_state->__pyx_float_11_17);
  Py_CLEAR(clear_module_state->__pyx_float_11_19);
  Py_CLEAR(clear_module_state->__pyx_float_11_23);
  Py_CLEAR(clear_module_state->__pyx_float_11_25);
  Py_CLEAR(clear_module_state->__pyx_float_11_30);
  Py_CLEAR(clear_module_state->__pyx_float_11_31);
  Py_CLEAR(clear_module_state->__pyx_float_11_35);
  Py_CLEAR(clear_module_state->__pyx_float_11_40);
  Py_CLEAR(clear_module_state->__pyx_float_11_44);
  Py_CLEAR(clear_module_state->__pyx_float_11_48);
  Py_CLEAR(clear_module_state->__pyx_float_11_53);
  Py_CLEAR(clear_module_state->__pyx_float_11_58);
  Py_CLEAR(clear_module_state->__pyx_float_11_75);
  Py_CLEAR(clear_module_state->__pyx_float_11_78);
  Py_CLEAR(clear_module_state->__pyx_float_11_94);
  Py_CLEAR(clear_module_state->__pyx_float_11_97);
  Py_CLEAR(clear_module_state->__pyx_float_11_98);
  Py_CLEAR(clear_module_state->__pyx_float_11_99);
  Py_CLEAR(clear_module_state->__pyx_float_12_00);
  Py_CLEAR(clear_module_state->__pyx_float_12_01);
  Py_CLEAR(clear_module_state->__pyx_float_12_02);
  Py_CLEAR(clear_module_state->__pyx_float_12_03);
  Py_CLEAR(clear_module_state->__pyx_float_12_05);
  Py_CLEAR(clear_module_state->__pyx_float_12_06);
  Py_CLEAR(clear_module_state->__pyx_float_12_07);
  Py_CLEAR(clear_module_state->__pyx_float_12_08);
  Py_CLEAR(clear_module_state->__pyx_float_12_10);
  Py_CLEAR(clear_module_state->__pyx_float_12_11);
  Py_CLEAR(clear_module_state->__pyx_float_12_12);
  Py_CLEAR(clear_module_state->__pyx_float_12_15);
  Py_CLEAR(clear_module_state->__pyx_float_12_18);
  Py_CLEAR(clear_module_state->__pyx_float_12_20);
  Py_CLEAR(clear_module_state->__pyx_float_12_24);
  Py_CLEAR(clear_module_state->__pyx_float_12_26);
  Py_CLEAR(clear_module_state->__pyx_float_12_30);
  Py_CLEAR(clear_module_state->__pyx_float_12_32);
  Py_CLEAR(clear_module_state->__pyx_float_12_33);
  Py_CLEAR(clear_module_state->__pyx_float_12_40);
  Py_CLEAR(clear_module_state->__pyx_float_12_45);
  Py_CLEAR(clear_module_state->__pyx_float_12_50);
  Py_CLEAR(clear_module_state->__pyx_float_12_58);
  Py_CLEAR(clear_module_state->__pyx_float_12_63);
  Py_CLEAR(clear_module_state->__pyx_float_12_67);
  Py_CLEAR(clear_module_state->__pyx_float_12_70);
  Py_CLEAR(clear_module_state->__pyx_float_12_78);
  Py_CLEAR(clear_module_state->__pyx_float_12_95);
  Py_CLEAR(clear_module_state->__pyx_float_12_96);
  Py_CLEAR(clear_module_state->__pyx_float_12_98);
  Py_CLEAR(clear_module_state->__pyx_float_12_99);
  Py_CLEAR(clear_module_state->__pyx_float_13_00);
  Py_CLEAR(clear_module_state->__pyx_float_13_01);
  Py_CLEAR(clear_module_state->__pyx_float_13_02);
  Py_CLEAR(clear_module_state->__pyx_float_13_03);
  Py_CLEAR(clear_module_state->__pyx_float_13_04);
  Py_CLEAR(clear_module_state->__pyx_float_13_05);
  Py_CLEAR(clear_module_state->__pyx_float_13_06);
  Py_CLEAR(clear_module_state->__pyx_float_13_08);
  Py_CLEAR(clear_module_state->__pyx_float_13_11);
  Py_CLEAR(clear_module_state->__pyx_float_13_13);
  Py_CLEAR(clear_module_state->__pyx_float_13_14);
  Py_CLEAR(clear_module_state->__pyx_float_13_15);
  Py_CLEAR(clear_module_state->__pyx_float_13_16);
  Py_CLEAR(clear_module_state->__pyx_float_13_21);
  Py_CLEAR(clear_module_state->__pyx_float_13_24);
  Py_CLEAR(clear_module_state->__pyx_float_13_25);
  Py_CLEAR(clear_module_state->__pyx_float_13_26);
  Py_CLEAR(clear_module_state->__pyx_float_13_29);
  Py_CLEAR(clear_module_state->__pyx_float_13_34);
  Py_CLEAR(clear_module_state->__pyx_float_13_38);
  Py_CLEAR(clear_module_state->__pyx_float_13_40);
  Py_CLEAR(clear_module_state->__pyx_float_13_42);
  Py_CLEAR(clear_module_state->__pyx_float_13_49);
  Py_CLEAR(clear_module_state->__pyx_float_13_53);
  Py_CLEAR(clear_module_state->__pyx_float_13_58);
  Py_CLEAR(clear_module_state->__pyx_float_13_66);
  Py_CLEAR(clear_module_state->__pyx_float_13_75);
  Py_CLEAR(clear_module_state->__pyx_float_13_79);
  Py_CLEAR(clear_module_state->__pyx_float_13_89);
  Py_CLEAR(clear_module_state->__pyx_float_13_91);
  Py_CLEAR(clear_module_state->__pyx_float_13_97);
  Py_CLEAR(clear_module_state->__pyx_float_13_99);
  Py_CLEAR(clear_module_state->__pyx_float_14_00);
  Py_CLEAR(clear_module_state->__pyx_float_14_02);
  Py_CLEAR(clear_module_state->__pyx_float_14_03);
  Py_CLEAR(clear_module_state->__pyx_float_14_04);
  Py_CLEAR(clear_module_state->__pyx_float_14_05);
  Py_CLEAR(clear_module_state->__pyx_float_14_06);
  Py_CLEAR(clear_module_state->__pyx_float_14_07);
  Py_CLEAR(clear_module_state->__pyx_float_14_08);
  Py_CLEAR(clear_module_state->__pyx_float_14_10);
  Py_CLEAR(clear_module_state->__pyx_float_14_12);
  Py_CLEAR(clear_module_state->__pyx_float_14_13);
  Py_CLEAR(clear_module_state->__pyx_float_14_19);
  Py_CLEAR(clear_module_state->__pyx_float_14_25);
  Py_CLEAR(clear_module_state->__pyx_float_14_32);
  Py_CLEAR(clear_module_state->__pyx_float_14_33);
  Py_CLEAR(clear_module_state->__pyx_float_14_39);
  Py_CLEAR(clear_module_state->__pyx_float_14_45);
  Py_CLEAR(clear_module_state->__pyx_float_14_50);
  Py_CLEAR(clear_module_state->__pyx_float_14_53);
  Py_CLEAR(clear_module_state->__pyx_float_14_54);
  Py_CLEAR(clear_module_state->__pyx_float_14_62);
  Py_CLEAR(clear_module_state->__pyx_float_14_66);
  Py_CLEAR(clear_module_state->__pyx_float_14_74);
  Py_CLEAR(clear_module_state->__pyx_float_14_84);
  Py_CLEAR(clear_module_state->__pyx_float_14_93);
  Py_CLEAR(clear_module_state->__pyx_float_14_95);
  Py_CLEAR(clear_module_state->__pyx_float_15_00);
  Py_CLEAR(clear_module_state->__pyx_float_15_01);
  Py_CLEAR(clear_module_state->__pyx_float_15_03);
  Py_CLEAR(clear_module_state->__pyx_float_15_04);
  Py_CLEAR(clear_module_state->__pyx_float_15_05);
  Py_CLEAR(clear_module_state->__pyx_float_15_06);
  Py_CLEAR(clear_module_state->__pyx_float_15_07);
  Py_CLEAR(clear_module_state->__pyx_float_15_08);
  Py_CLEAR(clear_module_state->__pyx_float_15_09);
  Py_CLEAR(clear_module_state->__pyx_float_15_11);
  Py_CLEAR(clear_module_state->__pyx_float_15_12);
  Py_CLEAR(clear_module_state->__pyx_float_15_15);
  Py_CLEAR(clear_module_state->__pyx_float_15_16);
  Py_CLEAR(clear_module_state->__pyx_float_15_21);
  Py_CLEAR(clear_module_state->__pyx_float_15_22);
  Py_CLEAR(clear_module_state->__pyx_float_15_24);
  Py_CLEAR(clear_module_state->__pyx_float_15_27);
  Py_CLEAR(clear_module_state->__pyx_float_15_31);
  Py_CLEAR(clear_module_state->__pyx_float_15_41);
  Py_CLEAR(clear_module_state->__pyx_float_15_42);
  Py_CLEAR(clear_module_state->__pyx_float_15_48);
  Py_CLEAR(clear_module_state->__pyx_float_15_53);
  Py_CLEAR(clear_module_state->__pyx_float_15_62);
  Py_CLEAR(clear_module_state->__pyx_float_15_65);
  Py_CLEAR(clear_module_state->__pyx_float_15_77);
  Py_CLEAR(clear_module_state->__pyx_float_15_81);
  Py_CLEAR(clear_module_state->__pyx_float_15_88);
  Py_CLEAR(clear_module_state->__pyx_float_15_91);
  Py_CLEAR(clear_module_state->__pyx_float_15_93);
  Py_CLEAR(clear_module_state->__pyx_float_15_99);
  Py_CLEAR(clear_module_state->__pyx_float_16_00);
  Py_CLEAR(clear_module_state->__pyx_float_16_01);
  Py_CLEAR(clear_module_state->__pyx_float_16_03);
  Py_CLEAR(clear_module_state->__pyx_float_16_04);
  Py_CLEAR(clear_module_state->__pyx_float_16_05);
  Py_CLEAR(clear_module_state->__pyx_float_16_06);
  Py_CLEAR(clear_module_state->__pyx_float_16_07);
  Py_CLEAR(clear_module_state->__pyx_float_16_08);
  Py_CLEAR(clear_module_state->__pyx_float_16_09);
  Py_CLEAR(clear_module_state->__pyx_float_16_11);
  Py_CLEAR(clear_module_state->__pyx_float_16_12);
  Py_CLEAR(clear_module_state->__pyx_float_16_13);
  Py_CLEAR(clear_module_state->__pyx_float_16_14);
  Py_CLEAR(clear_module_state->__pyx_float_16_15);
  Py_CLEAR(clear_module_state->__pyx_float_16_16);
  Py_CLEAR(clear_module_state->__pyx_float_16_18);
  Py_CLEAR(clear_module_state->__pyx_float_16_20);
  Py_CLEAR(clear_module_state->__pyx_float_16_21);
  Py_CLEAR(clear_module_state->__pyx_float_16_24);
  Py_CLEAR(clear_module_state->__pyx_float_16_29);
  Py_CLEAR(clear_module_state->__pyx_float_16_30);
  Py_CLEAR(clear_module_state->__pyx_float_16_34);
  Py_CLEAR(clear_module_state->__pyx_float_16_40);
  Py_CLEAR(clear_module_state->__pyx_float_16_49);
  Py_CLEAR(clear_module_state->__pyx_float_16_50);
  Py_CLEAR(clear_module_state->__pyx_float_16_59);
  Py_CLEAR(clear_module_state->__pyx_float_16_66);
  Py_CLEAR(clear_module_state->__pyx_float_16_80);
  Py_CLEAR(clear_module_state->__pyx_float_17_01);
  Py_CLEAR(clear_module_state->__pyx_float_17_03);
  Py_CLEAR(clear_module_state->__pyx_float_17_04);
  Py_CLEAR(clear_module_state->__pyx_float_17_06);
  Py_CLEAR(clear_module_state->__pyx_float_17_07);
  Py_CLEAR(clear_module_state->__pyx_float_17_08);
  Py_CLEAR(clear_module_state->__pyx_float_17_09);
  Py_CLEAR(clear_module_state->__pyx_float_17_10);
  Py_CLEAR(clear_module_state->__pyx_float_17_11);
  Py_CLEAR(clear_module_state->__pyx_float_17_12);
  Py_CLEAR(clear_module_state->__pyx_float_17_14);
  Py_CLEAR(clear_module_state->__pyx_float_17_15);
  Py_CLEAR(clear_module_state->__pyx_float_17_17);
  Py_CLEAR(clear_module_state->__pyx_float_17_21);
  Py_CLEAR(clear_module_state->__pyx_float_17_22);
  Py_CLEAR(clear_module_state->__pyx_float_17_24);
  Py_CLEAR(clear_module_state->__pyx_float_17_25);
  Py_CLEAR(clear_module_state->__pyx_float_17_30);
  Py_CLEAR(clear_module_state->__pyx_float_17_36);
  Py_CLEAR(clear_module_state->__pyx_float_17_38);
  Py_CLEAR(clear_module_state->__pyx_float_17_42);
  Py_CLEAR(clear_module_state->__pyx_float_17_48);
  Py_CLEAR(clear_module_state->__pyx_float_17_59);
  Py_CLEAR(clear_module_state->__pyx_float_17_61);
  Py_CLEAR(clear_module_state->__pyx_float_17_62);
  Py_CLEAR(clear_module_state->__pyx_float_17_70);
  Py_CLEAR(clear_module_state->__pyx_float_17_79);
  Py_CLEAR(clear_module_state->__pyx_float_17_95);
  Py_CLEAR(clear_module_state->__pyx_float_17_97);
  Py_CLEAR(clear_module_state->__pyx_float_18_02);
  Py_CLEAR(clear_module_state->__pyx_float_18_03);
  Py_CLEAR(clear_module_state->__pyx_float_18_08);
  Py_CLEAR(clear_module_state->__pyx_float_18_09);
  Py_CLEAR(clear_module_state->__pyx_float_18_10);
  Py_CLEAR(clear_module_state->__pyx_float_18_11);
  Py_CLEAR(clear_module_state->__pyx_float_18_12);
  Py_CLEAR(clear_module_state->__pyx_float_18_13);
  Py_CLEAR(clear_module_state->__pyx_float_18_14);
  Py_CLEAR(clear_module_state->__pyx_float_18_15);
  Py_CLEAR(clear_module_state->__pyx_float_18_16);
  Py_CLEAR(clear_module_state->__pyx_float_18_18);
  Py_CLEAR(clear_module_state->__pyx_float_18_19);
  Py_CLEAR(clear_module_state->__pyx_float_18_20);
  Py_CLEAR(clear_module_state->__pyx_float_18_21);
  Py_CLEAR(clear_module_state->__pyx_float_18_23);
  Py_CLEAR(clear_module_state->__pyx_float_18_25);
  Py_CLEAR(clear_module_state->__pyx_float_18_27);
  Py_CLEAR(clear_module_state->__pyx_float_18_30);
  Py_CLEAR(clear_module_state->__pyx_float_18_35);
  Py_CLEAR(clear_module_state->__pyx_float_18_43);
  Py_CLEAR(clear_module_state->__pyx_float_18_45);
  Py_CLEAR(clear_module_state->__pyx_float_18_51);
  Py_CLEAR(clear_module_state->__pyx_float_18_59);
  Py_CLEAR(clear_module_state->__pyx_float_18_71);
  Py_CLEAR(clear_module_state->__pyx_float_18_75);
  Py_CLEAR(clear_module_state->__pyx_float_18_84);
  Py_CLEAR(clear_module_state->__pyx_float_18_94);
  Py_CLEAR(clear_module_state->__pyx_float_18_95);
  Py_CLEAR(clear_module_state->__pyx_float_19_04);
  Py_CLEAR(clear_module_state->__pyx_float_19_05);
  Py_CLEAR(clear_module_state->__pyx_float_19_06);
  Py_CLEAR(clear_module_state->__pyx_float_19_11);
  Py_CLEAR(clear_module_state->__pyx_float_19_12);
  Py_CLEAR(clear_module_state->__pyx_float_19_14);
  Py_CLEAR(clear_module_state->__pyx_float_19_15);
  Py_CLEAR(clear_module_state->__pyx_float_19_16);
  Py_CLEAR(clear_module_state->__pyx_float_19_17);
  Py_CLEAR(clear_module_state->__pyx_float_19_18);
  Py_CLEAR(clear_module_state->__pyx_float_19_21);
  Py_CLEAR(clear_module_state->__pyx_float_19_22);
  Py_CLEAR(clear_module_state->__pyx_float_19_23);
  Py_CLEAR(clear_module_state->__pyx_float_19_24);
  Py_CLEAR(clear_module_state->__pyx_float_19_26);
  Py_CLEAR(clear_module_state->__pyx_float_19_29);
  Py_CLEAR(clear_module_state->__pyx_float_19_34);
  Py_CLEAR(clear_module_state->__pyx_float_19_36);
  Py_CLEAR(clear_module_state->__pyx_float_19_39);
  Py_CLEAR(clear_module_state->__pyx_float_19_41);
  Py_CLEAR(clear_module_state->__pyx_float_19_51);
  Py_CLEAR(clear_module_state->__pyx_float_19_56);
  Py_CLEAR(clear_module_state->__pyx_float_19_57);
  Py_CLEAR(clear_module_state->__pyx_float_19_61);
  Py_CLEAR(clear_module_state->__pyx_float_19_71);
  Py_CLEAR(clear_module_state->__pyx_float_19_83);
  Py_CLEAR(clear_module_state->__pyx_float_19_84);
  Py_CLEAR(clear_module_state->__pyx_float_19_89);
  Py_CLEAR(clear_module_state->__pyx_float_20_00);
  Py_CLEAR(clear_module_state->__pyx_float_20_08);
  Py_CLEAR(clear_module_state->__pyx_float_20_11);
  Py_CLEAR(clear_module_state->__pyx_float_20_12);
  Py_CLEAR(clear_module_state->__pyx_float_20_13);
  Py_CLEAR(clear_module_state->__pyx_float_20_16);
  Py_CLEAR(clear_module_state->__pyx_float_20_17);
  Py_CLEAR(clear_module_state->__pyx_float_20_18);
  Py_CLEAR(clear_module_state->__pyx_float_20_19);
  Py_CLEAR(clear_module_state->__pyx_float_20_21);
  Py_CLEAR(clear_module_state->__pyx_float_20_23);
  Py_CLEAR(clear_module_state->__pyx_float_20_24);
  Py_CLEAR(clear_module_state->__pyx_float_20_25);
  Py_CLEAR(clear_module_state->__pyx_float_20_26);
  Py_CLEAR(clear_module_state->__pyx_float_20_27);
  Py_CLEAR(clear_module_state->__pyx_float_20_28);
  Py_CLEAR(clear_module_state->__pyx_float_20_29);
  Py_CLEAR(clear_module_state->__pyx_float_20_30);
  Py_CLEAR(clear_module_state->__pyx_float_20_34);
  Py_CLEAR(clear_module_state->__pyx_float_20_36);
  Py_CLEAR(clear_module_state->__pyx_float_20_40);
  Py_CLEAR(clear_module_state->__pyx_float_20_43);
  Py_CLEAR(clear_module_state->__pyx_float_20_44);
  Py_CLEAR(clear_module_state->__pyx_float_20_50);
  Py_CLEAR(clear_module_state->__pyx_float_20_60);
  Py_CLEAR(clear_module_state->__pyx_float_20_65);
  Py_CLEAR(clear_module_state->__pyx_float_20_72);
  Py_CLEAR(clear_module_state->__pyx_float_20_84);
  Py_CLEAR(clear_module_state->__pyx_float_20_98);
  Py_CLEAR(clear_module_state->__pyx_float_20_99);
  Py_CLEAR(clear_module_state->__pyx_float_21_04);
  Py_CLEAR(clear_module_state->__pyx_float_21_11);
  Py_CLEAR(clear_module_state->__pyx_float_21_13);
  Py_CLEAR(clear_module_state->__pyx_float_21_17);
  Py_CLEAR(clear_module_state->__pyx_float_21_18);
  Py_CLEAR(clear_module_state->__pyx_float_21_20);
  Py_CLEAR(clear_module_state->__pyx_float_21_21);
  Py_CLEAR(clear_module_state->__pyx_float_21_22);
  Py_CLEAR(clear_module_state->__pyx_float_21_23);
  Py_CLEAR(clear_module_state->__pyx_float_21_24);
  Py_CLEAR(clear_module_state->__pyx_float_21_25);
  Py_CLEAR(clear_module_state->__pyx_float_21_27);
  Py_CLEAR(clear_module_state->__pyx_float_21_29);
  Py_CLEAR(clear_module_state->__pyx_float_21_30);
  Py_CLEAR(clear_module_state->__pyx_float_21_31);
  Py_CLEAR(clear_module_state->__pyx_float_21_32);
  Py_CLEAR(clear_module_state->__pyx_float_21_33);
  Py_CLEAR(clear_module_state->__pyx_float_21_41);
  Py_CLEAR(clear_module_state->__pyx_float_21_47);
  Py_CLEAR(clear_module_state->__pyx_float_21_50);
  Py_CLEAR(clear_module_state->__pyx_float_21_51);
  Py_CLEAR(clear_module_state->__pyx_float_21_56);
  Py_CLEAR(clear_module_state->__pyx_float_21_57);
  Py_CLEAR(clear_module_state->__pyx_float_21_70);
  Py_CLEAR(clear_module_state->__pyx_float_21_74);
  Py_CLEAR(clear_module_state->__pyx_float_21_84);
  Py_CLEAR(clear_module_state->__pyx_float_21_98);
  Py_CLEAR(clear_module_state->__pyx_float_22_12);
  Py_CLEAR(clear_module_state->__pyx_float_22_13);
  Py_CLEAR(clear_module_state->__pyx_float_22_14);
  Py_CLEAR(clear_module_state->__pyx_float_22_16);
  Py_CLEAR(clear_module_state->__pyx_float_22_20);
  Py_CLEAR(clear_module_state->__pyx_float_22_21);
  Py_CLEAR(clear_module_state->__pyx_float_22_22);
  Py_CLEAR(clear_module_state->__pyx_float_22_25);
  Py_CLEAR(clear_module_state->__pyx_float_22_26);
  Py_CLEAR(clear_module_state->__pyx_float_22_27);
  Py_CLEAR(clear_module_state->__pyx_float_22_28);
  Py_CLEAR(clear_module_state->__pyx_float_22_29);
  Py_CLEAR(clear_module_state->__pyx_float_22_30);
  Py_CLEAR(clear_module_state->__pyx_float_22_32);
  Py_CLEAR(clear_module_state->__pyx_float_22_35);
  Py_CLEAR(clear_module_state->__pyx_float_22_36);
  Py_CLEAR(clear_module_state->__pyx_float_22_37);
  Py_CLEAR(clear_module_state->__pyx_float_22_38);
  Py_CLEAR(clear_module_state->__pyx_float_22_48);
  Py_CLEAR(clear_module_state->__pyx_float_22_49);
  Py_CLEAR(clear_module_state->__pyx_float_22_54);
  Py_CLEAR(clear_module_state->__pyx_float_22_59);
  Py_CLEAR(clear_module_state->__pyx_float_22_60);
  Py_CLEAR(clear_module_state->__pyx_float_22_68);
  Py_CLEAR(clear_module_state->__pyx_float_22_82);
  Py_CLEAR(clear_module_state->__pyx_float_22_87);
  Py_CLEAR(clear_module_state->__pyx_float_22_99);
  Py_CLEAR(clear_module_state->__pyx_float_23_11);
  Py_CLEAR(clear_module_state->__pyx_float_23_19);
  Py_CLEAR(clear_module_state->__pyx_float_23_20);
  Py_CLEAR(clear_module_state->__pyx_float_23_22);
  Py_CLEAR(clear_module_state->__pyx_float_23_24);
  Py_CLEAR(clear_module_state->__pyx_float_23_26);
  Py_CLEAR(clear_module_state->__pyx_float_23_28);
  Py_CLEAR(clear_module_state->__pyx_float_23_29);
  Py_CLEAR(clear_module_state->__pyx_float_23_30);
  Py_CLEAR(clear_module_state->__pyx_float_23_31);
  Py_CLEAR(clear_module_state->__pyx_float_23_32);
  Py_CLEAR(clear_module_state->__pyx_float_23_33);
  Py_CLEAR(clear_module_state->__pyx_float_23_34);
  Py_CLEAR(clear_module_state->__pyx_float_23_35);
  Py_CLEAR(clear_module_state->__pyx_float_23_38);
  Py_CLEAR(clear_module_state->__pyx_float_23_39);
  Py_CLEAR(clear_module_state->__pyx_float_23_41);
  Py_CLEAR(clear_module_state->__pyx_float_23_42);
  Py_CLEAR(clear_module_state->__pyx_float_23_46);
  Py_CLEAR(clear_module_state->__pyx_float_23_55);
  Py_CLEAR(clear_module_state->__pyx_float_23_56);
  Py_CLEAR(clear_module_state->__pyx_float_23_62);
  Py_CLEAR(clear_module_state->__pyx_float_23_69);
  Py_CLEAR(clear_module_state->__pyx_float_23_72);
  Py_CLEAR(clear_module_state->__pyx_float_23_73);
  Py_CLEAR(clear_module_state->__pyx_float_23_80);
  Py_CLEAR(clear_module_state->__pyx_float_23_90);
  Py_CLEAR(clear_module_state->__pyx_float_23_95);
  Py_CLEAR(clear_module_state->__pyx_float_23_99);
  Py_CLEAR(clear_module_state->__pyx_float_24_00);
  Py_CLEAR(clear_module_state->__pyx_float_24_05);
  Py_CLEAR(clear_module_state->__pyx_float_24_13);
  Py_CLEAR(clear_module_state->__pyx_float_24_14);
  Py_CLEAR(clear_module_state->__pyx_float_24_19);
  Py_CLEAR(clear_module_state->__pyx_float_24_26);
  Py_CLEAR(clear_module_state->__pyx_float_24_29);
  Py_CLEAR(clear_module_state->__pyx_float_24_31);
  Py_CLEAR(clear_module_state->__pyx_float_24_32);
  Py_CLEAR(clear_module_state->__pyx_float_24_33);
  Py_CLEAR(clear_module_state->__pyx_float_24_34);
  Py_CLEAR(clear_module_state->__pyx_float_24_35);
  Py_CLEAR(clear_module_state->__pyx_float_24_36);
  Py_CLEAR(clear_module_state->__pyx_float_24_37);
  Py_CLEAR(clear_module_state->__pyx_float_24_38);
  Py_CLEAR(clear_module_state->__pyx_float_24_40);
  Py_CLEAR(clear_module_state->__pyx_float_24_48);
  Py_CLEAR(clear_module_state->__pyx_float_24_50);
  Py_CLEAR(clear_module_state->__pyx_float_24_52);
  Py_CLEAR(clear_module_state->__pyx_float_24_64);
  Py_CLEAR(clear_module_state->__pyx_float_24_71);
  Py_CLEAR(clear_module_state->__pyx_float_24_80);
  Py_CLEAR(clear_module_state->__pyx_float_24_90);
  Py_CLEAR(clear_module_state->__pyx_float_24_97);
  Py_CLEAR(clear_module_state->__pyx_float_25_05);
  Py_CLEAR(clear_module_state->__pyx_float_25_15);
  Py_CLEAR(clear_module_state->__pyx_float_25_20);
  Py_CLEAR(clear_module_state->__pyx_float_25_24);
  Py_CLEAR(clear_module_state->__pyx_float_25_30);
  Py_CLEAR(clear_module_state->__pyx_float_25_31);
  Py_CLEAR(clear_module_state->__pyx_float_25_32);
  Py_CLEAR(clear_module_state->__pyx_float_25_33);
  Py_CLEAR(clear_module_state->__pyx_float_25_35);
  Py_CLEAR(clear_module_state->__pyx_float_25_38);
  Py_CLEAR(clear_module_state->__pyx_float_25_41);
  Py_CLEAR(clear_module_state->__pyx_float_25_42);
  Py_CLEAR(clear_module_state->__pyx_float_25_43);
  Py_CLEAR(clear_module_state->__pyx_float_25_45);
  Py_CLEAR(clear_module_state->__pyx_float_25_49);
  Py_CLEAR(clear_module_state->__pyx_float_25_54);
  Py_CLEAR(clear_module_state->__pyx_float_25_55);
  Py_CLEAR(clear_module_state->__pyx_float_25_56);
  Py_CLEAR(clear_module_state->__pyx_float_25_58);
  Py_CLEAR(clear_module_state->__pyx_float_25_59);
  Py_CLEAR(clear_module_state->__pyx_float_25_70);
  Py_CLEAR(clear_module_state->__pyx_float_25_74);
  Py_CLEAR(clear_module_state->__pyx_float_25_82);
  Py_CLEAR(clear_module_state->__pyx_float_25_87);
  Py_CLEAR(clear_module_state->__pyx_float_25_88);
  Py_CLEAR(clear_module_state->__pyx_float_26_02);
  Py_CLEAR(clear_module_state->__pyx_float_26_17);
  Py_CLEAR(clear_module_state->__pyx_float_26_22);
  Py_CLEAR(clear_module_state->__pyx_float_26_24);
  Py_CLEAR(clear_module_state->__pyx_float_26_36);
  Py_CLEAR(clear_module_state->__pyx_float_26_38);
  Py_CLEAR(clear_module_state->__pyx_float_26_39);
  Py_CLEAR(clear_module_state->__pyx_float_26_40);
  Py_CLEAR(clear_module_state->__pyx_float_26_41);
  Py_CLEAR(clear_module_state->__pyx_float_26_43);
  Py_CLEAR(clear_module_state->__pyx_float_26_46);
  Py_CLEAR(clear_module_state->__pyx_float_26_47);
  Py_CLEAR(clear_module_state->__pyx_float_26_49);
  Py_CLEAR(clear_module_state->__pyx_float_26_50);
  Py_CLEAR(clear_module_state->__pyx_float_26_54);
  Py_CLEAR(clear_module_state->__pyx_float_26_55);
  Py_CLEAR(clear_module_state->__pyx_float_26_61);
  Py_CLEAR(clear_module_state->__pyx_float_26_62);
  Py_CLEAR(clear_module_state->__pyx_float_26_64);
  Py_CLEAR(clear_module_state->__pyx_float_26_67);
  Py_CLEAR(clear_module_state->__pyx_float_26_68);
  Py_CLEAR(clear_module_state->__pyx_float_26_69);
  Py_CLEAR(clear_module_state->__pyx_float_26_70);
  Py_CLEAR(clear_module_state->__pyx_float_26_81);
  Py_CLEAR(clear_module_state->__pyx_float_26_91);
  Py_CLEAR(clear_module_state->__pyx_float_27_02);
  Py_CLEAR(clear_module_state->__pyx_float_27_03);
  Py_CLEAR(clear_module_state->__pyx_float_27_04);
  Py_CLEAR(clear_module_state->__pyx_float_27_16);
  Py_CLEAR(clear_module_state->__pyx_float_27_36);
  Py_CLEAR(clear_module_state->__pyx_float_27_41);
  Py_CLEAR(clear_module_state->__pyx_float_27_43);
  Py_CLEAR(clear_module_state->__pyx_float_27_44);
  Py_CLEAR(clear_module_state->__pyx_float_27_46);
  Py_CLEAR(clear_module_state->__pyx_float_27_47);
  Py_CLEAR(clear_module_state->__pyx_float_27_49);
  Py_CLEAR(clear_module_state->__pyx_float_27_50);
  Py_CLEAR(clear_module_state->__pyx_float_27_52);
  Py_CLEAR(clear_module_state->__pyx_float_27_53);
  Py_CLEAR(clear_module_state->__pyx_float_27_54);
  Py_CLEAR(clear_module_state->__pyx_float_27_55);
  Py_CLEAR(clear_module_state->__pyx_float_27_59);
  Py_CLEAR(clear_module_state->__pyx_float_27_60);
  Py_CLEAR(clear_module_state->__pyx_float_27_61);
  Py_CLEAR(clear_module_state->__pyx_float_27_70);
  Py_CLEAR(clear_module_state->__pyx_float_27_72);
  Py_CLEAR(clear_module_state->__pyx_float_27_76);
  Py_CLEAR(clear_module_state->__pyx_float_27_77);
  Py_CLEAR(clear_module_state->__pyx_float_27_78);
  Py_CLEAR(clear_module_state->__pyx_float_27_95);
  Py_CLEAR(clear_module_state->__pyx_float_27_97);
  Py_CLEAR(clear_module_state->__pyx_float_28_03);
  Py_CLEAR(clear_module_state->__pyx_float_28_09);
  Py_CLEAR(clear_module_state->__pyx_float_28_12);
  Py_CLEAR(clear_module_state->__pyx_float_28_16);
  Py_CLEAR(clear_module_state->__pyx_float_28_30);
  Py_CLEAR(clear_module_state->__pyx_float_28_39);
  Py_CLEAR(clear_module_state->__pyx_float_28_44);
  Py_CLEAR(clear_module_state->__pyx_float_28_45);
  Py_CLEAR(clear_module_state->__pyx_float_28_47);
  Py_CLEAR(clear_module_state->__pyx_float_28_50);
  Py_CLEAR(clear_module_state->__pyx_float_28_51);
  Py_CLEAR(clear_module_state->__pyx_float_28_52);
  Py_CLEAR(clear_module_state->__pyx_float_28_53);
  Py_CLEAR(clear_module_state->__pyx_float_28_55);
  Py_CLEAR(clear_module_state->__pyx_float_28_59);
  Py_CLEAR(clear_module_state->__pyx_float_28_60);
  Py_CLEAR(clear_module_state->__pyx_float_28_62);
  Py_CLEAR(clear_module_state->__pyx_float_28_63);
  Py_CLEAR(clear_module_state->__pyx_float_28_64);
  Py_CLEAR(clear_module_state->__pyx_float_28_65);
  Py_CLEAR(clear_module_state->__pyx_float_28_66);
  Py_CLEAR(clear_module_state->__pyx_float_28_70);
  Py_CLEAR(clear_module_state->__pyx_float_28_73);
  Py_CLEAR(clear_module_state->__pyx_float_28_78);
  Py_CLEAR(clear_module_state->__pyx_float_28_79);
  Py_CLEAR(clear_module_state->__pyx_float_28_80);
  Py_CLEAR(clear_module_state->__pyx_float_28_82);
  Py_CLEAR(clear_module_state->__pyx_float_28_87);
  Py_CLEAR(clear_module_state->__pyx_float_28_88);
  Py_CLEAR(clear_module_state->__pyx_float_29_03);
  Py_CLEAR(clear_module_state->__pyx_float_29_12);
  Py_CLEAR(clear_module_state->__pyx_float_29_17);
  Py_CLEAR(clear_module_state->__pyx_float_29_28);
  Py_CLEAR(clear_module_state->__pyx_float_29_31);
  Py_CLEAR(clear_module_state->__pyx_float_29_40);
  Py_CLEAR(clear_module_state->__pyx_float_29_48);
  Py_CLEAR(clear_module_state->__pyx_float_29_54);
  Py_CLEAR(clear_module_state->__pyx_float_29_57);
  Py_CLEAR(clear_module_state->__pyx_float_29_58);
  Py_CLEAR(clear_module_state->__pyx_float_29_60);
  Py_CLEAR(clear_module_state->__pyx_float_29_61);
  Py_CLEAR(clear_module_state->__pyx_float_29_64);
  Py_CLEAR(clear_module_state->__pyx_float_29_65);
  Py_CLEAR(clear_module_state->__pyx_float_29_66);
  Py_CLEAR(clear_module_state->__pyx_float_29_68);
  Py_CLEAR(clear_module_state->__pyx_float_29_71);
  Py_CLEAR(clear_module_state->__pyx_float_29_72);
  Py_CLEAR(clear_module_state->__pyx_float_29_76);
  Py_CLEAR(clear_module_state->__pyx_float_29_80);
  Py_CLEAR(clear_module_state->__pyx_float_29_81);
  Py_CLEAR(clear_module_state->__pyx_float_29_85);
  Py_CLEAR(clear_module_state->__pyx_float_29_89);
  Py_CLEAR(clear_module_state->__pyx_float_29_90);
  Py_CLEAR(clear_module_state->__pyx_float_29_91);
  Py_CLEAR(clear_module_state->__pyx_float_29_95);
  Py_CLEAR(clear_module_state->__pyx_float_30_03);
  Py_CLEAR(clear_module_state->__pyx_float_30_08);
  Py_CLEAR(clear_module_state->__pyx_float_30_09);
  Py_CLEAR(clear_module_state->__pyx_float_30_10);
  Py_CLEAR(clear_module_state->__pyx_float_30_22);
  Py_CLEAR(clear_module_state->__pyx_float_30_32);
  Py_CLEAR(clear_module_state->__pyx_float_30_37);
  Py_CLEAR(clear_module_state->__pyx_float_30_43);
  Py_CLEAR(clear_module_state->__pyx_float_30_46);
  Py_CLEAR(clear_module_state->__pyx_float_30_58);
  Py_CLEAR(clear_module_state->__pyx_float_30_60);
  Py_CLEAR(clear_module_state->__pyx_float_30_65);
  Py_CLEAR(clear_module_state->__pyx_float_30_70);
  Py_CLEAR(clear_module_state->__pyx_float_30_73);
  Py_CLEAR(clear_module_state->__pyx_float_30_75);
  Py_CLEAR(clear_module_state->__pyx_float_30_77);
  Py_CLEAR(clear_module_state->__pyx_float_30_78);
  Py_CLEAR(clear_module_state->__pyx_float_30_81);
  Py_CLEAR(clear_module_state->__pyx_float_30_85);
  Py_CLEAR(clear_module_state->__pyx_float_30_91);
  Py_CLEAR(clear_module_state->__pyx_float_30_97);
  Py_CLEAR(clear_module_state->__pyx_float_30_98);
  Py_CLEAR(clear_module_state->__pyx_float_31_01);
  Py_CLEAR(clear_module_state->__pyx_float_31_05);
  Py_CLEAR(clear_module_state->__pyx_float_31_06);
  Py_CLEAR(clear_module_state->__pyx_float_31_13);
  Py_CLEAR(clear_module_state->__pyx_float_31_14);
  Py_CLEAR(clear_module_state->__pyx_float_31_30);
  Py_CLEAR(clear_module_state->__pyx_float_31_45);
  Py_CLEAR(clear_module_state->__pyx_float_31_59);
  Py_CLEAR(clear_module_state->__pyx_float_31_61);
  Py_CLEAR(clear_module_state->__pyx_float_31_62);
  Py_CLEAR(clear_module_state->__pyx_float_31_66);
  Py_CLEAR(clear_module_state->__pyx_float_31_67);
  Py_CLEAR(clear_module_state->__pyx_float_31_71);
  Py_CLEAR(clear_module_state->__pyx_float_31_73);
  Py_CLEAR(clear_module_state->__pyx_float_31_74);
  Py_CLEAR(clear_module_state->__pyx_float_31_78);
  Py_CLEAR(clear_module_state->__pyx_float_31_81);
  Py_CLEAR(clear_module_state->__pyx_float_31_82);
  Py_CLEAR(clear_module_state->__pyx_float_31_84);
  Py_CLEAR(clear_module_state->__pyx_float_31_86);
  Py_CLEAR(clear_module_state->__pyx_float_31_88);
  Py_CLEAR(clear_module_state->__pyx_float_31_91);
  Py_CLEAR(clear_module_state->__pyx_float_31_95);
  Py_CLEAR(clear_module_state->__pyx_float_31_96);
  Py_CLEAR(clear_module_state->__pyx_float_31_97);
  Py_CLEAR(clear_module_state->__pyx_float_32_00);
  Py_CLEAR(clear_module_state->__pyx_float_32_03);
  Py_CLEAR(clear_module_state->__pyx_float_32_07);
  Py_CLEAR(clear_module_state->__pyx_float_32_08);
  Py_CLEAR(clear_module_state->__pyx_float_32_09);
  Py_CLEAR(clear_module_state->__pyx_float_32_12);
  Py_CLEAR(clear_module_state->__pyx_float_32_13);
  Py_CLEAR(clear_module_state->__pyx_float_32_14);
  Py_CLEAR(clear_module_state->__pyx_float_32_17);
  Py_CLEAR(clear_module_state->__pyx_float_32_24);
  Py_CLEAR(clear_module_state->__pyx_float_32_27);
  Py_CLEAR(clear_module_state->__pyx_float_32_48);
  Py_CLEAR(clear_module_state->__pyx_float_32_61);
  Py_CLEAR(clear_module_state->__pyx_float_32_65);
  Py_CLEAR(clear_module_state->__pyx_float_32_73);
  Py_CLEAR(clear_module_state->__pyx_float_32_75);
  Py_CLEAR(clear_module_state->__pyx_float_32_77);
  Py_CLEAR(clear_module_state->__pyx_float_32_78);
  Py_CLEAR(clear_module_state->__pyx_float_32_79);
  Py_CLEAR(clear_module_state->__pyx_float_32_83);
  Py_CLEAR(clear_module_state->__pyx_float_32_84);
  Py_CLEAR(clear_module_state->__pyx_float_32_89);
  Py_CLEAR(clear_module_state->__pyx_float_32_92);
  Py_CLEAR(clear_module_state->__pyx_float_32_93);
  Py_CLEAR(clear_module_state->__pyx_float_32_94);
  Py_CLEAR(clear_module_state->__pyx_float_32_96);
  Py_CLEAR(clear_module_state->__pyx_float_32_97);
  Py_CLEAR(clear_module_state->__pyx_float_33_00);
  Py_CLEAR(clear_module_state->__pyx_float_33_02);
  Py_CLEAR(clear_module_state->__pyx_float_33_14);
  Py_CLEAR(clear_module_state->__pyx_float_33_21);
  Py_CLEAR(clear_module_state->__pyx_float_33_28);
  Py_CLEAR(clear_module_state->__pyx_float_33_29);
  Py_CLEAR(clear_module_state->__pyx_float_33_32);
  Py_CLEAR(clear_module_state->__pyx_float_33_36);
  Py_CLEAR(clear_module_state->__pyx_float_33_37);
  Py_CLEAR(clear_module_state->__pyx_float_33_52);
  Py_CLEAR(clear_module_state->__pyx_float_33_59);
  Py_CLEAR(clear_module_state->__pyx_float_33_60);
  Py_CLEAR(clear_module_state->__pyx_float_33_77);
  Py_CLEAR(clear_module_state->__pyx_float_33_81);
  Py_CLEAR(clear_module_state->__pyx_float_33_82);
  Py_CLEAR(clear_module_state->__pyx_float_33_85);
  Py_CLEAR(clear_module_state->__pyx_float_33_95);
  Py_CLEAR(clear_module_state->__pyx_float_34_03);
  Py_CLEAR(clear_module_state->__pyx_float_34_04);
  Py_CLEAR(clear_module_state->__pyx_float_34_08);
  Py_CLEAR(clear_module_state->__pyx_float_34_09);
  Py_CLEAR(clear_module_state->__pyx_float_34_12);
  Py_CLEAR(clear_module_state->__pyx_float_34_13);
  Py_CLEAR(clear_module_state->__pyx_float_34_19);
  Py_CLEAR(clear_module_state->__pyx_float_34_21);
  Py_CLEAR(clear_module_state->__pyx_float_34_26);
  Py_CLEAR(clear_module_state->__pyx_float_34_30);
  Py_CLEAR(clear_module_state->__pyx_float_34_31);
  Py_CLEAR(clear_module_state->__pyx_float_34_37);
  Py_CLEAR(clear_module_state->__pyx_float_34_42);
  Py_CLEAR(clear_module_state->__pyx_float_34_52);
  Py_CLEAR(clear_module_state->__pyx_float_34_76);
  Py_CLEAR(clear_module_state->__pyx_float_34_79);
  Py_CLEAR(clear_module_state->__pyx_float_34_91);
  Py_CLEAR(clear_module_state->__pyx_float_34_95);
  Py_CLEAR(clear_module_state->__pyx_float_34_96);
  Py_CLEAR(clear_module_state->__pyx_float_34_98);
  Py_CLEAR(clear_module_state->__pyx_float_35_06);
  Py_CLEAR(clear_module_state->__pyx_float_35_07);
  Py_CLEAR(clear_module_state->__pyx_float_35_13);
  Py_CLEAR(clear_module_state->__pyx_float_35_14);
  Py_CLEAR(clear_module_state->__pyx_float_35_16);
  Py_CLEAR(clear_module_state->__pyx_float_35_17);
  Py_CLEAR(clear_module_state->__pyx_float_35_18);
  Py_CLEAR(clear_module_state->__pyx_float_35_25);
  Py_CLEAR(clear_module_state->__pyx_float_35_30);
  Py_CLEAR(clear_module_state->__pyx_float_35_38);
  Py_CLEAR(clear_module_state->__pyx_float_35_44);
  Py_CLEAR(clear_module_state->__pyx_float_35_46);
  Py_CLEAR(clear_module_state->__pyx_float_35_50);
  Py_CLEAR(clear_module_state->__pyx_float_35_59);
  Py_CLEAR(clear_module_state->__pyx_float_35_64);
  Py_CLEAR(clear_module_state->__pyx_float_35_67);
  Py_CLEAR(clear_module_state->__pyx_float_35_75);
  Py_CLEAR(clear_module_state->__pyx_float_36_00);
  Py_CLEAR(clear_module_state->__pyx_float_36_01);
  Py_CLEAR(clear_module_state->__pyx_float_36_02);
  Py_CLEAR(clear_module_state->__pyx_float_36_07);
  Py_CLEAR(clear_module_state->__pyx_float_36_09);
  Py_CLEAR(clear_module_state->__pyx_float_36_14);
  Py_CLEAR(clear_module_state->__pyx_float_36_16);
  Py_CLEAR(clear_module_state->__pyx_float_36_20);
  Py_CLEAR(clear_module_state->__pyx_float_36_25);
  Py_CLEAR(clear_module_state->__pyx_float_36_27);
  Py_CLEAR(clear_module_state->__pyx_float_36_28);
  Py_CLEAR(clear_module_state->__pyx_float_36_29);
  Py_CLEAR(clear_module_state->__pyx_float_36_30);
  Py_CLEAR(clear_module_state->__pyx_float_36_32);
  Py_CLEAR(clear_module_state->__pyx_float_36_34);
  Py_CLEAR(clear_module_state->__pyx_float_36_40);
  Py_CLEAR(clear_module_state->__pyx_float_36_43);
  Py_CLEAR(clear_module_state->__pyx_float_36_55);
  Py_CLEAR(clear_module_state->__pyx_float_36_56);
  Py_CLEAR(clear_module_state->__pyx_float_36_62);
  Py_CLEAR(clear_module_state->__pyx_float_36_74);
  Py_CLEAR(clear_module_state->__pyx_float_36_78);
  Py_CLEAR(clear_module_state->__pyx_float_36_88);
  Py_CLEAR(clear_module_state->__pyx_float_37_11);
  Py_CLEAR(clear_module_state->__pyx_float_37_16);
  Py_CLEAR(clear_module_state->__pyx_float_37_20);
  Py_CLEAR(clear_module_state->__pyx_float_37_23);
  Py_CLEAR(clear_module_state->__pyx_float_37_27);
  Py_CLEAR(clear_module_state->__pyx_float_37_32);
  Py_CLEAR(clear_module_state->__pyx_float_37_34);
  Py_CLEAR(clear_module_state->__pyx_float_37_35);
  Py_CLEAR(clear_module_state->__pyx_float_37_38);
  Py_CLEAR(clear_module_state->__pyx_float_37_39);
  Py_CLEAR(clear_module_state->__pyx_float_37_41);
  Py_CLEAR(clear_module_state->__pyx_float_37_51);
  Py_CLEAR(clear_module_state->__pyx_float_37_56);
  Py_CLEAR(clear_module_state->__pyx_float_37_57);
  Py_CLEAR(clear_module_state->__pyx_float_37_68);
  Py_CLEAR(clear_module_state->__pyx_float_37_69);
  Py_CLEAR(clear_module_state->__pyx_float_37_76);
  Py_CLEAR(clear_module_state->__pyx_float_37_77);
  Py_CLEAR(clear_module_state->__pyx_float_37_88);
  Py_CLEAR(clear_module_state->__pyx_float_37_89);
  Py_CLEAR(clear_module_state->__pyx_float_37_90);
  Py_CLEAR(clear_module_state->__pyx_float_37_93);
  Py_CLEAR(clear_module_state->__pyx_float_37_99);
  Py_CLEAR(clear_module_state->__pyx_float_38_10);
  Py_CLEAR(clear_module_state->__pyx_float_38_18);
  Py_CLEAR(clear_module_state->__pyx_float_38_20);
  Py_CLEAR(clear_module_state->__pyx_float_38_25);
  Py_CLEAR(clear_module_state->__pyx_float_38_26);
  Py_CLEAR(clear_module_state->__pyx_float_38_30);
  Py_CLEAR(clear_module_state->__pyx_float_38_33);
  Py_CLEAR(clear_module_state->__pyx_float_38_36);
  Py_CLEAR(clear_module_state->__pyx_float_38_46);
  Py_CLEAR(clear_module_state->__pyx_float_38_51);
  Py_CLEAR(clear_module_state->__pyx_float_38_53);
  Py_CLEAR(clear_module_state->__pyx_float_38_54);
  Py_CLEAR(clear_module_state->__pyx_float_38_55);
  Py_CLEAR(clear_module_state->__pyx_float_38_58);
  Py_CLEAR(clear_module_state->__pyx_float_38_64);
  Py_CLEAR(clear_module_state->__pyx_float_38_68);
  Py_CLEAR(clear_module_state->__pyx_float_38_80);
  Py_CLEAR(clear_module_state->__pyx_float_38_85);
  Py_CLEAR(clear_module_state->__pyx_float_38_86);
  Py_CLEAR(clear_module_state->__pyx_float_38_92);
  Py_CLEAR(clear_module_state->__pyx_float_38_93);
  Py_CLEAR(clear_module_state->__pyx_float_39_00);
  Py_CLEAR(clear_module_state->__pyx_float_39_03);
  Py_CLEAR(clear_module_state->__pyx_float_39_06);
  Py_CLEAR(clear_module_state->__pyx_float_39_11);
  Py_CLEAR(clear_module_state->__pyx_float_39_17);
  Py_CLEAR(clear_module_state->__pyx_float_39_23);
  Py_CLEAR(clear_module_state->__pyx_float_39_29);
  Py_CLEAR(clear_module_state->__pyx_float_39_35);
  Py_CLEAR(clear_module_state->__pyx_float_39_41);
  Py_CLEAR(clear_module_state->__pyx_float_39_47);
  Py_CLEAR(clear_module_state->__pyx_float_39_59);
  Py_CLEAR(clear_module_state->__pyx_float_39_60);
  Py_CLEAR(clear_module_state->__pyx_float_39_62);
  Py_CLEAR(clear_module_state->__pyx_float_39_64);
  Py_CLEAR(clear_module_state->__pyx_float_39_65);
  Py_CLEAR(clear_module_state->__pyx_float_39_66);
  Py_CLEAR(clear_module_state->__pyx_float_39_67);
  Py_CLEAR(clear_module_state->__pyx_float_39_75);
  Py_CLEAR(clear_module_state->__pyx_float_39_80);
  Py_CLEAR(clear_module_state->__pyx_float_39_88);
  Py_CLEAR(clear_module_state->__pyx_float_40_02);
  Py_CLEAR(clear_module_state->__pyx_float_40_03);
  Py_CLEAR(clear_module_state->__pyx_float_40_05);
  Py_CLEAR(clear_module_state->__pyx_float_40_07);
  Py_CLEAR(clear_module_state->__pyx_float_40_08);
  Py_CLEAR(clear_module_state->__pyx_float_40_09);
  Py_CLEAR(clear_module_state->__pyx_float_40_11);
  Py_CLEAR(clear_module_state->__pyx_float_40_19);
  Py_CLEAR(clear_module_state->__pyx_float_40_25);
  Py_CLEAR(clear_module_state->__pyx_float_40_32);
  Py_CLEAR(clear_module_state->__pyx_float_40_33);
  Py_CLEAR(clear_module_state->__pyx_float_40_35);
  Py_CLEAR(clear_module_state->__pyx_float_40_43);
  Py_CLEAR(clear_module_state->__pyx_float_40_53);
  Py_CLEAR(clear_module_state->__pyx_float_40_54);
  Py_CLEAR(clear_module_state->__pyx_float_40_55);
  Py_CLEAR(clear_module_state->__pyx_float_40_58);
  Py_CLEAR(clear_module_state->__pyx_float_40_69);
  Py_CLEAR(clear_module_state->__pyx_float_40_71);
  Py_CLEAR(clear_module_state->__pyx_float_40_77);
  Py_CLEAR(clear_module_state->__pyx_float_40_80);
  Py_CLEAR(clear_module_state->__pyx_float_40_85);
  Py_CLEAR(clear_module_state->__pyx_float_40_90);
  Py_CLEAR(clear_module_state->__pyx_float_40_95);
  Py_CLEAR(clear_module_state->__pyx_float_40_97);
  Py_CLEAR(clear_module_state->__pyx_float_41_00);
  Py_CLEAR(clear_module_state->__pyx_float_41_16);
  Py_CLEAR(clear_module_state->__pyx_float_41_20);
  Py_CLEAR(clear_module_state->__pyx_float_41_23);
  Py_CLEAR(clear_module_state->__pyx_float_41_28);
  Py_CLEAR(clear_module_state->__pyx_float_41_37);
  Py_CLEAR(clear_module_state->__pyx_float_41_40);
  Py_CLEAR(clear_module_state->__pyx_float_41_43);
  Py_CLEAR(clear_module_state->__pyx_float_41_47);
  Py_CLEAR(clear_module_state->__pyx_float_41_50);
  Py_CLEAR(clear_module_state->__pyx_float_41_51);
  Py_CLEAR(clear_module_state->__pyx_float_41_52);
  Py_CLEAR(clear_module_state->__pyx_float_41_53);
  Py_CLEAR(clear_module_state->__pyx_float_41_61);
  Py_CLEAR(clear_module_state->__pyx_float_41_77);
  Py_CLEAR(clear_module_state->__pyx_float_41_82);
  Py_CLEAR(clear_module_state->__pyx_float_41_86);
  Py_CLEAR(clear_module_state->__pyx_float_41_89);
  Py_CLEAR(clear_module_state->__pyx_float_41_90);
  Py_CLEAR(clear_module_state->__pyx_float_41_93);
  Py_CLEAR(clear_module_state->__pyx_float_41_97);
  Py_CLEAR(clear_module_state->__pyx_float_42_09);
  Py_CLEAR(clear_module_state->__pyx_float_42_23);
  Py_CLEAR(clear_module_state->__pyx_float_42_37);
  Py_CLEAR(clear_module_state->__pyx_float_42_42);
  Py_CLEAR(clear_module_state->__pyx_float_42_43);
  Py_CLEAR(clear_module_state->__pyx_float_42_50);
  Py_CLEAR(clear_module_state->__pyx_float_42_53);
  Py_CLEAR(clear_module_state->__pyx_float_42_66);
  Py_CLEAR(clear_module_state->__pyx_float_42_67);
  Py_CLEAR(clear_module_state->__pyx_float_42_80);
  Py_CLEAR(clear_module_state->__pyx_float_42_92);
  Py_CLEAR(clear_module_state->__pyx_float_42_95);
  Py_CLEAR(clear_module_state->__pyx_float_43_04);
  Py_CLEAR(clear_module_state->__pyx_float_43_06);
  Py_CLEAR(clear_module_state->__pyx_float_43_08);
  Py_CLEAR(clear_module_state->__pyx_float_43_24);
  Py_CLEAR(clear_module_state->__pyx_float_43_25);
  Py_CLEAR(clear_module_state->__pyx_float_43_27);
  Py_CLEAR(clear_module_state->__pyx_float_43_35);
  Py_CLEAR(clear_module_state->__pyx_float_43_38);
  Py_CLEAR(clear_module_state->__pyx_float_43_40);
  Py_CLEAR(clear_module_state->__pyx_float_43_41);
  Py_CLEAR(clear_module_state->__pyx_float_43_46);
  Py_CLEAR(clear_module_state->__pyx_float_43_48);
  Py_CLEAR(clear_module_state->__pyx_float_43_49);
  Py_CLEAR(clear_module_state->__pyx_float_43_54);
  Py_CLEAR(clear_module_state->__pyx_float_43_60);
  Py_CLEAR(clear_module_state->__pyx_float_43_65);
  Py_CLEAR(clear_module_state->__pyx_float_43_74);
  Py_CLEAR(clear_module_state->__pyx_float_43_75);
  Py_CLEAR(clear_module_state->__pyx_float_43_87);
  Py_CLEAR(clear_module_state->__pyx_float_43_89);
  Py_CLEAR(clear_module_state->__pyx_float_43_93);
  Py_CLEAR(clear_module_state->__pyx_float_44_02);
  Py_CLEAR(clear_module_state->__pyx_float_44_03);
  Py_CLEAR(clear_module_state->__pyx_float_44_10);
  Py_CLEAR(clear_module_state->__pyx_float_44_14);
  Py_CLEAR(clear_module_state->__pyx_float_44_17);
  Py_CLEAR(clear_module_state->__pyx_float_44_21);
  Py_CLEAR(clear_module_state->__pyx_float_44_24);
  Py_CLEAR(clear_module_state->__pyx_float_44_31);
  Py_CLEAR(clear_module_state->__pyx_float_44_48);
  Py_CLEAR(clear_module_state->__pyx_float_44_51);
  Py_CLEAR(clear_module_state->__pyx_float_44_52);
  Py_CLEAR(clear_module_state->__pyx_float_44_53);
  Py_CLEAR(clear_module_state->__pyx_float_44_68);
  Py_CLEAR(clear_module_state->__pyx_float_44_73);
  Py_CLEAR(clear_module_state->__pyx_float_44_74);
  Py_CLEAR(clear_module_state->__pyx_float_44_94);
  Py_CLEAR(clear_module_state->__pyx_float_45_03);
  Py_CLEAR(clear_module_state->__pyx_float_45_05);
  Py_CLEAR(clear_module_state->__pyx_float_45_12);
  Py_CLEAR(clear_module_state->__pyx_float_45_13);
  Py_CLEAR(clear_module_state->__pyx_float_45_24);
  Py_CLEAR(clear_module_state->__pyx_float_45_25);
  Py_CLEAR(clear_module_state->__pyx_float_45_29);
  Py_CLEAR(clear_module_state->__pyx_float_45_36);
  Py_CLEAR(clear_module_state->__pyx_float_45_38);
  Py_CLEAR(clear_module_state->__pyx_float_45_42);
  Py_CLEAR(clear_module_state->__pyx_float_45_49);
  Py_CLEAR(clear_module_state->__pyx_float_45_62);
  Py_CLEAR(clear_module_state->__pyx_float_45_65);
  Py_CLEAR(clear_module_state->__pyx_float_45_68);
  Py_CLEAR(clear_module_state->__pyx_float_45_85);
  Py_CLEAR(clear_module_state->__pyx_float_45_95);
  Py_CLEAR(clear_module_state->__pyx_float_45_99);
  Py_CLEAR(clear_module_state->__pyx_float_46_02);
  Py_CLEAR(clear_module_state->__pyx_float_46_11);
  Py_CLEAR(clear_module_state->__pyx_float_46_13);
  Py_CLEAR(clear_module_state->__pyx_float_46_18);
  Py_CLEAR(clear_module_state->__pyx_float_46_20);
  Py_CLEAR(clear_module_state->__pyx_float_46_33);
  Py_CLEAR(clear_module_state->__pyx_float_46_35);
  Py_CLEAR(clear_module_state->__pyx_float_46_36);
  Py_CLEAR(clear_module_state->__pyx_float_46_42);
  Py_CLEAR(clear_module_state->__pyx_float_46_43);
  Py_CLEAR(clear_module_state->__pyx_float_46_54);
  Py_CLEAR(clear_module_state->__pyx_float_46_56);
  Py_CLEAR(clear_module_state->__pyx_float_46_57);
  Py_CLEAR(clear_module_state->__pyx_float_46_62);
  Py_CLEAR(clear_module_state->__pyx_float_46_65);
  Py_CLEAR(clear_module_state->__pyx_float_46_68);
  Py_CLEAR(clear_module_state->__pyx_float_46_86);
  Py_CLEAR(clear_module_state->__pyx_float_46_93);
  Py_CLEAR(clear_module_state->__pyx_float_47_00);
  Py_CLEAR(clear_module_state->__pyx_float_47_07);
  Py_CLEAR(clear_module_state->__pyx_float_47_11);
  Py_CLEAR(clear_module_state->__pyx_float_47_16);
  Py_CLEAR(clear_module_state->__pyx_float_47_20);
  Py_CLEAR(clear_module_state->__pyx_float_47_21);
  Py_CLEAR(clear_module_state->__pyx_float_47_25);
  Py_CLEAR(clear_module_state->__pyx_float_47_40);
  Py_CLEAR(clear_module_state->__pyx_float_47_41);
  Py_CLEAR(clear_module_state->__pyx_float_47_46);
  Py_CLEAR(clear_module_state->__pyx_float_47_50);
  Py_CLEAR(clear_module_state->__pyx_float_47_60);
  Py_CLEAR(clear_module_state->__pyx_float_47_64);
  Py_CLEAR(clear_module_state->__pyx_float_47_65);
  Py_CLEAR(clear_module_state->__pyx_float_47_72);
  Py_CLEAR(clear_module_state->__pyx_float_47_78);
  Py_CLEAR(clear_module_state->__pyx_float_47_79);
  Py_CLEAR(clear_module_state->__pyx_float_47_81);
  Py_CLEAR(clear_module_state->__pyx_float_47_82);
  Py_CLEAR(clear_module_state->__pyx_float_47_87);
  Py_CLEAR(clear_module_state->__pyx_float_48_07);
  Py_CLEAR(clear_module_state->__pyx_float_48_11);
  Py_CLEAR(clear_module_state->__pyx_float_48_28);
  Py_CLEAR(clear_module_state->__pyx_float_48_36);
  Py_CLEAR(clear_module_state->__pyx_float_48_40);
  Py_CLEAR(clear_module_state->__pyx_float_48_42);
  Py_CLEAR(clear_module_state->__pyx_float_48_43);
  Py_CLEAR(clear_module_state->__pyx_float_48_44);
  Py_CLEAR(clear_module_state->__pyx_float_48_52);
  Py_CLEAR(clear_module_state->__pyx_float_48_57);
  Py_CLEAR(clear_module_state->__pyx_float_48_65);
  Py_CLEAR(clear_module_state->__pyx_float_48_68);
  Py_CLEAR(clear_module_state->__pyx_float_48_71);
  Py_CLEAR(clear_module_state->__pyx_float_48_76);
  Py_CLEAR(clear_module_state->__pyx_float_48_79);
  Py_CLEAR(clear_module_state->__pyx_float_48_83);
  Py_CLEAR(clear_module_state->__pyx_float_48_87);
  Py_CLEAR(clear_module_state->__pyx_float_48_88);
  Py_CLEAR(clear_module_state->__pyx_float_48_89);
  Py_CLEAR(clear_module_state->__pyx_float_48_95);
  Py_CLEAR(clear_module_state->__pyx_float_48_98);
  Py_CLEAR(clear_module_state->__pyx_float_48_99);
  Py_CLEAR(clear_module_state->__pyx_float_49_01);
  Py_CLEAR(clear_module_state->__pyx_float_49_14);
  Py_CLEAR(clear_module_state->__pyx_float_49_27);
  Py_CLEAR(clear_module_state->__pyx_float_49_29);
  Py_CLEAR(clear_module_state->__pyx_float_49_44);
  Py_CLEAR(clear_module_state->__pyx_float_49_48);
  Py_CLEAR(clear_module_state->__pyx_float_49_55);
  Py_CLEAR(clear_module_state->__pyx_float_49_65);
  Py_CLEAR(clear_module_state->__pyx_float_49_67);
  Py_CLEAR(clear_module_state->__pyx_float_49_68);
  Py_CLEAR(clear_module_state->__pyx_float_49_87);
  Py_CLEAR(clear_module_state->__pyx_float_49_93);
  Py_CLEAR(clear_module_state->__pyx_float_49_95);
  Py_CLEAR(clear_module_state->__pyx_float_50_01);
  Py_CLEAR(clear_module_state->__pyx_float_50_03);
  Py_CLEAR(clear_module_state->__pyx_float_50_10);
  Py_CLEAR(clear_module_state->__pyx_float_50_11);
  Py_CLEAR(clear_module_state->__pyx_float_50_13);
  Py_CLEAR(clear_module_state->__pyx_float_50_18);
  Py_CLEAR(clear_module_state->__pyx_float_50_21);
  Py_CLEAR(clear_module_state->__pyx_float_50_48);
  Py_CLEAR(clear_module_state->__pyx_float_50_49);
  Py_CLEAR(clear_module_state->__pyx_float_50_63);
  Py_CLEAR(clear_module_state->__pyx_float_50_72);
  Py_CLEAR(clear_module_state->__pyx_float_50_76);
  Py_CLEAR(clear_module_state->__pyx_float_50_83);
  Py_CLEAR(clear_module_state->__pyx_float_50_88);
  Py_CLEAR(clear_module_state->__pyx_float_50_92);
  Py_CLEAR(clear_module_state->__pyx_float_50_96);
  Py_CLEAR(clear_module_state->__pyx_float_51_09);
  Py_CLEAR(clear_module_state->__pyx_float_51_12);
  Py_CLEAR(clear_module_state->__pyx_float_51_19);
  Py_CLEAR(clear_module_state->__pyx_float_51_27);
  Py_CLEAR(clear_module_state->__pyx_float_51_29);
  Py_CLEAR(clear_module_state->__pyx_float_51_34);
  Py_CLEAR(clear_module_state->__pyx_float_51_37);
  Py_CLEAR(clear_module_state->__pyx_float_51_42);
  Py_CLEAR(clear_module_state->__pyx_float_51_54);
  Py_CLEAR(clear_module_state->__pyx_float_51_62);
  Py_CLEAR(clear_module_state->__pyx_float_51_81);
  Py_CLEAR(clear_module_state->__pyx_float_51_93);
  Py_CLEAR(clear_module_state->__pyx_float_52_00);
  Py_CLEAR(clear_module_state->__pyx_float_52_15);
  Py_CLEAR(clear_module_state->__pyx_float_52_17);
  Py_CLEAR(clear_module_state->__pyx_float_52_33);
  Py_CLEAR(clear_module_state->__pyx_float_52_43);
  Py_CLEAR(clear_module_state->__pyx_float_52_47);
  Py_CLEAR(clear_module_state->__pyx_float_52_70);
  Py_CLEAR(clear_module_state->__pyx_float_52_73);
  Py_CLEAR(clear_module_state->__pyx_float_52_95);
  Py_CLEAR(clear_module_state->__pyx_float_53_42);
  Py_CLEAR(clear_module_state->__pyx_float_53_46);
  Py_CLEAR(clear_module_state->__pyx_float_53_52);
  Py_CLEAR(clear_module_state->__pyx_float_53_56);
  Py_CLEAR(clear_module_state->__pyx_float_53_68);
  Py_CLEAR(clear_module_state->__pyx_float_53_79);
  Py_CLEAR(clear_module_state->__pyx_float_53_82);
  Py_CLEAR(clear_module_state->__pyx_float_53_90);
  Py_CLEAR(clear_module_state->__pyx_float_53_91);
  Py_CLEAR(clear_module_state->__pyx_float_54_00);
  Py_CLEAR(clear_module_state->__pyx_float_54_05);
  Py_CLEAR(clear_module_state->__pyx_float_54_15);
  Py_CLEAR(clear_module_state->__pyx_float_54_23);
  Py_CLEAR(clear_module_state->__pyx_float_54_39);
  Py_CLEAR(clear_module_state->__pyx_float_54_69);
  Py_CLEAR(clear_module_state->__pyx_float_54_74);
  Py_CLEAR(clear_module_state->__pyx_float_54_75);
  Py_CLEAR(clear_module_state->__pyx_float_54_76);
  Py_CLEAR(clear_module_state->__pyx_float_55_02);
  Py_CLEAR(clear_module_state->__pyx_float_55_06);
  Py_CLEAR(clear_module_state->__pyx_float_55_09);
  Py_CLEAR(clear_module_state->__pyx_float_55_14);
  Py_CLEAR(clear_module_state->__pyx_float_55_24);
  Py_CLEAR(clear_module_state->__pyx_float_55_30);
  Py_CLEAR(clear_module_state->__pyx_float_55_31);
  Py_CLEAR(clear_module_state->__pyx_float_55_34);
  Py_CLEAR(clear_module_state->__pyx_float_55_45);
  Py_CLEAR(clear_module_state->__pyx_float_55_50);
  Py_CLEAR(clear_module_state->__pyx_float_55_58);
  Py_CLEAR(clear_module_state->__pyx_float_55_75);
  Py_CLEAR(clear_module_state->__pyx_float_55_88);
  Py_CLEAR(clear_module_state->__pyx_float_55_90);
  Py_CLEAR(clear_module_state->__pyx_float_55_98);
  Py_CLEAR(clear_module_state->__pyx_float_56_04);
  Py_CLEAR(clear_module_state->__pyx_float_56_06);
  Py_CLEAR(clear_module_state->__pyx_float_56_17);
  Py_CLEAR(clear_module_state->__pyx_float_56_18);
  Py_CLEAR(clear_module_state->__pyx_float_56_20);
  Py_CLEAR(clear_module_state->__pyx_float_56_23);
  Py_CLEAR(clear_module_state->__pyx_float_56_31);
  Py_CLEAR(clear_module_state->__pyx_float_56_60);
  Py_CLEAR(clear_module_state->__pyx_float_56_62);
  Py_CLEAR(clear_module_state->__pyx_float_56_68);
  Py_CLEAR(clear_module_state->__pyx_float_56_73);
  Py_CLEAR(clear_module_state->__pyx_float_56_90);
  Py_CLEAR(clear_module_state->__pyx_float_56_98);
  Py_CLEAR(clear_module_state->__pyx_float_57_02);
  Py_CLEAR(clear_module_state->__pyx_float_57_10);
  Py_CLEAR(clear_module_state->__pyx_float_57_21);
  Py_CLEAR(clear_module_state->__pyx_float_57_26);
  Py_CLEAR(clear_module_state->__pyx_float_57_34);
  Py_CLEAR(clear_module_state->__pyx_float_57_40);
  Py_CLEAR(clear_module_state->__pyx_float_57_41);
  Py_CLEAR(clear_module_state->__pyx_float_57_53);
  Py_CLEAR(clear_module_state->__pyx_float_57_63);
  Py_CLEAR(clear_module_state->__pyx_float_57_69);
  Py_CLEAR(clear_module_state->__pyx_float_57_81);
  Py_CLEAR(clear_module_state->__pyx_float_57_90);
  Py_CLEAR(clear_module_state->__pyx_float_57_96);
  Py_CLEAR(clear_module_state->__pyx_float_57_97);
  Py_CLEAR(clear_module_state->__pyx_float_57_98);
  Py_CLEAR(clear_module_state->__pyx_float_58_03);
  Py_CLEAR(clear_module_state->__pyx_float_58_11);
  Py_CLEAR(clear_module_state->__pyx_float_58_17);
  Py_CLEAR(clear_module_state->__pyx_float_58_28);
  Py_CLEAR(clear_module_state->__pyx_float_58_34);
  Py_CLEAR(clear_module_state->__pyx_float_58_42);
  Py_CLEAR(clear_module_state->__pyx_float_58_53);
  Py_CLEAR(clear_module_state->__pyx_float_58_56);
  Py_CLEAR(clear_module_state->__pyx_float_58_67);
  Py_CLEAR(clear_module_state->__pyx_float_58_71);
  Py_CLEAR(clear_module_state->__pyx_float_58_75);
  Py_CLEAR(clear_module_state->__pyx_float_58_79);
  Py_CLEAR(clear_module_state->__pyx_float_58_86);
  Py_CLEAR(clear_module_state->__pyx_float_59_02);
  Py_CLEAR(clear_module_state->__pyx_float_59_13);
  Py_CLEAR(clear_module_state->__pyx_float_59_16);
  Py_CLEAR(clear_module_state->__pyx_float_59_21);
  Py_CLEAR(clear_module_state->__pyx_float_59_29);
  Py_CLEAR(clear_module_state->__pyx_float_59_30);
  Py_CLEAR(clear_module_state->__pyx_float_59_33);
  Py_CLEAR(clear_module_state->__pyx_float_59_40);
  Py_CLEAR(clear_module_state->__pyx_float_59_47);
  Py_CLEAR(clear_module_state->__pyx_float_59_70);
  Py_CLEAR(clear_module_state->__pyx_float_59_81);
  Py_CLEAR(clear_module_state->__pyx_float_59_83);
  Py_CLEAR(clear_module_state->__pyx_float_59_86);
  Py_CLEAR(clear_module_state->__pyx_float_59_95);
  Py_CLEAR(clear_module_state->__pyx_float_59_96);
  Py_CLEAR(clear_module_state->__pyx_float_60_00);
  Py_CLEAR(clear_module_state->__pyx_float_60_03);
  Py_CLEAR(clear_module_state->__pyx_float_60_09);
  Py_CLEAR(clear_module_state->__pyx_float_60_23);
  Py_CLEAR(clear_module_state->__pyx_float_60_33);
  Py_CLEAR(clear_module_state->__pyx_float_60_35);
  Py_CLEAR(clear_module_state->__pyx_float_60_42);
  Py_CLEAR(clear_module_state->__pyx_float_60_59);
  Py_CLEAR(clear_module_state->__pyx_float_60_62);
  Py_CLEAR(clear_module_state->__pyx_float_60_67);
  Py_CLEAR(clear_module_state->__pyx_float_60_75);
  Py_CLEAR(clear_module_state->__pyx_float_61_05);
  Py_CLEAR(clear_module_state->__pyx_float_61_13);
  Py_CLEAR(clear_module_state->__pyx_float_61_21);
  Py_CLEAR(clear_module_state->__pyx_float_61_29);
  Py_CLEAR(clear_module_state->__pyx_float_61_34);
  Py_CLEAR(clear_module_state->__pyx_float_61_37);
  Py_CLEAR(clear_module_state->__pyx_float_61_39);
  Py_CLEAR(clear_module_state->__pyx_float_61_40);
  Py_CLEAR(clear_module_state->__pyx_float_61_56);
  Py_CLEAR(clear_module_state->__pyx_float_61_59);
  Py_CLEAR(clear_module_state->__pyx_float_61_62);
  Py_CLEAR(clear_module_state->__pyx_float_61_66);
  Py_CLEAR(clear_module_state->__pyx_float_61_99);
  Py_CLEAR(clear_module_state->__pyx_float_62_04);
  Py_CLEAR(clear_module_state->__pyx_float_62_20);
  Py_CLEAR(clear_module_state->__pyx_float_62_22);
  Py_CLEAR(clear_module_state->__pyx_float_62_54);
  Py_CLEAR(clear_module_state->__pyx_float_62_56);
  Py_CLEAR(clear_module_state->__pyx_float_62_72);
  Py_CLEAR(clear_module_state->__pyx_float_62_84);
  Py_CLEAR(clear_module_state->__pyx_float_62_86);
  Py_CLEAR(clear_module_state->__pyx_float_63_08);
  Py_CLEAR(clear_module_state->__pyx_float_63_48);
  Py_CLEAR(clear_module_state->__pyx_float_63_55);
  Py_CLEAR(clear_module_state->__pyx_float_63_78);
  Py_CLEAR(clear_module_state->__pyx_float_63_97);
  Py_CLEAR(clear_module_state->__pyx_float_64_29);
  Py_CLEAR(clear_module_state->__pyx_float_64_36);
  Py_CLEAR(clear_module_state->__pyx_float_64_82);
  Py_CLEAR(clear_module_state->__pyx_float_64_83);
  Py_CLEAR(clear_module_state->__pyx_float_64_86);
  Py_CLEAR(clear_module_state->__pyx_float_64_97);
  Py_CLEAR(clear_module_state->__pyx_float_65_09);
  Py_CLEAR(clear_module_state->__pyx_float_65_19);
  Py_CLEAR(clear_module_state->__pyx_float_65_20);
  Py_CLEAR(clear_module_state->__pyx_float_65_42);
  Py_CLEAR(clear_module_state->__pyx_float_65_54);
  Py_CLEAR(clear_module_state->__pyx_float_65_65);
  Py_CLEAR(clear_module_state->__pyx_float_65_89);
  Py_CLEAR(clear_module_state->__pyx_float_66_12);
  Py_CLEAR(clear_module_state->__pyx_float_66_24);
  Py_CLEAR(clear_module_state->__pyx_float_66_26);
  Py_CLEAR(clear_module_state->__pyx_float_66_34);
  Py_CLEAR(clear_module_state->__pyx_float_66_44);
  Py_CLEAR(clear_module_state->__pyx_float_66_47);
  Py_CLEAR(clear_module_state->__pyx_float_66_48);
  Py_CLEAR(clear_module_state->__pyx_float_66_56);
  Py_CLEAR(clear_module_state->__pyx_float_66_60);
  Py_CLEAR(clear_module_state->__pyx_float_66_64);
  Py_CLEAR(clear_module_state->__pyx_float_66_72);
  Py_CLEAR(clear_module_state->__pyx_float_66_91);
  Py_CLEAR(clear_module_state->__pyx_float_66_92);
  Py_CLEAR(clear_module_state->__pyx_float_66_96);
  Py_CLEAR(clear_module_state->__pyx_float_67_18);
  Py_CLEAR(clear_module_state->__pyx_float_67_23);
  Py_CLEAR(clear_module_state->__pyx_float_67_37);
  Py_CLEAR(clear_module_state->__pyx_float_67_46);
  Py_CLEAR(clear_module_state->__pyx_float_67_63);
  Py_CLEAR(clear_module_state->__pyx_float_67_70);
  Py_CLEAR(clear_module_state->__pyx_float_67_83);
  Py_CLEAR(clear_module_state->__pyx_float_67_94);
  Py_CLEAR(clear_module_state->__pyx_float_67_97);
  Py_CLEAR(clear_module_state->__pyx_float_67_98);
  Py_CLEAR(clear_module_state->__pyx_float_68_18);
  Py_CLEAR(clear_module_state->__pyx_float_68_50);
  Py_CLEAR(clear_module_state->__pyx_float_68_61);
  Py_CLEAR(clear_module_state->__pyx_float_68_65);
  Py_CLEAR(clear_module_state->__pyx_float_68_81);
  Py_CLEAR(clear_module_state->__pyx_float_68_85);
  Py_CLEAR(clear_module_state->__pyx_float_68_90);
  Py_CLEAR(clear_module_state->__pyx_float_68_94);
  Py_CLEAR(clear_module_state->__pyx_float_68_99);
  Py_CLEAR(clear_module_state->__pyx_float_69_09);
  Py_CLEAR(clear_module_state->__pyx_float_69_13);
  Py_CLEAR(clear_module_state->__pyx_float_69_17);
  Py_CLEAR(clear_module_state->__pyx_float_69_33);
  Py_CLEAR(clear_module_state->__pyx_float_69_37);
  Py_CLEAR(clear_module_state->__pyx_float_69_60);
  Py_CLEAR(clear_module_state->__pyx_float_69_78);
  Py_CLEAR(clear_module_state->__pyx_float_69_91);
  Py_CLEAR(clear_module_state->__pyx_float_69_97);
  Py_CLEAR(clear_module_state->__pyx_float_70_10);
  Py_CLEAR(clear_module_state->__pyx_float_70_25);
  Py_CLEAR(clear_module_state->__pyx_float_70_42);
  Py_CLEAR(clear_module_state->__pyx_float_70_43);
  Py_CLEAR(clear_module_state->__pyx_float_70_52);
  Py_CLEAR(clear_module_state->__pyx_float_70_53);
  Py_CLEAR(clear_module_state->__pyx_float_70_58);
  Py_CLEAR(clear_module_state->__pyx_float_70_68);
  Py_CLEAR(clear_module_state->__pyx_float_70_73);
  Py_CLEAR(clear_module_state->__pyx_float_70_88);
  Py_CLEAR(clear_module_state->__pyx_float_71_17);
  Py_CLEAR(clear_module_state->__pyx_float_71_21);
  Py_CLEAR(clear_module_state->__pyx_float_71_25);
  Py_CLEAR(clear_module_state->__pyx_float_71_36);
  Py_CLEAR(clear_module_state->__pyx_float_71_59);
  Py_CLEAR(clear_module_state->__pyx_float_71_72);
  Py_CLEAR(clear_module_state->__pyx_float_71_84);
  Py_CLEAR(clear_module_state->__pyx_float_72_00);
  Py_CLEAR(clear_module_state->__pyx_float_72_01);
  Py_CLEAR(clear_module_state->__pyx_float_72_14);
  Py_CLEAR(clear_module_state->__pyx_float_72_24);
  Py_CLEAR(clear_module_state->__pyx_float_72_45);
  Py_CLEAR(clear_module_state->__pyx_float_73_05);
  Py_CLEAR(clear_module_state->__pyx_float_73_28);
  Py_CLEAR(clear_module_state->__pyx_float_73_50);
  Py_CLEAR(clear_module_state->__pyx_float_73_53);
  Py_CLEAR(clear_module_state->__pyx_float_73_57);
  Py_CLEAR(clear_module_state->__pyx_float_73_62);
  Py_CLEAR(clear_module_state->__pyx_float_73_68);
  Py_CLEAR(clear_module_state->__pyx_float_74_22);
  Py_CLEAR(clear_module_state->__pyx_float_74_40);
  Py_CLEAR(clear_module_state->__pyx_float_74_95);
  Py_CLEAR(clear_module_state->__pyx_float_75_00);
  Py_CLEAR(clear_module_state->__pyx_float_75_44);
  Py_CLEAR(clear_module_state->__pyx_float_75_49);
  Py_CLEAR(clear_module_state->__pyx_float_75_52);
  Py_CLEAR(clear_module_state->__pyx_float_75_76);
  Py_CLEAR(clear_module_state->__pyx_float_75_92);
  Py_CLEAR(clear_module_state->__pyx_float_76_12);
  Py_CLEAR(clear_module_state->__pyx_float_76_45);
  Py_CLEAR(clear_module_state->__pyx_float_76_52);
  Py_CLEAR(clear_module_state->__pyx_float_76_53);
  Py_CLEAR(clear_module_state->__pyx_float_76_96);
  Py_CLEAR(clear_module_state->__pyx_float_77_03);
  Py_CLEAR(clear_module_state->__pyx_float_77_08);
  Py_CLEAR(clear_module_state->__pyx_float_77_13);
  Py_CLEAR(clear_module_state->__pyx_float_77_93);
  Py_CLEAR(clear_module_state->__pyx_float_77_97);
  Py_CLEAR(clear_module_state->__pyx_float_78_00);
  Py_CLEAR(clear_module_state->__pyx_float_78_02);
  Py_CLEAR(clear_module_state->__pyx_float_78_31);
  Py_CLEAR(clear_module_state->__pyx_float_78_32);
  Py_CLEAR(clear_module_state->__pyx_float_78_45);
  Py_CLEAR(clear_module_state->__pyx_float_78_94);
  Py_CLEAR(clear_module_state->__pyx_float_79_27);
  Py_CLEAR(clear_module_state->__pyx_float_79_38);
  Py_CLEAR(clear_module_state->__pyx_float_79_48);
  Py_CLEAR(clear_module_state->__pyx_float_79_49);
  Py_CLEAR(clear_module_state->__pyx_float_79_57);
  Py_CLEAR(clear_module_state->__pyx_float_79_65);
  Py_CLEAR(clear_module_state->__pyx_float_79_73);
  Py_CLEAR(clear_module_state->__pyx_float_79_83);
  Py_CLEAR(clear_module_state->__pyx_float_80_16);
  Py_CLEAR(clear_module_state->__pyx_float_80_39);
  Py_CLEAR(clear_module_state->__pyx_float_80_44);
  Py_CLEAR(clear_module_state->__pyx_float_80_77);
  Py_CLEAR(clear_module_state->__pyx_float_81_00);
  Py_CLEAR(clear_module_state->__pyx_float_81_16);
  Py_CLEAR(clear_module_state->__pyx_float_81_33);
  Py_CLEAR(clear_module_state->__pyx_float_81_56);
  Py_CLEAR(clear_module_state->__pyx_float_81_69);
  Py_CLEAR(clear_module_state->__pyx_float_81_75);
  Py_CLEAR(clear_module_state->__pyx_float_81_87);
  Py_CLEAR(clear_module_state->__pyx_float_82_06);
  Py_CLEAR(clear_module_state->__pyx_float_82_39);
  Py_CLEAR(clear_module_state->__pyx_float_82_44);
  Py_CLEAR(clear_module_state->__pyx_float_82_47);
  Py_CLEAR(clear_module_state->__pyx_float_82_52);
  Py_CLEAR(clear_module_state->__pyx_float_82_59);
  Py_CLEAR(clear_module_state->__pyx_float_82_70);
  Py_CLEAR(clear_module_state->__pyx_float_82_99);
  Py_CLEAR(clear_module_state->__pyx_float_83_17);
  Py_CLEAR(clear_module_state->__pyx_float_83_53);
  Py_CLEAR(clear_module_state->__pyx_float_83_91);
  Py_CLEAR(clear_module_state->__pyx_float_83_97);
  Py_CLEAR(clear_module_state->__pyx_float_84_01);
  Py_CLEAR(clear_module_state->__pyx_float_84_10);
  Py_CLEAR(clear_module_state->__pyx_float_84_22);
  Py_CLEAR(clear_module_state->__pyx_float_84_61);
  Py_CLEAR(clear_module_state->__pyx_float_84_81);
  Py_CLEAR(clear_module_state->__pyx_float_85_28);
  Py_CLEAR(clear_module_state->__pyx_float_85_41);
  Py_CLEAR(clear_module_state->__pyx_float_85_42);
  Py_CLEAR(clear_module_state->__pyx_float_85_97);
  Py_CLEAR(clear_module_state->__pyx_float_86_88);
  Py_CLEAR(clear_module_state->__pyx_float_87_27);
  Py_CLEAR(clear_module_state->__pyx_float_87_46);
  Py_CLEAR(clear_module_state->__pyx_float_87_56);
  Py_CLEAR(clear_module_state->__pyx_float_87_65);
  Py_CLEAR(clear_module_state->__pyx_float_88_36);
  Py_CLEAR(clear_module_state->__pyx_float_88_39);
  Py_CLEAR(clear_module_state->__pyx_float_89_08);
  Py_CLEAR(clear_module_state->__pyx_float_89_85);
  Py_CLEAR(clear_module_state->__pyx_float_90_29);
  Py_CLEAR(clear_module_state->__pyx_float_90_30);
  Py_CLEAR(clear_module_state->__pyx_float_90_55);
  Py_CLEAR(clear_module_state->__pyx_float_90_70);
  Py_CLEAR(clear_module_state->__pyx_float_91_35);
  Py_CLEAR(clear_module_state->__pyx_float_91_75);
  Py_CLEAR(clear_module_state->__pyx_float_92_09);
  Py_CLEAR(clear_module_state->__pyx_float_92_30);
  Py_CLEAR(clear_module_state->__pyx_float_92_86);
  Py_CLEAR(clear_module_state->__pyx_float_93_28);
  Py_CLEAR(clear_module_state->__pyx_float_93_40);
  Py_CLEAR(clear_module_state->__pyx_float_93_52);
  Py_CLEAR(clear_module_state->__pyx_float_94_02);
  Py_CLEAR(clear_module_state->__pyx_float_95_26);
  Py_CLEAR(clear_module_state->__pyx_float_95_59);
  Py_CLEAR(clear_module_state->__pyx_float_97_07);
  Py_CLEAR(clear_module_state->__pyx_float_97_22);
  Py_CLEAR(clear_module_state->__pyx_float_98_86);
  Py_CLEAR(clear_module_state->__pyx_float_105_28);
  Py_CLEAR(clear_module_state->__pyx_float_106_22);
  Py_CLEAR(clear_module_state->__pyx_float_107_45);
  Py_CLEAR(clear_module_state->__pyx_float_108_37);
  Py_CLEAR(clear_module_state->__pyx_float_108_70);
  Py_CLEAR(clear_module_state->__pyx_float_109_39);
  Py_CLEAR(clear_module_state->__pyx_float_122_08);
  Py_CLEAR(clear_module_state->__pyx_float_125_54);
  Py_CLEAR(clear_module_state->__pyx_float_32_703);
  Py_CLEAR(clear_module_state->__pyx_float_10000_0);
  Py_CLEAR(clear_module_state->__pyx_int_0);
  Py_CLEAR(clear_module_state->__pyx_int_1);
  Py_CLEAR(clear_module_state->__pyx_int_2);
  Py_CLEAR(clear_module_state->__pyx_int_4);
  Py_CLEAR(clear_module_state->__pyx_int_12);
  Py_CLEAR(clear_module_state->__pyx_int_45);
  Py_CLEAR(clear_module_state->__pyx_int_100);
  Py_CLEAR(clear_module_state->__pyx_int_220);
  Py_CLEAR(clear_module_state->__pyx_int_261);
  Py_CLEAR(clear_module_state->__pyx_int_440);
  Py_CLEAR(clear_module_state->__pyx_tuple_);
  Py_CLEAR(clear_module_state->__pyx_slice__3);
  Py_CLEAR(clear_module_state->__pyx_slice__4);
  Py_CLEAR(clear_module_state->__pyx_tuple__2);
  Py_CLEAR(clear_module_state->__pyx_tuple__6);
  Py_CLEAR(clear_module_state->__pyx_tuple__7);
  Py_CLEAR(clear_module_state->__pyx_tuple__9);
  Py_CLEAR(clear_module_state->__pyx_tuple__11);
  Py_CLEAR(clear_module_state->__pyx_tuple__13);
  Py_CLEAR(clear_module_state->__pyx_tuple__14);
  Py_CLEAR(clear_module_state->__pyx_codeobj__8);
  Py_CLEAR(clear_module_state->__pyx_codeobj__10);
  Py_CLEAR(clear_module_state->__pyx_codeobj__12);
  return 0;
}
#endif
/* #### Code section: module_state_traverse ### */
#if CYTHON_USE_MODULE_STATE
static int __pyx_m_traverse(PyObject *m, visitproc visit, void *arg) {
  __pyx_mstate *traverse_module_state = __pyx_mstate(m);
  if (!traverse_module_state) return 0;
  Py_VISIT(traverse_module_state->__pyx_d);
  Py_VISIT(traverse_module_state->__pyx_b);
  Py_VISIT(traverse_module_state->__pyx_cython_runtime);
  Py_VISIT(traverse_module_state->__pyx_empty_tuple);
  Py_VISIT(traverse_module_state->__pyx_empty_bytes);
  Py_VISIT(traverse_module_state->__pyx_empty_unicode);
  #ifdef __Pyx_CyFunction_USED
  Py_VISIT(traverse_module_state->__pyx_CyFunctionType);
  #endif
  #ifdef __Pyx_FusedFunction_USED
  Py_VISIT(traverse_module_state->__pyx_FusedFunctionType);
  #endif
  Py_VISIT(traverse_module_state->__pyx_ptype_7cpython_4type_type);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_dtype);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_flatiter);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_broadcast);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_ndarray);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_generic);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_number);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_integer);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_signedinteger);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_unsignedinteger);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_inexact);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_floating);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_complexfloating);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_flexible);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_character);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_ufunc);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_3clm_mus_any);
  Py_VISIT(traverse_module_state->__pyx_n_s_ImportError);
  Py_VISIT(traverse_module_state->__pyx_n_s_PIANO_SPECTRA);
  Py_VISIT(traverse_module_state->__pyx_n_s_Sound);
  Py_VISIT(traverse_module_state->__pyx_n_s__15);
  Py_VISIT(traverse_module_state->__pyx_n_s__5);
  Py_VISIT(traverse_module_state->__pyx_n_s_amplitude);
  Py_VISIT(traverse_module_state->__pyx_n_s_asyncio_coroutines);
  Py_VISIT(traverse_module_state->__pyx_n_s_attackTime);
  Py_VISIT(traverse_module_state->__pyx_n_s_base);
  Py_VISIT(traverse_module_state->__pyx_n_s_class_getitem);
  Py_VISIT(traverse_module_state->__pyx_n_s_cline_in_traceback);
  Py_VISIT(traverse_module_state->__pyx_n_s_clm);
  Py_VISIT(traverse_module_state->__pyx_n_s_db_drop_per_second);
  Py_VISIT(traverse_module_state->__pyx_n_s_degree);
  Py_VISIT(traverse_module_state->__pyx_n_s_distance);
  Py_VISIT(traverse_module_state->__pyx_n_s_double);
  Py_VISIT(traverse_module_state->__pyx_n_s_dtype);
  Py_VISIT(traverse_module_state->__pyx_n_s_dur);
  Py_VISIT(traverse_module_state->__pyx_n_s_duration);
  Py_VISIT(traverse_module_state->__pyx_n_s_enter);
  Py_VISIT(traverse_module_state->__pyx_n_s_exit);
  Py_VISIT(traverse_module_state->__pyx_n_s_freq);
  Py_VISIT(traverse_module_state->__pyx_n_s_frequency);
  Py_VISIT(traverse_module_state->__pyx_n_s_get_piano_partials);
  Py_VISIT(traverse_module_state->__pyx_n_s_import);
  Py_VISIT(traverse_module_state->__pyx_n_s_initializing);
  Py_VISIT(traverse_module_state->__pyx_n_s_is_coroutine);
  Py_VISIT(traverse_module_state->__pyx_n_s_is_number);
  Py_VISIT(traverse_module_state->__pyx_n_s_lbj_piano);
  Py_VISIT(traverse_module_state->__pyx_n_s_log);
  Py_VISIT(traverse_module_state->__pyx_n_s_main);
  Py_VISIT(traverse_module_state->__pyx_n_u_main);
  Py_VISIT(traverse_module_state->__pyx_n_s_make_env);
  Py_VISIT(traverse_module_state->__pyx_n_s_make_locsig);
  Py_VISIT(traverse_module_state->__pyx_n_s_make_oscil_bank);
  Py_VISIT(traverse_module_state->__pyx_n_s_make_piano_ampfun);
  Py_VISIT(traverse_module_state->__pyx_n_s_math);
  Py_VISIT(traverse_module_state->__pyx_n_s_name);
  Py_VISIT(traverse_module_state->__pyx_n_s_np);
  Py_VISIT(traverse_module_state->__pyx_n_s_numpy);
  Py_VISIT(traverse_module_state->__pyx_kp_u_numpy_core_multiarray_failed_to);
  Py_VISIT(traverse_module_state->__pyx_kp_u_numpy_core_umath_failed_to_impor);
  Py_VISIT(traverse_module_state->__pyx_n_s_pfreq);
  Py_VISIT(traverse_module_state->__pyx_n_s_piano_attack_duration);
  Py_VISIT(traverse_module_state->__pyx_n_s_play);
  Py_VISIT(traverse_module_state->__pyx_n_s_pysndlib);
  Py_VISIT(traverse_module_state->__pyx_n_s_pysndlib_clm);
  Py_VISIT(traverse_module_state->__pyx_n_s_pysndlib_instruments_lbj_piano);
  Py_VISIT(traverse_module_state->__pyx_n_s_pyx_vtable);
  Py_VISIT(traverse_module_state->__pyx_n_s_random);
  Py_VISIT(traverse_module_state->__pyx_n_s_range);
  Py_VISIT(traverse_module_state->__pyx_n_s_releaseAmp);
  Py_VISIT(traverse_module_state->__pyx_n_s_reverb_amount);
  Py_VISIT(traverse_module_state->__pyx_n_s_round);
  Py_VISIT(traverse_module_state->__pyx_n_s_scaler);
  Py_VISIT(traverse_module_state->__pyx_n_s_spec);
  Py_VISIT(traverse_module_state->__pyx_kp_s_src_pysndlib_instruments_lbj_pia);
  Py_VISIT(traverse_module_state->__pyx_n_s_stable);
  Py_VISIT(traverse_module_state->__pyx_n_s_startime);
  Py_VISIT(traverse_module_state->__pyx_n_s_statistics);
  Py_VISIT(traverse_module_state->__pyx_n_s_test);
  Py_VISIT(traverse_module_state->__pyx_n_s_zeros);
  Py_VISIT(traverse_module_state->__pyx_float__4);
  Py_VISIT(traverse_module_state->__pyx_float__34);
  Py_VISIT(traverse_module_state->__pyx_float_1_0);
  Py_VISIT(traverse_module_state->__pyx_float_0_97);
  Py_VISIT(traverse_module_state->__pyx_float_0_98);
  Py_VISIT(traverse_module_state->__pyx_float_0_99);
  Py_VISIT(traverse_module_state->__pyx_float_1_00);
  Py_VISIT(traverse_module_state->__pyx_float_1_01);
  Py_VISIT(traverse_module_state->__pyx_float_1_02);
  Py_VISIT(traverse_module_state->__pyx_float_1_97);
  Py_VISIT(traverse_module_state->__pyx_float_1_98);
  Py_VISIT(traverse_module_state->__pyx_float_1_99);
  Py_VISIT(traverse_module_state->__pyx_float_2_00);
  Py_VISIT(traverse_module_state->__pyx_float_2_01);
  Py_VISIT(traverse_module_state->__pyx_float_2_02);
  Py_VISIT(traverse_module_state->__pyx_float_2_03);
  Py_VISIT(traverse_module_state->__pyx_float_2_07);
  Py_VISIT(traverse_module_state->__pyx_float_2_09);
  Py_VISIT(traverse_module_state->__pyx_float_2_96);
  Py_VISIT(traverse_module_state->__pyx_float_2_97);
  Py_VISIT(traverse_module_state->__pyx_float_2_98);
  Py_VISIT(traverse_module_state->__pyx_float_2_99);
  Py_VISIT(traverse_module_state->__pyx_float_3_00);
  Py_VISIT(traverse_module_state->__pyx_float_3_01);
  Py_VISIT(traverse_module_state->__pyx_float_3_02);
  Py_VISIT(traverse_module_state->__pyx_float_3_03);
  Py_VISIT(traverse_module_state->__pyx_float_3_04);
  Py_VISIT(traverse_module_state->__pyx_float_3_09);
  Py_VISIT(traverse_module_state->__pyx_float_3_11);
  Py_VISIT(traverse_module_state->__pyx_float_3_12);
  Py_VISIT(traverse_module_state->__pyx_float_3_15);
  Py_VISIT(traverse_module_state->__pyx_float_3_19);
  Py_VISIT(traverse_module_state->__pyx_float_3_95);
  Py_VISIT(traverse_module_state->__pyx_float_3_97);
  Py_VISIT(traverse_module_state->__pyx_float_3_98);
  Py_VISIT(traverse_module_state->__pyx_float_3_99);
  Py_VISIT(traverse_module_state->__pyx_float_4_00);
  Py_VISIT(traverse_module_state->__pyx_float_4_01);
  Py_VISIT(traverse_module_state->__pyx_float_4_02);
  Py_VISIT(traverse_module_state->__pyx_float_4_03);
  Py_VISIT(traverse_module_state->__pyx_float_4_04);
  Py_VISIT(traverse_module_state->__pyx_float_4_05);
  Py_VISIT(traverse_module_state->__pyx_float_4_06);
  Py_VISIT(traverse_module_state->__pyx_float_4_10);
  Py_VISIT(traverse_module_state->__pyx_float_4_12);
  Py_VISIT(traverse_module_state->__pyx_float_4_13);
  Py_VISIT(traverse_module_state->__pyx_float_4_17);
  Py_VISIT(traverse_module_state->__pyx_float_4_19);
  Py_VISIT(traverse_module_state->__pyx_float_4_26);
  Py_VISIT(traverse_module_state->__pyx_float_4_94);
  Py_VISIT(traverse_module_state->__pyx_float_4_95);
  Py_VISIT(traverse_module_state->__pyx_float_4_96);
  Py_VISIT(traverse_module_state->__pyx_float_4_97);
  Py_VISIT(traverse_module_state->__pyx_float_4_98);
  Py_VISIT(traverse_module_state->__pyx_float_4_99);
  Py_VISIT(traverse_module_state->__pyx_float_5_00);
  Py_VISIT(traverse_module_state->__pyx_float_5_01);
  Py_VISIT(traverse_module_state->__pyx_float_5_02);
  Py_VISIT(traverse_module_state->__pyx_float_5_03);
  Py_VISIT(traverse_module_state->__pyx_float_5_04);
  Py_VISIT(traverse_module_state->__pyx_float_5_05);
  Py_VISIT(traverse_module_state->__pyx_float_5_07);
  Py_VISIT(traverse_module_state->__pyx_float_5_09);
  Py_VISIT(traverse_module_state->__pyx_float_5_10);
  Py_VISIT(traverse_module_state->__pyx_float_5_11);
  Py_VISIT(traverse_module_state->__pyx_float_5_12);
  Py_VISIT(traverse_module_state->__pyx_float_5_14);
  Py_VISIT(traverse_module_state->__pyx_float_5_15);
  Py_VISIT(traverse_module_state->__pyx_float_5_19);
  Py_VISIT(traverse_module_state->__pyx_float_5_20);
  Py_VISIT(traverse_module_state->__pyx_float_5_22);
  Py_VISIT(traverse_module_state->__pyx_float_5_95);
  Py_VISIT(traverse_module_state->__pyx_float_5_96);
  Py_VISIT(traverse_module_state->__pyx_float_5_97);
  Py_VISIT(traverse_module_state->__pyx_float_5_98);
  Py_VISIT(traverse_module_state->__pyx_float_5_99);
  Py_VISIT(traverse_module_state->__pyx_float_6_00);
  Py_VISIT(traverse_module_state->__pyx_float_6_01);
  Py_VISIT(traverse_module_state->__pyx_float_6_02);
  Py_VISIT(traverse_module_state->__pyx_float_6_03);
  Py_VISIT(traverse_module_state->__pyx_float_6_04);
  Py_VISIT(traverse_module_state->__pyx_float_6_06);
  Py_VISIT(traverse_module_state->__pyx_float_6_07);
  Py_VISIT(traverse_module_state->__pyx_float_6_08);
  Py_VISIT(traverse_module_state->__pyx_float_6_09);
  Py_VISIT(traverse_module_state->__pyx_float_6_12);
  Py_VISIT(traverse_module_state->__pyx_float_6_13);
  Py_VISIT(traverse_module_state->__pyx_float_6_14);
  Py_VISIT(traverse_module_state->__pyx_float_6_16);
  Py_VISIT(traverse_module_state->__pyx_float_6_17);
  Py_VISIT(traverse_module_state->__pyx_float_6_20);
  Py_VISIT(traverse_module_state->__pyx_float_6_22);
  Py_VISIT(traverse_module_state->__pyx_float_6_26);
  Py_VISIT(traverse_module_state->__pyx_float_6_27);
  Py_VISIT(traverse_module_state->__pyx_float_6_32);
  Py_VISIT(traverse_module_state->__pyx_float_6_35);
  Py_VISIT(traverse_module_state->__pyx_float_6_94);
  Py_VISIT(traverse_module_state->__pyx_float_6_95);
  Py_VISIT(traverse_module_state->__pyx_float_6_96);
  Py_VISIT(traverse_module_state->__pyx_float_6_97);
  Py_VISIT(traverse_module_state->__pyx_float_6_98);
  Py_VISIT(traverse_module_state->__pyx_float_6_99);
  Py_VISIT(traverse_module_state->__pyx_float_7_00);
  Py_VISIT(traverse_module_state->__pyx_float_7_01);
  Py_VISIT(traverse_module_state->__pyx_float_7_03);
  Py_VISIT(traverse_module_state->__pyx_float_7_04);
  Py_VISIT(traverse_module_state->__pyx_float_7_05);
  Py_VISIT(traverse_module_state->__pyx_float_7_06);
  Py_VISIT(traverse_module_state->__pyx_float_7_07);
  Py_VISIT(traverse_module_state->__pyx_float_7_09);
  Py_VISIT(traverse_module_state->__pyx_float_7_11);
  Py_VISIT(traverse_module_state->__pyx_float_7_12);
  Py_VISIT(traverse_module_state->__pyx_float_7_13);
  Py_VISIT(traverse_module_state->__pyx_float_7_15);
  Py_VISIT(traverse_module_state->__pyx_float_7_19);
  Py_VISIT(traverse_module_state->__pyx_float_7_22);
  Py_VISIT(traverse_module_state->__pyx_float_7_23);
  Py_VISIT(traverse_module_state->__pyx_float_7_25);
  Py_VISIT(traverse_module_state->__pyx_float_7_26);
  Py_VISIT(traverse_module_state->__pyx_float_7_27);
  Py_VISIT(traverse_module_state->__pyx_float_7_32);
  Py_VISIT(traverse_module_state->__pyx_float_7_33);
  Py_VISIT(traverse_module_state->__pyx_float_7_35);
  Py_VISIT(traverse_module_state->__pyx_float_7_41);
  Py_VISIT(traverse_module_state->__pyx_float_7_59);
  Py_VISIT(traverse_module_state->__pyx_float_7_93);
  Py_VISIT(traverse_module_state->__pyx_float_7_94);
  Py_VISIT(traverse_module_state->__pyx_float_7_96);
  Py_VISIT(traverse_module_state->__pyx_float_7_97);
  Py_VISIT(traverse_module_state->__pyx_float_7_98);
  Py_VISIT(traverse_module_state->__pyx_float_7_99);
  Py_VISIT(traverse_module_state->__pyx_float_8_00);
  Py_VISIT(traverse_module_state->__pyx_float_8_01);
  Py_VISIT(traverse_module_state->__pyx_float_8_02);
  Py_VISIT(traverse_module_state->__pyx_float_8_04);
  Py_VISIT(traverse_module_state->__pyx_float_8_05);
  Py_VISIT(traverse_module_state->__pyx_float_8_06);
  Py_VISIT(traverse_module_state->__pyx_float_8_08);
  Py_VISIT(traverse_module_state->__pyx_float_8_09);
  Py_VISIT(traverse_module_state->__pyx_float_8_11);
  Py_VISIT(traverse_module_state->__pyx_float_8_12);
  Py_VISIT(traverse_module_state->__pyx_float_8_14);
  Py_VISIT(traverse_module_state->__pyx_float_8_17);
  Py_VISIT(traverse_module_state->__pyx_float_8_19);
  Py_VISIT(traverse_module_state->__pyx_float_8_20);
  Py_VISIT(traverse_module_state->__pyx_float_8_22);
  Py_VISIT(traverse_module_state->__pyx_float_8_29);
  Py_VISIT(traverse_module_state->__pyx_float_8_30);
  Py_VISIT(traverse_module_state->__pyx_float_8_32);
  Py_VISIT(traverse_module_state->__pyx_float_8_33);
  Py_VISIT(traverse_module_state->__pyx_float_8_36);
  Py_VISIT(traverse_module_state->__pyx_float_8_39);
  Py_VISIT(traverse_module_state->__pyx_float_8_40);
  Py_VISIT(traverse_module_state->__pyx_float_8_46);
  Py_VISIT(traverse_module_state->__pyx_float_8_50);
  Py_VISIT(traverse_module_state->__pyx_float_8_63);
  Py_VISIT(traverse_module_state->__pyx_float_8_94);
  Py_VISIT(traverse_module_state->__pyx_float_8_95);
  Py_VISIT(traverse_module_state->__pyx_float_8_96);
  Py_VISIT(traverse_module_state->__pyx_float_8_97);
  Py_VISIT(traverse_module_state->__pyx_float_8_98);
  Py_VISIT(traverse_module_state->__pyx_float_8_99);
  Py_VISIT(traverse_module_state->__pyx_float_9_00);
  Py_VISIT(traverse_module_state->__pyx_float_9_01);
  Py_VISIT(traverse_module_state->__pyx_float_9_04);
  Py_VISIT(traverse_module_state->__pyx_float_9_07);
  Py_VISIT(traverse_module_state->__pyx_float_9_08);
  Py_VISIT(traverse_module_state->__pyx_float_9_10);
  Py_VISIT(traverse_module_state->__pyx_float_9_11);
  Py_VISIT(traverse_module_state->__pyx_float_9_13);
  Py_VISIT(traverse_module_state->__pyx_float_9_15);
  Py_VISIT(traverse_module_state->__pyx_float_9_16);
  Py_VISIT(traverse_module_state->__pyx_float_9_19);
  Py_VISIT(traverse_module_state->__pyx_float_9_21);
  Py_VISIT(traverse_module_state->__pyx_float_9_23);
  Py_VISIT(traverse_module_state->__pyx_float_9_25);
  Py_VISIT(traverse_module_state->__pyx_float_9_27);
  Py_VISIT(traverse_module_state->__pyx_float_9_29);
  Py_VISIT(traverse_module_state->__pyx_float_9_32);
  Py_VISIT(traverse_module_state->__pyx_float_9_43);
  Py_VISIT(traverse_module_state->__pyx_float_9_50);
  Py_VISIT(traverse_module_state->__pyx_float_9_51);
  Py_VISIT(traverse_module_state->__pyx_float_9_53);
  Py_VISIT(traverse_module_state->__pyx_float_9_58);
  Py_VISIT(traverse_module_state->__pyx_float_9_66);
  Py_VISIT(traverse_module_state->__pyx_float_9_92);
  Py_VISIT(traverse_module_state->__pyx_float_9_95);
  Py_VISIT(traverse_module_state->__pyx_float_9_96);
  Py_VISIT(traverse_module_state->__pyx_float_9_98);
  Py_VISIT(traverse_module_state->__pyx_float_9_99);
  Py_VISIT(traverse_module_state->__pyx_float__0001);
  Py_VISIT(traverse_module_state->__pyx_float__0002);
  Py_VISIT(traverse_module_state->__pyx_float__0003);
  Py_VISIT(traverse_module_state->__pyx_float__0004);
  Py_VISIT(traverse_module_state->__pyx_float__0005);
  Py_VISIT(traverse_module_state->__pyx_float__0006);
  Py_VISIT(traverse_module_state->__pyx_float__0007);
  Py_VISIT(traverse_module_state->__pyx_float__0008);
  Py_VISIT(traverse_module_state->__pyx_float__0009);
  Py_VISIT(traverse_module_state->__pyx_float__0010);
  Py_VISIT(traverse_module_state->__pyx_float__0011);
  Py_VISIT(traverse_module_state->__pyx_float__0012);
  Py_VISIT(traverse_module_state->__pyx_float__0013);
  Py_VISIT(traverse_module_state->__pyx_float__0014);
  Py_VISIT(traverse_module_state->__pyx_float__0015);
  Py_VISIT(traverse_module_state->__pyx_float__0016);
  Py_VISIT(traverse_module_state->__pyx_float__0017);
  Py_VISIT(traverse_module_state->__pyx_float__0018);
  Py_VISIT(traverse_module_state->__pyx_float__0019);
  Py_VISIT(traverse_module_state->__pyx_float__0020);
  Py_VISIT(traverse_module_state->__pyx_float__0021);
  Py_VISIT(traverse_module_state->__pyx_float__0022);
  Py_VISIT(traverse_module_state->__pyx_float__0023);
  Py_VISIT(traverse_module_state->__pyx_float__0024);
  Py_VISIT(traverse_module_state->__pyx_float__0025);
  Py_VISIT(traverse_module_state->__pyx_float__0026);
  Py_VISIT(traverse_module_state->__pyx_float__0027);
  Py_VISIT(traverse_module_state->__pyx_float__0028);
  Py_VISIT(traverse_module_state->__pyx_float__0029);
  Py_VISIT(traverse_module_state->__pyx_float__0030);
  Py_VISIT(traverse_module_state->__pyx_float__0031);
  Py_VISIT(traverse_module_state->__pyx_float__0032);
  Py_VISIT(traverse_module_state->__pyx_float__0033);
  Py_VISIT(traverse_module_state->__pyx_float__0034);
  Py_VISIT(traverse_module_state->__pyx_float__0035);
  Py_VISIT(traverse_module_state->__pyx_float__0036);
  Py_VISIT(traverse_module_state->__pyx_float__0037);
  Py_VISIT(traverse_module_state->__pyx_float__0038);
  Py_VISIT(traverse_module_state->__pyx_float__0039);
  Py_VISIT(traverse_module_state->__pyx_float__0040);
  Py_VISIT(traverse_module_state->__pyx_float__0041);
  Py_VISIT(traverse_module_state->__pyx_float__0042);
  Py_VISIT(traverse_module_state->__pyx_float__0043);
  Py_VISIT(traverse_module_state->__pyx_float__0044);
  Py_VISIT(traverse_module_state->__pyx_float__0045);
  Py_VISIT(traverse_module_state->__pyx_float__0046);
  Py_VISIT(traverse_module_state->__pyx_float__0047);
  Py_VISIT(traverse_module_state->__pyx_float__0048);
  Py_VISIT(traverse_module_state->__pyx_float__0049);
  Py_VISIT(traverse_module_state->__pyx_float__0050);
  Py_VISIT(traverse_module_state->__pyx_float__0051);
  Py_VISIT(traverse_module_state->__pyx_float__0052);
  Py_VISIT(traverse_module_state->__pyx_float__0053);
  Py_VISIT(traverse_module_state->__pyx_float__0054);
  Py_VISIT(traverse_module_state->__pyx_float__0055);
  Py_VISIT(traverse_module_state->__pyx_float__0056);
  Py_VISIT(traverse_module_state->__pyx_float__0057);
  Py_VISIT(traverse_module_state->__pyx_float__0058);
  Py_VISIT(traverse_module_state->__pyx_float__0059);
  Py_VISIT(traverse_module_state->__pyx_float__0060);
  Py_VISIT(traverse_module_state->__pyx_float__0061);
  Py_VISIT(traverse_module_state->__pyx_float__0062);
  Py_VISIT(traverse_module_state->__pyx_float__0063);
  Py_VISIT(traverse_module_state->__pyx_float__0064);
  Py_VISIT(traverse_module_state->__pyx_float__0065);
  Py_VISIT(traverse_module_state->__pyx_float__0066);
  Py_VISIT(traverse_module_state->__pyx_float__0067);
  Py_VISIT(traverse_module_state->__pyx_float__0068);
  Py_VISIT(traverse_module_state->__pyx_float__0069);
  Py_VISIT(traverse_module_state->__pyx_float__0070);
  Py_VISIT(traverse_module_state->__pyx_float__0071);
  Py_VISIT(traverse_module_state->__pyx_float__0072);
  Py_VISIT(traverse_module_state->__pyx_float__0073);
  Py_VISIT(traverse_module_state->__pyx_float__0074);
  Py_VISIT(traverse_module_state->__pyx_float__0075);
  Py_VISIT(traverse_module_state->__pyx_float__0076);
  Py_VISIT(traverse_module_state->__pyx_float__0077);
  Py_VISIT(traverse_module_state->__pyx_float__0078);
  Py_VISIT(traverse_module_state->__pyx_float__0080);
  Py_VISIT(traverse_module_state->__pyx_float__0081);
  Py_VISIT(traverse_module_state->__pyx_float__0082);
  Py_VISIT(traverse_module_state->__pyx_float__0083);
  Py_VISIT(traverse_module_state->__pyx_float__0084);
  Py_VISIT(traverse_module_state->__pyx_float__0085);
  Py_VISIT(traverse_module_state->__pyx_float__0086);
  Py_VISIT(traverse_module_state->__pyx_float__0087);
  Py_VISIT(traverse_module_state->__pyx_float__0088);
  Py_VISIT(traverse_module_state->__pyx_float__0089);
  Py_VISIT(traverse_module_state->__pyx_float__0090);
  Py_VISIT(traverse_module_state->__pyx_float__0091);
  Py_VISIT(traverse_module_state->__pyx_float__0092);
  Py_VISIT(traverse_module_state->__pyx_float__0093);
  Py_VISIT(traverse_module_state->__pyx_float__0094);
  Py_VISIT(traverse_module_state->__pyx_float__0095);
  Py_VISIT(traverse_module_state->__pyx_float__0097);
  Py_VISIT(traverse_module_state->__pyx_float__0098);
  Py_VISIT(traverse_module_state->__pyx_float__0099);
  Py_VISIT(traverse_module_state->__pyx_float__0100);
  Py_VISIT(traverse_module_state->__pyx_float__0101);
  Py_VISIT(traverse_module_state->__pyx_float__0102);
  Py_VISIT(traverse_module_state->__pyx_float__0103);
  Py_VISIT(traverse_module_state->__pyx_float__0104);
  Py_VISIT(traverse_module_state->__pyx_float__0105);
  Py_VISIT(traverse_module_state->__pyx_float__0106);
  Py_VISIT(traverse_module_state->__pyx_float__0107);
  Py_VISIT(traverse_module_state->__pyx_float__0108);
  Py_VISIT(traverse_module_state->__pyx_float__0109);
  Py_VISIT(traverse_module_state->__pyx_float__0111);
  Py_VISIT(traverse_module_state->__pyx_float__0112);
  Py_VISIT(traverse_module_state->__pyx_float__0113);
  Py_VISIT(traverse_module_state->__pyx_float__0114);
  Py_VISIT(traverse_module_state->__pyx_float__0115);
  Py_VISIT(traverse_module_state->__pyx_float__0116);
  Py_VISIT(traverse_module_state->__pyx_float__0117);
  Py_VISIT(traverse_module_state->__pyx_float__0118);
  Py_VISIT(traverse_module_state->__pyx_float__0119);
  Py_VISIT(traverse_module_state->__pyx_float__0120);
  Py_VISIT(traverse_module_state->__pyx_float__0121);
  Py_VISIT(traverse_module_state->__pyx_float__0122);
  Py_VISIT(traverse_module_state->__pyx_float__0123);
  Py_VISIT(traverse_module_state->__pyx_float__0124);
  Py_VISIT(traverse_module_state->__pyx_float__0125);
  Py_VISIT(traverse_module_state->__pyx_float__0126);
  Py_VISIT(traverse_module_state->__pyx_float__0127);
  Py_VISIT(traverse_module_state->__pyx_float__0128);
  Py_VISIT(traverse_module_state->__pyx_float__0129);
  Py_VISIT(traverse_module_state->__pyx_float__0132);
  Py_VISIT(traverse_module_state->__pyx_float__0133);
  Py_VISIT(traverse_module_state->__pyx_float__0134);
  Py_VISIT(traverse_module_state->__pyx_float__0135);
  Py_VISIT(traverse_module_state->__pyx_float__0136);
  Py_VISIT(traverse_module_state->__pyx_float__0137);
  Py_VISIT(traverse_module_state->__pyx_float__0138);
  Py_VISIT(traverse_module_state->__pyx_float__0139);
  Py_VISIT(traverse_module_state->__pyx_float__0140);
  Py_VISIT(traverse_module_state->__pyx_float__0141);
  Py_VISIT(traverse_module_state->__pyx_float__0142);
  Py_VISIT(traverse_module_state->__pyx_float__0144);
  Py_VISIT(traverse_module_state->__pyx_float__0145);
  Py_VISIT(traverse_module_state->__pyx_float__0147);
  Py_VISIT(traverse_module_state->__pyx_float__0148);
  Py_VISIT(traverse_module_state->__pyx_float__0151);
  Py_VISIT(traverse_module_state->__pyx_float__0152);
  Py_VISIT(traverse_module_state->__pyx_float__0153);
  Py_VISIT(traverse_module_state->__pyx_float__0154);
  Py_VISIT(traverse_module_state->__pyx_float__0155);
  Py_VISIT(traverse_module_state->__pyx_float__0156);
  Py_VISIT(traverse_module_state->__pyx_float__0158);
  Py_VISIT(traverse_module_state->__pyx_float__0159);
  Py_VISIT(traverse_module_state->__pyx_float__0160);
  Py_VISIT(traverse_module_state->__pyx_float__0161);
  Py_VISIT(traverse_module_state->__pyx_float__0162);
  Py_VISIT(traverse_module_state->__pyx_float__0163);
  Py_VISIT(traverse_module_state->__pyx_float__0165);
  Py_VISIT(traverse_module_state->__pyx_float__0166);
  Py_VISIT(traverse_module_state->__pyx_float__0167);
  Py_VISIT(traverse_module_state->__pyx_float__0170);
  Py_VISIT(traverse_module_state->__pyx_float__0171);
  Py_VISIT(traverse_module_state->__pyx_float__0172);
  Py_VISIT(traverse_module_state->__pyx_float__0173);
  Py_VISIT(traverse_module_state->__pyx_float__0174);
  Py_VISIT(traverse_module_state->__pyx_float__0175);
  Py_VISIT(traverse_module_state->__pyx_float__0177);
  Py_VISIT(traverse_module_state->__pyx_float__0178);
  Py_VISIT(traverse_module_state->__pyx_float__0179);
  Py_VISIT(traverse_module_state->__pyx_float__0181);
  Py_VISIT(traverse_module_state->__pyx_float__0182);
  Py_VISIT(traverse_module_state->__pyx_float__0183);
  Py_VISIT(traverse_module_state->__pyx_float__0184);
  Py_VISIT(traverse_module_state->__pyx_float__0185);
  Py_VISIT(traverse_module_state->__pyx_float__0186);
  Py_VISIT(traverse_module_state->__pyx_float__0189);
  Py_VISIT(traverse_module_state->__pyx_float__0190);
  Py_VISIT(traverse_module_state->__pyx_float__0191);
  Py_VISIT(traverse_module_state->__pyx_float__0192);
  Py_VISIT(traverse_module_state->__pyx_float__0194);
  Py_VISIT(traverse_module_state->__pyx_float__0196);
  Py_VISIT(traverse_module_state->__pyx_float__0197);
  Py_VISIT(traverse_module_state->__pyx_float__0201);
  Py_VISIT(traverse_module_state->__pyx_float__0202);
  Py_VISIT(traverse_module_state->__pyx_float__0203);
  Py_VISIT(traverse_module_state->__pyx_float__0206);
  Py_VISIT(traverse_module_state->__pyx_float__0207);
  Py_VISIT(traverse_module_state->__pyx_float__0209);
  Py_VISIT(traverse_module_state->__pyx_float__0210);
  Py_VISIT(traverse_module_state->__pyx_float__0211);
  Py_VISIT(traverse_module_state->__pyx_float__0212);
  Py_VISIT(traverse_module_state->__pyx_float__0213);
  Py_VISIT(traverse_module_state->__pyx_float__0215);
  Py_VISIT(traverse_module_state->__pyx_float__0217);
  Py_VISIT(traverse_module_state->__pyx_float__0218);
  Py_VISIT(traverse_module_state->__pyx_float__0221);
  Py_VISIT(traverse_module_state->__pyx_float__0222);
  Py_VISIT(traverse_module_state->__pyx_float__0225);
  Py_VISIT(traverse_module_state->__pyx_float__0227);
  Py_VISIT(traverse_module_state->__pyx_float__0228);
  Py_VISIT(traverse_module_state->__pyx_float__0229);
  Py_VISIT(traverse_module_state->__pyx_float__0230);
  Py_VISIT(traverse_module_state->__pyx_float__0232);
  Py_VISIT(traverse_module_state->__pyx_float__0233);
  Py_VISIT(traverse_module_state->__pyx_float__0234);
  Py_VISIT(traverse_module_state->__pyx_float__0235);
  Py_VISIT(traverse_module_state->__pyx_float__0236);
  Py_VISIT(traverse_module_state->__pyx_float__0237);
  Py_VISIT(traverse_module_state->__pyx_float__0238);
  Py_VISIT(traverse_module_state->__pyx_float__0240);
  Py_VISIT(traverse_module_state->__pyx_float__0241);
  Py_VISIT(traverse_module_state->__pyx_float__0242);
  Py_VISIT(traverse_module_state->__pyx_float__0245);
  Py_VISIT(traverse_module_state->__pyx_float__0246);
  Py_VISIT(traverse_module_state->__pyx_float__0247);
  Py_VISIT(traverse_module_state->__pyx_float__0248);
  Py_VISIT(traverse_module_state->__pyx_float__0249);
  Py_VISIT(traverse_module_state->__pyx_float__0250);
  Py_VISIT(traverse_module_state->__pyx_float__0252);
  Py_VISIT(traverse_module_state->__pyx_float__0256);
  Py_VISIT(traverse_module_state->__pyx_float__0257);
  Py_VISIT(traverse_module_state->__pyx_float__0258);
  Py_VISIT(traverse_module_state->__pyx_float__0259);
  Py_VISIT(traverse_module_state->__pyx_float__0260);
  Py_VISIT(traverse_module_state->__pyx_float__0264);
  Py_VISIT(traverse_module_state->__pyx_float__0265);
  Py_VISIT(traverse_module_state->__pyx_float__0266);
  Py_VISIT(traverse_module_state->__pyx_float__0268);
  Py_VISIT(traverse_module_state->__pyx_float__0272);
  Py_VISIT(traverse_module_state->__pyx_float__0274);
  Py_VISIT(traverse_module_state->__pyx_float__0276);
  Py_VISIT(traverse_module_state->__pyx_float__0278);
  Py_VISIT(traverse_module_state->__pyx_float__0279);
  Py_VISIT(traverse_module_state->__pyx_float__0280);
  Py_VISIT(traverse_module_state->__pyx_float__0284);
  Py_VISIT(traverse_module_state->__pyx_float__0285);
  Py_VISIT(traverse_module_state->__pyx_float__0287);
  Py_VISIT(traverse_module_state->__pyx_float__0289);
  Py_VISIT(traverse_module_state->__pyx_float__0290);
  Py_VISIT(traverse_module_state->__pyx_float__0291);
  Py_VISIT(traverse_module_state->__pyx_float__0292);
  Py_VISIT(traverse_module_state->__pyx_float__0293);
  Py_VISIT(traverse_module_state->__pyx_float__0297);
  Py_VISIT(traverse_module_state->__pyx_float__0298);
  Py_VISIT(traverse_module_state->__pyx_float__0299);
  Py_VISIT(traverse_module_state->__pyx_float__0300);
  Py_VISIT(traverse_module_state->__pyx_float__0301);
  Py_VISIT(traverse_module_state->__pyx_float__0305);
  Py_VISIT(traverse_module_state->__pyx_float__0306);
  Py_VISIT(traverse_module_state->__pyx_float__0307);
  Py_VISIT(traverse_module_state->__pyx_float__0309);
  Py_VISIT(traverse_module_state->__pyx_float__0310);
  Py_VISIT(traverse_module_state->__pyx_float__0311);
  Py_VISIT(traverse_module_state->__pyx_float__0313);
  Py_VISIT(traverse_module_state->__pyx_float__0314);
  Py_VISIT(traverse_module_state->__pyx_float__0315);
  Py_VISIT(traverse_module_state->__pyx_float__0319);
  Py_VISIT(traverse_module_state->__pyx_float__0322);
  Py_VISIT(traverse_module_state->__pyx_float__0324);
  Py_VISIT(traverse_module_state->__pyx_float__0326);
  Py_VISIT(traverse_module_state->__pyx_float__0328);
  Py_VISIT(traverse_module_state->__pyx_float__0329);
  Py_VISIT(traverse_module_state->__pyx_float__0332);
  Py_VISIT(traverse_module_state->__pyx_float__0334);
  Py_VISIT(traverse_module_state->__pyx_float__0336);
  Py_VISIT(traverse_module_state->__pyx_float__0340);
  Py_VISIT(traverse_module_state->__pyx_float__0342);
  Py_VISIT(traverse_module_state->__pyx_float__0345);
  Py_VISIT(traverse_module_state->__pyx_float__0349);
  Py_VISIT(traverse_module_state->__pyx_float__0353);
  Py_VISIT(traverse_module_state->__pyx_float__0355);
  Py_VISIT(traverse_module_state->__pyx_float__0363);
  Py_VISIT(traverse_module_state->__pyx_float__0364);
  Py_VISIT(traverse_module_state->__pyx_float__0365);
  Py_VISIT(traverse_module_state->__pyx_float__0367);
  Py_VISIT(traverse_module_state->__pyx_float__0368);
  Py_VISIT(traverse_module_state->__pyx_float__0369);
  Py_VISIT(traverse_module_state->__pyx_float__0370);
  Py_VISIT(traverse_module_state->__pyx_float__0373);
  Py_VISIT(traverse_module_state->__pyx_float__0381);
  Py_VISIT(traverse_module_state->__pyx_float__0387);
  Py_VISIT(traverse_module_state->__pyx_float__0389);
  Py_VISIT(traverse_module_state->__pyx_float__0390);
  Py_VISIT(traverse_module_state->__pyx_float__0392);
  Py_VISIT(traverse_module_state->__pyx_float__0396);
  Py_VISIT(traverse_module_state->__pyx_float__0402);
  Py_VISIT(traverse_module_state->__pyx_float__0403);
  Py_VISIT(traverse_module_state->__pyx_float__0404);
  Py_VISIT(traverse_module_state->__pyx_float__0410);
  Py_VISIT(traverse_module_state->__pyx_float__0411);
  Py_VISIT(traverse_module_state->__pyx_float__0415);
  Py_VISIT(traverse_module_state->__pyx_float__0418);
  Py_VISIT(traverse_module_state->__pyx_float__0420);
  Py_VISIT(traverse_module_state->__pyx_float__0421);
  Py_VISIT(traverse_module_state->__pyx_float__0423);
  Py_VISIT(traverse_module_state->__pyx_float__0425);
  Py_VISIT(traverse_module_state->__pyx_float__0437);
  Py_VISIT(traverse_module_state->__pyx_float__0445);
  Py_VISIT(traverse_module_state->__pyx_float__0448);
  Py_VISIT(traverse_module_state->__pyx_float__0449);
  Py_VISIT(traverse_module_state->__pyx_float__0465);
  Py_VISIT(traverse_module_state->__pyx_float__0472);
  Py_VISIT(traverse_module_state->__pyx_float__0473);
  Py_VISIT(traverse_module_state->__pyx_float__0475);
  Py_VISIT(traverse_module_state->__pyx_float__0482);
  Py_VISIT(traverse_module_state->__pyx_float__0492);
  Py_VISIT(traverse_module_state->__pyx_float__0493);
  Py_VISIT(traverse_module_state->__pyx_float__0497);
  Py_VISIT(traverse_module_state->__pyx_float__0498);
  Py_VISIT(traverse_module_state->__pyx_float__0500);
  Py_VISIT(traverse_module_state->__pyx_float__0503);
  Py_VISIT(traverse_module_state->__pyx_float__0506);
  Py_VISIT(traverse_module_state->__pyx_float__0512);
  Py_VISIT(traverse_module_state->__pyx_float__0514);
  Py_VISIT(traverse_module_state->__pyx_float__0516);
  Py_VISIT(traverse_module_state->__pyx_float__0517);
  Py_VISIT(traverse_module_state->__pyx_float__0522);
  Py_VISIT(traverse_module_state->__pyx_float__0523);
  Py_VISIT(traverse_module_state->__pyx_float__0525);
  Py_VISIT(traverse_module_state->__pyx_float__0578);
  Py_VISIT(traverse_module_state->__pyx_float__0593);
  Py_VISIT(traverse_module_state->__pyx_float__0596);
  Py_VISIT(traverse_module_state->__pyx_float__0613);
  Py_VISIT(traverse_module_state->__pyx_float__0621);
  Py_VISIT(traverse_module_state->__pyx_float__0629);
  Py_VISIT(traverse_module_state->__pyx_float__0650);
  Py_VISIT(traverse_module_state->__pyx_float__0654);
  Py_VISIT(traverse_module_state->__pyx_float__0669);
  Py_VISIT(traverse_module_state->__pyx_float__0674);
  Py_VISIT(traverse_module_state->__pyx_float__0676);
  Py_VISIT(traverse_module_state->__pyx_float__0678);
  Py_VISIT(traverse_module_state->__pyx_float__0683);
  Py_VISIT(traverse_module_state->__pyx_float__0692);
  Py_VISIT(traverse_module_state->__pyx_float__0698);
  Py_VISIT(traverse_module_state->__pyx_float__0708);
  Py_VISIT(traverse_module_state->__pyx_float__0714);
  Py_VISIT(traverse_module_state->__pyx_float__0719);
  Py_VISIT(traverse_module_state->__pyx_float__0726);
  Py_VISIT(traverse_module_state->__pyx_float__0727);
  Py_VISIT(traverse_module_state->__pyx_float__0752);
  Py_VISIT(traverse_module_state->__pyx_float__0763);
  Py_VISIT(traverse_module_state->__pyx_float__0765);
  Py_VISIT(traverse_module_state->__pyx_float__0773);
  Py_VISIT(traverse_module_state->__pyx_float__0813);
  Py_VISIT(traverse_module_state->__pyx_float__0820);
  Py_VISIT(traverse_module_state->__pyx_float__0835);
  Py_VISIT(traverse_module_state->__pyx_float__0841);
  Py_VISIT(traverse_module_state->__pyx_float__0846);
  Py_VISIT(traverse_module_state->__pyx_float__0877);
  Py_VISIT(traverse_module_state->__pyx_float__0909);
  Py_VISIT(traverse_module_state->__pyx_float__0919);
  Py_VISIT(traverse_module_state->__pyx_float__0920);
  Py_VISIT(traverse_module_state->__pyx_float__0921);
  Py_VISIT(traverse_module_state->__pyx_float__0933);
  Py_VISIT(traverse_module_state->__pyx_float__0938);
  Py_VISIT(traverse_module_state->__pyx_float__0951);
  Py_VISIT(traverse_module_state->__pyx_float__0963);
  Py_VISIT(traverse_module_state->__pyx_float__0967);
  Py_VISIT(traverse_module_state->__pyx_float__0976);
  Py_VISIT(traverse_module_state->__pyx_float__0982);
  Py_VISIT(traverse_module_state->__pyx_float__0985);
  Py_VISIT(traverse_module_state->__pyx_float__1008);
  Py_VISIT(traverse_module_state->__pyx_float__1011);
  Py_VISIT(traverse_module_state->__pyx_float__1015);
  Py_VISIT(traverse_module_state->__pyx_float__1018);
  Py_VISIT(traverse_module_state->__pyx_float__1054);
  Py_VISIT(traverse_module_state->__pyx_float__1066);
  Py_VISIT(traverse_module_state->__pyx_float__1085);
  Py_VISIT(traverse_module_state->__pyx_float__1126);
  Py_VISIT(traverse_module_state->__pyx_float__1132);
  Py_VISIT(traverse_module_state->__pyx_float__1174);
  Py_VISIT(traverse_module_state->__pyx_float__1193);
  Py_VISIT(traverse_module_state->__pyx_float__1238);
  Py_VISIT(traverse_module_state->__pyx_float__1243);
  Py_VISIT(traverse_module_state->__pyx_float__1273);
  Py_VISIT(traverse_module_state->__pyx_float__1296);
  Py_VISIT(traverse_module_state->__pyx_float__1304);
  Py_VISIT(traverse_module_state->__pyx_float__1311);
  Py_VISIT(traverse_module_state->__pyx_float__1357);
  Py_VISIT(traverse_module_state->__pyx_float__1400);
  Py_VISIT(traverse_module_state->__pyx_float__1440);
  Py_VISIT(traverse_module_state->__pyx_float__1441);
  Py_VISIT(traverse_module_state->__pyx_float__1486);
  Py_VISIT(traverse_module_state->__pyx_float__1541);
  Py_VISIT(traverse_module_state->__pyx_float__1555);
  Py_VISIT(traverse_module_state->__pyx_float__1611);
  Py_VISIT(traverse_module_state->__pyx_float__1655);
  Py_VISIT(traverse_module_state->__pyx_float__1715);
  Py_VISIT(traverse_module_state->__pyx_float__1821);
  Py_VISIT(traverse_module_state->__pyx_float__1822);
  Py_VISIT(traverse_module_state->__pyx_float__1868);
  Py_VISIT(traverse_module_state->__pyx_float__1889);
  Py_VISIT(traverse_module_state->__pyx_float__1919);
  Py_VISIT(traverse_module_state->__pyx_float__1933);
  Py_VISIT(traverse_module_state->__pyx_float__2095);
  Py_VISIT(traverse_module_state->__pyx_float__2113);
  Py_VISIT(traverse_module_state->__pyx_float__2185);
  Py_VISIT(traverse_module_state->__pyx_float__2270);
  Py_VISIT(traverse_module_state->__pyx_float__2388);
  Py_VISIT(traverse_module_state->__pyx_float__2455);
  Py_VISIT(traverse_module_state->__pyx_float__2598);
  Py_VISIT(traverse_module_state->__pyx_float__2735);
  Py_VISIT(traverse_module_state->__pyx_float__3637);
  Py_VISIT(traverse_module_state->__pyx_float_10_00);
  Py_VISIT(traverse_module_state->__pyx_float_10_01);
  Py_VISIT(traverse_module_state->__pyx_float_10_02);
  Py_VISIT(traverse_module_state->__pyx_float_10_03);
  Py_VISIT(traverse_module_state->__pyx_float_10_04);
  Py_VISIT(traverse_module_state->__pyx_float_10_05);
  Py_VISIT(traverse_module_state->__pyx_float_10_08);
  Py_VISIT(traverse_module_state->__pyx_float_10_10);
  Py_VISIT(traverse_module_state->__pyx_float_10_11);
  Py_VISIT(traverse_module_state->__pyx_float_10_12);
  Py_VISIT(traverse_module_state->__pyx_float_10_15);
  Py_VISIT(traverse_module_state->__pyx_float_10_17);
  Py_VISIT(traverse_module_state->__pyx_float_10_18);
  Py_VISIT(traverse_module_state->__pyx_float_10_22);
  Py_VISIT(traverse_module_state->__pyx_float_10_23);
  Py_VISIT(traverse_module_state->__pyx_float_10_25);
  Py_VISIT(traverse_module_state->__pyx_float_10_30);
  Py_VISIT(traverse_module_state->__pyx_float_10_33);
  Py_VISIT(traverse_module_state->__pyx_float_10_36);
  Py_VISIT(traverse_module_state->__pyx_float_10_38);
  Py_VISIT(traverse_module_state->__pyx_float_10_40);
  Py_VISIT(traverse_module_state->__pyx_float_10_45);
  Py_VISIT(traverse_module_state->__pyx_float_10_59);
  Py_VISIT(traverse_module_state->__pyx_float_10_69);
  Py_VISIT(traverse_module_state->__pyx_float_10_93);
  Py_VISIT(traverse_module_state->__pyx_float_10_96);
  Py_VISIT(traverse_module_state->__pyx_float_10_98);
  Py_VISIT(traverse_module_state->__pyx_float_10_99);
  Py_VISIT(traverse_module_state->__pyx_float_11_00);
  Py_VISIT(traverse_module_state->__pyx_float_11_01);
  Py_VISIT(traverse_module_state->__pyx_float_11_02);
  Py_VISIT(traverse_module_state->__pyx_float_11_03);
  Py_VISIT(traverse_module_state->__pyx_float_11_04);
  Py_VISIT(traverse_module_state->__pyx_float_11_05);
  Py_VISIT(traverse_module_state->__pyx_float_11_07);
  Py_VISIT(traverse_module_state->__pyx_float_11_08);
  Py_VISIT(traverse_module_state->__pyx_float_11_11);
  Py_VISIT(traverse_module_state->__pyx_float_11_14);
  Py_VISIT(traverse_module_state->__pyx_float_11_15);
  Py_VISIT(traverse_module_state->__pyx_float_11_17);
  Py_VISIT(traverse_module_state->__pyx_float_11_19);
  Py_VISIT(traverse_module_state->__pyx_float_11_23);
  Py_VISIT(traverse_module_state->__pyx_float_11_25);
  Py_VISIT(traverse_module_state->__pyx_float_11_30);
  Py_VISIT(traverse_module_state->__pyx_float_11_31);
  Py_VISIT(traverse_module_state->__pyx_float_11_35);
  Py_VISIT(traverse_module_state->__pyx_float_11_40);
  Py_VISIT(traverse_module_state->__pyx_float_11_44);
  Py_VISIT(traverse_module_state->__pyx_float_11_48);
  Py_VISIT(traverse_module_state->__pyx_float_11_53);
  Py_VISIT(traverse_module_state->__pyx_float_11_58);
  Py_VISIT(traverse_module_state->__pyx_float_11_75);
  Py_VISIT(traverse_module_state->__pyx_float_11_78);
  Py_VISIT(traverse_module_state->__pyx_float_11_94);
  Py_VISIT(traverse_module_state->__pyx_float_11_97);
  Py_VISIT(traverse_module_state->__pyx_float_11_98);
  Py_VISIT(traverse_module_state->__pyx_float_11_99);
  Py_VISIT(traverse_module_state->__pyx_float_12_00);
  Py_VISIT(traverse_module_state->__pyx_float_12_01);
  Py_VISIT(traverse_module_state->__pyx_float_12_02);
  Py_VISIT(traverse_module_state->__pyx_float_12_03);
  Py_VISIT(traverse_module_state->__pyx_float_12_05);
  Py_VISIT(traverse_module_state->__pyx_float_12_06);
  Py_VISIT(traverse_module_state->__pyx_float_12_07);
  Py_VISIT(traverse_module_state->__pyx_float_12_08);
  Py_VISIT(traverse_module_state->__pyx_float_12_10);
  Py_VISIT(traverse_module_state->__pyx_float_12_11);
  Py_VISIT(traverse_module_state->__pyx_float_12_12);
  Py_VISIT(traverse_module_state->__pyx_float_12_15);
  Py_VISIT(traverse_module_state->__pyx_float_12_18);
  Py_VISIT(traverse_module_state->__pyx_float_12_20);
  Py_VISIT(traverse_module_state->__pyx_float_12_24);
  Py_VISIT(traverse_module_state->__pyx_float_12_26);
  Py_VISIT(traverse_module_state->__pyx_float_12_30);
  Py_VISIT(traverse_module_state->__pyx_float_12_32);
  Py_VISIT(traverse_module_state->__pyx_float_12_33);
  Py_VISIT(traverse_module_state->__pyx_float_12_40);
  Py_VISIT(traverse_module_state->__pyx_float_12_45);
  Py_VISIT(traverse_module_state->__pyx_float_12_50);
  Py_VISIT(traverse_module_state->__pyx_float_12_58);
  Py_VISIT(traverse_module_state->__pyx_float_12_63);
  Py_VISIT(traverse_module_state->__pyx_float_12_67);
  Py_VISIT(traverse_module_state->__pyx_float_12_70);
  Py_VISIT(traverse_module_state->__pyx_float_12_78);
  Py_VISIT(traverse_module_state->__pyx_float_12_95);
  Py_VISIT(traverse_module_state->__pyx_float_12_96);
  Py_VISIT(traverse_module_state->__pyx_float_12_98);
  Py_VISIT(traverse_module_state->__pyx_float_12_99);
  Py_VISIT(traverse_module_state->__pyx_float_13_00);
  Py_VISIT(traverse_module_state->__pyx_float_13_01);
  Py_VISIT(traverse_module_state->__pyx_float_13_02);
  Py_VISIT(traverse_module_state->__pyx_float_13_03);
  Py_VISIT(traverse_module_state->__pyx_float_13_04);
  Py_VISIT(traverse_module_state->__pyx_float_13_05);
  Py_VISIT(traverse_module_state->__pyx_float_13_06);
  Py_VISIT(traverse_module_state->__pyx_float_13_08);
  Py_VISIT(traverse_module_state->__pyx_float_13_11);
  Py_VISIT(traverse_module_state->__pyx_float_13_13);
  Py_VISIT(traverse_module_state->__pyx_float_13_14);
  Py_VISIT(traverse_module_state->__pyx_float_13_15);
  Py_VISIT(traverse_module_state->__pyx_float_13_16);
  Py_VISIT(traverse_module_state->__pyx_float_13_21);
  Py_VISIT(traverse_module_state->__pyx_float_13_24);
  Py_VISIT(traverse_module_state->__pyx_float_13_25);
  Py_VISIT(traverse_module_state->__pyx_float_13_26);
  Py_VISIT(traverse_module_state->__pyx_float_13_29);
  Py_VISIT(traverse_module_state->__pyx_float_13_34);
  Py_VISIT(traverse_module_state->__pyx_float_13_38);
  Py_VISIT(traverse_module_state->__pyx_float_13_40);
  Py_VISIT(traverse_module_state->__pyx_float_13_42);
  Py_VISIT(traverse_module_state->__pyx_float_13_49);
  Py_VISIT(traverse_module_state->__pyx_float_13_53);
  Py_VISIT(traverse_module_state->__pyx_float_13_58);
  Py_VISIT(traverse_module_state->__pyx_float_13_66);
  Py_VISIT(traverse_module_state->__pyx_float_13_75);
  Py_VISIT(traverse_module_state->__pyx_float_13_79);
  Py_VISIT(traverse_module_state->__pyx_float_13_89);
  Py_VISIT(traverse_module_state->__pyx_float_13_91);
  Py_VISIT(traverse_module_state->__pyx_float_13_97);
  Py_VISIT(traverse_module_state->__pyx_float_13_99);
  Py_VISIT(traverse_module_state->__pyx_float_14_00);
  Py_VISIT(traverse_module_state->__pyx_float_14_02);
  Py_VISIT(traverse_module_state->__pyx_float_14_03);
  Py_VISIT(traverse_module_state->__pyx_float_14_04);
  Py_VISIT(traverse_module_state->__pyx_float_14_05);
  Py_VISIT(traverse_module_state->__pyx_float_14_06);
  Py_VISIT(traverse_module_state->__pyx_float_14_07);
  Py_VISIT(traverse_module_state->__pyx_float_14_08);
  Py_VISIT(traverse_module_state->__pyx_float_14_10);
  Py_VISIT(traverse_module_state->__pyx_float_14_12);
  Py_VISIT(traverse_module_state->__pyx_float_14_13);
  Py_VISIT(traverse_module_state->__pyx_float_14_19);
  Py_VISIT(traverse_module_state->__pyx_float_14_25);
  Py_VISIT(traverse_module_state->__pyx_float_14_32);
  Py_VISIT(traverse_module_state->__pyx_float_14_33);
  Py_VISIT(traverse_module_state->__pyx_float_14_39);
  Py_VISIT(traverse_module_state->__pyx_float_14_45);
  Py_VISIT(traverse_module_state->__pyx_float_14_50);
  Py_VISIT(traverse_module_state->__pyx_float_14_53);
  Py_VISIT(traverse_module_state->__pyx_float_14_54);
  Py_VISIT(traverse_module_state->__pyx_float_14_62);
  Py_VISIT(traverse_module_state->__pyx_float_14_66);
  Py_VISIT(traverse_module_state->__pyx_float_14_74);
  Py_VISIT(traverse_module_state->__pyx_float_14_84);
  Py_VISIT(traverse_module_state->__pyx_float_14_93);
  Py_VISIT(traverse_module_state->__pyx_float_14_95);
  Py_VISIT(traverse_module_state->__pyx_float_15_00);
  Py_VISIT(traverse_module_state->__pyx_float_15_01);
  Py_VISIT(traverse_module_state->__pyx_float_15_03);
  Py_VISIT(traverse_module_state->__pyx_float_15_04);
  Py_VISIT(traverse_module_state->__pyx_float_15_05);
  Py_VISIT(traverse_module_state->__pyx_float_15_06);
  Py_VISIT(traverse_module_state->__pyx_float_15_07);
  Py_VISIT(traverse_module_state->__pyx_float_15_08);
  Py_VISIT(traverse_module_state->__pyx_float_15_09);
  Py_VISIT(traverse_module_state->__pyx_float_15_11);
  Py_VISIT(traverse_module_state->__pyx_float_15_12);
  Py_VISIT(traverse_module_state->__pyx_float_15_15);
  Py_VISIT(traverse_module_state->__pyx_float_15_16);
  Py_VISIT(traverse_module_state->__pyx_float_15_21);
  Py_VISIT(traverse_module_state->__pyx_float_15_22);
  Py_VISIT(traverse_module_state->__pyx_float_15_24);
  Py_VISIT(traverse_module_state->__pyx_float_15_27);
  Py_VISIT(traverse_module_state->__pyx_float_15_31);
  Py_VISIT(traverse_module_state->__pyx_float_15_41);
  Py_VISIT(traverse_module_state->__pyx_float_15_42);
  Py_VISIT(traverse_module_state->__pyx_float_15_48);
  Py_VISIT(traverse_module_state->__pyx_float_15_53);
  Py_VISIT(traverse_module_state->__pyx_float_15_62);
  Py_VISIT(traverse_module_state->__pyx_float_15_65);
  Py_VISIT(traverse_module_state->__pyx_float_15_77);
  Py_VISIT(traverse_module_state->__pyx_float_15_81);
  Py_VISIT(traverse_module_state->__pyx_float_15_88);
  Py_VISIT(traverse_module_state->__pyx_float_15_91);
  Py_VISIT(traverse_module_state->__pyx_float_15_93);
  Py_VISIT(traverse_module_state->__pyx_float_15_99);
  Py_VISIT(traverse_module_state->__pyx_float_16_00);
  Py_VISIT(traverse_module_state->__pyx_float_16_01);
  Py_VISIT(traverse_module_state->__pyx_float_16_03);
  Py_VISIT(traverse_module_state->__pyx_float_16_04);
  Py_VISIT(traverse_module_state->__pyx_float_16_05);
  Py_VISIT(traverse_module_state->__pyx_float_16_06);
  Py_VISIT(traverse_module_state->__pyx_float_16_07);
  Py_VISIT(traverse_module_state->__pyx_float_16_08);
  Py_VISIT(traverse_module_state->__pyx_float_16_09);
  Py_VISIT(traverse_module_state->__pyx_float_16_11);
  Py_VISIT(traverse_module_state->__pyx_float_16_12);
  Py_VISIT(traverse_module_state->__pyx_float_16_13);
  Py_VISIT(traverse_module_state->__pyx_float_16_14);
  Py_VISIT(traverse_module_state->__pyx_float_16_15);
  Py_VISIT(traverse_module_state->__pyx_float_16_16);
  Py_VISIT(traverse_module_state->__pyx_float_16_18);
  Py_VISIT(traverse_module_state->__pyx_float_16_20);
  Py_VISIT(traverse_module_state->__pyx_float_16_21);
  Py_VISIT(traverse_module_state->__pyx_float_16_24);
  Py_VISIT(traverse_module_state->__pyx_float_16_29);
  Py_VISIT(traverse_module_state->__pyx_float_16_30);
  Py_VISIT(traverse_module_state->__pyx_float_16_34);
  Py_VISIT(traverse_module_state->__pyx_float_16_40);
  Py_VISIT(traverse_module_state->__pyx_float_16_49);
  Py_VISIT(traverse_module_state->__pyx_float_16_50);
  Py_VISIT(traverse_module_state->__pyx_float_16_59);
  Py_VISIT(traverse_module_state->__pyx_float_16_66);
  Py_VISIT(traverse_module_state->__pyx_float_16_80);
  Py_VISIT(traverse_module_state->__pyx_float_17_01);
  Py_VISIT(traverse_module_state->__pyx_float_17_03);
  Py_VISIT(traverse_module_state->__pyx_float_17_04);
  Py_VISIT(traverse_module_state->__pyx_float_17_06);
  Py_VISIT(traverse_module_state->__pyx_float_17_07);
  Py_VISIT(traverse_module_state->__pyx_float_17_08);
  Py_VISIT(traverse_module_state->__pyx_float_17_09);
  Py_VISIT(traverse_module_state->__pyx_float_17_10);
  Py_VISIT(traverse_module_state->__pyx_float_17_11);
  Py_VISIT(traverse_module_state->__pyx_float_17_12);
  Py_VISIT(traverse_module_state->__pyx_float_17_14);
  Py_VISIT(traverse_module_state->__pyx_float_17_15);
  Py_VISIT(traverse_module_state->__pyx_float_17_17);
  Py_VISIT(traverse_module_state->__pyx_float_17_21);
  Py_VISIT(traverse_module_state->__pyx_float_17_22);
  Py_VISIT(traverse_module_state->__pyx_float_17_24);
  Py_VISIT(traverse_module_state->__pyx_float_17_25);
  Py_VISIT(traverse_module_state->__pyx_float_17_30);
  Py_VISIT(traverse_module_state->__pyx_float_17_36);
  Py_VISIT(traverse_module_state->__pyx_float_17_38);
  Py_VISIT(traverse_module_state->__pyx_float_17_42);
  Py_VISIT(traverse_module_state->__pyx_float_17_48);
  Py_VISIT(traverse_module_state->__pyx_float_17_59);
  Py_VISIT(traverse_module_state->__pyx_float_17_61);
  Py_VISIT(traverse_module_state->__pyx_float_17_62);
  Py_VISIT(traverse_module_state->__pyx_float_17_70);
  Py_VISIT(traverse_module_state->__pyx_float_17_79);
  Py_VISIT(traverse_module_state->__pyx_float_17_95);
  Py_VISIT(traverse_module_state->__pyx_float_17_97);
  Py_VISIT(traverse_module_state->__pyx_float_18_02);
  Py_VISIT(traverse_module_state->__pyx_float_18_03);
  Py_VISIT(traverse_module_state->__pyx_float_18_08);
  Py_VISIT(traverse_module_state->__pyx_float_18_09);
  Py_VISIT(traverse_module_state->__pyx_float_18_10);
  Py_VISIT(traverse_module_state->__pyx_float_18_11);
  Py_VISIT(traverse_module_state->__pyx_float_18_12);
  Py_VISIT(traverse_module_state->__pyx_float_18_13);
  Py_VISIT(traverse_module_state->__pyx_float_18_14);
  Py_VISIT(traverse_module_state->__pyx_float_18_15);
  Py_VISIT(traverse_module_state->__pyx_float_18_16);
  Py_VISIT(traverse_module_state->__pyx_float_18_18);
  Py_VISIT(traverse_module_state->__pyx_float_18_19);
  Py_VISIT(traverse_module_state->__pyx_float_18_20);
  Py_VISIT(traverse_module_state->__pyx_float_18_21);
  Py_VISIT(traverse_module_state->__pyx_float_18_23);
  Py_VISIT(traverse_module_state->__pyx_float_18_25);
  Py_VISIT(traverse_module_state->__pyx_float_18_27);
  Py_VISIT(traverse_module_state->__pyx_float_18_30);
  Py_VISIT(traverse_module_state->__pyx_float_18_35);
  Py_VISIT(traverse_module_state->__pyx_float_18_43);
  Py_VISIT(traverse_module_state->__pyx_float_18_45);
  Py_VISIT(traverse_module_state->__pyx_float_18_51);
  Py_VISIT(traverse_module_state->__pyx_float_18_59);
  Py_VISIT(traverse_module_state->__pyx_float_18_71);
  Py_VISIT(traverse_module_state->__pyx_float_18_75);
  Py_VISIT(traverse_module_state->__pyx_float_18_84);
  Py_VISIT(traverse_module_state->__pyx_float_18_94);
  Py_VISIT(traverse_module_state->__pyx_float_18_95);
  Py_VISIT(traverse_module_state->__pyx_float_19_04);
  Py_VISIT(traverse_module_state->__pyx_float_19_05);
  Py_VISIT(traverse_module_state->__pyx_float_19_06);
  Py_VISIT(traverse_module_state->__pyx_float_19_11);
  Py_VISIT(traverse_module_state->__pyx_float_19_12);
  Py_VISIT(traverse_module_state->__pyx_float_19_14);
  Py_VISIT(traverse_module_state->__pyx_float_19_15);
  Py_VISIT(traverse_module_state->__pyx_float_19_16);
  Py_VISIT(traverse_module_state->__pyx_float_19_17);
  Py_VISIT(traverse_module_state->__pyx_float_19_18);
  Py_VISIT(traverse_module_state->__pyx_float_19_21);
  Py_VISIT(traverse_module_state->__pyx_float_19_22);
  Py_VISIT(traverse_module_state->__pyx_float_19_23);
  Py_VISIT(traverse_module_state->__pyx_float_19_24);
  Py_VISIT(traverse_module_state->__pyx_float_19_26);
  Py_VISIT(traverse_module_state->__pyx_float_19_29);
  Py_VISIT(traverse_module_state->__pyx_float_19_34);
  Py_VISIT(traverse_module_state->__pyx_float_19_36);
  Py_VISIT(traverse_module_state->__pyx_float_19_39);
  Py_VISIT(traverse_module_state->__pyx_float_19_41);
  Py_VISIT(traverse_module_state->__pyx_float_19_51);
  Py_VISIT(traverse_module_state->__pyx_float_19_56);
  Py_VISIT(traverse_module_state->__pyx_float_19_57);
  Py_VISIT(traverse_module_state->__pyx_float_19_61);
  Py_VISIT(traverse_module_state->__pyx_float_19_71);
  Py_VISIT(traverse_module_state->__pyx_float_19_83);
  Py_VISIT(traverse_module_state->__pyx_float_19_84);
  Py_VISIT(traverse_module_state->__pyx_float_19_89);
  Py_VISIT(traverse_module_state->__pyx_float_20_00);
  Py_VISIT(traverse_module_state->__pyx_float_20_08);
  Py_VISIT(traverse_module_state->__pyx_float_20_11);
  Py_VISIT(traverse_module_state->__pyx_float_20_12);
  Py_VISIT(traverse_module_state->__pyx_float_20_13);
  Py_VISIT(traverse_module_state->__pyx_float_20_16);
  Py_VISIT(traverse_module_state->__pyx_float_20_17);
  Py_VISIT(traverse_module_state->__pyx_float_20_18);
  Py_VISIT(traverse_module_state->__pyx_float_20_19);
  Py_VISIT(traverse_module_state->__pyx_float_20_21);
  Py_VISIT(traverse_module_state->__pyx_float_20_23);
  Py_VISIT(traverse_module_state->__pyx_float_20_24);
  Py_VISIT(traverse_module_state->__pyx_float_20_25);
  Py_VISIT(traverse_module_state->__pyx_float_20_26);
  Py_VISIT(traverse_module_state->__pyx_float_20_27);
  Py_VISIT(traverse_module_state->__pyx_float_20_28);
  Py_VISIT(traverse_module_state->__pyx_float_20_29);
  Py_VISIT(traverse_module_state->__pyx_float_20_30);
  Py_VISIT(traverse_module_state->__pyx_float_20_34);
  Py_VISIT(traverse_module_state->__pyx_float_20_36);
  Py_VISIT(traverse_module_state->__pyx_float_20_40);
  Py_VISIT(traverse_module_state->__pyx_float_20_43);
  Py_VISIT(traverse_module_state->__pyx_float_20_44);
  Py_VISIT(traverse_module_state->__pyx_float_20_50);
  Py_VISIT(traverse_module_state->__pyx_float_20_60);
  Py_VISIT(traverse_module_state->__pyx_float_20_65);
  Py_VISIT(traverse_module_state->__pyx_float_20_72);
  Py_VISIT(traverse_module_state->__pyx_float_20_84);
  Py_VISIT(traverse_module_state->__pyx_float_20_98);
  Py_VISIT(traverse_module_state->__pyx_float_20_99);
  Py_VISIT(traverse_module_state->__pyx_float_21_04);
  Py_VISIT(traverse_module_state->__pyx_float_21_11);
  Py_VISIT(traverse_module_state->__pyx_float_21_13);
  Py_VISIT(traverse_module_state->__pyx_float_21_17);
  Py_VISIT(traverse_module_state->__pyx_float_21_18);
  Py_VISIT(traverse_module_state->__pyx_float_21_20);
  Py_VISIT(traverse_module_state->__pyx_float_21_21);
  Py_VISIT(traverse_module_state->__pyx_float_21_22);
  Py_VISIT(traverse_module_state->__pyx_float_21_23);
  Py_VISIT(traverse_module_state->__pyx_float_21_24);
  Py_VISIT(traverse_module_state->__pyx_float_21_25);
  Py_VISIT(traverse_module_state->__pyx_float_21_27);
  Py_VISIT(traverse_module_state->__pyx_float_21_29);
  Py_VISIT(traverse_module_state->__pyx_float_21_30);
  Py_VISIT(traverse_module_state->__pyx_float_21_31);
  Py_VISIT(traverse_module_state->__pyx_float_21_32);
  Py_VISIT(traverse_module_state->__pyx_float_21_33);
  Py_VISIT(traverse_module_state->__pyx_float_21_41);
  Py_VISIT(traverse_module_state->__pyx_float_21_47);
  Py_VISIT(traverse_module_state->__pyx_float_21_50);
  Py_VISIT(traverse_module_state->__pyx_float_21_51);
  Py_VISIT(traverse_module_state->__pyx_float_21_56);
  Py_VISIT(traverse_module_state->__pyx_float_21_57);
  Py_VISIT(traverse_module_state->__pyx_float_21_70);
  Py_VISIT(traverse_module_state->__pyx_float_21_74);
  Py_VISIT(traverse_module_state->__pyx_float_21_84);
  Py_VISIT(traverse_module_state->__pyx_float_21_98);
  Py_VISIT(traverse_module_state->__pyx_float_22_12);
  Py_VISIT(traverse_module_state->__pyx_float_22_13);
  Py_VISIT(traverse_module_state->__pyx_float_22_14);
  Py_VISIT(traverse_module_state->__pyx_float_22_16);
  Py_VISIT(traverse_module_state->__pyx_float_22_20);
  Py_VISIT(traverse_module_state->__pyx_float_22_21);
  Py_VISIT(traverse_module_state->__pyx_float_22_22);
  Py_VISIT(traverse_module_state->__pyx_float_22_25);
  Py_VISIT(traverse_module_state->__pyx_float_22_26);
  Py_VISIT(traverse_module_state->__pyx_float_22_27);
  Py_VISIT(traverse_module_state->__pyx_float_22_28);
  Py_VISIT(traverse_module_state->__pyx_float_22_29);
  Py_VISIT(traverse_module_state->__pyx_float_22_30);
  Py_VISIT(traverse_module_state->__pyx_float_22_32);
  Py_VISIT(traverse_module_state->__pyx_float_22_35);
  Py_VISIT(traverse_module_state->__pyx_float_22_36);
  Py_VISIT(traverse_module_state->__pyx_float_22_37);
  Py_VISIT(traverse_module_state->__pyx_float_22_38);
  Py_VISIT(traverse_module_state->__pyx_float_22_48);
  Py_VISIT(traverse_module_state->__pyx_float_22_49);
  Py_VISIT(traverse_module_state->__pyx_float_22_54);
  Py_VISIT(traverse_module_state->__pyx_float_22_59);
  Py_VISIT(traverse_module_state->__pyx_float_22_60);
  Py_VISIT(traverse_module_state->__pyx_float_22_68);
  Py_VISIT(traverse_module_state->__pyx_float_22_82);
  Py_VISIT(traverse_module_state->__pyx_float_22_87);
  Py_VISIT(traverse_module_state->__pyx_float_22_99);
  Py_VISIT(traverse_module_state->__pyx_float_23_11);
  Py_VISIT(traverse_module_state->__pyx_float_23_19);
  Py_VISIT(traverse_module_state->__pyx_float_23_20);
  Py_VISIT(traverse_module_state->__pyx_float_23_22);
  Py_VISIT(traverse_module_state->__pyx_float_23_24);
  Py_VISIT(traverse_module_state->__pyx_float_23_26);
  Py_VISIT(traverse_module_state->__pyx_float_23_28);
  Py_VISIT(traverse_module_state->__pyx_float_23_29);
  Py_VISIT(traverse_module_state->__pyx_float_23_30);
  Py_VISIT(traverse_module_state->__pyx_float_23_31);
  Py_VISIT(traverse_module_state->__pyx_float_23_32);
  Py_VISIT(traverse_module_state->__pyx_float_23_33);
  Py_VISIT(traverse_module_state->__pyx_float_23_34);
  Py_VISIT(traverse_module_state->__pyx_float_23_35);
  Py_VISIT(traverse_module_state->__pyx_float_23_38);
  Py_VISIT(traverse_module_state->__pyx_float_23_39);
  Py_VISIT(traverse_module_state->__pyx_float_23_41);
  Py_VISIT(traverse_module_state->__pyx_float_23_42);
  Py_VISIT(traverse_module_state->__pyx_float_23_46);
  Py_VISIT(traverse_module_state->__pyx_float_23_55);
  Py_VISIT(traverse_module_state->__pyx_float_23_56);
  Py_VISIT(traverse_module_state->__pyx_float_23_62);
  Py_VISIT(traverse_module_state->__pyx_float_23_69);
  Py_VISIT(traverse_module_state->__pyx_float_23_72);
  Py_VISIT(traverse_module_state->__pyx_float_23_73);
  Py_VISIT(traverse_module_state->__pyx_float_23_80);
  Py_VISIT(traverse_module_state->__pyx_float_23_90);
  Py_VISIT(traverse_module_state->__pyx_float_23_95);
  Py_VISIT(traverse_module_state->__pyx_float_23_99);
  Py_VISIT(traverse_module_state->__pyx_float_24_00);
  Py_VISIT(traverse_module_state->__pyx_float_24_05);
  Py_VISIT(traverse_module_state->__pyx_float_24_13);
  Py_VISIT(traverse_module_state->__pyx_float_24_14);
  Py_VISIT(traverse_module_state->__pyx_float_24_19);
  Py_VISIT(traverse_module_state->__pyx_float_24_26);
  Py_VISIT(traverse_module_state->__pyx_float_24_29);
  Py_VISIT(traverse_module_state->__pyx_float_24_31);
  Py_VISIT(traverse_module_state->__pyx_float_24_32);
  Py_VISIT(traverse_module_state->__pyx_float_24_33);
  Py_VISIT(traverse_module_state->__pyx_float_24_34);
  Py_VISIT(traverse_module_state->__pyx_float_24_35);
  Py_VISIT(traverse_module_state->__pyx_float_24_36);
  Py_VISIT(traverse_module_state->__pyx_float_24_37);
  Py_VISIT(traverse_module_state->__pyx_float_24_38);
  Py_VISIT(traverse_module_state->__pyx_float_24_40);
  Py_VISIT(traverse_module_state->__pyx_float_24_48);
  Py_VISIT(traverse_module_state->__pyx_float_24_50);
  Py_VISIT(traverse_module_state->__pyx_float_24_52);
  Py_VISIT(traverse_module_state->__pyx_float_24_64);
  Py_VISIT(traverse_module_state->__pyx_float_24_71);
  Py_VISIT(traverse_module_state->__pyx_float_24_80);
  Py_VISIT(traverse_module_state->__pyx_float_24_90);
  Py_VISIT(traverse_module_state->__pyx_float_24_97);
  Py_VISIT(traverse_module_state->__pyx_float_25_05);
  Py_VISIT(traverse_module_state->__pyx_float_25_15);
  Py_VISIT(traverse_module_state->__pyx_float_25_20);
  Py_VISIT(traverse_module_state->__pyx_float_25_24);
  Py_VISIT(traverse_module_state->__pyx_float_25_30);
  Py_VISIT(traverse_module_state->__pyx_float_25_31);
  Py_VISIT(traverse_module_state->__pyx_float_25_32);
  Py_VISIT(traverse_module_state->__pyx_float_25_33);
  Py_VISIT(traverse_module_state->__pyx_float_25_35);
  Py_VISIT(traverse_module_state->__pyx_float_25_38);
  Py_VISIT(traverse_module_state->__pyx_float_25_41);
  Py_VISIT(traverse_module_state->__pyx_float_25_42);
  Py_VISIT(traverse_module_state->__pyx_float_25_43);
  Py_VISIT(traverse_module_state->__pyx_float_25_45);
  Py_VISIT(traverse_module_state->__pyx_float_25_49);
  Py_VISIT(traverse_module_state->__pyx_float_25_54);
  Py_VISIT(traverse_module_state->__pyx_float_25_55);
  Py_VISIT(traverse_module_state->__pyx_float_25_56);
  Py_VISIT(traverse_module_state->__pyx_float_25_58);
  Py_VISIT(traverse_module_state->__pyx_float_25_59);
  Py_VISIT(traverse_module_state->__pyx_float_25_70);
  Py_VISIT(traverse_module_state->__pyx_float_25_74);
  Py_VISIT(traverse_module_state->__pyx_float_25_82);
  Py_VISIT(traverse_module_state->__pyx_float_25_87);
  Py_VISIT(traverse_module_state->__pyx_float_25_88);
  Py_VISIT(traverse_module_state->__pyx_float_26_02);
  Py_VISIT(traverse_module_state->__pyx_float_26_17);
  Py_VISIT(traverse_module_state->__pyx_float_26_22);
  Py_VISIT(traverse_module_state->__pyx_float_26_24);
  Py_VISIT(traverse_module_state->__pyx_float_26_36);
  Py_VISIT(traverse_module_state->__pyx_float_26_38);
  Py_VISIT(traverse_module_state->__pyx_float_26_39);
  Py_VISIT(traverse_module_state->__pyx_float_26_40);
  Py_VISIT(traverse_module_state->__pyx_float_26_41);
  Py_VISIT(traverse_module_state->__pyx_float_26_43);
  Py_VISIT(traverse_module_state->__pyx_float_26_46);
  Py_VISIT(traverse_module_state->__pyx_float_26_47);
  Py_VISIT(traverse_module_state->__pyx_float_26_49);
  Py_VISIT(traverse_module_state->__pyx_float_26_50);
  Py_VISIT(traverse_module_state->__pyx_float_26_54);
  Py_VISIT(traverse_module_state->__pyx_float_26_55);
  Py_VISIT(traverse_module_state->__pyx_float_26_61);
  Py_VISIT(traverse_module_state->__pyx_float_26_62);
  Py_VISIT(traverse_module_state->__pyx_float_26_64);
  Py_VISIT(traverse_module_state->__pyx_float_26_67);
  Py_VISIT(traverse_module_state->__pyx_float_26_68);
  Py_VISIT(traverse_module_state->__pyx_float_26_69);
  Py_VISIT(traverse_module_state->__pyx_float_26_70);
  Py_VISIT(traverse_module_state->__pyx_float_26_81);
  Py_VISIT(traverse_module_state->__pyx_float_26_91);
  Py_VISIT(traverse_module_state->__pyx_float_27_02);
  Py_VISIT(traverse_module_state->__pyx_float_27_03);
  Py_VISIT(traverse_module_state->__pyx_float_27_04);
  Py_VISIT(traverse_module_state->__pyx_float_27_16);
  Py_VISIT(traverse_module_state->__pyx_float_27_36);
  Py_VISIT(traverse_module_state->__pyx_float_27_41);
  Py_VISIT(traverse_module_state->__pyx_float_27_43);
  Py_VISIT(traverse_module_state->__pyx_float_27_44);
  Py_VISIT(traverse_module_state->__pyx_float_27_46);
  Py_VISIT(traverse_module_state->__pyx_float_27_47);
  Py_VISIT(traverse_module_state->__pyx_float_27_49);
  Py_VISIT(traverse_module_state->__pyx_float_27_50);
  Py_VISIT(traverse_module_state->__pyx_float_27_52);
  Py_VISIT(traverse_module_state->__pyx_float_27_53);
  Py_VISIT(traverse_module_state->__pyx_float_27_54);
  Py_VISIT(traverse_module_state->__pyx_float_27_55);
  Py_VISIT(traverse_module_state->__pyx_float_27_59);
  Py_VISIT(traverse_module_state->__pyx_float_27_60);
  Py_VISIT(traverse_module_state->__pyx_float_27_61);
  Py_VISIT(traverse_module_state->__pyx_float_27_70);
  Py_VISIT(traverse_module_state->__pyx_float_27_72);
  Py_VISIT(traverse_module_state->__pyx_float_27_76);
  Py_VISIT(traverse_module_state->__pyx_float_27_77);
  Py_VISIT(traverse_module_state->__pyx_float_27_78);
  Py_VISIT(traverse_module_state->__pyx_float_27_95);
  Py_VISIT(traverse_module_state->__pyx_float_27_97);
  Py_VISIT(traverse_module_state->__pyx_float_28_03);
  Py_VISIT(traverse_module_state->__pyx_float_28_09);
  Py_VISIT(traverse_module_state->__pyx_float_28_12);
  Py_VISIT(traverse_module_state->__pyx_float_28_16);
  Py_VISIT(traverse_module_state->__pyx_float_28_30);
  Py_VISIT(traverse_module_state->__pyx_float_28_39);
  Py_VISIT(traverse_module_state->__pyx_float_28_44);
  Py_VISIT(traverse_module_state->__pyx_float_28_45);
  Py_VISIT(traverse_module_state->__pyx_float_28_47);
  Py_VISIT(traverse_module_state->__pyx_float_28_50);
  Py_VISIT(traverse_module_state->__pyx_float_28_51);
  Py_VISIT(traverse_module_state->__pyx_float_28_52);
  Py_VISIT(traverse_module_state->__pyx_float_28_53);
  Py_VISIT(traverse_module_state->__pyx_float_28_55);
  Py_VISIT(traverse_module_state->__pyx_float_28_59);
  Py_VISIT(traverse_module_state->__pyx_float_28_60);
  Py_VISIT(traverse_module_state->__pyx_float_28_62);
  Py_VISIT(traverse_module_state->__pyx_float_28_63);
  Py_VISIT(traverse_module_state->__pyx_float_28_64);
  Py_VISIT(traverse_module_state->__pyx_float_28_65);
  Py_VISIT(traverse_module_state->__pyx_float_28_66);
  Py_VISIT(traverse_module_state->__pyx_float_28_70);
  Py_VISIT(traverse_module_state->__pyx_float_28_73);
  Py_VISIT(traverse_module_state->__pyx_float_28_78);
  Py_VISIT(traverse_module_state->__pyx_float_28_79);
  Py_VISIT(traverse_module_state->__pyx_float_28_80);
  Py_VISIT(traverse_module_state->__pyx_float_28_82);
  Py_VISIT(traverse_module_state->__pyx_float_28_87);
  Py_VISIT(traverse_module_state->__pyx_float_28_88);
  Py_VISIT(traverse_module_state->__pyx_float_29_03);
  Py_VISIT(traverse_module_state->__pyx_float_29_12);
  Py_VISIT(traverse_module_state->__pyx_float_29_17);
  Py_VISIT(traverse_module_state->__pyx_float_29_28);
  Py_VISIT(traverse_module_state->__pyx_float_29_31);
  Py_VISIT(traverse_module_state->__pyx_float_29_40);
  Py_VISIT(traverse_module_state->__pyx_float_29_48);
  Py_VISIT(traverse_module_state->__pyx_float_29_54);
  Py_VISIT(traverse_module_state->__pyx_float_29_57);
  Py_VISIT(traverse_module_state->__pyx_float_29_58);
  Py_VISIT(traverse_module_state->__pyx_float_29_60);
  Py_VISIT(traverse_module_state->__pyx_float_29_61);
  Py_VISIT(traverse_module_state->__pyx_float_29_64);
  Py_VISIT(traverse_module_state->__pyx_float_29_65);
  Py_VISIT(traverse_module_state->__pyx_float_29_66);
  Py_VISIT(traverse_module_state->__pyx_float_29_68);
  Py_VISIT(traverse_module_state->__pyx_float_29_71);
  Py_VISIT(traverse_module_state->__pyx_float_29_72);
  Py_VISIT(traverse_module_state->__pyx_float_29_76);
  Py_VISIT(traverse_module_state->__pyx_float_29_80);
  Py_VISIT(traverse_module_state->__pyx_float_29_81);
  Py_VISIT(traverse_module_state->__pyx_float_29_85);
  Py_VISIT(traverse_module_state->__pyx_float_29_89);
  Py_VISIT(traverse_module_state->__pyx_float_29_90);
  Py_VISIT(traverse_module_state->__pyx_float_29_91);
  Py_VISIT(traverse_module_state->__pyx_float_29_95);
  Py_VISIT(traverse_module_state->__pyx_float_30_03);
  Py_VISIT(traverse_module_state->__pyx_float_30_08);
  Py_VISIT(traverse_module_state->__pyx_float_30_09);
  Py_VISIT(traverse_module_state->__pyx_float_30_10);
  Py_VISIT(traverse_module_state->__pyx_float_30_22);
  Py_VISIT(traverse_module_state->__pyx_float_30_32);
  Py_VISIT(traverse_module_state->__pyx_float_30_37);
  Py_VISIT(traverse_module_state->__pyx_float_30_43);
  Py_VISIT(traverse_module_state->__pyx_float_30_46);
  Py_VISIT(traverse_module_state->__pyx_float_30_58);
  Py_VISIT(traverse_module_state->__pyx_float_30_60);
  Py_VISIT(traverse_module_state->__pyx_float_30_65);
  Py_VISIT(traverse_module_state->__pyx_float_30_70);
  Py_VISIT(traverse_module_state->__pyx_float_30_73);
  Py_VISIT(traverse_module_state->__pyx_float_30_75);
  Py_VISIT(traverse_module_state->__pyx_float_30_77);
  Py_VISIT(traverse_module_state->__pyx_float_30_78);
  Py_VISIT(traverse_module_state->__pyx_float_30_81);
  Py_VISIT(traverse_module_state->__pyx_float_30_85);
  Py_VISIT(traverse_module_state->__pyx_float_30_91);
  Py_VISIT(traverse_module_state->__pyx_float_30_97);
  Py_VISIT(traverse_module_state->__pyx_float_30_98);
  Py_VISIT(traverse_module_state->__pyx_float_31_01);
  Py_VISIT(traverse_module_state->__pyx_float_31_05);
  Py_VISIT(traverse_module_state->__pyx_float_31_06);
  Py_VISIT(traverse_module_state->__pyx_float_31_13);
  Py_VISIT(traverse_module_state->__pyx_float_31_14);
  Py_VISIT(traverse_module_state->__pyx_float_31_30);
  Py_VISIT(traverse_module_state->__pyx_float_31_45);
  Py_VISIT(traverse_module_state->__pyx_float_31_59);
  Py_VISIT(traverse_module_state->__pyx_float_31_61);
  Py_VISIT(traverse_module_state->__pyx_float_31_62);
  Py_VISIT(traverse_module_state->__pyx_float_31_66);
  Py_VISIT(traverse_module_state->__pyx_float_31_67);
  Py_VISIT(traverse_module_state->__pyx_float_31_71);
  Py_VISIT(traverse_module_state->__pyx_float_31_73);
  Py_VISIT(traverse_module_state->__pyx_float_31_74);
  Py_VISIT(traverse_module_state->__pyx_float_31_78);
  Py_VISIT(traverse_module_state->__pyx_float_31_81);
  Py_VISIT(traverse_module_state->__pyx_float_31_82);
  Py_VISIT(traverse_module_state->__pyx_float_31_84);
  Py_VISIT(traverse_module_state->__pyx_float_31_86);
  Py_VISIT(traverse_module_state->__pyx_float_31_88);
  Py_VISIT(traverse_module_state->__pyx_float_31_91);
  Py_VISIT(traverse_module_state->__pyx_float_31_95);
  Py_VISIT(traverse_module_state->__pyx_float_31_96);
  Py_VISIT(traverse_module_state->__pyx_float_31_97);
  Py_VISIT(traverse_module_state->__pyx_float_32_00);
  Py_VISIT(traverse_module_state->__pyx_float_32_03);
  Py_VISIT(traverse_module_state->__pyx_float_32_07);
  Py_VISIT(traverse_module_state->__pyx_float_32_08);
  Py_VISIT(traverse_module_state->__pyx_float_32_09);
  Py_VISIT(traverse_module_state->__pyx_float_32_12);
  Py_VISIT(traverse_module_state->__pyx_float_32_13);
  Py_VISIT(traverse_module_state->__pyx_float_32_14);
  Py_VISIT(traverse_module_state->__pyx_float_32_17);
  Py_VISIT(traverse_module_state->__pyx_float_32_24);
  Py_VISIT(traverse_module_state->__pyx_float_32_27);
  Py_VISIT(traverse_module_state->__pyx_float_32_48);
  Py_VISIT(traverse_module_state->__pyx_float_32_61);
  Py_VISIT(traverse_module_state->__pyx_float_32_65);
  Py_VISIT(traverse_module_state->__pyx_float_32_73);
  Py_VISIT(traverse_module_state->__pyx_float_32_75);
  Py_VISIT(traverse_module_state->__pyx_float_32_77);
  Py_VISIT(traverse_module_state->__pyx_float_32_78);
  Py_VISIT(traverse_module_state->__pyx_float_32_79);
  Py_VISIT(traverse_module_state->__pyx_float_32_83);
  Py_VISIT(traverse_module_state->__pyx_float_32_84);
  Py_VISIT(traverse_module_state->__pyx_float_32_89);
  Py_VISIT(traverse_module_state->__pyx_float_32_92);
  Py_VISIT(traverse_module_state->__pyx_float_32_93);
  Py_VISIT(traverse_module_state->__pyx_float_32_94);
  Py_VISIT(traverse_module_state->__pyx_float_32_96);
  Py_VISIT(traverse_module_state->__pyx_float_32_97);
  Py_VISIT(traverse_module_state->__pyx_float_33_00);
  Py_VISIT(traverse_module_state->__pyx_float_33_02);
  Py_VISIT(traverse_module_state->__pyx_float_33_14);
  Py_VISIT(traverse_module_state->__pyx_float_33_21);
  Py_VISIT(traverse_module_state->__pyx_float_33_28);
  Py_VISIT(traverse_module_state->__pyx_float_33_29);
  Py_VISIT(traverse_module_state->__pyx_float_33_32);
  Py_VISIT(traverse_module_state->__pyx_float_33_36);
  Py_VISIT(traverse_module_state->__pyx_float_33_37);
  Py_VISIT(traverse_module_state->__pyx_float_33_52);
  Py_VISIT(traverse_module_state->__pyx_float_33_59);
  Py_VISIT(traverse_module_state->__pyx_float_33_60);
  Py_VISIT(traverse_module_state->__pyx_float_33_77);
  Py_VISIT(traverse_module_state->__pyx_float_33_81);
  Py_VISIT(traverse_module_state->__pyx_float_33_82);
  Py_VISIT(traverse_module_state->__pyx_float_33_85);
  Py_VISIT(traverse_module_state->__pyx_float_33_95);
  Py_VISIT(traverse_module_state->__pyx_float_34_03);
  Py_VISIT(traverse_module_state->__pyx_float_34_04);
  Py_VISIT(traverse_module_state->__pyx_float_34_08);
  Py_VISIT(traverse_module_state->__pyx_float_34_09);
  Py_VISIT(traverse_module_state->__pyx_float_34_12);
  Py_VISIT(traverse_module_state->__pyx_float_34_13);
  Py_VISIT(traverse_module_state->__pyx_float_34_19);
  Py_VISIT(traverse_module_state->__pyx_float_34_21);
  Py_VISIT(traverse_module_state->__pyx_float_34_26);
  Py_VISIT(traverse_module_state->__pyx_float_34_30);
  Py_VISIT(traverse_module_state->__pyx_float_34_31);
  Py_VISIT(traverse_module_state->__pyx_float_34_37);
  Py_VISIT(traverse_module_state->__pyx_float_34_42);
  Py_VISIT(traverse_module_state->__pyx_float_34_52);
  Py_VISIT(traverse_module_state->__pyx_float_34_76);
  Py_VISIT(traverse_module_state->__pyx_float_34_79);
  Py_VISIT(traverse_module_state->__pyx_float_34_91);
  Py_VISIT(traverse_module_state->__pyx_float_34_95);
  Py_VISIT(traverse_module_state->__pyx_float_34_96);
  Py_VISIT(traverse_module_state->__pyx_float_34_98);
  Py_VISIT(traverse_module_state->__pyx_float_35_06);
  Py_VISIT(traverse_module_state->__pyx_float_35_07);
  Py_VISIT(traverse_module_state->__pyx_float_35_13);
  Py_VISIT(traverse_module_state->__pyx_float_35_14);
  Py_VISIT(traverse_module_state->__pyx_float_35_16);
  Py_VISIT(traverse_module_state->__pyx_float_35_17);
  Py_VISIT(traverse_module_state->__pyx_float_35_18);
  Py_VISIT(traverse_module_state->__pyx_float_35_25);
  Py_VISIT(traverse_module_state->__pyx_float_35_30);
  Py_VISIT(traverse_module_state->__pyx_float_35_38);
  Py_VISIT(traverse_module_state->__pyx_float_35_44);
  Py_VISIT(traverse_module_state->__pyx_float_35_46);
  Py_VISIT(traverse_module_state->__pyx_float_35_50);
  Py_VISIT(traverse_module_state->__pyx_float_35_59);
  Py_VISIT(traverse_module_state->__pyx_float_35_64);
  Py_VISIT(traverse_module_state->__pyx_float_35_67);
  Py_VISIT(traverse_module_state->__pyx_float_35_75);
  Py_VISIT(traverse_module_state->__pyx_float_36_00);
  Py_VISIT(traverse_module_state->__pyx_float_36_01);
  Py_VISIT(traverse_module_state->__pyx_float_36_02);
  Py_VISIT(traverse_module_state->__pyx_float_36_07);
  Py_VISIT(traverse_module_state->__pyx_float_36_09);
  Py_VISIT(traverse_module_state->__pyx_float_36_14);
  Py_VISIT(traverse_module_state->__pyx_float_36_16);
  Py_VISIT(traverse_module_state->__pyx_float_36_20);
  Py_VISIT(traverse_module_state->__pyx_float_36_25);
  Py_VISIT(traverse_module_state->__pyx_float_36_27);
  Py_VISIT(traverse_module_state->__pyx_float_36_28);
  Py_VISIT(traverse_module_state->__pyx_float_36_29);
  Py_VISIT(traverse_module_state->__pyx_float_36_30);
  Py_VISIT(traverse_module_state->__pyx_float_36_32);
  Py_VISIT(traverse_module_state->__pyx_float_36_34);
  Py_VISIT(traverse_module_state->__pyx_float_36_40);
  Py_VISIT(traverse_module_state->__pyx_float_36_43);
  Py_VISIT(traverse_module_state->__pyx_float_36_55);
  Py_VISIT(traverse_module_state->__pyx_float_36_56);
  Py_VISIT(traverse_module_state->__pyx_float_36_62);
  Py_VISIT(traverse_module_state->__pyx_float_36_74);
  Py_VISIT(traverse_module_state->__pyx_float_36_78);
  Py_VISIT(traverse_module_state->__pyx_float_36_88);
  Py_VISIT(traverse_module_state->__pyx_float_37_11);
  Py_VISIT(traverse_module_state->__pyx_float_37_16);
  Py_VISIT(traverse_module_state->__pyx_float_37_20);
  Py_VISIT(traverse_module_state->__pyx_float_37_23);
  Py_VISIT(traverse_module_state->__pyx_float_37_27);
  Py_VISIT(traverse_module_state->__pyx_float_37_32);
  Py_VISIT(traverse_module_state->__pyx_float_37_34);
  Py_VISIT(traverse_module_state->__pyx_float_37_35);
  Py_VISIT(traverse_module_state->__pyx_float_37_38);
  Py_VISIT(traverse_module_state->__pyx_float_37_39);
  Py_VISIT(traverse_module_state->__pyx_float_37_41);
  Py_VISIT(traverse_module_state->__pyx_float_37_51);
  Py_VISIT(traverse_module_state->__pyx_float_37_56);
  Py_VISIT(traverse_module_state->__pyx_float_37_57);
  Py_VISIT(traverse_module_state->__pyx_float_37_68);
  Py_VISIT(traverse_module_state->__pyx_float_37_69);
  Py_VISIT(traverse_module_state->__pyx_float_37_76);
  Py_VISIT(traverse_module_state->__pyx_float_37_77);
  Py_VISIT(traverse_module_state->__pyx_float_37_88);
  Py_VISIT(traverse_module_state->__pyx_float_37_89);
  Py_VISIT(traverse_module_state->__pyx_float_37_90);
  Py_VISIT(traverse_module_state->__pyx_float_37_93);
  Py_VISIT(traverse_module_state->__pyx_float_37_99);
  Py_VISIT(traverse_module_state->__pyx_float_38_10);
  Py_VISIT(traverse_module_state->__pyx_float_38_18);
  Py_VISIT(traverse_module_state->__pyx_float_38_20);
  Py_VISIT(traverse_module_state->__pyx_float_38_25);
  Py_VISIT(traverse_module_state->__pyx_float_38_26);
  Py_VISIT(traverse_module_state->__pyx_float_38_30);
  Py_VISIT(traverse_module_state->__pyx_float_38_33);
  Py_VISIT(traverse_module_state->__pyx_float_38_36);
  Py_VISIT(traverse_module_state->__pyx_float_38_46);
  Py_VISIT(traverse_module_state->__pyx_float_38_51);
  Py_VISIT(traverse_module_state->__pyx_float_38_53);
  Py_VISIT(traverse_module_state->__pyx_float_38_54);
  Py_VISIT(traverse_module_state->__pyx_float_38_55);
  Py_VISIT(traverse_module_state->__pyx_float_38_58);
  Py_VISIT(traverse_module_state->__pyx_float_38_64);
  Py_VISIT(traverse_module_state->__pyx_float_38_68);
  Py_VISIT(traverse_module_state->__pyx_float_38_80);
  Py_VISIT(traverse_module_state->__pyx_float_38_85);
  Py_VISIT(traverse_module_state->__pyx_float_38_86);
  Py_VISIT(traverse_module_state->__pyx_float_38_92);
  Py_VISIT(traverse_module_state->__pyx_float_38_93);
  Py_VISIT(traverse_module_state->__pyx_float_39_00);
  Py_VISIT(traverse_module_state->__pyx_float_39_03);
  Py_VISIT(traverse_module_state->__pyx_float_39_06);
  Py_VISIT(traverse_module_state->__pyx_float_39_11);
  Py_VISIT(traverse_module_state->__pyx_float_39_17);
  Py_VISIT(traverse_module_state->__pyx_float_39_23);
  Py_VISIT(traverse_module_state->__pyx_float_39_29);
  Py_VISIT(traverse_module_state->__pyx_float_39_35);
  Py_VISIT(traverse_module_state->__pyx_float_39_41);
  Py_VISIT(traverse_module_state->__pyx_float_39_47);
  Py_VISIT(traverse_module_state->__pyx_float_39_59);
  Py_VISIT(traverse_module_state->__pyx_float_39_60);
  Py_VISIT(traverse_module_state->__pyx_float_39_62);
  Py_VISIT(traverse_module_state->__pyx_float_39_64);
  Py_VISIT(traverse_module_state->__pyx_float_39_65);
  Py_VISIT(traverse_module_state->__pyx_float_39_66);
  Py_VISIT(traverse_module_state->__pyx_float_39_67);
  Py_VISIT(traverse_module_state->__pyx_float_39_75);
  Py_VISIT(traverse_module_state->__pyx_float_39_80);
  Py_VISIT(traverse_module_state->__pyx_float_39_88);
  Py_VISIT(traverse_module_state->__pyx_float_40_02);
  Py_VISIT(traverse_module_state->__pyx_float_40_03);
  Py_VISIT(traverse_module_state->__pyx_float_40_05);
  Py_VISIT(traverse_module_state->__pyx_float_40_07);
  Py_VISIT(traverse_module_state->__pyx_float_40_08);
  Py_VISIT(traverse_module_state->__pyx_float_40_09);
  Py_VISIT(traverse_module_state->__pyx_float_40_11);
  Py_VISIT(traverse_module_state->__pyx_float_40_19);
  Py_VISIT(traverse_module_state->__pyx_float_40_25);
  Py_VISIT(traverse_module_state->__pyx_float_40_32);
  Py_VISIT(traverse_module_state->__pyx_float_40_33);
  Py_VISIT(traverse_module_state->__pyx_float_40_35);
  Py_VISIT(traverse_module_state->__pyx_float_40_43);
  Py_VISIT(traverse_module_state->__pyx_float_40_53);
  Py_VISIT(traverse_module_state->__pyx_float_40_54);
  Py_VISIT(traverse_module_state->__pyx_float_40_55);
  Py_VISIT(traverse_module_state->__pyx_float_40_58);
  Py_VISIT(traverse_module_state->__pyx_float_40_69);
  Py_VISIT(traverse_module_state->__pyx_float_40_71);
  Py_VISIT(traverse_module_state->__pyx_float_40_77);
  Py_VISIT(traverse_module_state->__pyx_float_40_80);
  Py_VISIT(traverse_module_state->__pyx_float_40_85);
  Py_VISIT(traverse_module_state->__pyx_float_40_90);
  Py_VISIT(traverse_module_state->__pyx_float_40_95);
  Py_VISIT(traverse_module_state->__pyx_float_40_97);
  Py_VISIT(traverse_module_state->__pyx_float_41_00);
  Py_VISIT(traverse_module_state->__pyx_float_41_16);
  Py_VISIT(traverse_module_state->__pyx_float_41_20);
  Py_VISIT(traverse_module_state->__pyx_float_41_23);
  Py_VISIT(traverse_module_state->__pyx_float_41_28);
  Py_VISIT(traverse_module_state->__pyx_float_41_37);
  Py_VISIT(traverse_module_state->__pyx_float_41_40);
  Py_VISIT(traverse_module_state->__pyx_float_41_43);
  Py_VISIT(traverse_module_state->__pyx_float_41_47);
  Py_VISIT(traverse_module_state->__pyx_float_41_50);
  Py_VISIT(traverse_module_state->__pyx_float_41_51);
  Py_VISIT(traverse_module_state->__pyx_float_41_52);
  Py_VISIT(traverse_module_state->__pyx_float_41_53);
  Py_VISIT(traverse_module_state->__pyx_float_41_61);
  Py_VISIT(traverse_module_state->__pyx_float_41_77);
  Py_VISIT(traverse_module_state->__pyx_float_41_82);
  Py_VISIT(traverse_module_state->__pyx_float_41_86);
  Py_VISIT(traverse_module_state->__pyx_float_41_89);
  Py_VISIT(traverse_module_state->__pyx_float_41_90);
  Py_VISIT(traverse_module_state->__pyx_float_41_93);
  Py_VISIT(traverse_module_state->__pyx_float_41_97);
  Py_VISIT(traverse_module_state->__pyx_float_42_09);
  Py_VISIT(traverse_module_state->__pyx_float_42_23);
  Py_VISIT(traverse_module_state->__pyx_float_42_37);
  Py_VISIT(traverse_module_state->__pyx_float_42_42);
  Py_VISIT(traverse_module_state->__pyx_float_42_43);
  Py_VISIT(traverse_module_state->__pyx_float_42_50);
  Py_VISIT(traverse_module_state->__pyx_float_42_53);
  Py_VISIT(traverse_module_state->__pyx_float_42_66);
  Py_VISIT(traverse_module_state->__pyx_float_42_67);
  Py_VISIT(traverse_module_state->__pyx_float_42_80);
  Py_VISIT(traverse_module_state->__pyx_float_42_92);
  Py_VISIT(traverse_module_state->__pyx_float_42_95);
  Py_VISIT(traverse_module_state->__pyx_float_43_04);
  Py_VISIT(traverse_module_state->__pyx_float_43_06);
  Py_VISIT(traverse_module_state->__pyx_float_43_08);
  Py_VISIT(traverse_module_state->__pyx_float_43_24);
  Py_VISIT(traverse_module_state->__pyx_float_43_25);
  Py_VISIT(traverse_module_state->__pyx_float_43_27);
  Py_VISIT(traverse_module_state->__pyx_float_43_35);
  Py_VISIT(traverse_module_state->__pyx_float_43_38);
  Py_VISIT(traverse_module_state->__pyx_float_43_40);
  Py_VISIT(traverse_module_state->__pyx_float_43_41);
  Py_VISIT(traverse_module_state->__pyx_float_43_46);
  Py_VISIT(traverse_module_state->__pyx_float_43_48);
  Py_VISIT(traverse_module_state->__pyx_float_43_49);
  Py_VISIT(traverse_module_state->__pyx_float_43_54);
  Py_VISIT(traverse_module_state->__pyx_float_43_60);
  Py_VISIT(traverse_module_state->__pyx_float_43_65);
  Py_VISIT(traverse_module_state->__pyx_float_43_74);
  Py_VISIT(traverse_module_state->__pyx_float_43_75);
  Py_VISIT(traverse_module_state->__pyx_float_43_87);
  Py_VISIT(traverse_module_state->__pyx_float_43_89);
  Py_VISIT(traverse_module_state->__pyx_float_43_93);
  Py_VISIT(traverse_module_state->__pyx_float_44_02);
  Py_VISIT(traverse_module_state->__pyx_float_44_03);
  Py_VISIT(traverse_module_state->__pyx_float_44_10);
  Py_VISIT(traverse_module_state->__pyx_float_44_14);
  Py_VISIT(traverse_module_state->__pyx_float_44_17);
  Py_VISIT(traverse_module_state->__pyx_float_44_21);
  Py_VISIT(traverse_module_state->__pyx_float_44_24);
  Py_VISIT(traverse_module_state->__pyx_float_44_31);
  Py_VISIT(traverse_module_state->__pyx_float_44_48);
  Py_VISIT(traverse_module_state->__pyx_float_44_51);
  Py_VISIT(traverse_module_state->__pyx_float_44_52);
  Py_VISIT(traverse_module_state->__pyx_float_44_53);
  Py_VISIT(traverse_module_state->__pyx_float_44_68);
  Py_VISIT(traverse_module_state->__pyx_float_44_73);
  Py_VISIT(traverse_module_state->__pyx_float_44_74);
  Py_VISIT(traverse_module_state->__pyx_float_44_94);
  Py_VISIT(traverse_module_state->__pyx_float_45_03);
  Py_VISIT(traverse_module_state->__pyx_float_45_05);
  Py_VISIT(traverse_module_state->__pyx_float_45_12);
  Py_VISIT(traverse_module_state->__pyx_float_45_13);
  Py_VISIT(traverse_module_state->__pyx_float_45_24);
  Py_VISIT(traverse_module_state->__pyx_float_45_25);
  Py_VISIT(traverse_module_state->__pyx_float_45_29);
  Py_VISIT(traverse_module_state->__pyx_float_45_36);
  Py_VISIT(traverse_module_state->__pyx_float_45_38);
  Py_VISIT(traverse_module_state->__pyx_float_45_42);
  Py_VISIT(traverse_module_state->__pyx_float_45_49);
  Py_VISIT(traverse_module_state->__pyx_float_45_62);
  Py_VISIT(traverse_module_state->__pyx_float_45_65);
  Py_VISIT(traverse_module_state->__pyx_float_45_68);
  Py_VISIT(traverse_module_state->__pyx_float_45_85);
  Py_VISIT(traverse_module_state->__pyx_float_45_95);
  Py_VISIT(traverse_module_state->__pyx_float_45_99);
  Py_VISIT(traverse_module_state->__pyx_float_46_02);
  Py_VISIT(traverse_module_state->__pyx_float_46_11);
  Py_VISIT(traverse_module_state->__pyx_float_46_13);
  Py_VISIT(traverse_module_state->__pyx_float_46_18);
  Py_VISIT(traverse_module_state->__pyx_float_46_20);
  Py_VISIT(traverse_module_state->__pyx_float_46_33);
  Py_VISIT(traverse_module_state->__pyx_float_46_35);
  Py_VISIT(traverse_module_state->__pyx_float_46_36);
  Py_VISIT(traverse_module_state->__pyx_float_46_42);
  Py_VISIT(traverse_module_state->__pyx_float_46_43);
  Py_VISIT(traverse_module_state->__pyx_float_46_54);
  Py_VISIT(traverse_module_state->__pyx_float_46_56);
  Py_VISIT(traverse_module_state->__pyx_float_46_57);
  Py_VISIT(traverse_module_state->__pyx_float_46_62);
  Py_VISIT(traverse_module_state->__pyx_float_46_65);
  Py_VISIT(traverse_module_state->__pyx_float_46_68);
  Py_VISIT(traverse_module_state->__pyx_float_46_86);
  Py_VISIT(traverse_module_state->__pyx_float_46_93);
  Py_VISIT(traverse_module_state->__pyx_float_47_00);
  Py_VISIT(traverse_module_state->__pyx_float_47_07);
  Py_VISIT(traverse_module_state->__pyx_float_47_11);
  Py_VISIT(traverse_module_state->__pyx_float_47_16);
  Py_VISIT(traverse_module_state->__pyx_float_47_20);
  Py_VISIT(traverse_module_state->__pyx_float_47_21);
  Py_VISIT(traverse_module_state->__pyx_float_47_25);
  Py_VISIT(traverse_module_state->__pyx_float_47_40);
  Py_VISIT(traverse_module_state->__pyx_float_47_41);
  Py_VISIT(traverse_module_state->__pyx_float_47_46);
  Py_VISIT(traverse_module_state->__pyx_float_47_50);
  Py_VISIT(traverse_module_state->__pyx_float_47_60);
  Py_VISIT(traverse_module_state->__pyx_float_47_64);
  Py_VISIT(traverse_module_state->__pyx_float_47_65);
  Py_VISIT(traverse_module_state->__pyx_float_47_72);
  Py_VISIT(traverse_module_state->__pyx_float_47_78);
  Py_VISIT(traverse_module_state->__pyx_float_47_79);
  Py_VISIT(traverse_module_state->__pyx_float_47_81);
  Py_VISIT(traverse_module_state->__pyx_float_47_82);
  Py_VISIT(traverse_module_state->__pyx_float_47_87);
  Py_VISIT(traverse_module_state->__pyx_float_48_07);
  Py_VISIT(traverse_module_state->__pyx_float_48_11);
  Py_VISIT(traverse_module_state->__pyx_float_48_28);
  Py_VISIT(traverse_module_state->__pyx_float_48_36);
  Py_VISIT(traverse_module_state->__pyx_float_48_40);
  Py_VISIT(traverse_module_state->__pyx_float_48_42);
  Py_VISIT(traverse_module_state->__pyx_float_48_43);
  Py_VISIT(traverse_module_state->__pyx_float_48_44);
  Py_VISIT(traverse_module_state->__pyx_float_48_52);
  Py_VISIT(traverse_module_state->__pyx_float_48_57);
  Py_VISIT(traverse_module_state->__pyx_float_48_65);
  Py_VISIT(traverse_module_state->__pyx_float_48_68);
  Py_VISIT(traverse_module_state->__pyx_float_48_71);
  Py_VISIT(traverse_module_state->__pyx_float_48_76);
  Py_VISIT(traverse_module_state->__pyx_float_48_79);
  Py_VISIT(traverse_module_state->__pyx_float_48_83);
  Py_VISIT(traverse_module_state->__pyx_float_48_87);
  Py_VISIT(traverse_module_state->__pyx_float_48_88);
  Py_VISIT(traverse_module_state->__pyx_float_48_89);
  Py_VISIT(traverse_module_state->__pyx_float_48_95);
  Py_VISIT(traverse_module_state->__pyx_float_48_98);
  Py_VISIT(traverse_module_state->__pyx_float_48_99);
  Py_VISIT(traverse_module_state->__pyx_float_49_01);
  Py_VISIT(traverse_module_state->__pyx_float_49_14);
  Py_VISIT(traverse_module_state->__pyx_float_49_27);
  Py_VISIT(traverse_module_state->__pyx_float_49_29);
  Py_VISIT(traverse_module_state->__pyx_float_49_44);
  Py_VISIT(traverse_module_state->__pyx_float_49_48);
  Py_VISIT(traverse_module_state->__pyx_float_49_55);
  Py_VISIT(traverse_module_state->__pyx_float_49_65);
  Py_VISIT(traverse_module_state->__pyx_float_49_67);
  Py_VISIT(traverse_module_state->__pyx_float_49_68);
  Py_VISIT(traverse_module_state->__pyx_float_49_87);
  Py_VISIT(traverse_module_state->__pyx_float_49_93);
  Py_VISIT(traverse_module_state->__pyx_float_49_95);
  Py_VISIT(traverse_module_state->__pyx_float_50_01);
  Py_VISIT(traverse_module_state->__pyx_float_50_03);
  Py_VISIT(traverse_module_state->__pyx_float_50_10);
  Py_VISIT(traverse_module_state->__pyx_float_50_11);
  Py_VISIT(traverse_module_state->__pyx_float_50_13);
  Py_VISIT(traverse_module_state->__pyx_float_50_18);
  Py_VISIT(traverse_module_state->__pyx_float_50_21);
  Py_VISIT(traverse_module_state->__pyx_float_50_48);
  Py_VISIT(traverse_module_state->__pyx_float_50_49);
  Py_VISIT(traverse_module_state->__pyx_float_50_63);
  Py_VISIT(traverse_module_state->__pyx_float_50_72);
  Py_VISIT(traverse_module_state->__pyx_float_50_76);
  Py_VISIT(traverse_module_state->__pyx_float_50_83);
  Py_VISIT(traverse_module_state->__pyx_float_50_88);
  Py_VISIT(traverse_module_state->__pyx_float_50_92);
  Py_VISIT(traverse_module_state->__pyx_float_50_96);
  Py_VISIT(traverse_module_state->__pyx_float_51_09);
  Py_VISIT(traverse_module_state->__pyx_float_51_12);
  Py_VISIT(traverse_module_state->__pyx_float_51_19);
  Py_VISIT(traverse_module_state->__pyx_float_51_27);
  Py_VISIT(traverse_module_state->__pyx_float_51_29);
  Py_VISIT(traverse_module_state->__pyx_float_51_34);
  Py_VISIT(traverse_module_state->__pyx_float_51_37);
  Py_VISIT(traverse_module_state->__pyx_float_51_42);
  Py_VISIT(traverse_module_state->__pyx_float_51_54);
  Py_VISIT(traverse_module_state->__pyx_float_51_62);
  Py_VISIT(traverse_module_state->__pyx_float_51_81);
  Py_VISIT(traverse_module_state->__pyx_float_51_93);
  Py_VISIT(traverse_module_state->__pyx_float_52_00);
  Py_VISIT(traverse_module_state->__pyx_float_52_15);
  Py_VISIT(traverse_module_state->__pyx_float_52_17);
  Py_VISIT(traverse_module_state->__pyx_float_52_33);
  Py_VISIT(traverse_module_state->__pyx_float_52_43);
  Py_VISIT(traverse_module_state->__pyx_float_52_47);
  Py_VISIT(traverse_module_state->__pyx_float_52_70);
  Py_VISIT(traverse_module_state->__pyx_float_52_73);
  Py_VISIT(traverse_module_state->__pyx_float_52_95);
  Py_VISIT(traverse_module_state->__pyx_float_53_42);
  Py_VISIT(traverse_module_state->__pyx_float_53_46);
  Py_VISIT(traverse_module_state->__pyx_float_53_52);
  Py_VISIT(traverse_module_state->__pyx_float_53_56);
  Py_VISIT(traverse_module_state->__pyx_float_53_68);
  Py_VISIT(traverse_module_state->__pyx_float_53_79);
  Py_VISIT(traverse_module_state->__pyx_float_53_82);
  Py_VISIT(traverse_module_state->__pyx_float_53_90);
  Py_VISIT(traverse_module_state->__pyx_float_53_91);
  Py_VISIT(traverse_module_state->__pyx_float_54_00);
  Py_VISIT(traverse_module_state->__pyx_float_54_05);
  Py_VISIT(traverse_module_state->__pyx_float_54_15);
  Py_VISIT(traverse_module_state->__pyx_float_54_23);
  Py_VISIT(traverse_module_state->__pyx_float_54_39);
  Py_VISIT(traverse_module_state->__pyx_float_54_69);
  Py_VISIT(traverse_module_state->__pyx_float_54_74);
  Py_VISIT(traverse_module_state->__pyx_float_54_75);
  Py_VISIT(traverse_module_state->__pyx_float_54_76);
  Py_VISIT(traverse_module_state->__pyx_float_55_02);
  Py_VISIT(traverse_module_state->__pyx_float_55_06);
  Py_VISIT(traverse_module_state->__pyx_float_55_09);
  Py_VISIT(traverse_module_state->__pyx_float_55_14);
  Py_VISIT(traverse_module_state->__pyx_float_55_24);
  Py_VISIT(traverse_module_state->__pyx_float_55_30);
  Py_VISIT(traverse_module_state->__pyx_float_55_31);
  Py_VISIT(traverse_module_state->__pyx_float_55_34);
  Py_VISIT(traverse_module_state->__pyx_float_55_45);
  Py_VISIT(traverse_module_state->__pyx_float_55_50);
  Py_VISIT(traverse_module_state->__pyx_float_55_58);
  Py_VISIT(traverse_module_state->__pyx_float_55_75);
  Py_VISIT(traverse_module_state->__pyx_float_55_88);
  Py_VISIT(traverse_module_state->__pyx_float_55_90);
  Py_VISIT(traverse_module_state->__pyx_float_55_98);
  Py_VISIT(traverse_module_state->__pyx_float_56_04);
  Py_VISIT(traverse_module_state->__pyx_float_56_06);
  Py_VISIT(traverse_module_state->__pyx_float_56_17);
  Py_VISIT(traverse_module_state->__pyx_float_56_18);
  Py_VISIT(traverse_module_state->__pyx_float_56_20);
  Py_VISIT(traverse_module_state->__pyx_float_56_23);
  Py_VISIT(traverse_module_state->__pyx_float_56_31);
  Py_VISIT(traverse_module_state->__pyx_float_56_60);
  Py_VISIT(traverse_module_state->__pyx_float_56_62);
  Py_VISIT(traverse_module_state->__pyx_float_56_68);
  Py_VISIT(traverse_module_state->__pyx_float_56_73);
  Py_VISIT(traverse_module_state->__pyx_float_56_90);
  Py_VISIT(traverse_module_state->__pyx_float_56_98);
  Py_VISIT(traverse_module_state->__pyx_float_57_02);
  Py_VISIT(traverse_module_state->__pyx_float_57_10);
  Py_VISIT(traverse_module_state->__pyx_float_57_21);
  Py_VISIT(traverse_module_state->__pyx_float_57_26);
  Py_VISIT(traverse_module_state->__pyx_float_57_34);
  Py_VISIT(traverse_module_state->__pyx_float_57_40);
  Py_VISIT(traverse_module_state->__pyx_float_57_41);
  Py_VISIT(traverse_module_state->__pyx_float_57_53);
  Py_VISIT(traverse_module_state->__pyx_float_57_63);
  Py_VISIT(traverse_module_state->__pyx_float_57_69);
  Py_VISIT(traverse_module_state->__pyx_float_57_81);
  Py_VISIT(traverse_module_state->__pyx_float_57_90);
  Py_VISIT(traverse_module_state->__pyx_float_57_96);
  Py_VISIT(traverse_module_state->__pyx_float_57_97);
  Py_VISIT(traverse_module_state->__pyx_float_57_98);
  Py_VISIT(traverse_module_state->__pyx_float_58_03);
  Py_VISIT(traverse_module_state->__pyx_float_58_11);
  Py_VISIT(traverse_module_state->__pyx_float_58_17);
  Py_VISIT(traverse_module_state->__pyx_float_58_28);
  Py_VISIT(traverse_module_state->__pyx_float_58_34);
  Py_VISIT(traverse_module_state->__pyx_float_58_42);
  Py_VISIT(traverse_module_state->__pyx_float_58_53);
  Py_VISIT(traverse_module_state->__pyx_float_58_56);
  Py_VISIT(traverse_module_state->__pyx_float_58_67);
  Py_VISIT(traverse_module_state->__pyx_float_58_71);
  Py_VISIT(traverse_module_state->__pyx_float_58_75);
  Py_VISIT(traverse_module_state->__pyx_float_58_79);
  Py_VISIT(traverse_module_state->__pyx_float_58_86);
  Py_VISIT(traverse_module_state->__pyx_float_59_02);
  Py_VISIT(traverse_module_state->__pyx_float_59_13);
  Py_VISIT(traverse_module_state->__pyx_float_59_16);
  Py_VISIT(traverse_module_state->__pyx_float_59_21);
  Py_VISIT(traverse_module_state->__pyx_float_59_29);
  Py_VISIT(traverse_module_state->__pyx_float_59_30);
  Py_VISIT(traverse_module_state->__pyx_float_59_33);
  Py_VISIT(traverse_module_state->__pyx_float_59_40);
  Py_VISIT(traverse_module_state->__pyx_float_59_47);
  Py_VISIT(traverse_module_state->__pyx_float_59_70);
  Py_VISIT(traverse_module_state->__pyx_float_59_81);
  Py_VISIT(traverse_module_state->__pyx_float_59_83);
  Py_VISIT(traverse_module_state->__pyx_float_59_86);
  Py_VISIT(traverse_module_state->__pyx_float_59_95);
  Py_VISIT(traverse_module_state->__pyx_float_59_96);
  Py_VISIT(traverse_module_state->__pyx_float_60_00);
  Py_VISIT(traverse_module_state->__pyx_float_60_03);
  Py_VISIT(traverse_module_state->__pyx_float_60_09);
  Py_VISIT(traverse_module_state->__pyx_float_60_23);
  Py_VISIT(traverse_module_state->__pyx_float_60_33);
  Py_VISIT(traverse_module_state->__pyx_float_60_35);
  Py_VISIT(traverse_module_state->__pyx_float_60_42);
  Py_VISIT(traverse_module_state->__pyx_float_60_59);
  Py_VISIT(traverse_module_state->__pyx_float_60_62);
  Py_VISIT(traverse_module_state->__pyx_float_60_67);
  Py_VISIT(traverse_module_state->__pyx_float_60_75);
  Py_VISIT(traverse_module_state->__pyx_float_61_05);
  Py_VISIT(traverse_module_state->__pyx_float_61_13);
  Py_VISIT(traverse_module_state->__pyx_float_61_21);
  Py_VISIT(traverse_module_state->__pyx_float_61_29);
  Py_VISIT(traverse_module_state->__pyx_float_61_34);
  Py_VISIT(traverse_module_state->__pyx_float_61_37);
  Py_VISIT(traverse_module_state->__pyx_float_61_39);
  Py_VISIT(traverse_module_state->__pyx_float_61_40);
  Py_VISIT(traverse_module_state->__pyx_float_61_56);
  Py_VISIT(traverse_module_state->__pyx_float_61_59);
  Py_VISIT(traverse_module_state->__pyx_float_61_62);
  Py_VISIT(traverse_module_state->__pyx_float_61_66);
  Py_VISIT(traverse_module_state->__pyx_float_61_99);
  Py_VISIT(traverse_module_state->__pyx_float_62_04);
  Py_VISIT(traverse_module_state->__pyx_float_62_20);
  Py_VISIT(traverse_module_state->__pyx_float_62_22);
  Py_VISIT(traverse_module_state->__pyx_float_62_54);
  Py_VISIT(traverse_module_state->__pyx_float_62_56);
  Py_VISIT(traverse_module_state->__pyx_float_62_72);
  Py_VISIT(traverse_module_state->__pyx_float_62_84);
  Py_VISIT(traverse_module_state->__pyx_float_62_86);
  Py_VISIT(traverse_module_state->__pyx_float_63_08);
  Py_VISIT(traverse_module_state->__pyx_float_63_48);
  Py_VISIT(traverse_module_state->__pyx_float_63_55);
  Py_VISIT(traverse_module_state->__pyx_float_63_78);
  Py_VISIT(traverse_module_state->__pyx_float_63_97);
  Py_VISIT(traverse_module_state->__pyx_float_64_29);
  Py_VISIT(traverse_module_state->__pyx_float_64_36);
  Py_VISIT(traverse_module_state->__pyx_float_64_82);
  Py_VISIT(traverse_module_state->__pyx_float_64_83);
  Py_VISIT(traverse_module_state->__pyx_float_64_86);
  Py_VISIT(traverse_module_state->__pyx_float_64_97);
  Py_VISIT(traverse_module_state->__pyx_float_65_09);
  Py_VISIT(traverse_module_state->__pyx_float_65_19);
  Py_VISIT(traverse_module_state->__pyx_float_65_20);
  Py_VISIT(traverse_module_state->__pyx_float_65_42);
  Py_VISIT(traverse_module_state->__pyx_float_65_54);
  Py_VISIT(traverse_module_state->__pyx_float_65_65);
  Py_VISIT(traverse_module_state->__pyx_float_65_89);
  Py_VISIT(traverse_module_state->__pyx_float_66_12);
  Py_VISIT(traverse_module_state->__pyx_float_66_24);
  Py_VISIT(traverse_module_state->__pyx_float_66_26);
  Py_VISIT(traverse_module_state->__pyx_float_66_34);
  Py_VISIT(traverse_module_state->__pyx_float_66_44);
  Py_VISIT(traverse_module_state->__pyx_float_66_47);
  Py_VISIT(traverse_module_state->__pyx_float_66_48);
  Py_VISIT(traverse_module_state->__pyx_float_66_56);
  Py_VISIT(traverse_module_state->__pyx_float_66_60);
  Py_VISIT(traverse_module_state->__pyx_float_66_64);
  Py_VISIT(traverse_module_state->__pyx_float_66_72);
  Py_VISIT(traverse_module_state->__pyx_float_66_91);
  Py_VISIT(traverse_module_state->__pyx_float_66_92);
  Py_VISIT(traverse_module_state->__pyx_float_66_96);
  Py_VISIT(traverse_module_state->__pyx_float_67_18);
  Py_VISIT(traverse_module_state->__pyx_float_67_23);
  Py_VISIT(traverse_module_state->__pyx_float_67_37);
  Py_VISIT(traverse_module_state->__pyx_float_67_46);
  Py_VISIT(traverse_module_state->__pyx_float_67_63);
  Py_VISIT(traverse_module_state->__pyx_float_67_70);
  Py_VISIT(traverse_module_state->__pyx_float_67_83);
  Py_VISIT(traverse_module_state->__pyx_float_67_94);
  Py_VISIT(traverse_module_state->__pyx_float_67_97);
  Py_VISIT(traverse_module_state->__pyx_float_67_98);
  Py_VISIT(traverse_module_state->__pyx_float_68_18);
  Py_VISIT(traverse_module_state->__pyx_float_68_50);
  Py_VISIT(traverse_module_state->__pyx_float_68_61);
  Py_VISIT(traverse_module_state->__pyx_float_68_65);
  Py_VISIT(traverse_module_state->__pyx_float_68_81);
  Py_VISIT(traverse_module_state->__pyx_float_68_85);
  Py_VISIT(traverse_module_state->__pyx_float_68_90);
  Py_VISIT(traverse_module_state->__pyx_float_68_94);
  Py_VISIT(traverse_module_state->__pyx_float_68_99);
  Py_VISIT(traverse_module_state->__pyx_float_69_09);
  Py_VISIT(traverse_module_state->__pyx_float_69_13);
  Py_VISIT(traverse_module_state->__pyx_float_69_17);
  Py_VISIT(traverse_module_state->__pyx_float_69_33);
  Py_VISIT(traverse_module_state->__pyx_float_69_37);
  Py_VISIT(traverse_module_state->__pyx_float_69_60);
  Py_VISIT(traverse_module_state->__pyx_float_69_78);
  Py_VISIT(traverse_module_state->__pyx_float_69_91);
  Py_VISIT(traverse_module_state->__pyx_float_69_97);
  Py_VISIT(traverse_module_state->__pyx_float_70_10);
  Py_VISIT(traverse_module_state->__pyx_float_70_25);
  Py_VISIT(traverse_module_state->__pyx_float_70_42);
  Py_VISIT(traverse_module_state->__pyx_float_70_43);
  Py_VISIT(traverse_module_state->__pyx_float_70_52);
  Py_VISIT(traverse_module_state->__pyx_float_70_53);
  Py_VISIT(traverse_module_state->__pyx_float_70_58);
  Py_VISIT(traverse_module_state->__pyx_float_70_68);
  Py_VISIT(traverse_module_state->__pyx_float_70_73);
  Py_VISIT(traverse_module_state->__pyx_float_70_88);
  Py_VISIT(traverse_module_state->__pyx_float_71_17);
  Py_VISIT(traverse_module_state->__pyx_float_71_21);
  Py_VISIT(traverse_module_state->__pyx_float_71_25);
  Py_VISIT(traverse_module_state->__pyx_float_71_36);
  Py_VISIT(traverse_module_state->__pyx_float_71_59);
  Py_VISIT(traverse_module_state->__pyx_float_71_72);
  Py_VISIT(traverse_module_state->__pyx_float_71_84);
  Py_VISIT(traverse_module_state->__pyx_float_72_00);
  Py_VISIT(traverse_module_state->__pyx_float_72_01);
  Py_VISIT(traverse_module_state->__pyx_float_72_14);
  Py_VISIT(traverse_module_state->__pyx_float_72_24);
  Py_VISIT(traverse_module_state->__pyx_float_72_45);
  Py_VISIT(traverse_module_state->__pyx_float_73_05);
  Py_VISIT(traverse_module_state->__pyx_float_73_28);
  Py_VISIT(traverse_module_state->__pyx_float_73_50);
  Py_VISIT(traverse_module_state->__pyx_float_73_53);
  Py_VISIT(traverse_module_state->__pyx_float_73_57);
  Py_VISIT(traverse_module_state->__pyx_float_73_62);
  Py_VISIT(traverse_module_state->__pyx_float_73_68);
  Py_VISIT(traverse_module_state->__pyx_float_74_22);
  Py_VISIT(traverse_module_state->__pyx_float_74_40);
  Py_VISIT(traverse_module_state->__pyx_float_74_95);
  Py_VISIT(traverse_module_state->__pyx_float_75_00);
  Py_VISIT(traverse_module_state->__pyx_float_75_44);
  Py_VISIT(traverse_module_state->__pyx_float_75_49);
  Py_VISIT(traverse_module_state->__pyx_float_75_52);
  Py_VISIT(traverse_module_state->__pyx_float_75_76);
  Py_VISIT(traverse_module_state->__pyx_float_75_92);
  Py_VISIT(traverse_module_state->__pyx_float_76_12);
  Py_VISIT(traverse_module_state->__pyx_float_76_45);
  Py_VISIT(traverse_module_state->__pyx_float_76_52);
  Py_VISIT(traverse_module_state->__pyx_float_76_53);
  Py_VISIT(traverse_module_state->__pyx_float_76_96);
  Py_VISIT(traverse_module_state->__pyx_float_77_03);
  Py_VISIT(traverse_module_state->__pyx_float_77_08);
  Py_VISIT(traverse_module_state->__pyx_float_77_13);
  Py_VISIT(traverse_module_state->__pyx_float_77_93);
  Py_VISIT(traverse_module_state->__pyx_float_77_97);
  Py_VISIT(traverse_module_state->__pyx_float_78_00);
  Py_VISIT(traverse_module_state->__pyx_float_78_02);
  Py_VISIT(traverse_module_state->__pyx_float_78_31);
  Py_VISIT(traverse_module_state->__pyx_float_78_32);
  Py_VISIT(traverse_module_state->__pyx_float_78_45);
  Py_VISIT(traverse_module_state->__pyx_float_78_94);
  Py_VISIT(traverse_module_state->__pyx_float_79_27);
  Py_VISIT(traverse_module_state->__pyx_float_79_38);
  Py_VISIT(traverse_module_state->__pyx_float_79_48);
  Py_VISIT(traverse_module_state->__pyx_float_79_49);
  Py_VISIT(traverse_module_state->__pyx_float_79_57);
  Py_VISIT(traverse_module_state->__pyx_float_79_65);
  Py_VISIT(traverse_module_state->__pyx_float_79_73);
  Py_VISIT(traverse_module_state->__pyx_float_79_83);
  Py_VISIT(traverse_module_state->__pyx_float_80_16);
  Py_VISIT(traverse_module_state->__pyx_float_80_39);
  Py_VISIT(traverse_module_state->__pyx_float_80_44);
  Py_VISIT(traverse_module_state->__pyx_float_80_77);
  Py_VISIT(traverse_module_state->__pyx_float_81_00);
  Py_VISIT(traverse_module_state->__pyx_float_81_16);
  Py_VISIT(traverse_module_state->__pyx_float_81_33);
  Py_VISIT(traverse_module_state->__pyx_float_81_56);
  Py_VISIT(traverse_module_state->__pyx_float_81_69);
  Py_VISIT(traverse_module_state->__pyx_float_81_75);
  Py_VISIT(traverse_module_state->__pyx_float_81_87);
  Py_VISIT(traverse_module_state->__pyx_float_82_06);
  Py_VISIT(traverse_module_state->__pyx_float_82_39);
  Py_VISIT(traverse_module_state->__pyx_float_82_44);
  Py_VISIT(traverse_module_state->__pyx_float_82_47);
  Py_VISIT(traverse_module_state->__pyx_float_82_52);
  Py_VISIT(traverse_module_state->__pyx_float_82_59);
  Py_VISIT(traverse_module_state->__pyx_float_82_70);
  Py_VISIT(traverse_module_state->__pyx_float_82_99);
  Py_VISIT(traverse_module_state->__pyx_float_83_17);
  Py_VISIT(traverse_module_state->__pyx_float_83_53);
  Py_VISIT(traverse_module_state->__pyx_float_83_91);
  Py_VISIT(traverse_module_state->__pyx_float_83_97);
  Py_VISIT(traverse_module_state->__pyx_float_84_01);
  Py_VISIT(traverse_module_state->__pyx_float_84_10);
  Py_VISIT(traverse_module_state->__pyx_float_84_22);
  Py_VISIT(traverse_module_state->__pyx_float_84_61);
  Py_VISIT(traverse_module_state->__pyx_float_84_81);
  Py_VISIT(traverse_module_state->__pyx_float_85_28);
  Py_VISIT(traverse_module_state->__pyx_float_85_41);
  Py_VISIT(traverse_module_state->__pyx_float_85_42);
  Py_VISIT(traverse_module_state->__pyx_float_85_97);
  Py_VISIT(traverse_module_state->__pyx_float_86_88);
  Py_VISIT(traverse_module_state->__pyx_float_87_27);
  Py_VISIT(traverse_module_state->__pyx_float_87_46);
  Py_VISIT(traverse_module_state->__pyx_float_87_56);
  Py_VISIT(traverse_module_state->__pyx_float_87_65);
  Py_VISIT(traverse_module_state->__pyx_float_88_36);
  Py_VISIT(traverse_module_state->__pyx_float_88_39);
  Py_VISIT(traverse_module_state->__pyx_float_89_08);
  Py_VISIT(traverse_module_state->__pyx_float_89_85);
  Py_VISIT(traverse_module_state->__pyx_float_90_29);
  Py_VISIT(traverse_module_state->__pyx_float_90_30);
  Py_VISIT(traverse_module_state->__pyx_float_90_55);
  Py_VISIT(traverse_module_state->__pyx_float_90_70);
  Py_VISIT(traverse_module_state->__pyx_float_91_35);
  Py_VISIT(traverse_module_state->__pyx_float_91_75);
  Py_VISIT(traverse_module_state->__pyx_float_92_09);
  Py_VISIT(traverse_module_state->__pyx_float_92_30);
  Py_VISIT(traverse_module_state->__pyx_float_92_86);
  Py_VISIT(traverse_module_state->__pyx_float_93_28);
  Py_VISIT(traverse_module_state->__pyx_float_93_40);
  Py_VISIT(traverse_module_state->__pyx_float_93_52);
  Py_VISIT(traverse_module_state->__pyx_float_94_02);
  Py_VISIT(traverse_module_state->__pyx_float_95_26);
  Py_VISIT(traverse_module_state->__pyx_float_95_59);
  Py_VISIT(traverse_module_state->__pyx_float_97_07);
  Py_VISIT(traverse_module_state->__pyx_float_97_22);
  Py_VISIT(traverse_module_state->__pyx_float_98_86);
  Py_VISIT(traverse_module_state->__pyx_float_105_28);
  Py_VISIT(traverse_module_state->__pyx_float_106_22);
  Py_VISIT(traverse_module_state->__pyx_float_107_45);
  Py_VISIT(traverse_module_state->__pyx_float_108_37);
  Py_VISIT(traverse_module_state->__pyx_float_108_70);
  Py_VISIT(traverse_module_state->__pyx_float_109_39);
  Py_VISIT(traverse_module_state->__pyx_float_122_08);
  Py_VISIT(traverse_module_state->__pyx_float_125_54);
  Py_VISIT(traverse_module_state->__pyx_float_32_703);
  Py_VISIT(traverse_module_state->__pyx_float_10000_0);
  Py_VISIT(traverse_module_state->__pyx_int_0);
  Py_VISIT(traverse_module_state->__pyx_int_1);
  Py_VISIT(traverse_module_state->__pyx_int_2);
  Py_VISIT(traverse_module_state->__pyx_int_4);
  Py_VISIT(traverse_module_state->__pyx_int_12);
  Py_VISIT(traverse_module_state->__pyx_int_45);
  Py_VISIT(traverse_module_state->__pyx_int_100);
  Py_VISIT(traverse_module_state->__pyx_int_220);
  Py_VISIT(traverse_module_state->__pyx_int_261);
  Py_VISIT(traverse_module_state->__pyx_int_440);
  Py_VISIT(traverse_module_state->__pyx_tuple_);
  Py_VISIT(traverse_module_state->__pyx_slice__3);
  Py_VISIT(traverse_module_state->__pyx_slice__4);
  Py_VISIT(traverse_module_state->__pyx_tuple__2);
  Py_VISIT(traverse_module_state->__pyx_tuple__6);
  Py_VISIT(traverse_module_state->__pyx_tuple__7);
  Py_VISIT(traverse_module_state->__pyx_tuple__9);
  Py_VISIT(traverse_module_state->__pyx_tuple__11);
  Py_VISIT(traverse_module_state->__pyx_tuple__13);
  Py_VISIT(traverse_module_state->__pyx_tuple__14);
  Py_VISIT(traverse_module_state->__pyx_codeobj__8);
  Py_VISIT(traverse_module_state->__pyx_codeobj__10);
  Py_VISIT(traverse_module_state->__pyx_codeobj__12);
  return 0;
}
#endif
/* #### Code section: module_state_defines ### */
#define __pyx_d __pyx_mstate_global->__pyx_d
#define __pyx_b __pyx_mstate_global->__pyx_b
#define __pyx_cython_runtime __pyx_mstate_global->__pyx_cython_runtime
#define __pyx_empty_tuple __pyx_mstate_global->__pyx_empty_tuple
#define __pyx_empty_bytes __pyx_mstate_global->__pyx_empty_bytes
#define __pyx_empty_unicode __pyx_mstate_global->__pyx_empty_unicode
#ifdef __Pyx_CyFunction_USED
#define __pyx_CyFunctionType __pyx_mstate_global->__pyx_CyFunctionType
#endif
#ifdef __Pyx_FusedFunction_USED
#define __pyx_FusedFunctionType __pyx_mstate_global->__pyx_FusedFunctionType
#endif
#ifdef __Pyx_Generator_USED
#define __pyx_GeneratorType __pyx_mstate_global->__pyx_GeneratorType
#endif
#ifdef __Pyx_IterableCoroutine_USED
#define __pyx_IterableCoroutineType __pyx_mstate_global->__pyx_IterableCoroutineType
#endif
#ifdef __Pyx_Coroutine_USED
#define __pyx_CoroutineAwaitType __pyx_mstate_global->__pyx_CoroutineAwaitType
#endif
#ifdef __Pyx_Coroutine_USED
#define __pyx_CoroutineType __pyx_mstate_global->__pyx_CoroutineType
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#define __pyx_ptype_7cpython_4type_type __pyx_mstate_global->__pyx_ptype_7cpython_4type_type
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#define __pyx_ptype_5numpy_dtype __pyx_mstate_global->__pyx_ptype_5numpy_dtype
#define __pyx_ptype_5numpy_flatiter __pyx_mstate_global->__pyx_ptype_5numpy_flatiter
#define __pyx_ptype_5numpy_broadcast __pyx_mstate_global->__pyx_ptype_5numpy_broadcast
#define __pyx_ptype_5numpy_ndarray __pyx_mstate_global->__pyx_ptype_5numpy_ndarray
#define __pyx_ptype_5numpy_generic __pyx_mstate_global->__pyx_ptype_5numpy_generic
#define __pyx_ptype_5numpy_number __pyx_mstate_global->__pyx_ptype_5numpy_number
#define __pyx_ptype_5numpy_integer __pyx_mstate_global->__pyx_ptype_5numpy_integer
#define __pyx_ptype_5numpy_signedinteger __pyx_mstate_global->__pyx_ptype_5numpy_signedinteger
#define __pyx_ptype_5numpy_unsignedinteger __pyx_mstate_global->__pyx_ptype_5numpy_unsignedinteger
#define __pyx_ptype_5numpy_inexact __pyx_mstate_global->__pyx_ptype_5numpy_inexact
#define __pyx_ptype_5numpy_floating __pyx_mstate_global->__pyx_ptype_5numpy_floating
#define __pyx_ptype_5numpy_complexfloating __pyx_mstate_global->__pyx_ptype_5numpy_complexfloating
#define __pyx_ptype_5numpy_flexible __pyx_mstate_global->__pyx_ptype_5numpy_flexible
#define __pyx_ptype_5numpy_character __pyx_mstate_global->__pyx_ptype_5numpy_character
#define __pyx_ptype_5numpy_ufunc __pyx_mstate_global->__pyx_ptype_5numpy_ufunc
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#define __pyx_ptype_8pysndlib_3clm_mus_any __pyx_mstate_global->__pyx_ptype_8pysndlib_3clm_mus_any
#if CYTHON_USE_MODULE_STATE
#endif
#define __pyx_n_s_ImportError __pyx_mstate_global->__pyx_n_s_ImportError
#define __pyx_n_s_PIANO_SPECTRA __pyx_mstate_global->__pyx_n_s_PIANO_SPECTRA
#define __pyx_n_s_Sound __pyx_mstate_global->__pyx_n_s_Sound
#define __pyx_n_s__15 __pyx_mstate_global->__pyx_n_s__15
#define __pyx_n_s__5 __pyx_mstate_global->__pyx_n_s__5
#define __pyx_n_s_amplitude __pyx_mstate_global->__pyx_n_s_amplitude
#define __pyx_n_s_asyncio_coroutines __pyx_mstate_global->__pyx_n_s_asyncio_coroutines
#define __pyx_n_s_attackTime __pyx_mstate_global->__pyx_n_s_attackTime
#define __pyx_n_s_base __pyx_mstate_global->__pyx_n_s_base
#define __pyx_n_s_class_getitem __pyx_mstate_global->__pyx_n_s_class_getitem
#define __pyx_n_s_cline_in_traceback __pyx_mstate_global->__pyx_n_s_cline_in_traceback
#define __pyx_n_s_clm __pyx_mstate_global->__pyx_n_s_clm
#define __pyx_n_s_db_drop_per_second __pyx_mstate_global->__pyx_n_s_db_drop_per_second
#define __pyx_n_s_degree __pyx_mstate_global->__pyx_n_s_degree
#define __pyx_n_s_distance __pyx_mstate_global->__pyx_n_s_distance
#define __pyx_n_s_double __pyx_mstate_global->__pyx_n_s_double
#define __pyx_n_s_dtype __pyx_mstate_global->__pyx_n_s_dtype
#define __pyx_n_s_dur __pyx_mstate_global->__pyx_n_s_dur
#define __pyx_n_s_duration __pyx_mstate_global->__pyx_n_s_duration
#define __pyx_n_s_enter __pyx_mstate_global->__pyx_n_s_enter
#define __pyx_n_s_exit __pyx_mstate_global->__pyx_n_s_exit
#define __pyx_n_s_freq __pyx_mstate_global->__pyx_n_s_freq
#define __pyx_n_s_frequency __pyx_mstate_global->__pyx_n_s_frequency
#define __pyx_n_s_get_piano_partials __pyx_mstate_global->__pyx_n_s_get_piano_partials
#define __pyx_n_s_import __pyx_mstate_global->__pyx_n_s_import
#define __pyx_n_s_initializing __pyx_mstate_global->__pyx_n_s_initializing
#define __pyx_n_s_is_coroutine __pyx_mstate_global->__pyx_n_s_is_coroutine
#define __pyx_n_s_is_number __pyx_mstate_global->__pyx_n_s_is_number
#define __pyx_n_s_lbj_piano __pyx_mstate_global->__pyx_n_s_lbj_piano
#define __pyx_n_s_log __pyx_mstate_global->__pyx_n_s_log
#define __pyx_n_s_main __pyx_mstate_global->__pyx_n_s_main
#define __pyx_n_u_main __pyx_mstate_global->__pyx_n_u_main
#define __pyx_n_s_make_env __pyx_mstate_global->__pyx_n_s_make_env
#define __pyx_n_s_make_locsig __pyx_mstate_global->__pyx_n_s_make_locsig
#define __pyx_n_s_make_oscil_bank __pyx_mstate_global->__pyx_n_s_make_oscil_bank
#define __pyx_n_s_make_piano_ampfun __pyx_mstate_global->__pyx_n_s_make_piano_ampfun
#define __pyx_n_s_math __pyx_mstate_global->__pyx_n_s_math
#define __pyx_n_s_name __pyx_mstate_global->__pyx_n_s_name
#define __pyx_n_s_np __pyx_mstate_global->__pyx_n_s_np
#define __pyx_n_s_numpy __pyx_mstate_global->__pyx_n_s_numpy
#define __pyx_kp_u_numpy_core_multiarray_failed_to __pyx_mstate_global->__pyx_kp_u_numpy_core_multiarray_failed_to
#define __pyx_kp_u_numpy_core_umath_failed_to_impor __pyx_mstate_global->__pyx_kp_u_numpy_core_umath_failed_to_impor
#define __pyx_n_s_pfreq __pyx_mstate_global->__pyx_n_s_pfreq
#define __pyx_n_s_piano_attack_duration __pyx_mstate_global->__pyx_n_s_piano_attack_duration
#define __pyx_n_s_play __pyx_mstate_global->__pyx_n_s_play
#define __pyx_n_s_pysndlib __pyx_mstate_global->__pyx_n_s_pysndlib
#define __pyx_n_s_pysndlib_clm __pyx_mstate_global->__pyx_n_s_pysndlib_clm
#define __pyx_n_s_pysndlib_instruments_lbj_piano __pyx_mstate_global->__pyx_n_s_pysndlib_instruments_lbj_piano
#define __pyx_n_s_pyx_vtable __pyx_mstate_global->__pyx_n_s_pyx_vtable
#define __pyx_n_s_random __pyx_mstate_global->__pyx_n_s_random
#define __pyx_n_s_range __pyx_mstate_global->__pyx_n_s_range
#define __pyx_n_s_releaseAmp __pyx_mstate_global->__pyx_n_s_releaseAmp
#define __pyx_n_s_reverb_amount __pyx_mstate_global->__pyx_n_s_reverb_amount
#define __pyx_n_s_round __pyx_mstate_global->__pyx_n_s_round
#define __pyx_n_s_scaler __pyx_mstate_global->__pyx_n_s_scaler
#define __pyx_n_s_spec __pyx_mstate_global->__pyx_n_s_spec
#define __pyx_kp_s_src_pysndlib_instruments_lbj_pia __pyx_mstate_global->__pyx_kp_s_src_pysndlib_instruments_lbj_pia
#define __pyx_n_s_stable __pyx_mstate_global->__pyx_n_s_stable
#define __pyx_n_s_startime __pyx_mstate_global->__pyx_n_s_startime
#define __pyx_n_s_statistics __pyx_mstate_global->__pyx_n_s_statistics
#define __pyx_n_s_test __pyx_mstate_global->__pyx_n_s_test
#define __pyx_n_s_zeros __pyx_mstate_global->__pyx_n_s_zeros
#define __pyx_float__4 __pyx_mstate_global->__pyx_float__4
#define __pyx_float__34 __pyx_mstate_global->__pyx_float__34
#define __pyx_float_1_0 __pyx_mstate_global->__pyx_float_1_0
#define __pyx_float_0_97 __pyx_mstate_global->__pyx_float_0_97
#define __pyx_float_0_98 __pyx_mstate_global->__pyx_float_0_98
#define __pyx_float_0_99 __pyx_mstate_global->__pyx_float_0_99
#define __pyx_float_1_00 __pyx_mstate_global->__pyx_float_1_00
#define __pyx_float_1_01 __pyx_mstate_global->__pyx_float_1_01
#define __pyx_float_1_02 __pyx_mstate_global->__pyx_float_1_02
#define __pyx_float_1_97 __pyx_mstate_global->__pyx_float_1_97
#define __pyx_float_1_98 __pyx_mstate_global->__pyx_float_1_98
#define __pyx_float_1_99 __pyx_mstate_global->__pyx_float_1_99
#define __pyx_float_2_00 __pyx_mstate_global->__pyx_float_2_00
#define __pyx_float_2_01 __pyx_mstate_global->__pyx_float_2_01
#define __pyx_float_2_02 __pyx_mstate_global->__pyx_float_2_02
#define __pyx_float_2_03 __pyx_mstate_global->__pyx_float_2_03
#define __pyx_float_2_07 __pyx_mstate_global->__pyx_float_2_07
#define __pyx_float_2_09 __pyx_mstate_global->__pyx_float_2_09
#define __pyx_float_2_96 __pyx_mstate_global->__pyx_float_2_96
#define __pyx_float_2_97 __pyx_mstate_global->__pyx_float_2_97
#define __pyx_float_2_98 __pyx_mstate_global->__pyx_float_2_98
#define __pyx_float_2_99 __pyx_mstate_global->__pyx_float_2_99
#define __pyx_float_3_00 __pyx_mstate_global->__pyx_float_3_00
#define __pyx_float_3_01 __pyx_mstate_global->__pyx_float_3_01
#define __pyx_float_3_02 __pyx_mstate_global->__pyx_float_3_02
#define __pyx_float_3_03 __pyx_mstate_global->__pyx_float_3_03
#define __pyx_float_3_04 __pyx_mstate_global->__pyx_float_3_04
#define __pyx_float_3_09 __pyx_mstate_global->__pyx_float_3_09
#define __pyx_float_3_11 __pyx_mstate_global->__pyx_float_3_11
#define __pyx_float_3_12 __pyx_mstate_global->__pyx_float_3_12
#define __pyx_float_3_15 __pyx_mstate_global->__pyx_float_3_15
#define __pyx_float_3_19 __pyx_mstate_global->__pyx_float_3_19
#define __pyx_float_3_95 __pyx_mstate_global->__pyx_float_3_95
#define __pyx_float_3_97 __pyx_mstate_global->__pyx_float_3_97
#define __pyx_float_3_98 __pyx_mstate_global->__pyx_float_3_98
#define __pyx_float_3_99 __pyx_mstate_global->__pyx_float_3_99
#define __pyx_float_4_00 __pyx_mstate_global->__pyx_float_4_00
#define __pyx_float_4_01 __pyx_mstate_global->__pyx_float_4_01
#define __pyx_float_4_02 __pyx_mstate_global->__pyx_float_4_02
#define __pyx_float_4_03 __pyx_mstate_global->__pyx_float_4_03
#define __pyx_float_4_04 __pyx_mstate_global->__pyx_float_4_04
#define __pyx_float_4_05 __pyx_mstate_global->__pyx_float_4_05
#define __pyx_float_4_06 __pyx_mstate_global->__pyx_float_4_06
#define __pyx_float_4_10 __pyx_mstate_global->__pyx_float_4_10
#define __pyx_float_4_12 __pyx_mstate_global->__pyx_float_4_12
#define __pyx_float_4_13 __pyx_mstate_global->__pyx_float_4_13
#define __pyx_float_4_17 __pyx_mstate_global->__pyx_float_4_17
#define __pyx_float_4_19 __pyx_mstate_global->__pyx_float_4_19
#define __pyx_float_4_26 __pyx_mstate_global->__pyx_float_4_26
#define __pyx_float_4_94 __pyx_mstate_global->__pyx_float_4_94
#define __pyx_float_4_95 __pyx_mstate_global->__pyx_float_4_95
#define __pyx_float_4_96 __pyx_mstate_global->__pyx_float_4_96
#define __pyx_float_4_97 __pyx_mstate_global->__pyx_float_4_97
#define __pyx_float_4_98 __pyx_mstate_global->__pyx_float_4_98
#define __pyx_float_4_99 __pyx_mstate_global->__pyx_float_4_99
#define __pyx_float_5_00 __pyx_mstate_global->__pyx_float_5_00
#define __pyx_float_5_01 __pyx_mstate_global->__pyx_float_5_01
#define __pyx_float_5_02 __pyx_mstate_global->__pyx_float_5_02
#define __pyx_float_5_03 __pyx_mstate_global->__pyx_float_5_03
#define __pyx_float_5_04 __pyx_mstate_global->__pyx_float_5_04
#define __pyx_float_5_05 __pyx_mstate_global->__pyx_float_5_05
#define __pyx_float_5_07 __pyx_mstate_global->__pyx_float_5_07
#define __pyx_float_5_09 __pyx_mstate_global->__pyx_float_5_09
#define __pyx_float_5_10 __pyx_mstate_global->__pyx_float_5_10
#define __pyx_float_5_11 __pyx_mstate_global->__pyx_float_5_11
#define __pyx_float_5_12 __pyx_mstate_global->__pyx_float_5_12
#define __pyx_float_5_14 __pyx_mstate_global->__pyx_float_5_14
#define __pyx_float_5_15 __pyx_mstate_global->__pyx_float_5_15
#define __pyx_float_5_19 __pyx_mstate_global->__pyx_float_5_19
#define __pyx_float_5_20 __pyx_mstate_global->__pyx_float_5_20
#define __pyx_float_5_22 __pyx_mstate_global->__pyx_float_5_22
#define __pyx_float_5_95 __pyx_mstate_global->__pyx_float_5_95
#define __pyx_float_5_96 __pyx_mstate_global->__pyx_float_5_96
#define __pyx_float_5_97 __pyx_mstate_global->__pyx_float_5_97
#define __pyx_float_5_98 __pyx_mstate_global->__pyx_float_5_98
#define __pyx_float_5_99 __pyx_mstate_global->__pyx_float_5_99
#define __pyx_float_6_00 __pyx_mstate_global->__pyx_float_6_00
#define __pyx_float_6_01 __pyx_mstate_global->__pyx_float_6_01
#define __pyx_float_6_02 __pyx_mstate_global->__pyx_float_6_02
#define __pyx_float_6_03 __pyx_mstate_global->__pyx_float_6_03
#define __pyx_float_6_04 __pyx_mstate_global->__pyx_float_6_04
#define __pyx_float_6_06 __pyx_mstate_global->__pyx_float_6_06
#define __pyx_float_6_07 __pyx_mstate_global->__pyx_float_6_07
#define __pyx_float_6_08 __pyx_mstate_global->__pyx_float_6_08
#define __pyx_float_6_09 __pyx_mstate_global->__pyx_float_6_09
#define __pyx_float_6_12 __pyx_mstate_global->__pyx_float_6_12
#define __pyx_float_6_13 __pyx_mstate_global->__pyx_float_6_13
#define __pyx_float_6_14 __pyx_mstate_global->__pyx_float_6_14
#define __pyx_float_6_16 __pyx_mstate_global->__pyx_float_6_16
#define __pyx_float_6_17 __pyx_mstate_global->__pyx_float_6_17
#define __pyx_float_6_20 __pyx_mstate_global->__pyx_float_6_20
#define __pyx_float_6_22 __pyx_mstate_global->__pyx_float_6_22
#define __pyx_float_6_26 __pyx_mstate_global->__pyx_float_6_26
#define __pyx_float_6_27 __pyx_mstate_global->__pyx_float_6_27
#define __pyx_float_6_32 __pyx_mstate_global->__pyx_float_6_32
#define __pyx_float_6_35 __pyx_mstate_global->__pyx_float_6_35
#define __pyx_float_6_94 __pyx_mstate_global->__pyx_float_6_94
#define __pyx_float_6_95 __pyx_mstate_global->__pyx_float_6_95
#define __pyx_float_6_96 __pyx_mstate_global->__pyx_float_6_96
#define __pyx_float_6_97 __pyx_mstate_global->__pyx_float_6_97
#define __pyx_float_6_98 __pyx_mstate_global->__pyx_float_6_98
#define __pyx_float_6_99 __pyx_mstate_global->__pyx_float_6_99
#define __pyx_float_7_00 __pyx_mstate_global->__pyx_float_7_00
#define __pyx_float_7_01 __pyx_mstate_global->__pyx_float_7_01
#define __pyx_float_7_03 __pyx_mstate_global->__pyx_float_7_03
#define __pyx_float_7_04 __pyx_mstate_global->__pyx_float_7_04
#define __pyx_float_7_05 __pyx_mstate_global->__pyx_float_7_05
#define __pyx_float_7_06 __pyx_mstate_global->__pyx_float_7_06
#define __pyx_float_7_07 __pyx_mstate_global->__pyx_float_7_07
#define __pyx_float_7_09 __pyx_mstate_global->__pyx_float_7_09
#define __pyx_float_7_11 __pyx_mstate_global->__pyx_float_7_11
#define __pyx_float_7_12 __pyx_mstate_global->__pyx_float_7_12
#define __pyx_float_7_13 __pyx_mstate_global->__pyx_float_7_13
#define __pyx_float_7_15 __pyx_mstate_global->__pyx_float_7_15
#define __pyx_float_7_19 __pyx_mstate_global->__pyx_float_7_19
#define __pyx_float_7_22 __pyx_mstate_global->__pyx_float_7_22
#define __pyx_float_7_23 __pyx_mstate_global->__pyx_float_7_23
#define __pyx_float_7_25 __pyx_mstate_global->__pyx_float_7_25
#define __pyx_float_7_26 __pyx_mstate_global->__pyx_float_7_26
#define __pyx_float_7_27 __pyx_mstate_global->__pyx_float_7_27
#define __pyx_float_7_32 __pyx_mstate_global->__pyx_float_7_32
#define __pyx_float_7_33 __pyx_mstate_global->__pyx_float_7_33
#define __pyx_float_7_35 __pyx_mstate_global->__pyx_float_7_35
#define __pyx_float_7_41 __pyx_mstate_global->__pyx_float_7_41
#define __pyx_float_7_59 __pyx_mstate_global->__pyx_float_7_59
#define __pyx_float_7_93 __pyx_mstate_global->__pyx_float_7_93
#define __pyx_float_7_94 __pyx_mstate_global->__pyx_float_7_94
#define __pyx_float_7_96 __pyx_mstate_global->__pyx_float_7_96
#define __pyx_float_7_97 __pyx_mstate_global->__pyx_float_7_97
#define __pyx_float_7_98 __pyx_mstate_global->__pyx_float_7_98
#define __pyx_float_7_99 __pyx_mstate_global->__pyx_float_7_99
#define __pyx_float_8_00 __pyx_mstate_global->__pyx_float_8_00
#define __pyx_float_8_01 __pyx_mstate_global->__pyx_float_8_01
#define __pyx_float_8_02 __pyx_mstate_global->__pyx_float_8_02
#define __pyx_float_8_04 __pyx_mstate_global->__pyx_float_8_04
#define __pyx_float_8_05 __pyx_mstate_global->__pyx_float_8_05
#define __pyx_float_8_06 __pyx_mstate_global->__pyx_float_8_06
#define __pyx_float_8_08 __pyx_mstate_global->__pyx_float_8_08
#define __pyx_float_8_09 __pyx_mstate_global->__pyx_float_8_09
#define __pyx_float_8_11 __pyx_mstate_global->__pyx_float_8_11
#define __pyx_float_8_12 __pyx_mstate_global->__pyx_float_8_12
#define __pyx_float_8_14 __pyx_mstate_global->__pyx_float_8_14
#define __pyx_float_8_17 __pyx_mstate_global->__pyx_float_8_17
#define __pyx_float_8_19 __pyx_mstate_global->__pyx_float_8_19
#define __pyx_float_8_20 __pyx_mstate_global->__pyx_float_8_20
#define __pyx_float_8_22 __pyx_mstate_global->__pyx_float_8_22
#define __pyx_float_8_29 __pyx_mstate_global->__pyx_float_8_29
#define __pyx_float_8_30 __pyx_mstate_global->__pyx_float_8_30
#define __pyx_float_8_32 __pyx_mstate_global->__pyx_float_8_32
#define __pyx_float_8_33 __pyx_mstate_global->__pyx_float_8_33
#define __pyx_float_8_36 __pyx_mstate_global->__pyx_float_8_36
#define __pyx_float_8_39 __pyx_mstate_global->__pyx_float_8_39
#define __pyx_float_8_40 __pyx_mstate_global->__pyx_float_8_40
#define __pyx_float_8_46 __pyx_mstate_global->__pyx_float_8_46
#define __pyx_float_8_50 __pyx_mstate_global->__pyx_float_8_50
#define __pyx_float_8_63 __pyx_mstate_global->__pyx_float_8_63
#define __pyx_float_8_94 __pyx_mstate_global->__pyx_float_8_94
#define __pyx_float_8_95 __pyx_mstate_global->__pyx_float_8_95
#define __pyx_float_8_96 __pyx_mstate_global->__pyx_float_8_96
#define __pyx_float_8_97 __pyx_mstate_global->__pyx_float_8_97
#define __pyx_float_8_98 __pyx_mstate_global->__pyx_float_8_98
#define __pyx_float_8_99 __pyx_mstate_global->__pyx_float_8_99
#define __pyx_float_9_00 __pyx_mstate_global->__pyx_float_9_00
#define __pyx_float_9_01 __pyx_mstate_global->__pyx_float_9_01
#define __pyx_float_9_04 __pyx_mstate_global->__pyx_float_9_04
#define __pyx_float_9_07 __pyx_mstate_global->__pyx_float_9_07
#define __pyx_float_9_08 __pyx_mstate_global->__pyx_float_9_08
#define __pyx_float_9_10 __pyx_mstate_global->__pyx_float_9_10
#define __pyx_float_9_11 __pyx_mstate_global->__pyx_float_9_11
#define __pyx_float_9_13 __pyx_mstate_global->__pyx_float_9_13
#define __pyx_float_9_15 __pyx_mstate_global->__pyx_float_9_15
#define __pyx_float_9_16 __pyx_mstate_global->__pyx_float_9_16
#define __pyx_float_9_19 __pyx_mstate_global->__pyx_float_9_19
#define __pyx_float_9_21 __pyx_mstate_global->__pyx_float_9_21
#define __pyx_float_9_23 __pyx_mstate_global->__pyx_float_9_23
#define __pyx_float_9_25 __pyx_mstate_global->__pyx_float_9_25
#define __pyx_float_9_27 __pyx_mstate_global->__pyx_float_9_27
#define __pyx_float_9_29 __pyx_mstate_global->__pyx_float_9_29
#define __pyx_float_9_32 __pyx_mstate_global->__pyx_float_9_32
#define __pyx_float_9_43 __pyx_mstate_global->__pyx_float_9_43
#define __pyx_float_9_50 __pyx_mstate_global->__pyx_float_9_50
#define __pyx_float_9_51 __pyx_mstate_global->__pyx_float_9_51
#define __pyx_float_9_53 __pyx_mstate_global->__pyx_float_9_53
#define __pyx_float_9_58 __pyx_mstate_global->__pyx_float_9_58
#define __pyx_float_9_66 __pyx_mstate_global->__pyx_float_9_66
#define __pyx_float_9_92 __pyx_mstate_global->__pyx_float_9_92
#define __pyx_float_9_95 __pyx_mstate_global->__pyx_float_9_95
#define __pyx_float_9_96 __pyx_mstate_global->__pyx_float_9_96
#define __pyx_float_9_98 __pyx_mstate_global->__pyx_float_9_98
#define __pyx_float_9_99 __pyx_mstate_global->__pyx_float_9_99
#define __pyx_float__0001 __pyx_mstate_global->__pyx_float__0001
#define __pyx_float__0002 __pyx_mstate_global->__pyx_float__0002
#define __pyx_float__0003 __pyx_mstate_global->__pyx_float__0003
#define __pyx_float__0004 __pyx_mstate_global->__pyx_float__0004
#define __pyx_float__0005 __pyx_mstate_global->__pyx_float__0005
#define __pyx_float__0006 __pyx_mstate_global->__pyx_float__0006
#define __pyx_float__0007 __pyx_mstate_global->__pyx_float__0007
#define __pyx_float__0008 __pyx_mstate_global->__pyx_float__0008
#define __pyx_float__0009 __pyx_mstate_global->__pyx_float__0009
#define __pyx_float__0010 __pyx_mstate_global->__pyx_float__0010
#define __pyx_float__0011 __pyx_mstate_global->__pyx_float__0011
#define __pyx_float__0012 __pyx_mstate_global->__pyx_float__0012
#define __pyx_float__0013 __pyx_mstate_global->__pyx_float__0013
#define __pyx_float__0014 __pyx_mstate_global->__pyx_float__0014
#define __pyx_float__0015 __pyx_mstate_global->__pyx_float__0015
#define __pyx_float__0016 __pyx_mstate_global->__pyx_float__0016
#define __pyx_float__0017 __pyx_mstate_global->__pyx_float__0017
#define __pyx_float__0018 __pyx_mstate_global->__pyx_float__0018
#define __pyx_float__0019 __pyx_mstate_global->__pyx_float__0019
#define __pyx_float__0020 __pyx_mstate_global->__pyx_float__0020
#define __pyx_float__0021 __pyx_mstate_global->__pyx_float__0021
#define __pyx_float__0022 __pyx_mstate_global->__pyx_float__0022
#define __pyx_float__0023 __pyx_mstate_global->__pyx_float__0023
#define __pyx_float__0024 __pyx_mstate_global->__pyx_float__0024
#define __pyx_float__0025 __pyx_mstate_global->__pyx_float__0025
#define __pyx_float__0026 __pyx_mstate_global->__pyx_float__0026
#define __pyx_float__0027 __pyx_mstate_global->__pyx_float__0027
#define __pyx_float__0028 __pyx_mstate_global->__pyx_float__0028
#define __pyx_float__0029 __pyx_mstate_global->__pyx_float__0029
#define __pyx_float__0030 __pyx_mstate_global->__pyx_float__0030
#define __pyx_float__0031 __pyx_mstate_global->__pyx_float__0031
#define __pyx_float__0032 __pyx_mstate_global->__pyx_float__0032
#define __pyx_float__0033 __pyx_mstate_global->__pyx_float__0033
#define __pyx_float__0034 __pyx_mstate_global->__pyx_float__0034
#define __pyx_float__0035 __pyx_mstate_global->__pyx_float__0035
#define __pyx_float__0036 __pyx_mstate_global->__pyx_float__0036
#define __pyx_float__0037 __pyx_mstate_global->__pyx_float__0037
#define __pyx_float__0038 __pyx_mstate_global->__pyx_float__0038
#define __pyx_float__0039 __pyx_mstate_global->__pyx_float__0039
#define __pyx_float__0040 __pyx_mstate_global->__pyx_float__0040
#define __pyx_float__0041 __pyx_mstate_global->__pyx_float__0041
#define __pyx_float__0042 __pyx_mstate_global->__pyx_float__0042
#define __pyx_float__0043 __pyx_mstate_global->__pyx_float__0043
#define __pyx_float__0044 __pyx_mstate_global->__pyx_float__0044
#define __pyx_float__0045 __pyx_mstate_global->__pyx_float__0045
#define __pyx_float__0046 __pyx_mstate_global->__pyx_float__0046
#define __pyx_float__0047 __pyx_mstate_global->__pyx_float__0047
#define __pyx_float__0048 __pyx_mstate_global->__pyx_float__0048
#define __pyx_float__0049 __pyx_mstate_global->__pyx_float__0049
#define __pyx_float__0050 __pyx_mstate_global->__pyx_float__0050
#define __pyx_float__0051 __pyx_mstate_global->__pyx_float__0051
#define __pyx_float__0052 __pyx_mstate_global->__pyx_float__0052
#define __pyx_float__0053 __pyx_mstate_global->__pyx_float__0053
#define __pyx_float__0054 __pyx_mstate_global->__pyx_float__0054
#define __pyx_float__0055 __pyx_mstate_global->__pyx_float__0055
#define __pyx_float__0056 __pyx_mstate_global->__pyx_float__0056
#define __pyx_float__0057 __pyx_mstate_global->__pyx_float__0057
#define __pyx_float__0058 __pyx_mstate_global->__pyx_float__0058
#define __pyx_float__0059 __pyx_mstate_global->__pyx_float__0059
#define __pyx_float__0060 __pyx_mstate_global->__pyx_float__0060
#define __pyx_float__0061 __pyx_mstate_global->__pyx_float__0061
#define __pyx_float__0062 __pyx_mstate_global->__pyx_float__0062
#define __pyx_float__0063 __pyx_mstate_global->__pyx_float__0063
#define __pyx_float__0064 __pyx_mstate_global->__pyx_float__0064
#define __pyx_float__0065 __pyx_mstate_global->__pyx_float__0065
#define __pyx_float__0066 __pyx_mstate_global->__pyx_float__0066
#define __pyx_float__0067 __pyx_mstate_global->__pyx_float__0067
#define __pyx_float__0068 __pyx_mstate_global->__pyx_float__0068
#define __pyx_float__0069 __pyx_mstate_global->__pyx_float__0069
#define __pyx_float__0070 __pyx_mstate_global->__pyx_float__0070
#define __pyx_float__0071 __pyx_mstate_global->__pyx_float__0071
#define __pyx_float__0072 __pyx_mstate_global->__pyx_float__0072
#define __pyx_float__0073 __pyx_mstate_global->__pyx_float__0073
#define __pyx_float__0074 __pyx_mstate_global->__pyx_float__0074
#define __pyx_float__0075 __pyx_mstate_global->__pyx_float__0075
#define __pyx_float__0076 __pyx_mstate_global->__pyx_float__0076
#define __pyx_float__0077 __pyx_mstate_global->__pyx_float__0077
#define __pyx_float__0078 __pyx_mstate_global->__pyx_float__0078
#define __pyx_float__0080 __pyx_mstate_global->__pyx_float__0080
#define __pyx_float__0081 __pyx_mstate_global->__pyx_float__0081
#define __pyx_float__0082 __pyx_mstate_global->__pyx_float__0082
#define __pyx_float__0083 __pyx_mstate_global->__pyx_float__0083
#define __pyx_float__0084 __pyx_mstate_global->__pyx_float__0084
#define __pyx_float__0085 __pyx_mstate_global->__pyx_float__0085
#define __pyx_float__0086 __pyx_mstate_global->__pyx_float__0086
#define __pyx_float__0087 __pyx_mstate_global->__pyx_float__0087
#define __pyx_float__0088 __pyx_mstate_global->__pyx_float__0088
#define __pyx_float__0089 __pyx_mstate_global->__pyx_float__0089
#define __pyx_float__0090 __pyx_mstate_global->__pyx_float__0090
#define __pyx_float__0091 __pyx_mstate_global->__pyx_float__0091
#define __pyx_float__0092 __pyx_mstate_global->__pyx_float__0092
#define __pyx_float__0093 __pyx_mstate_global->__pyx_float__0093
#define __pyx_float__0094 __pyx_mstate_global->__pyx_float__0094
#define __pyx_float__0095 __pyx_mstate_global->__pyx_float__0095
#define __pyx_float__0097 __pyx_mstate_global->__pyx_float__0097
#define __pyx_float__0098 __pyx_mstate_global->__pyx_float__0098
#define __pyx_float__0099 __pyx_mstate_global->__pyx_float__0099
#define __pyx_float__0100 __pyx_mstate_global->__pyx_float__0100
#define __pyx_float__0101 __pyx_mstate_global->__pyx_float__0101
#define __pyx_float__0102 __pyx_mstate_global->__pyx_float__0102
#define __pyx_float__0103 __pyx_mstate_global->__pyx_float__0103
#define __pyx_float__0104 __pyx_mstate_global->__pyx_float__0104
#define __pyx_float__0105 __pyx_mstate_global->__pyx_float__0105
#define __pyx_float__0106 __pyx_mstate_global->__pyx_float__0106
#define __pyx_float__0107 __pyx_mstate_global->__pyx_float__0107
#define __pyx_float__0108 __pyx_mstate_global->__pyx_float__0108
#define __pyx_float__0109 __pyx_mstate_global->__pyx_float__0109
#define __pyx_float__0111 __pyx_mstate_global->__pyx_float__0111
#define __pyx_float__0112 __pyx_mstate_global->__pyx_float__0112
#define __pyx_float__0113 __pyx_mstate_global->__pyx_float__0113
#define __pyx_float__0114 __pyx_mstate_global->__pyx_float__0114
#define __pyx_float__0115 __pyx_mstate_global->__pyx_float__0115
#define __pyx_float__0116 __pyx_mstate_global->__pyx_float__0116
#define __pyx_float__0117 __pyx_mstate_global->__pyx_float__0117
#define __pyx_float__0118 __pyx_mstate_global->__pyx_float__0118
#define __pyx_float__0119 __pyx_mstate_global->__pyx_float__0119
#define __pyx_float__0120 __pyx_mstate_global->__pyx_float__0120
#define __pyx_float__0121 __pyx_mstate_global->__pyx_float__0121
#define __pyx_float__0122 __pyx_mstate_global->__pyx_float__0122
#define __pyx_float__0123 __pyx_mstate_global->__pyx_float__0123
#define __pyx_float__0124 __pyx_mstate_global->__pyx_float__0124
#define __pyx_float__0125 __pyx_mstate_global->__pyx_float__0125
#define __pyx_float__0126 __pyx_mstate_global->__pyx_float__0126
#define __pyx_float__0127 __pyx_mstate_global->__pyx_float__0127
#define __pyx_float__0128 __pyx_mstate_global->__pyx_float__0128
#define __pyx_float__0129 __pyx_mstate_global->__pyx_float__0129
#define __pyx_float__0132 __pyx_mstate_global->__pyx_float__0132
#define __pyx_float__0133 __pyx_mstate_global->__pyx_float__0133
#define __pyx_float__0134 __pyx_mstate_global->__pyx_float__0134
#define __pyx_float__0135 __pyx_mstate_global->__pyx_float__0135
#define __pyx_float__0136 __pyx_mstate_global->__pyx_float__0136
#define __pyx_float__0137 __pyx_mstate_global->__pyx_float__0137
#define __pyx_float__0138 __pyx_mstate_global->__pyx_float__0138
#define __pyx_float__0139 __pyx_mstate_global->__pyx_float__0139
#define __pyx_float__0140 __pyx_mstate_global->__pyx_float__0140
#define __pyx_float__0141 __pyx_mstate_global->__pyx_float__0141
#define __pyx_float__0142 __pyx_mstate_global->__pyx_float__0142
#define __pyx_float__0144 __pyx_mstate_global->__pyx_float__0144
#define __pyx_float__0145 __pyx_mstate_global->__pyx_float__0145
#define __pyx_float__0147 __pyx_mstate_global->__pyx_float__0147
#define __pyx_float__0148 __pyx_mstate_global->__pyx_float__0148
#define __pyx_float__0151 __pyx_mstate_global->__pyx_float__0151
#define __pyx_float__0152 __pyx_mstate_global->__pyx_float__0152
#define __pyx_float__0153 __pyx_mstate_global->__pyx_float__0153
#define __pyx_float__0154 __pyx_mstate_global->__pyx_float__0154
#define __pyx_float__0155 __pyx_mstate_global->__pyx_float__0155
#define __pyx_float__0156 __pyx_mstate_global->__pyx_float__0156
#define __pyx_float__0158 __pyx_mstate_global->__pyx_float__0158
#define __pyx_float__0159 __pyx_mstate_global->__pyx_float__0159
#define __pyx_float__0160 __pyx_mstate_global->__pyx_float__0160
#define __pyx_float__0161 __pyx_mstate_global->__pyx_float__0161
#define __pyx_float__0162 __pyx_mstate_global->__pyx_float__0162
#define __pyx_float__0163 __pyx_mstate_global->__pyx_float__0163
#define __pyx_float__0165 __pyx_mstate_global->__pyx_float__0165
#define __pyx_float__0166 __pyx_mstate_global->__pyx_float__0166
#define __pyx_float__0167 __pyx_mstate_global->__pyx_float__0167
#define __pyx_float__0170 __pyx_mstate_global->__pyx_float__0170
#define __pyx_float__0171 __pyx_mstate_global->__pyx_float__0171
#define __pyx_float__0172 __pyx_mstate_global->__pyx_float__0172
#define __pyx_float__0173 __pyx_mstate_global->__pyx_float__0173
#define __pyx_float__0174 __pyx_mstate_global->__pyx_float__0174
#define __pyx_float__0175 __pyx_mstate_global->__pyx_float__0175
#define __pyx_float__0177 __pyx_mstate_global->__pyx_float__0177
#define __pyx_float__0178 __pyx_mstate_global->__pyx_float__0178
#define __pyx_float__0179 __pyx_mstate_global->__pyx_float__0179
#define __pyx_float__0181 __pyx_mstate_global->__pyx_float__0181
#define __pyx_float__0182 __pyx_mstate_global->__pyx_float__0182
#define __pyx_float__0183 __pyx_mstate_global->__pyx_float__0183
#define __pyx_float__0184 __pyx_mstate_global->__pyx_float__0184
#define __pyx_float__0185 __pyx_mstate_global->__pyx_float__0185
#define __pyx_float__0186 __pyx_mstate_global->__pyx_float__0186
#define __pyx_float__0189 __pyx_mstate_global->__pyx_float__0189
#define __pyx_float__0190 __pyx_mstate_global->__pyx_float__0190
#define __pyx_float__0191 __pyx_mstate_global->__pyx_float__0191
#define __pyx_float__0192 __pyx_mstate_global->__pyx_float__0192
#define __pyx_float__0194 __pyx_mstate_global->__pyx_float__0194
#define __pyx_float__0196 __pyx_mstate_global->__pyx_float__0196
#define __pyx_float__0197 __pyx_mstate_global->__pyx_float__0197
#define __pyx_float__0201 __pyx_mstate_global->__pyx_float__0201
#define __pyx_float__0202 __pyx_mstate_global->__pyx_float__0202
#define __pyx_float__0203 __pyx_mstate_global->__pyx_float__0203
#define __pyx_float__0206 __pyx_mstate_global->__pyx_float__0206
#define __pyx_float__0207 __pyx_mstate_global->__pyx_float__0207
#define __pyx_float__0209 __pyx_mstate_global->__pyx_float__0209
#define __pyx_float__0210 __pyx_mstate_global->__pyx_float__0210
#define __pyx_float__0211 __pyx_mstate_global->__pyx_float__0211
#define __pyx_float__0212 __pyx_mstate_global->__pyx_float__0212
#define __pyx_float__0213 __pyx_mstate_global->__pyx_float__0213
#define __pyx_float__0215 __pyx_mstate_global->__pyx_float__0215
#define __pyx_float__0217 __pyx_mstate_global->__pyx_float__0217
#define __pyx_float__0218 __pyx_mstate_global->__pyx_float__0218
#define __pyx_float__0221 __pyx_mstate_global->__pyx_float__0221
#define __pyx_float__0222 __pyx_mstate_global->__pyx_float__0222
#define __pyx_float__0225 __pyx_mstate_global->__pyx_float__0225
#define __pyx_float__0227 __pyx_mstate_global->__pyx_float__0227
#define __pyx_float__0228 __pyx_mstate_global->__pyx_float__0228
#define __pyx_float__0229 __pyx_mstate_global->__pyx_float__0229
#define __pyx_float__0230 __pyx_mstate_global->__pyx_float__0230
#define __pyx_float__0232 __pyx_mstate_global->__pyx_float__0232
#define __pyx_float__0233 __pyx_mstate_global->__pyx_float__0233
#define __pyx_float__0234 __pyx_mstate_global->__pyx_float__0234
#define __pyx_float__0235 __pyx_mstate_global->__pyx_float__0235
#define __pyx_float__0236 __pyx_mstate_global->__pyx_float__0236
#define __pyx_float__0237 __pyx_mstate_global->__pyx_float__0237
#define __pyx_float__0238 __pyx_mstate_global->__pyx_float__0238
#define __pyx_float__0240 __pyx_mstate_global->__pyx_float__0240
#define __pyx_float__0241 __pyx_mstate_global->__pyx_float__0241
#define __pyx_float__0242 __pyx_mstate_global->__pyx_float__0242
#define __pyx_float__0245 __pyx_mstate_global->__pyx_float__0245
#define __pyx_float__0246 __pyx_mstate_global->__pyx_float__0246
#define __pyx_float__0247 __pyx_mstate_global->__pyx_float__0247
#define __pyx_float__0248 __pyx_mstate_global->__pyx_float__0248
#define __pyx_float__0249 __pyx_mstate_global->__pyx_float__0249
#define __pyx_float__0250 __pyx_mstate_global->__pyx_float__0250
#define __pyx_float__0252 __pyx_mstate_global->__pyx_float__0252
#define __pyx_float__0256 __pyx_mstate_global->__pyx_float__0256
#define __pyx_float__0257 __pyx_mstate_global->__pyx_float__0257
#define __pyx_float__0258 __pyx_mstate_global->__pyx_float__0258
#define __pyx_float__0259 __pyx_mstate_global->__pyx_float__0259
#define __pyx_float__0260 __pyx_mstate_global->__pyx_float__0260
#define __pyx_float__0264 __pyx_mstate_global->__pyx_float__0264
#define __pyx_float__0265 __pyx_mstate_global->__pyx_float__0265
#define __pyx_float__0266 __pyx_mstate_global->__pyx_float__0266
#define __pyx_float__0268 __pyx_mstate_global->__pyx_float__0268
#define __pyx_float__0272 __pyx_mstate_global->__pyx_float__0272
#define __pyx_float__0274 __pyx_mstate_global->__pyx_float__0274
#define __pyx_float__0276 __pyx_mstate_global->__pyx_float__0276
#define __pyx_float__0278 __pyx_mstate_global->__pyx_float__0278
#define __pyx_float__0279 __pyx_mstate_global->__pyx_float__0279
#define __pyx_float__0280 __pyx_mstate_global->__pyx_float__0280
#define __pyx_float__0284 __pyx_mstate_global->__pyx_float__0284
#define __pyx_float__0285 __pyx_mstate_global->__pyx_float__0285
#define __pyx_float__0287 __pyx_mstate_global->__pyx_float__0287
#define __pyx_float__0289 __pyx_mstate_global->__pyx_float__0289
#define __pyx_float__0290 __pyx_mstate_global->__pyx_float__0290
#define __pyx_float__0291 __pyx_mstate_global->__pyx_float__0291
#define __pyx_float__0292 __pyx_mstate_global->__pyx_float__0292
#define __pyx_float__0293 __pyx_mstate_global->__pyx_float__0293
#define __pyx_float__0297 __pyx_mstate_global->__pyx_float__0297
#define __pyx_float__0298 __pyx_mstate_global->__pyx_float__0298
#define __pyx_float__0299 __pyx_mstate_global->__pyx_float__0299
#define __pyx_float__0300 __pyx_mstate_global->__pyx_float__0300
#define __pyx_float__0301 __pyx_mstate_global->__pyx_float__0301
#define __pyx_float__0305 __pyx_mstate_global->__pyx_float__0305
#define __pyx_float__0306 __pyx_mstate_global->__pyx_float__0306
#define __pyx_float__0307 __pyx_mstate_global->__pyx_float__0307
#define __pyx_float__0309 __pyx_mstate_global->__pyx_float__0309
#define __pyx_float__0310 __pyx_mstate_global->__pyx_float__0310
#define __pyx_float__0311 __pyx_mstate_global->__pyx_float__0311
#define __pyx_float__0313 __pyx_mstate_global->__pyx_float__0313
#define __pyx_float__0314 __pyx_mstate_global->__pyx_float__0314
#define __pyx_float__0315 __pyx_mstate_global->__pyx_float__0315
#define __pyx_float__0319 __pyx_mstate_global->__pyx_float__0319
#define __pyx_float__0322 __pyx_mstate_global->__pyx_float__0322
#define __pyx_float__0324 __pyx_mstate_global->__pyx_float__0324
#define __pyx_float__0326 __pyx_mstate_global->__pyx_float__0326
#define __pyx_float__0328 __pyx_mstate_global->__pyx_float__0328
#define __pyx_float__0329 __pyx_mstate_global->__pyx_float__0329
#define __pyx_float__0332 __pyx_mstate_global->__pyx_float__0332
#define __pyx_float__0334 __pyx_mstate_global->__pyx_float__0334
#define __pyx_float__0336 __pyx_mstate_global->__pyx_float__0336
#define __pyx_float__0340 __pyx_mstate_global->__pyx_float__0340
#define __pyx_float__0342 __pyx_mstate_global->__pyx_float__0342
#define __pyx_float__0345 __pyx_mstate_global->__pyx_float__0345
#define __pyx_float__0349 __pyx_mstate_global->__pyx_float__0349
#define __pyx_float__0353 __pyx_mstate_global->__pyx_float__0353
#define __pyx_float__0355 __pyx_mstate_global->__pyx_float__0355
#define __pyx_float__0363 __pyx_mstate_global->__pyx_float__0363
#define __pyx_float__0364 __pyx_mstate_global->__pyx_float__0364
#define __pyx_float__0365 __pyx_mstate_global->__pyx_float__0365
#define __pyx_float__0367 __pyx_mstate_global->__pyx_float__0367
#define __pyx_float__0368 __pyx_mstate_global->__pyx_float__0368
#define __pyx_float__0369 __pyx_mstate_global->__pyx_float__0369
#define __pyx_float__0370 __pyx_mstate_global->__pyx_float__0370
#define __pyx_float__0373 __pyx_mstate_global->__pyx_float__0373
#define __pyx_float__0381 __pyx_mstate_global->__pyx_float__0381
#define __pyx_float__0387 __pyx_mstate_global->__pyx_float__0387
#define __pyx_float__0389 __pyx_mstate_global->__pyx_float__0389
#define __pyx_float__0390 __pyx_mstate_global->__pyx_float__0390
#define __pyx_float__0392 __pyx_mstate_global->__pyx_float__0392
#define __pyx_float__0396 __pyx_mstate_global->__pyx_float__0396
#define __pyx_float__0402 __pyx_mstate_global->__pyx_float__0402
#define __pyx_float__0403 __pyx_mstate_global->__pyx_float__0403
#define __pyx_float__0404 __pyx_mstate_global->__pyx_float__0404
#define __pyx_float__0410 __pyx_mstate_global->__pyx_float__0410
#define __pyx_float__0411 __pyx_mstate_global->__pyx_float__0411
#define __pyx_float__0415 __pyx_mstate_global->__pyx_float__0415
#define __pyx_float__0418 __pyx_mstate_global->__pyx_float__0418
#define __pyx_float__0420 __pyx_mstate_global->__pyx_float__0420
#define __pyx_float__0421 __pyx_mstate_global->__pyx_float__0421
#define __pyx_float__0423 __pyx_mstate_global->__pyx_float__0423
#define __pyx_float__0425 __pyx_mstate_global->__pyx_float__0425
#define __pyx_float__0437 __pyx_mstate_global->__pyx_float__0437
#define __pyx_float__0445 __pyx_mstate_global->__pyx_float__0445
#define __pyx_float__0448 __pyx_mstate_global->__pyx_float__0448
#define __pyx_float__0449 __pyx_mstate_global->__pyx_float__0449
#define __pyx_float__0465 __pyx_mstate_global->__pyx_float__0465
#define __pyx_float__0472 __pyx_mstate_global->__pyx_float__0472
#define __pyx_float__0473 __pyx_mstate_global->__pyx_float__0473
#define __pyx_float__0475 __pyx_mstate_global->__pyx_float__0475
#define __pyx_float__0482 __pyx_mstate_global->__pyx_float__0482
#define __pyx_float__0492 __pyx_mstate_global->__pyx_float__0492
#define __pyx_float__0493 __pyx_mstate_global->__pyx_float__0493
#define __pyx_float__0497 __pyx_mstate_global->__pyx_float__0497
#define __pyx_float__0498 __pyx_mstate_global->__pyx_float__0498
#define __pyx_float__0500 __pyx_mstate_global->__pyx_float__0500
#define __pyx_float__0503 __pyx_mstate_global->__pyx_float__0503
#define __pyx_float__0506 __pyx_mstate_global->__pyx_float__0506
#define __pyx_float__0512 __pyx_mstate_global->__pyx_float__0512
#define __pyx_float__0514 __pyx_mstate_global->__pyx_float__0514
#define __pyx_float__0516 __pyx_mstate_global->__pyx_float__0516
#define __pyx_float__0517 __pyx_mstate_global->__pyx_float__0517
#define __pyx_float__0522 __pyx_mstate_global->__pyx_float__0522
#define __pyx_float__0523 __pyx_mstate_global->__pyx_float__0523
#define __pyx_float__0525 __pyx_mstate_global->__pyx_float__0525
#define __pyx_float__0578 __pyx_mstate_global->__pyx_float__0578
#define __pyx_float__0593 __pyx_mstate_global->__pyx_float__0593
#define __pyx_float__0596 __pyx_mstate_global->__pyx_float__0596
#define __pyx_float__0613 __pyx_mstate_global->__pyx_float__0613
#define __pyx_float__0621 __pyx_mstate_global->__pyx_float__0621
#define __pyx_float__0629 __pyx_mstate_global->__pyx_float__0629
#define __pyx_float__0650 __pyx_mstate_global->__pyx_float__0650
#define __pyx_float__0654 __pyx_mstate_global->__pyx_float__0654
#define __pyx_float__0669 __pyx_mstate_global->__pyx_float__0669
#define __pyx_float__0674 __pyx_mstate_global->__pyx_float__0674
#define __pyx_float__0676 __pyx_mstate_global->__pyx_float__0676
#define __pyx_float__0678 __pyx_mstate_global->__pyx_float__0678
#define __pyx_float__0683 __pyx_mstate_global->__pyx_float__0683
#define __pyx_float__0692 __pyx_mstate_global->__pyx_float__0692
#define __pyx_float__0698 __pyx_mstate_global->__pyx_float__0698
#define __pyx_float__0708 __pyx_mstate_global->__pyx_float__0708
#define __pyx_float__0714 __pyx_mstate_global->__pyx_float__0714
#define __pyx_float__0719 __pyx_mstate_global->__pyx_float__0719
#define __pyx_float__0726 __pyx_mstate_global->__pyx_float__0726
#define __pyx_float__0727 __pyx_mstate_global->__pyx_float__0727
#define __pyx_float__0752 __pyx_mstate_global->__pyx_float__0752
#define __pyx_float__0763 __pyx_mstate_global->__pyx_float__0763
#define __pyx_float__0765 __pyx_mstate_global->__pyx_float__0765
#define __pyx_float__0773 __pyx_mstate_global->__pyx_float__0773
#define __pyx_float__0813 __pyx_mstate_global->__pyx_float__0813
#define __pyx_float__0820 __pyx_mstate_global->__pyx_float__0820
#define __pyx_float__0835 __pyx_mstate_global->__pyx_float__0835
#define __pyx_float__0841 __pyx_mstate_global->__pyx_float__0841
#define __pyx_float__0846 __pyx_mstate_global->__pyx_float__0846
#define __pyx_float__0877 __pyx_mstate_global->__pyx_float__0877
#define __pyx_float__0909 __pyx_mstate_global->__pyx_float__0909
#define __pyx_float__0919 __pyx_mstate_global->__pyx_float__0919
#define __pyx_float__0920 __pyx_mstate_global->__pyx_float__0920
#define __pyx_float__0921 __pyx_mstate_global->__pyx_float__0921
#define __pyx_float__0933 __pyx_mstate_global->__pyx_float__0933
#define __pyx_float__0938 __pyx_mstate_global->__pyx_float__0938
#define __pyx_float__0951 __pyx_mstate_global->__pyx_float__0951
#define __pyx_float__0963 __pyx_mstate_global->__pyx_float__0963
#define __pyx_float__0967 __pyx_mstate_global->__pyx_float__0967
#define __pyx_float__0976 __pyx_mstate_global->__pyx_float__0976
#define __pyx_float__0982 __pyx_mstate_global->__pyx_float__0982
#define __pyx_float__0985 __pyx_mstate_global->__pyx_float__0985
#define __pyx_float__1008 __pyx_mstate_global->__pyx_float__1008
#define __pyx_float__1011 __pyx_mstate_global->__pyx_float__1011
#define __pyx_float__1015 __pyx_mstate_global->__pyx_float__1015
#define __pyx_float__1018 __pyx_mstate_global->__pyx_float__1018
#define __pyx_float__1054 __pyx_mstate_global->__pyx_float__1054
#define __pyx_float__1066 __pyx_mstate_global->__pyx_float__1066
#define __pyx_float__1085 __pyx_mstate_global->__pyx_float__1085
#define __pyx_float__1126 __pyx_mstate_global->__pyx_float__1126
#define __pyx_float__1132 __pyx_mstate_global->__pyx_float__1132
#define __pyx_float__1174 __pyx_mstate_global->__pyx_float__1174
#define __pyx_float__1193 __pyx_mstate_global->__pyx_float__1193
#define __pyx_float__1238 __pyx_mstate_global->__pyx_float__1238
#define __pyx_float__1243 __pyx_mstate_global->__pyx_float__1243
#define __pyx_float__1273 __pyx_mstate_global->__pyx_float__1273
#define __pyx_float__1296 __pyx_mstate_global->__pyx_float__1296
#define __pyx_float__1304 __pyx_mstate_global->__pyx_float__1304
#define __pyx_float__1311 __pyx_mstate_global->__pyx_float__1311
#define __pyx_float__1357 __pyx_mstate_global->__pyx_float__1357
#define __pyx_float__1400 __pyx_mstate_global->__pyx_float__1400
#define __pyx_float__1440 __pyx_mstate_global->__pyx_float__1440
#define __pyx_float__1441 __pyx_mstate_global->__pyx_float__1441
#define __pyx_float__1486 __pyx_mstate_global->__pyx_float__1486
#define __pyx_float__1541 __pyx_mstate_global->__pyx_float__1541
#define __pyx_float__1555 __pyx_mstate_global->__pyx_float__1555
#define __pyx_float__1611 __pyx_mstate_global->__pyx_float__1611
#define __pyx_float__1655 __pyx_mstate_global->__pyx_float__1655
#define __pyx_float__1715 __pyx_mstate_global->__pyx_float__1715
#define __pyx_float__1821 __pyx_mstate_global->__pyx_float__1821
#define __pyx_float__1822 __pyx_mstate_global->__pyx_float__1822
#define __pyx_float__1868 __pyx_mstate_global->__pyx_float__1868
#define __pyx_float__1889 __pyx_mstate_global->__pyx_float__1889
#define __pyx_float__1919 __pyx_mstate_global->__pyx_float__1919
#define __pyx_float__1933 __pyx_mstate_global->__pyx_float__1933
#define __pyx_float__2095 __pyx_mstate_global->__pyx_float__2095
#define __pyx_float__2113 __pyx_mstate_global->__pyx_float__2113
#define __pyx_float__2185 __pyx_mstate_global->__pyx_float__2185
#define __pyx_float__2270 __pyx_mstate_global->__pyx_float__2270
#define __pyx_float__2388 __pyx_mstate_global->__pyx_float__2388
#define __pyx_float__2455 __pyx_mstate_global->__pyx_float__2455
#define __pyx_float__2598 __pyx_mstate_global->__pyx_float__2598
#define __pyx_float__2735 __pyx_mstate_global->__pyx_float__2735
#define __pyx_float__3637 __pyx_mstate_global->__pyx_float__3637
#define __pyx_float_10_00 __pyx_mstate_global->__pyx_float_10_00
#define __pyx_float_10_01 __pyx_mstate_global->__pyx_float_10_01
#define __pyx_float_10_02 __pyx_mstate_global->__pyx_float_10_02
#define __pyx_float_10_03 __pyx_mstate_global->__pyx_float_10_03
#define __pyx_float_10_04 __pyx_mstate_global->__pyx_float_10_04
#define __pyx_float_10_05 __pyx_mstate_global->__pyx_float_10_05
#define __pyx_float_10_08 __pyx_mstate_global->__pyx_float_10_08
#define __pyx_float_10_10 __pyx_mstate_global->__pyx_float_10_10
#define __pyx_float_10_11 __pyx_mstate_global->__pyx_float_10_11
#define __pyx_float_10_12 __pyx_mstate_global->__pyx_float_10_12
#define __pyx_float_10_15 __pyx_mstate_global->__pyx_float_10_15
#define __pyx_float_10_17 __pyx_mstate_global->__pyx_float_10_17
#define __pyx_float_10_18 __pyx_mstate_global->__pyx_float_10_18
#define __pyx_float_10_22 __pyx_mstate_global->__pyx_float_10_22
#define __pyx_float_10_23 __pyx_mstate_global->__pyx_float_10_23
#define __pyx_float_10_25 __pyx_mstate_global->__pyx_float_10_25
#define __pyx_float_10_30 __pyx_mstate_global->__pyx_float_10_30
#define __pyx_float_10_33 __pyx_mstate_global->__pyx_float_10_33
#define __pyx_float_10_36 __pyx_mstate_global->__pyx_float_10_36
#define __pyx_float_10_38 __pyx_mstate_global->__pyx_float_10_38
#define __pyx_float_10_40 __pyx_mstate_global->__pyx_float_10_40
#define __pyx_float_10_45 __pyx_mstate_global->__pyx_float_10_45
#define __pyx_float_10_59 __pyx_mstate_global->__pyx_float_10_59
#define __pyx_float_10_69 __pyx_mstate_global->__pyx_float_10_69
#define __pyx_float_10_93 __pyx_mstate_global->__pyx_float_10_93
#define __pyx_float_10_96 __pyx_mstate_global->__pyx_float_10_96
#define __pyx_float_10_98 __pyx_mstate_global->__pyx_float_10_98
#define __pyx_float_10_99 __pyx_mstate_global->__pyx_float_10_99
#define __pyx_float_11_00 __pyx_mstate_global->__pyx_float_11_00
#define __pyx_float_11_01 __pyx_mstate_global->__pyx_float_11_01
#define __pyx_float_11_02 __pyx_mstate_global->__pyx_float_11_02
#define __pyx_float_11_03 __pyx_mstate_global->__pyx_float_11_03
#define __pyx_float_11_04 __pyx_mstate_global->__pyx_float_11_04
#define __pyx_float_11_05 __pyx_mstate_global->__pyx_float_11_05
#define __pyx_float_11_07 __pyx_mstate_global->__pyx_float_11_07
#define __pyx_float_11_08 __pyx_mstate_global->__pyx_float_11_08
#define __pyx_float_11_11 __pyx_mstate_global->__pyx_float_11_11
#define __pyx_float_11_14 __pyx_mstate_global->__pyx_float_11_14
#define __pyx_float_11_15 __pyx_mstate_global->__pyx_float_11_15
#define __pyx_float_11_17 __pyx_mstate_global->__pyx_float_11_17
#define __pyx_float_11_19 __pyx_mstate_global->__pyx_float_11_19
#define __pyx_float_11_23 __pyx_mstate_global->__pyx_float_11_23
#define __pyx_float_11_25 __pyx_mstate_global->__pyx_float_11_25
#define __pyx_float_11_30 __pyx_mstate_global->__pyx_float_11_30
#define __pyx_float_11_31 __pyx_mstate_global->__pyx_float_11_31
#define __pyx_float_11_35 __pyx_mstate_global->__pyx_float_11_35
#define __pyx_float_11_40 __pyx_mstate_global->__pyx_float_11_40
#define __pyx_float_11_44 __pyx_mstate_global->__pyx_float_11_44
#define __pyx_float_11_48 __pyx_mstate_global->__pyx_float_11_48
#define __pyx_float_11_53 __pyx_mstate_global->__pyx_float_11_53
#define __pyx_float_11_58 __pyx_mstate_global->__pyx_float_11_58
#define __pyx_float_11_75 __pyx_mstate_global->__pyx_float_11_75
#define __pyx_float_11_78 __pyx_mstate_global->__pyx_float_11_78
#define __pyx_float_11_94 __pyx_mstate_global->__pyx_float_11_94
#define __pyx_float_11_97 __pyx_mstate_global->__pyx_float_11_97
#define __pyx_float_11_98 __pyx_mstate_global->__pyx_float_11_98
#define __pyx_float_11_99 __pyx_mstate_global->__pyx_float_11_99
#define __pyx_float_12_00 __pyx_mstate_global->__pyx_float_12_00
#define __pyx_float_12_01 __pyx_mstate_global->__pyx_float_12_01
#define __pyx_float_12_02 __pyx_mstate_global->__pyx_float_12_02
#define __pyx_float_12_03 __pyx_mstate_global->__pyx_float_12_03
#define __pyx_float_12_05 __pyx_mstate_global->__pyx_float_12_05
#define __pyx_float_12_06 __pyx_mstate_global->__pyx_float_12_06
#define __pyx_float_12_07 __pyx_mstate_global->__pyx_float_12_07
#define __pyx_float_12_08 __pyx_mstate_global->__pyx_float_12_08
#define __pyx_float_12_10 __pyx_mstate_global->__pyx_float_12_10
#define __pyx_float_12_11 __pyx_mstate_global->__pyx_float_12_11
#define __pyx_float_12_12 __pyx_mstate_global->__pyx_float_12_12
#define __pyx_float_12_15 __pyx_mstate_global->__pyx_float_12_15
#define __pyx_float_12_18 __pyx_mstate_global->__pyx_float_12_18
#define __pyx_float_12_20 __pyx_mstate_global->__pyx_float_12_20
#define __pyx_float_12_24 __pyx_mstate_global->__pyx_float_12_24
#define __pyx_float_12_26 __pyx_mstate_global->__pyx_float_12_26
#define __pyx_float_12_30 __pyx_mstate_global->__pyx_float_12_30
#define __pyx_float_12_32 __pyx_mstate_global->__pyx_float_12_32
#define __pyx_float_12_33 __pyx_mstate_global->__pyx_float_12_33
#define __pyx_float_12_40 __pyx_mstate_global->__pyx_float_12_40
#define __pyx_float_12_45 __pyx_mstate_global->__pyx_float_12_45
#define __pyx_float_12_50 __pyx_mstate_global->__pyx_float_12_50
#define __pyx_float_12_58 __pyx_mstate_global->__pyx_float_12_58
#define __pyx_float_12_63 __pyx_mstate_global->__pyx_float_12_63
#define __pyx_float_12_67 __pyx_mstate_global->__pyx_float_12_67
#define __pyx_float_12_70 __pyx_mstate_global->__pyx_float_12_70
#define __pyx_float_12_78 __pyx_mstate_global->__pyx_float_12_78
#define __pyx_float_12_95 __pyx_mstate_global->__pyx_float_12_95
#define __pyx_float_12_96 __pyx_mstate_global->__pyx_float_12_96
#define __pyx_float_12_98 __pyx_mstate_global->__pyx_float_12_98
#define __pyx_float_12_99 __pyx_mstate_global->__pyx_float_12_99
#define __pyx_float_13_00 __pyx_mstate_global->__pyx_float_13_00
#define __pyx_float_13_01 __pyx_mstate_global->__pyx_float_13_01
#define __pyx_float_13_02 __pyx_mstate_global->__pyx_float_13_02
#define __pyx_float_13_03 __pyx_mstate_global->__pyx_float_13_03
#define __pyx_float_13_04 __pyx_mstate_global->__pyx_float_13_04
#define __pyx_float_13_05 __pyx_mstate_global->__pyx_float_13_05
#define __pyx_float_13_06 __pyx_mstate_global->__pyx_float_13_06
#define __pyx_float_13_08 __pyx_mstate_global->__pyx_float_13_08
#define __pyx_float_13_11 __pyx_mstate_global->__pyx_float_13_11
#define __pyx_float_13_13 __pyx_mstate_global->__pyx_float_13_13
#define __pyx_float_13_14 __pyx_mstate_global->__pyx_float_13_14
#define __pyx_float_13_15 __pyx_mstate_global->__pyx_float_13_15
#define __pyx_float_13_16 __pyx_mstate_global->__pyx_float_13_16
#define __pyx_float_13_21 __pyx_mstate_global->__pyx_float_13_21
#define __pyx_float_13_24 __pyx_mstate_global->__pyx_float_13_24
#define __pyx_float_13_25 __pyx_mstate_global->__pyx_float_13_25
#define __pyx_float_13_26 __pyx_mstate_global->__pyx_float_13_26
#define __pyx_float_13_29 __pyx_mstate_global->__pyx_float_13_29
#define __pyx_float_13_34 __pyx_mstate_global->__pyx_float_13_34
#define __pyx_float_13_38 __pyx_mstate_global->__pyx_float_13_38
#define __pyx_float_13_40 __pyx_mstate_global->__pyx_float_13_40
#define __pyx_float_13_42 __pyx_mstate_global->__pyx_float_13_42
#define __pyx_float_13_49 __pyx_mstate_global->__pyx_float_13_49
#define __pyx_float_13_53 __pyx_mstate_global->__pyx_float_13_53
#define __pyx_float_13_58 __pyx_mstate_global->__pyx_float_13_58
#define __pyx_float_13_66 __pyx_mstate_global->__pyx_float_13_66
#define __pyx_float_13_75 __pyx_mstate_global->__pyx_float_13_75
#define __pyx_float_13_79 __pyx_mstate_global->__pyx_float_13_79
#define __pyx_float_13_89 __pyx_mstate_global->__pyx_float_13_89
#define __pyx_float_13_91 __pyx_mstate_global->__pyx_float_13_91
#define __pyx_float_13_97 __pyx_mstate_global->__pyx_float_13_97
#define __pyx_float_13_99 __pyx_mstate_global->__pyx_float_13_99
#define __pyx_float_14_00 __pyx_mstate_global->__pyx_float_14_00
#define __pyx_float_14_02 __pyx_mstate_global->__pyx_float_14_02
#define __pyx_float_14_03 __pyx_mstate_global->__pyx_float_14_03
#define __pyx_float_14_04 __pyx_mstate_global->__pyx_float_14_04
#define __pyx_float_14_05 __pyx_mstate_global->__pyx_float_14_05
#define __pyx_float_14_06 __pyx_mstate_global->__pyx_float_14_06
#define __pyx_float_14_07 __pyx_mstate_global->__pyx_float_14_07
#define __pyx_float_14_08 __pyx_mstate_global->__pyx_float_14_08
#define __pyx_float_14_10 __pyx_mstate_global->__pyx_float_14_10
#define __pyx_float_14_12 __pyx_mstate_global->__pyx_float_14_12
#define __pyx_float_14_13 __pyx_mstate_global->__pyx_float_14_13
#define __pyx_float_14_19 __pyx_mstate_global->__pyx_float_14_19
#define __pyx_float_14_25 __pyx_mstate_global->__pyx_float_14_25
#define __pyx_float_14_32 __pyx_mstate_global->__pyx_float_14_32
#define __pyx_float_14_33 __pyx_mstate_global->__pyx_float_14_33
#define __pyx_float_14_39 __pyx_mstate_global->__pyx_float_14_39
#define __pyx_float_14_45 __pyx_mstate_global->__pyx_float_14_45
#define __pyx_float_14_50 __pyx_mstate_global->__pyx_float_14_50
#define __pyx_float_14_53 __pyx_mstate_global->__pyx_float_14_53
#define __pyx_float_14_54 __pyx_mstate_global->__pyx_float_14_54
#define __pyx_float_14_62 __pyx_mstate_global->__pyx_float_14_62
#define __pyx_float_14_66 __pyx_mstate_global->__pyx_float_14_66
#define __pyx_float_14_74 __pyx_mstate_global->__pyx_float_14_74
#define __pyx_float_14_84 __pyx_mstate_global->__pyx_float_14_84
#define __pyx_float_14_93 __pyx_mstate_global->__pyx_float_14_93
#define __pyx_float_14_95 __pyx_mstate_global->__pyx_float_14_95
#define __pyx_float_15_00 __pyx_mstate_global->__pyx_float_15_00
#define __pyx_float_15_01 __pyx_mstate_global->__pyx_float_15_01
#define __pyx_float_15_03 __pyx_mstate_global->__pyx_float_15_03
#define __pyx_float_15_04 __pyx_mstate_global->__pyx_float_15_04
#define __pyx_float_15_05 __pyx_mstate_global->__pyx_float_15_05
#define __pyx_float_15_06 __pyx_mstate_global->__pyx_float_15_06
#define __pyx_float_15_07 __pyx_mstate_global->__pyx_float_15_07
#define __pyx_float_15_08 __pyx_mstate_global->__pyx_float_15_08
#define __pyx_float_15_09 __pyx_mstate_global->__pyx_float_15_09
#define __pyx_float_15_11 __pyx_mstate_global->__pyx_float_15_11
#define __pyx_float_15_12 __pyx_mstate_global->__pyx_float_15_12
#define __pyx_float_15_15 __pyx_mstate_global->__pyx_float_15_15
#define __pyx_float_15_16 __pyx_mstate_global->__pyx_float_15_16
#define __pyx_float_15_21 __pyx_mstate_global->__pyx_float_15_21
#define __pyx_float_15_22 __pyx_mstate_global->__pyx_float_15_22
#define __pyx_float_15_24 __pyx_mstate_global->__pyx_float_15_24
#define __pyx_float_15_27 __pyx_mstate_global->__pyx_float_15_27
#define __pyx_float_15_31 __pyx_mstate_global->__pyx_float_15_31
#define __pyx_float_15_41 __pyx_mstate_global->__pyx_float_15_41
#define __pyx_float_15_42 __pyx_mstate_global->__pyx_float_15_42
#define __pyx_float_15_48 __pyx_mstate_global->__pyx_float_15_48
#define __pyx_float_15_53 __pyx_mstate_global->__pyx_float_15_53
#define __pyx_float_15_62 __pyx_mstate_global->__pyx_float_15_62
#define __pyx_float_15_65 __pyx_mstate_global->__pyx_float_15_65
#define __pyx_float_15_77 __pyx_mstate_global->__pyx_float_15_77
#define __pyx_float_15_81 __pyx_mstate_global->__pyx_float_15_81
#define __pyx_float_15_88 __pyx_mstate_global->__pyx_float_15_88
#define __pyx_float_15_91 __pyx_mstate_global->__pyx_float_15_91
#define __pyx_float_15_93 __pyx_mstate_global->__pyx_float_15_93
#define __pyx_float_15_99 __pyx_mstate_global->__pyx_float_15_99
#define __pyx_float_16_00 __pyx_mstate_global->__pyx_float_16_00
#define __pyx_float_16_01 __pyx_mstate_global->__pyx_float_16_01
#define __pyx_float_16_03 __pyx_mstate_global->__pyx_float_16_03
#define __pyx_float_16_04 __pyx_mstate_global->__pyx_float_16_04
#define __pyx_float_16_05 __pyx_mstate_global->__pyx_float_16_05
#define __pyx_float_16_06 __pyx_mstate_global->__pyx_float_16_06
#define __pyx_float_16_07 __pyx_mstate_global->__pyx_float_16_07
#define __pyx_float_16_08 __pyx_mstate_global->__pyx_float_16_08
#define __pyx_float_16_09 __pyx_mstate_global->__pyx_float_16_09
#define __pyx_float_16_11 __pyx_mstate_global->__pyx_float_16_11
#define __pyx_float_16_12 __pyx_mstate_global->__pyx_float_16_12
#define __pyx_float_16_13 __pyx_mstate_global->__pyx_float_16_13
#define __pyx_float_16_14 __pyx_mstate_global->__pyx_float_16_14
#define __pyx_float_16_15 __pyx_mstate_global->__pyx_float_16_15
#define __pyx_float_16_16 __pyx_mstate_global->__pyx_float_16_16
#define __pyx_float_16_18 __pyx_mstate_global->__pyx_float_16_18
#define __pyx_float_16_20 __pyx_mstate_global->__pyx_float_16_20
#define __pyx_float_16_21 __pyx_mstate_global->__pyx_float_16_21
#define __pyx_float_16_24 __pyx_mstate_global->__pyx_float_16_24
#define __pyx_float_16_29 __pyx_mstate_global->__pyx_float_16_29
#define __pyx_float_16_30 __pyx_mstate_global->__pyx_float_16_30
#define __pyx_float_16_34 __pyx_mstate_global->__pyx_float_16_34
#define __pyx_float_16_40 __pyx_mstate_global->__pyx_float_16_40
#define __pyx_float_16_49 __pyx_mstate_global->__pyx_float_16_49
#define __pyx_float_16_50 __pyx_mstate_global->__pyx_float_16_50
#define __pyx_float_16_59 __pyx_mstate_global->__pyx_float_16_59
#define __pyx_float_16_66 __pyx_mstate_global->__pyx_float_16_66
#define __pyx_float_16_80 __pyx_mstate_global->__pyx_float_16_80
#define __pyx_float_17_01 __pyx_mstate_global->__pyx_float_17_01
#define __pyx_float_17_03 __pyx_mstate_global->__pyx_float_17_03
#define __pyx_float_17_04 __pyx_mstate_global->__pyx_float_17_04
#define __pyx_float_17_06 __pyx_mstate_global->__pyx_float_17_06
#define __pyx_float_17_07 __pyx_mstate_global->__pyx_float_17_07
#define __pyx_float_17_08 __pyx_mstate_global->__pyx_float_17_08
#define __pyx_float_17_09 __pyx_mstate_global->__pyx_float_17_09
#define __pyx_float_17_10 __pyx_mstate_global->__pyx_float_17_10
#define __pyx_float_17_11 __pyx_mstate_global->__pyx_float_17_11
#define __pyx_float_17_12 __pyx_mstate_global->__pyx_float_17_12
#define __pyx_float_17_14 __pyx_mstate_global->__pyx_float_17_14
#define __pyx_float_17_15 __pyx_mstate_global->__pyx_float_17_15
#define __pyx_float_17_17 __pyx_mstate_global->__pyx_float_17_17
#define __pyx_float_17_21 __pyx_mstate_global->__pyx_float_17_21
#define __pyx_float_17_22 __pyx_mstate_global->__pyx_float_17_22
#define __pyx_float_17_24 __pyx_mstate_global->__pyx_float_17_24
#define __pyx_float_17_25 __pyx_mstate_global->__pyx_float_17_25
#define __pyx_float_17_30 __pyx_mstate_global->__pyx_float_17_30
#define __pyx_float_17_36 __pyx_mstate_global->__pyx_float_17_36
#define __pyx_float_17_38 __pyx_mstate_global->__pyx_float_17_38
#define __pyx_float_17_42 __pyx_mstate_global->__pyx_float_17_42
#define __pyx_float_17_48 __pyx_mstate_global->__pyx_float_17_48
#define __pyx_float_17_59 __pyx_mstate_global->__pyx_float_17_59
#define __pyx_float_17_61 __pyx_mstate_global->__pyx_float_17_61
#define __pyx_float_17_62 __pyx_mstate_global->__pyx_float_17_62
#define __pyx_float_17_70 __pyx_mstate_global->__pyx_float_17_70
#define __pyx_float_17_79 __pyx_mstate_global->__pyx_float_17_79
#define __pyx_float_17_95 __pyx_mstate_global->__pyx_float_17_95
#define __pyx_float_17_97 __pyx_mstate_global->__pyx_float_17_97
#define __pyx_float_18_02 __pyx_mstate_global->__pyx_float_18_02
#define __pyx_float_18_03 __pyx_mstate_global->__pyx_float_18_03
#define __pyx_float_18_08 __pyx_mstate_global->__pyx_float_18_08
#define __pyx_float_18_09 __pyx_mstate_global->__pyx_float_18_09
#define __pyx_float_18_10 __pyx_mstate_global->__pyx_float_18_10
#define __pyx_float_18_11 __pyx_mstate_global->__pyx_float_18_11
#define __pyx_float_18_12 __pyx_mstate_global->__pyx_float_18_12
#define __pyx_float_18_13 __pyx_mstate_global->__pyx_float_18_13
#define __pyx_float_18_14 __pyx_mstate_global->__pyx_float_18_14
#define __pyx_float_18_15 __pyx_mstate_global->__pyx_float_18_15
#define __pyx_float_18_16 __pyx_mstate_global->__pyx_float_18_16
#define __pyx_float_18_18 __pyx_mstate_global->__pyx_float_18_18
#define __pyx_float_18_19 __pyx_mstate_global->__pyx_float_18_19
#define __pyx_float_18_20 __pyx_mstate_global->__pyx_float_18_20
#define __pyx_float_18_21 __pyx_mstate_global->__pyx_float_18_21
#define __pyx_float_18_23 __pyx_mstate_global->__pyx_float_18_23
#define __pyx_float_18_25 __pyx_mstate_global->__pyx_float_18_25
#define __pyx_float_18_27 __pyx_mstate_global->__pyx_float_18_27
#define __pyx_float_18_30 __pyx_mstate_global->__pyx_float_18_30
#define __pyx_float_18_35 __pyx_mstate_global->__pyx_float_18_35
#define __pyx_float_18_43 __pyx_mstate_global->__pyx_float_18_43
#define __pyx_float_18_45 __pyx_mstate_global->__pyx_float_18_45
#define __pyx_float_18_51 __pyx_mstate_global->__pyx_float_18_51
#define __pyx_float_18_59 __pyx_mstate_global->__pyx_float_18_59
#define __pyx_float_18_71 __pyx_mstate_global->__pyx_float_18_71
#define __pyx_float_18_75 __pyx_mstate_global->__pyx_float_18_75
#define __pyx_float_18_84 __pyx_mstate_global->__pyx_float_18_84
#define __pyx_float_18_94 __pyx_mstate_global->__pyx_float_18_94
#define __pyx_float_18_95 __pyx_mstate_global->__pyx_float_18_95
#define __pyx_float_19_04 __pyx_mstate_global->__pyx_float_19_04
#define __pyx_float_19_05 __pyx_mstate_global->__pyx_float_19_05
#define __pyx_float_19_06 __pyx_mstate_global->__pyx_float_19_06
#define __pyx_float_19_11 __pyx_mstate_global->__pyx_float_19_11
#define __pyx_float_19_12 __pyx_mstate_global->__pyx_float_19_12
#define __pyx_float_19_14 __pyx_mstate_global->__pyx_float_19_14
#define __pyx_float_19_15 __pyx_mstate_global->__pyx_float_19_15
#define __pyx_float_19_16 __pyx_mstate_global->__pyx_float_19_16
#define __pyx_float_19_17 __pyx_mstate_global->__pyx_float_19_17
#define __pyx_float_19_18 __pyx_mstate_global->__pyx_float_19_18
#define __pyx_float_19_21 __pyx_mstate_global->__pyx_float_19_21
#define __pyx_float_19_22 __pyx_mstate_global->__pyx_float_19_22
#define __pyx_float_19_23 __pyx_mstate_global->__pyx_float_19_23
#define __pyx_float_19_24 __pyx_mstate_global->__pyx_float_19_24
#define __pyx_float_19_26 __pyx_mstate_global->__pyx_float_19_26
#define __pyx_float_19_29 __pyx_mstate_global->__pyx_float_19_29
#define __pyx_float_19_34 __pyx_mstate_global->__pyx_float_19_34
#define __pyx_float_19_36 __pyx_mstate_global->__pyx_float_19_36
#define __pyx_float_19_39 __pyx_mstate_global->__pyx_float_19_39
#define __pyx_float_19_41 __pyx_mstate_global->__pyx_float_19_41
#define __pyx_float_19_51 __pyx_mstate_global->__pyx_float_19_51
#define __pyx_float_19_56 __pyx_mstate_global->__pyx_float_19_56
#define __pyx_float_19_57 __pyx_mstate_global->__pyx_float_19_57
#define __pyx_float_19_61 __pyx_mstate_global->__pyx_float_19_61
#define __pyx_float_19_71 __pyx_mstate_global->__pyx_float_19_71
#define __pyx_float_19_83 __pyx_mstate_global->__pyx_float_19_83
#define __pyx_float_19_84 __pyx_mstate_global->__pyx_float_19_84
#define __pyx_float_19_89 __pyx_mstate_global->__pyx_float_19_89
#define __pyx_float_20_00 __pyx_mstate_global->__pyx_float_20_00
#define __pyx_float_20_08 __pyx_mstate_global->__pyx_float_20_08
#define __pyx_float_20_11 __pyx_mstate_global->__pyx_float_20_11
#define __pyx_float_20_12 __pyx_mstate_global->__pyx_float_20_12
#define __pyx_float_20_13 __pyx_mstate_global->__pyx_float_20_13
#define __pyx_float_20_16 __pyx_mstate_global->__pyx_float_20_16
#define __pyx_float_20_17 __pyx_mstate_global->__pyx_float_20_17
#define __pyx_float_20_18 __pyx_mstate_global->__pyx_float_20_18
#define __pyx_float_20_19 __pyx_mstate_global->__pyx_float_20_19
#define __pyx_float_20_21 __pyx_mstate_global->__pyx_float_20_21
#define __pyx_float_20_23 __pyx_mstate_global->__pyx_float_20_23
#define __pyx_float_20_24 __pyx_mstate_global->__pyx_float_20_24
#define __pyx_float_20_25 __pyx_mstate_global->__pyx_float_20_25
#define __pyx_float_20_26 __pyx_mstate_global->__pyx_float_20_26
#define __pyx_float_20_27 __pyx_mstate_global->__pyx_float_20_27
#define __pyx_float_20_28 __pyx_mstate_global->__pyx_float_20_28
#define __pyx_float_20_29 __pyx_mstate_global->__pyx_float_20_29
#define __pyx_float_20_30 __pyx_mstate_global->__pyx_float_20_30
#define __pyx_float_20_34 __pyx_mstate_global->__pyx_float_20_34
#define __pyx_float_20_36 __pyx_mstate_global->__pyx_float_20_36
#define __pyx_float_20_40 __pyx_mstate_global->__pyx_float_20_40
#define __pyx_float_20_43 __pyx_mstate_global->__pyx_float_20_43
#define __pyx_float_20_44 __pyx_mstate_global->__pyx_float_20_44
#define __pyx_float_20_50 __pyx_mstate_global->__pyx_float_20_50
#define __pyx_float_20_60 __pyx_mstate_global->__pyx_float_20_60
#define __pyx_float_20_65 __pyx_mstate_global->__pyx_float_20_65
#define __pyx_float_20_72 __pyx_mstate_global->__pyx_float_20_72
#define __pyx_float_20_84 __pyx_mstate_global->__pyx_float_20_84
#define __pyx_float_20_98 __pyx_mstate_global->__pyx_float_20_98
#define __pyx_float_20_99 __pyx_mstate_global->__pyx_float_20_99
#define __pyx_float_21_04 __pyx_mstate_global->__pyx_float_21_04
#define __pyx_float_21_11 __pyx_mstate_global->__pyx_float_21_11
#define __pyx_float_21_13 __pyx_mstate_global->__pyx_float_21_13
#define __pyx_float_21_17 __pyx_mstate_global->__pyx_float_21_17
#define __pyx_float_21_18 __pyx_mstate_global->__pyx_float_21_18
#define __pyx_float_21_20 __pyx_mstate_global->__pyx_float_21_20
#define __pyx_float_21_21 __pyx_mstate_global->__pyx_float_21_21
#define __pyx_float_21_22 __pyx_mstate_global->__pyx_float_21_22
#define __pyx_float_21_23 __pyx_mstate_global->__pyx_float_21_23
#define __pyx_float_21_24 __pyx_mstate_global->__pyx_float_21_24
#define __pyx_float_21_25 __pyx_mstate_global->__pyx_float_21_25
#define __pyx_float_21_27 __pyx_mstate_global->__pyx_float_21_27
#define __pyx_float_21_29 __pyx_mstate_global->__pyx_float_21_29
#define __pyx_float_21_30 __pyx_mstate_global->__pyx_float_21_30
#define __pyx_float_21_31 __pyx_mstate_global->__pyx_float_21_31
#define __pyx_float_21_32 __pyx_mstate_global->__pyx_float_21_32
#define __pyx_float_21_33 __pyx_mstate_global->__pyx_float_21_33
#define __pyx_float_21_41 __pyx_mstate_global->__pyx_float_21_41
#define __pyx_float_21_47 __pyx_mstate_global->__pyx_float_21_47
#define __pyx_float_21_50 __pyx_mstate_global->__pyx_float_21_50
#define __pyx_float_21_51 __pyx_mstate_global->__pyx_float_21_51
#define __pyx_float_21_56 __pyx_mstate_global->__pyx_float_21_56
#define __pyx_float_21_57 __pyx_mstate_global->__pyx_float_21_57
#define __pyx_float_21_70 __pyx_mstate_global->__pyx_float_21_70
#define __pyx_float_21_74 __pyx_mstate_global->__pyx_float_21_74
#define __pyx_float_21_84 __pyx_mstate_global->__pyx_float_21_84
#define __pyx_float_21_98 __pyx_mstate_global->__pyx_float_21_98
#define __pyx_float_22_12 __pyx_mstate_global->__pyx_float_22_12
#define __pyx_float_22_13 __pyx_mstate_global->__pyx_float_22_13
#define __pyx_float_22_14 __pyx_mstate_global->__pyx_float_22_14
#define __pyx_float_22_16 __pyx_mstate_global->__pyx_float_22_16
#define __pyx_float_22_20 __pyx_mstate_global->__pyx_float_22_20
#define __pyx_float_22_21 __pyx_mstate_global->__pyx_float_22_21
#define __pyx_float_22_22 __pyx_mstate_global->__pyx_float_22_22
#define __pyx_float_22_25 __pyx_mstate_global->__pyx_float_22_25
#define __pyx_float_22_26 __pyx_mstate_global->__pyx_float_22_26
#define __pyx_float_22_27 __pyx_mstate_global->__pyx_float_22_27
#define __pyx_float_22_28 __pyx_mstate_global->__pyx_float_22_28
#define __pyx_float_22_29 __pyx_mstate_global->__pyx_float_22_29
#define __pyx_float_22_30 __pyx_mstate_global->__pyx_float_22_30
#define __pyx_float_22_32 __pyx_mstate_global->__pyx_float_22_32
#define __pyx_float_22_35 __pyx_mstate_global->__pyx_float_22_35
#define __pyx_float_22_36 __pyx_mstate_global->__pyx_float_22_36
#define __pyx_float_22_37 __pyx_mstate_global->__pyx_float_22_37
#define __pyx_float_22_38 __pyx_mstate_global->__pyx_float_22_38
#define __pyx_float_22_48 __pyx_mstate_global->__pyx_float_22_48
#define __pyx_float_22_49 __pyx_mstate_global->__pyx_float_22_49
#define __pyx_float_22_54 __pyx_mstate_global->__pyx_float_22_54
#define __pyx_float_22_59 __pyx_mstate_global->__pyx_float_22_59
#define __pyx_float_22_60 __pyx_mstate_global->__pyx_float_22_60
#define __pyx_float_22_68 __pyx_mstate_global->__pyx_float_22_68
#define __pyx_float_22_82 __pyx_mstate_global->__pyx_float_22_82
#define __pyx_float_22_87 __pyx_mstate_global->__pyx_float_22_87
#define __pyx_float_22_99 __pyx_mstate_global->__pyx_float_22_99
#define __pyx_float_23_11 __pyx_mstate_global->__pyx_float_23_11
#define __pyx_float_23_19 __pyx_mstate_global->__pyx_float_23_19
#define __pyx_float_23_20 __pyx_mstate_global->__pyx_float_23_20
#define __pyx_float_23_22 __pyx_mstate_global->__pyx_float_23_22
#define __pyx_float_23_24 __pyx_mstate_global->__pyx_float_23_24
#define __pyx_float_23_26 __pyx_mstate_global->__pyx_float_23_26
#define __pyx_float_23_28 __pyx_mstate_global->__pyx_float_23_28
#define __pyx_float_23_29 __pyx_mstate_global->__pyx_float_23_29
#define __pyx_float_23_30 __pyx_mstate_global->__pyx_float_23_30
#define __pyx_float_23_31 __pyx_mstate_global->__pyx_float_23_31
#define __pyx_float_23_32 __pyx_mstate_global->__pyx_float_23_32
#define __pyx_float_23_33 __pyx_mstate_global->__pyx_float_23_33
#define __pyx_float_23_34 __pyx_mstate_global->__pyx_float_23_34
#define __pyx_float_23_35 __pyx_mstate_global->__pyx_float_23_35
#define __pyx_float_23_38 __pyx_mstate_global->__pyx_float_23_38
#define __pyx_float_23_39 __pyx_mstate_global->__pyx_float_23_39
#define __pyx_float_23_41 __pyx_mstate_global->__pyx_float_23_41
#define __pyx_float_23_42 __pyx_mstate_global->__pyx_float_23_42
#define __pyx_float_23_46 __pyx_mstate_global->__pyx_float_23_46
#define __pyx_float_23_55 __pyx_mstate_global->__pyx_float_23_55
#define __pyx_float_23_56 __pyx_mstate_global->__pyx_float_23_56
#define __pyx_float_23_62 __pyx_mstate_global->__pyx_float_23_62
#define __pyx_float_23_69 __pyx_mstate_global->__pyx_float_23_69
#define __pyx_float_23_72 __pyx_mstate_global->__pyx_float_23_72
#define __pyx_float_23_73 __pyx_mstate_global->__pyx_float_23_73
#define __pyx_float_23_80 __pyx_mstate_global->__pyx_float_23_80
#define __pyx_float_23_90 __pyx_mstate_global->__pyx_float_23_90
#define __pyx_float_23_95 __pyx_mstate_global->__pyx_float_23_95
#define __pyx_float_23_99 __pyx_mstate_global->__pyx_float_23_99
#define __pyx_float_24_00 __pyx_mstate_global->__pyx_float_24_00
#define __pyx_float_24_05 __pyx_mstate_global->__pyx_float_24_05
#define __pyx_float_24_13 __pyx_mstate_global->__pyx_float_24_13
#define __pyx_float_24_14 __pyx_mstate_global->__pyx_float_24_14
#define __pyx_float_24_19 __pyx_mstate_global->__pyx_float_24_19
#define __pyx_float_24_26 __pyx_mstate_global->__pyx_float_24_26
#define __pyx_float_24_29 __pyx_mstate_global->__pyx_float_24_29
#define __pyx_float_24_31 __pyx_mstate_global->__pyx_float_24_31
#define __pyx_float_24_32 __pyx_mstate_global->__pyx_float_24_32
#define __pyx_float_24_33 __pyx_mstate_global->__pyx_float_24_33
#define __pyx_float_24_34 __pyx_mstate_global->__pyx_float_24_34
#define __pyx_float_24_35 __pyx_mstate_global->__pyx_float_24_35
#define __pyx_float_24_36 __pyx_mstate_global->__pyx_float_24_36
#define __pyx_float_24_37 __pyx_mstate_global->__pyx_float_24_37
#define __pyx_float_24_38 __pyx_mstate_global->__pyx_float_24_38
#define __pyx_float_24_40 __pyx_mstate_global->__pyx_float_24_40
#define __pyx_float_24_48 __pyx_mstate_global->__pyx_float_24_48
#define __pyx_float_24_50 __pyx_mstate_global->__pyx_float_24_50
#define __pyx_float_24_52 __pyx_mstate_global->__pyx_float_24_52
#define __pyx_float_24_64 __pyx_mstate_global->__pyx_float_24_64
#define __pyx_float_24_71 __pyx_mstate_global->__pyx_float_24_71
#define __pyx_float_24_80 __pyx_mstate_global->__pyx_float_24_80
#define __pyx_float_24_90 __pyx_mstate_global->__pyx_float_24_90
#define __pyx_float_24_97 __pyx_mstate_global->__pyx_float_24_97
#define __pyx_float_25_05 __pyx_mstate_global->__pyx_float_25_05
#define __pyx_float_25_15 __pyx_mstate_global->__pyx_float_25_15
#define __pyx_float_25_20 __pyx_mstate_global->__pyx_float_25_20
#define __pyx_float_25_24 __pyx_mstate_global->__pyx_float_25_24
#define __pyx_float_25_30 __pyx_mstate_global->__pyx_float_25_30
#define __pyx_float_25_31 __pyx_mstate_global->__pyx_float_25_31
#define __pyx_float_25_32 __pyx_mstate_global->__pyx_float_25_32
#define __pyx_float_25_33 __pyx_mstate_global->__pyx_float_25_33
#define __pyx_float_25_35 __pyx_mstate_global->__pyx_float_25_35
#define __pyx_float_25_38 __pyx_mstate_global->__pyx_float_25_38
#define __pyx_float_25_41 __pyx_mstate_global->__pyx_float_25_41
#define __pyx_float_25_42 __pyx_mstate_global->__pyx_float_25_42
#define __pyx_float_25_43 __pyx_mstate_global->__pyx_float_25_43
#define __pyx_float_25_45 __pyx_mstate_global->__pyx_float_25_45
#define __pyx_float_25_49 __pyx_mstate_global->__pyx_float_25_49
#define __pyx_float_25_54 __pyx_mstate_global->__pyx_float_25_54
#define __pyx_float_25_55 __pyx_mstate_global->__pyx_float_25_55
#define __pyx_float_25_56 __pyx_mstate_global->__pyx_float_25_56
#define __pyx_float_25_58 __pyx_mstate_global->__pyx_float_25_58
#define __pyx_float_25_59 __pyx_mstate_global->__pyx_float_25_59
#define __pyx_float_25_70 __pyx_mstate_global->__pyx_float_25_70
#define __pyx_float_25_74 __pyx_mstate_global->__pyx_float_25_74
#define __pyx_float_25_82 __pyx_mstate_global->__pyx_float_25_82
#define __pyx_float_25_87 __pyx_mstate_global->__pyx_float_25_87
#define __pyx_float_25_88 __pyx_mstate_global->__pyx_float_25_88
#define __pyx_float_26_02 __pyx_mstate_global->__pyx_float_26_02
#define __pyx_float_26_17 __pyx_mstate_global->__pyx_float_26_17
#define __pyx_float_26_22 __pyx_mstate_global->__pyx_float_26_22
#define __pyx_float_26_24 __pyx_mstate_global->__pyx_float_26_24
#define __pyx_float_26_36 __pyx_mstate_global->__pyx_float_26_36
#define __pyx_float_26_38 __pyx_mstate_global->__pyx_float_26_38
#define __pyx_float_26_39 __pyx_mstate_global->__pyx_float_26_39
#define __pyx_float_26_40 __pyx_mstate_global->__pyx_float_26_40
#define __pyx_float_26_41 __pyx_mstate_global->__pyx_float_26_41
#define __pyx_float_26_43 __pyx_mstate_global->__pyx_float_26_43
#define __pyx_float_26_46 __pyx_mstate_global->__pyx_float_26_46
#define __pyx_float_26_47 __pyx_mstate_global->__pyx_float_26_47
#define __pyx_float_26_49 __pyx_mstate_global->__pyx_float_26_49
#define __pyx_float_26_50 __pyx_mstate_global->__pyx_float_26_50
#define __pyx_float_26_54 __pyx_mstate_global->__pyx_float_26_54
#define __pyx_float_26_55 __pyx_mstate_global->__pyx_float_26_55
#define __pyx_float_26_61 __pyx_mstate_global->__pyx_float_26_61
#define __pyx_float_26_62 __pyx_mstate_global->__pyx_float_26_62
#define __pyx_float_26_64 __pyx_mstate_global->__pyx_float_26_64
#define __pyx_float_26_67 __pyx_mstate_global->__pyx_float_26_67
#define __pyx_float_26_68 __pyx_mstate_global->__pyx_float_26_68
#define __pyx_float_26_69 __pyx_mstate_global->__pyx_float_26_69
#define __pyx_float_26_70 __pyx_mstate_global->__pyx_float_26_70
#define __pyx_float_26_81 __pyx_mstate_global->__pyx_float_26_81
#define __pyx_float_26_91 __pyx_mstate_global->__pyx_float_26_91
#define __pyx_float_27_02 __pyx_mstate_global->__pyx_float_27_02
#define __pyx_float_27_03 __pyx_mstate_global->__pyx_float_27_03
#define __pyx_float_27_04 __pyx_mstate_global->__pyx_float_27_04
#define __pyx_float_27_16 __pyx_mstate_global->__pyx_float_27_16
#define __pyx_float_27_36 __pyx_mstate_global->__pyx_float_27_36
#define __pyx_float_27_41 __pyx_mstate_global->__pyx_float_27_41
#define __pyx_float_27_43 __pyx_mstate_global->__pyx_float_27_43
#define __pyx_float_27_44 __pyx_mstate_global->__pyx_float_27_44
#define __pyx_float_27_46 __pyx_mstate_global->__pyx_float_27_46
#define __pyx_float_27_47 __pyx_mstate_global->__pyx_float_27_47
#define __pyx_float_27_49 __pyx_mstate_global->__pyx_float_27_49
#define __pyx_float_27_50 __pyx_mstate_global->__pyx_float_27_50
#define __pyx_float_27_52 __pyx_mstate_global->__pyx_float_27_52
#define __pyx_float_27_53 __pyx_mstate_global->__pyx_float_27_53
#define __pyx_float_27_54 __pyx_mstate_global->__pyx_float_27_54
#define __pyx_float_27_55 __pyx_mstate_global->__pyx_float_27_55
#define __pyx_float_27_59 __pyx_mstate_global->__pyx_float_27_59
#define __pyx_float_27_60 __pyx_mstate_global->__pyx_float_27_60
#define __pyx_float_27_61 __pyx_mstate_global->__pyx_float_27_61
#define __pyx_float_27_70 __pyx_mstate_global->__pyx_float_27_70
#define __pyx_float_27_72 __pyx_mstate_global->__pyx_float_27_72
#define __pyx_float_27_76 __pyx_mstate_global->__pyx_float_27_76
#define __pyx_float_27_77 __pyx_mstate_global->__pyx_float_27_77
#define __pyx_float_27_78 __pyx_mstate_global->__pyx_float_27_78
#define __pyx_float_27_95 __pyx_mstate_global->__pyx_float_27_95
#define __pyx_float_27_97 __pyx_mstate_global->__pyx_float_27_97
#define __pyx_float_28_03 __pyx_mstate_global->__pyx_float_28_03
#define __pyx_float_28_09 __pyx_mstate_global->__pyx_float_28_09
#define __pyx_float_28_12 __pyx_mstate_global->__pyx_float_28_12
#define __pyx_float_28_16 __pyx_mstate_global->__pyx_float_28_16
#define __pyx_float_28_30 __pyx_mstate_global->__pyx_float_28_30
#define __pyx_float_28_39 __pyx_mstate_global->__pyx_float_28_39
#define __pyx_float_28_44 __pyx_mstate_global->__pyx_float_28_44
#define __pyx_float_28_45 __pyx_mstate_global->__pyx_float_28_45
#define __pyx_float_28_47 __pyx_mstate_global->__pyx_float_28_47
#define __pyx_float_28_50 __pyx_mstate_global->__pyx_float_28_50
#define __pyx_float_28_51 __pyx_mstate_global->__pyx_float_28_51
#define __pyx_float_28_52 __pyx_mstate_global->__pyx_float_28_52
#define __pyx_float_28_53 __pyx_mstate_global->__pyx_float_28_53
#define __pyx_float_28_55 __pyx_mstate_global->__pyx_float_28_55
#define __pyx_float_28_59 __pyx_mstate_global->__pyx_float_28_59
#define __pyx_float_28_60 __pyx_mstate_global->__pyx_float_28_60
#define __pyx_float_28_62 __pyx_mstate_global->__pyx_float_28_62
#define __pyx_float_28_63 __pyx_mstate_global->__pyx_float_28_63
#define __pyx_float_28_64 __pyx_mstate_global->__pyx_float_28_64
#define __pyx_float_28_65 __pyx_mstate_global->__pyx_float_28_65
#define __pyx_float_28_66 __pyx_mstate_global->__pyx_float_28_66
#define __pyx_float_28_70 __pyx_mstate_global->__pyx_float_28_70
#define __pyx_float_28_73 __pyx_mstate_global->__pyx_float_28_73
#define __pyx_float_28_78 __pyx_mstate_global->__pyx_float_28_78
#define __pyx_float_28_79 __pyx_mstate_global->__pyx_float_28_79
#define __pyx_float_28_80 __pyx_mstate_global->__pyx_float_28_80
#define __pyx_float_28_82 __pyx_mstate_global->__pyx_float_28_82
#define __pyx_float_28_87 __pyx_mstate_global->__pyx_float_28_87
#define __pyx_float_28_88 __pyx_mstate_global->__pyx_float_28_88
#define __pyx_float_29_03 __pyx_mstate_global->__pyx_float_29_03
#define __pyx_float_29_12 __pyx_mstate_global->__pyx_float_29_12
#define __pyx_float_29_17 __pyx_mstate_global->__pyx_float_29_17
#define __pyx_float_29_28 __pyx_mstate_global->__pyx_float_29_28
#define __pyx_float_29_31 __pyx_mstate_global->__pyx_float_29_31
#define __pyx_float_29_40 __pyx_mstate_global->__pyx_float_29_40
#define __pyx_float_29_48 __pyx_mstate_global->__pyx_float_29_48
#define __pyx_float_29_54 __pyx_mstate_global->__pyx_float_29_54
#define __pyx_float_29_57 __pyx_mstate_global->__pyx_float_29_57
#define __pyx_float_29_58 __pyx_mstate_global->__pyx_float_29_58
#define __pyx_float_29_60 __pyx_mstate_global->__pyx_float_29_60
#define __pyx_float_29_61 __pyx_mstate_global->__pyx_float_29_61
#define __pyx_float_29_64 __pyx_mstate_global->__pyx_float_29_64
#define __pyx_float_29_65 __pyx_mstate_global->__pyx_float_29_65
#define __pyx_float_29_66 __pyx_mstate_global->__pyx_float_29_66
#define __pyx_float_29_68 __pyx_mstate_global->__pyx_float_29_68
#define __pyx_float_29_71 __pyx_mstate_global->__pyx_float_29_71
#define __pyx_float_29_72 __pyx_mstate_global->__pyx_float_29_72
#define __pyx_float_29_76 __pyx_mstate_global->__pyx_float_29_76
#define __pyx_float_29_80 __pyx_mstate_global->__pyx_float_29_80
#define __pyx_float_29_81 __pyx_mstate_global->__pyx_float_29_81
#define __pyx_float_29_85 __pyx_mstate_global->__pyx_float_29_85
#define __pyx_float_29_89 __pyx_mstate_global->__pyx_float_29_89
#define __pyx_float_29_90 __pyx_mstate_global->__pyx_float_29_90
#define __pyx_float_29_91 __pyx_mstate_global->__pyx_float_29_91
#define __pyx_float_29_95 __pyx_mstate_global->__pyx_float_29_95
#define __pyx_float_30_03 __pyx_mstate_global->__pyx_float_30_03
#define __pyx_float_30_08 __pyx_mstate_global->__pyx_float_30_08
#define __pyx_float_30_09 __pyx_mstate_global->__pyx_float_30_09
#define __pyx_float_30_10 __pyx_mstate_global->__pyx_float_30_10
#define __pyx_float_30_22 __pyx_mstate_global->__pyx_float_30_22
#define __pyx_float_30_32 __pyx_mstate_global->__pyx_float_30_32
#define __pyx_float_30_37 __pyx_mstate_global->__pyx_float_30_37
#define __pyx_float_30_43 __pyx_mstate_global->__pyx_float_30_43
#define __pyx_float_30_46 __pyx_mstate_global->__pyx_float_30_46
#define __pyx_float_30_58 __pyx_mstate_global->__pyx_float_30_58
#define __pyx_float_30_60 __pyx_mstate_global->__pyx_float_30_60
#define __pyx_float_30_65 __pyx_mstate_global->__pyx_float_30_65
#define __pyx_float_30_70 __pyx_mstate_global->__pyx_float_30_70
#define __pyx_float_30_73 __pyx_mstate_global->__pyx_float_30_73
#define __pyx_float_30_75 __pyx_mstate_global->__pyx_float_30_75
#define __pyx_float_30_77 __pyx_mstate_global->__pyx_float_30_77
#define __pyx_float_30_78 __pyx_mstate_global->__pyx_float_30_78
#define __pyx_float_30_81 __pyx_mstate_global->__pyx_float_30_81
#define __pyx_float_30_85 __pyx_mstate_global->__pyx_float_30_85
#define __pyx_float_30_91 __pyx_mstate_global->__pyx_float_30_91
#define __pyx_float_30_97 __pyx_mstate_global->__pyx_float_30_97
#define __pyx_float_30_98 __pyx_mstate_global->__pyx_float_30_98
#define __pyx_float_31_01 __pyx_mstate_global->__pyx_float_31_01
#define __pyx_float_31_05 __pyx_mstate_global->__pyx_float_31_05
#define __pyx_float_31_06 __pyx_mstate_global->__pyx_float_31_06
#define __pyx_float_31_13 __pyx_mstate_global->__pyx_float_31_13
#define __pyx_float_31_14 __pyx_mstate_global->__pyx_float_31_14
#define __pyx_float_31_30 __pyx_mstate_global->__pyx_float_31_30
#define __pyx_float_31_45 __pyx_mstate_global->__pyx_float_31_45
#define __pyx_float_31_59 __pyx_mstate_global->__pyx_float_31_59
#define __pyx_float_31_61 __pyx_mstate_global->__pyx_float_31_61
#define __pyx_float_31_62 __pyx_mstate_global->__pyx_float_31_62
#define __pyx_float_31_66 __pyx_mstate_global->__pyx_float_31_66
#define __pyx_float_31_67 __pyx_mstate_global->__pyx_float_31_67
#define __pyx_float_31_71 __pyx_mstate_global->__pyx_float_31_71
#define __pyx_float_31_73 __pyx_mstate_global->__pyx_float_31_73
#define __pyx_float_31_74 __pyx_mstate_global->__pyx_float_31_74
#define __pyx_float_31_78 __pyx_mstate_global->__pyx_float_31_78
#define __pyx_float_31_81 __pyx_mstate_global->__pyx_float_31_81
#define __pyx_float_31_82 __pyx_mstate_global->__pyx_float_31_82
#define __pyx_float_31_84 __pyx_mstate_global->__pyx_float_31_84
#define __pyx_float_31_86 __pyx_mstate_global->__pyx_float_31_86
#define __pyx_float_31_88 __pyx_mstate_global->__pyx_float_31_88
#define __pyx_float_31_91 __pyx_mstate_global->__pyx_float_31_91
#define __pyx_float_31_95 __pyx_mstate_global->__pyx_float_31_95
#define __pyx_float_31_96 __pyx_mstate_global->__pyx_float_31_96
#define __pyx_float_31_97 __pyx_mstate_global->__pyx_float_31_97
#define __pyx_float_32_00 __pyx_mstate_global->__pyx_float_32_00
#define __pyx_float_32_03 __pyx_mstate_global->__pyx_float_32_03
#define __pyx_float_32_07 __pyx_mstate_global->__pyx_float_32_07
#define __pyx_float_32_08 __pyx_mstate_global->__pyx_float_32_08
#define __pyx_float_32_09 __pyx_mstate_global->__pyx_float_32_09
#define __pyx_float_32_12 __pyx_mstate_global->__pyx_float_32_12
#define __pyx_float_32_13 __pyx_mstate_global->__pyx_float_32_13
#define __pyx_float_32_14 __pyx_mstate_global->__pyx_float_32_14
#define __pyx_float_32_17 __pyx_mstate_global->__pyx_float_32_17
#define __pyx_float_32_24 __pyx_mstate_global->__pyx_float_32_24
#define __pyx_float_32_27 __pyx_mstate_global->__pyx_float_32_27
#define __pyx_float_32_48 __pyx_mstate_global->__pyx_float_32_48
#define __pyx_float_32_61 __pyx_mstate_global->__pyx_float_32_61
#define __pyx_float_32_65 __pyx_mstate_global->__pyx_float_32_65
#define __pyx_float_32_73 __pyx_mstate_global->__pyx_float_32_73
#define __pyx_float_32_75 __pyx_mstate_global->__pyx_float_32_75
#define __pyx_float_32_77 __pyx_mstate_global->__pyx_float_32_77
#define __pyx_float_32_78 __pyx_mstate_global->__pyx_float_32_78
#define __pyx_float_32_79 __pyx_mstate_global->__pyx_float_32_79
#define __pyx_float_32_83 __pyx_mstate_global->__pyx_float_32_83
#define __pyx_float_32_84 __pyx_mstate_global->__pyx_float_32_84
#define __pyx_float_32_89 __pyx_mstate_global->__pyx_float_32_89
#define __pyx_float_32_92 __pyx_mstate_global->__pyx_float_32_92
#define __pyx_float_32_93 __pyx_mstate_global->__pyx_float_32_93
#define __pyx_float_32_94 __pyx_mstate_global->__pyx_float_32_94
#define __pyx_float_32_96 __pyx_mstate_global->__pyx_float_32_96
#define __pyx_float_32_97 __pyx_mstate_global->__pyx_float_32_97
#define __pyx_float_33_00 __pyx_mstate_global->__pyx_float_33_00
#define __pyx_float_33_02 __pyx_mstate_global->__pyx_float_33_02
#define __pyx_float_33_14 __pyx_mstate_global->__pyx_float_33_14
#define __pyx_float_33_21 __pyx_mstate_global->__pyx_float_33_21
#define __pyx_float_33_28 __pyx_mstate_global->__pyx_float_33_28
#define __pyx_float_33_29 __pyx_mstate_global->__pyx_float_33_29
#define __pyx_float_33_32 __pyx_mstate_global->__pyx_float_33_32
#define __pyx_float_33_36 __pyx_mstate_global->__pyx_float_33_36
#define __pyx_float_33_37 __pyx_mstate_global->__pyx_float_33_37
#define __pyx_float_33_52 __pyx_mstate_global->__pyx_float_33_52
#define __pyx_float_33_59 __pyx_mstate_global->__pyx_float_33_59
#define __pyx_float_33_60 __pyx_mstate_global->__pyx_float_33_60
#define __pyx_float_33_77 __pyx_mstate_global->__pyx_float_33_77
#define __pyx_float_33_81 __pyx_mstate_global->__pyx_float_33_81
#define __pyx_float_33_82 __pyx_mstate_global->__pyx_float_33_82
#define __pyx_float_33_85 __pyx_mstate_global->__pyx_float_33_85
#define __pyx_float_33_95 __pyx_mstate_global->__pyx_float_33_95
#define __pyx_float_34_03 __pyx_mstate_global->__pyx_float_34_03
#define __pyx_float_34_04 __pyx_mstate_global->__pyx_float_34_04
#define __pyx_float_34_08 __pyx_mstate_global->__pyx_float_34_08
#define __pyx_float_34_09 __pyx_mstate_global->__pyx_float_34_09
#define __pyx_float_34_12 __pyx_mstate_global->__pyx_float_34_12
#define __pyx_float_34_13 __pyx_mstate_global->__pyx_float_34_13
#define __pyx_float_34_19 __pyx_mstate_global->__pyx_float_34_19
#define __pyx_float_34_21 __pyx_mstate_global->__pyx_float_34_21
#define __pyx_float_34_26 __pyx_mstate_global->__pyx_float_34_26
#define __pyx_float_34_30 __pyx_mstate_global->__pyx_float_34_30
#define __pyx_float_34_31 __pyx_mstate_global->__pyx_float_34_31
#define __pyx_float_34_37 __pyx_mstate_global->__pyx_float_34_37
#define __pyx_float_34_42 __pyx_mstate_global->__pyx_float_34_42
#define __pyx_float_34_52 __pyx_mstate_global->__pyx_float_34_52
#define __pyx_float_34_76 __pyx_mstate_global->__pyx_float_34_76
#define __pyx_float_34_79 __pyx_mstate_global->__pyx_float_34_79
#define __pyx_float_34_91 __pyx_mstate_global->__pyx_float_34_91
#define __pyx_float_34_95 __pyx_mstate_global->__pyx_float_34_95
#define __pyx_float_34_96 __pyx_mstate_global->__pyx_float_34_96
#define __pyx_float_34_98 __pyx_mstate_global->__pyx_float_34_98
#define __pyx_float_35_06 __pyx_mstate_global->__pyx_float_35_06
#define __pyx_float_35_07 __pyx_mstate_global->__pyx_float_35_07
#define __pyx_float_35_13 __pyx_mstate_global->__pyx_float_35_13
#define __pyx_float_35_14 __pyx_mstate_global->__pyx_float_35_14
#define __pyx_float_35_16 __pyx_mstate_global->__pyx_float_35_16
#define __pyx_float_35_17 __pyx_mstate_global->__pyx_float_35_17
#define __pyx_float_35_18 __pyx_mstate_global->__pyx_float_35_18
#define __pyx_float_35_25 __pyx_mstate_global->__pyx_float_35_25
#define __pyx_float_35_30 __pyx_mstate_global->__pyx_float_35_30
#define __pyx_float_35_38 __pyx_mstate_global->__pyx_float_35_38
#define __pyx_float_35_44 __pyx_mstate_global->__pyx_float_35_44
#define __pyx_float_35_46 __pyx_mstate_global->__pyx_float_35_46
#define __pyx_float_35_50 __pyx_mstate_global->__pyx_float_35_50
#define __pyx_float_35_59 __pyx_mstate_global->__pyx_float_35_59
#define __pyx_float_35_64 __pyx_mstate_global->__pyx_float_35_64
#define __pyx_float_35_67 __pyx_mstate_global->__pyx_float_35_67
#define __pyx_float_35_75 __pyx_mstate_global->__pyx_float_35_75
#define __pyx_float_36_00 __pyx_mstate_global->__pyx_float_36_00
#define __pyx_float_36_01 __pyx_mstate_global->__pyx_float_36_01
#define __pyx_float_36_02 __pyx_mstate_global->__pyx_float_36_02
#define __pyx_float_36_07 __pyx_mstate_global->__pyx_float_36_07
#define __pyx_float_36_09 __pyx_mstate_global->__pyx_float_36_09
#define __pyx_float_36_14 __pyx_mstate_global->__pyx_float_36_14
#define __pyx_float_36_16 __pyx_mstate_global->__pyx_float_36_16
#define __pyx_float_36_20 __pyx_mstate_global->__pyx_float_36_20
#define __pyx_float_36_25 __pyx_mstate_global->__pyx_float_36_25
#define __pyx_float_36_27 __pyx_mstate_global->__pyx_float_36_27
#define __pyx_float_36_28 __pyx_mstate_global->__pyx_float_36_28
#define __pyx_float_36_29 __pyx_mstate_global->__pyx_float_36_29
#define __pyx_float_36_30 __pyx_mstate_global->__pyx_float_36_30
#define __pyx_float_36_32 __pyx_mstate_global->__pyx_float_36_32
#define __pyx_float_36_34 __pyx_mstate_global->__pyx_float_36_34
#define __pyx_float_36_40 __pyx_mstate_global->__pyx_float_36_40
#define __pyx_float_36_43 __pyx_mstate_global->__pyx_float_36_43
#define __pyx_float_36_55 __pyx_mstate_global->__pyx_float_36_55
#define __pyx_float_36_56 __pyx_mstate_global->__pyx_float_36_56
#define __pyx_float_36_62 __pyx_mstate_global->__pyx_float_36_62
#define __pyx_float_36_74 __pyx_mstate_global->__pyx_float_36_74
#define __pyx_float_36_78 __pyx_mstate_global->__pyx_float_36_78
#define __pyx_float_36_88 __pyx_mstate_global->__pyx_float_36_88
#define __pyx_float_37_11 __pyx_mstate_global->__pyx_float_37_11
#define __pyx_float_37_16 __pyx_mstate_global->__pyx_float_37_16
#define __pyx_float_37_20 __pyx_mstate_global->__pyx_float_37_20
#define __pyx_float_37_23 __pyx_mstate_global->__pyx_float_37_23
#define __pyx_float_37_27 __pyx_mstate_global->__pyx_float_37_27
#define __pyx_float_37_32 __pyx_mstate_global->__pyx_float_37_32
#define __pyx_float_37_34 __pyx_mstate_global->__pyx_float_37_34
#define __pyx_float_37_35 __pyx_mstate_global->__pyx_float_37_35
#define __pyx_float_37_38 __pyx_mstate_global->__pyx_float_37_38
#define __pyx_float_37_39 __pyx_mstate_global->__pyx_float_37_39
#define __pyx_float_37_41 __pyx_mstate_global->__pyx_float_37_41
#define __pyx_float_37_51 __pyx_mstate_global->__pyx_float_37_51
#define __pyx_float_37_56 __pyx_mstate_global->__pyx_float_37_56
#define __pyx_float_37_57 __pyx_mstate_global->__pyx_float_37_57
#define __pyx_float_37_68 __pyx_mstate_global->__pyx_float_37_68
#define __pyx_float_37_69 __pyx_mstate_global->__pyx_float_37_69
#define __pyx_float_37_76 __pyx_mstate_global->__pyx_float_37_76
#define __pyx_float_37_77 __pyx_mstate_global->__pyx_float_37_77
#define __pyx_float_37_88 __pyx_mstate_global->__pyx_float_37_88
#define __pyx_float_37_89 __pyx_mstate_global->__pyx_float_37_89
#define __pyx_float_37_90 __pyx_mstate_global->__pyx_float_37_90
#define __pyx_float_37_93 __pyx_mstate_global->__pyx_float_37_93
#define __pyx_float_37_99 __pyx_mstate_global->__pyx_float_37_99
#define __pyx_float_38_10 __pyx_mstate_global->__pyx_float_38_10
#define __pyx_float_38_18 __pyx_mstate_global->__pyx_float_38_18
#define __pyx_float_38_20 __pyx_mstate_global->__pyx_float_38_20
#define __pyx_float_38_25 __pyx_mstate_global->__pyx_float_38_25
#define __pyx_float_38_26 __pyx_mstate_global->__pyx_float_38_26
#define __pyx_float_38_30 __pyx_mstate_global->__pyx_float_38_30
#define __pyx_float_38_33 __pyx_mstate_global->__pyx_float_38_33
#define __pyx_float_38_36 __pyx_mstate_global->__pyx_float_38_36
#define __pyx_float_38_46 __pyx_mstate_global->__pyx_float_38_46
#define __pyx_float_38_51 __pyx_mstate_global->__pyx_float_38_51
#define __pyx_float_38_53 __pyx_mstate_global->__pyx_float_38_53
#define __pyx_float_38_54 __pyx_mstate_global->__pyx_float_38_54
#define __pyx_float_38_55 __pyx_mstate_global->__pyx_float_38_55
#define __pyx_float_38_58 __pyx_mstate_global->__pyx_float_38_58
#define __pyx_float_38_64 __pyx_mstate_global->__pyx_float_38_64
#define __pyx_float_38_68 __pyx_mstate_global->__pyx_float_38_68
#define __pyx_float_38_80 __pyx_mstate_global->__pyx_float_38_80
#define __pyx_float_38_85 __pyx_mstate_global->__pyx_float_38_85
#define __pyx_float_38_86 __pyx_mstate_global->__pyx_float_38_86
#define __pyx_float_38_92 __pyx_mstate_global->__pyx_float_38_92
#define __pyx_float_38_93 __pyx_mstate_global->__pyx_float_38_93
#define __pyx_float_39_00 __pyx_mstate_global->__pyx_float_39_00
#define __pyx_float_39_03 __pyx_mstate_global->__pyx_float_39_03
#define __pyx_float_39_06 __pyx_mstate_global->__pyx_float_39_06
#define __pyx_float_39_11 __pyx_mstate_global->__pyx_float_39_11
#define __pyx_float_39_17 __pyx_mstate_global->__pyx_float_39_17
#define __pyx_float_39_23 __pyx_mstate_global->__pyx_float_39_23
#define __pyx_float_39_29 __pyx_mstate_global->__pyx_float_39_29
#define __pyx_float_39_35 __pyx_mstate_global->__pyx_float_39_35
#define __pyx_float_39_41 __pyx_mstate_global->__pyx_float_39_41
#define __pyx_float_39_47 __pyx_mstate_global->__pyx_float_39_47
#define __pyx_float_39_59 __pyx_mstate_global->__pyx_float_39_59
#define __pyx_float_39_60 __pyx_mstate_global->__pyx_float_39_60
#define __pyx_float_39_62 __pyx_mstate_global->__pyx_float_39_62
#define __pyx_float_39_64 __pyx_mstate_global->__pyx_float_39_64
#define __pyx_float_39_65 __pyx_mstate_global->__pyx_float_39_65
#define __pyx_float_39_66 __pyx_mstate_global->__pyx_float_39_66
#define __pyx_float_39_67 __pyx_mstate_global->__pyx_float_39_67
#define __pyx_float_39_75 __pyx_mstate_global->__pyx_float_39_75
#define __pyx_float_39_80 __pyx_mstate_global->__pyx_float_39_80
#define __pyx_float_39_88 __pyx_mstate_global->__pyx_float_39_88
#define __pyx_float_40_02 __pyx_mstate_global->__pyx_float_40_02
#define __pyx_float_40_03 __pyx_mstate_global->__pyx_float_40_03
#define __pyx_float_40_05 __pyx_mstate_global->__pyx_float_40_05
#define __pyx_float_40_07 __pyx_mstate_global->__pyx_float_40_07
#define __pyx_float_40_08 __pyx_mstate_global->__pyx_float_40_08
#define __pyx_float_40_09 __pyx_mstate_global->__pyx_float_40_09
#define __pyx_float_40_11 __pyx_mstate_global->__pyx_float_40_11
#define __pyx_float_40_19 __pyx_mstate_global->__pyx_float_40_19
#define __pyx_float_40_25 __pyx_mstate_global->__pyx_float_40_25
#define __pyx_float_40_32 __pyx_mstate_global->__pyx_float_40_32
#define __pyx_float_40_33 __pyx_mstate_global->__pyx_float_40_33
#define __pyx_float_40_35 __pyx_mstate_global->__pyx_float_40_35
#define __pyx_float_40_43 __pyx_mstate_global->__pyx_float_40_43
#define __pyx_float_40_53 __pyx_mstate_global->__pyx_float_40_53
#define __pyx_float_40_54 __pyx_mstate_global->__pyx_float_40_54
#define __pyx_float_40_55 __pyx_mstate_global->__pyx_float_40_55
#define __pyx_float_40_58 __pyx_mstate_global->__pyx_float_40_58
#define __pyx_float_40_69 __pyx_mstate_global->__pyx_float_40_69
#define __pyx_float_40_71 __pyx_mstate_global->__pyx_float_40_71
#define __pyx_float_40_77 __pyx_mstate_global->__pyx_float_40_77
#define __pyx_float_40_80 __pyx_mstate_global->__pyx_float_40_80
#define __pyx_float_40_85 __pyx_mstate_global->__pyx_float_40_85
#define __pyx_float_40_90 __pyx_mstate_global->__pyx_float_40_90
#define __pyx_float_40_95 __pyx_mstate_global->__pyx_float_40_95
#define __pyx_float_40_97 __pyx_mstate_global->__pyx_float_40_97
#define __pyx_float_41_00 __pyx_mstate_global->__pyx_float_41_00
#define __pyx_float_41_16 __pyx_mstate_global->__pyx_float_41_16
#define __pyx_float_41_20 __pyx_mstate_global->__pyx_float_41_20
#define __pyx_float_41_23 __pyx_mstate_global->__pyx_float_41_23
#define __pyx_float_41_28 __pyx_mstate_global->__pyx_float_41_28
#define __pyx_float_41_37 __pyx_mstate_global->__pyx_float_41_37
#define __pyx_float_41_40 __pyx_mstate_global->__pyx_float_41_40
#define __pyx_float_41_43 __pyx_mstate_global->__pyx_float_41_43
#define __pyx_float_41_47 __pyx_mstate_global->__pyx_float_41_47
#define __pyx_float_41_50 __pyx_mstate_global->__pyx_float_41_50
#define __pyx_float_41_51 __pyx_mstate_global->__pyx_float_41_51
#define __pyx_float_41_52 __pyx_mstate_global->__pyx_float_41_52
#define __pyx_float_41_53 __pyx_mstate_global->__pyx_float_41_53
#define __pyx_float_41_61 __pyx_mstate_global->__pyx_float_41_61
#define __pyx_float_41_77 __pyx_mstate_global->__pyx_float_41_77
#define __pyx_float_41_82 __pyx_mstate_global->__pyx_float_41_82
#define __pyx_float_41_86 __pyx_mstate_global->__pyx_float_41_86
#define __pyx_float_41_89 __pyx_mstate_global->__pyx_float_41_89
#define __pyx_float_41_90 __pyx_mstate_global->__pyx_float_41_90
#define __pyx_float_41_93 __pyx_mstate_global->__pyx_float_41_93
#define __pyx_float_41_97 __pyx_mstate_global->__pyx_float_41_97
#define __pyx_float_42_09 __pyx_mstate_global->__pyx_float_42_09
#define __pyx_float_42_23 __pyx_mstate_global->__pyx_float_42_23
#define __pyx_float_42_37 __pyx_mstate_global->__pyx_float_42_37
#define __pyx_float_42_42 __pyx_mstate_global->__pyx_float_42_42
#define __pyx_float_42_43 __pyx_mstate_global->__pyx_float_42_43
#define __pyx_float_42_50 __pyx_mstate_global->__pyx_float_42_50
#define __pyx_float_42_53 __pyx_mstate_global->__pyx_float_42_53
#define __pyx_float_42_66 __pyx_mstate_global->__pyx_float_42_66
#define __pyx_float_42_67 __pyx_mstate_global->__pyx_float_42_67
#define __pyx_float_42_80 __pyx_mstate_global->__pyx_float_42_80
#define __pyx_float_42_92 __pyx_mstate_global->__pyx_float_42_92
#define __pyx_float_42_95 __pyx_mstate_global->__pyx_float_42_95
#define __pyx_float_43_04 __pyx_mstate_global->__pyx_float_43_04
#define __pyx_float_43_06 __pyx_mstate_global->__pyx_float_43_06
#define __pyx_float_43_08 __pyx_mstate_global->__pyx_float_43_08
#define __pyx_float_43_24 __pyx_mstate_global->__pyx_float_43_24
#define __pyx_float_43_25 __pyx_mstate_global->__pyx_float_43_25
#define __pyx_float_43_27 __pyx_mstate_global->__pyx_float_43_27
#define __pyx_float_43_35 __pyx_mstate_global->__pyx_float_43_35
#define __pyx_float_43_38 __pyx_mstate_global->__pyx_float_43_38
#define __pyx_float_43_40 __pyx_mstate_global->__pyx_float_43_40
#define __pyx_float_43_41 __pyx_mstate_global->__pyx_float_43_41
#define __pyx_float_43_46 __pyx_mstate_global->__pyx_float_43_46
#define __pyx_float_43_48 __pyx_mstate_global->__pyx_float_43_48
#define __pyx_float_43_49 __pyx_mstate_global->__pyx_float_43_49
#define __pyx_float_43_54 __pyx_mstate_global->__pyx_float_43_54
#define __pyx_float_43_60 __pyx_mstate_global->__pyx_float_43_60
#define __pyx_float_43_65 __pyx_mstate_global->__pyx_float_43_65
#define __pyx_float_43_74 __pyx_mstate_global->__pyx_float_43_74
#define __pyx_float_43_75 __pyx_mstate_global->__pyx_float_43_75
#define __pyx_float_43_87 __pyx_mstate_global->__pyx_float_43_87
#define __pyx_float_43_89 __pyx_mstate_global->__pyx_float_43_89
#define __pyx_float_43_93 __pyx_mstate_global->__pyx_float_43_93
#define __pyx_float_44_02 __pyx_mstate_global->__pyx_float_44_02
#define __pyx_float_44_03 __pyx_mstate_global->__pyx_float_44_03
#define __pyx_float_44_10 __pyx_mstate_global->__pyx_float_44_10
#define __pyx_float_44_14 __pyx_mstate_global->__pyx_float_44_14
#define __pyx_float_44_17 __pyx_mstate_global->__pyx_float_44_17
#define __pyx_float_44_21 __pyx_mstate_global->__pyx_float_44_21
#define __pyx_float_44_24 __pyx_mstate_global->__pyx_float_44_24
#define __pyx_float_44_31 __pyx_mstate_global->__pyx_float_44_31
#define __pyx_float_44_48 __pyx_mstate_global->__pyx_float_44_48
#define __pyx_float_44_51 __pyx_mstate_global->__pyx_float_44_51
#define __pyx_float_44_52 __pyx_mstate_global->__pyx_float_44_52
#define __pyx_float_44_53 __pyx_mstate_global->__pyx_float_44_53
#define __pyx_float_44_68 __pyx_mstate_global->__pyx_float_44_68
#define __pyx_float_44_73 __pyx_mstate_global->__pyx_float_44_73
#define __pyx_float_44_74 __pyx_mstate_global->__pyx_float_44_74
#define __pyx_float_44_94 __pyx_mstate_global->__pyx_float_44_94
#define __pyx_float_45_03 __pyx_mstate_global->__pyx_float_45_03
#define __pyx_float_45_05 __pyx_mstate_global->__pyx_float_45_05
#define __pyx_float_45_12 __pyx_mstate_global->__pyx_float_45_12
#define __pyx_float_45_13 __pyx_mstate_global->__pyx_float_45_13
#define __pyx_float_45_24 __pyx_mstate_global->__pyx_float_45_24
#define __pyx_float_45_25 __pyx_mstate_global->__pyx_float_45_25
#define __pyx_float_45_29 __pyx_mstate_global->__pyx_float_45_29
#define __pyx_float_45_36 __pyx_mstate_global->__pyx_float_45_36
#define __pyx_float_45_38 __pyx_mstate_global->__pyx_float_45_38
#define __pyx_float_45_42 __pyx_mstate_global->__pyx_float_45_42
#define __pyx_float_45_49 __pyx_mstate_global->__pyx_float_45_49
#define __pyx_float_45_62 __pyx_mstate_global->__pyx_float_45_62
#define __pyx_float_45_65 __pyx_mstate_global->__pyx_float_45_65
#define __pyx_float_45_68 __pyx_mstate_global->__pyx_float_45_68
#define __pyx_float_45_85 __pyx_mstate_global->__pyx_float_45_85
#define __pyx_float_45_95 __pyx_mstate_global->__pyx_float_45_95
#define __pyx_float_45_99 __pyx_mstate_global->__pyx_float_45_99
#define __pyx_float_46_02 __pyx_mstate_global->__pyx_float_46_02
#define __pyx_float_46_11 __pyx_mstate_global->__pyx_float_46_11
#define __pyx_float_46_13 __pyx_mstate_global->__pyx_float_46_13
#define __pyx_float_46_18 __pyx_mstate_global->__pyx_float_46_18
#define __pyx_float_46_20 __pyx_mstate_global->__pyx_float_46_20
#define __pyx_float_46_33 __pyx_mstate_global->__pyx_float_46_33
#define __pyx_float_46_35 __pyx_mstate_global->__pyx_float_46_35
#define __pyx_float_46_36 __pyx_mstate_global->__pyx_float_46_36
#define __pyx_float_46_42 __pyx_mstate_global->__pyx_float_46_42
#define __pyx_float_46_43 __pyx_mstate_global->__pyx_float_46_43
#define __pyx_float_46_54 __pyx_mstate_global->__pyx_float_46_54
#define __pyx_float_46_56 __pyx_mstate_global->__pyx_float_46_56
#define __pyx_float_46_57 __pyx_mstate_global->__pyx_float_46_57
#define __pyx_float_46_62 __pyx_mstate_global->__pyx_float_46_62
#define __pyx_float_46_65 __pyx_mstate_global->__pyx_float_46_65
#define __pyx_float_46_68 __pyx_mstate_global->__pyx_float_46_68
#define __pyx_float_46_86 __pyx_mstate_global->__pyx_float_46_86
#define __pyx_float_46_93 __pyx_mstate_global->__pyx_float_46_93
#define __pyx_float_47_00 __pyx_mstate_global->__pyx_float_47_00
#define __pyx_float_47_07 __pyx_mstate_global->__pyx_float_47_07
#define __pyx_float_47_11 __pyx_mstate_global->__pyx_float_47_11
#define __pyx_float_47_16 __pyx_mstate_global->__pyx_float_47_16
#define __pyx_float_47_20 __pyx_mstate_global->__pyx_float_47_20
#define __pyx_float_47_21 __pyx_mstate_global->__pyx_float_47_21
#define __pyx_float_47_25 __pyx_mstate_global->__pyx_float_47_25
#define __pyx_float_47_40 __pyx_mstate_global->__pyx_float_47_40
#define __pyx_float_47_41 __pyx_mstate_global->__pyx_float_47_41
#define __pyx_float_47_46 __pyx_mstate_global->__pyx_float_47_46
#define __pyx_float_47_50 __pyx_mstate_global->__pyx_float_47_50
#define __pyx_float_47_60 __pyx_mstate_global->__pyx_float_47_60
#define __pyx_float_47_64 __pyx_mstate_global->__pyx_float_47_64
#define __pyx_float_47_65 __pyx_mstate_global->__pyx_float_47_65
#define __pyx_float_47_72 __pyx_mstate_global->__pyx_float_47_72
#define __pyx_float_47_78 __pyx_mstate_global->__pyx_float_47_78
#define __pyx_float_47_79 __pyx_mstate_global->__pyx_float_47_79
#define __pyx_float_47_81 __pyx_mstate_global->__pyx_float_47_81
#define __pyx_float_47_82 __pyx_mstate_global->__pyx_float_47_82
#define __pyx_float_47_87 __pyx_mstate_global->__pyx_float_47_87
#define __pyx_float_48_07 __pyx_mstate_global->__pyx_float_48_07
#define __pyx_float_48_11 __pyx_mstate_global->__pyx_float_48_11
#define __pyx_float_48_28 __pyx_mstate_global->__pyx_float_48_28
#define __pyx_float_48_36 __pyx_mstate_global->__pyx_float_48_36
#define __pyx_float_48_40 __pyx_mstate_global->__pyx_float_48_40
#define __pyx_float_48_42 __pyx_mstate_global->__pyx_float_48_42
#define __pyx_float_48_43 __pyx_mstate_global->__pyx_float_48_43
#define __pyx_float_48_44 __pyx_mstate_global->__pyx_float_48_44
#define __pyx_float_48_52 __pyx_mstate_global->__pyx_float_48_52
#define __pyx_float_48_57 __pyx_mstate_global->__pyx_float_48_57
#define __pyx_float_48_65 __pyx_mstate_global->__pyx_float_48_65
#define __pyx_float_48_68 __pyx_mstate_global->__pyx_float_48_68
#define __pyx_float_48_71 __pyx_mstate_global->__pyx_float_48_71
#define __pyx_float_48_76 __pyx_mstate_global->__pyx_float_48_76
#define __pyx_float_48_79 __pyx_mstate_global->__pyx_float_48_79
#define __pyx_float_48_83 __pyx_mstate_global->__pyx_float_48_83
#define __pyx_float_48_87 __pyx_mstate_global->__pyx_float_48_87
#define __pyx_float_48_88 __pyx_mstate_global->__pyx_float_48_88
#define __pyx_float_48_89 __pyx_mstate_global->__pyx_float_48_89
#define __pyx_float_48_95 __pyx_mstate_global->__pyx_float_48_95
#define __pyx_float_48_98 __pyx_mstate_global->__pyx_float_48_98
#define __pyx_float_48_99 __pyx_mstate_global->__pyx_float_48_99
#define __pyx_float_49_01 __pyx_mstate_global->__pyx_float_49_01
#define __pyx_float_49_14 __pyx_mstate_global->__pyx_float_49_14
#define __pyx_float_49_27 __pyx_mstate_global->__pyx_float_49_27
#define __pyx_float_49_29 __pyx_mstate_global->__pyx_float_49_29
#define __pyx_float_49_44 __pyx_mstate_global->__pyx_float_49_44
#define __pyx_float_49_48 __pyx_mstate_global->__pyx_float_49_48
#define __pyx_float_49_55 __pyx_mstate_global->__pyx_float_49_55
#define __pyx_float_49_65 __pyx_mstate_global->__pyx_float_49_65
#define __pyx_float_49_67 __pyx_mstate_global->__pyx_float_49_67
#define __pyx_float_49_68 __pyx_mstate_global->__pyx_float_49_68
#define __pyx_float_49_87 __pyx_mstate_global->__pyx_float_49_87
#define __pyx_float_49_93 __pyx_mstate_global->__pyx_float_49_93
#define __pyx_float_49_95 __pyx_mstate_global->__pyx_float_49_95
#define __pyx_float_50_01 __pyx_mstate_global->__pyx_float_50_01
#define __pyx_float_50_03 __pyx_mstate_global->__pyx_float_50_03
#define __pyx_float_50_10 __pyx_mstate_global->__pyx_float_50_10
#define __pyx_float_50_11 __pyx_mstate_global->__pyx_float_50_11
#define __pyx_float_50_13 __pyx_mstate_global->__pyx_float_50_13
#define __pyx_float_50_18 __pyx_mstate_global->__pyx_float_50_18
#define __pyx_float_50_21 __pyx_mstate_global->__pyx_float_50_21
#define __pyx_float_50_48 __pyx_mstate_global->__pyx_float_50_48
#define __pyx_float_50_49 __pyx_mstate_global->__pyx_float_50_49
#define __pyx_float_50_63 __pyx_mstate_global->__pyx_float_50_63
#define __pyx_float_50_72 __pyx_mstate_global->__pyx_float_50_72
#define __pyx_float_50_76 __pyx_mstate_global->__pyx_float_50_76
#define __pyx_float_50_83 __pyx_mstate_global->__pyx_float_50_83
#define __pyx_float_50_88 __pyx_mstate_global->__pyx_float_50_88
#define __pyx_float_50_92 __pyx_mstate_global->__pyx_float_50_92
#define __pyx_float_50_96 __pyx_mstate_global->__pyx_float_50_96
#define __pyx_float_51_09 __pyx_mstate_global->__pyx_float_51_09
#define __pyx_float_51_12 __pyx_mstate_global->__pyx_float_51_12
#define __pyx_float_51_19 __pyx_mstate_global->__pyx_float_51_19
#define __pyx_float_51_27 __pyx_mstate_global->__pyx_float_51_27
#define __pyx_float_51_29 __pyx_mstate_global->__pyx_float_51_29
#define __pyx_float_51_34 __pyx_mstate_global->__pyx_float_51_34
#define __pyx_float_51_37 __pyx_mstate_global->__pyx_float_51_37
#define __pyx_float_51_42 __pyx_mstate_global->__pyx_float_51_42
#define __pyx_float_51_54 __pyx_mstate_global->__pyx_float_51_54
#define __pyx_float_51_62 __pyx_mstate_global->__pyx_float_51_62
#define __pyx_float_51_81 __pyx_mstate_global->__pyx_float_51_81
#define __pyx_float_51_93 __pyx_mstate_global->__pyx_float_51_93
#define __pyx_float_52_00 __pyx_mstate_global->__pyx_float_52_00
#define __pyx_float_52_15 __pyx_mstate_global->__pyx_float_52_15
#define __pyx_float_52_17 __pyx_mstate_global->__pyx_float_52_17
#define __pyx_float_52_33 __pyx_mstate_global->__pyx_float_52_33
#define __pyx_float_52_43 __pyx_mstate_global->__pyx_float_52_43
#define __pyx_float_52_47 __pyx_mstate_global->__pyx_float_52_47
#define __pyx_float_52_70 __pyx_mstate_global->__pyx_float_52_70
#define __pyx_float_52_73 __pyx_mstate_global->__pyx_float_52_73
#define __pyx_float_52_95 __pyx_mstate_global->__pyx_float_52_95
#define __pyx_float_53_42 __pyx_mstate_global->__pyx_float_53_42
#define __pyx_float_53_46 __pyx_mstate_global->__pyx_float_53_46
#define __pyx_float_53_52 __pyx_mstate_global->__pyx_float_53_52
#define __pyx_float_53_56 __pyx_mstate_global->__pyx_float_53_56
#define __pyx_float_53_68 __pyx_mstate_global->__pyx_float_53_68
#define __pyx_float_53_79 __pyx_mstate_global->__pyx_float_53_79
#define __pyx_float_53_82 __pyx_mstate_global->__pyx_float_53_82
#define __pyx_float_53_90 __pyx_mstate_global->__pyx_float_53_90
#define __pyx_float_53_91 __pyx_mstate_global->__pyx_float_53_91
#define __pyx_float_54_00 __pyx_mstate_global->__pyx_float_54_00
#define __pyx_float_54_05 __pyx_mstate_global->__pyx_float_54_05
#define __pyx_float_54_15 __pyx_mstate_global->__pyx_float_54_15
#define __pyx_float_54_23 __pyx_mstate_global->__pyx_float_54_23
#define __pyx_float_54_39 __pyx_mstate_global->__pyx_float_54_39
#define __pyx_float_54_69 __pyx_mstate_global->__pyx_float_54_69
#define __pyx_float_54_74 __pyx_mstate_global->__pyx_float_54_74
#define __pyx_float_54_75 __pyx_mstate_global->__pyx_float_54_75
#define __pyx_float_54_76 __pyx_mstate_global->__pyx_float_54_76
#define __pyx_float_55_02 __pyx_mstate_global->__pyx_float_55_02
#define __pyx_float_55_06 __pyx_mstate_global->__pyx_float_55_06
#define __pyx_float_55_09 __pyx_mstate_global->__pyx_float_55_09
#define __pyx_float_55_14 __pyx_mstate_global->__pyx_float_55_14
#define __pyx_float_55_24 __pyx_mstate_global->__pyx_float_55_24
#define __pyx_float_55_30 __pyx_mstate_global->__pyx_float_55_30
#define __pyx_float_55_31 __pyx_mstate_global->__pyx_float_55_31
#define __pyx_float_55_34 __pyx_mstate_global->__pyx_float_55_34
#define __pyx_float_55_45 __pyx_mstate_global->__pyx_float_55_45
#define __pyx_float_55_50 __pyx_mstate_global->__pyx_float_55_50
#define __pyx_float_55_58 __pyx_mstate_global->__pyx_float_55_58
#define __pyx_float_55_75 __pyx_mstate_global->__pyx_float_55_75
#define __pyx_float_55_88 __pyx_mstate_global->__pyx_float_55_88
#define __pyx_float_55_90 __pyx_mstate_global->__pyx_float_55_90
#define __pyx_float_55_98 __pyx_mstate_global->__pyx_float_55_98
#define __pyx_float_56_04 __pyx_mstate_global->__pyx_float_56_04
#define __pyx_float_56_06 __pyx_mstate_global->__pyx_float_56_06
#define __pyx_float_56_17 __pyx_mstate_global->__pyx_float_56_17
#define __pyx_float_56_18 __pyx_mstate_global->__pyx_float_56_18
#define __pyx_float_56_20 __pyx_mstate_global->__pyx_float_56_20
#define __pyx_float_56_23 __pyx_mstate_global->__pyx_float_56_23
#define __pyx_float_56_31 __pyx_mstate_global->__pyx_float_56_31
#define __pyx_float_56_60 __pyx_mstate_global->__pyx_float_56_60
#define __pyx_float_56_62 __pyx_mstate_global->__pyx_float_56_62
#define __pyx_float_56_68 __pyx_mstate_global->__pyx_float_56_68
#define __pyx_float_56_73 __pyx_mstate_global->__pyx_float_56_73
#define __pyx_float_56_90 __pyx_mstate_global->__pyx_float_56_90
#define __pyx_float_56_98 __pyx_mstate_global->__pyx_float_56_98
#define __pyx_float_57_02 __pyx_mstate_global->__pyx_float_57_02
#define __pyx_float_57_10 __pyx_mstate_global->__pyx_float_57_10
#define __pyx_float_57_21 __pyx_mstate_global->__pyx_float_57_21
#define __pyx_float_57_26 __pyx_mstate_global->__pyx_float_57_26
#define __pyx_float_57_34 __pyx_mstate_global->__pyx_float_57_34
#define __pyx_float_57_40 __pyx_mstate_global->__pyx_float_57_40
#define __pyx_float_57_41 __pyx_mstate_global->__pyx_float_57_41
#define __pyx_float_57_53 __pyx_mstate_global->__pyx_float_57_53
#define __pyx_float_57_63 __pyx_mstate_global->__pyx_float_57_63
#define __pyx_float_57_69 __pyx_mstate_global->__pyx_float_57_69
#define __pyx_float_57_81 __pyx_mstate_global->__pyx_float_57_81
#define __pyx_float_57_90 __pyx_mstate_global->__pyx_float_57_90
#define __pyx_float_57_96 __pyx_mstate_global->__pyx_float_57_96
#define __pyx_float_57_97 __pyx_mstate_global->__pyx_float_57_97
#define __pyx_float_57_98 __pyx_mstate_global->__pyx_float_57_98
#define __pyx_float_58_03 __pyx_mstate_global->__pyx_float_58_03
#define __pyx_float_58_11 __pyx_mstate_global->__pyx_float_58_11
#define __pyx_float_58_17 __pyx_mstate_global->__pyx_float_58_17
#define __pyx_float_58_28 __pyx_mstate_global->__pyx_float_58_28
#define __pyx_float_58_34 __pyx_mstate_global->__pyx_float_58_34
#define __pyx_float_58_42 __pyx_mstate_global->__pyx_float_58_42
#define __pyx_float_58_53 __pyx_mstate_global->__pyx_float_58_53
#define __pyx_float_58_56 __pyx_mstate_global->__pyx_float_58_56
#define __pyx_float_58_67 __pyx_mstate_global->__pyx_float_58_67
#define __pyx_float_58_71 __pyx_mstate_global->__pyx_float_58_71
#define __pyx_float_58_75 __pyx_mstate_global->__pyx_float_58_75
#define __pyx_float_58_79 __pyx_mstate_global->__pyx_float_58_79
#define __pyx_float_58_86 __pyx_mstate_global->__pyx_float_58_86
#define __pyx_float_59_02 __pyx_mstate_global->__pyx_float_59_02
#define __pyx_float_59_13 __pyx_mstate_global->__pyx_float_59_13
#define __pyx_float_59_16 __pyx_mstate_global->__pyx_float_59_16
#define __pyx_float_59_21 __pyx_mstate_global->__pyx_float_59_21
#define __pyx_float_59_29 __pyx_mstate_global->__pyx_float_59_29
#define __pyx_float_59_30 __pyx_mstate_global->__pyx_float_59_30
#define __pyx_float_59_33 __pyx_mstate_global->__pyx_float_59_33
#define __pyx_float_59_40 __pyx_mstate_global->__pyx_float_59_40
#define __pyx_float_59_47 __pyx_mstate_global->__pyx_float_59_47
#define __pyx_float_59_70 __pyx_mstate_global->__pyx_float_59_70
#define __pyx_float_59_81 __pyx_mstate_global->__pyx_float_59_81
#define __pyx_float_59_83 __pyx_mstate_global->__pyx_float_59_83
#define __pyx_float_59_86 __pyx_mstate_global->__pyx_float_59_86
#define __pyx_float_59_95 __pyx_mstate_global->__pyx_float_59_95
#define __pyx_float_59_96 __pyx_mstate_global->__pyx_float_59_96
#define __pyx_float_60_00 __pyx_mstate_global->__pyx_float_60_00
#define __pyx_float_60_03 __pyx_mstate_global->__pyx_float_60_03
#define __pyx_float_60_09 __pyx_mstate_global->__pyx_float_60_09
#define __pyx_float_60_23 __pyx_mstate_global->__pyx_float_60_23
#define __pyx_float_60_33 __pyx_mstate_global->__pyx_float_60_33
#define __pyx_float_60_35 __pyx_mstate_global->__pyx_float_60_35
#define __pyx_float_60_42 __pyx_mstate_global->__pyx_float_60_42
#define __pyx_float_60_59 __pyx_mstate_global->__pyx_float_60_59
#define __pyx_float_60_62 __pyx_mstate_global->__pyx_float_60_62
#define __pyx_float_60_67 __pyx_mstate_global->__pyx_float_60_67
#define __pyx_float_60_75 __pyx_mstate_global->__pyx_float_60_75
#define __pyx_float_61_05 __pyx_mstate_global->__pyx_float_61_05
#define __pyx_float_61_13 __pyx_mstate_global->__pyx_float_61_13
#define __pyx_float_61_21 __pyx_mstate_global->__pyx_float_61_21
#define __pyx_float_61_29 __pyx_mstate_global->__pyx_float_61_29
#define __pyx_float_61_34 __pyx_mstate_global->__pyx_float_61_34
#define __pyx_float_61_37 __pyx_mstate_global->__pyx_float_61_37
#define __pyx_float_61_39 __pyx_mstate_global->__pyx_float_61_39
#define __pyx_float_61_40 __pyx_mstate_global->__pyx_float_61_40
#define __pyx_float_61_56 __pyx_mstate_global->__pyx_float_61_56
#define __pyx_float_61_59 __pyx_mstate_global->__pyx_float_61_59
#define __pyx_float_61_62 __pyx_mstate_global->__pyx_float_61_62
#define __pyx_float_61_66 __pyx_mstate_global->__pyx_float_61_66
#define __pyx_float_61_99 __pyx_mstate_global->__pyx_float_61_99
#define __pyx_float_62_04 __pyx_mstate_global->__pyx_float_62_04
#define __pyx_float_62_20 __pyx_mstate_global->__pyx_float_62_20
#define __pyx_float_62_22 __pyx_mstate_global->__pyx_float_62_22
#define __pyx_float_62_54 __pyx_mstate_global->__pyx_float_62_54
#define __pyx_float_62_56 __pyx_mstate_global->__pyx_float_62_56
#define __pyx_float_62_72 __pyx_mstate_global->__pyx_float_62_72
#define __pyx_float_62_84 __pyx_mstate_global->__pyx_float_62_84
#define __pyx_float_62_86 __pyx_mstate_global->__pyx_float_62_86
#define __pyx_float_63_08 __pyx_mstate_global->__pyx_float_63_08
#define __pyx_float_63_48 __pyx_mstate_global->__pyx_float_63_48
#define __pyx_float_63_55 __pyx_mstate_global->__pyx_float_63_55
#define __pyx_float_63_78 __pyx_mstate_global->__pyx_float_63_78
#define __pyx_float_63_97 __pyx_mstate_global->__pyx_float_63_97
#define __pyx_float_64_29 __pyx_mstate_global->__pyx_float_64_29
#define __pyx_float_64_36 __pyx_mstate_global->__pyx_float_64_36
#define __pyx_float_64_82 __pyx_mstate_global->__pyx_float_64_82
#define __pyx_float_64_83 __pyx_mstate_global->__pyx_float_64_83
#define __pyx_float_64_86 __pyx_mstate_global->__pyx_float_64_86
#define __pyx_float_64_97 __pyx_mstate_global->__pyx_float_64_97
#define __pyx_float_65_09 __pyx_mstate_global->__pyx_float_65_09
#define __pyx_float_65_19 __pyx_mstate_global->__pyx_float_65_19
#define __pyx_float_65_20 __pyx_mstate_global->__pyx_float_65_20
#define __pyx_float_65_42 __pyx_mstate_global->__pyx_float_65_42
#define __pyx_float_65_54 __pyx_mstate_global->__pyx_float_65_54
#define __pyx_float_65_65 __pyx_mstate_global->__pyx_float_65_65
#define __pyx_float_65_89 __pyx_mstate_global->__pyx_float_65_89
#define __pyx_float_66_12 __pyx_mstate_global->__pyx_float_66_12
#define __pyx_float_66_24 __pyx_mstate_global->__pyx_float_66_24
#define __pyx_float_66_26 __pyx_mstate_global->__pyx_float_66_26
#define __pyx_float_66_34 __pyx_mstate_global->__pyx_float_66_34
#define __pyx_float_66_44 __pyx_mstate_global->__pyx_float_66_44
#define __pyx_float_66_47 __pyx_mstate_global->__pyx_float_66_47
#define __pyx_float_66_48 __pyx_mstate_global->__pyx_float_66_48
#define __pyx_float_66_56 __pyx_mstate_global->__pyx_float_66_56
#define __pyx_float_66_60 __pyx_mstate_global->__pyx_float_66_60
#define __pyx_float_66_64 __pyx_mstate_global->__pyx_float_66_64
#define __pyx_float_66_72 __pyx_mstate_global->__pyx_float_66_72
#define __pyx_float_66_91 __pyx_mstate_global->__pyx_float_66_91
#define __pyx_float_66_92 __pyx_mstate_global->__pyx_float_66_92
#define __pyx_float_66_96 __pyx_mstate_global->__pyx_float_66_96
#define __pyx_float_67_18 __pyx_mstate_global->__pyx_float_67_18
#define __pyx_float_67_23 __pyx_mstate_global->__pyx_float_67_23
#define __pyx_float_67_37 __pyx_mstate_global->__pyx_float_67_37
#define __pyx_float_67_46 __pyx_mstate_global->__pyx_float_67_46
#define __pyx_float_67_63 __pyx_mstate_global->__pyx_float_67_63
#define __pyx_float_67_70 __pyx_mstate_global->__pyx_float_67_70
#define __pyx_float_67_83 __pyx_mstate_global->__pyx_float_67_83
#define __pyx_float_67_94 __pyx_mstate_global->__pyx_float_67_94
#define __pyx_float_67_97 __pyx_mstate_global->__pyx_float_67_97
#define __pyx_float_67_98 __pyx_mstate_global->__pyx_float_67_98
#define __pyx_float_68_18 __pyx_mstate_global->__pyx_float_68_18
#define __pyx_float_68_50 __pyx_mstate_global->__pyx_float_68_50
#define __pyx_float_68_61 __pyx_mstate_global->__pyx_float_68_61
#define __pyx_float_68_65 __pyx_mstate_global->__pyx_float_68_65
#define __pyx_float_68_81 __pyx_mstate_global->__pyx_float_68_81
#define __pyx_float_68_85 __pyx_mstate_global->__pyx_float_68_85
#define __pyx_float_68_90 __pyx_mstate_global->__pyx_float_68_90
#define __pyx_float_68_94 __pyx_mstate_global->__pyx_float_68_94
#define __pyx_float_68_99 __pyx_mstate_global->__pyx_float_68_99
#define __pyx_float_69_09 __pyx_mstate_global->__pyx_float_69_09
#define __pyx_float_69_13 __pyx_mstate_global->__pyx_float_69_13
#define __pyx_float_69_17 __pyx_mstate_global->__pyx_float_69_17
#define __pyx_float_69_33 __pyx_mstate_global->__pyx_float_69_33
#define __pyx_float_69_37 __pyx_mstate_global->__pyx_float_69_37
#define __pyx_float_69_60 __pyx_mstate_global->__pyx_float_69_60
#define __pyx_float_69_78 __pyx_mstate_global->__pyx_float_69_78
#define __pyx_float_69_91 __pyx_mstate_global->__pyx_float_69_91
#define __pyx_float_69_97 __pyx_mstate_global->__pyx_float_69_97
#define __pyx_float_70_10 __pyx_mstate_global->__pyx_float_70_10
#define __pyx_float_70_25 __pyx_mstate_global->__pyx_float_70_25
#define __pyx_float_70_42 __pyx_mstate_global->__pyx_float_70_42
#define __pyx_float_70_43 __pyx_mstate_global->__pyx_float_70_43
#define __pyx_float_70_52 __pyx_mstate_global->__pyx_float_70_52
#define __pyx_float_70_53 __pyx_mstate_global->__pyx_float_70_53
#define __pyx_float_70_58 __pyx_mstate_global->__pyx_float_70_58
#define __pyx_float_70_68 __pyx_mstate_global->__pyx_float_70_68
#define __pyx_float_70_73 __pyx_mstate_global->__pyx_float_70_73
#define __pyx_float_70_88 __pyx_mstate_global->__pyx_float_70_88
#define __pyx_float_71_17 __pyx_mstate_global->__pyx_float_71_17
#define __pyx_float_71_21 __pyx_mstate_global->__pyx_float_71_21
#define __pyx_float_71_25 __pyx_mstate_global->__pyx_float_71_25
#define __pyx_float_71_36 __pyx_mstate_global->__pyx_float_71_36
#define __pyx_float_71_59 __pyx_mstate_global->__pyx_float_71_59
#define __pyx_float_71_72 __pyx_mstate_global->__pyx_float_71_72
#define __pyx_float_71_84 __pyx_mstate_global->__pyx_float_71_84
#define __pyx_float_72_00 __pyx_mstate_global->__pyx_float_72_00
#define __pyx_float_72_01 __pyx_mstate_global->__pyx_float_72_01
#define __pyx_float_72_14 __pyx_mstate_global->__pyx_float_72_14
#define __pyx_float_72_24 __pyx_mstate_global->__pyx_float_72_24
#define __pyx_float_72_45 __pyx_mstate_global->__pyx_float_72_45
#define __pyx_float_73_05 __pyx_mstate_global->__pyx_float_73_05
#define __pyx_float_73_28 __pyx_mstate_global->__pyx_float_73_28
#define __pyx_float_73_50 __pyx_mstate_global->__pyx_float_73_50
#define __pyx_float_73_53 __pyx_mstate_global->__pyx_float_73_53
#define __pyx_float_73_57 __pyx_mstate_global->__pyx_float_73_57
#define __pyx_float_73_62 __pyx_mstate_global->__pyx_float_73_62
#define __pyx_float_73_68 __pyx_mstate_global->__pyx_float_73_68
#define __pyx_float_74_22 __pyx_mstate_global->__pyx_float_74_22
#define __pyx_float_74_40 __pyx_mstate_global->__pyx_float_74_40
#define __pyx_float_74_95 __pyx_mstate_global->__pyx_float_74_95
#define __pyx_float_75_00 __pyx_mstate_global->__pyx_float_75_00
#define __pyx_float_75_44 __pyx_mstate_global->__pyx_float_75_44
#define __pyx_float_75_49 __pyx_mstate_global->__pyx_float_75_49
#define __pyx_float_75_52 __pyx_mstate_global->__pyx_float_75_52
#define __pyx_float_75_76 __pyx_mstate_global->__pyx_float_75_76
#define __pyx_float_75_92 __pyx_mstate_global->__pyx_float_75_92
#define __pyx_float_76_12 __pyx_mstate_global->__pyx_float_76_12
#define __pyx_float_76_45 __pyx_mstate_global->__pyx_float_76_45
#define __pyx_float_76_52 __pyx_mstate_global->__pyx_float_76_52
#define __pyx_float_76_53 __pyx_mstate_global->__pyx_float_76_53
#define __pyx_float_76_96 __pyx_mstate_global->__pyx_float_76_96
#define __pyx_float_77_03 __pyx_mstate_global->__pyx_float_77_03
#define __pyx_float_77_08 __pyx_mstate_global->__pyx_float_77_08
#define __pyx_float_77_13 __pyx_mstate_global->__pyx_float_77_13
#define __pyx_float_77_93 __pyx_mstate_global->__pyx_float_77_93
#define __pyx_float_77_97 __pyx_mstate_global->__pyx_float_77_97
#define __pyx_float_78_00 __pyx_mstate_global->__pyx_float_78_00
#define __pyx_float_78_02 __pyx_mstate_global->__pyx_float_78_02
#define __pyx_float_78_31 __pyx_mstate_global->__pyx_float_78_31
#define __pyx_float_78_32 __pyx_mstate_global->__pyx_float_78_32
#define __pyx_float_78_45 __pyx_mstate_global->__pyx_float_78_45
#define __pyx_float_78_94 __pyx_mstate_global->__pyx_float_78_94
#define __pyx_float_79_27 __pyx_mstate_global->__pyx_float_79_27
#define __pyx_float_79_38 __pyx_mstate_global->__pyx_float_79_38
#define __pyx_float_79_48 __pyx_mstate_global->__pyx_float_79_48
#define __pyx_float_79_49 __pyx_mstate_global->__pyx_float_79_49
#define __pyx_float_79_57 __pyx_mstate_global->__pyx_float_79_57
#define __pyx_float_79_65 __pyx_mstate_global->__pyx_float_79_65
#define __pyx_float_79_73 __pyx_mstate_global->__pyx_float_79_73
#define __pyx_float_79_83 __pyx_mstate_global->__pyx_float_79_83
#define __pyx_float_80_16 __pyx_mstate_global->__pyx_float_80_16
#define __pyx_float_80_39 __pyx_mstate_global->__pyx_float_80_39
#define __pyx_float_80_44 __pyx_mstate_global->__pyx_float_80_44
#define __pyx_float_80_77 __pyx_mstate_global->__pyx_float_80_77
#define __pyx_float_81_00 __pyx_mstate_global->__pyx_float_81_00
#define __pyx_float_81_16 __pyx_mstate_global->__pyx_float_81_16
#define __pyx_float_81_33 __pyx_mstate_global->__pyx_float_81_33
#define __pyx_float_81_56 __pyx_mstate_global->__pyx_float_81_56
#define __pyx_float_81_69 __pyx_mstate_global->__pyx_float_81_69
#define __pyx_float_81_75 __pyx_mstate_global->__pyx_float_81_75
#define __pyx_float_81_87 __pyx_mstate_global->__pyx_float_81_87
#define __pyx_float_82_06 __pyx_mstate_global->__pyx_float_82_06
#define __pyx_float_82_39 __pyx_mstate_global->__pyx_float_82_39
#define __pyx_float_82_44 __pyx_mstate_global->__pyx_float_82_44
#define __pyx_float_82_47 __pyx_mstate_global->__pyx_float_82_47
#define __pyx_float_82_52 __pyx_mstate_global->__pyx_float_82_52
#define __pyx_float_82_59 __pyx_mstate_global->__pyx_float_82_59
#define __pyx_float_82_70 __pyx_mstate_global->__pyx_float_82_70
#define __pyx_float_82_99 __pyx_mstate_global->__pyx_float_82_99
#define __pyx_float_83_17 __pyx_mstate_global->__pyx_float_83_17
#define __pyx_float_83_53 __pyx_mstate_global->__pyx_float_83_53
#define __pyx_float_83_91 __pyx_mstate_global->__pyx_float_83_91
#define __pyx_float_83_97 __pyx_mstate_global->__pyx_float_83_97
#define __pyx_float_84_01 __pyx_mstate_global->__pyx_float_84_01
#define __pyx_float_84_10 __pyx_mstate_global->__pyx_float_84_10
#define __pyx_float_84_22 __pyx_mstate_global->__pyx_float_84_22
#define __pyx_float_84_61 __pyx_mstate_global->__pyx_float_84_61
#define __pyx_float_84_81 __pyx_mstate_global->__pyx_float_84_81
#define __pyx_float_85_28 __pyx_mstate_global->__pyx_float_85_28
#define __pyx_float_85_41 __pyx_mstate_global->__pyx_float_85_41
#define __pyx_float_85_42 __pyx_mstate_global->__pyx_float_85_42
#define __pyx_float_85_97 __pyx_mstate_global->__pyx_float_85_97
#define __pyx_float_86_88 __pyx_mstate_global->__pyx_float_86_88
#define __pyx_float_87_27 __pyx_mstate_global->__pyx_float_87_27
#define __pyx_float_87_46 __pyx_mstate_global->__pyx_float_87_46
#define __pyx_float_87_56 __pyx_mstate_global->__pyx_float_87_56
#define __pyx_float_87_65 __pyx_mstate_global->__pyx_float_87_65
#define __pyx_float_88_36 __pyx_mstate_global->__pyx_float_88_36
#define __pyx_float_88_39 __pyx_mstate_global->__pyx_float_88_39
#define __pyx_float_89_08 __pyx_mstate_global->__pyx_float_89_08
#define __pyx_float_89_85 __pyx_mstate_global->__pyx_float_89_85
#define __pyx_float_90_29 __pyx_mstate_global->__pyx_float_90_29
#define __pyx_float_90_30 __pyx_mstate_global->__pyx_float_90_30
#define __pyx_float_90_55 __pyx_mstate_global->__pyx_float_90_55
#define __pyx_float_90_70 __pyx_mstate_global->__pyx_float_90_70
#define __pyx_float_91_35 __pyx_mstate_global->__pyx_float_91_35
#define __pyx_float_91_75 __pyx_mstate_global->__pyx_float_91_75
#define __pyx_float_92_09 __pyx_mstate_global->__pyx_float_92_09
#define __pyx_float_92_30 __pyx_mstate_global->__pyx_float_92_30
#define __pyx_float_92_86 __pyx_mstate_global->__pyx_float_92_86
#define __pyx_float_93_28 __pyx_mstate_global->__pyx_float_93_28
#define __pyx_float_93_40 __pyx_mstate_global->__pyx_float_93_40
#define __pyx_float_93_52 __pyx_mstate_global->__pyx_float_93_52
#define __pyx_float_94_02 __pyx_mstate_global->__pyx_float_94_02
#define __pyx_float_95_26 __pyx_mstate_global->__pyx_float_95_26
#define __pyx_float_95_59 __pyx_mstate_global->__pyx_float_95_59
#define __pyx_float_97_07 __pyx_mstate_global->__pyx_float_97_07
#define __pyx_float_97_22 __pyx_mstate_global->__pyx_float_97_22
#define __pyx_float_98_86 __pyx_mstate_global->__pyx_float_98_86
#define __pyx_float_105_28 __pyx_mstate_global->__pyx_float_105_28
#define __pyx_float_106_22 __pyx_mstate_global->__pyx_float_106_22
#define __pyx_float_107_45 __pyx_mstate_global->__pyx_float_107_45
#define __pyx_float_108_37 __pyx_mstate_global->__pyx_float_108_37
#define __pyx_float_108_70 __pyx_mstate_global->__pyx_float_108_70
#define __pyx_float_109_39 __pyx_mstate_global->__pyx_float_109_39
#define __pyx_float_122_08 __pyx_mstate_global->__pyx_float_122_08
#define __pyx_float_125_54 __pyx_mstate_global->__pyx_float_125_54
#define __pyx_float_32_703 __pyx_mstate_global->__pyx_float_32_703
#define __pyx_float_10000_0 __pyx_mstate_global->__pyx_float_10000_0
#define __pyx_int_0 __pyx_mstate_global->__pyx_int_0
#define __pyx_int_1 __pyx_mstate_global->__pyx_int_1
#define __pyx_int_2 __pyx_mstate_global->__pyx_int_2
#define __pyx_int_4 __pyx_mstate_global->__pyx_int_4
#define __pyx_int_12 __pyx_mstate_global->__pyx_int_12
#define __pyx_int_45 __pyx_mstate_global->__pyx_int_45
#define __pyx_int_100 __pyx_mstate_global->__pyx_int_100
#define __pyx_int_220 __pyx_mstate_global->__pyx_int_220
#define __pyx_int_261 __pyx_mstate_global->__pyx_int_261
#define __pyx_int_440 __pyx_mstate_global->__pyx_int_440
#define __pyx_tuple_ __pyx_mstate_global->__pyx_tuple_
#define __pyx_slice__3 __pyx_mstate_global->__pyx_slice__3
#define __pyx_slice__4 __pyx_mstate_global->__pyx_slice__4
#define __pyx_tuple__2 __pyx_mstate_global->__pyx_tuple__2
#define __pyx_tuple__6 __pyx_mstate_global->__pyx_tuple__6
#define __pyx_tuple__7 __pyx_mstate_global->__pyx_tuple__7
#define __pyx_tuple__9 __pyx_mstate_global->__pyx_tuple__9
#define __pyx_tuple__11 __pyx_mstate_global->__pyx_tuple__11
#define __pyx_tuple__13 __pyx_mstate_global->__pyx_tuple__13
#define __pyx_tuple__14 __pyx_mstate_global->__pyx_tuple__14
#define __pyx_codeobj__8 __pyx_mstate_global->__pyx_codeobj__8
#define __pyx_codeobj__10 __pyx_mstate_global->__pyx_codeobj__10
#define __pyx_codeobj__12 __pyx_mstate_global->__pyx_codeobj__12
/* #### Code section: module_code ### */

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":245
 * 
 *         @property
 *         cdef inline PyObject* base(self) nogil:             # <<<<<<<<<<<<<<
 *             """Returns a borrowed reference to the object owning the data/memory.
 *             """
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_7ndarray_4base_base(PyArrayObject *__pyx_v_self) {
  PyObject *__pyx_r;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":248
 *             """Returns a borrowed reference to the object owning the data/memory.
 *             """
 *             return PyArray_BASE(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyArray_BASE(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":245
 * 
 *         @property
 *         cdef inline PyObject* base(self) nogil:             # <<<<<<<<<<<<<<
 *             """Returns a borrowed reference to the object owning the data/memory.
 *             """
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":251
 * 
 *         @property
 *         cdef inline dtype descr(self):             # <<<<<<<<<<<<<<
 *             """Returns an owned reference to the dtype of the array.
 *             """
 */

static CYTHON_INLINE PyArray_Descr *__pyx_f_5numpy_7ndarray_5descr_descr(PyArrayObject *__pyx_v_self) {
  PyArray_Descr *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyArray_Descr *__pyx_t_1;
  __Pyx_RefNannySetupContext("descr", 1);

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":254
 *             """Returns an owned reference to the dtype of the array.
 *             """
 *             return <dtype>PyArray_DESCR(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __pyx_t_1 = PyArray_DESCR(__pyx_v_self);
  __Pyx_INCREF((PyObject *)((PyArray_Descr *)__pyx_t_1));
  __pyx_r = ((PyArray_Descr *)__pyx_t_1);
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":251
 * 
 *         @property
 *         cdef inline dtype descr(self):             # <<<<<<<<<<<<<<
 *             """Returns an owned reference to the dtype of the array.
 *             """
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":257
 * 
 *         @property
 *         cdef inline int ndim(self) nogil:             # <<<<<<<<<<<<<<
 *             """Returns the number of dimensions in the array.
 *             """
 */

static CYTHON_INLINE int __pyx_f_5numpy_7ndarray_4ndim_ndim(PyArrayObject *__pyx_v_self) {
  int __pyx_r;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":260
 *             """Returns the number of dimensions in the array.
 *             """
 *             return PyArray_NDIM(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyArray_NDIM(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":257
 * 
 *         @property
 *         cdef inline int ndim(self) nogil:             # <<<<<<<<<<<<<<
 *             """Returns the number of dimensions in the array.
 *             """
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":263
 * 
 *         @property
 *         cdef inline npy_intp *shape(self) nogil:             # <<<<<<<<<<<<<<
 *             """Returns a pointer to the dimensions/shape of the array.
 *             The number of elements matches the number of dimensions of the array (ndim).
 */

static CYTHON_INLINE npy_intp *__pyx_f_5numpy_7ndarray_5shape_shape(PyArrayObject *__pyx_v_self) {
  npy_intp *__pyx_r;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":268
 *             Can return NULL for 0-dimensional arrays.
 *             """
 *             return PyArray_DIMS(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyArray_DIMS(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":263
 * 
 *         @property
 *         cdef inline npy_intp *shape(self) nogil:             # <<<<<<<<<<<<<<
 *             """Returns a pointer to the dimensions/shape of the array.
 *             The number of elements matches the number of dimensions of the array (ndim).
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":271
 * 
 *         @property
 *         cdef inline npy_intp *strides(self) nogil:             # <<<<<<<<<<<<<<
 *             """Returns a pointer to the strides of the array.
 *             The number of elements matches the number of dimensions of the array (ndim).
 */

static CYTHON_INLINE npy_intp *__pyx_f_5numpy_7ndarray_7strides_strides(PyArrayObject *__pyx_v_self) {
  npy_intp *__pyx_r;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":275
 *             The number of elements matches the number of dimensions of the array (ndim).
 *             """
 *             return PyArray_STRIDES(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyArray_STRIDES(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":271
 * 
 *         @property
 *         cdef inline npy_intp *strides(self) nogil:             # <<<<<<<<<<<<<<
 *             """Returns a pointer to the strides of the array.
 *             The number of elements matches the number of dimensions of the array (ndim).
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":278
 * 
 *         @property
 *         cdef inline npy_intp size(self) nogil:             # <<<<<<<<<<<<<<
 *             """Returns the total size (in number of elements) of the array.
 *             """
 */

static CYTHON_INLINE npy_intp __pyx_f_5numpy_7ndarray_4size_size(PyArrayObject *__pyx_v_self) {
  npy_intp __pyx_r;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":281
 *             """Returns the total size (in number of elements) of the array.
 *             """
 *             return PyArray_SIZE(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyArray_SIZE(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":278
 * 
 *         @property
 *         cdef inline npy_intp size(self) nogil:             # <<<<<<<<<<<<<<
 *             """Returns the total size (in number of elements) of the array.
 *             """
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":284
 * 
 *         @property
 *         cdef inline char* data(self) nogil:             # <<<<<<<<<<<<<<
 *             """The pointer to the data buffer as a char*.
 *             This is provided for legacy reasons to avoid direct struct field access.
 */

static CYTHON_INLINE char *__pyx_f_5numpy_7ndarray_4data_data(PyArrayObject *__pyx_v_self) {
  char *__pyx_r;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":290
 *             of `PyArray_DATA()` instead, which returns a 'void*'.
 *             """
 *             return PyArray_BYTES(self)             # <<<<<<<<<<<<<<
 * 
 *     ctypedef unsigned char      npy_bool
 */
  __pyx_r = PyArray_BYTES(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":284
 * 
 *         @property
 *         cdef inline char* data(self) nogil:             # <<<<<<<<<<<<<<
 *             """The pointer to the data buffer as a char*.
 *             This is provided for legacy reasons to avoid direct struct field access.
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":773
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew1(PyObject *__pyx_v_a) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew1", 1);

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":774
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 *     return PyArray_MultiIterNew(1, <void*>a)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(1, ((void *)__pyx_v_a)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 774, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":773
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":776
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew2(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew2", 1);

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":777
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(2, ((void *)__pyx_v_a), ((void *)__pyx_v_b)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 777, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":776
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew2", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":779
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew3(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew3", 1);

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":780
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(3, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 780, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":779
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew3", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":782
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew4(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew4", 1);

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":783
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(4, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 783, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":782
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew4", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":785
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew5(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d, PyObject *__pyx_v_e) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew5", 1);

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":786
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)             # <<<<<<<<<<<<<<
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(5, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d), ((void *)__pyx_v_e)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 786, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":785
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew5", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":788
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyDataType_SHAPE(PyArray_Descr *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("PyDataType_SHAPE", 1);

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":789
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */
  __pyx_t_1 = PyDataType_HASSUBARRAY(__pyx_v_d);
  if (__pyx_t_1) {

    /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":790
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape             # <<<<<<<<<<<<<<
 *     else:
 *         return ()
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject*)__pyx_v_d->subarray->shape));
    __pyx_r = ((PyObject*)__pyx_v_d->subarray->shape);
    goto __pyx_L0;

    /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":789
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */
  }

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":792
 *         return <tuple>d.subarray.shape
 *     else:
 *         return ()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_empty_tuple);
    __pyx_r = __pyx_empty_tuple;
    goto __pyx_L0;
  }

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":788
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":968
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

static CYTHON_INLINE void __pyx_f_5numpy_set_array_base(PyArrayObject *__pyx_v_arr, PyObject *__pyx_v_base) {
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":969
 * 
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!             # <<<<<<<<<<<<<<
 *     PyArray_SetBaseObject(arr, base)
 * 
 */
  Py_INCREF(__pyx_v_base);

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":970
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object get_array_base(ndarray arr):
 */
  __pyx_t_1 = PyArray_SetBaseObject(__pyx_v_arr, __pyx_v_base); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(1, 970, __pyx_L1_error)

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":968
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("numpy.set_array_base", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":972
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_get_array_base(PyArrayObject *__pyx_v_arr) {
  PyObject *__pyx_v_base;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("get_array_base", 1);

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":973
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)             # <<<<<<<<<<<<<<
 *     if base is NULL:
 *         return None
 */
  __pyx_v_base = PyArray_BASE(__pyx_v_arr);

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":974
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */
  __pyx_t_1 = (__pyx_v_base == NULL);
  if (__pyx_t_1) {

    /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":975
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 *         return None             # <<<<<<<<<<<<<<
 *     return <object>base
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":974
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */
  }

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":976
 *     if base is NULL:
 *         return None
 *     return <object>base             # <<<<<<<<<<<<<<
 * 
 * # Versions of the import_* functions which are more suitable for
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_base));
  __pyx_r = ((PyObject *)__pyx_v_base);
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":972
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":980
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         __pyx_import_array()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_array(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_array", 1);

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":981
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":982
 * cdef inline int import_array() except -1:
 *     try:
 *         __pyx_import_array()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")
 */
      __pyx_t_4 = _import_array(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 982, __pyx_L3_error)

      /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":981
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":983
 *     try:
 *         __pyx_import_array()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(1, 983, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_7);

      /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":984
 *         __pyx_import_array()
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 984, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(1, 984, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;

    /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":981
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
    __pyx_L5_except_error:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":980
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         __pyx_import_array()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":986
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_umath(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_umath", 1);

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":987
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":988
 * cdef inline int import_umath() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")
 */
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 988, __pyx_L3_error)

      /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":987
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":989
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(1, 989, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_7);

      /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":990
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 990, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(1, 990, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;

    /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":987
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    __pyx_L5_except_error:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":986
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":992
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_ufunc(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_ufunc", 1);

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":993
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":994
 * cdef inline int import_ufunc() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")
 */
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 994, __pyx_L3_error)

      /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":993
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":995
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(1, 995, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_7);

      /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":996
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 996, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(1, 996, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;

    /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":993
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    __pyx_L5_except_error:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":992
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":999
 * 
 * 
 * cdef inline bint is_timedelta64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.timedelta64)`
 */

static CYTHON_INLINE int __pyx_f_5numpy_is_timedelta64_object(PyObject *__pyx_v_obj) {
  int __pyx_r;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1011
 *     bool
 *     """
 *     return PyObject_TypeCheck(obj, &PyTimedeltaArrType_Type)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = PyObject_TypeCheck(__pyx_v_obj, (&PyTimedeltaArrType_Type));
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":999
 * 
 * 
 * cdef inline bint is_timedelta64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.timedelta64)`
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1014
 * 
 * 
 * cdef inline bint is_datetime64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.datetime64)`
 */

static CYTHON_INLINE int __pyx_f_5numpy_is_datetime64_object(PyObject *__pyx_v_obj) {
  int __pyx_r;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1026
 *     bool
 *     """
 *     return PyObject_TypeCheck(obj, &PyDatetimeArrType_Type)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = PyObject_TypeCheck(__pyx_v_obj, (&PyDatetimeArrType_Type));
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1014
 * 
 * 
 * cdef inline bint is_datetime64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.datetime64)`
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1029
 * 
 * 
 * cdef inline npy_datetime get_datetime64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy datetime64 object
 */

static CYTHON_INLINE npy_datetime __pyx_f_5numpy_get_datetime64_value(PyObject *__pyx_v_obj) {
  npy_datetime __pyx_r;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1036
 *     also needed.  That can be found using `get_datetime64_unit`.
 *     """
 *     return (<PyDatetimeScalarObject*>obj).obval             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((PyDatetimeScalarObject *)__pyx_v_obj)->obval;
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1029
 * 
 * 
 * cdef inline npy_datetime get_datetime64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy datetime64 object
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1039
 * 
 * 
 * cdef inline npy_timedelta get_timedelta64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy timedelta64 object
 */

static CYTHON_INLINE npy_timedelta __pyx_f_5numpy_get_timedelta64_value(PyObject *__pyx_v_obj) {
  npy_timedelta __pyx_r;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1043
 *     returns the int64 value underlying scalar numpy timedelta64 object
 *     """
 *     return (<PyTimedeltaScalarObject*>obj).obval             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((PyTimedeltaScalarObject *)__pyx_v_obj)->obval;
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1039
 * 
 * 
 * cdef inline npy_timedelta get_timedelta64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy timedelta64 object
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1046
 * 
 * 
 * cdef inline NPY_DATETIMEUNIT get_datetime64_unit(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the unit part of the dtype for a numpy datetime64 object.
 */

static CYTHON_INLINE NPY_DATETIMEUNIT __pyx_f_5numpy_get_datetime64_unit(PyObject *__pyx_v_obj) {
  NPY_DATETIMEUNIT __pyx_r;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1050
 *     returns the unit part of the dtype for a numpy datetime64 object.
 *     """
 *     return <NPY_DATETIMEUNIT>(<PyDatetimeScalarObject*>obj).obmeta.base             # <<<<<<<<<<<<<<
 */
  __pyx_r = ((NPY_DATETIMEUNIT)((PyDatetimeScalarObject *)__pyx_v_obj)->obmeta.base);
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1046
 * 
 * 
 * cdef inline NPY_DATETIMEUNIT get_datetime64_unit(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the unit part of the dtype for a numpy datetime64 object.
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "pysndlib/instruments/lbj_piano.py":111
 * 
 * 
 * def get_piano_partials(freq):             # <<<<<<<<<<<<<<
 *     return PIANO_SPECTRA[round(12 * math.log(freq / 32.703, 2))]
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pysndlib_11instruments_9lbj_piano_1get_piano_partials(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_8pysndlib_11instruments_9lbj_piano_1get_piano_partials = {"get_piano_partials", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_8pysndlib_11instruments_9lbj_piano_1get_piano_partials, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_8pysndlib_11instruments_9lbj_piano_1get_piano_partials(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_freq = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_piano_partials (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_freq,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_freq)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 111, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "get_piano_partials") < 0)) __PYX_ERR(0, 111, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_freq = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_piano_partials", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 111, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("pysndlib.instruments.lbj_piano.get_piano_partials", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_8pysndlib_11instruments_9lbj_piano_get_piano_partials(__pyx_self, __pyx_v_freq);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pysndlib_11instruments_9lbj_piano_get_piano_partials(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_freq) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_piano_partials", 1);

  /* "pysndlib/instruments/lbj_piano.py":112
 * 
 * def get_piano_partials(freq):
 *     return PIANO_SPECTRA[round(12 * math.log(freq / 32.703, 2))]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_PIANO_SPECTRA); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 112, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_math); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 112, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_log); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 112, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyFloat_TrueDivideObjC(__pyx_v_freq, __pyx_float_32_703, 32.703, 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 112, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
      __pyx_t_6 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_5, __pyx_t_3, __pyx_int_2};
    __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+1-__pyx_t_6, 2+__pyx_t_6);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 112, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __pyx_t_4 = __Pyx_PyInt_MultiplyCObj(__pyx_int_12, __pyx_t_2, 12, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 112, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_round, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 112, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetItem(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 112, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "pysndlib/instruments/lbj_piano.py":111
 * 
 * 
 * def get_piano_partials(freq):             # <<<<<<<<<<<<<<
 *     return PIANO_SPECTRA[round(12 * math.log(freq / 32.703, 2))]
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pysndlib.instruments.lbj_piano.get_piano_partials", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pysndlib/instruments/lbj_piano.py":115
 * 
 * 
 * def make_piano_ampfun(dur, piano_attack_duration):             # <<<<<<<<<<<<<<
 *     db_drop_per_second = -10.0
 *     releaseAmp = clm.db2linear(db_drop_per_second * dur)
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pysndlib_11instruments_9lbj_piano_3make_piano_ampfun(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_8pysndlib_11instruments_9lbj_piano_3make_piano_ampfun = {"make_piano_ampfun", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_8pysndlib_11instruments_9lbj_piano_3make_piano_ampfun, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_8pysndlib_11instruments_9lbj_piano_3make_piano_ampfun(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_dur = 0;
  PyObject *__pyx_v_piano_attack_duration = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("make_piano_ampfun (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_dur,&__pyx_n_s_piano_attack_duration,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_dur)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 115, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_piano_attack_duration)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 115, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("make_piano_ampfun", 1, 2, 2, 1); __PYX_ERR(0, 115, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "make_piano_ampfun") < 0)) __PYX_ERR(0, 115, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
      values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
    }
    __pyx_v_dur = values[0];
    __pyx_v_piano_attack_duration = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("make_piano_ampfun", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 115, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("pysndlib.instruments.lbj_piano.make_piano_ampfun", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_8pysndlib_11instruments_9lbj_piano_2make_piano_ampfun(__pyx_self, __pyx_v_dur, __pyx_v_piano_attack_duration);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pysndlib_11instruments_9lbj_piano_2make_piano_ampfun(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_dur, PyObject *__pyx_v_piano_attack_duration) {
  double __pyx_v_db_drop_per_second;
  double __pyx_v_releaseAmp;
  PyObject *__pyx_v_attackTime = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  double __pyx_t_3;
  double __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("make_piano_ampfun", 1);

  /* "pysndlib/instruments/lbj_piano.py":116
 * 
 * def make_piano_ampfun(dur, piano_attack_duration):
 *     db_drop_per_second = -10.0             # <<<<<<<<<<<<<<
 *     releaseAmp = clm.db2linear(db_drop_per_second * dur)
 *     attackTime = (piano_attack_duration*100) / dur
 */
  __pyx_v_db_drop_per_second = -10.0;

  /* "pysndlib/instruments/lbj_piano.py":117
 * def make_piano_ampfun(dur, piano_attack_duration):
 *     db_drop_per_second = -10.0
 *     releaseAmp = clm.db2linear(db_drop_per_second * dur)             # <<<<<<<<<<<<<<
 *     attackTime = (piano_attack_duration*100) / dur
 *     return [0,0,(attackTime/4), 1.0, attackTime, 1.0, 100, releaseAmp]
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_db_drop_per_second); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 117, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyNumber_Multiply(__pyx_t_1, __pyx_v_dur); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 117, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_3 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 117, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __pyx_f_8pysndlib_3clm_db2linear(__pyx_t_3, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 117, __pyx_L1_error)
  __pyx_v_releaseAmp = __pyx_t_4;

  /* "pysndlib/instruments/lbj_piano.py":118
 *     db_drop_per_second = -10.0
 *     releaseAmp = clm.db2linear(db_drop_per_second * dur)
 *     attackTime = (piano_attack_duration*100) / dur             # <<<<<<<<<<<<<<
 *     return [0,0,(attackTime/4), 1.0, attackTime, 1.0, 100, releaseAmp]
 * 
 */
  __pyx_t_2 = __Pyx_PyInt_MultiplyObjC(__pyx_v_piano_attack_duration, __pyx_int_100, 0x64, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyNumber_Divide(__pyx_t_2, __pyx_v_dur); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_attackTime = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pysndlib/instruments/lbj_piano.py":119
 *     releaseAmp = clm.db2linear(db_drop_per_second * dur)
 *     attackTime = (piano_attack_duration*100) / dur
 *     return [0,0,(attackTime/4), 1.0, attackTime, 1.0, 100, releaseAmp]             # <<<<<<<<<<<<<<
 * 
 * @cython.ccall
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_TrueDivideObjC(__pyx_v_attackTime, __pyx_int_4, 4, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 119, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_releaseAmp); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 119, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = PyList_New(8); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 119, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 0, __pyx_int_0)) __PYX_ERR(0, 119, __pyx_L1_error);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 1, __pyx_int_0)) __PYX_ERR(0, 119, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 2, __pyx_t_1)) __PYX_ERR(0, 119, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_1_0);
  __Pyx_GIVEREF(__pyx_float_1_0);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 3, __pyx_float_1_0)) __PYX_ERR(0, 119, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_attackTime);
  __Pyx_GIVEREF(__pyx_v_attackTime);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 4, __pyx_v_attackTime)) __PYX_ERR(0, 119, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_1_0);
  __Pyx_GIVEREF(__pyx_float_1_0);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 5, __pyx_float_1_0)) __PYX_ERR(0, 119, __pyx_L1_error);
  __Pyx_INCREF(__pyx_int_100);
  __Pyx_GIVEREF(__pyx_int_100);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 6, __pyx_int_100)) __PYX_ERR(0, 119, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_2);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 7, __pyx_t_2)) __PYX_ERR(0, 119, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "pysndlib/instruments/lbj_piano.py":115
 * 
 * 
 * def make_piano_ampfun(dur, piano_attack_duration):             # <<<<<<<<<<<<<<
 *     db_drop_per_second = -10.0
 *     releaseAmp = clm.db2linear(db_drop_per_second * dur)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pysndlib.instruments.lbj_piano.make_piano_ampfun", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_attackTime);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pysndlib/instruments/lbj_piano.py":121
 *     return [0,0,(attackTime/4), 1.0, attackTime, 1.0, 100, releaseAmp]
 * 
 * @cython.ccall             # <<<<<<<<<<<<<<
 * def lbj_piano(startime, duration, frequency, amplitude, pfreq=None, degree=45, reverb_amount=0, distance=1):
 *     piano_attack_duration = .04
 */

static PyObject *__pyx_pw_8pysndlib_11instruments_9lbj_piano_5lbj_piano(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_8pysndlib_11instruments_9lbj_piano_lbj_piano(PyObject *__pyx_v_startime, PyObject *__pyx_v_duration, PyObject *__pyx_v_frequency, PyObject *__pyx_v_amplitude, CYTHON_UNUSED int __pyx_skip_dispatch, struct __pyx_opt_args_8pysndlib_11instruments_9lbj_piano_lbj_piano *__pyx_optional_args) {

  /* "pysndlib/instruments/lbj_piano.py":122
 * 
 * @cython.ccall
 * def lbj_piano(startime, duration, frequency, amplitude, pfreq=None, degree=45, reverb_amount=0, distance=1):             # <<<<<<<<<<<<<<
 *     piano_attack_duration = .04
 *     piano_release_duration = .2
 */
  PyObject *__pyx_v_pfreq = ((PyObject *)Py_None);
  PyObject *__pyx_v_degree = ((PyObject *)__pyx_int_45);
  PyObject *__pyx_v_reverb_amount = ((PyObject *)__pyx_int_0);
  PyObject *__pyx_v_distance = ((PyObject *)__pyx_int_1);
  double __pyx_v_piano_attack_duration;
  double __pyx_v_piano_release_duration;
  PyArrayObject *__pyx_v_partials = NULL;
  long __pyx_v_beg;
  long __pyx_v_newdur;
  long __pyx_v_end;
  double __pyx_v_env1dur;
  Py_ssize_t __pyx_v_siz;
  long __pyx_v_env1samples;
  PyObject *__pyx_v_freqs = NULL;
  PyObject *__pyx_v_phases = NULL;
  PyObject *__pyx_v_alist = NULL;
  PyObject *__pyx_v_locs = NULL;
  PyObject *__pyx_v_ampfun1 = NULL;
  PyObject *__pyx_v_ampenv1 = NULL;
  PyObject *__pyx_v_ampenv2 = NULL;
  long __pyx_v_i;
  struct __pyx_obj_8pysndlib_3clm_mus_any *__pyx_v_obank = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  double __pyx_t_7;
  long __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  Py_ssize_t __pyx_t_12;
  double __pyx_t_13;
  long __pyx_t_14;
  long __pyx_t_15;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("lbj_piano", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_pfreq = __pyx_optional_args->pfreq;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_degree = __pyx_optional_args->degree;
        if (__pyx_optional_args->__pyx_n > 2) {
          __pyx_v_reverb_amount = __pyx_optional_args->reverb_amount;
          if (__pyx_optional_args->__pyx_n > 3) {
            __pyx_v_distance = __pyx_optional_args->distance;
          }
        }
      }
    }
  }
  __Pyx_INCREF(__pyx_v_pfreq);

  /* "pysndlib/instruments/lbj_piano.py":123
 * @cython.ccall
 * def lbj_piano(startime, duration, frequency, amplitude, pfreq=None, degree=45, reverb_amount=0, distance=1):
 *     piano_attack_duration = .04             # <<<<<<<<<<<<<<
 *     piano_release_duration = .2
 * 
 */
  __pyx_v_piano_attack_duration = .04;

  /* "pysndlib/instruments/lbj_piano.py":124
 * def lbj_piano(startime, duration, frequency, amplitude, pfreq=None, degree=45, reverb_amount=0, distance=1):
 *     piano_attack_duration = .04
 *     piano_release_duration = .2             # <<<<<<<<<<<<<<
 * 
 *     if not clm.is_number(pfreq):
 */
  __pyx_v_piano_release_duration = .2;

  /* "pysndlib/instruments/lbj_piano.py":126
 *     piano_release_duration = .2
 * 
 *     if not clm.is_number(pfreq):             # <<<<<<<<<<<<<<
 *         pfreq = frequency
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_clm); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_is_number); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  __pyx_t_4 = 0;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_4 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_pfreq};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_4, 1+__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 126, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 126, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = (!__pyx_t_5);
  if (__pyx_t_6) {

    /* "pysndlib/instruments/lbj_piano.py":127
 * 
 *     if not clm.is_number(pfreq):
 *         pfreq = frequency             # <<<<<<<<<<<<<<
 * 
 *     partials = clm.normalize_partials(get_piano_partials(pfreq))
 */
    __Pyx_INCREF(__pyx_v_frequency);
    __Pyx_DECREF_SET(__pyx_v_pfreq, __pyx_v_frequency);

    /* "pysndlib/instruments/lbj_piano.py":126
 *     piano_release_duration = .2
 * 
 *     if not clm.is_number(pfreq):             # <<<<<<<<<<<<<<
 *         pfreq = frequency
 * 
 */
  }

  /* "pysndlib/instruments/lbj_piano.py":129
 *         pfreq = frequency
 * 
 *     partials = clm.normalize_partials(get_piano_partials(pfreq))             # <<<<<<<<<<<<<<
 *     beg: cython.long = clm.seconds2samples(startime)
 *     newdur: cython.long = duration + piano_attack_duration + piano_release_duration
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_get_piano_partials); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 129, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = NULL;
  __pyx_t_4 = 0;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_4 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_pfreq};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_4, 1+__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 129, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __pyx_t_3 = ((PyObject *)__pyx_f_8pysndlib_3clm_normalize_partials(__pyx_t_1, 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 129, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_partials = ((PyArrayObject *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "pysndlib/instruments/lbj_piano.py":130
 * 
 *     partials = clm.normalize_partials(get_piano_partials(pfreq))
 *     beg: cython.long = clm.seconds2samples(startime)             # <<<<<<<<<<<<<<
 *     newdur: cython.long = duration + piano_attack_duration + piano_release_duration
 *     end: cython.long = beg + clm.seconds2samples(newdur)
 */
  __pyx_t_7 = __pyx_PyFloat_AsDouble(__pyx_v_startime); if (unlikely((__pyx_t_7 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 130, __pyx_L1_error)
  __pyx_t_8 = __pyx_f_8pysndlib_3clm_seconds2samples(__pyx_t_7, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 130, __pyx_L1_error)
  __pyx_v_beg = __pyx_t_8;

  /* "pysndlib/instruments/lbj_piano.py":131
 *     partials = clm.normalize_partials(get_piano_partials(pfreq))
 *     beg: cython.long = clm.seconds2samples(startime)
 *     newdur: cython.long = duration + piano_attack_duration + piano_release_duration             # <<<<<<<<<<<<<<
 *     end: cython.long = beg + clm.seconds2samples(newdur)
 *     env1dur: cython.double = newdur - piano_release_duration
 */
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_piano_attack_duration); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 131, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = PyNumber_Add(__pyx_v_duration, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 131, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_piano_release_duration); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 131, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = PyNumber_Add(__pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 131, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_8 = __Pyx_PyInt_As_long(__pyx_t_2); if (unlikely((__pyx_t_8 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 131, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_newdur = __pyx_t_8;

  /* "pysndlib/instruments/lbj_piano.py":132
 *     beg: cython.long = clm.seconds2samples(startime)
 *     newdur: cython.long = duration + piano_attack_duration + piano_release_duration
 *     end: cython.long = beg + clm.seconds2samples(newdur)             # <<<<<<<<<<<<<<
 *     env1dur: cython.double = newdur - piano_release_duration
 *     siz = len(partials) // 2
 */
  __pyx_t_8 = __pyx_f_8pysndlib_3clm_seconds2samples(__pyx_v_newdur, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 132, __pyx_L1_error)
  __pyx_v_end = (__pyx_v_beg + __pyx_t_8);

  /* "pysndlib/instruments/lbj_piano.py":133
 *     newdur: cython.long = duration + piano_attack_duration + piano_release_duration
 *     end: cython.long = beg + clm.seconds2samples(newdur)
 *     env1dur: cython.double = newdur - piano_release_duration             # <<<<<<<<<<<<<<
 *     siz = len(partials) // 2
 *     env1samples: cython.long = beg + clm.seconds2samples(env1dur)
 */
  __pyx_v_env1dur = (__pyx_v_newdur - __pyx_v_piano_release_duration);

  /* "pysndlib/instruments/lbj_piano.py":134
 *     end: cython.long = beg + clm.seconds2samples(newdur)
 *     env1dur: cython.double = newdur - piano_release_duration
 *     siz = len(partials) // 2             # <<<<<<<<<<<<<<
 *     env1samples: cython.long = beg + clm.seconds2samples(env1dur)
 *     freqs: np.ndarray  = np.zeros(siz, dtype=np.double)
 */
  __pyx_t_9 = PyObject_Length(((PyObject *)__pyx_v_partials)); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(0, 134, __pyx_L1_error)
  __pyx_v_siz = __Pyx_div_Py_ssize_t(__pyx_t_9, 2);

  /* "pysndlib/instruments/lbj_piano.py":135
 *     env1dur: cython.double = newdur - piano_release_duration
 *     siz = len(partials) // 2
 *     env1samples: cython.long = beg + clm.seconds2samples(env1dur)             # <<<<<<<<<<<<<<
 *     freqs: np.ndarray  = np.zeros(siz, dtype=np.double)
 *     phases: np.ndarray  = np.zeros(siz, dtype=np.double)
 */
  __pyx_t_8 = __pyx_f_8pysndlib_3clm_seconds2samples(__pyx_v_env1dur, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 135, __pyx_L1_error)
  __pyx_v_env1samples = (__pyx_v_beg + __pyx_t_8);

  /* "pysndlib/instruments/lbj_piano.py":136
 *     siz = len(partials) // 2
 *     env1samples: cython.long = beg + clm.seconds2samples(env1dur)
 *     freqs: np.ndarray  = np.zeros(siz, dtype=np.double)             # <<<<<<<<<<<<<<
 *     phases: np.ndarray  = np.zeros(siz, dtype=np.double)
 *     alist: np.ndarray  = np.zeros(siz, dtype=np.double)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 136, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_zeros); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 136, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyInt_FromSsize_t(__pyx_v_siz); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 136, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 136, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2)) __PYX_ERR(0, 136, __pyx_L1_error);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 136, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_np); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 136, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_double); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 136, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_11) < 0) __PYX_ERR(0, 136, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 136, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_freqs = __pyx_t_11;
  __pyx_t_11 = 0;

  /* "pysndlib/instruments/lbj_piano.py":137
 *     env1samples: cython.long = beg + clm.seconds2samples(env1dur)
 *     freqs: np.ndarray  = np.zeros(siz, dtype=np.double)
 *     phases: np.ndarray  = np.zeros(siz, dtype=np.double)             # <<<<<<<<<<<<<<
 *     alist: np.ndarray  = np.zeros(siz, dtype=np.double)
 *     locs = clm.make_locsig(degree, distance, reverb_amount)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_n_s_np); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 137, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 137, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = PyInt_FromSsize_t(__pyx_v_siz); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 137, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 137, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_11);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_11)) __PYX_ERR(0, 137, __pyx_L1_error);
  __pyx_t_11 = 0;
  __pyx_t_11 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 137, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 137, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_double); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 137, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_11, __pyx_n_s_dtype, __pyx_t_10) < 0) __PYX_ERR(0, 137, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, __pyx_t_11); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 137, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_v_phases = __pyx_t_10;
  __pyx_t_10 = 0;

  /* "pysndlib/instruments/lbj_piano.py":138
 *     freqs: np.ndarray  = np.zeros(siz, dtype=np.double)
 *     phases: np.ndarray  = np.zeros(siz, dtype=np.double)
 *     alist: np.ndarray  = np.zeros(siz, dtype=np.double)             # <<<<<<<<<<<<<<
 *     locs = clm.make_locsig(degree, distance, reverb_amount)
 *     ampfun1 = make_piano_ampfun(env1dur, piano_attack_duration)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_np); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 138, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_zeros); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 138, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = PyInt_FromSsize_t(__pyx_v_siz); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 138, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 138, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_10);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_10)) __PYX_ERR(0, 138, __pyx_L1_error);
  __pyx_t_10 = 0;
  __pyx_t_10 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 138, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 138, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_double); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 138, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_10, __pyx_n_s_dtype, __pyx_t_3) < 0) __PYX_ERR(0, 138, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_11, __pyx_t_1, __pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 138, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_v_alist = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pysndlib/instruments/lbj_piano.py":139
 *     phases: np.ndarray  = np.zeros(siz, dtype=np.double)
 *     alist: np.ndarray  = np.zeros(siz, dtype=np.double)
 *     locs = clm.make_locsig(degree, distance, reverb_amount)             # <<<<<<<<<<<<<<
 *     ampfun1 = make_piano_ampfun(env1dur, piano_attack_duration)
 *     ampenv1 = clm.make_env(ampfun1, scaler=amplitude, duration=env1dur, base=10000.0)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_clm); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 139, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_make_locsig); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 139, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = NULL;
  __pyx_t_4 = 0;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
      __pyx_t_4 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[4] = {__pyx_t_10, __pyx_v_degree, __pyx_v_distance, __pyx_v_reverb_amount};
    __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_1, __pyx_callargs+1-__pyx_t_4, 3+__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 139, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __pyx_v_locs = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pysndlib/instruments/lbj_piano.py":140
 *     alist: np.ndarray  = np.zeros(siz, dtype=np.double)
 *     locs = clm.make_locsig(degree, distance, reverb_amount)
 *     ampfun1 = make_piano_ampfun(env1dur, piano_attack_duration)             # <<<<<<<<<<<<<<
 *     ampenv1 = clm.make_env(ampfun1, scaler=amplitude, duration=env1dur, base=10000.0)
 *     ampenv2 = clm.make_env([0,1,100,0], scaler=amplitude * ampfun1[len(ampfun1)-1], duration=env1dur, base=1.0)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_make_piano_ampfun); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 140, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_10 = PyFloat_FromDouble(__pyx_v_env1dur); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 140, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_11 = PyFloat_FromDouble(__pyx_v_piano_attack_duration); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 140, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_2 = NULL;
  __pyx_t_4 = 0;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
      __pyx_t_4 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_2, __pyx_t_10, __pyx_t_11};
    __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_1, __pyx_callargs+1-__pyx_t_4, 2+__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 140, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __pyx_v_ampfun1 = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pysndlib/instruments/lbj_piano.py":141
 *     locs = clm.make_locsig(degree, distance, reverb_amount)
 *     ampfun1 = make_piano_ampfun(env1dur, piano_attack_duration)
 *     ampenv1 = clm.make_env(ampfun1, scaler=amplitude, duration=env1dur, base=10000.0)             # <<<<<<<<<<<<<<
 *     ampenv2 = clm.make_env([0,1,100,0], scaler=amplitude * ampfun1[len(ampfun1)-1], duration=env1dur, base=1.0)
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_clm); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 141, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_make_env); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 141, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 141, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_ampfun1);
  __Pyx_GIVEREF(__pyx_v_ampfun1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_ampfun1)) __PYX_ERR(0, 141, __pyx_L1_error);
  __pyx_t_11 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 141, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  if (PyDict_SetItem(__pyx_t_11, __pyx_n_s_scaler, __pyx_v_amplitude) < 0) __PYX_ERR(0, 141, __pyx_L1_error)
  __pyx_t_10 = PyFloat_FromDouble(__pyx_v_env1dur); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 141, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  if (PyDict_SetItem(__pyx_t_11, __pyx_n_s_duration, __pyx_t_10) < 0) __PYX_ERR(0, 141, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (PyDict_SetItem(__pyx_t_11, __pyx_n_s_base, __pyx_float_10000_0) < 0) __PYX_ERR(0, 141, __pyx_L1_error)
  __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_3, __pyx_t_11); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 141, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_v_ampenv1 = __pyx_t_10;
  __pyx_t_10 = 0;

  /* "pysndlib/instruments/lbj_piano.py":142
 *     ampfun1 = make_piano_ampfun(env1dur, piano_attack_duration)
 *     ampenv1 = clm.make_env(ampfun1, scaler=amplitude, duration=env1dur, base=10000.0)
 *     ampenv2 = clm.make_env([0,1,100,0], scaler=amplitude * ampfun1[len(ampfun1)-1], duration=env1dur, base=1.0)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_clm); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 142, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_make_env); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 142, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = PyList_New(4); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 142, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 0, __pyx_int_0)) __PYX_ERR(0, 142, __pyx_L1_error);
  __Pyx_INCREF(__pyx_int_1);
  __Pyx_GIVEREF(__pyx_int_1);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 1, __pyx_int_1)) __PYX_ERR(0, 142, __pyx_L1_error);
  __Pyx_INCREF(__pyx_int_100);
  __Pyx_GIVEREF(__pyx_int_100);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 2, __pyx_int_100)) __PYX_ERR(0, 142, __pyx_L1_error);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 3, __pyx_int_0)) __PYX_ERR(0, 142, __pyx_L1_error);
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 142, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_10);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_10)) __PYX_ERR(0, 142, __pyx_L1_error);
  __pyx_t_10 = 0;
  __pyx_t_10 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 142, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_9 = PyObject_Length(__pyx_v_ampfun1); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(0, 142, __pyx_L1_error)
  __pyx_t_12 = (__pyx_t_9 - 1);
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_ampfun1, __pyx_t_12, Py_ssize_t, 1, PyInt_FromSsize_t, 0, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 142, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyNumber_Multiply(__pyx_v_amplitude, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 142, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_10, __pyx_n_s_scaler, __pyx_t_2) < 0) __PYX_ERR(0, 142, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_env1dur); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 142, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_10, __pyx_n_s_duration, __pyx_t_2) < 0) __PYX_ERR(0, 142, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_10, __pyx_n_s_base, __pyx_float_1_0) < 0) __PYX_ERR(0, 142, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_11, __pyx_t_3, __pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 142, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_v_ampenv2 = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pysndlib/instruments/lbj_piano.py":145
 * 
 * 
 *     i: cython.long = 0             # <<<<<<<<<<<<<<
 * 
 *     for i in range(siz):
 */
  __pyx_v_i = 0;

  /* "pysndlib/instruments/lbj_piano.py":147
 *     i: cython.long = 0
 * 
 *     for i in range(siz):             # <<<<<<<<<<<<<<
 *         freqs[i] =  clm.hz2radians(partials[::2][i]*frequency)
 *         alist[i] =  partials[1::2][i]
 */
  __pyx_t_12 = __pyx_v_siz;
  __pyx_t_9 = __pyx_t_12;
  for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_9; __pyx_t_8+=1) {
    __pyx_v_i = __pyx_t_8;

    /* "pysndlib/instruments/lbj_piano.py":148
 * 
 *     for i in range(siz):
 *         freqs[i] =  clm.hz2radians(partials[::2][i]*frequency)             # <<<<<<<<<<<<<<
 *         alist[i] =  partials[1::2][i]
 * 
 */
    __pyx_t_2 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_partials), __pyx_slice__3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 148, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_10 = __Pyx_GetItemInt(__pyx_t_2, __pyx_v_i, long, 1, __Pyx_PyInt_From_long, 0, 1, 1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 148, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PyNumber_Multiply(__pyx_t_10, __pyx_v_frequency); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 148, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_7 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_7 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 148, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_13 = __pyx_f_8pysndlib_3clm_hz2radians(__pyx_t_7, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 148, __pyx_L1_error)
    __pyx_t_2 = PyFloat_FromDouble(__pyx_t_13); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 148, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (unlikely((__Pyx_SetItemInt(__pyx_v_freqs, __pyx_v_i, __pyx_t_2, long, 1, __Pyx_PyInt_From_long, 0, 1, 1) < 0))) __PYX_ERR(0, 148, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pysndlib/instruments/lbj_piano.py":149
 *     for i in range(siz):
 *         freqs[i] =  clm.hz2radians(partials[::2][i]*frequency)
 *         alist[i] =  partials[1::2][i]             # <<<<<<<<<<<<<<
 * 
 *     obank: clm.mus_any = clm.make_oscil_bank(freqs, phases, alist, stable=True)
 */
    __pyx_t_2 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_partials), __pyx_slice__4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 149, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_10 = __Pyx_GetItemInt(__pyx_t_2, __pyx_v_i, long, 1, __Pyx_PyInt_From_long, 0, 1, 1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 149, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely((__Pyx_SetItemInt(__pyx_v_alist, __pyx_v_i, __pyx_t_10, long, 1, __Pyx_PyInt_From_long, 0, 1, 1) < 0))) __PYX_ERR(0, 149, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  }

  /* "pysndlib/instruments/lbj_piano.py":151
 *         alist[i] =  partials[1::2][i]
 * 
 *     obank: clm.mus_any = clm.make_oscil_bank(freqs, phases, alist, stable=True)             # <<<<<<<<<<<<<<
 * 
 *     for i in range(beg,env1samples):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_clm); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 151, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_make_oscil_bank); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 151, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = PyTuple_New(3); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 151, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_INCREF(__pyx_v_freqs);
  __Pyx_GIVEREF(__pyx_v_freqs);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_v_freqs)) __PYX_ERR(0, 151, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_phases);
  __Pyx_GIVEREF(__pyx_v_phases);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_v_phases)) __PYX_ERR(0, 151, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_alist);
  __Pyx_GIVEREF(__pyx_v_alist);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 2, __pyx_v_alist)) __PYX_ERR(0, 151, __pyx_L1_error);
  __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 151, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_stable, Py_True) < 0) __PYX_ERR(0, 151, __pyx_L1_error)
  __pyx_t_11 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_10, __pyx_t_3); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 151, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (!(likely(((__pyx_t_11) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_11, __pyx_ptype_8pysndlib_3clm_mus_any))))) __PYX_ERR(0, 151, __pyx_L1_error)
  __pyx_v_obank = ((struct __pyx_obj_8pysndlib_3clm_mus_any *)__pyx_t_11);
  __pyx_t_11 = 0;

  /* "pysndlib/instruments/lbj_piano.py":153
 *     obank: clm.mus_any = clm.make_oscil_bank(freqs, phases, alist, stable=True)
 * 
 *     for i in range(beg,env1samples):             # <<<<<<<<<<<<<<
 *         clm.locsig(locs, i, clm.env(ampenv1) * clm.oscil_bank(obank))
 *     for i in range(env1samples, end):
 */
  __pyx_t_8 = __pyx_v_env1samples;
  __pyx_t_14 = __pyx_t_8;
  for (__pyx_t_15 = __pyx_v_beg; __pyx_t_15 < __pyx_t_14; __pyx_t_15+=1) {
    __pyx_v_i = __pyx_t_15;

    /* "pysndlib/instruments/lbj_piano.py":154
 * 
 *     for i in range(beg,env1samples):
 *         clm.locsig(locs, i, clm.env(ampenv1) * clm.oscil_bank(obank))             # <<<<<<<<<<<<<<
 *     for i in range(env1samples, end):
 *         clm.locsig(locs, i, clm.env(ampenv2) * clm.oscil_bank(obank))
 */
    if (!(likely(((__pyx_v_locs) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_locs, __pyx_ptype_8pysndlib_3clm_mus_any))))) __PYX_ERR(0, 154, __pyx_L1_error)
    if (!(likely(((__pyx_v_ampenv1) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_ampenv1, __pyx_ptype_8pysndlib_3clm_mus_any))))) __PYX_ERR(0, 154, __pyx_L1_error)
    __pyx_t_13 = __pyx_f_8pysndlib_3clm_env(((struct __pyx_obj_8pysndlib_3clm_mus_any *)__pyx_v_ampenv1), 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 154, __pyx_L1_error)
    __pyx_t_7 = __pyx_f_8pysndlib_3clm_oscil_bank(__pyx_v_obank, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 154, __pyx_L1_error)
    __pyx_f_8pysndlib_3clm_locsig(((struct __pyx_obj_8pysndlib_3clm_mus_any *)__pyx_v_locs), __pyx_v_i, (__pyx_t_13 * __pyx_t_7), 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 154, __pyx_L1_error)
  }

  /* "pysndlib/instruments/lbj_piano.py":155
 *     for i in range(beg,env1samples):
 *         clm.locsig(locs, i, clm.env(ampenv1) * clm.oscil_bank(obank))
 *     for i in range(env1samples, end):             # <<<<<<<<<<<<<<
 *         clm.locsig(locs, i, clm.env(ampenv2) * clm.oscil_bank(obank))
 * 
 */
  __pyx_t_8 = __pyx_v_end;
  __pyx_t_14 = __pyx_t_8;
  for (__pyx_t_15 = __pyx_v_env1samples; __pyx_t_15 < __pyx_t_14; __pyx_t_15+=1) {
    __pyx_v_i = __pyx_t_15;

    /* "pysndlib/instruments/lbj_piano.py":156
 *         clm.locsig(locs, i, clm.env(ampenv1) * clm.oscil_bank(obank))
 *     for i in range(env1samples, end):
 *         clm.locsig(locs, i, clm.env(ampenv2) * clm.oscil_bank(obank))             # <<<<<<<<<<<<<<
 * 
 * if __name__ == '__main__':
 */
    if (!(likely(((__pyx_v_locs) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_locs, __pyx_ptype_8pysndlib_3clm_mus_any))))) __PYX_ERR(0, 156, __pyx_L1_error)
    if (!(likely(((__pyx_v_ampenv2) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_ampenv2, __pyx_ptype_8pysndlib_3clm_mus_any))))) __PYX_ERR(0, 156, __pyx_L1_error)
    __pyx_t_7 = __pyx_f_8pysndlib_3clm_env(((struct __pyx_obj_8pysndlib_3clm_mus_any *)__pyx_v_ampenv2), 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 156, __pyx_L1_error)
    __pyx_t_13 = __pyx_f_8pysndlib_3clm_oscil_bank(__pyx_v_obank, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 156, __pyx_L1_error)
    __pyx_f_8pysndlib_3clm_locsig(((struct __pyx_obj_8pysndlib_3clm_mus_any *)__pyx_v_locs), __pyx_v_i, (__pyx_t_7 * __pyx_t_13), 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 156, __pyx_L1_error)
  }

  /* "pysndlib/instruments/lbj_piano.py":121
 *     return [0,0,(attackTime/4), 1.0, attackTime, 1.0, 100, releaseAmp]
 * 
 * @cython.ccall             # <<<<<<<<<<<<<<
 * def lbj_piano(startime, duration, frequency, amplitude, pfreq=None, degree=45, reverb_amount=0, distance=1):
 *     piano_attack_duration = .04
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("pysndlib.instruments.lbj_piano.lbj_piano", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_partials);
  __Pyx_XDECREF(__pyx_v_freqs);
  __Pyx_XDECREF(__pyx_v_phases);
  __Pyx_XDECREF(__pyx_v_alist);
  __Pyx_XDECREF(__pyx_v_locs);
  __Pyx_XDECREF(__pyx_v_ampfun1);
  __Pyx_XDECREF(__pyx_v_ampenv1);
  __Pyx_XDECREF(__pyx_v_ampenv2);
  __Pyx_XDECREF((PyObject *)__pyx_v_obank);
  __Pyx_XDECREF(__pyx_v_pfreq);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_8pysndlib_11instruments_9lbj_piano_5lbj_piano(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_8pysndlib_11instruments_9lbj_piano_5lbj_piano = {"lbj_piano", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_8pysndlib_11instruments_9lbj_piano_5lbj_piano, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_8pysndlib_11instruments_9lbj_piano_5lbj_piano(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_startime = 0;
  PyObject *__pyx_v_duration = 0;
  PyObject *__pyx_v_frequency = 0;
  PyObject *__pyx_v_amplitude = 0;
  PyObject *__pyx_v_pfreq = 0;
  PyObject *__pyx_v_degree = 0;
  PyObject *__pyx_v_reverb_amount = 0;
  PyObject *__pyx_v_distance = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[8] = {0,0,0,0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("lbj_piano (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_startime,&__pyx_n_s_duration,&__pyx_n_s_frequency,&__pyx_n_s_amplitude,&__pyx_n_s_pfreq,&__pyx_n_s_degree,&__pyx_n_s_reverb_amount,&__pyx_n_s_distance,0};

    /* "pysndlib/instruments/lbj_piano.py":122
 * 
 * @cython.ccall
 * def lbj_piano(startime, duration, frequency, amplitude, pfreq=None, degree=45, reverb_amount=0, distance=1):             # <<<<<<<<<<<<<<
 *     piano_attack_duration = .04
 *     piano_release_duration = .2
 */
    values[4] = __Pyx_Arg_NewRef_FASTCALL(((PyObject *)Py_None));
    values[5] = __Pyx_Arg_NewRef_FASTCALL(((PyObject *)__pyx_int_45));
    values[6] = __Pyx_Arg_NewRef_FASTCALL(((PyObject *)__pyx_int_0));
    values[7] = __Pyx_Arg_NewRef_FASTCALL(((PyObject *)__pyx_int_1));
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  8: values[7] = __Pyx_Arg_FASTCALL(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = __Pyx_Arg_FASTCALL(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = __Pyx_Arg_FASTCALL(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = __Pyx_Arg_FASTCALL(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = __Pyx_Arg_FASTCALL(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_startime)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 121, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_duration)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 121, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("lbj_piano", 0, 4, 8, 1); __PYX_ERR(0, 121, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_frequency)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[2]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 121, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("lbj_piano", 0, 4, 8, 2); __PYX_ERR(0, 121, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_amplitude)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[3]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 121, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("lbj_piano", 0, 4, 8, 3); __PYX_ERR(0, 121, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_pfreq);
          if (value) { values[4] = __Pyx_Arg_NewRef_FASTCALL(value); kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 121, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_degree);
          if (value) { values[5] = __Pyx_Arg_NewRef_FASTCALL(value); kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 121, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_reverb_amount);
          if (value) { values[6] = __Pyx_Arg_NewRef_FASTCALL(value); kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 121, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_distance);
          if (value) { values[7] = __Pyx_Arg_NewRef_FASTCALL(value); kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 121, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "lbj_piano") < 0)) __PYX_ERR(0, 121, __pyx_L3_error)
      }
    } else {
      switch (__pyx_nargs) {
        case  8: values[7] = __Pyx_Arg_FASTCALL(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = __Pyx_Arg_FASTCALL(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = __Pyx_Arg_FASTCALL(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = __Pyx_Arg_FASTCALL(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = __Pyx_Arg_FASTCALL(__pyx_args, 3);
        values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_startime = values[0];
    __pyx_v_duration = values[1];
    __pyx_v_frequency = values[2];
    __pyx_v_amplitude = values[3];
    __pyx_v_pfreq = values[4];
    __pyx_v_degree = values[5];
    __pyx_v_reverb_amount = values[6];
    __pyx_v_distance = values[7];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("lbj_piano", 0, 4, 8, __pyx_nargs); __PYX_ERR(0, 121, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("pysndlib.instruments.lbj_piano.lbj_piano", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_8pysndlib_11instruments_9lbj_piano_4lbj_piano(__pyx_self, __pyx_v_startime, __pyx_v_duration, __pyx_v_frequency, __pyx_v_amplitude, __pyx_v_pfreq, __pyx_v_degree, __pyx_v_reverb_amount, __pyx_v_distance);

  /* "pysndlib/instruments/lbj_piano.py":121
 *     return [0,0,(attackTime/4), 1.0, attackTime, 1.0, 100, releaseAmp]
 * 
 * @cython.ccall             # <<<<<<<<<<<<<<
 * def lbj_piano(startime, duration, frequency, amplitude, pfreq=None, degree=45, reverb_amount=0, distance=1):
 *     piano_attack_duration = .04
 */

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pysndlib_11instruments_9lbj_piano_4lbj_piano(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_startime, PyObject *__pyx_v_duration, PyObject *__pyx_v_frequency, PyObject *__pyx_v_amplitude, PyObject *__pyx_v_pfreq, PyObject *__pyx_v_degree, PyObject *__pyx_v_reverb_amount, PyObject *__pyx_v_distance) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_8pysndlib_11instruments_9lbj_piano_lbj_piano __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("lbj_piano", 1);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 4;
  __pyx_t_2.pfreq = __pyx_v_pfreq;
  __pyx_t_2.degree = __pyx_v_degree;
  __pyx_t_2.reverb_amount = __pyx_v_reverb_amount;
  __pyx_t_2.distance = __pyx_v_distance;
  __pyx_t_1 = __pyx_f_8pysndlib_11instruments_9lbj_piano_lbj_piano(__pyx_v_startime, __pyx_v_duration, __pyx_v_frequency, __pyx_v_amplitude, 0, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pysndlib.instruments.lbj_piano.lbj_piano", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif
/* #### Code section: pystring_table ### */

static int __Pyx_CreateStringTabAndInitStrings(void) {
  __Pyx_StringTabEntry __pyx_string_tab[] = {
    {&__pyx_n_s_ImportError, __pyx_k_ImportError, sizeof(__pyx_k_ImportError), 0, 0, 1, 1},
    {&__pyx_n_s_PIANO_SPECTRA, __pyx_k_PIANO_SPECTRA, sizeof(__pyx_k_PIANO_SPECTRA), 0, 0, 1, 1},
    {&__pyx_n_s_Sound, __pyx_k_Sound, sizeof(__pyx_k_Sound), 0, 0, 1, 1},
    {&__pyx_n_s__15, __pyx_k__15, sizeof(__pyx_k__15), 0, 0, 1, 1},
    {&__pyx_n_s__5, __pyx_k__5, sizeof(__pyx_k__5), 0, 0, 1, 1},
    {&__pyx_n_s_amplitude, __pyx_k_amplitude, sizeof(__pyx_k_amplitude), 0, 0, 1, 1},
    {&__pyx_n_s_asyncio_coroutines, __pyx_k_asyncio_coroutines, sizeof(__pyx_k_asyncio_coroutines), 0, 0, 1, 1},
    {&__pyx_n_s_attackTime, __pyx_k_attackTime, sizeof(__pyx_k_attackTime), 0, 0, 1, 1},
    {&__pyx_n_s_base, __pyx_k_base, sizeof(__pyx_k_base), 0, 0, 1, 1},
    {&__pyx_n_s_class_getitem, __pyx_k_class_getitem, sizeof(__pyx_k_class_getitem), 0, 0, 1, 1},
    {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
    {&__pyx_n_s_clm, __pyx_k_clm, sizeof(__pyx_k_clm), 0, 0, 1, 1},
    {&__pyx_n_s_db_drop_per_second, __pyx_k_db_drop_per_second, sizeof(__pyx_k_db_drop_per_second), 0, 0, 1, 1},
    {&__pyx_n_s_degree, __pyx_k_degree, sizeof(__pyx_k_degree), 0, 0, 1, 1},
    {&__pyx_n_s_distance, __pyx_k_distance, sizeof(__pyx_k_distance), 0, 0, 1, 1},
    {&__pyx_n_s_double, __pyx_k_double, sizeof(__pyx_k_double), 0, 0, 1, 1},
    {&__pyx_n_s_dtype, __pyx_k_dtype, sizeof(__pyx_k_dtype), 0, 0, 1, 1},
    {&__pyx_n_s_dur, __pyx_k_dur, sizeof(__pyx_k_dur), 0, 0, 1, 1},
    {&__pyx_n_s_duration, __pyx_k_duration, sizeof(__pyx_k_duration), 0, 0, 1, 1},
    {&__pyx_n_s_enter, __pyx_k_enter, sizeof(__pyx_k_enter), 0, 0, 1, 1},
    {&__pyx_n_s_exit, __pyx_k_exit, sizeof(__pyx_k_exit), 0, 0, 1, 1},
    {&__pyx_n_s_freq, __pyx_k_freq, sizeof(__pyx_k_freq), 0, 0, 1, 1},
    {&__pyx_n_s_frequency, __pyx_k_frequency, sizeof(__pyx_k_frequency), 0, 0, 1, 1},
    {&__pyx_n_s_get_piano_partials, __pyx_k_get_piano_partials, sizeof(__pyx_k_get_piano_partials), 0, 0, 1, 1},
    {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
    {&__pyx_n_s_initializing, __pyx_k_initializing, sizeof(__pyx_k_initializing), 0, 0, 1, 1},
    {&__pyx_n_s_is_coroutine, __pyx_k_is_coroutine, sizeof(__pyx_k_is_coroutine), 0, 0, 1, 1},
    {&__pyx_n_s_is_number, __pyx_k_is_number, sizeof(__pyx_k_is_number), 0, 0, 1, 1},
    {&__pyx_n_s_lbj_piano, __pyx_k_lbj_piano, sizeof(__pyx_k_lbj_piano), 0, 0, 1, 1},
    {&__pyx_n_s_log, __pyx_k_log, sizeof(__pyx_k_log), 0, 0, 1, 1},
    {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
    {&__pyx_n_u_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 1, 0, 1},
    {&__pyx_n_s_make_env, __pyx_k_make_env, sizeof(__pyx_k_make_env), 0, 0, 1, 1},
    {&__pyx_n_s_make_locsig, __pyx_k_make_locsig, sizeof(__pyx_k_make_locsig), 0, 0, 1, 1},
    {&__pyx_n_s_make_oscil_bank, __pyx_k_make_oscil_bank, sizeof(__pyx_k_make_oscil_bank), 0, 0, 1, 1},
    {&__pyx_n_s_make_piano_ampfun, __pyx_k_make_piano_ampfun, sizeof(__pyx_k_make_piano_ampfun), 0, 0, 1, 1},
    {&__pyx_n_s_math, __pyx_k_math, sizeof(__pyx_k_math), 0, 0, 1, 1},
    {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
    {&__pyx_n_s_np, __pyx_k_np, sizeof(__pyx_k_np), 0, 0, 1, 1},
    {&__pyx_n_s_numpy, __pyx_k_numpy, sizeof(__pyx_k_numpy), 0, 0, 1, 1},
    {&__pyx_kp_u_numpy_core_multiarray_failed_to, __pyx_k_numpy_core_multiarray_failed_to, sizeof(__pyx_k_numpy_core_multiarray_failed_to), 0, 1, 0, 0},
    {&__pyx_kp_u_numpy_core_umath_failed_to_impor, __pyx_k_numpy_core_umath_failed_to_impor, sizeof(__pyx_k_numpy_core_umath_failed_to_impor), 0, 1, 0, 0},
    {&__pyx_n_s_pfreq, __pyx_k_pfreq, sizeof(__pyx_k_pfreq), 0, 0, 1, 1},
    {&__pyx_n_s_piano_attack_duration, __pyx_k_piano_attack_duration, sizeof(__pyx_k_piano_attack_duration), 0, 0, 1, 1},
    {&__pyx_n_s_play, __pyx_k_play, sizeof(__pyx_k_play), 0, 0, 1, 1},
    {&__pyx_n_s_pysndlib, __pyx_k_pysndlib, sizeof(__pyx_k_pysndlib), 0, 0, 1, 1},
    {&__pyx_n_s_pysndlib_clm, __pyx_k_pysndlib_clm, sizeof(__pyx_k_pysndlib_clm), 0, 0, 1, 1},
    {&__pyx_n_s_pysndlib_instruments_lbj_piano, __pyx_k_pysndlib_instruments_lbj_piano, sizeof(__pyx_k_pysndlib_instruments_lbj_piano), 0, 0, 1, 1},
    {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
    {&__pyx_n_s_random, __pyx_k_random, sizeof(__pyx_k_random), 0, 0, 1, 1},
    {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
    {&__pyx_n_s_releaseAmp, __pyx_k_releaseAmp, sizeof(__pyx_k_releaseAmp), 0, 0, 1, 1},
    {&__pyx_n_s_reverb_amount, __pyx_k_reverb_amount, sizeof(__pyx_k_reverb_amount), 0, 0, 1, 1},
    {&__pyx_n_s_round, __pyx_k_round, sizeof(__pyx_k_round), 0, 0, 1, 1},
    {&__pyx_n_s_scaler, __pyx_k_scaler, sizeof(__pyx_k_scaler), 0, 0, 1, 1},
    {&__pyx_n_s_spec, __pyx_k_spec, sizeof(__pyx_k_spec), 0, 0, 1, 1},
    {&__pyx_kp_s_src_pysndlib_instruments_lbj_pia, __pyx_k_src_pysndlib_instruments_lbj_pia, sizeof(__pyx_k_src_pysndlib_instruments_lbj_pia), 0, 0, 1, 0},
    {&__pyx_n_s_stable, __pyx_k_stable, sizeof(__pyx_k_stable), 0, 0, 1, 1},
    {&__pyx_n_s_startime, __pyx_k_startime, sizeof(__pyx_k_startime), 0, 0, 1, 1},
    {&__pyx_n_s_statistics, __pyx_k_statistics, sizeof(__pyx_k_statistics), 0, 0, 1, 1},
    {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
    {&__pyx_n_s_zeros, __pyx_k_zeros, sizeof(__pyx_k_zeros), 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 0, 0}
  };
  return __Pyx_InitStrings(__pyx_string_tab);
}
/* #### Code section: cached_builtins ### */
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_round = __Pyx_GetBuiltinName(__pyx_n_s_round); if (!__pyx_builtin_round) __PYX_ERR(0, 112, __pyx_L1_error)
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 147, __pyx_L1_error)
  __pyx_builtin_ImportError = __Pyx_GetBuiltinName(__pyx_n_s_ImportError); if (!__pyx_builtin_ImportError) __PYX_ERR(1, 984, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: cached_constants ### */

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":984
 *         __pyx_import_array()
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_u_numpy_core_multiarray_failed_to); if (unlikely(!__pyx_tuple_)) __PYX_ERR(1, 984, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":990
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_u_numpy_core_umath_failed_to_impor); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(1, 990, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "pysndlib/instruments/lbj_piano.py":148
 * 
 *     for i in range(siz):
 *         freqs[i] =  clm.hz2radians(partials[::2][i]*frequency)             # <<<<<<<<<<<<<<
 *         alist[i] =  partials[1::2][i]
 * 
 */
  __pyx_slice__3 = PySlice_New(Py_None, Py_None, __pyx_int_2); if (unlikely(!__pyx_slice__3)) __PYX_ERR(0, 148, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__3);
  __Pyx_GIVEREF(__pyx_slice__3);

  /* "pysndlib/instruments/lbj_piano.py":149
 *     for i in range(siz):
 *         freqs[i] =  clm.hz2radians(partials[::2][i]*frequency)
 *         alist[i] =  partials[1::2][i]             # <<<<<<<<<<<<<<
 * 
 *     obank: clm.mus_any = clm.make_oscil_bank(freqs, phases, alist, stable=True)
 */
  __pyx_slice__4 = PySlice_New(__pyx_int_1, Py_None, __pyx_int_2); if (unlikely(!__pyx_slice__4)) __PYX_ERR(0, 149, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__4);
  __Pyx_GIVEREF(__pyx_slice__4);

  /* "pysndlib/instruments/lbj_piano.py":10
 * import cython
 * import numpy as np
 * import pysndlib.clm as clm             # <<<<<<<<<<<<<<
 * if cython.compiled:
 *     from cython.cimports.pysndlib import clm
 */
  __pyx_tuple__6 = PyTuple_Pack(2, __pyx_n_s_pysndlib, __pyx_n_s_clm); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "pysndlib/instruments/lbj_piano.py":111
 * 
 * 
 * def get_piano_partials(freq):             # <<<<<<<<<<<<<<
 *     return PIANO_SPECTRA[round(12 * math.log(freq / 32.703, 2))]
 * 
 */
  __pyx_tuple__7 = PyTuple_Pack(1, __pyx_n_s_freq); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(0, 111, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);
  __pyx_codeobj__8 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__7, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_pysndlib_instruments_lbj_pia, __pyx_n_s_get_piano_partials, 111, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__8)) __PYX_ERR(0, 111, __pyx_L1_error)

  /* "pysndlib/instruments/lbj_piano.py":115
 * 
 * 
 * def make_piano_ampfun(dur, piano_attack_duration):             # <<<<<<<<<<<<<<
 *     db_drop_per_second = -10.0
 *     releaseAmp = clm.db2linear(db_drop_per_second * dur)
 */
  __pyx_tuple__9 = PyTuple_Pack(5, __pyx_n_s_dur, __pyx_n_s_piano_attack_duration, __pyx_n_s_db_drop_per_second, __pyx_n_s_releaseAmp, __pyx_n_s_attackTime); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(0, 115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);
  __pyx_codeobj__10 = (PyObject*)__Pyx_PyCode_New(2, 0, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__9, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_pysndlib_instruments_lbj_pia, __pyx_n_s_make_piano_ampfun, 115, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__10)) __PYX_ERR(0, 115, __pyx_L1_error)

  /* "pysndlib/instruments/lbj_piano.py":121
 *     return [0,0,(attackTime/4), 1.0, attackTime, 1.0, 100, releaseAmp]
 * 
 * @cython.ccall             # <<<<<<<<<<<<<<
 * def lbj_piano(startime, duration, frequency, amplitude, pfreq=None, degree=45, reverb_amount=0, distance=1):
 *     piano_attack_duration = .04
 */
  __pyx_tuple__11 = PyTuple_Pack(8, __pyx_n_s_startime, __pyx_n_s_duration, __pyx_n_s_frequency, __pyx_n_s_amplitude, __pyx_n_s_pfreq, __pyx_n_s_degree, __pyx_n_s_reverb_amount, __pyx_n_s_distance); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);
  __pyx_codeobj__12 = (PyObject*)__Pyx_PyCode_New(8, 0, 0, 8, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__11, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_pysndlib_instruments_lbj_pia, __pyx_n_s_lbj_piano, 121, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__12)) __PYX_ERR(0, 121, __pyx_L1_error)
  __pyx_tuple__13 = PyTuple_Pack(4, Py_None, __pyx_int_45, __pyx_int_0, __pyx_int_1); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);

  /* "pysndlib/instruments/lbj_piano.py":159
 * 
 * if __name__ == '__main__':
 *     with clm.Sound( play = True, statistics=True):             # <<<<<<<<<<<<<<
 *         lbj_piano(0,4,220,.34)
 *         lbj_piano(1,4,440,.34)
 */
  __pyx_tuple__14 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple__14)) __PYX_ERR(0, 159, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__14);
  __Pyx_GIVEREF(__pyx_tuple__14);
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}
/* #### Code section: init_constants ### */

static CYTHON_SMALL_CODE int __Pyx_InitConstants(void) {
  if (__Pyx_CreateStringTabAndInitStrings() < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_float__4 = PyFloat_FromDouble(.4); if (unlikely(!__pyx_float__4)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__34 = PyFloat_FromDouble(.34); if (unlikely(!__pyx_float__34)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_1_0 = PyFloat_FromDouble(1.0); if (unlikely(!__pyx_float_1_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_0_97 = PyFloat_FromDouble(0.97); if (unlikely(!__pyx_float_0_97)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_0_98 = PyFloat_FromDouble(0.98); if (unlikely(!__pyx_float_0_98)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_0_99 = PyFloat_FromDouble(0.99); if (unlikely(!__pyx_float_0_99)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_1_00 = PyFloat_FromDouble(1.00); if (unlikely(!__pyx_float_1_00)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_1_01 = PyFloat_FromDouble(1.01); if (unlikely(!__pyx_float_1_01)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_1_02 = PyFloat_FromDouble(1.02); if (unlikely(!__pyx_float_1_02)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_1_97 = PyFloat_FromDouble(1.97); if (unlikely(!__pyx_float_1_97)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_1_98 = PyFloat_FromDouble(1.98); if (unlikely(!__pyx_float_1_98)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_1_99 = PyFloat_FromDouble(1.99); if (unlikely(!__pyx_float_1_99)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_2_00 = PyFloat_FromDouble(2.00); if (unlikely(!__pyx_float_2_00)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_2_01 = PyFloat_FromDouble(2.01); if (unlikely(!__pyx_float_2_01)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_2_02 = PyFloat_FromDouble(2.02); if (unlikely(!__pyx_float_2_02)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_2_03 = PyFloat_FromDouble(2.03); if (unlikely(!__pyx_float_2_03)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_2_07 = PyFloat_FromDouble(2.07); if (unlikely(!__pyx_float_2_07)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_2_09 = PyFloat_FromDouble(2.09); if (unlikely(!__pyx_float_2_09)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_2_96 = PyFloat_FromDouble(2.96); if (unlikely(!__pyx_float_2_96)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_2_97 = PyFloat_FromDouble(2.97); if (unlikely(!__pyx_float_2_97)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_2_98 = PyFloat_FromDouble(2.98); if (unlikely(!__pyx_float_2_98)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_2_99 = PyFloat_FromDouble(2.99); if (unlikely(!__pyx_float_2_99)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_3_00 = PyFloat_FromDouble(3.00); if (unlikely(!__pyx_float_3_00)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_3_01 = PyFloat_FromDouble(3.01); if (unlikely(!__pyx_float_3_01)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_3_02 = PyFloat_FromDouble(3.02); if (unlikely(!__pyx_float_3_02)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_3_03 = PyFloat_FromDouble(3.03); if (unlikely(!__pyx_float_3_03)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_3_04 = PyFloat_FromDouble(3.04); if (unlikely(!__pyx_float_3_04)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_3_09 = PyFloat_FromDouble(3.09); if (unlikely(!__pyx_float_3_09)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_3_11 = PyFloat_FromDouble(3.11); if (unlikely(!__pyx_float_3_11)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_3_12 = PyFloat_FromDouble(3.12); if (unlikely(!__pyx_float_3_12)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_3_15 = PyFloat_FromDouble(3.15); if (unlikely(!__pyx_float_3_15)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_3_19 = PyFloat_FromDouble(3.19); if (unlikely(!__pyx_float_3_19)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_3_95 = PyFloat_FromDouble(3.95); if (unlikely(!__pyx_float_3_95)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_3_97 = PyFloat_FromDouble(3.97); if (unlikely(!__pyx_float_3_97)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_3_98 = PyFloat_FromDouble(3.98); if (unlikely(!__pyx_float_3_98)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_3_99 = PyFloat_FromDouble(3.99); if (unlikely(!__pyx_float_3_99)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_4_00 = PyFloat_FromDouble(4.00); if (unlikely(!__pyx_float_4_00)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_4_01 = PyFloat_FromDouble(4.01); if (unlikely(!__pyx_float_4_01)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_4_02 = PyFloat_FromDouble(4.02); if (unlikely(!__pyx_float_4_02)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_4_03 = PyFloat_FromDouble(4.03); if (unlikely(!__pyx_float_4_03)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_4_04 = PyFloat_FromDouble(4.04); if (unlikely(!__pyx_float_4_04)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_4_05 = PyFloat_FromDouble(4.05); if (unlikely(!__pyx_float_4_05)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_4_06 = PyFloat_FromDouble(4.06); if (unlikely(!__pyx_float_4_06)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_4_10 = PyFloat_FromDouble(4.10); if (unlikely(!__pyx_float_4_10)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_4_12 = PyFloat_FromDouble(4.12); if (unlikely(!__pyx_float_4_12)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_4_13 = PyFloat_FromDouble(4.13); if (unlikely(!__pyx_float_4_13)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_4_17 = PyFloat_FromDouble(4.17); if (unlikely(!__pyx_float_4_17)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_4_19 = PyFloat_FromDouble(4.19); if (unlikely(!__pyx_float_4_19)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_4_26 = PyFloat_FromDouble(4.26); if (unlikely(!__pyx_float_4_26)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_4_94 = PyFloat_FromDouble(4.94); if (unlikely(!__pyx_float_4_94)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_4_95 = PyFloat_FromDouble(4.95); if (unlikely(!__pyx_float_4_95)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_4_96 = PyFloat_FromDouble(4.96); if (unlikely(!__pyx_float_4_96)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_4_97 = PyFloat_FromDouble(4.97); if (unlikely(!__pyx_float_4_97)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_4_98 = PyFloat_FromDouble(4.98); if (unlikely(!__pyx_float_4_98)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_4_99 = PyFloat_FromDouble(4.99); if (unlikely(!__pyx_float_4_99)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_5_00 = PyFloat_FromDouble(5.00); if (unlikely(!__pyx_float_5_00)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_5_01 = PyFloat_FromDouble(5.01); if (unlikely(!__pyx_float_5_01)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_5_02 = PyFloat_FromDouble(5.02); if (unlikely(!__pyx_float_5_02)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_5_03 = PyFloat_FromDouble(5.03); if (unlikely(!__pyx_float_5_03)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_5_04 = PyFloat_FromDouble(5.04); if (unlikely(!__pyx_float_5_04)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_5_05 = PyFloat_FromDouble(5.05); if (unlikely(!__pyx_float_5_05)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_5_07 = PyFloat_FromDouble(5.07); if (unlikely(!__pyx_float_5_07)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_5_09 = PyFloat_FromDouble(5.09); if (unlikely(!__pyx_float_5_09)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_5_10 = PyFloat_FromDouble(5.10); if (unlikely(!__pyx_float_5_10)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_5_11 = PyFloat_FromDouble(5.11); if (unlikely(!__pyx_float_5_11)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_5_12 = PyFloat_FromDouble(5.12); if (unlikely(!__pyx_float_5_12)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_5_14 = PyFloat_FromDouble(5.14); if (unlikely(!__pyx_float_5_14)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_5_15 = PyFloat_FromDouble(5.15); if (unlikely(!__pyx_float_5_15)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_5_19 = PyFloat_FromDouble(5.19); if (unlikely(!__pyx_float_5_19)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_5_20 = PyFloat_FromDouble(5.20); if (unlikely(!__pyx_float_5_20)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_5_22 = PyFloat_FromDouble(5.22); if (unlikely(!__pyx_float_5_22)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_5_95 = PyFloat_FromDouble(5.95); if (unlikely(!__pyx_float_5_95)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_5_96 = PyFloat_FromDouble(5.96); if (unlikely(!__pyx_float_5_96)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_5_97 = PyFloat_FromDouble(5.97); if (unlikely(!__pyx_float_5_97)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_5_98 = PyFloat_FromDouble(5.98); if (unlikely(!__pyx_float_5_98)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_5_99 = PyFloat_FromDouble(5.99); if (unlikely(!__pyx_float_5_99)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_6_00 = PyFloat_FromDouble(6.00); if (unlikely(!__pyx_float_6_00)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_6_01 = PyFloat_FromDouble(6.01); if (unlikely(!__pyx_float_6_01)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_6_02 = PyFloat_FromDouble(6.02); if (unlikely(!__pyx_float_6_02)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_6_03 = PyFloat_FromDouble(6.03); if (unlikely(!__pyx_float_6_03)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_6_04 = PyFloat_FromDouble(6.04); if (unlikely(!__pyx_float_6_04)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_6_06 = PyFloat_FromDouble(6.06); if (unlikely(!__pyx_float_6_06)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_6_07 = PyFloat_FromDouble(6.07); if (unlikely(!__pyx_float_6_07)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_6_08 = PyFloat_FromDouble(6.08); if (unlikely(!__pyx_float_6_08)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_6_09 = PyFloat_FromDouble(6.09); if (unlikely(!__pyx_float_6_09)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_6_12 = PyFloat_FromDouble(6.12); if (unlikely(!__pyx_float_6_12)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_6_13 = PyFloat_FromDouble(6.13); if (unlikely(!__pyx_float_6_13)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_6_14 = PyFloat_FromDouble(6.14); if (unlikely(!__pyx_float_6_14)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_6_16 = PyFloat_FromDouble(6.16); if (unlikely(!__pyx_float_6_16)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_6_17 = PyFloat_FromDouble(6.17); if (unlikely(!__pyx_float_6_17)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_6_20 = PyFloat_FromDouble(6.20); if (unlikely(!__pyx_float_6_20)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_6_22 = PyFloat_FromDouble(6.22); if (unlikely(!__pyx_float_6_22)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_6_26 = PyFloat_FromDouble(6.26); if (unlikely(!__pyx_float_6_26)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_6_27 = PyFloat_FromDouble(6.27); if (unlikely(!__pyx_float_6_27)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_6_32 = PyFloat_FromDouble(6.32); if (unlikely(!__pyx_float_6_32)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_6_35 = PyFloat_FromDouble(6.35); if (unlikely(!__pyx_float_6_35)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_6_94 = PyFloat_FromDouble(6.94); if (unlikely(!__pyx_float_6_94)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_6_95 = PyFloat_FromDouble(6.95); if (unlikely(!__pyx_float_6_95)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_6_96 = PyFloat_FromDouble(6.96); if (unlikely(!__pyx_float_6_96)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_6_97 = PyFloat_FromDouble(6.97); if (unlikely(!__pyx_float_6_97)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_6_98 = PyFloat_FromDouble(6.98); if (unlikely(!__pyx_float_6_98)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_6_99 = PyFloat_FromDouble(6.99); if (unlikely(!__pyx_float_6_99)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_7_00 = PyFloat_FromDouble(7.00); if (unlikely(!__pyx_float_7_00)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_7_01 = PyFloat_FromDouble(7.01); if (unlikely(!__pyx_float_7_01)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_7_03 = PyFloat_FromDouble(7.03); if (unlikely(!__pyx_float_7_03)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_7_04 = PyFloat_FromDouble(7.04); if (unlikely(!__pyx_float_7_04)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_7_05 = PyFloat_FromDouble(7.05); if (unlikely(!__pyx_float_7_05)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_7_06 = PyFloat_FromDouble(7.06); if (unlikely(!__pyx_float_7_06)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_7_07 = PyFloat_FromDouble(7.07); if (unlikely(!__pyx_float_7_07)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_7_09 = PyFloat_FromDouble(7.09); if (unlikely(!__pyx_float_7_09)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_7_11 = PyFloat_FromDouble(7.11); if (unlikely(!__pyx_float_7_11)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_7_12 = PyFloat_FromDouble(7.12); if (unlikely(!__pyx_float_7_12)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_7_13 = PyFloat_FromDouble(7.13); if (unlikely(!__pyx_float_7_13)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_7_15 = PyFloat_FromDouble(7.15); if (unlikely(!__pyx_float_7_15)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_7_19 = PyFloat_FromDouble(7.19); if (unlikely(!__pyx_float_7_19)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_7_22 = PyFloat_FromDouble(7.22); if (unlikely(!__pyx_float_7_22)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_7_23 = PyFloat_FromDouble(7.23); if (unlikely(!__pyx_float_7_23)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_7_25 = PyFloat_FromDouble(7.25); if (unlikely(!__pyx_float_7_25)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_7_26 = PyFloat_FromDouble(7.26); if (unlikely(!__pyx_float_7_26)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_7_27 = PyFloat_FromDouble(7.27); if (unlikely(!__pyx_float_7_27)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_7_32 = PyFloat_FromDouble(7.32); if (unlikely(!__pyx_float_7_32)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_7_33 = PyFloat_FromDouble(7.33); if (unlikely(!__pyx_float_7_33)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_7_35 = PyFloat_FromDouble(7.35); if (unlikely(!__pyx_float_7_35)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_7_41 = PyFloat_FromDouble(7.41); if (unlikely(!__pyx_float_7_41)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_7_59 = PyFloat_FromDouble(7.59); if (unlikely(!__pyx_float_7_59)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_7_93 = PyFloat_FromDouble(7.93); if (unlikely(!__pyx_float_7_93)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_7_94 = PyFloat_FromDouble(7.94); if (unlikely(!__pyx_float_7_94)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_7_96 = PyFloat_FromDouble(7.96); if (unlikely(!__pyx_float_7_96)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_7_97 = PyFloat_FromDouble(7.97); if (unlikely(!__pyx_float_7_97)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_7_98 = PyFloat_FromDouble(7.98); if (unlikely(!__pyx_float_7_98)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_7_99 = PyFloat_FromDouble(7.99); if (unlikely(!__pyx_float_7_99)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_8_00 = PyFloat_FromDouble(8.00); if (unlikely(!__pyx_float_8_00)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_8_01 = PyFloat_FromDouble(8.01); if (unlikely(!__pyx_float_8_01)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_8_02 = PyFloat_FromDouble(8.02); if (unlikely(!__pyx_float_8_02)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_8_04 = PyFloat_FromDouble(8.04); if (unlikely(!__pyx_float_8_04)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_8_05 = PyFloat_FromDouble(8.05); if (unlikely(!__pyx_float_8_05)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_8_06 = PyFloat_FromDouble(8.06); if (unlikely(!__pyx_float_8_06)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_8_08 = PyFloat_FromDouble(8.08); if (unlikely(!__pyx_float_8_08)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_8_09 = PyFloat_FromDouble(8.09); if (unlikely(!__pyx_float_8_09)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_8_11 = PyFloat_FromDouble(8.11); if (unlikely(!__pyx_float_8_11)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_8_12 = PyFloat_FromDouble(8.12); if (unlikely(!__pyx_float_8_12)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_8_14 = PyFloat_FromDouble(8.14); if (unlikely(!__pyx_float_8_14)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_8_17 = PyFloat_FromDouble(8.17); if (unlikely(!__pyx_float_8_17)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_8_19 = PyFloat_FromDouble(8.19); if (unlikely(!__pyx_float_8_19)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_8_20 = PyFloat_FromDouble(8.20); if (unlikely(!__pyx_float_8_20)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_8_22 = PyFloat_FromDouble(8.22); if (unlikely(!__pyx_float_8_22)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_8_29 = PyFloat_FromDouble(8.29); if (unlikely(!__pyx_float_8_29)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_8_30 = PyFloat_FromDouble(8.30); if (unlikely(!__pyx_float_8_30)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_8_32 = PyFloat_FromDouble(8.32); if (unlikely(!__pyx_float_8_32)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_8_33 = PyFloat_FromDouble(8.33); if (unlikely(!__pyx_float_8_33)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_8_36 = PyFloat_FromDouble(8.36); if (unlikely(!__pyx_float_8_36)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_8_39 = PyFloat_FromDouble(8.39); if (unlikely(!__pyx_float_8_39)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_8_40 = PyFloat_FromDouble(8.40); if (unlikely(!__pyx_float_8_40)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_8_46 = PyFloat_FromDouble(8.46); if (unlikely(!__pyx_float_8_46)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_8_50 = PyFloat_FromDouble(8.50); if (unlikely(!__pyx_float_8_50)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_8_63 = PyFloat_FromDouble(8.63); if (unlikely(!__pyx_float_8_63)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_8_94 = PyFloat_FromDouble(8.94); if (unlikely(!__pyx_float_8_94)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_8_95 = PyFloat_FromDouble(8.95); if (unlikely(!__pyx_float_8_95)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_8_96 = PyFloat_FromDouble(8.96); if (unlikely(!__pyx_float_8_96)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_8_97 = PyFloat_FromDouble(8.97); if (unlikely(!__pyx_float_8_97)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_8_98 = PyFloat_FromDouble(8.98); if (unlikely(!__pyx_float_8_98)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_8_99 = PyFloat_FromDouble(8.99); if (unlikely(!__pyx_float_8_99)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_9_00 = PyFloat_FromDouble(9.00); if (unlikely(!__pyx_float_9_00)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_9_01 = PyFloat_FromDouble(9.01); if (unlikely(!__pyx_float_9_01)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_9_04 = PyFloat_FromDouble(9.04); if (unlikely(!__pyx_float_9_04)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_9_07 = PyFloat_FromDouble(9.07); if (unlikely(!__pyx_float_9_07)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_9_08 = PyFloat_FromDouble(9.08); if (unlikely(!__pyx_float_9_08)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_9_10 = PyFloat_FromDouble(9.10); if (unlikely(!__pyx_float_9_10)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_9_11 = PyFloat_FromDouble(9.11); if (unlikely(!__pyx_float_9_11)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_9_13 = PyFloat_FromDouble(9.13); if (unlikely(!__pyx_float_9_13)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_9_15 = PyFloat_FromDouble(9.15); if (unlikely(!__pyx_float_9_15)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_9_16 = PyFloat_FromDouble(9.16); if (unlikely(!__pyx_float_9_16)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_9_19 = PyFloat_FromDouble(9.19); if (unlikely(!__pyx_float_9_19)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_9_21 = PyFloat_FromDouble(9.21); if (unlikely(!__pyx_float_9_21)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_9_23 = PyFloat_FromDouble(9.23); if (unlikely(!__pyx_float_9_23)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_9_25 = PyFloat_FromDouble(9.25); if (unlikely(!__pyx_float_9_25)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_9_27 = PyFloat_FromDouble(9.27); if (unlikely(!__pyx_float_9_27)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_9_29 = PyFloat_FromDouble(9.29); if (unlikely(!__pyx_float_9_29)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_9_32 = PyFloat_FromDouble(9.32); if (unlikely(!__pyx_float_9_32)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_9_43 = PyFloat_FromDouble(9.43); if (unlikely(!__pyx_float_9_43)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_9_50 = PyFloat_FromDouble(9.50); if (unlikely(!__pyx_float_9_50)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_9_51 = PyFloat_FromDouble(9.51); if (unlikely(!__pyx_float_9_51)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_9_53 = PyFloat_FromDouble(9.53); if (unlikely(!__pyx_float_9_53)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_9_58 = PyFloat_FromDouble(9.58); if (unlikely(!__pyx_float_9_58)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_9_66 = PyFloat_FromDouble(9.66); if (unlikely(!__pyx_float_9_66)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_9_92 = PyFloat_FromDouble(9.92); if (unlikely(!__pyx_float_9_92)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_9_95 = PyFloat_FromDouble(9.95); if (unlikely(!__pyx_float_9_95)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_9_96 = PyFloat_FromDouble(9.96); if (unlikely(!__pyx_float_9_96)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_9_98 = PyFloat_FromDouble(9.98); if (unlikely(!__pyx_float_9_98)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_9_99 = PyFloat_FromDouble(9.99); if (unlikely(!__pyx_float_9_99)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0001 = PyFloat_FromDouble(.0001); if (unlikely(!__pyx_float__0001)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0002 = PyFloat_FromDouble(.0002); if (unlikely(!__pyx_float__0002)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0003 = PyFloat_FromDouble(.0003); if (unlikely(!__pyx_float__0003)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0004 = PyFloat_FromDouble(.0004); if (unlikely(!__pyx_float__0004)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0005 = PyFloat_FromDouble(.0005); if (unlikely(!__pyx_float__0005)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0006 = PyFloat_FromDouble(.0006); if (unlikely(!__pyx_float__0006)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0007 = PyFloat_FromDouble(.0007); if (unlikely(!__pyx_float__0007)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0008 = PyFloat_FromDouble(.0008); if (unlikely(!__pyx_float__0008)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0009 = PyFloat_FromDouble(.0009); if (unlikely(!__pyx_float__0009)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0010 = PyFloat_FromDouble(.0010); if (unlikely(!__pyx_float__0010)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0011 = PyFloat_FromDouble(.0011); if (unlikely(!__pyx_float__0011)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0012 = PyFloat_FromDouble(.0012); if (unlikely(!__pyx_float__0012)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0013 = PyFloat_FromDouble(.0013); if (unlikely(!__pyx_float__0013)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0014 = PyFloat_FromDouble(.0014); if (unlikely(!__pyx_float__0014)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0015 = PyFloat_FromDouble(.0015); if (unlikely(!__pyx_float__0015)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0016 = PyFloat_FromDouble(.0016); if (unlikely(!__pyx_float__0016)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0017 = PyFloat_FromDouble(.0017); if (unlikely(!__pyx_float__0017)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0018 = PyFloat_FromDouble(.0018); if (unlikely(!__pyx_float__0018)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0019 = PyFloat_FromDouble(.0019); if (unlikely(!__pyx_float__0019)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0020 = PyFloat_FromDouble(.0020); if (unlikely(!__pyx_float__0020)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0021 = PyFloat_FromDouble(.0021); if (unlikely(!__pyx_float__0021)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0022 = PyFloat_FromDouble(.0022); if (unlikely(!__pyx_float__0022)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0023 = PyFloat_FromDouble(.0023); if (unlikely(!__pyx_float__0023)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0024 = PyFloat_FromDouble(.0024); if (unlikely(!__pyx_float__0024)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0025 = PyFloat_FromDouble(.0025); if (unlikely(!__pyx_float__0025)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0026 = PyFloat_FromDouble(.0026); if (unlikely(!__pyx_float__0026)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0027 = PyFloat_FromDouble(.0027); if (unlikely(!__pyx_float__0027)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0028 = PyFloat_FromDouble(.0028); if (unlikely(!__pyx_float__0028)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0029 = PyFloat_FromDouble(.0029); if (unlikely(!__pyx_float__0029)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0030 = PyFloat_FromDouble(.0030); if (unlikely(!__pyx_float__0030)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0031 = PyFloat_FromDouble(.0031); if (unlikely(!__pyx_float__0031)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0032 = PyFloat_FromDouble(.0032); if (unlikely(!__pyx_float__0032)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0033 = PyFloat_FromDouble(.0033); if (unlikely(!__pyx_float__0033)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0034 = PyFloat_FromDouble(.0034); if (unlikely(!__pyx_float__0034)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0035 = PyFloat_FromDouble(.0035); if (unlikely(!__pyx_float__0035)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0036 = PyFloat_FromDouble(.0036); if (unlikely(!__pyx_float__0036)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0037 = PyFloat_FromDouble(.0037); if (unlikely(!__pyx_float__0037)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0038 = PyFloat_FromDouble(.0038); if (unlikely(!__pyx_float__0038)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0039 = PyFloat_FromDouble(.0039); if (unlikely(!__pyx_float__0039)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0040 = PyFloat_FromDouble(.0040); if (unlikely(!__pyx_float__0040)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0041 = PyFloat_FromDouble(.0041); if (unlikely(!__pyx_float__0041)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0042 = PyFloat_FromDouble(.0042); if (unlikely(!__pyx_float__0042)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0043 = PyFloat_FromDouble(.0043); if (unlikely(!__pyx_float__0043)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0044 = PyFloat_FromDouble(.0044); if (unlikely(!__pyx_float__0044)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0045 = PyFloat_FromDouble(.0045); if (unlikely(!__pyx_float__0045)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0046 = PyFloat_FromDouble(.0046); if (unlikely(!__pyx_float__0046)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0047 = PyFloat_FromDouble(.0047); if (unlikely(!__pyx_float__0047)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0048 = PyFloat_FromDouble(.0048); if (unlikely(!__pyx_float__0048)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0049 = PyFloat_FromDouble(.0049); if (unlikely(!__pyx_float__0049)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0050 = PyFloat_FromDouble(.0050); if (unlikely(!__pyx_float__0050)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0051 = PyFloat_FromDouble(.0051); if (unlikely(!__pyx_float__0051)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0052 = PyFloat_FromDouble(.0052); if (unlikely(!__pyx_float__0052)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0053 = PyFloat_FromDouble(.0053); if (unlikely(!__pyx_float__0053)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0054 = PyFloat_FromDouble(.0054); if (unlikely(!__pyx_float__0054)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0055 = PyFloat_FromDouble(.0055); if (unlikely(!__pyx_float__0055)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0056 = PyFloat_FromDouble(.0056); if (unlikely(!__pyx_float__0056)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0057 = PyFloat_FromDouble(.0057); if (unlikely(!__pyx_float__0057)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0058 = PyFloat_FromDouble(.0058); if (unlikely(!__pyx_float__0058)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0059 = PyFloat_FromDouble(.0059); if (unlikely(!__pyx_float__0059)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0060 = PyFloat_FromDouble(.0060); if (unlikely(!__pyx_float__0060)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0061 = PyFloat_FromDouble(.0061); if (unlikely(!__pyx_float__0061)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0062 = PyFloat_FromDouble(.0062); if (unlikely(!__pyx_float__0062)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0063 = PyFloat_FromDouble(.0063); if (unlikely(!__pyx_float__0063)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0064 = PyFloat_FromDouble(.0064); if (unlikely(!__pyx_float__0064)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0065 = PyFloat_FromDouble(.0065); if (unlikely(!__pyx_float__0065)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0066 = PyFloat_FromDouble(.0066); if (unlikely(!__pyx_float__0066)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0067 = PyFloat_FromDouble(.0067); if (unlikely(!__pyx_float__0067)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0068 = PyFloat_FromDouble(.0068); if (unlikely(!__pyx_float__0068)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0069 = PyFloat_FromDouble(.0069); if (unlikely(!__pyx_float__0069)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0070 = PyFloat_FromDouble(.0070); if (unlikely(!__pyx_float__0070)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0071 = PyFloat_FromDouble(.0071); if (unlikely(!__pyx_float__0071)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0072 = PyFloat_FromDouble(.0072); if (unlikely(!__pyx_float__0072)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0073 = PyFloat_FromDouble(.0073); if (unlikely(!__pyx_float__0073)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0074 = PyFloat_FromDouble(.0074); if (unlikely(!__pyx_float__0074)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0075 = PyFloat_FromDouble(.0075); if (unlikely(!__pyx_float__0075)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0076 = PyFloat_FromDouble(.0076); if (unlikely(!__pyx_float__0076)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0077 = PyFloat_FromDouble(.0077); if (unlikely(!__pyx_float__0077)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0078 = PyFloat_FromDouble(.0078); if (unlikely(!__pyx_float__0078)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0080 = PyFloat_FromDouble(.0080); if (unlikely(!__pyx_float__0080)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0081 = PyFloat_FromDouble(.0081); if (unlikely(!__pyx_float__0081)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0082 = PyFloat_FromDouble(.0082); if (unlikely(!__pyx_float__0082)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0083 = PyFloat_FromDouble(.0083); if (unlikely(!__pyx_float__0083)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0084 = PyFloat_FromDouble(.0084); if (unlikely(!__pyx_float__0084)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0085 = PyFloat_FromDouble(.0085); if (unlikely(!__pyx_float__0085)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0086 = PyFloat_FromDouble(.0086); if (unlikely(!__pyx_float__0086)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0087 = PyFloat_FromDouble(.0087); if (unlikely(!__pyx_float__0087)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0088 = PyFloat_FromDouble(.0088); if (unlikely(!__pyx_float__0088)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0089 = PyFloat_FromDouble(.0089); if (unlikely(!__pyx_float__0089)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0090 = PyFloat_FromDouble(.0090); if (unlikely(!__pyx_float__0090)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0091 = PyFloat_FromDouble(.0091); if (unlikely(!__pyx_float__0091)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0092 = PyFloat_FromDouble(.0092); if (unlikely(!__pyx_float__0092)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0093 = PyFloat_FromDouble(.0093); if (unlikely(!__pyx_float__0093)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0094 = PyFloat_FromDouble(.0094); if (unlikely(!__pyx_float__0094)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0095 = PyFloat_FromDouble(.0095); if (unlikely(!__pyx_float__0095)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0097 = PyFloat_FromDouble(.0097); if (unlikely(!__pyx_float__0097)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0098 = PyFloat_FromDouble(.0098); if (unlikely(!__pyx_float__0098)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0099 = PyFloat_FromDouble(.0099); if (unlikely(!__pyx_float__0099)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0100 = PyFloat_FromDouble(.0100); if (unlikely(!__pyx_float__0100)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0101 = PyFloat_FromDouble(.0101); if (unlikely(!__pyx_float__0101)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0102 = PyFloat_FromDouble(.0102); if (unlikely(!__pyx_float__0102)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0103 = PyFloat_FromDouble(.0103); if (unlikely(!__pyx_float__0103)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0104 = PyFloat_FromDouble(.0104); if (unlikely(!__pyx_float__0104)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0105 = PyFloat_FromDouble(.0105); if (unlikely(!__pyx_float__0105)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0106 = PyFloat_FromDouble(.0106); if (unlikely(!__pyx_float__0106)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0107 = PyFloat_FromDouble(.0107); if (unlikely(!__pyx_float__0107)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0108 = PyFloat_FromDouble(.0108); if (unlikely(!__pyx_float__0108)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0109 = PyFloat_FromDouble(.0109); if (unlikely(!__pyx_float__0109)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0111 = PyFloat_FromDouble(.0111); if (unlikely(!__pyx_float__0111)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0112 = PyFloat_FromDouble(.0112); if (unlikely(!__pyx_float__0112)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0113 = PyFloat_FromDouble(.0113); if (unlikely(!__pyx_float__0113)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0114 = PyFloat_FromDouble(.0114); if (unlikely(!__pyx_float__0114)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0115 = PyFloat_FromDouble(.0115); if (unlikely(!__pyx_float__0115)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0116 = PyFloat_FromDouble(.0116); if (unlikely(!__pyx_float__0116)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0117 = PyFloat_FromDouble(.0117); if (unlikely(!__pyx_float__0117)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0118 = PyFloat_FromDouble(.0118); if (unlikely(!__pyx_float__0118)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0119 = PyFloat_FromDouble(.0119); if (unlikely(!__pyx_float__0119)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0120 = PyFloat_FromDouble(.0120); if (unlikely(!__pyx_float__0120)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0121 = PyFloat_FromDouble(.0121); if (unlikely(!__pyx_float__0121)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0122 = PyFloat_FromDouble(.0122); if (unlikely(!__pyx_float__0122)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0123 = PyFloat_FromDouble(.0123); if (unlikely(!__pyx_float__0123)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0124 = PyFloat_FromDouble(.0124); if (unlikely(!__pyx_float__0124)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0125 = PyFloat_FromDouble(.0125); if (unlikely(!__pyx_float__0125)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0126 = PyFloat_FromDouble(.0126); if (unlikely(!__pyx_float__0126)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0127 = PyFloat_FromDouble(.0127); if (unlikely(!__pyx_float__0127)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0128 = PyFloat_FromDouble(.0128); if (unlikely(!__pyx_float__0128)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0129 = PyFloat_FromDouble(.0129); if (unlikely(!__pyx_float__0129)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0132 = PyFloat_FromDouble(.0132); if (unlikely(!__pyx_float__0132)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0133 = PyFloat_FromDouble(.0133); if (unlikely(!__pyx_float__0133)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0134 = PyFloat_FromDouble(.0134); if (unlikely(!__pyx_float__0134)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0135 = PyFloat_FromDouble(.0135); if (unlikely(!__pyx_float__0135)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0136 = PyFloat_FromDouble(.0136); if (unlikely(!__pyx_float__0136)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0137 = PyFloat_FromDouble(.0137); if (unlikely(!__pyx_float__0137)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0138 = PyFloat_FromDouble(.0138); if (unlikely(!__pyx_float__0138)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0139 = PyFloat_FromDouble(.0139); if (unlikely(!__pyx_float__0139)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0140 = PyFloat_FromDouble(.0140); if (unlikely(!__pyx_float__0140)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0141 = PyFloat_FromDouble(.0141); if (unlikely(!__pyx_float__0141)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0142 = PyFloat_FromDouble(.0142); if (unlikely(!__pyx_float__0142)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0144 = PyFloat_FromDouble(.0144); if (unlikely(!__pyx_float__0144)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0145 = PyFloat_FromDouble(.0145); if (unlikely(!__pyx_float__0145)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0147 = PyFloat_FromDouble(.0147); if (unlikely(!__pyx_float__0147)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0148 = PyFloat_FromDouble(.0148); if (unlikely(!__pyx_float__0148)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0151 = PyFloat_FromDouble(.0151); if (unlikely(!__pyx_float__0151)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0152 = PyFloat_FromDouble(.0152); if (unlikely(!__pyx_float__0152)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0153 = PyFloat_FromDouble(.0153); if (unlikely(!__pyx_float__0153)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0154 = PyFloat_FromDouble(.0154); if (unlikely(!__pyx_float__0154)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0155 = PyFloat_FromDouble(.0155); if (unlikely(!__pyx_float__0155)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0156 = PyFloat_FromDouble(.0156); if (unlikely(!__pyx_float__0156)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0158 = PyFloat_FromDouble(.0158); if (unlikely(!__pyx_float__0158)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0159 = PyFloat_FromDouble(.0159); if (unlikely(!__pyx_float__0159)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0160 = PyFloat_FromDouble(.0160); if (unlikely(!__pyx_float__0160)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0161 = PyFloat_FromDouble(.0161); if (unlikely(!__pyx_float__0161)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0162 = PyFloat_FromDouble(.0162); if (unlikely(!__pyx_float__0162)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0163 = PyFloat_FromDouble(.0163); if (unlikely(!__pyx_float__0163)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0165 = PyFloat_FromDouble(.0165); if (unlikely(!__pyx_float__0165)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0166 = PyFloat_FromDouble(.0166); if (unlikely(!__pyx_float__0166)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0167 = PyFloat_FromDouble(.0167); if (unlikely(!__pyx_float__0167)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0170 = PyFloat_FromDouble(.0170); if (unlikely(!__pyx_float__0170)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0171 = PyFloat_FromDouble(.0171); if (unlikely(!__pyx_float__0171)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0172 = PyFloat_FromDouble(.0172); if (unlikely(!__pyx_float__0172)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0173 = PyFloat_FromDouble(.0173); if (unlikely(!__pyx_float__0173)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0174 = PyFloat_FromDouble(.0174); if (unlikely(!__pyx_float__0174)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0175 = PyFloat_FromDouble(.0175); if (unlikely(!__pyx_float__0175)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0177 = PyFloat_FromDouble(.0177); if (unlikely(!__pyx_float__0177)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0178 = PyFloat_FromDouble(.0178); if (unlikely(!__pyx_float__0178)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0179 = PyFloat_FromDouble(.0179); if (unlikely(!__pyx_float__0179)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0181 = PyFloat_FromDouble(.0181); if (unlikely(!__pyx_float__0181)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0182 = PyFloat_FromDouble(.0182); if (unlikely(!__pyx_float__0182)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0183 = PyFloat_FromDouble(.0183); if (unlikely(!__pyx_float__0183)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0184 = PyFloat_FromDouble(.0184); if (unlikely(!__pyx_float__0184)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0185 = PyFloat_FromDouble(.0185); if (unlikely(!__pyx_float__0185)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0186 = PyFloat_FromDouble(.0186); if (unlikely(!__pyx_float__0186)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0189 = PyFloat_FromDouble(.0189); if (unlikely(!__pyx_float__0189)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0190 = PyFloat_FromDouble(.0190); if (unlikely(!__pyx_float__0190)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0191 = PyFloat_FromDouble(.0191); if (unlikely(!__pyx_float__0191)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0192 = PyFloat_FromDouble(.0192); if (unlikely(!__pyx_float__0192)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0194 = PyFloat_FromDouble(.0194); if (unlikely(!__pyx_float__0194)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0196 = PyFloat_FromDouble(.0196); if (unlikely(!__pyx_float__0196)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0197 = PyFloat_FromDouble(.0197); if (unlikely(!__pyx_float__0197)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0201 = PyFloat_FromDouble(.0201); if (unlikely(!__pyx_float__0201)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0202 = PyFloat_FromDouble(.0202); if (unlikely(!__pyx_float__0202)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0203 = PyFloat_FromDouble(.0203); if (unlikely(!__pyx_float__0203)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0206 = PyFloat_FromDouble(.0206); if (unlikely(!__pyx_float__0206)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0207 = PyFloat_FromDouble(.0207); if (unlikely(!__pyx_float__0207)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0209 = PyFloat_FromDouble(.0209); if (unlikely(!__pyx_float__0209)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0210 = PyFloat_FromDouble(.0210); if (unlikely(!__pyx_float__0210)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0211 = PyFloat_FromDouble(.0211); if (unlikely(!__pyx_float__0211)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0212 = PyFloat_FromDouble(.0212); if (unlikely(!__pyx_float__0212)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0213 = PyFloat_FromDouble(.0213); if (unlikely(!__pyx_float__0213)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0215 = PyFloat_FromDouble(.0215); if (unlikely(!__pyx_float__0215)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0217 = PyFloat_FromDouble(.0217); if (unlikely(!__pyx_float__0217)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0218 = PyFloat_FromDouble(.0218); if (unlikely(!__pyx_float__0218)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0221 = PyFloat_FromDouble(.0221); if (unlikely(!__pyx_float__0221)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0222 = PyFloat_FromDouble(.0222); if (unlikely(!__pyx_float__0222)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0225 = PyFloat_FromDouble(.0225); if (unlikely(!__pyx_float__0225)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0227 = PyFloat_FromDouble(.0227); if (unlikely(!__pyx_float__0227)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0228 = PyFloat_FromDouble(.0228); if (unlikely(!__pyx_float__0228)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0229 = PyFloat_FromDouble(.0229); if (unlikely(!__pyx_float__0229)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0230 = PyFloat_FromDouble(.0230); if (unlikely(!__pyx_float__0230)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0232 = PyFloat_FromDouble(.0232); if (unlikely(!__pyx_float__0232)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0233 = PyFloat_FromDouble(.0233); if (unlikely(!__pyx_float__0233)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0234 = PyFloat_FromDouble(.0234); if (unlikely(!__pyx_float__0234)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0235 = PyFloat_FromDouble(.0235); if (unlikely(!__pyx_float__0235)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0236 = PyFloat_FromDouble(.0236); if (unlikely(!__pyx_float__0236)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0237 = PyFloat_FromDouble(.0237); if (unlikely(!__pyx_float__0237)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0238 = PyFloat_FromDouble(.0238); if (unlikely(!__pyx_float__0238)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0240 = PyFloat_FromDouble(.0240); if (unlikely(!__pyx_float__0240)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0241 = PyFloat_FromDouble(.0241); if (unlikely(!__pyx_float__0241)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0242 = PyFloat_FromDouble(.0242); if (unlikely(!__pyx_float__0242)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0245 = PyFloat_FromDouble(.0245); if (unlikely(!__pyx_float__0245)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0246 = PyFloat_FromDouble(.0246); if (unlikely(!__pyx_float__0246)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0247 = PyFloat_FromDouble(.0247); if (unlikely(!__pyx_float__0247)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0248 = PyFloat_FromDouble(.0248); if (unlikely(!__pyx_float__0248)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0249 = PyFloat_FromDouble(.0249); if (unlikely(!__pyx_float__0249)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0250 = PyFloat_FromDouble(.0250); if (unlikely(!__pyx_float__0250)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0252 = PyFloat_FromDouble(.0252); if (unlikely(!__pyx_float__0252)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0256 = PyFloat_FromDouble(.0256); if (unlikely(!__pyx_float__0256)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0257 = PyFloat_FromDouble(.0257); if (unlikely(!__pyx_float__0257)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0258 = PyFloat_FromDouble(.0258); if (unlikely(!__pyx_float__0258)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0259 = PyFloat_FromDouble(.0259); if (unlikely(!__pyx_float__0259)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0260 = PyFloat_FromDouble(.0260); if (unlikely(!__pyx_float__0260)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0264 = PyFloat_FromDouble(.0264); if (unlikely(!__pyx_float__0264)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0265 = PyFloat_FromDouble(.0265); if (unlikely(!__pyx_float__0265)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0266 = PyFloat_FromDouble(.0266); if (unlikely(!__pyx_float__0266)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0268 = PyFloat_FromDouble(.0268); if (unlikely(!__pyx_float__0268)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0272 = PyFloat_FromDouble(.0272); if (unlikely(!__pyx_float__0272)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0274 = PyFloat_FromDouble(.0274); if (unlikely(!__pyx_float__0274)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0276 = PyFloat_FromDouble(.0276); if (unlikely(!__pyx_float__0276)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0278 = PyFloat_FromDouble(.0278); if (unlikely(!__pyx_float__0278)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0279 = PyFloat_FromDouble(.0279); if (unlikely(!__pyx_float__0279)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0280 = PyFloat_FromDouble(.0280); if (unlikely(!__pyx_float__0280)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0284 = PyFloat_FromDouble(.0284); if (unlikely(!__pyx_float__0284)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0285 = PyFloat_FromDouble(.0285); if (unlikely(!__pyx_float__0285)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0287 = PyFloat_FromDouble(.0287); if (unlikely(!__pyx_float__0287)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0289 = PyFloat_FromDouble(.0289); if (unlikely(!__pyx_float__0289)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0290 = PyFloat_FromDouble(.0290); if (unlikely(!__pyx_float__0290)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0291 = PyFloat_FromDouble(.0291); if (unlikely(!__pyx_float__0291)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0292 = PyFloat_FromDouble(.0292); if (unlikely(!__pyx_float__0292)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0293 = PyFloat_FromDouble(.0293); if (unlikely(!__pyx_float__0293)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0297 = PyFloat_FromDouble(.0297); if (unlikely(!__pyx_float__0297)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0298 = PyFloat_FromDouble(.0298); if (unlikely(!__pyx_float__0298)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0299 = PyFloat_FromDouble(.0299); if (unlikely(!__pyx_float__0299)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0300 = PyFloat_FromDouble(.0300); if (unlikely(!__pyx_float__0300)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0301 = PyFloat_FromDouble(.0301); if (unlikely(!__pyx_float__0301)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0305 = PyFloat_FromDouble(.0305); if (unlikely(!__pyx_float__0305)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0306 = PyFloat_FromDouble(.0306); if (unlikely(!__pyx_float__0306)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0307 = PyFloat_FromDouble(.0307); if (unlikely(!__pyx_float__0307)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0309 = PyFloat_FromDouble(.0309); if (unlikely(!__pyx_float__0309)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0310 = PyFloat_FromDouble(.0310); if (unlikely(!__pyx_float__0310)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0311 = PyFloat_FromDouble(.0311); if (unlikely(!__pyx_float__0311)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0313 = PyFloat_FromDouble(.0313); if (unlikely(!__pyx_float__0313)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0314 = PyFloat_FromDouble(.0314); if (unlikely(!__pyx_float__0314)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0315 = PyFloat_FromDouble(.0315); if (unlikely(!__pyx_float__0315)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0319 = PyFloat_FromDouble(.0319); if (unlikely(!__pyx_float__0319)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0322 = PyFloat_FromDouble(.0322); if (unlikely(!__pyx_float__0322)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0324 = PyFloat_FromDouble(.0324); if (unlikely(!__pyx_float__0324)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0326 = PyFloat_FromDouble(.0326); if (unlikely(!__pyx_float__0326)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0328 = PyFloat_FromDouble(.0328); if (unlikely(!__pyx_float__0328)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0329 = PyFloat_FromDouble(.0329); if (unlikely(!__pyx_float__0329)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0332 = PyFloat_FromDouble(.0332); if (unlikely(!__pyx_float__0332)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0334 = PyFloat_FromDouble(.0334); if (unlikely(!__pyx_float__0334)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0336 = PyFloat_FromDouble(.0336); if (unlikely(!__pyx_float__0336)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0340 = PyFloat_FromDouble(.0340); if (unlikely(!__pyx_float__0340)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0342 = PyFloat_FromDouble(.0342); if (unlikely(!__pyx_float__0342)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0345 = PyFloat_FromDouble(.0345); if (unlikely(!__pyx_float__0345)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0349 = PyFloat_FromDouble(.0349); if (unlikely(!__pyx_float__0349)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0353 = PyFloat_FromDouble(.0353); if (unlikely(!__pyx_float__0353)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0355 = PyFloat_FromDouble(.0355); if (unlikely(!__pyx_float__0355)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0363 = PyFloat_FromDouble(.0363); if (unlikely(!__pyx_float__0363)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0364 = PyFloat_FromDouble(.0364); if (unlikely(!__pyx_float__0364)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0365 = PyFloat_FromDouble(.0365); if (unlikely(!__pyx_float__0365)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0367 = PyFloat_FromDouble(.0367); if (unlikely(!__pyx_float__0367)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0368 = PyFloat_FromDouble(.0368); if (unlikely(!__pyx_float__0368)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0369 = PyFloat_FromDouble(.0369); if (unlikely(!__pyx_float__0369)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0370 = PyFloat_FromDouble(.0370); if (unlikely(!__pyx_float__0370)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0373 = PyFloat_FromDouble(.0373); if (unlikely(!__pyx_float__0373)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0381 = PyFloat_FromDouble(.0381); if (unlikely(!__pyx_float__0381)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0387 = PyFloat_FromDouble(.0387); if (unlikely(!__pyx_float__0387)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0389 = PyFloat_FromDouble(.0389); if (unlikely(!__pyx_float__0389)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0390 = PyFloat_FromDouble(.0390); if (unlikely(!__pyx_float__0390)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0392 = PyFloat_FromDouble(.0392); if (unlikely(!__pyx_float__0392)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0396 = PyFloat_FromDouble(.0396); if (unlikely(!__pyx_float__0396)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0402 = PyFloat_FromDouble(.0402); if (unlikely(!__pyx_float__0402)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0403 = PyFloat_FromDouble(.0403); if (unlikely(!__pyx_float__0403)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0404 = PyFloat_FromDouble(.0404); if (unlikely(!__pyx_float__0404)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0410 = PyFloat_FromDouble(.0410); if (unlikely(!__pyx_float__0410)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0411 = PyFloat_FromDouble(.0411); if (unlikely(!__pyx_float__0411)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0415 = PyFloat_FromDouble(.0415); if (unlikely(!__pyx_float__0415)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0418 = PyFloat_FromDouble(.0418); if (unlikely(!__pyx_float__0418)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0420 = PyFloat_FromDouble(.0420); if (unlikely(!__pyx_float__0420)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0421 = PyFloat_FromDouble(.0421); if (unlikely(!__pyx_float__0421)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0423 = PyFloat_FromDouble(.0423); if (unlikely(!__pyx_float__0423)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0425 = PyFloat_FromDouble(.0425); if (unlikely(!__pyx_float__0425)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0437 = PyFloat_FromDouble(.0437); if (unlikely(!__pyx_float__0437)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0445 = PyFloat_FromDouble(.0445); if (unlikely(!__pyx_float__0445)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0448 = PyFloat_FromDouble(.0448); if (unlikely(!__pyx_float__0448)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0449 = PyFloat_FromDouble(.0449); if (unlikely(!__pyx_float__0449)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0465 = PyFloat_FromDouble(.0465); if (unlikely(!__pyx_float__0465)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0472 = PyFloat_FromDouble(.0472); if (unlikely(!__pyx_float__0472)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0473 = PyFloat_FromDouble(.0473); if (unlikely(!__pyx_float__0473)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0475 = PyFloat_FromDouble(.0475); if (unlikely(!__pyx_float__0475)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0482 = PyFloat_FromDouble(.0482); if (unlikely(!__pyx_float__0482)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0492 = PyFloat_FromDouble(.0492); if (unlikely(!__pyx_float__0492)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0493 = PyFloat_FromDouble(.0493); if (unlikely(!__pyx_float__0493)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0497 = PyFloat_FromDouble(.0497); if (unlikely(!__pyx_float__0497)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0498 = PyFloat_FromDouble(.0498); if (unlikely(!__pyx_float__0498)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0500 = PyFloat_FromDouble(.0500); if (unlikely(!__pyx_float__0500)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0503 = PyFloat_FromDouble(.0503); if (unlikely(!__pyx_float__0503)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0506 = PyFloat_FromDouble(.0506); if (unlikely(!__pyx_float__0506)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0512 = PyFloat_FromDouble(.0512); if (unlikely(!__pyx_float__0512)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0514 = PyFloat_FromDouble(.0514); if (unlikely(!__pyx_float__0514)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0516 = PyFloat_FromDouble(.0516); if (unlikely(!__pyx_float__0516)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0517 = PyFloat_FromDouble(.0517); if (unlikely(!__pyx_float__0517)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0522 = PyFloat_FromDouble(.0522); if (unlikely(!__pyx_float__0522)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0523 = PyFloat_FromDouble(.0523); if (unlikely(!__pyx_float__0523)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0525 = PyFloat_FromDouble(.0525); if (unlikely(!__pyx_float__0525)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0578 = PyFloat_FromDouble(.0578); if (unlikely(!__pyx_float__0578)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0593 = PyFloat_FromDouble(.0593); if (unlikely(!__pyx_float__0593)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0596 = PyFloat_FromDouble(.0596); if (unlikely(!__pyx_float__0596)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0613 = PyFloat_FromDouble(.0613); if (unlikely(!__pyx_float__0613)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0621 = PyFloat_FromDouble(.0621); if (unlikely(!__pyx_float__0621)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0629 = PyFloat_FromDouble(.0629); if (unlikely(!__pyx_float__0629)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0650 = PyFloat_FromDouble(.0650); if (unlikely(!__pyx_float__0650)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0654 = PyFloat_FromDouble(.0654); if (unlikely(!__pyx_float__0654)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0669 = PyFloat_FromDouble(.0669); if (unlikely(!__pyx_float__0669)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0674 = PyFloat_FromDouble(.0674); if (unlikely(!__pyx_float__0674)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0676 = PyFloat_FromDouble(.0676); if (unlikely(!__pyx_float__0676)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0678 = PyFloat_FromDouble(.0678); if (unlikely(!__pyx_float__0678)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0683 = PyFloat_FromDouble(.0683); if (unlikely(!__pyx_float__0683)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0692 = PyFloat_FromDouble(.0692); if (unlikely(!__pyx_float__0692)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0698 = PyFloat_FromDouble(.0698); if (unlikely(!__pyx_float__0698)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0708 = PyFloat_FromDouble(.0708); if (unlikely(!__pyx_float__0708)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0714 = PyFloat_FromDouble(.0714); if (unlikely(!__pyx_float__0714)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0719 = PyFloat_FromDouble(.0719); if (unlikely(!__pyx_float__0719)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0726 = PyFloat_FromDouble(.0726); if (unlikely(!__pyx_float__0726)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0727 = PyFloat_FromDouble(.0727); if (unlikely(!__pyx_float__0727)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0752 = PyFloat_FromDouble(.0752); if (unlikely(!__pyx_float__0752)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0763 = PyFloat_FromDouble(.0763); if (unlikely(!__pyx_float__0763)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0765 = PyFloat_FromDouble(.0765); if (unlikely(!__pyx_float__0765)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0773 = PyFloat_FromDouble(.0773); if (unlikely(!__pyx_float__0773)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0813 = PyFloat_FromDouble(.0813); if (unlikely(!__pyx_float__0813)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0820 = PyFloat_FromDouble(.0820); if (unlikely(!__pyx_float__0820)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0835 = PyFloat_FromDouble(.0835); if (unlikely(!__pyx_float__0835)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0841 = PyFloat_FromDouble(.0841); if (unlikely(!__pyx_float__0841)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0846 = PyFloat_FromDouble(.0846); if (unlikely(!__pyx_float__0846)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0877 = PyFloat_FromDouble(.0877); if (unlikely(!__pyx_float__0877)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0909 = PyFloat_FromDouble(.0909); if (unlikely(!__pyx_float__0909)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0919 = PyFloat_FromDouble(.0919); if (unlikely(!__pyx_float__0919)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0920 = PyFloat_FromDouble(.0920); if (unlikely(!__pyx_float__0920)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0921 = PyFloat_FromDouble(.0921); if (unlikely(!__pyx_float__0921)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0933 = PyFloat_FromDouble(.0933); if (unlikely(!__pyx_float__0933)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0938 = PyFloat_FromDouble(.0938); if (unlikely(!__pyx_float__0938)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0951 = PyFloat_FromDouble(.0951); if (unlikely(!__pyx_float__0951)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0963 = PyFloat_FromDouble(.0963); if (unlikely(!__pyx_float__0963)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0967 = PyFloat_FromDouble(.0967); if (unlikely(!__pyx_float__0967)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0976 = PyFloat_FromDouble(.0976); if (unlikely(!__pyx_float__0976)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0982 = PyFloat_FromDouble(.0982); if (unlikely(!__pyx_float__0982)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__0985 = PyFloat_FromDouble(.0985); if (unlikely(!__pyx_float__0985)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1008 = PyFloat_FromDouble(.1008); if (unlikely(!__pyx_float__1008)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1011 = PyFloat_FromDouble(.1011); if (unlikely(!__pyx_float__1011)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1015 = PyFloat_FromDouble(.1015); if (unlikely(!__pyx_float__1015)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1018 = PyFloat_FromDouble(.1018); if (unlikely(!__pyx_float__1018)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1054 = PyFloat_FromDouble(.1054); if (unlikely(!__pyx_float__1054)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1066 = PyFloat_FromDouble(.1066); if (unlikely(!__pyx_float__1066)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1085 = PyFloat_FromDouble(.1085); if (unlikely(!__pyx_float__1085)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1126 = PyFloat_FromDouble(.1126); if (unlikely(!__pyx_float__1126)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1132 = PyFloat_FromDouble(.1132); if (unlikely(!__pyx_float__1132)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1174 = PyFloat_FromDouble(.1174); if (unlikely(!__pyx_float__1174)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1193 = PyFloat_FromDouble(.1193); if (unlikely(!__pyx_float__1193)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1238 = PyFloat_FromDouble(.1238); if (unlikely(!__pyx_float__1238)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1243 = PyFloat_FromDouble(.1243); if (unlikely(!__pyx_float__1243)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1273 = PyFloat_FromDouble(.1273); if (unlikely(!__pyx_float__1273)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1296 = PyFloat_FromDouble(.1296); if (unlikely(!__pyx_float__1296)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1304 = PyFloat_FromDouble(.1304); if (unlikely(!__pyx_float__1304)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1311 = PyFloat_FromDouble(.1311); if (unlikely(!__pyx_float__1311)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1357 = PyFloat_FromDouble(.1357); if (unlikely(!__pyx_float__1357)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1400 = PyFloat_FromDouble(.1400); if (unlikely(!__pyx_float__1400)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1440 = PyFloat_FromDouble(.1440); if (unlikely(!__pyx_float__1440)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1441 = PyFloat_FromDouble(.1441); if (unlikely(!__pyx_float__1441)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1486 = PyFloat_FromDouble(.1486); if (unlikely(!__pyx_float__1486)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1541 = PyFloat_FromDouble(.1541); if (unlikely(!__pyx_float__1541)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1555 = PyFloat_FromDouble(.1555); if (unlikely(!__pyx_float__1555)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1611 = PyFloat_FromDouble(.1611); if (unlikely(!__pyx_float__1611)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1655 = PyFloat_FromDouble(.1655); if (unlikely(!__pyx_float__1655)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1715 = PyFloat_FromDouble(.1715); if (unlikely(!__pyx_float__1715)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1821 = PyFloat_FromDouble(.1821); if (unlikely(!__pyx_float__1821)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1822 = PyFloat_FromDouble(.1822); if (unlikely(!__pyx_float__1822)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1868 = PyFloat_FromDouble(.1868); if (unlikely(!__pyx_float__1868)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1889 = PyFloat_FromDouble(.1889); if (unlikely(!__pyx_float__1889)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1919 = PyFloat_FromDouble(.1919); if (unlikely(!__pyx_float__1919)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__1933 = PyFloat_FromDouble(.1933); if (unlikely(!__pyx_float__1933)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__2095 = PyFloat_FromDouble(.2095); if (unlikely(!__pyx_float__2095)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__2113 = PyFloat_FromDouble(.2113); if (unlikely(!__pyx_float__2113)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__2185 = PyFloat_FromDouble(.2185); if (unlikely(!__pyx_float__2185)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__2270 = PyFloat_FromDouble(.2270); if (unlikely(!__pyx_float__2270)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__2388 = PyFloat_FromDouble(.2388); if (unlikely(!__pyx_float__2388)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__2455 = PyFloat_FromDouble(.2455); if (unlikely(!__pyx_float__2455)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__2598 = PyFloat_FromDouble(.2598); if (unlikely(!__pyx_float__2598)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__2735 = PyFloat_FromDouble(.2735); if (unlikely(!__pyx_float__2735)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__3637 = PyFloat_FromDouble(.3637); if (unlikely(!__pyx_float__3637)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_10_00 = PyFloat_FromDouble(10.00); if (unlikely(!__pyx_float_10_00)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_10_01 = PyFloat_FromDouble(10.01); if (unlikely(!__pyx_float_10_01)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_10_02 = PyFloat_FromDouble(10.02); if (unlikely(!__pyx_float_10_02)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_10_03 = PyFloat_FromDouble(10.03); if (unlikely(!__pyx_float_10_03)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_10_04 = PyFloat_FromDouble(10.04); if (unlikely(!__pyx_float_10_04)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_10_05 = PyFloat_FromDouble(10.05); if (unlikely(!__pyx_float_10_05)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_10_08 = PyFloat_FromDouble(10.08); if (unlikely(!__pyx_float_10_08)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_10_10 = PyFloat_FromDouble(10.10); if (unlikely(!__pyx_float_10_10)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_10_11 = PyFloat_FromDouble(10.11); if (unlikely(!__pyx_float_10_11)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_10_12 = PyFloat_FromDouble(10.12); if (unlikely(!__pyx_float_10_12)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_10_15 = PyFloat_FromDouble(10.15); if (unlikely(!__pyx_float_10_15)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_10_17 = PyFloat_FromDouble(10.17); if (unlikely(!__pyx_float_10_17)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_10_18 = PyFloat_FromDouble(10.18); if (unlikely(!__pyx_float_10_18)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_10_22 = PyFloat_FromDouble(10.22); if (unlikely(!__pyx_float_10_22)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_10_23 = PyFloat_FromDouble(10.23); if (unlikely(!__pyx_float_10_23)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_10_25 = PyFloat_FromDouble(10.25); if (unlikely(!__pyx_float_10_25)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_10_30 = PyFloat_FromDouble(10.30); if (unlikely(!__pyx_float_10_30)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_10_33 = PyFloat_FromDouble(10.33); if (unlikely(!__pyx_float_10_33)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_10_36 = PyFloat_FromDouble(10.36); if (unlikely(!__pyx_float_10_36)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_10_38 = PyFloat_FromDouble(10.38); if (unlikely(!__pyx_float_10_38)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_10_40 = PyFloat_FromDouble(10.40); if (unlikely(!__pyx_float_10_40)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_10_45 = PyFloat_FromDouble(10.45); if (unlikely(!__pyx_float_10_45)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_10_59 = PyFloat_FromDouble(10.59); if (unlikely(!__pyx_float_10_59)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_10_69 = PyFloat_FromDouble(10.69); if (unlikely(!__pyx_float_10_69)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_10_93 = PyFloat_FromDouble(10.93); if (unlikely(!__pyx_float_10_93)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_10_96 = PyFloat_FromDouble(10.96); if (unlikely(!__pyx_float_10_96)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_10_98 = PyFloat_FromDouble(10.98); if (unlikely(!__pyx_float_10_98)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_10_99 = PyFloat_FromDouble(10.99); if (unlikely(!__pyx_float_10_99)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_11_00 = PyFloat_FromDouble(11.00); if (unlikely(!__pyx_float_11_00)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_11_01 = PyFloat_FromDouble(11.01); if (unlikely(!__pyx_float_11_01)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_11_02 = PyFloat_FromDouble(11.02); if (unlikely(!__pyx_float_11_02)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_11_03 = PyFloat_FromDouble(11.03); if (unlikely(!__pyx_float_11_03)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_11_04 = PyFloat_FromDouble(11.04); if (unlikely(!__pyx_float_11_04)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_11_05 = PyFloat_FromDouble(11.05); if (unlikely(!__pyx_float_11_05)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_11_07 = PyFloat_FromDouble(11.07); if (unlikely(!__pyx_float_11_07)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_11_08 = PyFloat_FromDouble(11.08); if (unlikely(!__pyx_float_11_08)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_11_11 = PyFloat_FromDouble(11.11); if (unlikely(!__pyx_float_11_11)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_11_14 = PyFloat_FromDouble(11.14); if (unlikely(!__pyx_float_11_14)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_11_15 = PyFloat_FromDouble(11.15); if (unlikely(!__pyx_float_11_15)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_11_17 = PyFloat_FromDouble(11.17); if (unlikely(!__pyx_float_11_17)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_11_19 = PyFloat_FromDouble(11.19); if (unlikely(!__pyx_float_11_19)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_11_23 = PyFloat_FromDouble(11.23); if (unlikely(!__pyx_float_11_23)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_11_25 = PyFloat_FromDouble(11.25); if (unlikely(!__pyx_float_11_25)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_11_30 = PyFloat_FromDouble(11.30); if (unlikely(!__pyx_float_11_30)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_11_31 = PyFloat_FromDouble(11.31); if (unlikely(!__pyx_float_11_31)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_11_35 = PyFloat_FromDouble(11.35); if (unlikely(!__pyx_float_11_35)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_11_40 = PyFloat_FromDouble(11.40); if (unlikely(!__pyx_float_11_40)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_11_44 = PyFloat_FromDouble(11.44); if (unlikely(!__pyx_float_11_44)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_11_48 = PyFloat_FromDouble(11.48); if (unlikely(!__pyx_float_11_48)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_11_53 = PyFloat_FromDouble(11.53); if (unlikely(!__pyx_float_11_53)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_11_58 = PyFloat_FromDouble(11.58); if (unlikely(!__pyx_float_11_58)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_11_75 = PyFloat_FromDouble(11.75); if (unlikely(!__pyx_float_11_75)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_11_78 = PyFloat_FromDouble(11.78); if (unlikely(!__pyx_float_11_78)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_11_94 = PyFloat_FromDouble(11.94); if (unlikely(!__pyx_float_11_94)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_11_97 = PyFloat_FromDouble(11.97); if (unlikely(!__pyx_float_11_97)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_11_98 = PyFloat_FromDouble(11.98); if (unlikely(!__pyx_float_11_98)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_11_99 = PyFloat_FromDouble(11.99); if (unlikely(!__pyx_float_11_99)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_12_00 = PyFloat_FromDouble(12.00); if (unlikely(!__pyx_float_12_00)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_12_01 = PyFloat_FromDouble(12.01); if (unlikely(!__pyx_float_12_01)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_12_02 = PyFloat_FromDouble(12.02); if (unlikely(!__pyx_float_12_02)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_12_03 = PyFloat_FromDouble(12.03); if (unlikely(!__pyx_float_12_03)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_12_05 = PyFloat_FromDouble(12.05); if (unlikely(!__pyx_float_12_05)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_12_06 = PyFloat_FromDouble(12.06); if (unlikely(!__pyx_float_12_06)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_12_07 = PyFloat_FromDouble(12.07); if (unlikely(!__pyx_float_12_07)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_12_08 = PyFloat_FromDouble(12.08); if (unlikely(!__pyx_float_12_08)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_12_10 = PyFloat_FromDouble(12.10); if (unlikely(!__pyx_float_12_10)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_12_11 = PyFloat_FromDouble(12.11); if (unlikely(!__pyx_float_12_11)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_12_12 = PyFloat_FromDouble(12.12); if (unlikely(!__pyx_float_12_12)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_12_15 = PyFloat_FromDouble(12.15); if (unlikely(!__pyx_float_12_15)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_12_18 = PyFloat_FromDouble(12.18); if (unlikely(!__pyx_float_12_18)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_12_20 = PyFloat_FromDouble(12.20); if (unlikely(!__pyx_float_12_20)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_12_24 = PyFloat_FromDouble(12.24); if (unlikely(!__pyx_float_12_24)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_12_26 = PyFloat_FromDouble(12.26); if (unlikely(!__pyx_float_12_26)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_12_30 = PyFloat_FromDouble(12.30); if (unlikely(!__pyx_float_12_30)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_12_32 = PyFloat_FromDouble(12.32); if (unlikely(!__pyx_float_12_32)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_12_33 = PyFloat_FromDouble(12.33); if (unlikely(!__pyx_float_12_33)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_12_40 = PyFloat_FromDouble(12.40); if (unlikely(!__pyx_float_12_40)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_12_45 = PyFloat_FromDouble(12.45); if (unlikely(!__pyx_float_12_45)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_12_50 = PyFloat_FromDouble(12.50); if (unlikely(!__pyx_float_12_50)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_12_58 = PyFloat_FromDouble(12.58); if (unlikely(!__pyx_float_12_58)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_12_63 = PyFloat_FromDouble(12.63); if (unlikely(!__pyx_float_12_63)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_12_67 = PyFloat_FromDouble(12.67); if (unlikely(!__pyx_float_12_67)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_12_70 = PyFloat_FromDouble(12.70); if (unlikely(!__pyx_float_12_70)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_12_78 = PyFloat_FromDouble(12.78); if (unlikely(!__pyx_float_12_78)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_12_95 = PyFloat_FromDouble(12.95); if (unlikely(!__pyx_float_12_95)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_12_96 = PyFloat_FromDouble(12.96); if (unlikely(!__pyx_float_12_96)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_12_98 = PyFloat_FromDouble(12.98); if (unlikely(!__pyx_float_12_98)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_12_99 = PyFloat_FromDouble(12.99); if (unlikely(!__pyx_float_12_99)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_13_00 = PyFloat_FromDouble(13.00); if (unlikely(!__pyx_float_13_00)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_13_01 = PyFloat_FromDouble(13.01); if (unlikely(!__pyx_float_13_01)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_13_02 = PyFloat_FromDouble(13.02); if (unlikely(!__pyx_float_13_02)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_13_03 = PyFloat_FromDouble(13.03); if (unlikely(!__pyx_float_13_03)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_13_04 = PyFloat_FromDouble(13.04); if (unlikely(!__pyx_float_13_04)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_13_05 = PyFloat_FromDouble(13.05); if (unlikely(!__pyx_float_13_05)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_13_06 = PyFloat_FromDouble(13.06); if (unlikely(!__pyx_float_13_06)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_13_08 = PyFloat_FromDouble(13.08); if (unlikely(!__pyx_float_13_08)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_13_11 = PyFloat_FromDouble(13.11); if (unlikely(!__pyx_float_13_11)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_13_13 = PyFloat_FromDouble(13.13); if (unlikely(!__pyx_float_13_13)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_13_14 = PyFloat_FromDouble(13.14); if (unlikely(!__pyx_float_13_14)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_13_15 = PyFloat_FromDouble(13.15); if (unlikely(!__pyx_float_13_15)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_13_16 = PyFloat_FromDouble(13.16); if (unlikely(!__pyx_float_13_16)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_13_21 = PyFloat_FromDouble(13.21); if (unlikely(!__pyx_float_13_21)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_13_24 = PyFloat_FromDouble(13.24); if (unlikely(!__pyx_float_13_24)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_13_25 = PyFloat_FromDouble(13.25); if (unlikely(!__pyx_float_13_25)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_13_26 = PyFloat_FromDouble(13.26); if (unlikely(!__pyx_float_13_26)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_13_29 = PyFloat_FromDouble(13.29); if (unlikely(!__pyx_float_13_29)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_13_34 = PyFloat_FromDouble(13.34); if (unlikely(!__pyx_float_13_34)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_13_38 = PyFloat_FromDouble(13.38); if (unlikely(!__pyx_float_13_38)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_13_40 = PyFloat_FromDouble(13.40); if (unlikely(!__pyx_float_13_40)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_13_42 = PyFloat_FromDouble(13.42); if (unlikely(!__pyx_float_13_42)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_13_49 = PyFloat_FromDouble(13.49); if (unlikely(!__pyx_float_13_49)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_13_53 = PyFloat_FromDouble(13.53); if (unlikely(!__pyx_float_13_53)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_13_58 = PyFloat_FromDouble(13.58); if (unlikely(!__pyx_float_13_58)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_13_66 = PyFloat_FromDouble(13.66); if (unlikely(!__pyx_float_13_66)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_13_75 = PyFloat_FromDouble(13.75); if (unlikely(!__pyx_float_13_75)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_13_79 = PyFloat_FromDouble(13.79); if (unlikely(!__pyx_float_13_79)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_13_89 = PyFloat_FromDouble(13.89); if (unlikely(!__pyx_float_13_89)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_13_91 = PyFloat_FromDouble(13.91); if (unlikely(!__pyx_float_13_91)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_13_97 = PyFloat_FromDouble(13.97); if (unlikely(!__pyx_float_13_97)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_13_99 = PyFloat_FromDouble(13.99); if (unlikely(!__pyx_float_13_99)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_14_00 = PyFloat_FromDouble(14.00); if (unlikely(!__pyx_float_14_00)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_14_02 = PyFloat_FromDouble(14.02); if (unlikely(!__pyx_float_14_02)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_14_03 = PyFloat_FromDouble(14.03); if (unlikely(!__pyx_float_14_03)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_14_04 = PyFloat_FromDouble(14.04); if (unlikely(!__pyx_float_14_04)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_14_05 = PyFloat_FromDouble(14.05); if (unlikely(!__pyx_float_14_05)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_14_06 = PyFloat_FromDouble(14.06); if (unlikely(!__pyx_float_14_06)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_14_07 = PyFloat_FromDouble(14.07); if (unlikely(!__pyx_float_14_07)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_14_08 = PyFloat_FromDouble(14.08); if (unlikely(!__pyx_float_14_08)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_14_10 = PyFloat_FromDouble(14.10); if (unlikely(!__pyx_float_14_10)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_14_12 = PyFloat_FromDouble(14.12); if (unlikely(!__pyx_float_14_12)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_14_13 = PyFloat_FromDouble(14.13); if (unlikely(!__pyx_float_14_13)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_14_19 = PyFloat_FromDouble(14.19); if (unlikely(!__pyx_float_14_19)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_14_25 = PyFloat_FromDouble(14.25); if (unlikely(!__pyx_float_14_25)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_14_32 = PyFloat_FromDouble(14.32); if (unlikely(!__pyx_float_14_32)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_14_33 = PyFloat_FromDouble(14.33); if (unlikely(!__pyx_float_14_33)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_14_39 = PyFloat_FromDouble(14.39); if (unlikely(!__pyx_float_14_39)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_14_45 = PyFloat_FromDouble(14.45); if (unlikely(!__pyx_float_14_45)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_14_50 = PyFloat_FromDouble(14.50); if (unlikely(!__pyx_float_14_50)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_14_53 = PyFloat_FromDouble(14.53); if (unlikely(!__pyx_float_14_53)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_14_54 = PyFloat_FromDouble(14.54); if (unlikely(!__pyx_float_14_54)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_14_62 = PyFloat_FromDouble(14.62); if (unlikely(!__pyx_float_14_62)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_14_66 = PyFloat_FromDouble(14.66); if (unlikely(!__pyx_float_14_66)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_14_74 = PyFloat_FromDouble(14.74); if (unlikely(!__pyx_float_14_74)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_14_84 = PyFloat_FromDouble(14.84); if (unlikely(!__pyx_float_14_84)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_14_93 = PyFloat_FromDouble(14.93); if (unlikely(!__pyx_float_14_93)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_14_95 = PyFloat_FromDouble(14.95); if (unlikely(!__pyx_float_14_95)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_15_00 = PyFloat_FromDouble(15.00); if (unlikely(!__pyx_float_15_00)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_15_01 = PyFloat_FromDouble(15.01); if (unlikely(!__pyx_float_15_01)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_15_03 = PyFloat_FromDouble(15.03); if (unlikely(!__pyx_float_15_03)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_15_04 = PyFloat_FromDouble(15.04); if (unlikely(!__pyx_float_15_04)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_15_05 = PyFloat_FromDouble(15.05); if (unlikely(!__pyx_float_15_05)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_15_06 = PyFloat_FromDouble(15.06); if (unlikely(!__pyx_float_15_06)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_15_07 = PyFloat_FromDouble(15.07); if (unlikely(!__pyx_float_15_07)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_15_08 = PyFloat_FromDouble(15.08); if (unlikely(!__pyx_float_15_08)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_15_09 = PyFloat_FromDouble(15.09); if (unlikely(!__pyx_float_15_09)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_15_11 = PyFloat_FromDouble(15.11); if (unlikely(!__pyx_float_15_11)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_15_12 = PyFloat_FromDouble(15.12); if (unlikely(!__pyx_float_15_12)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_15_15 = PyFloat_FromDouble(15.15); if (unlikely(!__pyx_float_15_15)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_15_16 = PyFloat_FromDouble(15.16); if (unlikely(!__pyx_float_15_16)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_15_21 = PyFloat_FromDouble(15.21); if (unlikely(!__pyx_float_15_21)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_15_22 = PyFloat_FromDouble(15.22); if (unlikely(!__pyx_float_15_22)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_15_24 = PyFloat_FromDouble(15.24); if (unlikely(!__pyx_float_15_24)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_15_27 = PyFloat_FromDouble(15.27); if (unlikely(!__pyx_float_15_27)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_15_31 = PyFloat_FromDouble(15.31); if (unlikely(!__pyx_float_15_31)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_15_41 = PyFloat_FromDouble(15.41); if (unlikely(!__pyx_float_15_41)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_15_42 = PyFloat_FromDouble(15.42); if (unlikely(!__pyx_float_15_42)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_15_48 = PyFloat_FromDouble(15.48); if (unlikely(!__pyx_float_15_48)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_15_53 = PyFloat_FromDouble(15.53); if (unlikely(!__pyx_float_15_53)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_15_62 = PyFloat_FromDouble(15.62); if (unlikely(!__pyx_float_15_62)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_15_65 = PyFloat_FromDouble(15.65); if (unlikely(!__pyx_float_15_65)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_15_77 = PyFloat_FromDouble(15.77); if (unlikely(!__pyx_float_15_77)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_15_81 = PyFloat_FromDouble(15.81); if (unlikely(!__pyx_float_15_81)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_15_88 = PyFloat_FromDouble(15.88); if (unlikely(!__pyx_float_15_88)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_15_91 = PyFloat_FromDouble(15.91); if (unlikely(!__pyx_float_15_91)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_15_93 = PyFloat_FromDouble(15.93); if (unlikely(!__pyx_float_15_93)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_15_99 = PyFloat_FromDouble(15.99); if (unlikely(!__pyx_float_15_99)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_16_00 = PyFloat_FromDouble(16.00); if (unlikely(!__pyx_float_16_00)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_16_01 = PyFloat_FromDouble(16.01); if (unlikely(!__pyx_float_16_01)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_16_03 = PyFloat_FromDouble(16.03); if (unlikely(!__pyx_float_16_03)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_16_04 = PyFloat_FromDouble(16.04); if (unlikely(!__pyx_float_16_04)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_16_05 = PyFloat_FromDouble(16.05); if (unlikely(!__pyx_float_16_05)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_16_06 = PyFloat_FromDouble(16.06); if (unlikely(!__pyx_float_16_06)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_16_07 = PyFloat_FromDouble(16.07); if (unlikely(!__pyx_float_16_07)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_16_08 = PyFloat_FromDouble(16.08); if (unlikely(!__pyx_float_16_08)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_16_09 = PyFloat_FromDouble(16.09); if (unlikely(!__pyx_float_16_09)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_16_11 = PyFloat_FromDouble(16.11); if (unlikely(!__pyx_float_16_11)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_16_12 = PyFloat_FromDouble(16.12); if (unlikely(!__pyx_float_16_12)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_16_13 = PyFloat_FromDouble(16.13); if (unlikely(!__pyx_float_16_13)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_16_14 = PyFloat_FromDouble(16.14); if (unlikely(!__pyx_float_16_14)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_16_15 = PyFloat_FromDouble(16.15); if (unlikely(!__pyx_float_16_15)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_16_16 = PyFloat_FromDouble(16.16); if (unlikely(!__pyx_float_16_16)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_16_18 = PyFloat_FromDouble(16.18); if (unlikely(!__pyx_float_16_18)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_16_20 = PyFloat_FromDouble(16.20); if (unlikely(!__pyx_float_16_20)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_16_21 = PyFloat_FromDouble(16.21); if (unlikely(!__pyx_float_16_21)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_16_24 = PyFloat_FromDouble(16.24); if (unlikely(!__pyx_float_16_24)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_16_29 = PyFloat_FromDouble(16.29); if (unlikely(!__pyx_float_16_29)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_16_30 = PyFloat_FromDouble(16.30); if (unlikely(!__pyx_float_16_30)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_16_34 = PyFloat_FromDouble(16.34); if (unlikely(!__pyx_float_16_34)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_16_40 = PyFloat_FromDouble(16.40); if (unlikely(!__pyx_float_16_40)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_16_49 = PyFloat_FromDouble(16.49); if (unlikely(!__pyx_float_16_49)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_16_50 = PyFloat_FromDouble(16.50); if (unlikely(!__pyx_float_16_50)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_16_59 = PyFloat_FromDouble(16.59); if (unlikely(!__pyx_float_16_59)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_16_66 = PyFloat_FromDouble(16.66); if (unlikely(!__pyx_float_16_66)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_16_80 = PyFloat_FromDouble(16.80); if (unlikely(!__pyx_float_16_80)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_17_01 = PyFloat_FromDouble(17.01); if (unlikely(!__pyx_float_17_01)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_17_03 = PyFloat_FromDouble(17.03); if (unlikely(!__pyx_float_17_03)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_17_04 = PyFloat_FromDouble(17.04); if (unlikely(!__pyx_float_17_04)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_17_06 = PyFloat_FromDouble(17.06); if (unlikely(!__pyx_float_17_06)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_17_07 = PyFloat_FromDouble(17.07); if (unlikely(!__pyx_float_17_07)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_17_08 = PyFloat_FromDouble(17.08); if (unlikely(!__pyx_float_17_08)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_17_09 = PyFloat_FromDouble(17.09); if (unlikely(!__pyx_float_17_09)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_17_10 = PyFloat_FromDouble(17.10); if (unlikely(!__pyx_float_17_10)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_17_11 = PyFloat_FromDouble(17.11); if (unlikely(!__pyx_float_17_11)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_17_12 = PyFloat_FromDouble(17.12); if (unlikely(!__pyx_float_17_12)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_17_14 = PyFloat_FromDouble(17.14); if (unlikely(!__pyx_float_17_14)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_17_15 = PyFloat_FromDouble(17.15); if (unlikely(!__pyx_float_17_15)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_17_17 = PyFloat_FromDouble(17.17); if (unlikely(!__pyx_float_17_17)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_17_21 = PyFloat_FromDouble(17.21); if (unlikely(!__pyx_float_17_21)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_17_22 = PyFloat_FromDouble(17.22); if (unlikely(!__pyx_float_17_22)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_17_24 = PyFloat_FromDouble(17.24); if (unlikely(!__pyx_float_17_24)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_17_25 = PyFloat_FromDouble(17.25); if (unlikely(!__pyx_float_17_25)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_17_30 = PyFloat_FromDouble(17.30); if (unlikely(!__pyx_float_17_30)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_17_36 = PyFloat_FromDouble(17.36); if (unlikely(!__pyx_float_17_36)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_17_38 = PyFloat_FromDouble(17.38); if (unlikely(!__pyx_float_17_38)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_17_42 = PyFloat_FromDouble(17.42); if (unlikely(!__pyx_float_17_42)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_17_48 = PyFloat_FromDouble(17.48); if (unlikely(!__pyx_float_17_48)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_17_59 = PyFloat_FromDouble(17.59); if (unlikely(!__pyx_float_17_59)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_17_61 = PyFloat_FromDouble(17.61); if (unlikely(!__pyx_float_17_61)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_17_62 = PyFloat_FromDouble(17.62); if (unlikely(!__pyx_float_17_62)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_17_70 = PyFloat_FromDouble(17.70); if (unlikely(!__pyx_float_17_70)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_17_79 = PyFloat_FromDouble(17.79); if (unlikely(!__pyx_float_17_79)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_17_95 = PyFloat_FromDouble(17.95); if (unlikely(!__pyx_float_17_95)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_17_97 = PyFloat_FromDouble(17.97); if (unlikely(!__pyx_float_17_97)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_18_02 = PyFloat_FromDouble(18.02); if (unlikely(!__pyx_float_18_02)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_18_03 = PyFloat_FromDouble(18.03); if (unlikely(!__pyx_float_18_03)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_18_08 = PyFloat_FromDouble(18.08); if (unlikely(!__pyx_float_18_08)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_18_09 = PyFloat_FromDouble(18.09); if (unlikely(!__pyx_float_18_09)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_18_10 = PyFloat_FromDouble(18.10); if (unlikely(!__pyx_float_18_10)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_18_11 = PyFloat_FromDouble(18.11); if (unlikely(!__pyx_float_18_11)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_18_12 = PyFloat_FromDouble(18.12); if (unlikely(!__pyx_float_18_12)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_18_13 = PyFloat_FromDouble(18.13); if (unlikely(!__pyx_float_18_13)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_18_14 = PyFloat_FromDouble(18.14); if (unlikely(!__pyx_float_18_14)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_18_15 = PyFloat_FromDouble(18.15); if (unlikely(!__pyx_float_18_15)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_18_16 = PyFloat_FromDouble(18.16); if (unlikely(!__pyx_float_18_16)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_18_18 = PyFloat_FromDouble(18.18); if (unlikely(!__pyx_float_18_18)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_18_19 = PyFloat_FromDouble(18.19); if (unlikely(!__pyx_float_18_19)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_18_20 = PyFloat_FromDouble(18.20); if (unlikely(!__pyx_float_18_20)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_18_21 = PyFloat_FromDouble(18.21); if (unlikely(!__pyx_float_18_21)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_18_23 = PyFloat_FromDouble(18.23); if (unlikely(!__pyx_float_18_23)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_18_25 = PyFloat_FromDouble(18.25); if (unlikely(!__pyx_float_18_25)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_18_27 = PyFloat_FromDouble(18.27); if (unlikely(!__pyx_float_18_27)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_18_30 = PyFloat_FromDouble(18.30); if (unlikely(!__pyx_float_18_30)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_18_35 = PyFloat_FromDouble(18.35); if (unlikely(!__pyx_float_18_35)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_18_43 = PyFloat_FromDouble(18.43); if (unlikely(!__pyx_float_18_43)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_18_45 = PyFloat_FromDouble(18.45); if (unlikely(!__pyx_float_18_45)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_18_51 = PyFloat_FromDouble(18.51); if (unlikely(!__pyx_float_18_51)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_18_59 = PyFloat_FromDouble(18.59); if (unlikely(!__pyx_float_18_59)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_18_71 = PyFloat_FromDouble(18.71); if (unlikely(!__pyx_float_18_71)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_18_75 = PyFloat_FromDouble(18.75); if (unlikely(!__pyx_float_18_75)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_18_84 = PyFloat_FromDouble(18.84); if (unlikely(!__pyx_float_18_84)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_18_94 = PyFloat_FromDouble(18.94); if (unlikely(!__pyx_float_18_94)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_18_95 = PyFloat_FromDouble(18.95); if (unlikely(!__pyx_float_18_95)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_19_04 = PyFloat_FromDouble(19.04); if (unlikely(!__pyx_float_19_04)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_19_05 = PyFloat_FromDouble(19.05); if (unlikely(!__pyx_float_19_05)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_19_06 = PyFloat_FromDouble(19.06); if (unlikely(!__pyx_float_19_06)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_19_11 = PyFloat_FromDouble(19.11); if (unlikely(!__pyx_float_19_11)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_19_12 = PyFloat_FromDouble(19.12); if (unlikely(!__pyx_float_19_12)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_19_14 = PyFloat_FromDouble(19.14); if (unlikely(!__pyx_float_19_14)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_19_15 = PyFloat_FromDouble(19.15); if (unlikely(!__pyx_float_19_15)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_19_16 = PyFloat_FromDouble(19.16); if (unlikely(!__pyx_float_19_16)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_19_17 = PyFloat_FromDouble(19.17); if (unlikely(!__pyx_float_19_17)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_19_18 = PyFloat_FromDouble(19.18); if (unlikely(!__pyx_float_19_18)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_19_21 = PyFloat_FromDouble(19.21); if (unlikely(!__pyx_float_19_21)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_19_22 = PyFloat_FromDouble(19.22); if (unlikely(!__pyx_float_19_22)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_19_23 = PyFloat_FromDouble(19.23); if (unlikely(!__pyx_float_19_23)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_19_24 = PyFloat_FromDouble(19.24); if (unlikely(!__pyx_float_19_24)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_19_26 = PyFloat_FromDouble(19.26); if (unlikely(!__pyx_float_19_26)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_19_29 = PyFloat_FromDouble(19.29); if (unlikely(!__pyx_float_19_29)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_19_34 = PyFloat_FromDouble(19.34); if (unlikely(!__pyx_float_19_34)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_19_36 = PyFloat_FromDouble(19.36); if (unlikely(!__pyx_float_19_36)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_19_39 = PyFloat_FromDouble(19.39); if (unlikely(!__pyx_float_19_39)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_19_41 = PyFloat_FromDouble(19.41); if (unlikely(!__pyx_float_19_41)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_19_51 = PyFloat_FromDouble(19.51); if (unlikely(!__pyx_float_19_51)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_19_56 = PyFloat_FromDouble(19.56); if (unlikely(!__pyx_float_19_56)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_19_57 = PyFloat_FromDouble(19.57); if (unlikely(!__pyx_float_19_57)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_19_61 = PyFloat_FromDouble(19.61); if (unlikely(!__pyx_float_19_61)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_19_71 = PyFloat_FromDouble(19.71); if (unlikely(!__pyx_float_19_71)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_19_83 = PyFloat_FromDouble(19.83); if (unlikely(!__pyx_float_19_83)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_19_84 = PyFloat_FromDouble(19.84); if (unlikely(!__pyx_float_19_84)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_19_89 = PyFloat_FromDouble(19.89); if (unlikely(!__pyx_float_19_89)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_20_00 = PyFloat_FromDouble(20.00); if (unlikely(!__pyx_float_20_00)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_20_08 = PyFloat_FromDouble(20.08); if (unlikely(!__pyx_float_20_08)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_20_11 = PyFloat_FromDouble(20.11); if (unlikely(!__pyx_float_20_11)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_20_12 = PyFloat_FromDouble(20.12); if (unlikely(!__pyx_float_20_12)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_20_13 = PyFloat_FromDouble(20.13); if (unlikely(!__pyx_float_20_13)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_20_16 = PyFloat_FromDouble(20.16); if (unlikely(!__pyx_float_20_16)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_20_17 = PyFloat_FromDouble(20.17); if (unlikely(!__pyx_float_20_17)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_20_18 = PyFloat_FromDouble(20.18); if (unlikely(!__pyx_float_20_18)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_20_19 = PyFloat_FromDouble(20.19); if (unlikely(!__pyx_float_20_19)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_20_21 = PyFloat_FromDouble(20.21); if (unlikely(!__pyx_float_20_21)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_20_23 = PyFloat_FromDouble(20.23); if (unlikely(!__pyx_float_20_23)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_20_24 = PyFloat_FromDouble(20.24); if (unlikely(!__pyx_float_20_24)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_20_25 = PyFloat_FromDouble(20.25); if (unlikely(!__pyx_float_20_25)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_20_26 = PyFloat_FromDouble(20.26); if (unlikely(!__pyx_float_20_26)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_20_27 = PyFloat_FromDouble(20.27); if (unlikely(!__pyx_float_20_27)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_20_28 = PyFloat_FromDouble(20.28); if (unlikely(!__pyx_float_20_28)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_20_29 = PyFloat_FromDouble(20.29); if (unlikely(!__pyx_float_20_29)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_20_30 = PyFloat_FromDouble(20.30); if (unlikely(!__pyx_float_20_30)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_20_34 = PyFloat_FromDouble(20.34); if (unlikely(!__pyx_float_20_34)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_20_36 = PyFloat_FromDouble(20.36); if (unlikely(!__pyx_float_20_36)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_20_40 = PyFloat_FromDouble(20.40); if (unlikely(!__pyx_float_20_40)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_20_43 = PyFloat_FromDouble(20.43); if (unlikely(!__pyx_float_20_43)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_20_44 = PyFloat_FromDouble(20.44); if (unlikely(!__pyx_float_20_44)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_20_50 = PyFloat_FromDouble(20.50); if (unlikely(!__pyx_float_20_50)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_20_60 = PyFloat_FromDouble(20.60); if (unlikely(!__pyx_float_20_60)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_20_65 = PyFloat_FromDouble(20.65); if (unlikely(!__pyx_float_20_65)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_20_72 = PyFloat_FromDouble(20.72); if (unlikely(!__pyx_float_20_72)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_20_84 = PyFloat_FromDouble(20.84); if (unlikely(!__pyx_float_20_84)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_20_98 = PyFloat_FromDouble(20.98); if (unlikely(!__pyx_float_20_98)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_20_99 = PyFloat_FromDouble(20.99); if (unlikely(!__pyx_float_20_99)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_21_04 = PyFloat_FromDouble(21.04); if (unlikely(!__pyx_float_21_04)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_21_11 = PyFloat_FromDouble(21.11); if (unlikely(!__pyx_float_21_11)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_21_13 = PyFloat_FromDouble(21.13); if (unlikely(!__pyx_float_21_13)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_21_17 = PyFloat_FromDouble(21.17); if (unlikely(!__pyx_float_21_17)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_21_18 = PyFloat_FromDouble(21.18); if (unlikely(!__pyx_float_21_18)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_21_20 = PyFloat_FromDouble(21.20); if (unlikely(!__pyx_float_21_20)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_21_21 = PyFloat_FromDouble(21.21); if (unlikely(!__pyx_float_21_21)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_21_22 = PyFloat_FromDouble(21.22); if (unlikely(!__pyx_float_21_22)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_21_23 = PyFloat_FromDouble(21.23); if (unlikely(!__pyx_float_21_23)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_21_24 = PyFloat_FromDouble(21.24); if (unlikely(!__pyx_float_21_24)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_21_25 = PyFloat_FromDouble(21.25); if (unlikely(!__pyx_float_21_25)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_21_27 = PyFloat_FromDouble(21.27); if (unlikely(!__pyx_float_21_27)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_21_29 = PyFloat_FromDouble(21.29); if (unlikely(!__pyx_float_21_29)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_21_30 = PyFloat_FromDouble(21.30); if (unlikely(!__pyx_float_21_30)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_21_31 = PyFloat_FromDouble(21.31); if (unlikely(!__pyx_float_21_31)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_21_32 = PyFloat_FromDouble(21.32); if (unlikely(!__pyx_float_21_32)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_21_33 = PyFloat_FromDouble(21.33); if (unlikely(!__pyx_float_21_33)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_21_41 = PyFloat_FromDouble(21.41); if (unlikely(!__pyx_float_21_41)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_21_47 = PyFloat_FromDouble(21.47); if (unlikely(!__pyx_float_21_47)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_21_50 = PyFloat_FromDouble(21.50); if (unlikely(!__pyx_float_21_50)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_21_51 = PyFloat_FromDouble(21.51); if (unlikely(!__pyx_float_21_51)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_21_56 = PyFloat_FromDouble(21.56); if (unlikely(!__pyx_float_21_56)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_21_57 = PyFloat_FromDouble(21.57); if (unlikely(!__pyx_float_21_57)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_21_70 = PyFloat_FromDouble(21.70); if (unlikely(!__pyx_float_21_70)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_21_74 = PyFloat_FromDouble(21.74); if (unlikely(!__pyx_float_21_74)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_21_84 = PyFloat_FromDouble(21.84); if (unlikely(!__pyx_float_21_84)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_21_98 = PyFloat_FromDouble(21.98); if (unlikely(!__pyx_float_21_98)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_22_12 = PyFloat_FromDouble(22.12); if (unlikely(!__pyx_float_22_12)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_22_13 = PyFloat_FromDouble(22.13); if (unlikely(!__pyx_float_22_13)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_22_14 = PyFloat_FromDouble(22.14); if (unlikely(!__pyx_float_22_14)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_22_16 = PyFloat_FromDouble(22.16); if (unlikely(!__pyx_float_22_16)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_22_20 = PyFloat_FromDouble(22.20); if (unlikely(!__pyx_float_22_20)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_22_21 = PyFloat_FromDouble(22.21); if (unlikely(!__pyx_float_22_21)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_22_22 = PyFloat_FromDouble(22.22); if (unlikely(!__pyx_float_22_22)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_22_25 = PyFloat_FromDouble(22.25); if (unlikely(!__pyx_float_22_25)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_22_26 = PyFloat_FromDouble(22.26); if (unlikely(!__pyx_float_22_26)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_22_27 = PyFloat_FromDouble(22.27); if (unlikely(!__pyx_float_22_27)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_22_28 = PyFloat_FromDouble(22.28); if (unlikely(!__pyx_float_22_28)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_22_29 = PyFloat_FromDouble(22.29); if (unlikely(!__pyx_float_22_29)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_22_30 = PyFloat_FromDouble(22.30); if (unlikely(!__pyx_float_22_30)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_22_32 = PyFloat_FromDouble(22.32); if (unlikely(!__pyx_float_22_32)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_22_35 = PyFloat_FromDouble(22.35); if (unlikely(!__pyx_float_22_35)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_22_36 = PyFloat_FromDouble(22.36); if (unlikely(!__pyx_float_22_36)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_22_37 = PyFloat_FromDouble(22.37); if (unlikely(!__pyx_float_22_37)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_22_38 = PyFloat_FromDouble(22.38); if (unlikely(!__pyx_float_22_38)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_22_48 = PyFloat_FromDouble(22.48); if (unlikely(!__pyx_float_22_48)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_22_49 = PyFloat_FromDouble(22.49); if (unlikely(!__pyx_float_22_49)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_22_54 = PyFloat_FromDouble(22.54); if (unlikely(!__pyx_float_22_54)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_22_59 = PyFloat_FromDouble(22.59); if (unlikely(!__pyx_float_22_59)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_22_60 = PyFloat_FromDouble(22.60); if (unlikely(!__pyx_float_22_60)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_22_68 = PyFloat_FromDouble(22.68); if (unlikely(!__pyx_float_22_68)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_22_82 = PyFloat_FromDouble(22.82); if (unlikely(!__pyx_float_22_82)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_22_87 = PyFloat_FromDouble(22.87); if (unlikely(!__pyx_float_22_87)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_22_99 = PyFloat_FromDouble(22.99); if (unlikely(!__pyx_float_22_99)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_23_11 = PyFloat_FromDouble(23.11); if (unlikely(!__pyx_float_23_11)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_23_19 = PyFloat_FromDouble(23.19); if (unlikely(!__pyx_float_23_19)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_23_20 = PyFloat_FromDouble(23.20); if (unlikely(!__pyx_float_23_20)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_23_22 = PyFloat_FromDouble(23.22); if (unlikely(!__pyx_float_23_22)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_23_24 = PyFloat_FromDouble(23.24); if (unlikely(!__pyx_float_23_24)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_23_26 = PyFloat_FromDouble(23.26); if (unlikely(!__pyx_float_23_26)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_23_28 = PyFloat_FromDouble(23.28); if (unlikely(!__pyx_float_23_28)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_23_29 = PyFloat_FromDouble(23.29); if (unlikely(!__pyx_float_23_29)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_23_30 = PyFloat_FromDouble(23.30); if (unlikely(!__pyx_float_23_30)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_23_31 = PyFloat_FromDouble(23.31); if (unlikely(!__pyx_float_23_31)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_23_32 = PyFloat_FromDouble(23.32); if (unlikely(!__pyx_float_23_32)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_23_33 = PyFloat_FromDouble(23.33); if (unlikely(!__pyx_float_23_33)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_23_34 = PyFloat_FromDouble(23.34); if (unlikely(!__pyx_float_23_34)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_23_35 = PyFloat_FromDouble(23.35); if (unlikely(!__pyx_float_23_35)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_23_38 = PyFloat_FromDouble(23.38); if (unlikely(!__pyx_float_23_38)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_23_39 = PyFloat_FromDouble(23.39); if (unlikely(!__pyx_float_23_39)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_23_41 = PyFloat_FromDouble(23.41); if (unlikely(!__pyx_float_23_41)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_23_42 = PyFloat_FromDouble(23.42); if (unlikely(!__pyx_float_23_42)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_23_46 = PyFloat_FromDouble(23.46); if (unlikely(!__pyx_float_23_46)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_23_55 = PyFloat_FromDouble(23.55); if (unlikely(!__pyx_float_23_55)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_23_56 = PyFloat_FromDouble(23.56); if (unlikely(!__pyx_float_23_56)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_23_62 = PyFloat_FromDouble(23.62); if (unlikely(!__pyx_float_23_62)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_23_69 = PyFloat_FromDouble(23.69); if (unlikely(!__pyx_float_23_69)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_23_72 = PyFloat_FromDouble(23.72); if (unlikely(!__pyx_float_23_72)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_23_73 = PyFloat_FromDouble(23.73); if (unlikely(!__pyx_float_23_73)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_23_80 = PyFloat_FromDouble(23.80); if (unlikely(!__pyx_float_23_80)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_23_90 = PyFloat_FromDouble(23.90); if (unlikely(!__pyx_float_23_90)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_23_95 = PyFloat_FromDouble(23.95); if (unlikely(!__pyx_float_23_95)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_23_99 = PyFloat_FromDouble(23.99); if (unlikely(!__pyx_float_23_99)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_24_00 = PyFloat_FromDouble(24.00); if (unlikely(!__pyx_float_24_00)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_24_05 = PyFloat_FromDouble(24.05); if (unlikely(!__pyx_float_24_05)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_24_13 = PyFloat_FromDouble(24.13); if (unlikely(!__pyx_float_24_13)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_24_14 = PyFloat_FromDouble(24.14); if (unlikely(!__pyx_float_24_14)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_24_19 = PyFloat_FromDouble(24.19); if (unlikely(!__pyx_float_24_19)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_24_26 = PyFloat_FromDouble(24.26); if (unlikely(!__pyx_float_24_26)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_24_29 = PyFloat_FromDouble(24.29); if (unlikely(!__pyx_float_24_29)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_24_31 = PyFloat_FromDouble(24.31); if (unlikely(!__pyx_float_24_31)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_24_32 = PyFloat_FromDouble(24.32); if (unlikely(!__pyx_float_24_32)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_24_33 = PyFloat_FromDouble(24.33); if (unlikely(!__pyx_float_24_33)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_24_34 = PyFloat_FromDouble(24.34); if (unlikely(!__pyx_float_24_34)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_24_35 = PyFloat_FromDouble(24.35); if (unlikely(!__pyx_float_24_35)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_24_36 = PyFloat_FromDouble(24.36); if (unlikely(!__pyx_float_24_36)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_24_37 = PyFloat_FromDouble(24.37); if (unlikely(!__pyx_float_24_37)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_24_38 = PyFloat_FromDouble(24.38); if (unlikely(!__pyx_float_24_38)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_24_40 = PyFloat_FromDouble(24.40); if (unlikely(!__pyx_float_24_40)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_24_48 = PyFloat_FromDouble(24.48); if (unlikely(!__pyx_float_24_48)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_24_50 = PyFloat_FromDouble(24.50); if (unlikely(!__pyx_float_24_50)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_24_52 = PyFloat_FromDouble(24.52); if (unlikely(!__pyx_float_24_52)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_24_64 = PyFloat_FromDouble(24.64); if (unlikely(!__pyx_float_24_64)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_24_71 = PyFloat_FromDouble(24.71); if (unlikely(!__pyx_float_24_71)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_24_80 = PyFloat_FromDouble(24.80); if (unlikely(!__pyx_float_24_80)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_24_90 = PyFloat_FromDouble(24.90); if (unlikely(!__pyx_float_24_90)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_24_97 = PyFloat_FromDouble(24.97); if (unlikely(!__pyx_float_24_97)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_25_05 = PyFloat_FromDouble(25.05); if (unlikely(!__pyx_float_25_05)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_25_15 = PyFloat_FromDouble(25.15); if (unlikely(!__pyx_float_25_15)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_25_20 = PyFloat_FromDouble(25.20); if (unlikely(!__pyx_float_25_20)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_25_24 = PyFloat_FromDouble(25.24); if (unlikely(!__pyx_float_25_24)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_25_30 = PyFloat_FromDouble(25.30); if (unlikely(!__pyx_float_25_30)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_25_31 = PyFloat_FromDouble(25.31); if (unlikely(!__pyx_float_25_31)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_25_32 = PyFloat_FromDouble(25.32); if (unlikely(!__pyx_float_25_32)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_25_33 = PyFloat_FromDouble(25.33); if (unlikely(!__pyx_float_25_33)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_25_35 = PyFloat_FromDouble(25.35); if (unlikely(!__pyx_float_25_35)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_25_38 = PyFloat_FromDouble(25.38); if (unlikely(!__pyx_float_25_38)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_25_41 = PyFloat_FromDouble(25.41); if (unlikely(!__pyx_float_25_41)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_25_42 = PyFloat_FromDouble(25.42); if (unlikely(!__pyx_float_25_42)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_25_43 = PyFloat_FromDouble(25.43); if (unlikely(!__pyx_float_25_43)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_25_45 = PyFloat_FromDouble(25.45); if (unlikely(!__pyx_float_25_45)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_25_49 = PyFloat_FromDouble(25.49); if (unlikely(!__pyx_float_25_49)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_25_54 = PyFloat_FromDouble(25.54); if (unlikely(!__pyx_float_25_54)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_25_55 = PyFloat_FromDouble(25.55); if (unlikely(!__pyx_float_25_55)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_25_56 = PyFloat_FromDouble(25.56); if (unlikely(!__pyx_float_25_56)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_25_58 = PyFloat_FromDouble(25.58); if (unlikely(!__pyx_float_25_58)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_25_59 = PyFloat_FromDouble(25.59); if (unlikely(!__pyx_float_25_59)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_25_70 = PyFloat_FromDouble(25.70); if (unlikely(!__pyx_float_25_70)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_25_74 = PyFloat_FromDouble(25.74); if (unlikely(!__pyx_float_25_74)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_25_82 = PyFloat_FromDouble(25.82); if (unlikely(!__pyx_float_25_82)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_25_87 = PyFloat_FromDouble(25.87); if (unlikely(!__pyx_float_25_87)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_25_88 = PyFloat_FromDouble(25.88); if (unlikely(!__pyx_float_25_88)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_26_02 = PyFloat_FromDouble(26.02); if (unlikely(!__pyx_float_26_02)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_26_17 = PyFloat_FromDouble(26.17); if (unlikely(!__pyx_float_26_17)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_26_22 = PyFloat_FromDouble(26.22); if (unlikely(!__pyx_float_26_22)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_26_24 = PyFloat_FromDouble(26.24); if (unlikely(!__pyx_float_26_24)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_26_36 = PyFloat_FromDouble(26.36); if (unlikely(!__pyx_float_26_36)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_26_38 = PyFloat_FromDouble(26.38); if (unlikely(!__pyx_float_26_38)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_26_39 = PyFloat_FromDouble(26.39); if (unlikely(!__pyx_float_26_39)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_26_40 = PyFloat_FromDouble(26.40); if (unlikely(!__pyx_float_26_40)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_26_41 = PyFloat_FromDouble(26.41); if (unlikely(!__pyx_float_26_41)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_26_43 = PyFloat_FromDouble(26.43); if (unlikely(!__pyx_float_26_43)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_26_46 = PyFloat_FromDouble(26.46); if (unlikely(!__pyx_float_26_46)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_26_47 = PyFloat_FromDouble(26.47); if (unlikely(!__pyx_float_26_47)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_26_49 = PyFloat_FromDouble(26.49); if (unlikely(!__pyx_float_26_49)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_26_50 = PyFloat_FromDouble(26.50); if (unlikely(!__pyx_float_26_50)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_26_54 = PyFloat_FromDouble(26.54); if (unlikely(!__pyx_float_26_54)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_26_55 = PyFloat_FromDouble(26.55); if (unlikely(!__pyx_float_26_55)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_26_61 = PyFloat_FromDouble(26.61); if (unlikely(!__pyx_float_26_61)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_26_62 = PyFloat_FromDouble(26.62); if (unlikely(!__pyx_float_26_62)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_26_64 = PyFloat_FromDouble(26.64); if (unlikely(!__pyx_float_26_64)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_26_67 = PyFloat_FromDouble(26.67); if (unlikely(!__pyx_float_26_67)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_26_68 = PyFloat_FromDouble(26.68); if (unlikely(!__pyx_float_26_68)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_26_69 = PyFloat_FromDouble(26.69); if (unlikely(!__pyx_float_26_69)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_26_70 = PyFloat_FromDouble(26.70); if (unlikely(!__pyx_float_26_70)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_26_81 = PyFloat_FromDouble(26.81); if (unlikely(!__pyx_float_26_81)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_26_91 = PyFloat_FromDouble(26.91); if (unlikely(!__pyx_float_26_91)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_27_02 = PyFloat_FromDouble(27.02); if (unlikely(!__pyx_float_27_02)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_27_03 = PyFloat_FromDouble(27.03); if (unlikely(!__pyx_float_27_03)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_27_04 = PyFloat_FromDouble(27.04); if (unlikely(!__pyx_float_27_04)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_27_16 = PyFloat_FromDouble(27.16); if (unlikely(!__pyx_float_27_16)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_27_36 = PyFloat_FromDouble(27.36); if (unlikely(!__pyx_float_27_36)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_27_41 = PyFloat_FromDouble(27.41); if (unlikely(!__pyx_float_27_41)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_27_43 = PyFloat_FromDouble(27.43); if (unlikely(!__pyx_float_27_43)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_27_44 = PyFloat_FromDouble(27.44); if (unlikely(!__pyx_float_27_44)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_27_46 = PyFloat_FromDouble(27.46); if (unlikely(!__pyx_float_27_46)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_27_47 = PyFloat_FromDouble(27.47); if (unlikely(!__pyx_float_27_47)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_27_49 = PyFloat_FromDouble(27.49); if (unlikely(!__pyx_float_27_49)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_27_50 = PyFloat_FromDouble(27.50); if (unlikely(!__pyx_float_27_50)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_27_52 = PyFloat_FromDouble(27.52); if (unlikely(!__pyx_float_27_52)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_27_53 = PyFloat_FromDouble(27.53); if (unlikely(!__pyx_float_27_53)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_27_54 = PyFloat_FromDouble(27.54); if (unlikely(!__pyx_float_27_54)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_27_55 = PyFloat_FromDouble(27.55); if (unlikely(!__pyx_float_27_55)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_27_59 = PyFloat_FromDouble(27.59); if (unlikely(!__pyx_float_27_59)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_27_60 = PyFloat_FromDouble(27.60); if (unlikely(!__pyx_float_27_60)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_27_61 = PyFloat_FromDouble(27.61); if (unlikely(!__pyx_float_27_61)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_27_70 = PyFloat_FromDouble(27.70); if (unlikely(!__pyx_float_27_70)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_27_72 = PyFloat_FromDouble(27.72); if (unlikely(!__pyx_float_27_72)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_27_76 = PyFloat_FromDouble(27.76); if (unlikely(!__pyx_float_27_76)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_27_77 = PyFloat_FromDouble(27.77); if (unlikely(!__pyx_float_27_77)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_27_78 = PyFloat_FromDouble(27.78); if (unlikely(!__pyx_float_27_78)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_27_95 = PyFloat_FromDouble(27.95); if (unlikely(!__pyx_float_27_95)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_27_97 = PyFloat_FromDouble(27.97); if (unlikely(!__pyx_float_27_97)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_28_03 = PyFloat_FromDouble(28.03); if (unlikely(!__pyx_float_28_03)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_28_09 = PyFloat_FromDouble(28.09); if (unlikely(!__pyx_float_28_09)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_28_12 = PyFloat_FromDouble(28.12); if (unlikely(!__pyx_float_28_12)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_28_16 = PyFloat_FromDouble(28.16); if (unlikely(!__pyx_float_28_16)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_28_30 = PyFloat_FromDouble(28.30); if (unlikely(!__pyx_float_28_30)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_28_39 = PyFloat_FromDouble(28.39); if (unlikely(!__pyx_float_28_39)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_28_44 = PyFloat_FromDouble(28.44); if (unlikely(!__pyx_float_28_44)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_28_45 = PyFloat_FromDouble(28.45); if (unlikely(!__pyx_float_28_45)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_28_47 = PyFloat_FromDouble(28.47); if (unlikely(!__pyx_float_28_47)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_28_50 = PyFloat_FromDouble(28.50); if (unlikely(!__pyx_float_28_50)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_28_51 = PyFloat_FromDouble(28.51); if (unlikely(!__pyx_float_28_51)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_28_52 = PyFloat_FromDouble(28.52); if (unlikely(!__pyx_float_28_52)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_28_53 = PyFloat_FromDouble(28.53); if (unlikely(!__pyx_float_28_53)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_28_55 = PyFloat_FromDouble(28.55); if (unlikely(!__pyx_float_28_55)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_28_59 = PyFloat_FromDouble(28.59); if (unlikely(!__pyx_float_28_59)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_28_60 = PyFloat_FromDouble(28.60); if (unlikely(!__pyx_float_28_60)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_28_62 = PyFloat_FromDouble(28.62); if (unlikely(!__pyx_float_28_62)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_28_63 = PyFloat_FromDouble(28.63); if (unlikely(!__pyx_float_28_63)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_28_64 = PyFloat_FromDouble(28.64); if (unlikely(!__pyx_float_28_64)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_28_65 = PyFloat_FromDouble(28.65); if (unlikely(!__pyx_float_28_65)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_28_66 = PyFloat_FromDouble(28.66); if (unlikely(!__pyx_float_28_66)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_28_70 = PyFloat_FromDouble(28.70); if (unlikely(!__pyx_float_28_70)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_28_73 = PyFloat_FromDouble(28.73); if (unlikely(!__pyx_float_28_73)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_28_78 = PyFloat_FromDouble(28.78); if (unlikely(!__pyx_float_28_78)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_28_79 = PyFloat_FromDouble(28.79); if (unlikely(!__pyx_float_28_79)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_28_80 = PyFloat_FromDouble(28.80); if (unlikely(!__pyx_float_28_80)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_28_82 = PyFloat_FromDouble(28.82); if (unlikely(!__pyx_float_28_82)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_28_87 = PyFloat_FromDouble(28.87); if (unlikely(!__pyx_float_28_87)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_28_88 = PyFloat_FromDouble(28.88); if (unlikely(!__pyx_float_28_88)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_29_03 = PyFloat_FromDouble(29.03); if (unlikely(!__pyx_float_29_03)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_29_12 = PyFloat_FromDouble(29.12); if (unlikely(!__pyx_float_29_12)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_29_17 = PyFloat_FromDouble(29.17); if (unlikely(!__pyx_float_29_17)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_29_28 = PyFloat_FromDouble(29.28); if (unlikely(!__pyx_float_29_28)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_29_31 = PyFloat_FromDouble(29.31); if (unlikely(!__pyx_float_29_31)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_29_40 = PyFloat_FromDouble(29.40); if (unlikely(!__pyx_float_29_40)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_29_48 = PyFloat_FromDouble(29.48); if (unlikely(!__pyx_float_29_48)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_29_54 = PyFloat_FromDouble(29.54); if (unlikely(!__pyx_float_29_54)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_29_57 = PyFloat_FromDouble(29.57); if (unlikely(!__pyx_float_29_57)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_29_58 = PyFloat_FromDouble(29.58); if (unlikely(!__pyx_float_29_58)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_29_60 = PyFloat_FromDouble(29.60); if (unlikely(!__pyx_float_29_60)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_29_61 = PyFloat_FromDouble(29.61); if (unlikely(!__pyx_float_29_61)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_29_64 = PyFloat_FromDouble(29.64); if (unlikely(!__pyx_float_29_64)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_29_65 = PyFloat_FromDouble(29.65); if (unlikely(!__pyx_float_29_65)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_29_66 = PyFloat_FromDouble(29.66); if (unlikely(!__pyx_float_29_66)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_29_68 = PyFloat_FromDouble(29.68); if (unlikely(!__pyx_float_29_68)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_29_71 = PyFloat_FromDouble(29.71); if (unlikely(!__pyx_float_29_71)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_29_72 = PyFloat_FromDouble(29.72); if (unlikely(!__pyx_float_29_72)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_29_76 = PyFloat_FromDouble(29.76); if (unlikely(!__pyx_float_29_76)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_29_80 = PyFloat_FromDouble(29.80); if (unlikely(!__pyx_float_29_80)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_29_81 = PyFloat_FromDouble(29.81); if (unlikely(!__pyx_float_29_81)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_29_85 = PyFloat_FromDouble(29.85); if (unlikely(!__pyx_float_29_85)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_29_89 = PyFloat_FromDouble(29.89); if (unlikely(!__pyx_float_29_89)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_29_90 = PyFloat_FromDouble(29.90); if (unlikely(!__pyx_float_29_90)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_29_91 = PyFloat_FromDouble(29.91); if (unlikely(!__pyx_float_29_91)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_29_95 = PyFloat_FromDouble(29.95); if (unlikely(!__pyx_float_29_95)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_30_03 = PyFloat_FromDouble(30.03); if (unlikely(!__pyx_float_30_03)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_30_08 = PyFloat_FromDouble(30.08); if (unlikely(!__pyx_float_30_08)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_30_09 = PyFloat_FromDouble(30.09); if (unlikely(!__pyx_float_30_09)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_30_10 = PyFloat_FromDouble(30.10); if (unlikely(!__pyx_float_30_10)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_30_22 = PyFloat_FromDouble(30.22); if (unlikely(!__pyx_float_30_22)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_30_32 = PyFloat_FromDouble(30.32); if (unlikely(!__pyx_float_30_32)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_30_37 = PyFloat_FromDouble(30.37); if (unlikely(!__pyx_float_30_37)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_30_43 = PyFloat_FromDouble(30.43); if (unlikely(!__pyx_float_30_43)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_30_46 = PyFloat_FromDouble(30.46); if (unlikely(!__pyx_float_30_46)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_30_58 = PyFloat_FromDouble(30.58); if (unlikely(!__pyx_float_30_58)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_30_60 = PyFloat_FromDouble(30.60); if (unlikely(!__pyx_float_30_60)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_30_65 = PyFloat_FromDouble(30.65); if (unlikely(!__pyx_float_30_65)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_30_70 = PyFloat_FromDouble(30.70); if (unlikely(!__pyx_float_30_70)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_30_73 = PyFloat_FromDouble(30.73); if (unlikely(!__pyx_float_30_73)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_30_75 = PyFloat_FromDouble(30.75); if (unlikely(!__pyx_float_30_75)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_30_77 = PyFloat_FromDouble(30.77); if (unlikely(!__pyx_float_30_77)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_30_78 = PyFloat_FromDouble(30.78); if (unlikely(!__pyx_float_30_78)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_30_81 = PyFloat_FromDouble(30.81); if (unlikely(!__pyx_float_30_81)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_30_85 = PyFloat_FromDouble(30.85); if (unlikely(!__pyx_float_30_85)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_30_91 = PyFloat_FromDouble(30.91); if (unlikely(!__pyx_float_30_91)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_30_97 = PyFloat_FromDouble(30.97); if (unlikely(!__pyx_float_30_97)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_30_98 = PyFloat_FromDouble(30.98); if (unlikely(!__pyx_float_30_98)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_31_01 = PyFloat_FromDouble(31.01); if (unlikely(!__pyx_float_31_01)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_31_05 = PyFloat_FromDouble(31.05); if (unlikely(!__pyx_float_31_05)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_31_06 = PyFloat_FromDouble(31.06); if (unlikely(!__pyx_float_31_06)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_31_13 = PyFloat_FromDouble(31.13); if (unlikely(!__pyx_float_31_13)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_31_14 = PyFloat_FromDouble(31.14); if (unlikely(!__pyx_float_31_14)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_31_30 = PyFloat_FromDouble(31.30); if (unlikely(!__pyx_float_31_30)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_31_45 = PyFloat_FromDouble(31.45); if (unlikely(!__pyx_float_31_45)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_31_59 = PyFloat_FromDouble(31.59); if (unlikely(!__pyx_float_31_59)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_31_61 = PyFloat_FromDouble(31.61); if (unlikely(!__pyx_float_31_61)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_31_62 = PyFloat_FromDouble(31.62); if (unlikely(!__pyx_float_31_62)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_31_66 = PyFloat_FromDouble(31.66); if (unlikely(!__pyx_float_31_66)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_31_67 = PyFloat_FromDouble(31.67); if (unlikely(!__pyx_float_31_67)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_31_71 = PyFloat_FromDouble(31.71); if (unlikely(!__pyx_float_31_71)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_31_73 = PyFloat_FromDouble(31.73); if (unlikely(!__pyx_float_31_73)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_31_74 = PyFloat_FromDouble(31.74); if (unlikely(!__pyx_float_31_74)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_31_78 = PyFloat_FromDouble(31.78); if (unlikely(!__pyx_float_31_78)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_31_81 = PyFloat_FromDouble(31.81); if (unlikely(!__pyx_float_31_81)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_31_82 = PyFloat_FromDouble(31.82); if (unlikely(!__pyx_float_31_82)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_31_84 = PyFloat_FromDouble(31.84); if (unlikely(!__pyx_float_31_84)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_31_86 = PyFloat_FromDouble(31.86); if (unlikely(!__pyx_float_31_86)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_31_88 = PyFloat_FromDouble(31.88); if (unlikely(!__pyx_float_31_88)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_31_91 = PyFloat_FromDouble(31.91); if (unlikely(!__pyx_float_31_91)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_31_95 = PyFloat_FromDouble(31.95); if (unlikely(!__pyx_float_31_95)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_31_96 = PyFloat_FromDouble(31.96); if (unlikely(!__pyx_float_31_96)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_31_97 = PyFloat_FromDouble(31.97); if (unlikely(!__pyx_float_31_97)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_32_00 = PyFloat_FromDouble(32.00); if (unlikely(!__pyx_float_32_00)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_32_03 = PyFloat_FromDouble(32.03); if (unlikely(!__pyx_float_32_03)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_32_07 = PyFloat_FromDouble(32.07); if (unlikely(!__pyx_float_32_07)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_32_08 = PyFloat_FromDouble(32.08); if (unlikely(!__pyx_float_32_08)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_32_09 = PyFloat_FromDouble(32.09); if (unlikely(!__pyx_float_32_09)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_32_12 = PyFloat_FromDouble(32.12); if (unlikely(!__pyx_float_32_12)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_32_13 = PyFloat_FromDouble(32.13); if (unlikely(!__pyx_float_32_13)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_32_14 = PyFloat_FromDouble(32.14); if (unlikely(!__pyx_float_32_14)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_32_17 = PyFloat_FromDouble(32.17); if (unlikely(!__pyx_float_32_17)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_32_24 = PyFloat_FromDouble(32.24); if (unlikely(!__pyx_float_32_24)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_32_27 = PyFloat_FromDouble(32.27); if (unlikely(!__pyx_float_32_27)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_32_48 = PyFloat_FromDouble(32.48); if (unlikely(!__pyx_float_32_48)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_32_61 = PyFloat_FromDouble(32.61); if (unlikely(!__pyx_float_32_61)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_32_65 = PyFloat_FromDouble(32.65); if (unlikely(!__pyx_float_32_65)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_32_73 = PyFloat_FromDouble(32.73); if (unlikely(!__pyx_float_32_73)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_32_75 = PyFloat_FromDouble(32.75); if (unlikely(!__pyx_float_32_75)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_32_77 = PyFloat_FromDouble(32.77); if (unlikely(!__pyx_float_32_77)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_32_78 = PyFloat_FromDouble(32.78); if (unlikely(!__pyx_float_32_78)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_32_79 = PyFloat_FromDouble(32.79); if (unlikely(!__pyx_float_32_79)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_32_83 = PyFloat_FromDouble(32.83); if (unlikely(!__pyx_float_32_83)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_32_84 = PyFloat_FromDouble(32.84); if (unlikely(!__pyx_float_32_84)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_32_89 = PyFloat_FromDouble(32.89); if (unlikely(!__pyx_float_32_89)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_32_92 = PyFloat_FromDouble(32.92); if (unlikely(!__pyx_float_32_92)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_32_93 = PyFloat_FromDouble(32.93); if (unlikely(!__pyx_float_32_93)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_32_94 = PyFloat_FromDouble(32.94); if (unlikely(!__pyx_float_32_94)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_32_96 = PyFloat_FromDouble(32.96); if (unlikely(!__pyx_float_32_96)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_32_97 = PyFloat_FromDouble(32.97); if (unlikely(!__pyx_float_32_97)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_33_00 = PyFloat_FromDouble(33.00); if (unlikely(!__pyx_float_33_00)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_33_02 = PyFloat_FromDouble(33.02); if (unlikely(!__pyx_float_33_02)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_33_14 = PyFloat_FromDouble(33.14); if (unlikely(!__pyx_float_33_14)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_33_21 = PyFloat_FromDouble(33.21); if (unlikely(!__pyx_float_33_21)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_33_28 = PyFloat_FromDouble(33.28); if (unlikely(!__pyx_float_33_28)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_33_29 = PyFloat_FromDouble(33.29); if (unlikely(!__pyx_float_33_29)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_33_32 = PyFloat_FromDouble(33.32); if (unlikely(!__pyx_float_33_32)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_33_36 = PyFloat_FromDouble(33.36); if (unlikely(!__pyx_float_33_36)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_33_37 = PyFloat_FromDouble(33.37); if (unlikely(!__pyx_float_33_37)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_33_52 = PyFloat_FromDouble(33.52); if (unlikely(!__pyx_float_33_52)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_33_59 = PyFloat_FromDouble(33.59); if (unlikely(!__pyx_float_33_59)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_33_60 = PyFloat_FromDouble(33.60); if (unlikely(!__pyx_float_33_60)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_33_77 = PyFloat_FromDouble(33.77); if (unlikely(!__pyx_float_33_77)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_33_81 = PyFloat_FromDouble(33.81); if (unlikely(!__pyx_float_33_81)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_33_82 = PyFloat_FromDouble(33.82); if (unlikely(!__pyx_float_33_82)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_33_85 = PyFloat_FromDouble(33.85); if (unlikely(!__pyx_float_33_85)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_33_95 = PyFloat_FromDouble(33.95); if (unlikely(!__pyx_float_33_95)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_34_03 = PyFloat_FromDouble(34.03); if (unlikely(!__pyx_float_34_03)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_34_04 = PyFloat_FromDouble(34.04); if (unlikely(!__pyx_float_34_04)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_34_08 = PyFloat_FromDouble(34.08); if (unlikely(!__pyx_float_34_08)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_34_09 = PyFloat_FromDouble(34.09); if (unlikely(!__pyx_float_34_09)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_34_12 = PyFloat_FromDouble(34.12); if (unlikely(!__pyx_float_34_12)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_34_13 = PyFloat_FromDouble(34.13); if (unlikely(!__pyx_float_34_13)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_34_19 = PyFloat_FromDouble(34.19); if (unlikely(!__pyx_float_34_19)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_34_21 = PyFloat_FromDouble(34.21); if (unlikely(!__pyx_float_34_21)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_34_26 = PyFloat_FromDouble(34.26); if (unlikely(!__pyx_float_34_26)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_34_30 = PyFloat_FromDouble(34.30); if (unlikely(!__pyx_float_34_30)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_34_31 = PyFloat_FromDouble(34.31); if (unlikely(!__pyx_float_34_31)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_34_37 = PyFloat_FromDouble(34.37); if (unlikely(!__pyx_float_34_37)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_34_42 = PyFloat_FromDouble(34.42); if (unlikely(!__pyx_float_34_42)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_34_52 = PyFloat_FromDouble(34.52); if (unlikely(!__pyx_float_34_52)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_34_76 = PyFloat_FromDouble(34.76); if (unlikely(!__pyx_float_34_76)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_34_79 = PyFloat_FromDouble(34.79); if (unlikely(!__pyx_float_34_79)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_34_91 = PyFloat_FromDouble(34.91); if (unlikely(!__pyx_float_34_91)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_34_95 = PyFloat_FromDouble(34.95); if (unlikely(!__pyx_float_34_95)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_34_96 = PyFloat_FromDouble(34.96); if (unlikely(!__pyx_float_34_96)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_34_98 = PyFloat_FromDouble(34.98); if (unlikely(!__pyx_float_34_98)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_35_06 = PyFloat_FromDouble(35.06); if (unlikely(!__pyx_float_35_06)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_35_07 = PyFloat_FromDouble(35.07); if (unlikely(!__pyx_float_35_07)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_35_13 = PyFloat_FromDouble(35.13); if (unlikely(!__pyx_float_35_13)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_35_14 = PyFloat_FromDouble(35.14); if (unlikely(!__pyx_float_35_14)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_35_16 = PyFloat_FromDouble(35.16); if (unlikely(!__pyx_float_35_16)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_35_17 = PyFloat_FromDouble(35.17); if (unlikely(!__pyx_float_35_17)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_35_18 = PyFloat_FromDouble(35.18); if (unlikely(!__pyx_float_35_18)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_35_25 = PyFloat_FromDouble(35.25); if (unlikely(!__pyx_float_35_25)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_35_30 = PyFloat_FromDouble(35.30); if (unlikely(!__pyx_float_35_30)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_35_38 = PyFloat_FromDouble(35.38); if (unlikely(!__pyx_float_35_38)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_35_44 = PyFloat_FromDouble(35.44); if (unlikely(!__pyx_float_35_44)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_35_46 = PyFloat_FromDouble(35.46); if (unlikely(!__pyx_float_35_46)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_35_50 = PyFloat_FromDouble(35.50); if (unlikely(!__pyx_float_35_50)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_35_59 = PyFloat_FromDouble(35.59); if (unlikely(!__pyx_float_35_59)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_35_64 = PyFloat_FromDouble(35.64); if (unlikely(!__pyx_float_35_64)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_35_67 = PyFloat_FromDouble(35.67); if (unlikely(!__pyx_float_35_67)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_35_75 = PyFloat_FromDouble(35.75); if (unlikely(!__pyx_float_35_75)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_36_00 = PyFloat_FromDouble(36.00); if (unlikely(!__pyx_float_36_00)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_36_01 = PyFloat_FromDouble(36.01); if (unlikely(!__pyx_float_36_01)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_36_02 = PyFloat_FromDouble(36.02); if (unlikely(!__pyx_float_36_02)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_36_07 = PyFloat_FromDouble(36.07); if (unlikely(!__pyx_float_36_07)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_36_09 = PyFloat_FromDouble(36.09); if (unlikely(!__pyx_float_36_09)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_36_14 = PyFloat_FromDouble(36.14); if (unlikely(!__pyx_float_36_14)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_36_16 = PyFloat_FromDouble(36.16); if (unlikely(!__pyx_float_36_16)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_36_20 = PyFloat_FromDouble(36.20); if (unlikely(!__pyx_float_36_20)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_36_25 = PyFloat_FromDouble(36.25); if (unlikely(!__pyx_float_36_25)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_36_27 = PyFloat_FromDouble(36.27); if (unlikely(!__pyx_float_36_27)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_36_28 = PyFloat_FromDouble(36.28); if (unlikely(!__pyx_float_36_28)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_36_29 = PyFloat_FromDouble(36.29); if (unlikely(!__pyx_float_36_29)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_36_30 = PyFloat_FromDouble(36.30); if (unlikely(!__pyx_float_36_30)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_36_32 = PyFloat_FromDouble(36.32); if (unlikely(!__pyx_float_36_32)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_36_34 = PyFloat_FromDouble(36.34); if (unlikely(!__pyx_float_36_34)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_36_40 = PyFloat_FromDouble(36.40); if (unlikely(!__pyx_float_36_40)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_36_43 = PyFloat_FromDouble(36.43); if (unlikely(!__pyx_float_36_43)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_36_55 = PyFloat_FromDouble(36.55); if (unlikely(!__pyx_float_36_55)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_36_56 = PyFloat_FromDouble(36.56); if (unlikely(!__pyx_float_36_56)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_36_62 = PyFloat_FromDouble(36.62); if (unlikely(!__pyx_float_36_62)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_36_74 = PyFloat_FromDouble(36.74); if (unlikely(!__pyx_float_36_74)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_36_78 = PyFloat_FromDouble(36.78); if (unlikely(!__pyx_float_36_78)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_36_88 = PyFloat_FromDouble(36.88); if (unlikely(!__pyx_float_36_88)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_37_11 = PyFloat_FromDouble(37.11); if (unlikely(!__pyx_float_37_11)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_37_16 = PyFloat_FromDouble(37.16); if (unlikely(!__pyx_float_37_16)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_37_20 = PyFloat_FromDouble(37.20); if (unlikely(!__pyx_float_37_20)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_37_23 = PyFloat_FromDouble(37.23); if (unlikely(!__pyx_float_37_23)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_37_27 = PyFloat_FromDouble(37.27); if (unlikely(!__pyx_float_37_27)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_37_32 = PyFloat_FromDouble(37.32); if (unlikely(!__pyx_float_37_32)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_37_34 = PyFloat_FromDouble(37.34); if (unlikely(!__pyx_float_37_34)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_37_35 = PyFloat_FromDouble(37.35); if (unlikely(!__pyx_float_37_35)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_37_38 = PyFloat_FromDouble(37.38); if (unlikely(!__pyx_float_37_38)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_37_39 = PyFloat_FromDouble(37.39); if (unlikely(!__pyx_float_37_39)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_37_41 = PyFloat_FromDouble(37.41); if (unlikely(!__pyx_float_37_41)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_37_51 = PyFloat_FromDouble(37.51); if (unlikely(!__pyx_float_37_51)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_37_56 = PyFloat_FromDouble(37.56); if (unlikely(!__pyx_float_37_56)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_37_57 = PyFloat_FromDouble(37.57); if (unlikely(!__pyx_float_37_57)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_37_68 = PyFloat_FromDouble(37.68); if (unlikely(!__pyx_float_37_68)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_37_69 = PyFloat_FromDouble(37.69); if (unlikely(!__pyx_float_37_69)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_37_76 = PyFloat_FromDouble(37.76); if (unlikely(!__pyx_float_37_76)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_37_77 = PyFloat_FromDouble(37.77); if (unlikely(!__pyx_float_37_77)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_37_88 = PyFloat_FromDouble(37.88); if (unlikely(!__pyx_float_37_88)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_37_89 = PyFloat_FromDouble(37.89); if (unlikely(!__pyx_float_37_89)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_37_90 = PyFloat_FromDouble(37.90); if (unlikely(!__pyx_float_37_90)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_37_93 = PyFloat_FromDouble(37.93); if (unlikely(!__pyx_float_37_93)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_37_99 = PyFloat_FromDouble(37.99); if (unlikely(!__pyx_float_37_99)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_38_10 = PyFloat_FromDouble(38.10); if (unlikely(!__pyx_float_38_10)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_38_18 = PyFloat_FromDouble(38.18); if (unlikely(!__pyx_float_38_18)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_38_20 = PyFloat_FromDouble(38.20); if (unlikely(!__pyx_float_38_20)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_38_25 = PyFloat_FromDouble(38.25); if (unlikely(!__pyx_float_38_25)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_38_26 = PyFloat_FromDouble(38.26); if (unlikely(!__pyx_float_38_26)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_38_30 = PyFloat_FromDouble(38.30); if (unlikely(!__pyx_float_38_30)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_38_33 = PyFloat_FromDouble(38.33); if (unlikely(!__pyx_float_38_33)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_38_36 = PyFloat_FromDouble(38.36); if (unlikely(!__pyx_float_38_36)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_38_46 = PyFloat_FromDouble(38.46); if (unlikely(!__pyx_float_38_46)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_38_51 = PyFloat_FromDouble(38.51); if (unlikely(!__pyx_float_38_51)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_38_53 = PyFloat_FromDouble(38.53); if (unlikely(!__pyx_float_38_53)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_38_54 = PyFloat_FromDouble(38.54); if (unlikely(!__pyx_float_38_54)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_38_55 = PyFloat_FromDouble(38.55); if (unlikely(!__pyx_float_38_55)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_38_58 = PyFloat_FromDouble(38.58); if (unlikely(!__pyx_float_38_58)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_38_64 = PyFloat_FromDouble(38.64); if (unlikely(!__pyx_float_38_64)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_38_68 = PyFloat_FromDouble(38.68); if (unlikely(!__pyx_float_38_68)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_38_80 = PyFloat_FromDouble(38.80); if (unlikely(!__pyx_float_38_80)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_38_85 = PyFloat_FromDouble(38.85); if (unlikely(!__pyx_float_38_85)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_38_86 = PyFloat_FromDouble(38.86); if (unlikely(!__pyx_float_38_86)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_38_92 = PyFloat_FromDouble(38.92); if (unlikely(!__pyx_float_38_92)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_38_93 = PyFloat_FromDouble(38.93); if (unlikely(!__pyx_float_38_93)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_39_00 = PyFloat_FromDouble(39.00); if (unlikely(!__pyx_float_39_00)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_39_03 = PyFloat_FromDouble(39.03); if (unlikely(!__pyx_float_39_03)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_39_06 = PyFloat_FromDouble(39.06); if (unlikely(!__pyx_float_39_06)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_39_11 = PyFloat_FromDouble(39.11); if (unlikely(!__pyx_float_39_11)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_39_17 = PyFloat_FromDouble(39.17); if (unlikely(!__pyx_float_39_17)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_39_23 = PyFloat_FromDouble(39.23); if (unlikely(!__pyx_float_39_23)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_39_29 = PyFloat_FromDouble(39.29); if (unlikely(!__pyx_float_39_29)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_39_35 = PyFloat_FromDouble(39.35); if (unlikely(!__pyx_float_39_35)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_39_41 = PyFloat_FromDouble(39.41); if (unlikely(!__pyx_float_39_41)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_39_47 = PyFloat_FromDouble(39.47); if (unlikely(!__pyx_float_39_47)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_39_59 = PyFloat_FromDouble(39.59); if (unlikely(!__pyx_float_39_59)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_39_60 = PyFloat_FromDouble(39.60); if (unlikely(!__pyx_float_39_60)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_39_62 = PyFloat_FromDouble(39.62); if (unlikely(!__pyx_float_39_62)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_39_64 = PyFloat_FromDouble(39.64); if (unlikely(!__pyx_float_39_64)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_39_65 = PyFloat_FromDouble(39.65); if (unlikely(!__pyx_float_39_65)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_39_66 = PyFloat_FromDouble(39.66); if (unlikely(!__pyx_float_39_66)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_39_67 = PyFloat_FromDouble(39.67); if (unlikely(!__pyx_float_39_67)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_39_75 = PyFloat_FromDouble(39.75); if (unlikely(!__pyx_float_39_75)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_39_80 = PyFloat_FromDouble(39.80); if (unlikely(!__pyx_float_39_80)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_39_88 = PyFloat_FromDouble(39.88); if (unlikely(!__pyx_float_39_88)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_40_02 = PyFloat_FromDouble(40.02); if (unlikely(!__pyx_float_40_02)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_40_03 = PyFloat_FromDouble(40.03); if (unlikely(!__pyx_float_40_03)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_40_05 = PyFloat_FromDouble(40.05); if (unlikely(!__pyx_float_40_05)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_40_07 = PyFloat_FromDouble(40.07); if (unlikely(!__pyx_float_40_07)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_40_08 = PyFloat_FromDouble(40.08); if (unlikely(!__pyx_float_40_08)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_40_09 = PyFloat_FromDouble(40.09); if (unlikely(!__pyx_float_40_09)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_40_11 = PyFloat_FromDouble(40.11); if (unlikely(!__pyx_float_40_11)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_40_19 = PyFloat_FromDouble(40.19); if (unlikely(!__pyx_float_40_19)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_40_25 = PyFloat_FromDouble(40.25); if (unlikely(!__pyx_float_40_25)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_40_32 = PyFloat_FromDouble(40.32); if (unlikely(!__pyx_float_40_32)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_40_33 = PyFloat_FromDouble(40.33); if (unlikely(!__pyx_float_40_33)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_40_35 = PyFloat_FromDouble(40.35); if (unlikely(!__pyx_float_40_35)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_40_43 = PyFloat_FromDouble(40.43); if (unlikely(!__pyx_float_40_43)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_40_53 = PyFloat_FromDouble(40.53); if (unlikely(!__pyx_float_40_53)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_40_54 = PyFloat_FromDouble(40.54); if (unlikely(!__pyx_float_40_54)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_40_55 = PyFloat_FromDouble(40.55); if (unlikely(!__pyx_float_40_55)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_40_58 = PyFloat_FromDouble(40.58); if (unlikely(!__pyx_float_40_58)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_40_69 = PyFloat_FromDouble(40.69); if (unlikely(!__pyx_float_40_69)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_40_71 = PyFloat_FromDouble(40.71); if (unlikely(!__pyx_float_40_71)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_40_77 = PyFloat_FromDouble(40.77); if (unlikely(!__pyx_float_40_77)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_40_80 = PyFloat_FromDouble(40.80); if (unlikely(!__pyx_float_40_80)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_40_85 = PyFloat_FromDouble(40.85); if (unlikely(!__pyx_float_40_85)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_40_90 = PyFloat_FromDouble(40.90); if (unlikely(!__pyx_float_40_90)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_40_95 = PyFloat_FromDouble(40.95); if (unlikely(!__pyx_float_40_95)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_40_97 = PyFloat_FromDouble(40.97); if (unlikely(!__pyx_float_40_97)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_41_00 = PyFloat_FromDouble(41.00); if (unlikely(!__pyx_float_41_00)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_41_16 = PyFloat_FromDouble(41.16); if (unlikely(!__pyx_float_41_16)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_41_20 = PyFloat_FromDouble(41.20); if (unlikely(!__pyx_float_41_20)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_41_23 = PyFloat_FromDouble(41.23); if (unlikely(!__pyx_float_41_23)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_41_28 = PyFloat_FromDouble(41.28); if (unlikely(!__pyx_float_41_28)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_41_37 = PyFloat_FromDouble(41.37); if (unlikely(!__pyx_float_41_37)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_41_40 = PyFloat_FromDouble(41.40); if (unlikely(!__pyx_float_41_40)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_41_43 = PyFloat_FromDouble(41.43); if (unlikely(!__pyx_float_41_43)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_41_47 = PyFloat_FromDouble(41.47); if (unlikely(!__pyx_float_41_47)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_41_50 = PyFloat_FromDouble(41.50); if (unlikely(!__pyx_float_41_50)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_41_51 = PyFloat_FromDouble(41.51); if (unlikely(!__pyx_float_41_51)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_41_52 = PyFloat_FromDouble(41.52); if (unlikely(!__pyx_float_41_52)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_41_53 = PyFloat_FromDouble(41.53); if (unlikely(!__pyx_float_41_53)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_41_61 = PyFloat_FromDouble(41.61); if (unlikely(!__pyx_float_41_61)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_41_77 = PyFloat_FromDouble(41.77); if (unlikely(!__pyx_float_41_77)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_41_82 = PyFloat_FromDouble(41.82); if (unlikely(!__pyx_float_41_82)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_41_86 = PyFloat_FromDouble(41.86); if (unlikely(!__pyx_float_41_86)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_41_89 = PyFloat_FromDouble(41.89); if (unlikely(!__pyx_float_41_89)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_41_90 = PyFloat_FromDouble(41.90); if (unlikely(!__pyx_float_41_90)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_41_93 = PyFloat_FromDouble(41.93); if (unlikely(!__pyx_float_41_93)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_41_97 = PyFloat_FromDouble(41.97); if (unlikely(!__pyx_float_41_97)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_42_09 = PyFloat_FromDouble(42.09); if (unlikely(!__pyx_float_42_09)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_42_23 = PyFloat_FromDouble(42.23); if (unlikely(!__pyx_float_42_23)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_42_37 = PyFloat_FromDouble(42.37); if (unlikely(!__pyx_float_42_37)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_42_42 = PyFloat_FromDouble(42.42); if (unlikely(!__pyx_float_42_42)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_42_43 = PyFloat_FromDouble(42.43); if (unlikely(!__pyx_float_42_43)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_42_50 = PyFloat_FromDouble(42.50); if (unlikely(!__pyx_float_42_50)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_42_53 = PyFloat_FromDouble(42.53); if (unlikely(!__pyx_float_42_53)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_42_66 = PyFloat_FromDouble(42.66); if (unlikely(!__pyx_float_42_66)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_42_67 = PyFloat_FromDouble(42.67); if (unlikely(!__pyx_float_42_67)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_42_80 = PyFloat_FromDouble(42.80); if (unlikely(!__pyx_float_42_80)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_42_92 = PyFloat_FromDouble(42.92); if (unlikely(!__pyx_float_42_92)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_42_95 = PyFloat_FromDouble(42.95); if (unlikely(!__pyx_float_42_95)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_43_04 = PyFloat_FromDouble(43.04); if (unlikely(!__pyx_float_43_04)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_43_06 = PyFloat_FromDouble(43.06); if (unlikely(!__pyx_float_43_06)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_43_08 = PyFloat_FromDouble(43.08); if (unlikely(!__pyx_float_43_08)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_43_24 = PyFloat_FromDouble(43.24); if (unlikely(!__pyx_float_43_24)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_43_25 = PyFloat_FromDouble(43.25); if (unlikely(!__pyx_float_43_25)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_43_27 = PyFloat_FromDouble(43.27); if (unlikely(!__pyx_float_43_27)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_43_35 = PyFloat_FromDouble(43.35); if (unlikely(!__pyx_float_43_35)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_43_38 = PyFloat_FromDouble(43.38); if (unlikely(!__pyx_float_43_38)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_43_40 = PyFloat_FromDouble(43.40); if (unlikely(!__pyx_float_43_40)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_43_41 = PyFloat_FromDouble(43.41); if (unlikely(!__pyx_float_43_41)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_43_46 = PyFloat_FromDouble(43.46); if (unlikely(!__pyx_float_43_46)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_43_48 = PyFloat_FromDouble(43.48); if (unlikely(!__pyx_float_43_48)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_43_49 = PyFloat_FromDouble(43.49); if (unlikely(!__pyx_float_43_49)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_43_54 = PyFloat_FromDouble(43.54); if (unlikely(!__pyx_float_43_54)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_43_60 = PyFloat_FromDouble(43.60); if (unlikely(!__pyx_float_43_60)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_43_65 = PyFloat_FromDouble(43.65); if (unlikely(!__pyx_float_43_65)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_43_74 = PyFloat_FromDouble(43.74); if (unlikely(!__pyx_float_43_74)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_43_75 = PyFloat_FromDouble(43.75); if (unlikely(!__pyx_float_43_75)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_43_87 = PyFloat_FromDouble(43.87); if (unlikely(!__pyx_float_43_87)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_43_89 = PyFloat_FromDouble(43.89); if (unlikely(!__pyx_float_43_89)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_43_93 = PyFloat_FromDouble(43.93); if (unlikely(!__pyx_float_43_93)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_44_02 = PyFloat_FromDouble(44.02); if (unlikely(!__pyx_float_44_02)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_44_03 = PyFloat_FromDouble(44.03); if (unlikely(!__pyx_float_44_03)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_44_10 = PyFloat_FromDouble(44.10); if (unlikely(!__pyx_float_44_10)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_44_14 = PyFloat_FromDouble(44.14); if (unlikely(!__pyx_float_44_14)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_44_17 = PyFloat_FromDouble(44.17); if (unlikely(!__pyx_float_44_17)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_44_21 = PyFloat_FromDouble(44.21); if (unlikely(!__pyx_float_44_21)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_44_24 = PyFloat_FromDouble(44.24); if (unlikely(!__pyx_float_44_24)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_44_31 = PyFloat_FromDouble(44.31); if (unlikely(!__pyx_float_44_31)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_44_48 = PyFloat_FromDouble(44.48); if (unlikely(!__pyx_float_44_48)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_44_51 = PyFloat_FromDouble(44.51); if (unlikely(!__pyx_float_44_51)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_44_52 = PyFloat_FromDouble(44.52); if (unlikely(!__pyx_float_44_52)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_44_53 = PyFloat_FromDouble(44.53); if (unlikely(!__pyx_float_44_53)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_44_68 = PyFloat_FromDouble(44.68); if (unlikely(!__pyx_float_44_68)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_44_73 = PyFloat_FromDouble(44.73); if (unlikely(!__pyx_float_44_73)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_44_74 = PyFloat_FromDouble(44.74); if (unlikely(!__pyx_float_44_74)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_44_94 = PyFloat_FromDouble(44.94); if (unlikely(!__pyx_float_44_94)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_45_03 = PyFloat_FromDouble(45.03); if (unlikely(!__pyx_float_45_03)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_45_05 = PyFloat_FromDouble(45.05); if (unlikely(!__pyx_float_45_05)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_45_12 = PyFloat_FromDouble(45.12); if (unlikely(!__pyx_float_45_12)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_45_13 = PyFloat_FromDouble(45.13); if (unlikely(!__pyx_float_45_13)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_45_24 = PyFloat_FromDouble(45.24); if (unlikely(!__pyx_float_45_24)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_45_25 = PyFloat_FromDouble(45.25); if (unlikely(!__pyx_float_45_25)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_45_29 = PyFloat_FromDouble(45.29); if (unlikely(!__pyx_float_45_29)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_45_36 = PyFloat_FromDouble(45.36); if (unlikely(!__pyx_float_45_36)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_45_38 = PyFloat_FromDouble(45.38); if (unlikely(!__pyx_float_45_38)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_45_42 = PyFloat_FromDouble(45.42); if (unlikely(!__pyx_float_45_42)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_45_49 = PyFloat_FromDouble(45.49); if (unlikely(!__pyx_float_45_49)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_45_62 = PyFloat_FromDouble(45.62); if (unlikely(!__pyx_float_45_62)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_45_65 = PyFloat_FromDouble(45.65); if (unlikely(!__pyx_float_45_65)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_45_68 = PyFloat_FromDouble(45.68); if (unlikely(!__pyx_float_45_68)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_45_85 = PyFloat_FromDouble(45.85); if (unlikely(!__pyx_float_45_85)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_45_95 = PyFloat_FromDouble(45.95); if (unlikely(!__pyx_float_45_95)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_45_99 = PyFloat_FromDouble(45.99); if (unlikely(!__pyx_float_45_99)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_46_02 = PyFloat_FromDouble(46.02); if (unlikely(!__pyx_float_46_02)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_46_11 = PyFloat_FromDouble(46.11); if (unlikely(!__pyx_float_46_11)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_46_13 = PyFloat_FromDouble(46.13); if (unlikely(!__pyx_float_46_13)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_46_18 = PyFloat_FromDouble(46.18); if (unlikely(!__pyx_float_46_18)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_46_20 = PyFloat_FromDouble(46.20); if (unlikely(!__pyx_float_46_20)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_46_33 = PyFloat_FromDouble(46.33); if (unlikely(!__pyx_float_46_33)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_46_35 = PyFloat_FromDouble(46.35); if (unlikely(!__pyx_float_46_35)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_46_36 = PyFloat_FromDouble(46.36); if (unlikely(!__pyx_float_46_36)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_46_42 = PyFloat_FromDouble(46.42); if (unlikely(!__pyx_float_46_42)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_46_43 = PyFloat_FromDouble(46.43); if (unlikely(!__pyx_float_46_43)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_46_54 = PyFloat_FromDouble(46.54); if (unlikely(!__pyx_float_46_54)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_46_56 = PyFloat_FromDouble(46.56); if (unlikely(!__pyx_float_46_56)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_46_57 = PyFloat_FromDouble(46.57); if (unlikely(!__pyx_float_46_57)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_46_62 = PyFloat_FromDouble(46.62); if (unlikely(!__pyx_float_46_62)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_46_65 = PyFloat_FromDouble(46.65); if (unlikely(!__pyx_float_46_65)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_46_68 = PyFloat_FromDouble(46.68); if (unlikely(!__pyx_float_46_68)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_46_86 = PyFloat_FromDouble(46.86); if (unlikely(!__pyx_float_46_86)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_46_93 = PyFloat_FromDouble(46.93); if (unlikely(!__pyx_float_46_93)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_47_00 = PyFloat_FromDouble(47.00); if (unlikely(!__pyx_float_47_00)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_47_07 = PyFloat_FromDouble(47.07); if (unlikely(!__pyx_float_47_07)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_47_11 = PyFloat_FromDouble(47.11); if (unlikely(!__pyx_float_47_11)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_47_16 = PyFloat_FromDouble(47.16); if (unlikely(!__pyx_float_47_16)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_47_20 = PyFloat_FromDouble(47.20); if (unlikely(!__pyx_float_47_20)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_47_21 = PyFloat_FromDouble(47.21); if (unlikely(!__pyx_float_47_21)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_47_25 = PyFloat_FromDouble(47.25); if (unlikely(!__pyx_float_47_25)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_47_40 = PyFloat_FromDouble(47.40); if (unlikely(!__pyx_float_47_40)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_47_41 = PyFloat_FromDouble(47.41); if (unlikely(!__pyx_float_47_41)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_47_46 = PyFloat_FromDouble(47.46); if (unlikely(!__pyx_float_47_46)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_47_50 = PyFloat_FromDouble(47.50); if (unlikely(!__pyx_float_47_50)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_47_60 = PyFloat_FromDouble(47.60); if (unlikely(!__pyx_float_47_60)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_47_64 = PyFloat_FromDouble(47.64); if (unlikely(!__pyx_float_47_64)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_47_65 = PyFloat_FromDouble(47.65); if (unlikely(!__pyx_float_47_65)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_47_72 = PyFloat_FromDouble(47.72); if (unlikely(!__pyx_float_47_72)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_47_78 = PyFloat_FromDouble(47.78); if (unlikely(!__pyx_float_47_78)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_47_79 = PyFloat_FromDouble(47.79); if (unlikely(!__pyx_float_47_79)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_47_81 = PyFloat_FromDouble(47.81); if (unlikely(!__pyx_float_47_81)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_47_82 = PyFloat_FromDouble(47.82); if (unlikely(!__pyx_float_47_82)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_47_87 = PyFloat_FromDouble(47.87); if (unlikely(!__pyx_float_47_87)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_48_07 = PyFloat_FromDouble(48.07); if (unlikely(!__pyx_float_48_07)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_48_11 = PyFloat_FromDouble(48.11); if (unlikely(!__pyx_float_48_11)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_48_28 = PyFloat_FromDouble(48.28); if (unlikely(!__pyx_float_48_28)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_48_36 = PyFloat_FromDouble(48.36); if (unlikely(!__pyx_float_48_36)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_48_40 = PyFloat_FromDouble(48.40); if (unlikely(!__pyx_float_48_40)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_48_42 = PyFloat_FromDouble(48.42); if (unlikely(!__pyx_float_48_42)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_48_43 = PyFloat_FromDouble(48.43); if (unlikely(!__pyx_float_48_43)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_48_44 = PyFloat_FromDouble(48.44); if (unlikely(!__pyx_float_48_44)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_48_52 = PyFloat_FromDouble(48.52); if (unlikely(!__pyx_float_48_52)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_48_57 = PyFloat_FromDouble(48.57); if (unlikely(!__pyx_float_48_57)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_48_65 = PyFloat_FromDouble(48.65); if (unlikely(!__pyx_float_48_65)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_48_68 = PyFloat_FromDouble(48.68); if (unlikely(!__pyx_float_48_68)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_48_71 = PyFloat_FromDouble(48.71); if (unlikely(!__pyx_float_48_71)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_48_76 = PyFloat_FromDouble(48.76); if (unlikely(!__pyx_float_48_76)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_48_79 = PyFloat_FromDouble(48.79); if (unlikely(!__pyx_float_48_79)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_48_83 = PyFloat_FromDouble(48.83); if (unlikely(!__pyx_float_48_83)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_48_87 = PyFloat_FromDouble(48.87); if (unlikely(!__pyx_float_48_87)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_48_88 = PyFloat_FromDouble(48.88); if (unlikely(!__pyx_float_48_88)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_48_89 = PyFloat_FromDouble(48.89); if (unlikely(!__pyx_float_48_89)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_48_95 = PyFloat_FromDouble(48.95); if (unlikely(!__pyx_float_48_95)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_48_98 = PyFloat_FromDouble(48.98); if (unlikely(!__pyx_float_48_98)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_48_99 = PyFloat_FromDouble(48.99); if (unlikely(!__pyx_float_48_99)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_49_01 = PyFloat_FromDouble(49.01); if (unlikely(!__pyx_float_49_01)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_49_14 = PyFloat_FromDouble(49.14); if (unlikely(!__pyx_float_49_14)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_49_27 = PyFloat_FromDouble(49.27); if (unlikely(!__pyx_float_49_27)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_49_29 = PyFloat_FromDouble(49.29); if (unlikely(!__pyx_float_49_29)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_49_44 = PyFloat_FromDouble(49.44); if (unlikely(!__pyx_float_49_44)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_49_48 = PyFloat_FromDouble(49.48); if (unlikely(!__pyx_float_49_48)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_49_55 = PyFloat_FromDouble(49.55); if (unlikely(!__pyx_float_49_55)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_49_65 = PyFloat_FromDouble(49.65); if (unlikely(!__pyx_float_49_65)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_49_67 = PyFloat_FromDouble(49.67); if (unlikely(!__pyx_float_49_67)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_49_68 = PyFloat_FromDouble(49.68); if (unlikely(!__pyx_float_49_68)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_49_87 = PyFloat_FromDouble(49.87); if (unlikely(!__pyx_float_49_87)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_49_93 = PyFloat_FromDouble(49.93); if (unlikely(!__pyx_float_49_93)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_49_95 = PyFloat_FromDouble(49.95); if (unlikely(!__pyx_float_49_95)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_50_01 = PyFloat_FromDouble(50.01); if (unlikely(!__pyx_float_50_01)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_50_03 = PyFloat_FromDouble(50.03); if (unlikely(!__pyx_float_50_03)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_50_10 = PyFloat_FromDouble(50.10); if (unlikely(!__pyx_float_50_10)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_50_11 = PyFloat_FromDouble(50.11); if (unlikely(!__pyx_float_50_11)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_50_13 = PyFloat_FromDouble(50.13); if (unlikely(!__pyx_float_50_13)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_50_18 = PyFloat_FromDouble(50.18); if (unlikely(!__pyx_float_50_18)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_50_21 = PyFloat_FromDouble(50.21); if (unlikely(!__pyx_float_50_21)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_50_48 = PyFloat_FromDouble(50.48); if (unlikely(!__pyx_float_50_48)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_50_49 = PyFloat_FromDouble(50.49); if (unlikely(!__pyx_float_50_49)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_50_63 = PyFloat_FromDouble(50.63); if (unlikely(!__pyx_float_50_63)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_50_72 = PyFloat_FromDouble(50.72); if (unlikely(!__pyx_float_50_72)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_50_76 = PyFloat_FromDouble(50.76); if (unlikely(!__pyx_float_50_76)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_50_83 = PyFloat_FromDouble(50.83); if (unlikely(!__pyx_float_50_83)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_50_88 = PyFloat_FromDouble(50.88); if (unlikely(!__pyx_float_50_88)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_50_92 = PyFloat_FromDouble(50.92); if (unlikely(!__pyx_float_50_92)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_50_96 = PyFloat_FromDouble(50.96); if (unlikely(!__pyx_float_50_96)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_51_09 = PyFloat_FromDouble(51.09); if (unlikely(!__pyx_float_51_09)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_51_12 = PyFloat_FromDouble(51.12); if (unlikely(!__pyx_float_51_12)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_51_19 = PyFloat_FromDouble(51.19); if (unlikely(!__pyx_float_51_19)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_51_27 = PyFloat_FromDouble(51.27); if (unlikely(!__pyx_float_51_27)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_51_29 = PyFloat_FromDouble(51.29); if (unlikely(!__pyx_float_51_29)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_51_34 = PyFloat_FromDouble(51.34); if (unlikely(!__pyx_float_51_34)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_51_37 = PyFloat_FromDouble(51.37); if (unlikely(!__pyx_float_51_37)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_51_42 = PyFloat_FromDouble(51.42); if (unlikely(!__pyx_float_51_42)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_51_54 = PyFloat_FromDouble(51.54); if (unlikely(!__pyx_float_51_54)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_51_62 = PyFloat_FromDouble(51.62); if (unlikely(!__pyx_float_51_62)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_51_81 = PyFloat_FromDouble(51.81); if (unlikely(!__pyx_float_51_81)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_51_93 = PyFloat_FromDouble(51.93); if (unlikely(!__pyx_float_51_93)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_52_00 = PyFloat_FromDouble(52.00); if (unlikely(!__pyx_float_52_00)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_52_15 = PyFloat_FromDouble(52.15); if (unlikely(!__pyx_float_52_15)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_52_17 = PyFloat_FromDouble(52.17); if (unlikely(!__pyx_float_52_17)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_52_33 = PyFloat_FromDouble(52.33); if (unlikely(!__pyx_float_52_33)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_52_43 = PyFloat_FromDouble(52.43); if (unlikely(!__pyx_float_52_43)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_52_47 = PyFloat_FromDouble(52.47); if (unlikely(!__pyx_float_52_47)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_52_70 = PyFloat_FromDouble(52.70); if (unlikely(!__pyx_float_52_70)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_52_73 = PyFloat_FromDouble(52.73); if (unlikely(!__pyx_float_52_73)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_52_95 = PyFloat_FromDouble(52.95); if (unlikely(!__pyx_float_52_95)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_53_42 = PyFloat_FromDouble(53.42); if (unlikely(!__pyx_float_53_42)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_53_46 = PyFloat_FromDouble(53.46); if (unlikely(!__pyx_float_53_46)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_53_52 = PyFloat_FromDouble(53.52); if (unlikely(!__pyx_float_53_52)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_53_56 = PyFloat_FromDouble(53.56); if (unlikely(!__pyx_float_53_56)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_53_68 = PyFloat_FromDouble(53.68); if (unlikely(!__pyx_float_53_68)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_53_79 = PyFloat_FromDouble(53.79); if (unlikely(!__pyx_float_53_79)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_53_82 = PyFloat_FromDouble(53.82); if (unlikely(!__pyx_float_53_82)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_53_90 = PyFloat_FromDouble(53.90); if (unlikely(!__pyx_float_53_90)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_53_91 = PyFloat_FromDouble(53.91); if (unlikely(!__pyx_float_53_91)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_54_00 = PyFloat_FromDouble(54.00); if (unlikely(!__pyx_float_54_00)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_54_05 = PyFloat_FromDouble(54.05); if (unlikely(!__pyx_float_54_05)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_54_15 = PyFloat_FromDouble(54.15); if (unlikely(!__pyx_float_54_15)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_54_23 = PyFloat_FromDouble(54.23); if (unlikely(!__pyx_float_54_23)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_54_39 = PyFloat_FromDouble(54.39); if (unlikely(!__pyx_float_54_39)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_54_69 = PyFloat_FromDouble(54.69); if (unlikely(!__pyx_float_54_69)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_54_74 = PyFloat_FromDouble(54.74); if (unlikely(!__pyx_float_54_74)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_54_75 = PyFloat_FromDouble(54.75); if (unlikely(!__pyx_float_54_75)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_54_76 = PyFloat_FromDouble(54.76); if (unlikely(!__pyx_float_54_76)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_55_02 = PyFloat_FromDouble(55.02); if (unlikely(!__pyx_float_55_02)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_55_06 = PyFloat_FromDouble(55.06); if (unlikely(!__pyx_float_55_06)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_55_09 = PyFloat_FromDouble(55.09); if (unlikely(!__pyx_float_55_09)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_55_14 = PyFloat_FromDouble(55.14); if (unlikely(!__pyx_float_55_14)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_55_24 = PyFloat_FromDouble(55.24); if (unlikely(!__pyx_float_55_24)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_55_30 = PyFloat_FromDouble(55.30); if (unlikely(!__pyx_float_55_30)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_55_31 = PyFloat_FromDouble(55.31); if (unlikely(!__pyx_float_55_31)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_55_34 = PyFloat_FromDouble(55.34); if (unlikely(!__pyx_float_55_34)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_55_45 = PyFloat_FromDouble(55.45); if (unlikely(!__pyx_float_55_45)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_55_50 = PyFloat_FromDouble(55.50); if (unlikely(!__pyx_float_55_50)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_55_58 = PyFloat_FromDouble(55.58); if (unlikely(!__pyx_float_55_58)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_55_75 = PyFloat_FromDouble(55.75); if (unlikely(!__pyx_float_55_75)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_55_88 = PyFloat_FromDouble(55.88); if (unlikely(!__pyx_float_55_88)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_55_90 = PyFloat_FromDouble(55.90); if (unlikely(!__pyx_float_55_90)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_55_98 = PyFloat_FromDouble(55.98); if (unlikely(!__pyx_float_55_98)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_56_04 = PyFloat_FromDouble(56.04); if (unlikely(!__pyx_float_56_04)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_56_06 = PyFloat_FromDouble(56.06); if (unlikely(!__pyx_float_56_06)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_56_17 = PyFloat_FromDouble(56.17); if (unlikely(!__pyx_float_56_17)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_56_18 = PyFloat_FromDouble(56.18); if (unlikely(!__pyx_float_56_18)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_56_20 = PyFloat_FromDouble(56.20); if (unlikely(!__pyx_float_56_20)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_56_23 = PyFloat_FromDouble(56.23); if (unlikely(!__pyx_float_56_23)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_56_31 = PyFloat_FromDouble(56.31); if (unlikely(!__pyx_float_56_31)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_56_60 = PyFloat_FromDouble(56.60); if (unlikely(!__pyx_float_56_60)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_56_62 = PyFloat_FromDouble(56.62); if (unlikely(!__pyx_float_56_62)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_56_68 = PyFloat_FromDouble(56.68); if (unlikely(!__pyx_float_56_68)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_56_73 = PyFloat_FromDouble(56.73); if (unlikely(!__pyx_float_56_73)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_56_90 = PyFloat_FromDouble(56.90); if (unlikely(!__pyx_float_56_90)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_56_98 = PyFloat_FromDouble(56.98); if (unlikely(!__pyx_float_56_98)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_57_02 = PyFloat_FromDouble(57.02); if (unlikely(!__pyx_float_57_02)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_57_10 = PyFloat_FromDouble(57.10); if (unlikely(!__pyx_float_57_10)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_57_21 = PyFloat_FromDouble(57.21); if (unlikely(!__pyx_float_57_21)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_57_26 = PyFloat_FromDouble(57.26); if (unlikely(!__pyx_float_57_26)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_57_34 = PyFloat_FromDouble(57.34); if (unlikely(!__pyx_float_57_34)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_57_40 = PyFloat_FromDouble(57.40); if (unlikely(!__pyx_float_57_40)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_57_41 = PyFloat_FromDouble(57.41); if (unlikely(!__pyx_float_57_41)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_57_53 = PyFloat_FromDouble(57.53); if (unlikely(!__pyx_float_57_53)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_57_63 = PyFloat_FromDouble(57.63); if (unlikely(!__pyx_float_57_63)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_57_69 = PyFloat_FromDouble(57.69); if (unlikely(!__pyx_float_57_69)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_57_81 = PyFloat_FromDouble(57.81); if (unlikely(!__pyx_float_57_81)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_57_90 = PyFloat_FromDouble(57.90); if (unlikely(!__pyx_float_57_90)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_57_96 = PyFloat_FromDouble(57.96); if (unlikely(!__pyx_float_57_96)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_57_97 = PyFloat_FromDouble(57.97); if (unlikely(!__pyx_float_57_97)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_57_98 = PyFloat_FromDouble(57.98); if (unlikely(!__pyx_float_57_98)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_58_03 = PyFloat_FromDouble(58.03); if (unlikely(!__pyx_float_58_03)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_58_11 = PyFloat_FromDouble(58.11); if (unlikely(!__pyx_float_58_11)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_58_17 = PyFloat_FromDouble(58.17); if (unlikely(!__pyx_float_58_17)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_58_28 = PyFloat_FromDouble(58.28); if (unlikely(!__pyx_float_58_28)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_58_34 = PyFloat_FromDouble(58.34); if (unlikely(!__pyx_float_58_34)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_58_42 = PyFloat_FromDouble(58.42); if (unlikely(!__pyx_float_58_42)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_58_53 = PyFloat_FromDouble(58.53); if (unlikely(!__pyx_float_58_53)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_58_56 = PyFloat_FromDouble(58.56); if (unlikely(!__pyx_float_58_56)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_58_67 = PyFloat_FromDouble(58.67); if (unlikely(!__pyx_float_58_67)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_58_71 = PyFloat_FromDouble(58.71); if (unlikely(!__pyx_float_58_71)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_58_75 = PyFloat_FromDouble(58.75); if (unlikely(!__pyx_float_58_75)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_58_79 = PyFloat_FromDouble(58.79); if (unlikely(!__pyx_float_58_79)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_58_86 = PyFloat_FromDouble(58.86); if (unlikely(!__pyx_float_58_86)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_59_02 = PyFloat_FromDouble(59.02); if (unlikely(!__pyx_float_59_02)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_59_13 = PyFloat_FromDouble(59.13); if (unlikely(!__pyx_float_59_13)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_59_16 = PyFloat_FromDouble(59.16); if (unlikely(!__pyx_float_59_16)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_59_21 = PyFloat_FromDouble(59.21); if (unlikely(!__pyx_float_59_21)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_59_29 = PyFloat_FromDouble(59.29); if (unlikely(!__pyx_float_59_29)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_59_30 = PyFloat_FromDouble(59.30); if (unlikely(!__pyx_float_59_30)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_59_33 = PyFloat_FromDouble(59.33); if (unlikely(!__pyx_float_59_33)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_59_40 = PyFloat_FromDouble(59.40); if (unlikely(!__pyx_float_59_40)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_59_47 = PyFloat_FromDouble(59.47); if (unlikely(!__pyx_float_59_47)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_59_70 = PyFloat_FromDouble(59.70); if (unlikely(!__pyx_float_59_70)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_59_81 = PyFloat_FromDouble(59.81); if (unlikely(!__pyx_float_59_81)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_59_83 = PyFloat_FromDouble(59.83); if (unlikely(!__pyx_float_59_83)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_59_86 = PyFloat_FromDouble(59.86); if (unlikely(!__pyx_float_59_86)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_59_95 = PyFloat_FromDouble(59.95); if (unlikely(!__pyx_float_59_95)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_59_96 = PyFloat_FromDouble(59.96); if (unlikely(!__pyx_float_59_96)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_60_00 = PyFloat_FromDouble(60.00); if (unlikely(!__pyx_float_60_00)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_60_03 = PyFloat_FromDouble(60.03); if (unlikely(!__pyx_float_60_03)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_60_09 = PyFloat_FromDouble(60.09); if (unlikely(!__pyx_float_60_09)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_60_23 = PyFloat_FromDouble(60.23); if (unlikely(!__pyx_float_60_23)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_60_33 = PyFloat_FromDouble(60.33); if (unlikely(!__pyx_float_60_33)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_60_35 = PyFloat_FromDouble(60.35); if (unlikely(!__pyx_float_60_35)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_60_42 = PyFloat_FromDouble(60.42); if (unlikely(!__pyx_float_60_42)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_60_59 = PyFloat_FromDouble(60.59); if (unlikely(!__pyx_float_60_59)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_60_62 = PyFloat_FromDouble(60.62); if (unlikely(!__pyx_float_60_62)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_60_67 = PyFloat_FromDouble(60.67); if (unlikely(!__pyx_float_60_67)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_60_75 = PyFloat_FromDouble(60.75); if (unlikely(!__pyx_float_60_75)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_61_05 = PyFloat_FromDouble(61.05); if (unlikely(!__pyx_float_61_05)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_61_13 = PyFloat_FromDouble(61.13); if (unlikely(!__pyx_float_61_13)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_61_21 = PyFloat_FromDouble(61.21); if (unlikely(!__pyx_float_61_21)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_61_29 = PyFloat_FromDouble(61.29); if (unlikely(!__pyx_float_61_29)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_61_34 = PyFloat_FromDouble(61.34); if (unlikely(!__pyx_float_61_34)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_61_37 = PyFloat_FromDouble(61.37); if (unlikely(!__pyx_float_61_37)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_61_39 = PyFloat_FromDouble(61.39); if (unlikely(!__pyx_float_61_39)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_61_40 = PyFloat_FromDouble(61.40); if (unlikely(!__pyx_float_61_40)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_61_56 = PyFloat_FromDouble(61.56); if (unlikely(!__pyx_float_61_56)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_61_59 = PyFloat_FromDouble(61.59); if (unlikely(!__pyx_float_61_59)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_61_62 = PyFloat_FromDouble(61.62); if (unlikely(!__pyx_float_61_62)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_61_66 = PyFloat_FromDouble(61.66); if (unlikely(!__pyx_float_61_66)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_61_99 = PyFloat_FromDouble(61.99); if (unlikely(!__pyx_float_61_99)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_62_04 = PyFloat_FromDouble(62.04); if (unlikely(!__pyx_float_62_04)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_62_20 = PyFloat_FromDouble(62.20); if (unlikely(!__pyx_float_62_20)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_62_22 = PyFloat_FromDouble(62.22); if (unlikely(!__pyx_float_62_22)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_62_54 = PyFloat_FromDouble(62.54); if (unlikely(!__pyx_float_62_54)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_62_56 = PyFloat_FromDouble(62.56); if (unlikely(!__pyx_float_62_56)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_62_72 = PyFloat_FromDouble(62.72); if (unlikely(!__pyx_float_62_72)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_62_84 = PyFloat_FromDouble(62.84); if (unlikely(!__pyx_float_62_84)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_62_86 = PyFloat_FromDouble(62.86); if (unlikely(!__pyx_float_62_86)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_63_08 = PyFloat_FromDouble(63.08); if (unlikely(!__pyx_float_63_08)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_63_48 = PyFloat_FromDouble(63.48); if (unlikely(!__pyx_float_63_48)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_63_55 = PyFloat_FromDouble(63.55); if (unlikely(!__pyx_float_63_55)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_63_78 = PyFloat_FromDouble(63.78); if (unlikely(!__pyx_float_63_78)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_63_97 = PyFloat_FromDouble(63.97); if (unlikely(!__pyx_float_63_97)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_64_29 = PyFloat_FromDouble(64.29); if (unlikely(!__pyx_float_64_29)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_64_36 = PyFloat_FromDouble(64.36); if (unlikely(!__pyx_float_64_36)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_64_82 = PyFloat_FromDouble(64.82); if (unlikely(!__pyx_float_64_82)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_64_83 = PyFloat_FromDouble(64.83); if (unlikely(!__pyx_float_64_83)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_64_86 = PyFloat_FromDouble(64.86); if (unlikely(!__pyx_float_64_86)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_64_97 = PyFloat_FromDouble(64.97); if (unlikely(!__pyx_float_64_97)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_65_09 = PyFloat_FromDouble(65.09); if (unlikely(!__pyx_float_65_09)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_65_19 = PyFloat_FromDouble(65.19); if (unlikely(!__pyx_float_65_19)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_65_20 = PyFloat_FromDouble(65.20); if (unlikely(!__pyx_float_65_20)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_65_42 = PyFloat_FromDouble(65.42); if (unlikely(!__pyx_float_65_42)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_65_54 = PyFloat_FromDouble(65.54); if (unlikely(!__pyx_float_65_54)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_65_65 = PyFloat_FromDouble(65.65); if (unlikely(!__pyx_float_65_65)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_65_89 = PyFloat_FromDouble(65.89); if (unlikely(!__pyx_float_65_89)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_66_12 = PyFloat_FromDouble(66.12); if (unlikely(!__pyx_float_66_12)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_66_24 = PyFloat_FromDouble(66.24); if (unlikely(!__pyx_float_66_24)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_66_26 = PyFloat_FromDouble(66.26); if (unlikely(!__pyx_float_66_26)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_66_34 = PyFloat_FromDouble(66.34); if (unlikely(!__pyx_float_66_34)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_66_44 = PyFloat_FromDouble(66.44); if (unlikely(!__pyx_float_66_44)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_66_47 = PyFloat_FromDouble(66.47); if (unlikely(!__pyx_float_66_47)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_66_48 = PyFloat_FromDouble(66.48); if (unlikely(!__pyx_float_66_48)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_66_56 = PyFloat_FromDouble(66.56); if (unlikely(!__pyx_float_66_56)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_66_60 = PyFloat_FromDouble(66.60); if (unlikely(!__pyx_float_66_60)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_66_64 = PyFloat_FromDouble(66.64); if (unlikely(!__pyx_float_66_64)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_66_72 = PyFloat_FromDouble(66.72); if (unlikely(!__pyx_float_66_72)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_66_91 = PyFloat_FromDouble(66.91); if (unlikely(!__pyx_float_66_91)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_66_92 = PyFloat_FromDouble(66.92); if (unlikely(!__pyx_float_66_92)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_66_96 = PyFloat_FromDouble(66.96); if (unlikely(!__pyx_float_66_96)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_67_18 = PyFloat_FromDouble(67.18); if (unlikely(!__pyx_float_67_18)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_67_23 = PyFloat_FromDouble(67.23); if (unlikely(!__pyx_float_67_23)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_67_37 = PyFloat_FromDouble(67.37); if (unlikely(!__pyx_float_67_37)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_67_46 = PyFloat_FromDouble(67.46); if (unlikely(!__pyx_float_67_46)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_67_63 = PyFloat_FromDouble(67.63); if (unlikely(!__pyx_float_67_63)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_67_70 = PyFloat_FromDouble(67.70); if (unlikely(!__pyx_float_67_70)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_67_83 = PyFloat_FromDouble(67.83); if (unlikely(!__pyx_float_67_83)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_67_94 = PyFloat_FromDouble(67.94); if (unlikely(!__pyx_float_67_94)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_67_97 = PyFloat_FromDouble(67.97); if (unlikely(!__pyx_float_67_97)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_67_98 = PyFloat_FromDouble(67.98); if (unlikely(!__pyx_float_67_98)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_68_18 = PyFloat_FromDouble(68.18); if (unlikely(!__pyx_float_68_18)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_68_50 = PyFloat_FromDouble(68.50); if (unlikely(!__pyx_float_68_50)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_68_61 = PyFloat_FromDouble(68.61); if (unlikely(!__pyx_float_68_61)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_68_65 = PyFloat_FromDouble(68.65); if (unlikely(!__pyx_float_68_65)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_68_81 = PyFloat_FromDouble(68.81); if (unlikely(!__pyx_float_68_81)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_68_85 = PyFloat_FromDouble(68.85); if (unlikely(!__pyx_float_68_85)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_68_90 = PyFloat_FromDouble(68.90); if (unlikely(!__pyx_float_68_90)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_68_94 = PyFloat_FromDouble(68.94); if (unlikely(!__pyx_float_68_94)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_68_99 = PyFloat_FromDouble(68.99); if (unlikely(!__pyx_float_68_99)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_69_09 = PyFloat_FromDouble(69.09); if (unlikely(!__pyx_float_69_09)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_69_13 = PyFloat_FromDouble(69.13); if (unlikely(!__pyx_float_69_13)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_69_17 = PyFloat_FromDouble(69.17); if (unlikely(!__pyx_float_69_17)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_69_33 = PyFloat_FromDouble(69.33); if (unlikely(!__pyx_float_69_33)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_69_37 = PyFloat_FromDouble(69.37); if (unlikely(!__pyx_float_69_37)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_69_60 = PyFloat_FromDouble(69.60); if (unlikely(!__pyx_float_69_60)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_69_78 = PyFloat_FromDouble(69.78); if (unlikely(!__pyx_float_69_78)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_69_91 = PyFloat_FromDouble(69.91); if (unlikely(!__pyx_float_69_91)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_69_97 = PyFloat_FromDouble(69.97); if (unlikely(!__pyx_float_69_97)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_70_10 = PyFloat_FromDouble(70.10); if (unlikely(!__pyx_float_70_10)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_70_25 = PyFloat_FromDouble(70.25); if (unlikely(!__pyx_float_70_25)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_70_42 = PyFloat_FromDouble(70.42); if (unlikely(!__pyx_float_70_42)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_70_43 = PyFloat_FromDouble(70.43); if (unlikely(!__pyx_float_70_43)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_70_52 = PyFloat_FromDouble(70.52); if (unlikely(!__pyx_float_70_52)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_70_53 = PyFloat_FromDouble(70.53); if (unlikely(!__pyx_float_70_53)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_70_58 = PyFloat_FromDouble(70.58); if (unlikely(!__pyx_float_70_58)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_70_68 = PyFloat_FromDouble(70.68); if (unlikely(!__pyx_float_70_68)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_70_73 = PyFloat_FromDouble(70.73); if (unlikely(!__pyx_float_70_73)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_70_88 = PyFloat_FromDouble(70.88); if (unlikely(!__pyx_float_70_88)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_71_17 = PyFloat_FromDouble(71.17); if (unlikely(!__pyx_float_71_17)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_71_21 = PyFloat_FromDouble(71.21); if (unlikely(!__pyx_float_71_21)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_71_25 = PyFloat_FromDouble(71.25); if (unlikely(!__pyx_float_71_25)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_71_36 = PyFloat_FromDouble(71.36); if (unlikely(!__pyx_float_71_36)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_71_59 = PyFloat_FromDouble(71.59); if (unlikely(!__pyx_float_71_59)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_71_72 = PyFloat_FromDouble(71.72); if (unlikely(!__pyx_float_71_72)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_71_84 = PyFloat_FromDouble(71.84); if (unlikely(!__pyx_float_71_84)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_72_00 = PyFloat_FromDouble(72.00); if (unlikely(!__pyx_float_72_00)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_72_01 = PyFloat_FromDouble(72.01); if (unlikely(!__pyx_float_72_01)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_72_14 = PyFloat_FromDouble(72.14); if (unlikely(!__pyx_float_72_14)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_72_24 = PyFloat_FromDouble(72.24); if (unlikely(!__pyx_float_72_24)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_72_45 = PyFloat_FromDouble(72.45); if (unlikely(!__pyx_float_72_45)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_73_05 = PyFloat_FromDouble(73.05); if (unlikely(!__pyx_float_73_05)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_73_28 = PyFloat_FromDouble(73.28); if (unlikely(!__pyx_float_73_28)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_73_50 = PyFloat_FromDouble(73.50); if (unlikely(!__pyx_float_73_50)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_73_53 = PyFloat_FromDouble(73.53); if (unlikely(!__pyx_float_73_53)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_73_57 = PyFloat_FromDouble(73.57); if (unlikely(!__pyx_float_73_57)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_73_62 = PyFloat_FromDouble(73.62); if (unlikely(!__pyx_float_73_62)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_73_68 = PyFloat_FromDouble(73.68); if (unlikely(!__pyx_float_73_68)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_74_22 = PyFloat_FromDouble(74.22); if (unlikely(!__pyx_float_74_22)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_74_40 = PyFloat_FromDouble(74.40); if (unlikely(!__pyx_float_74_40)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_74_95 = PyFloat_FromDouble(74.95); if (unlikely(!__pyx_float_74_95)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_75_00 = PyFloat_FromDouble(75.00); if (unlikely(!__pyx_float_75_00)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_75_44 = PyFloat_FromDouble(75.44); if (unlikely(!__pyx_float_75_44)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_75_49 = PyFloat_FromDouble(75.49); if (unlikely(!__pyx_float_75_49)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_75_52 = PyFloat_FromDouble(75.52); if (unlikely(!__pyx_float_75_52)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_75_76 = PyFloat_FromDouble(75.76); if (unlikely(!__pyx_float_75_76)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_75_92 = PyFloat_FromDouble(75.92); if (unlikely(!__pyx_float_75_92)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_76_12 = PyFloat_FromDouble(76.12); if (unlikely(!__pyx_float_76_12)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_76_45 = PyFloat_FromDouble(76.45); if (unlikely(!__pyx_float_76_45)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_76_52 = PyFloat_FromDouble(76.52); if (unlikely(!__pyx_float_76_52)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_76_53 = PyFloat_FromDouble(76.53); if (unlikely(!__pyx_float_76_53)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_76_96 = PyFloat_FromDouble(76.96); if (unlikely(!__pyx_float_76_96)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_77_03 = PyFloat_FromDouble(77.03); if (unlikely(!__pyx_float_77_03)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_77_08 = PyFloat_FromDouble(77.08); if (unlikely(!__pyx_float_77_08)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_77_13 = PyFloat_FromDouble(77.13); if (unlikely(!__pyx_float_77_13)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_77_93 = PyFloat_FromDouble(77.93); if (unlikely(!__pyx_float_77_93)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_77_97 = PyFloat_FromDouble(77.97); if (unlikely(!__pyx_float_77_97)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_78_00 = PyFloat_FromDouble(78.00); if (unlikely(!__pyx_float_78_00)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_78_02 = PyFloat_FromDouble(78.02); if (unlikely(!__pyx_float_78_02)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_78_31 = PyFloat_FromDouble(78.31); if (unlikely(!__pyx_float_78_31)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_78_32 = PyFloat_FromDouble(78.32); if (unlikely(!__pyx_float_78_32)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_78_45 = PyFloat_FromDouble(78.45); if (unlikely(!__pyx_float_78_45)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_78_94 = PyFloat_FromDouble(78.94); if (unlikely(!__pyx_float_78_94)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_79_27 = PyFloat_FromDouble(79.27); if (unlikely(!__pyx_float_79_27)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_79_38 = PyFloat_FromDouble(79.38); if (unlikely(!__pyx_float_79_38)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_79_48 = PyFloat_FromDouble(79.48); if (unlikely(!__pyx_float_79_48)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_79_49 = PyFloat_FromDouble(79.49); if (unlikely(!__pyx_float_79_49)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_79_57 = PyFloat_FromDouble(79.57); if (unlikely(!__pyx_float_79_57)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_79_65 = PyFloat_FromDouble(79.65); if (unlikely(!__pyx_float_79_65)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_79_73 = PyFloat_FromDouble(79.73); if (unlikely(!__pyx_float_79_73)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_79_83 = PyFloat_FromDouble(79.83); if (unlikely(!__pyx_float_79_83)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_80_16 = PyFloat_FromDouble(80.16); if (unlikely(!__pyx_float_80_16)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_80_39 = PyFloat_FromDouble(80.39); if (unlikely(!__pyx_float_80_39)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_80_44 = PyFloat_FromDouble(80.44); if (unlikely(!__pyx_float_80_44)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_80_77 = PyFloat_FromDouble(80.77); if (unlikely(!__pyx_float_80_77)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_81_00 = PyFloat_FromDouble(81.00); if (unlikely(!__pyx_float_81_00)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_81_16 = PyFloat_FromDouble(81.16); if (unlikely(!__pyx_float_81_16)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_81_33 = PyFloat_FromDouble(81.33); if (unlikely(!__pyx_float_81_33)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_81_56 = PyFloat_FromDouble(81.56); if (unlikely(!__pyx_float_81_56)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_81_69 = PyFloat_FromDouble(81.69); if (unlikely(!__pyx_float_81_69)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_81_75 = PyFloat_FromDouble(81.75); if (unlikely(!__pyx_float_81_75)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_81_87 = PyFloat_FromDouble(81.87); if (unlikely(!__pyx_float_81_87)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_82_06 = PyFloat_FromDouble(82.06); if (unlikely(!__pyx_float_82_06)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_82_39 = PyFloat_FromDouble(82.39); if (unlikely(!__pyx_float_82_39)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_82_44 = PyFloat_FromDouble(82.44); if (unlikely(!__pyx_float_82_44)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_82_47 = PyFloat_FromDouble(82.47); if (unlikely(!__pyx_float_82_47)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_82_52 = PyFloat_FromDouble(82.52); if (unlikely(!__pyx_float_82_52)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_82_59 = PyFloat_FromDouble(82.59); if (unlikely(!__pyx_float_82_59)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_82_70 = PyFloat_FromDouble(82.70); if (unlikely(!__pyx_float_82_70)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_82_99 = PyFloat_FromDouble(82.99); if (unlikely(!__pyx_float_82_99)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_83_17 = PyFloat_FromDouble(83.17); if (unlikely(!__pyx_float_83_17)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_83_53 = PyFloat_FromDouble(83.53); if (unlikely(!__pyx_float_83_53)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_83_91 = PyFloat_FromDouble(83.91); if (unlikely(!__pyx_float_83_91)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_83_97 = PyFloat_FromDouble(83.97); if (unlikely(!__pyx_float_83_97)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_84_01 = PyFloat_FromDouble(84.01); if (unlikely(!__pyx_float_84_01)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_84_10 = PyFloat_FromDouble(84.10); if (unlikely(!__pyx_float_84_10)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_84_22 = PyFloat_FromDouble(84.22); if (unlikely(!__pyx_float_84_22)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_84_61 = PyFloat_FromDouble(84.61); if (unlikely(!__pyx_float_84_61)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_84_81 = PyFloat_FromDouble(84.81); if (unlikely(!__pyx_float_84_81)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_85_28 = PyFloat_FromDouble(85.28); if (unlikely(!__pyx_float_85_28)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_85_41 = PyFloat_FromDouble(85.41); if (unlikely(!__pyx_float_85_41)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_85_42 = PyFloat_FromDouble(85.42); if (unlikely(!__pyx_float_85_42)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_85_97 = PyFloat_FromDouble(85.97); if (unlikely(!__pyx_float_85_97)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_86_88 = PyFloat_FromDouble(86.88); if (unlikely(!__pyx_float_86_88)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_87_27 = PyFloat_FromDouble(87.27); if (unlikely(!__pyx_float_87_27)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_87_46 = PyFloat_FromDouble(87.46); if (unlikely(!__pyx_float_87_46)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_87_56 = PyFloat_FromDouble(87.56); if (unlikely(!__pyx_float_87_56)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_87_65 = PyFloat_FromDouble(87.65); if (unlikely(!__pyx_float_87_65)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_88_36 = PyFloat_FromDouble(88.36); if (unlikely(!__pyx_float_88_36)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_88_39 = PyFloat_FromDouble(88.39); if (unlikely(!__pyx_float_88_39)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_89_08 = PyFloat_FromDouble(89.08); if (unlikely(!__pyx_float_89_08)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_89_85 = PyFloat_FromDouble(89.85); if (unlikely(!__pyx_float_89_85)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_90_29 = PyFloat_FromDouble(90.29); if (unlikely(!__pyx_float_90_29)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_90_30 = PyFloat_FromDouble(90.30); if (unlikely(!__pyx_float_90_30)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_90_55 = PyFloat_FromDouble(90.55); if (unlikely(!__pyx_float_90_55)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_90_70 = PyFloat_FromDouble(90.70); if (unlikely(!__pyx_float_90_70)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_91_35 = PyFloat_FromDouble(91.35); if (unlikely(!__pyx_float_91_35)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_91_75 = PyFloat_FromDouble(91.75); if (unlikely(!__pyx_float_91_75)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_92_09 = PyFloat_FromDouble(92.09); if (unlikely(!__pyx_float_92_09)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_92_30 = PyFloat_FromDouble(92.30); if (unlikely(!__pyx_float_92_30)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_92_86 = PyFloat_FromDouble(92.86); if (unlikely(!__pyx_float_92_86)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_93_28 = PyFloat_FromDouble(93.28); if (unlikely(!__pyx_float_93_28)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_93_40 = PyFloat_FromDouble(93.40); if (unlikely(!__pyx_float_93_40)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_93_52 = PyFloat_FromDouble(93.52); if (unlikely(!__pyx_float_93_52)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_94_02 = PyFloat_FromDouble(94.02); if (unlikely(!__pyx_float_94_02)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_95_26 = PyFloat_FromDouble(95.26); if (unlikely(!__pyx_float_95_26)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_95_59 = PyFloat_FromDouble(95.59); if (unlikely(!__pyx_float_95_59)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_97_07 = PyFloat_FromDouble(97.07); if (unlikely(!__pyx_float_97_07)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_97_22 = PyFloat_FromDouble(97.22); if (unlikely(!__pyx_float_97_22)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_98_86 = PyFloat_FromDouble(98.86); if (unlikely(!__pyx_float_98_86)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_105_28 = PyFloat_FromDouble(105.28); if (unlikely(!__pyx_float_105_28)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_106_22 = PyFloat_FromDouble(106.22); if (unlikely(!__pyx_float_106_22)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_107_45 = PyFloat_FromDouble(107.45); if (unlikely(!__pyx_float_107_45)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_108_37 = PyFloat_FromDouble(108.37); if (unlikely(!__pyx_float_108_37)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_108_70 = PyFloat_FromDouble(108.70); if (unlikely(!__pyx_float_108_70)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_109_39 = PyFloat_FromDouble(109.39); if (unlikely(!__pyx_float_109_39)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_122_08 = PyFloat_FromDouble(122.08); if (unlikely(!__pyx_float_122_08)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_125_54 = PyFloat_FromDouble(125.54); if (unlikely(!__pyx_float_125_54)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_32_703 = PyFloat_FromDouble(32.703); if (unlikely(!__pyx_float_32_703)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_10000_0 = PyFloat_FromDouble(10000.0); if (unlikely(!__pyx_float_10000_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_2 = PyInt_FromLong(2); if (unlikely(!__pyx_int_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_4 = PyInt_FromLong(4); if (unlikely(!__pyx_int_4)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_12 = PyInt_FromLong(12); if (unlikely(!__pyx_int_12)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_45 = PyInt_FromLong(45); if (unlikely(!__pyx_int_45)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_100 = PyInt_FromLong(100); if (unlikely(!__pyx_int_100)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_220 = PyInt_FromLong(220); if (unlikely(!__pyx_int_220)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_261 = PyInt_FromLong(261); if (unlikely(!__pyx_int_261)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_440 = PyInt_FromLong(440); if (unlikely(!__pyx_int_440)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: init_globals ### */

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  return 0;
}
/* #### Code section: init_module ### */

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType_3_0_8(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type", 
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyTypeObject),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(PyTypeObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_7cpython_4type_type) __PYX_ERR(2, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("numpy"); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 202, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_5numpy_dtype = __Pyx_ImportType_3_0_8(__pyx_t_1, "numpy", "dtype", sizeof(PyArray_Descr), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyArray_Descr),__Pyx_ImportType_CheckSize_Ignore_3_0_8); if (!__pyx_ptype_5numpy_dtype) __PYX_ERR(1, 202, __pyx_L1_error)
  __pyx_ptype_5numpy_flatiter = __Pyx_ImportType_3_0_8(__pyx_t_1, "numpy", "flatiter", sizeof(PyArrayIterObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyArrayIterObject),__Pyx_ImportType_CheckSize_Ignore_3_0_8); if (!__pyx_ptype_5numpy_flatiter) __PYX_ERR(1, 225, __pyx_L1_error)
  __pyx_ptype_5numpy_broadcast = __Pyx_ImportType_3_0_8(__pyx_t_1, "numpy", "broadcast", sizeof(PyArrayMultiIterObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyArrayMultiIterObject),__Pyx_ImportType_CheckSize_Ignore_3_0_8); if (!__pyx_ptype_5numpy_broadcast) __PYX_ERR(1, 229, __pyx_L1_error)
  __pyx_ptype_5numpy_ndarray = __Pyx_ImportType_3_0_8(__pyx_t_1, "numpy", "ndarray", sizeof(PyArrayObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyArrayObject),__Pyx_ImportType_CheckSize_Ignore_3_0_8); if (!__pyx_ptype_5numpy_ndarray) __PYX_ERR(1, 238, __pyx_L1_error)
  __pyx_ptype_5numpy_generic = __Pyx_ImportType_3_0_8(__pyx_t_1, "numpy", "generic", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyObject),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_5numpy_generic) __PYX_ERR(1, 809, __pyx_L1_error)
  __pyx_ptype_5numpy_number = __Pyx_ImportType_3_0_8(__pyx_t_1, "numpy", "number", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyObject),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_5numpy_number) __PYX_ERR(1, 811, __pyx_L1_error)
  __pyx_ptype_5numpy_integer = __Pyx_ImportType_3_0_8(__pyx_t_1, "numpy", "integer", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyObject),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_5numpy_integer) __PYX_ERR(1, 813, __pyx_L1_error)
  __pyx_ptype_5numpy_signedinteger = __Pyx_ImportType_3_0_8(__pyx_t_1, "numpy", "signedinteger", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyObject),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_5numpy_signedinteger) __PYX_ERR(1, 815, __pyx_L1_error)
  __pyx_ptype_5numpy_unsignedinteger = __Pyx_ImportType_3_0_8(__pyx_t_1, "numpy", "unsignedinteger", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyObject),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_5numpy_unsignedinteger) __PYX_ERR(1, 817, __pyx_L1_error)
  __pyx_ptype_5numpy_inexact = __Pyx_ImportType_3_0_8(__pyx_t_1, "numpy", "inexact", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyObject),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_5numpy_inexact) __PYX_ERR(1, 819, __pyx_L1_error)
  __pyx_ptype_5numpy_floating = __Pyx_ImportType_3_0_8(__pyx_t_1, "numpy", "floating", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyObject),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_5numpy_floating) __PYX_ERR(1, 821, __pyx_L1_error)
  __pyx_ptype_5numpy_complexfloating = __Pyx_ImportType_3_0_8(__pyx_t_1, "numpy", "complexfloating", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyObject),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_5numpy_complexfloating) __PYX_ERR(1, 823, __pyx_L1_error)
  __pyx_ptype_5numpy_flexible = __Pyx_ImportType_3_0_8(__pyx_t_1, "numpy", "flexible", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyObject),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_5numpy_flexible) __PYX_ERR(1, 825, __pyx_L1_error)
  __pyx_ptype_5numpy_character = __Pyx_ImportType_3_0_8(__pyx_t_1, "numpy", "character", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyObject),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_5numpy_character) __PYX_ERR(1, 827, __pyx_L1_error)
  __pyx_ptype_5numpy_ufunc = __Pyx_ImportType_3_0_8(__pyx_t_1, "numpy", "ufunc", sizeof(PyUFuncObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyUFuncObject),__Pyx_ImportType_CheckSize_Ignore_3_0_8); if (!__pyx_ptype_5numpy_ufunc) __PYX_ERR(1, 866, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("pysndlib.clm"); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_8pysndlib_3clm_mus_any = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.clm", "mus_any", sizeof(struct __pyx_obj_8pysndlib_3clm_mus_any), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_3clm_mus_any),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_3clm_mus_any) __PYX_ERR(3, 40, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_3clm_mus_any = (struct __pyx_vtabstruct_8pysndlib_3clm_mus_any*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_3clm_mus_any); if (unlikely(!__pyx_vtabptr_8pysndlib_3clm_mus_any)) __PYX_ERR(3, 40, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __pyx_t_1 = PyImport_ImportModule("pysndlib.clm"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportFunction_3_0_8(__pyx_t_1, "hz2radians", (void (**)(void))&__pyx_f_8pysndlib_3clm_hz2radians, "double (double, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction_3_0_8(__pyx_t_1, "db2linear", (void (**)(void))&__pyx_f_8pysndlib_3clm_db2linear, "double (double, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction_3_0_8(__pyx_t_1, "seconds2samples", (void (**)(void))&__pyx_f_8pysndlib_3clm_seconds2samples, "long (double, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction_3_0_8(__pyx_t_1, "normalize_partials", (void (**)(void))&__pyx_f_8pysndlib_3clm_normalize_partials, "PyArrayObject *(PyObject *, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction_3_0_8(__pyx_t_1, "oscil_bank", (void (**)(void))&__pyx_f_8pysndlib_3clm_oscil_bank, "double (struct __pyx_obj_8pysndlib_3clm_mus_any *, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction_3_0_8(__pyx_t_1, "env", (void (**)(void))&__pyx_f_8pysndlib_3clm_env, "double (struct __pyx_obj_8pysndlib_3clm_mus_any *, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction_3_0_8(__pyx_t_1, "locsig", (void (**)(void))&__pyx_f_8pysndlib_3clm_locsig, "double (struct __pyx_obj_8pysndlib_3clm_mus_any *, long, double, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}


#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_lbj_piano(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_lbj_piano},
  {0, NULL}
};
#endif

#ifdef __cplusplus
namespace {
  struct PyModuleDef __pyx_moduledef =
  #else
  static struct PyModuleDef __pyx_moduledef =
  #endif
  {
      PyModuleDef_HEAD_INIT,
      "lbj_piano",
      0, /* m_doc */
    #if CYTHON_PEP489_MULTI_PHASE_INIT
      0, /* m_size */
    #elif CYTHON_USE_MODULE_STATE
      sizeof(__pyx_mstate), /* m_size */
    #else
      -1, /* m_size */
    #endif
      __pyx_methods /* m_methods */,
    #if CYTHON_PEP489_MULTI_PHASE_INIT
      __pyx_moduledef_slots, /* m_slots */
    #else
      NULL, /* m_reload */
    #endif
    #if CYTHON_USE_MODULE_STATE
      __pyx_m_traverse, /* m_traverse */
      __pyx_m_clear, /* m_clear */
      NULL /* m_free */
    #else
      NULL, /* m_traverse */
      NULL, /* m_clear */
      NULL /* m_free */
    #endif
  };
  #ifdef __cplusplus
} /* anonymous namespace */
#endif
#endif

#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initlbj_piano(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initlbj_piano(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_lbj_piano(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_lbj_piano(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *module, const char* from_name, const char* to_name, int allow_none)
#else
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none)
#endif
{
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
#if CYTHON_COMPILING_IN_LIMITED_API
            result = PyModule_AddObject(module, to_name, value);
#else
            result = PyDict_SetItemString(moddict, to_name, value);
#endif
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    CYTHON_UNUSED_VAR(def);
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
#if CYTHON_COMPILING_IN_LIMITED_API
    moddict = module;
#else
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
#endif
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_lbj_piano(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  int stringtab_initialized = 0;
  #if CYTHON_USE_MODULE_STATE
  int pystate_addmodule_run = 0;
  #endif
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  PyObject *__pyx_t_22 = NULL;
  PyObject *__pyx_t_23 = NULL;
  PyObject *__pyx_t_24 = NULL;
  PyObject *__pyx_t_25 = NULL;
  PyObject *__pyx_t_26 = NULL;
  PyObject *__pyx_t_27 = NULL;
  PyObject *__pyx_t_28 = NULL;
  PyObject *__pyx_t_29 = NULL;
  PyObject *__pyx_t_30 = NULL;
  PyObject *__pyx_t_31 = NULL;
  PyObject *__pyx_t_32 = NULL;
  PyObject *__pyx_t_33 = NULL;
  PyObject *__pyx_t_34 = NULL;
  PyObject *__pyx_t_35 = NULL;
  PyObject *__pyx_t_36 = NULL;
  PyObject *__pyx_t_37 = NULL;
  PyObject *__pyx_t_38 = NULL;
  PyObject *__pyx_t_39 = NULL;
  PyObject *__pyx_t_40 = NULL;
  PyObject *__pyx_t_41 = NULL;
  PyObject *__pyx_t_42 = NULL;
  PyObject *__pyx_t_43 = NULL;
  PyObject *__pyx_t_44 = NULL;
  PyObject *__pyx_t_45 = NULL;
  PyObject *__pyx_t_46 = NULL;
  PyObject *__pyx_t_47 = NULL;
  PyObject *__pyx_t_48 = NULL;
  PyObject *__pyx_t_49 = NULL;
  PyObject *__pyx_t_50 = NULL;
  PyObject *__pyx_t_51 = NULL;
  PyObject *__pyx_t_52 = NULL;
  PyObject *__pyx_t_53 = NULL;
  PyObject *__pyx_t_54 = NULL;
  PyObject *__pyx_t_55 = NULL;
  PyObject *__pyx_t_56 = NULL;
  PyObject *__pyx_t_57 = NULL;
  PyObject *__pyx_t_58 = NULL;
  PyObject *__pyx_t_59 = NULL;
  PyObject *__pyx_t_60 = NULL;
  PyObject *__pyx_t_61 = NULL;
  PyObject *__pyx_t_62 = NULL;
  PyObject *__pyx_t_63 = NULL;
  PyObject *__pyx_t_64 = NULL;
  PyObject *__pyx_t_65 = NULL;
  PyObject *__pyx_t_66 = NULL;
  PyObject *__pyx_t_67 = NULL;
  PyObject *__pyx_t_68 = NULL;
  PyObject *__pyx_t_69 = NULL;
  PyObject *__pyx_t_70 = NULL;
  PyObject *__pyx_t_71 = NULL;
  PyObject *__pyx_t_72 = NULL;
  PyObject *__pyx_t_73 = NULL;
  PyObject *__pyx_t_74 = NULL;
  PyObject *__pyx_t_75 = NULL;
  PyObject *__pyx_t_76 = NULL;
  PyObject *__pyx_t_77 = NULL;
  PyObject *__pyx_t_78 = NULL;
  PyObject *__pyx_t_79 = NULL;
  PyObject *__pyx_t_80 = NULL;
  PyObject *__pyx_t_81 = NULL;
  PyObject *__pyx_t_82 = NULL;
  int __pyx_t_83;
  PyObject *__pyx_t_84 = NULL;
  PyObject *__pyx_t_85 = NULL;
  PyObject *__pyx_t_86 = NULL;
  PyObject *__pyx_t_87 = NULL;
  int __pyx_t_88;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'lbj_piano' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("lbj_piano", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #elif CYTHON_USE_MODULE_STATE
  __pyx_t_1 = PyModule_Create(&__pyx_moduledef); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  {
    int add_module_result = PyState_AddModule(__pyx_t_1, &__pyx_moduledef);
    __pyx_t_1 = 0; /* transfer ownership from __pyx_t_1 to "lbj_piano" pseudovariable */
    if (unlikely((add_module_result < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
    pystate_addmodule_run = 1;
  }
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #endif
  CYTHON_UNUSED_VAR(__pyx_t_1);
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = __Pyx_PyImport_AddModuleRef(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_cython_runtime = __Pyx_PyImport_AddModuleRef((const char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_lbj_piano(void)", 0);
  if (__Pyx_check_binary_version(__PYX_LIMITED_VERSION_HEX, __Pyx_get_runtime_version(), CYTHON_COMPILING_IN_LIMITED_API) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(WITH_THREAD) && PY_VERSION_HEX < 0x030700F0 && defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  PyEval_InitThreads();
  #endif
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  stringtab_initialized = 1;
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_pysndlib__instruments__lbj_piano) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "pysndlib.instruments.lbj_piano")) {
      if (unlikely((PyDict_SetItemString(modules, "pysndlib.instruments.lbj_piano", __pyx_m) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  (void)__Pyx_modinit_type_init_code();
  if (unlikely((__Pyx_modinit_type_import_code() < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_variable_import_code();
  if (unlikely((__Pyx_modinit_function_import_code() < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "pysndlib/instruments/lbj_piano.py":6
 * #==================================================================================
 * 
 * import math             # <<<<<<<<<<<<<<
 * import random
 * import cython
 */
  __pyx_t_2 = __Pyx_ImportDottedModule(__pyx_n_s_math, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_math, __pyx_t_2) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pysndlib/instruments/lbj_piano.py":7
 * 
 * import math
 * import random             # <<<<<<<<<<<<<<
 * import cython
 * import numpy as np
 */
  __pyx_t_2 = __Pyx_ImportDottedModule(__pyx_n_s_random, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_random, __pyx_t_2) < 0) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pysndlib/instruments/lbj_piano.py":9
 * import random
 * import cython
 * import numpy as np             # <<<<<<<<<<<<<<
 * import pysndlib.clm as clm
 * if cython.compiled:
 */
  __pyx_t_2 = __Pyx_ImportDottedModule(__pyx_n_s_numpy, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_np, __pyx_t_2) < 0) __PYX_ERR(0, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pysndlib/instruments/lbj_piano.py":10
 * import cython
 * import numpy as np
 * import pysndlib.clm as clm             # <<<<<<<<<<<<<<
 * if cython.compiled:
 *     from cython.cimports.pysndlib import clm
 */
  __pyx_t_2 = __Pyx_ImportDottedModule(__pyx_n_s_pysndlib_clm, __pyx_tuple__6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_clm, __pyx_t_2) < 0) __PYX_ERR(0, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pysndlib/instruments/lbj_piano.py":28
 * 
 * # moved this outside function definition
 * PIANO_SPECTRA = [[1.97,.0326,2.99,.0086,3.95,.0163,4.97,.0178,5.98,.0177,6.95,.0315,8.02,.0001 ,8.94,.0076,9.96,.0134,10.99,.0284,11.98,.0229,13.02,.0229,13.89,.0010,15.06,.0090,16.00,.0003 ,17.08,.0078,18.16,.0064,19.18,.0129,20.21,.0085,21.27,.0225,22.32,.0061,23.41,.0102,24.48,.0005 ,25.56,.0016,26.64,.0018,27.70,.0113,28.80,.0111,29.91,.0158,31.06,.0093,32.17,.0017,33.32,.0002 ,34.42,.0018,35.59,.0027,36.74,.0055,37.90,.0037,39.06,.0064,40.25,.0033,41.47,.0014,42.53,.0004 ,43.89,.0010,45.12,.0039,46.33,.0039,47.64,.0009,48.88,.0016,50.13,.0006,51.37,.0010,52.70,.0002 ,54.00,.0004,55.30,.0008,56.60,.0025,57.96,.0010,59.30,.0012,60.67,.0011,61.99,.0003,62.86,.0001 ,64.36,.0005,64.86,.0001,66.26,.0004,67.70,.0006,68.94,.0002,70.10,.0001,70.58,.0002,72.01,.0007 ,73.53,.0006,75.00,.0002,77.03,.0005,78.00,.0002,79.57,.0006,81.16,.0005,82.70,.0005,84.22,.0003 ,85.41,.0002,87.46,.0001,90.30,.0001,94.02,.0001,95.26,.0002,109.39,.0003],             # <<<<<<<<<<<<<<
 * 			        [1.98,.0194,2.99,.0210,3.97,.0276,4.96,.0297,5.96,.0158,6.99,.0207,8.01,.0009 ,9.00,.0101,10.00,.0297,11.01,.0289,12.02,.0211,13.04,.0127,14.07,.0061,15.08,.0174,16.13,.0009 ,17.12,.0093,18.16,.0117,19.21,.0122,20.29,.0108,21.30,.0077,22.38,.0132,23.46,.0073,24.14,.0002 ,25.58,.0026,26.69,.0035,27.77,.0053,28.88,.0024,30.08,.0027,31.13,.0075,32.24,.0027,33.36,.0004 ,34.42,.0004,35.64,.0019,36.78,.0037,38.10,.0009,39.11,.0027,40.32,.0010,41.51,.0013,42.66,.0019 ,43.87,.0007,45.13,.0017,46.35,.0019,47.65,.0021,48.89,.0014,50.18,.0023,51.42,.0015,52.73,.0002 ,54.00,.0005,55.34,.0006,56.60,.0010,57.96,.0016,58.86,.0005,59.30,.0004,60.75,.0005,62.22,.0003 ,63.55,.0005,64.82,.0003,66.24,.0003,67.63,.0011,69.09,.0007,70.52,.0004,72.00,.0005,73.50,.0008 ,74.95,.0003,77.13,.0013,78.02,.0002,79.48,.0004,82.59,.0004,84.10,.0003],
 * 			        [2.00,.0313,2.99,.0109,4.00,.0215,5.00,.0242,5.98,.0355,7.01,.0132,8.01,.0009 ,9.01,.0071,10.00,.0258,11.03,.0221,12.02,.0056,13.06,.0196,14.05,.0160,15.11,.0107,16.11,.0003 ,17.14,.0111,18.21,.0085,19.23,.0010,20.28,.0048,21.31,.0128,22.36,.0051,23.41,.0041,24.05,.0006 ,25.54,.0019,26.62,.0028,27.72,.0034,28.82,.0062,29.89,.0039,30.98,.0058,32.08,.0011,33.21,.0002 ,34.37,.0008,35.46,.0018,36.62,.0036,37.77,.0018,38.92,.0042,40.07,.0037,41.23,.0011,42.67,.0003 ,43.65,.0018,44.68,.0025,45.99,.0044,47.21,.0051,48.40,.0044,49.67,.0005,50.88,.0019,52.15,.0003 ,53.42,.0008,54.69,.0010,55.98,.0005,57.26,.0013,58.53,.0027,59.83,.0011,61.21,.0027,62.54,.0003 ,63.78,.0003,65.20,.0001,66.60,.0006,67.98,.0008,69.37,.0019,70.73,.0007,72.14,.0004,73.62,.0002 ,74.40,.0003,76.52,.0006,77.97,.0002,79.49,.0004,80.77,.0003,81.00,.0001,82.47,.0005,83.97,.0001 ,87.27,.0002],
 */
  __pyx_t_2 = PyList_New(154); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_float_1_97);
  __Pyx_GIVEREF(__pyx_float_1_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 0, __pyx_float_1_97)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0326);
  __Pyx_GIVEREF(__pyx_float__0326);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 1, __pyx_float__0326)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_99);
  __Pyx_GIVEREF(__pyx_float_2_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 2, __pyx_float_2_99)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0086);
  __Pyx_GIVEREF(__pyx_float__0086);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 3, __pyx_float__0086)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_95);
  __Pyx_GIVEREF(__pyx_float_3_95);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 4, __pyx_float_3_95)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0163);
  __Pyx_GIVEREF(__pyx_float__0163);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 5, __pyx_float__0163)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_97);
  __Pyx_GIVEREF(__pyx_float_4_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 6, __pyx_float_4_97)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0178);
  __Pyx_GIVEREF(__pyx_float__0178);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 7, __pyx_float__0178)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_98);
  __Pyx_GIVEREF(__pyx_float_5_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 8, __pyx_float_5_98)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0177);
  __Pyx_GIVEREF(__pyx_float__0177);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 9, __pyx_float__0177)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_95);
  __Pyx_GIVEREF(__pyx_float_6_95);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 10, __pyx_float_6_95)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0315);
  __Pyx_GIVEREF(__pyx_float__0315);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 11, __pyx_float__0315)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_02);
  __Pyx_GIVEREF(__pyx_float_8_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 12, __pyx_float_8_02)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 13, __pyx_float__0001)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_94);
  __Pyx_GIVEREF(__pyx_float_8_94);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 14, __pyx_float_8_94)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0076);
  __Pyx_GIVEREF(__pyx_float__0076);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 15, __pyx_float__0076)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_96);
  __Pyx_GIVEREF(__pyx_float_9_96);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 16, __pyx_float_9_96)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0134);
  __Pyx_GIVEREF(__pyx_float__0134);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 17, __pyx_float__0134)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_99);
  __Pyx_GIVEREF(__pyx_float_10_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 18, __pyx_float_10_99)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0284);
  __Pyx_GIVEREF(__pyx_float__0284);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 19, __pyx_float__0284)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_98);
  __Pyx_GIVEREF(__pyx_float_11_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 20, __pyx_float_11_98)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0229);
  __Pyx_GIVEREF(__pyx_float__0229);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 21, __pyx_float__0229)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_02);
  __Pyx_GIVEREF(__pyx_float_13_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 22, __pyx_float_13_02)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0229);
  __Pyx_GIVEREF(__pyx_float__0229);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 23, __pyx_float__0229)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_89);
  __Pyx_GIVEREF(__pyx_float_13_89);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 24, __pyx_float_13_89)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 25, __pyx_float__0010)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_06);
  __Pyx_GIVEREF(__pyx_float_15_06);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 26, __pyx_float_15_06)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0090);
  __Pyx_GIVEREF(__pyx_float__0090);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 27, __pyx_float__0090)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_00);
  __Pyx_GIVEREF(__pyx_float_16_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 28, __pyx_float_16_00)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 29, __pyx_float__0003)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_08);
  __Pyx_GIVEREF(__pyx_float_17_08);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 30, __pyx_float_17_08)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0078);
  __Pyx_GIVEREF(__pyx_float__0078);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 31, __pyx_float__0078)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_16);
  __Pyx_GIVEREF(__pyx_float_18_16);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 32, __pyx_float_18_16)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0064);
  __Pyx_GIVEREF(__pyx_float__0064);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 33, __pyx_float__0064)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_18);
  __Pyx_GIVEREF(__pyx_float_19_18);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 34, __pyx_float_19_18)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0129);
  __Pyx_GIVEREF(__pyx_float__0129);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 35, __pyx_float__0129)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_21);
  __Pyx_GIVEREF(__pyx_float_20_21);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 36, __pyx_float_20_21)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0085);
  __Pyx_GIVEREF(__pyx_float__0085);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 37, __pyx_float__0085)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_27);
  __Pyx_GIVEREF(__pyx_float_21_27);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 38, __pyx_float_21_27)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0225);
  __Pyx_GIVEREF(__pyx_float__0225);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 39, __pyx_float__0225)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_32);
  __Pyx_GIVEREF(__pyx_float_22_32);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 40, __pyx_float_22_32)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0061);
  __Pyx_GIVEREF(__pyx_float__0061);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 41, __pyx_float__0061)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_41);
  __Pyx_GIVEREF(__pyx_float_23_41);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 42, __pyx_float_23_41)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0102);
  __Pyx_GIVEREF(__pyx_float__0102);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 43, __pyx_float__0102)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_24_48);
  __Pyx_GIVEREF(__pyx_float_24_48);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 44, __pyx_float_24_48)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 45, __pyx_float__0005)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_25_56);
  __Pyx_GIVEREF(__pyx_float_25_56);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 46, __pyx_float_25_56)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0016);
  __Pyx_GIVEREF(__pyx_float__0016);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 47, __pyx_float__0016)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_26_64);
  __Pyx_GIVEREF(__pyx_float_26_64);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 48, __pyx_float_26_64)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0018);
  __Pyx_GIVEREF(__pyx_float__0018);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 49, __pyx_float__0018)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_27_70);
  __Pyx_GIVEREF(__pyx_float_27_70);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 50, __pyx_float_27_70)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0113);
  __Pyx_GIVEREF(__pyx_float__0113);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 51, __pyx_float__0113)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_80);
  __Pyx_GIVEREF(__pyx_float_28_80);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 52, __pyx_float_28_80)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0111);
  __Pyx_GIVEREF(__pyx_float__0111);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 53, __pyx_float__0111)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_29_91);
  __Pyx_GIVEREF(__pyx_float_29_91);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 54, __pyx_float_29_91)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0158);
  __Pyx_GIVEREF(__pyx_float__0158);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 55, __pyx_float__0158)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_31_06);
  __Pyx_GIVEREF(__pyx_float_31_06);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 56, __pyx_float_31_06)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0093);
  __Pyx_GIVEREF(__pyx_float__0093);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 57, __pyx_float__0093)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_32_17);
  __Pyx_GIVEREF(__pyx_float_32_17);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 58, __pyx_float_32_17)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0017);
  __Pyx_GIVEREF(__pyx_float__0017);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 59, __pyx_float__0017)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_33_32);
  __Pyx_GIVEREF(__pyx_float_33_32);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 60, __pyx_float_33_32)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 61, __pyx_float__0002)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_34_42);
  __Pyx_GIVEREF(__pyx_float_34_42);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 62, __pyx_float_34_42)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0018);
  __Pyx_GIVEREF(__pyx_float__0018);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 63, __pyx_float__0018)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_35_59);
  __Pyx_GIVEREF(__pyx_float_35_59);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 64, __pyx_float_35_59)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0027);
  __Pyx_GIVEREF(__pyx_float__0027);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 65, __pyx_float__0027)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_36_74);
  __Pyx_GIVEREF(__pyx_float_36_74);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 66, __pyx_float_36_74)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0055);
  __Pyx_GIVEREF(__pyx_float__0055);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 67, __pyx_float__0055)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_37_90);
  __Pyx_GIVEREF(__pyx_float_37_90);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 68, __pyx_float_37_90)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0037);
  __Pyx_GIVEREF(__pyx_float__0037);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 69, __pyx_float__0037)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_39_06);
  __Pyx_GIVEREF(__pyx_float_39_06);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 70, __pyx_float_39_06)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0064);
  __Pyx_GIVEREF(__pyx_float__0064);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 71, __pyx_float__0064)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_40_25);
  __Pyx_GIVEREF(__pyx_float_40_25);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 72, __pyx_float_40_25)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0033);
  __Pyx_GIVEREF(__pyx_float__0033);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 73, __pyx_float__0033)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_41_47);
  __Pyx_GIVEREF(__pyx_float_41_47);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 74, __pyx_float_41_47)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0014);
  __Pyx_GIVEREF(__pyx_float__0014);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 75, __pyx_float__0014)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_42_53);
  __Pyx_GIVEREF(__pyx_float_42_53);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 76, __pyx_float_42_53)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 77, __pyx_float__0004)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_43_89);
  __Pyx_GIVEREF(__pyx_float_43_89);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 78, __pyx_float_43_89)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 79, __pyx_float__0010)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_45_12);
  __Pyx_GIVEREF(__pyx_float_45_12);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 80, __pyx_float_45_12)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0039);
  __Pyx_GIVEREF(__pyx_float__0039);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 81, __pyx_float__0039)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_46_33);
  __Pyx_GIVEREF(__pyx_float_46_33);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 82, __pyx_float_46_33)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0039);
  __Pyx_GIVEREF(__pyx_float__0039);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 83, __pyx_float__0039)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_47_64);
  __Pyx_GIVEREF(__pyx_float_47_64);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 84, __pyx_float_47_64)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 85, __pyx_float__0009)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_48_88);
  __Pyx_GIVEREF(__pyx_float_48_88);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 86, __pyx_float_48_88)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0016);
  __Pyx_GIVEREF(__pyx_float__0016);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 87, __pyx_float__0016)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_50_13);
  __Pyx_GIVEREF(__pyx_float_50_13);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 88, __pyx_float_50_13)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 89, __pyx_float__0006)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_51_37);
  __Pyx_GIVEREF(__pyx_float_51_37);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 90, __pyx_float_51_37)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 91, __pyx_float__0010)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_52_70);
  __Pyx_GIVEREF(__pyx_float_52_70);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 92, __pyx_float_52_70)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 93, __pyx_float__0002)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_54_00);
  __Pyx_GIVEREF(__pyx_float_54_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 94, __pyx_float_54_00)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 95, __pyx_float__0004)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_55_30);
  __Pyx_GIVEREF(__pyx_float_55_30);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 96, __pyx_float_55_30)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 97, __pyx_float__0008)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_56_60);
  __Pyx_GIVEREF(__pyx_float_56_60);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 98, __pyx_float_56_60)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0025);
  __Pyx_GIVEREF(__pyx_float__0025);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 99, __pyx_float__0025)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_57_96);
  __Pyx_GIVEREF(__pyx_float_57_96);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 100, __pyx_float_57_96)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 101, __pyx_float__0010)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_59_30);
  __Pyx_GIVEREF(__pyx_float_59_30);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 102, __pyx_float_59_30)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 103, __pyx_float__0012)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_60_67);
  __Pyx_GIVEREF(__pyx_float_60_67);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 104, __pyx_float_60_67)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 105, __pyx_float__0011)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_61_99);
  __Pyx_GIVEREF(__pyx_float_61_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 106, __pyx_float_61_99)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 107, __pyx_float__0003)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_62_86);
  __Pyx_GIVEREF(__pyx_float_62_86);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 108, __pyx_float_62_86)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 109, __pyx_float__0001)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_64_36);
  __Pyx_GIVEREF(__pyx_float_64_36);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 110, __pyx_float_64_36)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 111, __pyx_float__0005)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_64_86);
  __Pyx_GIVEREF(__pyx_float_64_86);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 112, __pyx_float_64_86)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 113, __pyx_float__0001)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_66_26);
  __Pyx_GIVEREF(__pyx_float_66_26);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 114, __pyx_float_66_26)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 115, __pyx_float__0004)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_67_70);
  __Pyx_GIVEREF(__pyx_float_67_70);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 116, __pyx_float_67_70)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 117, __pyx_float__0006)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_68_94);
  __Pyx_GIVEREF(__pyx_float_68_94);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 118, __pyx_float_68_94)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 119, __pyx_float__0002)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_70_10);
  __Pyx_GIVEREF(__pyx_float_70_10);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 120, __pyx_float_70_10)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 121, __pyx_float__0001)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_70_58);
  __Pyx_GIVEREF(__pyx_float_70_58);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 122, __pyx_float_70_58)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 123, __pyx_float__0002)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_72_01);
  __Pyx_GIVEREF(__pyx_float_72_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 124, __pyx_float_72_01)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 125, __pyx_float__0007)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_73_53);
  __Pyx_GIVEREF(__pyx_float_73_53);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 126, __pyx_float_73_53)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 127, __pyx_float__0006)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_75_00);
  __Pyx_GIVEREF(__pyx_float_75_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 128, __pyx_float_75_00)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 129, __pyx_float__0002)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_77_03);
  __Pyx_GIVEREF(__pyx_float_77_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 130, __pyx_float_77_03)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 131, __pyx_float__0005)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_78_00);
  __Pyx_GIVEREF(__pyx_float_78_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 132, __pyx_float_78_00)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 133, __pyx_float__0002)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_79_57);
  __Pyx_GIVEREF(__pyx_float_79_57);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 134, __pyx_float_79_57)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 135, __pyx_float__0006)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_81_16);
  __Pyx_GIVEREF(__pyx_float_81_16);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 136, __pyx_float_81_16)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 137, __pyx_float__0005)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_82_70);
  __Pyx_GIVEREF(__pyx_float_82_70);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 138, __pyx_float_82_70)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 139, __pyx_float__0005)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_84_22);
  __Pyx_GIVEREF(__pyx_float_84_22);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 140, __pyx_float_84_22)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 141, __pyx_float__0003)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_85_41);
  __Pyx_GIVEREF(__pyx_float_85_41);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 142, __pyx_float_85_41)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 143, __pyx_float__0002)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_87_46);
  __Pyx_GIVEREF(__pyx_float_87_46);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 144, __pyx_float_87_46)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 145, __pyx_float__0001)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_90_30);
  __Pyx_GIVEREF(__pyx_float_90_30);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 146, __pyx_float_90_30)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 147, __pyx_float__0001)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_94_02);
  __Pyx_GIVEREF(__pyx_float_94_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 148, __pyx_float_94_02)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 149, __pyx_float__0001)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_95_26);
  __Pyx_GIVEREF(__pyx_float_95_26);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 150, __pyx_float_95_26)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 151, __pyx_float__0002)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_109_39);
  __Pyx_GIVEREF(__pyx_float_109_39);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 152, __pyx_float_109_39)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 153, __pyx_float__0003)) __PYX_ERR(0, 28, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":29
 * # moved this outside function definition
 * PIANO_SPECTRA = [[1.97,.0326,2.99,.0086,3.95,.0163,4.97,.0178,5.98,.0177,6.95,.0315,8.02,.0001 ,8.94,.0076,9.96,.0134,10.99,.0284,11.98,.0229,13.02,.0229,13.89,.0010,15.06,.0090,16.00,.0003 ,17.08,.0078,18.16,.0064,19.18,.0129,20.21,.0085,21.27,.0225,22.32,.0061,23.41,.0102,24.48,.0005 ,25.56,.0016,26.64,.0018,27.70,.0113,28.80,.0111,29.91,.0158,31.06,.0093,32.17,.0017,33.32,.0002 ,34.42,.0018,35.59,.0027,36.74,.0055,37.90,.0037,39.06,.0064,40.25,.0033,41.47,.0014,42.53,.0004 ,43.89,.0010,45.12,.0039,46.33,.0039,47.64,.0009,48.88,.0016,50.13,.0006,51.37,.0010,52.70,.0002 ,54.00,.0004,55.30,.0008,56.60,.0025,57.96,.0010,59.30,.0012,60.67,.0011,61.99,.0003,62.86,.0001 ,64.36,.0005,64.86,.0001,66.26,.0004,67.70,.0006,68.94,.0002,70.10,.0001,70.58,.0002,72.01,.0007 ,73.53,.0006,75.00,.0002,77.03,.0005,78.00,.0002,79.57,.0006,81.16,.0005,82.70,.0005,84.22,.0003 ,85.41,.0002,87.46,.0001,90.30,.0001,94.02,.0001,95.26,.0002,109.39,.0003],
 * 			        [1.98,.0194,2.99,.0210,3.97,.0276,4.96,.0297,5.96,.0158,6.99,.0207,8.01,.0009 ,9.00,.0101,10.00,.0297,11.01,.0289,12.02,.0211,13.04,.0127,14.07,.0061,15.08,.0174,16.13,.0009 ,17.12,.0093,18.16,.0117,19.21,.0122,20.29,.0108,21.30,.0077,22.38,.0132,23.46,.0073,24.14,.0002 ,25.58,.0026,26.69,.0035,27.77,.0053,28.88,.0024,30.08,.0027,31.13,.0075,32.24,.0027,33.36,.0004 ,34.42,.0004,35.64,.0019,36.78,.0037,38.10,.0009,39.11,.0027,40.32,.0010,41.51,.0013,42.66,.0019 ,43.87,.0007,45.13,.0017,46.35,.0019,47.65,.0021,48.89,.0014,50.18,.0023,51.42,.0015,52.73,.0002 ,54.00,.0005,55.34,.0006,56.60,.0010,57.96,.0016,58.86,.0005,59.30,.0004,60.75,.0005,62.22,.0003 ,63.55,.0005,64.82,.0003,66.24,.0003,67.63,.0011,69.09,.0007,70.52,.0004,72.00,.0005,73.50,.0008 ,74.95,.0003,77.13,.0013,78.02,.0002,79.48,.0004,82.59,.0004,84.10,.0003],             # <<<<<<<<<<<<<<
 * 			        [2.00,.0313,2.99,.0109,4.00,.0215,5.00,.0242,5.98,.0355,7.01,.0132,8.01,.0009 ,9.01,.0071,10.00,.0258,11.03,.0221,12.02,.0056,13.06,.0196,14.05,.0160,15.11,.0107,16.11,.0003 ,17.14,.0111,18.21,.0085,19.23,.0010,20.28,.0048,21.31,.0128,22.36,.0051,23.41,.0041,24.05,.0006 ,25.54,.0019,26.62,.0028,27.72,.0034,28.82,.0062,29.89,.0039,30.98,.0058,32.08,.0011,33.21,.0002 ,34.37,.0008,35.46,.0018,36.62,.0036,37.77,.0018,38.92,.0042,40.07,.0037,41.23,.0011,42.67,.0003 ,43.65,.0018,44.68,.0025,45.99,.0044,47.21,.0051,48.40,.0044,49.67,.0005,50.88,.0019,52.15,.0003 ,53.42,.0008,54.69,.0010,55.98,.0005,57.26,.0013,58.53,.0027,59.83,.0011,61.21,.0027,62.54,.0003 ,63.78,.0003,65.20,.0001,66.60,.0006,67.98,.0008,69.37,.0019,70.73,.0007,72.14,.0004,73.62,.0002 ,74.40,.0003,76.52,.0006,77.97,.0002,79.49,.0004,80.77,.0003,81.00,.0001,82.47,.0005,83.97,.0001 ,87.27,.0002],
 * 			        [2.00,.0257,2.99,.0142,3.97,.0202,4.95,.0148,5.95,.0420,6.95,.0037,7.94,.0004 ,8.94,.0172,9.95,.0191,10.96,.0115,11.97,.0059,12.98,.0140,14.00,.0178,15.03,.0121,16.09,.0002 ,17.07,.0066,18.08,.0033,19.15,.0022,20.18,.0057,21.22,.0077,22.29,.0037,23.33,.0066,24.97,.0002 ,25.49,.0019,26.55,.0042,27.61,.0043,28.73,.0038,29.81,.0084,30.91,.0040,32.03,.0025,33.14,.0005 ,34.26,.0003,35.38,.0019,36.56,.0037,37.68,.0049,38.86,.0036,40.11,.0011,41.28,.0008,42.50,.0004 ,43.60,.0002,44.74,.0022,45.99,.0050,47.20,.0009,48.40,.0036,49.68,.0004,50.92,.0009,52.17,.0005 ,53.46,.0007,54.76,.0006,56.06,.0005,57.34,.0011,58.67,.0005,59.95,.0015,61.37,.0008,62.72,.0004 ,65.42,.0009,66.96,.0003,68.18,.0003,69.78,.0003,71.21,.0004,72.45,.0002,74.22,.0003,75.44,.0001 ,76.53,.0003,78.31,.0004,79.83,.0003,80.16,.0001,81.33,.0003,82.44,.0001,83.17,.0002,84.81,.0003 ,85.97,.0003,89.08,.0001,90.70,.0002,92.30,.0002,95.59,.0002,97.22,.0003,98.86,.0001,108.37,.0001 ,125.54,.0001],
 */
  __pyx_t_3 = PyList_New(138); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_float_1_98);
  __Pyx_GIVEREF(__pyx_float_1_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 0, __pyx_float_1_98)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0194);
  __Pyx_GIVEREF(__pyx_float__0194);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 1, __pyx_float__0194)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_99);
  __Pyx_GIVEREF(__pyx_float_2_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 2, __pyx_float_2_99)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0210);
  __Pyx_GIVEREF(__pyx_float__0210);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 3, __pyx_float__0210)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_97);
  __Pyx_GIVEREF(__pyx_float_3_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 4, __pyx_float_3_97)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0276);
  __Pyx_GIVEREF(__pyx_float__0276);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 5, __pyx_float__0276)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_96);
  __Pyx_GIVEREF(__pyx_float_4_96);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 6, __pyx_float_4_96)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0297);
  __Pyx_GIVEREF(__pyx_float__0297);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 7, __pyx_float__0297)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_96);
  __Pyx_GIVEREF(__pyx_float_5_96);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 8, __pyx_float_5_96)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0158);
  __Pyx_GIVEREF(__pyx_float__0158);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 9, __pyx_float__0158)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_99);
  __Pyx_GIVEREF(__pyx_float_6_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 10, __pyx_float_6_99)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0207);
  __Pyx_GIVEREF(__pyx_float__0207);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 11, __pyx_float__0207)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_01);
  __Pyx_GIVEREF(__pyx_float_8_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 12, __pyx_float_8_01)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 13, __pyx_float__0009)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_00);
  __Pyx_GIVEREF(__pyx_float_9_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 14, __pyx_float_9_00)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0101);
  __Pyx_GIVEREF(__pyx_float__0101);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 15, __pyx_float__0101)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_00);
  __Pyx_GIVEREF(__pyx_float_10_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 16, __pyx_float_10_00)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0297);
  __Pyx_GIVEREF(__pyx_float__0297);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 17, __pyx_float__0297)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_01);
  __Pyx_GIVEREF(__pyx_float_11_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 18, __pyx_float_11_01)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0289);
  __Pyx_GIVEREF(__pyx_float__0289);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 19, __pyx_float__0289)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_02);
  __Pyx_GIVEREF(__pyx_float_12_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 20, __pyx_float_12_02)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0211);
  __Pyx_GIVEREF(__pyx_float__0211);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 21, __pyx_float__0211)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_04);
  __Pyx_GIVEREF(__pyx_float_13_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 22, __pyx_float_13_04)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0127);
  __Pyx_GIVEREF(__pyx_float__0127);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 23, __pyx_float__0127)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_07);
  __Pyx_GIVEREF(__pyx_float_14_07);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 24, __pyx_float_14_07)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0061);
  __Pyx_GIVEREF(__pyx_float__0061);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 25, __pyx_float__0061)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_08);
  __Pyx_GIVEREF(__pyx_float_15_08);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 26, __pyx_float_15_08)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0174);
  __Pyx_GIVEREF(__pyx_float__0174);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 27, __pyx_float__0174)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_13);
  __Pyx_GIVEREF(__pyx_float_16_13);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 28, __pyx_float_16_13)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 29, __pyx_float__0009)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_12);
  __Pyx_GIVEREF(__pyx_float_17_12);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 30, __pyx_float_17_12)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0093);
  __Pyx_GIVEREF(__pyx_float__0093);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 31, __pyx_float__0093)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_16);
  __Pyx_GIVEREF(__pyx_float_18_16);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 32, __pyx_float_18_16)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0117);
  __Pyx_GIVEREF(__pyx_float__0117);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 33, __pyx_float__0117)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_21);
  __Pyx_GIVEREF(__pyx_float_19_21);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 34, __pyx_float_19_21)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0122);
  __Pyx_GIVEREF(__pyx_float__0122);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 35, __pyx_float__0122)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_29);
  __Pyx_GIVEREF(__pyx_float_20_29);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 36, __pyx_float_20_29)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0108);
  __Pyx_GIVEREF(__pyx_float__0108);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 37, __pyx_float__0108)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_30);
  __Pyx_GIVEREF(__pyx_float_21_30);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 38, __pyx_float_21_30)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0077);
  __Pyx_GIVEREF(__pyx_float__0077);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 39, __pyx_float__0077)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_38);
  __Pyx_GIVEREF(__pyx_float_22_38);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 40, __pyx_float_22_38)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0132);
  __Pyx_GIVEREF(__pyx_float__0132);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 41, __pyx_float__0132)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_46);
  __Pyx_GIVEREF(__pyx_float_23_46);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 42, __pyx_float_23_46)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0073);
  __Pyx_GIVEREF(__pyx_float__0073);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 43, __pyx_float__0073)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_24_14);
  __Pyx_GIVEREF(__pyx_float_24_14);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 44, __pyx_float_24_14)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 45, __pyx_float__0002)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_25_58);
  __Pyx_GIVEREF(__pyx_float_25_58);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 46, __pyx_float_25_58)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0026);
  __Pyx_GIVEREF(__pyx_float__0026);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 47, __pyx_float__0026)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_26_69);
  __Pyx_GIVEREF(__pyx_float_26_69);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 48, __pyx_float_26_69)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0035);
  __Pyx_GIVEREF(__pyx_float__0035);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 49, __pyx_float__0035)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_27_77);
  __Pyx_GIVEREF(__pyx_float_27_77);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 50, __pyx_float_27_77)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0053);
  __Pyx_GIVEREF(__pyx_float__0053);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 51, __pyx_float__0053)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_88);
  __Pyx_GIVEREF(__pyx_float_28_88);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 52, __pyx_float_28_88)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0024);
  __Pyx_GIVEREF(__pyx_float__0024);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 53, __pyx_float__0024)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_30_08);
  __Pyx_GIVEREF(__pyx_float_30_08);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 54, __pyx_float_30_08)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0027);
  __Pyx_GIVEREF(__pyx_float__0027);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 55, __pyx_float__0027)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_31_13);
  __Pyx_GIVEREF(__pyx_float_31_13);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 56, __pyx_float_31_13)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0075);
  __Pyx_GIVEREF(__pyx_float__0075);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 57, __pyx_float__0075)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_32_24);
  __Pyx_GIVEREF(__pyx_float_32_24);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 58, __pyx_float_32_24)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0027);
  __Pyx_GIVEREF(__pyx_float__0027);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 59, __pyx_float__0027)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_33_36);
  __Pyx_GIVEREF(__pyx_float_33_36);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 60, __pyx_float_33_36)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 61, __pyx_float__0004)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_34_42);
  __Pyx_GIVEREF(__pyx_float_34_42);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 62, __pyx_float_34_42)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 63, __pyx_float__0004)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_35_64);
  __Pyx_GIVEREF(__pyx_float_35_64);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 64, __pyx_float_35_64)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0019);
  __Pyx_GIVEREF(__pyx_float__0019);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 65, __pyx_float__0019)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_36_78);
  __Pyx_GIVEREF(__pyx_float_36_78);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 66, __pyx_float_36_78)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0037);
  __Pyx_GIVEREF(__pyx_float__0037);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 67, __pyx_float__0037)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_38_10);
  __Pyx_GIVEREF(__pyx_float_38_10);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 68, __pyx_float_38_10)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 69, __pyx_float__0009)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_39_11);
  __Pyx_GIVEREF(__pyx_float_39_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 70, __pyx_float_39_11)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0027);
  __Pyx_GIVEREF(__pyx_float__0027);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 71, __pyx_float__0027)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_40_32);
  __Pyx_GIVEREF(__pyx_float_40_32);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 72, __pyx_float_40_32)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 73, __pyx_float__0010)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_41_51);
  __Pyx_GIVEREF(__pyx_float_41_51);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 74, __pyx_float_41_51)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0013);
  __Pyx_GIVEREF(__pyx_float__0013);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 75, __pyx_float__0013)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_42_66);
  __Pyx_GIVEREF(__pyx_float_42_66);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 76, __pyx_float_42_66)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0019);
  __Pyx_GIVEREF(__pyx_float__0019);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 77, __pyx_float__0019)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_43_87);
  __Pyx_GIVEREF(__pyx_float_43_87);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 78, __pyx_float_43_87)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 79, __pyx_float__0007)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_45_13);
  __Pyx_GIVEREF(__pyx_float_45_13);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 80, __pyx_float_45_13)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0017);
  __Pyx_GIVEREF(__pyx_float__0017);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 81, __pyx_float__0017)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_46_35);
  __Pyx_GIVEREF(__pyx_float_46_35);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 82, __pyx_float_46_35)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0019);
  __Pyx_GIVEREF(__pyx_float__0019);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 83, __pyx_float__0019)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_47_65);
  __Pyx_GIVEREF(__pyx_float_47_65);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 84, __pyx_float_47_65)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0021);
  __Pyx_GIVEREF(__pyx_float__0021);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 85, __pyx_float__0021)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_48_89);
  __Pyx_GIVEREF(__pyx_float_48_89);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 86, __pyx_float_48_89)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0014);
  __Pyx_GIVEREF(__pyx_float__0014);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 87, __pyx_float__0014)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_50_18);
  __Pyx_GIVEREF(__pyx_float_50_18);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 88, __pyx_float_50_18)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0023);
  __Pyx_GIVEREF(__pyx_float__0023);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 89, __pyx_float__0023)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_51_42);
  __Pyx_GIVEREF(__pyx_float_51_42);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 90, __pyx_float_51_42)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0015);
  __Pyx_GIVEREF(__pyx_float__0015);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 91, __pyx_float__0015)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_52_73);
  __Pyx_GIVEREF(__pyx_float_52_73);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 92, __pyx_float_52_73)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 93, __pyx_float__0002)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_54_00);
  __Pyx_GIVEREF(__pyx_float_54_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 94, __pyx_float_54_00)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 95, __pyx_float__0005)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_55_34);
  __Pyx_GIVEREF(__pyx_float_55_34);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 96, __pyx_float_55_34)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 97, __pyx_float__0006)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_56_60);
  __Pyx_GIVEREF(__pyx_float_56_60);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 98, __pyx_float_56_60)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 99, __pyx_float__0010)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_57_96);
  __Pyx_GIVEREF(__pyx_float_57_96);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 100, __pyx_float_57_96)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0016);
  __Pyx_GIVEREF(__pyx_float__0016);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 101, __pyx_float__0016)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_58_86);
  __Pyx_GIVEREF(__pyx_float_58_86);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 102, __pyx_float_58_86)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 103, __pyx_float__0005)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_59_30);
  __Pyx_GIVEREF(__pyx_float_59_30);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 104, __pyx_float_59_30)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 105, __pyx_float__0004)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_60_75);
  __Pyx_GIVEREF(__pyx_float_60_75);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 106, __pyx_float_60_75)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 107, __pyx_float__0005)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_62_22);
  __Pyx_GIVEREF(__pyx_float_62_22);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 108, __pyx_float_62_22)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 109, __pyx_float__0003)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_63_55);
  __Pyx_GIVEREF(__pyx_float_63_55);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 110, __pyx_float_63_55)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 111, __pyx_float__0005)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_64_82);
  __Pyx_GIVEREF(__pyx_float_64_82);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 112, __pyx_float_64_82)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 113, __pyx_float__0003)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_66_24);
  __Pyx_GIVEREF(__pyx_float_66_24);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 114, __pyx_float_66_24)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 115, __pyx_float__0003)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_67_63);
  __Pyx_GIVEREF(__pyx_float_67_63);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 116, __pyx_float_67_63)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 117, __pyx_float__0011)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_69_09);
  __Pyx_GIVEREF(__pyx_float_69_09);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 118, __pyx_float_69_09)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 119, __pyx_float__0007)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_70_52);
  __Pyx_GIVEREF(__pyx_float_70_52);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 120, __pyx_float_70_52)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 121, __pyx_float__0004)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_72_00);
  __Pyx_GIVEREF(__pyx_float_72_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 122, __pyx_float_72_00)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 123, __pyx_float__0005)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_73_50);
  __Pyx_GIVEREF(__pyx_float_73_50);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 124, __pyx_float_73_50)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 125, __pyx_float__0008)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_74_95);
  __Pyx_GIVEREF(__pyx_float_74_95);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 126, __pyx_float_74_95)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 127, __pyx_float__0003)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_77_13);
  __Pyx_GIVEREF(__pyx_float_77_13);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 128, __pyx_float_77_13)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0013);
  __Pyx_GIVEREF(__pyx_float__0013);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 129, __pyx_float__0013)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_78_02);
  __Pyx_GIVEREF(__pyx_float_78_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 130, __pyx_float_78_02)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 131, __pyx_float__0002)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_79_48);
  __Pyx_GIVEREF(__pyx_float_79_48);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 132, __pyx_float_79_48)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 133, __pyx_float__0004)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_82_59);
  __Pyx_GIVEREF(__pyx_float_82_59);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 134, __pyx_float_82_59)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 135, __pyx_float__0004)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_84_10);
  __Pyx_GIVEREF(__pyx_float_84_10);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 136, __pyx_float_84_10)) __PYX_ERR(0, 29, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 137, __pyx_float__0003)) __PYX_ERR(0, 29, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":30
 * PIANO_SPECTRA = [[1.97,.0326,2.99,.0086,3.95,.0163,4.97,.0178,5.98,.0177,6.95,.0315,8.02,.0001 ,8.94,.0076,9.96,.0134,10.99,.0284,11.98,.0229,13.02,.0229,13.89,.0010,15.06,.0090,16.00,.0003 ,17.08,.0078,18.16,.0064,19.18,.0129,20.21,.0085,21.27,.0225,22.32,.0061,23.41,.0102,24.48,.0005 ,25.56,.0016,26.64,.0018,27.70,.0113,28.80,.0111,29.91,.0158,31.06,.0093,32.17,.0017,33.32,.0002 ,34.42,.0018,35.59,.0027,36.74,.0055,37.90,.0037,39.06,.0064,40.25,.0033,41.47,.0014,42.53,.0004 ,43.89,.0010,45.12,.0039,46.33,.0039,47.64,.0009,48.88,.0016,50.13,.0006,51.37,.0010,52.70,.0002 ,54.00,.0004,55.30,.0008,56.60,.0025,57.96,.0010,59.30,.0012,60.67,.0011,61.99,.0003,62.86,.0001 ,64.36,.0005,64.86,.0001,66.26,.0004,67.70,.0006,68.94,.0002,70.10,.0001,70.58,.0002,72.01,.0007 ,73.53,.0006,75.00,.0002,77.03,.0005,78.00,.0002,79.57,.0006,81.16,.0005,82.70,.0005,84.22,.0003 ,85.41,.0002,87.46,.0001,90.30,.0001,94.02,.0001,95.26,.0002,109.39,.0003],
 * 			        [1.98,.0194,2.99,.0210,3.97,.0276,4.96,.0297,5.96,.0158,6.99,.0207,8.01,.0009 ,9.00,.0101,10.00,.0297,11.01,.0289,12.02,.0211,13.04,.0127,14.07,.0061,15.08,.0174,16.13,.0009 ,17.12,.0093,18.16,.0117,19.21,.0122,20.29,.0108,21.30,.0077,22.38,.0132,23.46,.0073,24.14,.0002 ,25.58,.0026,26.69,.0035,27.77,.0053,28.88,.0024,30.08,.0027,31.13,.0075,32.24,.0027,33.36,.0004 ,34.42,.0004,35.64,.0019,36.78,.0037,38.10,.0009,39.11,.0027,40.32,.0010,41.51,.0013,42.66,.0019 ,43.87,.0007,45.13,.0017,46.35,.0019,47.65,.0021,48.89,.0014,50.18,.0023,51.42,.0015,52.73,.0002 ,54.00,.0005,55.34,.0006,56.60,.0010,57.96,.0016,58.86,.0005,59.30,.0004,60.75,.0005,62.22,.0003 ,63.55,.0005,64.82,.0003,66.24,.0003,67.63,.0011,69.09,.0007,70.52,.0004,72.00,.0005,73.50,.0008 ,74.95,.0003,77.13,.0013,78.02,.0002,79.48,.0004,82.59,.0004,84.10,.0003],
 * 			        [2.00,.0313,2.99,.0109,4.00,.0215,5.00,.0242,5.98,.0355,7.01,.0132,8.01,.0009 ,9.01,.0071,10.00,.0258,11.03,.0221,12.02,.0056,13.06,.0196,14.05,.0160,15.11,.0107,16.11,.0003 ,17.14,.0111,18.21,.0085,19.23,.0010,20.28,.0048,21.31,.0128,22.36,.0051,23.41,.0041,24.05,.0006 ,25.54,.0019,26.62,.0028,27.72,.0034,28.82,.0062,29.89,.0039,30.98,.0058,32.08,.0011,33.21,.0002 ,34.37,.0008,35.46,.0018,36.62,.0036,37.77,.0018,38.92,.0042,40.07,.0037,41.23,.0011,42.67,.0003 ,43.65,.0018,44.68,.0025,45.99,.0044,47.21,.0051,48.40,.0044,49.67,.0005,50.88,.0019,52.15,.0003 ,53.42,.0008,54.69,.0010,55.98,.0005,57.26,.0013,58.53,.0027,59.83,.0011,61.21,.0027,62.54,.0003 ,63.78,.0003,65.20,.0001,66.60,.0006,67.98,.0008,69.37,.0019,70.73,.0007,72.14,.0004,73.62,.0002 ,74.40,.0003,76.52,.0006,77.97,.0002,79.49,.0004,80.77,.0003,81.00,.0001,82.47,.0005,83.97,.0001 ,87.27,.0002],             # <<<<<<<<<<<<<<
 * 			        [2.00,.0257,2.99,.0142,3.97,.0202,4.95,.0148,5.95,.0420,6.95,.0037,7.94,.0004 ,8.94,.0172,9.95,.0191,10.96,.0115,11.97,.0059,12.98,.0140,14.00,.0178,15.03,.0121,16.09,.0002 ,17.07,.0066,18.08,.0033,19.15,.0022,20.18,.0057,21.22,.0077,22.29,.0037,23.33,.0066,24.97,.0002 ,25.49,.0019,26.55,.0042,27.61,.0043,28.73,.0038,29.81,.0084,30.91,.0040,32.03,.0025,33.14,.0005 ,34.26,.0003,35.38,.0019,36.56,.0037,37.68,.0049,38.86,.0036,40.11,.0011,41.28,.0008,42.50,.0004 ,43.60,.0002,44.74,.0022,45.99,.0050,47.20,.0009,48.40,.0036,49.68,.0004,50.92,.0009,52.17,.0005 ,53.46,.0007,54.76,.0006,56.06,.0005,57.34,.0011,58.67,.0005,59.95,.0015,61.37,.0008,62.72,.0004 ,65.42,.0009,66.96,.0003,68.18,.0003,69.78,.0003,71.21,.0004,72.45,.0002,74.22,.0003,75.44,.0001 ,76.53,.0003,78.31,.0004,79.83,.0003,80.16,.0001,81.33,.0003,82.44,.0001,83.17,.0002,84.81,.0003 ,85.97,.0003,89.08,.0001,90.70,.0002,92.30,.0002,95.59,.0002,97.22,.0003,98.86,.0001,108.37,.0001 ,125.54,.0001],
 * 			        [1.99,.0650,3.03,.0040,4.03,.0059,5.02,.0090,5.97,.0227,6.98,.0050,8.04,.0020 ,9.00,.0082,9.96,.0078,11.01,.0056,12.01,.0095,13.02,.0050,14.04,.0093,15.08,.0064,16.14,.0017 ,17.06,.0020,18.10,.0025,19.14,.0023,20.18,.0015,21.24,.0032,22.29,.0029,23.32,.0014,24.37,.0005 ,25.43,.0030,26.50,.0022,27.60,.0027,28.64,.0024,29.76,.0035,30.81,.0136,31.96,.0025,33.02,.0003 ,34.13,.0005,35.25,.0007,36.40,.0014,37.51,.0020,38.64,.0012,39.80,.0019,40.97,.0004,42.09,.0003 ,43.24,.0003,44.48,.0002,45.65,.0024,46.86,.0005,48.07,.0013,49.27,.0008,50.49,.0006,52.95,.0001 ,54.23,.0005,55.45,.0004,56.73,.0001,58.03,.0003,59.29,.0002,60.59,.0003,62.04,.0002,65.89,.0002 ,67.23,.0002,68.61,.0002,69.97,.0004,71.36,.0005,85.42,.0001],
 */
  __pyx_t_4 = PyList_New(144); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_float_2_00);
  __Pyx_GIVEREF(__pyx_float_2_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 0, __pyx_float_2_00)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0313);
  __Pyx_GIVEREF(__pyx_float__0313);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 1, __pyx_float__0313)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_99);
  __Pyx_GIVEREF(__pyx_float_2_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 2, __pyx_float_2_99)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0109);
  __Pyx_GIVEREF(__pyx_float__0109);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 3, __pyx_float__0109)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_00);
  __Pyx_GIVEREF(__pyx_float_4_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 4, __pyx_float_4_00)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0215);
  __Pyx_GIVEREF(__pyx_float__0215);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 5, __pyx_float__0215)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_00);
  __Pyx_GIVEREF(__pyx_float_5_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 6, __pyx_float_5_00)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0242);
  __Pyx_GIVEREF(__pyx_float__0242);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 7, __pyx_float__0242)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_98);
  __Pyx_GIVEREF(__pyx_float_5_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 8, __pyx_float_5_98)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0355);
  __Pyx_GIVEREF(__pyx_float__0355);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 9, __pyx_float__0355)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_01);
  __Pyx_GIVEREF(__pyx_float_7_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 10, __pyx_float_7_01)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0132);
  __Pyx_GIVEREF(__pyx_float__0132);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 11, __pyx_float__0132)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_01);
  __Pyx_GIVEREF(__pyx_float_8_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 12, __pyx_float_8_01)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 13, __pyx_float__0009)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_01);
  __Pyx_GIVEREF(__pyx_float_9_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 14, __pyx_float_9_01)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0071);
  __Pyx_GIVEREF(__pyx_float__0071);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 15, __pyx_float__0071)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_00);
  __Pyx_GIVEREF(__pyx_float_10_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 16, __pyx_float_10_00)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0258);
  __Pyx_GIVEREF(__pyx_float__0258);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 17, __pyx_float__0258)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_03);
  __Pyx_GIVEREF(__pyx_float_11_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 18, __pyx_float_11_03)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0221);
  __Pyx_GIVEREF(__pyx_float__0221);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 19, __pyx_float__0221)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_02);
  __Pyx_GIVEREF(__pyx_float_12_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 20, __pyx_float_12_02)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0056);
  __Pyx_GIVEREF(__pyx_float__0056);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 21, __pyx_float__0056)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_06);
  __Pyx_GIVEREF(__pyx_float_13_06);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 22, __pyx_float_13_06)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0196);
  __Pyx_GIVEREF(__pyx_float__0196);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 23, __pyx_float__0196)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_05);
  __Pyx_GIVEREF(__pyx_float_14_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 24, __pyx_float_14_05)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0160);
  __Pyx_GIVEREF(__pyx_float__0160);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 25, __pyx_float__0160)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_11);
  __Pyx_GIVEREF(__pyx_float_15_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 26, __pyx_float_15_11)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0107);
  __Pyx_GIVEREF(__pyx_float__0107);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 27, __pyx_float__0107)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_11);
  __Pyx_GIVEREF(__pyx_float_16_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 28, __pyx_float_16_11)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 29, __pyx_float__0003)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_14);
  __Pyx_GIVEREF(__pyx_float_17_14);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 30, __pyx_float_17_14)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0111);
  __Pyx_GIVEREF(__pyx_float__0111);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 31, __pyx_float__0111)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_21);
  __Pyx_GIVEREF(__pyx_float_18_21);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 32, __pyx_float_18_21)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0085);
  __Pyx_GIVEREF(__pyx_float__0085);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 33, __pyx_float__0085)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_23);
  __Pyx_GIVEREF(__pyx_float_19_23);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 34, __pyx_float_19_23)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 35, __pyx_float__0010)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_28);
  __Pyx_GIVEREF(__pyx_float_20_28);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 36, __pyx_float_20_28)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0048);
  __Pyx_GIVEREF(__pyx_float__0048);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 37, __pyx_float__0048)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_31);
  __Pyx_GIVEREF(__pyx_float_21_31);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 38, __pyx_float_21_31)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0128);
  __Pyx_GIVEREF(__pyx_float__0128);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 39, __pyx_float__0128)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_36);
  __Pyx_GIVEREF(__pyx_float_22_36);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 40, __pyx_float_22_36)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0051);
  __Pyx_GIVEREF(__pyx_float__0051);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 41, __pyx_float__0051)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_41);
  __Pyx_GIVEREF(__pyx_float_23_41);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 42, __pyx_float_23_41)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0041);
  __Pyx_GIVEREF(__pyx_float__0041);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 43, __pyx_float__0041)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_24_05);
  __Pyx_GIVEREF(__pyx_float_24_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 44, __pyx_float_24_05)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 45, __pyx_float__0006)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_25_54);
  __Pyx_GIVEREF(__pyx_float_25_54);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 46, __pyx_float_25_54)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0019);
  __Pyx_GIVEREF(__pyx_float__0019);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 47, __pyx_float__0019)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_26_62);
  __Pyx_GIVEREF(__pyx_float_26_62);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 48, __pyx_float_26_62)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0028);
  __Pyx_GIVEREF(__pyx_float__0028);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 49, __pyx_float__0028)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_27_72);
  __Pyx_GIVEREF(__pyx_float_27_72);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 50, __pyx_float_27_72)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0034);
  __Pyx_GIVEREF(__pyx_float__0034);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 51, __pyx_float__0034)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_82);
  __Pyx_GIVEREF(__pyx_float_28_82);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 52, __pyx_float_28_82)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0062);
  __Pyx_GIVEREF(__pyx_float__0062);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 53, __pyx_float__0062)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_29_89);
  __Pyx_GIVEREF(__pyx_float_29_89);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 54, __pyx_float_29_89)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0039);
  __Pyx_GIVEREF(__pyx_float__0039);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 55, __pyx_float__0039)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_30_98);
  __Pyx_GIVEREF(__pyx_float_30_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 56, __pyx_float_30_98)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0058);
  __Pyx_GIVEREF(__pyx_float__0058);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 57, __pyx_float__0058)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_32_08);
  __Pyx_GIVEREF(__pyx_float_32_08);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 58, __pyx_float_32_08)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 59, __pyx_float__0011)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_33_21);
  __Pyx_GIVEREF(__pyx_float_33_21);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 60, __pyx_float_33_21)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 61, __pyx_float__0002)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_34_37);
  __Pyx_GIVEREF(__pyx_float_34_37);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 62, __pyx_float_34_37)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 63, __pyx_float__0008)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_35_46);
  __Pyx_GIVEREF(__pyx_float_35_46);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 64, __pyx_float_35_46)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0018);
  __Pyx_GIVEREF(__pyx_float__0018);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 65, __pyx_float__0018)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_36_62);
  __Pyx_GIVEREF(__pyx_float_36_62);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 66, __pyx_float_36_62)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0036);
  __Pyx_GIVEREF(__pyx_float__0036);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 67, __pyx_float__0036)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_37_77);
  __Pyx_GIVEREF(__pyx_float_37_77);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 68, __pyx_float_37_77)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0018);
  __Pyx_GIVEREF(__pyx_float__0018);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 69, __pyx_float__0018)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_38_92);
  __Pyx_GIVEREF(__pyx_float_38_92);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 70, __pyx_float_38_92)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0042);
  __Pyx_GIVEREF(__pyx_float__0042);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 71, __pyx_float__0042)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_40_07);
  __Pyx_GIVEREF(__pyx_float_40_07);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 72, __pyx_float_40_07)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0037);
  __Pyx_GIVEREF(__pyx_float__0037);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 73, __pyx_float__0037)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_41_23);
  __Pyx_GIVEREF(__pyx_float_41_23);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 74, __pyx_float_41_23)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 75, __pyx_float__0011)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_42_67);
  __Pyx_GIVEREF(__pyx_float_42_67);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 76, __pyx_float_42_67)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 77, __pyx_float__0003)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_43_65);
  __Pyx_GIVEREF(__pyx_float_43_65);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 78, __pyx_float_43_65)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0018);
  __Pyx_GIVEREF(__pyx_float__0018);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 79, __pyx_float__0018)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_44_68);
  __Pyx_GIVEREF(__pyx_float_44_68);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 80, __pyx_float_44_68)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0025);
  __Pyx_GIVEREF(__pyx_float__0025);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 81, __pyx_float__0025)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_45_99);
  __Pyx_GIVEREF(__pyx_float_45_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 82, __pyx_float_45_99)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0044);
  __Pyx_GIVEREF(__pyx_float__0044);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 83, __pyx_float__0044)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_47_21);
  __Pyx_GIVEREF(__pyx_float_47_21);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 84, __pyx_float_47_21)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0051);
  __Pyx_GIVEREF(__pyx_float__0051);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 85, __pyx_float__0051)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_48_40);
  __Pyx_GIVEREF(__pyx_float_48_40);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 86, __pyx_float_48_40)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0044);
  __Pyx_GIVEREF(__pyx_float__0044);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 87, __pyx_float__0044)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_49_67);
  __Pyx_GIVEREF(__pyx_float_49_67);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 88, __pyx_float_49_67)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 89, __pyx_float__0005)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_50_88);
  __Pyx_GIVEREF(__pyx_float_50_88);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 90, __pyx_float_50_88)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0019);
  __Pyx_GIVEREF(__pyx_float__0019);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 91, __pyx_float__0019)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_52_15);
  __Pyx_GIVEREF(__pyx_float_52_15);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 92, __pyx_float_52_15)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 93, __pyx_float__0003)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_53_42);
  __Pyx_GIVEREF(__pyx_float_53_42);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 94, __pyx_float_53_42)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 95, __pyx_float__0008)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_54_69);
  __Pyx_GIVEREF(__pyx_float_54_69);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 96, __pyx_float_54_69)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 97, __pyx_float__0010)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_55_98);
  __Pyx_GIVEREF(__pyx_float_55_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 98, __pyx_float_55_98)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 99, __pyx_float__0005)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_57_26);
  __Pyx_GIVEREF(__pyx_float_57_26);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 100, __pyx_float_57_26)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0013);
  __Pyx_GIVEREF(__pyx_float__0013);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 101, __pyx_float__0013)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_58_53);
  __Pyx_GIVEREF(__pyx_float_58_53);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 102, __pyx_float_58_53)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0027);
  __Pyx_GIVEREF(__pyx_float__0027);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 103, __pyx_float__0027)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_59_83);
  __Pyx_GIVEREF(__pyx_float_59_83);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 104, __pyx_float_59_83)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 105, __pyx_float__0011)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_61_21);
  __Pyx_GIVEREF(__pyx_float_61_21);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 106, __pyx_float_61_21)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0027);
  __Pyx_GIVEREF(__pyx_float__0027);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 107, __pyx_float__0027)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_62_54);
  __Pyx_GIVEREF(__pyx_float_62_54);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 108, __pyx_float_62_54)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 109, __pyx_float__0003)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_63_78);
  __Pyx_GIVEREF(__pyx_float_63_78);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 110, __pyx_float_63_78)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 111, __pyx_float__0003)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_65_20);
  __Pyx_GIVEREF(__pyx_float_65_20);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 112, __pyx_float_65_20)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 113, __pyx_float__0001)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_66_60);
  __Pyx_GIVEREF(__pyx_float_66_60);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 114, __pyx_float_66_60)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 115, __pyx_float__0006)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_67_98);
  __Pyx_GIVEREF(__pyx_float_67_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 116, __pyx_float_67_98)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 117, __pyx_float__0008)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_69_37);
  __Pyx_GIVEREF(__pyx_float_69_37);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 118, __pyx_float_69_37)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0019);
  __Pyx_GIVEREF(__pyx_float__0019);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 119, __pyx_float__0019)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_70_73);
  __Pyx_GIVEREF(__pyx_float_70_73);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 120, __pyx_float_70_73)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 121, __pyx_float__0007)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_72_14);
  __Pyx_GIVEREF(__pyx_float_72_14);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 122, __pyx_float_72_14)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 123, __pyx_float__0004)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_73_62);
  __Pyx_GIVEREF(__pyx_float_73_62);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 124, __pyx_float_73_62)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 125, __pyx_float__0002)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_74_40);
  __Pyx_GIVEREF(__pyx_float_74_40);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 126, __pyx_float_74_40)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 127, __pyx_float__0003)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_76_52);
  __Pyx_GIVEREF(__pyx_float_76_52);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 128, __pyx_float_76_52)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 129, __pyx_float__0006)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_77_97);
  __Pyx_GIVEREF(__pyx_float_77_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 130, __pyx_float_77_97)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 131, __pyx_float__0002)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_79_49);
  __Pyx_GIVEREF(__pyx_float_79_49);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 132, __pyx_float_79_49)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 133, __pyx_float__0004)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_80_77);
  __Pyx_GIVEREF(__pyx_float_80_77);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 134, __pyx_float_80_77)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 135, __pyx_float__0003)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_81_00);
  __Pyx_GIVEREF(__pyx_float_81_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 136, __pyx_float_81_00)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 137, __pyx_float__0001)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_82_47);
  __Pyx_GIVEREF(__pyx_float_82_47);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 138, __pyx_float_82_47)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 139, __pyx_float__0005)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_83_97);
  __Pyx_GIVEREF(__pyx_float_83_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 140, __pyx_float_83_97)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 141, __pyx_float__0001)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_87_27);
  __Pyx_GIVEREF(__pyx_float_87_27);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 142, __pyx_float_87_27)) __PYX_ERR(0, 30, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 143, __pyx_float__0002)) __PYX_ERR(0, 30, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":31
 * 			        [1.98,.0194,2.99,.0210,3.97,.0276,4.96,.0297,5.96,.0158,6.99,.0207,8.01,.0009 ,9.00,.0101,10.00,.0297,11.01,.0289,12.02,.0211,13.04,.0127,14.07,.0061,15.08,.0174,16.13,.0009 ,17.12,.0093,18.16,.0117,19.21,.0122,20.29,.0108,21.30,.0077,22.38,.0132,23.46,.0073,24.14,.0002 ,25.58,.0026,26.69,.0035,27.77,.0053,28.88,.0024,30.08,.0027,31.13,.0075,32.24,.0027,33.36,.0004 ,34.42,.0004,35.64,.0019,36.78,.0037,38.10,.0009,39.11,.0027,40.32,.0010,41.51,.0013,42.66,.0019 ,43.87,.0007,45.13,.0017,46.35,.0019,47.65,.0021,48.89,.0014,50.18,.0023,51.42,.0015,52.73,.0002 ,54.00,.0005,55.34,.0006,56.60,.0010,57.96,.0016,58.86,.0005,59.30,.0004,60.75,.0005,62.22,.0003 ,63.55,.0005,64.82,.0003,66.24,.0003,67.63,.0011,69.09,.0007,70.52,.0004,72.00,.0005,73.50,.0008 ,74.95,.0003,77.13,.0013,78.02,.0002,79.48,.0004,82.59,.0004,84.10,.0003],
 * 			        [2.00,.0313,2.99,.0109,4.00,.0215,5.00,.0242,5.98,.0355,7.01,.0132,8.01,.0009 ,9.01,.0071,10.00,.0258,11.03,.0221,12.02,.0056,13.06,.0196,14.05,.0160,15.11,.0107,16.11,.0003 ,17.14,.0111,18.21,.0085,19.23,.0010,20.28,.0048,21.31,.0128,22.36,.0051,23.41,.0041,24.05,.0006 ,25.54,.0019,26.62,.0028,27.72,.0034,28.82,.0062,29.89,.0039,30.98,.0058,32.08,.0011,33.21,.0002 ,34.37,.0008,35.46,.0018,36.62,.0036,37.77,.0018,38.92,.0042,40.07,.0037,41.23,.0011,42.67,.0003 ,43.65,.0018,44.68,.0025,45.99,.0044,47.21,.0051,48.40,.0044,49.67,.0005,50.88,.0019,52.15,.0003 ,53.42,.0008,54.69,.0010,55.98,.0005,57.26,.0013,58.53,.0027,59.83,.0011,61.21,.0027,62.54,.0003 ,63.78,.0003,65.20,.0001,66.60,.0006,67.98,.0008,69.37,.0019,70.73,.0007,72.14,.0004,73.62,.0002 ,74.40,.0003,76.52,.0006,77.97,.0002,79.49,.0004,80.77,.0003,81.00,.0001,82.47,.0005,83.97,.0001 ,87.27,.0002],
 * 			        [2.00,.0257,2.99,.0142,3.97,.0202,4.95,.0148,5.95,.0420,6.95,.0037,7.94,.0004 ,8.94,.0172,9.95,.0191,10.96,.0115,11.97,.0059,12.98,.0140,14.00,.0178,15.03,.0121,16.09,.0002 ,17.07,.0066,18.08,.0033,19.15,.0022,20.18,.0057,21.22,.0077,22.29,.0037,23.33,.0066,24.97,.0002 ,25.49,.0019,26.55,.0042,27.61,.0043,28.73,.0038,29.81,.0084,30.91,.0040,32.03,.0025,33.14,.0005 ,34.26,.0003,35.38,.0019,36.56,.0037,37.68,.0049,38.86,.0036,40.11,.0011,41.28,.0008,42.50,.0004 ,43.60,.0002,44.74,.0022,45.99,.0050,47.20,.0009,48.40,.0036,49.68,.0004,50.92,.0009,52.17,.0005 ,53.46,.0007,54.76,.0006,56.06,.0005,57.34,.0011,58.67,.0005,59.95,.0015,61.37,.0008,62.72,.0004 ,65.42,.0009,66.96,.0003,68.18,.0003,69.78,.0003,71.21,.0004,72.45,.0002,74.22,.0003,75.44,.0001 ,76.53,.0003,78.31,.0004,79.83,.0003,80.16,.0001,81.33,.0003,82.44,.0001,83.17,.0002,84.81,.0003 ,85.97,.0003,89.08,.0001,90.70,.0002,92.30,.0002,95.59,.0002,97.22,.0003,98.86,.0001,108.37,.0001 ,125.54,.0001],             # <<<<<<<<<<<<<<
 * 			        [1.99,.0650,3.03,.0040,4.03,.0059,5.02,.0090,5.97,.0227,6.98,.0050,8.04,.0020 ,9.00,.0082,9.96,.0078,11.01,.0056,12.01,.0095,13.02,.0050,14.04,.0093,15.08,.0064,16.14,.0017 ,17.06,.0020,18.10,.0025,19.14,.0023,20.18,.0015,21.24,.0032,22.29,.0029,23.32,.0014,24.37,.0005 ,25.43,.0030,26.50,.0022,27.60,.0027,28.64,.0024,29.76,.0035,30.81,.0136,31.96,.0025,33.02,.0003 ,34.13,.0005,35.25,.0007,36.40,.0014,37.51,.0020,38.64,.0012,39.80,.0019,40.97,.0004,42.09,.0003 ,43.24,.0003,44.48,.0002,45.65,.0024,46.86,.0005,48.07,.0013,49.27,.0008,50.49,.0006,52.95,.0001 ,54.23,.0005,55.45,.0004,56.73,.0001,58.03,.0003,59.29,.0002,60.59,.0003,62.04,.0002,65.89,.0002 ,67.23,.0002,68.61,.0002,69.97,.0004,71.36,.0005,85.42,.0001],
 * 			        [1.98,.0256,2.96,.0158,3.95,.0310,4.94,.0411,5.95,.0238,6.94,.0152,7.93,.0011 ,8.95,.0185,9.92,.0166,10.93,.0306,11.94,.0258,12.96,.0202,13.97,.0403,14.95,.0228,15.93,.0005 ,17.01,.0072,18.02,.0034,19.06,.0028,20.08,.0124,21.13,.0137,22.16,.0102,23.19,.0058,23.90,.0013 ,25.30,.0039,26.36,.0039,27.41,.0025,28.47,.0071,29.64,.0031,30.60,.0027,31.71,.0021,32.84,.0003 ,33.82,.0002,35.07,.0019,36.09,.0054,37.20,.0038,38.33,.0024,39.47,.0055,40.55,.0016,41.77,.0006 ,42.95,.0002,43.27,.0018,44.03,.0006,45.25,.0019,46.36,.0033,47.50,.0024,48.87,.0012,50.03,.0016 ,51.09,.0004,53.52,.0017,54.74,.0012,56.17,.0003,57.40,.0011,58.42,.0020,59.70,.0007,61.29,.0008 ,62.56,.0003,63.48,.0002,64.83,.0002,66.12,.0012,67.46,.0017,68.81,.0003,69.13,.0003,70.53,.0002 ,71.84,.0001,73.28,.0002,75.52,.0010,76.96,.0005,77.93,.0003,78.32,.0003,79.73,.0003,81.69,.0002 ,82.52,.0001,84.01,.0001,84.61,.0002,86.88,.0001,88.36,.0002,89.85,.0002,91.35,.0003,92.86,.0002 ,93.40,.0001,105.28,.0002,106.22,.0002,107.45,.0001,108.70,.0003,122.08,.0002],
 */
  __pyx_t_5 = PyList_New(160); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_float_2_00);
  __Pyx_GIVEREF(__pyx_float_2_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 0, __pyx_float_2_00)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0257);
  __Pyx_GIVEREF(__pyx_float__0257);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 1, __pyx_float__0257)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_99);
  __Pyx_GIVEREF(__pyx_float_2_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 2, __pyx_float_2_99)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0142);
  __Pyx_GIVEREF(__pyx_float__0142);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 3, __pyx_float__0142)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_97);
  __Pyx_GIVEREF(__pyx_float_3_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 4, __pyx_float_3_97)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0202);
  __Pyx_GIVEREF(__pyx_float__0202);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 5, __pyx_float__0202)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_95);
  __Pyx_GIVEREF(__pyx_float_4_95);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 6, __pyx_float_4_95)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0148);
  __Pyx_GIVEREF(__pyx_float__0148);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 7, __pyx_float__0148)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_95);
  __Pyx_GIVEREF(__pyx_float_5_95);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 8, __pyx_float_5_95)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0420);
  __Pyx_GIVEREF(__pyx_float__0420);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 9, __pyx_float__0420)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_95);
  __Pyx_GIVEREF(__pyx_float_6_95);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 10, __pyx_float_6_95)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0037);
  __Pyx_GIVEREF(__pyx_float__0037);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 11, __pyx_float__0037)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_94);
  __Pyx_GIVEREF(__pyx_float_7_94);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 12, __pyx_float_7_94)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 13, __pyx_float__0004)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_94);
  __Pyx_GIVEREF(__pyx_float_8_94);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 14, __pyx_float_8_94)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0172);
  __Pyx_GIVEREF(__pyx_float__0172);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 15, __pyx_float__0172)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_95);
  __Pyx_GIVEREF(__pyx_float_9_95);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 16, __pyx_float_9_95)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0191);
  __Pyx_GIVEREF(__pyx_float__0191);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 17, __pyx_float__0191)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_96);
  __Pyx_GIVEREF(__pyx_float_10_96);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 18, __pyx_float_10_96)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0115);
  __Pyx_GIVEREF(__pyx_float__0115);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 19, __pyx_float__0115)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_97);
  __Pyx_GIVEREF(__pyx_float_11_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 20, __pyx_float_11_97)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0059);
  __Pyx_GIVEREF(__pyx_float__0059);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 21, __pyx_float__0059)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_98);
  __Pyx_GIVEREF(__pyx_float_12_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 22, __pyx_float_12_98)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0140);
  __Pyx_GIVEREF(__pyx_float__0140);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 23, __pyx_float__0140)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_00);
  __Pyx_GIVEREF(__pyx_float_14_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 24, __pyx_float_14_00)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0178);
  __Pyx_GIVEREF(__pyx_float__0178);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 25, __pyx_float__0178)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_03);
  __Pyx_GIVEREF(__pyx_float_15_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 26, __pyx_float_15_03)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0121);
  __Pyx_GIVEREF(__pyx_float__0121);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 27, __pyx_float__0121)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_09);
  __Pyx_GIVEREF(__pyx_float_16_09);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 28, __pyx_float_16_09)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 29, __pyx_float__0002)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_07);
  __Pyx_GIVEREF(__pyx_float_17_07);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 30, __pyx_float_17_07)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0066);
  __Pyx_GIVEREF(__pyx_float__0066);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 31, __pyx_float__0066)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_08);
  __Pyx_GIVEREF(__pyx_float_18_08);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 32, __pyx_float_18_08)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0033);
  __Pyx_GIVEREF(__pyx_float__0033);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 33, __pyx_float__0033)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_15);
  __Pyx_GIVEREF(__pyx_float_19_15);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 34, __pyx_float_19_15)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0022);
  __Pyx_GIVEREF(__pyx_float__0022);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 35, __pyx_float__0022)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_18);
  __Pyx_GIVEREF(__pyx_float_20_18);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 36, __pyx_float_20_18)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0057);
  __Pyx_GIVEREF(__pyx_float__0057);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 37, __pyx_float__0057)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_22);
  __Pyx_GIVEREF(__pyx_float_21_22);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 38, __pyx_float_21_22)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0077);
  __Pyx_GIVEREF(__pyx_float__0077);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 39, __pyx_float__0077)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_29);
  __Pyx_GIVEREF(__pyx_float_22_29);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 40, __pyx_float_22_29)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0037);
  __Pyx_GIVEREF(__pyx_float__0037);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 41, __pyx_float__0037)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_33);
  __Pyx_GIVEREF(__pyx_float_23_33);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 42, __pyx_float_23_33)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0066);
  __Pyx_GIVEREF(__pyx_float__0066);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 43, __pyx_float__0066)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_24_97);
  __Pyx_GIVEREF(__pyx_float_24_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 44, __pyx_float_24_97)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 45, __pyx_float__0002)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_25_49);
  __Pyx_GIVEREF(__pyx_float_25_49);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 46, __pyx_float_25_49)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0019);
  __Pyx_GIVEREF(__pyx_float__0019);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 47, __pyx_float__0019)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_26_55);
  __Pyx_GIVEREF(__pyx_float_26_55);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 48, __pyx_float_26_55)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0042);
  __Pyx_GIVEREF(__pyx_float__0042);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 49, __pyx_float__0042)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_27_61);
  __Pyx_GIVEREF(__pyx_float_27_61);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 50, __pyx_float_27_61)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0043);
  __Pyx_GIVEREF(__pyx_float__0043);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 51, __pyx_float__0043)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_73);
  __Pyx_GIVEREF(__pyx_float_28_73);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 52, __pyx_float_28_73)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0038);
  __Pyx_GIVEREF(__pyx_float__0038);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 53, __pyx_float__0038)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_29_81);
  __Pyx_GIVEREF(__pyx_float_29_81);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 54, __pyx_float_29_81)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0084);
  __Pyx_GIVEREF(__pyx_float__0084);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 55, __pyx_float__0084)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_30_91);
  __Pyx_GIVEREF(__pyx_float_30_91);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 56, __pyx_float_30_91)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0040);
  __Pyx_GIVEREF(__pyx_float__0040);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 57, __pyx_float__0040)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_32_03);
  __Pyx_GIVEREF(__pyx_float_32_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 58, __pyx_float_32_03)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0025);
  __Pyx_GIVEREF(__pyx_float__0025);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 59, __pyx_float__0025)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_33_14);
  __Pyx_GIVEREF(__pyx_float_33_14);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 60, __pyx_float_33_14)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 61, __pyx_float__0005)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_34_26);
  __Pyx_GIVEREF(__pyx_float_34_26);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 62, __pyx_float_34_26)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 63, __pyx_float__0003)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_35_38);
  __Pyx_GIVEREF(__pyx_float_35_38);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 64, __pyx_float_35_38)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0019);
  __Pyx_GIVEREF(__pyx_float__0019);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 65, __pyx_float__0019)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_36_56);
  __Pyx_GIVEREF(__pyx_float_36_56);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 66, __pyx_float_36_56)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0037);
  __Pyx_GIVEREF(__pyx_float__0037);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 67, __pyx_float__0037)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_37_68);
  __Pyx_GIVEREF(__pyx_float_37_68);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 68, __pyx_float_37_68)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0049);
  __Pyx_GIVEREF(__pyx_float__0049);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 69, __pyx_float__0049)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_38_86);
  __Pyx_GIVEREF(__pyx_float_38_86);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 70, __pyx_float_38_86)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0036);
  __Pyx_GIVEREF(__pyx_float__0036);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 71, __pyx_float__0036)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_40_11);
  __Pyx_GIVEREF(__pyx_float_40_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 72, __pyx_float_40_11)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 73, __pyx_float__0011)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_41_28);
  __Pyx_GIVEREF(__pyx_float_41_28);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 74, __pyx_float_41_28)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 75, __pyx_float__0008)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_42_50);
  __Pyx_GIVEREF(__pyx_float_42_50);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 76, __pyx_float_42_50)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 77, __pyx_float__0004)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_43_60);
  __Pyx_GIVEREF(__pyx_float_43_60);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 78, __pyx_float_43_60)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 79, __pyx_float__0002)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_44_74);
  __Pyx_GIVEREF(__pyx_float_44_74);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 80, __pyx_float_44_74)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0022);
  __Pyx_GIVEREF(__pyx_float__0022);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 81, __pyx_float__0022)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_45_99);
  __Pyx_GIVEREF(__pyx_float_45_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 82, __pyx_float_45_99)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0050);
  __Pyx_GIVEREF(__pyx_float__0050);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 83, __pyx_float__0050)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_47_20);
  __Pyx_GIVEREF(__pyx_float_47_20);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 84, __pyx_float_47_20)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 85, __pyx_float__0009)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_48_40);
  __Pyx_GIVEREF(__pyx_float_48_40);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 86, __pyx_float_48_40)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0036);
  __Pyx_GIVEREF(__pyx_float__0036);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 87, __pyx_float__0036)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_49_68);
  __Pyx_GIVEREF(__pyx_float_49_68);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 88, __pyx_float_49_68)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 89, __pyx_float__0004)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_50_92);
  __Pyx_GIVEREF(__pyx_float_50_92);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 90, __pyx_float_50_92)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 91, __pyx_float__0009)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_52_17);
  __Pyx_GIVEREF(__pyx_float_52_17);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 92, __pyx_float_52_17)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 93, __pyx_float__0005)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_53_46);
  __Pyx_GIVEREF(__pyx_float_53_46);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 94, __pyx_float_53_46)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 95, __pyx_float__0007)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_54_76);
  __Pyx_GIVEREF(__pyx_float_54_76);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 96, __pyx_float_54_76)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 97, __pyx_float__0006)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_56_06);
  __Pyx_GIVEREF(__pyx_float_56_06);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 98, __pyx_float_56_06)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 99, __pyx_float__0005)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_57_34);
  __Pyx_GIVEREF(__pyx_float_57_34);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 100, __pyx_float_57_34)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 101, __pyx_float__0011)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_58_67);
  __Pyx_GIVEREF(__pyx_float_58_67);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 102, __pyx_float_58_67)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 103, __pyx_float__0005)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_59_95);
  __Pyx_GIVEREF(__pyx_float_59_95);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 104, __pyx_float_59_95)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0015);
  __Pyx_GIVEREF(__pyx_float__0015);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 105, __pyx_float__0015)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_61_37);
  __Pyx_GIVEREF(__pyx_float_61_37);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 106, __pyx_float_61_37)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 107, __pyx_float__0008)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_62_72);
  __Pyx_GIVEREF(__pyx_float_62_72);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 108, __pyx_float_62_72)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 109, __pyx_float__0004)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_65_42);
  __Pyx_GIVEREF(__pyx_float_65_42);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 110, __pyx_float_65_42)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 111, __pyx_float__0009)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_66_96);
  __Pyx_GIVEREF(__pyx_float_66_96);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 112, __pyx_float_66_96)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 113, __pyx_float__0003)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_68_18);
  __Pyx_GIVEREF(__pyx_float_68_18);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 114, __pyx_float_68_18)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 115, __pyx_float__0003)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_69_78);
  __Pyx_GIVEREF(__pyx_float_69_78);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 116, __pyx_float_69_78)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 117, __pyx_float__0003)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_71_21);
  __Pyx_GIVEREF(__pyx_float_71_21);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 118, __pyx_float_71_21)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 119, __pyx_float__0004)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_72_45);
  __Pyx_GIVEREF(__pyx_float_72_45);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 120, __pyx_float_72_45)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 121, __pyx_float__0002)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_74_22);
  __Pyx_GIVEREF(__pyx_float_74_22);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 122, __pyx_float_74_22)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 123, __pyx_float__0003)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_75_44);
  __Pyx_GIVEREF(__pyx_float_75_44);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 124, __pyx_float_75_44)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 125, __pyx_float__0001)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_76_53);
  __Pyx_GIVEREF(__pyx_float_76_53);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 126, __pyx_float_76_53)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 127, __pyx_float__0003)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_78_31);
  __Pyx_GIVEREF(__pyx_float_78_31);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 128, __pyx_float_78_31)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 129, __pyx_float__0004)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_79_83);
  __Pyx_GIVEREF(__pyx_float_79_83);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 130, __pyx_float_79_83)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 131, __pyx_float__0003)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_80_16);
  __Pyx_GIVEREF(__pyx_float_80_16);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 132, __pyx_float_80_16)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 133, __pyx_float__0001)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_81_33);
  __Pyx_GIVEREF(__pyx_float_81_33);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 134, __pyx_float_81_33)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 135, __pyx_float__0003)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_82_44);
  __Pyx_GIVEREF(__pyx_float_82_44);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 136, __pyx_float_82_44)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 137, __pyx_float__0001)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_83_17);
  __Pyx_GIVEREF(__pyx_float_83_17);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 138, __pyx_float_83_17)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 139, __pyx_float__0002)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_84_81);
  __Pyx_GIVEREF(__pyx_float_84_81);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 140, __pyx_float_84_81)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 141, __pyx_float__0003)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_85_97);
  __Pyx_GIVEREF(__pyx_float_85_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 142, __pyx_float_85_97)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 143, __pyx_float__0003)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_89_08);
  __Pyx_GIVEREF(__pyx_float_89_08);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 144, __pyx_float_89_08)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 145, __pyx_float__0001)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_90_70);
  __Pyx_GIVEREF(__pyx_float_90_70);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 146, __pyx_float_90_70)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 147, __pyx_float__0002)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_92_30);
  __Pyx_GIVEREF(__pyx_float_92_30);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 148, __pyx_float_92_30)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 149, __pyx_float__0002)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_95_59);
  __Pyx_GIVEREF(__pyx_float_95_59);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 150, __pyx_float_95_59)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 151, __pyx_float__0002)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_97_22);
  __Pyx_GIVEREF(__pyx_float_97_22);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 152, __pyx_float_97_22)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 153, __pyx_float__0003)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_98_86);
  __Pyx_GIVEREF(__pyx_float_98_86);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 154, __pyx_float_98_86)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 155, __pyx_float__0001)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_108_37);
  __Pyx_GIVEREF(__pyx_float_108_37);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 156, __pyx_float_108_37)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 157, __pyx_float__0001)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_125_54);
  __Pyx_GIVEREF(__pyx_float_125_54);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 158, __pyx_float_125_54)) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_5, 159, __pyx_float__0001)) __PYX_ERR(0, 31, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":32
 * 			        [2.00,.0313,2.99,.0109,4.00,.0215,5.00,.0242,5.98,.0355,7.01,.0132,8.01,.0009 ,9.01,.0071,10.00,.0258,11.03,.0221,12.02,.0056,13.06,.0196,14.05,.0160,15.11,.0107,16.11,.0003 ,17.14,.0111,18.21,.0085,19.23,.0010,20.28,.0048,21.31,.0128,22.36,.0051,23.41,.0041,24.05,.0006 ,25.54,.0019,26.62,.0028,27.72,.0034,28.82,.0062,29.89,.0039,30.98,.0058,32.08,.0011,33.21,.0002 ,34.37,.0008,35.46,.0018,36.62,.0036,37.77,.0018,38.92,.0042,40.07,.0037,41.23,.0011,42.67,.0003 ,43.65,.0018,44.68,.0025,45.99,.0044,47.21,.0051,48.40,.0044,49.67,.0005,50.88,.0019,52.15,.0003 ,53.42,.0008,54.69,.0010,55.98,.0005,57.26,.0013,58.53,.0027,59.83,.0011,61.21,.0027,62.54,.0003 ,63.78,.0003,65.20,.0001,66.60,.0006,67.98,.0008,69.37,.0019,70.73,.0007,72.14,.0004,73.62,.0002 ,74.40,.0003,76.52,.0006,77.97,.0002,79.49,.0004,80.77,.0003,81.00,.0001,82.47,.0005,83.97,.0001 ,87.27,.0002],
 * 			        [2.00,.0257,2.99,.0142,3.97,.0202,4.95,.0148,5.95,.0420,6.95,.0037,7.94,.0004 ,8.94,.0172,9.95,.0191,10.96,.0115,11.97,.0059,12.98,.0140,14.00,.0178,15.03,.0121,16.09,.0002 ,17.07,.0066,18.08,.0033,19.15,.0022,20.18,.0057,21.22,.0077,22.29,.0037,23.33,.0066,24.97,.0002 ,25.49,.0019,26.55,.0042,27.61,.0043,28.73,.0038,29.81,.0084,30.91,.0040,32.03,.0025,33.14,.0005 ,34.26,.0003,35.38,.0019,36.56,.0037,37.68,.0049,38.86,.0036,40.11,.0011,41.28,.0008,42.50,.0004 ,43.60,.0002,44.74,.0022,45.99,.0050,47.20,.0009,48.40,.0036,49.68,.0004,50.92,.0009,52.17,.0005 ,53.46,.0007,54.76,.0006,56.06,.0005,57.34,.0011,58.67,.0005,59.95,.0015,61.37,.0008,62.72,.0004 ,65.42,.0009,66.96,.0003,68.18,.0003,69.78,.0003,71.21,.0004,72.45,.0002,74.22,.0003,75.44,.0001 ,76.53,.0003,78.31,.0004,79.83,.0003,80.16,.0001,81.33,.0003,82.44,.0001,83.17,.0002,84.81,.0003 ,85.97,.0003,89.08,.0001,90.70,.0002,92.30,.0002,95.59,.0002,97.22,.0003,98.86,.0001,108.37,.0001 ,125.54,.0001],
 * 			        [1.99,.0650,3.03,.0040,4.03,.0059,5.02,.0090,5.97,.0227,6.98,.0050,8.04,.0020 ,9.00,.0082,9.96,.0078,11.01,.0056,12.01,.0095,13.02,.0050,14.04,.0093,15.08,.0064,16.14,.0017 ,17.06,.0020,18.10,.0025,19.14,.0023,20.18,.0015,21.24,.0032,22.29,.0029,23.32,.0014,24.37,.0005 ,25.43,.0030,26.50,.0022,27.60,.0027,28.64,.0024,29.76,.0035,30.81,.0136,31.96,.0025,33.02,.0003 ,34.13,.0005,35.25,.0007,36.40,.0014,37.51,.0020,38.64,.0012,39.80,.0019,40.97,.0004,42.09,.0003 ,43.24,.0003,44.48,.0002,45.65,.0024,46.86,.0005,48.07,.0013,49.27,.0008,50.49,.0006,52.95,.0001 ,54.23,.0005,55.45,.0004,56.73,.0001,58.03,.0003,59.29,.0002,60.59,.0003,62.04,.0002,65.89,.0002 ,67.23,.0002,68.61,.0002,69.97,.0004,71.36,.0005,85.42,.0001],             # <<<<<<<<<<<<<<
 * 			        [1.98,.0256,2.96,.0158,3.95,.0310,4.94,.0411,5.95,.0238,6.94,.0152,7.93,.0011 ,8.95,.0185,9.92,.0166,10.93,.0306,11.94,.0258,12.96,.0202,13.97,.0403,14.95,.0228,15.93,.0005 ,17.01,.0072,18.02,.0034,19.06,.0028,20.08,.0124,21.13,.0137,22.16,.0102,23.19,.0058,23.90,.0013 ,25.30,.0039,26.36,.0039,27.41,.0025,28.47,.0071,29.64,.0031,30.60,.0027,31.71,.0021,32.84,.0003 ,33.82,.0002,35.07,.0019,36.09,.0054,37.20,.0038,38.33,.0024,39.47,.0055,40.55,.0016,41.77,.0006 ,42.95,.0002,43.27,.0018,44.03,.0006,45.25,.0019,46.36,.0033,47.50,.0024,48.87,.0012,50.03,.0016 ,51.09,.0004,53.52,.0017,54.74,.0012,56.17,.0003,57.40,.0011,58.42,.0020,59.70,.0007,61.29,.0008 ,62.56,.0003,63.48,.0002,64.83,.0002,66.12,.0012,67.46,.0017,68.81,.0003,69.13,.0003,70.53,.0002 ,71.84,.0001,73.28,.0002,75.52,.0010,76.96,.0005,77.93,.0003,78.32,.0003,79.73,.0003,81.69,.0002 ,82.52,.0001,84.01,.0001,84.61,.0002,86.88,.0001,88.36,.0002,89.85,.0002,91.35,.0003,92.86,.0002 ,93.40,.0001,105.28,.0002,106.22,.0002,107.45,.0001,108.70,.0003,122.08,.0002],
 * 			        [1.97,.0264,2.97,.0211,3.98,.0234,4.98,.0307,5.96,.0085,6.94,.0140,7.93,.0005 ,8.96,.0112,9.96,.0209,10.98,.0194,11.98,.0154,12.99,.0274,13.99,.0127,15.01,.0101,15.99,.0002 ,17.04,.0011,18.08,.0032,19.14,.0028,20.12,.0054,21.20,.0053,22.13,.0028,23.22,.0030,24.32,.0006 ,25.24,.0004,26.43,.0028,27.53,.0048,28.52,.0039,29.54,.0047,30.73,.0044,31.82,.0007,32.94,.0008 ,34.04,.0012,35.13,.0018,36.29,.0007,37.35,.0075,38.51,.0045,39.66,.0014,40.90,.0004,41.90,.0002 ,43.08,.0002,44.24,.0017,45.36,.0013,46.68,.0020,47.79,.0015,48.98,.0010,50.21,.0012,51.34,.0001 ,53.82,.0003,55.09,.0004,56.23,.0005,57.53,.0004,58.79,.0005,59.30,.0002,60.03,.0002,61.40,.0003 ,62.84,.0001,66.64,.0001,67.97,.0001,69.33,.0001,70.68,.0001,73.57,.0002,75.76,.0002,76.45,.0001 ,79.27,.0001,80.44,.0002,81.87,.0002],
 */
  __pyx_t_6 = PyList_New(120); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_float_1_99);
  __Pyx_GIVEREF(__pyx_float_1_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 0, __pyx_float_1_99)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0650);
  __Pyx_GIVEREF(__pyx_float__0650);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 1, __pyx_float__0650)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_03);
  __Pyx_GIVEREF(__pyx_float_3_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 2, __pyx_float_3_03)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0040);
  __Pyx_GIVEREF(__pyx_float__0040);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 3, __pyx_float__0040)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_03);
  __Pyx_GIVEREF(__pyx_float_4_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 4, __pyx_float_4_03)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0059);
  __Pyx_GIVEREF(__pyx_float__0059);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 5, __pyx_float__0059)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_02);
  __Pyx_GIVEREF(__pyx_float_5_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 6, __pyx_float_5_02)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0090);
  __Pyx_GIVEREF(__pyx_float__0090);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 7, __pyx_float__0090)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_97);
  __Pyx_GIVEREF(__pyx_float_5_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 8, __pyx_float_5_97)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0227);
  __Pyx_GIVEREF(__pyx_float__0227);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 9, __pyx_float__0227)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_98);
  __Pyx_GIVEREF(__pyx_float_6_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 10, __pyx_float_6_98)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0050);
  __Pyx_GIVEREF(__pyx_float__0050);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 11, __pyx_float__0050)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_04);
  __Pyx_GIVEREF(__pyx_float_8_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 12, __pyx_float_8_04)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0020);
  __Pyx_GIVEREF(__pyx_float__0020);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 13, __pyx_float__0020)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_00);
  __Pyx_GIVEREF(__pyx_float_9_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 14, __pyx_float_9_00)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0082);
  __Pyx_GIVEREF(__pyx_float__0082);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 15, __pyx_float__0082)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_96);
  __Pyx_GIVEREF(__pyx_float_9_96);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 16, __pyx_float_9_96)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0078);
  __Pyx_GIVEREF(__pyx_float__0078);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 17, __pyx_float__0078)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_01);
  __Pyx_GIVEREF(__pyx_float_11_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 18, __pyx_float_11_01)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0056);
  __Pyx_GIVEREF(__pyx_float__0056);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 19, __pyx_float__0056)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_01);
  __Pyx_GIVEREF(__pyx_float_12_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 20, __pyx_float_12_01)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0095);
  __Pyx_GIVEREF(__pyx_float__0095);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 21, __pyx_float__0095)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_02);
  __Pyx_GIVEREF(__pyx_float_13_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 22, __pyx_float_13_02)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0050);
  __Pyx_GIVEREF(__pyx_float__0050);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 23, __pyx_float__0050)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_04);
  __Pyx_GIVEREF(__pyx_float_14_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 24, __pyx_float_14_04)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0093);
  __Pyx_GIVEREF(__pyx_float__0093);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 25, __pyx_float__0093)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_08);
  __Pyx_GIVEREF(__pyx_float_15_08);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 26, __pyx_float_15_08)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0064);
  __Pyx_GIVEREF(__pyx_float__0064);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 27, __pyx_float__0064)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_14);
  __Pyx_GIVEREF(__pyx_float_16_14);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 28, __pyx_float_16_14)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0017);
  __Pyx_GIVEREF(__pyx_float__0017);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 29, __pyx_float__0017)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_06);
  __Pyx_GIVEREF(__pyx_float_17_06);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 30, __pyx_float_17_06)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0020);
  __Pyx_GIVEREF(__pyx_float__0020);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 31, __pyx_float__0020)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_10);
  __Pyx_GIVEREF(__pyx_float_18_10);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 32, __pyx_float_18_10)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0025);
  __Pyx_GIVEREF(__pyx_float__0025);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 33, __pyx_float__0025)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_14);
  __Pyx_GIVEREF(__pyx_float_19_14);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 34, __pyx_float_19_14)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0023);
  __Pyx_GIVEREF(__pyx_float__0023);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 35, __pyx_float__0023)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_18);
  __Pyx_GIVEREF(__pyx_float_20_18);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 36, __pyx_float_20_18)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0015);
  __Pyx_GIVEREF(__pyx_float__0015);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 37, __pyx_float__0015)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_24);
  __Pyx_GIVEREF(__pyx_float_21_24);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 38, __pyx_float_21_24)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0032);
  __Pyx_GIVEREF(__pyx_float__0032);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 39, __pyx_float__0032)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_29);
  __Pyx_GIVEREF(__pyx_float_22_29);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 40, __pyx_float_22_29)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0029);
  __Pyx_GIVEREF(__pyx_float__0029);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 41, __pyx_float__0029)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_32);
  __Pyx_GIVEREF(__pyx_float_23_32);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 42, __pyx_float_23_32)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0014);
  __Pyx_GIVEREF(__pyx_float__0014);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 43, __pyx_float__0014)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_24_37);
  __Pyx_GIVEREF(__pyx_float_24_37);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 44, __pyx_float_24_37)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 45, __pyx_float__0005)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_25_43);
  __Pyx_GIVEREF(__pyx_float_25_43);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 46, __pyx_float_25_43)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0030);
  __Pyx_GIVEREF(__pyx_float__0030);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 47, __pyx_float__0030)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_26_50);
  __Pyx_GIVEREF(__pyx_float_26_50);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 48, __pyx_float_26_50)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0022);
  __Pyx_GIVEREF(__pyx_float__0022);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 49, __pyx_float__0022)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_27_60);
  __Pyx_GIVEREF(__pyx_float_27_60);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 50, __pyx_float_27_60)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0027);
  __Pyx_GIVEREF(__pyx_float__0027);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 51, __pyx_float__0027)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_64);
  __Pyx_GIVEREF(__pyx_float_28_64);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 52, __pyx_float_28_64)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0024);
  __Pyx_GIVEREF(__pyx_float__0024);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 53, __pyx_float__0024)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_29_76);
  __Pyx_GIVEREF(__pyx_float_29_76);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 54, __pyx_float_29_76)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0035);
  __Pyx_GIVEREF(__pyx_float__0035);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 55, __pyx_float__0035)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_30_81);
  __Pyx_GIVEREF(__pyx_float_30_81);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 56, __pyx_float_30_81)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0136);
  __Pyx_GIVEREF(__pyx_float__0136);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 57, __pyx_float__0136)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_31_96);
  __Pyx_GIVEREF(__pyx_float_31_96);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 58, __pyx_float_31_96)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0025);
  __Pyx_GIVEREF(__pyx_float__0025);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 59, __pyx_float__0025)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_33_02);
  __Pyx_GIVEREF(__pyx_float_33_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 60, __pyx_float_33_02)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 61, __pyx_float__0003)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_34_13);
  __Pyx_GIVEREF(__pyx_float_34_13);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 62, __pyx_float_34_13)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 63, __pyx_float__0005)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_35_25);
  __Pyx_GIVEREF(__pyx_float_35_25);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 64, __pyx_float_35_25)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 65, __pyx_float__0007)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_36_40);
  __Pyx_GIVEREF(__pyx_float_36_40);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 66, __pyx_float_36_40)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0014);
  __Pyx_GIVEREF(__pyx_float__0014);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 67, __pyx_float__0014)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_37_51);
  __Pyx_GIVEREF(__pyx_float_37_51);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 68, __pyx_float_37_51)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0020);
  __Pyx_GIVEREF(__pyx_float__0020);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 69, __pyx_float__0020)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_38_64);
  __Pyx_GIVEREF(__pyx_float_38_64);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 70, __pyx_float_38_64)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 71, __pyx_float__0012)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_39_80);
  __Pyx_GIVEREF(__pyx_float_39_80);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 72, __pyx_float_39_80)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0019);
  __Pyx_GIVEREF(__pyx_float__0019);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 73, __pyx_float__0019)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_40_97);
  __Pyx_GIVEREF(__pyx_float_40_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 74, __pyx_float_40_97)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 75, __pyx_float__0004)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_42_09);
  __Pyx_GIVEREF(__pyx_float_42_09);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 76, __pyx_float_42_09)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 77, __pyx_float__0003)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_43_24);
  __Pyx_GIVEREF(__pyx_float_43_24);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 78, __pyx_float_43_24)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 79, __pyx_float__0003)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_44_48);
  __Pyx_GIVEREF(__pyx_float_44_48);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 80, __pyx_float_44_48)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 81, __pyx_float__0002)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_45_65);
  __Pyx_GIVEREF(__pyx_float_45_65);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 82, __pyx_float_45_65)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0024);
  __Pyx_GIVEREF(__pyx_float__0024);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 83, __pyx_float__0024)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_46_86);
  __Pyx_GIVEREF(__pyx_float_46_86);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 84, __pyx_float_46_86)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 85, __pyx_float__0005)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_48_07);
  __Pyx_GIVEREF(__pyx_float_48_07);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 86, __pyx_float_48_07)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0013);
  __Pyx_GIVEREF(__pyx_float__0013);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 87, __pyx_float__0013)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_49_27);
  __Pyx_GIVEREF(__pyx_float_49_27);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 88, __pyx_float_49_27)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 89, __pyx_float__0008)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_50_49);
  __Pyx_GIVEREF(__pyx_float_50_49);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 90, __pyx_float_50_49)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 91, __pyx_float__0006)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_52_95);
  __Pyx_GIVEREF(__pyx_float_52_95);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 92, __pyx_float_52_95)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 93, __pyx_float__0001)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_54_23);
  __Pyx_GIVEREF(__pyx_float_54_23);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 94, __pyx_float_54_23)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 95, __pyx_float__0005)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_55_45);
  __Pyx_GIVEREF(__pyx_float_55_45);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 96, __pyx_float_55_45)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 97, __pyx_float__0004)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_56_73);
  __Pyx_GIVEREF(__pyx_float_56_73);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 98, __pyx_float_56_73)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 99, __pyx_float__0001)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_58_03);
  __Pyx_GIVEREF(__pyx_float_58_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 100, __pyx_float_58_03)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 101, __pyx_float__0003)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_59_29);
  __Pyx_GIVEREF(__pyx_float_59_29);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 102, __pyx_float_59_29)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 103, __pyx_float__0002)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_60_59);
  __Pyx_GIVEREF(__pyx_float_60_59);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 104, __pyx_float_60_59)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 105, __pyx_float__0003)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_62_04);
  __Pyx_GIVEREF(__pyx_float_62_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 106, __pyx_float_62_04)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 107, __pyx_float__0002)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_65_89);
  __Pyx_GIVEREF(__pyx_float_65_89);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 108, __pyx_float_65_89)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 109, __pyx_float__0002)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_67_23);
  __Pyx_GIVEREF(__pyx_float_67_23);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 110, __pyx_float_67_23)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 111, __pyx_float__0002)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_68_61);
  __Pyx_GIVEREF(__pyx_float_68_61);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 112, __pyx_float_68_61)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 113, __pyx_float__0002)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_69_97);
  __Pyx_GIVEREF(__pyx_float_69_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 114, __pyx_float_69_97)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 115, __pyx_float__0004)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_71_36);
  __Pyx_GIVEREF(__pyx_float_71_36);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 116, __pyx_float_71_36)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 117, __pyx_float__0005)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_85_42);
  __Pyx_GIVEREF(__pyx_float_85_42);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 118, __pyx_float_85_42)) __PYX_ERR(0, 32, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_6, 119, __pyx_float__0001)) __PYX_ERR(0, 32, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":33
 * 			        [2.00,.0257,2.99,.0142,3.97,.0202,4.95,.0148,5.95,.0420,6.95,.0037,7.94,.0004 ,8.94,.0172,9.95,.0191,10.96,.0115,11.97,.0059,12.98,.0140,14.00,.0178,15.03,.0121,16.09,.0002 ,17.07,.0066,18.08,.0033,19.15,.0022,20.18,.0057,21.22,.0077,22.29,.0037,23.33,.0066,24.97,.0002 ,25.49,.0019,26.55,.0042,27.61,.0043,28.73,.0038,29.81,.0084,30.91,.0040,32.03,.0025,33.14,.0005 ,34.26,.0003,35.38,.0019,36.56,.0037,37.68,.0049,38.86,.0036,40.11,.0011,41.28,.0008,42.50,.0004 ,43.60,.0002,44.74,.0022,45.99,.0050,47.20,.0009,48.40,.0036,49.68,.0004,50.92,.0009,52.17,.0005 ,53.46,.0007,54.76,.0006,56.06,.0005,57.34,.0011,58.67,.0005,59.95,.0015,61.37,.0008,62.72,.0004 ,65.42,.0009,66.96,.0003,68.18,.0003,69.78,.0003,71.21,.0004,72.45,.0002,74.22,.0003,75.44,.0001 ,76.53,.0003,78.31,.0004,79.83,.0003,80.16,.0001,81.33,.0003,82.44,.0001,83.17,.0002,84.81,.0003 ,85.97,.0003,89.08,.0001,90.70,.0002,92.30,.0002,95.59,.0002,97.22,.0003,98.86,.0001,108.37,.0001 ,125.54,.0001],
 * 			        [1.99,.0650,3.03,.0040,4.03,.0059,5.02,.0090,5.97,.0227,6.98,.0050,8.04,.0020 ,9.00,.0082,9.96,.0078,11.01,.0056,12.01,.0095,13.02,.0050,14.04,.0093,15.08,.0064,16.14,.0017 ,17.06,.0020,18.10,.0025,19.14,.0023,20.18,.0015,21.24,.0032,22.29,.0029,23.32,.0014,24.37,.0005 ,25.43,.0030,26.50,.0022,27.60,.0027,28.64,.0024,29.76,.0035,30.81,.0136,31.96,.0025,33.02,.0003 ,34.13,.0005,35.25,.0007,36.40,.0014,37.51,.0020,38.64,.0012,39.80,.0019,40.97,.0004,42.09,.0003 ,43.24,.0003,44.48,.0002,45.65,.0024,46.86,.0005,48.07,.0013,49.27,.0008,50.49,.0006,52.95,.0001 ,54.23,.0005,55.45,.0004,56.73,.0001,58.03,.0003,59.29,.0002,60.59,.0003,62.04,.0002,65.89,.0002 ,67.23,.0002,68.61,.0002,69.97,.0004,71.36,.0005,85.42,.0001],
 * 			        [1.98,.0256,2.96,.0158,3.95,.0310,4.94,.0411,5.95,.0238,6.94,.0152,7.93,.0011 ,8.95,.0185,9.92,.0166,10.93,.0306,11.94,.0258,12.96,.0202,13.97,.0403,14.95,.0228,15.93,.0005 ,17.01,.0072,18.02,.0034,19.06,.0028,20.08,.0124,21.13,.0137,22.16,.0102,23.19,.0058,23.90,.0013 ,25.30,.0039,26.36,.0039,27.41,.0025,28.47,.0071,29.64,.0031,30.60,.0027,31.71,.0021,32.84,.0003 ,33.82,.0002,35.07,.0019,36.09,.0054,37.20,.0038,38.33,.0024,39.47,.0055,40.55,.0016,41.77,.0006 ,42.95,.0002,43.27,.0018,44.03,.0006,45.25,.0019,46.36,.0033,47.50,.0024,48.87,.0012,50.03,.0016 ,51.09,.0004,53.52,.0017,54.74,.0012,56.17,.0003,57.40,.0011,58.42,.0020,59.70,.0007,61.29,.0008 ,62.56,.0003,63.48,.0002,64.83,.0002,66.12,.0012,67.46,.0017,68.81,.0003,69.13,.0003,70.53,.0002 ,71.84,.0001,73.28,.0002,75.52,.0010,76.96,.0005,77.93,.0003,78.32,.0003,79.73,.0003,81.69,.0002 ,82.52,.0001,84.01,.0001,84.61,.0002,86.88,.0001,88.36,.0002,89.85,.0002,91.35,.0003,92.86,.0002 ,93.40,.0001,105.28,.0002,106.22,.0002,107.45,.0001,108.70,.0003,122.08,.0002],             # <<<<<<<<<<<<<<
 * 			        [1.97,.0264,2.97,.0211,3.98,.0234,4.98,.0307,5.96,.0085,6.94,.0140,7.93,.0005 ,8.96,.0112,9.96,.0209,10.98,.0194,11.98,.0154,12.99,.0274,13.99,.0127,15.01,.0101,15.99,.0002 ,17.04,.0011,18.08,.0032,19.14,.0028,20.12,.0054,21.20,.0053,22.13,.0028,23.22,.0030,24.32,.0006 ,25.24,.0004,26.43,.0028,27.53,.0048,28.52,.0039,29.54,.0047,30.73,.0044,31.82,.0007,32.94,.0008 ,34.04,.0012,35.13,.0018,36.29,.0007,37.35,.0075,38.51,.0045,39.66,.0014,40.90,.0004,41.90,.0002 ,43.08,.0002,44.24,.0017,45.36,.0013,46.68,.0020,47.79,.0015,48.98,.0010,50.21,.0012,51.34,.0001 ,53.82,.0003,55.09,.0004,56.23,.0005,57.53,.0004,58.79,.0005,59.30,.0002,60.03,.0002,61.40,.0003 ,62.84,.0001,66.64,.0001,67.97,.0001,69.33,.0001,70.68,.0001,73.57,.0002,75.76,.0002,76.45,.0001 ,79.27,.0001,80.44,.0002,81.87,.0002],
 * 			        [2.00,.0311,2.99,.0086,3.99,.0266,4.97,.0123,5.98,.0235,6.97,.0161,7.97,.0008 ,8.96,.0088,9.96,.0621,10.99,.0080,11.99,.0034,12.99,.0300,14.03,.0228,15.04,.0105,16.03,.0004 ,17.06,.0036,18.09,.0094,18.95,.0009,20.17,.0071,21.21,.0161,22.25,.0106,23.28,.0104,24.33,.0008 ,25.38,.0030,26.46,.0035,27.50,.0026,28.59,.0028,29.66,.0128,30.75,.0139,31.81,.0038,32.93,.0006 ,34.04,.0004,35.16,.0005,36.25,.0023,37.35,.0012,38.46,.0021,39.59,.0035,40.71,.0006,41.86,.0007 ,42.42,.0001,43.46,.0003,44.17,.0032,45.29,.0013,46.57,.0004,47.72,.0011,48.79,.0005,50.11,.0005 ,51.29,.0003,52.47,.0002,53.68,.0004,55.02,.0005,56.18,.0003,57.41,.0003,58.75,.0007,59.33,.0009 ,60.00,.0004,61.34,.0001,64.97,.0003,65.20,.0002,66.48,.0002,67.83,.0002,68.90,.0003,70.25,.0003 ,71.59,.0002,73.68,.0001,75.92,.0001,77.08,.0002,78.45,.0002,81.56,.0002,82.99,.0001,88.39,.0001],
 */
  __pyx_t_7 = PyList_New(170); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_INCREF(__pyx_float_1_98);
  __Pyx_GIVEREF(__pyx_float_1_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 0, __pyx_float_1_98)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0256);
  __Pyx_GIVEREF(__pyx_float__0256);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 1, __pyx_float__0256)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_96);
  __Pyx_GIVEREF(__pyx_float_2_96);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 2, __pyx_float_2_96)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0158);
  __Pyx_GIVEREF(__pyx_float__0158);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 3, __pyx_float__0158)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_95);
  __Pyx_GIVEREF(__pyx_float_3_95);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 4, __pyx_float_3_95)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0310);
  __Pyx_GIVEREF(__pyx_float__0310);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 5, __pyx_float__0310)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_94);
  __Pyx_GIVEREF(__pyx_float_4_94);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 6, __pyx_float_4_94)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0411);
  __Pyx_GIVEREF(__pyx_float__0411);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 7, __pyx_float__0411)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_95);
  __Pyx_GIVEREF(__pyx_float_5_95);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 8, __pyx_float_5_95)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0238);
  __Pyx_GIVEREF(__pyx_float__0238);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 9, __pyx_float__0238)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_94);
  __Pyx_GIVEREF(__pyx_float_6_94);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 10, __pyx_float_6_94)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0152);
  __Pyx_GIVEREF(__pyx_float__0152);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 11, __pyx_float__0152)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_93);
  __Pyx_GIVEREF(__pyx_float_7_93);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 12, __pyx_float_7_93)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 13, __pyx_float__0011)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_95);
  __Pyx_GIVEREF(__pyx_float_8_95);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 14, __pyx_float_8_95)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0185);
  __Pyx_GIVEREF(__pyx_float__0185);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 15, __pyx_float__0185)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_92);
  __Pyx_GIVEREF(__pyx_float_9_92);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 16, __pyx_float_9_92)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0166);
  __Pyx_GIVEREF(__pyx_float__0166);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 17, __pyx_float__0166)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_93);
  __Pyx_GIVEREF(__pyx_float_10_93);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 18, __pyx_float_10_93)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0306);
  __Pyx_GIVEREF(__pyx_float__0306);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 19, __pyx_float__0306)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_94);
  __Pyx_GIVEREF(__pyx_float_11_94);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 20, __pyx_float_11_94)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0258);
  __Pyx_GIVEREF(__pyx_float__0258);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 21, __pyx_float__0258)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_96);
  __Pyx_GIVEREF(__pyx_float_12_96);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 22, __pyx_float_12_96)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0202);
  __Pyx_GIVEREF(__pyx_float__0202);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 23, __pyx_float__0202)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_97);
  __Pyx_GIVEREF(__pyx_float_13_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 24, __pyx_float_13_97)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0403);
  __Pyx_GIVEREF(__pyx_float__0403);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 25, __pyx_float__0403)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_95);
  __Pyx_GIVEREF(__pyx_float_14_95);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 26, __pyx_float_14_95)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0228);
  __Pyx_GIVEREF(__pyx_float__0228);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 27, __pyx_float__0228)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_93);
  __Pyx_GIVEREF(__pyx_float_15_93);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 28, __pyx_float_15_93)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 29, __pyx_float__0005)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_01);
  __Pyx_GIVEREF(__pyx_float_17_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 30, __pyx_float_17_01)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0072);
  __Pyx_GIVEREF(__pyx_float__0072);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 31, __pyx_float__0072)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_02);
  __Pyx_GIVEREF(__pyx_float_18_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 32, __pyx_float_18_02)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0034);
  __Pyx_GIVEREF(__pyx_float__0034);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 33, __pyx_float__0034)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_06);
  __Pyx_GIVEREF(__pyx_float_19_06);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 34, __pyx_float_19_06)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0028);
  __Pyx_GIVEREF(__pyx_float__0028);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 35, __pyx_float__0028)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_08);
  __Pyx_GIVEREF(__pyx_float_20_08);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 36, __pyx_float_20_08)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0124);
  __Pyx_GIVEREF(__pyx_float__0124);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 37, __pyx_float__0124)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_13);
  __Pyx_GIVEREF(__pyx_float_21_13);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 38, __pyx_float_21_13)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0137);
  __Pyx_GIVEREF(__pyx_float__0137);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 39, __pyx_float__0137)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_16);
  __Pyx_GIVEREF(__pyx_float_22_16);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 40, __pyx_float_22_16)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0102);
  __Pyx_GIVEREF(__pyx_float__0102);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 41, __pyx_float__0102)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_19);
  __Pyx_GIVEREF(__pyx_float_23_19);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 42, __pyx_float_23_19)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0058);
  __Pyx_GIVEREF(__pyx_float__0058);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 43, __pyx_float__0058)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_90);
  __Pyx_GIVEREF(__pyx_float_23_90);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 44, __pyx_float_23_90)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0013);
  __Pyx_GIVEREF(__pyx_float__0013);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 45, __pyx_float__0013)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_25_30);
  __Pyx_GIVEREF(__pyx_float_25_30);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 46, __pyx_float_25_30)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0039);
  __Pyx_GIVEREF(__pyx_float__0039);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 47, __pyx_float__0039)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_26_36);
  __Pyx_GIVEREF(__pyx_float_26_36);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 48, __pyx_float_26_36)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0039);
  __Pyx_GIVEREF(__pyx_float__0039);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 49, __pyx_float__0039)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_27_41);
  __Pyx_GIVEREF(__pyx_float_27_41);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 50, __pyx_float_27_41)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0025);
  __Pyx_GIVEREF(__pyx_float__0025);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 51, __pyx_float__0025)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_47);
  __Pyx_GIVEREF(__pyx_float_28_47);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 52, __pyx_float_28_47)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0071);
  __Pyx_GIVEREF(__pyx_float__0071);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 53, __pyx_float__0071)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_29_64);
  __Pyx_GIVEREF(__pyx_float_29_64);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 54, __pyx_float_29_64)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0031);
  __Pyx_GIVEREF(__pyx_float__0031);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 55, __pyx_float__0031)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_30_60);
  __Pyx_GIVEREF(__pyx_float_30_60);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 56, __pyx_float_30_60)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0027);
  __Pyx_GIVEREF(__pyx_float__0027);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 57, __pyx_float__0027)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_31_71);
  __Pyx_GIVEREF(__pyx_float_31_71);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 58, __pyx_float_31_71)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0021);
  __Pyx_GIVEREF(__pyx_float__0021);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 59, __pyx_float__0021)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_32_84);
  __Pyx_GIVEREF(__pyx_float_32_84);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 60, __pyx_float_32_84)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 61, __pyx_float__0003)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_33_82);
  __Pyx_GIVEREF(__pyx_float_33_82);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 62, __pyx_float_33_82)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 63, __pyx_float__0002)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_35_07);
  __Pyx_GIVEREF(__pyx_float_35_07);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 64, __pyx_float_35_07)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0019);
  __Pyx_GIVEREF(__pyx_float__0019);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 65, __pyx_float__0019)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_36_09);
  __Pyx_GIVEREF(__pyx_float_36_09);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 66, __pyx_float_36_09)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0054);
  __Pyx_GIVEREF(__pyx_float__0054);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 67, __pyx_float__0054)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_37_20);
  __Pyx_GIVEREF(__pyx_float_37_20);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 68, __pyx_float_37_20)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0038);
  __Pyx_GIVEREF(__pyx_float__0038);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 69, __pyx_float__0038)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_38_33);
  __Pyx_GIVEREF(__pyx_float_38_33);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 70, __pyx_float_38_33)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0024);
  __Pyx_GIVEREF(__pyx_float__0024);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 71, __pyx_float__0024)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_39_47);
  __Pyx_GIVEREF(__pyx_float_39_47);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 72, __pyx_float_39_47)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0055);
  __Pyx_GIVEREF(__pyx_float__0055);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 73, __pyx_float__0055)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_40_55);
  __Pyx_GIVEREF(__pyx_float_40_55);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 74, __pyx_float_40_55)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0016);
  __Pyx_GIVEREF(__pyx_float__0016);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 75, __pyx_float__0016)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_41_77);
  __Pyx_GIVEREF(__pyx_float_41_77);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 76, __pyx_float_41_77)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 77, __pyx_float__0006)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_42_95);
  __Pyx_GIVEREF(__pyx_float_42_95);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 78, __pyx_float_42_95)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 79, __pyx_float__0002)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_43_27);
  __Pyx_GIVEREF(__pyx_float_43_27);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 80, __pyx_float_43_27)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0018);
  __Pyx_GIVEREF(__pyx_float__0018);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 81, __pyx_float__0018)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_44_03);
  __Pyx_GIVEREF(__pyx_float_44_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 82, __pyx_float_44_03)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 83, __pyx_float__0006)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_45_25);
  __Pyx_GIVEREF(__pyx_float_45_25);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 84, __pyx_float_45_25)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0019);
  __Pyx_GIVEREF(__pyx_float__0019);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 85, __pyx_float__0019)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_46_36);
  __Pyx_GIVEREF(__pyx_float_46_36);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 86, __pyx_float_46_36)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0033);
  __Pyx_GIVEREF(__pyx_float__0033);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 87, __pyx_float__0033)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_47_50);
  __Pyx_GIVEREF(__pyx_float_47_50);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 88, __pyx_float_47_50)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0024);
  __Pyx_GIVEREF(__pyx_float__0024);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 89, __pyx_float__0024)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_48_87);
  __Pyx_GIVEREF(__pyx_float_48_87);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 90, __pyx_float_48_87)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 91, __pyx_float__0012)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_50_03);
  __Pyx_GIVEREF(__pyx_float_50_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 92, __pyx_float_50_03)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0016);
  __Pyx_GIVEREF(__pyx_float__0016);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 93, __pyx_float__0016)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_51_09);
  __Pyx_GIVEREF(__pyx_float_51_09);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 94, __pyx_float_51_09)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 95, __pyx_float__0004)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_53_52);
  __Pyx_GIVEREF(__pyx_float_53_52);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 96, __pyx_float_53_52)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0017);
  __Pyx_GIVEREF(__pyx_float__0017);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 97, __pyx_float__0017)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_54_74);
  __Pyx_GIVEREF(__pyx_float_54_74);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 98, __pyx_float_54_74)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 99, __pyx_float__0012)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_56_17);
  __Pyx_GIVEREF(__pyx_float_56_17);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 100, __pyx_float_56_17)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 101, __pyx_float__0003)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_57_40);
  __Pyx_GIVEREF(__pyx_float_57_40);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 102, __pyx_float_57_40)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 103, __pyx_float__0011)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_58_42);
  __Pyx_GIVEREF(__pyx_float_58_42);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 104, __pyx_float_58_42)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0020);
  __Pyx_GIVEREF(__pyx_float__0020);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 105, __pyx_float__0020)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_59_70);
  __Pyx_GIVEREF(__pyx_float_59_70);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 106, __pyx_float_59_70)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 107, __pyx_float__0007)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_61_29);
  __Pyx_GIVEREF(__pyx_float_61_29);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 108, __pyx_float_61_29)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 109, __pyx_float__0008)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_62_56);
  __Pyx_GIVEREF(__pyx_float_62_56);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 110, __pyx_float_62_56)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 111, __pyx_float__0003)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_63_48);
  __Pyx_GIVEREF(__pyx_float_63_48);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 112, __pyx_float_63_48)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 113, __pyx_float__0002)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_64_83);
  __Pyx_GIVEREF(__pyx_float_64_83);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 114, __pyx_float_64_83)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 115, __pyx_float__0002)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_66_12);
  __Pyx_GIVEREF(__pyx_float_66_12);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 116, __pyx_float_66_12)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 117, __pyx_float__0012)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_67_46);
  __Pyx_GIVEREF(__pyx_float_67_46);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 118, __pyx_float_67_46)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0017);
  __Pyx_GIVEREF(__pyx_float__0017);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 119, __pyx_float__0017)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_68_81);
  __Pyx_GIVEREF(__pyx_float_68_81);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 120, __pyx_float_68_81)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 121, __pyx_float__0003)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_69_13);
  __Pyx_GIVEREF(__pyx_float_69_13);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 122, __pyx_float_69_13)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 123, __pyx_float__0003)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_70_53);
  __Pyx_GIVEREF(__pyx_float_70_53);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 124, __pyx_float_70_53)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 125, __pyx_float__0002)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_71_84);
  __Pyx_GIVEREF(__pyx_float_71_84);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 126, __pyx_float_71_84)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 127, __pyx_float__0001)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_73_28);
  __Pyx_GIVEREF(__pyx_float_73_28);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 128, __pyx_float_73_28)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 129, __pyx_float__0002)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_75_52);
  __Pyx_GIVEREF(__pyx_float_75_52);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 130, __pyx_float_75_52)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 131, __pyx_float__0010)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_76_96);
  __Pyx_GIVEREF(__pyx_float_76_96);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 132, __pyx_float_76_96)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 133, __pyx_float__0005)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_77_93);
  __Pyx_GIVEREF(__pyx_float_77_93);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 134, __pyx_float_77_93)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 135, __pyx_float__0003)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_78_32);
  __Pyx_GIVEREF(__pyx_float_78_32);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 136, __pyx_float_78_32)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 137, __pyx_float__0003)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_79_73);
  __Pyx_GIVEREF(__pyx_float_79_73);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 138, __pyx_float_79_73)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 139, __pyx_float__0003)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_81_69);
  __Pyx_GIVEREF(__pyx_float_81_69);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 140, __pyx_float_81_69)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 141, __pyx_float__0002)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_82_52);
  __Pyx_GIVEREF(__pyx_float_82_52);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 142, __pyx_float_82_52)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 143, __pyx_float__0001)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_84_01);
  __Pyx_GIVEREF(__pyx_float_84_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 144, __pyx_float_84_01)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 145, __pyx_float__0001)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_84_61);
  __Pyx_GIVEREF(__pyx_float_84_61);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 146, __pyx_float_84_61)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 147, __pyx_float__0002)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_86_88);
  __Pyx_GIVEREF(__pyx_float_86_88);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 148, __pyx_float_86_88)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 149, __pyx_float__0001)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_88_36);
  __Pyx_GIVEREF(__pyx_float_88_36);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 150, __pyx_float_88_36)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 151, __pyx_float__0002)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_89_85);
  __Pyx_GIVEREF(__pyx_float_89_85);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 152, __pyx_float_89_85)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 153, __pyx_float__0002)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_91_35);
  __Pyx_GIVEREF(__pyx_float_91_35);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 154, __pyx_float_91_35)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 155, __pyx_float__0003)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_92_86);
  __Pyx_GIVEREF(__pyx_float_92_86);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 156, __pyx_float_92_86)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 157, __pyx_float__0002)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_93_40);
  __Pyx_GIVEREF(__pyx_float_93_40);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 158, __pyx_float_93_40)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 159, __pyx_float__0001)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_105_28);
  __Pyx_GIVEREF(__pyx_float_105_28);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 160, __pyx_float_105_28)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 161, __pyx_float__0002)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_106_22);
  __Pyx_GIVEREF(__pyx_float_106_22);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 162, __pyx_float_106_22)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 163, __pyx_float__0002)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_107_45);
  __Pyx_GIVEREF(__pyx_float_107_45);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 164, __pyx_float_107_45)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 165, __pyx_float__0001)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_108_70);
  __Pyx_GIVEREF(__pyx_float_108_70);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 166, __pyx_float_108_70)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 167, __pyx_float__0003)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_122_08);
  __Pyx_GIVEREF(__pyx_float_122_08);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 168, __pyx_float_122_08)) __PYX_ERR(0, 33, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_7, 169, __pyx_float__0002)) __PYX_ERR(0, 33, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":34
 * 			        [1.99,.0650,3.03,.0040,4.03,.0059,5.02,.0090,5.97,.0227,6.98,.0050,8.04,.0020 ,9.00,.0082,9.96,.0078,11.01,.0056,12.01,.0095,13.02,.0050,14.04,.0093,15.08,.0064,16.14,.0017 ,17.06,.0020,18.10,.0025,19.14,.0023,20.18,.0015,21.24,.0032,22.29,.0029,23.32,.0014,24.37,.0005 ,25.43,.0030,26.50,.0022,27.60,.0027,28.64,.0024,29.76,.0035,30.81,.0136,31.96,.0025,33.02,.0003 ,34.13,.0005,35.25,.0007,36.40,.0014,37.51,.0020,38.64,.0012,39.80,.0019,40.97,.0004,42.09,.0003 ,43.24,.0003,44.48,.0002,45.65,.0024,46.86,.0005,48.07,.0013,49.27,.0008,50.49,.0006,52.95,.0001 ,54.23,.0005,55.45,.0004,56.73,.0001,58.03,.0003,59.29,.0002,60.59,.0003,62.04,.0002,65.89,.0002 ,67.23,.0002,68.61,.0002,69.97,.0004,71.36,.0005,85.42,.0001],
 * 			        [1.98,.0256,2.96,.0158,3.95,.0310,4.94,.0411,5.95,.0238,6.94,.0152,7.93,.0011 ,8.95,.0185,9.92,.0166,10.93,.0306,11.94,.0258,12.96,.0202,13.97,.0403,14.95,.0228,15.93,.0005 ,17.01,.0072,18.02,.0034,19.06,.0028,20.08,.0124,21.13,.0137,22.16,.0102,23.19,.0058,23.90,.0013 ,25.30,.0039,26.36,.0039,27.41,.0025,28.47,.0071,29.64,.0031,30.60,.0027,31.71,.0021,32.84,.0003 ,33.82,.0002,35.07,.0019,36.09,.0054,37.20,.0038,38.33,.0024,39.47,.0055,40.55,.0016,41.77,.0006 ,42.95,.0002,43.27,.0018,44.03,.0006,45.25,.0019,46.36,.0033,47.50,.0024,48.87,.0012,50.03,.0016 ,51.09,.0004,53.52,.0017,54.74,.0012,56.17,.0003,57.40,.0011,58.42,.0020,59.70,.0007,61.29,.0008 ,62.56,.0003,63.48,.0002,64.83,.0002,66.12,.0012,67.46,.0017,68.81,.0003,69.13,.0003,70.53,.0002 ,71.84,.0001,73.28,.0002,75.52,.0010,76.96,.0005,77.93,.0003,78.32,.0003,79.73,.0003,81.69,.0002 ,82.52,.0001,84.01,.0001,84.61,.0002,86.88,.0001,88.36,.0002,89.85,.0002,91.35,.0003,92.86,.0002 ,93.40,.0001,105.28,.0002,106.22,.0002,107.45,.0001,108.70,.0003,122.08,.0002],
 * 			        [1.97,.0264,2.97,.0211,3.98,.0234,4.98,.0307,5.96,.0085,6.94,.0140,7.93,.0005 ,8.96,.0112,9.96,.0209,10.98,.0194,11.98,.0154,12.99,.0274,13.99,.0127,15.01,.0101,15.99,.0002 ,17.04,.0011,18.08,.0032,19.14,.0028,20.12,.0054,21.20,.0053,22.13,.0028,23.22,.0030,24.32,.0006 ,25.24,.0004,26.43,.0028,27.53,.0048,28.52,.0039,29.54,.0047,30.73,.0044,31.82,.0007,32.94,.0008 ,34.04,.0012,35.13,.0018,36.29,.0007,37.35,.0075,38.51,.0045,39.66,.0014,40.90,.0004,41.90,.0002 ,43.08,.0002,44.24,.0017,45.36,.0013,46.68,.0020,47.79,.0015,48.98,.0010,50.21,.0012,51.34,.0001 ,53.82,.0003,55.09,.0004,56.23,.0005,57.53,.0004,58.79,.0005,59.30,.0002,60.03,.0002,61.40,.0003 ,62.84,.0001,66.64,.0001,67.97,.0001,69.33,.0001,70.68,.0001,73.57,.0002,75.76,.0002,76.45,.0001 ,79.27,.0001,80.44,.0002,81.87,.0002],             # <<<<<<<<<<<<<<
 * 			        [2.00,.0311,2.99,.0086,3.99,.0266,4.97,.0123,5.98,.0235,6.97,.0161,7.97,.0008 ,8.96,.0088,9.96,.0621,10.99,.0080,11.99,.0034,12.99,.0300,14.03,.0228,15.04,.0105,16.03,.0004 ,17.06,.0036,18.09,.0094,18.95,.0009,20.17,.0071,21.21,.0161,22.25,.0106,23.28,.0104,24.33,.0008 ,25.38,.0030,26.46,.0035,27.50,.0026,28.59,.0028,29.66,.0128,30.75,.0139,31.81,.0038,32.93,.0006 ,34.04,.0004,35.16,.0005,36.25,.0023,37.35,.0012,38.46,.0021,39.59,.0035,40.71,.0006,41.86,.0007 ,42.42,.0001,43.46,.0003,44.17,.0032,45.29,.0013,46.57,.0004,47.72,.0011,48.79,.0005,50.11,.0005 ,51.29,.0003,52.47,.0002,53.68,.0004,55.02,.0005,56.18,.0003,57.41,.0003,58.75,.0007,59.33,.0009 ,60.00,.0004,61.34,.0001,64.97,.0003,65.20,.0002,66.48,.0002,67.83,.0002,68.90,.0003,70.25,.0003 ,71.59,.0002,73.68,.0001,75.92,.0001,77.08,.0002,78.45,.0002,81.56,.0002,82.99,.0001,88.39,.0001],
 * 			        [0.97,.0059,1.98,.0212,2.99,.0153,3.99,.0227,4.96,.0215,5.97,.0153,6.98,.0085 ,7.98,.0007,8.97,.0179,9.98,.0512,10.98,.0322,12.00,.0098,13.02,.0186,14.00,.0099,15.05,.0109 ,15.88,.0011,17.07,.0076,18.11,.0071,19.12,.0045,20.16,.0038,21.23,.0213,22.27,.0332,23.34,.0082 ,24.34,.0014,25.42,.0024,26.47,.0012,27.54,.0014,28.60,.0024,29.72,.0026,30.10,.0008,31.91,.0021 ,32.13,.0011,33.02,.0007,34.09,.0014,35.17,.0007,36.27,.0024,37.39,.0029,38.58,.0014,39.65,.0017 ,40.95,.0012,41.97,.0004,42.43,.0002,43.49,.0001,44.31,.0012,45.42,.0031,46.62,.0017,47.82,.0013 ,49.14,.0013,50.18,.0010,51.54,.0003,53.90,.0006,55.06,.0010,56.31,.0003,57.63,.0001,59.02,.0003 ,60.09,.0004,60.35,.0004,61.62,.0009,63.97,.0001,65.19,.0001,65.54,.0002,66.92,.0002,67.94,.0002 ,69.17,.0003,69.60,.0004,70.88,.0002,72.24,.0002,76.12,.0001,78.94,.0001,81.75,.0001,82.06,.0001 ,83.53,.0001,90.29,.0002,91.75,.0001,92.09,.0002,93.28,.0001,97.07,.0001],
 */
  __pyx_t_8 = PyList_New(132); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 34, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_INCREF(__pyx_float_1_97);
  __Pyx_GIVEREF(__pyx_float_1_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 0, __pyx_float_1_97)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0264);
  __Pyx_GIVEREF(__pyx_float__0264);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 1, __pyx_float__0264)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_97);
  __Pyx_GIVEREF(__pyx_float_2_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 2, __pyx_float_2_97)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0211);
  __Pyx_GIVEREF(__pyx_float__0211);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 3, __pyx_float__0211)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_98);
  __Pyx_GIVEREF(__pyx_float_3_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 4, __pyx_float_3_98)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0234);
  __Pyx_GIVEREF(__pyx_float__0234);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 5, __pyx_float__0234)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_98);
  __Pyx_GIVEREF(__pyx_float_4_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 6, __pyx_float_4_98)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0307);
  __Pyx_GIVEREF(__pyx_float__0307);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 7, __pyx_float__0307)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_96);
  __Pyx_GIVEREF(__pyx_float_5_96);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 8, __pyx_float_5_96)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0085);
  __Pyx_GIVEREF(__pyx_float__0085);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 9, __pyx_float__0085)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_94);
  __Pyx_GIVEREF(__pyx_float_6_94);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 10, __pyx_float_6_94)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0140);
  __Pyx_GIVEREF(__pyx_float__0140);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 11, __pyx_float__0140)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_93);
  __Pyx_GIVEREF(__pyx_float_7_93);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 12, __pyx_float_7_93)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 13, __pyx_float__0005)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_96);
  __Pyx_GIVEREF(__pyx_float_8_96);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 14, __pyx_float_8_96)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0112);
  __Pyx_GIVEREF(__pyx_float__0112);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 15, __pyx_float__0112)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_96);
  __Pyx_GIVEREF(__pyx_float_9_96);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 16, __pyx_float_9_96)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0209);
  __Pyx_GIVEREF(__pyx_float__0209);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 17, __pyx_float__0209)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_98);
  __Pyx_GIVEREF(__pyx_float_10_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 18, __pyx_float_10_98)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0194);
  __Pyx_GIVEREF(__pyx_float__0194);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 19, __pyx_float__0194)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_98);
  __Pyx_GIVEREF(__pyx_float_11_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 20, __pyx_float_11_98)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0154);
  __Pyx_GIVEREF(__pyx_float__0154);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 21, __pyx_float__0154)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_99);
  __Pyx_GIVEREF(__pyx_float_12_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 22, __pyx_float_12_99)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0274);
  __Pyx_GIVEREF(__pyx_float__0274);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 23, __pyx_float__0274)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_99);
  __Pyx_GIVEREF(__pyx_float_13_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 24, __pyx_float_13_99)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0127);
  __Pyx_GIVEREF(__pyx_float__0127);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 25, __pyx_float__0127)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_01);
  __Pyx_GIVEREF(__pyx_float_15_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 26, __pyx_float_15_01)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0101);
  __Pyx_GIVEREF(__pyx_float__0101);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 27, __pyx_float__0101)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_99);
  __Pyx_GIVEREF(__pyx_float_15_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 28, __pyx_float_15_99)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 29, __pyx_float__0002)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_04);
  __Pyx_GIVEREF(__pyx_float_17_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 30, __pyx_float_17_04)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 31, __pyx_float__0011)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_08);
  __Pyx_GIVEREF(__pyx_float_18_08);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 32, __pyx_float_18_08)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0032);
  __Pyx_GIVEREF(__pyx_float__0032);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 33, __pyx_float__0032)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_14);
  __Pyx_GIVEREF(__pyx_float_19_14);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 34, __pyx_float_19_14)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0028);
  __Pyx_GIVEREF(__pyx_float__0028);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 35, __pyx_float__0028)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_12);
  __Pyx_GIVEREF(__pyx_float_20_12);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 36, __pyx_float_20_12)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0054);
  __Pyx_GIVEREF(__pyx_float__0054);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 37, __pyx_float__0054)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_20);
  __Pyx_GIVEREF(__pyx_float_21_20);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 38, __pyx_float_21_20)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0053);
  __Pyx_GIVEREF(__pyx_float__0053);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 39, __pyx_float__0053)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_13);
  __Pyx_GIVEREF(__pyx_float_22_13);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 40, __pyx_float_22_13)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0028);
  __Pyx_GIVEREF(__pyx_float__0028);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 41, __pyx_float__0028)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_22);
  __Pyx_GIVEREF(__pyx_float_23_22);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 42, __pyx_float_23_22)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0030);
  __Pyx_GIVEREF(__pyx_float__0030);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 43, __pyx_float__0030)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_24_32);
  __Pyx_GIVEREF(__pyx_float_24_32);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 44, __pyx_float_24_32)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 45, __pyx_float__0006)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_25_24);
  __Pyx_GIVEREF(__pyx_float_25_24);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 46, __pyx_float_25_24)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 47, __pyx_float__0004)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_26_43);
  __Pyx_GIVEREF(__pyx_float_26_43);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 48, __pyx_float_26_43)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0028);
  __Pyx_GIVEREF(__pyx_float__0028);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 49, __pyx_float__0028)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_27_53);
  __Pyx_GIVEREF(__pyx_float_27_53);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 50, __pyx_float_27_53)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0048);
  __Pyx_GIVEREF(__pyx_float__0048);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 51, __pyx_float__0048)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_52);
  __Pyx_GIVEREF(__pyx_float_28_52);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 52, __pyx_float_28_52)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0039);
  __Pyx_GIVEREF(__pyx_float__0039);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 53, __pyx_float__0039)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_29_54);
  __Pyx_GIVEREF(__pyx_float_29_54);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 54, __pyx_float_29_54)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0047);
  __Pyx_GIVEREF(__pyx_float__0047);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 55, __pyx_float__0047)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_30_73);
  __Pyx_GIVEREF(__pyx_float_30_73);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 56, __pyx_float_30_73)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0044);
  __Pyx_GIVEREF(__pyx_float__0044);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 57, __pyx_float__0044)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_31_82);
  __Pyx_GIVEREF(__pyx_float_31_82);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 58, __pyx_float_31_82)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 59, __pyx_float__0007)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_32_94);
  __Pyx_GIVEREF(__pyx_float_32_94);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 60, __pyx_float_32_94)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 61, __pyx_float__0008)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_34_04);
  __Pyx_GIVEREF(__pyx_float_34_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 62, __pyx_float_34_04)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 63, __pyx_float__0012)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_35_13);
  __Pyx_GIVEREF(__pyx_float_35_13);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 64, __pyx_float_35_13)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0018);
  __Pyx_GIVEREF(__pyx_float__0018);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 65, __pyx_float__0018)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_36_29);
  __Pyx_GIVEREF(__pyx_float_36_29);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 66, __pyx_float_36_29)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 67, __pyx_float__0007)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_37_35);
  __Pyx_GIVEREF(__pyx_float_37_35);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 68, __pyx_float_37_35)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0075);
  __Pyx_GIVEREF(__pyx_float__0075);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 69, __pyx_float__0075)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_38_51);
  __Pyx_GIVEREF(__pyx_float_38_51);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 70, __pyx_float_38_51)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0045);
  __Pyx_GIVEREF(__pyx_float__0045);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 71, __pyx_float__0045)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_39_66);
  __Pyx_GIVEREF(__pyx_float_39_66);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 72, __pyx_float_39_66)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0014);
  __Pyx_GIVEREF(__pyx_float__0014);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 73, __pyx_float__0014)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_40_90);
  __Pyx_GIVEREF(__pyx_float_40_90);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 74, __pyx_float_40_90)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 75, __pyx_float__0004)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_41_90);
  __Pyx_GIVEREF(__pyx_float_41_90);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 76, __pyx_float_41_90)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 77, __pyx_float__0002)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_43_08);
  __Pyx_GIVEREF(__pyx_float_43_08);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 78, __pyx_float_43_08)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 79, __pyx_float__0002)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_44_24);
  __Pyx_GIVEREF(__pyx_float_44_24);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 80, __pyx_float_44_24)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0017);
  __Pyx_GIVEREF(__pyx_float__0017);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 81, __pyx_float__0017)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_45_36);
  __Pyx_GIVEREF(__pyx_float_45_36);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 82, __pyx_float_45_36)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0013);
  __Pyx_GIVEREF(__pyx_float__0013);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 83, __pyx_float__0013)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_46_68);
  __Pyx_GIVEREF(__pyx_float_46_68);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 84, __pyx_float_46_68)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0020);
  __Pyx_GIVEREF(__pyx_float__0020);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 85, __pyx_float__0020)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_47_79);
  __Pyx_GIVEREF(__pyx_float_47_79);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 86, __pyx_float_47_79)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0015);
  __Pyx_GIVEREF(__pyx_float__0015);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 87, __pyx_float__0015)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_48_98);
  __Pyx_GIVEREF(__pyx_float_48_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 88, __pyx_float_48_98)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 89, __pyx_float__0010)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_50_21);
  __Pyx_GIVEREF(__pyx_float_50_21);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 90, __pyx_float_50_21)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 91, __pyx_float__0012)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_51_34);
  __Pyx_GIVEREF(__pyx_float_51_34);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 92, __pyx_float_51_34)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 93, __pyx_float__0001)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_53_82);
  __Pyx_GIVEREF(__pyx_float_53_82);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 94, __pyx_float_53_82)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 95, __pyx_float__0003)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_55_09);
  __Pyx_GIVEREF(__pyx_float_55_09);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 96, __pyx_float_55_09)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 97, __pyx_float__0004)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_56_23);
  __Pyx_GIVEREF(__pyx_float_56_23);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 98, __pyx_float_56_23)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 99, __pyx_float__0005)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_57_53);
  __Pyx_GIVEREF(__pyx_float_57_53);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 100, __pyx_float_57_53)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 101, __pyx_float__0004)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_58_79);
  __Pyx_GIVEREF(__pyx_float_58_79);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 102, __pyx_float_58_79)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 103, __pyx_float__0005)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_59_30);
  __Pyx_GIVEREF(__pyx_float_59_30);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 104, __pyx_float_59_30)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 105, __pyx_float__0002)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_60_03);
  __Pyx_GIVEREF(__pyx_float_60_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 106, __pyx_float_60_03)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 107, __pyx_float__0002)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_61_40);
  __Pyx_GIVEREF(__pyx_float_61_40);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 108, __pyx_float_61_40)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 109, __pyx_float__0003)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_62_84);
  __Pyx_GIVEREF(__pyx_float_62_84);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 110, __pyx_float_62_84)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 111, __pyx_float__0001)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_66_64);
  __Pyx_GIVEREF(__pyx_float_66_64);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 112, __pyx_float_66_64)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 113, __pyx_float__0001)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_67_97);
  __Pyx_GIVEREF(__pyx_float_67_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 114, __pyx_float_67_97)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 115, __pyx_float__0001)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_69_33);
  __Pyx_GIVEREF(__pyx_float_69_33);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 116, __pyx_float_69_33)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 117, __pyx_float__0001)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_70_68);
  __Pyx_GIVEREF(__pyx_float_70_68);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 118, __pyx_float_70_68)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 119, __pyx_float__0001)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_73_57);
  __Pyx_GIVEREF(__pyx_float_73_57);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 120, __pyx_float_73_57)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 121, __pyx_float__0002)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_75_76);
  __Pyx_GIVEREF(__pyx_float_75_76);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 122, __pyx_float_75_76)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 123, __pyx_float__0002)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_76_45);
  __Pyx_GIVEREF(__pyx_float_76_45);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 124, __pyx_float_76_45)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 125, __pyx_float__0001)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_79_27);
  __Pyx_GIVEREF(__pyx_float_79_27);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 126, __pyx_float_79_27)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 127, __pyx_float__0001)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_80_44);
  __Pyx_GIVEREF(__pyx_float_80_44);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 128, __pyx_float_80_44)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 129, __pyx_float__0002)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_81_87);
  __Pyx_GIVEREF(__pyx_float_81_87);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 130, __pyx_float_81_87)) __PYX_ERR(0, 34, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 131, __pyx_float__0002)) __PYX_ERR(0, 34, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":35
 * 			        [1.98,.0256,2.96,.0158,3.95,.0310,4.94,.0411,5.95,.0238,6.94,.0152,7.93,.0011 ,8.95,.0185,9.92,.0166,10.93,.0306,11.94,.0258,12.96,.0202,13.97,.0403,14.95,.0228,15.93,.0005 ,17.01,.0072,18.02,.0034,19.06,.0028,20.08,.0124,21.13,.0137,22.16,.0102,23.19,.0058,23.90,.0013 ,25.30,.0039,26.36,.0039,27.41,.0025,28.47,.0071,29.64,.0031,30.60,.0027,31.71,.0021,32.84,.0003 ,33.82,.0002,35.07,.0019,36.09,.0054,37.20,.0038,38.33,.0024,39.47,.0055,40.55,.0016,41.77,.0006 ,42.95,.0002,43.27,.0018,44.03,.0006,45.25,.0019,46.36,.0033,47.50,.0024,48.87,.0012,50.03,.0016 ,51.09,.0004,53.52,.0017,54.74,.0012,56.17,.0003,57.40,.0011,58.42,.0020,59.70,.0007,61.29,.0008 ,62.56,.0003,63.48,.0002,64.83,.0002,66.12,.0012,67.46,.0017,68.81,.0003,69.13,.0003,70.53,.0002 ,71.84,.0001,73.28,.0002,75.52,.0010,76.96,.0005,77.93,.0003,78.32,.0003,79.73,.0003,81.69,.0002 ,82.52,.0001,84.01,.0001,84.61,.0002,86.88,.0001,88.36,.0002,89.85,.0002,91.35,.0003,92.86,.0002 ,93.40,.0001,105.28,.0002,106.22,.0002,107.45,.0001,108.70,.0003,122.08,.0002],
 * 			        [1.97,.0264,2.97,.0211,3.98,.0234,4.98,.0307,5.96,.0085,6.94,.0140,7.93,.0005 ,8.96,.0112,9.96,.0209,10.98,.0194,11.98,.0154,12.99,.0274,13.99,.0127,15.01,.0101,15.99,.0002 ,17.04,.0011,18.08,.0032,19.14,.0028,20.12,.0054,21.20,.0053,22.13,.0028,23.22,.0030,24.32,.0006 ,25.24,.0004,26.43,.0028,27.53,.0048,28.52,.0039,29.54,.0047,30.73,.0044,31.82,.0007,32.94,.0008 ,34.04,.0012,35.13,.0018,36.29,.0007,37.35,.0075,38.51,.0045,39.66,.0014,40.90,.0004,41.90,.0002 ,43.08,.0002,44.24,.0017,45.36,.0013,46.68,.0020,47.79,.0015,48.98,.0010,50.21,.0012,51.34,.0001 ,53.82,.0003,55.09,.0004,56.23,.0005,57.53,.0004,58.79,.0005,59.30,.0002,60.03,.0002,61.40,.0003 ,62.84,.0001,66.64,.0001,67.97,.0001,69.33,.0001,70.68,.0001,73.57,.0002,75.76,.0002,76.45,.0001 ,79.27,.0001,80.44,.0002,81.87,.0002],
 * 			        [2.00,.0311,2.99,.0086,3.99,.0266,4.97,.0123,5.98,.0235,6.97,.0161,7.97,.0008 ,8.96,.0088,9.96,.0621,10.99,.0080,11.99,.0034,12.99,.0300,14.03,.0228,15.04,.0105,16.03,.0004 ,17.06,.0036,18.09,.0094,18.95,.0009,20.17,.0071,21.21,.0161,22.25,.0106,23.28,.0104,24.33,.0008 ,25.38,.0030,26.46,.0035,27.50,.0026,28.59,.0028,29.66,.0128,30.75,.0139,31.81,.0038,32.93,.0006 ,34.04,.0004,35.16,.0005,36.25,.0023,37.35,.0012,38.46,.0021,39.59,.0035,40.71,.0006,41.86,.0007 ,42.42,.0001,43.46,.0003,44.17,.0032,45.29,.0013,46.57,.0004,47.72,.0011,48.79,.0005,50.11,.0005 ,51.29,.0003,52.47,.0002,53.68,.0004,55.02,.0005,56.18,.0003,57.41,.0003,58.75,.0007,59.33,.0009 ,60.00,.0004,61.34,.0001,64.97,.0003,65.20,.0002,66.48,.0002,67.83,.0002,68.90,.0003,70.25,.0003 ,71.59,.0002,73.68,.0001,75.92,.0001,77.08,.0002,78.45,.0002,81.56,.0002,82.99,.0001,88.39,.0001],             # <<<<<<<<<<<<<<
 * 			        [0.97,.0059,1.98,.0212,2.99,.0153,3.99,.0227,4.96,.0215,5.97,.0153,6.98,.0085 ,7.98,.0007,8.97,.0179,9.98,.0512,10.98,.0322,12.00,.0098,13.02,.0186,14.00,.0099,15.05,.0109 ,15.88,.0011,17.07,.0076,18.11,.0071,19.12,.0045,20.16,.0038,21.23,.0213,22.27,.0332,23.34,.0082 ,24.34,.0014,25.42,.0024,26.47,.0012,27.54,.0014,28.60,.0024,29.72,.0026,30.10,.0008,31.91,.0021 ,32.13,.0011,33.02,.0007,34.09,.0014,35.17,.0007,36.27,.0024,37.39,.0029,38.58,.0014,39.65,.0017 ,40.95,.0012,41.97,.0004,42.43,.0002,43.49,.0001,44.31,.0012,45.42,.0031,46.62,.0017,47.82,.0013 ,49.14,.0013,50.18,.0010,51.54,.0003,53.90,.0006,55.06,.0010,56.31,.0003,57.63,.0001,59.02,.0003 ,60.09,.0004,60.35,.0004,61.62,.0009,63.97,.0001,65.19,.0001,65.54,.0002,66.92,.0002,67.94,.0002 ,69.17,.0003,69.60,.0004,70.88,.0002,72.24,.0002,76.12,.0001,78.94,.0001,81.75,.0001,82.06,.0001 ,83.53,.0001,90.29,.0002,91.75,.0001,92.09,.0002,93.28,.0001,97.07,.0001],
 * 			        [1.98,.0159,2.98,.1008,3.98,.0365,4.98,.0133,5.97,.0101,6.97,.0115,7.97,.0007 ,8.99,.0349,10.01,.0342,11.01,.0236,12.00,.0041,13.02,.0114,14.05,.0137,15.06,.0100,16.05,.0007 ,17.04,.0009,18.12,.0077,19.15,.0023,20.12,.0017,21.24,.0113,22.26,.0126,23.30,.0093,24.36,.0007 ,25.43,.0007,26.47,.0009,27.55,.0013,28.59,.0025,29.61,.0010,30.77,.0021,31.86,.0023,32.96,.0003 ,34.03,.0007,35.06,.0005,36.20,.0006,37.34,.0006,38.36,.0009,39.60,.0016,40.69,.0005,41.77,.0002 ,42.92,.0002,44.02,.0003,45.24,.0006,46.33,.0004,47.50,.0007,48.71,.0007,49.87,.0002,51.27,.0002 ,53.42,.0003,55.88,.0003,57.10,.0004,58.34,.0002,59.86,.0003,61.13,.0003,67.18,.0001,68.50,.0001 ,71.17,.0001,83.91,.0001,90.55,.0001],
 */
  __pyx_t_9 = PyList_New(142); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_INCREF(__pyx_float_2_00);
  __Pyx_GIVEREF(__pyx_float_2_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 0, __pyx_float_2_00)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0311);
  __Pyx_GIVEREF(__pyx_float__0311);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 1, __pyx_float__0311)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_99);
  __Pyx_GIVEREF(__pyx_float_2_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 2, __pyx_float_2_99)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0086);
  __Pyx_GIVEREF(__pyx_float__0086);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 3, __pyx_float__0086)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_99);
  __Pyx_GIVEREF(__pyx_float_3_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 4, __pyx_float_3_99)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0266);
  __Pyx_GIVEREF(__pyx_float__0266);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 5, __pyx_float__0266)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_97);
  __Pyx_GIVEREF(__pyx_float_4_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 6, __pyx_float_4_97)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0123);
  __Pyx_GIVEREF(__pyx_float__0123);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 7, __pyx_float__0123)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_98);
  __Pyx_GIVEREF(__pyx_float_5_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 8, __pyx_float_5_98)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0235);
  __Pyx_GIVEREF(__pyx_float__0235);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 9, __pyx_float__0235)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_97);
  __Pyx_GIVEREF(__pyx_float_6_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 10, __pyx_float_6_97)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0161);
  __Pyx_GIVEREF(__pyx_float__0161);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 11, __pyx_float__0161)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_97);
  __Pyx_GIVEREF(__pyx_float_7_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 12, __pyx_float_7_97)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 13, __pyx_float__0008)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_96);
  __Pyx_GIVEREF(__pyx_float_8_96);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 14, __pyx_float_8_96)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0088);
  __Pyx_GIVEREF(__pyx_float__0088);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 15, __pyx_float__0088)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_96);
  __Pyx_GIVEREF(__pyx_float_9_96);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 16, __pyx_float_9_96)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0621);
  __Pyx_GIVEREF(__pyx_float__0621);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 17, __pyx_float__0621)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_99);
  __Pyx_GIVEREF(__pyx_float_10_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 18, __pyx_float_10_99)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0080);
  __Pyx_GIVEREF(__pyx_float__0080);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 19, __pyx_float__0080)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_99);
  __Pyx_GIVEREF(__pyx_float_11_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 20, __pyx_float_11_99)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0034);
  __Pyx_GIVEREF(__pyx_float__0034);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 21, __pyx_float__0034)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_99);
  __Pyx_GIVEREF(__pyx_float_12_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 22, __pyx_float_12_99)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0300);
  __Pyx_GIVEREF(__pyx_float__0300);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 23, __pyx_float__0300)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_03);
  __Pyx_GIVEREF(__pyx_float_14_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 24, __pyx_float_14_03)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0228);
  __Pyx_GIVEREF(__pyx_float__0228);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 25, __pyx_float__0228)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_04);
  __Pyx_GIVEREF(__pyx_float_15_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 26, __pyx_float_15_04)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0105);
  __Pyx_GIVEREF(__pyx_float__0105);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 27, __pyx_float__0105)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_03);
  __Pyx_GIVEREF(__pyx_float_16_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 28, __pyx_float_16_03)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 29, __pyx_float__0004)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_06);
  __Pyx_GIVEREF(__pyx_float_17_06);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 30, __pyx_float_17_06)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0036);
  __Pyx_GIVEREF(__pyx_float__0036);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 31, __pyx_float__0036)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_09);
  __Pyx_GIVEREF(__pyx_float_18_09);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 32, __pyx_float_18_09)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0094);
  __Pyx_GIVEREF(__pyx_float__0094);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 33, __pyx_float__0094)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_95);
  __Pyx_GIVEREF(__pyx_float_18_95);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 34, __pyx_float_18_95)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 35, __pyx_float__0009)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_17);
  __Pyx_GIVEREF(__pyx_float_20_17);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 36, __pyx_float_20_17)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0071);
  __Pyx_GIVEREF(__pyx_float__0071);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 37, __pyx_float__0071)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_21);
  __Pyx_GIVEREF(__pyx_float_21_21);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 38, __pyx_float_21_21)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0161);
  __Pyx_GIVEREF(__pyx_float__0161);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 39, __pyx_float__0161)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_25);
  __Pyx_GIVEREF(__pyx_float_22_25);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 40, __pyx_float_22_25)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0106);
  __Pyx_GIVEREF(__pyx_float__0106);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 41, __pyx_float__0106)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_28);
  __Pyx_GIVEREF(__pyx_float_23_28);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 42, __pyx_float_23_28)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0104);
  __Pyx_GIVEREF(__pyx_float__0104);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 43, __pyx_float__0104)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_24_33);
  __Pyx_GIVEREF(__pyx_float_24_33);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 44, __pyx_float_24_33)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 45, __pyx_float__0008)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_25_38);
  __Pyx_GIVEREF(__pyx_float_25_38);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 46, __pyx_float_25_38)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0030);
  __Pyx_GIVEREF(__pyx_float__0030);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 47, __pyx_float__0030)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_26_46);
  __Pyx_GIVEREF(__pyx_float_26_46);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 48, __pyx_float_26_46)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0035);
  __Pyx_GIVEREF(__pyx_float__0035);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 49, __pyx_float__0035)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_27_50);
  __Pyx_GIVEREF(__pyx_float_27_50);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 50, __pyx_float_27_50)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0026);
  __Pyx_GIVEREF(__pyx_float__0026);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 51, __pyx_float__0026)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_59);
  __Pyx_GIVEREF(__pyx_float_28_59);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 52, __pyx_float_28_59)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0028);
  __Pyx_GIVEREF(__pyx_float__0028);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 53, __pyx_float__0028)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_29_66);
  __Pyx_GIVEREF(__pyx_float_29_66);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 54, __pyx_float_29_66)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0128);
  __Pyx_GIVEREF(__pyx_float__0128);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 55, __pyx_float__0128)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_30_75);
  __Pyx_GIVEREF(__pyx_float_30_75);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 56, __pyx_float_30_75)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0139);
  __Pyx_GIVEREF(__pyx_float__0139);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 57, __pyx_float__0139)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_31_81);
  __Pyx_GIVEREF(__pyx_float_31_81);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 58, __pyx_float_31_81)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0038);
  __Pyx_GIVEREF(__pyx_float__0038);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 59, __pyx_float__0038)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_32_93);
  __Pyx_GIVEREF(__pyx_float_32_93);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 60, __pyx_float_32_93)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 61, __pyx_float__0006)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_34_04);
  __Pyx_GIVEREF(__pyx_float_34_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 62, __pyx_float_34_04)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 63, __pyx_float__0004)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_35_16);
  __Pyx_GIVEREF(__pyx_float_35_16);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 64, __pyx_float_35_16)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 65, __pyx_float__0005)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_36_25);
  __Pyx_GIVEREF(__pyx_float_36_25);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 66, __pyx_float_36_25)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0023);
  __Pyx_GIVEREF(__pyx_float__0023);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 67, __pyx_float__0023)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_37_35);
  __Pyx_GIVEREF(__pyx_float_37_35);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 68, __pyx_float_37_35)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 69, __pyx_float__0012)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_38_46);
  __Pyx_GIVEREF(__pyx_float_38_46);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 70, __pyx_float_38_46)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0021);
  __Pyx_GIVEREF(__pyx_float__0021);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 71, __pyx_float__0021)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_39_59);
  __Pyx_GIVEREF(__pyx_float_39_59);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 72, __pyx_float_39_59)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0035);
  __Pyx_GIVEREF(__pyx_float__0035);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 73, __pyx_float__0035)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_40_71);
  __Pyx_GIVEREF(__pyx_float_40_71);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 74, __pyx_float_40_71)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 75, __pyx_float__0006)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_41_86);
  __Pyx_GIVEREF(__pyx_float_41_86);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 76, __pyx_float_41_86)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 77, __pyx_float__0007)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_42_42);
  __Pyx_GIVEREF(__pyx_float_42_42);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 78, __pyx_float_42_42)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 79, __pyx_float__0001)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_43_46);
  __Pyx_GIVEREF(__pyx_float_43_46);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 80, __pyx_float_43_46)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 81, __pyx_float__0003)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_44_17);
  __Pyx_GIVEREF(__pyx_float_44_17);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 82, __pyx_float_44_17)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0032);
  __Pyx_GIVEREF(__pyx_float__0032);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 83, __pyx_float__0032)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_45_29);
  __Pyx_GIVEREF(__pyx_float_45_29);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 84, __pyx_float_45_29)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0013);
  __Pyx_GIVEREF(__pyx_float__0013);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 85, __pyx_float__0013)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_46_57);
  __Pyx_GIVEREF(__pyx_float_46_57);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 86, __pyx_float_46_57)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 87, __pyx_float__0004)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_47_72);
  __Pyx_GIVEREF(__pyx_float_47_72);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 88, __pyx_float_47_72)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 89, __pyx_float__0011)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_48_79);
  __Pyx_GIVEREF(__pyx_float_48_79);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 90, __pyx_float_48_79)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 91, __pyx_float__0005)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_50_11);
  __Pyx_GIVEREF(__pyx_float_50_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 92, __pyx_float_50_11)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 93, __pyx_float__0005)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_51_29);
  __Pyx_GIVEREF(__pyx_float_51_29);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 94, __pyx_float_51_29)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 95, __pyx_float__0003)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_52_47);
  __Pyx_GIVEREF(__pyx_float_52_47);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 96, __pyx_float_52_47)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 97, __pyx_float__0002)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_53_68);
  __Pyx_GIVEREF(__pyx_float_53_68);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 98, __pyx_float_53_68)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 99, __pyx_float__0004)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_55_02);
  __Pyx_GIVEREF(__pyx_float_55_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 100, __pyx_float_55_02)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 101, __pyx_float__0005)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_56_18);
  __Pyx_GIVEREF(__pyx_float_56_18);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 102, __pyx_float_56_18)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 103, __pyx_float__0003)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_57_41);
  __Pyx_GIVEREF(__pyx_float_57_41);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 104, __pyx_float_57_41)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 105, __pyx_float__0003)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_58_75);
  __Pyx_GIVEREF(__pyx_float_58_75);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 106, __pyx_float_58_75)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 107, __pyx_float__0007)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_59_33);
  __Pyx_GIVEREF(__pyx_float_59_33);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 108, __pyx_float_59_33)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 109, __pyx_float__0009)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_60_00);
  __Pyx_GIVEREF(__pyx_float_60_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 110, __pyx_float_60_00)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 111, __pyx_float__0004)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_61_34);
  __Pyx_GIVEREF(__pyx_float_61_34);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 112, __pyx_float_61_34)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 113, __pyx_float__0001)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_64_97);
  __Pyx_GIVEREF(__pyx_float_64_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 114, __pyx_float_64_97)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 115, __pyx_float__0003)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_65_20);
  __Pyx_GIVEREF(__pyx_float_65_20);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 116, __pyx_float_65_20)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 117, __pyx_float__0002)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_66_48);
  __Pyx_GIVEREF(__pyx_float_66_48);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 118, __pyx_float_66_48)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 119, __pyx_float__0002)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_67_83);
  __Pyx_GIVEREF(__pyx_float_67_83);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 120, __pyx_float_67_83)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 121, __pyx_float__0002)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_68_90);
  __Pyx_GIVEREF(__pyx_float_68_90);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 122, __pyx_float_68_90)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 123, __pyx_float__0003)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_70_25);
  __Pyx_GIVEREF(__pyx_float_70_25);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 124, __pyx_float_70_25)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 125, __pyx_float__0003)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_71_59);
  __Pyx_GIVEREF(__pyx_float_71_59);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 126, __pyx_float_71_59)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 127, __pyx_float__0002)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_73_68);
  __Pyx_GIVEREF(__pyx_float_73_68);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 128, __pyx_float_73_68)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 129, __pyx_float__0001)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_75_92);
  __Pyx_GIVEREF(__pyx_float_75_92);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 130, __pyx_float_75_92)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 131, __pyx_float__0001)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_77_08);
  __Pyx_GIVEREF(__pyx_float_77_08);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 132, __pyx_float_77_08)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 133, __pyx_float__0002)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_78_45);
  __Pyx_GIVEREF(__pyx_float_78_45);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 134, __pyx_float_78_45)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 135, __pyx_float__0002)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_81_56);
  __Pyx_GIVEREF(__pyx_float_81_56);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 136, __pyx_float_81_56)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 137, __pyx_float__0002)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_82_99);
  __Pyx_GIVEREF(__pyx_float_82_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 138, __pyx_float_82_99)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 139, __pyx_float__0001)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_88_39);
  __Pyx_GIVEREF(__pyx_float_88_39);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 140, __pyx_float_88_39)) __PYX_ERR(0, 35, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 141, __pyx_float__0001)) __PYX_ERR(0, 35, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":36
 * 			        [1.97,.0264,2.97,.0211,3.98,.0234,4.98,.0307,5.96,.0085,6.94,.0140,7.93,.0005 ,8.96,.0112,9.96,.0209,10.98,.0194,11.98,.0154,12.99,.0274,13.99,.0127,15.01,.0101,15.99,.0002 ,17.04,.0011,18.08,.0032,19.14,.0028,20.12,.0054,21.20,.0053,22.13,.0028,23.22,.0030,24.32,.0006 ,25.24,.0004,26.43,.0028,27.53,.0048,28.52,.0039,29.54,.0047,30.73,.0044,31.82,.0007,32.94,.0008 ,34.04,.0012,35.13,.0018,36.29,.0007,37.35,.0075,38.51,.0045,39.66,.0014,40.90,.0004,41.90,.0002 ,43.08,.0002,44.24,.0017,45.36,.0013,46.68,.0020,47.79,.0015,48.98,.0010,50.21,.0012,51.34,.0001 ,53.82,.0003,55.09,.0004,56.23,.0005,57.53,.0004,58.79,.0005,59.30,.0002,60.03,.0002,61.40,.0003 ,62.84,.0001,66.64,.0001,67.97,.0001,69.33,.0001,70.68,.0001,73.57,.0002,75.76,.0002,76.45,.0001 ,79.27,.0001,80.44,.0002,81.87,.0002],
 * 			        [2.00,.0311,2.99,.0086,3.99,.0266,4.97,.0123,5.98,.0235,6.97,.0161,7.97,.0008 ,8.96,.0088,9.96,.0621,10.99,.0080,11.99,.0034,12.99,.0300,14.03,.0228,15.04,.0105,16.03,.0004 ,17.06,.0036,18.09,.0094,18.95,.0009,20.17,.0071,21.21,.0161,22.25,.0106,23.28,.0104,24.33,.0008 ,25.38,.0030,26.46,.0035,27.50,.0026,28.59,.0028,29.66,.0128,30.75,.0139,31.81,.0038,32.93,.0006 ,34.04,.0004,35.16,.0005,36.25,.0023,37.35,.0012,38.46,.0021,39.59,.0035,40.71,.0006,41.86,.0007 ,42.42,.0001,43.46,.0003,44.17,.0032,45.29,.0013,46.57,.0004,47.72,.0011,48.79,.0005,50.11,.0005 ,51.29,.0003,52.47,.0002,53.68,.0004,55.02,.0005,56.18,.0003,57.41,.0003,58.75,.0007,59.33,.0009 ,60.00,.0004,61.34,.0001,64.97,.0003,65.20,.0002,66.48,.0002,67.83,.0002,68.90,.0003,70.25,.0003 ,71.59,.0002,73.68,.0001,75.92,.0001,77.08,.0002,78.45,.0002,81.56,.0002,82.99,.0001,88.39,.0001],
 * 			        [0.97,.0059,1.98,.0212,2.99,.0153,3.99,.0227,4.96,.0215,5.97,.0153,6.98,.0085 ,7.98,.0007,8.97,.0179,9.98,.0512,10.98,.0322,12.00,.0098,13.02,.0186,14.00,.0099,15.05,.0109 ,15.88,.0011,17.07,.0076,18.11,.0071,19.12,.0045,20.16,.0038,21.23,.0213,22.27,.0332,23.34,.0082 ,24.34,.0014,25.42,.0024,26.47,.0012,27.54,.0014,28.60,.0024,29.72,.0026,30.10,.0008,31.91,.0021 ,32.13,.0011,33.02,.0007,34.09,.0014,35.17,.0007,36.27,.0024,37.39,.0029,38.58,.0014,39.65,.0017 ,40.95,.0012,41.97,.0004,42.43,.0002,43.49,.0001,44.31,.0012,45.42,.0031,46.62,.0017,47.82,.0013 ,49.14,.0013,50.18,.0010,51.54,.0003,53.90,.0006,55.06,.0010,56.31,.0003,57.63,.0001,59.02,.0003 ,60.09,.0004,60.35,.0004,61.62,.0009,63.97,.0001,65.19,.0001,65.54,.0002,66.92,.0002,67.94,.0002 ,69.17,.0003,69.60,.0004,70.88,.0002,72.24,.0002,76.12,.0001,78.94,.0001,81.75,.0001,82.06,.0001 ,83.53,.0001,90.29,.0002,91.75,.0001,92.09,.0002,93.28,.0001,97.07,.0001],             # <<<<<<<<<<<<<<
 * 			        [1.98,.0159,2.98,.1008,3.98,.0365,4.98,.0133,5.97,.0101,6.97,.0115,7.97,.0007 ,8.99,.0349,10.01,.0342,11.01,.0236,12.00,.0041,13.02,.0114,14.05,.0137,15.06,.0100,16.05,.0007 ,17.04,.0009,18.12,.0077,19.15,.0023,20.12,.0017,21.24,.0113,22.26,.0126,23.30,.0093,24.36,.0007 ,25.43,.0007,26.47,.0009,27.55,.0013,28.59,.0025,29.61,.0010,30.77,.0021,31.86,.0023,32.96,.0003 ,34.03,.0007,35.06,.0005,36.20,.0006,37.34,.0006,38.36,.0009,39.60,.0016,40.69,.0005,41.77,.0002 ,42.92,.0002,44.02,.0003,45.24,.0006,46.33,.0004,47.50,.0007,48.71,.0007,49.87,.0002,51.27,.0002 ,53.42,.0003,55.88,.0003,57.10,.0004,58.34,.0002,59.86,.0003,61.13,.0003,67.18,.0001,68.50,.0001 ,71.17,.0001,83.91,.0001,90.55,.0001],
 * 			        [0.98,.0099,2.00,.0181,2.99,.0353,3.98,.0285,4.97,.0514,5.96,.0402,6.96,.0015 ,7.98,.0012,8.98,.0175,9.98,.0264,10.98,.0392,11.98,.0236,13.00,.0153,14.04,.0049,15.00,.0089 ,16.01,.0001,17.03,.0106,18.03,.0028,19.05,.0024,20.08,.0040,21.11,.0103,22.12,.0104,23.20,.0017 ,24.19,.0008,25.20,.0007,26.24,.0011,27.36,.0009,27.97,.0030,29.40,.0044,30.37,.0019,31.59,.0017 ,32.65,.0008,33.59,.0005,34.79,.0009,35.75,.0027,36.88,.0035,37.93,.0039,39.00,.0031,40.08,.0025 ,41.16,.0010,43.25,.0004,44.52,.0012,45.62,.0023,45.85,.0012,47.00,.0006,47.87,.0008,48.99,.0003 ,50.48,.0003,51.62,.0001,52.43,.0001,53.56,.0002,54.76,.0002,56.04,.0002,56.68,.0006,57.10,.0003 ,58.28,.0005,59.47,.0003,59.96,.0002,60.67,.0001,63.08,.0002,64.29,.0002,66.72,.0001,67.97,.0001 ,68.65,.0001,70.43,.0001,79.38,.0001,80.39,.0001,82.39,.0001],
 */
  __pyx_t_10 = PyList_New(154); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_INCREF(__pyx_float_0_97);
  __Pyx_GIVEREF(__pyx_float_0_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 0, __pyx_float_0_97)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0059);
  __Pyx_GIVEREF(__pyx_float__0059);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 1, __pyx_float__0059)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_1_98);
  __Pyx_GIVEREF(__pyx_float_1_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 2, __pyx_float_1_98)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0212);
  __Pyx_GIVEREF(__pyx_float__0212);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 3, __pyx_float__0212)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_99);
  __Pyx_GIVEREF(__pyx_float_2_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 4, __pyx_float_2_99)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0153);
  __Pyx_GIVEREF(__pyx_float__0153);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 5, __pyx_float__0153)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_99);
  __Pyx_GIVEREF(__pyx_float_3_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 6, __pyx_float_3_99)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0227);
  __Pyx_GIVEREF(__pyx_float__0227);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 7, __pyx_float__0227)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_96);
  __Pyx_GIVEREF(__pyx_float_4_96);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 8, __pyx_float_4_96)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0215);
  __Pyx_GIVEREF(__pyx_float__0215);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 9, __pyx_float__0215)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_97);
  __Pyx_GIVEREF(__pyx_float_5_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 10, __pyx_float_5_97)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0153);
  __Pyx_GIVEREF(__pyx_float__0153);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 11, __pyx_float__0153)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_98);
  __Pyx_GIVEREF(__pyx_float_6_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 12, __pyx_float_6_98)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0085);
  __Pyx_GIVEREF(__pyx_float__0085);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 13, __pyx_float__0085)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_98);
  __Pyx_GIVEREF(__pyx_float_7_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 14, __pyx_float_7_98)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 15, __pyx_float__0007)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_97);
  __Pyx_GIVEREF(__pyx_float_8_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 16, __pyx_float_8_97)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0179);
  __Pyx_GIVEREF(__pyx_float__0179);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 17, __pyx_float__0179)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_98);
  __Pyx_GIVEREF(__pyx_float_9_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 18, __pyx_float_9_98)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0512);
  __Pyx_GIVEREF(__pyx_float__0512);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 19, __pyx_float__0512)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_98);
  __Pyx_GIVEREF(__pyx_float_10_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 20, __pyx_float_10_98)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0322);
  __Pyx_GIVEREF(__pyx_float__0322);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 21, __pyx_float__0322)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_00);
  __Pyx_GIVEREF(__pyx_float_12_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 22, __pyx_float_12_00)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0098);
  __Pyx_GIVEREF(__pyx_float__0098);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 23, __pyx_float__0098)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_02);
  __Pyx_GIVEREF(__pyx_float_13_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 24, __pyx_float_13_02)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0186);
  __Pyx_GIVEREF(__pyx_float__0186);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 25, __pyx_float__0186)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_00);
  __Pyx_GIVEREF(__pyx_float_14_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 26, __pyx_float_14_00)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0099);
  __Pyx_GIVEREF(__pyx_float__0099);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 27, __pyx_float__0099)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_05);
  __Pyx_GIVEREF(__pyx_float_15_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 28, __pyx_float_15_05)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0109);
  __Pyx_GIVEREF(__pyx_float__0109);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 29, __pyx_float__0109)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_88);
  __Pyx_GIVEREF(__pyx_float_15_88);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 30, __pyx_float_15_88)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 31, __pyx_float__0011)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_07);
  __Pyx_GIVEREF(__pyx_float_17_07);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 32, __pyx_float_17_07)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0076);
  __Pyx_GIVEREF(__pyx_float__0076);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 33, __pyx_float__0076)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_11);
  __Pyx_GIVEREF(__pyx_float_18_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 34, __pyx_float_18_11)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0071);
  __Pyx_GIVEREF(__pyx_float__0071);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 35, __pyx_float__0071)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_12);
  __Pyx_GIVEREF(__pyx_float_19_12);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 36, __pyx_float_19_12)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0045);
  __Pyx_GIVEREF(__pyx_float__0045);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 37, __pyx_float__0045)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_16);
  __Pyx_GIVEREF(__pyx_float_20_16);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 38, __pyx_float_20_16)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0038);
  __Pyx_GIVEREF(__pyx_float__0038);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 39, __pyx_float__0038)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_23);
  __Pyx_GIVEREF(__pyx_float_21_23);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 40, __pyx_float_21_23)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0213);
  __Pyx_GIVEREF(__pyx_float__0213);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 41, __pyx_float__0213)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_27);
  __Pyx_GIVEREF(__pyx_float_22_27);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 42, __pyx_float_22_27)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0332);
  __Pyx_GIVEREF(__pyx_float__0332);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 43, __pyx_float__0332)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_34);
  __Pyx_GIVEREF(__pyx_float_23_34);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 44, __pyx_float_23_34)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0082);
  __Pyx_GIVEREF(__pyx_float__0082);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 45, __pyx_float__0082)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_24_34);
  __Pyx_GIVEREF(__pyx_float_24_34);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 46, __pyx_float_24_34)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0014);
  __Pyx_GIVEREF(__pyx_float__0014);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 47, __pyx_float__0014)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_25_42);
  __Pyx_GIVEREF(__pyx_float_25_42);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 48, __pyx_float_25_42)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0024);
  __Pyx_GIVEREF(__pyx_float__0024);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 49, __pyx_float__0024)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_26_47);
  __Pyx_GIVEREF(__pyx_float_26_47);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 50, __pyx_float_26_47)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 51, __pyx_float__0012)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_27_54);
  __Pyx_GIVEREF(__pyx_float_27_54);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 52, __pyx_float_27_54)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0014);
  __Pyx_GIVEREF(__pyx_float__0014);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 53, __pyx_float__0014)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_60);
  __Pyx_GIVEREF(__pyx_float_28_60);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 54, __pyx_float_28_60)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0024);
  __Pyx_GIVEREF(__pyx_float__0024);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 55, __pyx_float__0024)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_29_72);
  __Pyx_GIVEREF(__pyx_float_29_72);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 56, __pyx_float_29_72)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0026);
  __Pyx_GIVEREF(__pyx_float__0026);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 57, __pyx_float__0026)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_30_10);
  __Pyx_GIVEREF(__pyx_float_30_10);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 58, __pyx_float_30_10)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 59, __pyx_float__0008)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_31_91);
  __Pyx_GIVEREF(__pyx_float_31_91);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 60, __pyx_float_31_91)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0021);
  __Pyx_GIVEREF(__pyx_float__0021);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 61, __pyx_float__0021)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_32_13);
  __Pyx_GIVEREF(__pyx_float_32_13);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 62, __pyx_float_32_13)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 63, __pyx_float__0011)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_33_02);
  __Pyx_GIVEREF(__pyx_float_33_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 64, __pyx_float_33_02)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 65, __pyx_float__0007)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_34_09);
  __Pyx_GIVEREF(__pyx_float_34_09);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 66, __pyx_float_34_09)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0014);
  __Pyx_GIVEREF(__pyx_float__0014);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 67, __pyx_float__0014)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_35_17);
  __Pyx_GIVEREF(__pyx_float_35_17);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 68, __pyx_float_35_17)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 69, __pyx_float__0007)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_36_27);
  __Pyx_GIVEREF(__pyx_float_36_27);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 70, __pyx_float_36_27)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0024);
  __Pyx_GIVEREF(__pyx_float__0024);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 71, __pyx_float__0024)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_37_39);
  __Pyx_GIVEREF(__pyx_float_37_39);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 72, __pyx_float_37_39)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0029);
  __Pyx_GIVEREF(__pyx_float__0029);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 73, __pyx_float__0029)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_38_58);
  __Pyx_GIVEREF(__pyx_float_38_58);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 74, __pyx_float_38_58)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0014);
  __Pyx_GIVEREF(__pyx_float__0014);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 75, __pyx_float__0014)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_39_65);
  __Pyx_GIVEREF(__pyx_float_39_65);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 76, __pyx_float_39_65)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0017);
  __Pyx_GIVEREF(__pyx_float__0017);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 77, __pyx_float__0017)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_40_95);
  __Pyx_GIVEREF(__pyx_float_40_95);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 78, __pyx_float_40_95)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 79, __pyx_float__0012)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_41_97);
  __Pyx_GIVEREF(__pyx_float_41_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 80, __pyx_float_41_97)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 81, __pyx_float__0004)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_42_43);
  __Pyx_GIVEREF(__pyx_float_42_43);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 82, __pyx_float_42_43)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 83, __pyx_float__0002)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_43_49);
  __Pyx_GIVEREF(__pyx_float_43_49);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 84, __pyx_float_43_49)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 85, __pyx_float__0001)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_44_31);
  __Pyx_GIVEREF(__pyx_float_44_31);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 86, __pyx_float_44_31)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 87, __pyx_float__0012)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_45_42);
  __Pyx_GIVEREF(__pyx_float_45_42);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 88, __pyx_float_45_42)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0031);
  __Pyx_GIVEREF(__pyx_float__0031);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 89, __pyx_float__0031)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_46_62);
  __Pyx_GIVEREF(__pyx_float_46_62);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 90, __pyx_float_46_62)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0017);
  __Pyx_GIVEREF(__pyx_float__0017);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 91, __pyx_float__0017)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_47_82);
  __Pyx_GIVEREF(__pyx_float_47_82);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 92, __pyx_float_47_82)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0013);
  __Pyx_GIVEREF(__pyx_float__0013);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 93, __pyx_float__0013)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_49_14);
  __Pyx_GIVEREF(__pyx_float_49_14);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 94, __pyx_float_49_14)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0013);
  __Pyx_GIVEREF(__pyx_float__0013);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 95, __pyx_float__0013)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_50_18);
  __Pyx_GIVEREF(__pyx_float_50_18);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 96, __pyx_float_50_18)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 97, __pyx_float__0010)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_51_54);
  __Pyx_GIVEREF(__pyx_float_51_54);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 98, __pyx_float_51_54)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 99, __pyx_float__0003)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_53_90);
  __Pyx_GIVEREF(__pyx_float_53_90);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 100, __pyx_float_53_90)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 101, __pyx_float__0006)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_55_06);
  __Pyx_GIVEREF(__pyx_float_55_06);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 102, __pyx_float_55_06)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 103, __pyx_float__0010)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_56_31);
  __Pyx_GIVEREF(__pyx_float_56_31);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 104, __pyx_float_56_31)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 105, __pyx_float__0003)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_57_63);
  __Pyx_GIVEREF(__pyx_float_57_63);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 106, __pyx_float_57_63)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 107, __pyx_float__0001)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_59_02);
  __Pyx_GIVEREF(__pyx_float_59_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 108, __pyx_float_59_02)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 109, __pyx_float__0003)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_60_09);
  __Pyx_GIVEREF(__pyx_float_60_09);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 110, __pyx_float_60_09)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 111, __pyx_float__0004)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_60_35);
  __Pyx_GIVEREF(__pyx_float_60_35);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 112, __pyx_float_60_35)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 113, __pyx_float__0004)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_61_62);
  __Pyx_GIVEREF(__pyx_float_61_62);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 114, __pyx_float_61_62)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 115, __pyx_float__0009)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_63_97);
  __Pyx_GIVEREF(__pyx_float_63_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 116, __pyx_float_63_97)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 117, __pyx_float__0001)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_65_19);
  __Pyx_GIVEREF(__pyx_float_65_19);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 118, __pyx_float_65_19)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 119, __pyx_float__0001)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_65_54);
  __Pyx_GIVEREF(__pyx_float_65_54);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 120, __pyx_float_65_54)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 121, __pyx_float__0002)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_66_92);
  __Pyx_GIVEREF(__pyx_float_66_92);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 122, __pyx_float_66_92)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 123, __pyx_float__0002)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_67_94);
  __Pyx_GIVEREF(__pyx_float_67_94);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 124, __pyx_float_67_94)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 125, __pyx_float__0002)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_69_17);
  __Pyx_GIVEREF(__pyx_float_69_17);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 126, __pyx_float_69_17)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 127, __pyx_float__0003)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_69_60);
  __Pyx_GIVEREF(__pyx_float_69_60);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 128, __pyx_float_69_60)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 129, __pyx_float__0004)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_70_88);
  __Pyx_GIVEREF(__pyx_float_70_88);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 130, __pyx_float_70_88)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 131, __pyx_float__0002)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_72_24);
  __Pyx_GIVEREF(__pyx_float_72_24);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 132, __pyx_float_72_24)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 133, __pyx_float__0002)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_76_12);
  __Pyx_GIVEREF(__pyx_float_76_12);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 134, __pyx_float_76_12)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 135, __pyx_float__0001)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_78_94);
  __Pyx_GIVEREF(__pyx_float_78_94);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 136, __pyx_float_78_94)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 137, __pyx_float__0001)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_81_75);
  __Pyx_GIVEREF(__pyx_float_81_75);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 138, __pyx_float_81_75)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 139, __pyx_float__0001)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_82_06);
  __Pyx_GIVEREF(__pyx_float_82_06);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 140, __pyx_float_82_06)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 141, __pyx_float__0001)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_83_53);
  __Pyx_GIVEREF(__pyx_float_83_53);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 142, __pyx_float_83_53)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 143, __pyx_float__0001)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_90_29);
  __Pyx_GIVEREF(__pyx_float_90_29);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 144, __pyx_float_90_29)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 145, __pyx_float__0002)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_91_75);
  __Pyx_GIVEREF(__pyx_float_91_75);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 146, __pyx_float_91_75)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 147, __pyx_float__0001)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_92_09);
  __Pyx_GIVEREF(__pyx_float_92_09);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 148, __pyx_float_92_09)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 149, __pyx_float__0002)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_93_28);
  __Pyx_GIVEREF(__pyx_float_93_28);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 150, __pyx_float_93_28)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 151, __pyx_float__0001)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_97_07);
  __Pyx_GIVEREF(__pyx_float_97_07);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 152, __pyx_float_97_07)) __PYX_ERR(0, 36, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_10, 153, __pyx_float__0001)) __PYX_ERR(0, 36, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":37
 * 			        [2.00,.0311,2.99,.0086,3.99,.0266,4.97,.0123,5.98,.0235,6.97,.0161,7.97,.0008 ,8.96,.0088,9.96,.0621,10.99,.0080,11.99,.0034,12.99,.0300,14.03,.0228,15.04,.0105,16.03,.0004 ,17.06,.0036,18.09,.0094,18.95,.0009,20.17,.0071,21.21,.0161,22.25,.0106,23.28,.0104,24.33,.0008 ,25.38,.0030,26.46,.0035,27.50,.0026,28.59,.0028,29.66,.0128,30.75,.0139,31.81,.0038,32.93,.0006 ,34.04,.0004,35.16,.0005,36.25,.0023,37.35,.0012,38.46,.0021,39.59,.0035,40.71,.0006,41.86,.0007 ,42.42,.0001,43.46,.0003,44.17,.0032,45.29,.0013,46.57,.0004,47.72,.0011,48.79,.0005,50.11,.0005 ,51.29,.0003,52.47,.0002,53.68,.0004,55.02,.0005,56.18,.0003,57.41,.0003,58.75,.0007,59.33,.0009 ,60.00,.0004,61.34,.0001,64.97,.0003,65.20,.0002,66.48,.0002,67.83,.0002,68.90,.0003,70.25,.0003 ,71.59,.0002,73.68,.0001,75.92,.0001,77.08,.0002,78.45,.0002,81.56,.0002,82.99,.0001,88.39,.0001],
 * 			        [0.97,.0059,1.98,.0212,2.99,.0153,3.99,.0227,4.96,.0215,5.97,.0153,6.98,.0085 ,7.98,.0007,8.97,.0179,9.98,.0512,10.98,.0322,12.00,.0098,13.02,.0186,14.00,.0099,15.05,.0109 ,15.88,.0011,17.07,.0076,18.11,.0071,19.12,.0045,20.16,.0038,21.23,.0213,22.27,.0332,23.34,.0082 ,24.34,.0014,25.42,.0024,26.47,.0012,27.54,.0014,28.60,.0024,29.72,.0026,30.10,.0008,31.91,.0021 ,32.13,.0011,33.02,.0007,34.09,.0014,35.17,.0007,36.27,.0024,37.39,.0029,38.58,.0014,39.65,.0017 ,40.95,.0012,41.97,.0004,42.43,.0002,43.49,.0001,44.31,.0012,45.42,.0031,46.62,.0017,47.82,.0013 ,49.14,.0013,50.18,.0010,51.54,.0003,53.90,.0006,55.06,.0010,56.31,.0003,57.63,.0001,59.02,.0003 ,60.09,.0004,60.35,.0004,61.62,.0009,63.97,.0001,65.19,.0001,65.54,.0002,66.92,.0002,67.94,.0002 ,69.17,.0003,69.60,.0004,70.88,.0002,72.24,.0002,76.12,.0001,78.94,.0001,81.75,.0001,82.06,.0001 ,83.53,.0001,90.29,.0002,91.75,.0001,92.09,.0002,93.28,.0001,97.07,.0001],
 * 			        [1.98,.0159,2.98,.1008,3.98,.0365,4.98,.0133,5.97,.0101,6.97,.0115,7.97,.0007 ,8.99,.0349,10.01,.0342,11.01,.0236,12.00,.0041,13.02,.0114,14.05,.0137,15.06,.0100,16.05,.0007 ,17.04,.0009,18.12,.0077,19.15,.0023,20.12,.0017,21.24,.0113,22.26,.0126,23.30,.0093,24.36,.0007 ,25.43,.0007,26.47,.0009,27.55,.0013,28.59,.0025,29.61,.0010,30.77,.0021,31.86,.0023,32.96,.0003 ,34.03,.0007,35.06,.0005,36.20,.0006,37.34,.0006,38.36,.0009,39.60,.0016,40.69,.0005,41.77,.0002 ,42.92,.0002,44.02,.0003,45.24,.0006,46.33,.0004,47.50,.0007,48.71,.0007,49.87,.0002,51.27,.0002 ,53.42,.0003,55.88,.0003,57.10,.0004,58.34,.0002,59.86,.0003,61.13,.0003,67.18,.0001,68.50,.0001 ,71.17,.0001,83.91,.0001,90.55,.0001],             # <<<<<<<<<<<<<<
 * 			        [0.98,.0099,2.00,.0181,2.99,.0353,3.98,.0285,4.97,.0514,5.96,.0402,6.96,.0015 ,7.98,.0012,8.98,.0175,9.98,.0264,10.98,.0392,11.98,.0236,13.00,.0153,14.04,.0049,15.00,.0089 ,16.01,.0001,17.03,.0106,18.03,.0028,19.05,.0024,20.08,.0040,21.11,.0103,22.12,.0104,23.20,.0017 ,24.19,.0008,25.20,.0007,26.24,.0011,27.36,.0009,27.97,.0030,29.40,.0044,30.37,.0019,31.59,.0017 ,32.65,.0008,33.59,.0005,34.79,.0009,35.75,.0027,36.88,.0035,37.93,.0039,39.00,.0031,40.08,.0025 ,41.16,.0010,43.25,.0004,44.52,.0012,45.62,.0023,45.85,.0012,47.00,.0006,47.87,.0008,48.99,.0003 ,50.48,.0003,51.62,.0001,52.43,.0001,53.56,.0002,54.76,.0002,56.04,.0002,56.68,.0006,57.10,.0003 ,58.28,.0005,59.47,.0003,59.96,.0002,60.67,.0001,63.08,.0002,64.29,.0002,66.72,.0001,67.97,.0001 ,68.65,.0001,70.43,.0001,79.38,.0001,80.39,.0001,82.39,.0001],
 * 			        [1.00,.0765,1.99,.0151,2.99,.0500,3.99,.0197,5.00,.0260,6.00,.0145,6.98,.0128 ,7.97,.0004,8.98,.0158,9.99,.0265,11.02,.0290,12.02,.0053,13.03,.0242,14.03,.0103,15.06,.0054 ,16.04,.0006,17.08,.0008,18.10,.0058,19.16,.0011,20.16,.0055,21.18,.0040,22.20,.0019,23.22,.0014 ,24.05,.0005,25.31,.0019,26.38,.0018,27.44,.0022,28.45,.0024,29.57,.0073,30.58,.0032,31.66,.0071 ,32.73,.0015,33.85,.0005,34.96,.0003,36.00,.0020,37.11,.0018,38.18,.0055,39.23,.0006,40.33,.0004 ,41.52,.0003,43.41,.0028,45.05,.0003,45.99,.0002,47.07,.0003,48.52,.0002,49.48,.0003,50.63,.0003 ,51.81,.0002,54.05,.0002,55.24,.0001,56.62,.0001,57.81,.0004,59.16,.0013,60.23,.0003,66.44,.0001 ,68.99,.0004,75.49,.0001,87.56,.0004],
 */
  __pyx_t_11 = PyList_New(116); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_INCREF(__pyx_float_1_98);
  __Pyx_GIVEREF(__pyx_float_1_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 0, __pyx_float_1_98)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0159);
  __Pyx_GIVEREF(__pyx_float__0159);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 1, __pyx_float__0159)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_98);
  __Pyx_GIVEREF(__pyx_float_2_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 2, __pyx_float_2_98)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1008);
  __Pyx_GIVEREF(__pyx_float__1008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 3, __pyx_float__1008)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_98);
  __Pyx_GIVEREF(__pyx_float_3_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 4, __pyx_float_3_98)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0365);
  __Pyx_GIVEREF(__pyx_float__0365);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 5, __pyx_float__0365)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_98);
  __Pyx_GIVEREF(__pyx_float_4_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 6, __pyx_float_4_98)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0133);
  __Pyx_GIVEREF(__pyx_float__0133);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 7, __pyx_float__0133)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_97);
  __Pyx_GIVEREF(__pyx_float_5_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 8, __pyx_float_5_97)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0101);
  __Pyx_GIVEREF(__pyx_float__0101);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 9, __pyx_float__0101)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_97);
  __Pyx_GIVEREF(__pyx_float_6_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 10, __pyx_float_6_97)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0115);
  __Pyx_GIVEREF(__pyx_float__0115);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 11, __pyx_float__0115)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_97);
  __Pyx_GIVEREF(__pyx_float_7_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 12, __pyx_float_7_97)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 13, __pyx_float__0007)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_99);
  __Pyx_GIVEREF(__pyx_float_8_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 14, __pyx_float_8_99)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0349);
  __Pyx_GIVEREF(__pyx_float__0349);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 15, __pyx_float__0349)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_01);
  __Pyx_GIVEREF(__pyx_float_10_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 16, __pyx_float_10_01)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0342);
  __Pyx_GIVEREF(__pyx_float__0342);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 17, __pyx_float__0342)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_01);
  __Pyx_GIVEREF(__pyx_float_11_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 18, __pyx_float_11_01)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0236);
  __Pyx_GIVEREF(__pyx_float__0236);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 19, __pyx_float__0236)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_00);
  __Pyx_GIVEREF(__pyx_float_12_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 20, __pyx_float_12_00)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0041);
  __Pyx_GIVEREF(__pyx_float__0041);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 21, __pyx_float__0041)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_02);
  __Pyx_GIVEREF(__pyx_float_13_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 22, __pyx_float_13_02)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0114);
  __Pyx_GIVEREF(__pyx_float__0114);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 23, __pyx_float__0114)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_05);
  __Pyx_GIVEREF(__pyx_float_14_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 24, __pyx_float_14_05)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0137);
  __Pyx_GIVEREF(__pyx_float__0137);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 25, __pyx_float__0137)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_06);
  __Pyx_GIVEREF(__pyx_float_15_06);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 26, __pyx_float_15_06)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0100);
  __Pyx_GIVEREF(__pyx_float__0100);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 27, __pyx_float__0100)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_05);
  __Pyx_GIVEREF(__pyx_float_16_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 28, __pyx_float_16_05)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 29, __pyx_float__0007)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_04);
  __Pyx_GIVEREF(__pyx_float_17_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 30, __pyx_float_17_04)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 31, __pyx_float__0009)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_12);
  __Pyx_GIVEREF(__pyx_float_18_12);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 32, __pyx_float_18_12)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0077);
  __Pyx_GIVEREF(__pyx_float__0077);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 33, __pyx_float__0077)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_15);
  __Pyx_GIVEREF(__pyx_float_19_15);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 34, __pyx_float_19_15)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0023);
  __Pyx_GIVEREF(__pyx_float__0023);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 35, __pyx_float__0023)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_12);
  __Pyx_GIVEREF(__pyx_float_20_12);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 36, __pyx_float_20_12)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0017);
  __Pyx_GIVEREF(__pyx_float__0017);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 37, __pyx_float__0017)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_24);
  __Pyx_GIVEREF(__pyx_float_21_24);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 38, __pyx_float_21_24)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0113);
  __Pyx_GIVEREF(__pyx_float__0113);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 39, __pyx_float__0113)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_26);
  __Pyx_GIVEREF(__pyx_float_22_26);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 40, __pyx_float_22_26)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0126);
  __Pyx_GIVEREF(__pyx_float__0126);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 41, __pyx_float__0126)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_30);
  __Pyx_GIVEREF(__pyx_float_23_30);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 42, __pyx_float_23_30)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0093);
  __Pyx_GIVEREF(__pyx_float__0093);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 43, __pyx_float__0093)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_24_36);
  __Pyx_GIVEREF(__pyx_float_24_36);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 44, __pyx_float_24_36)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 45, __pyx_float__0007)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_25_43);
  __Pyx_GIVEREF(__pyx_float_25_43);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 46, __pyx_float_25_43)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 47, __pyx_float__0007)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_26_47);
  __Pyx_GIVEREF(__pyx_float_26_47);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 48, __pyx_float_26_47)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 49, __pyx_float__0009)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_27_55);
  __Pyx_GIVEREF(__pyx_float_27_55);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 50, __pyx_float_27_55)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0013);
  __Pyx_GIVEREF(__pyx_float__0013);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 51, __pyx_float__0013)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_59);
  __Pyx_GIVEREF(__pyx_float_28_59);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 52, __pyx_float_28_59)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0025);
  __Pyx_GIVEREF(__pyx_float__0025);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 53, __pyx_float__0025)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_29_61);
  __Pyx_GIVEREF(__pyx_float_29_61);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 54, __pyx_float_29_61)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 55, __pyx_float__0010)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_30_77);
  __Pyx_GIVEREF(__pyx_float_30_77);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 56, __pyx_float_30_77)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0021);
  __Pyx_GIVEREF(__pyx_float__0021);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 57, __pyx_float__0021)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_31_86);
  __Pyx_GIVEREF(__pyx_float_31_86);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 58, __pyx_float_31_86)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0023);
  __Pyx_GIVEREF(__pyx_float__0023);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 59, __pyx_float__0023)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_32_96);
  __Pyx_GIVEREF(__pyx_float_32_96);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 60, __pyx_float_32_96)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 61, __pyx_float__0003)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_34_03);
  __Pyx_GIVEREF(__pyx_float_34_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 62, __pyx_float_34_03)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 63, __pyx_float__0007)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_35_06);
  __Pyx_GIVEREF(__pyx_float_35_06);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 64, __pyx_float_35_06)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 65, __pyx_float__0005)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_36_20);
  __Pyx_GIVEREF(__pyx_float_36_20);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 66, __pyx_float_36_20)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 67, __pyx_float__0006)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_37_34);
  __Pyx_GIVEREF(__pyx_float_37_34);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 68, __pyx_float_37_34)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 69, __pyx_float__0006)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_38_36);
  __Pyx_GIVEREF(__pyx_float_38_36);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 70, __pyx_float_38_36)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 71, __pyx_float__0009)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_39_60);
  __Pyx_GIVEREF(__pyx_float_39_60);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 72, __pyx_float_39_60)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0016);
  __Pyx_GIVEREF(__pyx_float__0016);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 73, __pyx_float__0016)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_40_69);
  __Pyx_GIVEREF(__pyx_float_40_69);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 74, __pyx_float_40_69)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 75, __pyx_float__0005)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_41_77);
  __Pyx_GIVEREF(__pyx_float_41_77);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 76, __pyx_float_41_77)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 77, __pyx_float__0002)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_42_92);
  __Pyx_GIVEREF(__pyx_float_42_92);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 78, __pyx_float_42_92)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 79, __pyx_float__0002)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_44_02);
  __Pyx_GIVEREF(__pyx_float_44_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 80, __pyx_float_44_02)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 81, __pyx_float__0003)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_45_24);
  __Pyx_GIVEREF(__pyx_float_45_24);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 82, __pyx_float_45_24)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 83, __pyx_float__0006)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_46_33);
  __Pyx_GIVEREF(__pyx_float_46_33);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 84, __pyx_float_46_33)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 85, __pyx_float__0004)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_47_50);
  __Pyx_GIVEREF(__pyx_float_47_50);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 86, __pyx_float_47_50)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 87, __pyx_float__0007)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_48_71);
  __Pyx_GIVEREF(__pyx_float_48_71);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 88, __pyx_float_48_71)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 89, __pyx_float__0007)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_49_87);
  __Pyx_GIVEREF(__pyx_float_49_87);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 90, __pyx_float_49_87)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 91, __pyx_float__0002)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_51_27);
  __Pyx_GIVEREF(__pyx_float_51_27);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 92, __pyx_float_51_27)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 93, __pyx_float__0002)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_53_42);
  __Pyx_GIVEREF(__pyx_float_53_42);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 94, __pyx_float_53_42)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 95, __pyx_float__0003)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_55_88);
  __Pyx_GIVEREF(__pyx_float_55_88);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 96, __pyx_float_55_88)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 97, __pyx_float__0003)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_57_10);
  __Pyx_GIVEREF(__pyx_float_57_10);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 98, __pyx_float_57_10)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 99, __pyx_float__0004)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_58_34);
  __Pyx_GIVEREF(__pyx_float_58_34);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 100, __pyx_float_58_34)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 101, __pyx_float__0002)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_59_86);
  __Pyx_GIVEREF(__pyx_float_59_86);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 102, __pyx_float_59_86)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 103, __pyx_float__0003)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_61_13);
  __Pyx_GIVEREF(__pyx_float_61_13);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 104, __pyx_float_61_13)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 105, __pyx_float__0003)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_67_18);
  __Pyx_GIVEREF(__pyx_float_67_18);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 106, __pyx_float_67_18)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 107, __pyx_float__0001)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_68_50);
  __Pyx_GIVEREF(__pyx_float_68_50);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 108, __pyx_float_68_50)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 109, __pyx_float__0001)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_71_17);
  __Pyx_GIVEREF(__pyx_float_71_17);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 110, __pyx_float_71_17)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 111, __pyx_float__0001)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_83_91);
  __Pyx_GIVEREF(__pyx_float_83_91);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 112, __pyx_float_83_91)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 113, __pyx_float__0001)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_90_55);
  __Pyx_GIVEREF(__pyx_float_90_55);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 114, __pyx_float_90_55)) __PYX_ERR(0, 37, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_11, 115, __pyx_float__0001)) __PYX_ERR(0, 37, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":38
 * 			        [0.97,.0059,1.98,.0212,2.99,.0153,3.99,.0227,4.96,.0215,5.97,.0153,6.98,.0085 ,7.98,.0007,8.97,.0179,9.98,.0512,10.98,.0322,12.00,.0098,13.02,.0186,14.00,.0099,15.05,.0109 ,15.88,.0011,17.07,.0076,18.11,.0071,19.12,.0045,20.16,.0038,21.23,.0213,22.27,.0332,23.34,.0082 ,24.34,.0014,25.42,.0024,26.47,.0012,27.54,.0014,28.60,.0024,29.72,.0026,30.10,.0008,31.91,.0021 ,32.13,.0011,33.02,.0007,34.09,.0014,35.17,.0007,36.27,.0024,37.39,.0029,38.58,.0014,39.65,.0017 ,40.95,.0012,41.97,.0004,42.43,.0002,43.49,.0001,44.31,.0012,45.42,.0031,46.62,.0017,47.82,.0013 ,49.14,.0013,50.18,.0010,51.54,.0003,53.90,.0006,55.06,.0010,56.31,.0003,57.63,.0001,59.02,.0003 ,60.09,.0004,60.35,.0004,61.62,.0009,63.97,.0001,65.19,.0001,65.54,.0002,66.92,.0002,67.94,.0002 ,69.17,.0003,69.60,.0004,70.88,.0002,72.24,.0002,76.12,.0001,78.94,.0001,81.75,.0001,82.06,.0001 ,83.53,.0001,90.29,.0002,91.75,.0001,92.09,.0002,93.28,.0001,97.07,.0001],
 * 			        [1.98,.0159,2.98,.1008,3.98,.0365,4.98,.0133,5.97,.0101,6.97,.0115,7.97,.0007 ,8.99,.0349,10.01,.0342,11.01,.0236,12.00,.0041,13.02,.0114,14.05,.0137,15.06,.0100,16.05,.0007 ,17.04,.0009,18.12,.0077,19.15,.0023,20.12,.0017,21.24,.0113,22.26,.0126,23.30,.0093,24.36,.0007 ,25.43,.0007,26.47,.0009,27.55,.0013,28.59,.0025,29.61,.0010,30.77,.0021,31.86,.0023,32.96,.0003 ,34.03,.0007,35.06,.0005,36.20,.0006,37.34,.0006,38.36,.0009,39.60,.0016,40.69,.0005,41.77,.0002 ,42.92,.0002,44.02,.0003,45.24,.0006,46.33,.0004,47.50,.0007,48.71,.0007,49.87,.0002,51.27,.0002 ,53.42,.0003,55.88,.0003,57.10,.0004,58.34,.0002,59.86,.0003,61.13,.0003,67.18,.0001,68.50,.0001 ,71.17,.0001,83.91,.0001,90.55,.0001],
 * 			        [0.98,.0099,2.00,.0181,2.99,.0353,3.98,.0285,4.97,.0514,5.96,.0402,6.96,.0015 ,7.98,.0012,8.98,.0175,9.98,.0264,10.98,.0392,11.98,.0236,13.00,.0153,14.04,.0049,15.00,.0089 ,16.01,.0001,17.03,.0106,18.03,.0028,19.05,.0024,20.08,.0040,21.11,.0103,22.12,.0104,23.20,.0017 ,24.19,.0008,25.20,.0007,26.24,.0011,27.36,.0009,27.97,.0030,29.40,.0044,30.37,.0019,31.59,.0017 ,32.65,.0008,33.59,.0005,34.79,.0009,35.75,.0027,36.88,.0035,37.93,.0039,39.00,.0031,40.08,.0025 ,41.16,.0010,43.25,.0004,44.52,.0012,45.62,.0023,45.85,.0012,47.00,.0006,47.87,.0008,48.99,.0003 ,50.48,.0003,51.62,.0001,52.43,.0001,53.56,.0002,54.76,.0002,56.04,.0002,56.68,.0006,57.10,.0003 ,58.28,.0005,59.47,.0003,59.96,.0002,60.67,.0001,63.08,.0002,64.29,.0002,66.72,.0001,67.97,.0001 ,68.65,.0001,70.43,.0001,79.38,.0001,80.39,.0001,82.39,.0001],             # <<<<<<<<<<<<<<
 * 			        [1.00,.0765,1.99,.0151,2.99,.0500,3.99,.0197,5.00,.0260,6.00,.0145,6.98,.0128 ,7.97,.0004,8.98,.0158,9.99,.0265,11.02,.0290,12.02,.0053,13.03,.0242,14.03,.0103,15.06,.0054 ,16.04,.0006,17.08,.0008,18.10,.0058,19.16,.0011,20.16,.0055,21.18,.0040,22.20,.0019,23.22,.0014 ,24.05,.0005,25.31,.0019,26.38,.0018,27.44,.0022,28.45,.0024,29.57,.0073,30.58,.0032,31.66,.0071 ,32.73,.0015,33.85,.0005,34.96,.0003,36.00,.0020,37.11,.0018,38.18,.0055,39.23,.0006,40.33,.0004 ,41.52,.0003,43.41,.0028,45.05,.0003,45.99,.0002,47.07,.0003,48.52,.0002,49.48,.0003,50.63,.0003 ,51.81,.0002,54.05,.0002,55.24,.0001,56.62,.0001,57.81,.0004,59.16,.0013,60.23,.0003,66.44,.0001 ,68.99,.0004,75.49,.0001,87.56,.0004],
 * 			        [0.98,.0629,1.99,.0232,2.98,.0217,4.00,.0396,4.98,.0171,5.97,.0098,6.99,.0167 ,7.99,.0003,8.98,.0192,9.98,.0266,10.99,.0256,12.01,.0061,13.02,.0135,14.02,.0062,15.05,.0158 ,16.06,.0018,17.08,.0101,18.09,.0053,19.11,.0074,20.13,.0020,21.17,.0052,22.22,.0077,23.24,.0035 ,24.00,.0009,25.32,.0016,26.40,.0022,27.43,.0005,28.55,.0026,29.60,.0026,30.65,.0010,31.67,.0019 ,32.77,.0008,33.81,.0003,34.91,.0003,36.01,.0005,37.11,.0010,38.20,.0014,39.29,.0039,40.43,.0012 ,41.50,.0006,43.38,.0017,43.75,.0002,44.94,.0005,46.13,.0002,47.11,.0003,48.28,.0005,48.42,.0005 ,49.44,.0003,50.76,.0004,51.93,.0002,54.15,.0003,55.31,.0005,55.50,.0003,56.98,.0003,57.90,.0004 ,60.33,.0002,61.39,.0001,61.59,.0001,65.09,.0002,66.34,.0001,68.85,.0001,70.42,.0002,71.72,.0001 ,73.05,.0003,79.65,.0001,85.28,.0002,93.52,.0001],
 */
  __pyx_t_12 = PyList_New(136); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_INCREF(__pyx_float_0_98);
  __Pyx_GIVEREF(__pyx_float_0_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 0, __pyx_float_0_98)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0099);
  __Pyx_GIVEREF(__pyx_float__0099);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 1, __pyx_float__0099)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_00);
  __Pyx_GIVEREF(__pyx_float_2_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 2, __pyx_float_2_00)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0181);
  __Pyx_GIVEREF(__pyx_float__0181);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 3, __pyx_float__0181)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_99);
  __Pyx_GIVEREF(__pyx_float_2_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 4, __pyx_float_2_99)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0353);
  __Pyx_GIVEREF(__pyx_float__0353);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 5, __pyx_float__0353)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_98);
  __Pyx_GIVEREF(__pyx_float_3_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 6, __pyx_float_3_98)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0285);
  __Pyx_GIVEREF(__pyx_float__0285);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 7, __pyx_float__0285)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_97);
  __Pyx_GIVEREF(__pyx_float_4_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 8, __pyx_float_4_97)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0514);
  __Pyx_GIVEREF(__pyx_float__0514);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 9, __pyx_float__0514)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_96);
  __Pyx_GIVEREF(__pyx_float_5_96);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 10, __pyx_float_5_96)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0402);
  __Pyx_GIVEREF(__pyx_float__0402);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 11, __pyx_float__0402)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_96);
  __Pyx_GIVEREF(__pyx_float_6_96);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 12, __pyx_float_6_96)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0015);
  __Pyx_GIVEREF(__pyx_float__0015);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 13, __pyx_float__0015)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_98);
  __Pyx_GIVEREF(__pyx_float_7_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 14, __pyx_float_7_98)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 15, __pyx_float__0012)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_98);
  __Pyx_GIVEREF(__pyx_float_8_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 16, __pyx_float_8_98)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0175);
  __Pyx_GIVEREF(__pyx_float__0175);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 17, __pyx_float__0175)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_98);
  __Pyx_GIVEREF(__pyx_float_9_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 18, __pyx_float_9_98)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0264);
  __Pyx_GIVEREF(__pyx_float__0264);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 19, __pyx_float__0264)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_98);
  __Pyx_GIVEREF(__pyx_float_10_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 20, __pyx_float_10_98)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0392);
  __Pyx_GIVEREF(__pyx_float__0392);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 21, __pyx_float__0392)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_98);
  __Pyx_GIVEREF(__pyx_float_11_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 22, __pyx_float_11_98)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0236);
  __Pyx_GIVEREF(__pyx_float__0236);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 23, __pyx_float__0236)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_00);
  __Pyx_GIVEREF(__pyx_float_13_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 24, __pyx_float_13_00)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0153);
  __Pyx_GIVEREF(__pyx_float__0153);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 25, __pyx_float__0153)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_04);
  __Pyx_GIVEREF(__pyx_float_14_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 26, __pyx_float_14_04)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0049);
  __Pyx_GIVEREF(__pyx_float__0049);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 27, __pyx_float__0049)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_00);
  __Pyx_GIVEREF(__pyx_float_15_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 28, __pyx_float_15_00)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0089);
  __Pyx_GIVEREF(__pyx_float__0089);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 29, __pyx_float__0089)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_01);
  __Pyx_GIVEREF(__pyx_float_16_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 30, __pyx_float_16_01)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 31, __pyx_float__0001)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_03);
  __Pyx_GIVEREF(__pyx_float_17_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 32, __pyx_float_17_03)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0106);
  __Pyx_GIVEREF(__pyx_float__0106);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 33, __pyx_float__0106)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_03);
  __Pyx_GIVEREF(__pyx_float_18_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 34, __pyx_float_18_03)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0028);
  __Pyx_GIVEREF(__pyx_float__0028);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 35, __pyx_float__0028)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_05);
  __Pyx_GIVEREF(__pyx_float_19_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 36, __pyx_float_19_05)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0024);
  __Pyx_GIVEREF(__pyx_float__0024);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 37, __pyx_float__0024)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_08);
  __Pyx_GIVEREF(__pyx_float_20_08);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 38, __pyx_float_20_08)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0040);
  __Pyx_GIVEREF(__pyx_float__0040);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 39, __pyx_float__0040)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_11);
  __Pyx_GIVEREF(__pyx_float_21_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 40, __pyx_float_21_11)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0103);
  __Pyx_GIVEREF(__pyx_float__0103);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 41, __pyx_float__0103)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_12);
  __Pyx_GIVEREF(__pyx_float_22_12);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 42, __pyx_float_22_12)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0104);
  __Pyx_GIVEREF(__pyx_float__0104);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 43, __pyx_float__0104)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_20);
  __Pyx_GIVEREF(__pyx_float_23_20);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 44, __pyx_float_23_20)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0017);
  __Pyx_GIVEREF(__pyx_float__0017);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 45, __pyx_float__0017)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_24_19);
  __Pyx_GIVEREF(__pyx_float_24_19);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 46, __pyx_float_24_19)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 47, __pyx_float__0008)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_25_20);
  __Pyx_GIVEREF(__pyx_float_25_20);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 48, __pyx_float_25_20)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 49, __pyx_float__0007)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_26_24);
  __Pyx_GIVEREF(__pyx_float_26_24);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 50, __pyx_float_26_24)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 51, __pyx_float__0011)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_27_36);
  __Pyx_GIVEREF(__pyx_float_27_36);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 52, __pyx_float_27_36)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 53, __pyx_float__0009)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_27_97);
  __Pyx_GIVEREF(__pyx_float_27_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 54, __pyx_float_27_97)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0030);
  __Pyx_GIVEREF(__pyx_float__0030);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 55, __pyx_float__0030)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_29_40);
  __Pyx_GIVEREF(__pyx_float_29_40);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 56, __pyx_float_29_40)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0044);
  __Pyx_GIVEREF(__pyx_float__0044);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 57, __pyx_float__0044)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_30_37);
  __Pyx_GIVEREF(__pyx_float_30_37);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 58, __pyx_float_30_37)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0019);
  __Pyx_GIVEREF(__pyx_float__0019);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 59, __pyx_float__0019)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_31_59);
  __Pyx_GIVEREF(__pyx_float_31_59);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 60, __pyx_float_31_59)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0017);
  __Pyx_GIVEREF(__pyx_float__0017);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 61, __pyx_float__0017)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_32_65);
  __Pyx_GIVEREF(__pyx_float_32_65);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 62, __pyx_float_32_65)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 63, __pyx_float__0008)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_33_59);
  __Pyx_GIVEREF(__pyx_float_33_59);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 64, __pyx_float_33_59)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 65, __pyx_float__0005)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_34_79);
  __Pyx_GIVEREF(__pyx_float_34_79);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 66, __pyx_float_34_79)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 67, __pyx_float__0009)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_35_75);
  __Pyx_GIVEREF(__pyx_float_35_75);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 68, __pyx_float_35_75)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0027);
  __Pyx_GIVEREF(__pyx_float__0027);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 69, __pyx_float__0027)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_36_88);
  __Pyx_GIVEREF(__pyx_float_36_88);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 70, __pyx_float_36_88)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0035);
  __Pyx_GIVEREF(__pyx_float__0035);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 71, __pyx_float__0035)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_37_93);
  __Pyx_GIVEREF(__pyx_float_37_93);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 72, __pyx_float_37_93)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0039);
  __Pyx_GIVEREF(__pyx_float__0039);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 73, __pyx_float__0039)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_39_00);
  __Pyx_GIVEREF(__pyx_float_39_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 74, __pyx_float_39_00)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0031);
  __Pyx_GIVEREF(__pyx_float__0031);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 75, __pyx_float__0031)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_40_08);
  __Pyx_GIVEREF(__pyx_float_40_08);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 76, __pyx_float_40_08)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0025);
  __Pyx_GIVEREF(__pyx_float__0025);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 77, __pyx_float__0025)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_41_16);
  __Pyx_GIVEREF(__pyx_float_41_16);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 78, __pyx_float_41_16)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 79, __pyx_float__0010)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_43_25);
  __Pyx_GIVEREF(__pyx_float_43_25);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 80, __pyx_float_43_25)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 81, __pyx_float__0004)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_44_52);
  __Pyx_GIVEREF(__pyx_float_44_52);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 82, __pyx_float_44_52)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 83, __pyx_float__0012)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_45_62);
  __Pyx_GIVEREF(__pyx_float_45_62);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 84, __pyx_float_45_62)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0023);
  __Pyx_GIVEREF(__pyx_float__0023);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 85, __pyx_float__0023)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_45_85);
  __Pyx_GIVEREF(__pyx_float_45_85);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 86, __pyx_float_45_85)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 87, __pyx_float__0012)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_47_00);
  __Pyx_GIVEREF(__pyx_float_47_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 88, __pyx_float_47_00)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 89, __pyx_float__0006)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_47_87);
  __Pyx_GIVEREF(__pyx_float_47_87);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 90, __pyx_float_47_87)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 91, __pyx_float__0008)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_48_99);
  __Pyx_GIVEREF(__pyx_float_48_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 92, __pyx_float_48_99)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 93, __pyx_float__0003)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_50_48);
  __Pyx_GIVEREF(__pyx_float_50_48);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 94, __pyx_float_50_48)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 95, __pyx_float__0003)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_51_62);
  __Pyx_GIVEREF(__pyx_float_51_62);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 96, __pyx_float_51_62)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 97, __pyx_float__0001)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_52_43);
  __Pyx_GIVEREF(__pyx_float_52_43);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 98, __pyx_float_52_43)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 99, __pyx_float__0001)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_53_56);
  __Pyx_GIVEREF(__pyx_float_53_56);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 100, __pyx_float_53_56)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 101, __pyx_float__0002)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_54_76);
  __Pyx_GIVEREF(__pyx_float_54_76);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 102, __pyx_float_54_76)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 103, __pyx_float__0002)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_56_04);
  __Pyx_GIVEREF(__pyx_float_56_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 104, __pyx_float_56_04)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 105, __pyx_float__0002)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_56_68);
  __Pyx_GIVEREF(__pyx_float_56_68);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 106, __pyx_float_56_68)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 107, __pyx_float__0006)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_57_10);
  __Pyx_GIVEREF(__pyx_float_57_10);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 108, __pyx_float_57_10)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 109, __pyx_float__0003)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_58_28);
  __Pyx_GIVEREF(__pyx_float_58_28);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 110, __pyx_float_58_28)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 111, __pyx_float__0005)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_59_47);
  __Pyx_GIVEREF(__pyx_float_59_47);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 112, __pyx_float_59_47)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 113, __pyx_float__0003)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_59_96);
  __Pyx_GIVEREF(__pyx_float_59_96);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 114, __pyx_float_59_96)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 115, __pyx_float__0002)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_60_67);
  __Pyx_GIVEREF(__pyx_float_60_67);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 116, __pyx_float_60_67)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 117, __pyx_float__0001)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_63_08);
  __Pyx_GIVEREF(__pyx_float_63_08);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 118, __pyx_float_63_08)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 119, __pyx_float__0002)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_64_29);
  __Pyx_GIVEREF(__pyx_float_64_29);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 120, __pyx_float_64_29)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 121, __pyx_float__0002)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_66_72);
  __Pyx_GIVEREF(__pyx_float_66_72);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 122, __pyx_float_66_72)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 123, __pyx_float__0001)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_67_97);
  __Pyx_GIVEREF(__pyx_float_67_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 124, __pyx_float_67_97)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 125, __pyx_float__0001)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_68_65);
  __Pyx_GIVEREF(__pyx_float_68_65);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 126, __pyx_float_68_65)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 127, __pyx_float__0001)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_70_43);
  __Pyx_GIVEREF(__pyx_float_70_43);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 128, __pyx_float_70_43)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 129, __pyx_float__0001)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_79_38);
  __Pyx_GIVEREF(__pyx_float_79_38);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 130, __pyx_float_79_38)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 131, __pyx_float__0001)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_80_39);
  __Pyx_GIVEREF(__pyx_float_80_39);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 132, __pyx_float_80_39)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 133, __pyx_float__0001)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_82_39);
  __Pyx_GIVEREF(__pyx_float_82_39);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 134, __pyx_float_82_39)) __PYX_ERR(0, 38, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_12, 135, __pyx_float__0001)) __PYX_ERR(0, 38, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":39
 * 			        [1.98,.0159,2.98,.1008,3.98,.0365,4.98,.0133,5.97,.0101,6.97,.0115,7.97,.0007 ,8.99,.0349,10.01,.0342,11.01,.0236,12.00,.0041,13.02,.0114,14.05,.0137,15.06,.0100,16.05,.0007 ,17.04,.0009,18.12,.0077,19.15,.0023,20.12,.0017,21.24,.0113,22.26,.0126,23.30,.0093,24.36,.0007 ,25.43,.0007,26.47,.0009,27.55,.0013,28.59,.0025,29.61,.0010,30.77,.0021,31.86,.0023,32.96,.0003 ,34.03,.0007,35.06,.0005,36.20,.0006,37.34,.0006,38.36,.0009,39.60,.0016,40.69,.0005,41.77,.0002 ,42.92,.0002,44.02,.0003,45.24,.0006,46.33,.0004,47.50,.0007,48.71,.0007,49.87,.0002,51.27,.0002 ,53.42,.0003,55.88,.0003,57.10,.0004,58.34,.0002,59.86,.0003,61.13,.0003,67.18,.0001,68.50,.0001 ,71.17,.0001,83.91,.0001,90.55,.0001],
 * 			        [0.98,.0099,2.00,.0181,2.99,.0353,3.98,.0285,4.97,.0514,5.96,.0402,6.96,.0015 ,7.98,.0012,8.98,.0175,9.98,.0264,10.98,.0392,11.98,.0236,13.00,.0153,14.04,.0049,15.00,.0089 ,16.01,.0001,17.03,.0106,18.03,.0028,19.05,.0024,20.08,.0040,21.11,.0103,22.12,.0104,23.20,.0017 ,24.19,.0008,25.20,.0007,26.24,.0011,27.36,.0009,27.97,.0030,29.40,.0044,30.37,.0019,31.59,.0017 ,32.65,.0008,33.59,.0005,34.79,.0009,35.75,.0027,36.88,.0035,37.93,.0039,39.00,.0031,40.08,.0025 ,41.16,.0010,43.25,.0004,44.52,.0012,45.62,.0023,45.85,.0012,47.00,.0006,47.87,.0008,48.99,.0003 ,50.48,.0003,51.62,.0001,52.43,.0001,53.56,.0002,54.76,.0002,56.04,.0002,56.68,.0006,57.10,.0003 ,58.28,.0005,59.47,.0003,59.96,.0002,60.67,.0001,63.08,.0002,64.29,.0002,66.72,.0001,67.97,.0001 ,68.65,.0001,70.43,.0001,79.38,.0001,80.39,.0001,82.39,.0001],
 * 			        [1.00,.0765,1.99,.0151,2.99,.0500,3.99,.0197,5.00,.0260,6.00,.0145,6.98,.0128 ,7.97,.0004,8.98,.0158,9.99,.0265,11.02,.0290,12.02,.0053,13.03,.0242,14.03,.0103,15.06,.0054 ,16.04,.0006,17.08,.0008,18.10,.0058,19.16,.0011,20.16,.0055,21.18,.0040,22.20,.0019,23.22,.0014 ,24.05,.0005,25.31,.0019,26.38,.0018,27.44,.0022,28.45,.0024,29.57,.0073,30.58,.0032,31.66,.0071 ,32.73,.0015,33.85,.0005,34.96,.0003,36.00,.0020,37.11,.0018,38.18,.0055,39.23,.0006,40.33,.0004 ,41.52,.0003,43.41,.0028,45.05,.0003,45.99,.0002,47.07,.0003,48.52,.0002,49.48,.0003,50.63,.0003 ,51.81,.0002,54.05,.0002,55.24,.0001,56.62,.0001,57.81,.0004,59.16,.0013,60.23,.0003,66.44,.0001 ,68.99,.0004,75.49,.0001,87.56,.0004],             # <<<<<<<<<<<<<<
 * 			        [0.98,.0629,1.99,.0232,2.98,.0217,4.00,.0396,4.98,.0171,5.97,.0098,6.99,.0167 ,7.99,.0003,8.98,.0192,9.98,.0266,10.99,.0256,12.01,.0061,13.02,.0135,14.02,.0062,15.05,.0158 ,16.06,.0018,17.08,.0101,18.09,.0053,19.11,.0074,20.13,.0020,21.17,.0052,22.22,.0077,23.24,.0035 ,24.00,.0009,25.32,.0016,26.40,.0022,27.43,.0005,28.55,.0026,29.60,.0026,30.65,.0010,31.67,.0019 ,32.77,.0008,33.81,.0003,34.91,.0003,36.01,.0005,37.11,.0010,38.20,.0014,39.29,.0039,40.43,.0012 ,41.50,.0006,43.38,.0017,43.75,.0002,44.94,.0005,46.13,.0002,47.11,.0003,48.28,.0005,48.42,.0005 ,49.44,.0003,50.76,.0004,51.93,.0002,54.15,.0003,55.31,.0005,55.50,.0003,56.98,.0003,57.90,.0004 ,60.33,.0002,61.39,.0001,61.59,.0001,65.09,.0002,66.34,.0001,68.85,.0001,70.42,.0002,71.72,.0001 ,73.05,.0003,79.65,.0001,85.28,.0002,93.52,.0001],
 * 			        [1.02,.0185,1.99,.0525,2.98,.0613,3.99,.0415,4.98,.0109,5.97,.0248,6.99,.0102 ,7.98,.0005,8.98,.0124,9.99,.0103,10.99,.0124,12.00,.0016,13.01,.0029,14.03,.0211,15.04,.0128 ,16.07,.0021,17.09,.0009,18.09,.0043,19.14,.0022,20.13,.0016,21.20,.0045,22.21,.0088,23.26,.0046 ,24.29,.0013,25.35,.0009,26.39,.0028,27.49,.0009,28.51,.0006,29.58,.0012,30.70,.0010,31.74,.0019 ,32.75,.0002,33.85,.0001,34.95,.0005,36.02,.0003,37.16,.0009,38.25,.0018,39.35,.0008,40.54,.0004 ,41.61,.0002,43.40,.0004,43.74,.0003,45.05,.0001,46.11,.0003,47.40,.0002,48.36,.0004,49.55,.0004 ,50.72,.0002,52.00,.0001,55.58,.0002,57.02,.0001,57.98,.0002,59.13,.0003,61.56,.0001,66.56,.0001 ,87.65,.0002],
 */
  __pyx_t_13 = PyList_New(116); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 0, __pyx_float_1_00)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0765);
  __Pyx_GIVEREF(__pyx_float__0765);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 1, __pyx_float__0765)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_1_99);
  __Pyx_GIVEREF(__pyx_float_1_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 2, __pyx_float_1_99)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0151);
  __Pyx_GIVEREF(__pyx_float__0151);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 3, __pyx_float__0151)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_99);
  __Pyx_GIVEREF(__pyx_float_2_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 4, __pyx_float_2_99)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0500);
  __Pyx_GIVEREF(__pyx_float__0500);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 5, __pyx_float__0500)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_99);
  __Pyx_GIVEREF(__pyx_float_3_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 6, __pyx_float_3_99)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0197);
  __Pyx_GIVEREF(__pyx_float__0197);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 7, __pyx_float__0197)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_00);
  __Pyx_GIVEREF(__pyx_float_5_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 8, __pyx_float_5_00)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0260);
  __Pyx_GIVEREF(__pyx_float__0260);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 9, __pyx_float__0260)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_00);
  __Pyx_GIVEREF(__pyx_float_6_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 10, __pyx_float_6_00)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0145);
  __Pyx_GIVEREF(__pyx_float__0145);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 11, __pyx_float__0145)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_98);
  __Pyx_GIVEREF(__pyx_float_6_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 12, __pyx_float_6_98)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0128);
  __Pyx_GIVEREF(__pyx_float__0128);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 13, __pyx_float__0128)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_97);
  __Pyx_GIVEREF(__pyx_float_7_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 14, __pyx_float_7_97)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 15, __pyx_float__0004)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_98);
  __Pyx_GIVEREF(__pyx_float_8_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 16, __pyx_float_8_98)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0158);
  __Pyx_GIVEREF(__pyx_float__0158);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 17, __pyx_float__0158)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_99);
  __Pyx_GIVEREF(__pyx_float_9_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 18, __pyx_float_9_99)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0265);
  __Pyx_GIVEREF(__pyx_float__0265);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 19, __pyx_float__0265)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_02);
  __Pyx_GIVEREF(__pyx_float_11_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 20, __pyx_float_11_02)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0290);
  __Pyx_GIVEREF(__pyx_float__0290);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 21, __pyx_float__0290)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_02);
  __Pyx_GIVEREF(__pyx_float_12_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 22, __pyx_float_12_02)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0053);
  __Pyx_GIVEREF(__pyx_float__0053);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 23, __pyx_float__0053)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_03);
  __Pyx_GIVEREF(__pyx_float_13_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 24, __pyx_float_13_03)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0242);
  __Pyx_GIVEREF(__pyx_float__0242);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 25, __pyx_float__0242)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_03);
  __Pyx_GIVEREF(__pyx_float_14_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 26, __pyx_float_14_03)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0103);
  __Pyx_GIVEREF(__pyx_float__0103);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 27, __pyx_float__0103)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_06);
  __Pyx_GIVEREF(__pyx_float_15_06);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 28, __pyx_float_15_06)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0054);
  __Pyx_GIVEREF(__pyx_float__0054);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 29, __pyx_float__0054)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_04);
  __Pyx_GIVEREF(__pyx_float_16_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 30, __pyx_float_16_04)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 31, __pyx_float__0006)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_08);
  __Pyx_GIVEREF(__pyx_float_17_08);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 32, __pyx_float_17_08)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 33, __pyx_float__0008)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_10);
  __Pyx_GIVEREF(__pyx_float_18_10);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 34, __pyx_float_18_10)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0058);
  __Pyx_GIVEREF(__pyx_float__0058);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 35, __pyx_float__0058)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_16);
  __Pyx_GIVEREF(__pyx_float_19_16);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 36, __pyx_float_19_16)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 37, __pyx_float__0011)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_16);
  __Pyx_GIVEREF(__pyx_float_20_16);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 38, __pyx_float_20_16)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0055);
  __Pyx_GIVEREF(__pyx_float__0055);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 39, __pyx_float__0055)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_18);
  __Pyx_GIVEREF(__pyx_float_21_18);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 40, __pyx_float_21_18)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0040);
  __Pyx_GIVEREF(__pyx_float__0040);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 41, __pyx_float__0040)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_20);
  __Pyx_GIVEREF(__pyx_float_22_20);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 42, __pyx_float_22_20)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0019);
  __Pyx_GIVEREF(__pyx_float__0019);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 43, __pyx_float__0019)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_22);
  __Pyx_GIVEREF(__pyx_float_23_22);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 44, __pyx_float_23_22)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0014);
  __Pyx_GIVEREF(__pyx_float__0014);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 45, __pyx_float__0014)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_24_05);
  __Pyx_GIVEREF(__pyx_float_24_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 46, __pyx_float_24_05)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 47, __pyx_float__0005)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_25_31);
  __Pyx_GIVEREF(__pyx_float_25_31);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 48, __pyx_float_25_31)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0019);
  __Pyx_GIVEREF(__pyx_float__0019);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 49, __pyx_float__0019)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_26_38);
  __Pyx_GIVEREF(__pyx_float_26_38);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 50, __pyx_float_26_38)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0018);
  __Pyx_GIVEREF(__pyx_float__0018);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 51, __pyx_float__0018)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_27_44);
  __Pyx_GIVEREF(__pyx_float_27_44);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 52, __pyx_float_27_44)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0022);
  __Pyx_GIVEREF(__pyx_float__0022);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 53, __pyx_float__0022)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_45);
  __Pyx_GIVEREF(__pyx_float_28_45);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 54, __pyx_float_28_45)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0024);
  __Pyx_GIVEREF(__pyx_float__0024);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 55, __pyx_float__0024)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_29_57);
  __Pyx_GIVEREF(__pyx_float_29_57);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 56, __pyx_float_29_57)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0073);
  __Pyx_GIVEREF(__pyx_float__0073);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 57, __pyx_float__0073)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_30_58);
  __Pyx_GIVEREF(__pyx_float_30_58);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 58, __pyx_float_30_58)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0032);
  __Pyx_GIVEREF(__pyx_float__0032);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 59, __pyx_float__0032)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_31_66);
  __Pyx_GIVEREF(__pyx_float_31_66);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 60, __pyx_float_31_66)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0071);
  __Pyx_GIVEREF(__pyx_float__0071);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 61, __pyx_float__0071)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_32_73);
  __Pyx_GIVEREF(__pyx_float_32_73);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 62, __pyx_float_32_73)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0015);
  __Pyx_GIVEREF(__pyx_float__0015);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 63, __pyx_float__0015)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_33_85);
  __Pyx_GIVEREF(__pyx_float_33_85);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 64, __pyx_float_33_85)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 65, __pyx_float__0005)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_34_96);
  __Pyx_GIVEREF(__pyx_float_34_96);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 66, __pyx_float_34_96)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 67, __pyx_float__0003)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_36_00);
  __Pyx_GIVEREF(__pyx_float_36_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 68, __pyx_float_36_00)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0020);
  __Pyx_GIVEREF(__pyx_float__0020);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 69, __pyx_float__0020)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_37_11);
  __Pyx_GIVEREF(__pyx_float_37_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 70, __pyx_float_37_11)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0018);
  __Pyx_GIVEREF(__pyx_float__0018);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 71, __pyx_float__0018)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_38_18);
  __Pyx_GIVEREF(__pyx_float_38_18);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 72, __pyx_float_38_18)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0055);
  __Pyx_GIVEREF(__pyx_float__0055);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 73, __pyx_float__0055)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_39_23);
  __Pyx_GIVEREF(__pyx_float_39_23);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 74, __pyx_float_39_23)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 75, __pyx_float__0006)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_40_33);
  __Pyx_GIVEREF(__pyx_float_40_33);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 76, __pyx_float_40_33)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 77, __pyx_float__0004)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_41_52);
  __Pyx_GIVEREF(__pyx_float_41_52);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 78, __pyx_float_41_52)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 79, __pyx_float__0003)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_43_41);
  __Pyx_GIVEREF(__pyx_float_43_41);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 80, __pyx_float_43_41)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0028);
  __Pyx_GIVEREF(__pyx_float__0028);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 81, __pyx_float__0028)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_45_05);
  __Pyx_GIVEREF(__pyx_float_45_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 82, __pyx_float_45_05)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 83, __pyx_float__0003)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_45_99);
  __Pyx_GIVEREF(__pyx_float_45_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 84, __pyx_float_45_99)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 85, __pyx_float__0002)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_47_07);
  __Pyx_GIVEREF(__pyx_float_47_07);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 86, __pyx_float_47_07)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 87, __pyx_float__0003)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_48_52);
  __Pyx_GIVEREF(__pyx_float_48_52);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 88, __pyx_float_48_52)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 89, __pyx_float__0002)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_49_48);
  __Pyx_GIVEREF(__pyx_float_49_48);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 90, __pyx_float_49_48)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 91, __pyx_float__0003)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_50_63);
  __Pyx_GIVEREF(__pyx_float_50_63);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 92, __pyx_float_50_63)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 93, __pyx_float__0003)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_51_81);
  __Pyx_GIVEREF(__pyx_float_51_81);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 94, __pyx_float_51_81)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 95, __pyx_float__0002)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_54_05);
  __Pyx_GIVEREF(__pyx_float_54_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 96, __pyx_float_54_05)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 97, __pyx_float__0002)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_55_24);
  __Pyx_GIVEREF(__pyx_float_55_24);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 98, __pyx_float_55_24)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 99, __pyx_float__0001)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_56_62);
  __Pyx_GIVEREF(__pyx_float_56_62);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 100, __pyx_float_56_62)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 101, __pyx_float__0001)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_57_81);
  __Pyx_GIVEREF(__pyx_float_57_81);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 102, __pyx_float_57_81)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 103, __pyx_float__0004)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_59_16);
  __Pyx_GIVEREF(__pyx_float_59_16);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 104, __pyx_float_59_16)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0013);
  __Pyx_GIVEREF(__pyx_float__0013);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 105, __pyx_float__0013)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_60_23);
  __Pyx_GIVEREF(__pyx_float_60_23);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 106, __pyx_float_60_23)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 107, __pyx_float__0003)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_66_44);
  __Pyx_GIVEREF(__pyx_float_66_44);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 108, __pyx_float_66_44)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 109, __pyx_float__0001)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_68_99);
  __Pyx_GIVEREF(__pyx_float_68_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 110, __pyx_float_68_99)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 111, __pyx_float__0004)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_75_49);
  __Pyx_GIVEREF(__pyx_float_75_49);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 112, __pyx_float_75_49)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 113, __pyx_float__0001)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_87_56);
  __Pyx_GIVEREF(__pyx_float_87_56);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 114, __pyx_float_87_56)) __PYX_ERR(0, 39, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_13, 115, __pyx_float__0004)) __PYX_ERR(0, 39, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":40
 * 			        [0.98,.0099,2.00,.0181,2.99,.0353,3.98,.0285,4.97,.0514,5.96,.0402,6.96,.0015 ,7.98,.0012,8.98,.0175,9.98,.0264,10.98,.0392,11.98,.0236,13.00,.0153,14.04,.0049,15.00,.0089 ,16.01,.0001,17.03,.0106,18.03,.0028,19.05,.0024,20.08,.0040,21.11,.0103,22.12,.0104,23.20,.0017 ,24.19,.0008,25.20,.0007,26.24,.0011,27.36,.0009,27.97,.0030,29.40,.0044,30.37,.0019,31.59,.0017 ,32.65,.0008,33.59,.0005,34.79,.0009,35.75,.0027,36.88,.0035,37.93,.0039,39.00,.0031,40.08,.0025 ,41.16,.0010,43.25,.0004,44.52,.0012,45.62,.0023,45.85,.0012,47.00,.0006,47.87,.0008,48.99,.0003 ,50.48,.0003,51.62,.0001,52.43,.0001,53.56,.0002,54.76,.0002,56.04,.0002,56.68,.0006,57.10,.0003 ,58.28,.0005,59.47,.0003,59.96,.0002,60.67,.0001,63.08,.0002,64.29,.0002,66.72,.0001,67.97,.0001 ,68.65,.0001,70.43,.0001,79.38,.0001,80.39,.0001,82.39,.0001],
 * 			        [1.00,.0765,1.99,.0151,2.99,.0500,3.99,.0197,5.00,.0260,6.00,.0145,6.98,.0128 ,7.97,.0004,8.98,.0158,9.99,.0265,11.02,.0290,12.02,.0053,13.03,.0242,14.03,.0103,15.06,.0054 ,16.04,.0006,17.08,.0008,18.10,.0058,19.16,.0011,20.16,.0055,21.18,.0040,22.20,.0019,23.22,.0014 ,24.05,.0005,25.31,.0019,26.38,.0018,27.44,.0022,28.45,.0024,29.57,.0073,30.58,.0032,31.66,.0071 ,32.73,.0015,33.85,.0005,34.96,.0003,36.00,.0020,37.11,.0018,38.18,.0055,39.23,.0006,40.33,.0004 ,41.52,.0003,43.41,.0028,45.05,.0003,45.99,.0002,47.07,.0003,48.52,.0002,49.48,.0003,50.63,.0003 ,51.81,.0002,54.05,.0002,55.24,.0001,56.62,.0001,57.81,.0004,59.16,.0013,60.23,.0003,66.44,.0001 ,68.99,.0004,75.49,.0001,87.56,.0004],
 * 			        [0.98,.0629,1.99,.0232,2.98,.0217,4.00,.0396,4.98,.0171,5.97,.0098,6.99,.0167 ,7.99,.0003,8.98,.0192,9.98,.0266,10.99,.0256,12.01,.0061,13.02,.0135,14.02,.0062,15.05,.0158 ,16.06,.0018,17.08,.0101,18.09,.0053,19.11,.0074,20.13,.0020,21.17,.0052,22.22,.0077,23.24,.0035 ,24.00,.0009,25.32,.0016,26.40,.0022,27.43,.0005,28.55,.0026,29.60,.0026,30.65,.0010,31.67,.0019 ,32.77,.0008,33.81,.0003,34.91,.0003,36.01,.0005,37.11,.0010,38.20,.0014,39.29,.0039,40.43,.0012 ,41.50,.0006,43.38,.0017,43.75,.0002,44.94,.0005,46.13,.0002,47.11,.0003,48.28,.0005,48.42,.0005 ,49.44,.0003,50.76,.0004,51.93,.0002,54.15,.0003,55.31,.0005,55.50,.0003,56.98,.0003,57.90,.0004 ,60.33,.0002,61.39,.0001,61.59,.0001,65.09,.0002,66.34,.0001,68.85,.0001,70.42,.0002,71.72,.0001 ,73.05,.0003,79.65,.0001,85.28,.0002,93.52,.0001],             # <<<<<<<<<<<<<<
 * 			        [1.02,.0185,1.99,.0525,2.98,.0613,3.99,.0415,4.98,.0109,5.97,.0248,6.99,.0102 ,7.98,.0005,8.98,.0124,9.99,.0103,10.99,.0124,12.00,.0016,13.01,.0029,14.03,.0211,15.04,.0128 ,16.07,.0021,17.09,.0009,18.09,.0043,19.14,.0022,20.13,.0016,21.20,.0045,22.21,.0088,23.26,.0046 ,24.29,.0013,25.35,.0009,26.39,.0028,27.49,.0009,28.51,.0006,29.58,.0012,30.70,.0010,31.74,.0019 ,32.75,.0002,33.85,.0001,34.95,.0005,36.02,.0003,37.16,.0009,38.25,.0018,39.35,.0008,40.54,.0004 ,41.61,.0002,43.40,.0004,43.74,.0003,45.05,.0001,46.11,.0003,47.40,.0002,48.36,.0004,49.55,.0004 ,50.72,.0002,52.00,.0001,55.58,.0002,57.02,.0001,57.98,.0002,59.13,.0003,61.56,.0001,66.56,.0001 ,87.65,.0002],
 * 			        [1.00,.0473,1.99,.0506,2.99,.0982,3.99,.0654,5.00,.0196,5.99,.0094,6.99,.0118 ,7.93,.0001,8.99,.0057,10.01,.0285,11.01,.0142,12.03,.0032,13.03,.0056,14.06,.0064,15.06,.0059 ,16.11,.0005,17.09,.0033,18.14,.0027,19.15,.0014,20.17,.0010,21.21,.0059,22.26,.0043,23.31,.0031 ,24.31,.0018,25.33,.0009,26.41,.0005,27.47,.0015,28.53,.0015,29.58,.0041,30.65,.0025,31.73,.0011 ,32.83,.0010,34.98,.0003,36.07,.0009,37.23,.0001,38.26,.0020,39.41,.0014,40.53,.0005,41.40,.0003 ,42.80,.0002,43.48,.0028,43.93,.0001,45.03,.0003,46.18,.0007,47.41,.0001,48.57,.0002,49.67,.0001 ,50.83,.0002,54.39,.0001,55.58,.0002,57.97,.0005,58.11,.0002,59.21,.0001,60.42,.0002,61.66,.0001],
 */
  __pyx_t_14 = PyList_New(134); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_INCREF(__pyx_float_0_98);
  __Pyx_GIVEREF(__pyx_float_0_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 0, __pyx_float_0_98)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0629);
  __Pyx_GIVEREF(__pyx_float__0629);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 1, __pyx_float__0629)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_1_99);
  __Pyx_GIVEREF(__pyx_float_1_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 2, __pyx_float_1_99)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0232);
  __Pyx_GIVEREF(__pyx_float__0232);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 3, __pyx_float__0232)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_98);
  __Pyx_GIVEREF(__pyx_float_2_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 4, __pyx_float_2_98)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0217);
  __Pyx_GIVEREF(__pyx_float__0217);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 5, __pyx_float__0217)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_00);
  __Pyx_GIVEREF(__pyx_float_4_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 6, __pyx_float_4_00)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0396);
  __Pyx_GIVEREF(__pyx_float__0396);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 7, __pyx_float__0396)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_98);
  __Pyx_GIVEREF(__pyx_float_4_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 8, __pyx_float_4_98)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0171);
  __Pyx_GIVEREF(__pyx_float__0171);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 9, __pyx_float__0171)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_97);
  __Pyx_GIVEREF(__pyx_float_5_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 10, __pyx_float_5_97)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0098);
  __Pyx_GIVEREF(__pyx_float__0098);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 11, __pyx_float__0098)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_99);
  __Pyx_GIVEREF(__pyx_float_6_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 12, __pyx_float_6_99)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0167);
  __Pyx_GIVEREF(__pyx_float__0167);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 13, __pyx_float__0167)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_99);
  __Pyx_GIVEREF(__pyx_float_7_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 14, __pyx_float_7_99)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 15, __pyx_float__0003)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_98);
  __Pyx_GIVEREF(__pyx_float_8_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 16, __pyx_float_8_98)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0192);
  __Pyx_GIVEREF(__pyx_float__0192);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 17, __pyx_float__0192)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_98);
  __Pyx_GIVEREF(__pyx_float_9_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 18, __pyx_float_9_98)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0266);
  __Pyx_GIVEREF(__pyx_float__0266);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 19, __pyx_float__0266)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_99);
  __Pyx_GIVEREF(__pyx_float_10_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 20, __pyx_float_10_99)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0256);
  __Pyx_GIVEREF(__pyx_float__0256);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 21, __pyx_float__0256)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_01);
  __Pyx_GIVEREF(__pyx_float_12_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 22, __pyx_float_12_01)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0061);
  __Pyx_GIVEREF(__pyx_float__0061);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 23, __pyx_float__0061)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_02);
  __Pyx_GIVEREF(__pyx_float_13_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 24, __pyx_float_13_02)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0135);
  __Pyx_GIVEREF(__pyx_float__0135);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 25, __pyx_float__0135)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_02);
  __Pyx_GIVEREF(__pyx_float_14_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 26, __pyx_float_14_02)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0062);
  __Pyx_GIVEREF(__pyx_float__0062);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 27, __pyx_float__0062)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_05);
  __Pyx_GIVEREF(__pyx_float_15_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 28, __pyx_float_15_05)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0158);
  __Pyx_GIVEREF(__pyx_float__0158);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 29, __pyx_float__0158)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_06);
  __Pyx_GIVEREF(__pyx_float_16_06);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 30, __pyx_float_16_06)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0018);
  __Pyx_GIVEREF(__pyx_float__0018);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 31, __pyx_float__0018)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_08);
  __Pyx_GIVEREF(__pyx_float_17_08);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 32, __pyx_float_17_08)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0101);
  __Pyx_GIVEREF(__pyx_float__0101);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 33, __pyx_float__0101)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_09);
  __Pyx_GIVEREF(__pyx_float_18_09);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 34, __pyx_float_18_09)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0053);
  __Pyx_GIVEREF(__pyx_float__0053);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 35, __pyx_float__0053)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_11);
  __Pyx_GIVEREF(__pyx_float_19_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 36, __pyx_float_19_11)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0074);
  __Pyx_GIVEREF(__pyx_float__0074);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 37, __pyx_float__0074)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_13);
  __Pyx_GIVEREF(__pyx_float_20_13);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 38, __pyx_float_20_13)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0020);
  __Pyx_GIVEREF(__pyx_float__0020);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 39, __pyx_float__0020)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_17);
  __Pyx_GIVEREF(__pyx_float_21_17);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 40, __pyx_float_21_17)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0052);
  __Pyx_GIVEREF(__pyx_float__0052);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 41, __pyx_float__0052)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_22);
  __Pyx_GIVEREF(__pyx_float_22_22);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 42, __pyx_float_22_22)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0077);
  __Pyx_GIVEREF(__pyx_float__0077);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 43, __pyx_float__0077)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_24);
  __Pyx_GIVEREF(__pyx_float_23_24);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 44, __pyx_float_23_24)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0035);
  __Pyx_GIVEREF(__pyx_float__0035);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 45, __pyx_float__0035)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_24_00);
  __Pyx_GIVEREF(__pyx_float_24_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 46, __pyx_float_24_00)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 47, __pyx_float__0009)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_25_32);
  __Pyx_GIVEREF(__pyx_float_25_32);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 48, __pyx_float_25_32)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0016);
  __Pyx_GIVEREF(__pyx_float__0016);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 49, __pyx_float__0016)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_26_40);
  __Pyx_GIVEREF(__pyx_float_26_40);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 50, __pyx_float_26_40)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0022);
  __Pyx_GIVEREF(__pyx_float__0022);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 51, __pyx_float__0022)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_27_43);
  __Pyx_GIVEREF(__pyx_float_27_43);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 52, __pyx_float_27_43)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 53, __pyx_float__0005)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_55);
  __Pyx_GIVEREF(__pyx_float_28_55);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 54, __pyx_float_28_55)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0026);
  __Pyx_GIVEREF(__pyx_float__0026);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 55, __pyx_float__0026)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_29_60);
  __Pyx_GIVEREF(__pyx_float_29_60);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 56, __pyx_float_29_60)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0026);
  __Pyx_GIVEREF(__pyx_float__0026);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 57, __pyx_float__0026)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_30_65);
  __Pyx_GIVEREF(__pyx_float_30_65);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 58, __pyx_float_30_65)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 59, __pyx_float__0010)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_31_67);
  __Pyx_GIVEREF(__pyx_float_31_67);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 60, __pyx_float_31_67)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0019);
  __Pyx_GIVEREF(__pyx_float__0019);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 61, __pyx_float__0019)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_32_77);
  __Pyx_GIVEREF(__pyx_float_32_77);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 62, __pyx_float_32_77)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 63, __pyx_float__0008)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_33_81);
  __Pyx_GIVEREF(__pyx_float_33_81);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 64, __pyx_float_33_81)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 65, __pyx_float__0003)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_34_91);
  __Pyx_GIVEREF(__pyx_float_34_91);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 66, __pyx_float_34_91)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 67, __pyx_float__0003)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_36_01);
  __Pyx_GIVEREF(__pyx_float_36_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 68, __pyx_float_36_01)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 69, __pyx_float__0005)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_37_11);
  __Pyx_GIVEREF(__pyx_float_37_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 70, __pyx_float_37_11)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 71, __pyx_float__0010)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_38_20);
  __Pyx_GIVEREF(__pyx_float_38_20);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 72, __pyx_float_38_20)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0014);
  __Pyx_GIVEREF(__pyx_float__0014);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 73, __pyx_float__0014)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_39_29);
  __Pyx_GIVEREF(__pyx_float_39_29);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 74, __pyx_float_39_29)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0039);
  __Pyx_GIVEREF(__pyx_float__0039);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 75, __pyx_float__0039)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_40_43);
  __Pyx_GIVEREF(__pyx_float_40_43);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 76, __pyx_float_40_43)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 77, __pyx_float__0012)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_41_50);
  __Pyx_GIVEREF(__pyx_float_41_50);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 78, __pyx_float_41_50)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 79, __pyx_float__0006)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_43_38);
  __Pyx_GIVEREF(__pyx_float_43_38);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 80, __pyx_float_43_38)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0017);
  __Pyx_GIVEREF(__pyx_float__0017);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 81, __pyx_float__0017)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_43_75);
  __Pyx_GIVEREF(__pyx_float_43_75);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 82, __pyx_float_43_75)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 83, __pyx_float__0002)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_44_94);
  __Pyx_GIVEREF(__pyx_float_44_94);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 84, __pyx_float_44_94)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 85, __pyx_float__0005)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_46_13);
  __Pyx_GIVEREF(__pyx_float_46_13);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 86, __pyx_float_46_13)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 87, __pyx_float__0002)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_47_11);
  __Pyx_GIVEREF(__pyx_float_47_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 88, __pyx_float_47_11)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 89, __pyx_float__0003)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_48_28);
  __Pyx_GIVEREF(__pyx_float_48_28);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 90, __pyx_float_48_28)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 91, __pyx_float__0005)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_48_42);
  __Pyx_GIVEREF(__pyx_float_48_42);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 92, __pyx_float_48_42)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 93, __pyx_float__0005)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_49_44);
  __Pyx_GIVEREF(__pyx_float_49_44);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 94, __pyx_float_49_44)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 95, __pyx_float__0003)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_50_76);
  __Pyx_GIVEREF(__pyx_float_50_76);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 96, __pyx_float_50_76)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 97, __pyx_float__0004)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_51_93);
  __Pyx_GIVEREF(__pyx_float_51_93);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 98, __pyx_float_51_93)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 99, __pyx_float__0002)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_54_15);
  __Pyx_GIVEREF(__pyx_float_54_15);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 100, __pyx_float_54_15)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 101, __pyx_float__0003)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_55_31);
  __Pyx_GIVEREF(__pyx_float_55_31);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 102, __pyx_float_55_31)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 103, __pyx_float__0005)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_55_50);
  __Pyx_GIVEREF(__pyx_float_55_50);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 104, __pyx_float_55_50)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 105, __pyx_float__0003)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_56_98);
  __Pyx_GIVEREF(__pyx_float_56_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 106, __pyx_float_56_98)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 107, __pyx_float__0003)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_57_90);
  __Pyx_GIVEREF(__pyx_float_57_90);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 108, __pyx_float_57_90)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 109, __pyx_float__0004)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_60_33);
  __Pyx_GIVEREF(__pyx_float_60_33);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 110, __pyx_float_60_33)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 111, __pyx_float__0002)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_61_39);
  __Pyx_GIVEREF(__pyx_float_61_39);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 112, __pyx_float_61_39)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 113, __pyx_float__0001)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_61_59);
  __Pyx_GIVEREF(__pyx_float_61_59);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 114, __pyx_float_61_59)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 115, __pyx_float__0001)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_65_09);
  __Pyx_GIVEREF(__pyx_float_65_09);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 116, __pyx_float_65_09)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 117, __pyx_float__0002)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_66_34);
  __Pyx_GIVEREF(__pyx_float_66_34);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 118, __pyx_float_66_34)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 119, __pyx_float__0001)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_68_85);
  __Pyx_GIVEREF(__pyx_float_68_85);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 120, __pyx_float_68_85)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 121, __pyx_float__0001)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_70_42);
  __Pyx_GIVEREF(__pyx_float_70_42);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 122, __pyx_float_70_42)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 123, __pyx_float__0002)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_71_72);
  __Pyx_GIVEREF(__pyx_float_71_72);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 124, __pyx_float_71_72)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 125, __pyx_float__0001)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_73_05);
  __Pyx_GIVEREF(__pyx_float_73_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 126, __pyx_float_73_05)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 127, __pyx_float__0003)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_79_65);
  __Pyx_GIVEREF(__pyx_float_79_65);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 128, __pyx_float_79_65)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 129, __pyx_float__0001)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_85_28);
  __Pyx_GIVEREF(__pyx_float_85_28);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 130, __pyx_float_85_28)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 131, __pyx_float__0002)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_93_52);
  __Pyx_GIVEREF(__pyx_float_93_52);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 132, __pyx_float_93_52)) __PYX_ERR(0, 40, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_14, 133, __pyx_float__0001)) __PYX_ERR(0, 40, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":41
 * 			        [1.00,.0765,1.99,.0151,2.99,.0500,3.99,.0197,5.00,.0260,6.00,.0145,6.98,.0128 ,7.97,.0004,8.98,.0158,9.99,.0265,11.02,.0290,12.02,.0053,13.03,.0242,14.03,.0103,15.06,.0054 ,16.04,.0006,17.08,.0008,18.10,.0058,19.16,.0011,20.16,.0055,21.18,.0040,22.20,.0019,23.22,.0014 ,24.05,.0005,25.31,.0019,26.38,.0018,27.44,.0022,28.45,.0024,29.57,.0073,30.58,.0032,31.66,.0071 ,32.73,.0015,33.85,.0005,34.96,.0003,36.00,.0020,37.11,.0018,38.18,.0055,39.23,.0006,40.33,.0004 ,41.52,.0003,43.41,.0028,45.05,.0003,45.99,.0002,47.07,.0003,48.52,.0002,49.48,.0003,50.63,.0003 ,51.81,.0002,54.05,.0002,55.24,.0001,56.62,.0001,57.81,.0004,59.16,.0013,60.23,.0003,66.44,.0001 ,68.99,.0004,75.49,.0001,87.56,.0004],
 * 			        [0.98,.0629,1.99,.0232,2.98,.0217,4.00,.0396,4.98,.0171,5.97,.0098,6.99,.0167 ,7.99,.0003,8.98,.0192,9.98,.0266,10.99,.0256,12.01,.0061,13.02,.0135,14.02,.0062,15.05,.0158 ,16.06,.0018,17.08,.0101,18.09,.0053,19.11,.0074,20.13,.0020,21.17,.0052,22.22,.0077,23.24,.0035 ,24.00,.0009,25.32,.0016,26.40,.0022,27.43,.0005,28.55,.0026,29.60,.0026,30.65,.0010,31.67,.0019 ,32.77,.0008,33.81,.0003,34.91,.0003,36.01,.0005,37.11,.0010,38.20,.0014,39.29,.0039,40.43,.0012 ,41.50,.0006,43.38,.0017,43.75,.0002,44.94,.0005,46.13,.0002,47.11,.0003,48.28,.0005,48.42,.0005 ,49.44,.0003,50.76,.0004,51.93,.0002,54.15,.0003,55.31,.0005,55.50,.0003,56.98,.0003,57.90,.0004 ,60.33,.0002,61.39,.0001,61.59,.0001,65.09,.0002,66.34,.0001,68.85,.0001,70.42,.0002,71.72,.0001 ,73.05,.0003,79.65,.0001,85.28,.0002,93.52,.0001],
 * 			        [1.02,.0185,1.99,.0525,2.98,.0613,3.99,.0415,4.98,.0109,5.97,.0248,6.99,.0102 ,7.98,.0005,8.98,.0124,9.99,.0103,10.99,.0124,12.00,.0016,13.01,.0029,14.03,.0211,15.04,.0128 ,16.07,.0021,17.09,.0009,18.09,.0043,19.14,.0022,20.13,.0016,21.20,.0045,22.21,.0088,23.26,.0046 ,24.29,.0013,25.35,.0009,26.39,.0028,27.49,.0009,28.51,.0006,29.58,.0012,30.70,.0010,31.74,.0019 ,32.75,.0002,33.85,.0001,34.95,.0005,36.02,.0003,37.16,.0009,38.25,.0018,39.35,.0008,40.54,.0004 ,41.61,.0002,43.40,.0004,43.74,.0003,45.05,.0001,46.11,.0003,47.40,.0002,48.36,.0004,49.55,.0004 ,50.72,.0002,52.00,.0001,55.58,.0002,57.02,.0001,57.98,.0002,59.13,.0003,61.56,.0001,66.56,.0001 ,87.65,.0002],             # <<<<<<<<<<<<<<
 * 			        [1.00,.0473,1.99,.0506,2.99,.0982,3.99,.0654,5.00,.0196,5.99,.0094,6.99,.0118 ,7.93,.0001,8.99,.0057,10.01,.0285,11.01,.0142,12.03,.0032,13.03,.0056,14.06,.0064,15.06,.0059 ,16.11,.0005,17.09,.0033,18.14,.0027,19.15,.0014,20.17,.0010,21.21,.0059,22.26,.0043,23.31,.0031 ,24.31,.0018,25.33,.0009,26.41,.0005,27.47,.0015,28.53,.0015,29.58,.0041,30.65,.0025,31.73,.0011 ,32.83,.0010,34.98,.0003,36.07,.0009,37.23,.0001,38.26,.0020,39.41,.0014,40.53,.0005,41.40,.0003 ,42.80,.0002,43.48,.0028,43.93,.0001,45.03,.0003,46.18,.0007,47.41,.0001,48.57,.0002,49.67,.0001 ,50.83,.0002,54.39,.0001,55.58,.0002,57.97,.0005,58.11,.0002,59.21,.0001,60.42,.0002,61.66,.0001],
 * 			        [1.00,.0503,2.00,.0963,2.99,.1304,3.99,.0218,4.98,.0041,5.98,.0292,6.98,.0482 ,7.99,.0005,8.99,.0280,10.00,.0237,11.00,.0152,12.02,.0036,12.95,.0022,14.06,.0111,15.07,.0196 ,16.08,.0016,17.11,.0044,18.13,.0073,19.17,.0055,20.19,.0028,21.20,.0012,22.27,.0068,23.30,.0036 ,24.35,.0012,25.35,.0002,26.46,.0005,27.47,.0005,28.59,.0009,29.65,.0021,30.70,.0020,31.78,.0012 ,32.89,.0010,35.06,.0005,36.16,.0008,37.27,.0010,38.36,.0010,39.47,.0014,40.58,.0004,41.43,.0007 ,41.82,.0003,43.48,.0008,44.53,.0001,45.25,.0003,46.43,.0002,47.46,.0002,48.76,.0005,49.95,.0004 ,50.96,.0002,51.12,.0002,52.33,.0001,54.75,.0001,55.75,.0002,56.90,.0002,58.17,.0002,59.40,.0004 ,60.62,.0002,65.65,.0001,66.91,.0002,69.91,.0001,71.25,.0002],
 */
  __pyx_t_15 = PyList_New(112); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  __Pyx_INCREF(__pyx_float_1_02);
  __Pyx_GIVEREF(__pyx_float_1_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 0, __pyx_float_1_02)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0185);
  __Pyx_GIVEREF(__pyx_float__0185);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 1, __pyx_float__0185)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_1_99);
  __Pyx_GIVEREF(__pyx_float_1_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 2, __pyx_float_1_99)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0525);
  __Pyx_GIVEREF(__pyx_float__0525);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 3, __pyx_float__0525)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_98);
  __Pyx_GIVEREF(__pyx_float_2_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 4, __pyx_float_2_98)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0613);
  __Pyx_GIVEREF(__pyx_float__0613);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 5, __pyx_float__0613)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_99);
  __Pyx_GIVEREF(__pyx_float_3_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 6, __pyx_float_3_99)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0415);
  __Pyx_GIVEREF(__pyx_float__0415);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 7, __pyx_float__0415)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_98);
  __Pyx_GIVEREF(__pyx_float_4_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 8, __pyx_float_4_98)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0109);
  __Pyx_GIVEREF(__pyx_float__0109);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 9, __pyx_float__0109)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_97);
  __Pyx_GIVEREF(__pyx_float_5_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 10, __pyx_float_5_97)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0248);
  __Pyx_GIVEREF(__pyx_float__0248);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 11, __pyx_float__0248)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_99);
  __Pyx_GIVEREF(__pyx_float_6_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 12, __pyx_float_6_99)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0102);
  __Pyx_GIVEREF(__pyx_float__0102);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 13, __pyx_float__0102)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_98);
  __Pyx_GIVEREF(__pyx_float_7_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 14, __pyx_float_7_98)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 15, __pyx_float__0005)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_98);
  __Pyx_GIVEREF(__pyx_float_8_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 16, __pyx_float_8_98)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0124);
  __Pyx_GIVEREF(__pyx_float__0124);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 17, __pyx_float__0124)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_99);
  __Pyx_GIVEREF(__pyx_float_9_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 18, __pyx_float_9_99)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0103);
  __Pyx_GIVEREF(__pyx_float__0103);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 19, __pyx_float__0103)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_99);
  __Pyx_GIVEREF(__pyx_float_10_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 20, __pyx_float_10_99)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0124);
  __Pyx_GIVEREF(__pyx_float__0124);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 21, __pyx_float__0124)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_00);
  __Pyx_GIVEREF(__pyx_float_12_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 22, __pyx_float_12_00)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0016);
  __Pyx_GIVEREF(__pyx_float__0016);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 23, __pyx_float__0016)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_01);
  __Pyx_GIVEREF(__pyx_float_13_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 24, __pyx_float_13_01)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0029);
  __Pyx_GIVEREF(__pyx_float__0029);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 25, __pyx_float__0029)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_03);
  __Pyx_GIVEREF(__pyx_float_14_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 26, __pyx_float_14_03)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0211);
  __Pyx_GIVEREF(__pyx_float__0211);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 27, __pyx_float__0211)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_04);
  __Pyx_GIVEREF(__pyx_float_15_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 28, __pyx_float_15_04)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0128);
  __Pyx_GIVEREF(__pyx_float__0128);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 29, __pyx_float__0128)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_07);
  __Pyx_GIVEREF(__pyx_float_16_07);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 30, __pyx_float_16_07)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0021);
  __Pyx_GIVEREF(__pyx_float__0021);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 31, __pyx_float__0021)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_09);
  __Pyx_GIVEREF(__pyx_float_17_09);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 32, __pyx_float_17_09)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 33, __pyx_float__0009)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_09);
  __Pyx_GIVEREF(__pyx_float_18_09);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 34, __pyx_float_18_09)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0043);
  __Pyx_GIVEREF(__pyx_float__0043);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 35, __pyx_float__0043)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_14);
  __Pyx_GIVEREF(__pyx_float_19_14);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 36, __pyx_float_19_14)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0022);
  __Pyx_GIVEREF(__pyx_float__0022);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 37, __pyx_float__0022)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_13);
  __Pyx_GIVEREF(__pyx_float_20_13);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 38, __pyx_float_20_13)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0016);
  __Pyx_GIVEREF(__pyx_float__0016);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 39, __pyx_float__0016)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_20);
  __Pyx_GIVEREF(__pyx_float_21_20);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 40, __pyx_float_21_20)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0045);
  __Pyx_GIVEREF(__pyx_float__0045);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 41, __pyx_float__0045)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_21);
  __Pyx_GIVEREF(__pyx_float_22_21);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 42, __pyx_float_22_21)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0088);
  __Pyx_GIVEREF(__pyx_float__0088);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 43, __pyx_float__0088)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_26);
  __Pyx_GIVEREF(__pyx_float_23_26);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 44, __pyx_float_23_26)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0046);
  __Pyx_GIVEREF(__pyx_float__0046);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 45, __pyx_float__0046)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_24_29);
  __Pyx_GIVEREF(__pyx_float_24_29);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 46, __pyx_float_24_29)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0013);
  __Pyx_GIVEREF(__pyx_float__0013);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 47, __pyx_float__0013)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_25_35);
  __Pyx_GIVEREF(__pyx_float_25_35);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 48, __pyx_float_25_35)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 49, __pyx_float__0009)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_26_39);
  __Pyx_GIVEREF(__pyx_float_26_39);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 50, __pyx_float_26_39)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0028);
  __Pyx_GIVEREF(__pyx_float__0028);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 51, __pyx_float__0028)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_27_49);
  __Pyx_GIVEREF(__pyx_float_27_49);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 52, __pyx_float_27_49)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 53, __pyx_float__0009)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_51);
  __Pyx_GIVEREF(__pyx_float_28_51);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 54, __pyx_float_28_51)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 55, __pyx_float__0006)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_29_58);
  __Pyx_GIVEREF(__pyx_float_29_58);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 56, __pyx_float_29_58)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 57, __pyx_float__0012)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_30_70);
  __Pyx_GIVEREF(__pyx_float_30_70);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 58, __pyx_float_30_70)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 59, __pyx_float__0010)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_31_74);
  __Pyx_GIVEREF(__pyx_float_31_74);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 60, __pyx_float_31_74)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0019);
  __Pyx_GIVEREF(__pyx_float__0019);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 61, __pyx_float__0019)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_32_75);
  __Pyx_GIVEREF(__pyx_float_32_75);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 62, __pyx_float_32_75)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 63, __pyx_float__0002)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_33_85);
  __Pyx_GIVEREF(__pyx_float_33_85);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 64, __pyx_float_33_85)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 65, __pyx_float__0001)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_34_95);
  __Pyx_GIVEREF(__pyx_float_34_95);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 66, __pyx_float_34_95)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 67, __pyx_float__0005)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_36_02);
  __Pyx_GIVEREF(__pyx_float_36_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 68, __pyx_float_36_02)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 69, __pyx_float__0003)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_37_16);
  __Pyx_GIVEREF(__pyx_float_37_16);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 70, __pyx_float_37_16)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 71, __pyx_float__0009)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_38_25);
  __Pyx_GIVEREF(__pyx_float_38_25);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 72, __pyx_float_38_25)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0018);
  __Pyx_GIVEREF(__pyx_float__0018);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 73, __pyx_float__0018)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_39_35);
  __Pyx_GIVEREF(__pyx_float_39_35);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 74, __pyx_float_39_35)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 75, __pyx_float__0008)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_40_54);
  __Pyx_GIVEREF(__pyx_float_40_54);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 76, __pyx_float_40_54)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 77, __pyx_float__0004)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_41_61);
  __Pyx_GIVEREF(__pyx_float_41_61);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 78, __pyx_float_41_61)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 79, __pyx_float__0002)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_43_40);
  __Pyx_GIVEREF(__pyx_float_43_40);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 80, __pyx_float_43_40)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 81, __pyx_float__0004)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_43_74);
  __Pyx_GIVEREF(__pyx_float_43_74);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 82, __pyx_float_43_74)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 83, __pyx_float__0003)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_45_05);
  __Pyx_GIVEREF(__pyx_float_45_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 84, __pyx_float_45_05)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 85, __pyx_float__0001)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_46_11);
  __Pyx_GIVEREF(__pyx_float_46_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 86, __pyx_float_46_11)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 87, __pyx_float__0003)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_47_40);
  __Pyx_GIVEREF(__pyx_float_47_40);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 88, __pyx_float_47_40)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 89, __pyx_float__0002)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_48_36);
  __Pyx_GIVEREF(__pyx_float_48_36);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 90, __pyx_float_48_36)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 91, __pyx_float__0004)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_49_55);
  __Pyx_GIVEREF(__pyx_float_49_55);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 92, __pyx_float_49_55)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 93, __pyx_float__0004)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_50_72);
  __Pyx_GIVEREF(__pyx_float_50_72);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 94, __pyx_float_50_72)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 95, __pyx_float__0002)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_52_00);
  __Pyx_GIVEREF(__pyx_float_52_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 96, __pyx_float_52_00)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 97, __pyx_float__0001)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_55_58);
  __Pyx_GIVEREF(__pyx_float_55_58);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 98, __pyx_float_55_58)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 99, __pyx_float__0002)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_57_02);
  __Pyx_GIVEREF(__pyx_float_57_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 100, __pyx_float_57_02)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 101, __pyx_float__0001)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_57_98);
  __Pyx_GIVEREF(__pyx_float_57_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 102, __pyx_float_57_98)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 103, __pyx_float__0002)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_59_13);
  __Pyx_GIVEREF(__pyx_float_59_13);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 104, __pyx_float_59_13)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 105, __pyx_float__0003)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_61_56);
  __Pyx_GIVEREF(__pyx_float_61_56);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 106, __pyx_float_61_56)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 107, __pyx_float__0001)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_66_56);
  __Pyx_GIVEREF(__pyx_float_66_56);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 108, __pyx_float_66_56)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 109, __pyx_float__0001)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_87_65);
  __Pyx_GIVEREF(__pyx_float_87_65);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 110, __pyx_float_87_65)) __PYX_ERR(0, 41, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_15, 111, __pyx_float__0002)) __PYX_ERR(0, 41, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":42
 * 			        [0.98,.0629,1.99,.0232,2.98,.0217,4.00,.0396,4.98,.0171,5.97,.0098,6.99,.0167 ,7.99,.0003,8.98,.0192,9.98,.0266,10.99,.0256,12.01,.0061,13.02,.0135,14.02,.0062,15.05,.0158 ,16.06,.0018,17.08,.0101,18.09,.0053,19.11,.0074,20.13,.0020,21.17,.0052,22.22,.0077,23.24,.0035 ,24.00,.0009,25.32,.0016,26.40,.0022,27.43,.0005,28.55,.0026,29.60,.0026,30.65,.0010,31.67,.0019 ,32.77,.0008,33.81,.0003,34.91,.0003,36.01,.0005,37.11,.0010,38.20,.0014,39.29,.0039,40.43,.0012 ,41.50,.0006,43.38,.0017,43.75,.0002,44.94,.0005,46.13,.0002,47.11,.0003,48.28,.0005,48.42,.0005 ,49.44,.0003,50.76,.0004,51.93,.0002,54.15,.0003,55.31,.0005,55.50,.0003,56.98,.0003,57.90,.0004 ,60.33,.0002,61.39,.0001,61.59,.0001,65.09,.0002,66.34,.0001,68.85,.0001,70.42,.0002,71.72,.0001 ,73.05,.0003,79.65,.0001,85.28,.0002,93.52,.0001],
 * 			        [1.02,.0185,1.99,.0525,2.98,.0613,3.99,.0415,4.98,.0109,5.97,.0248,6.99,.0102 ,7.98,.0005,8.98,.0124,9.99,.0103,10.99,.0124,12.00,.0016,13.01,.0029,14.03,.0211,15.04,.0128 ,16.07,.0021,17.09,.0009,18.09,.0043,19.14,.0022,20.13,.0016,21.20,.0045,22.21,.0088,23.26,.0046 ,24.29,.0013,25.35,.0009,26.39,.0028,27.49,.0009,28.51,.0006,29.58,.0012,30.70,.0010,31.74,.0019 ,32.75,.0002,33.85,.0001,34.95,.0005,36.02,.0003,37.16,.0009,38.25,.0018,39.35,.0008,40.54,.0004 ,41.61,.0002,43.40,.0004,43.74,.0003,45.05,.0001,46.11,.0003,47.40,.0002,48.36,.0004,49.55,.0004 ,50.72,.0002,52.00,.0001,55.58,.0002,57.02,.0001,57.98,.0002,59.13,.0003,61.56,.0001,66.56,.0001 ,87.65,.0002],
 * 			        [1.00,.0473,1.99,.0506,2.99,.0982,3.99,.0654,5.00,.0196,5.99,.0094,6.99,.0118 ,7.93,.0001,8.99,.0057,10.01,.0285,11.01,.0142,12.03,.0032,13.03,.0056,14.06,.0064,15.06,.0059 ,16.11,.0005,17.09,.0033,18.14,.0027,19.15,.0014,20.17,.0010,21.21,.0059,22.26,.0043,23.31,.0031 ,24.31,.0018,25.33,.0009,26.41,.0005,27.47,.0015,28.53,.0015,29.58,.0041,30.65,.0025,31.73,.0011 ,32.83,.0010,34.98,.0003,36.07,.0009,37.23,.0001,38.26,.0020,39.41,.0014,40.53,.0005,41.40,.0003 ,42.80,.0002,43.48,.0028,43.93,.0001,45.03,.0003,46.18,.0007,47.41,.0001,48.57,.0002,49.67,.0001 ,50.83,.0002,54.39,.0001,55.58,.0002,57.97,.0005,58.11,.0002,59.21,.0001,60.42,.0002,61.66,.0001],             # <<<<<<<<<<<<<<
 * 			        [1.00,.0503,2.00,.0963,2.99,.1304,3.99,.0218,4.98,.0041,5.98,.0292,6.98,.0482 ,7.99,.0005,8.99,.0280,10.00,.0237,11.00,.0152,12.02,.0036,12.95,.0022,14.06,.0111,15.07,.0196 ,16.08,.0016,17.11,.0044,18.13,.0073,19.17,.0055,20.19,.0028,21.20,.0012,22.27,.0068,23.30,.0036 ,24.35,.0012,25.35,.0002,26.46,.0005,27.47,.0005,28.59,.0009,29.65,.0021,30.70,.0020,31.78,.0012 ,32.89,.0010,35.06,.0005,36.16,.0008,37.27,.0010,38.36,.0010,39.47,.0014,40.58,.0004,41.43,.0007 ,41.82,.0003,43.48,.0008,44.53,.0001,45.25,.0003,46.43,.0002,47.46,.0002,48.76,.0005,49.95,.0004 ,50.96,.0002,51.12,.0002,52.33,.0001,54.75,.0001,55.75,.0002,56.90,.0002,58.17,.0002,59.40,.0004 ,60.62,.0002,65.65,.0001,66.91,.0002,69.91,.0001,71.25,.0002],
 * 			        [1.00,.1243,1.98,.1611,3.00,.0698,3.98,.0390,5.00,.0138,5.99,.0154,7.01,.0287 ,8.01,.0014,9.01,.0049,10.00,.0144,11.01,.0055,12.05,.0052,13.01,.0011,14.05,.0118,15.07,.0154 ,16.12,.0028,17.14,.0061,18.25,.0007,19.22,.0020,20.24,.0011,21.27,.0029,22.30,.0046,23.34,.0049 ,24.35,.0004,25.45,.0003,26.47,.0007,27.59,.0008,28.16,.0009,29.12,.0002,29.81,.0006,30.81,.0009 ,31.95,.0004,33.00,.0011,34.12,.0005,35.18,.0003,36.30,.0008,37.38,.0003,38.55,.0003,39.64,.0006 ,40.77,.0007,41.52,.0006,41.89,.0006,43.04,.0011,43.60,.0009,44.31,.0002,45.68,.0002,46.56,.0003 ,47.60,.0001,48.83,.0006,50.01,.0003,51.27,.0003,56.04,.0005,57.21,.0003,58.56,.0004,59.83,.0003 ,61.05,.0001,62.20,.0001,67.37,.0002,76.53,.0001],
 */
  __pyx_t_16 = PyList_New(110); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_16);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 0, __pyx_float_1_00)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0473);
  __Pyx_GIVEREF(__pyx_float__0473);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 1, __pyx_float__0473)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_1_99);
  __Pyx_GIVEREF(__pyx_float_1_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 2, __pyx_float_1_99)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0506);
  __Pyx_GIVEREF(__pyx_float__0506);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 3, __pyx_float__0506)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_99);
  __Pyx_GIVEREF(__pyx_float_2_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 4, __pyx_float_2_99)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0982);
  __Pyx_GIVEREF(__pyx_float__0982);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 5, __pyx_float__0982)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_99);
  __Pyx_GIVEREF(__pyx_float_3_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 6, __pyx_float_3_99)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0654);
  __Pyx_GIVEREF(__pyx_float__0654);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 7, __pyx_float__0654)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_00);
  __Pyx_GIVEREF(__pyx_float_5_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 8, __pyx_float_5_00)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0196);
  __Pyx_GIVEREF(__pyx_float__0196);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 9, __pyx_float__0196)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_99);
  __Pyx_GIVEREF(__pyx_float_5_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 10, __pyx_float_5_99)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0094);
  __Pyx_GIVEREF(__pyx_float__0094);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 11, __pyx_float__0094)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_99);
  __Pyx_GIVEREF(__pyx_float_6_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 12, __pyx_float_6_99)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0118);
  __Pyx_GIVEREF(__pyx_float__0118);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 13, __pyx_float__0118)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_93);
  __Pyx_GIVEREF(__pyx_float_7_93);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 14, __pyx_float_7_93)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 15, __pyx_float__0001)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_99);
  __Pyx_GIVEREF(__pyx_float_8_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 16, __pyx_float_8_99)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0057);
  __Pyx_GIVEREF(__pyx_float__0057);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 17, __pyx_float__0057)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_01);
  __Pyx_GIVEREF(__pyx_float_10_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 18, __pyx_float_10_01)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0285);
  __Pyx_GIVEREF(__pyx_float__0285);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 19, __pyx_float__0285)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_01);
  __Pyx_GIVEREF(__pyx_float_11_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 20, __pyx_float_11_01)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0142);
  __Pyx_GIVEREF(__pyx_float__0142);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 21, __pyx_float__0142)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_03);
  __Pyx_GIVEREF(__pyx_float_12_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 22, __pyx_float_12_03)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0032);
  __Pyx_GIVEREF(__pyx_float__0032);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 23, __pyx_float__0032)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_03);
  __Pyx_GIVEREF(__pyx_float_13_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 24, __pyx_float_13_03)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0056);
  __Pyx_GIVEREF(__pyx_float__0056);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 25, __pyx_float__0056)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_06);
  __Pyx_GIVEREF(__pyx_float_14_06);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 26, __pyx_float_14_06)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0064);
  __Pyx_GIVEREF(__pyx_float__0064);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 27, __pyx_float__0064)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_06);
  __Pyx_GIVEREF(__pyx_float_15_06);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 28, __pyx_float_15_06)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0059);
  __Pyx_GIVEREF(__pyx_float__0059);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 29, __pyx_float__0059)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_11);
  __Pyx_GIVEREF(__pyx_float_16_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 30, __pyx_float_16_11)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 31, __pyx_float__0005)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_09);
  __Pyx_GIVEREF(__pyx_float_17_09);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 32, __pyx_float_17_09)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0033);
  __Pyx_GIVEREF(__pyx_float__0033);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 33, __pyx_float__0033)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_14);
  __Pyx_GIVEREF(__pyx_float_18_14);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 34, __pyx_float_18_14)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0027);
  __Pyx_GIVEREF(__pyx_float__0027);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 35, __pyx_float__0027)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_15);
  __Pyx_GIVEREF(__pyx_float_19_15);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 36, __pyx_float_19_15)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0014);
  __Pyx_GIVEREF(__pyx_float__0014);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 37, __pyx_float__0014)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_17);
  __Pyx_GIVEREF(__pyx_float_20_17);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 38, __pyx_float_20_17)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 39, __pyx_float__0010)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_21);
  __Pyx_GIVEREF(__pyx_float_21_21);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 40, __pyx_float_21_21)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0059);
  __Pyx_GIVEREF(__pyx_float__0059);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 41, __pyx_float__0059)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_26);
  __Pyx_GIVEREF(__pyx_float_22_26);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 42, __pyx_float_22_26)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0043);
  __Pyx_GIVEREF(__pyx_float__0043);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 43, __pyx_float__0043)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_31);
  __Pyx_GIVEREF(__pyx_float_23_31);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 44, __pyx_float_23_31)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0031);
  __Pyx_GIVEREF(__pyx_float__0031);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 45, __pyx_float__0031)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_24_31);
  __Pyx_GIVEREF(__pyx_float_24_31);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 46, __pyx_float_24_31)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0018);
  __Pyx_GIVEREF(__pyx_float__0018);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 47, __pyx_float__0018)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_25_33);
  __Pyx_GIVEREF(__pyx_float_25_33);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 48, __pyx_float_25_33)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 49, __pyx_float__0009)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_26_41);
  __Pyx_GIVEREF(__pyx_float_26_41);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 50, __pyx_float_26_41)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 51, __pyx_float__0005)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_27_47);
  __Pyx_GIVEREF(__pyx_float_27_47);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 52, __pyx_float_27_47)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0015);
  __Pyx_GIVEREF(__pyx_float__0015);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 53, __pyx_float__0015)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_53);
  __Pyx_GIVEREF(__pyx_float_28_53);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 54, __pyx_float_28_53)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0015);
  __Pyx_GIVEREF(__pyx_float__0015);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 55, __pyx_float__0015)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_29_58);
  __Pyx_GIVEREF(__pyx_float_29_58);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 56, __pyx_float_29_58)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0041);
  __Pyx_GIVEREF(__pyx_float__0041);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 57, __pyx_float__0041)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_30_65);
  __Pyx_GIVEREF(__pyx_float_30_65);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 58, __pyx_float_30_65)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0025);
  __Pyx_GIVEREF(__pyx_float__0025);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 59, __pyx_float__0025)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_31_73);
  __Pyx_GIVEREF(__pyx_float_31_73);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 60, __pyx_float_31_73)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 61, __pyx_float__0011)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_32_83);
  __Pyx_GIVEREF(__pyx_float_32_83);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 62, __pyx_float_32_83)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 63, __pyx_float__0010)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_34_98);
  __Pyx_GIVEREF(__pyx_float_34_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 64, __pyx_float_34_98)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 65, __pyx_float__0003)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_36_07);
  __Pyx_GIVEREF(__pyx_float_36_07);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 66, __pyx_float_36_07)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 67, __pyx_float__0009)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_37_23);
  __Pyx_GIVEREF(__pyx_float_37_23);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 68, __pyx_float_37_23)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 69, __pyx_float__0001)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_38_26);
  __Pyx_GIVEREF(__pyx_float_38_26);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 70, __pyx_float_38_26)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0020);
  __Pyx_GIVEREF(__pyx_float__0020);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 71, __pyx_float__0020)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_39_41);
  __Pyx_GIVEREF(__pyx_float_39_41);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 72, __pyx_float_39_41)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0014);
  __Pyx_GIVEREF(__pyx_float__0014);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 73, __pyx_float__0014)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_40_53);
  __Pyx_GIVEREF(__pyx_float_40_53);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 74, __pyx_float_40_53)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 75, __pyx_float__0005)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_41_40);
  __Pyx_GIVEREF(__pyx_float_41_40);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 76, __pyx_float_41_40)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 77, __pyx_float__0003)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_42_80);
  __Pyx_GIVEREF(__pyx_float_42_80);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 78, __pyx_float_42_80)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 79, __pyx_float__0002)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_43_48);
  __Pyx_GIVEREF(__pyx_float_43_48);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 80, __pyx_float_43_48)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0028);
  __Pyx_GIVEREF(__pyx_float__0028);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 81, __pyx_float__0028)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_43_93);
  __Pyx_GIVEREF(__pyx_float_43_93);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 82, __pyx_float_43_93)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 83, __pyx_float__0001)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_45_03);
  __Pyx_GIVEREF(__pyx_float_45_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 84, __pyx_float_45_03)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 85, __pyx_float__0003)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_46_18);
  __Pyx_GIVEREF(__pyx_float_46_18);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 86, __pyx_float_46_18)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 87, __pyx_float__0007)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_47_41);
  __Pyx_GIVEREF(__pyx_float_47_41);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 88, __pyx_float_47_41)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 89, __pyx_float__0001)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_48_57);
  __Pyx_GIVEREF(__pyx_float_48_57);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 90, __pyx_float_48_57)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 91, __pyx_float__0002)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_49_67);
  __Pyx_GIVEREF(__pyx_float_49_67);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 92, __pyx_float_49_67)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 93, __pyx_float__0001)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_50_83);
  __Pyx_GIVEREF(__pyx_float_50_83);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 94, __pyx_float_50_83)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 95, __pyx_float__0002)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_54_39);
  __Pyx_GIVEREF(__pyx_float_54_39);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 96, __pyx_float_54_39)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 97, __pyx_float__0001)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_55_58);
  __Pyx_GIVEREF(__pyx_float_55_58);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 98, __pyx_float_55_58)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 99, __pyx_float__0002)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_57_97);
  __Pyx_GIVEREF(__pyx_float_57_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 100, __pyx_float_57_97)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 101, __pyx_float__0005)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_58_11);
  __Pyx_GIVEREF(__pyx_float_58_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 102, __pyx_float_58_11)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 103, __pyx_float__0002)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_59_21);
  __Pyx_GIVEREF(__pyx_float_59_21);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 104, __pyx_float_59_21)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 105, __pyx_float__0001)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_60_42);
  __Pyx_GIVEREF(__pyx_float_60_42);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 106, __pyx_float_60_42)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 107, __pyx_float__0002)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_61_66);
  __Pyx_GIVEREF(__pyx_float_61_66);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 108, __pyx_float_61_66)) __PYX_ERR(0, 42, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_16, 109, __pyx_float__0001)) __PYX_ERR(0, 42, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":43
 * 			        [1.02,.0185,1.99,.0525,2.98,.0613,3.99,.0415,4.98,.0109,5.97,.0248,6.99,.0102 ,7.98,.0005,8.98,.0124,9.99,.0103,10.99,.0124,12.00,.0016,13.01,.0029,14.03,.0211,15.04,.0128 ,16.07,.0021,17.09,.0009,18.09,.0043,19.14,.0022,20.13,.0016,21.20,.0045,22.21,.0088,23.26,.0046 ,24.29,.0013,25.35,.0009,26.39,.0028,27.49,.0009,28.51,.0006,29.58,.0012,30.70,.0010,31.74,.0019 ,32.75,.0002,33.85,.0001,34.95,.0005,36.02,.0003,37.16,.0009,38.25,.0018,39.35,.0008,40.54,.0004 ,41.61,.0002,43.40,.0004,43.74,.0003,45.05,.0001,46.11,.0003,47.40,.0002,48.36,.0004,49.55,.0004 ,50.72,.0002,52.00,.0001,55.58,.0002,57.02,.0001,57.98,.0002,59.13,.0003,61.56,.0001,66.56,.0001 ,87.65,.0002],
 * 			        [1.00,.0473,1.99,.0506,2.99,.0982,3.99,.0654,5.00,.0196,5.99,.0094,6.99,.0118 ,7.93,.0001,8.99,.0057,10.01,.0285,11.01,.0142,12.03,.0032,13.03,.0056,14.06,.0064,15.06,.0059 ,16.11,.0005,17.09,.0033,18.14,.0027,19.15,.0014,20.17,.0010,21.21,.0059,22.26,.0043,23.31,.0031 ,24.31,.0018,25.33,.0009,26.41,.0005,27.47,.0015,28.53,.0015,29.58,.0041,30.65,.0025,31.73,.0011 ,32.83,.0010,34.98,.0003,36.07,.0009,37.23,.0001,38.26,.0020,39.41,.0014,40.53,.0005,41.40,.0003 ,42.80,.0002,43.48,.0028,43.93,.0001,45.03,.0003,46.18,.0007,47.41,.0001,48.57,.0002,49.67,.0001 ,50.83,.0002,54.39,.0001,55.58,.0002,57.97,.0005,58.11,.0002,59.21,.0001,60.42,.0002,61.66,.0001],
 * 			        [1.00,.0503,2.00,.0963,2.99,.1304,3.99,.0218,4.98,.0041,5.98,.0292,6.98,.0482 ,7.99,.0005,8.99,.0280,10.00,.0237,11.00,.0152,12.02,.0036,12.95,.0022,14.06,.0111,15.07,.0196 ,16.08,.0016,17.11,.0044,18.13,.0073,19.17,.0055,20.19,.0028,21.20,.0012,22.27,.0068,23.30,.0036 ,24.35,.0012,25.35,.0002,26.46,.0005,27.47,.0005,28.59,.0009,29.65,.0021,30.70,.0020,31.78,.0012 ,32.89,.0010,35.06,.0005,36.16,.0008,37.27,.0010,38.36,.0010,39.47,.0014,40.58,.0004,41.43,.0007 ,41.82,.0003,43.48,.0008,44.53,.0001,45.25,.0003,46.43,.0002,47.46,.0002,48.76,.0005,49.95,.0004 ,50.96,.0002,51.12,.0002,52.33,.0001,54.75,.0001,55.75,.0002,56.90,.0002,58.17,.0002,59.40,.0004 ,60.62,.0002,65.65,.0001,66.91,.0002,69.91,.0001,71.25,.0002],             # <<<<<<<<<<<<<<
 * 			        [1.00,.1243,1.98,.1611,3.00,.0698,3.98,.0390,5.00,.0138,5.99,.0154,7.01,.0287 ,8.01,.0014,9.01,.0049,10.00,.0144,11.01,.0055,12.05,.0052,13.01,.0011,14.05,.0118,15.07,.0154 ,16.12,.0028,17.14,.0061,18.25,.0007,19.22,.0020,20.24,.0011,21.27,.0029,22.30,.0046,23.34,.0049 ,24.35,.0004,25.45,.0003,26.47,.0007,27.59,.0008,28.16,.0009,29.12,.0002,29.81,.0006,30.81,.0009 ,31.95,.0004,33.00,.0011,34.12,.0005,35.18,.0003,36.30,.0008,37.38,.0003,38.55,.0003,39.64,.0006 ,40.77,.0007,41.52,.0006,41.89,.0006,43.04,.0011,43.60,.0009,44.31,.0002,45.68,.0002,46.56,.0003 ,47.60,.0001,48.83,.0006,50.01,.0003,51.27,.0003,56.04,.0005,57.21,.0003,58.56,.0004,59.83,.0003 ,61.05,.0001,62.20,.0001,67.37,.0002,76.53,.0001],
 * 			        [0.99,.0222,1.99,.0678,2.99,.0683,4.00,.0191,5.00,.0119,6.01,.0232,6.98,.0336 ,7.99,.0082,9.01,.0201,10.01,.0189,11.01,.0041,12.01,.0053,13.05,.0154,14.04,.0159,15.06,.0092 ,16.11,.0038,17.12,.0014,18.15,.0091,19.16,.0006,20.30,.0012,21.25,.0061,22.28,.0099,23.34,.0028 ,24.38,.0012,25.43,.0016,26.49,.0048,27.55,.0025,28.62,.0015,29.71,.0032,30.78,.0077,31.88,.0011 ,32.97,.0007,34.08,.0006,35.16,.0008,36.28,.0004,37.41,.0006,38.54,.0005,39.62,.0002,40.80,.0003 ,41.93,.0001,43.06,.0002,44.21,.0003,45.38,.0002,46.54,.0007,47.78,.0003,48.95,.0004,50.10,.0003 ,51.37,.0002,53.79,.0003,56.20,.0001,58.71,.0002,66.47,.0003],
 */
  __pyx_t_17 = PyList_New(120); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_17);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 0, __pyx_float_1_00)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0503);
  __Pyx_GIVEREF(__pyx_float__0503);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 1, __pyx_float__0503)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_00);
  __Pyx_GIVEREF(__pyx_float_2_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 2, __pyx_float_2_00)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0963);
  __Pyx_GIVEREF(__pyx_float__0963);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 3, __pyx_float__0963)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_99);
  __Pyx_GIVEREF(__pyx_float_2_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 4, __pyx_float_2_99)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1304);
  __Pyx_GIVEREF(__pyx_float__1304);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 5, __pyx_float__1304)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_99);
  __Pyx_GIVEREF(__pyx_float_3_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 6, __pyx_float_3_99)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0218);
  __Pyx_GIVEREF(__pyx_float__0218);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 7, __pyx_float__0218)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_98);
  __Pyx_GIVEREF(__pyx_float_4_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 8, __pyx_float_4_98)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0041);
  __Pyx_GIVEREF(__pyx_float__0041);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 9, __pyx_float__0041)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_98);
  __Pyx_GIVEREF(__pyx_float_5_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 10, __pyx_float_5_98)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0292);
  __Pyx_GIVEREF(__pyx_float__0292);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 11, __pyx_float__0292)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_98);
  __Pyx_GIVEREF(__pyx_float_6_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 12, __pyx_float_6_98)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0482);
  __Pyx_GIVEREF(__pyx_float__0482);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 13, __pyx_float__0482)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_99);
  __Pyx_GIVEREF(__pyx_float_7_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 14, __pyx_float_7_99)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 15, __pyx_float__0005)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_99);
  __Pyx_GIVEREF(__pyx_float_8_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 16, __pyx_float_8_99)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0280);
  __Pyx_GIVEREF(__pyx_float__0280);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 17, __pyx_float__0280)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_00);
  __Pyx_GIVEREF(__pyx_float_10_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 18, __pyx_float_10_00)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0237);
  __Pyx_GIVEREF(__pyx_float__0237);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 19, __pyx_float__0237)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_00);
  __Pyx_GIVEREF(__pyx_float_11_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 20, __pyx_float_11_00)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0152);
  __Pyx_GIVEREF(__pyx_float__0152);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 21, __pyx_float__0152)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_02);
  __Pyx_GIVEREF(__pyx_float_12_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 22, __pyx_float_12_02)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0036);
  __Pyx_GIVEREF(__pyx_float__0036);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 23, __pyx_float__0036)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_95);
  __Pyx_GIVEREF(__pyx_float_12_95);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 24, __pyx_float_12_95)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0022);
  __Pyx_GIVEREF(__pyx_float__0022);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 25, __pyx_float__0022)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_06);
  __Pyx_GIVEREF(__pyx_float_14_06);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 26, __pyx_float_14_06)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0111);
  __Pyx_GIVEREF(__pyx_float__0111);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 27, __pyx_float__0111)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_07);
  __Pyx_GIVEREF(__pyx_float_15_07);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 28, __pyx_float_15_07)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0196);
  __Pyx_GIVEREF(__pyx_float__0196);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 29, __pyx_float__0196)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_08);
  __Pyx_GIVEREF(__pyx_float_16_08);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 30, __pyx_float_16_08)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0016);
  __Pyx_GIVEREF(__pyx_float__0016);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 31, __pyx_float__0016)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_11);
  __Pyx_GIVEREF(__pyx_float_17_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 32, __pyx_float_17_11)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0044);
  __Pyx_GIVEREF(__pyx_float__0044);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 33, __pyx_float__0044)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_13);
  __Pyx_GIVEREF(__pyx_float_18_13);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 34, __pyx_float_18_13)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0073);
  __Pyx_GIVEREF(__pyx_float__0073);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 35, __pyx_float__0073)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_17);
  __Pyx_GIVEREF(__pyx_float_19_17);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 36, __pyx_float_19_17)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0055);
  __Pyx_GIVEREF(__pyx_float__0055);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 37, __pyx_float__0055)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_19);
  __Pyx_GIVEREF(__pyx_float_20_19);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 38, __pyx_float_20_19)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0028);
  __Pyx_GIVEREF(__pyx_float__0028);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 39, __pyx_float__0028)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_20);
  __Pyx_GIVEREF(__pyx_float_21_20);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 40, __pyx_float_21_20)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 41, __pyx_float__0012)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_27);
  __Pyx_GIVEREF(__pyx_float_22_27);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 42, __pyx_float_22_27)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0068);
  __Pyx_GIVEREF(__pyx_float__0068);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 43, __pyx_float__0068)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_30);
  __Pyx_GIVEREF(__pyx_float_23_30);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 44, __pyx_float_23_30)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0036);
  __Pyx_GIVEREF(__pyx_float__0036);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 45, __pyx_float__0036)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_24_35);
  __Pyx_GIVEREF(__pyx_float_24_35);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 46, __pyx_float_24_35)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 47, __pyx_float__0012)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_25_35);
  __Pyx_GIVEREF(__pyx_float_25_35);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 48, __pyx_float_25_35)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 49, __pyx_float__0002)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_26_46);
  __Pyx_GIVEREF(__pyx_float_26_46);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 50, __pyx_float_26_46)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 51, __pyx_float__0005)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_27_47);
  __Pyx_GIVEREF(__pyx_float_27_47);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 52, __pyx_float_27_47)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 53, __pyx_float__0005)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_59);
  __Pyx_GIVEREF(__pyx_float_28_59);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 54, __pyx_float_28_59)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 55, __pyx_float__0009)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_29_65);
  __Pyx_GIVEREF(__pyx_float_29_65);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 56, __pyx_float_29_65)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0021);
  __Pyx_GIVEREF(__pyx_float__0021);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 57, __pyx_float__0021)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_30_70);
  __Pyx_GIVEREF(__pyx_float_30_70);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 58, __pyx_float_30_70)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0020);
  __Pyx_GIVEREF(__pyx_float__0020);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 59, __pyx_float__0020)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_31_78);
  __Pyx_GIVEREF(__pyx_float_31_78);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 60, __pyx_float_31_78)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 61, __pyx_float__0012)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_32_89);
  __Pyx_GIVEREF(__pyx_float_32_89);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 62, __pyx_float_32_89)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 63, __pyx_float__0010)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_35_06);
  __Pyx_GIVEREF(__pyx_float_35_06);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 64, __pyx_float_35_06)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 65, __pyx_float__0005)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_36_16);
  __Pyx_GIVEREF(__pyx_float_36_16);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 66, __pyx_float_36_16)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 67, __pyx_float__0008)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_37_27);
  __Pyx_GIVEREF(__pyx_float_37_27);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 68, __pyx_float_37_27)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 69, __pyx_float__0010)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_38_36);
  __Pyx_GIVEREF(__pyx_float_38_36);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 70, __pyx_float_38_36)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 71, __pyx_float__0010)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_39_47);
  __Pyx_GIVEREF(__pyx_float_39_47);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 72, __pyx_float_39_47)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0014);
  __Pyx_GIVEREF(__pyx_float__0014);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 73, __pyx_float__0014)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_40_58);
  __Pyx_GIVEREF(__pyx_float_40_58);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 74, __pyx_float_40_58)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 75, __pyx_float__0004)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_41_43);
  __Pyx_GIVEREF(__pyx_float_41_43);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 76, __pyx_float_41_43)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 77, __pyx_float__0007)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_41_82);
  __Pyx_GIVEREF(__pyx_float_41_82);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 78, __pyx_float_41_82)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 79, __pyx_float__0003)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_43_48);
  __Pyx_GIVEREF(__pyx_float_43_48);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 80, __pyx_float_43_48)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 81, __pyx_float__0008)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_44_53);
  __Pyx_GIVEREF(__pyx_float_44_53);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 82, __pyx_float_44_53)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 83, __pyx_float__0001)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_45_25);
  __Pyx_GIVEREF(__pyx_float_45_25);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 84, __pyx_float_45_25)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 85, __pyx_float__0003)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_46_43);
  __Pyx_GIVEREF(__pyx_float_46_43);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 86, __pyx_float_46_43)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 87, __pyx_float__0002)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_47_46);
  __Pyx_GIVEREF(__pyx_float_47_46);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 88, __pyx_float_47_46)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 89, __pyx_float__0002)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_48_76);
  __Pyx_GIVEREF(__pyx_float_48_76);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 90, __pyx_float_48_76)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 91, __pyx_float__0005)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_49_95);
  __Pyx_GIVEREF(__pyx_float_49_95);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 92, __pyx_float_49_95)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 93, __pyx_float__0004)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_50_96);
  __Pyx_GIVEREF(__pyx_float_50_96);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 94, __pyx_float_50_96)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 95, __pyx_float__0002)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_51_12);
  __Pyx_GIVEREF(__pyx_float_51_12);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 96, __pyx_float_51_12)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 97, __pyx_float__0002)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_52_33);
  __Pyx_GIVEREF(__pyx_float_52_33);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 98, __pyx_float_52_33)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 99, __pyx_float__0001)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_54_75);
  __Pyx_GIVEREF(__pyx_float_54_75);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 100, __pyx_float_54_75)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 101, __pyx_float__0001)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_55_75);
  __Pyx_GIVEREF(__pyx_float_55_75);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 102, __pyx_float_55_75)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 103, __pyx_float__0002)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_56_90);
  __Pyx_GIVEREF(__pyx_float_56_90);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 104, __pyx_float_56_90)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 105, __pyx_float__0002)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_58_17);
  __Pyx_GIVEREF(__pyx_float_58_17);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 106, __pyx_float_58_17)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 107, __pyx_float__0002)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_59_40);
  __Pyx_GIVEREF(__pyx_float_59_40);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 108, __pyx_float_59_40)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 109, __pyx_float__0004)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_60_62);
  __Pyx_GIVEREF(__pyx_float_60_62);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 110, __pyx_float_60_62)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 111, __pyx_float__0002)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_65_65);
  __Pyx_GIVEREF(__pyx_float_65_65);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 112, __pyx_float_65_65)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 113, __pyx_float__0001)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_66_91);
  __Pyx_GIVEREF(__pyx_float_66_91);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 114, __pyx_float_66_91)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 115, __pyx_float__0002)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_69_91);
  __Pyx_GIVEREF(__pyx_float_69_91);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 116, __pyx_float_69_91)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 117, __pyx_float__0001)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_71_25);
  __Pyx_GIVEREF(__pyx_float_71_25);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 118, __pyx_float_71_25)) __PYX_ERR(0, 43, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 119, __pyx_float__0002)) __PYX_ERR(0, 43, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":44
 * 			        [1.00,.0473,1.99,.0506,2.99,.0982,3.99,.0654,5.00,.0196,5.99,.0094,6.99,.0118 ,7.93,.0001,8.99,.0057,10.01,.0285,11.01,.0142,12.03,.0032,13.03,.0056,14.06,.0064,15.06,.0059 ,16.11,.0005,17.09,.0033,18.14,.0027,19.15,.0014,20.17,.0010,21.21,.0059,22.26,.0043,23.31,.0031 ,24.31,.0018,25.33,.0009,26.41,.0005,27.47,.0015,28.53,.0015,29.58,.0041,30.65,.0025,31.73,.0011 ,32.83,.0010,34.98,.0003,36.07,.0009,37.23,.0001,38.26,.0020,39.41,.0014,40.53,.0005,41.40,.0003 ,42.80,.0002,43.48,.0028,43.93,.0001,45.03,.0003,46.18,.0007,47.41,.0001,48.57,.0002,49.67,.0001 ,50.83,.0002,54.39,.0001,55.58,.0002,57.97,.0005,58.11,.0002,59.21,.0001,60.42,.0002,61.66,.0001],
 * 			        [1.00,.0503,2.00,.0963,2.99,.1304,3.99,.0218,4.98,.0041,5.98,.0292,6.98,.0482 ,7.99,.0005,8.99,.0280,10.00,.0237,11.00,.0152,12.02,.0036,12.95,.0022,14.06,.0111,15.07,.0196 ,16.08,.0016,17.11,.0044,18.13,.0073,19.17,.0055,20.19,.0028,21.20,.0012,22.27,.0068,23.30,.0036 ,24.35,.0012,25.35,.0002,26.46,.0005,27.47,.0005,28.59,.0009,29.65,.0021,30.70,.0020,31.78,.0012 ,32.89,.0010,35.06,.0005,36.16,.0008,37.27,.0010,38.36,.0010,39.47,.0014,40.58,.0004,41.43,.0007 ,41.82,.0003,43.48,.0008,44.53,.0001,45.25,.0003,46.43,.0002,47.46,.0002,48.76,.0005,49.95,.0004 ,50.96,.0002,51.12,.0002,52.33,.0001,54.75,.0001,55.75,.0002,56.90,.0002,58.17,.0002,59.40,.0004 ,60.62,.0002,65.65,.0001,66.91,.0002,69.91,.0001,71.25,.0002],
 * 			        [1.00,.1243,1.98,.1611,3.00,.0698,3.98,.0390,5.00,.0138,5.99,.0154,7.01,.0287 ,8.01,.0014,9.01,.0049,10.00,.0144,11.01,.0055,12.05,.0052,13.01,.0011,14.05,.0118,15.07,.0154 ,16.12,.0028,17.14,.0061,18.25,.0007,19.22,.0020,20.24,.0011,21.27,.0029,22.30,.0046,23.34,.0049 ,24.35,.0004,25.45,.0003,26.47,.0007,27.59,.0008,28.16,.0009,29.12,.0002,29.81,.0006,30.81,.0009 ,31.95,.0004,33.00,.0011,34.12,.0005,35.18,.0003,36.30,.0008,37.38,.0003,38.55,.0003,39.64,.0006 ,40.77,.0007,41.52,.0006,41.89,.0006,43.04,.0011,43.60,.0009,44.31,.0002,45.68,.0002,46.56,.0003 ,47.60,.0001,48.83,.0006,50.01,.0003,51.27,.0003,56.04,.0005,57.21,.0003,58.56,.0004,59.83,.0003 ,61.05,.0001,62.20,.0001,67.37,.0002,76.53,.0001],             # <<<<<<<<<<<<<<
 * 			        [0.99,.0222,1.99,.0678,2.99,.0683,4.00,.0191,5.00,.0119,6.01,.0232,6.98,.0336 ,7.99,.0082,9.01,.0201,10.01,.0189,11.01,.0041,12.01,.0053,13.05,.0154,14.04,.0159,15.06,.0092 ,16.11,.0038,17.12,.0014,18.15,.0091,19.16,.0006,20.30,.0012,21.25,.0061,22.28,.0099,23.34,.0028 ,24.38,.0012,25.43,.0016,26.49,.0048,27.55,.0025,28.62,.0015,29.71,.0032,30.78,.0077,31.88,.0011 ,32.97,.0007,34.08,.0006,35.16,.0008,36.28,.0004,37.41,.0006,38.54,.0005,39.62,.0002,40.80,.0003 ,41.93,.0001,43.06,.0002,44.21,.0003,45.38,.0002,46.54,.0007,47.78,.0003,48.95,.0004,50.10,.0003 ,51.37,.0002,53.79,.0003,56.20,.0001,58.71,.0002,66.47,.0003],
 * 			        [1.01,.0241,1.99,.1011,2.98,.0938,3.98,.0081,4.99,.0062,5.99,.0291,6.99,.0676 ,7.59,.0004,8.98,.0127,9.99,.0112,10.99,.0142,12.00,.0029,13.02,.0071,14.02,.0184,15.03,.0064 ,16.07,.0010,17.09,.0011,18.11,.0010,19.15,.0060,20.19,.0019,21.24,.0025,22.29,.0013,23.31,.0050 ,25.41,.0030,26.50,.0018,27.53,.0006,28.63,.0012,29.66,.0013,30.77,.0020,31.84,.0006,34.04,.0001 ,35.14,.0001,36.32,.0004,37.41,.0007,38.53,.0007,39.67,.0009,40.85,.0003,45.49,.0002,46.65,.0001 ,47.81,.0004,49.01,.0002,53.91,.0002,55.14,.0002,57.69,.0002],
 */
  __pyx_t_18 = PyList_New(118); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_18);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 0, __pyx_float_1_00)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1243);
  __Pyx_GIVEREF(__pyx_float__1243);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 1, __pyx_float__1243)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_1_98);
  __Pyx_GIVEREF(__pyx_float_1_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 2, __pyx_float_1_98)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1611);
  __Pyx_GIVEREF(__pyx_float__1611);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 3, __pyx_float__1611)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_00);
  __Pyx_GIVEREF(__pyx_float_3_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 4, __pyx_float_3_00)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0698);
  __Pyx_GIVEREF(__pyx_float__0698);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 5, __pyx_float__0698)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_98);
  __Pyx_GIVEREF(__pyx_float_3_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 6, __pyx_float_3_98)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0390);
  __Pyx_GIVEREF(__pyx_float__0390);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 7, __pyx_float__0390)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_00);
  __Pyx_GIVEREF(__pyx_float_5_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 8, __pyx_float_5_00)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0138);
  __Pyx_GIVEREF(__pyx_float__0138);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 9, __pyx_float__0138)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_99);
  __Pyx_GIVEREF(__pyx_float_5_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 10, __pyx_float_5_99)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0154);
  __Pyx_GIVEREF(__pyx_float__0154);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 11, __pyx_float__0154)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_01);
  __Pyx_GIVEREF(__pyx_float_7_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 12, __pyx_float_7_01)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0287);
  __Pyx_GIVEREF(__pyx_float__0287);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 13, __pyx_float__0287)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_01);
  __Pyx_GIVEREF(__pyx_float_8_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 14, __pyx_float_8_01)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0014);
  __Pyx_GIVEREF(__pyx_float__0014);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 15, __pyx_float__0014)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_01);
  __Pyx_GIVEREF(__pyx_float_9_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 16, __pyx_float_9_01)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0049);
  __Pyx_GIVEREF(__pyx_float__0049);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 17, __pyx_float__0049)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_00);
  __Pyx_GIVEREF(__pyx_float_10_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 18, __pyx_float_10_00)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0144);
  __Pyx_GIVEREF(__pyx_float__0144);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 19, __pyx_float__0144)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_01);
  __Pyx_GIVEREF(__pyx_float_11_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 20, __pyx_float_11_01)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0055);
  __Pyx_GIVEREF(__pyx_float__0055);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 21, __pyx_float__0055)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_05);
  __Pyx_GIVEREF(__pyx_float_12_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 22, __pyx_float_12_05)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0052);
  __Pyx_GIVEREF(__pyx_float__0052);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 23, __pyx_float__0052)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_01);
  __Pyx_GIVEREF(__pyx_float_13_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 24, __pyx_float_13_01)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 25, __pyx_float__0011)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_05);
  __Pyx_GIVEREF(__pyx_float_14_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 26, __pyx_float_14_05)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0118);
  __Pyx_GIVEREF(__pyx_float__0118);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 27, __pyx_float__0118)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_07);
  __Pyx_GIVEREF(__pyx_float_15_07);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 28, __pyx_float_15_07)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0154);
  __Pyx_GIVEREF(__pyx_float__0154);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 29, __pyx_float__0154)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_12);
  __Pyx_GIVEREF(__pyx_float_16_12);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 30, __pyx_float_16_12)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0028);
  __Pyx_GIVEREF(__pyx_float__0028);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 31, __pyx_float__0028)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_14);
  __Pyx_GIVEREF(__pyx_float_17_14);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 32, __pyx_float_17_14)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0061);
  __Pyx_GIVEREF(__pyx_float__0061);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 33, __pyx_float__0061)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_25);
  __Pyx_GIVEREF(__pyx_float_18_25);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 34, __pyx_float_18_25)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 35, __pyx_float__0007)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_22);
  __Pyx_GIVEREF(__pyx_float_19_22);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 36, __pyx_float_19_22)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0020);
  __Pyx_GIVEREF(__pyx_float__0020);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 37, __pyx_float__0020)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_24);
  __Pyx_GIVEREF(__pyx_float_20_24);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 38, __pyx_float_20_24)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 39, __pyx_float__0011)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_27);
  __Pyx_GIVEREF(__pyx_float_21_27);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 40, __pyx_float_21_27)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0029);
  __Pyx_GIVEREF(__pyx_float__0029);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 41, __pyx_float__0029)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_30);
  __Pyx_GIVEREF(__pyx_float_22_30);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 42, __pyx_float_22_30)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0046);
  __Pyx_GIVEREF(__pyx_float__0046);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 43, __pyx_float__0046)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_34);
  __Pyx_GIVEREF(__pyx_float_23_34);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 44, __pyx_float_23_34)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0049);
  __Pyx_GIVEREF(__pyx_float__0049);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 45, __pyx_float__0049)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_24_35);
  __Pyx_GIVEREF(__pyx_float_24_35);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 46, __pyx_float_24_35)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 47, __pyx_float__0004)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_25_45);
  __Pyx_GIVEREF(__pyx_float_25_45);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 48, __pyx_float_25_45)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 49, __pyx_float__0003)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_26_47);
  __Pyx_GIVEREF(__pyx_float_26_47);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 50, __pyx_float_26_47)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 51, __pyx_float__0007)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_27_59);
  __Pyx_GIVEREF(__pyx_float_27_59);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 52, __pyx_float_27_59)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 53, __pyx_float__0008)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_16);
  __Pyx_GIVEREF(__pyx_float_28_16);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 54, __pyx_float_28_16)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 55, __pyx_float__0009)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_29_12);
  __Pyx_GIVEREF(__pyx_float_29_12);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 56, __pyx_float_29_12)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 57, __pyx_float__0002)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_29_81);
  __Pyx_GIVEREF(__pyx_float_29_81);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 58, __pyx_float_29_81)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 59, __pyx_float__0006)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_30_81);
  __Pyx_GIVEREF(__pyx_float_30_81);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 60, __pyx_float_30_81)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 61, __pyx_float__0009)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_31_95);
  __Pyx_GIVEREF(__pyx_float_31_95);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 62, __pyx_float_31_95)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 63, __pyx_float__0004)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_33_00);
  __Pyx_GIVEREF(__pyx_float_33_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 64, __pyx_float_33_00)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 65, __pyx_float__0011)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_34_12);
  __Pyx_GIVEREF(__pyx_float_34_12);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 66, __pyx_float_34_12)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 67, __pyx_float__0005)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_35_18);
  __Pyx_GIVEREF(__pyx_float_35_18);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 68, __pyx_float_35_18)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 69, __pyx_float__0003)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_36_30);
  __Pyx_GIVEREF(__pyx_float_36_30);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 70, __pyx_float_36_30)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 71, __pyx_float__0008)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_37_38);
  __Pyx_GIVEREF(__pyx_float_37_38);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 72, __pyx_float_37_38)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 73, __pyx_float__0003)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_38_55);
  __Pyx_GIVEREF(__pyx_float_38_55);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 74, __pyx_float_38_55)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 75, __pyx_float__0003)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_39_64);
  __Pyx_GIVEREF(__pyx_float_39_64);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 76, __pyx_float_39_64)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 77, __pyx_float__0006)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_40_77);
  __Pyx_GIVEREF(__pyx_float_40_77);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 78, __pyx_float_40_77)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 79, __pyx_float__0007)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_41_52);
  __Pyx_GIVEREF(__pyx_float_41_52);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 80, __pyx_float_41_52)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 81, __pyx_float__0006)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_41_89);
  __Pyx_GIVEREF(__pyx_float_41_89);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 82, __pyx_float_41_89)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 83, __pyx_float__0006)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_43_04);
  __Pyx_GIVEREF(__pyx_float_43_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 84, __pyx_float_43_04)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 85, __pyx_float__0011)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_43_60);
  __Pyx_GIVEREF(__pyx_float_43_60);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 86, __pyx_float_43_60)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 87, __pyx_float__0009)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_44_31);
  __Pyx_GIVEREF(__pyx_float_44_31);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 88, __pyx_float_44_31)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 89, __pyx_float__0002)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_45_68);
  __Pyx_GIVEREF(__pyx_float_45_68);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 90, __pyx_float_45_68)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 91, __pyx_float__0002)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_46_56);
  __Pyx_GIVEREF(__pyx_float_46_56);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 92, __pyx_float_46_56)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 93, __pyx_float__0003)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_47_60);
  __Pyx_GIVEREF(__pyx_float_47_60);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 94, __pyx_float_47_60)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 95, __pyx_float__0001)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_48_83);
  __Pyx_GIVEREF(__pyx_float_48_83);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 96, __pyx_float_48_83)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 97, __pyx_float__0006)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_50_01);
  __Pyx_GIVEREF(__pyx_float_50_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 98, __pyx_float_50_01)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 99, __pyx_float__0003)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_51_27);
  __Pyx_GIVEREF(__pyx_float_51_27);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 100, __pyx_float_51_27)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 101, __pyx_float__0003)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_56_04);
  __Pyx_GIVEREF(__pyx_float_56_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 102, __pyx_float_56_04)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 103, __pyx_float__0005)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_57_21);
  __Pyx_GIVEREF(__pyx_float_57_21);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 104, __pyx_float_57_21)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 105, __pyx_float__0003)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_58_56);
  __Pyx_GIVEREF(__pyx_float_58_56);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 106, __pyx_float_58_56)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 107, __pyx_float__0004)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_59_83);
  __Pyx_GIVEREF(__pyx_float_59_83);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 108, __pyx_float_59_83)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 109, __pyx_float__0003)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_61_05);
  __Pyx_GIVEREF(__pyx_float_61_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 110, __pyx_float_61_05)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 111, __pyx_float__0001)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_62_20);
  __Pyx_GIVEREF(__pyx_float_62_20);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 112, __pyx_float_62_20)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 113, __pyx_float__0001)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_67_37);
  __Pyx_GIVEREF(__pyx_float_67_37);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 114, __pyx_float_67_37)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 115, __pyx_float__0002)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_76_53);
  __Pyx_GIVEREF(__pyx_float_76_53);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 116, __pyx_float_76_53)) __PYX_ERR(0, 44, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_18, 117, __pyx_float__0001)) __PYX_ERR(0, 44, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":45
 * 			        [1.00,.0503,2.00,.0963,2.99,.1304,3.99,.0218,4.98,.0041,5.98,.0292,6.98,.0482 ,7.99,.0005,8.99,.0280,10.00,.0237,11.00,.0152,12.02,.0036,12.95,.0022,14.06,.0111,15.07,.0196 ,16.08,.0016,17.11,.0044,18.13,.0073,19.17,.0055,20.19,.0028,21.20,.0012,22.27,.0068,23.30,.0036 ,24.35,.0012,25.35,.0002,26.46,.0005,27.47,.0005,28.59,.0009,29.65,.0021,30.70,.0020,31.78,.0012 ,32.89,.0010,35.06,.0005,36.16,.0008,37.27,.0010,38.36,.0010,39.47,.0014,40.58,.0004,41.43,.0007 ,41.82,.0003,43.48,.0008,44.53,.0001,45.25,.0003,46.43,.0002,47.46,.0002,48.76,.0005,49.95,.0004 ,50.96,.0002,51.12,.0002,52.33,.0001,54.75,.0001,55.75,.0002,56.90,.0002,58.17,.0002,59.40,.0004 ,60.62,.0002,65.65,.0001,66.91,.0002,69.91,.0001,71.25,.0002],
 * 			        [1.00,.1243,1.98,.1611,3.00,.0698,3.98,.0390,5.00,.0138,5.99,.0154,7.01,.0287 ,8.01,.0014,9.01,.0049,10.00,.0144,11.01,.0055,12.05,.0052,13.01,.0011,14.05,.0118,15.07,.0154 ,16.12,.0028,17.14,.0061,18.25,.0007,19.22,.0020,20.24,.0011,21.27,.0029,22.30,.0046,23.34,.0049 ,24.35,.0004,25.45,.0003,26.47,.0007,27.59,.0008,28.16,.0009,29.12,.0002,29.81,.0006,30.81,.0009 ,31.95,.0004,33.00,.0011,34.12,.0005,35.18,.0003,36.30,.0008,37.38,.0003,38.55,.0003,39.64,.0006 ,40.77,.0007,41.52,.0006,41.89,.0006,43.04,.0011,43.60,.0009,44.31,.0002,45.68,.0002,46.56,.0003 ,47.60,.0001,48.83,.0006,50.01,.0003,51.27,.0003,56.04,.0005,57.21,.0003,58.56,.0004,59.83,.0003 ,61.05,.0001,62.20,.0001,67.37,.0002,76.53,.0001],
 * 			        [0.99,.0222,1.99,.0678,2.99,.0683,4.00,.0191,5.00,.0119,6.01,.0232,6.98,.0336 ,7.99,.0082,9.01,.0201,10.01,.0189,11.01,.0041,12.01,.0053,13.05,.0154,14.04,.0159,15.06,.0092 ,16.11,.0038,17.12,.0014,18.15,.0091,19.16,.0006,20.30,.0012,21.25,.0061,22.28,.0099,23.34,.0028 ,24.38,.0012,25.43,.0016,26.49,.0048,27.55,.0025,28.62,.0015,29.71,.0032,30.78,.0077,31.88,.0011 ,32.97,.0007,34.08,.0006,35.16,.0008,36.28,.0004,37.41,.0006,38.54,.0005,39.62,.0002,40.80,.0003 ,41.93,.0001,43.06,.0002,44.21,.0003,45.38,.0002,46.54,.0007,47.78,.0003,48.95,.0004,50.10,.0003 ,51.37,.0002,53.79,.0003,56.20,.0001,58.71,.0002,66.47,.0003],             # <<<<<<<<<<<<<<
 * 			        [1.01,.0241,1.99,.1011,2.98,.0938,3.98,.0081,4.99,.0062,5.99,.0291,6.99,.0676 ,7.59,.0004,8.98,.0127,9.99,.0112,10.99,.0142,12.00,.0029,13.02,.0071,14.02,.0184,15.03,.0064 ,16.07,.0010,17.09,.0011,18.11,.0010,19.15,.0060,20.19,.0019,21.24,.0025,22.29,.0013,23.31,.0050 ,25.41,.0030,26.50,.0018,27.53,.0006,28.63,.0012,29.66,.0013,30.77,.0020,31.84,.0006,34.04,.0001 ,35.14,.0001,36.32,.0004,37.41,.0007,38.53,.0007,39.67,.0009,40.85,.0003,45.49,.0002,46.65,.0001 ,47.81,.0004,49.01,.0002,53.91,.0002,55.14,.0002,57.69,.0002],
 * 			        [1.00,.0326,2.00,.1066,2.99,.1015,4.00,.0210,4.97,.0170,5.99,.0813,6.98,.0820 ,7.96,.0011,8.99,.0248,10.03,.0107,11.01,.0126,12.01,.0027,13.01,.0233,14.04,.0151,15.05,.0071 ,16.04,.0002,17.10,.0061,18.12,.0059,19.15,.0087,20.23,.0005,21.25,.0040,22.30,.0032,23.35,.0004 ,24.40,.0001,25.45,.0030,26.54,.0022,27.60,.0003,28.70,.0009,29.80,.0029,30.85,.0006,31.97,.0006 ,34.19,.0004,35.30,.0003,36.43,.0007,37.56,.0005,38.68,.0019,39.88,.0013,41.00,.0003,43.35,.0003 ,44.51,.0002,45.68,.0006,46.93,.0010,48.11,.0006,49.29,.0003,55.58,.0002],
 */
  __pyx_t_19 = PyList_New(104); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 45, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_19);
  __Pyx_INCREF(__pyx_float_0_99);
  __Pyx_GIVEREF(__pyx_float_0_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 0, __pyx_float_0_99)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0222);
  __Pyx_GIVEREF(__pyx_float__0222);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 1, __pyx_float__0222)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_1_99);
  __Pyx_GIVEREF(__pyx_float_1_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 2, __pyx_float_1_99)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0678);
  __Pyx_GIVEREF(__pyx_float__0678);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 3, __pyx_float__0678)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_99);
  __Pyx_GIVEREF(__pyx_float_2_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 4, __pyx_float_2_99)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0683);
  __Pyx_GIVEREF(__pyx_float__0683);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 5, __pyx_float__0683)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_00);
  __Pyx_GIVEREF(__pyx_float_4_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 6, __pyx_float_4_00)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0191);
  __Pyx_GIVEREF(__pyx_float__0191);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 7, __pyx_float__0191)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_00);
  __Pyx_GIVEREF(__pyx_float_5_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 8, __pyx_float_5_00)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0119);
  __Pyx_GIVEREF(__pyx_float__0119);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 9, __pyx_float__0119)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_01);
  __Pyx_GIVEREF(__pyx_float_6_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 10, __pyx_float_6_01)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0232);
  __Pyx_GIVEREF(__pyx_float__0232);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 11, __pyx_float__0232)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_98);
  __Pyx_GIVEREF(__pyx_float_6_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 12, __pyx_float_6_98)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0336);
  __Pyx_GIVEREF(__pyx_float__0336);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 13, __pyx_float__0336)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_99);
  __Pyx_GIVEREF(__pyx_float_7_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 14, __pyx_float_7_99)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0082);
  __Pyx_GIVEREF(__pyx_float__0082);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 15, __pyx_float__0082)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_01);
  __Pyx_GIVEREF(__pyx_float_9_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 16, __pyx_float_9_01)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0201);
  __Pyx_GIVEREF(__pyx_float__0201);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 17, __pyx_float__0201)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_01);
  __Pyx_GIVEREF(__pyx_float_10_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 18, __pyx_float_10_01)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0189);
  __Pyx_GIVEREF(__pyx_float__0189);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 19, __pyx_float__0189)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_01);
  __Pyx_GIVEREF(__pyx_float_11_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 20, __pyx_float_11_01)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0041);
  __Pyx_GIVEREF(__pyx_float__0041);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 21, __pyx_float__0041)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_01);
  __Pyx_GIVEREF(__pyx_float_12_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 22, __pyx_float_12_01)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0053);
  __Pyx_GIVEREF(__pyx_float__0053);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 23, __pyx_float__0053)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_05);
  __Pyx_GIVEREF(__pyx_float_13_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 24, __pyx_float_13_05)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0154);
  __Pyx_GIVEREF(__pyx_float__0154);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 25, __pyx_float__0154)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_04);
  __Pyx_GIVEREF(__pyx_float_14_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 26, __pyx_float_14_04)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0159);
  __Pyx_GIVEREF(__pyx_float__0159);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 27, __pyx_float__0159)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_06);
  __Pyx_GIVEREF(__pyx_float_15_06);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 28, __pyx_float_15_06)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0092);
  __Pyx_GIVEREF(__pyx_float__0092);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 29, __pyx_float__0092)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_11);
  __Pyx_GIVEREF(__pyx_float_16_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 30, __pyx_float_16_11)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0038);
  __Pyx_GIVEREF(__pyx_float__0038);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 31, __pyx_float__0038)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_12);
  __Pyx_GIVEREF(__pyx_float_17_12);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 32, __pyx_float_17_12)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0014);
  __Pyx_GIVEREF(__pyx_float__0014);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 33, __pyx_float__0014)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_15);
  __Pyx_GIVEREF(__pyx_float_18_15);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 34, __pyx_float_18_15)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0091);
  __Pyx_GIVEREF(__pyx_float__0091);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 35, __pyx_float__0091)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_16);
  __Pyx_GIVEREF(__pyx_float_19_16);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 36, __pyx_float_19_16)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 37, __pyx_float__0006)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_30);
  __Pyx_GIVEREF(__pyx_float_20_30);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 38, __pyx_float_20_30)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 39, __pyx_float__0012)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_25);
  __Pyx_GIVEREF(__pyx_float_21_25);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 40, __pyx_float_21_25)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0061);
  __Pyx_GIVEREF(__pyx_float__0061);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 41, __pyx_float__0061)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_28);
  __Pyx_GIVEREF(__pyx_float_22_28);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 42, __pyx_float_22_28)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0099);
  __Pyx_GIVEREF(__pyx_float__0099);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 43, __pyx_float__0099)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_34);
  __Pyx_GIVEREF(__pyx_float_23_34);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 44, __pyx_float_23_34)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0028);
  __Pyx_GIVEREF(__pyx_float__0028);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 45, __pyx_float__0028)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_24_38);
  __Pyx_GIVEREF(__pyx_float_24_38);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 46, __pyx_float_24_38)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 47, __pyx_float__0012)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_25_43);
  __Pyx_GIVEREF(__pyx_float_25_43);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 48, __pyx_float_25_43)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0016);
  __Pyx_GIVEREF(__pyx_float__0016);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 49, __pyx_float__0016)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_26_49);
  __Pyx_GIVEREF(__pyx_float_26_49);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 50, __pyx_float_26_49)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0048);
  __Pyx_GIVEREF(__pyx_float__0048);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 51, __pyx_float__0048)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_27_55);
  __Pyx_GIVEREF(__pyx_float_27_55);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 52, __pyx_float_27_55)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0025);
  __Pyx_GIVEREF(__pyx_float__0025);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 53, __pyx_float__0025)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_62);
  __Pyx_GIVEREF(__pyx_float_28_62);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 54, __pyx_float_28_62)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0015);
  __Pyx_GIVEREF(__pyx_float__0015);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 55, __pyx_float__0015)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_29_71);
  __Pyx_GIVEREF(__pyx_float_29_71);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 56, __pyx_float_29_71)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0032);
  __Pyx_GIVEREF(__pyx_float__0032);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 57, __pyx_float__0032)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_30_78);
  __Pyx_GIVEREF(__pyx_float_30_78);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 58, __pyx_float_30_78)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0077);
  __Pyx_GIVEREF(__pyx_float__0077);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 59, __pyx_float__0077)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_31_88);
  __Pyx_GIVEREF(__pyx_float_31_88);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 60, __pyx_float_31_88)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 61, __pyx_float__0011)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_32_97);
  __Pyx_GIVEREF(__pyx_float_32_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 62, __pyx_float_32_97)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 63, __pyx_float__0007)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_34_08);
  __Pyx_GIVEREF(__pyx_float_34_08);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 64, __pyx_float_34_08)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 65, __pyx_float__0006)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_35_16);
  __Pyx_GIVEREF(__pyx_float_35_16);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 66, __pyx_float_35_16)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 67, __pyx_float__0008)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_36_28);
  __Pyx_GIVEREF(__pyx_float_36_28);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 68, __pyx_float_36_28)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 69, __pyx_float__0004)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_37_41);
  __Pyx_GIVEREF(__pyx_float_37_41);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 70, __pyx_float_37_41)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 71, __pyx_float__0006)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_38_54);
  __Pyx_GIVEREF(__pyx_float_38_54);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 72, __pyx_float_38_54)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 73, __pyx_float__0005)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_39_62);
  __Pyx_GIVEREF(__pyx_float_39_62);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 74, __pyx_float_39_62)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 75, __pyx_float__0002)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_40_80);
  __Pyx_GIVEREF(__pyx_float_40_80);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 76, __pyx_float_40_80)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 77, __pyx_float__0003)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_41_93);
  __Pyx_GIVEREF(__pyx_float_41_93);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 78, __pyx_float_41_93)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 79, __pyx_float__0001)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_43_06);
  __Pyx_GIVEREF(__pyx_float_43_06);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 80, __pyx_float_43_06)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 81, __pyx_float__0002)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_44_21);
  __Pyx_GIVEREF(__pyx_float_44_21);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 82, __pyx_float_44_21)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 83, __pyx_float__0003)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_45_38);
  __Pyx_GIVEREF(__pyx_float_45_38);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 84, __pyx_float_45_38)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 85, __pyx_float__0002)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_46_54);
  __Pyx_GIVEREF(__pyx_float_46_54);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 86, __pyx_float_46_54)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 87, __pyx_float__0007)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_47_78);
  __Pyx_GIVEREF(__pyx_float_47_78);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 88, __pyx_float_47_78)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 89, __pyx_float__0003)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_48_95);
  __Pyx_GIVEREF(__pyx_float_48_95);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 90, __pyx_float_48_95)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 91, __pyx_float__0004)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_50_10);
  __Pyx_GIVEREF(__pyx_float_50_10);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 92, __pyx_float_50_10)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 93, __pyx_float__0003)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_51_37);
  __Pyx_GIVEREF(__pyx_float_51_37);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 94, __pyx_float_51_37)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 95, __pyx_float__0002)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_53_79);
  __Pyx_GIVEREF(__pyx_float_53_79);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 96, __pyx_float_53_79)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 97, __pyx_float__0003)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_56_20);
  __Pyx_GIVEREF(__pyx_float_56_20);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 98, __pyx_float_56_20)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 99, __pyx_float__0001)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_58_71);
  __Pyx_GIVEREF(__pyx_float_58_71);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 100, __pyx_float_58_71)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 101, __pyx_float__0002)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_66_47);
  __Pyx_GIVEREF(__pyx_float_66_47);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 102, __pyx_float_66_47)) __PYX_ERR(0, 45, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_19, 103, __pyx_float__0003)) __PYX_ERR(0, 45, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":46
 * 			        [1.00,.1243,1.98,.1611,3.00,.0698,3.98,.0390,5.00,.0138,5.99,.0154,7.01,.0287 ,8.01,.0014,9.01,.0049,10.00,.0144,11.01,.0055,12.05,.0052,13.01,.0011,14.05,.0118,15.07,.0154 ,16.12,.0028,17.14,.0061,18.25,.0007,19.22,.0020,20.24,.0011,21.27,.0029,22.30,.0046,23.34,.0049 ,24.35,.0004,25.45,.0003,26.47,.0007,27.59,.0008,28.16,.0009,29.12,.0002,29.81,.0006,30.81,.0009 ,31.95,.0004,33.00,.0011,34.12,.0005,35.18,.0003,36.30,.0008,37.38,.0003,38.55,.0003,39.64,.0006 ,40.77,.0007,41.52,.0006,41.89,.0006,43.04,.0011,43.60,.0009,44.31,.0002,45.68,.0002,46.56,.0003 ,47.60,.0001,48.83,.0006,50.01,.0003,51.27,.0003,56.04,.0005,57.21,.0003,58.56,.0004,59.83,.0003 ,61.05,.0001,62.20,.0001,67.37,.0002,76.53,.0001],
 * 			        [0.99,.0222,1.99,.0678,2.99,.0683,4.00,.0191,5.00,.0119,6.01,.0232,6.98,.0336 ,7.99,.0082,9.01,.0201,10.01,.0189,11.01,.0041,12.01,.0053,13.05,.0154,14.04,.0159,15.06,.0092 ,16.11,.0038,17.12,.0014,18.15,.0091,19.16,.0006,20.30,.0012,21.25,.0061,22.28,.0099,23.34,.0028 ,24.38,.0012,25.43,.0016,26.49,.0048,27.55,.0025,28.62,.0015,29.71,.0032,30.78,.0077,31.88,.0011 ,32.97,.0007,34.08,.0006,35.16,.0008,36.28,.0004,37.41,.0006,38.54,.0005,39.62,.0002,40.80,.0003 ,41.93,.0001,43.06,.0002,44.21,.0003,45.38,.0002,46.54,.0007,47.78,.0003,48.95,.0004,50.10,.0003 ,51.37,.0002,53.79,.0003,56.20,.0001,58.71,.0002,66.47,.0003],
 * 			        [1.01,.0241,1.99,.1011,2.98,.0938,3.98,.0081,4.99,.0062,5.99,.0291,6.99,.0676 ,7.59,.0004,8.98,.0127,9.99,.0112,10.99,.0142,12.00,.0029,13.02,.0071,14.02,.0184,15.03,.0064 ,16.07,.0010,17.09,.0011,18.11,.0010,19.15,.0060,20.19,.0019,21.24,.0025,22.29,.0013,23.31,.0050 ,25.41,.0030,26.50,.0018,27.53,.0006,28.63,.0012,29.66,.0013,30.77,.0020,31.84,.0006,34.04,.0001 ,35.14,.0001,36.32,.0004,37.41,.0007,38.53,.0007,39.67,.0009,40.85,.0003,45.49,.0002,46.65,.0001 ,47.81,.0004,49.01,.0002,53.91,.0002,55.14,.0002,57.69,.0002],             # <<<<<<<<<<<<<<
 * 			        [1.00,.0326,2.00,.1066,2.99,.1015,4.00,.0210,4.97,.0170,5.99,.0813,6.98,.0820 ,7.96,.0011,8.99,.0248,10.03,.0107,11.01,.0126,12.01,.0027,13.01,.0233,14.04,.0151,15.05,.0071 ,16.04,.0002,17.10,.0061,18.12,.0059,19.15,.0087,20.23,.0005,21.25,.0040,22.30,.0032,23.35,.0004 ,24.40,.0001,25.45,.0030,26.54,.0022,27.60,.0003,28.70,.0009,29.80,.0029,30.85,.0006,31.97,.0006 ,34.19,.0004,35.30,.0003,36.43,.0007,37.56,.0005,38.68,.0019,39.88,.0013,41.00,.0003,43.35,.0003 ,44.51,.0002,45.68,.0006,46.93,.0010,48.11,.0006,49.29,.0003,55.58,.0002],
 * 			        [0.98,.0113,1.99,.0967,3.00,.0719,3.98,.0345,4.98,.0121,6.00,.0621,7.00,.0137 ,7.98,.0006,9.01,.0314,10.01,.0171,11.02,.0060,12.03,.0024,13.05,.0077,14.07,.0040,15.12,.0032 ,16.13,.0004,17.15,.0011,18.20,.0028,19.18,.0003,20.26,.0003,21.31,.0025,22.35,.0021,23.39,.0005 ,25.55,.0002,26.62,.0014,27.70,.0003,28.78,.0005,29.90,.0030,31.01,.0011,32.12,.0005,34.31,.0001 ,35.50,.0002,36.62,.0002,37.76,.0005,38.85,.0002,40.09,.0004,43.60,.0001,44.73,.0002,46.02,.0002 ,47.25,.0004,48.44,.0004],
 */
  __pyx_t_20 = PyList_New(88); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 46, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_20);
  __Pyx_INCREF(__pyx_float_1_01);
  __Pyx_GIVEREF(__pyx_float_1_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 0, __pyx_float_1_01)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0241);
  __Pyx_GIVEREF(__pyx_float__0241);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 1, __pyx_float__0241)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_1_99);
  __Pyx_GIVEREF(__pyx_float_1_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 2, __pyx_float_1_99)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1011);
  __Pyx_GIVEREF(__pyx_float__1011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 3, __pyx_float__1011)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_98);
  __Pyx_GIVEREF(__pyx_float_2_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 4, __pyx_float_2_98)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0938);
  __Pyx_GIVEREF(__pyx_float__0938);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 5, __pyx_float__0938)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_98);
  __Pyx_GIVEREF(__pyx_float_3_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 6, __pyx_float_3_98)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0081);
  __Pyx_GIVEREF(__pyx_float__0081);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 7, __pyx_float__0081)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_99);
  __Pyx_GIVEREF(__pyx_float_4_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 8, __pyx_float_4_99)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0062);
  __Pyx_GIVEREF(__pyx_float__0062);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 9, __pyx_float__0062)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_99);
  __Pyx_GIVEREF(__pyx_float_5_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 10, __pyx_float_5_99)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0291);
  __Pyx_GIVEREF(__pyx_float__0291);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 11, __pyx_float__0291)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_99);
  __Pyx_GIVEREF(__pyx_float_6_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 12, __pyx_float_6_99)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0676);
  __Pyx_GIVEREF(__pyx_float__0676);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 13, __pyx_float__0676)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_59);
  __Pyx_GIVEREF(__pyx_float_7_59);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 14, __pyx_float_7_59)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 15, __pyx_float__0004)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_98);
  __Pyx_GIVEREF(__pyx_float_8_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 16, __pyx_float_8_98)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0127);
  __Pyx_GIVEREF(__pyx_float__0127);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 17, __pyx_float__0127)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_99);
  __Pyx_GIVEREF(__pyx_float_9_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 18, __pyx_float_9_99)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0112);
  __Pyx_GIVEREF(__pyx_float__0112);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 19, __pyx_float__0112)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_99);
  __Pyx_GIVEREF(__pyx_float_10_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 20, __pyx_float_10_99)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0142);
  __Pyx_GIVEREF(__pyx_float__0142);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 21, __pyx_float__0142)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_00);
  __Pyx_GIVEREF(__pyx_float_12_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 22, __pyx_float_12_00)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0029);
  __Pyx_GIVEREF(__pyx_float__0029);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 23, __pyx_float__0029)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_02);
  __Pyx_GIVEREF(__pyx_float_13_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 24, __pyx_float_13_02)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0071);
  __Pyx_GIVEREF(__pyx_float__0071);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 25, __pyx_float__0071)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_02);
  __Pyx_GIVEREF(__pyx_float_14_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 26, __pyx_float_14_02)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0184);
  __Pyx_GIVEREF(__pyx_float__0184);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 27, __pyx_float__0184)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_03);
  __Pyx_GIVEREF(__pyx_float_15_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 28, __pyx_float_15_03)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0064);
  __Pyx_GIVEREF(__pyx_float__0064);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 29, __pyx_float__0064)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_07);
  __Pyx_GIVEREF(__pyx_float_16_07);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 30, __pyx_float_16_07)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 31, __pyx_float__0010)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_09);
  __Pyx_GIVEREF(__pyx_float_17_09);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 32, __pyx_float_17_09)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 33, __pyx_float__0011)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_11);
  __Pyx_GIVEREF(__pyx_float_18_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 34, __pyx_float_18_11)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 35, __pyx_float__0010)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_15);
  __Pyx_GIVEREF(__pyx_float_19_15);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 36, __pyx_float_19_15)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0060);
  __Pyx_GIVEREF(__pyx_float__0060);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 37, __pyx_float__0060)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_19);
  __Pyx_GIVEREF(__pyx_float_20_19);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 38, __pyx_float_20_19)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0019);
  __Pyx_GIVEREF(__pyx_float__0019);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 39, __pyx_float__0019)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_24);
  __Pyx_GIVEREF(__pyx_float_21_24);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 40, __pyx_float_21_24)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0025);
  __Pyx_GIVEREF(__pyx_float__0025);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 41, __pyx_float__0025)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_29);
  __Pyx_GIVEREF(__pyx_float_22_29);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 42, __pyx_float_22_29)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0013);
  __Pyx_GIVEREF(__pyx_float__0013);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 43, __pyx_float__0013)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_31);
  __Pyx_GIVEREF(__pyx_float_23_31);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 44, __pyx_float_23_31)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0050);
  __Pyx_GIVEREF(__pyx_float__0050);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 45, __pyx_float__0050)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_25_41);
  __Pyx_GIVEREF(__pyx_float_25_41);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 46, __pyx_float_25_41)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0030);
  __Pyx_GIVEREF(__pyx_float__0030);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 47, __pyx_float__0030)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_26_50);
  __Pyx_GIVEREF(__pyx_float_26_50);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 48, __pyx_float_26_50)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0018);
  __Pyx_GIVEREF(__pyx_float__0018);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 49, __pyx_float__0018)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_27_53);
  __Pyx_GIVEREF(__pyx_float_27_53);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 50, __pyx_float_27_53)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 51, __pyx_float__0006)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_63);
  __Pyx_GIVEREF(__pyx_float_28_63);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 52, __pyx_float_28_63)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 53, __pyx_float__0012)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_29_66);
  __Pyx_GIVEREF(__pyx_float_29_66);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 54, __pyx_float_29_66)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0013);
  __Pyx_GIVEREF(__pyx_float__0013);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 55, __pyx_float__0013)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_30_77);
  __Pyx_GIVEREF(__pyx_float_30_77);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 56, __pyx_float_30_77)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0020);
  __Pyx_GIVEREF(__pyx_float__0020);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 57, __pyx_float__0020)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_31_84);
  __Pyx_GIVEREF(__pyx_float_31_84);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 58, __pyx_float_31_84)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 59, __pyx_float__0006)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_34_04);
  __Pyx_GIVEREF(__pyx_float_34_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 60, __pyx_float_34_04)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 61, __pyx_float__0001)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_35_14);
  __Pyx_GIVEREF(__pyx_float_35_14);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 62, __pyx_float_35_14)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 63, __pyx_float__0001)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_36_32);
  __Pyx_GIVEREF(__pyx_float_36_32);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 64, __pyx_float_36_32)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 65, __pyx_float__0004)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_37_41);
  __Pyx_GIVEREF(__pyx_float_37_41);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 66, __pyx_float_37_41)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 67, __pyx_float__0007)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_38_53);
  __Pyx_GIVEREF(__pyx_float_38_53);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 68, __pyx_float_38_53)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 69, __pyx_float__0007)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_39_67);
  __Pyx_GIVEREF(__pyx_float_39_67);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 70, __pyx_float_39_67)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 71, __pyx_float__0009)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_40_85);
  __Pyx_GIVEREF(__pyx_float_40_85);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 72, __pyx_float_40_85)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 73, __pyx_float__0003)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_45_49);
  __Pyx_GIVEREF(__pyx_float_45_49);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 74, __pyx_float_45_49)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 75, __pyx_float__0002)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_46_65);
  __Pyx_GIVEREF(__pyx_float_46_65);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 76, __pyx_float_46_65)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 77, __pyx_float__0001)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_47_81);
  __Pyx_GIVEREF(__pyx_float_47_81);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 78, __pyx_float_47_81)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 79, __pyx_float__0004)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_49_01);
  __Pyx_GIVEREF(__pyx_float_49_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 80, __pyx_float_49_01)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 81, __pyx_float__0002)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_53_91);
  __Pyx_GIVEREF(__pyx_float_53_91);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 82, __pyx_float_53_91)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 83, __pyx_float__0002)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_55_14);
  __Pyx_GIVEREF(__pyx_float_55_14);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 84, __pyx_float_55_14)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 85, __pyx_float__0002)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_57_69);
  __Pyx_GIVEREF(__pyx_float_57_69);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 86, __pyx_float_57_69)) __PYX_ERR(0, 46, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 87, __pyx_float__0002)) __PYX_ERR(0, 46, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":47
 * 			        [0.99,.0222,1.99,.0678,2.99,.0683,4.00,.0191,5.00,.0119,6.01,.0232,6.98,.0336 ,7.99,.0082,9.01,.0201,10.01,.0189,11.01,.0041,12.01,.0053,13.05,.0154,14.04,.0159,15.06,.0092 ,16.11,.0038,17.12,.0014,18.15,.0091,19.16,.0006,20.30,.0012,21.25,.0061,22.28,.0099,23.34,.0028 ,24.38,.0012,25.43,.0016,26.49,.0048,27.55,.0025,28.62,.0015,29.71,.0032,30.78,.0077,31.88,.0011 ,32.97,.0007,34.08,.0006,35.16,.0008,36.28,.0004,37.41,.0006,38.54,.0005,39.62,.0002,40.80,.0003 ,41.93,.0001,43.06,.0002,44.21,.0003,45.38,.0002,46.54,.0007,47.78,.0003,48.95,.0004,50.10,.0003 ,51.37,.0002,53.79,.0003,56.20,.0001,58.71,.0002,66.47,.0003],
 * 			        [1.01,.0241,1.99,.1011,2.98,.0938,3.98,.0081,4.99,.0062,5.99,.0291,6.99,.0676 ,7.59,.0004,8.98,.0127,9.99,.0112,10.99,.0142,12.00,.0029,13.02,.0071,14.02,.0184,15.03,.0064 ,16.07,.0010,17.09,.0011,18.11,.0010,19.15,.0060,20.19,.0019,21.24,.0025,22.29,.0013,23.31,.0050 ,25.41,.0030,26.50,.0018,27.53,.0006,28.63,.0012,29.66,.0013,30.77,.0020,31.84,.0006,34.04,.0001 ,35.14,.0001,36.32,.0004,37.41,.0007,38.53,.0007,39.67,.0009,40.85,.0003,45.49,.0002,46.65,.0001 ,47.81,.0004,49.01,.0002,53.91,.0002,55.14,.0002,57.69,.0002],
 * 			        [1.00,.0326,2.00,.1066,2.99,.1015,4.00,.0210,4.97,.0170,5.99,.0813,6.98,.0820 ,7.96,.0011,8.99,.0248,10.03,.0107,11.01,.0126,12.01,.0027,13.01,.0233,14.04,.0151,15.05,.0071 ,16.04,.0002,17.10,.0061,18.12,.0059,19.15,.0087,20.23,.0005,21.25,.0040,22.30,.0032,23.35,.0004 ,24.40,.0001,25.45,.0030,26.54,.0022,27.60,.0003,28.70,.0009,29.80,.0029,30.85,.0006,31.97,.0006 ,34.19,.0004,35.30,.0003,36.43,.0007,37.56,.0005,38.68,.0019,39.88,.0013,41.00,.0003,43.35,.0003 ,44.51,.0002,45.68,.0006,46.93,.0010,48.11,.0006,49.29,.0003,55.58,.0002],             # <<<<<<<<<<<<<<
 * 			        [0.98,.0113,1.99,.0967,3.00,.0719,3.98,.0345,4.98,.0121,6.00,.0621,7.00,.0137 ,7.98,.0006,9.01,.0314,10.01,.0171,11.02,.0060,12.03,.0024,13.05,.0077,14.07,.0040,15.12,.0032 ,16.13,.0004,17.15,.0011,18.20,.0028,19.18,.0003,20.26,.0003,21.31,.0025,22.35,.0021,23.39,.0005 ,25.55,.0002,26.62,.0014,27.70,.0003,28.78,.0005,29.90,.0030,31.01,.0011,32.12,.0005,34.31,.0001 ,35.50,.0002,36.62,.0002,37.76,.0005,38.85,.0002,40.09,.0004,43.60,.0001,44.73,.0002,46.02,.0002 ,47.25,.0004,48.44,.0004],
 * 			        [0.99,.0156,1.98,.0846,2.98,.0178,3.98,.0367,4.98,.0448,5.98,.0113,6.99,.0189 ,8.00,.0011,9.01,.0247,10.02,.0089,11.01,.0184,12.03,.0105,13.00,.0039,14.07,.0116,15.09,.0078 ,16.13,.0008,17.14,.0064,18.19,.0029,19.22,.0028,20.25,.0017,21.32,.0043,22.37,.0055,23.42,.0034 ,24.48,.0004,25.54,.0002,26.61,.0017,27.70,.0011,28.80,.0002,29.89,.0019,30.97,.0028,32.09,.0007 ,34.30,.0002,35.44,.0003,36.55,.0001,37.69,.0004,38.93,.0002,40.05,.0005,41.20,.0005,42.37,.0002 ,43.54,.0003,44.73,.0001,45.95,.0002,47.16,.0001,48.43,.0005,49.65,.0004,55.90,.0002,59.81,.0004],
 */
  __pyx_t_21 = PyList_New(90); if (unlikely(!__pyx_t_21)) __PYX_ERR(0, 47, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_21);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 0, __pyx_float_1_00)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0326);
  __Pyx_GIVEREF(__pyx_float__0326);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 1, __pyx_float__0326)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_00);
  __Pyx_GIVEREF(__pyx_float_2_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 2, __pyx_float_2_00)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1066);
  __Pyx_GIVEREF(__pyx_float__1066);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 3, __pyx_float__1066)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_99);
  __Pyx_GIVEREF(__pyx_float_2_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 4, __pyx_float_2_99)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1015);
  __Pyx_GIVEREF(__pyx_float__1015);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 5, __pyx_float__1015)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_00);
  __Pyx_GIVEREF(__pyx_float_4_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 6, __pyx_float_4_00)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0210);
  __Pyx_GIVEREF(__pyx_float__0210);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 7, __pyx_float__0210)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_97);
  __Pyx_GIVEREF(__pyx_float_4_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 8, __pyx_float_4_97)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0170);
  __Pyx_GIVEREF(__pyx_float__0170);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 9, __pyx_float__0170)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_99);
  __Pyx_GIVEREF(__pyx_float_5_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 10, __pyx_float_5_99)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0813);
  __Pyx_GIVEREF(__pyx_float__0813);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 11, __pyx_float__0813)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_98);
  __Pyx_GIVEREF(__pyx_float_6_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 12, __pyx_float_6_98)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0820);
  __Pyx_GIVEREF(__pyx_float__0820);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 13, __pyx_float__0820)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_96);
  __Pyx_GIVEREF(__pyx_float_7_96);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 14, __pyx_float_7_96)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 15, __pyx_float__0011)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_99);
  __Pyx_GIVEREF(__pyx_float_8_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 16, __pyx_float_8_99)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0248);
  __Pyx_GIVEREF(__pyx_float__0248);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 17, __pyx_float__0248)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_03);
  __Pyx_GIVEREF(__pyx_float_10_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 18, __pyx_float_10_03)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0107);
  __Pyx_GIVEREF(__pyx_float__0107);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 19, __pyx_float__0107)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_01);
  __Pyx_GIVEREF(__pyx_float_11_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 20, __pyx_float_11_01)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0126);
  __Pyx_GIVEREF(__pyx_float__0126);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 21, __pyx_float__0126)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_01);
  __Pyx_GIVEREF(__pyx_float_12_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 22, __pyx_float_12_01)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0027);
  __Pyx_GIVEREF(__pyx_float__0027);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 23, __pyx_float__0027)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_01);
  __Pyx_GIVEREF(__pyx_float_13_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 24, __pyx_float_13_01)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0233);
  __Pyx_GIVEREF(__pyx_float__0233);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 25, __pyx_float__0233)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_04);
  __Pyx_GIVEREF(__pyx_float_14_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 26, __pyx_float_14_04)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0151);
  __Pyx_GIVEREF(__pyx_float__0151);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 27, __pyx_float__0151)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_05);
  __Pyx_GIVEREF(__pyx_float_15_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 28, __pyx_float_15_05)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0071);
  __Pyx_GIVEREF(__pyx_float__0071);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 29, __pyx_float__0071)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_04);
  __Pyx_GIVEREF(__pyx_float_16_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 30, __pyx_float_16_04)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 31, __pyx_float__0002)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_10);
  __Pyx_GIVEREF(__pyx_float_17_10);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 32, __pyx_float_17_10)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0061);
  __Pyx_GIVEREF(__pyx_float__0061);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 33, __pyx_float__0061)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_12);
  __Pyx_GIVEREF(__pyx_float_18_12);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 34, __pyx_float_18_12)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0059);
  __Pyx_GIVEREF(__pyx_float__0059);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 35, __pyx_float__0059)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_15);
  __Pyx_GIVEREF(__pyx_float_19_15);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 36, __pyx_float_19_15)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0087);
  __Pyx_GIVEREF(__pyx_float__0087);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 37, __pyx_float__0087)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_23);
  __Pyx_GIVEREF(__pyx_float_20_23);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 38, __pyx_float_20_23)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 39, __pyx_float__0005)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_25);
  __Pyx_GIVEREF(__pyx_float_21_25);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 40, __pyx_float_21_25)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0040);
  __Pyx_GIVEREF(__pyx_float__0040);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 41, __pyx_float__0040)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_30);
  __Pyx_GIVEREF(__pyx_float_22_30);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 42, __pyx_float_22_30)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0032);
  __Pyx_GIVEREF(__pyx_float__0032);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 43, __pyx_float__0032)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_35);
  __Pyx_GIVEREF(__pyx_float_23_35);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 44, __pyx_float_23_35)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 45, __pyx_float__0004)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_24_40);
  __Pyx_GIVEREF(__pyx_float_24_40);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 46, __pyx_float_24_40)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 47, __pyx_float__0001)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_25_45);
  __Pyx_GIVEREF(__pyx_float_25_45);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 48, __pyx_float_25_45)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0030);
  __Pyx_GIVEREF(__pyx_float__0030);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 49, __pyx_float__0030)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_26_54);
  __Pyx_GIVEREF(__pyx_float_26_54);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 50, __pyx_float_26_54)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0022);
  __Pyx_GIVEREF(__pyx_float__0022);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 51, __pyx_float__0022)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_27_60);
  __Pyx_GIVEREF(__pyx_float_27_60);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 52, __pyx_float_27_60)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 53, __pyx_float__0003)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_70);
  __Pyx_GIVEREF(__pyx_float_28_70);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 54, __pyx_float_28_70)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 55, __pyx_float__0009)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_29_80);
  __Pyx_GIVEREF(__pyx_float_29_80);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 56, __pyx_float_29_80)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0029);
  __Pyx_GIVEREF(__pyx_float__0029);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 57, __pyx_float__0029)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_30_85);
  __Pyx_GIVEREF(__pyx_float_30_85);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 58, __pyx_float_30_85)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 59, __pyx_float__0006)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_31_97);
  __Pyx_GIVEREF(__pyx_float_31_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 60, __pyx_float_31_97)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 61, __pyx_float__0006)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_34_19);
  __Pyx_GIVEREF(__pyx_float_34_19);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 62, __pyx_float_34_19)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 63, __pyx_float__0004)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_35_30);
  __Pyx_GIVEREF(__pyx_float_35_30);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 64, __pyx_float_35_30)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 65, __pyx_float__0003)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_36_43);
  __Pyx_GIVEREF(__pyx_float_36_43);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 66, __pyx_float_36_43)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 67, __pyx_float__0007)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_37_56);
  __Pyx_GIVEREF(__pyx_float_37_56);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 68, __pyx_float_37_56)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 69, __pyx_float__0005)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_38_68);
  __Pyx_GIVEREF(__pyx_float_38_68);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 70, __pyx_float_38_68)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0019);
  __Pyx_GIVEREF(__pyx_float__0019);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 71, __pyx_float__0019)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_39_88);
  __Pyx_GIVEREF(__pyx_float_39_88);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 72, __pyx_float_39_88)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0013);
  __Pyx_GIVEREF(__pyx_float__0013);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 73, __pyx_float__0013)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_41_00);
  __Pyx_GIVEREF(__pyx_float_41_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 74, __pyx_float_41_00)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 75, __pyx_float__0003)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_43_35);
  __Pyx_GIVEREF(__pyx_float_43_35);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 76, __pyx_float_43_35)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 77, __pyx_float__0003)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_44_51);
  __Pyx_GIVEREF(__pyx_float_44_51);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 78, __pyx_float_44_51)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 79, __pyx_float__0002)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_45_68);
  __Pyx_GIVEREF(__pyx_float_45_68);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 80, __pyx_float_45_68)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 81, __pyx_float__0006)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_46_93);
  __Pyx_GIVEREF(__pyx_float_46_93);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 82, __pyx_float_46_93)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 83, __pyx_float__0010)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_48_11);
  __Pyx_GIVEREF(__pyx_float_48_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 84, __pyx_float_48_11)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 85, __pyx_float__0006)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_49_29);
  __Pyx_GIVEREF(__pyx_float_49_29);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 86, __pyx_float_49_29)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 87, __pyx_float__0003)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_55_58);
  __Pyx_GIVEREF(__pyx_float_55_58);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 88, __pyx_float_55_58)) __PYX_ERR(0, 47, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 89, __pyx_float__0002)) __PYX_ERR(0, 47, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":48
 * 			        [1.01,.0241,1.99,.1011,2.98,.0938,3.98,.0081,4.99,.0062,5.99,.0291,6.99,.0676 ,7.59,.0004,8.98,.0127,9.99,.0112,10.99,.0142,12.00,.0029,13.02,.0071,14.02,.0184,15.03,.0064 ,16.07,.0010,17.09,.0011,18.11,.0010,19.15,.0060,20.19,.0019,21.24,.0025,22.29,.0013,23.31,.0050 ,25.41,.0030,26.50,.0018,27.53,.0006,28.63,.0012,29.66,.0013,30.77,.0020,31.84,.0006,34.04,.0001 ,35.14,.0001,36.32,.0004,37.41,.0007,38.53,.0007,39.67,.0009,40.85,.0003,45.49,.0002,46.65,.0001 ,47.81,.0004,49.01,.0002,53.91,.0002,55.14,.0002,57.69,.0002],
 * 			        [1.00,.0326,2.00,.1066,2.99,.1015,4.00,.0210,4.97,.0170,5.99,.0813,6.98,.0820 ,7.96,.0011,8.99,.0248,10.03,.0107,11.01,.0126,12.01,.0027,13.01,.0233,14.04,.0151,15.05,.0071 ,16.04,.0002,17.10,.0061,18.12,.0059,19.15,.0087,20.23,.0005,21.25,.0040,22.30,.0032,23.35,.0004 ,24.40,.0001,25.45,.0030,26.54,.0022,27.60,.0003,28.70,.0009,29.80,.0029,30.85,.0006,31.97,.0006 ,34.19,.0004,35.30,.0003,36.43,.0007,37.56,.0005,38.68,.0019,39.88,.0013,41.00,.0003,43.35,.0003 ,44.51,.0002,45.68,.0006,46.93,.0010,48.11,.0006,49.29,.0003,55.58,.0002],
 * 			        [0.98,.0113,1.99,.0967,3.00,.0719,3.98,.0345,4.98,.0121,6.00,.0621,7.00,.0137 ,7.98,.0006,9.01,.0314,10.01,.0171,11.02,.0060,12.03,.0024,13.05,.0077,14.07,.0040,15.12,.0032 ,16.13,.0004,17.15,.0011,18.20,.0028,19.18,.0003,20.26,.0003,21.31,.0025,22.35,.0021,23.39,.0005 ,25.55,.0002,26.62,.0014,27.70,.0003,28.78,.0005,29.90,.0030,31.01,.0011,32.12,.0005,34.31,.0001 ,35.50,.0002,36.62,.0002,37.76,.0005,38.85,.0002,40.09,.0004,43.60,.0001,44.73,.0002,46.02,.0002 ,47.25,.0004,48.44,.0004],             # <<<<<<<<<<<<<<
 * 			        [0.99,.0156,1.98,.0846,2.98,.0178,3.98,.0367,4.98,.0448,5.98,.0113,6.99,.0189 ,8.00,.0011,9.01,.0247,10.02,.0089,11.01,.0184,12.03,.0105,13.00,.0039,14.07,.0116,15.09,.0078 ,16.13,.0008,17.14,.0064,18.19,.0029,19.22,.0028,20.25,.0017,21.32,.0043,22.37,.0055,23.42,.0034 ,24.48,.0004,25.54,.0002,26.61,.0017,27.70,.0011,28.80,.0002,29.89,.0019,30.97,.0028,32.09,.0007 ,34.30,.0002,35.44,.0003,36.55,.0001,37.69,.0004,38.93,.0002,40.05,.0005,41.20,.0005,42.37,.0002 ,43.54,.0003,44.73,.0001,45.95,.0002,47.16,.0001,48.43,.0005,49.65,.0004,55.90,.0002,59.81,.0004],
 * 			        [1.01,.0280,2.00,.0708,2.99,.0182,3.99,.0248,4.98,.0245,5.98,.0279,6.98,.0437 ,7.99,.0065,8.99,.0299,10.00,.0073,10.99,.0011,12.03,.0122,13.03,.0028,14.08,.0044,15.11,.0097 ,16.15,.0010,17.17,.0025,18.19,.0017,19.24,.0008,20.28,.0040,21.32,.0024,22.38,.0008,23.46,.0032 ,24.52,.0010,25.59,.0008,26.68,.0009,27.76,.0012,28.88,.0003,29.95,.0005,31.05,.0017,32.14,.0002 ,33.29,.0003,37.88,.0002,39.03,.0002,40.19,.0004,41.37,.0003,43.74,.0002,46.20,.0001,48.68,.0001 ,49.93,.0001,51.19,.0002],
 */
  __pyx_t_22 = PyList_New(82); if (unlikely(!__pyx_t_22)) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_22);
  __Pyx_INCREF(__pyx_float_0_98);
  __Pyx_GIVEREF(__pyx_float_0_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 0, __pyx_float_0_98)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0113);
  __Pyx_GIVEREF(__pyx_float__0113);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 1, __pyx_float__0113)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_1_99);
  __Pyx_GIVEREF(__pyx_float_1_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 2, __pyx_float_1_99)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0967);
  __Pyx_GIVEREF(__pyx_float__0967);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 3, __pyx_float__0967)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_00);
  __Pyx_GIVEREF(__pyx_float_3_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 4, __pyx_float_3_00)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0719);
  __Pyx_GIVEREF(__pyx_float__0719);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 5, __pyx_float__0719)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_98);
  __Pyx_GIVEREF(__pyx_float_3_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 6, __pyx_float_3_98)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0345);
  __Pyx_GIVEREF(__pyx_float__0345);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 7, __pyx_float__0345)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_98);
  __Pyx_GIVEREF(__pyx_float_4_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 8, __pyx_float_4_98)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0121);
  __Pyx_GIVEREF(__pyx_float__0121);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 9, __pyx_float__0121)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_00);
  __Pyx_GIVEREF(__pyx_float_6_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 10, __pyx_float_6_00)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0621);
  __Pyx_GIVEREF(__pyx_float__0621);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 11, __pyx_float__0621)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_00);
  __Pyx_GIVEREF(__pyx_float_7_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 12, __pyx_float_7_00)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0137);
  __Pyx_GIVEREF(__pyx_float__0137);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 13, __pyx_float__0137)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_98);
  __Pyx_GIVEREF(__pyx_float_7_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 14, __pyx_float_7_98)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 15, __pyx_float__0006)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_01);
  __Pyx_GIVEREF(__pyx_float_9_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 16, __pyx_float_9_01)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0314);
  __Pyx_GIVEREF(__pyx_float__0314);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 17, __pyx_float__0314)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_01);
  __Pyx_GIVEREF(__pyx_float_10_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 18, __pyx_float_10_01)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0171);
  __Pyx_GIVEREF(__pyx_float__0171);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 19, __pyx_float__0171)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_02);
  __Pyx_GIVEREF(__pyx_float_11_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 20, __pyx_float_11_02)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0060);
  __Pyx_GIVEREF(__pyx_float__0060);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 21, __pyx_float__0060)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_03);
  __Pyx_GIVEREF(__pyx_float_12_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 22, __pyx_float_12_03)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0024);
  __Pyx_GIVEREF(__pyx_float__0024);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 23, __pyx_float__0024)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_05);
  __Pyx_GIVEREF(__pyx_float_13_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 24, __pyx_float_13_05)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0077);
  __Pyx_GIVEREF(__pyx_float__0077);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 25, __pyx_float__0077)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_07);
  __Pyx_GIVEREF(__pyx_float_14_07);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 26, __pyx_float_14_07)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0040);
  __Pyx_GIVEREF(__pyx_float__0040);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 27, __pyx_float__0040)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_12);
  __Pyx_GIVEREF(__pyx_float_15_12);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 28, __pyx_float_15_12)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0032);
  __Pyx_GIVEREF(__pyx_float__0032);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 29, __pyx_float__0032)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_13);
  __Pyx_GIVEREF(__pyx_float_16_13);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 30, __pyx_float_16_13)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 31, __pyx_float__0004)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_15);
  __Pyx_GIVEREF(__pyx_float_17_15);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 32, __pyx_float_17_15)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 33, __pyx_float__0011)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_20);
  __Pyx_GIVEREF(__pyx_float_18_20);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 34, __pyx_float_18_20)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0028);
  __Pyx_GIVEREF(__pyx_float__0028);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 35, __pyx_float__0028)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_18);
  __Pyx_GIVEREF(__pyx_float_19_18);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 36, __pyx_float_19_18)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 37, __pyx_float__0003)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_26);
  __Pyx_GIVEREF(__pyx_float_20_26);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 38, __pyx_float_20_26)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 39, __pyx_float__0003)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_31);
  __Pyx_GIVEREF(__pyx_float_21_31);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 40, __pyx_float_21_31)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0025);
  __Pyx_GIVEREF(__pyx_float__0025);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 41, __pyx_float__0025)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_35);
  __Pyx_GIVEREF(__pyx_float_22_35);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 42, __pyx_float_22_35)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0021);
  __Pyx_GIVEREF(__pyx_float__0021);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 43, __pyx_float__0021)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_39);
  __Pyx_GIVEREF(__pyx_float_23_39);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 44, __pyx_float_23_39)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 45, __pyx_float__0005)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_25_55);
  __Pyx_GIVEREF(__pyx_float_25_55);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 46, __pyx_float_25_55)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 47, __pyx_float__0002)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_26_62);
  __Pyx_GIVEREF(__pyx_float_26_62);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 48, __pyx_float_26_62)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0014);
  __Pyx_GIVEREF(__pyx_float__0014);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 49, __pyx_float__0014)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_27_70);
  __Pyx_GIVEREF(__pyx_float_27_70);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 50, __pyx_float_27_70)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 51, __pyx_float__0003)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_78);
  __Pyx_GIVEREF(__pyx_float_28_78);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 52, __pyx_float_28_78)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 53, __pyx_float__0005)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_29_90);
  __Pyx_GIVEREF(__pyx_float_29_90);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 54, __pyx_float_29_90)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0030);
  __Pyx_GIVEREF(__pyx_float__0030);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 55, __pyx_float__0030)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_31_01);
  __Pyx_GIVEREF(__pyx_float_31_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 56, __pyx_float_31_01)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 57, __pyx_float__0011)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_32_12);
  __Pyx_GIVEREF(__pyx_float_32_12);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 58, __pyx_float_32_12)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 59, __pyx_float__0005)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_34_31);
  __Pyx_GIVEREF(__pyx_float_34_31);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 60, __pyx_float_34_31)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 61, __pyx_float__0001)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_35_50);
  __Pyx_GIVEREF(__pyx_float_35_50);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 62, __pyx_float_35_50)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 63, __pyx_float__0002)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_36_62);
  __Pyx_GIVEREF(__pyx_float_36_62);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 64, __pyx_float_36_62)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 65, __pyx_float__0002)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_37_76);
  __Pyx_GIVEREF(__pyx_float_37_76);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 66, __pyx_float_37_76)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 67, __pyx_float__0005)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_38_85);
  __Pyx_GIVEREF(__pyx_float_38_85);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 68, __pyx_float_38_85)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 69, __pyx_float__0002)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_40_09);
  __Pyx_GIVEREF(__pyx_float_40_09);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 70, __pyx_float_40_09)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 71, __pyx_float__0004)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_43_60);
  __Pyx_GIVEREF(__pyx_float_43_60);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 72, __pyx_float_43_60)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 73, __pyx_float__0001)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_44_73);
  __Pyx_GIVEREF(__pyx_float_44_73);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 74, __pyx_float_44_73)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 75, __pyx_float__0002)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_46_02);
  __Pyx_GIVEREF(__pyx_float_46_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 76, __pyx_float_46_02)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 77, __pyx_float__0002)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_47_25);
  __Pyx_GIVEREF(__pyx_float_47_25);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 78, __pyx_float_47_25)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 79, __pyx_float__0004)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_48_44);
  __Pyx_GIVEREF(__pyx_float_48_44);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 80, __pyx_float_48_44)) __PYX_ERR(0, 48, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_22, 81, __pyx_float__0004)) __PYX_ERR(0, 48, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":49
 * 			        [1.00,.0326,2.00,.1066,2.99,.1015,4.00,.0210,4.97,.0170,5.99,.0813,6.98,.0820 ,7.96,.0011,8.99,.0248,10.03,.0107,11.01,.0126,12.01,.0027,13.01,.0233,14.04,.0151,15.05,.0071 ,16.04,.0002,17.10,.0061,18.12,.0059,19.15,.0087,20.23,.0005,21.25,.0040,22.30,.0032,23.35,.0004 ,24.40,.0001,25.45,.0030,26.54,.0022,27.60,.0003,28.70,.0009,29.80,.0029,30.85,.0006,31.97,.0006 ,34.19,.0004,35.30,.0003,36.43,.0007,37.56,.0005,38.68,.0019,39.88,.0013,41.00,.0003,43.35,.0003 ,44.51,.0002,45.68,.0006,46.93,.0010,48.11,.0006,49.29,.0003,55.58,.0002],
 * 			        [0.98,.0113,1.99,.0967,3.00,.0719,3.98,.0345,4.98,.0121,6.00,.0621,7.00,.0137 ,7.98,.0006,9.01,.0314,10.01,.0171,11.02,.0060,12.03,.0024,13.05,.0077,14.07,.0040,15.12,.0032 ,16.13,.0004,17.15,.0011,18.20,.0028,19.18,.0003,20.26,.0003,21.31,.0025,22.35,.0021,23.39,.0005 ,25.55,.0002,26.62,.0014,27.70,.0003,28.78,.0005,29.90,.0030,31.01,.0011,32.12,.0005,34.31,.0001 ,35.50,.0002,36.62,.0002,37.76,.0005,38.85,.0002,40.09,.0004,43.60,.0001,44.73,.0002,46.02,.0002 ,47.25,.0004,48.44,.0004],
 * 			        [0.99,.0156,1.98,.0846,2.98,.0178,3.98,.0367,4.98,.0448,5.98,.0113,6.99,.0189 ,8.00,.0011,9.01,.0247,10.02,.0089,11.01,.0184,12.03,.0105,13.00,.0039,14.07,.0116,15.09,.0078 ,16.13,.0008,17.14,.0064,18.19,.0029,19.22,.0028,20.25,.0017,21.32,.0043,22.37,.0055,23.42,.0034 ,24.48,.0004,25.54,.0002,26.61,.0017,27.70,.0011,28.80,.0002,29.89,.0019,30.97,.0028,32.09,.0007 ,34.30,.0002,35.44,.0003,36.55,.0001,37.69,.0004,38.93,.0002,40.05,.0005,41.20,.0005,42.37,.0002 ,43.54,.0003,44.73,.0001,45.95,.0002,47.16,.0001,48.43,.0005,49.65,.0004,55.90,.0002,59.81,.0004],             # <<<<<<<<<<<<<<
 * 			        [1.01,.0280,2.00,.0708,2.99,.0182,3.99,.0248,4.98,.0245,5.98,.0279,6.98,.0437 ,7.99,.0065,8.99,.0299,10.00,.0073,10.99,.0011,12.03,.0122,13.03,.0028,14.08,.0044,15.11,.0097 ,16.15,.0010,17.17,.0025,18.19,.0017,19.24,.0008,20.28,.0040,21.32,.0024,22.38,.0008,23.46,.0032 ,24.52,.0010,25.59,.0008,26.68,.0009,27.76,.0012,28.88,.0003,29.95,.0005,31.05,.0017,32.14,.0002 ,33.29,.0003,37.88,.0002,39.03,.0002,40.19,.0004,41.37,.0003,43.74,.0002,46.20,.0001,48.68,.0001 ,49.93,.0001,51.19,.0002],
 * 			        [1.00,.0225,1.99,.0921,2.98,.0933,3.99,.0365,4.99,.0100,5.98,.0213,6.98,.0049 ,7.98,.0041,8.98,.0090,9.99,.0068,11.01,.0040,12.03,.0086,13.02,.0015,14.04,.0071,15.09,.0082 ,16.14,.0011,17.15,.0014,18.18,.0010,19.26,.0013,20.26,.0005,21.33,.0006,22.36,.0011,23.46,.0016 ,24.52,.0004,25.59,.0002,26.70,.0006,27.78,.0007,28.87,.0002,30.03,.0008,31.14,.0010,32.24,.0006 ,33.37,.0002,35.67,.0003,37.99,.0004,39.17,.0004,40.35,.0005,41.53,.0001,46.42,.0001],
 */
  __pyx_t_23 = PyList_New(94); if (unlikely(!__pyx_t_23)) __PYX_ERR(0, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_23);
  __Pyx_INCREF(__pyx_float_0_99);
  __Pyx_GIVEREF(__pyx_float_0_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 0, __pyx_float_0_99)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0156);
  __Pyx_GIVEREF(__pyx_float__0156);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 1, __pyx_float__0156)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_1_98);
  __Pyx_GIVEREF(__pyx_float_1_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 2, __pyx_float_1_98)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0846);
  __Pyx_GIVEREF(__pyx_float__0846);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 3, __pyx_float__0846)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_98);
  __Pyx_GIVEREF(__pyx_float_2_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 4, __pyx_float_2_98)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0178);
  __Pyx_GIVEREF(__pyx_float__0178);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 5, __pyx_float__0178)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_98);
  __Pyx_GIVEREF(__pyx_float_3_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 6, __pyx_float_3_98)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0367);
  __Pyx_GIVEREF(__pyx_float__0367);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 7, __pyx_float__0367)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_98);
  __Pyx_GIVEREF(__pyx_float_4_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 8, __pyx_float_4_98)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0448);
  __Pyx_GIVEREF(__pyx_float__0448);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 9, __pyx_float__0448)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_98);
  __Pyx_GIVEREF(__pyx_float_5_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 10, __pyx_float_5_98)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0113);
  __Pyx_GIVEREF(__pyx_float__0113);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 11, __pyx_float__0113)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_99);
  __Pyx_GIVEREF(__pyx_float_6_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 12, __pyx_float_6_99)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0189);
  __Pyx_GIVEREF(__pyx_float__0189);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 13, __pyx_float__0189)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_00);
  __Pyx_GIVEREF(__pyx_float_8_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 14, __pyx_float_8_00)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 15, __pyx_float__0011)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_01);
  __Pyx_GIVEREF(__pyx_float_9_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 16, __pyx_float_9_01)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0247);
  __Pyx_GIVEREF(__pyx_float__0247);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 17, __pyx_float__0247)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_02);
  __Pyx_GIVEREF(__pyx_float_10_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 18, __pyx_float_10_02)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0089);
  __Pyx_GIVEREF(__pyx_float__0089);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 19, __pyx_float__0089)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_01);
  __Pyx_GIVEREF(__pyx_float_11_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 20, __pyx_float_11_01)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0184);
  __Pyx_GIVEREF(__pyx_float__0184);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 21, __pyx_float__0184)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_03);
  __Pyx_GIVEREF(__pyx_float_12_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 22, __pyx_float_12_03)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0105);
  __Pyx_GIVEREF(__pyx_float__0105);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 23, __pyx_float__0105)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_00);
  __Pyx_GIVEREF(__pyx_float_13_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 24, __pyx_float_13_00)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0039);
  __Pyx_GIVEREF(__pyx_float__0039);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 25, __pyx_float__0039)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_07);
  __Pyx_GIVEREF(__pyx_float_14_07);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 26, __pyx_float_14_07)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0116);
  __Pyx_GIVEREF(__pyx_float__0116);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 27, __pyx_float__0116)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_09);
  __Pyx_GIVEREF(__pyx_float_15_09);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 28, __pyx_float_15_09)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0078);
  __Pyx_GIVEREF(__pyx_float__0078);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 29, __pyx_float__0078)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_13);
  __Pyx_GIVEREF(__pyx_float_16_13);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 30, __pyx_float_16_13)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 31, __pyx_float__0008)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_14);
  __Pyx_GIVEREF(__pyx_float_17_14);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 32, __pyx_float_17_14)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0064);
  __Pyx_GIVEREF(__pyx_float__0064);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 33, __pyx_float__0064)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_19);
  __Pyx_GIVEREF(__pyx_float_18_19);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 34, __pyx_float_18_19)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0029);
  __Pyx_GIVEREF(__pyx_float__0029);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 35, __pyx_float__0029)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_22);
  __Pyx_GIVEREF(__pyx_float_19_22);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 36, __pyx_float_19_22)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0028);
  __Pyx_GIVEREF(__pyx_float__0028);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 37, __pyx_float__0028)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_25);
  __Pyx_GIVEREF(__pyx_float_20_25);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 38, __pyx_float_20_25)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0017);
  __Pyx_GIVEREF(__pyx_float__0017);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 39, __pyx_float__0017)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_32);
  __Pyx_GIVEREF(__pyx_float_21_32);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 40, __pyx_float_21_32)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0043);
  __Pyx_GIVEREF(__pyx_float__0043);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 41, __pyx_float__0043)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_37);
  __Pyx_GIVEREF(__pyx_float_22_37);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 42, __pyx_float_22_37)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0055);
  __Pyx_GIVEREF(__pyx_float__0055);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 43, __pyx_float__0055)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_42);
  __Pyx_GIVEREF(__pyx_float_23_42);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 44, __pyx_float_23_42)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0034);
  __Pyx_GIVEREF(__pyx_float__0034);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 45, __pyx_float__0034)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_24_48);
  __Pyx_GIVEREF(__pyx_float_24_48);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 46, __pyx_float_24_48)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 47, __pyx_float__0004)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_25_54);
  __Pyx_GIVEREF(__pyx_float_25_54);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 48, __pyx_float_25_54)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 49, __pyx_float__0002)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_26_61);
  __Pyx_GIVEREF(__pyx_float_26_61);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 50, __pyx_float_26_61)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0017);
  __Pyx_GIVEREF(__pyx_float__0017);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 51, __pyx_float__0017)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_27_70);
  __Pyx_GIVEREF(__pyx_float_27_70);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 52, __pyx_float_27_70)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 53, __pyx_float__0011)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_80);
  __Pyx_GIVEREF(__pyx_float_28_80);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 54, __pyx_float_28_80)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 55, __pyx_float__0002)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_29_89);
  __Pyx_GIVEREF(__pyx_float_29_89);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 56, __pyx_float_29_89)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0019);
  __Pyx_GIVEREF(__pyx_float__0019);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 57, __pyx_float__0019)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_30_97);
  __Pyx_GIVEREF(__pyx_float_30_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 58, __pyx_float_30_97)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0028);
  __Pyx_GIVEREF(__pyx_float__0028);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 59, __pyx_float__0028)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_32_09);
  __Pyx_GIVEREF(__pyx_float_32_09);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 60, __pyx_float_32_09)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 61, __pyx_float__0007)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_34_30);
  __Pyx_GIVEREF(__pyx_float_34_30);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 62, __pyx_float_34_30)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 63, __pyx_float__0002)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_35_44);
  __Pyx_GIVEREF(__pyx_float_35_44);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 64, __pyx_float_35_44)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 65, __pyx_float__0003)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_36_55);
  __Pyx_GIVEREF(__pyx_float_36_55);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 66, __pyx_float_36_55)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 67, __pyx_float__0001)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_37_69);
  __Pyx_GIVEREF(__pyx_float_37_69);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 68, __pyx_float_37_69)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 69, __pyx_float__0004)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_38_93);
  __Pyx_GIVEREF(__pyx_float_38_93);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 70, __pyx_float_38_93)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 71, __pyx_float__0002)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_40_05);
  __Pyx_GIVEREF(__pyx_float_40_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 72, __pyx_float_40_05)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 73, __pyx_float__0005)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_41_20);
  __Pyx_GIVEREF(__pyx_float_41_20);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 74, __pyx_float_41_20)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 75, __pyx_float__0005)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_42_37);
  __Pyx_GIVEREF(__pyx_float_42_37);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 76, __pyx_float_42_37)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 77, __pyx_float__0002)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_43_54);
  __Pyx_GIVEREF(__pyx_float_43_54);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 78, __pyx_float_43_54)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 79, __pyx_float__0003)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_44_73);
  __Pyx_GIVEREF(__pyx_float_44_73);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 80, __pyx_float_44_73)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 81, __pyx_float__0001)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_45_95);
  __Pyx_GIVEREF(__pyx_float_45_95);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 82, __pyx_float_45_95)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 83, __pyx_float__0002)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_47_16);
  __Pyx_GIVEREF(__pyx_float_47_16);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 84, __pyx_float_47_16)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 85, __pyx_float__0001)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_48_43);
  __Pyx_GIVEREF(__pyx_float_48_43);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 86, __pyx_float_48_43)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 87, __pyx_float__0005)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_49_65);
  __Pyx_GIVEREF(__pyx_float_49_65);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 88, __pyx_float_49_65)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 89, __pyx_float__0004)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_55_90);
  __Pyx_GIVEREF(__pyx_float_55_90);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 90, __pyx_float_55_90)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 91, __pyx_float__0002)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_59_81);
  __Pyx_GIVEREF(__pyx_float_59_81);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 92, __pyx_float_59_81)) __PYX_ERR(0, 49, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_23, 93, __pyx_float__0004)) __PYX_ERR(0, 49, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":50
 * 			        [0.98,.0113,1.99,.0967,3.00,.0719,3.98,.0345,4.98,.0121,6.00,.0621,7.00,.0137 ,7.98,.0006,9.01,.0314,10.01,.0171,11.02,.0060,12.03,.0024,13.05,.0077,14.07,.0040,15.12,.0032 ,16.13,.0004,17.15,.0011,18.20,.0028,19.18,.0003,20.26,.0003,21.31,.0025,22.35,.0021,23.39,.0005 ,25.55,.0002,26.62,.0014,27.70,.0003,28.78,.0005,29.90,.0030,31.01,.0011,32.12,.0005,34.31,.0001 ,35.50,.0002,36.62,.0002,37.76,.0005,38.85,.0002,40.09,.0004,43.60,.0001,44.73,.0002,46.02,.0002 ,47.25,.0004,48.44,.0004],
 * 			        [0.99,.0156,1.98,.0846,2.98,.0178,3.98,.0367,4.98,.0448,5.98,.0113,6.99,.0189 ,8.00,.0011,9.01,.0247,10.02,.0089,11.01,.0184,12.03,.0105,13.00,.0039,14.07,.0116,15.09,.0078 ,16.13,.0008,17.14,.0064,18.19,.0029,19.22,.0028,20.25,.0017,21.32,.0043,22.37,.0055,23.42,.0034 ,24.48,.0004,25.54,.0002,26.61,.0017,27.70,.0011,28.80,.0002,29.89,.0019,30.97,.0028,32.09,.0007 ,34.30,.0002,35.44,.0003,36.55,.0001,37.69,.0004,38.93,.0002,40.05,.0005,41.20,.0005,42.37,.0002 ,43.54,.0003,44.73,.0001,45.95,.0002,47.16,.0001,48.43,.0005,49.65,.0004,55.90,.0002,59.81,.0004],
 * 			        [1.01,.0280,2.00,.0708,2.99,.0182,3.99,.0248,4.98,.0245,5.98,.0279,6.98,.0437 ,7.99,.0065,8.99,.0299,10.00,.0073,10.99,.0011,12.03,.0122,13.03,.0028,14.08,.0044,15.11,.0097 ,16.15,.0010,17.17,.0025,18.19,.0017,19.24,.0008,20.28,.0040,21.32,.0024,22.38,.0008,23.46,.0032 ,24.52,.0010,25.59,.0008,26.68,.0009,27.76,.0012,28.88,.0003,29.95,.0005,31.05,.0017,32.14,.0002 ,33.29,.0003,37.88,.0002,39.03,.0002,40.19,.0004,41.37,.0003,43.74,.0002,46.20,.0001,48.68,.0001 ,49.93,.0001,51.19,.0002],             # <<<<<<<<<<<<<<
 * 			        [1.00,.0225,1.99,.0921,2.98,.0933,3.99,.0365,4.99,.0100,5.98,.0213,6.98,.0049 ,7.98,.0041,8.98,.0090,9.99,.0068,11.01,.0040,12.03,.0086,13.02,.0015,14.04,.0071,15.09,.0082 ,16.14,.0011,17.15,.0014,18.18,.0010,19.26,.0013,20.26,.0005,21.33,.0006,22.36,.0011,23.46,.0016 ,24.52,.0004,25.59,.0002,26.70,.0006,27.78,.0007,28.87,.0002,30.03,.0008,31.14,.0010,32.24,.0006 ,33.37,.0002,35.67,.0003,37.99,.0004,39.17,.0004,40.35,.0005,41.53,.0001,46.42,.0001],
 * 			        [1.00,.0465,1.99,.0976,2.98,.0678,4.00,.0727,4.99,.0305,5.98,.0210,6.98,.0227 ,8.00,.0085,9.01,.0183,10.02,.0258,11.05,.0003,12.06,.0061,13.05,.0021,14.10,.0089,15.12,.0077 ,16.16,.0016,17.21,.0061,18.23,.0011,19.29,.0031,20.36,.0031,21.41,.0007,22.48,.0013,23.55,.0020 ,24.64,.0004,25.74,.0005,26.81,.0006,27.95,.0006,29.03,.0001,30.22,.0010,31.30,.0004,32.48,.0001 ,33.60,.0002,38.30,.0003],
 */
  __pyx_t_24 = PyList_New(82); if (unlikely(!__pyx_t_24)) __PYX_ERR(0, 50, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_24);
  __Pyx_INCREF(__pyx_float_1_01);
  __Pyx_GIVEREF(__pyx_float_1_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 0, __pyx_float_1_01)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0280);
  __Pyx_GIVEREF(__pyx_float__0280);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 1, __pyx_float__0280)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_00);
  __Pyx_GIVEREF(__pyx_float_2_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 2, __pyx_float_2_00)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0708);
  __Pyx_GIVEREF(__pyx_float__0708);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 3, __pyx_float__0708)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_99);
  __Pyx_GIVEREF(__pyx_float_2_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 4, __pyx_float_2_99)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0182);
  __Pyx_GIVEREF(__pyx_float__0182);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 5, __pyx_float__0182)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_99);
  __Pyx_GIVEREF(__pyx_float_3_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 6, __pyx_float_3_99)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0248);
  __Pyx_GIVEREF(__pyx_float__0248);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 7, __pyx_float__0248)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_98);
  __Pyx_GIVEREF(__pyx_float_4_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 8, __pyx_float_4_98)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0245);
  __Pyx_GIVEREF(__pyx_float__0245);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 9, __pyx_float__0245)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_98);
  __Pyx_GIVEREF(__pyx_float_5_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 10, __pyx_float_5_98)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0279);
  __Pyx_GIVEREF(__pyx_float__0279);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 11, __pyx_float__0279)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_98);
  __Pyx_GIVEREF(__pyx_float_6_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 12, __pyx_float_6_98)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0437);
  __Pyx_GIVEREF(__pyx_float__0437);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 13, __pyx_float__0437)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_99);
  __Pyx_GIVEREF(__pyx_float_7_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 14, __pyx_float_7_99)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0065);
  __Pyx_GIVEREF(__pyx_float__0065);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 15, __pyx_float__0065)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_99);
  __Pyx_GIVEREF(__pyx_float_8_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 16, __pyx_float_8_99)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0299);
  __Pyx_GIVEREF(__pyx_float__0299);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 17, __pyx_float__0299)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_00);
  __Pyx_GIVEREF(__pyx_float_10_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 18, __pyx_float_10_00)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0073);
  __Pyx_GIVEREF(__pyx_float__0073);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 19, __pyx_float__0073)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_99);
  __Pyx_GIVEREF(__pyx_float_10_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 20, __pyx_float_10_99)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 21, __pyx_float__0011)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_03);
  __Pyx_GIVEREF(__pyx_float_12_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 22, __pyx_float_12_03)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0122);
  __Pyx_GIVEREF(__pyx_float__0122);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 23, __pyx_float__0122)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_03);
  __Pyx_GIVEREF(__pyx_float_13_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 24, __pyx_float_13_03)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0028);
  __Pyx_GIVEREF(__pyx_float__0028);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 25, __pyx_float__0028)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_08);
  __Pyx_GIVEREF(__pyx_float_14_08);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 26, __pyx_float_14_08)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0044);
  __Pyx_GIVEREF(__pyx_float__0044);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 27, __pyx_float__0044)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_11);
  __Pyx_GIVEREF(__pyx_float_15_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 28, __pyx_float_15_11)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0097);
  __Pyx_GIVEREF(__pyx_float__0097);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 29, __pyx_float__0097)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_15);
  __Pyx_GIVEREF(__pyx_float_16_15);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 30, __pyx_float_16_15)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 31, __pyx_float__0010)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_17);
  __Pyx_GIVEREF(__pyx_float_17_17);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 32, __pyx_float_17_17)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0025);
  __Pyx_GIVEREF(__pyx_float__0025);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 33, __pyx_float__0025)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_19);
  __Pyx_GIVEREF(__pyx_float_18_19);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 34, __pyx_float_18_19)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0017);
  __Pyx_GIVEREF(__pyx_float__0017);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 35, __pyx_float__0017)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_24);
  __Pyx_GIVEREF(__pyx_float_19_24);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 36, __pyx_float_19_24)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 37, __pyx_float__0008)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_28);
  __Pyx_GIVEREF(__pyx_float_20_28);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 38, __pyx_float_20_28)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0040);
  __Pyx_GIVEREF(__pyx_float__0040);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 39, __pyx_float__0040)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_32);
  __Pyx_GIVEREF(__pyx_float_21_32);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 40, __pyx_float_21_32)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0024);
  __Pyx_GIVEREF(__pyx_float__0024);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 41, __pyx_float__0024)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_38);
  __Pyx_GIVEREF(__pyx_float_22_38);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 42, __pyx_float_22_38)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 43, __pyx_float__0008)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_46);
  __Pyx_GIVEREF(__pyx_float_23_46);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 44, __pyx_float_23_46)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0032);
  __Pyx_GIVEREF(__pyx_float__0032);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 45, __pyx_float__0032)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_24_52);
  __Pyx_GIVEREF(__pyx_float_24_52);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 46, __pyx_float_24_52)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 47, __pyx_float__0010)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_25_59);
  __Pyx_GIVEREF(__pyx_float_25_59);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 48, __pyx_float_25_59)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 49, __pyx_float__0008)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_26_68);
  __Pyx_GIVEREF(__pyx_float_26_68);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 50, __pyx_float_26_68)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 51, __pyx_float__0009)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_27_76);
  __Pyx_GIVEREF(__pyx_float_27_76);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 52, __pyx_float_27_76)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 53, __pyx_float__0012)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_88);
  __Pyx_GIVEREF(__pyx_float_28_88);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 54, __pyx_float_28_88)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 55, __pyx_float__0003)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_29_95);
  __Pyx_GIVEREF(__pyx_float_29_95);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 56, __pyx_float_29_95)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 57, __pyx_float__0005)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_31_05);
  __Pyx_GIVEREF(__pyx_float_31_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 58, __pyx_float_31_05)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0017);
  __Pyx_GIVEREF(__pyx_float__0017);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 59, __pyx_float__0017)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_32_14);
  __Pyx_GIVEREF(__pyx_float_32_14);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 60, __pyx_float_32_14)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 61, __pyx_float__0002)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_33_29);
  __Pyx_GIVEREF(__pyx_float_33_29);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 62, __pyx_float_33_29)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 63, __pyx_float__0003)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_37_88);
  __Pyx_GIVEREF(__pyx_float_37_88);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 64, __pyx_float_37_88)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 65, __pyx_float__0002)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_39_03);
  __Pyx_GIVEREF(__pyx_float_39_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 66, __pyx_float_39_03)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 67, __pyx_float__0002)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_40_19);
  __Pyx_GIVEREF(__pyx_float_40_19);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 68, __pyx_float_40_19)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 69, __pyx_float__0004)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_41_37);
  __Pyx_GIVEREF(__pyx_float_41_37);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 70, __pyx_float_41_37)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 71, __pyx_float__0003)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_43_74);
  __Pyx_GIVEREF(__pyx_float_43_74);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 72, __pyx_float_43_74)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 73, __pyx_float__0002)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_46_20);
  __Pyx_GIVEREF(__pyx_float_46_20);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 74, __pyx_float_46_20)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 75, __pyx_float__0001)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_48_68);
  __Pyx_GIVEREF(__pyx_float_48_68);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 76, __pyx_float_48_68)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 77, __pyx_float__0001)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_49_93);
  __Pyx_GIVEREF(__pyx_float_49_93);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 78, __pyx_float_49_93)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 79, __pyx_float__0001)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_51_19);
  __Pyx_GIVEREF(__pyx_float_51_19);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 80, __pyx_float_51_19)) __PYX_ERR(0, 50, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_24, 81, __pyx_float__0002)) __PYX_ERR(0, 50, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":51
 * 			        [0.99,.0156,1.98,.0846,2.98,.0178,3.98,.0367,4.98,.0448,5.98,.0113,6.99,.0189 ,8.00,.0011,9.01,.0247,10.02,.0089,11.01,.0184,12.03,.0105,13.00,.0039,14.07,.0116,15.09,.0078 ,16.13,.0008,17.14,.0064,18.19,.0029,19.22,.0028,20.25,.0017,21.32,.0043,22.37,.0055,23.42,.0034 ,24.48,.0004,25.54,.0002,26.61,.0017,27.70,.0011,28.80,.0002,29.89,.0019,30.97,.0028,32.09,.0007 ,34.30,.0002,35.44,.0003,36.55,.0001,37.69,.0004,38.93,.0002,40.05,.0005,41.20,.0005,42.37,.0002 ,43.54,.0003,44.73,.0001,45.95,.0002,47.16,.0001,48.43,.0005,49.65,.0004,55.90,.0002,59.81,.0004],
 * 			        [1.01,.0280,2.00,.0708,2.99,.0182,3.99,.0248,4.98,.0245,5.98,.0279,6.98,.0437 ,7.99,.0065,8.99,.0299,10.00,.0073,10.99,.0011,12.03,.0122,13.03,.0028,14.08,.0044,15.11,.0097 ,16.15,.0010,17.17,.0025,18.19,.0017,19.24,.0008,20.28,.0040,21.32,.0024,22.38,.0008,23.46,.0032 ,24.52,.0010,25.59,.0008,26.68,.0009,27.76,.0012,28.88,.0003,29.95,.0005,31.05,.0017,32.14,.0002 ,33.29,.0003,37.88,.0002,39.03,.0002,40.19,.0004,41.37,.0003,43.74,.0002,46.20,.0001,48.68,.0001 ,49.93,.0001,51.19,.0002],
 * 			        [1.00,.0225,1.99,.0921,2.98,.0933,3.99,.0365,4.99,.0100,5.98,.0213,6.98,.0049 ,7.98,.0041,8.98,.0090,9.99,.0068,11.01,.0040,12.03,.0086,13.02,.0015,14.04,.0071,15.09,.0082 ,16.14,.0011,17.15,.0014,18.18,.0010,19.26,.0013,20.26,.0005,21.33,.0006,22.36,.0011,23.46,.0016 ,24.52,.0004,25.59,.0002,26.70,.0006,27.78,.0007,28.87,.0002,30.03,.0008,31.14,.0010,32.24,.0006 ,33.37,.0002,35.67,.0003,37.99,.0004,39.17,.0004,40.35,.0005,41.53,.0001,46.42,.0001],             # <<<<<<<<<<<<<<
 * 			        [1.00,.0465,1.99,.0976,2.98,.0678,4.00,.0727,4.99,.0305,5.98,.0210,6.98,.0227 ,8.00,.0085,9.01,.0183,10.02,.0258,11.05,.0003,12.06,.0061,13.05,.0021,14.10,.0089,15.12,.0077 ,16.16,.0016,17.21,.0061,18.23,.0011,19.29,.0031,20.36,.0031,21.41,.0007,22.48,.0013,23.55,.0020 ,24.64,.0004,25.74,.0005,26.81,.0006,27.95,.0006,29.03,.0001,30.22,.0010,31.30,.0004,32.48,.0001 ,33.60,.0002,38.30,.0003],
 * 			        [1.00,.0674,1.99,.0841,2.98,.0920,3.99,.0328,4.99,.0368,5.98,.0206,6.99,.0246 ,8.01,.0048,9.01,.0218,10.03,.0155,11.05,.0048,12.06,.0077,13.00,.0020,14.10,.0083,15.15,.0084 ,16.18,.0015,17.22,.0039,18.27,.0032,19.34,.0026,20.40,.0012,21.47,.0009,22.54,.0008,23.62,.0016 ,24.71,.0005,25.82,.0004,26.91,.0002,28.03,.0008,29.17,.0002,30.32,.0028,31.45,.0004,32.61,.0005 ,33.77,.0001,36.14,.0003,37.32,.0002,38.54,.0005,39.75,.0002,42.23,.0002,48.65,.0001],
 */
  __pyx_t_25 = PyList_New(76); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 51, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_25);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 0, __pyx_float_1_00)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0225);
  __Pyx_GIVEREF(__pyx_float__0225);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 1, __pyx_float__0225)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_1_99);
  __Pyx_GIVEREF(__pyx_float_1_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 2, __pyx_float_1_99)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0921);
  __Pyx_GIVEREF(__pyx_float__0921);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 3, __pyx_float__0921)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_98);
  __Pyx_GIVEREF(__pyx_float_2_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 4, __pyx_float_2_98)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0933);
  __Pyx_GIVEREF(__pyx_float__0933);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 5, __pyx_float__0933)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_99);
  __Pyx_GIVEREF(__pyx_float_3_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 6, __pyx_float_3_99)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0365);
  __Pyx_GIVEREF(__pyx_float__0365);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 7, __pyx_float__0365)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_99);
  __Pyx_GIVEREF(__pyx_float_4_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 8, __pyx_float_4_99)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0100);
  __Pyx_GIVEREF(__pyx_float__0100);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 9, __pyx_float__0100)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_98);
  __Pyx_GIVEREF(__pyx_float_5_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 10, __pyx_float_5_98)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0213);
  __Pyx_GIVEREF(__pyx_float__0213);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 11, __pyx_float__0213)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_98);
  __Pyx_GIVEREF(__pyx_float_6_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 12, __pyx_float_6_98)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0049);
  __Pyx_GIVEREF(__pyx_float__0049);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 13, __pyx_float__0049)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_98);
  __Pyx_GIVEREF(__pyx_float_7_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 14, __pyx_float_7_98)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0041);
  __Pyx_GIVEREF(__pyx_float__0041);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 15, __pyx_float__0041)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_98);
  __Pyx_GIVEREF(__pyx_float_8_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 16, __pyx_float_8_98)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0090);
  __Pyx_GIVEREF(__pyx_float__0090);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 17, __pyx_float__0090)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_99);
  __Pyx_GIVEREF(__pyx_float_9_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 18, __pyx_float_9_99)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0068);
  __Pyx_GIVEREF(__pyx_float__0068);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 19, __pyx_float__0068)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_01);
  __Pyx_GIVEREF(__pyx_float_11_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 20, __pyx_float_11_01)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0040);
  __Pyx_GIVEREF(__pyx_float__0040);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 21, __pyx_float__0040)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_03);
  __Pyx_GIVEREF(__pyx_float_12_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 22, __pyx_float_12_03)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0086);
  __Pyx_GIVEREF(__pyx_float__0086);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 23, __pyx_float__0086)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_02);
  __Pyx_GIVEREF(__pyx_float_13_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 24, __pyx_float_13_02)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0015);
  __Pyx_GIVEREF(__pyx_float__0015);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 25, __pyx_float__0015)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_04);
  __Pyx_GIVEREF(__pyx_float_14_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 26, __pyx_float_14_04)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0071);
  __Pyx_GIVEREF(__pyx_float__0071);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 27, __pyx_float__0071)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_09);
  __Pyx_GIVEREF(__pyx_float_15_09);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 28, __pyx_float_15_09)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0082);
  __Pyx_GIVEREF(__pyx_float__0082);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 29, __pyx_float__0082)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_14);
  __Pyx_GIVEREF(__pyx_float_16_14);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 30, __pyx_float_16_14)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 31, __pyx_float__0011)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_15);
  __Pyx_GIVEREF(__pyx_float_17_15);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 32, __pyx_float_17_15)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0014);
  __Pyx_GIVEREF(__pyx_float__0014);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 33, __pyx_float__0014)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_18);
  __Pyx_GIVEREF(__pyx_float_18_18);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 34, __pyx_float_18_18)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 35, __pyx_float__0010)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_26);
  __Pyx_GIVEREF(__pyx_float_19_26);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 36, __pyx_float_19_26)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0013);
  __Pyx_GIVEREF(__pyx_float__0013);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 37, __pyx_float__0013)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_26);
  __Pyx_GIVEREF(__pyx_float_20_26);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 38, __pyx_float_20_26)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 39, __pyx_float__0005)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_33);
  __Pyx_GIVEREF(__pyx_float_21_33);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 40, __pyx_float_21_33)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 41, __pyx_float__0006)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_36);
  __Pyx_GIVEREF(__pyx_float_22_36);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 42, __pyx_float_22_36)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 43, __pyx_float__0011)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_46);
  __Pyx_GIVEREF(__pyx_float_23_46);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 44, __pyx_float_23_46)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0016);
  __Pyx_GIVEREF(__pyx_float__0016);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 45, __pyx_float__0016)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_24_52);
  __Pyx_GIVEREF(__pyx_float_24_52);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 46, __pyx_float_24_52)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 47, __pyx_float__0004)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_25_59);
  __Pyx_GIVEREF(__pyx_float_25_59);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 48, __pyx_float_25_59)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 49, __pyx_float__0002)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_26_70);
  __Pyx_GIVEREF(__pyx_float_26_70);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 50, __pyx_float_26_70)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 51, __pyx_float__0006)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_27_78);
  __Pyx_GIVEREF(__pyx_float_27_78);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 52, __pyx_float_27_78)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 53, __pyx_float__0007)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_87);
  __Pyx_GIVEREF(__pyx_float_28_87);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 54, __pyx_float_28_87)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 55, __pyx_float__0002)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_30_03);
  __Pyx_GIVEREF(__pyx_float_30_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 56, __pyx_float_30_03)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 57, __pyx_float__0008)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_31_14);
  __Pyx_GIVEREF(__pyx_float_31_14);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 58, __pyx_float_31_14)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 59, __pyx_float__0010)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_32_24);
  __Pyx_GIVEREF(__pyx_float_32_24);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 60, __pyx_float_32_24)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 61, __pyx_float__0006)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_33_37);
  __Pyx_GIVEREF(__pyx_float_33_37);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 62, __pyx_float_33_37)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 63, __pyx_float__0002)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_35_67);
  __Pyx_GIVEREF(__pyx_float_35_67);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 64, __pyx_float_35_67)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 65, __pyx_float__0003)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_37_99);
  __Pyx_GIVEREF(__pyx_float_37_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 66, __pyx_float_37_99)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 67, __pyx_float__0004)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_39_17);
  __Pyx_GIVEREF(__pyx_float_39_17);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 68, __pyx_float_39_17)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 69, __pyx_float__0004)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_40_35);
  __Pyx_GIVEREF(__pyx_float_40_35);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 70, __pyx_float_40_35)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 71, __pyx_float__0005)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_41_53);
  __Pyx_GIVEREF(__pyx_float_41_53);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 72, __pyx_float_41_53)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 73, __pyx_float__0001)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_46_42);
  __Pyx_GIVEREF(__pyx_float_46_42);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 74, __pyx_float_46_42)) __PYX_ERR(0, 51, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 75, __pyx_float__0001)) __PYX_ERR(0, 51, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":52
 * 			        [1.01,.0280,2.00,.0708,2.99,.0182,3.99,.0248,4.98,.0245,5.98,.0279,6.98,.0437 ,7.99,.0065,8.99,.0299,10.00,.0073,10.99,.0011,12.03,.0122,13.03,.0028,14.08,.0044,15.11,.0097 ,16.15,.0010,17.17,.0025,18.19,.0017,19.24,.0008,20.28,.0040,21.32,.0024,22.38,.0008,23.46,.0032 ,24.52,.0010,25.59,.0008,26.68,.0009,27.76,.0012,28.88,.0003,29.95,.0005,31.05,.0017,32.14,.0002 ,33.29,.0003,37.88,.0002,39.03,.0002,40.19,.0004,41.37,.0003,43.74,.0002,46.20,.0001,48.68,.0001 ,49.93,.0001,51.19,.0002],
 * 			        [1.00,.0225,1.99,.0921,2.98,.0933,3.99,.0365,4.99,.0100,5.98,.0213,6.98,.0049 ,7.98,.0041,8.98,.0090,9.99,.0068,11.01,.0040,12.03,.0086,13.02,.0015,14.04,.0071,15.09,.0082 ,16.14,.0011,17.15,.0014,18.18,.0010,19.26,.0013,20.26,.0005,21.33,.0006,22.36,.0011,23.46,.0016 ,24.52,.0004,25.59,.0002,26.70,.0006,27.78,.0007,28.87,.0002,30.03,.0008,31.14,.0010,32.24,.0006 ,33.37,.0002,35.67,.0003,37.99,.0004,39.17,.0004,40.35,.0005,41.53,.0001,46.42,.0001],
 * 			        [1.00,.0465,1.99,.0976,2.98,.0678,4.00,.0727,4.99,.0305,5.98,.0210,6.98,.0227 ,8.00,.0085,9.01,.0183,10.02,.0258,11.05,.0003,12.06,.0061,13.05,.0021,14.10,.0089,15.12,.0077 ,16.16,.0016,17.21,.0061,18.23,.0011,19.29,.0031,20.36,.0031,21.41,.0007,22.48,.0013,23.55,.0020 ,24.64,.0004,25.74,.0005,26.81,.0006,27.95,.0006,29.03,.0001,30.22,.0010,31.30,.0004,32.48,.0001 ,33.60,.0002,38.30,.0003],             # <<<<<<<<<<<<<<
 * 			        [1.00,.0674,1.99,.0841,2.98,.0920,3.99,.0328,4.99,.0368,5.98,.0206,6.99,.0246 ,8.01,.0048,9.01,.0218,10.03,.0155,11.05,.0048,12.06,.0077,13.00,.0020,14.10,.0083,15.15,.0084 ,16.18,.0015,17.22,.0039,18.27,.0032,19.34,.0026,20.40,.0012,21.47,.0009,22.54,.0008,23.62,.0016 ,24.71,.0005,25.82,.0004,26.91,.0002,28.03,.0008,29.17,.0002,30.32,.0028,31.45,.0004,32.61,.0005 ,33.77,.0001,36.14,.0003,37.32,.0002,38.54,.0005,39.75,.0002,42.23,.0002,48.65,.0001],
 * 			        [1.01,.0423,1.99,.0240,2.98,.0517,4.00,.0493,5.00,.0324,6.00,.0094,6.99,.0449 ,7.99,.0050,9.00,.0197,10.03,.0132,11.03,.0009,12.07,.0017,13.08,.0023,14.12,.0094,15.16,.0071 ,16.21,.0020,17.25,.0005,18.30,.0027,19.04,.0004,20.43,.0022,21.51,.0002,22.59,.0006,23.72,.0018 ,24.80,.0002,25.88,.0002,27.03,.0002,28.09,.0006,29.31,.0002,30.46,.0004,31.61,.0007,32.78,.0005 ,33.95,.0001,36.34,.0002,37.56,.0001,38.80,.0001,40.02,.0001,44.14,.0001],
 */
  __pyx_t_26 = PyList_New(66); if (unlikely(!__pyx_t_26)) __PYX_ERR(0, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_26);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 0, __pyx_float_1_00)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0465);
  __Pyx_GIVEREF(__pyx_float__0465);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 1, __pyx_float__0465)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_1_99);
  __Pyx_GIVEREF(__pyx_float_1_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 2, __pyx_float_1_99)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0976);
  __Pyx_GIVEREF(__pyx_float__0976);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 3, __pyx_float__0976)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_98);
  __Pyx_GIVEREF(__pyx_float_2_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 4, __pyx_float_2_98)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0678);
  __Pyx_GIVEREF(__pyx_float__0678);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 5, __pyx_float__0678)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_00);
  __Pyx_GIVEREF(__pyx_float_4_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 6, __pyx_float_4_00)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0727);
  __Pyx_GIVEREF(__pyx_float__0727);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 7, __pyx_float__0727)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_99);
  __Pyx_GIVEREF(__pyx_float_4_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 8, __pyx_float_4_99)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0305);
  __Pyx_GIVEREF(__pyx_float__0305);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 9, __pyx_float__0305)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_98);
  __Pyx_GIVEREF(__pyx_float_5_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 10, __pyx_float_5_98)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0210);
  __Pyx_GIVEREF(__pyx_float__0210);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 11, __pyx_float__0210)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_98);
  __Pyx_GIVEREF(__pyx_float_6_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 12, __pyx_float_6_98)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0227);
  __Pyx_GIVEREF(__pyx_float__0227);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 13, __pyx_float__0227)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_00);
  __Pyx_GIVEREF(__pyx_float_8_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 14, __pyx_float_8_00)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0085);
  __Pyx_GIVEREF(__pyx_float__0085);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 15, __pyx_float__0085)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_01);
  __Pyx_GIVEREF(__pyx_float_9_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 16, __pyx_float_9_01)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0183);
  __Pyx_GIVEREF(__pyx_float__0183);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 17, __pyx_float__0183)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_02);
  __Pyx_GIVEREF(__pyx_float_10_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 18, __pyx_float_10_02)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0258);
  __Pyx_GIVEREF(__pyx_float__0258);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 19, __pyx_float__0258)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_05);
  __Pyx_GIVEREF(__pyx_float_11_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 20, __pyx_float_11_05)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 21, __pyx_float__0003)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_06);
  __Pyx_GIVEREF(__pyx_float_12_06);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 22, __pyx_float_12_06)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0061);
  __Pyx_GIVEREF(__pyx_float__0061);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 23, __pyx_float__0061)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_05);
  __Pyx_GIVEREF(__pyx_float_13_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 24, __pyx_float_13_05)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0021);
  __Pyx_GIVEREF(__pyx_float__0021);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 25, __pyx_float__0021)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_10);
  __Pyx_GIVEREF(__pyx_float_14_10);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 26, __pyx_float_14_10)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0089);
  __Pyx_GIVEREF(__pyx_float__0089);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 27, __pyx_float__0089)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_12);
  __Pyx_GIVEREF(__pyx_float_15_12);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 28, __pyx_float_15_12)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0077);
  __Pyx_GIVEREF(__pyx_float__0077);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 29, __pyx_float__0077)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_16);
  __Pyx_GIVEREF(__pyx_float_16_16);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 30, __pyx_float_16_16)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0016);
  __Pyx_GIVEREF(__pyx_float__0016);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 31, __pyx_float__0016)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_21);
  __Pyx_GIVEREF(__pyx_float_17_21);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 32, __pyx_float_17_21)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0061);
  __Pyx_GIVEREF(__pyx_float__0061);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 33, __pyx_float__0061)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_23);
  __Pyx_GIVEREF(__pyx_float_18_23);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 34, __pyx_float_18_23)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 35, __pyx_float__0011)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_29);
  __Pyx_GIVEREF(__pyx_float_19_29);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 36, __pyx_float_19_29)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0031);
  __Pyx_GIVEREF(__pyx_float__0031);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 37, __pyx_float__0031)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_36);
  __Pyx_GIVEREF(__pyx_float_20_36);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 38, __pyx_float_20_36)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0031);
  __Pyx_GIVEREF(__pyx_float__0031);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 39, __pyx_float__0031)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_41);
  __Pyx_GIVEREF(__pyx_float_21_41);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 40, __pyx_float_21_41)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 41, __pyx_float__0007)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_48);
  __Pyx_GIVEREF(__pyx_float_22_48);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 42, __pyx_float_22_48)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0013);
  __Pyx_GIVEREF(__pyx_float__0013);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 43, __pyx_float__0013)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_55);
  __Pyx_GIVEREF(__pyx_float_23_55);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 44, __pyx_float_23_55)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0020);
  __Pyx_GIVEREF(__pyx_float__0020);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 45, __pyx_float__0020)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_24_64);
  __Pyx_GIVEREF(__pyx_float_24_64);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 46, __pyx_float_24_64)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 47, __pyx_float__0004)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_25_74);
  __Pyx_GIVEREF(__pyx_float_25_74);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 48, __pyx_float_25_74)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 49, __pyx_float__0005)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_26_81);
  __Pyx_GIVEREF(__pyx_float_26_81);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 50, __pyx_float_26_81)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 51, __pyx_float__0006)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_27_95);
  __Pyx_GIVEREF(__pyx_float_27_95);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 52, __pyx_float_27_95)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 53, __pyx_float__0006)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_29_03);
  __Pyx_GIVEREF(__pyx_float_29_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 54, __pyx_float_29_03)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 55, __pyx_float__0001)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_30_22);
  __Pyx_GIVEREF(__pyx_float_30_22);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 56, __pyx_float_30_22)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 57, __pyx_float__0010)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_31_30);
  __Pyx_GIVEREF(__pyx_float_31_30);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 58, __pyx_float_31_30)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 59, __pyx_float__0004)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_32_48);
  __Pyx_GIVEREF(__pyx_float_32_48);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 60, __pyx_float_32_48)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 61, __pyx_float__0001)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_33_60);
  __Pyx_GIVEREF(__pyx_float_33_60);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 62, __pyx_float_33_60)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 63, __pyx_float__0002)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_38_30);
  __Pyx_GIVEREF(__pyx_float_38_30);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 64, __pyx_float_38_30)) __PYX_ERR(0, 52, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_26, 65, __pyx_float__0003)) __PYX_ERR(0, 52, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":53
 * 			        [1.00,.0225,1.99,.0921,2.98,.0933,3.99,.0365,4.99,.0100,5.98,.0213,6.98,.0049 ,7.98,.0041,8.98,.0090,9.99,.0068,11.01,.0040,12.03,.0086,13.02,.0015,14.04,.0071,15.09,.0082 ,16.14,.0011,17.15,.0014,18.18,.0010,19.26,.0013,20.26,.0005,21.33,.0006,22.36,.0011,23.46,.0016 ,24.52,.0004,25.59,.0002,26.70,.0006,27.78,.0007,28.87,.0002,30.03,.0008,31.14,.0010,32.24,.0006 ,33.37,.0002,35.67,.0003,37.99,.0004,39.17,.0004,40.35,.0005,41.53,.0001,46.42,.0001],
 * 			        [1.00,.0465,1.99,.0976,2.98,.0678,4.00,.0727,4.99,.0305,5.98,.0210,6.98,.0227 ,8.00,.0085,9.01,.0183,10.02,.0258,11.05,.0003,12.06,.0061,13.05,.0021,14.10,.0089,15.12,.0077 ,16.16,.0016,17.21,.0061,18.23,.0011,19.29,.0031,20.36,.0031,21.41,.0007,22.48,.0013,23.55,.0020 ,24.64,.0004,25.74,.0005,26.81,.0006,27.95,.0006,29.03,.0001,30.22,.0010,31.30,.0004,32.48,.0001 ,33.60,.0002,38.30,.0003],
 * 			        [1.00,.0674,1.99,.0841,2.98,.0920,3.99,.0328,4.99,.0368,5.98,.0206,6.99,.0246 ,8.01,.0048,9.01,.0218,10.03,.0155,11.05,.0048,12.06,.0077,13.00,.0020,14.10,.0083,15.15,.0084 ,16.18,.0015,17.22,.0039,18.27,.0032,19.34,.0026,20.40,.0012,21.47,.0009,22.54,.0008,23.62,.0016 ,24.71,.0005,25.82,.0004,26.91,.0002,28.03,.0008,29.17,.0002,30.32,.0028,31.45,.0004,32.61,.0005 ,33.77,.0001,36.14,.0003,37.32,.0002,38.54,.0005,39.75,.0002,42.23,.0002,48.65,.0001],             # <<<<<<<<<<<<<<
 * 			        [1.01,.0423,1.99,.0240,2.98,.0517,4.00,.0493,5.00,.0324,6.00,.0094,6.99,.0449 ,7.99,.0050,9.00,.0197,10.03,.0132,11.03,.0009,12.07,.0017,13.08,.0023,14.12,.0094,15.16,.0071 ,16.21,.0020,17.25,.0005,18.30,.0027,19.04,.0004,20.43,.0022,21.51,.0002,22.59,.0006,23.72,.0018 ,24.80,.0002,25.88,.0002,27.03,.0002,28.09,.0006,29.31,.0002,30.46,.0004,31.61,.0007,32.78,.0005 ,33.95,.0001,36.34,.0002,37.56,.0001,38.80,.0001,40.02,.0001,44.14,.0001],
 * 			        [1.00,.0669,1.99,.0909,2.99,.0410,3.98,.0292,4.98,.0259,5.98,.0148,6.98,.0319 ,7.99,.0076,9.01,.0056,10.02,.0206,11.04,.0032,12.05,.0085,13.08,.0040,14.12,.0037,15.16,.0030 ,16.20,.0013,17.24,.0021,18.30,.0010,19.36,.0015,20.44,.0013,21.50,.0009,22.60,.0015,23.69,.0014 ,24.80,.0006,25.87,.0002,27.02,.0006,28.12,.0002,29.28,.0003,30.43,.0002,31.59,.0007,32.79,.0001 ,35.14,.0001,37.57,.0001,40.03,.0002,41.28,.0004,44.10,.0001],
 */
  __pyx_t_27 = PyList_New(76); if (unlikely(!__pyx_t_27)) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_27);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 0, __pyx_float_1_00)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0674);
  __Pyx_GIVEREF(__pyx_float__0674);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 1, __pyx_float__0674)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_1_99);
  __Pyx_GIVEREF(__pyx_float_1_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 2, __pyx_float_1_99)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0841);
  __Pyx_GIVEREF(__pyx_float__0841);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 3, __pyx_float__0841)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_98);
  __Pyx_GIVEREF(__pyx_float_2_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 4, __pyx_float_2_98)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0920);
  __Pyx_GIVEREF(__pyx_float__0920);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 5, __pyx_float__0920)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_99);
  __Pyx_GIVEREF(__pyx_float_3_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 6, __pyx_float_3_99)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0328);
  __Pyx_GIVEREF(__pyx_float__0328);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 7, __pyx_float__0328)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_99);
  __Pyx_GIVEREF(__pyx_float_4_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 8, __pyx_float_4_99)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0368);
  __Pyx_GIVEREF(__pyx_float__0368);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 9, __pyx_float__0368)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_98);
  __Pyx_GIVEREF(__pyx_float_5_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 10, __pyx_float_5_98)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0206);
  __Pyx_GIVEREF(__pyx_float__0206);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 11, __pyx_float__0206)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_99);
  __Pyx_GIVEREF(__pyx_float_6_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 12, __pyx_float_6_99)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0246);
  __Pyx_GIVEREF(__pyx_float__0246);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 13, __pyx_float__0246)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_01);
  __Pyx_GIVEREF(__pyx_float_8_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 14, __pyx_float_8_01)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0048);
  __Pyx_GIVEREF(__pyx_float__0048);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 15, __pyx_float__0048)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_01);
  __Pyx_GIVEREF(__pyx_float_9_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 16, __pyx_float_9_01)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0218);
  __Pyx_GIVEREF(__pyx_float__0218);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 17, __pyx_float__0218)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_03);
  __Pyx_GIVEREF(__pyx_float_10_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 18, __pyx_float_10_03)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0155);
  __Pyx_GIVEREF(__pyx_float__0155);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 19, __pyx_float__0155)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_05);
  __Pyx_GIVEREF(__pyx_float_11_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 20, __pyx_float_11_05)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0048);
  __Pyx_GIVEREF(__pyx_float__0048);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 21, __pyx_float__0048)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_06);
  __Pyx_GIVEREF(__pyx_float_12_06);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 22, __pyx_float_12_06)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0077);
  __Pyx_GIVEREF(__pyx_float__0077);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 23, __pyx_float__0077)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_00);
  __Pyx_GIVEREF(__pyx_float_13_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 24, __pyx_float_13_00)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0020);
  __Pyx_GIVEREF(__pyx_float__0020);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 25, __pyx_float__0020)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_10);
  __Pyx_GIVEREF(__pyx_float_14_10);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 26, __pyx_float_14_10)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0083);
  __Pyx_GIVEREF(__pyx_float__0083);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 27, __pyx_float__0083)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_15);
  __Pyx_GIVEREF(__pyx_float_15_15);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 28, __pyx_float_15_15)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0084);
  __Pyx_GIVEREF(__pyx_float__0084);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 29, __pyx_float__0084)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_18);
  __Pyx_GIVEREF(__pyx_float_16_18);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 30, __pyx_float_16_18)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0015);
  __Pyx_GIVEREF(__pyx_float__0015);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 31, __pyx_float__0015)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_22);
  __Pyx_GIVEREF(__pyx_float_17_22);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 32, __pyx_float_17_22)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0039);
  __Pyx_GIVEREF(__pyx_float__0039);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 33, __pyx_float__0039)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_27);
  __Pyx_GIVEREF(__pyx_float_18_27);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 34, __pyx_float_18_27)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0032);
  __Pyx_GIVEREF(__pyx_float__0032);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 35, __pyx_float__0032)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_34);
  __Pyx_GIVEREF(__pyx_float_19_34);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 36, __pyx_float_19_34)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0026);
  __Pyx_GIVEREF(__pyx_float__0026);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 37, __pyx_float__0026)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_40);
  __Pyx_GIVEREF(__pyx_float_20_40);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 38, __pyx_float_20_40)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 39, __pyx_float__0012)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_47);
  __Pyx_GIVEREF(__pyx_float_21_47);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 40, __pyx_float_21_47)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 41, __pyx_float__0009)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_54);
  __Pyx_GIVEREF(__pyx_float_22_54);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 42, __pyx_float_22_54)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 43, __pyx_float__0008)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_62);
  __Pyx_GIVEREF(__pyx_float_23_62);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 44, __pyx_float_23_62)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0016);
  __Pyx_GIVEREF(__pyx_float__0016);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 45, __pyx_float__0016)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_24_71);
  __Pyx_GIVEREF(__pyx_float_24_71);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 46, __pyx_float_24_71)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 47, __pyx_float__0005)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_25_82);
  __Pyx_GIVEREF(__pyx_float_25_82);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 48, __pyx_float_25_82)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 49, __pyx_float__0004)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_26_91);
  __Pyx_GIVEREF(__pyx_float_26_91);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 50, __pyx_float_26_91)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 51, __pyx_float__0002)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_03);
  __Pyx_GIVEREF(__pyx_float_28_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 52, __pyx_float_28_03)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 53, __pyx_float__0008)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_29_17);
  __Pyx_GIVEREF(__pyx_float_29_17);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 54, __pyx_float_29_17)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 55, __pyx_float__0002)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_30_32);
  __Pyx_GIVEREF(__pyx_float_30_32);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 56, __pyx_float_30_32)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0028);
  __Pyx_GIVEREF(__pyx_float__0028);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 57, __pyx_float__0028)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_31_45);
  __Pyx_GIVEREF(__pyx_float_31_45);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 58, __pyx_float_31_45)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 59, __pyx_float__0004)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_32_61);
  __Pyx_GIVEREF(__pyx_float_32_61);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 60, __pyx_float_32_61)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 61, __pyx_float__0005)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_33_77);
  __Pyx_GIVEREF(__pyx_float_33_77);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 62, __pyx_float_33_77)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 63, __pyx_float__0001)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_36_14);
  __Pyx_GIVEREF(__pyx_float_36_14);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 64, __pyx_float_36_14)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 65, __pyx_float__0003)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_37_32);
  __Pyx_GIVEREF(__pyx_float_37_32);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 66, __pyx_float_37_32)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 67, __pyx_float__0002)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_38_54);
  __Pyx_GIVEREF(__pyx_float_38_54);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 68, __pyx_float_38_54)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 69, __pyx_float__0005)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_39_75);
  __Pyx_GIVEREF(__pyx_float_39_75);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 70, __pyx_float_39_75)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 71, __pyx_float__0002)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_42_23);
  __Pyx_GIVEREF(__pyx_float_42_23);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 72, __pyx_float_42_23)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 73, __pyx_float__0002)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_48_65);
  __Pyx_GIVEREF(__pyx_float_48_65);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 74, __pyx_float_48_65)) __PYX_ERR(0, 53, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_27, 75, __pyx_float__0001)) __PYX_ERR(0, 53, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":54
 * 			        [1.00,.0465,1.99,.0976,2.98,.0678,4.00,.0727,4.99,.0305,5.98,.0210,6.98,.0227 ,8.00,.0085,9.01,.0183,10.02,.0258,11.05,.0003,12.06,.0061,13.05,.0021,14.10,.0089,15.12,.0077 ,16.16,.0016,17.21,.0061,18.23,.0011,19.29,.0031,20.36,.0031,21.41,.0007,22.48,.0013,23.55,.0020 ,24.64,.0004,25.74,.0005,26.81,.0006,27.95,.0006,29.03,.0001,30.22,.0010,31.30,.0004,32.48,.0001 ,33.60,.0002,38.30,.0003],
 * 			        [1.00,.0674,1.99,.0841,2.98,.0920,3.99,.0328,4.99,.0368,5.98,.0206,6.99,.0246 ,8.01,.0048,9.01,.0218,10.03,.0155,11.05,.0048,12.06,.0077,13.00,.0020,14.10,.0083,15.15,.0084 ,16.18,.0015,17.22,.0039,18.27,.0032,19.34,.0026,20.40,.0012,21.47,.0009,22.54,.0008,23.62,.0016 ,24.71,.0005,25.82,.0004,26.91,.0002,28.03,.0008,29.17,.0002,30.32,.0028,31.45,.0004,32.61,.0005 ,33.77,.0001,36.14,.0003,37.32,.0002,38.54,.0005,39.75,.0002,42.23,.0002,48.65,.0001],
 * 			        [1.01,.0423,1.99,.0240,2.98,.0517,4.00,.0493,5.00,.0324,6.00,.0094,6.99,.0449 ,7.99,.0050,9.00,.0197,10.03,.0132,11.03,.0009,12.07,.0017,13.08,.0023,14.12,.0094,15.16,.0071 ,16.21,.0020,17.25,.0005,18.30,.0027,19.04,.0004,20.43,.0022,21.51,.0002,22.59,.0006,23.72,.0018 ,24.80,.0002,25.88,.0002,27.03,.0002,28.09,.0006,29.31,.0002,30.46,.0004,31.61,.0007,32.78,.0005 ,33.95,.0001,36.34,.0002,37.56,.0001,38.80,.0001,40.02,.0001,44.14,.0001],             # <<<<<<<<<<<<<<
 * 			        [1.00,.0669,1.99,.0909,2.99,.0410,3.98,.0292,4.98,.0259,5.98,.0148,6.98,.0319 ,7.99,.0076,9.01,.0056,10.02,.0206,11.04,.0032,12.05,.0085,13.08,.0040,14.12,.0037,15.16,.0030 ,16.20,.0013,17.24,.0021,18.30,.0010,19.36,.0015,20.44,.0013,21.50,.0009,22.60,.0015,23.69,.0014 ,24.80,.0006,25.87,.0002,27.02,.0006,28.12,.0002,29.28,.0003,30.43,.0002,31.59,.0007,32.79,.0001 ,35.14,.0001,37.57,.0001,40.03,.0002,41.28,.0004,44.10,.0001],
 * 			        [0.99,.0421,1.99,.1541,2.98,.0596,3.98,.0309,4.98,.0301,5.99,.0103,7.00,.0240 ,8.01,.0073,9.01,.0222,10.04,.0140,11.05,.0033,12.08,.0045,13.13,.0009,14.13,.0015,15.21,.0026 ,16.24,.0003,17.30,.0004,18.35,.0010,19.39,.0003,20.50,.0015,21.57,.0003,22.68,.0011,23.80,.0005 ,24.90,.0008,26.02,.0002,27.16,.0001,28.30,.0006,29.48,.0002,31.81,.0005,33.00,.0003,34.21,.0001 ,37.89,.0001],
 */
  __pyx_t_28 = PyList_New(74); if (unlikely(!__pyx_t_28)) __PYX_ERR(0, 54, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_28);
  __Pyx_INCREF(__pyx_float_1_01);
  __Pyx_GIVEREF(__pyx_float_1_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 0, __pyx_float_1_01)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0423);
  __Pyx_GIVEREF(__pyx_float__0423);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 1, __pyx_float__0423)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_1_99);
  __Pyx_GIVEREF(__pyx_float_1_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 2, __pyx_float_1_99)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0240);
  __Pyx_GIVEREF(__pyx_float__0240);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 3, __pyx_float__0240)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_98);
  __Pyx_GIVEREF(__pyx_float_2_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 4, __pyx_float_2_98)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0517);
  __Pyx_GIVEREF(__pyx_float__0517);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 5, __pyx_float__0517)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_00);
  __Pyx_GIVEREF(__pyx_float_4_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 6, __pyx_float_4_00)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0493);
  __Pyx_GIVEREF(__pyx_float__0493);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 7, __pyx_float__0493)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_00);
  __Pyx_GIVEREF(__pyx_float_5_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 8, __pyx_float_5_00)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0324);
  __Pyx_GIVEREF(__pyx_float__0324);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 9, __pyx_float__0324)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_00);
  __Pyx_GIVEREF(__pyx_float_6_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 10, __pyx_float_6_00)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0094);
  __Pyx_GIVEREF(__pyx_float__0094);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 11, __pyx_float__0094)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_99);
  __Pyx_GIVEREF(__pyx_float_6_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 12, __pyx_float_6_99)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0449);
  __Pyx_GIVEREF(__pyx_float__0449);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 13, __pyx_float__0449)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_99);
  __Pyx_GIVEREF(__pyx_float_7_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 14, __pyx_float_7_99)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0050);
  __Pyx_GIVEREF(__pyx_float__0050);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 15, __pyx_float__0050)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_00);
  __Pyx_GIVEREF(__pyx_float_9_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 16, __pyx_float_9_00)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0197);
  __Pyx_GIVEREF(__pyx_float__0197);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 17, __pyx_float__0197)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_03);
  __Pyx_GIVEREF(__pyx_float_10_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 18, __pyx_float_10_03)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0132);
  __Pyx_GIVEREF(__pyx_float__0132);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 19, __pyx_float__0132)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_03);
  __Pyx_GIVEREF(__pyx_float_11_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 20, __pyx_float_11_03)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 21, __pyx_float__0009)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_07);
  __Pyx_GIVEREF(__pyx_float_12_07);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 22, __pyx_float_12_07)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0017);
  __Pyx_GIVEREF(__pyx_float__0017);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 23, __pyx_float__0017)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_08);
  __Pyx_GIVEREF(__pyx_float_13_08);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 24, __pyx_float_13_08)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0023);
  __Pyx_GIVEREF(__pyx_float__0023);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 25, __pyx_float__0023)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_12);
  __Pyx_GIVEREF(__pyx_float_14_12);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 26, __pyx_float_14_12)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0094);
  __Pyx_GIVEREF(__pyx_float__0094);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 27, __pyx_float__0094)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_16);
  __Pyx_GIVEREF(__pyx_float_15_16);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 28, __pyx_float_15_16)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0071);
  __Pyx_GIVEREF(__pyx_float__0071);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 29, __pyx_float__0071)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_21);
  __Pyx_GIVEREF(__pyx_float_16_21);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 30, __pyx_float_16_21)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0020);
  __Pyx_GIVEREF(__pyx_float__0020);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 31, __pyx_float__0020)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_25);
  __Pyx_GIVEREF(__pyx_float_17_25);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 32, __pyx_float_17_25)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 33, __pyx_float__0005)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_30);
  __Pyx_GIVEREF(__pyx_float_18_30);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 34, __pyx_float_18_30)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0027);
  __Pyx_GIVEREF(__pyx_float__0027);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 35, __pyx_float__0027)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_04);
  __Pyx_GIVEREF(__pyx_float_19_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 36, __pyx_float_19_04)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 37, __pyx_float__0004)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_43);
  __Pyx_GIVEREF(__pyx_float_20_43);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 38, __pyx_float_20_43)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0022);
  __Pyx_GIVEREF(__pyx_float__0022);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 39, __pyx_float__0022)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_51);
  __Pyx_GIVEREF(__pyx_float_21_51);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 40, __pyx_float_21_51)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 41, __pyx_float__0002)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_59);
  __Pyx_GIVEREF(__pyx_float_22_59);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 42, __pyx_float_22_59)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 43, __pyx_float__0006)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_72);
  __Pyx_GIVEREF(__pyx_float_23_72);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 44, __pyx_float_23_72)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0018);
  __Pyx_GIVEREF(__pyx_float__0018);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 45, __pyx_float__0018)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_24_80);
  __Pyx_GIVEREF(__pyx_float_24_80);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 46, __pyx_float_24_80)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 47, __pyx_float__0002)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_25_88);
  __Pyx_GIVEREF(__pyx_float_25_88);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 48, __pyx_float_25_88)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 49, __pyx_float__0002)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_27_03);
  __Pyx_GIVEREF(__pyx_float_27_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 50, __pyx_float_27_03)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 51, __pyx_float__0002)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_09);
  __Pyx_GIVEREF(__pyx_float_28_09);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 52, __pyx_float_28_09)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 53, __pyx_float__0006)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_29_31);
  __Pyx_GIVEREF(__pyx_float_29_31);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 54, __pyx_float_29_31)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 55, __pyx_float__0002)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_30_46);
  __Pyx_GIVEREF(__pyx_float_30_46);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 56, __pyx_float_30_46)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 57, __pyx_float__0004)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_31_61);
  __Pyx_GIVEREF(__pyx_float_31_61);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 58, __pyx_float_31_61)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 59, __pyx_float__0007)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_32_78);
  __Pyx_GIVEREF(__pyx_float_32_78);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 60, __pyx_float_32_78)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 61, __pyx_float__0005)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_33_95);
  __Pyx_GIVEREF(__pyx_float_33_95);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 62, __pyx_float_33_95)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 63, __pyx_float__0001)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_36_34);
  __Pyx_GIVEREF(__pyx_float_36_34);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 64, __pyx_float_36_34)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 65, __pyx_float__0002)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_37_56);
  __Pyx_GIVEREF(__pyx_float_37_56);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 66, __pyx_float_37_56)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 67, __pyx_float__0001)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_38_80);
  __Pyx_GIVEREF(__pyx_float_38_80);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 68, __pyx_float_38_80)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 69, __pyx_float__0001)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_40_02);
  __Pyx_GIVEREF(__pyx_float_40_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 70, __pyx_float_40_02)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 71, __pyx_float__0001)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_44_14);
  __Pyx_GIVEREF(__pyx_float_44_14);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 72, __pyx_float_44_14)) __PYX_ERR(0, 54, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_28, 73, __pyx_float__0001)) __PYX_ERR(0, 54, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":55
 * 			        [1.00,.0674,1.99,.0841,2.98,.0920,3.99,.0328,4.99,.0368,5.98,.0206,6.99,.0246 ,8.01,.0048,9.01,.0218,10.03,.0155,11.05,.0048,12.06,.0077,13.00,.0020,14.10,.0083,15.15,.0084 ,16.18,.0015,17.22,.0039,18.27,.0032,19.34,.0026,20.40,.0012,21.47,.0009,22.54,.0008,23.62,.0016 ,24.71,.0005,25.82,.0004,26.91,.0002,28.03,.0008,29.17,.0002,30.32,.0028,31.45,.0004,32.61,.0005 ,33.77,.0001,36.14,.0003,37.32,.0002,38.54,.0005,39.75,.0002,42.23,.0002,48.65,.0001],
 * 			        [1.01,.0423,1.99,.0240,2.98,.0517,4.00,.0493,5.00,.0324,6.00,.0094,6.99,.0449 ,7.99,.0050,9.00,.0197,10.03,.0132,11.03,.0009,12.07,.0017,13.08,.0023,14.12,.0094,15.16,.0071 ,16.21,.0020,17.25,.0005,18.30,.0027,19.04,.0004,20.43,.0022,21.51,.0002,22.59,.0006,23.72,.0018 ,24.80,.0002,25.88,.0002,27.03,.0002,28.09,.0006,29.31,.0002,30.46,.0004,31.61,.0007,32.78,.0005 ,33.95,.0001,36.34,.0002,37.56,.0001,38.80,.0001,40.02,.0001,44.14,.0001],
 * 			        [1.00,.0669,1.99,.0909,2.99,.0410,3.98,.0292,4.98,.0259,5.98,.0148,6.98,.0319 ,7.99,.0076,9.01,.0056,10.02,.0206,11.04,.0032,12.05,.0085,13.08,.0040,14.12,.0037,15.16,.0030 ,16.20,.0013,17.24,.0021,18.30,.0010,19.36,.0015,20.44,.0013,21.50,.0009,22.60,.0015,23.69,.0014 ,24.80,.0006,25.87,.0002,27.02,.0006,28.12,.0002,29.28,.0003,30.43,.0002,31.59,.0007,32.79,.0001 ,35.14,.0001,37.57,.0001,40.03,.0002,41.28,.0004,44.10,.0001],             # <<<<<<<<<<<<<<
 * 			        [0.99,.0421,1.99,.1541,2.98,.0596,3.98,.0309,4.98,.0301,5.99,.0103,7.00,.0240 ,8.01,.0073,9.01,.0222,10.04,.0140,11.05,.0033,12.08,.0045,13.13,.0009,14.13,.0015,15.21,.0026 ,16.24,.0003,17.30,.0004,18.35,.0010,19.39,.0003,20.50,.0015,21.57,.0003,22.68,.0011,23.80,.0005 ,24.90,.0008,26.02,.0002,27.16,.0001,28.30,.0006,29.48,.0002,31.81,.0005,33.00,.0003,34.21,.0001 ,37.89,.0001],
 * 			        [0.99,.0389,2.00,.2095,3.00,.0835,3.99,.0289,5.00,.0578,5.99,.0363,7.01,.0387 ,8.01,.0056,9.04,.0173,10.05,.0175,11.08,.0053,12.10,.0056,13.15,.0064,14.19,.0036,15.22,.0019 ,16.29,.0010,17.36,.0017,18.43,.0018,19.51,.0004,20.60,.0011,21.70,.0003,22.82,.0003,23.95,.0001 ,25.05,.0004,26.17,.0001,28.50,.0003,29.68,.0001,32.07,.0003,33.28,.0004,34.52,.0001],
 */
  __pyx_t_29 = PyList_New(72); if (unlikely(!__pyx_t_29)) __PYX_ERR(0, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_29);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 0, __pyx_float_1_00)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0669);
  __Pyx_GIVEREF(__pyx_float__0669);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 1, __pyx_float__0669)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_1_99);
  __Pyx_GIVEREF(__pyx_float_1_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 2, __pyx_float_1_99)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0909);
  __Pyx_GIVEREF(__pyx_float__0909);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 3, __pyx_float__0909)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_99);
  __Pyx_GIVEREF(__pyx_float_2_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 4, __pyx_float_2_99)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0410);
  __Pyx_GIVEREF(__pyx_float__0410);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 5, __pyx_float__0410)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_98);
  __Pyx_GIVEREF(__pyx_float_3_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 6, __pyx_float_3_98)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0292);
  __Pyx_GIVEREF(__pyx_float__0292);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 7, __pyx_float__0292)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_98);
  __Pyx_GIVEREF(__pyx_float_4_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 8, __pyx_float_4_98)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0259);
  __Pyx_GIVEREF(__pyx_float__0259);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 9, __pyx_float__0259)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_98);
  __Pyx_GIVEREF(__pyx_float_5_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 10, __pyx_float_5_98)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0148);
  __Pyx_GIVEREF(__pyx_float__0148);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 11, __pyx_float__0148)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_98);
  __Pyx_GIVEREF(__pyx_float_6_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 12, __pyx_float_6_98)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0319);
  __Pyx_GIVEREF(__pyx_float__0319);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 13, __pyx_float__0319)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_99);
  __Pyx_GIVEREF(__pyx_float_7_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 14, __pyx_float_7_99)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0076);
  __Pyx_GIVEREF(__pyx_float__0076);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 15, __pyx_float__0076)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_01);
  __Pyx_GIVEREF(__pyx_float_9_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 16, __pyx_float_9_01)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0056);
  __Pyx_GIVEREF(__pyx_float__0056);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 17, __pyx_float__0056)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_02);
  __Pyx_GIVEREF(__pyx_float_10_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 18, __pyx_float_10_02)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0206);
  __Pyx_GIVEREF(__pyx_float__0206);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 19, __pyx_float__0206)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_04);
  __Pyx_GIVEREF(__pyx_float_11_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 20, __pyx_float_11_04)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0032);
  __Pyx_GIVEREF(__pyx_float__0032);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 21, __pyx_float__0032)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_05);
  __Pyx_GIVEREF(__pyx_float_12_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 22, __pyx_float_12_05)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0085);
  __Pyx_GIVEREF(__pyx_float__0085);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 23, __pyx_float__0085)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_08);
  __Pyx_GIVEREF(__pyx_float_13_08);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 24, __pyx_float_13_08)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0040);
  __Pyx_GIVEREF(__pyx_float__0040);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 25, __pyx_float__0040)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_12);
  __Pyx_GIVEREF(__pyx_float_14_12);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 26, __pyx_float_14_12)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0037);
  __Pyx_GIVEREF(__pyx_float__0037);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 27, __pyx_float__0037)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_16);
  __Pyx_GIVEREF(__pyx_float_15_16);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 28, __pyx_float_15_16)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0030);
  __Pyx_GIVEREF(__pyx_float__0030);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 29, __pyx_float__0030)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_20);
  __Pyx_GIVEREF(__pyx_float_16_20);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 30, __pyx_float_16_20)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0013);
  __Pyx_GIVEREF(__pyx_float__0013);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 31, __pyx_float__0013)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_24);
  __Pyx_GIVEREF(__pyx_float_17_24);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 32, __pyx_float_17_24)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0021);
  __Pyx_GIVEREF(__pyx_float__0021);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 33, __pyx_float__0021)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_30);
  __Pyx_GIVEREF(__pyx_float_18_30);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 34, __pyx_float_18_30)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 35, __pyx_float__0010)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_36);
  __Pyx_GIVEREF(__pyx_float_19_36);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 36, __pyx_float_19_36)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0015);
  __Pyx_GIVEREF(__pyx_float__0015);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 37, __pyx_float__0015)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_44);
  __Pyx_GIVEREF(__pyx_float_20_44);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 38, __pyx_float_20_44)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0013);
  __Pyx_GIVEREF(__pyx_float__0013);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 39, __pyx_float__0013)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_50);
  __Pyx_GIVEREF(__pyx_float_21_50);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 40, __pyx_float_21_50)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 41, __pyx_float__0009)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_60);
  __Pyx_GIVEREF(__pyx_float_22_60);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 42, __pyx_float_22_60)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0015);
  __Pyx_GIVEREF(__pyx_float__0015);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 43, __pyx_float__0015)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_69);
  __Pyx_GIVEREF(__pyx_float_23_69);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 44, __pyx_float_23_69)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0014);
  __Pyx_GIVEREF(__pyx_float__0014);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 45, __pyx_float__0014)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_24_80);
  __Pyx_GIVEREF(__pyx_float_24_80);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 46, __pyx_float_24_80)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 47, __pyx_float__0006)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_25_87);
  __Pyx_GIVEREF(__pyx_float_25_87);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 48, __pyx_float_25_87)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 49, __pyx_float__0002)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_27_02);
  __Pyx_GIVEREF(__pyx_float_27_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 50, __pyx_float_27_02)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 51, __pyx_float__0006)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_12);
  __Pyx_GIVEREF(__pyx_float_28_12);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 52, __pyx_float_28_12)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 53, __pyx_float__0002)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_29_28);
  __Pyx_GIVEREF(__pyx_float_29_28);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 54, __pyx_float_29_28)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 55, __pyx_float__0003)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_30_43);
  __Pyx_GIVEREF(__pyx_float_30_43);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 56, __pyx_float_30_43)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 57, __pyx_float__0002)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_31_59);
  __Pyx_GIVEREF(__pyx_float_31_59);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 58, __pyx_float_31_59)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 59, __pyx_float__0007)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_32_79);
  __Pyx_GIVEREF(__pyx_float_32_79);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 60, __pyx_float_32_79)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 61, __pyx_float__0001)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_35_14);
  __Pyx_GIVEREF(__pyx_float_35_14);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 62, __pyx_float_35_14)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 63, __pyx_float__0001)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_37_57);
  __Pyx_GIVEREF(__pyx_float_37_57);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 64, __pyx_float_37_57)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 65, __pyx_float__0001)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_40_03);
  __Pyx_GIVEREF(__pyx_float_40_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 66, __pyx_float_40_03)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 67, __pyx_float__0002)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_41_28);
  __Pyx_GIVEREF(__pyx_float_41_28);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 68, __pyx_float_41_28)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 69, __pyx_float__0004)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_44_10);
  __Pyx_GIVEREF(__pyx_float_44_10);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 70, __pyx_float_44_10)) __PYX_ERR(0, 55, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_29, 71, __pyx_float__0001)) __PYX_ERR(0, 55, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":56
 * 			        [1.01,.0423,1.99,.0240,2.98,.0517,4.00,.0493,5.00,.0324,6.00,.0094,6.99,.0449 ,7.99,.0050,9.00,.0197,10.03,.0132,11.03,.0009,12.07,.0017,13.08,.0023,14.12,.0094,15.16,.0071 ,16.21,.0020,17.25,.0005,18.30,.0027,19.04,.0004,20.43,.0022,21.51,.0002,22.59,.0006,23.72,.0018 ,24.80,.0002,25.88,.0002,27.03,.0002,28.09,.0006,29.31,.0002,30.46,.0004,31.61,.0007,32.78,.0005 ,33.95,.0001,36.34,.0002,37.56,.0001,38.80,.0001,40.02,.0001,44.14,.0001],
 * 			        [1.00,.0669,1.99,.0909,2.99,.0410,3.98,.0292,4.98,.0259,5.98,.0148,6.98,.0319 ,7.99,.0076,9.01,.0056,10.02,.0206,11.04,.0032,12.05,.0085,13.08,.0040,14.12,.0037,15.16,.0030 ,16.20,.0013,17.24,.0021,18.30,.0010,19.36,.0015,20.44,.0013,21.50,.0009,22.60,.0015,23.69,.0014 ,24.80,.0006,25.87,.0002,27.02,.0006,28.12,.0002,29.28,.0003,30.43,.0002,31.59,.0007,32.79,.0001 ,35.14,.0001,37.57,.0001,40.03,.0002,41.28,.0004,44.10,.0001],
 * 			        [0.99,.0421,1.99,.1541,2.98,.0596,3.98,.0309,4.98,.0301,5.99,.0103,7.00,.0240 ,8.01,.0073,9.01,.0222,10.04,.0140,11.05,.0033,12.08,.0045,13.13,.0009,14.13,.0015,15.21,.0026 ,16.24,.0003,17.30,.0004,18.35,.0010,19.39,.0003,20.50,.0015,21.57,.0003,22.68,.0011,23.80,.0005 ,24.90,.0008,26.02,.0002,27.16,.0001,28.30,.0006,29.48,.0002,31.81,.0005,33.00,.0003,34.21,.0001 ,37.89,.0001],             # <<<<<<<<<<<<<<
 * 			        [0.99,.0389,2.00,.2095,3.00,.0835,3.99,.0289,5.00,.0578,5.99,.0363,7.01,.0387 ,8.01,.0056,9.04,.0173,10.05,.0175,11.08,.0053,12.10,.0056,13.15,.0064,14.19,.0036,15.22,.0019 ,16.29,.0010,17.36,.0017,18.43,.0018,19.51,.0004,20.60,.0011,21.70,.0003,22.82,.0003,23.95,.0001 ,25.05,.0004,26.17,.0001,28.50,.0003,29.68,.0001,32.07,.0003,33.28,.0004,34.52,.0001],
 * 			        [1.00,.1238,1.99,.2270,3.00,.0102,3.99,.0181,4.98,.0415,6.00,.0165,7.01,.0314 ,8.02,.0148,9.04,.0203,10.05,.0088,11.07,.0062,12.11,.0070,13.14,.0054,14.19,.0028,15.24,.0044 ,16.30,.0029,17.38,.0009,18.45,.0026,19.56,.0003,20.65,.0025,21.74,.0014,22.87,.0013,23.99,.0007 ,25.15,.0002,27.46,.0004,28.39,.0006,28.65,.0004,29.85,.0001,31.05,.0002,32.27,.0003,33.52,.0002 ,34.76,.0003],
 */
  __pyx_t_30 = PyList_New(64); if (unlikely(!__pyx_t_30)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_30);
  __Pyx_INCREF(__pyx_float_0_99);
  __Pyx_GIVEREF(__pyx_float_0_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 0, __pyx_float_0_99)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0421);
  __Pyx_GIVEREF(__pyx_float__0421);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 1, __pyx_float__0421)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_1_99);
  __Pyx_GIVEREF(__pyx_float_1_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 2, __pyx_float_1_99)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1541);
  __Pyx_GIVEREF(__pyx_float__1541);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 3, __pyx_float__1541)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_98);
  __Pyx_GIVEREF(__pyx_float_2_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 4, __pyx_float_2_98)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0596);
  __Pyx_GIVEREF(__pyx_float__0596);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 5, __pyx_float__0596)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_98);
  __Pyx_GIVEREF(__pyx_float_3_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 6, __pyx_float_3_98)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0309);
  __Pyx_GIVEREF(__pyx_float__0309);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 7, __pyx_float__0309)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_98);
  __Pyx_GIVEREF(__pyx_float_4_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 8, __pyx_float_4_98)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0301);
  __Pyx_GIVEREF(__pyx_float__0301);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 9, __pyx_float__0301)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_99);
  __Pyx_GIVEREF(__pyx_float_5_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 10, __pyx_float_5_99)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0103);
  __Pyx_GIVEREF(__pyx_float__0103);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 11, __pyx_float__0103)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_00);
  __Pyx_GIVEREF(__pyx_float_7_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 12, __pyx_float_7_00)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0240);
  __Pyx_GIVEREF(__pyx_float__0240);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 13, __pyx_float__0240)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_01);
  __Pyx_GIVEREF(__pyx_float_8_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 14, __pyx_float_8_01)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0073);
  __Pyx_GIVEREF(__pyx_float__0073);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 15, __pyx_float__0073)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_01);
  __Pyx_GIVEREF(__pyx_float_9_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 16, __pyx_float_9_01)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0222);
  __Pyx_GIVEREF(__pyx_float__0222);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 17, __pyx_float__0222)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_04);
  __Pyx_GIVEREF(__pyx_float_10_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 18, __pyx_float_10_04)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0140);
  __Pyx_GIVEREF(__pyx_float__0140);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 19, __pyx_float__0140)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_05);
  __Pyx_GIVEREF(__pyx_float_11_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 20, __pyx_float_11_05)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0033);
  __Pyx_GIVEREF(__pyx_float__0033);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 21, __pyx_float__0033)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_08);
  __Pyx_GIVEREF(__pyx_float_12_08);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 22, __pyx_float_12_08)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0045);
  __Pyx_GIVEREF(__pyx_float__0045);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 23, __pyx_float__0045)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_13);
  __Pyx_GIVEREF(__pyx_float_13_13);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 24, __pyx_float_13_13)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 25, __pyx_float__0009)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_13);
  __Pyx_GIVEREF(__pyx_float_14_13);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 26, __pyx_float_14_13)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0015);
  __Pyx_GIVEREF(__pyx_float__0015);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 27, __pyx_float__0015)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_21);
  __Pyx_GIVEREF(__pyx_float_15_21);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 28, __pyx_float_15_21)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0026);
  __Pyx_GIVEREF(__pyx_float__0026);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 29, __pyx_float__0026)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_24);
  __Pyx_GIVEREF(__pyx_float_16_24);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 30, __pyx_float_16_24)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 31, __pyx_float__0003)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_30);
  __Pyx_GIVEREF(__pyx_float_17_30);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 32, __pyx_float_17_30)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 33, __pyx_float__0004)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_35);
  __Pyx_GIVEREF(__pyx_float_18_35);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 34, __pyx_float_18_35)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 35, __pyx_float__0010)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_39);
  __Pyx_GIVEREF(__pyx_float_19_39);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 36, __pyx_float_19_39)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 37, __pyx_float__0003)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_50);
  __Pyx_GIVEREF(__pyx_float_20_50);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 38, __pyx_float_20_50)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0015);
  __Pyx_GIVEREF(__pyx_float__0015);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 39, __pyx_float__0015)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_57);
  __Pyx_GIVEREF(__pyx_float_21_57);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 40, __pyx_float_21_57)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 41, __pyx_float__0003)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_68);
  __Pyx_GIVEREF(__pyx_float_22_68);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 42, __pyx_float_22_68)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 43, __pyx_float__0011)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_80);
  __Pyx_GIVEREF(__pyx_float_23_80);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 44, __pyx_float_23_80)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 45, __pyx_float__0005)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_24_90);
  __Pyx_GIVEREF(__pyx_float_24_90);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 46, __pyx_float_24_90)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 47, __pyx_float__0008)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_26_02);
  __Pyx_GIVEREF(__pyx_float_26_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 48, __pyx_float_26_02)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 49, __pyx_float__0002)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_27_16);
  __Pyx_GIVEREF(__pyx_float_27_16);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 50, __pyx_float_27_16)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 51, __pyx_float__0001)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_30);
  __Pyx_GIVEREF(__pyx_float_28_30);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 52, __pyx_float_28_30)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 53, __pyx_float__0006)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_29_48);
  __Pyx_GIVEREF(__pyx_float_29_48);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 54, __pyx_float_29_48)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 55, __pyx_float__0002)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_31_81);
  __Pyx_GIVEREF(__pyx_float_31_81);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 56, __pyx_float_31_81)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 57, __pyx_float__0005)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_33_00);
  __Pyx_GIVEREF(__pyx_float_33_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 58, __pyx_float_33_00)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 59, __pyx_float__0003)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_34_21);
  __Pyx_GIVEREF(__pyx_float_34_21);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 60, __pyx_float_34_21)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 61, __pyx_float__0001)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_37_89);
  __Pyx_GIVEREF(__pyx_float_37_89);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 62, __pyx_float_37_89)) __PYX_ERR(0, 56, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_30, 63, __pyx_float__0001)) __PYX_ERR(0, 56, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":57
 * 			        [1.00,.0669,1.99,.0909,2.99,.0410,3.98,.0292,4.98,.0259,5.98,.0148,6.98,.0319 ,7.99,.0076,9.01,.0056,10.02,.0206,11.04,.0032,12.05,.0085,13.08,.0040,14.12,.0037,15.16,.0030 ,16.20,.0013,17.24,.0021,18.30,.0010,19.36,.0015,20.44,.0013,21.50,.0009,22.60,.0015,23.69,.0014 ,24.80,.0006,25.87,.0002,27.02,.0006,28.12,.0002,29.28,.0003,30.43,.0002,31.59,.0007,32.79,.0001 ,35.14,.0001,37.57,.0001,40.03,.0002,41.28,.0004,44.10,.0001],
 * 			        [0.99,.0421,1.99,.1541,2.98,.0596,3.98,.0309,4.98,.0301,5.99,.0103,7.00,.0240 ,8.01,.0073,9.01,.0222,10.04,.0140,11.05,.0033,12.08,.0045,13.13,.0009,14.13,.0015,15.21,.0026 ,16.24,.0003,17.30,.0004,18.35,.0010,19.39,.0003,20.50,.0015,21.57,.0003,22.68,.0011,23.80,.0005 ,24.90,.0008,26.02,.0002,27.16,.0001,28.30,.0006,29.48,.0002,31.81,.0005,33.00,.0003,34.21,.0001 ,37.89,.0001],
 * 			        [0.99,.0389,2.00,.2095,3.00,.0835,3.99,.0289,5.00,.0578,5.99,.0363,7.01,.0387 ,8.01,.0056,9.04,.0173,10.05,.0175,11.08,.0053,12.10,.0056,13.15,.0064,14.19,.0036,15.22,.0019 ,16.29,.0010,17.36,.0017,18.43,.0018,19.51,.0004,20.60,.0011,21.70,.0003,22.82,.0003,23.95,.0001 ,25.05,.0004,26.17,.0001,28.50,.0003,29.68,.0001,32.07,.0003,33.28,.0004,34.52,.0001],             # <<<<<<<<<<<<<<
 * 			        [1.00,.1238,1.99,.2270,3.00,.0102,3.99,.0181,4.98,.0415,6.00,.0165,7.01,.0314 ,8.02,.0148,9.04,.0203,10.05,.0088,11.07,.0062,12.11,.0070,13.14,.0054,14.19,.0028,15.24,.0044 ,16.30,.0029,17.38,.0009,18.45,.0026,19.56,.0003,20.65,.0025,21.74,.0014,22.87,.0013,23.99,.0007 ,25.15,.0002,27.46,.0004,28.39,.0006,28.65,.0004,29.85,.0001,31.05,.0002,32.27,.0003,33.52,.0002 ,34.76,.0003],
 * 			        [1.00,.1054,2.00,.2598,2.99,.0369,3.98,.0523,4.99,.0020,5.99,.0051,7.00,.0268 ,8.01,.0027,9.04,.0029,10.05,.0081,11.08,.0047,12.12,.0051,13.16,.0091,14.19,.0015,15.27,.0030 ,16.34,.0017,17.42,.0006,18.51,.0003,19.61,.0007,20.72,.0003,21.84,.0001,22.99,.0010,24.13,.0001 ,28.44,.0001,30.09,.0001],
 */
  __pyx_t_31 = PyList_New(60); if (unlikely(!__pyx_t_31)) __PYX_ERR(0, 57, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_31);
  __Pyx_INCREF(__pyx_float_0_99);
  __Pyx_GIVEREF(__pyx_float_0_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 0, __pyx_float_0_99)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0389);
  __Pyx_GIVEREF(__pyx_float__0389);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 1, __pyx_float__0389)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_00);
  __Pyx_GIVEREF(__pyx_float_2_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 2, __pyx_float_2_00)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__2095);
  __Pyx_GIVEREF(__pyx_float__2095);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 3, __pyx_float__2095)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_00);
  __Pyx_GIVEREF(__pyx_float_3_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 4, __pyx_float_3_00)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0835);
  __Pyx_GIVEREF(__pyx_float__0835);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 5, __pyx_float__0835)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_99);
  __Pyx_GIVEREF(__pyx_float_3_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 6, __pyx_float_3_99)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0289);
  __Pyx_GIVEREF(__pyx_float__0289);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 7, __pyx_float__0289)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_00);
  __Pyx_GIVEREF(__pyx_float_5_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 8, __pyx_float_5_00)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0578);
  __Pyx_GIVEREF(__pyx_float__0578);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 9, __pyx_float__0578)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_99);
  __Pyx_GIVEREF(__pyx_float_5_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 10, __pyx_float_5_99)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0363);
  __Pyx_GIVEREF(__pyx_float__0363);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 11, __pyx_float__0363)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_01);
  __Pyx_GIVEREF(__pyx_float_7_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 12, __pyx_float_7_01)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0387);
  __Pyx_GIVEREF(__pyx_float__0387);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 13, __pyx_float__0387)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_01);
  __Pyx_GIVEREF(__pyx_float_8_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 14, __pyx_float_8_01)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0056);
  __Pyx_GIVEREF(__pyx_float__0056);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 15, __pyx_float__0056)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_04);
  __Pyx_GIVEREF(__pyx_float_9_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 16, __pyx_float_9_04)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0173);
  __Pyx_GIVEREF(__pyx_float__0173);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 17, __pyx_float__0173)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_05);
  __Pyx_GIVEREF(__pyx_float_10_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 18, __pyx_float_10_05)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0175);
  __Pyx_GIVEREF(__pyx_float__0175);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 19, __pyx_float__0175)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_08);
  __Pyx_GIVEREF(__pyx_float_11_08);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 20, __pyx_float_11_08)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0053);
  __Pyx_GIVEREF(__pyx_float__0053);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 21, __pyx_float__0053)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_10);
  __Pyx_GIVEREF(__pyx_float_12_10);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 22, __pyx_float_12_10)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0056);
  __Pyx_GIVEREF(__pyx_float__0056);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 23, __pyx_float__0056)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_15);
  __Pyx_GIVEREF(__pyx_float_13_15);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 24, __pyx_float_13_15)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0064);
  __Pyx_GIVEREF(__pyx_float__0064);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 25, __pyx_float__0064)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_19);
  __Pyx_GIVEREF(__pyx_float_14_19);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 26, __pyx_float_14_19)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0036);
  __Pyx_GIVEREF(__pyx_float__0036);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 27, __pyx_float__0036)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_22);
  __Pyx_GIVEREF(__pyx_float_15_22);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 28, __pyx_float_15_22)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0019);
  __Pyx_GIVEREF(__pyx_float__0019);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 29, __pyx_float__0019)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_29);
  __Pyx_GIVEREF(__pyx_float_16_29);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 30, __pyx_float_16_29)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 31, __pyx_float__0010)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_36);
  __Pyx_GIVEREF(__pyx_float_17_36);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 32, __pyx_float_17_36)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0017);
  __Pyx_GIVEREF(__pyx_float__0017);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 33, __pyx_float__0017)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_43);
  __Pyx_GIVEREF(__pyx_float_18_43);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 34, __pyx_float_18_43)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0018);
  __Pyx_GIVEREF(__pyx_float__0018);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 35, __pyx_float__0018)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_51);
  __Pyx_GIVEREF(__pyx_float_19_51);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 36, __pyx_float_19_51)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 37, __pyx_float__0004)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_60);
  __Pyx_GIVEREF(__pyx_float_20_60);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 38, __pyx_float_20_60)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 39, __pyx_float__0011)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_70);
  __Pyx_GIVEREF(__pyx_float_21_70);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 40, __pyx_float_21_70)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 41, __pyx_float__0003)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_82);
  __Pyx_GIVEREF(__pyx_float_22_82);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 42, __pyx_float_22_82)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 43, __pyx_float__0003)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_95);
  __Pyx_GIVEREF(__pyx_float_23_95);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 44, __pyx_float_23_95)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 45, __pyx_float__0001)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_25_05);
  __Pyx_GIVEREF(__pyx_float_25_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 46, __pyx_float_25_05)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 47, __pyx_float__0004)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_26_17);
  __Pyx_GIVEREF(__pyx_float_26_17);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 48, __pyx_float_26_17)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 49, __pyx_float__0001)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_50);
  __Pyx_GIVEREF(__pyx_float_28_50);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 50, __pyx_float_28_50)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 51, __pyx_float__0003)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_29_68);
  __Pyx_GIVEREF(__pyx_float_29_68);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 52, __pyx_float_29_68)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 53, __pyx_float__0001)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_32_07);
  __Pyx_GIVEREF(__pyx_float_32_07);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 54, __pyx_float_32_07)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 55, __pyx_float__0003)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_33_28);
  __Pyx_GIVEREF(__pyx_float_33_28);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 56, __pyx_float_33_28)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 57, __pyx_float__0004)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_34_52);
  __Pyx_GIVEREF(__pyx_float_34_52);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 58, __pyx_float_34_52)) __PYX_ERR(0, 57, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_31, 59, __pyx_float__0001)) __PYX_ERR(0, 57, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":58
 * 			        [0.99,.0421,1.99,.1541,2.98,.0596,3.98,.0309,4.98,.0301,5.99,.0103,7.00,.0240 ,8.01,.0073,9.01,.0222,10.04,.0140,11.05,.0033,12.08,.0045,13.13,.0009,14.13,.0015,15.21,.0026 ,16.24,.0003,17.30,.0004,18.35,.0010,19.39,.0003,20.50,.0015,21.57,.0003,22.68,.0011,23.80,.0005 ,24.90,.0008,26.02,.0002,27.16,.0001,28.30,.0006,29.48,.0002,31.81,.0005,33.00,.0003,34.21,.0001 ,37.89,.0001],
 * 			        [0.99,.0389,2.00,.2095,3.00,.0835,3.99,.0289,5.00,.0578,5.99,.0363,7.01,.0387 ,8.01,.0056,9.04,.0173,10.05,.0175,11.08,.0053,12.10,.0056,13.15,.0064,14.19,.0036,15.22,.0019 ,16.29,.0010,17.36,.0017,18.43,.0018,19.51,.0004,20.60,.0011,21.70,.0003,22.82,.0003,23.95,.0001 ,25.05,.0004,26.17,.0001,28.50,.0003,29.68,.0001,32.07,.0003,33.28,.0004,34.52,.0001],
 * 			        [1.00,.1238,1.99,.2270,3.00,.0102,3.99,.0181,4.98,.0415,6.00,.0165,7.01,.0314 ,8.02,.0148,9.04,.0203,10.05,.0088,11.07,.0062,12.11,.0070,13.14,.0054,14.19,.0028,15.24,.0044 ,16.30,.0029,17.38,.0009,18.45,.0026,19.56,.0003,20.65,.0025,21.74,.0014,22.87,.0013,23.99,.0007 ,25.15,.0002,27.46,.0004,28.39,.0006,28.65,.0004,29.85,.0001,31.05,.0002,32.27,.0003,33.52,.0002 ,34.76,.0003],             # <<<<<<<<<<<<<<
 * 			        [1.00,.1054,2.00,.2598,2.99,.0369,3.98,.0523,4.99,.0020,5.99,.0051,7.00,.0268 ,8.01,.0027,9.04,.0029,10.05,.0081,11.08,.0047,12.12,.0051,13.16,.0091,14.19,.0015,15.27,.0030 ,16.34,.0017,17.42,.0006,18.51,.0003,19.61,.0007,20.72,.0003,21.84,.0001,22.99,.0010,24.13,.0001 ,28.44,.0001,30.09,.0001],
 * 			        [0.99,.0919,2.00,.0418,2.99,.0498,3.99,.0135,4.99,.0026,6.00,.0155,7.01,.0340 ,8.02,.0033,9.04,.0218,10.08,.0084,11.11,.0057,12.15,.0051,13.21,.0043,14.25,.0015,15.31,.0023 ,16.40,.0008,17.48,.0004,18.59,.0016,19.71,.0010,20.84,.0018,21.98,.0002,23.11,.0013,24.26,.0003 ,26.67,.0002,29.12,.0002,30.37,.0002,31.62,.0003,32.92,.0001],
 */
  __pyx_t_32 = PyList_New(64); if (unlikely(!__pyx_t_32)) __PYX_ERR(0, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_32);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 0, __pyx_float_1_00)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1238);
  __Pyx_GIVEREF(__pyx_float__1238);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 1, __pyx_float__1238)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_1_99);
  __Pyx_GIVEREF(__pyx_float_1_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 2, __pyx_float_1_99)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__2270);
  __Pyx_GIVEREF(__pyx_float__2270);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 3, __pyx_float__2270)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_00);
  __Pyx_GIVEREF(__pyx_float_3_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 4, __pyx_float_3_00)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0102);
  __Pyx_GIVEREF(__pyx_float__0102);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 5, __pyx_float__0102)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_99);
  __Pyx_GIVEREF(__pyx_float_3_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 6, __pyx_float_3_99)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0181);
  __Pyx_GIVEREF(__pyx_float__0181);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 7, __pyx_float__0181)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_98);
  __Pyx_GIVEREF(__pyx_float_4_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 8, __pyx_float_4_98)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0415);
  __Pyx_GIVEREF(__pyx_float__0415);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 9, __pyx_float__0415)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_00);
  __Pyx_GIVEREF(__pyx_float_6_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 10, __pyx_float_6_00)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0165);
  __Pyx_GIVEREF(__pyx_float__0165);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 11, __pyx_float__0165)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_01);
  __Pyx_GIVEREF(__pyx_float_7_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 12, __pyx_float_7_01)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0314);
  __Pyx_GIVEREF(__pyx_float__0314);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 13, __pyx_float__0314)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_02);
  __Pyx_GIVEREF(__pyx_float_8_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 14, __pyx_float_8_02)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0148);
  __Pyx_GIVEREF(__pyx_float__0148);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 15, __pyx_float__0148)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_04);
  __Pyx_GIVEREF(__pyx_float_9_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 16, __pyx_float_9_04)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0203);
  __Pyx_GIVEREF(__pyx_float__0203);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 17, __pyx_float__0203)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_05);
  __Pyx_GIVEREF(__pyx_float_10_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 18, __pyx_float_10_05)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0088);
  __Pyx_GIVEREF(__pyx_float__0088);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 19, __pyx_float__0088)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_07);
  __Pyx_GIVEREF(__pyx_float_11_07);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 20, __pyx_float_11_07)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0062);
  __Pyx_GIVEREF(__pyx_float__0062);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 21, __pyx_float__0062)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_11);
  __Pyx_GIVEREF(__pyx_float_12_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 22, __pyx_float_12_11)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0070);
  __Pyx_GIVEREF(__pyx_float__0070);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 23, __pyx_float__0070)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_14);
  __Pyx_GIVEREF(__pyx_float_13_14);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 24, __pyx_float_13_14)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0054);
  __Pyx_GIVEREF(__pyx_float__0054);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 25, __pyx_float__0054)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_19);
  __Pyx_GIVEREF(__pyx_float_14_19);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 26, __pyx_float_14_19)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0028);
  __Pyx_GIVEREF(__pyx_float__0028);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 27, __pyx_float__0028)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_24);
  __Pyx_GIVEREF(__pyx_float_15_24);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 28, __pyx_float_15_24)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0044);
  __Pyx_GIVEREF(__pyx_float__0044);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 29, __pyx_float__0044)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_30);
  __Pyx_GIVEREF(__pyx_float_16_30);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 30, __pyx_float_16_30)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0029);
  __Pyx_GIVEREF(__pyx_float__0029);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 31, __pyx_float__0029)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_38);
  __Pyx_GIVEREF(__pyx_float_17_38);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 32, __pyx_float_17_38)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 33, __pyx_float__0009)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_45);
  __Pyx_GIVEREF(__pyx_float_18_45);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 34, __pyx_float_18_45)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0026);
  __Pyx_GIVEREF(__pyx_float__0026);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 35, __pyx_float__0026)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_56);
  __Pyx_GIVEREF(__pyx_float_19_56);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 36, __pyx_float_19_56)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 37, __pyx_float__0003)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_65);
  __Pyx_GIVEREF(__pyx_float_20_65);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 38, __pyx_float_20_65)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0025);
  __Pyx_GIVEREF(__pyx_float__0025);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 39, __pyx_float__0025)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_74);
  __Pyx_GIVEREF(__pyx_float_21_74);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 40, __pyx_float_21_74)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0014);
  __Pyx_GIVEREF(__pyx_float__0014);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 41, __pyx_float__0014)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_87);
  __Pyx_GIVEREF(__pyx_float_22_87);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 42, __pyx_float_22_87)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0013);
  __Pyx_GIVEREF(__pyx_float__0013);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 43, __pyx_float__0013)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_99);
  __Pyx_GIVEREF(__pyx_float_23_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 44, __pyx_float_23_99)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 45, __pyx_float__0007)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_25_15);
  __Pyx_GIVEREF(__pyx_float_25_15);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 46, __pyx_float_25_15)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 47, __pyx_float__0002)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_27_46);
  __Pyx_GIVEREF(__pyx_float_27_46);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 48, __pyx_float_27_46)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 49, __pyx_float__0004)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_39);
  __Pyx_GIVEREF(__pyx_float_28_39);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 50, __pyx_float_28_39)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 51, __pyx_float__0006)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_65);
  __Pyx_GIVEREF(__pyx_float_28_65);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 52, __pyx_float_28_65)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 53, __pyx_float__0004)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_29_85);
  __Pyx_GIVEREF(__pyx_float_29_85);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 54, __pyx_float_29_85)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 55, __pyx_float__0001)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_31_05);
  __Pyx_GIVEREF(__pyx_float_31_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 56, __pyx_float_31_05)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 57, __pyx_float__0002)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_32_27);
  __Pyx_GIVEREF(__pyx_float_32_27);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 58, __pyx_float_32_27)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 59, __pyx_float__0003)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_33_52);
  __Pyx_GIVEREF(__pyx_float_33_52);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 60, __pyx_float_33_52)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 61, __pyx_float__0002)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_34_76);
  __Pyx_GIVEREF(__pyx_float_34_76);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 62, __pyx_float_34_76)) __PYX_ERR(0, 58, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_32, 63, __pyx_float__0003)) __PYX_ERR(0, 58, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":59
 * 			        [0.99,.0389,2.00,.2095,3.00,.0835,3.99,.0289,5.00,.0578,5.99,.0363,7.01,.0387 ,8.01,.0056,9.04,.0173,10.05,.0175,11.08,.0053,12.10,.0056,13.15,.0064,14.19,.0036,15.22,.0019 ,16.29,.0010,17.36,.0017,18.43,.0018,19.51,.0004,20.60,.0011,21.70,.0003,22.82,.0003,23.95,.0001 ,25.05,.0004,26.17,.0001,28.50,.0003,29.68,.0001,32.07,.0003,33.28,.0004,34.52,.0001],
 * 			        [1.00,.1238,1.99,.2270,3.00,.0102,3.99,.0181,4.98,.0415,6.00,.0165,7.01,.0314 ,8.02,.0148,9.04,.0203,10.05,.0088,11.07,.0062,12.11,.0070,13.14,.0054,14.19,.0028,15.24,.0044 ,16.30,.0029,17.38,.0009,18.45,.0026,19.56,.0003,20.65,.0025,21.74,.0014,22.87,.0013,23.99,.0007 ,25.15,.0002,27.46,.0004,28.39,.0006,28.65,.0004,29.85,.0001,31.05,.0002,32.27,.0003,33.52,.0002 ,34.76,.0003],
 * 			        [1.00,.1054,2.00,.2598,2.99,.0369,3.98,.0523,4.99,.0020,5.99,.0051,7.00,.0268 ,8.01,.0027,9.04,.0029,10.05,.0081,11.08,.0047,12.12,.0051,13.16,.0091,14.19,.0015,15.27,.0030 ,16.34,.0017,17.42,.0006,18.51,.0003,19.61,.0007,20.72,.0003,21.84,.0001,22.99,.0010,24.13,.0001 ,28.44,.0001,30.09,.0001],             # <<<<<<<<<<<<<<
 * 			        [0.99,.0919,2.00,.0418,2.99,.0498,3.99,.0135,4.99,.0026,6.00,.0155,7.01,.0340 ,8.02,.0033,9.04,.0218,10.08,.0084,11.11,.0057,12.15,.0051,13.21,.0043,14.25,.0015,15.31,.0023 ,16.40,.0008,17.48,.0004,18.59,.0016,19.71,.0010,20.84,.0018,21.98,.0002,23.11,.0013,24.26,.0003 ,26.67,.0002,29.12,.0002,30.37,.0002,31.62,.0003,32.92,.0001],
 * 			        [0.99,.1174,1.99,.1126,2.99,.0370,3.99,.0159,5.01,.0472,6.01,.0091,7.03,.0211 ,8.05,.0015,9.07,.0098,10.11,.0038,11.15,.0042,12.20,.0018,13.24,.0041,14.32,.0033,15.41,.0052 ,16.49,.0001,17.61,.0004,18.71,.0004,19.84,.0004,20.99,.0002,22.14,.0006,23.31,.0006,24.50,.0004 ,25.70,.0002,28.09,.0002,28.66,.0002,32.00,.0001],
 */
  __pyx_t_33 = PyList_New(50); if (unlikely(!__pyx_t_33)) __PYX_ERR(0, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_33);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 0, __pyx_float_1_00)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1054);
  __Pyx_GIVEREF(__pyx_float__1054);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 1, __pyx_float__1054)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_00);
  __Pyx_GIVEREF(__pyx_float_2_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 2, __pyx_float_2_00)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__2598);
  __Pyx_GIVEREF(__pyx_float__2598);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 3, __pyx_float__2598)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_99);
  __Pyx_GIVEREF(__pyx_float_2_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 4, __pyx_float_2_99)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0369);
  __Pyx_GIVEREF(__pyx_float__0369);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 5, __pyx_float__0369)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_98);
  __Pyx_GIVEREF(__pyx_float_3_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 6, __pyx_float_3_98)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0523);
  __Pyx_GIVEREF(__pyx_float__0523);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 7, __pyx_float__0523)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_99);
  __Pyx_GIVEREF(__pyx_float_4_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 8, __pyx_float_4_99)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0020);
  __Pyx_GIVEREF(__pyx_float__0020);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 9, __pyx_float__0020)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_99);
  __Pyx_GIVEREF(__pyx_float_5_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 10, __pyx_float_5_99)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0051);
  __Pyx_GIVEREF(__pyx_float__0051);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 11, __pyx_float__0051)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_00);
  __Pyx_GIVEREF(__pyx_float_7_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 12, __pyx_float_7_00)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0268);
  __Pyx_GIVEREF(__pyx_float__0268);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 13, __pyx_float__0268)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_01);
  __Pyx_GIVEREF(__pyx_float_8_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 14, __pyx_float_8_01)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0027);
  __Pyx_GIVEREF(__pyx_float__0027);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 15, __pyx_float__0027)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_04);
  __Pyx_GIVEREF(__pyx_float_9_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 16, __pyx_float_9_04)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0029);
  __Pyx_GIVEREF(__pyx_float__0029);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 17, __pyx_float__0029)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_05);
  __Pyx_GIVEREF(__pyx_float_10_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 18, __pyx_float_10_05)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0081);
  __Pyx_GIVEREF(__pyx_float__0081);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 19, __pyx_float__0081)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_08);
  __Pyx_GIVEREF(__pyx_float_11_08);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 20, __pyx_float_11_08)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0047);
  __Pyx_GIVEREF(__pyx_float__0047);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 21, __pyx_float__0047)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_12);
  __Pyx_GIVEREF(__pyx_float_12_12);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 22, __pyx_float_12_12)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0051);
  __Pyx_GIVEREF(__pyx_float__0051);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 23, __pyx_float__0051)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_16);
  __Pyx_GIVEREF(__pyx_float_13_16);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 24, __pyx_float_13_16)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0091);
  __Pyx_GIVEREF(__pyx_float__0091);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 25, __pyx_float__0091)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_19);
  __Pyx_GIVEREF(__pyx_float_14_19);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 26, __pyx_float_14_19)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0015);
  __Pyx_GIVEREF(__pyx_float__0015);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 27, __pyx_float__0015)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_27);
  __Pyx_GIVEREF(__pyx_float_15_27);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 28, __pyx_float_15_27)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0030);
  __Pyx_GIVEREF(__pyx_float__0030);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 29, __pyx_float__0030)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_34);
  __Pyx_GIVEREF(__pyx_float_16_34);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 30, __pyx_float_16_34)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0017);
  __Pyx_GIVEREF(__pyx_float__0017);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 31, __pyx_float__0017)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_42);
  __Pyx_GIVEREF(__pyx_float_17_42);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 32, __pyx_float_17_42)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 33, __pyx_float__0006)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_51);
  __Pyx_GIVEREF(__pyx_float_18_51);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 34, __pyx_float_18_51)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 35, __pyx_float__0003)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_61);
  __Pyx_GIVEREF(__pyx_float_19_61);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 36, __pyx_float_19_61)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 37, __pyx_float__0007)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_72);
  __Pyx_GIVEREF(__pyx_float_20_72);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 38, __pyx_float_20_72)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 39, __pyx_float__0003)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_84);
  __Pyx_GIVEREF(__pyx_float_21_84);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 40, __pyx_float_21_84)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 41, __pyx_float__0001)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_99);
  __Pyx_GIVEREF(__pyx_float_22_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 42, __pyx_float_22_99)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 43, __pyx_float__0010)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_24_13);
  __Pyx_GIVEREF(__pyx_float_24_13);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 44, __pyx_float_24_13)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 45, __pyx_float__0001)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_44);
  __Pyx_GIVEREF(__pyx_float_28_44);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 46, __pyx_float_28_44)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 47, __pyx_float__0001)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_30_09);
  __Pyx_GIVEREF(__pyx_float_30_09);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 48, __pyx_float_30_09)) __PYX_ERR(0, 59, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_33, 49, __pyx_float__0001)) __PYX_ERR(0, 59, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":60
 * 			        [1.00,.1238,1.99,.2270,3.00,.0102,3.99,.0181,4.98,.0415,6.00,.0165,7.01,.0314 ,8.02,.0148,9.04,.0203,10.05,.0088,11.07,.0062,12.11,.0070,13.14,.0054,14.19,.0028,15.24,.0044 ,16.30,.0029,17.38,.0009,18.45,.0026,19.56,.0003,20.65,.0025,21.74,.0014,22.87,.0013,23.99,.0007 ,25.15,.0002,27.46,.0004,28.39,.0006,28.65,.0004,29.85,.0001,31.05,.0002,32.27,.0003,33.52,.0002 ,34.76,.0003],
 * 			        [1.00,.1054,2.00,.2598,2.99,.0369,3.98,.0523,4.99,.0020,5.99,.0051,7.00,.0268 ,8.01,.0027,9.04,.0029,10.05,.0081,11.08,.0047,12.12,.0051,13.16,.0091,14.19,.0015,15.27,.0030 ,16.34,.0017,17.42,.0006,18.51,.0003,19.61,.0007,20.72,.0003,21.84,.0001,22.99,.0010,24.13,.0001 ,28.44,.0001,30.09,.0001],
 * 			        [0.99,.0919,2.00,.0418,2.99,.0498,3.99,.0135,4.99,.0026,6.00,.0155,7.01,.0340 ,8.02,.0033,9.04,.0218,10.08,.0084,11.11,.0057,12.15,.0051,13.21,.0043,14.25,.0015,15.31,.0023 ,16.40,.0008,17.48,.0004,18.59,.0016,19.71,.0010,20.84,.0018,21.98,.0002,23.11,.0013,24.26,.0003 ,26.67,.0002,29.12,.0002,30.37,.0002,31.62,.0003,32.92,.0001],             # <<<<<<<<<<<<<<
 * 			        [0.99,.1174,1.99,.1126,2.99,.0370,3.99,.0159,5.01,.0472,6.01,.0091,7.03,.0211 ,8.05,.0015,9.07,.0098,10.11,.0038,11.15,.0042,12.20,.0018,13.24,.0041,14.32,.0033,15.41,.0052 ,16.49,.0001,17.61,.0004,18.71,.0004,19.84,.0004,20.99,.0002,22.14,.0006,23.31,.0006,24.50,.0004 ,25.70,.0002,28.09,.0002,28.66,.0002,32.00,.0001],
 * 			        [1.00,.1085,2.00,.1400,2.99,.0173,3.99,.0229,5.00,.0272,6.02,.0077,7.03,.0069 ,8.04,.0017,9.08,.0045,10.10,.0030,11.15,.0040,12.20,.0007,13.25,.0019,14.32,.0008,15.42,.0024 ,16.50,.0002,17.59,.0005,18.71,.0003,19.83,.0002,20.98,.0005,23.29,.0008],
 */
  __pyx_t_34 = PyList_New(56); if (unlikely(!__pyx_t_34)) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_34);
  __Pyx_INCREF(__pyx_float_0_99);
  __Pyx_GIVEREF(__pyx_float_0_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 0, __pyx_float_0_99)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0919);
  __Pyx_GIVEREF(__pyx_float__0919);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 1, __pyx_float__0919)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_00);
  __Pyx_GIVEREF(__pyx_float_2_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 2, __pyx_float_2_00)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0418);
  __Pyx_GIVEREF(__pyx_float__0418);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 3, __pyx_float__0418)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_99);
  __Pyx_GIVEREF(__pyx_float_2_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 4, __pyx_float_2_99)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0498);
  __Pyx_GIVEREF(__pyx_float__0498);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 5, __pyx_float__0498)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_99);
  __Pyx_GIVEREF(__pyx_float_3_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 6, __pyx_float_3_99)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0135);
  __Pyx_GIVEREF(__pyx_float__0135);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 7, __pyx_float__0135)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_99);
  __Pyx_GIVEREF(__pyx_float_4_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 8, __pyx_float_4_99)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0026);
  __Pyx_GIVEREF(__pyx_float__0026);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 9, __pyx_float__0026)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_00);
  __Pyx_GIVEREF(__pyx_float_6_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 10, __pyx_float_6_00)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0155);
  __Pyx_GIVEREF(__pyx_float__0155);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 11, __pyx_float__0155)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_01);
  __Pyx_GIVEREF(__pyx_float_7_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 12, __pyx_float_7_01)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0340);
  __Pyx_GIVEREF(__pyx_float__0340);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 13, __pyx_float__0340)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_02);
  __Pyx_GIVEREF(__pyx_float_8_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 14, __pyx_float_8_02)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0033);
  __Pyx_GIVEREF(__pyx_float__0033);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 15, __pyx_float__0033)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_04);
  __Pyx_GIVEREF(__pyx_float_9_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 16, __pyx_float_9_04)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0218);
  __Pyx_GIVEREF(__pyx_float__0218);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 17, __pyx_float__0218)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_08);
  __Pyx_GIVEREF(__pyx_float_10_08);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 18, __pyx_float_10_08)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0084);
  __Pyx_GIVEREF(__pyx_float__0084);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 19, __pyx_float__0084)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_11);
  __Pyx_GIVEREF(__pyx_float_11_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 20, __pyx_float_11_11)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0057);
  __Pyx_GIVEREF(__pyx_float__0057);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 21, __pyx_float__0057)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_15);
  __Pyx_GIVEREF(__pyx_float_12_15);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 22, __pyx_float_12_15)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0051);
  __Pyx_GIVEREF(__pyx_float__0051);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 23, __pyx_float__0051)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_21);
  __Pyx_GIVEREF(__pyx_float_13_21);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 24, __pyx_float_13_21)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0043);
  __Pyx_GIVEREF(__pyx_float__0043);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 25, __pyx_float__0043)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_25);
  __Pyx_GIVEREF(__pyx_float_14_25);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 26, __pyx_float_14_25)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0015);
  __Pyx_GIVEREF(__pyx_float__0015);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 27, __pyx_float__0015)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_31);
  __Pyx_GIVEREF(__pyx_float_15_31);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 28, __pyx_float_15_31)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0023);
  __Pyx_GIVEREF(__pyx_float__0023);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 29, __pyx_float__0023)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_40);
  __Pyx_GIVEREF(__pyx_float_16_40);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 30, __pyx_float_16_40)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 31, __pyx_float__0008)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_48);
  __Pyx_GIVEREF(__pyx_float_17_48);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 32, __pyx_float_17_48)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 33, __pyx_float__0004)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_59);
  __Pyx_GIVEREF(__pyx_float_18_59);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 34, __pyx_float_18_59)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0016);
  __Pyx_GIVEREF(__pyx_float__0016);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 35, __pyx_float__0016)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_71);
  __Pyx_GIVEREF(__pyx_float_19_71);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 36, __pyx_float_19_71)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 37, __pyx_float__0010)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_84);
  __Pyx_GIVEREF(__pyx_float_20_84);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 38, __pyx_float_20_84)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0018);
  __Pyx_GIVEREF(__pyx_float__0018);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 39, __pyx_float__0018)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_98);
  __Pyx_GIVEREF(__pyx_float_21_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 40, __pyx_float_21_98)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 41, __pyx_float__0002)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_11);
  __Pyx_GIVEREF(__pyx_float_23_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 42, __pyx_float_23_11)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0013);
  __Pyx_GIVEREF(__pyx_float__0013);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 43, __pyx_float__0013)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_24_26);
  __Pyx_GIVEREF(__pyx_float_24_26);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 44, __pyx_float_24_26)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 45, __pyx_float__0003)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_26_67);
  __Pyx_GIVEREF(__pyx_float_26_67);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 46, __pyx_float_26_67)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 47, __pyx_float__0002)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_29_12);
  __Pyx_GIVEREF(__pyx_float_29_12);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 48, __pyx_float_29_12)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 49, __pyx_float__0002)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_30_37);
  __Pyx_GIVEREF(__pyx_float_30_37);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 50, __pyx_float_30_37)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 51, __pyx_float__0002)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_31_62);
  __Pyx_GIVEREF(__pyx_float_31_62);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 52, __pyx_float_31_62)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 53, __pyx_float__0003)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_32_92);
  __Pyx_GIVEREF(__pyx_float_32_92);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 54, __pyx_float_32_92)) __PYX_ERR(0, 60, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_34, 55, __pyx_float__0001)) __PYX_ERR(0, 60, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":61
 * 			        [1.00,.1054,2.00,.2598,2.99,.0369,3.98,.0523,4.99,.0020,5.99,.0051,7.00,.0268 ,8.01,.0027,9.04,.0029,10.05,.0081,11.08,.0047,12.12,.0051,13.16,.0091,14.19,.0015,15.27,.0030 ,16.34,.0017,17.42,.0006,18.51,.0003,19.61,.0007,20.72,.0003,21.84,.0001,22.99,.0010,24.13,.0001 ,28.44,.0001,30.09,.0001],
 * 			        [0.99,.0919,2.00,.0418,2.99,.0498,3.99,.0135,4.99,.0026,6.00,.0155,7.01,.0340 ,8.02,.0033,9.04,.0218,10.08,.0084,11.11,.0057,12.15,.0051,13.21,.0043,14.25,.0015,15.31,.0023 ,16.40,.0008,17.48,.0004,18.59,.0016,19.71,.0010,20.84,.0018,21.98,.0002,23.11,.0013,24.26,.0003 ,26.67,.0002,29.12,.0002,30.37,.0002,31.62,.0003,32.92,.0001],
 * 			        [0.99,.1174,1.99,.1126,2.99,.0370,3.99,.0159,5.01,.0472,6.01,.0091,7.03,.0211 ,8.05,.0015,9.07,.0098,10.11,.0038,11.15,.0042,12.20,.0018,13.24,.0041,14.32,.0033,15.41,.0052 ,16.49,.0001,17.61,.0004,18.71,.0004,19.84,.0004,20.99,.0002,22.14,.0006,23.31,.0006,24.50,.0004 ,25.70,.0002,28.09,.0002,28.66,.0002,32.00,.0001],             # <<<<<<<<<<<<<<
 * 			        [1.00,.1085,2.00,.1400,2.99,.0173,3.99,.0229,5.00,.0272,6.02,.0077,7.03,.0069 ,8.04,.0017,9.08,.0045,10.10,.0030,11.15,.0040,12.20,.0007,13.25,.0019,14.32,.0008,15.42,.0024 ,16.50,.0002,17.59,.0005,18.71,.0003,19.83,.0002,20.98,.0005,23.29,.0008],
 * 			        [1.00,.0985,2.00,.1440,2.99,.0364,3.99,.0425,5.00,.0190,6.01,.0089,7.03,.0278 ,8.04,.0006,9.07,.0083,10.10,.0021,11.14,.0050,12.18,.0005,13.26,.0036,14.33,.0005,15.41,.0026 ,17.62,.0004,18.75,.0004,19.89,.0003,21.04,.0012,22.21,.0002,23.38,.0004,27.04,.0001],
 */
  __pyx_t_35 = PyList_New(54); if (unlikely(!__pyx_t_35)) __PYX_ERR(0, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_35);
  __Pyx_INCREF(__pyx_float_0_99);
  __Pyx_GIVEREF(__pyx_float_0_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 0, __pyx_float_0_99)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1174);
  __Pyx_GIVEREF(__pyx_float__1174);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 1, __pyx_float__1174)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_1_99);
  __Pyx_GIVEREF(__pyx_float_1_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 2, __pyx_float_1_99)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1126);
  __Pyx_GIVEREF(__pyx_float__1126);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 3, __pyx_float__1126)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_99);
  __Pyx_GIVEREF(__pyx_float_2_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 4, __pyx_float_2_99)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0370);
  __Pyx_GIVEREF(__pyx_float__0370);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 5, __pyx_float__0370)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_99);
  __Pyx_GIVEREF(__pyx_float_3_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 6, __pyx_float_3_99)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0159);
  __Pyx_GIVEREF(__pyx_float__0159);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 7, __pyx_float__0159)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_01);
  __Pyx_GIVEREF(__pyx_float_5_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 8, __pyx_float_5_01)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0472);
  __Pyx_GIVEREF(__pyx_float__0472);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 9, __pyx_float__0472)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_01);
  __Pyx_GIVEREF(__pyx_float_6_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 10, __pyx_float_6_01)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0091);
  __Pyx_GIVEREF(__pyx_float__0091);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 11, __pyx_float__0091)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_03);
  __Pyx_GIVEREF(__pyx_float_7_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 12, __pyx_float_7_03)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0211);
  __Pyx_GIVEREF(__pyx_float__0211);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 13, __pyx_float__0211)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_05);
  __Pyx_GIVEREF(__pyx_float_8_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 14, __pyx_float_8_05)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0015);
  __Pyx_GIVEREF(__pyx_float__0015);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 15, __pyx_float__0015)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_07);
  __Pyx_GIVEREF(__pyx_float_9_07);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 16, __pyx_float_9_07)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0098);
  __Pyx_GIVEREF(__pyx_float__0098);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 17, __pyx_float__0098)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_11);
  __Pyx_GIVEREF(__pyx_float_10_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 18, __pyx_float_10_11)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0038);
  __Pyx_GIVEREF(__pyx_float__0038);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 19, __pyx_float__0038)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_15);
  __Pyx_GIVEREF(__pyx_float_11_15);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 20, __pyx_float_11_15)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0042);
  __Pyx_GIVEREF(__pyx_float__0042);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 21, __pyx_float__0042)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_20);
  __Pyx_GIVEREF(__pyx_float_12_20);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 22, __pyx_float_12_20)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0018);
  __Pyx_GIVEREF(__pyx_float__0018);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 23, __pyx_float__0018)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_24);
  __Pyx_GIVEREF(__pyx_float_13_24);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 24, __pyx_float_13_24)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0041);
  __Pyx_GIVEREF(__pyx_float__0041);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 25, __pyx_float__0041)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_32);
  __Pyx_GIVEREF(__pyx_float_14_32);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 26, __pyx_float_14_32)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0033);
  __Pyx_GIVEREF(__pyx_float__0033);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 27, __pyx_float__0033)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_41);
  __Pyx_GIVEREF(__pyx_float_15_41);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 28, __pyx_float_15_41)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0052);
  __Pyx_GIVEREF(__pyx_float__0052);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 29, __pyx_float__0052)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_49);
  __Pyx_GIVEREF(__pyx_float_16_49);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 30, __pyx_float_16_49)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 31, __pyx_float__0001)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_61);
  __Pyx_GIVEREF(__pyx_float_17_61);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 32, __pyx_float_17_61)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 33, __pyx_float__0004)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_71);
  __Pyx_GIVEREF(__pyx_float_18_71);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 34, __pyx_float_18_71)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 35, __pyx_float__0004)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_84);
  __Pyx_GIVEREF(__pyx_float_19_84);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 36, __pyx_float_19_84)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 37, __pyx_float__0004)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_99);
  __Pyx_GIVEREF(__pyx_float_20_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 38, __pyx_float_20_99)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 39, __pyx_float__0002)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_14);
  __Pyx_GIVEREF(__pyx_float_22_14);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 40, __pyx_float_22_14)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 41, __pyx_float__0006)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_31);
  __Pyx_GIVEREF(__pyx_float_23_31);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 42, __pyx_float_23_31)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 43, __pyx_float__0006)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_24_50);
  __Pyx_GIVEREF(__pyx_float_24_50);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 44, __pyx_float_24_50)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 45, __pyx_float__0004)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_25_70);
  __Pyx_GIVEREF(__pyx_float_25_70);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 46, __pyx_float_25_70)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 47, __pyx_float__0002)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_09);
  __Pyx_GIVEREF(__pyx_float_28_09);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 48, __pyx_float_28_09)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 49, __pyx_float__0002)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_66);
  __Pyx_GIVEREF(__pyx_float_28_66);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 50, __pyx_float_28_66)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 51, __pyx_float__0002)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_32_00);
  __Pyx_GIVEREF(__pyx_float_32_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 52, __pyx_float_32_00)) __PYX_ERR(0, 61, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_35, 53, __pyx_float__0001)) __PYX_ERR(0, 61, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":62
 * 			        [0.99,.0919,2.00,.0418,2.99,.0498,3.99,.0135,4.99,.0026,6.00,.0155,7.01,.0340 ,8.02,.0033,9.04,.0218,10.08,.0084,11.11,.0057,12.15,.0051,13.21,.0043,14.25,.0015,15.31,.0023 ,16.40,.0008,17.48,.0004,18.59,.0016,19.71,.0010,20.84,.0018,21.98,.0002,23.11,.0013,24.26,.0003 ,26.67,.0002,29.12,.0002,30.37,.0002,31.62,.0003,32.92,.0001],
 * 			        [0.99,.1174,1.99,.1126,2.99,.0370,3.99,.0159,5.01,.0472,6.01,.0091,7.03,.0211 ,8.05,.0015,9.07,.0098,10.11,.0038,11.15,.0042,12.20,.0018,13.24,.0041,14.32,.0033,15.41,.0052 ,16.49,.0001,17.61,.0004,18.71,.0004,19.84,.0004,20.99,.0002,22.14,.0006,23.31,.0006,24.50,.0004 ,25.70,.0002,28.09,.0002,28.66,.0002,32.00,.0001],
 * 			        [1.00,.1085,2.00,.1400,2.99,.0173,3.99,.0229,5.00,.0272,6.02,.0077,7.03,.0069 ,8.04,.0017,9.08,.0045,10.10,.0030,11.15,.0040,12.20,.0007,13.25,.0019,14.32,.0008,15.42,.0024 ,16.50,.0002,17.59,.0005,18.71,.0003,19.83,.0002,20.98,.0005,23.29,.0008],             # <<<<<<<<<<<<<<
 * 			        [1.00,.0985,2.00,.1440,2.99,.0364,3.99,.0425,5.00,.0190,6.01,.0089,7.03,.0278 ,8.04,.0006,9.07,.0083,10.10,.0021,11.14,.0050,12.18,.0005,13.26,.0036,14.33,.0005,15.41,.0026 ,17.62,.0004,18.75,.0004,19.89,.0003,21.04,.0012,22.21,.0002,23.38,.0004,27.04,.0001],
 * 		            [0.99,.1273,2.00,.1311,2.99,.0120,4.00,.0099,5.00,.0235,6.02,.0068,7.03,.0162 ,8.06,.0009,9.08,.0083,10.12,.0014,11.17,.0050,12.24,.0010,13.29,.0013,14.39,.0022,15.48,.0011 ,16.59,.0002,17.70,.0003,18.84,.0010,20.00,.0003,21.17,.0003,23.56,.0004,28.79,.0003],
 */
  __pyx_t_36 = PyList_New(42); if (unlikely(!__pyx_t_36)) __PYX_ERR(0, 62, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_36);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 0, __pyx_float_1_00)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1085);
  __Pyx_GIVEREF(__pyx_float__1085);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 1, __pyx_float__1085)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_00);
  __Pyx_GIVEREF(__pyx_float_2_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 2, __pyx_float_2_00)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1400);
  __Pyx_GIVEREF(__pyx_float__1400);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 3, __pyx_float__1400)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_99);
  __Pyx_GIVEREF(__pyx_float_2_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 4, __pyx_float_2_99)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0173);
  __Pyx_GIVEREF(__pyx_float__0173);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 5, __pyx_float__0173)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_99);
  __Pyx_GIVEREF(__pyx_float_3_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 6, __pyx_float_3_99)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0229);
  __Pyx_GIVEREF(__pyx_float__0229);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 7, __pyx_float__0229)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_00);
  __Pyx_GIVEREF(__pyx_float_5_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 8, __pyx_float_5_00)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0272);
  __Pyx_GIVEREF(__pyx_float__0272);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 9, __pyx_float__0272)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_02);
  __Pyx_GIVEREF(__pyx_float_6_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 10, __pyx_float_6_02)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0077);
  __Pyx_GIVEREF(__pyx_float__0077);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 11, __pyx_float__0077)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_03);
  __Pyx_GIVEREF(__pyx_float_7_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 12, __pyx_float_7_03)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0069);
  __Pyx_GIVEREF(__pyx_float__0069);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 13, __pyx_float__0069)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_04);
  __Pyx_GIVEREF(__pyx_float_8_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 14, __pyx_float_8_04)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0017);
  __Pyx_GIVEREF(__pyx_float__0017);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 15, __pyx_float__0017)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_08);
  __Pyx_GIVEREF(__pyx_float_9_08);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 16, __pyx_float_9_08)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0045);
  __Pyx_GIVEREF(__pyx_float__0045);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 17, __pyx_float__0045)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_10);
  __Pyx_GIVEREF(__pyx_float_10_10);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 18, __pyx_float_10_10)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0030);
  __Pyx_GIVEREF(__pyx_float__0030);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 19, __pyx_float__0030)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_15);
  __Pyx_GIVEREF(__pyx_float_11_15);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 20, __pyx_float_11_15)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0040);
  __Pyx_GIVEREF(__pyx_float__0040);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 21, __pyx_float__0040)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_20);
  __Pyx_GIVEREF(__pyx_float_12_20);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 22, __pyx_float_12_20)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 23, __pyx_float__0007)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_25);
  __Pyx_GIVEREF(__pyx_float_13_25);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 24, __pyx_float_13_25)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0019);
  __Pyx_GIVEREF(__pyx_float__0019);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 25, __pyx_float__0019)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_32);
  __Pyx_GIVEREF(__pyx_float_14_32);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 26, __pyx_float_14_32)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 27, __pyx_float__0008)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_42);
  __Pyx_GIVEREF(__pyx_float_15_42);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 28, __pyx_float_15_42)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0024);
  __Pyx_GIVEREF(__pyx_float__0024);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 29, __pyx_float__0024)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_50);
  __Pyx_GIVEREF(__pyx_float_16_50);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 30, __pyx_float_16_50)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 31, __pyx_float__0002)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_59);
  __Pyx_GIVEREF(__pyx_float_17_59);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 32, __pyx_float_17_59)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 33, __pyx_float__0005)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_71);
  __Pyx_GIVEREF(__pyx_float_18_71);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 34, __pyx_float_18_71)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 35, __pyx_float__0003)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_83);
  __Pyx_GIVEREF(__pyx_float_19_83);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 36, __pyx_float_19_83)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 37, __pyx_float__0002)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_98);
  __Pyx_GIVEREF(__pyx_float_20_98);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 38, __pyx_float_20_98)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 39, __pyx_float__0005)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_29);
  __Pyx_GIVEREF(__pyx_float_23_29);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 40, __pyx_float_23_29)) __PYX_ERR(0, 62, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_36, 41, __pyx_float__0008)) __PYX_ERR(0, 62, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":63
 * 			        [0.99,.1174,1.99,.1126,2.99,.0370,3.99,.0159,5.01,.0472,6.01,.0091,7.03,.0211 ,8.05,.0015,9.07,.0098,10.11,.0038,11.15,.0042,12.20,.0018,13.24,.0041,14.32,.0033,15.41,.0052 ,16.49,.0001,17.61,.0004,18.71,.0004,19.84,.0004,20.99,.0002,22.14,.0006,23.31,.0006,24.50,.0004 ,25.70,.0002,28.09,.0002,28.66,.0002,32.00,.0001],
 * 			        [1.00,.1085,2.00,.1400,2.99,.0173,3.99,.0229,5.00,.0272,6.02,.0077,7.03,.0069 ,8.04,.0017,9.08,.0045,10.10,.0030,11.15,.0040,12.20,.0007,13.25,.0019,14.32,.0008,15.42,.0024 ,16.50,.0002,17.59,.0005,18.71,.0003,19.83,.0002,20.98,.0005,23.29,.0008],
 * 			        [1.00,.0985,2.00,.1440,2.99,.0364,3.99,.0425,5.00,.0190,6.01,.0089,7.03,.0278 ,8.04,.0006,9.07,.0083,10.10,.0021,11.14,.0050,12.18,.0005,13.26,.0036,14.33,.0005,15.41,.0026 ,17.62,.0004,18.75,.0004,19.89,.0003,21.04,.0012,22.21,.0002,23.38,.0004,27.04,.0001],             # <<<<<<<<<<<<<<
 * 		            [0.99,.1273,2.00,.1311,2.99,.0120,4.00,.0099,5.00,.0235,6.02,.0068,7.03,.0162 ,8.06,.0009,9.08,.0083,10.12,.0014,11.17,.0050,12.24,.0010,13.29,.0013,14.39,.0022,15.48,.0011 ,16.59,.0002,17.70,.0003,18.84,.0010,20.00,.0003,21.17,.0003,23.56,.0004,28.79,.0003],
 * 		            [1.00,.1018,2.00,.1486,3.00,.0165,4.00,.0186,5.01,.0194,6.02,.0045,7.04,.0083 ,8.06,.0012,9.10,.0066,10.15,.0009,11.19,.0008,12.26,.0011,13.34,.0028,14.45,.0006,15.53,.0009 ,16.66,.0002,17.79,.0006,18.94,.0005,20.11,.0003,21.29,.0005,22.49,.0003,23.73,.0005,26.22,.0001 ,27.52,.0001,28.88,.0002],
 */
  __pyx_t_37 = PyList_New(44); if (unlikely(!__pyx_t_37)) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_37);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 0, __pyx_float_1_00)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0985);
  __Pyx_GIVEREF(__pyx_float__0985);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 1, __pyx_float__0985)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_00);
  __Pyx_GIVEREF(__pyx_float_2_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 2, __pyx_float_2_00)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1440);
  __Pyx_GIVEREF(__pyx_float__1440);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 3, __pyx_float__1440)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_99);
  __Pyx_GIVEREF(__pyx_float_2_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 4, __pyx_float_2_99)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0364);
  __Pyx_GIVEREF(__pyx_float__0364);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 5, __pyx_float__0364)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_99);
  __Pyx_GIVEREF(__pyx_float_3_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 6, __pyx_float_3_99)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0425);
  __Pyx_GIVEREF(__pyx_float__0425);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 7, __pyx_float__0425)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_00);
  __Pyx_GIVEREF(__pyx_float_5_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 8, __pyx_float_5_00)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0190);
  __Pyx_GIVEREF(__pyx_float__0190);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 9, __pyx_float__0190)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_01);
  __Pyx_GIVEREF(__pyx_float_6_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 10, __pyx_float_6_01)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0089);
  __Pyx_GIVEREF(__pyx_float__0089);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 11, __pyx_float__0089)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_03);
  __Pyx_GIVEREF(__pyx_float_7_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 12, __pyx_float_7_03)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0278);
  __Pyx_GIVEREF(__pyx_float__0278);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 13, __pyx_float__0278)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_04);
  __Pyx_GIVEREF(__pyx_float_8_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 14, __pyx_float_8_04)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 15, __pyx_float__0006)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_07);
  __Pyx_GIVEREF(__pyx_float_9_07);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 16, __pyx_float_9_07)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0083);
  __Pyx_GIVEREF(__pyx_float__0083);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 17, __pyx_float__0083)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_10);
  __Pyx_GIVEREF(__pyx_float_10_10);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 18, __pyx_float_10_10)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0021);
  __Pyx_GIVEREF(__pyx_float__0021);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 19, __pyx_float__0021)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_14);
  __Pyx_GIVEREF(__pyx_float_11_14);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 20, __pyx_float_11_14)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0050);
  __Pyx_GIVEREF(__pyx_float__0050);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 21, __pyx_float__0050)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_18);
  __Pyx_GIVEREF(__pyx_float_12_18);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 22, __pyx_float_12_18)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 23, __pyx_float__0005)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_26);
  __Pyx_GIVEREF(__pyx_float_13_26);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 24, __pyx_float_13_26)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0036);
  __Pyx_GIVEREF(__pyx_float__0036);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 25, __pyx_float__0036)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_33);
  __Pyx_GIVEREF(__pyx_float_14_33);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 26, __pyx_float_14_33)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 27, __pyx_float__0005)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_41);
  __Pyx_GIVEREF(__pyx_float_15_41);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 28, __pyx_float_15_41)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0026);
  __Pyx_GIVEREF(__pyx_float__0026);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 29, __pyx_float__0026)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_62);
  __Pyx_GIVEREF(__pyx_float_17_62);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 30, __pyx_float_17_62)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 31, __pyx_float__0004)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_75);
  __Pyx_GIVEREF(__pyx_float_18_75);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 32, __pyx_float_18_75)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 33, __pyx_float__0004)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_89);
  __Pyx_GIVEREF(__pyx_float_19_89);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 34, __pyx_float_19_89)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 35, __pyx_float__0003)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_04);
  __Pyx_GIVEREF(__pyx_float_21_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 36, __pyx_float_21_04)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 37, __pyx_float__0012)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_21);
  __Pyx_GIVEREF(__pyx_float_22_21);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 38, __pyx_float_22_21)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 39, __pyx_float__0002)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_38);
  __Pyx_GIVEREF(__pyx_float_23_38);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 40, __pyx_float_23_38)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 41, __pyx_float__0004)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_27_04);
  __Pyx_GIVEREF(__pyx_float_27_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 42, __pyx_float_27_04)) __PYX_ERR(0, 63, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_37, 43, __pyx_float__0001)) __PYX_ERR(0, 63, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":64
 * 			        [1.00,.1085,2.00,.1400,2.99,.0173,3.99,.0229,5.00,.0272,6.02,.0077,7.03,.0069 ,8.04,.0017,9.08,.0045,10.10,.0030,11.15,.0040,12.20,.0007,13.25,.0019,14.32,.0008,15.42,.0024 ,16.50,.0002,17.59,.0005,18.71,.0003,19.83,.0002,20.98,.0005,23.29,.0008],
 * 			        [1.00,.0985,2.00,.1440,2.99,.0364,3.99,.0425,5.00,.0190,6.01,.0089,7.03,.0278 ,8.04,.0006,9.07,.0083,10.10,.0021,11.14,.0050,12.18,.0005,13.26,.0036,14.33,.0005,15.41,.0026 ,17.62,.0004,18.75,.0004,19.89,.0003,21.04,.0012,22.21,.0002,23.38,.0004,27.04,.0001],
 * 		            [0.99,.1273,2.00,.1311,2.99,.0120,4.00,.0099,5.00,.0235,6.02,.0068,7.03,.0162 ,8.06,.0009,9.08,.0083,10.12,.0014,11.17,.0050,12.24,.0010,13.29,.0013,14.39,.0022,15.48,.0011 ,16.59,.0002,17.70,.0003,18.84,.0010,20.00,.0003,21.17,.0003,23.56,.0004,28.79,.0003],             # <<<<<<<<<<<<<<
 * 		            [1.00,.1018,2.00,.1486,3.00,.0165,4.00,.0186,5.01,.0194,6.02,.0045,7.04,.0083 ,8.06,.0012,9.10,.0066,10.15,.0009,11.19,.0008,12.26,.0011,13.34,.0028,14.45,.0006,15.53,.0009 ,16.66,.0002,17.79,.0006,18.94,.0005,20.11,.0003,21.29,.0005,22.49,.0003,23.73,.0005,26.22,.0001 ,27.52,.0001,28.88,.0002],
 * 			        [1.00,.1889,1.99,.1822,3.00,.0363,4.00,.0047,5.01,.0202,6.03,.0053,7.05,.0114 ,8.01,.0002,9.13,.0048,10.17,.0010,11.23,.0033,12.30,.0010,13.38,.0006,14.50,.0002,15.62,.0010 ,20.27,.0001,21.47,.0001],
 */
  __pyx_t_38 = PyList_New(44); if (unlikely(!__pyx_t_38)) __PYX_ERR(0, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_38);
  __Pyx_INCREF(__pyx_float_0_99);
  __Pyx_GIVEREF(__pyx_float_0_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 0, __pyx_float_0_99)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1273);
  __Pyx_GIVEREF(__pyx_float__1273);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 1, __pyx_float__1273)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_00);
  __Pyx_GIVEREF(__pyx_float_2_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 2, __pyx_float_2_00)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1311);
  __Pyx_GIVEREF(__pyx_float__1311);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 3, __pyx_float__1311)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_99);
  __Pyx_GIVEREF(__pyx_float_2_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 4, __pyx_float_2_99)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0120);
  __Pyx_GIVEREF(__pyx_float__0120);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 5, __pyx_float__0120)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_00);
  __Pyx_GIVEREF(__pyx_float_4_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 6, __pyx_float_4_00)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0099);
  __Pyx_GIVEREF(__pyx_float__0099);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 7, __pyx_float__0099)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_00);
  __Pyx_GIVEREF(__pyx_float_5_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 8, __pyx_float_5_00)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0235);
  __Pyx_GIVEREF(__pyx_float__0235);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 9, __pyx_float__0235)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_02);
  __Pyx_GIVEREF(__pyx_float_6_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 10, __pyx_float_6_02)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0068);
  __Pyx_GIVEREF(__pyx_float__0068);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 11, __pyx_float__0068)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_03);
  __Pyx_GIVEREF(__pyx_float_7_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 12, __pyx_float_7_03)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0162);
  __Pyx_GIVEREF(__pyx_float__0162);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 13, __pyx_float__0162)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_06);
  __Pyx_GIVEREF(__pyx_float_8_06);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 14, __pyx_float_8_06)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 15, __pyx_float__0009)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_08);
  __Pyx_GIVEREF(__pyx_float_9_08);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 16, __pyx_float_9_08)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0083);
  __Pyx_GIVEREF(__pyx_float__0083);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 17, __pyx_float__0083)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_12);
  __Pyx_GIVEREF(__pyx_float_10_12);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 18, __pyx_float_10_12)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0014);
  __Pyx_GIVEREF(__pyx_float__0014);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 19, __pyx_float__0014)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_17);
  __Pyx_GIVEREF(__pyx_float_11_17);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 20, __pyx_float_11_17)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0050);
  __Pyx_GIVEREF(__pyx_float__0050);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 21, __pyx_float__0050)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_24);
  __Pyx_GIVEREF(__pyx_float_12_24);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 22, __pyx_float_12_24)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 23, __pyx_float__0010)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_29);
  __Pyx_GIVEREF(__pyx_float_13_29);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 24, __pyx_float_13_29)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0013);
  __Pyx_GIVEREF(__pyx_float__0013);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 25, __pyx_float__0013)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_39);
  __Pyx_GIVEREF(__pyx_float_14_39);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 26, __pyx_float_14_39)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0022);
  __Pyx_GIVEREF(__pyx_float__0022);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 27, __pyx_float__0022)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_48);
  __Pyx_GIVEREF(__pyx_float_15_48);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 28, __pyx_float_15_48)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 29, __pyx_float__0011)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_59);
  __Pyx_GIVEREF(__pyx_float_16_59);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 30, __pyx_float_16_59)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 31, __pyx_float__0002)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_70);
  __Pyx_GIVEREF(__pyx_float_17_70);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 32, __pyx_float_17_70)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 33, __pyx_float__0003)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_84);
  __Pyx_GIVEREF(__pyx_float_18_84);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 34, __pyx_float_18_84)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 35, __pyx_float__0010)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_00);
  __Pyx_GIVEREF(__pyx_float_20_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 36, __pyx_float_20_00)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 37, __pyx_float__0003)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_17);
  __Pyx_GIVEREF(__pyx_float_21_17);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 38, __pyx_float_21_17)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 39, __pyx_float__0003)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_56);
  __Pyx_GIVEREF(__pyx_float_23_56);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 40, __pyx_float_23_56)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 41, __pyx_float__0004)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_79);
  __Pyx_GIVEREF(__pyx_float_28_79);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 42, __pyx_float_28_79)) __PYX_ERR(0, 64, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_38, 43, __pyx_float__0003)) __PYX_ERR(0, 64, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":65
 * 			        [1.00,.0985,2.00,.1440,2.99,.0364,3.99,.0425,5.00,.0190,6.01,.0089,7.03,.0278 ,8.04,.0006,9.07,.0083,10.10,.0021,11.14,.0050,12.18,.0005,13.26,.0036,14.33,.0005,15.41,.0026 ,17.62,.0004,18.75,.0004,19.89,.0003,21.04,.0012,22.21,.0002,23.38,.0004,27.04,.0001],
 * 		            [0.99,.1273,2.00,.1311,2.99,.0120,4.00,.0099,5.00,.0235,6.02,.0068,7.03,.0162 ,8.06,.0009,9.08,.0083,10.12,.0014,11.17,.0050,12.24,.0010,13.29,.0013,14.39,.0022,15.48,.0011 ,16.59,.0002,17.70,.0003,18.84,.0010,20.00,.0003,21.17,.0003,23.56,.0004,28.79,.0003],
 * 		            [1.00,.1018,2.00,.1486,3.00,.0165,4.00,.0186,5.01,.0194,6.02,.0045,7.04,.0083 ,8.06,.0012,9.10,.0066,10.15,.0009,11.19,.0008,12.26,.0011,13.34,.0028,14.45,.0006,15.53,.0009 ,16.66,.0002,17.79,.0006,18.94,.0005,20.11,.0003,21.29,.0005,22.49,.0003,23.73,.0005,26.22,.0001 ,27.52,.0001,28.88,.0002],             # <<<<<<<<<<<<<<
 * 			        [1.00,.1889,1.99,.1822,3.00,.0363,4.00,.0047,5.01,.0202,6.03,.0053,7.05,.0114 ,8.01,.0002,9.13,.0048,10.17,.0010,11.23,.0033,12.30,.0010,13.38,.0006,14.50,.0002,15.62,.0010 ,20.27,.0001,21.47,.0001],
 * 			        [1.00,.0522,1.99,.0763,2.99,.0404,4.00,.0139,5.01,.0185,6.01,.0021,7.06,.0045 ,8.09,.0002,9.11,.0003,10.17,.0006,11.25,.0004,12.32,.0005,13.40,.0003,14.53,.0003,15.65,.0007 ,16.80,.0001,17.95,.0002,19.14,.0006,20.34,.0002,21.56,.0003],
 */
  __pyx_t_39 = PyList_New(50); if (unlikely(!__pyx_t_39)) __PYX_ERR(0, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_39);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 0, __pyx_float_1_00)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1018);
  __Pyx_GIVEREF(__pyx_float__1018);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 1, __pyx_float__1018)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_00);
  __Pyx_GIVEREF(__pyx_float_2_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 2, __pyx_float_2_00)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1486);
  __Pyx_GIVEREF(__pyx_float__1486);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 3, __pyx_float__1486)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_00);
  __Pyx_GIVEREF(__pyx_float_3_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 4, __pyx_float_3_00)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0165);
  __Pyx_GIVEREF(__pyx_float__0165);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 5, __pyx_float__0165)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_00);
  __Pyx_GIVEREF(__pyx_float_4_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 6, __pyx_float_4_00)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0186);
  __Pyx_GIVEREF(__pyx_float__0186);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 7, __pyx_float__0186)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_01);
  __Pyx_GIVEREF(__pyx_float_5_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 8, __pyx_float_5_01)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0194);
  __Pyx_GIVEREF(__pyx_float__0194);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 9, __pyx_float__0194)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_02);
  __Pyx_GIVEREF(__pyx_float_6_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 10, __pyx_float_6_02)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0045);
  __Pyx_GIVEREF(__pyx_float__0045);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 11, __pyx_float__0045)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_04);
  __Pyx_GIVEREF(__pyx_float_7_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 12, __pyx_float_7_04)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0083);
  __Pyx_GIVEREF(__pyx_float__0083);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 13, __pyx_float__0083)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_06);
  __Pyx_GIVEREF(__pyx_float_8_06);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 14, __pyx_float_8_06)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 15, __pyx_float__0012)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_10);
  __Pyx_GIVEREF(__pyx_float_9_10);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 16, __pyx_float_9_10)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0066);
  __Pyx_GIVEREF(__pyx_float__0066);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 17, __pyx_float__0066)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_15);
  __Pyx_GIVEREF(__pyx_float_10_15);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 18, __pyx_float_10_15)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 19, __pyx_float__0009)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_19);
  __Pyx_GIVEREF(__pyx_float_11_19);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 20, __pyx_float_11_19)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 21, __pyx_float__0008)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_26);
  __Pyx_GIVEREF(__pyx_float_12_26);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 22, __pyx_float_12_26)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 23, __pyx_float__0011)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_34);
  __Pyx_GIVEREF(__pyx_float_13_34);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 24, __pyx_float_13_34)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0028);
  __Pyx_GIVEREF(__pyx_float__0028);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 25, __pyx_float__0028)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_45);
  __Pyx_GIVEREF(__pyx_float_14_45);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 26, __pyx_float_14_45)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 27, __pyx_float__0006)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_53);
  __Pyx_GIVEREF(__pyx_float_15_53);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 28, __pyx_float_15_53)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 29, __pyx_float__0009)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_66);
  __Pyx_GIVEREF(__pyx_float_16_66);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 30, __pyx_float_16_66)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 31, __pyx_float__0002)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_79);
  __Pyx_GIVEREF(__pyx_float_17_79);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 32, __pyx_float_17_79)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 33, __pyx_float__0006)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_94);
  __Pyx_GIVEREF(__pyx_float_18_94);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 34, __pyx_float_18_94)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 35, __pyx_float__0005)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_11);
  __Pyx_GIVEREF(__pyx_float_20_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 36, __pyx_float_20_11)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 37, __pyx_float__0003)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_29);
  __Pyx_GIVEREF(__pyx_float_21_29);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 38, __pyx_float_21_29)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 39, __pyx_float__0005)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_22_49);
  __Pyx_GIVEREF(__pyx_float_22_49);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 40, __pyx_float_22_49)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 41, __pyx_float__0003)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_23_73);
  __Pyx_GIVEREF(__pyx_float_23_73);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 42, __pyx_float_23_73)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 43, __pyx_float__0005)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_26_22);
  __Pyx_GIVEREF(__pyx_float_26_22);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 44, __pyx_float_26_22)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 45, __pyx_float__0001)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_27_52);
  __Pyx_GIVEREF(__pyx_float_27_52);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 46, __pyx_float_27_52)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 47, __pyx_float__0001)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_28_88);
  __Pyx_GIVEREF(__pyx_float_28_88);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 48, __pyx_float_28_88)) __PYX_ERR(0, 65, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_39, 49, __pyx_float__0002)) __PYX_ERR(0, 65, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":66
 * 		            [0.99,.1273,2.00,.1311,2.99,.0120,4.00,.0099,5.00,.0235,6.02,.0068,7.03,.0162 ,8.06,.0009,9.08,.0083,10.12,.0014,11.17,.0050,12.24,.0010,13.29,.0013,14.39,.0022,15.48,.0011 ,16.59,.0002,17.70,.0003,18.84,.0010,20.00,.0003,21.17,.0003,23.56,.0004,28.79,.0003],
 * 		            [1.00,.1018,2.00,.1486,3.00,.0165,4.00,.0186,5.01,.0194,6.02,.0045,7.04,.0083 ,8.06,.0012,9.10,.0066,10.15,.0009,11.19,.0008,12.26,.0011,13.34,.0028,14.45,.0006,15.53,.0009 ,16.66,.0002,17.79,.0006,18.94,.0005,20.11,.0003,21.29,.0005,22.49,.0003,23.73,.0005,26.22,.0001 ,27.52,.0001,28.88,.0002],
 * 			        [1.00,.1889,1.99,.1822,3.00,.0363,4.00,.0047,5.01,.0202,6.03,.0053,7.05,.0114 ,8.01,.0002,9.13,.0048,10.17,.0010,11.23,.0033,12.30,.0010,13.38,.0006,14.50,.0002,15.62,.0010 ,20.27,.0001,21.47,.0001],             # <<<<<<<<<<<<<<
 * 			        [1.00,.0522,1.99,.0763,2.99,.0404,4.00,.0139,5.01,.0185,6.01,.0021,7.06,.0045 ,8.09,.0002,9.11,.0003,10.17,.0006,11.25,.0004,12.32,.0005,13.40,.0003,14.53,.0003,15.65,.0007 ,16.80,.0001,17.95,.0002,19.14,.0006,20.34,.0002,21.56,.0003],
 * 			        [0.99,.1821,1.99,.0773,3.00,.0125,4.01,.0065,5.01,.0202,6.03,.0071,7.05,.0090 ,8.08,.0006,9.13,.0008,10.18,.0013,11.25,.0010,12.33,.0012,13.42,.0006,14.54,.0005,15.65,.0004 ,17.97,.0002,19.15,.0001],
 */
  __pyx_t_40 = PyList_New(34); if (unlikely(!__pyx_t_40)) __PYX_ERR(0, 66, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_40);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 0, __pyx_float_1_00)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1889);
  __Pyx_GIVEREF(__pyx_float__1889);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 1, __pyx_float__1889)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_1_99);
  __Pyx_GIVEREF(__pyx_float_1_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 2, __pyx_float_1_99)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1822);
  __Pyx_GIVEREF(__pyx_float__1822);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 3, __pyx_float__1822)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_00);
  __Pyx_GIVEREF(__pyx_float_3_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 4, __pyx_float_3_00)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0363);
  __Pyx_GIVEREF(__pyx_float__0363);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 5, __pyx_float__0363)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_00);
  __Pyx_GIVEREF(__pyx_float_4_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 6, __pyx_float_4_00)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0047);
  __Pyx_GIVEREF(__pyx_float__0047);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 7, __pyx_float__0047)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_01);
  __Pyx_GIVEREF(__pyx_float_5_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 8, __pyx_float_5_01)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0202);
  __Pyx_GIVEREF(__pyx_float__0202);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 9, __pyx_float__0202)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_03);
  __Pyx_GIVEREF(__pyx_float_6_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 10, __pyx_float_6_03)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0053);
  __Pyx_GIVEREF(__pyx_float__0053);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 11, __pyx_float__0053)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_05);
  __Pyx_GIVEREF(__pyx_float_7_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 12, __pyx_float_7_05)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0114);
  __Pyx_GIVEREF(__pyx_float__0114);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 13, __pyx_float__0114)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_01);
  __Pyx_GIVEREF(__pyx_float_8_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 14, __pyx_float_8_01)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 15, __pyx_float__0002)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_13);
  __Pyx_GIVEREF(__pyx_float_9_13);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 16, __pyx_float_9_13)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0048);
  __Pyx_GIVEREF(__pyx_float__0048);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 17, __pyx_float__0048)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_17);
  __Pyx_GIVEREF(__pyx_float_10_17);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 18, __pyx_float_10_17)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 19, __pyx_float__0010)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_23);
  __Pyx_GIVEREF(__pyx_float_11_23);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 20, __pyx_float_11_23)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0033);
  __Pyx_GIVEREF(__pyx_float__0033);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 21, __pyx_float__0033)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_30);
  __Pyx_GIVEREF(__pyx_float_12_30);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 22, __pyx_float_12_30)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 23, __pyx_float__0010)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_38);
  __Pyx_GIVEREF(__pyx_float_13_38);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 24, __pyx_float_13_38)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 25, __pyx_float__0006)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_50);
  __Pyx_GIVEREF(__pyx_float_14_50);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 26, __pyx_float_14_50)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 27, __pyx_float__0002)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_62);
  __Pyx_GIVEREF(__pyx_float_15_62);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 28, __pyx_float_15_62)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 29, __pyx_float__0010)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_27);
  __Pyx_GIVEREF(__pyx_float_20_27);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 30, __pyx_float_20_27)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 31, __pyx_float__0001)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_47);
  __Pyx_GIVEREF(__pyx_float_21_47);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 32, __pyx_float_21_47)) __PYX_ERR(0, 66, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_40, 33, __pyx_float__0001)) __PYX_ERR(0, 66, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":67
 * 		            [1.00,.1018,2.00,.1486,3.00,.0165,4.00,.0186,5.01,.0194,6.02,.0045,7.04,.0083 ,8.06,.0012,9.10,.0066,10.15,.0009,11.19,.0008,12.26,.0011,13.34,.0028,14.45,.0006,15.53,.0009 ,16.66,.0002,17.79,.0006,18.94,.0005,20.11,.0003,21.29,.0005,22.49,.0003,23.73,.0005,26.22,.0001 ,27.52,.0001,28.88,.0002],
 * 			        [1.00,.1889,1.99,.1822,3.00,.0363,4.00,.0047,5.01,.0202,6.03,.0053,7.05,.0114 ,8.01,.0002,9.13,.0048,10.17,.0010,11.23,.0033,12.30,.0010,13.38,.0006,14.50,.0002,15.62,.0010 ,20.27,.0001,21.47,.0001],
 * 			        [1.00,.0522,1.99,.0763,2.99,.0404,4.00,.0139,5.01,.0185,6.01,.0021,7.06,.0045 ,8.09,.0002,9.11,.0003,10.17,.0006,11.25,.0004,12.32,.0005,13.40,.0003,14.53,.0003,15.65,.0007 ,16.80,.0001,17.95,.0002,19.14,.0006,20.34,.0002,21.56,.0003],             # <<<<<<<<<<<<<<
 * 			        [0.99,.1821,1.99,.0773,3.00,.0125,4.01,.0065,5.01,.0202,6.03,.0071,7.05,.0090 ,8.08,.0006,9.13,.0008,10.18,.0013,11.25,.0010,12.33,.0012,13.42,.0006,14.54,.0005,15.65,.0004 ,17.97,.0002,19.15,.0001],
 * 			        [1.00,.1868,2.00,.0951,3.00,.0147,4.01,.0134,5.02,.0184,6.04,.0132,7.06,.0011 ,8.11,.0008,9.15,.0010,10.22,.0012,11.30,.0011,12.40,.0003,13.11,.0004,13.49,.0002,14.62,.0003 ,15.77,.0001],
 */
  __pyx_t_41 = PyList_New(40); if (unlikely(!__pyx_t_41)) __PYX_ERR(0, 67, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_41);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 0, __pyx_float_1_00)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0522);
  __Pyx_GIVEREF(__pyx_float__0522);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 1, __pyx_float__0522)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_1_99);
  __Pyx_GIVEREF(__pyx_float_1_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 2, __pyx_float_1_99)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0763);
  __Pyx_GIVEREF(__pyx_float__0763);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 3, __pyx_float__0763)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_99);
  __Pyx_GIVEREF(__pyx_float_2_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 4, __pyx_float_2_99)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0404);
  __Pyx_GIVEREF(__pyx_float__0404);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 5, __pyx_float__0404)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_00);
  __Pyx_GIVEREF(__pyx_float_4_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 6, __pyx_float_4_00)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0139);
  __Pyx_GIVEREF(__pyx_float__0139);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 7, __pyx_float__0139)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_01);
  __Pyx_GIVEREF(__pyx_float_5_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 8, __pyx_float_5_01)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0185);
  __Pyx_GIVEREF(__pyx_float__0185);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 9, __pyx_float__0185)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_01);
  __Pyx_GIVEREF(__pyx_float_6_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 10, __pyx_float_6_01)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0021);
  __Pyx_GIVEREF(__pyx_float__0021);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 11, __pyx_float__0021)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_06);
  __Pyx_GIVEREF(__pyx_float_7_06);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 12, __pyx_float_7_06)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0045);
  __Pyx_GIVEREF(__pyx_float__0045);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 13, __pyx_float__0045)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_09);
  __Pyx_GIVEREF(__pyx_float_8_09);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 14, __pyx_float_8_09)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 15, __pyx_float__0002)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_11);
  __Pyx_GIVEREF(__pyx_float_9_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 16, __pyx_float_9_11)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 17, __pyx_float__0003)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_17);
  __Pyx_GIVEREF(__pyx_float_10_17);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 18, __pyx_float_10_17)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 19, __pyx_float__0006)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_25);
  __Pyx_GIVEREF(__pyx_float_11_25);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 20, __pyx_float_11_25)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 21, __pyx_float__0004)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_32);
  __Pyx_GIVEREF(__pyx_float_12_32);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 22, __pyx_float_12_32)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 23, __pyx_float__0005)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_40);
  __Pyx_GIVEREF(__pyx_float_13_40);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 24, __pyx_float_13_40)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 25, __pyx_float__0003)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_53);
  __Pyx_GIVEREF(__pyx_float_14_53);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 26, __pyx_float_14_53)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 27, __pyx_float__0003)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_65);
  __Pyx_GIVEREF(__pyx_float_15_65);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 28, __pyx_float_15_65)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 29, __pyx_float__0007)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_80);
  __Pyx_GIVEREF(__pyx_float_16_80);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 30, __pyx_float_16_80)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 31, __pyx_float__0001)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_95);
  __Pyx_GIVEREF(__pyx_float_17_95);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 32, __pyx_float_17_95)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 33, __pyx_float__0002)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_14);
  __Pyx_GIVEREF(__pyx_float_19_14);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 34, __pyx_float_19_14)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 35, __pyx_float__0006)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_20_34);
  __Pyx_GIVEREF(__pyx_float_20_34);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 36, __pyx_float_20_34)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 37, __pyx_float__0002)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_21_56);
  __Pyx_GIVEREF(__pyx_float_21_56);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 38, __pyx_float_21_56)) __PYX_ERR(0, 67, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_41, 39, __pyx_float__0003)) __PYX_ERR(0, 67, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":68
 * 			        [1.00,.1889,1.99,.1822,3.00,.0363,4.00,.0047,5.01,.0202,6.03,.0053,7.05,.0114 ,8.01,.0002,9.13,.0048,10.17,.0010,11.23,.0033,12.30,.0010,13.38,.0006,14.50,.0002,15.62,.0010 ,20.27,.0001,21.47,.0001],
 * 			        [1.00,.0522,1.99,.0763,2.99,.0404,4.00,.0139,5.01,.0185,6.01,.0021,7.06,.0045 ,8.09,.0002,9.11,.0003,10.17,.0006,11.25,.0004,12.32,.0005,13.40,.0003,14.53,.0003,15.65,.0007 ,16.80,.0001,17.95,.0002,19.14,.0006,20.34,.0002,21.56,.0003],
 * 			        [0.99,.1821,1.99,.0773,3.00,.0125,4.01,.0065,5.01,.0202,6.03,.0071,7.05,.0090 ,8.08,.0006,9.13,.0008,10.18,.0013,11.25,.0010,12.33,.0012,13.42,.0006,14.54,.0005,15.65,.0004 ,17.97,.0002,19.15,.0001],             # <<<<<<<<<<<<<<
 * 			        [1.00,.1868,2.00,.0951,3.00,.0147,4.01,.0134,5.02,.0184,6.04,.0132,7.06,.0011 ,8.11,.0008,9.15,.0010,10.22,.0012,11.30,.0011,12.40,.0003,13.11,.0004,13.49,.0002,14.62,.0003 ,15.77,.0001],
 * 			        [1.00,.1933,2.00,.0714,3.00,.0373,4.00,.0108,5.02,.0094,6.02,.0010,7.07,.0022 ,8.11,.0002,9.16,.0065,10.23,.0015,11.31,.0023,12.40,.0003,13.53,.0014,14.66,.0002,15.81,.0011 ,18.20,.0002,19.41,.0001],
 */
  __pyx_t_42 = PyList_New(34); if (unlikely(!__pyx_t_42)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_42);
  __Pyx_INCREF(__pyx_float_0_99);
  __Pyx_GIVEREF(__pyx_float_0_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 0, __pyx_float_0_99)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1821);
  __Pyx_GIVEREF(__pyx_float__1821);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 1, __pyx_float__1821)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_1_99);
  __Pyx_GIVEREF(__pyx_float_1_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 2, __pyx_float_1_99)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0773);
  __Pyx_GIVEREF(__pyx_float__0773);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 3, __pyx_float__0773)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_00);
  __Pyx_GIVEREF(__pyx_float_3_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 4, __pyx_float_3_00)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0125);
  __Pyx_GIVEREF(__pyx_float__0125);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 5, __pyx_float__0125)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_01);
  __Pyx_GIVEREF(__pyx_float_4_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 6, __pyx_float_4_01)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0065);
  __Pyx_GIVEREF(__pyx_float__0065);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 7, __pyx_float__0065)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_01);
  __Pyx_GIVEREF(__pyx_float_5_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 8, __pyx_float_5_01)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0202);
  __Pyx_GIVEREF(__pyx_float__0202);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 9, __pyx_float__0202)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_03);
  __Pyx_GIVEREF(__pyx_float_6_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 10, __pyx_float_6_03)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0071);
  __Pyx_GIVEREF(__pyx_float__0071);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 11, __pyx_float__0071)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_05);
  __Pyx_GIVEREF(__pyx_float_7_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 12, __pyx_float_7_05)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0090);
  __Pyx_GIVEREF(__pyx_float__0090);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 13, __pyx_float__0090)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_08);
  __Pyx_GIVEREF(__pyx_float_8_08);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 14, __pyx_float_8_08)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 15, __pyx_float__0006)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_13);
  __Pyx_GIVEREF(__pyx_float_9_13);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 16, __pyx_float_9_13)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 17, __pyx_float__0008)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_18);
  __Pyx_GIVEREF(__pyx_float_10_18);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 18, __pyx_float_10_18)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0013);
  __Pyx_GIVEREF(__pyx_float__0013);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 19, __pyx_float__0013)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_25);
  __Pyx_GIVEREF(__pyx_float_11_25);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 20, __pyx_float_11_25)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 21, __pyx_float__0010)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_33);
  __Pyx_GIVEREF(__pyx_float_12_33);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 22, __pyx_float_12_33)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 23, __pyx_float__0012)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_42);
  __Pyx_GIVEREF(__pyx_float_13_42);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 24, __pyx_float_13_42)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 25, __pyx_float__0006)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_54);
  __Pyx_GIVEREF(__pyx_float_14_54);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 26, __pyx_float_14_54)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 27, __pyx_float__0005)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_65);
  __Pyx_GIVEREF(__pyx_float_15_65);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 28, __pyx_float_15_65)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 29, __pyx_float__0004)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_17_97);
  __Pyx_GIVEREF(__pyx_float_17_97);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 30, __pyx_float_17_97)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 31, __pyx_float__0002)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_15);
  __Pyx_GIVEREF(__pyx_float_19_15);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 32, __pyx_float_19_15)) __PYX_ERR(0, 68, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_42, 33, __pyx_float__0001)) __PYX_ERR(0, 68, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":69
 * 			        [1.00,.0522,1.99,.0763,2.99,.0404,4.00,.0139,5.01,.0185,6.01,.0021,7.06,.0045 ,8.09,.0002,9.11,.0003,10.17,.0006,11.25,.0004,12.32,.0005,13.40,.0003,14.53,.0003,15.65,.0007 ,16.80,.0001,17.95,.0002,19.14,.0006,20.34,.0002,21.56,.0003],
 * 			        [0.99,.1821,1.99,.0773,3.00,.0125,4.01,.0065,5.01,.0202,6.03,.0071,7.05,.0090 ,8.08,.0006,9.13,.0008,10.18,.0013,11.25,.0010,12.33,.0012,13.42,.0006,14.54,.0005,15.65,.0004 ,17.97,.0002,19.15,.0001],
 * 			        [1.00,.1868,2.00,.0951,3.00,.0147,4.01,.0134,5.02,.0184,6.04,.0132,7.06,.0011 ,8.11,.0008,9.15,.0010,10.22,.0012,11.30,.0011,12.40,.0003,13.11,.0004,13.49,.0002,14.62,.0003 ,15.77,.0001],             # <<<<<<<<<<<<<<
 * 			        [1.00,.1933,2.00,.0714,3.00,.0373,4.00,.0108,5.02,.0094,6.02,.0010,7.07,.0022 ,8.11,.0002,9.16,.0065,10.23,.0015,11.31,.0023,12.40,.0003,13.53,.0014,14.66,.0002,15.81,.0011 ,18.20,.0002,19.41,.0001],
 * 			        [0.99,.2113,1.99,.0877,3.00,.0492,4.01,.0094,5.02,.0144,6.04,.0103,7.07,.0117 ,8.12,.0006,9.19,.0019,10.25,.0007,11.35,.0017,12.45,.0010,13.58,.0003,14.74,.0003,15.91,.0003 ,19.57,.0002],
 */
  __pyx_t_43 = PyList_New(32); if (unlikely(!__pyx_t_43)) __PYX_ERR(0, 69, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_43);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_43, 0, __pyx_float_1_00)) __PYX_ERR(0, 69, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1868);
  __Pyx_GIVEREF(__pyx_float__1868);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_43, 1, __pyx_float__1868)) __PYX_ERR(0, 69, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_00);
  __Pyx_GIVEREF(__pyx_float_2_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_43, 2, __pyx_float_2_00)) __PYX_ERR(0, 69, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0951);
  __Pyx_GIVEREF(__pyx_float__0951);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_43, 3, __pyx_float__0951)) __PYX_ERR(0, 69, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_00);
  __Pyx_GIVEREF(__pyx_float_3_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_43, 4, __pyx_float_3_00)) __PYX_ERR(0, 69, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0147);
  __Pyx_GIVEREF(__pyx_float__0147);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_43, 5, __pyx_float__0147)) __PYX_ERR(0, 69, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_01);
  __Pyx_GIVEREF(__pyx_float_4_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_43, 6, __pyx_float_4_01)) __PYX_ERR(0, 69, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0134);
  __Pyx_GIVEREF(__pyx_float__0134);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_43, 7, __pyx_float__0134)) __PYX_ERR(0, 69, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_02);
  __Pyx_GIVEREF(__pyx_float_5_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_43, 8, __pyx_float_5_02)) __PYX_ERR(0, 69, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0184);
  __Pyx_GIVEREF(__pyx_float__0184);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_43, 9, __pyx_float__0184)) __PYX_ERR(0, 69, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_04);
  __Pyx_GIVEREF(__pyx_float_6_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_43, 10, __pyx_float_6_04)) __PYX_ERR(0, 69, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0132);
  __Pyx_GIVEREF(__pyx_float__0132);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_43, 11, __pyx_float__0132)) __PYX_ERR(0, 69, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_06);
  __Pyx_GIVEREF(__pyx_float_7_06);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_43, 12, __pyx_float_7_06)) __PYX_ERR(0, 69, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_43, 13, __pyx_float__0011)) __PYX_ERR(0, 69, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_11);
  __Pyx_GIVEREF(__pyx_float_8_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_43, 14, __pyx_float_8_11)) __PYX_ERR(0, 69, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_43, 15, __pyx_float__0008)) __PYX_ERR(0, 69, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_15);
  __Pyx_GIVEREF(__pyx_float_9_15);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_43, 16, __pyx_float_9_15)) __PYX_ERR(0, 69, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_43, 17, __pyx_float__0010)) __PYX_ERR(0, 69, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_22);
  __Pyx_GIVEREF(__pyx_float_10_22);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_43, 18, __pyx_float_10_22)) __PYX_ERR(0, 69, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_43, 19, __pyx_float__0012)) __PYX_ERR(0, 69, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_30);
  __Pyx_GIVEREF(__pyx_float_11_30);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_43, 20, __pyx_float_11_30)) __PYX_ERR(0, 69, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_43, 21, __pyx_float__0011)) __PYX_ERR(0, 69, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_40);
  __Pyx_GIVEREF(__pyx_float_12_40);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_43, 22, __pyx_float_12_40)) __PYX_ERR(0, 69, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_43, 23, __pyx_float__0003)) __PYX_ERR(0, 69, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_11);
  __Pyx_GIVEREF(__pyx_float_13_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_43, 24, __pyx_float_13_11)) __PYX_ERR(0, 69, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_43, 25, __pyx_float__0004)) __PYX_ERR(0, 69, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_49);
  __Pyx_GIVEREF(__pyx_float_13_49);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_43, 26, __pyx_float_13_49)) __PYX_ERR(0, 69, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_43, 27, __pyx_float__0002)) __PYX_ERR(0, 69, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_62);
  __Pyx_GIVEREF(__pyx_float_14_62);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_43, 28, __pyx_float_14_62)) __PYX_ERR(0, 69, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_43, 29, __pyx_float__0003)) __PYX_ERR(0, 69, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_77);
  __Pyx_GIVEREF(__pyx_float_15_77);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_43, 30, __pyx_float_15_77)) __PYX_ERR(0, 69, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_43, 31, __pyx_float__0001)) __PYX_ERR(0, 69, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":70
 * 			        [0.99,.1821,1.99,.0773,3.00,.0125,4.01,.0065,5.01,.0202,6.03,.0071,7.05,.0090 ,8.08,.0006,9.13,.0008,10.18,.0013,11.25,.0010,12.33,.0012,13.42,.0006,14.54,.0005,15.65,.0004 ,17.97,.0002,19.15,.0001],
 * 			        [1.00,.1868,2.00,.0951,3.00,.0147,4.01,.0134,5.02,.0184,6.04,.0132,7.06,.0011 ,8.11,.0008,9.15,.0010,10.22,.0012,11.30,.0011,12.40,.0003,13.11,.0004,13.49,.0002,14.62,.0003 ,15.77,.0001],
 * 			        [1.00,.1933,2.00,.0714,3.00,.0373,4.00,.0108,5.02,.0094,6.02,.0010,7.07,.0022 ,8.11,.0002,9.16,.0065,10.23,.0015,11.31,.0023,12.40,.0003,13.53,.0014,14.66,.0002,15.81,.0011 ,18.20,.0002,19.41,.0001],             # <<<<<<<<<<<<<<
 * 			        [0.99,.2113,1.99,.0877,3.00,.0492,4.01,.0094,5.02,.0144,6.04,.0103,7.07,.0117 ,8.12,.0006,9.19,.0019,10.25,.0007,11.35,.0017,12.45,.0010,13.58,.0003,14.74,.0003,15.91,.0003 ,19.57,.0002],
 * 			        [0.99,.2455,1.99,.0161,3.00,.0215,4.01,.0036,5.03,.0049,6.04,.0012,7.09,.0036 ,8.14,.0011,9.21,.0009,10.30,.0001,11.40,.0012,12.50,.0001,13.66,.0005,14.84,.0001],
 */
  __pyx_t_44 = PyList_New(34); if (unlikely(!__pyx_t_44)) __PYX_ERR(0, 70, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_44);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 0, __pyx_float_1_00)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1933);
  __Pyx_GIVEREF(__pyx_float__1933);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 1, __pyx_float__1933)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_00);
  __Pyx_GIVEREF(__pyx_float_2_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 2, __pyx_float_2_00)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0714);
  __Pyx_GIVEREF(__pyx_float__0714);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 3, __pyx_float__0714)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_00);
  __Pyx_GIVEREF(__pyx_float_3_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 4, __pyx_float_3_00)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0373);
  __Pyx_GIVEREF(__pyx_float__0373);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 5, __pyx_float__0373)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_00);
  __Pyx_GIVEREF(__pyx_float_4_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 6, __pyx_float_4_00)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0108);
  __Pyx_GIVEREF(__pyx_float__0108);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 7, __pyx_float__0108)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_02);
  __Pyx_GIVEREF(__pyx_float_5_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 8, __pyx_float_5_02)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0094);
  __Pyx_GIVEREF(__pyx_float__0094);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 9, __pyx_float__0094)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_02);
  __Pyx_GIVEREF(__pyx_float_6_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 10, __pyx_float_6_02)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 11, __pyx_float__0010)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_07);
  __Pyx_GIVEREF(__pyx_float_7_07);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 12, __pyx_float_7_07)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0022);
  __Pyx_GIVEREF(__pyx_float__0022);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 13, __pyx_float__0022)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_11);
  __Pyx_GIVEREF(__pyx_float_8_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 14, __pyx_float_8_11)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 15, __pyx_float__0002)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_16);
  __Pyx_GIVEREF(__pyx_float_9_16);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 16, __pyx_float_9_16)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0065);
  __Pyx_GIVEREF(__pyx_float__0065);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 17, __pyx_float__0065)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_23);
  __Pyx_GIVEREF(__pyx_float_10_23);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 18, __pyx_float_10_23)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0015);
  __Pyx_GIVEREF(__pyx_float__0015);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 19, __pyx_float__0015)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_31);
  __Pyx_GIVEREF(__pyx_float_11_31);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 20, __pyx_float_11_31)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0023);
  __Pyx_GIVEREF(__pyx_float__0023);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 21, __pyx_float__0023)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_40);
  __Pyx_GIVEREF(__pyx_float_12_40);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 22, __pyx_float_12_40)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 23, __pyx_float__0003)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_53);
  __Pyx_GIVEREF(__pyx_float_13_53);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 24, __pyx_float_13_53)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0014);
  __Pyx_GIVEREF(__pyx_float__0014);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 25, __pyx_float__0014)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_66);
  __Pyx_GIVEREF(__pyx_float_14_66);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 26, __pyx_float_14_66)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 27, __pyx_float__0002)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_81);
  __Pyx_GIVEREF(__pyx_float_15_81);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 28, __pyx_float_15_81)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 29, __pyx_float__0011)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_18_20);
  __Pyx_GIVEREF(__pyx_float_18_20);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 30, __pyx_float_18_20)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 31, __pyx_float__0002)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_41);
  __Pyx_GIVEREF(__pyx_float_19_41);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 32, __pyx_float_19_41)) __PYX_ERR(0, 70, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_44, 33, __pyx_float__0001)) __PYX_ERR(0, 70, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":71
 * 			        [1.00,.1868,2.00,.0951,3.00,.0147,4.01,.0134,5.02,.0184,6.04,.0132,7.06,.0011 ,8.11,.0008,9.15,.0010,10.22,.0012,11.30,.0011,12.40,.0003,13.11,.0004,13.49,.0002,14.62,.0003 ,15.77,.0001],
 * 			        [1.00,.1933,2.00,.0714,3.00,.0373,4.00,.0108,5.02,.0094,6.02,.0010,7.07,.0022 ,8.11,.0002,9.16,.0065,10.23,.0015,11.31,.0023,12.40,.0003,13.53,.0014,14.66,.0002,15.81,.0011 ,18.20,.0002,19.41,.0001],
 * 			        [0.99,.2113,1.99,.0877,3.00,.0492,4.01,.0094,5.02,.0144,6.04,.0103,7.07,.0117 ,8.12,.0006,9.19,.0019,10.25,.0007,11.35,.0017,12.45,.0010,13.58,.0003,14.74,.0003,15.91,.0003 ,19.57,.0002],             # <<<<<<<<<<<<<<
 * 			        [0.99,.2455,1.99,.0161,3.00,.0215,4.01,.0036,5.03,.0049,6.04,.0012,7.09,.0036 ,8.14,.0011,9.21,.0009,10.30,.0001,11.40,.0012,12.50,.0001,13.66,.0005,14.84,.0001],
 * 			        [1.00,.1132,2.00,.0252,3.00,.0292,4.01,.0136,5.03,.0045,6.06,.0022,7.11,.0101 ,8.17,.0004,9.23,.0010,10.33,.0012,11.44,.0013,12.58,.0011,13.75,.0002,14.93,.0005,16.14,.0002],
 */
  __pyx_t_45 = PyList_New(32); if (unlikely(!__pyx_t_45)) __PYX_ERR(0, 71, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_45);
  __Pyx_INCREF(__pyx_float_0_99);
  __Pyx_GIVEREF(__pyx_float_0_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_45, 0, __pyx_float_0_99)) __PYX_ERR(0, 71, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__2113);
  __Pyx_GIVEREF(__pyx_float__2113);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_45, 1, __pyx_float__2113)) __PYX_ERR(0, 71, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_1_99);
  __Pyx_GIVEREF(__pyx_float_1_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_45, 2, __pyx_float_1_99)) __PYX_ERR(0, 71, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0877);
  __Pyx_GIVEREF(__pyx_float__0877);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_45, 3, __pyx_float__0877)) __PYX_ERR(0, 71, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_00);
  __Pyx_GIVEREF(__pyx_float_3_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_45, 4, __pyx_float_3_00)) __PYX_ERR(0, 71, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0492);
  __Pyx_GIVEREF(__pyx_float__0492);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_45, 5, __pyx_float__0492)) __PYX_ERR(0, 71, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_01);
  __Pyx_GIVEREF(__pyx_float_4_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_45, 6, __pyx_float_4_01)) __PYX_ERR(0, 71, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0094);
  __Pyx_GIVEREF(__pyx_float__0094);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_45, 7, __pyx_float__0094)) __PYX_ERR(0, 71, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_02);
  __Pyx_GIVEREF(__pyx_float_5_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_45, 8, __pyx_float_5_02)) __PYX_ERR(0, 71, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0144);
  __Pyx_GIVEREF(__pyx_float__0144);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_45, 9, __pyx_float__0144)) __PYX_ERR(0, 71, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_04);
  __Pyx_GIVEREF(__pyx_float_6_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_45, 10, __pyx_float_6_04)) __PYX_ERR(0, 71, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0103);
  __Pyx_GIVEREF(__pyx_float__0103);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_45, 11, __pyx_float__0103)) __PYX_ERR(0, 71, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_07);
  __Pyx_GIVEREF(__pyx_float_7_07);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_45, 12, __pyx_float_7_07)) __PYX_ERR(0, 71, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0117);
  __Pyx_GIVEREF(__pyx_float__0117);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_45, 13, __pyx_float__0117)) __PYX_ERR(0, 71, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_12);
  __Pyx_GIVEREF(__pyx_float_8_12);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_45, 14, __pyx_float_8_12)) __PYX_ERR(0, 71, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_45, 15, __pyx_float__0006)) __PYX_ERR(0, 71, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_19);
  __Pyx_GIVEREF(__pyx_float_9_19);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_45, 16, __pyx_float_9_19)) __PYX_ERR(0, 71, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0019);
  __Pyx_GIVEREF(__pyx_float__0019);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_45, 17, __pyx_float__0019)) __PYX_ERR(0, 71, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_25);
  __Pyx_GIVEREF(__pyx_float_10_25);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_45, 18, __pyx_float_10_25)) __PYX_ERR(0, 71, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_45, 19, __pyx_float__0007)) __PYX_ERR(0, 71, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_35);
  __Pyx_GIVEREF(__pyx_float_11_35);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_45, 20, __pyx_float_11_35)) __PYX_ERR(0, 71, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0017);
  __Pyx_GIVEREF(__pyx_float__0017);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_45, 21, __pyx_float__0017)) __PYX_ERR(0, 71, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_45);
  __Pyx_GIVEREF(__pyx_float_12_45);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_45, 22, __pyx_float_12_45)) __PYX_ERR(0, 71, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_45, 23, __pyx_float__0010)) __PYX_ERR(0, 71, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_58);
  __Pyx_GIVEREF(__pyx_float_13_58);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_45, 24, __pyx_float_13_58)) __PYX_ERR(0, 71, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_45, 25, __pyx_float__0003)) __PYX_ERR(0, 71, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_74);
  __Pyx_GIVEREF(__pyx_float_14_74);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_45, 26, __pyx_float_14_74)) __PYX_ERR(0, 71, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_45, 27, __pyx_float__0003)) __PYX_ERR(0, 71, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_91);
  __Pyx_GIVEREF(__pyx_float_15_91);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_45, 28, __pyx_float_15_91)) __PYX_ERR(0, 71, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_45, 29, __pyx_float__0003)) __PYX_ERR(0, 71, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_19_57);
  __Pyx_GIVEREF(__pyx_float_19_57);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_45, 30, __pyx_float_19_57)) __PYX_ERR(0, 71, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_45, 31, __pyx_float__0002)) __PYX_ERR(0, 71, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":72
 * 			        [1.00,.1933,2.00,.0714,3.00,.0373,4.00,.0108,5.02,.0094,6.02,.0010,7.07,.0022 ,8.11,.0002,9.16,.0065,10.23,.0015,11.31,.0023,12.40,.0003,13.53,.0014,14.66,.0002,15.81,.0011 ,18.20,.0002,19.41,.0001],
 * 			        [0.99,.2113,1.99,.0877,3.00,.0492,4.01,.0094,5.02,.0144,6.04,.0103,7.07,.0117 ,8.12,.0006,9.19,.0019,10.25,.0007,11.35,.0017,12.45,.0010,13.58,.0003,14.74,.0003,15.91,.0003 ,19.57,.0002],
 * 			        [0.99,.2455,1.99,.0161,3.00,.0215,4.01,.0036,5.03,.0049,6.04,.0012,7.09,.0036 ,8.14,.0011,9.21,.0009,10.30,.0001,11.40,.0012,12.50,.0001,13.66,.0005,14.84,.0001],             # <<<<<<<<<<<<<<
 * 			        [1.00,.1132,2.00,.0252,3.00,.0292,4.01,.0136,5.03,.0045,6.06,.0022,7.11,.0101 ,8.17,.0004,9.23,.0010,10.33,.0012,11.44,.0013,12.58,.0011,13.75,.0002,14.93,.0005,16.14,.0002],
 * 			        [1.00,.1655,2.00,.0445,3.00,.0120,4.00,.0038,5.02,.0015,6.07,.0038,7.11,.0003 ,8.19,.0002,9.25,.0010,10.36,.0011,11.48,.0005,12.63,.0002,13.79,.0003,16.24,.0002],
 */
  __pyx_t_46 = PyList_New(28); if (unlikely(!__pyx_t_46)) __PYX_ERR(0, 72, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_46);
  __Pyx_INCREF(__pyx_float_0_99);
  __Pyx_GIVEREF(__pyx_float_0_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_46, 0, __pyx_float_0_99)) __PYX_ERR(0, 72, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__2455);
  __Pyx_GIVEREF(__pyx_float__2455);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_46, 1, __pyx_float__2455)) __PYX_ERR(0, 72, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_1_99);
  __Pyx_GIVEREF(__pyx_float_1_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_46, 2, __pyx_float_1_99)) __PYX_ERR(0, 72, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0161);
  __Pyx_GIVEREF(__pyx_float__0161);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_46, 3, __pyx_float__0161)) __PYX_ERR(0, 72, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_00);
  __Pyx_GIVEREF(__pyx_float_3_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_46, 4, __pyx_float_3_00)) __PYX_ERR(0, 72, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0215);
  __Pyx_GIVEREF(__pyx_float__0215);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_46, 5, __pyx_float__0215)) __PYX_ERR(0, 72, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_01);
  __Pyx_GIVEREF(__pyx_float_4_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_46, 6, __pyx_float_4_01)) __PYX_ERR(0, 72, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0036);
  __Pyx_GIVEREF(__pyx_float__0036);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_46, 7, __pyx_float__0036)) __PYX_ERR(0, 72, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_03);
  __Pyx_GIVEREF(__pyx_float_5_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_46, 8, __pyx_float_5_03)) __PYX_ERR(0, 72, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0049);
  __Pyx_GIVEREF(__pyx_float__0049);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_46, 9, __pyx_float__0049)) __PYX_ERR(0, 72, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_04);
  __Pyx_GIVEREF(__pyx_float_6_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_46, 10, __pyx_float_6_04)) __PYX_ERR(0, 72, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_46, 11, __pyx_float__0012)) __PYX_ERR(0, 72, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_09);
  __Pyx_GIVEREF(__pyx_float_7_09);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_46, 12, __pyx_float_7_09)) __PYX_ERR(0, 72, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0036);
  __Pyx_GIVEREF(__pyx_float__0036);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_46, 13, __pyx_float__0036)) __PYX_ERR(0, 72, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_14);
  __Pyx_GIVEREF(__pyx_float_8_14);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_46, 14, __pyx_float_8_14)) __PYX_ERR(0, 72, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_46, 15, __pyx_float__0011)) __PYX_ERR(0, 72, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_21);
  __Pyx_GIVEREF(__pyx_float_9_21);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_46, 16, __pyx_float_9_21)) __PYX_ERR(0, 72, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_46, 17, __pyx_float__0009)) __PYX_ERR(0, 72, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_30);
  __Pyx_GIVEREF(__pyx_float_10_30);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_46, 18, __pyx_float_10_30)) __PYX_ERR(0, 72, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_46, 19, __pyx_float__0001)) __PYX_ERR(0, 72, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_40);
  __Pyx_GIVEREF(__pyx_float_11_40);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_46, 20, __pyx_float_11_40)) __PYX_ERR(0, 72, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_46, 21, __pyx_float__0012)) __PYX_ERR(0, 72, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_50);
  __Pyx_GIVEREF(__pyx_float_12_50);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_46, 22, __pyx_float_12_50)) __PYX_ERR(0, 72, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_46, 23, __pyx_float__0001)) __PYX_ERR(0, 72, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_66);
  __Pyx_GIVEREF(__pyx_float_13_66);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_46, 24, __pyx_float_13_66)) __PYX_ERR(0, 72, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_46, 25, __pyx_float__0005)) __PYX_ERR(0, 72, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_84);
  __Pyx_GIVEREF(__pyx_float_14_84);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_46, 26, __pyx_float_14_84)) __PYX_ERR(0, 72, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_46, 27, __pyx_float__0001)) __PYX_ERR(0, 72, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":73
 * 			        [0.99,.2113,1.99,.0877,3.00,.0492,4.01,.0094,5.02,.0144,6.04,.0103,7.07,.0117 ,8.12,.0006,9.19,.0019,10.25,.0007,11.35,.0017,12.45,.0010,13.58,.0003,14.74,.0003,15.91,.0003 ,19.57,.0002],
 * 			        [0.99,.2455,1.99,.0161,3.00,.0215,4.01,.0036,5.03,.0049,6.04,.0012,7.09,.0036 ,8.14,.0011,9.21,.0009,10.30,.0001,11.40,.0012,12.50,.0001,13.66,.0005,14.84,.0001],
 * 			        [1.00,.1132,2.00,.0252,3.00,.0292,4.01,.0136,5.03,.0045,6.06,.0022,7.11,.0101 ,8.17,.0004,9.23,.0010,10.33,.0012,11.44,.0013,12.58,.0011,13.75,.0002,14.93,.0005,16.14,.0002],             # <<<<<<<<<<<<<<
 * 			        [1.00,.1655,2.00,.0445,3.00,.0120,4.00,.0038,5.02,.0015,6.07,.0038,7.11,.0003 ,8.19,.0002,9.25,.0010,10.36,.0011,11.48,.0005,12.63,.0002,13.79,.0003,16.24,.0002],
 * 			        [0.99,.3637,1.99,.0259,3.01,.0038,4.01,.0057,5.03,.0040,6.07,.0067,7.12,.0014 ,8.19,.0004,9.27,.0003,10.38,.0002,12.67,.0001],
 */
  __pyx_t_47 = PyList_New(30); if (unlikely(!__pyx_t_47)) __PYX_ERR(0, 73, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_47);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_47, 0, __pyx_float_1_00)) __PYX_ERR(0, 73, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1132);
  __Pyx_GIVEREF(__pyx_float__1132);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_47, 1, __pyx_float__1132)) __PYX_ERR(0, 73, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_00);
  __Pyx_GIVEREF(__pyx_float_2_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_47, 2, __pyx_float_2_00)) __PYX_ERR(0, 73, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0252);
  __Pyx_GIVEREF(__pyx_float__0252);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_47, 3, __pyx_float__0252)) __PYX_ERR(0, 73, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_00);
  __Pyx_GIVEREF(__pyx_float_3_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_47, 4, __pyx_float_3_00)) __PYX_ERR(0, 73, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0292);
  __Pyx_GIVEREF(__pyx_float__0292);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_47, 5, __pyx_float__0292)) __PYX_ERR(0, 73, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_01);
  __Pyx_GIVEREF(__pyx_float_4_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_47, 6, __pyx_float_4_01)) __PYX_ERR(0, 73, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0136);
  __Pyx_GIVEREF(__pyx_float__0136);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_47, 7, __pyx_float__0136)) __PYX_ERR(0, 73, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_03);
  __Pyx_GIVEREF(__pyx_float_5_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_47, 8, __pyx_float_5_03)) __PYX_ERR(0, 73, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0045);
  __Pyx_GIVEREF(__pyx_float__0045);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_47, 9, __pyx_float__0045)) __PYX_ERR(0, 73, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_06);
  __Pyx_GIVEREF(__pyx_float_6_06);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_47, 10, __pyx_float_6_06)) __PYX_ERR(0, 73, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0022);
  __Pyx_GIVEREF(__pyx_float__0022);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_47, 11, __pyx_float__0022)) __PYX_ERR(0, 73, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_11);
  __Pyx_GIVEREF(__pyx_float_7_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_47, 12, __pyx_float_7_11)) __PYX_ERR(0, 73, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0101);
  __Pyx_GIVEREF(__pyx_float__0101);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_47, 13, __pyx_float__0101)) __PYX_ERR(0, 73, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_17);
  __Pyx_GIVEREF(__pyx_float_8_17);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_47, 14, __pyx_float_8_17)) __PYX_ERR(0, 73, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_47, 15, __pyx_float__0004)) __PYX_ERR(0, 73, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_23);
  __Pyx_GIVEREF(__pyx_float_9_23);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_47, 16, __pyx_float_9_23)) __PYX_ERR(0, 73, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_47, 17, __pyx_float__0010)) __PYX_ERR(0, 73, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_33);
  __Pyx_GIVEREF(__pyx_float_10_33);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_47, 18, __pyx_float_10_33)) __PYX_ERR(0, 73, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_47, 19, __pyx_float__0012)) __PYX_ERR(0, 73, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_44);
  __Pyx_GIVEREF(__pyx_float_11_44);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_47, 20, __pyx_float_11_44)) __PYX_ERR(0, 73, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0013);
  __Pyx_GIVEREF(__pyx_float__0013);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_47, 21, __pyx_float__0013)) __PYX_ERR(0, 73, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_58);
  __Pyx_GIVEREF(__pyx_float_12_58);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_47, 22, __pyx_float_12_58)) __PYX_ERR(0, 73, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_47, 23, __pyx_float__0011)) __PYX_ERR(0, 73, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_75);
  __Pyx_GIVEREF(__pyx_float_13_75);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_47, 24, __pyx_float_13_75)) __PYX_ERR(0, 73, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_47, 25, __pyx_float__0002)) __PYX_ERR(0, 73, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_14_93);
  __Pyx_GIVEREF(__pyx_float_14_93);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_47, 26, __pyx_float_14_93)) __PYX_ERR(0, 73, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_47, 27, __pyx_float__0005)) __PYX_ERR(0, 73, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_14);
  __Pyx_GIVEREF(__pyx_float_16_14);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_47, 28, __pyx_float_16_14)) __PYX_ERR(0, 73, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_47, 29, __pyx_float__0002)) __PYX_ERR(0, 73, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":74
 * 			        [0.99,.2455,1.99,.0161,3.00,.0215,4.01,.0036,5.03,.0049,6.04,.0012,7.09,.0036 ,8.14,.0011,9.21,.0009,10.30,.0001,11.40,.0012,12.50,.0001,13.66,.0005,14.84,.0001],
 * 			        [1.00,.1132,2.00,.0252,3.00,.0292,4.01,.0136,5.03,.0045,6.06,.0022,7.11,.0101 ,8.17,.0004,9.23,.0010,10.33,.0012,11.44,.0013,12.58,.0011,13.75,.0002,14.93,.0005,16.14,.0002],
 * 			        [1.00,.1655,2.00,.0445,3.00,.0120,4.00,.0038,5.02,.0015,6.07,.0038,7.11,.0003 ,8.19,.0002,9.25,.0010,10.36,.0011,11.48,.0005,12.63,.0002,13.79,.0003,16.24,.0002],             # <<<<<<<<<<<<<<
 * 			        [0.99,.3637,1.99,.0259,3.01,.0038,4.01,.0057,5.03,.0040,6.07,.0067,7.12,.0014 ,8.19,.0004,9.27,.0003,10.38,.0002,12.67,.0001],
 * 			        [1.00,.1193,2.00,.0230,3.00,.0104,4.01,.0084,5.04,.0047,6.08,.0035,7.13,.0041 ,8.20,.0002,9.29,.0005,10.40,.0005,11.53,.0003,12.70,.0002,13.91,.0002],
 */
  __pyx_t_48 = PyList_New(28); if (unlikely(!__pyx_t_48)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_48);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_48, 0, __pyx_float_1_00)) __PYX_ERR(0, 74, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1655);
  __Pyx_GIVEREF(__pyx_float__1655);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_48, 1, __pyx_float__1655)) __PYX_ERR(0, 74, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_00);
  __Pyx_GIVEREF(__pyx_float_2_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_48, 2, __pyx_float_2_00)) __PYX_ERR(0, 74, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0445);
  __Pyx_GIVEREF(__pyx_float__0445);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_48, 3, __pyx_float__0445)) __PYX_ERR(0, 74, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_00);
  __Pyx_GIVEREF(__pyx_float_3_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_48, 4, __pyx_float_3_00)) __PYX_ERR(0, 74, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0120);
  __Pyx_GIVEREF(__pyx_float__0120);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_48, 5, __pyx_float__0120)) __PYX_ERR(0, 74, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_00);
  __Pyx_GIVEREF(__pyx_float_4_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_48, 6, __pyx_float_4_00)) __PYX_ERR(0, 74, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0038);
  __Pyx_GIVEREF(__pyx_float__0038);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_48, 7, __pyx_float__0038)) __PYX_ERR(0, 74, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_02);
  __Pyx_GIVEREF(__pyx_float_5_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_48, 8, __pyx_float_5_02)) __PYX_ERR(0, 74, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0015);
  __Pyx_GIVEREF(__pyx_float__0015);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_48, 9, __pyx_float__0015)) __PYX_ERR(0, 74, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_07);
  __Pyx_GIVEREF(__pyx_float_6_07);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_48, 10, __pyx_float_6_07)) __PYX_ERR(0, 74, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0038);
  __Pyx_GIVEREF(__pyx_float__0038);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_48, 11, __pyx_float__0038)) __PYX_ERR(0, 74, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_11);
  __Pyx_GIVEREF(__pyx_float_7_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_48, 12, __pyx_float_7_11)) __PYX_ERR(0, 74, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_48, 13, __pyx_float__0003)) __PYX_ERR(0, 74, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_19);
  __Pyx_GIVEREF(__pyx_float_8_19);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_48, 14, __pyx_float_8_19)) __PYX_ERR(0, 74, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_48, 15, __pyx_float__0002)) __PYX_ERR(0, 74, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_25);
  __Pyx_GIVEREF(__pyx_float_9_25);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_48, 16, __pyx_float_9_25)) __PYX_ERR(0, 74, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_48, 17, __pyx_float__0010)) __PYX_ERR(0, 74, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_36);
  __Pyx_GIVEREF(__pyx_float_10_36);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_48, 18, __pyx_float_10_36)) __PYX_ERR(0, 74, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_48, 19, __pyx_float__0011)) __PYX_ERR(0, 74, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_48);
  __Pyx_GIVEREF(__pyx_float_11_48);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_48, 20, __pyx_float_11_48)) __PYX_ERR(0, 74, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_48, 21, __pyx_float__0005)) __PYX_ERR(0, 74, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_63);
  __Pyx_GIVEREF(__pyx_float_12_63);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_48, 22, __pyx_float_12_63)) __PYX_ERR(0, 74, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_48, 23, __pyx_float__0002)) __PYX_ERR(0, 74, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_79);
  __Pyx_GIVEREF(__pyx_float_13_79);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_48, 24, __pyx_float_13_79)) __PYX_ERR(0, 74, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_48, 25, __pyx_float__0003)) __PYX_ERR(0, 74, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_16_24);
  __Pyx_GIVEREF(__pyx_float_16_24);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_48, 26, __pyx_float_16_24)) __PYX_ERR(0, 74, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_48, 27, __pyx_float__0002)) __PYX_ERR(0, 74, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":75
 * 			        [1.00,.1132,2.00,.0252,3.00,.0292,4.01,.0136,5.03,.0045,6.06,.0022,7.11,.0101 ,8.17,.0004,9.23,.0010,10.33,.0012,11.44,.0013,12.58,.0011,13.75,.0002,14.93,.0005,16.14,.0002],
 * 			        [1.00,.1655,2.00,.0445,3.00,.0120,4.00,.0038,5.02,.0015,6.07,.0038,7.11,.0003 ,8.19,.0002,9.25,.0010,10.36,.0011,11.48,.0005,12.63,.0002,13.79,.0003,16.24,.0002],
 * 			        [0.99,.3637,1.99,.0259,3.01,.0038,4.01,.0057,5.03,.0040,6.07,.0067,7.12,.0014 ,8.19,.0004,9.27,.0003,10.38,.0002,12.67,.0001],             # <<<<<<<<<<<<<<
 * 			        [1.00,.1193,2.00,.0230,3.00,.0104,4.01,.0084,5.04,.0047,6.08,.0035,7.13,.0041 ,8.20,.0002,9.29,.0005,10.40,.0005,11.53,.0003,12.70,.0002,13.91,.0002],
 * 			        [1.00,.0752,2.00,.0497,3.00,.0074,4.02,.0076,5.05,.0053,6.09,.0043,7.15,.0024 ,8.22,.0001,9.32,.0006,10.45,.0002,11.58,.0001,12.78,.0001,15.22,.0001],
 */
  __pyx_t_49 = PyList_New(22); if (unlikely(!__pyx_t_49)) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_49);
  __Pyx_INCREF(__pyx_float_0_99);
  __Pyx_GIVEREF(__pyx_float_0_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_49, 0, __pyx_float_0_99)) __PYX_ERR(0, 75, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__3637);
  __Pyx_GIVEREF(__pyx_float__3637);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_49, 1, __pyx_float__3637)) __PYX_ERR(0, 75, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_1_99);
  __Pyx_GIVEREF(__pyx_float_1_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_49, 2, __pyx_float_1_99)) __PYX_ERR(0, 75, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0259);
  __Pyx_GIVEREF(__pyx_float__0259);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_49, 3, __pyx_float__0259)) __PYX_ERR(0, 75, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_01);
  __Pyx_GIVEREF(__pyx_float_3_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_49, 4, __pyx_float_3_01)) __PYX_ERR(0, 75, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0038);
  __Pyx_GIVEREF(__pyx_float__0038);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_49, 5, __pyx_float__0038)) __PYX_ERR(0, 75, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_01);
  __Pyx_GIVEREF(__pyx_float_4_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_49, 6, __pyx_float_4_01)) __PYX_ERR(0, 75, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0057);
  __Pyx_GIVEREF(__pyx_float__0057);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_49, 7, __pyx_float__0057)) __PYX_ERR(0, 75, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_03);
  __Pyx_GIVEREF(__pyx_float_5_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_49, 8, __pyx_float_5_03)) __PYX_ERR(0, 75, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0040);
  __Pyx_GIVEREF(__pyx_float__0040);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_49, 9, __pyx_float__0040)) __PYX_ERR(0, 75, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_07);
  __Pyx_GIVEREF(__pyx_float_6_07);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_49, 10, __pyx_float_6_07)) __PYX_ERR(0, 75, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0067);
  __Pyx_GIVEREF(__pyx_float__0067);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_49, 11, __pyx_float__0067)) __PYX_ERR(0, 75, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_12);
  __Pyx_GIVEREF(__pyx_float_7_12);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_49, 12, __pyx_float_7_12)) __PYX_ERR(0, 75, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0014);
  __Pyx_GIVEREF(__pyx_float__0014);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_49, 13, __pyx_float__0014)) __PYX_ERR(0, 75, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_19);
  __Pyx_GIVEREF(__pyx_float_8_19);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_49, 14, __pyx_float_8_19)) __PYX_ERR(0, 75, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_49, 15, __pyx_float__0004)) __PYX_ERR(0, 75, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_27);
  __Pyx_GIVEREF(__pyx_float_9_27);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_49, 16, __pyx_float_9_27)) __PYX_ERR(0, 75, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_49, 17, __pyx_float__0003)) __PYX_ERR(0, 75, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_38);
  __Pyx_GIVEREF(__pyx_float_10_38);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_49, 18, __pyx_float_10_38)) __PYX_ERR(0, 75, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_49, 19, __pyx_float__0002)) __PYX_ERR(0, 75, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_67);
  __Pyx_GIVEREF(__pyx_float_12_67);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_49, 20, __pyx_float_12_67)) __PYX_ERR(0, 75, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_49, 21, __pyx_float__0001)) __PYX_ERR(0, 75, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":76
 * 			        [1.00,.1655,2.00,.0445,3.00,.0120,4.00,.0038,5.02,.0015,6.07,.0038,7.11,.0003 ,8.19,.0002,9.25,.0010,10.36,.0011,11.48,.0005,12.63,.0002,13.79,.0003,16.24,.0002],
 * 			        [0.99,.3637,1.99,.0259,3.01,.0038,4.01,.0057,5.03,.0040,6.07,.0067,7.12,.0014 ,8.19,.0004,9.27,.0003,10.38,.0002,12.67,.0001],
 * 			        [1.00,.1193,2.00,.0230,3.00,.0104,4.01,.0084,5.04,.0047,6.08,.0035,7.13,.0041 ,8.20,.0002,9.29,.0005,10.40,.0005,11.53,.0003,12.70,.0002,13.91,.0002],             # <<<<<<<<<<<<<<
 * 			        [1.00,.0752,2.00,.0497,3.00,.0074,4.02,.0076,5.05,.0053,6.09,.0043,7.15,.0024 ,8.22,.0001,9.32,.0006,10.45,.0002,11.58,.0001,12.78,.0001,15.22,.0001],
 * 			        [1.00,.2388,2.00,.0629,3.01,.0159,4.04,.0063,5.07,.0051,6.12,.0045,7.19,.0026 ,8.29,.0015,9.43,.0001,11.75,.0002],
 */
  __pyx_t_50 = PyList_New(26); if (unlikely(!__pyx_t_50)) __PYX_ERR(0, 76, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_50);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_50, 0, __pyx_float_1_00)) __PYX_ERR(0, 76, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1193);
  __Pyx_GIVEREF(__pyx_float__1193);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_50, 1, __pyx_float__1193)) __PYX_ERR(0, 76, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_00);
  __Pyx_GIVEREF(__pyx_float_2_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_50, 2, __pyx_float_2_00)) __PYX_ERR(0, 76, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0230);
  __Pyx_GIVEREF(__pyx_float__0230);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_50, 3, __pyx_float__0230)) __PYX_ERR(0, 76, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_00);
  __Pyx_GIVEREF(__pyx_float_3_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_50, 4, __pyx_float_3_00)) __PYX_ERR(0, 76, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0104);
  __Pyx_GIVEREF(__pyx_float__0104);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_50, 5, __pyx_float__0104)) __PYX_ERR(0, 76, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_01);
  __Pyx_GIVEREF(__pyx_float_4_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_50, 6, __pyx_float_4_01)) __PYX_ERR(0, 76, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0084);
  __Pyx_GIVEREF(__pyx_float__0084);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_50, 7, __pyx_float__0084)) __PYX_ERR(0, 76, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_04);
  __Pyx_GIVEREF(__pyx_float_5_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_50, 8, __pyx_float_5_04)) __PYX_ERR(0, 76, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0047);
  __Pyx_GIVEREF(__pyx_float__0047);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_50, 9, __pyx_float__0047)) __PYX_ERR(0, 76, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_08);
  __Pyx_GIVEREF(__pyx_float_6_08);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_50, 10, __pyx_float_6_08)) __PYX_ERR(0, 76, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0035);
  __Pyx_GIVEREF(__pyx_float__0035);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_50, 11, __pyx_float__0035)) __PYX_ERR(0, 76, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_13);
  __Pyx_GIVEREF(__pyx_float_7_13);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_50, 12, __pyx_float_7_13)) __PYX_ERR(0, 76, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0041);
  __Pyx_GIVEREF(__pyx_float__0041);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_50, 13, __pyx_float__0041)) __PYX_ERR(0, 76, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_20);
  __Pyx_GIVEREF(__pyx_float_8_20);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_50, 14, __pyx_float_8_20)) __PYX_ERR(0, 76, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_50, 15, __pyx_float__0002)) __PYX_ERR(0, 76, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_29);
  __Pyx_GIVEREF(__pyx_float_9_29);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_50, 16, __pyx_float_9_29)) __PYX_ERR(0, 76, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_50, 17, __pyx_float__0005)) __PYX_ERR(0, 76, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_40);
  __Pyx_GIVEREF(__pyx_float_10_40);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_50, 18, __pyx_float_10_40)) __PYX_ERR(0, 76, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_50, 19, __pyx_float__0005)) __PYX_ERR(0, 76, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_53);
  __Pyx_GIVEREF(__pyx_float_11_53);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_50, 20, __pyx_float_11_53)) __PYX_ERR(0, 76, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_50, 21, __pyx_float__0003)) __PYX_ERR(0, 76, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_70);
  __Pyx_GIVEREF(__pyx_float_12_70);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_50, 22, __pyx_float_12_70)) __PYX_ERR(0, 76, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_50, 23, __pyx_float__0002)) __PYX_ERR(0, 76, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_91);
  __Pyx_GIVEREF(__pyx_float_13_91);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_50, 24, __pyx_float_13_91)) __PYX_ERR(0, 76, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_50, 25, __pyx_float__0002)) __PYX_ERR(0, 76, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":77
 * 			        [0.99,.3637,1.99,.0259,3.01,.0038,4.01,.0057,5.03,.0040,6.07,.0067,7.12,.0014 ,8.19,.0004,9.27,.0003,10.38,.0002,12.67,.0001],
 * 			        [1.00,.1193,2.00,.0230,3.00,.0104,4.01,.0084,5.04,.0047,6.08,.0035,7.13,.0041 ,8.20,.0002,9.29,.0005,10.40,.0005,11.53,.0003,12.70,.0002,13.91,.0002],
 * 			        [1.00,.0752,2.00,.0497,3.00,.0074,4.02,.0076,5.05,.0053,6.09,.0043,7.15,.0024 ,8.22,.0001,9.32,.0006,10.45,.0002,11.58,.0001,12.78,.0001,15.22,.0001],             # <<<<<<<<<<<<<<
 * 			        [1.00,.2388,2.00,.0629,3.01,.0159,4.04,.0063,5.07,.0051,6.12,.0045,7.19,.0026 ,8.29,.0015,9.43,.0001,11.75,.0002],
 * 			        [1.00,.1919,2.01,.0116,3.01,.0031,4.03,.0090,5.07,.0061,6.13,.0036,7.19,.0013 ,8.30,.0016,9.13,.0001,10.59,.0002,11.78,.0002],
 */
  __pyx_t_51 = PyList_New(26); if (unlikely(!__pyx_t_51)) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_51);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_51, 0, __pyx_float_1_00)) __PYX_ERR(0, 77, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0752);
  __Pyx_GIVEREF(__pyx_float__0752);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_51, 1, __pyx_float__0752)) __PYX_ERR(0, 77, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_00);
  __Pyx_GIVEREF(__pyx_float_2_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_51, 2, __pyx_float_2_00)) __PYX_ERR(0, 77, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0497);
  __Pyx_GIVEREF(__pyx_float__0497);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_51, 3, __pyx_float__0497)) __PYX_ERR(0, 77, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_00);
  __Pyx_GIVEREF(__pyx_float_3_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_51, 4, __pyx_float_3_00)) __PYX_ERR(0, 77, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0074);
  __Pyx_GIVEREF(__pyx_float__0074);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_51, 5, __pyx_float__0074)) __PYX_ERR(0, 77, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_02);
  __Pyx_GIVEREF(__pyx_float_4_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_51, 6, __pyx_float_4_02)) __PYX_ERR(0, 77, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0076);
  __Pyx_GIVEREF(__pyx_float__0076);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_51, 7, __pyx_float__0076)) __PYX_ERR(0, 77, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_05);
  __Pyx_GIVEREF(__pyx_float_5_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_51, 8, __pyx_float_5_05)) __PYX_ERR(0, 77, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0053);
  __Pyx_GIVEREF(__pyx_float__0053);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_51, 9, __pyx_float__0053)) __PYX_ERR(0, 77, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_09);
  __Pyx_GIVEREF(__pyx_float_6_09);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_51, 10, __pyx_float_6_09)) __PYX_ERR(0, 77, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0043);
  __Pyx_GIVEREF(__pyx_float__0043);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_51, 11, __pyx_float__0043)) __PYX_ERR(0, 77, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_15);
  __Pyx_GIVEREF(__pyx_float_7_15);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_51, 12, __pyx_float_7_15)) __PYX_ERR(0, 77, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0024);
  __Pyx_GIVEREF(__pyx_float__0024);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_51, 13, __pyx_float__0024)) __PYX_ERR(0, 77, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_22);
  __Pyx_GIVEREF(__pyx_float_8_22);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_51, 14, __pyx_float_8_22)) __PYX_ERR(0, 77, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_51, 15, __pyx_float__0001)) __PYX_ERR(0, 77, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_32);
  __Pyx_GIVEREF(__pyx_float_9_32);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_51, 16, __pyx_float_9_32)) __PYX_ERR(0, 77, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_51, 17, __pyx_float__0006)) __PYX_ERR(0, 77, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_45);
  __Pyx_GIVEREF(__pyx_float_10_45);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_51, 18, __pyx_float_10_45)) __PYX_ERR(0, 77, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_51, 19, __pyx_float__0002)) __PYX_ERR(0, 77, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_58);
  __Pyx_GIVEREF(__pyx_float_11_58);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_51, 20, __pyx_float_11_58)) __PYX_ERR(0, 77, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_51, 21, __pyx_float__0001)) __PYX_ERR(0, 77, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_12_78);
  __Pyx_GIVEREF(__pyx_float_12_78);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_51, 22, __pyx_float_12_78)) __PYX_ERR(0, 77, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_51, 23, __pyx_float__0001)) __PYX_ERR(0, 77, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_15_22);
  __Pyx_GIVEREF(__pyx_float_15_22);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_51, 24, __pyx_float_15_22)) __PYX_ERR(0, 77, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_51, 25, __pyx_float__0001)) __PYX_ERR(0, 77, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":78
 * 			        [1.00,.1193,2.00,.0230,3.00,.0104,4.01,.0084,5.04,.0047,6.08,.0035,7.13,.0041 ,8.20,.0002,9.29,.0005,10.40,.0005,11.53,.0003,12.70,.0002,13.91,.0002],
 * 			        [1.00,.0752,2.00,.0497,3.00,.0074,4.02,.0076,5.05,.0053,6.09,.0043,7.15,.0024 ,8.22,.0001,9.32,.0006,10.45,.0002,11.58,.0001,12.78,.0001,15.22,.0001],
 * 			        [1.00,.2388,2.00,.0629,3.01,.0159,4.04,.0063,5.07,.0051,6.12,.0045,7.19,.0026 ,8.29,.0015,9.43,.0001,11.75,.0002],             # <<<<<<<<<<<<<<
 * 			        [1.00,.1919,2.01,.0116,3.01,.0031,4.03,.0090,5.07,.0061,6.13,.0036,7.19,.0013 ,8.30,.0016,9.13,.0001,10.59,.0002,11.78,.0002],
 * 			        [1.00,.1296,2.00,.0135,3.01,.0041,4.04,.0045,5.09,.0028,6.14,.0046,7.23,.0007 ,8.32,.0007,9.50,.0001],
 */
  __pyx_t_52 = PyList_New(20); if (unlikely(!__pyx_t_52)) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_52);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_52, 0, __pyx_float_1_00)) __PYX_ERR(0, 78, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__2388);
  __Pyx_GIVEREF(__pyx_float__2388);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_52, 1, __pyx_float__2388)) __PYX_ERR(0, 78, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_00);
  __Pyx_GIVEREF(__pyx_float_2_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_52, 2, __pyx_float_2_00)) __PYX_ERR(0, 78, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0629);
  __Pyx_GIVEREF(__pyx_float__0629);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_52, 3, __pyx_float__0629)) __PYX_ERR(0, 78, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_01);
  __Pyx_GIVEREF(__pyx_float_3_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_52, 4, __pyx_float_3_01)) __PYX_ERR(0, 78, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0159);
  __Pyx_GIVEREF(__pyx_float__0159);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_52, 5, __pyx_float__0159)) __PYX_ERR(0, 78, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_04);
  __Pyx_GIVEREF(__pyx_float_4_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_52, 6, __pyx_float_4_04)) __PYX_ERR(0, 78, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0063);
  __Pyx_GIVEREF(__pyx_float__0063);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_52, 7, __pyx_float__0063)) __PYX_ERR(0, 78, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_07);
  __Pyx_GIVEREF(__pyx_float_5_07);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_52, 8, __pyx_float_5_07)) __PYX_ERR(0, 78, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0051);
  __Pyx_GIVEREF(__pyx_float__0051);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_52, 9, __pyx_float__0051)) __PYX_ERR(0, 78, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_12);
  __Pyx_GIVEREF(__pyx_float_6_12);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_52, 10, __pyx_float_6_12)) __PYX_ERR(0, 78, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0045);
  __Pyx_GIVEREF(__pyx_float__0045);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_52, 11, __pyx_float__0045)) __PYX_ERR(0, 78, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_19);
  __Pyx_GIVEREF(__pyx_float_7_19);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_52, 12, __pyx_float_7_19)) __PYX_ERR(0, 78, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0026);
  __Pyx_GIVEREF(__pyx_float__0026);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_52, 13, __pyx_float__0026)) __PYX_ERR(0, 78, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_29);
  __Pyx_GIVEREF(__pyx_float_8_29);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_52, 14, __pyx_float_8_29)) __PYX_ERR(0, 78, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0015);
  __Pyx_GIVEREF(__pyx_float__0015);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_52, 15, __pyx_float__0015)) __PYX_ERR(0, 78, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_43);
  __Pyx_GIVEREF(__pyx_float_9_43);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_52, 16, __pyx_float_9_43)) __PYX_ERR(0, 78, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_52, 17, __pyx_float__0001)) __PYX_ERR(0, 78, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_75);
  __Pyx_GIVEREF(__pyx_float_11_75);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_52, 18, __pyx_float_11_75)) __PYX_ERR(0, 78, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_52, 19, __pyx_float__0002)) __PYX_ERR(0, 78, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":79
 * 			        [1.00,.0752,2.00,.0497,3.00,.0074,4.02,.0076,5.05,.0053,6.09,.0043,7.15,.0024 ,8.22,.0001,9.32,.0006,10.45,.0002,11.58,.0001,12.78,.0001,15.22,.0001],
 * 			        [1.00,.2388,2.00,.0629,3.01,.0159,4.04,.0063,5.07,.0051,6.12,.0045,7.19,.0026 ,8.29,.0015,9.43,.0001,11.75,.0002],
 * 			        [1.00,.1919,2.01,.0116,3.01,.0031,4.03,.0090,5.07,.0061,6.13,.0036,7.19,.0013 ,8.30,.0016,9.13,.0001,10.59,.0002,11.78,.0002],             # <<<<<<<<<<<<<<
 * 			        [1.00,.1296,2.00,.0135,3.01,.0041,4.04,.0045,5.09,.0028,6.14,.0046,7.23,.0007 ,8.32,.0007,9.50,.0001],
 * 			        [1.00,.0692,2.00,.0209,3.02,.0025,4.05,.0030,5.09,.0047,6.17,.0022,7.25,.0015 ,8.36,.0015,9.53,.0010,10.69,.0001,13.40,.0001],
 */
  __pyx_t_53 = PyList_New(22); if (unlikely(!__pyx_t_53)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_53);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_53, 0, __pyx_float_1_00)) __PYX_ERR(0, 79, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1919);
  __Pyx_GIVEREF(__pyx_float__1919);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_53, 1, __pyx_float__1919)) __PYX_ERR(0, 79, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_01);
  __Pyx_GIVEREF(__pyx_float_2_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_53, 2, __pyx_float_2_01)) __PYX_ERR(0, 79, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0116);
  __Pyx_GIVEREF(__pyx_float__0116);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_53, 3, __pyx_float__0116)) __PYX_ERR(0, 79, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_01);
  __Pyx_GIVEREF(__pyx_float_3_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_53, 4, __pyx_float_3_01)) __PYX_ERR(0, 79, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0031);
  __Pyx_GIVEREF(__pyx_float__0031);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_53, 5, __pyx_float__0031)) __PYX_ERR(0, 79, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_03);
  __Pyx_GIVEREF(__pyx_float_4_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_53, 6, __pyx_float_4_03)) __PYX_ERR(0, 79, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0090);
  __Pyx_GIVEREF(__pyx_float__0090);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_53, 7, __pyx_float__0090)) __PYX_ERR(0, 79, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_07);
  __Pyx_GIVEREF(__pyx_float_5_07);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_53, 8, __pyx_float_5_07)) __PYX_ERR(0, 79, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0061);
  __Pyx_GIVEREF(__pyx_float__0061);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_53, 9, __pyx_float__0061)) __PYX_ERR(0, 79, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_13);
  __Pyx_GIVEREF(__pyx_float_6_13);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_53, 10, __pyx_float_6_13)) __PYX_ERR(0, 79, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0036);
  __Pyx_GIVEREF(__pyx_float__0036);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_53, 11, __pyx_float__0036)) __PYX_ERR(0, 79, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_19);
  __Pyx_GIVEREF(__pyx_float_7_19);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_53, 12, __pyx_float_7_19)) __PYX_ERR(0, 79, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0013);
  __Pyx_GIVEREF(__pyx_float__0013);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_53, 13, __pyx_float__0013)) __PYX_ERR(0, 79, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_30);
  __Pyx_GIVEREF(__pyx_float_8_30);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_53, 14, __pyx_float_8_30)) __PYX_ERR(0, 79, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0016);
  __Pyx_GIVEREF(__pyx_float__0016);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_53, 15, __pyx_float__0016)) __PYX_ERR(0, 79, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_13);
  __Pyx_GIVEREF(__pyx_float_9_13);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_53, 16, __pyx_float_9_13)) __PYX_ERR(0, 79, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_53, 17, __pyx_float__0001)) __PYX_ERR(0, 79, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_59);
  __Pyx_GIVEREF(__pyx_float_10_59);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_53, 18, __pyx_float_10_59)) __PYX_ERR(0, 79, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_53, 19, __pyx_float__0002)) __PYX_ERR(0, 79, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_11_78);
  __Pyx_GIVEREF(__pyx_float_11_78);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_53, 20, __pyx_float_11_78)) __PYX_ERR(0, 79, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_53, 21, __pyx_float__0002)) __PYX_ERR(0, 79, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":80
 * 			        [1.00,.2388,2.00,.0629,3.01,.0159,4.04,.0063,5.07,.0051,6.12,.0045,7.19,.0026 ,8.29,.0015,9.43,.0001,11.75,.0002],
 * 			        [1.00,.1919,2.01,.0116,3.01,.0031,4.03,.0090,5.07,.0061,6.13,.0036,7.19,.0013 ,8.30,.0016,9.13,.0001,10.59,.0002,11.78,.0002],
 * 			        [1.00,.1296,2.00,.0135,3.01,.0041,4.04,.0045,5.09,.0028,6.14,.0046,7.23,.0007 ,8.32,.0007,9.50,.0001],             # <<<<<<<<<<<<<<
 * 			        [1.00,.0692,2.00,.0209,3.02,.0025,4.05,.0030,5.09,.0047,6.17,.0022,7.25,.0015 ,8.36,.0015,9.53,.0010,10.69,.0001,13.40,.0001],
 * 			        [1.00,.1715,2.00,.0142,3.01,.0024,4.03,.0015,5.07,.0017,6.13,.0018,7.22,.0009 ,8.33,.0014,9.51,.0007,10.69,.0002],
 */
  __pyx_t_54 = PyList_New(18); if (unlikely(!__pyx_t_54)) __PYX_ERR(0, 80, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_54);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_54, 0, __pyx_float_1_00)) __PYX_ERR(0, 80, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1296);
  __Pyx_GIVEREF(__pyx_float__1296);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_54, 1, __pyx_float__1296)) __PYX_ERR(0, 80, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_00);
  __Pyx_GIVEREF(__pyx_float_2_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_54, 2, __pyx_float_2_00)) __PYX_ERR(0, 80, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0135);
  __Pyx_GIVEREF(__pyx_float__0135);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_54, 3, __pyx_float__0135)) __PYX_ERR(0, 80, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_01);
  __Pyx_GIVEREF(__pyx_float_3_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_54, 4, __pyx_float_3_01)) __PYX_ERR(0, 80, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0041);
  __Pyx_GIVEREF(__pyx_float__0041);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_54, 5, __pyx_float__0041)) __PYX_ERR(0, 80, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_04);
  __Pyx_GIVEREF(__pyx_float_4_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_54, 6, __pyx_float_4_04)) __PYX_ERR(0, 80, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0045);
  __Pyx_GIVEREF(__pyx_float__0045);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_54, 7, __pyx_float__0045)) __PYX_ERR(0, 80, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_09);
  __Pyx_GIVEREF(__pyx_float_5_09);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_54, 8, __pyx_float_5_09)) __PYX_ERR(0, 80, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0028);
  __Pyx_GIVEREF(__pyx_float__0028);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_54, 9, __pyx_float__0028)) __PYX_ERR(0, 80, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_14);
  __Pyx_GIVEREF(__pyx_float_6_14);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_54, 10, __pyx_float_6_14)) __PYX_ERR(0, 80, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0046);
  __Pyx_GIVEREF(__pyx_float__0046);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_54, 11, __pyx_float__0046)) __PYX_ERR(0, 80, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_23);
  __Pyx_GIVEREF(__pyx_float_7_23);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_54, 12, __pyx_float_7_23)) __PYX_ERR(0, 80, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_54, 13, __pyx_float__0007)) __PYX_ERR(0, 80, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_32);
  __Pyx_GIVEREF(__pyx_float_8_32);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_54, 14, __pyx_float_8_32)) __PYX_ERR(0, 80, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_54, 15, __pyx_float__0007)) __PYX_ERR(0, 80, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_50);
  __Pyx_GIVEREF(__pyx_float_9_50);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_54, 16, __pyx_float_9_50)) __PYX_ERR(0, 80, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_54, 17, __pyx_float__0001)) __PYX_ERR(0, 80, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":81
 * 			        [1.00,.1919,2.01,.0116,3.01,.0031,4.03,.0090,5.07,.0061,6.13,.0036,7.19,.0013 ,8.30,.0016,9.13,.0001,10.59,.0002,11.78,.0002],
 * 			        [1.00,.1296,2.00,.0135,3.01,.0041,4.04,.0045,5.09,.0028,6.14,.0046,7.23,.0007 ,8.32,.0007,9.50,.0001],
 * 			        [1.00,.0692,2.00,.0209,3.02,.0025,4.05,.0030,5.09,.0047,6.17,.0022,7.25,.0015 ,8.36,.0015,9.53,.0010,10.69,.0001,13.40,.0001],             # <<<<<<<<<<<<<<
 * 			        [1.00,.1715,2.00,.0142,3.01,.0024,4.03,.0015,5.07,.0017,6.13,.0018,7.22,.0009 ,8.33,.0014,9.51,.0007,10.69,.0002],
 * 			        [1.00,.1555,2.01,.0148,3.02,.0007,4.06,.0006,5.10,.0005,6.16,.0008,7.26,.0009 ,8.39,.0008,9.58,.0002],
 */
  __pyx_t_55 = PyList_New(22); if (unlikely(!__pyx_t_55)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_55);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_55, 0, __pyx_float_1_00)) __PYX_ERR(0, 81, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0692);
  __Pyx_GIVEREF(__pyx_float__0692);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_55, 1, __pyx_float__0692)) __PYX_ERR(0, 81, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_00);
  __Pyx_GIVEREF(__pyx_float_2_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_55, 2, __pyx_float_2_00)) __PYX_ERR(0, 81, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0209);
  __Pyx_GIVEREF(__pyx_float__0209);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_55, 3, __pyx_float__0209)) __PYX_ERR(0, 81, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_02);
  __Pyx_GIVEREF(__pyx_float_3_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_55, 4, __pyx_float_3_02)) __PYX_ERR(0, 81, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0025);
  __Pyx_GIVEREF(__pyx_float__0025);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_55, 5, __pyx_float__0025)) __PYX_ERR(0, 81, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_05);
  __Pyx_GIVEREF(__pyx_float_4_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_55, 6, __pyx_float_4_05)) __PYX_ERR(0, 81, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0030);
  __Pyx_GIVEREF(__pyx_float__0030);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_55, 7, __pyx_float__0030)) __PYX_ERR(0, 81, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_09);
  __Pyx_GIVEREF(__pyx_float_5_09);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_55, 8, __pyx_float_5_09)) __PYX_ERR(0, 81, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0047);
  __Pyx_GIVEREF(__pyx_float__0047);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_55, 9, __pyx_float__0047)) __PYX_ERR(0, 81, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_17);
  __Pyx_GIVEREF(__pyx_float_6_17);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_55, 10, __pyx_float_6_17)) __PYX_ERR(0, 81, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0022);
  __Pyx_GIVEREF(__pyx_float__0022);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_55, 11, __pyx_float__0022)) __PYX_ERR(0, 81, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_25);
  __Pyx_GIVEREF(__pyx_float_7_25);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_55, 12, __pyx_float_7_25)) __PYX_ERR(0, 81, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0015);
  __Pyx_GIVEREF(__pyx_float__0015);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_55, 13, __pyx_float__0015)) __PYX_ERR(0, 81, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_36);
  __Pyx_GIVEREF(__pyx_float_8_36);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_55, 14, __pyx_float_8_36)) __PYX_ERR(0, 81, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0015);
  __Pyx_GIVEREF(__pyx_float__0015);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_55, 15, __pyx_float__0015)) __PYX_ERR(0, 81, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_53);
  __Pyx_GIVEREF(__pyx_float_9_53);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_55, 16, __pyx_float_9_53)) __PYX_ERR(0, 81, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0010);
  __Pyx_GIVEREF(__pyx_float__0010);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_55, 17, __pyx_float__0010)) __PYX_ERR(0, 81, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_69);
  __Pyx_GIVEREF(__pyx_float_10_69);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_55, 18, __pyx_float_10_69)) __PYX_ERR(0, 81, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_55, 19, __pyx_float__0001)) __PYX_ERR(0, 81, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_13_40);
  __Pyx_GIVEREF(__pyx_float_13_40);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_55, 20, __pyx_float_13_40)) __PYX_ERR(0, 81, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_55, 21, __pyx_float__0001)) __PYX_ERR(0, 81, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":82
 * 			        [1.00,.1296,2.00,.0135,3.01,.0041,4.04,.0045,5.09,.0028,6.14,.0046,7.23,.0007 ,8.32,.0007,9.50,.0001],
 * 			        [1.00,.0692,2.00,.0209,3.02,.0025,4.05,.0030,5.09,.0047,6.17,.0022,7.25,.0015 ,8.36,.0015,9.53,.0010,10.69,.0001,13.40,.0001],
 * 			        [1.00,.1715,2.00,.0142,3.01,.0024,4.03,.0015,5.07,.0017,6.13,.0018,7.22,.0009 ,8.33,.0014,9.51,.0007,10.69,.0002],             # <<<<<<<<<<<<<<
 * 			        [1.00,.1555,2.01,.0148,3.02,.0007,4.06,.0006,5.10,.0005,6.16,.0008,7.26,.0009 ,8.39,.0008,9.58,.0002],
 * 			        [1.00,.1357,2.00,.0116,3.02,.0026,4.04,.0009,5.09,.0004,6.17,.0005,7.27,.0002 ,8.40,.0001],
 */
  __pyx_t_56 = PyList_New(20); if (unlikely(!__pyx_t_56)) __PYX_ERR(0, 82, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_56);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_56, 0, __pyx_float_1_00)) __PYX_ERR(0, 82, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1715);
  __Pyx_GIVEREF(__pyx_float__1715);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_56, 1, __pyx_float__1715)) __PYX_ERR(0, 82, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_00);
  __Pyx_GIVEREF(__pyx_float_2_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_56, 2, __pyx_float_2_00)) __PYX_ERR(0, 82, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0142);
  __Pyx_GIVEREF(__pyx_float__0142);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_56, 3, __pyx_float__0142)) __PYX_ERR(0, 82, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_01);
  __Pyx_GIVEREF(__pyx_float_3_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_56, 4, __pyx_float_3_01)) __PYX_ERR(0, 82, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0024);
  __Pyx_GIVEREF(__pyx_float__0024);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_56, 5, __pyx_float__0024)) __PYX_ERR(0, 82, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_03);
  __Pyx_GIVEREF(__pyx_float_4_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_56, 6, __pyx_float_4_03)) __PYX_ERR(0, 82, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0015);
  __Pyx_GIVEREF(__pyx_float__0015);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_56, 7, __pyx_float__0015)) __PYX_ERR(0, 82, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_07);
  __Pyx_GIVEREF(__pyx_float_5_07);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_56, 8, __pyx_float_5_07)) __PYX_ERR(0, 82, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0017);
  __Pyx_GIVEREF(__pyx_float__0017);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_56, 9, __pyx_float__0017)) __PYX_ERR(0, 82, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_13);
  __Pyx_GIVEREF(__pyx_float_6_13);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_56, 10, __pyx_float_6_13)) __PYX_ERR(0, 82, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0018);
  __Pyx_GIVEREF(__pyx_float__0018);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_56, 11, __pyx_float__0018)) __PYX_ERR(0, 82, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_22);
  __Pyx_GIVEREF(__pyx_float_7_22);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_56, 12, __pyx_float_7_22)) __PYX_ERR(0, 82, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_56, 13, __pyx_float__0009)) __PYX_ERR(0, 82, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_33);
  __Pyx_GIVEREF(__pyx_float_8_33);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_56, 14, __pyx_float_8_33)) __PYX_ERR(0, 82, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0014);
  __Pyx_GIVEREF(__pyx_float__0014);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_56, 15, __pyx_float__0014)) __PYX_ERR(0, 82, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_51);
  __Pyx_GIVEREF(__pyx_float_9_51);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_56, 16, __pyx_float_9_51)) __PYX_ERR(0, 82, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_56, 17, __pyx_float__0007)) __PYX_ERR(0, 82, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_10_69);
  __Pyx_GIVEREF(__pyx_float_10_69);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_56, 18, __pyx_float_10_69)) __PYX_ERR(0, 82, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_56, 19, __pyx_float__0002)) __PYX_ERR(0, 82, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":83
 * 			        [1.00,.0692,2.00,.0209,3.02,.0025,4.05,.0030,5.09,.0047,6.17,.0022,7.25,.0015 ,8.36,.0015,9.53,.0010,10.69,.0001,13.40,.0001],
 * 			        [1.00,.1715,2.00,.0142,3.01,.0024,4.03,.0015,5.07,.0017,6.13,.0018,7.22,.0009 ,8.33,.0014,9.51,.0007,10.69,.0002],
 * 			        [1.00,.1555,2.01,.0148,3.02,.0007,4.06,.0006,5.10,.0005,6.16,.0008,7.26,.0009 ,8.39,.0008,9.58,.0002],             # <<<<<<<<<<<<<<
 * 			        [1.00,.1357,2.00,.0116,3.02,.0026,4.04,.0009,5.09,.0004,6.17,.0005,7.27,.0002 ,8.40,.0001],
 * 			        [1.00,.2185,2.01,.0087,3.03,.0018,4.06,.0025,5.11,.0020,6.20,.0012,7.32,.0005 ,8.46,.0001,9.66,.0003],
 */
  __pyx_t_57 = PyList_New(18); if (unlikely(!__pyx_t_57)) __PYX_ERR(0, 83, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_57);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_57, 0, __pyx_float_1_00)) __PYX_ERR(0, 83, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1555);
  __Pyx_GIVEREF(__pyx_float__1555);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_57, 1, __pyx_float__1555)) __PYX_ERR(0, 83, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_01);
  __Pyx_GIVEREF(__pyx_float_2_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_57, 2, __pyx_float_2_01)) __PYX_ERR(0, 83, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0148);
  __Pyx_GIVEREF(__pyx_float__0148);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_57, 3, __pyx_float__0148)) __PYX_ERR(0, 83, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_02);
  __Pyx_GIVEREF(__pyx_float_3_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_57, 4, __pyx_float_3_02)) __PYX_ERR(0, 83, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_57, 5, __pyx_float__0007)) __PYX_ERR(0, 83, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_06);
  __Pyx_GIVEREF(__pyx_float_4_06);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_57, 6, __pyx_float_4_06)) __PYX_ERR(0, 83, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0006);
  __Pyx_GIVEREF(__pyx_float__0006);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_57, 7, __pyx_float__0006)) __PYX_ERR(0, 83, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_10);
  __Pyx_GIVEREF(__pyx_float_5_10);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_57, 8, __pyx_float_5_10)) __PYX_ERR(0, 83, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_57, 9, __pyx_float__0005)) __PYX_ERR(0, 83, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_16);
  __Pyx_GIVEREF(__pyx_float_6_16);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_57, 10, __pyx_float_6_16)) __PYX_ERR(0, 83, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_57, 11, __pyx_float__0008)) __PYX_ERR(0, 83, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_26);
  __Pyx_GIVEREF(__pyx_float_7_26);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_57, 12, __pyx_float_7_26)) __PYX_ERR(0, 83, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_57, 13, __pyx_float__0009)) __PYX_ERR(0, 83, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_39);
  __Pyx_GIVEREF(__pyx_float_8_39);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_57, 14, __pyx_float_8_39)) __PYX_ERR(0, 83, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_57, 15, __pyx_float__0008)) __PYX_ERR(0, 83, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_58);
  __Pyx_GIVEREF(__pyx_float_9_58);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_57, 16, __pyx_float_9_58)) __PYX_ERR(0, 83, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_57, 17, __pyx_float__0002)) __PYX_ERR(0, 83, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":84
 * 			        [1.00,.1715,2.00,.0142,3.01,.0024,4.03,.0015,5.07,.0017,6.13,.0018,7.22,.0009 ,8.33,.0014,9.51,.0007,10.69,.0002],
 * 			        [1.00,.1555,2.01,.0148,3.02,.0007,4.06,.0006,5.10,.0005,6.16,.0008,7.26,.0009 ,8.39,.0008,9.58,.0002],
 * 			        [1.00,.1357,2.00,.0116,3.02,.0026,4.04,.0009,5.09,.0004,6.17,.0005,7.27,.0002 ,8.40,.0001],             # <<<<<<<<<<<<<<
 * 			        [1.00,.2185,2.01,.0087,3.03,.0018,4.06,.0025,5.11,.0020,6.20,.0012,7.32,.0005 ,8.46,.0001,9.66,.0003],
 * 			        [1.00,.2735,2.00,.0038,3.02,.0008,4.06,.0012,5.12,.0008,6.22,.0011,7.35,.0003 ,8.50,.0002],
 */
  __pyx_t_58 = PyList_New(16); if (unlikely(!__pyx_t_58)) __PYX_ERR(0, 84, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_58);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_58, 0, __pyx_float_1_00)) __PYX_ERR(0, 84, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1357);
  __Pyx_GIVEREF(__pyx_float__1357);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_58, 1, __pyx_float__1357)) __PYX_ERR(0, 84, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_00);
  __Pyx_GIVEREF(__pyx_float_2_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_58, 2, __pyx_float_2_00)) __PYX_ERR(0, 84, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0116);
  __Pyx_GIVEREF(__pyx_float__0116);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_58, 3, __pyx_float__0116)) __PYX_ERR(0, 84, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_02);
  __Pyx_GIVEREF(__pyx_float_3_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_58, 4, __pyx_float_3_02)) __PYX_ERR(0, 84, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0026);
  __Pyx_GIVEREF(__pyx_float__0026);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_58, 5, __pyx_float__0026)) __PYX_ERR(0, 84, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_04);
  __Pyx_GIVEREF(__pyx_float_4_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_58, 6, __pyx_float_4_04)) __PYX_ERR(0, 84, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0009);
  __Pyx_GIVEREF(__pyx_float__0009);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_58, 7, __pyx_float__0009)) __PYX_ERR(0, 84, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_09);
  __Pyx_GIVEREF(__pyx_float_5_09);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_58, 8, __pyx_float_5_09)) __PYX_ERR(0, 84, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_58, 9, __pyx_float__0004)) __PYX_ERR(0, 84, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_17);
  __Pyx_GIVEREF(__pyx_float_6_17);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_58, 10, __pyx_float_6_17)) __PYX_ERR(0, 84, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_58, 11, __pyx_float__0005)) __PYX_ERR(0, 84, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_27);
  __Pyx_GIVEREF(__pyx_float_7_27);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_58, 12, __pyx_float_7_27)) __PYX_ERR(0, 84, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_58, 13, __pyx_float__0002)) __PYX_ERR(0, 84, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_40);
  __Pyx_GIVEREF(__pyx_float_8_40);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_58, 14, __pyx_float_8_40)) __PYX_ERR(0, 84, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_58, 15, __pyx_float__0001)) __PYX_ERR(0, 84, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":85
 * 			        [1.00,.1555,2.01,.0148,3.02,.0007,4.06,.0006,5.10,.0005,6.16,.0008,7.26,.0009 ,8.39,.0008,9.58,.0002],
 * 			        [1.00,.1357,2.00,.0116,3.02,.0026,4.04,.0009,5.09,.0004,6.17,.0005,7.27,.0002 ,8.40,.0001],
 * 			        [1.00,.2185,2.01,.0087,3.03,.0018,4.06,.0025,5.11,.0020,6.20,.0012,7.32,.0005 ,8.46,.0001,9.66,.0003],             # <<<<<<<<<<<<<<
 * 			        [1.00,.2735,2.00,.0038,3.02,.0008,4.06,.0012,5.12,.0008,6.22,.0011,7.35,.0003 ,8.50,.0002],
 * 			        [1.00,.1441,1.99,.0062,3.01,.0023,4.05,.0011,5.11,.0012,6.20,.0003,7.33,.0004 ,8.50,.0001],
 */
  __pyx_t_59 = PyList_New(18); if (unlikely(!__pyx_t_59)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_59);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_59, 0, __pyx_float_1_00)) __PYX_ERR(0, 85, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__2185);
  __Pyx_GIVEREF(__pyx_float__2185);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_59, 1, __pyx_float__2185)) __PYX_ERR(0, 85, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_01);
  __Pyx_GIVEREF(__pyx_float_2_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_59, 2, __pyx_float_2_01)) __PYX_ERR(0, 85, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0087);
  __Pyx_GIVEREF(__pyx_float__0087);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_59, 3, __pyx_float__0087)) __PYX_ERR(0, 85, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_03);
  __Pyx_GIVEREF(__pyx_float_3_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_59, 4, __pyx_float_3_03)) __PYX_ERR(0, 85, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0018);
  __Pyx_GIVEREF(__pyx_float__0018);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_59, 5, __pyx_float__0018)) __PYX_ERR(0, 85, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_06);
  __Pyx_GIVEREF(__pyx_float_4_06);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_59, 6, __pyx_float_4_06)) __PYX_ERR(0, 85, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0025);
  __Pyx_GIVEREF(__pyx_float__0025);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_59, 7, __pyx_float__0025)) __PYX_ERR(0, 85, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_11);
  __Pyx_GIVEREF(__pyx_float_5_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_59, 8, __pyx_float_5_11)) __PYX_ERR(0, 85, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0020);
  __Pyx_GIVEREF(__pyx_float__0020);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_59, 9, __pyx_float__0020)) __PYX_ERR(0, 85, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_20);
  __Pyx_GIVEREF(__pyx_float_6_20);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_59, 10, __pyx_float_6_20)) __PYX_ERR(0, 85, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_59, 11, __pyx_float__0012)) __PYX_ERR(0, 85, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_32);
  __Pyx_GIVEREF(__pyx_float_7_32);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_59, 12, __pyx_float_7_32)) __PYX_ERR(0, 85, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_59, 13, __pyx_float__0005)) __PYX_ERR(0, 85, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_46);
  __Pyx_GIVEREF(__pyx_float_8_46);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_59, 14, __pyx_float_8_46)) __PYX_ERR(0, 85, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_59, 15, __pyx_float__0001)) __PYX_ERR(0, 85, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_9_66);
  __Pyx_GIVEREF(__pyx_float_9_66);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_59, 16, __pyx_float_9_66)) __PYX_ERR(0, 85, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_59, 17, __pyx_float__0003)) __PYX_ERR(0, 85, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":86
 * 			        [1.00,.1357,2.00,.0116,3.02,.0026,4.04,.0009,5.09,.0004,6.17,.0005,7.27,.0002 ,8.40,.0001],
 * 			        [1.00,.2185,2.01,.0087,3.03,.0018,4.06,.0025,5.11,.0020,6.20,.0012,7.32,.0005 ,8.46,.0001,9.66,.0003],
 * 			        [1.00,.2735,2.00,.0038,3.02,.0008,4.06,.0012,5.12,.0008,6.22,.0011,7.35,.0003 ,8.50,.0002],             # <<<<<<<<<<<<<<
 * 			        [1.00,.1441,1.99,.0062,3.01,.0023,4.05,.0011,5.11,.0012,6.20,.0003,7.33,.0004 ,8.50,.0001],
 * 			        [1.00,.0726,2.01,.0293,3.03,.0022,5.14,.0005,6.26,.0011,7.41,.0002,8.63,.0002],
 */
  __pyx_t_60 = PyList_New(16); if (unlikely(!__pyx_t_60)) __PYX_ERR(0, 86, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_60);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_60, 0, __pyx_float_1_00)) __PYX_ERR(0, 86, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__2735);
  __Pyx_GIVEREF(__pyx_float__2735);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_60, 1, __pyx_float__2735)) __PYX_ERR(0, 86, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_00);
  __Pyx_GIVEREF(__pyx_float_2_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_60, 2, __pyx_float_2_00)) __PYX_ERR(0, 86, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0038);
  __Pyx_GIVEREF(__pyx_float__0038);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_60, 3, __pyx_float__0038)) __PYX_ERR(0, 86, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_02);
  __Pyx_GIVEREF(__pyx_float_3_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_60, 4, __pyx_float_3_02)) __PYX_ERR(0, 86, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_60, 5, __pyx_float__0008)) __PYX_ERR(0, 86, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_06);
  __Pyx_GIVEREF(__pyx_float_4_06);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_60, 6, __pyx_float_4_06)) __PYX_ERR(0, 86, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_60, 7, __pyx_float__0012)) __PYX_ERR(0, 86, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_12);
  __Pyx_GIVEREF(__pyx_float_5_12);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_60, 8, __pyx_float_5_12)) __PYX_ERR(0, 86, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0008);
  __Pyx_GIVEREF(__pyx_float__0008);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_60, 9, __pyx_float__0008)) __PYX_ERR(0, 86, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_22);
  __Pyx_GIVEREF(__pyx_float_6_22);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_60, 10, __pyx_float_6_22)) __PYX_ERR(0, 86, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_60, 11, __pyx_float__0011)) __PYX_ERR(0, 86, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_35);
  __Pyx_GIVEREF(__pyx_float_7_35);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_60, 12, __pyx_float_7_35)) __PYX_ERR(0, 86, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_60, 13, __pyx_float__0003)) __PYX_ERR(0, 86, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_50);
  __Pyx_GIVEREF(__pyx_float_8_50);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_60, 14, __pyx_float_8_50)) __PYX_ERR(0, 86, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_60, 15, __pyx_float__0002)) __PYX_ERR(0, 86, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":87
 * 			        [1.00,.2185,2.01,.0087,3.03,.0018,4.06,.0025,5.11,.0020,6.20,.0012,7.32,.0005 ,8.46,.0001,9.66,.0003],
 * 			        [1.00,.2735,2.00,.0038,3.02,.0008,4.06,.0012,5.12,.0008,6.22,.0011,7.35,.0003 ,8.50,.0002],
 * 			        [1.00,.1441,1.99,.0062,3.01,.0023,4.05,.0011,5.11,.0012,6.20,.0003,7.33,.0004 ,8.50,.0001],             # <<<<<<<<<<<<<<
 * 			        [1.00,.0726,2.01,.0293,3.03,.0022,5.14,.0005,6.26,.0011,7.41,.0002,8.63,.0002],
 * 			        [1.00,.0516,2.00,.0104,3.02,.0029,5.15,.0002,6.27,.0001],
 */
  __pyx_t_61 = PyList_New(16); if (unlikely(!__pyx_t_61)) __PYX_ERR(0, 87, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_61);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_61, 0, __pyx_float_1_00)) __PYX_ERR(0, 87, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__1441);
  __Pyx_GIVEREF(__pyx_float__1441);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_61, 1, __pyx_float__1441)) __PYX_ERR(0, 87, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_1_99);
  __Pyx_GIVEREF(__pyx_float_1_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_61, 2, __pyx_float_1_99)) __PYX_ERR(0, 87, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0062);
  __Pyx_GIVEREF(__pyx_float__0062);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_61, 3, __pyx_float__0062)) __PYX_ERR(0, 87, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_01);
  __Pyx_GIVEREF(__pyx_float_3_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_61, 4, __pyx_float_3_01)) __PYX_ERR(0, 87, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0023);
  __Pyx_GIVEREF(__pyx_float__0023);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_61, 5, __pyx_float__0023)) __PYX_ERR(0, 87, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_05);
  __Pyx_GIVEREF(__pyx_float_4_05);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_61, 6, __pyx_float_4_05)) __PYX_ERR(0, 87, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_61, 7, __pyx_float__0011)) __PYX_ERR(0, 87, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_11);
  __Pyx_GIVEREF(__pyx_float_5_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_61, 8, __pyx_float_5_11)) __PYX_ERR(0, 87, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_61, 9, __pyx_float__0012)) __PYX_ERR(0, 87, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_20);
  __Pyx_GIVEREF(__pyx_float_6_20);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_61, 10, __pyx_float_6_20)) __PYX_ERR(0, 87, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_61, 11, __pyx_float__0003)) __PYX_ERR(0, 87, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_33);
  __Pyx_GIVEREF(__pyx_float_7_33);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_61, 12, __pyx_float_7_33)) __PYX_ERR(0, 87, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_61, 13, __pyx_float__0004)) __PYX_ERR(0, 87, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_50);
  __Pyx_GIVEREF(__pyx_float_8_50);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_61, 14, __pyx_float_8_50)) __PYX_ERR(0, 87, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_61, 15, __pyx_float__0001)) __PYX_ERR(0, 87, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":88
 * 			        [1.00,.2735,2.00,.0038,3.02,.0008,4.06,.0012,5.12,.0008,6.22,.0011,7.35,.0003 ,8.50,.0002],
 * 			        [1.00,.1441,1.99,.0062,3.01,.0023,4.05,.0011,5.11,.0012,6.20,.0003,7.33,.0004 ,8.50,.0001],
 * 			        [1.00,.0726,2.01,.0293,3.03,.0022,5.14,.0005,6.26,.0011,7.41,.0002,8.63,.0002],             # <<<<<<<<<<<<<<
 * 			        [1.00,.0516,2.00,.0104,3.02,.0029,5.15,.0002,6.27,.0001],
 * 			        [1.00,.0329,2.00,.0033,3.03,.0013,4.10,.0005,5.19,.0004,6.32,.0002],
 */
  __pyx_t_62 = PyList_New(14); if (unlikely(!__pyx_t_62)) __PYX_ERR(0, 88, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_62);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_62, 0, __pyx_float_1_00)) __PYX_ERR(0, 88, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0726);
  __Pyx_GIVEREF(__pyx_float__0726);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_62, 1, __pyx_float__0726)) __PYX_ERR(0, 88, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_01);
  __Pyx_GIVEREF(__pyx_float_2_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_62, 2, __pyx_float_2_01)) __PYX_ERR(0, 88, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0293);
  __Pyx_GIVEREF(__pyx_float__0293);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_62, 3, __pyx_float__0293)) __PYX_ERR(0, 88, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_03);
  __Pyx_GIVEREF(__pyx_float_3_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_62, 4, __pyx_float_3_03)) __PYX_ERR(0, 88, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0022);
  __Pyx_GIVEREF(__pyx_float__0022);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_62, 5, __pyx_float__0022)) __PYX_ERR(0, 88, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_14);
  __Pyx_GIVEREF(__pyx_float_5_14);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_62, 6, __pyx_float_5_14)) __PYX_ERR(0, 88, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_62, 7, __pyx_float__0005)) __PYX_ERR(0, 88, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_26);
  __Pyx_GIVEREF(__pyx_float_6_26);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_62, 8, __pyx_float_6_26)) __PYX_ERR(0, 88, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_62, 9, __pyx_float__0011)) __PYX_ERR(0, 88, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_7_41);
  __Pyx_GIVEREF(__pyx_float_7_41);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_62, 10, __pyx_float_7_41)) __PYX_ERR(0, 88, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_62, 11, __pyx_float__0002)) __PYX_ERR(0, 88, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_8_63);
  __Pyx_GIVEREF(__pyx_float_8_63);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_62, 12, __pyx_float_8_63)) __PYX_ERR(0, 88, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_62, 13, __pyx_float__0002)) __PYX_ERR(0, 88, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":89
 * 			        [1.00,.1441,1.99,.0062,3.01,.0023,4.05,.0011,5.11,.0012,6.20,.0003,7.33,.0004 ,8.50,.0001],
 * 			        [1.00,.0726,2.01,.0293,3.03,.0022,5.14,.0005,6.26,.0011,7.41,.0002,8.63,.0002],
 * 			        [1.00,.0516,2.00,.0104,3.02,.0029,5.15,.0002,6.27,.0001],             # <<<<<<<<<<<<<<
 * 			        [1.00,.0329,2.00,.0033,3.03,.0013,4.10,.0005,5.19,.0004,6.32,.0002],
 * 			        [1.00,.0179,1.99,.0012,3.04,.0005,4.10,.0017,5.20,.0005,6.35,.0001],
 */
  __pyx_t_63 = PyList_New(10); if (unlikely(!__pyx_t_63)) __PYX_ERR(0, 89, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_63);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_63, 0, __pyx_float_1_00)) __PYX_ERR(0, 89, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0516);
  __Pyx_GIVEREF(__pyx_float__0516);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_63, 1, __pyx_float__0516)) __PYX_ERR(0, 89, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_00);
  __Pyx_GIVEREF(__pyx_float_2_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_63, 2, __pyx_float_2_00)) __PYX_ERR(0, 89, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0104);
  __Pyx_GIVEREF(__pyx_float__0104);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_63, 3, __pyx_float__0104)) __PYX_ERR(0, 89, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_02);
  __Pyx_GIVEREF(__pyx_float_3_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_63, 4, __pyx_float_3_02)) __PYX_ERR(0, 89, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0029);
  __Pyx_GIVEREF(__pyx_float__0029);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_63, 5, __pyx_float__0029)) __PYX_ERR(0, 89, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_15);
  __Pyx_GIVEREF(__pyx_float_5_15);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_63, 6, __pyx_float_5_15)) __PYX_ERR(0, 89, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_63, 7, __pyx_float__0002)) __PYX_ERR(0, 89, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_27);
  __Pyx_GIVEREF(__pyx_float_6_27);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_63, 8, __pyx_float_6_27)) __PYX_ERR(0, 89, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_63, 9, __pyx_float__0001)) __PYX_ERR(0, 89, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":90
 * 			        [1.00,.0726,2.01,.0293,3.03,.0022,5.14,.0005,6.26,.0011,7.41,.0002,8.63,.0002],
 * 			        [1.00,.0516,2.00,.0104,3.02,.0029,5.15,.0002,6.27,.0001],
 * 			        [1.00,.0329,2.00,.0033,3.03,.0013,4.10,.0005,5.19,.0004,6.32,.0002],             # <<<<<<<<<<<<<<
 * 			        [1.00,.0179,1.99,.0012,3.04,.0005,4.10,.0017,5.20,.0005,6.35,.0001],
 * 			        [1.00,.0334,2.01,.0033,3.04,.0011,4.13,.0003,5.22,.0003],
 */
  __pyx_t_64 = PyList_New(12); if (unlikely(!__pyx_t_64)) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_64);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_64, 0, __pyx_float_1_00)) __PYX_ERR(0, 90, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0329);
  __Pyx_GIVEREF(__pyx_float__0329);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_64, 1, __pyx_float__0329)) __PYX_ERR(0, 90, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_00);
  __Pyx_GIVEREF(__pyx_float_2_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_64, 2, __pyx_float_2_00)) __PYX_ERR(0, 90, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0033);
  __Pyx_GIVEREF(__pyx_float__0033);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_64, 3, __pyx_float__0033)) __PYX_ERR(0, 90, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_03);
  __Pyx_GIVEREF(__pyx_float_3_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_64, 4, __pyx_float_3_03)) __PYX_ERR(0, 90, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0013);
  __Pyx_GIVEREF(__pyx_float__0013);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_64, 5, __pyx_float__0013)) __PYX_ERR(0, 90, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_10);
  __Pyx_GIVEREF(__pyx_float_4_10);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_64, 6, __pyx_float_4_10)) __PYX_ERR(0, 90, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_64, 7, __pyx_float__0005)) __PYX_ERR(0, 90, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_19);
  __Pyx_GIVEREF(__pyx_float_5_19);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_64, 8, __pyx_float_5_19)) __PYX_ERR(0, 90, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_64, 9, __pyx_float__0004)) __PYX_ERR(0, 90, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_32);
  __Pyx_GIVEREF(__pyx_float_6_32);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_64, 10, __pyx_float_6_32)) __PYX_ERR(0, 90, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_64, 11, __pyx_float__0002)) __PYX_ERR(0, 90, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":91
 * 			        [1.00,.0516,2.00,.0104,3.02,.0029,5.15,.0002,6.27,.0001],
 * 			        [1.00,.0329,2.00,.0033,3.03,.0013,4.10,.0005,5.19,.0004,6.32,.0002],
 * 			        [1.00,.0179,1.99,.0012,3.04,.0005,4.10,.0017,5.20,.0005,6.35,.0001],             # <<<<<<<<<<<<<<
 * 			        [1.00,.0334,2.01,.0033,3.04,.0011,4.13,.0003,5.22,.0003],
 * 			        [0.99,.0161,2.01,.0100,3.04,.0020,4.13,.0003],
 */
  __pyx_t_65 = PyList_New(12); if (unlikely(!__pyx_t_65)) __PYX_ERR(0, 91, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_65);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_65, 0, __pyx_float_1_00)) __PYX_ERR(0, 91, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0179);
  __Pyx_GIVEREF(__pyx_float__0179);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_65, 1, __pyx_float__0179)) __PYX_ERR(0, 91, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_1_99);
  __Pyx_GIVEREF(__pyx_float_1_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_65, 2, __pyx_float_1_99)) __PYX_ERR(0, 91, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0012);
  __Pyx_GIVEREF(__pyx_float__0012);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_65, 3, __pyx_float__0012)) __PYX_ERR(0, 91, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_04);
  __Pyx_GIVEREF(__pyx_float_3_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_65, 4, __pyx_float_3_04)) __PYX_ERR(0, 91, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_65, 5, __pyx_float__0005)) __PYX_ERR(0, 91, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_10);
  __Pyx_GIVEREF(__pyx_float_4_10);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_65, 6, __pyx_float_4_10)) __PYX_ERR(0, 91, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0017);
  __Pyx_GIVEREF(__pyx_float__0017);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_65, 7, __pyx_float__0017)) __PYX_ERR(0, 91, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_20);
  __Pyx_GIVEREF(__pyx_float_5_20);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_65, 8, __pyx_float_5_20)) __PYX_ERR(0, 91, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_65, 9, __pyx_float__0005)) __PYX_ERR(0, 91, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_6_35);
  __Pyx_GIVEREF(__pyx_float_6_35);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_65, 10, __pyx_float_6_35)) __PYX_ERR(0, 91, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_65, 11, __pyx_float__0001)) __PYX_ERR(0, 91, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":92
 * 			        [1.00,.0329,2.00,.0033,3.03,.0013,4.10,.0005,5.19,.0004,6.32,.0002],
 * 			        [1.00,.0179,1.99,.0012,3.04,.0005,4.10,.0017,5.20,.0005,6.35,.0001],
 * 			        [1.00,.0334,2.01,.0033,3.04,.0011,4.13,.0003,5.22,.0003],             # <<<<<<<<<<<<<<
 * 			        [0.99,.0161,2.01,.0100,3.04,.0020,4.13,.0003],
 * 			        [1.00,.0475,1.99,.0045,3.03,.0035,4.12,.0011],
 */
  __pyx_t_66 = PyList_New(10); if (unlikely(!__pyx_t_66)) __PYX_ERR(0, 92, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_66);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_66, 0, __pyx_float_1_00)) __PYX_ERR(0, 92, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0334);
  __Pyx_GIVEREF(__pyx_float__0334);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_66, 1, __pyx_float__0334)) __PYX_ERR(0, 92, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_01);
  __Pyx_GIVEREF(__pyx_float_2_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_66, 2, __pyx_float_2_01)) __PYX_ERR(0, 92, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0033);
  __Pyx_GIVEREF(__pyx_float__0033);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_66, 3, __pyx_float__0033)) __PYX_ERR(0, 92, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_04);
  __Pyx_GIVEREF(__pyx_float_3_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_66, 4, __pyx_float_3_04)) __PYX_ERR(0, 92, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_66, 5, __pyx_float__0011)) __PYX_ERR(0, 92, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_13);
  __Pyx_GIVEREF(__pyx_float_4_13);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_66, 6, __pyx_float_4_13)) __PYX_ERR(0, 92, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_66, 7, __pyx_float__0003)) __PYX_ERR(0, 92, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_5_22);
  __Pyx_GIVEREF(__pyx_float_5_22);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_66, 8, __pyx_float_5_22)) __PYX_ERR(0, 92, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_66, 9, __pyx_float__0003)) __PYX_ERR(0, 92, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":93
 * 			        [1.00,.0179,1.99,.0012,3.04,.0005,4.10,.0017,5.20,.0005,6.35,.0001],
 * 			        [1.00,.0334,2.01,.0033,3.04,.0011,4.13,.0003,5.22,.0003],
 * 			        [0.99,.0161,2.01,.0100,3.04,.0020,4.13,.0003],             # <<<<<<<<<<<<<<
 * 			        [1.00,.0475,1.99,.0045,3.03,.0035,4.12,.0011],
 * 			        [1.00,.0593,2.00,.0014,4.17,.0002],
 */
  __pyx_t_67 = PyList_New(8); if (unlikely(!__pyx_t_67)) __PYX_ERR(0, 93, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_67);
  __Pyx_INCREF(__pyx_float_0_99);
  __Pyx_GIVEREF(__pyx_float_0_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_67, 0, __pyx_float_0_99)) __PYX_ERR(0, 93, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0161);
  __Pyx_GIVEREF(__pyx_float__0161);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_67, 1, __pyx_float__0161)) __PYX_ERR(0, 93, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_01);
  __Pyx_GIVEREF(__pyx_float_2_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_67, 2, __pyx_float_2_01)) __PYX_ERR(0, 93, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0100);
  __Pyx_GIVEREF(__pyx_float__0100);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_67, 3, __pyx_float__0100)) __PYX_ERR(0, 93, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_04);
  __Pyx_GIVEREF(__pyx_float_3_04);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_67, 4, __pyx_float_3_04)) __PYX_ERR(0, 93, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0020);
  __Pyx_GIVEREF(__pyx_float__0020);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_67, 5, __pyx_float__0020)) __PYX_ERR(0, 93, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_13);
  __Pyx_GIVEREF(__pyx_float_4_13);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_67, 6, __pyx_float_4_13)) __PYX_ERR(0, 93, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_67, 7, __pyx_float__0003)) __PYX_ERR(0, 93, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":94
 * 			        [1.00,.0334,2.01,.0033,3.04,.0011,4.13,.0003,5.22,.0003],
 * 			        [0.99,.0161,2.01,.0100,3.04,.0020,4.13,.0003],
 * 			        [1.00,.0475,1.99,.0045,3.03,.0035,4.12,.0011],             # <<<<<<<<<<<<<<
 * 			        [1.00,.0593,2.00,.0014,4.17,.0002],
 * 			        [1.00,.0249,2.01,.0016],
 */
  __pyx_t_68 = PyList_New(8); if (unlikely(!__pyx_t_68)) __PYX_ERR(0, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_68);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_68, 0, __pyx_float_1_00)) __PYX_ERR(0, 94, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0475);
  __Pyx_GIVEREF(__pyx_float__0475);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_68, 1, __pyx_float__0475)) __PYX_ERR(0, 94, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_1_99);
  __Pyx_GIVEREF(__pyx_float_1_99);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_68, 2, __pyx_float_1_99)) __PYX_ERR(0, 94, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0045);
  __Pyx_GIVEREF(__pyx_float__0045);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_68, 3, __pyx_float__0045)) __PYX_ERR(0, 94, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_03);
  __Pyx_GIVEREF(__pyx_float_3_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_68, 4, __pyx_float_3_03)) __PYX_ERR(0, 94, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0035);
  __Pyx_GIVEREF(__pyx_float__0035);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_68, 5, __pyx_float__0035)) __PYX_ERR(0, 94, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_12);
  __Pyx_GIVEREF(__pyx_float_4_12);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_68, 6, __pyx_float_4_12)) __PYX_ERR(0, 94, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0011);
  __Pyx_GIVEREF(__pyx_float__0011);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_68, 7, __pyx_float__0011)) __PYX_ERR(0, 94, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":95
 * 			        [0.99,.0161,2.01,.0100,3.04,.0020,4.13,.0003],
 * 			        [1.00,.0475,1.99,.0045,3.03,.0035,4.12,.0011],
 * 			        [1.00,.0593,2.00,.0014,4.17,.0002],             # <<<<<<<<<<<<<<
 * 			        [1.00,.0249,2.01,.0016],
 * 			        [1.00,.0242,2.00,.0038,4.19,.0002],
 */
  __pyx_t_69 = PyList_New(6); if (unlikely(!__pyx_t_69)) __PYX_ERR(0, 95, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_69);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_69, 0, __pyx_float_1_00)) __PYX_ERR(0, 95, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0593);
  __Pyx_GIVEREF(__pyx_float__0593);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_69, 1, __pyx_float__0593)) __PYX_ERR(0, 95, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_00);
  __Pyx_GIVEREF(__pyx_float_2_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_69, 2, __pyx_float_2_00)) __PYX_ERR(0, 95, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0014);
  __Pyx_GIVEREF(__pyx_float__0014);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_69, 3, __pyx_float__0014)) __PYX_ERR(0, 95, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_17);
  __Pyx_GIVEREF(__pyx_float_4_17);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_69, 4, __pyx_float_4_17)) __PYX_ERR(0, 95, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_69, 5, __pyx_float__0002)) __PYX_ERR(0, 95, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":96
 * 			        [1.00,.0475,1.99,.0045,3.03,.0035,4.12,.0011],
 * 			        [1.00,.0593,2.00,.0014,4.17,.0002],
 * 			        [1.00,.0249,2.01,.0016],             # <<<<<<<<<<<<<<
 * 			        [1.00,.0242,2.00,.0038,4.19,.0002],
 * 			        [1.00,.0170,2.02,.0030],
 */
  __pyx_t_70 = PyList_New(4); if (unlikely(!__pyx_t_70)) __PYX_ERR(0, 96, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_70);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_70, 0, __pyx_float_1_00)) __PYX_ERR(0, 96, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0249);
  __Pyx_GIVEREF(__pyx_float__0249);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_70, 1, __pyx_float__0249)) __PYX_ERR(0, 96, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_01);
  __Pyx_GIVEREF(__pyx_float_2_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_70, 2, __pyx_float_2_01)) __PYX_ERR(0, 96, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0016);
  __Pyx_GIVEREF(__pyx_float__0016);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_70, 3, __pyx_float__0016)) __PYX_ERR(0, 96, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":97
 * 			        [1.00,.0593,2.00,.0014,4.17,.0002],
 * 			        [1.00,.0249,2.01,.0016],
 * 			        [1.00,.0242,2.00,.0038,4.19,.0002],             # <<<<<<<<<<<<<<
 * 			        [1.00,.0170,2.02,.0030],
 * 			        [1.00,.0381,2.00,.0017,3.09,.0002],
 */
  __pyx_t_71 = PyList_New(6); if (unlikely(!__pyx_t_71)) __PYX_ERR(0, 97, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_71);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_71, 0, __pyx_float_1_00)) __PYX_ERR(0, 97, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0242);
  __Pyx_GIVEREF(__pyx_float__0242);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_71, 1, __pyx_float__0242)) __PYX_ERR(0, 97, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_00);
  __Pyx_GIVEREF(__pyx_float_2_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_71, 2, __pyx_float_2_00)) __PYX_ERR(0, 97, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0038);
  __Pyx_GIVEREF(__pyx_float__0038);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_71, 3, __pyx_float__0038)) __PYX_ERR(0, 97, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_19);
  __Pyx_GIVEREF(__pyx_float_4_19);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_71, 4, __pyx_float_4_19)) __PYX_ERR(0, 97, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_71, 5, __pyx_float__0002)) __PYX_ERR(0, 97, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":98
 * 			        [1.00,.0249,2.01,.0016],
 * 			        [1.00,.0242,2.00,.0038,4.19,.0002],
 * 			        [1.00,.0170,2.02,.0030],             # <<<<<<<<<<<<<<
 * 			        [1.00,.0381,2.00,.0017,3.09,.0002],
 * 			        [1.00,.0141,2.03,.0005,3.11,.0003,4.26,.0001],
 */
  __pyx_t_72 = PyList_New(4); if (unlikely(!__pyx_t_72)) __PYX_ERR(0, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_72);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_72, 0, __pyx_float_1_00)) __PYX_ERR(0, 98, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0170);
  __Pyx_GIVEREF(__pyx_float__0170);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_72, 1, __pyx_float__0170)) __PYX_ERR(0, 98, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_02);
  __Pyx_GIVEREF(__pyx_float_2_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_72, 2, __pyx_float_2_02)) __PYX_ERR(0, 98, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0030);
  __Pyx_GIVEREF(__pyx_float__0030);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_72, 3, __pyx_float__0030)) __PYX_ERR(0, 98, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":99
 * 			        [1.00,.0242,2.00,.0038,4.19,.0002],
 * 			        [1.00,.0170,2.02,.0030],
 * 			        [1.00,.0381,2.00,.0017,3.09,.0002],             # <<<<<<<<<<<<<<
 * 			        [1.00,.0141,2.03,.0005,3.11,.0003,4.26,.0001],
 * 		            [1.00,.0122,2.03,.0024],
 */
  __pyx_t_73 = PyList_New(6); if (unlikely(!__pyx_t_73)) __PYX_ERR(0, 99, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_73);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_73, 0, __pyx_float_1_00)) __PYX_ERR(0, 99, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0381);
  __Pyx_GIVEREF(__pyx_float__0381);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_73, 1, __pyx_float__0381)) __PYX_ERR(0, 99, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_00);
  __Pyx_GIVEREF(__pyx_float_2_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_73, 2, __pyx_float_2_00)) __PYX_ERR(0, 99, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0017);
  __Pyx_GIVEREF(__pyx_float__0017);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_73, 3, __pyx_float__0017)) __PYX_ERR(0, 99, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_09);
  __Pyx_GIVEREF(__pyx_float_3_09);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_73, 4, __pyx_float_3_09)) __PYX_ERR(0, 99, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_73, 5, __pyx_float__0002)) __PYX_ERR(0, 99, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":100
 * 			        [1.00,.0170,2.02,.0030],
 * 			        [1.00,.0381,2.00,.0017,3.09,.0002],
 * 			        [1.00,.0141,2.03,.0005,3.11,.0003,4.26,.0001],             # <<<<<<<<<<<<<<
 * 		            [1.00,.0122,2.03,.0024],
 * 			        [1.00,.0107,2.07,.0007,3.12,.0004],
 */
  __pyx_t_74 = PyList_New(8); if (unlikely(!__pyx_t_74)) __PYX_ERR(0, 100, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_74);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_74, 0, __pyx_float_1_00)) __PYX_ERR(0, 100, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0141);
  __Pyx_GIVEREF(__pyx_float__0141);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_74, 1, __pyx_float__0141)) __PYX_ERR(0, 100, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_03);
  __Pyx_GIVEREF(__pyx_float_2_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_74, 2, __pyx_float_2_03)) __PYX_ERR(0, 100, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_74, 3, __pyx_float__0005)) __PYX_ERR(0, 100, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_11);
  __Pyx_GIVEREF(__pyx_float_3_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_74, 4, __pyx_float_3_11)) __PYX_ERR(0, 100, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0003);
  __Pyx_GIVEREF(__pyx_float__0003);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_74, 5, __pyx_float__0003)) __PYX_ERR(0, 100, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_4_26);
  __Pyx_GIVEREF(__pyx_float_4_26);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_74, 6, __pyx_float_4_26)) __PYX_ERR(0, 100, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_74, 7, __pyx_float__0001)) __PYX_ERR(0, 100, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":101
 * 			        [1.00,.0381,2.00,.0017,3.09,.0002],
 * 			        [1.00,.0141,2.03,.0005,3.11,.0003,4.26,.0001],
 * 		            [1.00,.0122,2.03,.0024],             # <<<<<<<<<<<<<<
 * 			        [1.00,.0107,2.07,.0007,3.12,.0004],
 * 		            [1.00,.0250,2.02,.0026,3.15,.0002],
 */
  __pyx_t_75 = PyList_New(4); if (unlikely(!__pyx_t_75)) __PYX_ERR(0, 101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_75);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_75, 0, __pyx_float_1_00)) __PYX_ERR(0, 101, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0122);
  __Pyx_GIVEREF(__pyx_float__0122);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_75, 1, __pyx_float__0122)) __PYX_ERR(0, 101, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_03);
  __Pyx_GIVEREF(__pyx_float_2_03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_75, 2, __pyx_float_2_03)) __PYX_ERR(0, 101, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0024);
  __Pyx_GIVEREF(__pyx_float__0024);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_75, 3, __pyx_float__0024)) __PYX_ERR(0, 101, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":102
 * 			        [1.00,.0141,2.03,.0005,3.11,.0003,4.26,.0001],
 * 		            [1.00,.0122,2.03,.0024],
 * 			        [1.00,.0107,2.07,.0007,3.12,.0004],             # <<<<<<<<<<<<<<
 * 		            [1.00,.0250,2.02,.0026,3.15,.0002],
 * 			        [1.01,.0092],
 */
  __pyx_t_76 = PyList_New(6); if (unlikely(!__pyx_t_76)) __PYX_ERR(0, 102, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_76);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_76, 0, __pyx_float_1_00)) __PYX_ERR(0, 102, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0107);
  __Pyx_GIVEREF(__pyx_float__0107);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_76, 1, __pyx_float__0107)) __PYX_ERR(0, 102, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_07);
  __Pyx_GIVEREF(__pyx_float_2_07);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_76, 2, __pyx_float_2_07)) __PYX_ERR(0, 102, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0007);
  __Pyx_GIVEREF(__pyx_float__0007);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_76, 3, __pyx_float__0007)) __PYX_ERR(0, 102, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_12);
  __Pyx_GIVEREF(__pyx_float_3_12);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_76, 4, __pyx_float_3_12)) __PYX_ERR(0, 102, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0004);
  __Pyx_GIVEREF(__pyx_float__0004);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_76, 5, __pyx_float__0004)) __PYX_ERR(0, 102, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":103
 * 		            [1.00,.0122,2.03,.0024],
 * 			        [1.00,.0107,2.07,.0007,3.12,.0004],
 * 		            [1.00,.0250,2.02,.0026,3.15,.0002],             # <<<<<<<<<<<<<<
 * 			        [1.01,.0092],
 * 			        [1.01,.0102,2.09,.0005],
 */
  __pyx_t_77 = PyList_New(6); if (unlikely(!__pyx_t_77)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_77);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_77, 0, __pyx_float_1_00)) __PYX_ERR(0, 103, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0250);
  __Pyx_GIVEREF(__pyx_float__0250);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_77, 1, __pyx_float__0250)) __PYX_ERR(0, 103, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_02);
  __Pyx_GIVEREF(__pyx_float_2_02);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_77, 2, __pyx_float_2_02)) __PYX_ERR(0, 103, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0026);
  __Pyx_GIVEREF(__pyx_float__0026);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_77, 3, __pyx_float__0026)) __PYX_ERR(0, 103, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_15);
  __Pyx_GIVEREF(__pyx_float_3_15);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_77, 4, __pyx_float_3_15)) __PYX_ERR(0, 103, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0002);
  __Pyx_GIVEREF(__pyx_float__0002);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_77, 5, __pyx_float__0002)) __PYX_ERR(0, 103, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":104
 * 			        [1.00,.0107,2.07,.0007,3.12,.0004],
 * 		            [1.00,.0250,2.02,.0026,3.15,.0002],
 * 			        [1.01,.0092],             # <<<<<<<<<<<<<<
 * 			        [1.01,.0102,2.09,.0005],
 * 			        [1.00,.0080,2.00,.0005,3.19,.0001],
 */
  __pyx_t_78 = PyList_New(2); if (unlikely(!__pyx_t_78)) __PYX_ERR(0, 104, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_78);
  __Pyx_INCREF(__pyx_float_1_01);
  __Pyx_GIVEREF(__pyx_float_1_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_78, 0, __pyx_float_1_01)) __PYX_ERR(0, 104, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0092);
  __Pyx_GIVEREF(__pyx_float__0092);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_78, 1, __pyx_float__0092)) __PYX_ERR(0, 104, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":105
 * 		            [1.00,.0250,2.02,.0026,3.15,.0002],
 * 			        [1.01,.0092],
 * 			        [1.01,.0102,2.09,.0005],             # <<<<<<<<<<<<<<
 * 			        [1.00,.0080,2.00,.0005,3.19,.0001],
 * 			        [1.01,.0298,2.01,.0005]]
 */
  __pyx_t_79 = PyList_New(4); if (unlikely(!__pyx_t_79)) __PYX_ERR(0, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_79);
  __Pyx_INCREF(__pyx_float_1_01);
  __Pyx_GIVEREF(__pyx_float_1_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_79, 0, __pyx_float_1_01)) __PYX_ERR(0, 105, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0102);
  __Pyx_GIVEREF(__pyx_float__0102);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_79, 1, __pyx_float__0102)) __PYX_ERR(0, 105, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_09);
  __Pyx_GIVEREF(__pyx_float_2_09);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_79, 2, __pyx_float_2_09)) __PYX_ERR(0, 105, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_79, 3, __pyx_float__0005)) __PYX_ERR(0, 105, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":106
 * 			        [1.01,.0092],
 * 			        [1.01,.0102,2.09,.0005],
 * 			        [1.00,.0080,2.00,.0005,3.19,.0001],             # <<<<<<<<<<<<<<
 * 			        [1.01,.0298,2.01,.0005]]
 * 
 */
  __pyx_t_80 = PyList_New(6); if (unlikely(!__pyx_t_80)) __PYX_ERR(0, 106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_80);
  __Pyx_INCREF(__pyx_float_1_00);
  __Pyx_GIVEREF(__pyx_float_1_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_80, 0, __pyx_float_1_00)) __PYX_ERR(0, 106, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0080);
  __Pyx_GIVEREF(__pyx_float__0080);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_80, 1, __pyx_float__0080)) __PYX_ERR(0, 106, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_00);
  __Pyx_GIVEREF(__pyx_float_2_00);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_80, 2, __pyx_float_2_00)) __PYX_ERR(0, 106, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_80, 3, __pyx_float__0005)) __PYX_ERR(0, 106, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_3_19);
  __Pyx_GIVEREF(__pyx_float_3_19);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_80, 4, __pyx_float_3_19)) __PYX_ERR(0, 106, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0001);
  __Pyx_GIVEREF(__pyx_float__0001);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_80, 5, __pyx_float__0001)) __PYX_ERR(0, 106, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":107
 * 			        [1.01,.0102,2.09,.0005],
 * 			        [1.00,.0080,2.00,.0005,3.19,.0001],
 * 			        [1.01,.0298,2.01,.0005]]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_81 = PyList_New(4); if (unlikely(!__pyx_t_81)) __PYX_ERR(0, 107, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_81);
  __Pyx_INCREF(__pyx_float_1_01);
  __Pyx_GIVEREF(__pyx_float_1_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_81, 0, __pyx_float_1_01)) __PYX_ERR(0, 107, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0298);
  __Pyx_GIVEREF(__pyx_float__0298);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_81, 1, __pyx_float__0298)) __PYX_ERR(0, 107, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_2_01);
  __Pyx_GIVEREF(__pyx_float_2_01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_81, 2, __pyx_float_2_01)) __PYX_ERR(0, 107, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__0005);
  __Pyx_GIVEREF(__pyx_float__0005);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_81, 3, __pyx_float__0005)) __PYX_ERR(0, 107, __pyx_L1_error);

  /* "pysndlib/instruments/lbj_piano.py":28
 * 
 * # moved this outside function definition
 * PIANO_SPECTRA = [[1.97,.0326,2.99,.0086,3.95,.0163,4.97,.0178,5.98,.0177,6.95,.0315,8.02,.0001 ,8.94,.0076,9.96,.0134,10.99,.0284,11.98,.0229,13.02,.0229,13.89,.0010,15.06,.0090,16.00,.0003 ,17.08,.0078,18.16,.0064,19.18,.0129,20.21,.0085,21.27,.0225,22.32,.0061,23.41,.0102,24.48,.0005 ,25.56,.0016,26.64,.0018,27.70,.0113,28.80,.0111,29.91,.0158,31.06,.0093,32.17,.0017,33.32,.0002 ,34.42,.0018,35.59,.0027,36.74,.0055,37.90,.0037,39.06,.0064,40.25,.0033,41.47,.0014,42.53,.0004 ,43.89,.0010,45.12,.0039,46.33,.0039,47.64,.0009,48.88,.0016,50.13,.0006,51.37,.0010,52.70,.0002 ,54.00,.0004,55.30,.0008,56.60,.0025,57.96,.0010,59.30,.0012,60.67,.0011,61.99,.0003,62.86,.0001 ,64.36,.0005,64.86,.0001,66.26,.0004,67.70,.0006,68.94,.0002,70.10,.0001,70.58,.0002,72.01,.0007 ,73.53,.0006,75.00,.0002,77.03,.0005,78.00,.0002,79.57,.0006,81.16,.0005,82.70,.0005,84.22,.0003 ,85.41,.0002,87.46,.0001,90.30,.0001,94.02,.0001,95.26,.0002,109.39,.0003],             # <<<<<<<<<<<<<<
 * 			        [1.98,.0194,2.99,.0210,3.97,.0276,4.96,.0297,5.96,.0158,6.99,.0207,8.01,.0009 ,9.00,.0101,10.00,.0297,11.01,.0289,12.02,.0211,13.04,.0127,14.07,.0061,15.08,.0174,16.13,.0009 ,17.12,.0093,18.16,.0117,19.21,.0122,20.29,.0108,21.30,.0077,22.38,.0132,23.46,.0073,24.14,.0002 ,25.58,.0026,26.69,.0035,27.77,.0053,28.88,.0024,30.08,.0027,31.13,.0075,32.24,.0027,33.36,.0004 ,34.42,.0004,35.64,.0019,36.78,.0037,38.10,.0009,39.11,.0027,40.32,.0010,41.51,.0013,42.66,.0019 ,43.87,.0007,45.13,.0017,46.35,.0019,47.65,.0021,48.89,.0014,50.18,.0023,51.42,.0015,52.73,.0002 ,54.00,.0005,55.34,.0006,56.60,.0010,57.96,.0016,58.86,.0005,59.30,.0004,60.75,.0005,62.22,.0003 ,63.55,.0005,64.82,.0003,66.24,.0003,67.63,.0011,69.09,.0007,70.52,.0004,72.00,.0005,73.50,.0008 ,74.95,.0003,77.13,.0013,78.02,.0002,79.48,.0004,82.59,.0004,84.10,.0003],
 * 			        [2.00,.0313,2.99,.0109,4.00,.0215,5.00,.0242,5.98,.0355,7.01,.0132,8.01,.0009 ,9.01,.0071,10.00,.0258,11.03,.0221,12.02,.0056,13.06,.0196,14.05,.0160,15.11,.0107,16.11,.0003 ,17.14,.0111,18.21,.0085,19.23,.0010,20.28,.0048,21.31,.0128,22.36,.0051,23.41,.0041,24.05,.0006 ,25.54,.0019,26.62,.0028,27.72,.0034,28.82,.0062,29.89,.0039,30.98,.0058,32.08,.0011,33.21,.0002 ,34.37,.0008,35.46,.0018,36.62,.0036,37.77,.0018,38.92,.0042,40.07,.0037,41.23,.0011,42.67,.0003 ,43.65,.0018,44.68,.0025,45.99,.0044,47.21,.0051,48.40,.0044,49.67,.0005,50.88,.0019,52.15,.0003 ,53.42,.0008,54.69,.0010,55.98,.0005,57.26,.0013,58.53,.0027,59.83,.0011,61.21,.0027,62.54,.0003 ,63.78,.0003,65.20,.0001,66.60,.0006,67.98,.0008,69.37,.0019,70.73,.0007,72.14,.0004,73.62,.0002 ,74.40,.0003,76.52,.0006,77.97,.0002,79.49,.0004,80.77,.0003,81.00,.0001,82.47,.0005,83.97,.0001 ,87.27,.0002],
 */
  __pyx_t_82 = PyList_New(80); if (unlikely(!__pyx_t_82)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_82);
  __Pyx_GIVEREF(__pyx_t_2);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 0, __pyx_t_2)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_3);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 1, __pyx_t_3)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_4);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 2, __pyx_t_4)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_5);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 3, __pyx_t_5)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_6);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 4, __pyx_t_6)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_7);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 5, __pyx_t_7)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_8);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 6, __pyx_t_8)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_9);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 7, __pyx_t_9)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_10);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 8, __pyx_t_10)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_11);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 9, __pyx_t_11)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_12);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 10, __pyx_t_12)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_13);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 11, __pyx_t_13)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_14);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 12, __pyx_t_14)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_15);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 13, __pyx_t_15)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_16);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 14, __pyx_t_16)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_17);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 15, __pyx_t_17)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_18);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 16, __pyx_t_18)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_19);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 17, __pyx_t_19)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_20);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 18, __pyx_t_20)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_21);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 19, __pyx_t_21)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_22);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 20, __pyx_t_22)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_23);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 21, __pyx_t_23)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_24);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 22, __pyx_t_24)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_25);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 23, __pyx_t_25)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_26);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 24, __pyx_t_26)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_27);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 25, __pyx_t_27)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_28);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 26, __pyx_t_28)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_29);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 27, __pyx_t_29)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_30);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 28, __pyx_t_30)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_31);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 29, __pyx_t_31)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_32);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 30, __pyx_t_32)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_33);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 31, __pyx_t_33)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_34);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 32, __pyx_t_34)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_35);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 33, __pyx_t_35)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_36);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 34, __pyx_t_36)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_37);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 35, __pyx_t_37)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_38);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 36, __pyx_t_38)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_39);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 37, __pyx_t_39)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_40);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 38, __pyx_t_40)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_41);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 39, __pyx_t_41)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_42);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 40, __pyx_t_42)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_43);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 41, __pyx_t_43)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_44);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 42, __pyx_t_44)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_45);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 43, __pyx_t_45)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_46);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 44, __pyx_t_46)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_47);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 45, __pyx_t_47)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_48);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 46, __pyx_t_48)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_49);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 47, __pyx_t_49)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_50);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 48, __pyx_t_50)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_51);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 49, __pyx_t_51)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_52);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 50, __pyx_t_52)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_53);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 51, __pyx_t_53)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_54);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 52, __pyx_t_54)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_55);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 53, __pyx_t_55)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_56);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 54, __pyx_t_56)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_57);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 55, __pyx_t_57)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_58);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 56, __pyx_t_58)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_59);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 57, __pyx_t_59)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_60);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 58, __pyx_t_60)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_61);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 59, __pyx_t_61)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_62);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 60, __pyx_t_62)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_63);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 61, __pyx_t_63)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_64);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 62, __pyx_t_64)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_65);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 63, __pyx_t_65)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_66);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 64, __pyx_t_66)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_67);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 65, __pyx_t_67)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_68);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 66, __pyx_t_68)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_69);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 67, __pyx_t_69)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_70);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 68, __pyx_t_70)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_71);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 69, __pyx_t_71)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_72);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 70, __pyx_t_72)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_73);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 71, __pyx_t_73)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_74);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 72, __pyx_t_74)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_75);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 73, __pyx_t_75)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_76);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 74, __pyx_t_76)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_77);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 75, __pyx_t_77)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_78);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 76, __pyx_t_78)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_79);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 77, __pyx_t_79)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_80);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 78, __pyx_t_80)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_81);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_82, 79, __pyx_t_81)) __PYX_ERR(0, 28, __pyx_L1_error);
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_8 = 0;
  __pyx_t_9 = 0;
  __pyx_t_10 = 0;
  __pyx_t_11 = 0;
  __pyx_t_12 = 0;
  __pyx_t_13 = 0;
  __pyx_t_14 = 0;
  __pyx_t_15 = 0;
  __pyx_t_16 = 0;
  __pyx_t_17 = 0;
  __pyx_t_18 = 0;
  __pyx_t_19 = 0;
  __pyx_t_20 = 0;
  __pyx_t_21 = 0;
  __pyx_t_22 = 0;
  __pyx_t_23 = 0;
  __pyx_t_24 = 0;
  __pyx_t_25 = 0;
  __pyx_t_26 = 0;
  __pyx_t_27 = 0;
  __pyx_t_28 = 0;
  __pyx_t_29 = 0;
  __pyx_t_30 = 0;
  __pyx_t_31 = 0;
  __pyx_t_32 = 0;
  __pyx_t_33 = 0;
  __pyx_t_34 = 0;
  __pyx_t_35 = 0;
  __pyx_t_36 = 0;
  __pyx_t_37 = 0;
  __pyx_t_38 = 0;
  __pyx_t_39 = 0;
  __pyx_t_40 = 0;
  __pyx_t_41 = 0;
  __pyx_t_42 = 0;
  __pyx_t_43 = 0;
  __pyx_t_44 = 0;
  __pyx_t_45 = 0;
  __pyx_t_46 = 0;
  __pyx_t_47 = 0;
  __pyx_t_48 = 0;
  __pyx_t_49 = 0;
  __pyx_t_50 = 0;
  __pyx_t_51 = 0;
  __pyx_t_52 = 0;
  __pyx_t_53 = 0;
  __pyx_t_54 = 0;
  __pyx_t_55 = 0;
  __pyx_t_56 = 0;
  __pyx_t_57 = 0;
  __pyx_t_58 = 0;
  __pyx_t_59 = 0;
  __pyx_t_60 = 0;
  __pyx_t_61 = 0;
  __pyx_t_62 = 0;
  __pyx_t_63 = 0;
  __pyx_t_64 = 0;
  __pyx_t_65 = 0;
  __pyx_t_66 = 0;
  __pyx_t_67 = 0;
  __pyx_t_68 = 0;
  __pyx_t_69 = 0;
  __pyx_t_70 = 0;
  __pyx_t_71 = 0;
  __pyx_t_72 = 0;
  __pyx_t_73 = 0;
  __pyx_t_74 = 0;
  __pyx_t_75 = 0;
  __pyx_t_76 = 0;
  __pyx_t_77 = 0;
  __pyx_t_78 = 0;
  __pyx_t_79 = 0;
  __pyx_t_80 = 0;
  __pyx_t_81 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PIANO_SPECTRA, __pyx_t_82) < 0) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_82); __pyx_t_82 = 0;

  /* "pysndlib/instruments/lbj_piano.py":111
 * 
 * 
 * def get_piano_partials(freq):             # <<<<<<<<<<<<<<
 *     return PIANO_SPECTRA[round(12 * math.log(freq / 32.703, 2))]
 * 
 */
  __pyx_t_82 = __Pyx_CyFunction_New(&__pyx_mdef_8pysndlib_11instruments_9lbj_piano_1get_piano_partials, 0, __pyx_n_s_get_piano_partials, NULL, __pyx_n_s_pysndlib_instruments_lbj_piano, __pyx_d, ((PyObject *)__pyx_codeobj__8)); if (unlikely(!__pyx_t_82)) __PYX_ERR(0, 111, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_82);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_get_piano_partials, __pyx_t_82) < 0) __PYX_ERR(0, 111, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_82); __pyx_t_82 = 0;

  /* "pysndlib/instruments/lbj_piano.py":115
 * 
 * 
 * def make_piano_ampfun(dur, piano_attack_duration):             # <<<<<<<<<<<<<<
 *     db_drop_per_second = -10.0
 *     releaseAmp = clm.db2linear(db_drop_per_second * dur)
 */
  __pyx_t_82 = __Pyx_CyFunction_New(&__pyx_mdef_8pysndlib_11instruments_9lbj_piano_3make_piano_ampfun, 0, __pyx_n_s_make_piano_ampfun, NULL, __pyx_n_s_pysndlib_instruments_lbj_piano, __pyx_d, ((PyObject *)__pyx_codeobj__10)); if (unlikely(!__pyx_t_82)) __PYX_ERR(0, 115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_82);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_make_piano_ampfun, __pyx_t_82) < 0) __PYX_ERR(0, 115, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_82); __pyx_t_82 = 0;

  /* "pysndlib/instruments/lbj_piano.py":121
 *     return [0,0,(attackTime/4), 1.0, attackTime, 1.0, 100, releaseAmp]
 * 
 * @cython.ccall             # <<<<<<<<<<<<<<
 * def lbj_piano(startime, duration, frequency, amplitude, pfreq=None, degree=45, reverb_amount=0, distance=1):
 *     piano_attack_duration = .04
 */
  __pyx_t_82 = __Pyx_CyFunction_New(&__pyx_mdef_8pysndlib_11instruments_9lbj_piano_5lbj_piano, 0, __pyx_n_s_lbj_piano, NULL, __pyx_n_s_pysndlib_instruments_lbj_piano, __pyx_d, ((PyObject *)__pyx_codeobj__12)); if (unlikely(!__pyx_t_82)) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_82);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_82, __pyx_tuple__13);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_lbj_piano, __pyx_t_82) < 0) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_82); __pyx_t_82 = 0;

  /* "pysndlib/instruments/lbj_piano.py":158
 *         clm.locsig(locs, i, clm.env(ampenv2) * clm.oscil_bank(obank))
 * 
 * if __name__ == '__main__':             # <<<<<<<<<<<<<<
 *     with clm.Sound( play = True, statistics=True):
 *         lbj_piano(0,4,220,.34)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_82, __pyx_n_s_name); if (unlikely(!__pyx_t_82)) __PYX_ERR(0, 158, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_82);
  __pyx_t_83 = (__Pyx_PyUnicode_Equals(__pyx_t_82, __pyx_n_u_main, Py_EQ)); if (unlikely((__pyx_t_83 < 0))) __PYX_ERR(0, 158, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_82); __pyx_t_82 = 0;
  if (__pyx_t_83) {

    /* "pysndlib/instruments/lbj_piano.py":159
 * 
 * if __name__ == '__main__':
 *     with clm.Sound( play = True, statistics=True):             # <<<<<<<<<<<<<<
 *         lbj_piano(0,4,220,.34)
 *         lbj_piano(1,4,440,.34)
 */
    /*with:*/ {
      __Pyx_GetModuleGlobalName(__pyx_t_82, __pyx_n_s_clm); if (unlikely(!__pyx_t_82)) __PYX_ERR(0, 159, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_82);
      __pyx_t_81 = __Pyx_PyObject_GetAttrStr(__pyx_t_82, __pyx_n_s_Sound); if (unlikely(!__pyx_t_81)) __PYX_ERR(0, 159, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_81);
      __Pyx_DECREF(__pyx_t_82); __pyx_t_82 = 0;
      __pyx_t_82 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_82)) __PYX_ERR(0, 159, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_82);
      if (PyDict_SetItem(__pyx_t_82, __pyx_n_s_play, Py_True) < 0) __PYX_ERR(0, 159, __pyx_L1_error)
      if (PyDict_SetItem(__pyx_t_82, __pyx_n_s_statistics, Py_True) < 0) __PYX_ERR(0, 159, __pyx_L1_error)
      __pyx_t_80 = __Pyx_PyObject_Call(__pyx_t_81, __pyx_empty_tuple, __pyx_t_82); if (unlikely(!__pyx_t_80)) __PYX_ERR(0, 159, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_80);
      __Pyx_DECREF(__pyx_t_81); __pyx_t_81 = 0;
      __Pyx_DECREF(__pyx_t_82); __pyx_t_82 = 0;
      __pyx_t_1 = __Pyx_PyObject_LookupSpecial(__pyx_t_80, __pyx_n_s_exit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 159, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_82 = __Pyx_PyObject_LookupSpecial(__pyx_t_80, __pyx_n_s_enter); if (unlikely(!__pyx_t_82)) __PYX_ERR(0, 159, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_82);
      __pyx_t_81 = __Pyx_PyObject_CallNoArg(__pyx_t_82); if (unlikely(!__pyx_t_81)) __PYX_ERR(0, 159, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_81);
      __Pyx_DECREF(__pyx_t_82); __pyx_t_82 = 0;
      __Pyx_DECREF(__pyx_t_81); __pyx_t_81 = 0;
      __Pyx_DECREF(__pyx_t_80); __pyx_t_80 = 0;
      /*try:*/ {
        {
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __Pyx_ExceptionSave(&__pyx_t_84, &__pyx_t_85, &__pyx_t_86);
          __Pyx_XGOTREF(__pyx_t_84);
          __Pyx_XGOTREF(__pyx_t_85);
          __Pyx_XGOTREF(__pyx_t_86);
          /*try:*/ {

            /* "pysndlib/instruments/lbj_piano.py":160
 * if __name__ == '__main__':
 *     with clm.Sound( play = True, statistics=True):
 *         lbj_piano(0,4,220,.34)             # <<<<<<<<<<<<<<
 *         lbj_piano(1,4,440,.34)
 *         lbj_piano(2,4,261,.4)
 */
            __pyx_t_80 = __pyx_f_8pysndlib_11instruments_9lbj_piano_lbj_piano(__pyx_int_0, __pyx_int_4, __pyx_int_220, __pyx_float__34, 0, NULL); if (unlikely(!__pyx_t_80)) __PYX_ERR(0, 160, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_80);
            __Pyx_DECREF(__pyx_t_80); __pyx_t_80 = 0;

            /* "pysndlib/instruments/lbj_piano.py":161
 *     with clm.Sound( play = True, statistics=True):
 *         lbj_piano(0,4,220,.34)
 *         lbj_piano(1,4,440,.34)             # <<<<<<<<<<<<<<
 *         lbj_piano(2,4,261,.4)
 *         lbj_piano(2,4,440,.4)
 */
            __pyx_t_80 = __pyx_f_8pysndlib_11instruments_9lbj_piano_lbj_piano(__pyx_int_1, __pyx_int_4, __pyx_int_440, __pyx_float__34, 0, NULL); if (unlikely(!__pyx_t_80)) __PYX_ERR(0, 161, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_80);
            __Pyx_DECREF(__pyx_t_80); __pyx_t_80 = 0;

            /* "pysndlib/instruments/lbj_piano.py":162
 *         lbj_piano(0,4,220,.34)
 *         lbj_piano(1,4,440,.34)
 *         lbj_piano(2,4,261,.4)             # <<<<<<<<<<<<<<
 *         lbj_piano(2,4,440,.4)
 */
            __pyx_t_80 = __pyx_f_8pysndlib_11instruments_9lbj_piano_lbj_piano(__pyx_int_2, __pyx_int_4, __pyx_int_261, __pyx_float__4, 0, NULL); if (unlikely(!__pyx_t_80)) __PYX_ERR(0, 162, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_80);
            __Pyx_DECREF(__pyx_t_80); __pyx_t_80 = 0;

            /* "pysndlib/instruments/lbj_piano.py":163
 *         lbj_piano(1,4,440,.34)
 *         lbj_piano(2,4,261,.4)
 *         lbj_piano(2,4,440,.4)             # <<<<<<<<<<<<<<
 */
            __pyx_t_80 = __pyx_f_8pysndlib_11instruments_9lbj_piano_lbj_piano(__pyx_int_2, __pyx_int_4, __pyx_int_440, __pyx_float__4, 0, NULL); if (unlikely(!__pyx_t_80)) __PYX_ERR(0, 163, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_80);
            __Pyx_DECREF(__pyx_t_80); __pyx_t_80 = 0;

            /* "pysndlib/instruments/lbj_piano.py":159
 * 
 * if __name__ == '__main__':
 *     with clm.Sound( play = True, statistics=True):             # <<<<<<<<<<<<<<
 *         lbj_piano(0,4,220,.34)
 *         lbj_piano(1,4,440,.34)
 */
          }
          __Pyx_XDECREF(__pyx_t_84); __pyx_t_84 = 0;
          __Pyx_XDECREF(__pyx_t_85); __pyx_t_85 = 0;
          __Pyx_XDECREF(__pyx_t_86); __pyx_t_86 = 0;
          goto __pyx_L12_try_end;
          __pyx_L7_error:;
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
          __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
          __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
          __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
          __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF(__pyx_t_20); __pyx_t_20 = 0;
          __Pyx_XDECREF(__pyx_t_21); __pyx_t_21 = 0;
          __Pyx_XDECREF(__pyx_t_22); __pyx_t_22 = 0;
          __Pyx_XDECREF(__pyx_t_23); __pyx_t_23 = 0;
          __Pyx_XDECREF(__pyx_t_24); __pyx_t_24 = 0;
          __Pyx_XDECREF(__pyx_t_25); __pyx_t_25 = 0;
          __Pyx_XDECREF(__pyx_t_26); __pyx_t_26 = 0;
          __Pyx_XDECREF(__pyx_t_27); __pyx_t_27 = 0;
          __Pyx_XDECREF(__pyx_t_28); __pyx_t_28 = 0;
          __Pyx_XDECREF(__pyx_t_29); __pyx_t_29 = 0;
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_XDECREF(__pyx_t_30); __pyx_t_30 = 0;
          __Pyx_XDECREF(__pyx_t_31); __pyx_t_31 = 0;
          __Pyx_XDECREF(__pyx_t_32); __pyx_t_32 = 0;
          __Pyx_XDECREF(__pyx_t_33); __pyx_t_33 = 0;
          __Pyx_XDECREF(__pyx_t_34); __pyx_t_34 = 0;
          __Pyx_XDECREF(__pyx_t_35); __pyx_t_35 = 0;
          __Pyx_XDECREF(__pyx_t_36); __pyx_t_36 = 0;
          __Pyx_XDECREF(__pyx_t_37); __pyx_t_37 = 0;
          __Pyx_XDECREF(__pyx_t_38); __pyx_t_38 = 0;
          __Pyx_XDECREF(__pyx_t_39); __pyx_t_39 = 0;
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_XDECREF(__pyx_t_40); __pyx_t_40 = 0;
          __Pyx_XDECREF(__pyx_t_41); __pyx_t_41 = 0;
          __Pyx_XDECREF(__pyx_t_42); __pyx_t_42 = 0;
          __Pyx_XDECREF(__pyx_t_43); __pyx_t_43 = 0;
          __Pyx_XDECREF(__pyx_t_44); __pyx_t_44 = 0;
          __Pyx_XDECREF(__pyx_t_45); __pyx_t_45 = 0;
          __Pyx_XDECREF(__pyx_t_46); __pyx_t_46 = 0;
          __Pyx_XDECREF(__pyx_t_47); __pyx_t_47 = 0;
          __Pyx_XDECREF(__pyx_t_48); __pyx_t_48 = 0;
          __Pyx_XDECREF(__pyx_t_49); __pyx_t_49 = 0;
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_XDECREF(__pyx_t_50); __pyx_t_50 = 0;
          __Pyx_XDECREF(__pyx_t_51); __pyx_t_51 = 0;
          __Pyx_XDECREF(__pyx_t_52); __pyx_t_52 = 0;
          __Pyx_XDECREF(__pyx_t_53); __pyx_t_53 = 0;
          __Pyx_XDECREF(__pyx_t_54); __pyx_t_54 = 0;
          __Pyx_XDECREF(__pyx_t_55); __pyx_t_55 = 0;
          __Pyx_XDECREF(__pyx_t_56); __pyx_t_56 = 0;
          __Pyx_XDECREF(__pyx_t_57); __pyx_t_57 = 0;
          __Pyx_XDECREF(__pyx_t_58); __pyx_t_58 = 0;
          __Pyx_XDECREF(__pyx_t_59); __pyx_t_59 = 0;
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_XDECREF(__pyx_t_60); __pyx_t_60 = 0;
          __Pyx_XDECREF(__pyx_t_61); __pyx_t_61 = 0;
          __Pyx_XDECREF(__pyx_t_62); __pyx_t_62 = 0;
          __Pyx_XDECREF(__pyx_t_63); __pyx_t_63 = 0;
          __Pyx_XDECREF(__pyx_t_64); __pyx_t_64 = 0;
          __Pyx_XDECREF(__pyx_t_65); __pyx_t_65 = 0;
          __Pyx_XDECREF(__pyx_t_66); __pyx_t_66 = 0;
          __Pyx_XDECREF(__pyx_t_67); __pyx_t_67 = 0;
          __Pyx_XDECREF(__pyx_t_68); __pyx_t_68 = 0;
          __Pyx_XDECREF(__pyx_t_69); __pyx_t_69 = 0;
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_XDECREF(__pyx_t_70); __pyx_t_70 = 0;
          __Pyx_XDECREF(__pyx_t_71); __pyx_t_71 = 0;
          __Pyx_XDECREF(__pyx_t_72); __pyx_t_72 = 0;
          __Pyx_XDECREF(__pyx_t_73); __pyx_t_73 = 0;
          __Pyx_XDECREF(__pyx_t_74); __pyx_t_74 = 0;
          __Pyx_XDECREF(__pyx_t_75); __pyx_t_75 = 0;
          __Pyx_XDECREF(__pyx_t_76); __pyx_t_76 = 0;
          __Pyx_XDECREF(__pyx_t_77); __pyx_t_77 = 0;
          __Pyx_XDECREF(__pyx_t_78); __pyx_t_78 = 0;
          __Pyx_XDECREF(__pyx_t_79); __pyx_t_79 = 0;
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_XDECREF(__pyx_t_80); __pyx_t_80 = 0;
          __Pyx_XDECREF(__pyx_t_81); __pyx_t_81 = 0;
          __Pyx_XDECREF(__pyx_t_82); __pyx_t_82 = 0;
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
          /*except:*/ {
            __Pyx_AddTraceback("pysndlib.instruments.lbj_piano", __pyx_clineno, __pyx_lineno, __pyx_filename);
            if (__Pyx_GetException(&__pyx_t_80, &__pyx_t_81, &__pyx_t_82) < 0) __PYX_ERR(0, 159, __pyx_L9_except_error)
            __Pyx_XGOTREF(__pyx_t_80);
            __Pyx_XGOTREF(__pyx_t_81);
            __Pyx_XGOTREF(__pyx_t_82);
            __pyx_t_79 = PyTuple_Pack(3, __pyx_t_80, __pyx_t_81, __pyx_t_82); if (unlikely(!__pyx_t_79)) __PYX_ERR(0, 159, __pyx_L9_except_error)
            __Pyx_GOTREF(__pyx_t_79);
            __pyx_t_87 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_79, NULL);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __Pyx_DECREF(__pyx_t_79); __pyx_t_79 = 0;
            if (unlikely(!__pyx_t_87)) __PYX_ERR(0, 159, __pyx_L9_except_error)
            __Pyx_GOTREF(__pyx_t_87);
            __pyx_t_83 = __Pyx_PyObject_IsTrue(__pyx_t_87);
            __Pyx_DECREF(__pyx_t_87); __pyx_t_87 = 0;
            if (__pyx_t_83 < 0) __PYX_ERR(0, 159, __pyx_L9_except_error)
            __pyx_t_88 = (!__pyx_t_83);
            if (unlikely(__pyx_t_88)) {
              __Pyx_GIVEREF(__pyx_t_80);
              __Pyx_GIVEREF(__pyx_t_81);
              __Pyx_XGIVEREF(__pyx_t_82);
              __Pyx_ErrRestoreWithState(__pyx_t_80, __pyx_t_81, __pyx_t_82);
              __pyx_t_80 = 0; __pyx_t_81 = 0; __pyx_t_82 = 0; 
              __PYX_ERR(0, 159, __pyx_L9_except_error)
            }
            __Pyx_XDECREF(__pyx_t_80); __pyx_t_80 = 0;
            __Pyx_XDECREF(__pyx_t_81); __pyx_t_81 = 0;
            __Pyx_XDECREF(__pyx_t_82); __pyx_t_82 = 0;
            goto __pyx_L8_exception_handled;
          }
          __pyx_L9_except_error:;
          __Pyx_XGIVEREF(__pyx_t_84);
          __Pyx_XGIVEREF(__pyx_t_85);
          __Pyx_XGIVEREF(__pyx_t_86);
          __Pyx_ExceptionReset(__pyx_t_84, __pyx_t_85, __pyx_t_86);
          goto __pyx_L1_error;
          __pyx_L8_exception_handled:;
          __Pyx_XGIVEREF(__pyx_t_84);
          __Pyx_XGIVEREF(__pyx_t_85);
          __Pyx_XGIVEREF(__pyx_t_86);
          __Pyx_ExceptionReset(__pyx_t_84, __pyx_t_85, __pyx_t_86);
          __pyx_L12_try_end:;
        }
      }
      /*finally:*/ {
        /*normal exit:*/{
          if (__pyx_t_1) {
            __pyx_t_86 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__14, NULL);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            if (unlikely(!__pyx_t_86)) __PYX_ERR(0, 159, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_86);
            __Pyx_DECREF(__pyx_t_86); __pyx_t_86 = 0;
          }
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
      goto __pyx_L16;
      __pyx_L3_error:;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L1_error;
      __pyx_L16:;
    }

    /* "pysndlib/instruments/lbj_piano.py":158
 *         clm.locsig(locs, i, clm.env(ampenv2) * clm.oscil_bank(obank))
 * 
 * if __name__ == '__main__':             # <<<<<<<<<<<<<<
 *     with clm.Sound( play = True, statistics=True):
 *         lbj_piano(0,4,220,.34)
 */
  }

  /* "pysndlib/instruments/lbj_piano.py":1
 * #==================================================================================             # <<<<<<<<<<<<<<
 * # The code is ported from Bill Schottstedaet's 'clm-ins.scm'
 * # file  available at https://ccrma.stanford.edu/software/snd/sndlib/
 */
  __pyx_t_82 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_82)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_82);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_82) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_82); __pyx_t_82 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_XDECREF(__pyx_t_19);
  __Pyx_XDECREF(__pyx_t_20);
  __Pyx_XDECREF(__pyx_t_21);
  __Pyx_XDECREF(__pyx_t_22);
  __Pyx_XDECREF(__pyx_t_23);
  __Pyx_XDECREF(__pyx_t_24);
  __Pyx_XDECREF(__pyx_t_25);
  __Pyx_XDECREF(__pyx_t_26);
  __Pyx_XDECREF(__pyx_t_27);
  __Pyx_XDECREF(__pyx_t_28);
  __Pyx_XDECREF(__pyx_t_29);
  __Pyx_XDECREF(__pyx_t_30);
  __Pyx_XDECREF(__pyx_t_31);
  __Pyx_XDECREF(__pyx_t_32);
  __Pyx_XDECREF(__pyx_t_33);
  __Pyx_XDECREF(__pyx_t_34);
  __Pyx_XDECREF(__pyx_t_35);
  __Pyx_XDECREF(__pyx_t_36);
  __Pyx_XDECREF(__pyx_t_37);
  __Pyx_XDECREF(__pyx_t_38);
  __Pyx_XDECREF(__pyx_t_39);
  __Pyx_XDECREF(__pyx_t_40);
  __Pyx_XDECREF(__pyx_t_41);
  __Pyx_XDECREF(__pyx_t_42);
  __Pyx_XDECREF(__pyx_t_43);
  __Pyx_XDECREF(__pyx_t_44);
  __Pyx_XDECREF(__pyx_t_45);
  __Pyx_XDECREF(__pyx_t_46);
  __Pyx_XDECREF(__pyx_t_47);
  __Pyx_XDECREF(__pyx_t_48);
  __Pyx_XDECREF(__pyx_t_49);
  __Pyx_XDECREF(__pyx_t_50);
  __Pyx_XDECREF(__pyx_t_51);
  __Pyx_XDECREF(__pyx_t_52);
  __Pyx_XDECREF(__pyx_t_53);
  __Pyx_XDECREF(__pyx_t_54);
  __Pyx_XDECREF(__pyx_t_55);
  __Pyx_XDECREF(__pyx_t_56);
  __Pyx_XDECREF(__pyx_t_57);
  __Pyx_XDECREF(__pyx_t_58);
  __Pyx_XDECREF(__pyx_t_59);
  __Pyx_XDECREF(__pyx_t_60);
  __Pyx_XDECREF(__pyx_t_61);
  __Pyx_XDECREF(__pyx_t_62);
  __Pyx_XDECREF(__pyx_t_63);
  __Pyx_XDECREF(__pyx_t_64);
  __Pyx_XDECREF(__pyx_t_65);
  __Pyx_XDECREF(__pyx_t_66);
  __Pyx_XDECREF(__pyx_t_67);
  __Pyx_XDECREF(__pyx_t_68);
  __Pyx_XDECREF(__pyx_t_69);
  __Pyx_XDECREF(__pyx_t_70);
  __Pyx_XDECREF(__pyx_t_71);
  __Pyx_XDECREF(__pyx_t_72);
  __Pyx_XDECREF(__pyx_t_73);
  __Pyx_XDECREF(__pyx_t_74);
  __Pyx_XDECREF(__pyx_t_75);
  __Pyx_XDECREF(__pyx_t_76);
  __Pyx_XDECREF(__pyx_t_77);
  __Pyx_XDECREF(__pyx_t_78);
  __Pyx_XDECREF(__pyx_t_79);
  __Pyx_XDECREF(__pyx_t_80);
  __Pyx_XDECREF(__pyx_t_81);
  __Pyx_XDECREF(__pyx_t_82);
  if (__pyx_m) {
    if (__pyx_d && stringtab_initialized) {
      __Pyx_AddTraceback("init pysndlib.instruments.lbj_piano", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    #if !CYTHON_USE_MODULE_STATE
    Py_CLEAR(__pyx_m);
    #else
    Py_DECREF(__pyx_m);
    if (pystate_addmodule_run) {
      PyObject *tp, *value, *tb;
      PyErr_Fetch(&tp, &value, &tb);
      PyState_RemoveModule(&__pyx_moduledef);
      PyErr_Restore(tp, value, tb);
    }
    #endif
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init pysndlib.instruments.lbj_piano");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}
/* #### Code section: cleanup_globals ### */
/* #### Code section: cleanup_module ### */
/* #### Code section: main_method ### */
/* #### Code section: utility_code_pragmas ### */
#ifdef _MSC_VER
#pragma warning( push )
/* Warning 4127: conditional expression is constant
 * Cython uses constant conditional expressions to allow in inline functions to be optimized at
 * compile-time, so this warning is not useful
 */
#pragma warning( disable : 4127 )
#endif



/* #### Code section: utility_code_def ### */

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    int result;
    PyObject *exc_type;
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *current_exception = tstate->current_exception;
    if (unlikely(!current_exception)) return 0;
    exc_type = (PyObject*) Py_TYPE(current_exception);
    if (exc_type == err) return 1;
#else
    exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
#endif
    #if CYTHON_AVOID_BORROWED_REFS
    Py_INCREF(exc_type);
    #endif
    if (unlikely(PyTuple_Check(err))) {
        result = __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    } else {
        result = __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
    }
    #if CYTHON_AVOID_BORROWED_REFS
    Py_DECREF(exc_type);
    #endif
    return result;
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *tmp_value;
    assert(type == NULL || (value != NULL && type == (PyObject*) Py_TYPE(value)));
    if (value) {
        #if CYTHON_COMPILING_IN_CPYTHON
        if (unlikely(((PyBaseExceptionObject*) value)->traceback != tb))
        #endif
            PyException_SetTraceback(value, tb);
    }
    tmp_value = tstate->current_exception;
    tstate->current_exception = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
#else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject* exc_value;
    exc_value = tstate->current_exception;
    tstate->current_exception = 0;
    *value = exc_value;
    *type = NULL;
    *tb = NULL;
    if (exc_value) {
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        #if CYTHON_COMPILING_IN_CPYTHON
        *tb = ((PyBaseExceptionObject*) exc_value)->traceback;
        Py_XINCREF(*tb);
        #else
        *tb = PyException_GetTraceback(exc_value);
        #endif
    }
#else
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#endif
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* PyObjectGetAttrStrNoError */
#if __PYX_LIMITED_VERSION_HEX < 0x030d00A1
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if __PYX_LIMITED_VERSION_HEX >= 0x030d00A1
    (void) PyObject_GetOptionalAttr(obj, attr_name, &result);
    return result;
#else
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS && PY_VERSION_HEX >= 0x030700B1
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
#endif
}

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStrNoError(__pyx_b, name);
    if (unlikely(!result) && !PyErr_Occurred()) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK && CYTHON_FAST_THREAD_STATE
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_value == NULL || exc_info->exc_value == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    PyObject *exc_value = exc_info->exc_value;
    if (exc_value == NULL || exc_value == Py_None) {
        *value = NULL;
        *type = NULL;
        *tb = NULL;
    } else {
        *value = exc_value;
        Py_INCREF(*value);
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        *tb = PyException_GetTraceback(exc_value);
    }
  #elif CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
  #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
  #endif
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = tstate->exc_info;
    PyObject *tmp_value = exc_info->exc_value;
    exc_info->exc_value = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
  #else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
  #endif
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type = NULL, *local_value, *local_tb = NULL;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
  #if PY_VERSION_HEX >= 0x030C00A6
    local_value = tstate->current_exception;
    tstate->current_exception = 0;
    if (likely(local_value)) {
        local_type = (PyObject*) Py_TYPE(local_value);
        Py_INCREF(local_type);
        local_tb = PyException_GetTraceback(local_value);
    }
  #else
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
  #endif
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE && PY_VERSION_HEX >= 0x030C00A6
    if (unlikely(tstate->current_exception))
#elif CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
      #if PY_VERSION_HEX >= 0x030B00a4
        tmp_value = exc_info->exc_value;
        exc_info->exc_value = local_value;
        tmp_type = NULL;
        tmp_tb = NULL;
        Py_XDECREF(local_type);
        Py_XDECREF(local_tb);
      #else
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
      #endif
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    #if PY_MAJOR_VERSION < 3
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    #else
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    #endif
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    __Pyx_PyThreadState_declare
    CYTHON_UNUSED_VAR(cause);
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
      #if PY_VERSION_HEX >= 0x030C00A6
        PyException_SetTraceback(value, tb);
      #elif CYTHON_FAST_THREAD_STATE
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#else
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* TupleAndListFromArray */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE void __Pyx_copy_object_array(PyObject *const *CYTHON_RESTRICT src, PyObject** CYTHON_RESTRICT dest, Py_ssize_t length) {
    PyObject *v;
    Py_ssize_t i;
    for (i = 0; i < length; i++) {
        v = dest[i] = src[i];
        Py_INCREF(v);
    }
}
static CYTHON_INLINE PyObject *
__Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        Py_INCREF(__pyx_empty_tuple);
        return __pyx_empty_tuple;
    }
    res = PyTuple_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyTupleObject*)res)->ob_item, n);
    return res;
}
static CYTHON_INLINE PyObject *
__Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        return PyList_New(0);
    }
    res = PyList_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyListObject*)res)->ob_item, n);
    return res;
}
#endif

/* BytesEquals */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS && (PY_VERSION_HEX < 0x030B0000)
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        #if PY_MAJOR_VERSION < 3
        Py_XDECREF(owned_ref);
        #endif
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* fastcall */
#if CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s)
{
    Py_ssize_t i, n = PyTuple_GET_SIZE(kwnames);
    for (i = 0; i < n; i++)
    {
        if (s == PyTuple_GET_ITEM(kwnames, i)) return kwvalues[i];
    }
    for (i = 0; i < n; i++)
    {
        int eq = __Pyx_PyUnicode_Equals(s, PyTuple_GET_ITEM(kwnames, i), Py_EQ);
        if (unlikely(eq != 0)) {
            if (unlikely(eq < 0)) return NULL;
            return kwvalues[i];
        }
    }
    return NULL;
}
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
CYTHON_UNUSED static PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues) {
    Py_ssize_t i, nkwargs = PyTuple_GET_SIZE(kwnames);
    PyObject *dict;
    dict = PyDict_New();
    if (unlikely(!dict))
        return NULL;
    for (i=0; i<nkwargs; i++) {
        PyObject *key = PyTuple_GET_ITEM(kwnames, i);
        if (unlikely(PyDict_SetItem(dict, key, kwvalues[i]) < 0))
            goto bad;
    }
    return dict;
bad:
    Py_DECREF(dict);
    return NULL;
}
#endif
#endif

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject *const *kwvalues,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    int kwds_is_tuple = CYTHON_METH_FASTCALL && likely(PyTuple_Check(kwds));
    while (1) {
        Py_XDECREF(key); key = NULL;
        Py_XDECREF(value); value = NULL;
        if (kwds_is_tuple) {
            Py_ssize_t size;
#if CYTHON_ASSUME_SAFE_MACROS
            size = PyTuple_GET_SIZE(kwds);
#else
            size = PyTuple_Size(kwds);
            if (size < 0) goto bad;
#endif
            if (pos >= size) break;
#if CYTHON_AVOID_BORROWED_REFS
            key = __Pyx_PySequence_ITEM(kwds, pos);
            if (!key) goto bad;
#elif CYTHON_ASSUME_SAFE_MACROS
            key = PyTuple_GET_ITEM(kwds, pos);
#else
            key = PyTuple_GetItem(kwds, pos);
            if (!key) goto bad;
#endif
            value = kwvalues[pos];
            pos++;
        }
        else
        {
            if (!PyDict_Next(kwds, &pos, &key, &value)) break;
#if CYTHON_AVOID_BORROWED_REFS
            Py_INCREF(key);
#endif
        }
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
#if CYTHON_AVOID_BORROWED_REFS
            Py_INCREF(value);
            Py_DECREF(key);
#endif
            key = NULL;
            value = NULL;
            continue;
        }
#if !CYTHON_AVOID_BORROWED_REFS
        Py_INCREF(key);
#endif
        Py_INCREF(value);
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
#if CYTHON_AVOID_BORROWED_REFS
                    value = NULL;
#endif
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key)
                );
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
#if CYTHON_AVOID_BORROWED_REFS
                    value = NULL;
#endif
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    Py_XDECREF(key);
    Py_XDECREF(value);
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    #if PY_MAJOR_VERSION < 3
    PyErr_Format(PyExc_TypeError,
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
    PyErr_Format(PyExc_TypeError,
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    Py_XDECREF(key);
    Py_XDECREF(value);
    return -1;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && PY_VERSION_HEX < 0x030d0000
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#elif CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(!__pyx_m)) {
        return NULL;
    }
    result = PyObject_GetAttr(__pyx_m, name);
    if (likely(result)) {
        return result;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* PyIntBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_MultiplyCObj(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check) {
    CYTHON_MAYBE_UNUSED_VAR(intval);
    CYTHON_MAYBE_UNUSED_VAR(inplace);
    CYTHON_UNUSED_VAR(zerodivision_check);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op2))) {
        const long a = intval;
        long b = PyInt_AS_LONG(op2);
        
#ifdef HAVE_LONG_LONG
            if (sizeof(PY_LONG_LONG) > sizeof(long)) {
                PY_LONG_LONG result = (PY_LONG_LONG)a * (PY_LONG_LONG)b;
                return (result >= LONG_MIN && result <= LONG_MAX) ?
                    PyInt_FromLong((long)result) : PyLong_FromLongLong(result);
            }
#endif
#if CYTHON_USE_TYPE_SLOTS
            return PyInt_Type.tp_as_number->nb_multiply(op1, op2);
#else
            return PyNumber_Multiply(op1, op2);
#endif
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op2))) {
        const long a = intval;
        long b, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG lla = intval;
        PY_LONG_LONG llb, llx;
#endif
        if (unlikely(__Pyx_PyLong_IsZero(op2))) {
            return __Pyx_NewRef(op2);
        }
        if (likely(__Pyx_PyLong_IsCompact(op2))) {
            b = __Pyx_PyLong_CompactValue(op2);
        } else {
            const digit* digits = __Pyx_PyLong_Digits(op2);
            const Py_ssize_t size = __Pyx_PyLong_SignedDigitCount(op2);
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT+30) {
                        b = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT+30) {
                        llb = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT+30) {
                        b = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT+30) {
                        llb = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT+30) {
                        b = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT+30) {
                        llb = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT+30) {
                        b = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT+30) {
                        llb = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT+30) {
                        b = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT+30) {
                        llb = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT+30) {
                        b = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT+30) {
                        llb = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_multiply(op1, op2);
            }
        }
                CYTHON_UNUSED_VAR(a);
                CYTHON_UNUSED_VAR(b);
                #ifdef HAVE_LONG_LONG
                llb = b;
                goto long_long;
                #else
                return PyLong_Type.tp_as_number->nb_multiply(op1, op2);
                #endif
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla * llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op2)) {
        const long a = intval;
#if CYTHON_COMPILING_IN_LIMITED_API
        double b = __pyx_PyFloat_AsDouble(op2);
#else
        double b = PyFloat_AS_DOUBLE(op2);
#endif
            double result;
            
            PyFPE_START_PROTECT("multiply", return NULL)
            result = ((double)a) * (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceMultiply : PyNumber_Multiply)(op1, op2);
}
#endif

/* PyFloatBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyFloat_TrueDivideObjC(PyObject *op1, PyObject *op2, double floatval, int inplace, int zerodivision_check) {
    const double b = floatval;
    double a, result;
    CYTHON_UNUSED_VAR(inplace);
    CYTHON_UNUSED_VAR(zerodivision_check);
    if (likely(PyFloat_CheckExact(op1))) {
#if CYTHON_COMPILING_IN_LIMITED_API
        a = __pyx_PyFloat_AsDouble(op1);
#else
        a = PyFloat_AS_DOUBLE(op1);
#endif
        
    } else
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        a = (double) PyInt_AS_LONG(op1);
        
    } else
    #endif
    if (likely(PyLong_CheckExact(op1))) {
        #if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsZero(op1)) {
            a = 0.0;
            
        } else if (__Pyx_PyLong_IsCompact(op1)) {
            a = (double) __Pyx_PyLong_CompactValue(op1);
        } else {
            const digit* digits = __Pyx_PyLong_Digits(op1);
            const Py_ssize_t size = __Pyx_PyLong_SignedDigitCount(op1);
            switch (size) {
                case -2:
                case 2:
                    if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (1 * PyLong_SHIFT < 53))) {
                        a = (double) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        if ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53) || (a < (double) ((PY_LONG_LONG)1 << 53))) {
                            if (size == -2)
                                a = -a;
                            break;
                        }
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                case 3:
                    if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53))) {
                        a = (double) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        if ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53) || (a < (double) ((PY_LONG_LONG)1 << 53))) {
                            if (size == -3)
                                a = -a;
                            break;
                        }
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                case 4:
                    if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53))) {
                        a = (double) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        if ((8 * sizeof(unsigned long) < 53) || (4 * PyLong_SHIFT < 53) || (a < (double) ((PY_LONG_LONG)1 << 53))) {
                            if (size == -4)
                                a = -a;
                            break;
                        }
                    }
                    CYTHON_FALLTHROUGH;
                default:
        #endif
                    a = PyLong_AsDouble(op1);
                    if (unlikely(a == -1.0 && PyErr_Occurred())) return NULL;
        #if CYTHON_USE_PYLONG_INTERNALS
            }
        }
        #endif
    } else {
        return (inplace ? PyNumber_InPlaceTrueDivide : PyNumber_TrueDivide)(op1, op2);
    }
        PyFPE_START_PROTECT("divide", return NULL)
        result = a / b;
        PyFPE_END_PROTECT(result)
        return PyFloat_FromDouble(result);
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL && !CYTHON_VECTORCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    #if PY_MAJOR_VERSION < 3
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object"))) {
        return NULL;
    }
    #else
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object"))) {
        return NULL;
    }
    #endif
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = __Pyx_CyOrPyCFunction_GET_FUNCTION(func);
    self = __Pyx_CyOrPyCFunction_GET_SELF(func);
    #if PY_MAJOR_VERSION < 3
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    #else
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    #endif
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectFastCall */
#if PY_VERSION_HEX < 0x03090000 || CYTHON_COMPILING_IN_LIMITED_API
static PyObject* __Pyx_PyObject_FastCall_fallback(PyObject *func, PyObject **args, size_t nargs, PyObject *kwargs) {
    PyObject *argstuple;
    PyObject *result = 0;
    size_t i;
    argstuple = PyTuple_New((Py_ssize_t)nargs);
    if (unlikely(!argstuple)) return NULL;
    for (i = 0; i < nargs; i++) {
        Py_INCREF(args[i]);
        if (__Pyx_PyTuple_SET_ITEM(argstuple, (Py_ssize_t)i, args[i]) < 0) goto bad;
    }
    result = __Pyx_PyObject_Call(func, argstuple, kwargs);
  bad:
    Py_DECREF(argstuple);
    return result;
}
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject **args, size_t _nargs, PyObject *kwargs) {
    Py_ssize_t nargs = __Pyx_PyVectorcall_NARGS(_nargs);
#if CYTHON_COMPILING_IN_CPYTHON
    if (nargs == 0 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_NOARGS))
            return __Pyx_PyObject_CallMethO(func, NULL);
    }
    else if (nargs == 1 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_O))
            return __Pyx_PyObject_CallMethO(func, args[0]);
    }
#endif
    #if PY_VERSION_HEX < 0x030800B1
    #if CYTHON_FAST_PYCCALL
    if (PyCFunction_Check(func)) {
        if (kwargs) {
            return _PyCFunction_FastCallDict(func, args, nargs, kwargs);
        } else {
            return _PyCFunction_FastCallKeywords(func, args, nargs, NULL);
        }
    }
    #if PY_VERSION_HEX >= 0x030700A1
    if (!kwargs && __Pyx_IS_TYPE(func, &PyMethodDescr_Type)) {
        return _PyMethodDescr_FastCallKeywords(func, args, nargs, NULL);
    }
    #endif
    #endif
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs);
    }
    #endif
    #endif
    if (kwargs == NULL) {
        #if CYTHON_VECTORCALL
        #if PY_VERSION_HEX < 0x03090000
        vectorcallfunc f = _PyVectorcall_Function(func);
        #else
        vectorcallfunc f = PyVectorcall_Function(func);
        #endif
        if (f) {
            return f(func, args, (size_t)nargs, NULL);
        }
        #elif defined(__Pyx_CyFunction_USED) && CYTHON_BACKPORT_VECTORCALL
        if (__Pyx_CyFunction_CheckExact(func)) {
            __pyx_vectorcallfunc f = __Pyx_CyFunction_func_vectorcall(func);
            if (f) return f(func, args, (size_t)nargs, NULL);
        }
        #endif
    }
    if (nargs == 0) {
        return __Pyx_PyObject_Call(func, __pyx_empty_tuple, kwargs);
    }
    #if PY_VERSION_HEX >= 0x03090000 && !CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_VectorcallDict(func, args, (size_t)nargs, kwargs);
    #else
    return __Pyx_PyObject_FastCall_fallback(func, args, (size_t)nargs, kwargs);
    #endif
}

/* PyObjectCallOneArg */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *args[2] = {NULL, arg};
    return __Pyx_PyObject_FastCall(func, args+1, 1 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (unlikely(!j)) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PyMappingMethods *mm = Py_TYPE(o)->tp_as_mapping;
        PySequenceMethods *sm = Py_TYPE(o)->tp_as_sequence;
        if (mm && mm->mp_subscript) {
            PyObject *r, *key = PyInt_FromSsize_t(i);
            if (unlikely(!key)) return NULL;
            r = mm->mp_subscript(o, key);
            Py_DECREF(key);
            return r;
        }
        if (likely(sm && sm->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(sm->sq_length)) {
                Py_ssize_t l = sm->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return sm->sq_item(o, i);
        }
    }
#else
    if (is_list || !PyMapping_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject *index) {
    PyObject *runerr = NULL;
    Py_ssize_t key_value;
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        __Pyx_TypeName index_type_name = __Pyx_PyType_GetName(Py_TYPE(index));
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError,
            "cannot fit '" __Pyx_FMT_TYPENAME "' into an index-sized integer", index_type_name);
        __Pyx_DECREF_TypeName(index_type_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem_Slow(PyObject *obj, PyObject *key) {
    __Pyx_TypeName obj_type_name;
    if (likely(PyType_Check(obj))) {
        PyObject *meth = __Pyx_PyObject_GetAttrStrNoError(obj, __pyx_n_s_class_getitem);
        if (!meth) {
            PyErr_Clear();
        } else {
            PyObject *result = __Pyx_PyObject_CallOneArg(meth, key);
            Py_DECREF(meth);
            return result;
        }
    }
    obj_type_name = __Pyx_PyType_GetName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "'" __Pyx_FMT_TYPENAME "' object is not subscriptable", obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject *key) {
    PyTypeObject *tp = Py_TYPE(obj);
    PyMappingMethods *mm = tp->tp_as_mapping;
    PySequenceMethods *sm = tp->tp_as_sequence;
    if (likely(mm && mm->mp_subscript)) {
        return mm->mp_subscript(obj, key);
    }
    if (likely(sm && sm->sq_item)) {
        return __Pyx_PyObject_GetIndex(obj, key);
    }
    return __Pyx_PyObject_GetItem_Slow(obj, key);
}
#endif

/* PyIntBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_MultiplyObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check) {
    CYTHON_MAYBE_UNUSED_VAR(intval);
    CYTHON_MAYBE_UNUSED_VAR(inplace);
    CYTHON_UNUSED_VAR(zerodivision_check);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        
#ifdef HAVE_LONG_LONG
            if (sizeof(PY_LONG_LONG) > sizeof(long)) {
                PY_LONG_LONG result = (PY_LONG_LONG)a * (PY_LONG_LONG)b;
                return (result >= LONG_MIN && result <= LONG_MAX) ?
                    PyInt_FromLong((long)result) : PyLong_FromLongLong(result);
            }
#endif
#if CYTHON_USE_TYPE_SLOTS
            return PyInt_Type.tp_as_number->nb_multiply(op1, op2);
#else
            return PyNumber_Multiply(op1, op2);
#endif
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        if (unlikely(__Pyx_PyLong_IsZero(op1))) {
            return __Pyx_NewRef(op1);
        }
        if (likely(__Pyx_PyLong_IsCompact(op1))) {
            a = __Pyx_PyLong_CompactValue(op1);
        } else {
            const digit* digits = __Pyx_PyLong_Digits(op1);
            const Py_ssize_t size = __Pyx_PyLong_SignedDigitCount(op1);
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT+30) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT+30) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT+30) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT+30) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT+30) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT+30) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT+30) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT+30) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT+30) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT+30) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT+30) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT+30) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_multiply(op1, op2);
            }
        }
                CYTHON_UNUSED_VAR(a);
                CYTHON_UNUSED_VAR(b);
                #ifdef HAVE_LONG_LONG
                lla = a;
                goto long_long;
                #else
                return PyLong_Type.tp_as_number->nb_multiply(op1, op2);
                #endif
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla * llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
#if CYTHON_COMPILING_IN_LIMITED_API
        double a = __pyx_PyFloat_AsDouble(op1);
#else
        double a = PyFloat_AS_DOUBLE(op1);
#endif
            double result;
            
            PyFPE_START_PROTECT("multiply", return NULL)
            result = ((double)a) * (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceMultiply : PyNumber_Multiply)(op1, op2);
}
#endif

/* PyIntBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_TrueDivideObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check) {
    CYTHON_MAYBE_UNUSED_VAR(intval);
    CYTHON_MAYBE_UNUSED_VAR(inplace);
    CYTHON_UNUSED_VAR(zerodivision_check);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        
            if (8 * sizeof(long) <= 53 || likely(labs(a) <= ((PY_LONG_LONG)1 << 53))) {
                return PyFloat_FromDouble((double)a / (double)b);
            }
            return PyInt_Type.tp_as_number->nb_true_divide(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
        if (unlikely(__Pyx_PyLong_IsZero(op1))) {
        }
        if (likely(__Pyx_PyLong_IsCompact(op1))) {
            a = __Pyx_PyLong_CompactValue(op1);
        } else {
            const digit* digits = __Pyx_PyLong_Digits(op1);
            const Py_ssize_t size = __Pyx_PyLong_SignedDigitCount(op1);
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT && 1 * PyLong_SHIFT < 53) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT && 1 * PyLong_SHIFT < 53) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT && 2 * PyLong_SHIFT < 53) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT && 2 * PyLong_SHIFT < 53) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT && 3 * PyLong_SHIFT < 53) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT && 3 * PyLong_SHIFT < 53) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_true_divide(op1, op2);
            }
        }
                if ((8 * sizeof(long) <= 53 || likely(labs(a) <= ((PY_LONG_LONG)1 << 53)))
                        || __Pyx_PyLong_DigitCount(op1) <= 52 / PyLong_SHIFT) {
                    return PyFloat_FromDouble((double)a / (double)b);
                }
                return PyLong_Type.tp_as_number->nb_true_divide(op1, op2);
            return PyLong_FromLong(x);
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
#if CYTHON_COMPILING_IN_LIMITED_API
        double a = __pyx_PyFloat_AsDouble(op1);
#else
        double a = PyFloat_AS_DOUBLE(op1);
#endif
            double result;
            
            PyFPE_START_PROTECT("divide", return NULL)
            result = ((double)a) / (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceTrueDivide : PyNumber_TrueDivide)(op1, op2);
}
#endif

/* DivInt[Py_ssize_t] */
static CYTHON_INLINE Py_ssize_t __Pyx_div_Py_ssize_t(Py_ssize_t a, Py_ssize_t b) {
    Py_ssize_t q = a / b;
    Py_ssize_t r = a - q*b;
    q -= ((r != 0) & ((r ^ b) < 0));
    return q;
}

/* SetItemInt */
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v) {
    int r;
    if (unlikely(!j)) return -1;
    r = PyObject_SetItem(o, j, v);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v, int is_list,
                                               CYTHON_NCP_UNUSED int wraparound, CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = (!wraparound) ? i : ((likely(i >= 0)) ? i : i + PyList_GET_SIZE(o));
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o)))) {
            PyObject* old = PyList_GET_ITEM(o, n);
            Py_INCREF(v);
            PyList_SET_ITEM(o, n, v);
            Py_DECREF(old);
            return 1;
        }
    } else {
        PyMappingMethods *mm = Py_TYPE(o)->tp_as_mapping;
        PySequenceMethods *sm = Py_TYPE(o)->tp_as_sequence;
        if (mm && mm->mp_ass_subscript) {
            int r;
            PyObject *key = PyInt_FromSsize_t(i);
            if (unlikely(!key)) return -1;
            r = mm->mp_ass_subscript(o, key, v);
            Py_DECREF(key);
            return r;
        }
        if (likely(sm && sm->sq_ass_item)) {
            if (wraparound && unlikely(i < 0) && likely(sm->sq_length)) {
                Py_ssize_t l = sm->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return -1;
                    PyErr_Clear();
                }
            }
            return sm->sq_ass_item(o, i, v);
        }
    }
#else
    if (is_list || !PyMapping_Check(o))
    {
        return PySequence_SetItem(o, i, v);
    }
#endif
    return __Pyx_SetItemInt_Generic(o, PyInt_FromSsize_t(i), v);
}

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    __Pyx_TypeName obj_type_name;
    __Pyx_TypeName type_name;
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    obj_type_name = __Pyx_PyType_GetName(Py_TYPE(obj));
    type_name = __Pyx_PyType_GetName(type);
    PyErr_Format(PyExc_TypeError,
                 "Cannot convert " __Pyx_FMT_TYPENAME " to " __Pyx_FMT_TYPENAME,
                 obj_type_name, type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    __Pyx_DECREF_TypeName(type_name);
    return 0;
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType_3_0_8
#define __PYX_HAVE_RT_ImportType_3_0_8
static PyTypeObject *__Pyx_ImportType_3_0_8(PyObject *module, const char *module_name, const char *class_name,
    size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_3_0_8 check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
    Py_ssize_t itemsize;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_basicsize;
    PyObject *py_itemsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#if !CYTHON_COMPILING_IN_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
    itemsize = ((PyTypeObject *)result)->tp_itemsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
    py_itemsize = PyObject_GetAttrString(result, "__itemsize__");
    if (!py_itemsize)
        goto bad;
    itemsize = PyLong_AsSsize_t(py_itemsize);
    Py_DECREF(py_itemsize);
    py_itemsize = 0;
    if (itemsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (itemsize) {
        if (size % alignment) {
            alignment = size % alignment;
        }
        if (itemsize < (Py_ssize_t)alignment)
            itemsize = (Py_ssize_t)alignment;
    }
    if ((size_t)(basicsize + itemsize) < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize+itemsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error_3_0_8 &&
            ((size_t)basicsize > size || (size_t)(basicsize + itemsize) < size)) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd-%zd from PyObject",
            module_name, class_name, size, basicsize, basicsize+itemsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn_3_0_8 && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* GetVTable */
static void* __Pyx_GetVtable(PyTypeObject *type) {
    void* ptr;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *ob = PyObject_GetAttr((PyObject *)type, __pyx_n_s_pyx_vtable);
#else
    PyObject *ob = PyObject_GetItem(type->tp_dict, __pyx_n_s_pyx_vtable);
#endif
    if (!ob)
        goto bad;
    ptr = PyCapsule_GetPointer(ob, 0);
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *module = 0;
    PyObject *empty_dict = 0;
    PyObject *empty_list = 0;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (unlikely(!py_import))
        goto bad;
    if (!from_list) {
        empty_list = PyList_New(0);
        if (unlikely(!empty_list))
            goto bad;
        from_list = empty_list;
    }
    #endif
    empty_dict = PyDict_New();
    if (unlikely(!empty_dict))
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.') != NULL) {
                module = PyImport_ImportModuleLevelObject(
                    name, __pyx_d, empty_dict, from_list, 1);
                if (unlikely(!module)) {
                    if (unlikely(!PyErr_ExceptionMatches(PyExc_ImportError)))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (unlikely(!py_level))
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, __pyx_d, empty_dict, from_list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, __pyx_d, empty_dict, from_list, level);
            #endif
        }
    }
bad:
    Py_XDECREF(empty_dict);
    Py_XDECREF(empty_list);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    return module;
}

/* ImportDottedModule */
#if PY_MAJOR_VERSION >= 3
static PyObject *__Pyx__ImportDottedModule_Error(PyObject *name, PyObject *parts_tuple, Py_ssize_t count) {
    PyObject *partial_name = NULL, *slice = NULL, *sep = NULL;
    if (unlikely(PyErr_Occurred())) {
        PyErr_Clear();
    }
    if (likely(PyTuple_GET_SIZE(parts_tuple) == count)) {
        partial_name = name;
    } else {
        slice = PySequence_GetSlice(parts_tuple, 0, count);
        if (unlikely(!slice))
            goto bad;
        sep = PyUnicode_FromStringAndSize(".", 1);
        if (unlikely(!sep))
            goto bad;
        partial_name = PyUnicode_Join(sep, slice);
    }
    PyErr_Format(
#if PY_MAJOR_VERSION < 3
        PyExc_ImportError,
        "No module named '%s'", PyString_AS_STRING(partial_name));
#else
#if PY_VERSION_HEX >= 0x030600B1
        PyExc_ModuleNotFoundError,
#else
        PyExc_ImportError,
#endif
        "No module named '%U'", partial_name);
#endif
bad:
    Py_XDECREF(sep);
    Py_XDECREF(slice);
    Py_XDECREF(partial_name);
    return NULL;
}
#endif
#if PY_MAJOR_VERSION >= 3
static PyObject *__Pyx__ImportDottedModule_Lookup(PyObject *name) {
    PyObject *imported_module;
#if PY_VERSION_HEX < 0x030700A1 || (CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM  < 0x07030400)
    PyObject *modules = PyImport_GetModuleDict();
    if (unlikely(!modules))
        return NULL;
    imported_module = __Pyx_PyDict_GetItemStr(modules, name);
    Py_XINCREF(imported_module);
#else
    imported_module = PyImport_GetModule(name);
#endif
    return imported_module;
}
#endif
#if PY_MAJOR_VERSION >= 3
static PyObject *__Pyx_ImportDottedModule_WalkParts(PyObject *module, PyObject *name, PyObject *parts_tuple) {
    Py_ssize_t i, nparts;
    nparts = PyTuple_GET_SIZE(parts_tuple);
    for (i=1; i < nparts && module; i++) {
        PyObject *part, *submodule;
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        part = PyTuple_GET_ITEM(parts_tuple, i);
#else
        part = PySequence_ITEM(parts_tuple, i);
#endif
        submodule = __Pyx_PyObject_GetAttrStrNoError(module, part);
#if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
        Py_DECREF(part);
#endif
        Py_DECREF(module);
        module = submodule;
    }
    if (unlikely(!module)) {
        return __Pyx__ImportDottedModule_Error(name, parts_tuple, i);
    }
    return module;
}
#endif
static PyObject *__Pyx__ImportDottedModule(PyObject *name, PyObject *parts_tuple) {
#if PY_MAJOR_VERSION < 3
    PyObject *module, *from_list, *star = __pyx_n_s__5;
    CYTHON_UNUSED_VAR(parts_tuple);
    from_list = PyList_New(1);
    if (unlikely(!from_list))
        return NULL;
    Py_INCREF(star);
    PyList_SET_ITEM(from_list, 0, star);
    module = __Pyx_Import(name, from_list, 0);
    Py_DECREF(from_list);
    return module;
#else
    PyObject *imported_module;
    PyObject *module = __Pyx_Import(name, NULL, 0);
    if (!parts_tuple || unlikely(!module))
        return module;
    imported_module = __Pyx__ImportDottedModule_Lookup(name);
    if (likely(imported_module)) {
        Py_DECREF(module);
        return imported_module;
    }
    PyErr_Clear();
    return __Pyx_ImportDottedModule_WalkParts(module, name, parts_tuple);
#endif
}
static PyObject *__Pyx_ImportDottedModule(PyObject *name, PyObject *parts_tuple) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030400B1
    PyObject *module = __Pyx__ImportDottedModule_Lookup(name);
    if (likely(module)) {
        PyObject *spec = __Pyx_PyObject_GetAttrStrNoError(module, __pyx_n_s_spec);
        if (likely(spec)) {
            PyObject *unsafe = __Pyx_PyObject_GetAttrStrNoError(spec, __pyx_n_s_initializing);
            if (likely(!unsafe || !__Pyx_PyObject_IsTrue(unsafe))) {
                Py_DECREF(spec);
                spec = NULL;
            }
            Py_XDECREF(unsafe);
        }
        if (likely(!spec)) {
            PyErr_Clear();
            return module;
        }
        Py_DECREF(spec);
        Py_DECREF(module);
    } else if (PyErr_Occurred()) {
        PyErr_Clear();
    }
#endif
    return __Pyx__ImportDottedModule(name, parts_tuple);
}

/* FixUpExtensionType */
#if CYTHON_USE_TYPE_SPECS
static int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type) {
#if PY_VERSION_HEX > 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    CYTHON_UNUSED_VAR(spec);
    CYTHON_UNUSED_VAR(type);
#else
    const PyType_Slot *slot = spec->slots;
    while (slot && slot->slot && slot->slot != Py_tp_members)
        slot++;
    if (slot && slot->slot == Py_tp_members) {
        int changed = 0;
#if !(PY_VERSION_HEX <= 0x030900b1 && CYTHON_COMPILING_IN_CPYTHON)
        const
#endif
            PyMemberDef *memb = (PyMemberDef*) slot->pfunc;
        while (memb && memb->name) {
            if (memb->name[0] == '_' && memb->name[1] == '_') {
#if PY_VERSION_HEX < 0x030900b1
                if (strcmp(memb->name, "__weaklistoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_weaklistoffset = memb->offset;
                    changed = 1;
                }
                else if (strcmp(memb->name, "__dictoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_dictoffset = memb->offset;
                    changed = 1;
                }
#if CYTHON_METH_FASTCALL
                else if (strcmp(memb->name, "__vectorcalloffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
#if PY_VERSION_HEX >= 0x030800b4
                    type->tp_vectorcall_offset = memb->offset;
#else
                    type->tp_print = (printfunc) memb->offset;
#endif
                    changed = 1;
                }
#endif
#else
                if ((0));
#endif
#if PY_VERSION_HEX <= 0x030900b1 && CYTHON_COMPILING_IN_CPYTHON
                else if (strcmp(memb->name, "__module__") == 0) {
                    PyObject *descr;
                    assert(memb->type == T_OBJECT);
                    assert(memb->flags == 0 || memb->flags == READONLY);
                    descr = PyDescr_NewMember(type, memb);
                    if (unlikely(!descr))
                        return -1;
                    if (unlikely(PyDict_SetItem(type->tp_dict, PyDescr_NAME(descr), descr) < 0)) {
                        Py_DECREF(descr);
                        return -1;
                    }
                    Py_DECREF(descr);
                    changed = 1;
                }
#endif
            }
            memb++;
        }
        if (changed)
            PyType_Modified(type);
    }
#endif
    return 0;
}
#endif

/* FetchSharedCythonModule */
static PyObject *__Pyx_FetchSharedCythonABIModule(void) {
    return __Pyx_PyImport_AddModuleRef((char*) __PYX_ABI_MODULE_NAME);
}

/* FetchCommonType */
static int __Pyx_VerifyCachedType(PyObject *cached_type,
                               const char *name,
                               Py_ssize_t basicsize,
                               Py_ssize_t expected_basicsize) {
    if (!PyType_Check(cached_type)) {
        PyErr_Format(PyExc_TypeError,
            "Shared Cython type %.200s is not a type object", name);
        return -1;
    }
    if (basicsize != expected_basicsize) {
        PyErr_Format(PyExc_TypeError,
            "Shared Cython type %.200s has the wrong size, try recompiling",
            name);
        return -1;
    }
    return 0;
}
#if !CYTHON_USE_TYPE_SPECS
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* abi_module;
    const char* object_name;
    PyTypeObject *cached_type = NULL;
    abi_module = __Pyx_FetchSharedCythonABIModule();
    if (!abi_module) return NULL;
    object_name = strrchr(type->tp_name, '.');
    object_name = object_name ? object_name+1 : type->tp_name;
    cached_type = (PyTypeObject*) PyObject_GetAttrString(abi_module, object_name);
    if (cached_type) {
        if (__Pyx_VerifyCachedType(
              (PyObject *)cached_type,
              object_name,
              cached_type->tp_basicsize,
              type->tp_basicsize) < 0) {
            goto bad;
        }
        goto done;
    }
    if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
    PyErr_Clear();
    if (PyType_Ready(type) < 0) goto bad;
    if (PyObject_SetAttrString(abi_module, object_name, (PyObject *)type) < 0)
        goto bad;
    Py_INCREF(type);
    cached_type = type;
done:
    Py_DECREF(abi_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}
#else
static PyTypeObject *__Pyx_FetchCommonTypeFromSpec(PyObject *module, PyType_Spec *spec, PyObject *bases) {
    PyObject *abi_module, *cached_type = NULL;
    const char* object_name = strrchr(spec->name, '.');
    object_name = object_name ? object_name+1 : spec->name;
    abi_module = __Pyx_FetchSharedCythonABIModule();
    if (!abi_module) return NULL;
    cached_type = PyObject_GetAttrString(abi_module, object_name);
    if (cached_type) {
        Py_ssize_t basicsize;
#if CYTHON_COMPILING_IN_LIMITED_API
        PyObject *py_basicsize;
        py_basicsize = PyObject_GetAttrString(cached_type, "__basicsize__");
        if (unlikely(!py_basicsize)) goto bad;
        basicsize = PyLong_AsSsize_t(py_basicsize);
        Py_DECREF(py_basicsize);
        py_basicsize = 0;
        if (unlikely(basicsize == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
#else
        basicsize = likely(PyType_Check(cached_type)) ? ((PyTypeObject*) cached_type)->tp_basicsize : -1;
#endif
        if (__Pyx_VerifyCachedType(
              cached_type,
              object_name,
              basicsize,
              spec->basicsize) < 0) {
            goto bad;
        }
        goto done;
    }
    if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
    PyErr_Clear();
    CYTHON_UNUSED_VAR(module);
    cached_type = __Pyx_PyType_FromModuleAndSpec(abi_module, spec, bases);
    if (unlikely(!cached_type)) goto bad;
    if (unlikely(__Pyx_fix_up_extension_type_from_spec(spec, (PyTypeObject *) cached_type) < 0)) goto bad;
    if (PyObject_SetAttrString(abi_module, object_name, cached_type) < 0) goto bad;
done:
    Py_DECREF(abi_module);
    assert(cached_type == NULL || PyType_Check(cached_type));
    return (PyTypeObject *) cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}
#endif

/* PyVectorcallFastCallDict */
#if CYTHON_METH_FASTCALL
static PyObject *__Pyx_PyVectorcall_FastCallDict_kw(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw)
{
    PyObject *res = NULL;
    PyObject *kwnames;
    PyObject **newargs;
    PyObject **kwvalues;
    Py_ssize_t i, pos;
    size_t j;
    PyObject *key, *value;
    unsigned long keys_are_strings;
    Py_ssize_t nkw = PyDict_GET_SIZE(kw);
    newargs = (PyObject **)PyMem_Malloc((nargs + (size_t)nkw) * sizeof(args[0]));
    if (unlikely(newargs == NULL)) {
        PyErr_NoMemory();
        return NULL;
    }
    for (j = 0; j < nargs; j++) newargs[j] = args[j];
    kwnames = PyTuple_New(nkw);
    if (unlikely(kwnames == NULL)) {
        PyMem_Free(newargs);
        return NULL;
    }
    kwvalues = newargs + nargs;
    pos = i = 0;
    keys_are_strings = Py_TPFLAGS_UNICODE_SUBCLASS;
    while (PyDict_Next(kw, &pos, &key, &value)) {
        keys_are_strings &= Py_TYPE(key)->tp_flags;
        Py_INCREF(key);
        Py_INCREF(value);
        PyTuple_SET_ITEM(kwnames, i, key);
        kwvalues[i] = value;
        i++;
    }
    if (unlikely(!keys_are_strings)) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        goto cleanup;
    }
    res = vc(func, newargs, nargs, kwnames);
cleanup:
    Py_DECREF(kwnames);
    for (i = 0; i < nkw; i++)
        Py_DECREF(kwvalues[i]);
    PyMem_Free(newargs);
    return res;
}
static CYTHON_INLINE PyObject *__Pyx_PyVectorcall_FastCallDict(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw)
{
    if (likely(kw == NULL) || PyDict_GET_SIZE(kw) == 0) {
        return vc(func, args, nargs, NULL);
    }
    return __Pyx_PyVectorcall_FastCallDict_kw(func, vc, args, nargs, kw);
}
#endif

/* CythonFunctionShared */
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void *cfunc) {
    if (__Pyx_CyFunction_Check(func)) {
        return PyCFunction_GetFunction(((__pyx_CyFunctionObject*)func)->func) == (PyCFunction) cfunc;
    } else if (PyCFunction_Check(func)) {
        return PyCFunction_GetFunction(func) == (PyCFunction) cfunc;
    }
    return 0;
}
#else
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void *cfunc) {
    return __Pyx_CyOrPyCFunction_Check(func) && __Pyx_CyOrPyCFunction_GET_FUNCTION(func) == (PyCFunction) cfunc;
}
#endif
static CYTHON_INLINE void __Pyx__CyFunction_SetClassObj(__pyx_CyFunctionObject* f, PyObject* classobj) {
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    __Pyx_Py_XDECREF_SET(
        __Pyx_CyFunction_GetClassObj(f),
            ((classobj) ? __Pyx_NewRef(classobj) : NULL));
#else
    __Pyx_Py_XDECREF_SET(
        ((PyCMethodObject *) (f))->mm_class,
        (PyTypeObject*)((classobj) ? __Pyx_NewRef(classobj) : NULL));
#endif
}
static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, void *closure)
{
    CYTHON_UNUSED_VAR(closure);
    if (unlikely(op->func_doc == NULL)) {
#if CYTHON_COMPILING_IN_LIMITED_API
        op->func_doc = PyObject_GetAttrString(op->func, "__doc__");
        if (unlikely(!op->func_doc)) return NULL;
#else
        if (((PyCFunctionObject*)op)->m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(((PyCFunctionObject*)op)->m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(((PyCFunctionObject*)op)->m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
#endif
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_doc, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(op->func_name == NULL)) {
#if CYTHON_COMPILING_IN_LIMITED_API
        op->func_name = PyObject_GetAttrString(op->func, "__name__");
#elif PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(((PyCFunctionObject*)op)->m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(((PyCFunctionObject*)op)->m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_name, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_qualname, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_dict, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(op);
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op, void *context)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = __Pyx_PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = __Pyx_PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value) {
        value = Py_None;
    } else if (unlikely(value != Py_None && !PyTuple_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    PyErr_WarnEx(PyExc_RuntimeWarning, "changes to cyfunction.__defaults__ will not "
                 "currently affect the values used in function calls", 1);
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->defaults_tuple, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result = op->defaults_tuple;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (unlikely(__Pyx_CyFunction_init_defaults(op) < 0)) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value) {
        value = Py_None;
    } else if (unlikely(value != Py_None && !PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    PyErr_WarnEx(PyExc_RuntimeWarning, "changes to cyfunction.__kwdefaults__ will not "
                 "currently affect the values used in function calls", 1);
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->defaults_kwdict, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result = op->defaults_kwdict;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (unlikely(__Pyx_CyFunction_init_defaults(op) < 0)) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value || value == Py_None) {
        value = NULL;
    } else if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_annotations, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result = op->func_annotations;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_is_coroutine(__pyx_CyFunctionObject *op, void *context) {
    int is_coroutine;
    CYTHON_UNUSED_VAR(context);
    if (op->func_is_coroutine) {
        return __Pyx_NewRef(op->func_is_coroutine);
    }
    is_coroutine = op->flags & __Pyx_CYFUNCTION_COROUTINE;
#if PY_VERSION_HEX >= 0x03050000
    if (is_coroutine) {
        PyObject *module, *fromlist, *marker = __pyx_n_s_is_coroutine;
        fromlist = PyList_New(1);
        if (unlikely(!fromlist)) return NULL;
        Py_INCREF(marker);
#if CYTHON_ASSUME_SAFE_MACROS
        PyList_SET_ITEM(fromlist, 0, marker);
#else
        if (unlikely(PyList_SetItem(fromlist, 0, marker) < 0)) {
            Py_DECREF(marker);
            Py_DECREF(fromlist);
            return NULL;
        }
#endif
        module = PyImport_ImportModuleLevelObject(__pyx_n_s_asyncio_coroutines, NULL, NULL, fromlist, 0);
        Py_DECREF(fromlist);
        if (unlikely(!module)) goto ignore;
        op->func_is_coroutine = __Pyx_PyObject_GetAttrStr(module, marker);
        Py_DECREF(module);
        if (likely(op->func_is_coroutine)) {
            return __Pyx_NewRef(op->func_is_coroutine);
        }
ignore:
        PyErr_Clear();
    }
#endif
    op->func_is_coroutine = __Pyx_PyBool_FromLong(is_coroutine);
    return __Pyx_NewRef(op->func_is_coroutine);
}
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *
__Pyx_CyFunction_get_module(__pyx_CyFunctionObject *op, void *context) {
    CYTHON_UNUSED_VAR(context);
    return PyObject_GetAttrString(op->func, "__module__");
}
static int
__Pyx_CyFunction_set_module(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    return PyObject_SetAttrString(op->func, "__module__", value);
}
#endif
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {(char *) "_is_coroutine", (getter)__Pyx_CyFunction_get_is_coroutine, 0, 0, 0},
#if CYTHON_COMPILING_IN_LIMITED_API
    {"__module__", (getter)__Pyx_CyFunction_get_module, (setter)__Pyx_CyFunction_set_module, 0, 0},
#endif
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
#if !CYTHON_COMPILING_IN_LIMITED_API
    {(char *) "__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), 0, 0},
#endif
#if CYTHON_USE_TYPE_SPECS
    {(char *) "__dictoffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_dict), READONLY, 0},
#if CYTHON_METH_FASTCALL
#if CYTHON_BACKPORT_VECTORCALL
    {(char *) "__vectorcalloffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_vectorcall), READONLY, 0},
#else
#if !CYTHON_COMPILING_IN_LIMITED_API
    {(char *) "__vectorcalloffset__", T_PYSSIZET, offsetof(PyCFunctionObject, vectorcall), READONLY, 0},
#endif
#endif
#endif
#if PY_VERSION_HEX < 0x030500A0 || CYTHON_COMPILING_IN_LIMITED_API
    {(char *) "__weaklistoffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_weakreflist), READONLY, 0},
#else
    {(char *) "__weaklistoffset__", T_PYSSIZET, offsetof(PyCFunctionObject, m_weakreflist), READONLY, 0},
#endif
#endif
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, PyObject *args)
{
    CYTHON_UNUSED_VAR(args);
#if PY_MAJOR_VERSION >= 3
    Py_INCREF(m->func_qualname);
    return m->func_qualname;
#else
    return PyString_FromString(((PyCFunctionObject*)m)->m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0 || CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) (((PyCFunctionObject*)cyfunc)->m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject *op, PyMethodDef *ml, int flags, PyObject* qualname,
                                       PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
#if !CYTHON_COMPILING_IN_LIMITED_API
    PyCFunctionObject *cf = (PyCFunctionObject*) op;
#endif
    if (unlikely(op == NULL))
        return NULL;
#if CYTHON_COMPILING_IN_LIMITED_API
    op->func = PyCFunction_NewEx(ml, (PyObject*)op, module);
    if (unlikely(!op->func)) return NULL;
#endif
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
#if !CYTHON_COMPILING_IN_LIMITED_API
    cf->m_ml = ml;
    cf->m_self = (PyObject *) op;
#endif
    Py_XINCREF(closure);
    op->func_closure = closure;
#if !CYTHON_COMPILING_IN_LIMITED_API
    Py_XINCREF(module);
    cf->m_module = module;
#endif
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    op->func_classobj = NULL;
#else
    ((PyCMethodObject*)op)->mm_class = NULL;
#endif
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults_size = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    op->func_is_coroutine = NULL;
#if CYTHON_METH_FASTCALL
    switch (ml->ml_flags & (METH_VARARGS | METH_FASTCALL | METH_NOARGS | METH_O | METH_KEYWORDS | METH_METHOD)) {
    case METH_NOARGS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_NOARGS;
        break;
    case METH_O:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_O;
        break;
    case METH_METHOD | METH_FASTCALL | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD;
        break;
    case METH_FASTCALL | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS;
        break;
    case METH_VARARGS | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = NULL;
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags for CyFunction");
        Py_DECREF(op);
        return NULL;
    }
#endif
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
#if CYTHON_COMPILING_IN_LIMITED_API
    Py_CLEAR(m->func);
#else
    Py_CLEAR(((PyCFunctionObject*)m)->m_module);
#endif
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
#if !CYTHON_COMPILING_IN_LIMITED_API
#if PY_VERSION_HEX < 0x030900B1
    Py_CLEAR(__Pyx_CyFunction_GetClassObj(m));
#else
    {
        PyObject *cls = (PyObject*) ((PyCMethodObject *) (m))->mm_class;
        ((PyCMethodObject *) (m))->mm_class = NULL;
        Py_XDECREF(cls);
    }
#endif
#endif
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    Py_CLEAR(m->func_is_coroutine);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyObject_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    __Pyx_PyHeapTypeObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
#if CYTHON_COMPILING_IN_LIMITED_API
    Py_VISIT(m->func);
#else
    Py_VISIT(((PyCFunctionObject*)m)->m_module);
#endif
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
#if !CYTHON_COMPILING_IN_LIMITED_API
    Py_VISIT(__Pyx_CyFunction_GetClassObj(m));
#endif
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    Py_VISIT(m->func_is_coroutine);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *f = ((__pyx_CyFunctionObject*)func)->func;
    PyObject *py_name = NULL;
    PyCFunction meth;
    int flags;
    meth = PyCFunction_GetFunction(f);
    if (unlikely(!meth)) return NULL;
    flags = PyCFunction_GetFlags(f);
    if (unlikely(flags < 0)) return NULL;
#else
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    int flags = f->m_ml->ml_flags;
#endif
    Py_ssize_t size;
    switch (flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)(void*)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
#if CYTHON_ASSUME_SAFE_MACROS
            size = PyTuple_GET_SIZE(arg);
#else
            size = PyTuple_Size(arg);
            if (unlikely(size < 0)) return NULL;
#endif
            if (likely(size == 0))
                return (*meth)(self, NULL);
#if CYTHON_COMPILING_IN_LIMITED_API
            py_name = __Pyx_CyFunction_get_name((__pyx_CyFunctionObject*)func, NULL);
            if (!py_name) return NULL;
            PyErr_Format(PyExc_TypeError,
                "%.200S() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                py_name, size);
            Py_DECREF(py_name);
#else
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
#endif
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
#if CYTHON_ASSUME_SAFE_MACROS
            size = PyTuple_GET_SIZE(arg);
#else
            size = PyTuple_Size(arg);
            if (unlikely(size < 0)) return NULL;
#endif
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = __Pyx_PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
#if CYTHON_COMPILING_IN_LIMITED_API
            py_name = __Pyx_CyFunction_get_name((__pyx_CyFunctionObject*)func, NULL);
            if (!py_name) return NULL;
            PyErr_Format(PyExc_TypeError,
                "%.200S() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                py_name, size);
            Py_DECREF(py_name);
#else
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
#endif
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags for CyFunction");
        return NULL;
    }
#if CYTHON_COMPILING_IN_LIMITED_API
    py_name = __Pyx_CyFunction_get_name((__pyx_CyFunctionObject*)func, NULL);
    if (!py_name) return NULL;
    PyErr_Format(PyExc_TypeError, "%.200S() takes no keyword arguments",
                 py_name);
    Py_DECREF(py_name);
#else
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
#endif
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *self, *result;
#if CYTHON_COMPILING_IN_LIMITED_API
    self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)func)->func);
    if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
    self = ((PyCFunctionObject*)func)->m_self;
#endif
    result = __Pyx_CyFunction_CallMethod(func, self, arg, kw);
    return result;
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
#if CYTHON_METH_FASTCALL
     __pyx_vectorcallfunc vc = __Pyx_CyFunction_func_vectorcall(cyfunc);
    if (vc) {
#if CYTHON_ASSUME_SAFE_MACROS
        return __Pyx_PyVectorcall_FastCallDict(func, vc, &PyTuple_GET_ITEM(args, 0), (size_t)PyTuple_GET_SIZE(args), kw);
#else
        (void) &__Pyx_PyVectorcall_FastCallDict;
        return PyVectorcall_Call(func, args, kw);
#endif
    }
#endif
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
#if CYTHON_ASSUME_SAFE_MACROS
        argc = PyTuple_GET_SIZE(args);
#else
        argc = PyTuple_Size(args);
        if (unlikely(!argc) < 0) return NULL;
#endif
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
#if PY_MAJOR_VERSION > 2
            PyErr_Format(PyExc_TypeError,
                         "unbound method %.200S() needs an argument",
                         cyfunc->func_qualname);
#else
            PyErr_SetString(PyExc_TypeError,
                            "unbound method needs an argument");
#endif
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
#if CYTHON_METH_FASTCALL
static CYTHON_INLINE int __Pyx_CyFunction_Vectorcall_CheckArgs(__pyx_CyFunctionObject *cyfunc, Py_ssize_t nargs, PyObject *kwnames)
{
    int ret = 0;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        if (unlikely(nargs < 1)) {
            PyErr_Format(PyExc_TypeError, "%.200s() needs an argument",
                         ((PyCFunctionObject*)cyfunc)->m_ml->ml_name);
            return -1;
        }
        ret = 1;
    }
    if (unlikely(kwnames) && unlikely(PyTuple_GET_SIZE(kwnames))) {
        PyErr_Format(PyExc_TypeError,
                     "%.200s() takes no keyword arguments", ((PyCFunctionObject*)cyfunc)->m_ml->ml_name);
        return -1;
    }
    return ret;
}
static PyObject * __Pyx_CyFunction_Vectorcall_NOARGS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyMethodDef* def = ((PyCFunctionObject*)cyfunc)->m_ml;
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, kwnames)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
        self = ((PyCFunctionObject*)cyfunc)->m_self;
        break;
    default:
        return NULL;
    }
    if (unlikely(nargs != 0)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
            def->ml_name, nargs);
        return NULL;
    }
    return def->ml_meth(self, NULL);
}
static PyObject * __Pyx_CyFunction_Vectorcall_O(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyMethodDef* def = ((PyCFunctionObject*)cyfunc)->m_ml;
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, kwnames)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
        self = ((PyCFunctionObject*)cyfunc)->m_self;
        break;
    default:
        return NULL;
    }
    if (unlikely(nargs != 1)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
            def->ml_name, nargs);
        return NULL;
    }
    return def->ml_meth(self, args[0]);
}
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyMethodDef* def = ((PyCFunctionObject*)cyfunc)->m_ml;
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, NULL)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
        self = ((PyCFunctionObject*)cyfunc)->m_self;
        break;
    default:
        return NULL;
    }
    return ((_PyCFunctionFastWithKeywords)(void(*)(void))def->ml_meth)(self, args, nargs, kwnames);
}
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyMethodDef* def = ((PyCFunctionObject*)cyfunc)->m_ml;
    PyTypeObject *cls = (PyTypeObject *) __Pyx_CyFunction_GetClassObj(cyfunc);
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, NULL)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
        self = ((PyCFunctionObject*)cyfunc)->m_self;
        break;
    default:
        return NULL;
    }
    return ((__Pyx_PyCMethod)(void(*)(void))def->ml_meth)(self, cls, args, (size_t)nargs, kwnames);
}
#endif
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_CyFunctionType_slots[] = {
    {Py_tp_dealloc, (void *)__Pyx_CyFunction_dealloc},
    {Py_tp_repr, (void *)__Pyx_CyFunction_repr},
    {Py_tp_call, (void *)__Pyx_CyFunction_CallAsMethod},
    {Py_tp_traverse, (void *)__Pyx_CyFunction_traverse},
    {Py_tp_clear, (void *)__Pyx_CyFunction_clear},
    {Py_tp_methods, (void *)__pyx_CyFunction_methods},
    {Py_tp_members, (void *)__pyx_CyFunction_members},
    {Py_tp_getset, (void *)__pyx_CyFunction_getsets},
    {Py_tp_descr_get, (void *)__Pyx_PyMethod_New},
    {0, 0},
};
static PyType_Spec __pyx_CyFunctionType_spec = {
    __PYX_TYPE_MODULE_PREFIX "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
#ifdef Py_TPFLAGS_METHOD_DESCRIPTOR
    Py_TPFLAGS_METHOD_DESCRIPTOR |
#endif
#if (defined(_Py_TPFLAGS_HAVE_VECTORCALL) && CYTHON_METH_FASTCALL)
    _Py_TPFLAGS_HAVE_VECTORCALL |
#endif
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_BASETYPE,
    __pyx_CyFunctionType_slots
};
#else
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    __PYX_TYPE_MODULE_PREFIX "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
#if !CYTHON_METH_FASTCALL
    0,
#elif CYTHON_BACKPORT_VECTORCALL
    (printfunc)offsetof(__pyx_CyFunctionObject, func_vectorcall),
#else
    offsetof(PyCFunctionObject, vectorcall),
#endif
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_CallAsMethod,
    0,
    0,
    0,
    0,
#ifdef Py_TPFLAGS_METHOD_DESCRIPTOR
    Py_TPFLAGS_METHOD_DESCRIPTOR |
#endif
#if defined(_Py_TPFLAGS_HAVE_VECTORCALL) && CYTHON_METH_FASTCALL
    _Py_TPFLAGS_HAVE_VECTORCALL |
#endif
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_BASETYPE,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_PyMethod_New,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
    0,
#endif
#if __PYX_NEED_TP_PRINT_SLOT
    0,
#endif
#if PY_VERSION_HEX >= 0x030C0000
    0,
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
    0,
#endif
};
#endif
static int __pyx_CyFunction_init(PyObject *module) {
#if CYTHON_USE_TYPE_SPECS
    __pyx_CyFunctionType = __Pyx_FetchCommonTypeFromSpec(module, &__pyx_CyFunctionType_spec, NULL);
#else
    CYTHON_UNUSED_VAR(module);
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
#endif
    if (unlikely(__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_Malloc(size);
    if (unlikely(!m->defaults))
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    m->defaults_size = size;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* CythonFunction */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    PyObject *op = __Pyx_CyFunction_Init(
        PyObject_GC_New(__pyx_CyFunctionObject, __pyx_CyFunctionType),
        ml, flags, qualname, closure, module, globals, code
    );
    if (likely(op)) {
        PyObject_GC_Track(op);
    }
    return op;
}

/* PyObjectLookupSpecial */
#if CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx__PyObject_LookupSpecial(PyObject* obj, PyObject* attr_name, int with_error) {
    PyObject *res;
    PyTypeObject *tp = Py_TYPE(obj);
#if PY_MAJOR_VERSION < 3
    if (unlikely(PyInstance_Check(obj)))
        return with_error ? __Pyx_PyObject_GetAttrStr(obj, attr_name) : __Pyx_PyObject_GetAttrStrNoError(obj, attr_name);
#endif
    res = _PyType_Lookup(tp, attr_name);
    if (likely(res)) {
        descrgetfunc f = Py_TYPE(res)->tp_descr_get;
        if (!f) {
            Py_INCREF(res);
        } else {
            res = f(res, obj, (PyObject *)tp);
        }
    } else if (with_error) {
        PyErr_SetObject(PyExc_AttributeError, attr_name);
    }
    return res;
}
#endif

/* PyObjectCallNoArg */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
    PyObject *arg[2] = {NULL, NULL};
    return __Pyx_PyObject_FastCall(func, arg + 1, 0 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    CYTHON_MAYBE_UNUSED_VAR(tstate);
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStrNoError(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        (void) PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
#if !CYTHON_COMPILING_IN_LIMITED_API
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}
#endif

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
#if PY_VERSION_HEX >= 0x030b00a6 && !CYTHON_COMPILING_IN_LIMITED_API
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_PyCode_Replace_For_AddTraceback(PyObject *code, PyObject *scratch_dict,
                                                       PyObject *firstlineno, PyObject *name) {
    PyObject *replace = NULL;
    if (unlikely(PyDict_SetItemString(scratch_dict, "co_firstlineno", firstlineno))) return NULL;
    if (unlikely(PyDict_SetItemString(scratch_dict, "co_name", name))) return NULL;
    replace = PyObject_GetAttrString(code, "replace");
    if (likely(replace)) {
        PyObject *result;
        result = PyObject_Call(replace, __pyx_empty_tuple, scratch_dict);
        Py_DECREF(replace);
        return result;
    }
    PyErr_Clear();
    #if __PYX_LIMITED_VERSION_HEX < 0x030780000
    {
        PyObject *compiled = NULL, *result = NULL;
        if (unlikely(PyDict_SetItemString(scratch_dict, "code", code))) return NULL;
        if (unlikely(PyDict_SetItemString(scratch_dict, "type", (PyObject*)(&PyType_Type)))) return NULL;
        compiled = Py_CompileString(
            "out = type(code)(\n"
            "  code.co_argcount, code.co_kwonlyargcount, code.co_nlocals, code.co_stacksize,\n"
            "  code.co_flags, code.co_code, code.co_consts, code.co_names,\n"
            "  code.co_varnames, code.co_filename, co_name, co_firstlineno,\n"
            "  code.co_lnotab)\n", "<dummy>", Py_file_input);
        if (!compiled) return NULL;
        result = PyEval_EvalCode(compiled, scratch_dict, scratch_dict);
        Py_DECREF(compiled);
        if (!result) PyErr_Print();
        Py_DECREF(result);
        result = PyDict_GetItemString(scratch_dict, "out");
        if (result) Py_INCREF(result);
        return result;
    }
    #else
    return NULL;
    #endif
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyObject *code_object = NULL, *py_py_line = NULL, *py_funcname = NULL, *dict = NULL;
    PyObject *replace = NULL, *getframe = NULL, *frame = NULL;
    PyObject *exc_type, *exc_value, *exc_traceback;
    int success = 0;
    if (c_line) {
        (void) __pyx_cfilenm;
        (void) __Pyx_CLineForTraceback(__Pyx_PyThreadState_Current, c_line);
    }
    PyErr_Fetch(&exc_type, &exc_value, &exc_traceback);
    code_object = Py_CompileString("_getframe()", filename, Py_eval_input);
    if (unlikely(!code_object)) goto bad;
    py_py_line = PyLong_FromLong(py_line);
    if (unlikely(!py_py_line)) goto bad;
    py_funcname = PyUnicode_FromString(funcname);
    if (unlikely(!py_funcname)) goto bad;
    dict = PyDict_New();
    if (unlikely(!dict)) goto bad;
    {
        PyObject *old_code_object = code_object;
        code_object = __Pyx_PyCode_Replace_For_AddTraceback(code_object, dict, py_py_line, py_funcname);
        Py_DECREF(old_code_object);
    }
    if (unlikely(!code_object)) goto bad;
    getframe = PySys_GetObject("_getframe");
    if (unlikely(!getframe)) goto bad;
    if (unlikely(PyDict_SetItemString(dict, "_getframe", getframe))) goto bad;
    frame = PyEval_EvalCode(code_object, dict, dict);
    if (unlikely(!frame) || frame == Py_None) goto bad;
    success = 1;
  bad:
    PyErr_Restore(exc_type, exc_value, exc_traceback);
    Py_XDECREF(code_object);
    Py_XDECREF(py_py_line);
    Py_XDECREF(py_funcname);
    Py_XDECREF(dict);
    Py_XDECREF(replace);
    if (success) {
        PyTraceBack_Here(
            (struct _frame*)frame);
    }
    Py_XDECREF(frame);
}
#else
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_srcfile = NULL;
    py_srcfile = PyString_FromString(filename);
    if (!py_srcfile) goto bad;
    #endif
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        if (!py_funcname) goto bad;
        #endif
    }
    #if PY_MAJOR_VERSION < 3
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    #else
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    #endif
    Py_XDECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_srcfile);
    #endif
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject *ptype, *pvalue, *ptraceback;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) {
            /* If the code object creation fails, then we should clear the
               fetched exception references and propagate the new exception */
            Py_XDECREF(ptype);
            Py_XDECREF(pvalue);
            Py_XDECREF(ptraceback);
            goto bad;
        }
        __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}
#endif

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* Declarations */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return ::std::complex< float >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return x + y*(__pyx_t_float_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      __pyx_t_float_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabsf(b.real) >= fabsf(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                float r = b.imag / b.real;
                float s = (float)(1.0) / (b.real + b.imag * r);
                return __pyx_t_float_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            float r = b.real / b.imag;
            float s = (float)(1.0) / (b.imag + b.real * r);
            return __pyx_t_float_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            float denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_float_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrtf(z.real*z.real + z.imag*z.imag);
          #else
            return hypotf(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
            __pyx_t_float_complex z;
            float r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    float denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_float(a, a);
                    case 3:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, a);
                    case 4:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if ((b.imag == 0) && (a.real >= 0)) {
                    z.real = powf(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2f(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_float(a);
                theta = atan2f(a.imag, a.real);
            }
            lnr = logf(r);
            z_r = expf(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cosf(z_theta);
            z.imag = z_r * sinf(z_theta);
            return z;
        }
    #endif
#endif

/* Declarations */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return ::std::complex< double >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return x + y*(__pyx_t_double_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      __pyx_t_double_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabs(b.real) >= fabs(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                double r = b.imag / b.real;
                double s = (double)(1.0) / (b.real + b.imag * r);
                return __pyx_t_double_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            double r = b.real / b.imag;
            double s = (double)(1.0) / (b.imag + b.real * r);
            return __pyx_t_double_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            double denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_double_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrt(z.real*z.real + z.imag*z.imag);
          #else
            return hypot(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
            __pyx_t_double_complex z;
            double r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    double denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_double(a, a);
                    case 3:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, a);
                    case 4:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if ((b.imag == 0) && (a.real >= 0)) {
                    z.real = pow(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_double(a);
                theta = atan2(a.imag, a.real);
            }
            lnr = log(r);
            z_r = exp(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cos(z_theta);
            z.imag = z_r * sin(z_theta);
            return z;
        }
    #endif
#endif

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(long) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            if (unlikely(__Pyx_PyLong_IsNeg(x))) {
                goto raise_neg_overflow;
            } else if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_DigitCount(x)) {
                    case 2:
                        if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(long) >= 2 * PyLong_SHIFT)) {
                                return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(long) >= 3 * PyLong_SHIFT)) {
                                return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(long) >= 4 * PyLong_SHIFT)) {
                                return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                            }
                        }
                        break;
                }
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if ((sizeof(long) <= sizeof(unsigned long))) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(long) <= sizeof(unsigned PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_SignedDigitCount(x)) {
                    case -2:
                        if ((8 * sizeof(long) - 1 > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                                return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                            }
                        }
                        break;
                    case 2:
                        if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                                return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                            }
                        }
                        break;
                    case -3:
                        if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                                return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                                return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                            }
                        }
                        break;
                    case -4:
                        if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                                return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                                return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                            }
                        }
                        break;
                }
            }
#endif
            if ((sizeof(long) <= sizeof(long))) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(long) <= sizeof(PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
#if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
#endif
            if (likely(v)) {
                int ret = -1;
#if PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                ret = _PyLong_AsByteArray((PyLongObject *)v,
                                           bytes, sizeof(val),
                                           is_little, !is_unsigned);
#else
                PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
                int bits, remaining_bits, is_negative = 0;
                long idigit;
                int chunk_size = (sizeof(long) < 8) ? 30 : 62;
                if (unlikely(!PyLong_CheckExact(v))) {
                    PyObject *tmp = v;
                    v = PyNumber_Long(v);
                    assert(PyLong_CheckExact(v));
                    Py_DECREF(tmp);
                    if (unlikely(!v)) return (long) -1;
                }
#if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
                if (Py_SIZE(x) == 0)
                    return (long) 0;
                is_negative = Py_SIZE(x) < 0;
#else
                {
                    int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                    if (unlikely(result < 0))
                        return (long) -1;
                    is_negative = result == 1;
                }
#endif
                if (is_unsigned && unlikely(is_negative)) {
                    goto raise_neg_overflow;
                } else if (is_negative) {
                    stepval = PyNumber_Invert(v);
                    if (unlikely(!stepval))
                        return (long) -1;
                } else {
                    stepval = __Pyx_NewRef(v);
                }
                val = (long) 0;
                mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
                shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
                for (bits = 0; bits < (int) sizeof(long) * 8 - chunk_size; bits += chunk_size) {
                    PyObject *tmp, *digit;
                    digit = PyNumber_And(stepval, mask);
                    if (unlikely(!digit)) goto done;
                    idigit = PyLong_AsLong(digit);
                    Py_DECREF(digit);
                    if (unlikely(idigit < 0)) goto done;
                    tmp = PyNumber_Rshift(stepval, shift);
                    if (unlikely(!tmp)) goto done;
                    Py_DECREF(stepval); stepval = tmp;
                    val |= ((long) idigit) << bits;
                    #if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
                    if (Py_SIZE(stepval) == 0)
                        goto unpacking_done;
                    #endif
                }
                idigit = PyLong_AsLong(stepval);
                if (unlikely(idigit < 0)) goto done;
                remaining_bits = ((int) sizeof(long) * 8) - bits - (is_unsigned ? 0 : 1);
                if (unlikely(idigit >= (1L << remaining_bits)))
                    goto raise_overflow;
                val |= ((long) idigit) << bits;
            #if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
            unpacking_done:
            #endif
                if (!is_unsigned) {
                    if (unlikely(val & (((long) 1) << (sizeof(long) * 8 - 1))))
                        goto raise_overflow;
                    if (is_negative)
                        val = ~val;
                }
                ret = 0;
            done:
                Py_XDECREF(shift);
                Py_XDECREF(mask);
                Py_XDECREF(stepval);
#endif
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
#else
        PyObject *from_bytes, *result = NULL;
        PyObject *py_bytes = NULL, *arg_tuple = NULL, *kwds = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(long));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        arg_tuple = PyTuple_Pack(2, py_bytes, order_str);
        if (!arg_tuple) goto limited_bad;
        if (!is_unsigned) {
            kwds = PyDict_New();
            if (!kwds) goto limited_bad;
            if (PyDict_SetItemString(kwds, "signed", __Pyx_NewRef(Py_True))) goto limited_bad;
        }
        result = PyObject_Call(from_bytes, arg_tuple, kwds);
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(arg_tuple);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* FormatTypeName */
#if CYTHON_COMPILING_IN_LIMITED_API
static __Pyx_TypeName
__Pyx_PyType_GetName(PyTypeObject* tp)
{
    PyObject *name = __Pyx_PyObject_GetAttrStr((PyObject *)tp,
                                               __pyx_n_s_name);
    if (unlikely(name == NULL) || unlikely(!PyUnicode_Check(name))) {
        PyErr_Clear();
        Py_XDECREF(name);
        name = __Pyx_NewRef(__pyx_n_s__15);
    }
    return name;
}
#endif

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(int) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            if (unlikely(__Pyx_PyLong_IsNeg(x))) {
                goto raise_neg_overflow;
            } else if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_DigitCount(x)) {
                    case 2:
                        if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(int) >= 2 * PyLong_SHIFT)) {
                                return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(int) >= 3 * PyLong_SHIFT)) {
                                return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(int) >= 4 * PyLong_SHIFT)) {
                                return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                            }
                        }
                        break;
                }
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if ((sizeof(int) <= sizeof(unsigned long))) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(int) <= sizeof(unsigned PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_SignedDigitCount(x)) {
                    case -2:
                        if ((8 * sizeof(int) - 1 > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                                return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                            }
                        }
                        break;
                    case 2:
                        if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                                return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                            }
                        }
                        break;
                    case -3:
                        if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                                return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                                return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                            }
                        }
                        break;
                    case -4:
                        if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                                return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                                return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                            }
                        }
                        break;
                }
            }
#endif
            if ((sizeof(int) <= sizeof(long))) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(int) <= sizeof(PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
#if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
#endif
            if (likely(v)) {
                int ret = -1;
#if PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                ret = _PyLong_AsByteArray((PyLongObject *)v,
                                           bytes, sizeof(val),
                                           is_little, !is_unsigned);
#else
                PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
                int bits, remaining_bits, is_negative = 0;
                long idigit;
                int chunk_size = (sizeof(long) < 8) ? 30 : 62;
                if (unlikely(!PyLong_CheckExact(v))) {
                    PyObject *tmp = v;
                    v = PyNumber_Long(v);
                    assert(PyLong_CheckExact(v));
                    Py_DECREF(tmp);
                    if (unlikely(!v)) return (int) -1;
                }
#if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
                if (Py_SIZE(x) == 0)
                    return (int) 0;
                is_negative = Py_SIZE(x) < 0;
#else
                {
                    int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                    if (unlikely(result < 0))
                        return (int) -1;
                    is_negative = result == 1;
                }
#endif
                if (is_unsigned && unlikely(is_negative)) {
                    goto raise_neg_overflow;
                } else if (is_negative) {
                    stepval = PyNumber_Invert(v);
                    if (unlikely(!stepval))
                        return (int) -1;
                } else {
                    stepval = __Pyx_NewRef(v);
                }
                val = (int) 0;
                mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
                shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
                for (bits = 0; bits < (int) sizeof(int) * 8 - chunk_size; bits += chunk_size) {
                    PyObject *tmp, *digit;
                    digit = PyNumber_And(stepval, mask);
                    if (unlikely(!digit)) goto done;
                    idigit = PyLong_AsLong(digit);
                    Py_DECREF(digit);
                    if (unlikely(idigit < 0)) goto done;
                    tmp = PyNumber_Rshift(stepval, shift);
                    if (unlikely(!tmp)) goto done;
                    Py_DECREF(stepval); stepval = tmp;
                    val |= ((int) idigit) << bits;
                    #if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
                    if (Py_SIZE(stepval) == 0)
                        goto unpacking_done;
                    #endif
                }
                idigit = PyLong_AsLong(stepval);
                if (unlikely(idigit < 0)) goto done;
                remaining_bits = ((int) sizeof(int) * 8) - bits - (is_unsigned ? 0 : 1);
                if (unlikely(idigit >= (1L << remaining_bits)))
                    goto raise_overflow;
                val |= ((int) idigit) << bits;
            #if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
            unpacking_done:
            #endif
                if (!is_unsigned) {
                    if (unlikely(val & (((int) 1) << (sizeof(int) * 8 - 1))))
                        goto raise_overflow;
                    if (is_negative)
                        val = ~val;
                }
                ret = 0;
            done:
                Py_XDECREF(shift);
                Py_XDECREF(mask);
                Py_XDECREF(stepval);
#endif
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = __Pyx_PyType_GetSlot(a, tp_base, PyTypeObject*);
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (cls == a || cls == b) return 1;
    mro = cls->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            PyObject *base = PyTuple_GET_ITEM(mro, i);
            if (base == (PyObject *)a || base == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(cls, a) || __Pyx_InBases(cls, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    if (exc_type1) {
        return __Pyx_IsAnySubtype2((PyTypeObject*)err, (PyTypeObject*)exc_type1, (PyTypeObject*)exc_type2);
    } else {
        return __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
static unsigned long __Pyx_get_runtime_version(void) {
#if __PYX_LIMITED_VERSION_HEX >= 0x030B00A4
    return Py_Version & ~0xFFUL;
#else
    const char* rt_version = Py_GetVersion();
    unsigned long version = 0;
    unsigned long factor = 0x01000000UL;
    unsigned int digit = 0;
    int i = 0;
    while (factor) {
        while ('0' <= rt_version[i] && rt_version[i] <= '9') {
            digit = digit * 10 + (unsigned int) (rt_version[i] - '0');
            ++i;
        }
        version += factor * digit;
        if (rt_version[i] != '.')
            break;
        digit = 0;
        factor >>= 8;
        ++i;
    }
    return version;
#endif
}
static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer) {
    const unsigned long MAJOR_MINOR = 0xFFFF0000UL;
    if ((rt_version & MAJOR_MINOR) == (ct_version & MAJOR_MINOR))
        return 0;
    if (likely(allow_newer && (rt_version & MAJOR_MINOR) > (ct_version & MAJOR_MINOR)))
        return 1;
    {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compile time Python version %d.%d "
                      "of module '%.100s' "
                      "%s "
                      "runtime version %d.%d",
                       (int) (ct_version >> 24), (int) ((ct_version >> 16) & 0xFF),
                       __Pyx_MODULE_NAME,
                       (allow_newer) ? "was newer than" : "does not match",
                       (int) (rt_version >> 24), (int) ((rt_version >> 16) & 0xFF)
       );
        return PyErr_WarnEx(NULL, message, 1);
    }
}

/* FunctionImport */
#ifndef __PYX_HAVE_RT_ImportFunction_3_0_8
#define __PYX_HAVE_RT_ImportFunction_3_0_8
static int __Pyx_ImportFunction_3_0_8(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, funcname);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C function %.200s",
                PyModule_GetName(module), funcname);
        goto bad;
    }
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    tmp.p = PyCapsule_GetPointer(cobj, sig);
    *f = tmp.fp;
    if (!(*f))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* InitStrings */
#if PY_MAJOR_VERSION >= 3
static int __Pyx_InitString(__Pyx_StringTabEntry t, PyObject **str) {
    if (t.is_unicode | t.is_str) {
        if (t.intern) {
            *str = PyUnicode_InternFromString(t.s);
        } else if (t.encoding) {
            *str = PyUnicode_Decode(t.s, t.n - 1, t.encoding, NULL);
        } else {
            *str = PyUnicode_FromStringAndSize(t.s, t.n - 1);
        }
    } else {
        *str = PyBytes_FromStringAndSize(t.s, t.n - 1);
    }
    if (!*str)
        return -1;
    if (PyObject_Hash(*str) == -1)
        return -1;
    return 0;
}
#endif
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION >= 3
        __Pyx_InitString(*t, t->p);
        #else
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        #endif
        ++t;
    }
    return 0;
}

#include <string.h>
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s) {
    size_t len = strlen(s);
    if (unlikely(len > (size_t) PY_SSIZE_T_MAX)) {
        PyErr_SetString(PyExc_OverflowError, "byte string is too long");
        return -1;
    }
    return (Py_ssize_t) len;
}
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return __Pyx_PyUnicode_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return PyByteArray_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY && !CYTHON_COMPILING_IN_LIMITED_API) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
    __Pyx_TypeName result_type_name = __Pyx_PyType_GetName(Py_TYPE(result));
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type " __Pyx_FMT_TYPENAME ").  "
                "The ability to return an instance of a strict subclass of int is deprecated, "
                "and may be removed in a future version of Python.",
                result_type_name)) {
            __Pyx_DECREF_TypeName(result_type_name);
            Py_DECREF(result);
            return NULL;
        }
        __Pyx_DECREF_TypeName(result_type_name);
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type " __Pyx_FMT_TYPENAME ")",
                 type_name, type_name, result_type_name);
    __Pyx_DECREF_TypeName(result_type_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(__Pyx_PyLong_IsCompact(b))) {
        return __Pyx_PyLong_CompactValue(b);
    } else {
      const digit* digits = __Pyx_PyLong_Digits(b);
      const Py_ssize_t size = __Pyx_PyLong_SignedDigitCount(b);
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
#if PY_MAJOR_VERSION < 3
  } else if (likely(PyInt_CheckExact(o))) {
    return PyInt_AS_LONG(o);
#endif
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyInt_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


/* #### Code section: utility_code_pragmas_end ### */
#ifdef _MSC_VER
#pragma warning( pop )
#endif



/* #### Code section: end ### */
#endif /* Py_PYTHON_H */
