/* Generated by Cython 3.0.8 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "define_macros": [
            [
                "NPY_NO_DEPRECATED_API",
                "NPY_1_7_API_VERSION"
            ]
        ],
        "extra_compile_args": [
            "-Wno-parentheses-equality",
            "-Wno-unreachable-code-fallthrough"
        ],
        "extra_objects": [
            "./sndlib/libsndlib.a"
        ],
        "include_dirs": [
            "/private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/core/include",
            "./sndlib"
        ],
        "name": "pysndlib.instruments.bouncy",
        "sources": [
            "src/pysndlib/instruments/bouncy.py"
        ]
    },
    "module_name": "pysndlib.instruments.bouncy"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
#if defined(CYTHON_LIMITED_API) && 0
  #ifndef Py_LIMITED_API
    #if CYTHON_LIMITED_API+0 > 0x03030000
      #define Py_LIMITED_API CYTHON_LIMITED_API
    #else
      #define Py_LIMITED_API 0x03030000
    #endif
  #endif
#endif

#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02070000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.7+ or Python 3.3+.
#else
#if defined(CYTHON_LIMITED_API) && CYTHON_LIMITED_API
#define __PYX_EXTRA_ABI_MODULE_NAME "limited"
#else
#define __PYX_EXTRA_ABI_MODULE_NAME ""
#endif
#define CYTHON_ABI "3_0_8" __PYX_EXTRA_ABI_MODULE_NAME
#define __PYX_ABI_MODULE_NAME "_cython_" CYTHON_ABI
#define __PYX_TYPE_MODULE_PREFIX __PYX_ABI_MODULE_NAME "."
#define CYTHON_HEX_VERSION 0x030008F0
#define CYTHON_FUTURE_DIVISION 1
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(_WIN32) && !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #define HAVE_LONG_LONG
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#define __PYX_LIMITED_VERSION_HEX PY_VERSION_HEX
#if defined(GRAALVM_PYTHON)
  /* For very preliminary testing purposes. Most variables are set the same as PyPy.
     The existence of this section does not imply that anything works or is even tested */
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 1
  #define CYTHON_COMPILING_IN_NOGIL 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS (PY_MAJOR_VERSION >= 3)
  #endif
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
#elif defined(PYPY_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS (PY_MAJOR_VERSION >= 3)
  #endif
  #if PY_VERSION_HEX < 0x03090000
    #undef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #elif !defined(CYTHON_PEP489_MULTI_PHASE_INIT)
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1 && PYPY_VERSION_NUM >= 0x07030C00)
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
#elif defined(CYTHON_LIMITED_API)
  #ifdef Py_LIMITED_API
    #undef __PYX_LIMITED_VERSION_HEX
    #define __PYX_LIMITED_VERSION_HEX Py_LIMITED_API
  #endif
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 1
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #undef CYTHON_CLINE_IN_TRACEBACK
  #define CYTHON_CLINE_IN_TRACEBACK 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 1
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #endif
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 1
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 0
  #endif
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
#elif defined(Py_GIL_DISABLED) || defined(Py_NOGIL)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_NOGIL 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #ifndef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 1
  #endif
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #ifndef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #ifndef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0 || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_GIL
    #define CYTHON_FAST_GIL (PY_MAJOR_VERSION < 3 || PY_VERSION_HEX >= 0x03060000 && PY_VERSION_HEX < 0x030C00A6)
  #endif
  #ifndef CYTHON_METH_FASTCALL
    #define CYTHON_METH_FASTCALL (PY_VERSION_HEX >= 0x030700A1)
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #elif !defined(CYTHON_PEP489_MULTI_PHASE_INIT)
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_MODULE_STATE
    #define CYTHON_USE_MODULE_STATE 0
  #endif
  #if PY_VERSION_HEX < 0x030400a1
    #undef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 0
  #elif !defined(CYTHON_USE_TP_FINALIZE)
    #define CYTHON_USE_TP_FINALIZE 1
  #endif
  #if PY_VERSION_HEX < 0x030600B1
    #undef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS 0
  #elif !defined(CYTHON_USE_DICT_VERSIONS)
    #define CYTHON_USE_DICT_VERSIONS  (PY_VERSION_HEX < 0x030C00A5)
  #endif
  #if PY_VERSION_HEX < 0x030700A3
    #undef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK 0
  #elif !defined(CYTHON_USE_EXC_INFO_STACK)
    #define CYTHON_USE_EXC_INFO_STACK 1
  #endif
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 1
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if !defined(CYTHON_VECTORCALL)
#define CYTHON_VECTORCALL  (CYTHON_FAST_PYCCALL && PY_VERSION_HEX >= 0x030800B1)
#endif
#define CYTHON_BACKPORT_VECTORCALL (CYTHON_METH_FASTCALL && PY_VERSION_HEX < 0x030800B1)
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_MAJOR_VERSION < 3
    #include "longintrepr.h"
  #endif
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(maybe_unused) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(maybe_unused)
        #define CYTHON_UNUSED [[maybe_unused]]
      #endif
    #endif
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
  #define CYTHON_MAYBE_UNUSED_VAR(x) CYTHON_UNUSED_VAR(x)
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_USE_CPP_STD_MOVE
  #if defined(__cplusplus) && (\
    __cplusplus >= 201103L || (defined(_MSC_VER) && _MSC_VER >= 1600))
    #define CYTHON_USE_CPP_STD_MOVE 1
  #else
    #define CYTHON_USE_CPP_STD_MOVE 0
  #endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
            typedef unsigned char     uint8_t;
            typedef unsigned short    uint16_t;
            typedef unsigned int      uint32_t;
        #else
            typedef unsigned __int8   uint8_t;
            typedef unsigned __int16  uint16_t;
            typedef unsigned __int32  uint32_t;
        #endif
    #endif
    #if _MSC_VER < 1300
        #ifdef _WIN64
            typedef unsigned long long  __pyx_uintptr_t;
        #else
            typedef unsigned int        __pyx_uintptr_t;
        #endif
    #else
        #ifdef _WIN64
            typedef unsigned __int64    __pyx_uintptr_t;
        #else
            typedef unsigned __int32    __pyx_uintptr_t;
        #endif
    #endif
#else
    #include <stdint.h>
    typedef uintptr_t  __pyx_uintptr_t;
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(fallthrough) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(fallthrough)
        #define CYTHON_FALLTHROUGH [[fallthrough]]
      #endif
    #endif
    #ifndef CYTHON_FALLTHROUGH
      #if __has_cpp_attribute(clang::fallthrough)
        #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
      #elif __has_cpp_attribute(gnu::fallthrough)
        #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
      #endif
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif
#ifdef __cplusplus
  template <typename T>
  struct __PYX_IS_UNSIGNED_IMPL {static const bool value = T(0) < T(-1);};
  #define __PYX_IS_UNSIGNED(type) (__PYX_IS_UNSIGNED_IMPL<type>::value)
#else
  #define __PYX_IS_UNSIGNED(type) (((type)-1) > 0)
#endif
#if CYTHON_COMPILING_IN_PYPY == 1
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x030A0000)
#else
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000)
#endif
#define __PYX_REINTERPRET_FUNCION(func_pointer, other_pointer) ((func_pointer)(void(*)(void))(other_pointer))

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_DefaultClassType PyClass_Type
  #define __Pyx_PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_DefaultClassType PyType_Type
#if CYTHON_COMPILING_IN_LIMITED_API
    static CYTHON_INLINE PyObject* __Pyx_PyCode_New(int a, int p, int k, int l, int s, int f,
                                                    PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                                    PyObject *fv, PyObject *cell, PyObject* fn,
                                                    PyObject *name, int fline, PyObject *lnos) {
        PyObject *exception_table = NULL;
        PyObject *types_module=NULL, *code_type=NULL, *result=NULL;
        #if __PYX_LIMITED_VERSION_HEX < 0x030B0000
        PyObject *version_info;
        PyObject *py_minor_version = NULL;
        #endif
        long minor_version = 0;
        PyObject *type, *value, *traceback;
        PyErr_Fetch(&type, &value, &traceback);
        #if __PYX_LIMITED_VERSION_HEX >= 0x030B0000
        minor_version = 11;
        #else
        if (!(version_info = PySys_GetObject("version_info"))) goto end;
        if (!(py_minor_version = PySequence_GetItem(version_info, 1))) goto end;
        minor_version = PyLong_AsLong(py_minor_version);
        Py_DECREF(py_minor_version);
        if (minor_version == -1 && PyErr_Occurred()) goto end;
        #endif
        if (!(types_module = PyImport_ImportModule("types"))) goto end;
        if (!(code_type = PyObject_GetAttrString(types_module, "CodeType"))) goto end;
        if (minor_version <= 7) {
            (void)p;
            result = PyObject_CallFunction(code_type, "iiiiiOOOOOOiOO", a, k, l, s, f, code,
                          c, n, v, fn, name, fline, lnos, fv, cell);
        } else if (minor_version <= 10) {
            result = PyObject_CallFunction(code_type, "iiiiiiOOOOOOiOO", a,p, k, l, s, f, code,
                          c, n, v, fn, name, fline, lnos, fv, cell);
        } else {
            if (!(exception_table = PyBytes_FromStringAndSize(NULL, 0))) goto end;
            result = PyObject_CallFunction(code_type, "iiiiiiOOOOOOOiOO", a,p, k, l, s, f, code,
                          c, n, v, fn, name, name, fline, lnos, exception_table, fv, cell);
        }
    end:
        Py_XDECREF(code_type);
        Py_XDECREF(exception_table);
        Py_XDECREF(types_module);
        if (type) {
            PyErr_Restore(type, value, traceback);
        }
        return result;
    }
    #ifndef CO_OPTIMIZED
    #define CO_OPTIMIZED 0x0001
    #endif
    #ifndef CO_NEWLOCALS
    #define CO_NEWLOCALS 0x0002
    #endif
    #ifndef CO_VARARGS
    #define CO_VARARGS 0x0004
    #endif
    #ifndef CO_VARKEYWORDS
    #define CO_VARKEYWORDS 0x0008
    #endif
    #ifndef CO_ASYNC_GENERATOR
    #define CO_ASYNC_GENERATOR 0x0200
    #endif
    #ifndef CO_GENERATOR
    #define CO_GENERATOR 0x0020
    #endif
    #ifndef CO_COROUTINE
    #define CO_COROUTINE 0x0080
    #endif
#elif PY_VERSION_HEX >= 0x030B0000
  static CYTHON_INLINE PyCodeObject* __Pyx_PyCode_New(int a, int p, int k, int l, int s, int f,
                                                    PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                                    PyObject *fv, PyObject *cell, PyObject* fn,
                                                    PyObject *name, int fline, PyObject *lnos) {
    PyCodeObject *result;
    PyObject *empty_bytes = PyBytes_FromStringAndSize("", 0);
    if (!empty_bytes) return NULL;
    result =
      #if PY_VERSION_HEX >= 0x030C0000
        PyUnstable_Code_NewWithPosOnlyArgs
      #else
        PyCode_NewWithPosOnlyArgs
      #endif
        (a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, name, fline, lnos, empty_bytes);
    Py_DECREF(empty_bytes);
    return result;
  }
#elif PY_VERSION_HEX >= 0x030800B2 && !CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_NewWithPosOnlyArgs(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
#endif
#if PY_VERSION_HEX >= 0x030900A4 || defined(Py_IS_TYPE)
  #define __Pyx_IS_TYPE(ob, type) Py_IS_TYPE(ob, type)
#else
  #define __Pyx_IS_TYPE(ob, type) (((const PyObject*)ob)->ob_type == (type))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_Is)
  #define __Pyx_Py_Is(x, y)  Py_Is(x, y)
#else
  #define __Pyx_Py_Is(x, y) ((x) == (y))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsNone)
  #define __Pyx_Py_IsNone(ob) Py_IsNone(ob)
#else
  #define __Pyx_Py_IsNone(ob) __Pyx_Py_Is((ob), Py_None)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsTrue)
  #define __Pyx_Py_IsTrue(ob) Py_IsTrue(ob)
#else
  #define __Pyx_Py_IsTrue(ob) __Pyx_Py_Is((ob), Py_True)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsFalse)
  #define __Pyx_Py_IsFalse(ob) Py_IsFalse(ob)
#else
  #define __Pyx_Py_IsFalse(ob) __Pyx_Py_Is((ob), Py_False)
#endif
#define __Pyx_NoneAsNull(obj)  (__Pyx_Py_IsNone(obj) ? NULL : (obj))
#if PY_VERSION_HEX >= 0x030900F0 && !CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyObject_GC_IsFinalized(o) PyObject_GC_IsFinalized(o)
#else
  #define __Pyx_PyObject_GC_IsFinalized(o) _PyGC_FINALIZED(o)
#endif
#ifndef CO_COROUTINE
  #define CO_COROUTINE 0x80
#endif
#ifndef CO_ASYNC_GENERATOR
  #define CO_ASYNC_GENERATOR 0x200
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef Py_TPFLAGS_SEQUENCE
  #define Py_TPFLAGS_SEQUENCE 0
#endif
#ifndef Py_TPFLAGS_MAPPING
  #define Py_TPFLAGS_MAPPING 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_METH_FASTCALL
  #define __Pyx_METH_FASTCALL METH_FASTCALL
  #define __Pyx_PyCFunction_FastCall __Pyx_PyCFunctionFast
  #define __Pyx_PyCFunction_FastCallWithKeywords __Pyx_PyCFunctionFastWithKeywords
#else
  #define __Pyx_METH_FASTCALL METH_VARARGS
  #define __Pyx_PyCFunction_FastCall PyCFunction
  #define __Pyx_PyCFunction_FastCallWithKeywords PyCFunctionWithKeywords
#endif
#if CYTHON_VECTORCALL
  #define __pyx_vectorcallfunc vectorcallfunc
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  PY_VECTORCALL_ARGUMENTS_OFFSET
  #define __Pyx_PyVectorcall_NARGS(n)  PyVectorcall_NARGS((size_t)(n))
#elif CYTHON_BACKPORT_VECTORCALL
  typedef PyObject *(*__pyx_vectorcallfunc)(PyObject *callable, PyObject *const *args,
                                            size_t nargsf, PyObject *kwnames);
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  ((size_t)1 << (8 * sizeof(size_t) - 1))
  #define __Pyx_PyVectorcall_NARGS(n)  ((Py_ssize_t)(((size_t)(n)) & ~__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET))
#else
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  0
  #define __Pyx_PyVectorcall_NARGS(n)  ((Py_ssize_t)(n))
#endif
#if PY_MAJOR_VERSION >= 0x030900B1
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_CheckExact(func)
#else
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_Check(func)
#endif
#define __Pyx_CyOrPyCFunction_Check(func)  PyCFunction_Check(func)
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  (((PyCFunctionObject*)(func))->m_ml->ml_meth)
#elif !CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  PyCFunction_GET_FUNCTION(func)
#endif
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FLAGS(func)  (((PyCFunctionObject*)(func))->m_ml->ml_flags)
static CYTHON_INLINE PyObject* __Pyx_CyOrPyCFunction_GET_SELF(PyObject *func) {
    return (__Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_STATIC) ? NULL : ((PyCFunctionObject*)func)->m_self;
}
#endif
static CYTHON_INLINE int __Pyx__IsSameCFunction(PyObject *func, void *cfunc) {
#if CYTHON_COMPILING_IN_LIMITED_API
    return PyCFunction_Check(func) && PyCFunction_GetFunction(func) == (PyCFunction) cfunc;
#else
    return PyCFunction_Check(func) && PyCFunction_GET_FUNCTION(func) == (PyCFunction) cfunc;
#endif
}
#define __Pyx_IsSameCFunction(func, cfunc)   __Pyx__IsSameCFunction(func, cfunc)
#if __PYX_LIMITED_VERSION_HEX < 0x030900B1
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  ((void)m, PyType_FromSpecWithBases(s, b))
  typedef PyObject *(*__Pyx_PyCMethod)(PyObject *, PyTypeObject *, PyObject *const *, size_t, PyObject *);
#else
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  PyType_FromModuleAndSpec(m, s, b)
  #define __Pyx_PyCMethod  PyCMethod
#endif
#ifndef METH_METHOD
  #define METH_METHOD 0x200
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyThreadState_Current PyThreadState_Get()
#elif !CYTHON_FAST_THREAD_STATE
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x030d00A1
  #define __Pyx_PyThreadState_Current PyThreadState_GetUnchecked()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE void *__Pyx_PyModule_GetState(PyObject *op)
{
    void *result;
    result = PyModule_GetState(op);
    if (!result)
        Py_FatalError("Couldn't find the module state");
    return result;
}
#endif
#define __Pyx_PyObject_GetSlot(obj, name, func_ctype)  __Pyx_PyType_GetSlot(Py_TYPE(obj), name, func_ctype)
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((func_ctype) PyType_GetSlot((type), Py_##name))
#else
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((type)->name)
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if PY_MAJOR_VERSION < 3
    #if CYTHON_COMPILING_IN_PYPY
        #if PYPY_VERSION_NUM < 0x07030600
            #if defined(__cplusplus) && __cplusplus >= 201402L
                [[deprecated("`with nogil:` inside a nogil function will not release the GIL in PyPy2 < 7.3.6")]]
            #elif defined(__GNUC__) || defined(__clang__)
                __attribute__ ((__deprecated__("`with nogil:` inside a nogil function will not release the GIL in PyPy2 < 7.3.6")))
            #elif defined(_MSC_VER)
                __declspec(deprecated("`with nogil:` inside a nogil function will not release the GIL in PyPy2 < 7.3.6"))
            #endif
            static CYTHON_INLINE int PyGILState_Check(void) {
                return 0;
            }
        #else  // PYPY_VERSION_NUM < 0x07030600
        #endif  // PYPY_VERSION_NUM < 0x07030600
    #else
        static CYTHON_INLINE int PyGILState_Check(void) {
            PyThreadState * tstate = _PyThreadState_Current;
            return tstate && (tstate == PyGILState_GetThisThreadState());
        }
    #endif
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030d0000 || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX > 0x030600B4 && PY_VERSION_HEX < 0x030d0000 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStrWithError(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStr(PyObject *dict, PyObject *name) {
    PyObject *res = __Pyx_PyDict_GetItemStrWithError(dict, name);
    if (res == NULL) PyErr_Clear();
    return res;
}
#elif PY_MAJOR_VERSION >= 3 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07020000)
#define __Pyx_PyDict_GetItemStrWithError  PyDict_GetItemWithError
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#else
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStrWithError(PyObject *dict, PyObject *name) {
#if CYTHON_COMPILING_IN_PYPY
    return PyDict_GetItem(dict, name);
#else
    PyDictEntry *ep;
    PyDictObject *mp = (PyDictObject*) dict;
    long hash = ((PyStringObject *) name)->ob_shash;
    assert(hash != -1);
    ep = (mp->ma_lookup)(mp, name, hash);
    if (ep == NULL) {
        return NULL;
    }
    return ep->me_value;
#endif
}
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#endif
#if CYTHON_USE_TYPE_SLOTS
  #define __Pyx_PyType_GetFlags(tp)   (((PyTypeObject *)tp)->tp_flags)
  #define __Pyx_PyType_HasFeature(type, feature)  ((__Pyx_PyType_GetFlags(type) & (feature)) != 0)
  #define __Pyx_PyObject_GetIterNextFunc(obj)  (Py_TYPE(obj)->tp_iternext)
#else
  #define __Pyx_PyType_GetFlags(tp)   (PyType_GetFlags((PyTypeObject *)tp))
  #define __Pyx_PyType_HasFeature(type, feature)  PyType_HasFeature(type, feature)
  #define __Pyx_PyObject_GetIterNextFunc(obj)  PyIter_Next
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_SetItemOnTypeDict(tp, k, v) PyObject_GenericSetAttr((PyObject*)tp, k, v)
#else
  #define __Pyx_SetItemOnTypeDict(tp, k, v) PyDict_SetItem(tp->tp_dict, k, v)
#endif
#if CYTHON_USE_TYPE_SPECS && PY_VERSION_HEX >= 0x03080000
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  {\
    PyTypeObject *type = Py_TYPE((PyObject*)obj);\
    assert(__Pyx_PyType_HasFeature(type, Py_TPFLAGS_HEAPTYPE));\
    PyObject_GC_Del(obj);\
    Py_DECREF(type);\
}
#else
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  PyObject_GC_Del(obj)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GetLength(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_ReadChar(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((void)u, 1114111U)
  #define __Pyx_PyUnicode_KIND(u)         ((void)u, (0))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)u)
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)k, PyUnicode_ReadChar((PyObject*)(d), i))
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GetLength(u))
#elif PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_READY(op)       (0)
  #else
    #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                                0 : _PyUnicode_Ready((PyObject *)(op)))
  #endif
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         ((int)PyUnicode_KIND(u))
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, (Py_UCS4) ch)
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #else
    #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
    #else
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
    #endif
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535U : 1114111U)
  #define __Pyx_PyUnicode_KIND(u)         ((int)sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = (Py_UNICODE) ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #if !defined(PyUnicode_DecodeUnicodeEscape)
    #define PyUnicode_DecodeUnicodeEscape(s, size, errors)  PyUnicode_Decode(s, size, "unicode_escape", errors)
  #endif
  #if !defined(PyUnicode_Contains) || (PY_MAJOR_VERSION == 2 && PYPY_VERSION_NUM < 0x07030500)
    #undef PyUnicode_Contains
    #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
  #endif
  #if !defined(PyByteArray_Check)
    #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
  #endif
  #if !defined(PyObject_Format)
    #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
  #endif
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#if CYTHON_COMPILING_IN_CPYTHON
  #define __Pyx_PySequence_ListKeepNew(obj)\
    (likely(PyList_CheckExact(obj) && Py_REFCNT(obj) == 1) ? __Pyx_NewRef(obj) : PySequence_List(obj))
#else
  #define __Pyx_PySequence_ListKeepNew(obj)  PySequence_List(obj)
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        __Pyx_IS_TYPE(obj, &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_ITEM(o, i) PySequence_ITEM(o, i)
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) (PyTuple_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyList_SET_ITEM(o, i, v) (PyList_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_GET_SIZE(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_GET_SIZE(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_GET_SIZE(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_GET_SIZE(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_GET_SIZE(o)
#else
  #define __Pyx_PySequence_ITEM(o, i) PySequence_GetItem(o, i)
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) PyTuple_SetItem(o, i, v)
  #define __Pyx_PyList_SET_ITEM(o, i, v) PyList_SetItem(o, i, v)
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_Size(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_Size(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_Size(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_Size(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_Size(o)
#endif
#if PY_VERSION_HEX >= 0x030d00A1
  #define __Pyx_PyImport_AddModuleRef(name) PyImport_AddModuleRef(name)
#else
  static CYTHON_INLINE PyObject *__Pyx_PyImport_AddModuleRef(const char *name) {
      PyObject *module = PyImport_AddModule(name);
      Py_XINCREF(module);
      return module;
  }
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define __Pyx_Py3Int_Check(op)       PyLong_Check(op)
  #define __Pyx_Py3Int_CheckExact(op)  PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#else
  #define __Pyx_Py3Int_Check(op)       (PyLong_Check(op) || PyInt_Check(op))
  #define __Pyx_Py3Int_CheckExact(op)  (PyLong_CheckExact(op) || PyInt_CheckExact(op))
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsHash_t
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsSsize_t
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)
  #if !defined(_USE_MATH_DEFINES)
    #define _USE_MATH_DEFINES
  #endif
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifdef CYTHON_EXTERN_C
    #undef __PYX_EXTERN_C
    #define __PYX_EXTERN_C CYTHON_EXTERN_C
#elif defined(__PYX_EXTERN_C)
    #ifdef _MSC_VER
    #pragma message ("Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.")
    #else
    #warning Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.
    #endif
#else
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__pysndlib__instruments__bouncy
#define __PYX_HAVE_API__pysndlib__instruments__bouncy
/* Early includes */
#include <string.h>
#include <stdio.h>

    /* Using NumPy API declarations from "numpy/__init__.cython-30.pxd" */
    
#include "numpy/arrayobject.h"
#include "numpy/ndarrayobject.h"
#include "numpy/ndarraytypes.h"
#include "numpy/arrayscalars.h"
#include "numpy/ufuncobject.h"
#include "sndlib.h"
#include "clm.h"

    typedef mus_float_t (*connect_points_cb)(mus_float_t val);
    typedef mus_float_t (*input_cb)(void *arg, int direction);
    typedef int (*edit_cb)(void *closure);
    typedef bool (*analyze_cb)(void *arg, mus_float_t (*input)(void *arg1, int direction));
	typedef mus_float_t (*synthesize_cb)(void *arg);
	typedef void (*detour_cb)(mus_any *ptr, mus_long_t val);

    typedef mus_any* mus_any_ptr;
    typedef mus_any** mus_any_ptr_ptr;
    
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s);
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char*);
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const wchar_t *u)
{
    const wchar_t *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#else
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u)
{
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#endif
#define __Pyx_PyUnicode_FromOrdinal(o)       PyUnicode_FromOrdinal((int)o)
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_VERSION_HEX >= 0x030C00A7
  #ifndef _PyLong_SIGN_MASK
    #define _PyLong_SIGN_MASK 3
  #endif
  #ifndef _PyLong_NON_SIZE_BITS
    #define _PyLong_NON_SIZE_BITS 3
  #endif
  #define __Pyx_PyLong_Sign(x)  (((PyLongObject*)x)->long_value.lv_tag & _PyLong_SIGN_MASK)
  #define __Pyx_PyLong_IsNeg(x)  ((__Pyx_PyLong_Sign(x) & 2) != 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (!__Pyx_PyLong_IsNeg(x))
  #define __Pyx_PyLong_IsZero(x)  (__Pyx_PyLong_Sign(x) & 1)
  #define __Pyx_PyLong_IsPos(x)  (__Pyx_PyLong_Sign(x) == 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  (__Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  ((Py_ssize_t) (((PyLongObject*)x)->long_value.lv_tag >> _PyLong_NON_SIZE_BITS))
  #define __Pyx_PyLong_SignedDigitCount(x)\
        ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * __Pyx_PyLong_DigitCount(x))
  #if defined(PyUnstable_Long_IsCompact) && defined(PyUnstable_Long_CompactValue)
    #define __Pyx_PyLong_IsCompact(x)     PyUnstable_Long_IsCompact((PyLongObject*) x)
    #define __Pyx_PyLong_CompactValue(x)  PyUnstable_Long_CompactValue((PyLongObject*) x)
  #else
    #define __Pyx_PyLong_IsCompact(x)     (((PyLongObject*)x)->long_value.lv_tag < (2 << _PyLong_NON_SIZE_BITS))
    #define __Pyx_PyLong_CompactValue(x)  ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * (Py_ssize_t) __Pyx_PyLong_Digits(x)[0])
  #endif
  typedef Py_ssize_t  __Pyx_compact_pylong;
  typedef size_t  __Pyx_compact_upylong;
  #else
  #define __Pyx_PyLong_IsNeg(x)  (Py_SIZE(x) < 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (Py_SIZE(x) >= 0)
  #define __Pyx_PyLong_IsZero(x)  (Py_SIZE(x) == 0)
  #define __Pyx_PyLong_IsPos(x)  (Py_SIZE(x) > 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  ((Py_SIZE(x) == 0) ? 0 : __Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  __Pyx_sst_abs(Py_SIZE(x))
  #define __Pyx_PyLong_SignedDigitCount(x)  Py_SIZE(x)
  #define __Pyx_PyLong_IsCompact(x)  (Py_SIZE(x) == 0 || Py_SIZE(x) == 1 || Py_SIZE(x) == -1)
  #define __Pyx_PyLong_CompactValue(x)\
        ((Py_SIZE(x) == 0) ? (sdigit) 0 : ((Py_SIZE(x) < 0) ? -(sdigit)__Pyx_PyLong_Digits(x)[0] : (sdigit)__Pyx_PyLong_Digits(x)[0]))
  typedef sdigit  __Pyx_compact_pylong;
  typedef digit  __Pyx_compact_upylong;
  #endif
  #if PY_VERSION_HEX >= 0x030C00A5
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->long_value.ob_digit)
  #else
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->ob_digit)
  #endif
#endif
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
#include <string.h>
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = (char) c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#include <string.h>
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

#if !CYTHON_USE_MODULE_STATE
static PyObject *__pyx_m = NULL;
#endif
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm = __FILE__;
static const char *__pyx_filename;

/* Header.proto */
#if !defined(CYTHON_CCOMPLEX)
  #if defined(__cplusplus)
    #define CYTHON_CCOMPLEX 1
  #elif (defined(_Complex_I) && !defined(_MSC_VER)) || ((defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L) && !defined(__STDC_NO_COMPLEX__) && !defined(_MSC_VER))
    #define CYTHON_CCOMPLEX 1
  #else
    #define CYTHON_CCOMPLEX 0
  #endif
#endif
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #include <complex>
  #else
    #include <complex.h>
  #endif
#endif
#if CYTHON_CCOMPLEX && !defined(__cplusplus) && defined(__sun__) && defined(__GNUC__)
  #undef _Complex_I
  #define _Complex_I 1.0fj
#endif

/* #### Code section: filename_table ### */

static const char *__pyx_f[] = {
  "src/pysndlib/instruments/bouncy.py",
  "__init__.cython-30.pxd",
  "type.pxd",
  "src/pysndlib/clm.pxd",
  "src/pysndlib/generators.pxd",
};
/* #### Code section: utility_code_proto_before_types ### */
/* ForceInitThreads.proto */
#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif

/* #### Code section: numeric_typedefs ### */

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":730
 * # in Cython to enable them only on the right systems.
 * 
 * ctypedef npy_int8       int8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 */
typedef npy_int8 __pyx_t_5numpy_int8_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":731
 * 
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t
 */
typedef npy_int16 __pyx_t_5numpy_int16_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":732
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int64      int64_t
 * #ctypedef npy_int96      int96_t
 */
typedef npy_int32 __pyx_t_5numpy_int32_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":733
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_int96      int96_t
 * #ctypedef npy_int128     int128_t
 */
typedef npy_int64 __pyx_t_5numpy_int64_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":737
 * #ctypedef npy_int128     int128_t
 * 
 * ctypedef npy_uint8      uint8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 */
typedef npy_uint8 __pyx_t_5numpy_uint8_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":738
 * 
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t
 */
typedef npy_uint16 __pyx_t_5numpy_uint16_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":739
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint64     uint64_t
 * #ctypedef npy_uint96     uint96_t
 */
typedef npy_uint32 __pyx_t_5numpy_uint32_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":740
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_uint96     uint96_t
 * #ctypedef npy_uint128    uint128_t
 */
typedef npy_uint64 __pyx_t_5numpy_uint64_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":744
 * #ctypedef npy_uint128    uint128_t
 * 
 * ctypedef npy_float32    float32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_float64    float64_t
 * #ctypedef npy_float80    float80_t
 */
typedef npy_float32 __pyx_t_5numpy_float32_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":745
 * 
 * ctypedef npy_float32    float32_t
 * ctypedef npy_float64    float64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_float80    float80_t
 * #ctypedef npy_float128   float128_t
 */
typedef npy_float64 __pyx_t_5numpy_float64_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":754
 * # The int types are mapped a bit surprising --
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   longlong_t
 * 
 */
typedef npy_long __pyx_t_5numpy_int_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":755
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   longlong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_ulong      uint_t
 */
typedef npy_longlong __pyx_t_5numpy_longlong_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":757
 * ctypedef npy_longlong   longlong_t
 * 
 * ctypedef npy_ulong      uint_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 */
typedef npy_ulong __pyx_t_5numpy_uint_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":758
 * 
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulonglong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_intp       intp_t
 */
typedef npy_ulonglong __pyx_t_5numpy_ulonglong_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":760
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 * ctypedef npy_intp       intp_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uintp      uintp_t
 * 
 */
typedef npy_intp __pyx_t_5numpy_intp_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":761
 * 
 * ctypedef npy_intp       intp_t
 * ctypedef npy_uintp      uintp_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_double     float_t
 */
typedef npy_uintp __pyx_t_5numpy_uintp_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":763
 * ctypedef npy_uintp      uintp_t
 * 
 * ctypedef npy_double     float_t             # <<<<<<<<<<<<<<
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t
 */
typedef npy_double __pyx_t_5numpy_float_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":764
 * 
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longdouble longdouble_t
 * 
 */
typedef npy_double __pyx_t_5numpy_double_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":765
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cfloat      cfloat_t
 */
typedef npy_longdouble __pyx_t_5numpy_longdouble_t;
/* #### Code section: complex_type_declarations ### */
/* Declarations.proto */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
  #ifdef __cplusplus
    typedef ::std::complex< float > __pyx_t_float_complex;
  #else
    typedef float _Complex __pyx_t_float_complex;
  #endif
#else
    typedef struct { float real, imag; } __pyx_t_float_complex;
#endif
static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float, float);

/* Declarations.proto */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
  #ifdef __cplusplus
    typedef ::std::complex< double > __pyx_t_double_complex;
  #else
    typedef double _Complex __pyx_t_double_complex;
  #endif
#else
    typedef struct { double real, imag; } __pyx_t_double_complex;
#endif
static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double, double);

/* #### Code section: type_declarations ### */

/*--- Type declarations ---*/
struct __pyx_obj_8pysndlib_3clm_mus_any;
struct __pyx_obj_8pysndlib_10generators_CLMGenerator;
struct __pyx_obj_8pysndlib_10generators_Nssb;
struct __pyx_obj_8pysndlib_10generators_Nxysin;
struct __pyx_obj_8pysndlib_10generators_Nxycos;
struct __pyx_obj_8pysndlib_10generators_Nxy1cos;
struct __pyx_obj_8pysndlib_10generators_Nxy1sin;
struct __pyx_obj_8pysndlib_10generators_Noddcos;
struct __pyx_obj_8pysndlib_10generators_Noddssb;
struct __pyx_obj_8pysndlib_10generators_Ncos2;
struct __pyx_obj_8pysndlib_10generators_Ncos4;
struct __pyx_obj_8pysndlib_10generators_Npcos;
struct __pyx_obj_8pysndlib_10generators_Ncos5;
struct __pyx_obj_8pysndlib_10generators_Nsin5;
struct __pyx_obj_8pysndlib_10generators_Nrcos;
struct __pyx_obj_8pysndlib_10generators_Nrssb;
struct __pyx_obj_8pysndlib_10generators_Nkssb;
struct __pyx_obj_8pysndlib_10generators_Nsincos;
struct __pyx_obj_8pysndlib_10generators_N1cos;
struct __pyx_obj_8pysndlib_10generators_Npos1cos;
struct __pyx_obj_8pysndlib_10generators_Npos3cos;
struct __pyx_obj_8pysndlib_10generators_Rcos;
struct __pyx_obj_8pysndlib_10generators_Rssb;
struct __pyx_obj_8pysndlib_10generators_Rxysin;
struct __pyx_obj_8pysndlib_10generators_Rxycos;
struct __pyx_obj_8pysndlib_10generators_SafeRxycos;
struct __pyx_obj_8pysndlib_10generators_Ercos;
struct __pyx_obj_8pysndlib_10generators_Erssb;
struct __pyx_obj_8pysndlib_10generators_R2cos;
struct __pyx_obj_8pysndlib_10generators_R2ssb;
struct __pyx_obj_8pysndlib_10generators_Eoddcos;
struct __pyx_obj_8pysndlib_10generators_Rkcos;
struct __pyx_obj_8pysndlib_10generators_Rksin;
struct __pyx_obj_8pysndlib_10generators_Rkssb;
struct __pyx_obj_8pysndlib_10generators_Rkfcos;
struct __pyx_obj_8pysndlib_10generators_Rkfssb;
struct __pyx_obj_8pysndlib_10generators_R2kfcos;
struct __pyx_obj_8pysndlib_10generators_K2sin;
struct __pyx_obj_8pysndlib_10generators_K2cos;
struct __pyx_obj_8pysndlib_10generators_K2ssb;
struct __pyx_obj_8pysndlib_10generators_Dblsum;
struct __pyx_obj_8pysndlib_10generators_Rkoddssb;
struct __pyx_obj_8pysndlib_10generators_Krksin;
struct __pyx_obj_8pysndlib_10generators_Absin;
struct __pyx_obj_8pysndlib_10generators_Abcos;
struct __pyx_obj_8pysndlib_10generators_R2k2cos;
struct __pyx_obj_8pysndlib_10generators_Blsaw;
struct __pyx_obj_8pysndlib_10generators_Asyfm;
struct __pyx_obj_8pysndlib_10generators_Bess;
struct __pyx_obj_8pysndlib_10generators_Jjcos;
struct __pyx_obj_8pysndlib_10generators_J0evencos;
struct __pyx_obj_8pysndlib_10generators_J2cos;
struct __pyx_obj_8pysndlib_10generators_Jpcos;
struct __pyx_obj_8pysndlib_10generators_Jncos;
struct __pyx_obj_8pysndlib_10generators_J0j1cos;
struct __pyx_obj_8pysndlib_10generators_Jycos;
struct __pyx_obj_8pysndlib_10generators_Jcos;
struct __pyx_obj_8pysndlib_10generators_Blackman;
struct __pyx_obj_8pysndlib_10generators_Fmssb;
struct __pyx_obj_8pysndlib_10generators_K3sin;
struct __pyx_obj_8pysndlib_10generators_Izcos;
struct __pyx_obj_8pysndlib_10generators_AdjustableSquareWave;
struct __pyx_obj_8pysndlib_10generators_AdjustableTriangleWave;
struct __pyx_obj_8pysndlib_10generators_AdjustableSawtoothWave;
struct __pyx_obj_8pysndlib_10generators_AdjustableOscil;
struct __pyx_obj_8pysndlib_10generators_RoundInterp;
struct __pyx_obj_8pysndlib_10generators_Nchoosekcos;
struct __pyx_obj_8pysndlib_10generators_SincTrain;
struct __pyx_obj_8pysndlib_10generators_PinkNoise;
struct __pyx_obj_8pysndlib_10generators_BrownNoise;
struct __pyx_obj_8pysndlib_10generators_GreenNoise;
struct __pyx_obj_8pysndlib_10generators_GreenNoiseInterp;
struct __pyx_obj_8pysndlib_10generators_MovingSum;
struct __pyx_obj_8pysndlib_10generators_MovingVariance;
struct __pyx_obj_8pysndlib_10generators_MovingRMS;
struct __pyx_obj_8pysndlib_10generators_MovingLength;
struct __pyx_obj_8pysndlib_10generators_WeightedMovingAverage;
struct __pyx_obj_8pysndlib_10generators_ExponentiallyWeightedMovingAverage;
struct __pyx_obj_8pysndlib_10generators_Tanhsin;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":767
 * ctypedef npy_longdouble longdouble_t
 * 
 * ctypedef npy_cfloat      cfloat_t             # <<<<<<<<<<<<<<
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t
 */
typedef npy_cfloat __pyx_t_5numpy_cfloat_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":768
 * 
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t             # <<<<<<<<<<<<<<
 * ctypedef npy_clongdouble clongdouble_t
 * 
 */
typedef npy_cdouble __pyx_t_5numpy_cdouble_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":769
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cdouble     complex_t
 */
typedef npy_clongdouble __pyx_t_5numpy_clongdouble_t;

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":771
 * ctypedef npy_clongdouble clongdouble_t
 * 
 * ctypedef npy_cdouble     complex_t             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 */
typedef npy_cdouble __pyx_t_5numpy_complex_t;

/* "pysndlib/sndlib.pxd":11
 * csndlib.mus_sound_initialize()
 * 
 * cpdef enum Header:             # <<<<<<<<<<<<<<
 *     """
 *     file types for audio files
 */
enum __pyx_t_8pysndlib_6sndlib_Header {
  __pyx_e_8pysndlib_6sndlib_UNKNOWN_HEADER,
  __pyx_e_8pysndlib_6sndlib_NEXT,
  __pyx_e_8pysndlib_6sndlib_AIFC,
  __pyx_e_8pysndlib_6sndlib_RIFF,
  __pyx_e_8pysndlib_6sndlib_RF64,
  __pyx_e_8pysndlib_6sndlib_BICSF,
  __pyx_e_8pysndlib_6sndlib_NIST,
  __pyx_e_8pysndlib_6sndlib_INRS,
  __pyx_e_8pysndlib_6sndlib_ESPS,
  __pyx_e_8pysndlib_6sndlib_SVX,
  __pyx_e_8pysndlib_6sndlib_VOC,
  __pyx_e_8pysndlib_6sndlib_SNDT,
  __pyx_e_8pysndlib_6sndlib_RAW,
  __pyx_e_8pysndlib_6sndlib_SMP,
  __pyx_e_8pysndlib_6sndlib_AVR,
  __pyx_e_8pysndlib_6sndlib_IRCAM,
  __pyx_e_8pysndlib_6sndlib_SD1,
  __pyx_e_8pysndlib_6sndlib_SPPACK,
  __pyx_e_8pysndlib_6sndlib_MUS10,
  __pyx_e_8pysndlib_6sndlib_HCOM,
  __pyx_e_8pysndlib_6sndlib_PSION,
  __pyx_e_8pysndlib_6sndlib_MAUD,
  __pyx_e_8pysndlib_6sndlib_IEEE,
  __pyx_e_8pysndlib_6sndlib_MATLAB,
  __pyx_e_8pysndlib_6sndlib_ADC,
  __pyx_e_8pysndlib_6sndlib_MIDI,
  __pyx_e_8pysndlib_6sndlib_SOUNDFONT,
  __pyx_e_8pysndlib_6sndlib_GRAVIS,
  __pyx_e_8pysndlib_6sndlib_COMDISCO,
  __pyx_e_8pysndlib_6sndlib_GOLDWAVE,
  __pyx_e_8pysndlib_6sndlib_SRFS,
  __pyx_e_8pysndlib_6sndlib_MIDI_SAMPLE_DUMP,
  __pyx_e_8pysndlib_6sndlib_DIAMONDWARE,
  __pyx_e_8pysndlib_6sndlib_ADF,
  __pyx_e_8pysndlib_6sndlib_SBSTUDIOII,
  __pyx_e_8pysndlib_6sndlib_DELUSION,
  __pyx_e_8pysndlib_6sndlib_FARANDOLE,
  __pyx_e_8pysndlib_6sndlib_SAMPLE_DUMP,
  __pyx_e_8pysndlib_6sndlib_ULTRATRACKER,
  __pyx_e_8pysndlib_6sndlib_YAMAHA_SY85,
  __pyx_e_8pysndlib_6sndlib_YAMAHA_TX16W,
  __pyx_e_8pysndlib_6sndlib_DIGIPLAYER,
  __pyx_e_8pysndlib_6sndlib_COVOX,
  __pyx_e_8pysndlib_6sndlib_AVI,
  __pyx_e_8pysndlib_6sndlib_OMF,
  __pyx_e_8pysndlib_6sndlib_QUICKTIME,
  __pyx_e_8pysndlib_6sndlib_ASF,
  __pyx_e_8pysndlib_6sndlib_YAMAHA_SY99,
  __pyx_e_8pysndlib_6sndlib_KURZWEIL_2000,
  __pyx_e_8pysndlib_6sndlib_AIFF,
  __pyx_e_8pysndlib_6sndlib_PAF,
  __pyx_e_8pysndlib_6sndlib_CSL,
  __pyx_e_8pysndlib_6sndlib_FILE_SAMP,
  __pyx_e_8pysndlib_6sndlib_PVF,
  __pyx_e_8pysndlib_6sndlib_SOUNDFORGE,
  __pyx_e_8pysndlib_6sndlib_TWINVQ,
  __pyx_e_8pysndlib_6sndlib_AKAI4,
  __pyx_e_8pysndlib_6sndlib_IMPULSETRACKER,
  __pyx_e_8pysndlib_6sndlib_KORG,
  __pyx_e_8pysndlib_6sndlib_NVF,
  __pyx_e_8pysndlib_6sndlib_CAFF,
  __pyx_e_8pysndlib_6sndlib_MAUI,
  __pyx_e_8pysndlib_6sndlib_SDIF,
  __pyx_e_8pysndlib_6sndlib_OGG,
  __pyx_e_8pysndlib_6sndlib_FLAC,
  __pyx_e_8pysndlib_6sndlib_SPEEX,
  __pyx_e_8pysndlib_6sndlib_MPEG,
  __pyx_e_8pysndlib_6sndlib_SHORTEN,
  __pyx_e_8pysndlib_6sndlib_TTA,
  __pyx_e_8pysndlib_6sndlib_WAVPACK,
  __pyx_e_8pysndlib_6sndlib_SOX,
  __pyx_e_8pysndlib_6sndlib_NUM_HEADERS
};

/* "pysndlib/sndlib.pxd":17
 *     UNKNOWN_HEADER, NEXT, AIFC, RIFF, RF64, BICSF, NIST, INRS, ESPS, SVX, VOC, SNDT, RAW, SMP, AVR, IRCAM, SD1, SPPACK, MUS10, HCOM, PSION, MAUD, IEEE, MATLAB, ADC, MIDI, SOUNDFONT, GRAVIS, COMDISCO, GOLDWAVE, SRFS, MIDI_SAMPLE_DUMP, DIAMONDWARE, ADF, SBSTUDIOII, DELUSION, FARANDOLE, SAMPLE_DUMP, ULTRATRACKER, YAMAHA_SY85, YAMAHA_TX16W, DIGIPLAYER, COVOX, AVI, OMF, QUICKTIME, ASF, YAMAHA_SY99, KURZWEIL_2000, AIFF, PAF, CSL, FILE_SAMP, PVF, SOUNDFORGE, TWINVQ, AKAI4, IMPULSETRACKER, KORG, NVF, CAFF, MAUI, SDIF, OGG, FLAC, SPEEX, MPEG, SHORTEN, TTA, WAVPACK, SOX, NUM_HEADERS,
 * 
 * cpdef enum Sample:             # <<<<<<<<<<<<<<
 *     """
 *     numerical sample types
 */
enum __pyx_t_8pysndlib_6sndlib_Sample {
  __pyx_e_8pysndlib_6sndlib_UNKNOWN_SAMPLE,
  __pyx_e_8pysndlib_6sndlib_BSHORT,
  __pyx_e_8pysndlib_6sndlib_MULAW,
  __pyx_e_8pysndlib_6sndlib_BYTE,
  __pyx_e_8pysndlib_6sndlib_BFLOAT,
  __pyx_e_8pysndlib_6sndlib_BINT,
  __pyx_e_8pysndlib_6sndlib_ALAW,
  __pyx_e_8pysndlib_6sndlib_UBYTE,
  __pyx_e_8pysndlib_6sndlib_B24INT,
  __pyx_e_8pysndlib_6sndlib_BDOUBLE,
  __pyx_e_8pysndlib_6sndlib_LSHORT,
  __pyx_e_8pysndlib_6sndlib_LINT,
  __pyx_e_8pysndlib_6sndlib_LFLOAT,
  __pyx_e_8pysndlib_6sndlib_LDOUBLE,
  __pyx_e_8pysndlib_6sndlib_UBSHORT,
  __pyx_e_8pysndlib_6sndlib_ULSHORT,
  __pyx_e_8pysndlib_6sndlib_L24INT,
  __pyx_e_8pysndlib_6sndlib_BINTN,
  __pyx_e_8pysndlib_6sndlib_LINTN,
  __pyx_e_8pysndlib_6sndlib_BFLOAT_UNSCALED,
  __pyx_e_8pysndlib_6sndlib_LFLOAT_UNSCALED,
  __pyx_e_8pysndlib_6sndlib_BDOUBLE_UNSCALED,
  __pyx_e_8pysndlib_6sndlib_LDOUBLE_UNSCALED,
  __pyx_e_8pysndlib_6sndlib_NUM_SAMPLES
};

/* "pysndlib/sndlib.pxd":23
 *     UNKNOWN_SAMPLE, BSHORT, MULAW, BYTE, BFLOAT, BINT, ALAW, UBYTE, B24INT, BDOUBLE, LSHORT, LINT, LFLOAT, LDOUBLE, UBSHORT, ULSHORT, L24INT, BINTN, LINTN, BFLOAT_UNSCALED, LFLOAT_UNSCALED, BDOUBLE_UNSCALED, LDOUBLE_UNSCALED, NUM_SAMPLES,
 * 
 * cpdef enum Error:             # <<<<<<<<<<<<<<
 *     """
 *     sndlib and clm errors
 */
enum __pyx_t_8pysndlib_6sndlib_Error {
  __pyx_e_8pysndlib_6sndlib_NO_ERROR,
  __pyx_e_8pysndlib_6sndlib_NO_FREQUENCY,
  __pyx_e_8pysndlib_6sndlib_NO_PHASE,
  __pyx_e_8pysndlib_6sndlib_NO_GEN,
  __pyx_e_8pysndlib_6sndlib_NO_LENGTH,
  __pyx_e_8pysndlib_6sndlib_NO_DESCRIBE,
  __pyx_e_8pysndlib_6sndlib_NO_DATA,
  __pyx_e_8pysndlib_6sndlib_NO_SCALER,
  __pyx_e_8pysndlib_6sndlib_MEMORY_ALLOCATION_FAILED,
  __pyx_e_8pysndlib_6sndlib_CANT_OPEN_FILE,
  __pyx_e_8pysndlib_6sndlib_NO_SAMPLE_INPUT,
  __pyx_e_8pysndlib_6sndlib_NO_SAMPLE_OUTPUT,
  __pyx_e_8pysndlib_6sndlib_NO_SUCH_CHANNEL,
  __pyx_e_8pysndlib_6sndlib_NO_FILE_NAME_PROVIDED,
  __pyx_e_8pysndlib_6sndlib_NO_LOCATION,
  __pyx_e_8pysndlib_6sndlib_NO_CHANNEL,
  __pyx_e_8pysndlib_6sndlib_NO_SUCH_FFT_WINDOW,
  __pyx_e_8pysndlib_6sndlib_UNSUPPORTED_SAMPLE_TYPE,
  __pyx_e_8pysndlib_6sndlib_HEADER_READ_FAILED,
  __pyx_e_8pysndlib_6sndlib_UNSUPPORTED_HEADER_TYPE,
  __pyx_e_8pysndlib_6sndlib_FILE_DESCRIPTORS_NOT_INITIALIZED,
  __pyx_e_8pysndlib_6sndlib_NOT_A_SOUND_FILE,
  __pyx_e_8pysndlib_6sndlib_FILE_CLOSED,
  __pyx_e_8pysndlib_6sndlib_WRITE_ERROR,
  __pyx_e_8pysndlib_6sndlib_HEADER_WRITE_FAILED,
  __pyx_e_8pysndlib_6sndlib_CANT_OPEN_TEMP_FILE,
  __pyx_e_8pysndlib_6sndlib_INTERRUPTED,
  __pyx_e_8pysndlib_6sndlib_BAD_ENVELOPE,
  __pyx_e_8pysndlib_6sndlib_AUDIO_CHANNELS_NOT_AVAILABLE,
  __pyx_e_8pysndlib_6sndlib_AUDIO_SRATE_NOT_AVAILABLE,
  __pyx_e_8pysndlib_6sndlib_AUDIO_SAMPLE_TYPE_NOT_AVAILABLE,
  __pyx_e_8pysndlib_6sndlib_AUDIO_NO_INPUT_AVAILABLE,
  __pyx_e_8pysndlib_6sndlib_AUDIO_CONFIGURATION_NOT_AVAILABLE,
  __pyx_e_8pysndlib_6sndlib_AUDIO_WRITE_ERROR,
  __pyx_e_8pysndlib_6sndlib_AUDIO_SIZE_NOT_AVAILABLE,
  __pyx_e_8pysndlib_6sndlib_AUDIO_DEVICE_NOT_AVAILABLE,
  __pyx_e_8pysndlib_6sndlib_AUDIO_CANT_CLOSE,
  __pyx_e_8pysndlib_6sndlib_AUDIO_CANT_OPEN,
  __pyx_e_8pysndlib_6sndlib_AUDIO_READ_ERROR,
  __pyx_e_8pysndlib_6sndlib_AUDIO_CANT_WRITE,
  __pyx_e_8pysndlib_6sndlib_AUDIO_CANT_READ,
  __pyx_e_8pysndlib_6sndlib_AUDIO_NO_READ_PERMISSION,
  __pyx_e_8pysndlib_6sndlib_CANT_CLOSE_FILE,
  __pyx_e_8pysndlib_6sndlib_ARG_OUT_OF_RANGE,
  __pyx_e_8pysndlib_6sndlib_NO_CHANNELS,
  __pyx_e_8pysndlib_6sndlib_NO_HOP,
  __pyx_e_8pysndlib_6sndlib_NO_WIDTH,
  __pyx_e_8pysndlib_6sndlib_NO_FILE_NAME,
  __pyx_e_8pysndlib_6sndlib_NO_RAMP,
  __pyx_e_8pysndlib_6sndlib_NO_RUN,
  __pyx_e_8pysndlib_6sndlib_NO_INCREMENT,
  __pyx_e_8pysndlib_6sndlib_NO_OFFSET,
  __pyx_e_8pysndlib_6sndlib_NO_XCOEFF,
  __pyx_e_8pysndlib_6sndlib_NO_YCOEFF,
  __pyx_e_8pysndlib_6sndlib_NO_XCOEFFS,
  __pyx_e_8pysndlib_6sndlib_NO_YCOEFFS,
  __pyx_e_8pysndlib_6sndlib_NO_RESET,
  __pyx_e_8pysndlib_6sndlib_BAD_SIZE,
  __pyx_e_8pysndlib_6sndlib_CANT_CONVERT,
  __pyx_e_8pysndlib_6sndlib_READ_ERROR,
  __pyx_e_8pysndlib_6sndlib_NO_FEEDFORWARD,
  __pyx_e_8pysndlib_6sndlib_NO_FEEDBACK,
  __pyx_e_8pysndlib_6sndlib_NO_INTERP_TYPE,
  __pyx_e_8pysndlib_6sndlib_NO_POSITION,
  __pyx_e_8pysndlib_6sndlib_NO_ORDER,
  __pyx_e_8pysndlib_6sndlib_NO_COPY,
  __pyx_e_8pysndlib_6sndlib_CANT_TRANSLATE,
  __pyx_e_8pysndlib_6sndlib_NUM_ERRORS
};
struct __pyx_opt_args_8pysndlib_3clm_7mus_any_from_ptr;
struct __pyx_opt_args_8pysndlib_3clm_7mus_any_set_up_data;
struct __pyx_opt_args_8pysndlib_3clm_7mus_any_next;
struct __pyx_opt_args_8pysndlib_3clm_get_srate;
struct __pyx_opt_args_8pysndlib_3clm_random;
struct __pyx_opt_args_8pysndlib_3clm_contrast_enhancement;
struct __pyx_opt_args_8pysndlib_3clm_mus_interpolate;
struct __pyx_opt_args_8pysndlib_3clm_make_fft_window;
struct __pyx_opt_args_8pysndlib_3clm_file2ndarray;
struct __pyx_opt_args_8pysndlib_3clm_ndarray2file;
struct __pyx_opt_args_8pysndlib_3clm_mus_spectrum;
struct __pyx_opt_args_8pysndlib_3clm_spectrum;
struct __pyx_opt_args_8pysndlib_3clm_convolution;
struct __pyx_opt_args_8pysndlib_3clm_mus_convolution;
struct __pyx_opt_args_8pysndlib_3clm_partials2wave;
struct __pyx_opt_args_8pysndlib_3clm_phase_partials2wave;
struct __pyx_opt_args_8pysndlib_3clm_partials2polynomial;
struct __pyx_opt_args_8pysndlib_3clm_oscil;
struct __pyx_opt_args_8pysndlib_3clm_pulsed_env;
struct __pyx_opt_args_8pysndlib_3clm_table_lookup;
struct __pyx_opt_args_8pysndlib_3clm_make_table_lookup_with_env;
struct __pyx_opt_args_8pysndlib_3clm_polywave;
struct __pyx_opt_args_8pysndlib_3clm_polyshape;
struct __pyx_opt_args_8pysndlib_3clm_polyshape_unmodulated;
struct __pyx_opt_args_8pysndlib_3clm_triangle_wave;
struct __pyx_opt_args_8pysndlib_3clm_square_wave;
struct __pyx_opt_args_8pysndlib_3clm_make_sawtooth_wave;
struct __pyx_opt_args_8pysndlib_3clm_sawtooth_wave;
struct __pyx_opt_args_8pysndlib_3clm_pulse_train;
struct __pyx_opt_args_8pysndlib_3clm_ncos;
struct __pyx_opt_args_8pysndlib_3clm_nsin;
struct __pyx_opt_args_8pysndlib_3clm_nrxysin;
struct __pyx_opt_args_8pysndlib_3clm_nrxycos;
struct __pyx_opt_args_8pysndlib_3clm_rxykcos;
struct __pyx_opt_args_8pysndlib_3clm_rxyksin;
struct __pyx_opt_args_8pysndlib_3clm_ssb_am;
struct __pyx_opt_args_8pysndlib_3clm_wave_train;
struct __pyx_opt_args_8pysndlib_3clm_rand;
struct __pyx_opt_args_8pysndlib_3clm_rand_interp;
struct __pyx_opt_args_8pysndlib_3clm_formant;
struct __pyx_opt_args_8pysndlib_3clm_firmant;
struct __pyx_opt_args_8pysndlib_3clm_delay;
struct __pyx_opt_args_8pysndlib_3clm_tap;
struct __pyx_opt_args_8pysndlib_3clm_comb;
struct __pyx_opt_args_8pysndlib_3clm_filtered_comb;
struct __pyx_opt_args_8pysndlib_3clm_notch;
struct __pyx_opt_args_8pysndlib_3clm_all_pass;
struct __pyx_opt_args_8pysndlib_3clm_asymmetric_fm;
struct __pyx_opt_args_8pysndlib_3clm_file2sample;
struct __pyx_opt_args_8pysndlib_3clm_src;
struct __pyx_opt_args_8pysndlib_3clm_out_any;
struct __pyx_opt_args_8pysndlib_3clm_outa;
struct __pyx_opt_args_8pysndlib_3clm_outb;
struct __pyx_opt_args_8pysndlib_3clm_outc;
struct __pyx_opt_args_8pysndlib_3clm_outd;
struct __pyx_opt_args_8pysndlib_3clm_convolve_files;
struct __pyx_opt_args_8pysndlib_3clm_biquad_set_resonance;
struct __pyx_opt_args_8pysndlib_3clm_edot_product;

/* "clm.pxd":14
 * 
 * # --------------- clm enums ---------------- #
 * cpdef enum Interp:             # <<<<<<<<<<<<<<
 *     """
 *     various interpolation types
 */
enum __pyx_t_8pysndlib_3clm_Interp {
  __pyx_e_8pysndlib_3clm_NONE,
  __pyx_e_8pysndlib_3clm_LINEAR,
  __pyx_e_8pysndlib_3clm_SINUSOIDAL,
  __pyx_e_8pysndlib_3clm_ALL_PASS,
  __pyx_e_8pysndlib_3clm_LAGRANGE,
  __pyx_e_8pysndlib_3clm_BEZIER,
  __pyx_e_8pysndlib_3clm_HERMITE
};

/* "clm.pxd":20
 *     NONE, LINEAR,SINUSOIDAL, ALL_PASS, LAGRANGE, BEZIER, HERMITE
 * 
 * cpdef enum Window:             # <<<<<<<<<<<<<<
 *     """
 *     many useful windows
 */
enum __pyx_t_8pysndlib_3clm_Window {
  __pyx_e_8pysndlib_3clm_RECTANGULAR,
  __pyx_e_8pysndlib_3clm_HANN,
  __pyx_e_8pysndlib_3clm_WELCH,
  __pyx_e_8pysndlib_3clm_PARZEN,
  __pyx_e_8pysndlib_3clm_BARTLETT,
  __pyx_e_8pysndlib_3clm_HAMMING,
  __pyx_e_8pysndlib_3clm_BLACKMAN2,
  __pyx_e_8pysndlib_3clm_BLACKMAN3,
  __pyx_e_8pysndlib_3clm_BLACKMAN4,
  __pyx_e_8pysndlib_3clm_EXPONENTIAL,
  __pyx_e_8pysndlib_3clm_RIEMANN,
  __pyx_e_8pysndlib_3clm_KAISER,
  __pyx_e_8pysndlib_3clm_CAUCHY,
  __pyx_e_8pysndlib_3clm_POISSON,
  __pyx_e_8pysndlib_3clm_GAUSSIAN,
  __pyx_e_8pysndlib_3clm_TUKEY,
  __pyx_e_8pysndlib_3clm_DOLPH_CHEBYSHEV,
  __pyx_e_8pysndlib_3clm_HANN_POISSON,
  __pyx_e_8pysndlib_3clm_CONNES,
  __pyx_e_8pysndlib_3clm_SAMARAKI,
  __pyx_e_8pysndlib_3clm_ULTRASPHERICAL,
  __pyx_e_8pysndlib_3clm_BARTLETT_HANN,
  __pyx_e_8pysndlib_3clm_BOHMAN,
  __pyx_e_8pysndlib_3clm_FLAT_TOP,
  __pyx_e_8pysndlib_3clm_BLACKMAN5,
  __pyx_e_8pysndlib_3clm_BLACKMAN6,
  __pyx_e_8pysndlib_3clm_BLACKMAN7,
  __pyx_e_8pysndlib_3clm_BLACKMAN8,
  __pyx_e_8pysndlib_3clm_BLACKMAN9,
  __pyx_e_8pysndlib_3clm_BLACKMAN10,
  __pyx_e_8pysndlib_3clm_RV2,
  __pyx_e_8pysndlib_3clm_RV3,
  __pyx_e_8pysndlib_3clm_RV4,
  __pyx_e_8pysndlib_3clm_MLT_SINE,
  __pyx_e_8pysndlib_3clm_PAPOULIS,
  __pyx_e_8pysndlib_3clm_DPSS,
  __pyx_e_8pysndlib_3clm_SINC
};

/* "clm.pxd":26
 *     RECTANGULAR, HANN, WELCH, PARZEN, BARTLETT, HAMMING, BLACKMAN2, BLACKMAN3, BLACKMAN4, EXPONENTIAL, RIEMANN, KAISER, CAUCHY, POISSON, GAUSSIAN, TUKEY, DOLPH_CHEBYSHEV, HANN_POISSON, CONNES, SAMARAKI, ULTRASPHERICAL, BARTLETT_HANN, BOHMAN, FLAT_TOP, BLACKMAN5, BLACKMAN6, BLACKMAN7, BLACKMAN8, BLACKMAN9, BLACKMAN10, RV2, RV3, RV4, MLT_SINE, PAPOULIS, DPSS, SINC,
 * 
 * cpdef enum Spectrum:             # <<<<<<<<<<<<<<
 *     """
 *     types of normalizations when using the spectrum function. the results are in db if in_db, or linear and normalized to 1.0 normalized, or linear unnormalized raw
 */
enum __pyx_t_8pysndlib_3clm_Spectrum {
  __pyx_e_8pysndlib_3clm_IN_DB,
  __pyx_e_8pysndlib_3clm_NORMALIZED,
  __pyx_e_8pysndlib_3clm_RAW
};

/* "clm.pxd":32
 *     IN_DB, NORMALIZED, RAW
 * 
 * cpdef enum Polynomial:             # <<<<<<<<<<<<<<
 *     """
 *     used for polynomial based gens
 */
enum __pyx_t_8pysndlib_3clm_Polynomial {
  __pyx_e_8pysndlib_3clm_EITHER_KIND,
  __pyx_e_8pysndlib_3clm_FIRST_KIND,
  __pyx_e_8pysndlib_3clm_SECOND_KIND,
  __pyx_e_8pysndlib_3clm_BOTH_KINDS
};

/* "clm.pxd":77
 *     # these should all be considered private
 *     @staticmethod
 *     cdef mus_any from_ptr(cclm.mus_any *_ptr, bint owner=*, cython.int length=*)             # <<<<<<<<<<<<<<
 * 
 *     cpdef cache_append(self, obj)
 */
struct __pyx_opt_args_8pysndlib_3clm_7mus_any_from_ptr {
  int __pyx_n;
  int owner;
  int length;
};

/* "clm.pxd":81
 *     cpdef cache_append(self, obj)
 *     cpdef cache_extend(self, obj)
 *     cpdef set_up_data(self , cython.int length=*)             # <<<<<<<<<<<<<<
 *     cpdef set_up_xcoeffs(self)
 *     cpdef set_up_ycoeffs(self)
 */
struct __pyx_opt_args_8pysndlib_3clm_7mus_any_set_up_data {
  int __pyx_n;
  int length;
};

/* "clm.pxd":88
 *     # these should all be considered public
 *     cpdef mus_reset(self)
 *     cpdef next(self, cython.double arg1=*, cython.double arg2=*)             # <<<<<<<<<<<<<<
 * 
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_7mus_any_next {
  int __pyx_n;
  double arg1;
  double arg2;
};

/* "clm.pxd":97
 * cpdef mus_reset(mus_any obj)
 * cpdef cython.long get_length(obj)
 * cpdef cython.double get_srate(obj=*)             # <<<<<<<<<<<<<<
 * cpdef cython.long get_framples(obj)
 * cpdef cython.long get_channels(obj)
 */
struct __pyx_opt_args_8pysndlib_3clm_get_srate {
  int __pyx_n;
  PyObject *obj;
};

/* "clm.pxd":100
 * cpdef cython.long get_framples(obj)
 * cpdef cython.long get_channels(obj)
 * cpdef cython.double random(cython.double x=*)             # <<<<<<<<<<<<<<
 * cpdef bint is_zero( cython.double x)
 * cpdef bint is_even(n)
 */
struct __pyx_opt_args_8pysndlib_3clm_random {
  int __pyx_n;
  double x;
};

/* "clm.pxd":119
 * cpdef cython.double ring_modulate(cython.double s1, cython.double s2)
 * cpdef cython.double amplitude_modulate(cython.double s1, cython.double s2, cython.double s3)
 * cpdef cython.double contrast_enhancement(cython.double insig, cython.double fm_index=* )             # <<<<<<<<<<<<<<
 * cpdef cython.double dot_product(np.ndarray data1, np.ndarray  data2)
 * cpdef cython.double polynomial(np.ndarray coeffs, cython.double x )
 */
struct __pyx_opt_args_8pysndlib_3clm_contrast_enhancement {
  int __pyx_n;
  double fm_index;
};

/* "clm.pxd":124
 * cpdef cython.double array_interp(np.ndarray  fn, cython.double  x)
 * cpdef cython.double bessi0(cython.double x)
 * cpdef cython.double mus_interpolate(Interp interp_type, cython.double x, np.ndarray table, cython.double y1 =*)             # <<<<<<<<<<<<<<
 * cpdef np.ndarray make_fft_window(Window window_type, cython.int size, cython.double beta=* , cython.double alpha=*)
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_mus_interpolate {
  int __pyx_n;
  double y1;
};

/* "clm.pxd":125
 * cpdef cython.double bessi0(cython.double x)
 * cpdef cython.double mus_interpolate(Interp interp_type, cython.double x, np.ndarray table, cython.double y1 =*)
 * cpdef np.ndarray make_fft_window(Window window_type, cython.int size, cython.double beta=* , cython.double alpha=*)             # <<<<<<<<<<<<<<
 * 
 * cpdef file2ndarray(str filename, channel=*, beg=*, size=*)
 */
struct __pyx_opt_args_8pysndlib_3clm_make_fft_window {
  int __pyx_n;
  double beta;
  double alpha;
};

/* "clm.pxd":127
 * cpdef np.ndarray make_fft_window(Window window_type, cython.int size, cython.double beta=* , cython.double alpha=*)
 * 
 * cpdef file2ndarray(str filename, channel=*, beg=*, size=*)             # <<<<<<<<<<<<<<
 * cpdef ndarray2file(str filename, np.ndarray arr, size=*, sr=*, sample_type=*, header_type=*, comment=* )
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_file2ndarray {
  int __pyx_n;
  PyObject *channel;
  PyObject *beg;
  PyObject *size;
};

/* "clm.pxd":128
 * 
 * cpdef file2ndarray(str filename, channel=*, beg=*, size=*)
 * cpdef ndarray2file(str filename, np.ndarray arr, size=*, sr=*, sample_type=*, header_type=*, comment=* )             # <<<<<<<<<<<<<<
 * 
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_ndarray2file {
  int __pyx_n;
  PyObject *size;
  PyObject *sr;
  PyObject *sample_type;
  PyObject *header_type;
  PyObject *comment;
};

/* "clm.pxd":139
 * cpdef np.ndarray mus_polar2rectangular(np.ndarray rdat,  np.ndarray idat)
 * cpdef tuple polar2rectangular(np.ndarray rdat, np.ndarray idat)
 * cpdef np.ndarray mus_spectrum(np.ndarray rdat, np.ndarray  idat, np.ndarray  window, Spectrum norm_type=*)             # <<<<<<<<<<<<<<
 * cpdef tuple spectrum(np.ndarray rdat,np.ndarray idat,np.ndarray window, Spectrum norm_type=*)
 * cpdef np.ndarray mus_spectrum(np.ndarray rdat, np.ndarray  idat, np.ndarray  window, Spectrum norm_type=*)
 */
struct __pyx_opt_args_8pysndlib_3clm_mus_spectrum {
  int __pyx_n;
  enum __pyx_t_8pysndlib_3clm_Spectrum norm_type;
};

/* "clm.pxd":140
 * cpdef tuple polar2rectangular(np.ndarray rdat, np.ndarray idat)
 * cpdef np.ndarray mus_spectrum(np.ndarray rdat, np.ndarray  idat, np.ndarray  window, Spectrum norm_type=*)
 * cpdef tuple spectrum(np.ndarray rdat,np.ndarray idat,np.ndarray window, Spectrum norm_type=*)             # <<<<<<<<<<<<<<
 * cpdef np.ndarray mus_spectrum(np.ndarray rdat, np.ndarray  idat, np.ndarray  window, Spectrum norm_type=*)
 * cpdef np.ndarray convolution(np.ndarray rl1, np.ndarray  rl2, fft_size = *)
 */
struct __pyx_opt_args_8pysndlib_3clm_spectrum {
  int __pyx_n;
  enum __pyx_t_8pysndlib_3clm_Spectrum norm_type;
};

/* "clm.pxd":142
 * cpdef tuple spectrum(np.ndarray rdat,np.ndarray idat,np.ndarray window, Spectrum norm_type=*)
 * cpdef np.ndarray mus_spectrum(np.ndarray rdat, np.ndarray  idat, np.ndarray  window, Spectrum norm_type=*)
 * cpdef np.ndarray convolution(np.ndarray rl1, np.ndarray  rl2, fft_size = *)             # <<<<<<<<<<<<<<
 * cpdef np.ndarray mus_convolution(np.ndarray  rl1, np.ndarray  rl2, fft_size=*)
 * cpdef np.ndarray autocorrelate(np.ndarray data)
 */
struct __pyx_opt_args_8pysndlib_3clm_convolution {
  int __pyx_n;
  PyObject *fft_size;
};

/* "clm.pxd":143
 * cpdef np.ndarray mus_spectrum(np.ndarray rdat, np.ndarray  idat, np.ndarray  window, Spectrum norm_type=*)
 * cpdef np.ndarray convolution(np.ndarray rl1, np.ndarray  rl2, fft_size = *)
 * cpdef np.ndarray mus_convolution(np.ndarray  rl1, np.ndarray  rl2, fft_size=*)             # <<<<<<<<<<<<<<
 * cpdef np.ndarray autocorrelate(np.ndarray data)
 * cpdef np.ndarray mus_autocorrelate(np.ndarray data)
 */
struct __pyx_opt_args_8pysndlib_3clm_mus_convolution {
  int __pyx_n;
  PyObject *fft_size;
};

/* "clm.pxd":150
 * cpdef np.ndarray mus_cepstrum( np.ndarray data)
 * cpdef np.ndarray cepstrum(np.ndarray data)
 * cpdef np.ndarray partials2wave(partials, np.ndarray wave=*, table_size=*, bint norm=* )             # <<<<<<<<<<<<<<
 * cpdef np.ndarray phase_partials2wave(partials, np.ndarray wave=*, table_size=*, bint norm=* )
 * cpdef np.ndarray partials2polynomial(partials, Polynomial kind=*)
 */
struct __pyx_opt_args_8pysndlib_3clm_partials2wave {
  int __pyx_n;
  PyArrayObject *wave;
  PyObject *table_size;
  int norm;
};

/* "clm.pxd":151
 * cpdef np.ndarray cepstrum(np.ndarray data)
 * cpdef np.ndarray partials2wave(partials, np.ndarray wave=*, table_size=*, bint norm=* )
 * cpdef np.ndarray phase_partials2wave(partials, np.ndarray wave=*, table_size=*, bint norm=* )             # <<<<<<<<<<<<<<
 * cpdef np.ndarray partials2polynomial(partials, Polynomial kind=*)
 * cpdef np.ndarray normalize_partials(partials)
 */
struct __pyx_opt_args_8pysndlib_3clm_phase_partials2wave {
  int __pyx_n;
  PyArrayObject *wave;
  PyObject *table_size;
  int norm;
};

/* "clm.pxd":152
 * cpdef np.ndarray partials2wave(partials, np.ndarray wave=*, table_size=*, bint norm=* )
 * cpdef np.ndarray phase_partials2wave(partials, np.ndarray wave=*, table_size=*, bint norm=* )
 * cpdef np.ndarray partials2polynomial(partials, Polynomial kind=*)             # <<<<<<<<<<<<<<
 * cpdef np.ndarray normalize_partials(partials)
 * cpdef cython.double chebyshev_tu_sum(cython.double x, tcoeffs, ucoeffs)
 */
struct __pyx_opt_args_8pysndlib_3clm_partials2polynomial {
  int __pyx_n;
  enum __pyx_t_8pysndlib_3clm_Polynomial kind;
};

/* "clm.pxd":160
 * # ---------------- clm gen ---------------- #
 * 
 * cpdef cython.double oscil(mus_any gen, cython.double fm=*, cython.double pm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double oscil_unmodulated(mus_any gen)
 * cpdef bint is_oscil(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_oscil {
  int __pyx_n;
  double fm;
  double pm;
};

/* "clm.pxd":176
 * cpdef cython.double env_any(mus_any gen , connection_function)
 * 
 * cpdef cython.double pulsed_env(mus_any gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double pulsed_env_unmodulated(mus_any gen)
 * cpdef bint is_pulsed_env(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_pulsed_env {
  int __pyx_n;
  double fm;
};

/* "clm.pxd":180
 * cpdef bint is_pulsed_env(mus_any gen)
 * 
 * cpdef cython.double table_lookup(mus_any gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double table_lookup_unmodulated(mus_any gen)
 * cpdef bint is_table_lookup(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_table_lookup {
  int __pyx_n;
  double fm;
};

/* "clm.pxd":183
 * cpdef cython.double table_lookup_unmodulated(mus_any gen)
 * cpdef bint is_table_lookup(mus_any gen)
 * cpdef mus_any make_table_lookup_with_env(cython.double frequency, envelope, size= *)             # <<<<<<<<<<<<<<
 * 
 * cpdef cython.double polywave(mus_any gen, cython.double fm=*)
 */
struct __pyx_opt_args_8pysndlib_3clm_make_table_lookup_with_env {
  int __pyx_n;
  PyObject *size;
};

/* "clm.pxd":185
 * cpdef mus_any make_table_lookup_with_env(cython.double frequency, envelope, size= *)
 * 
 * cpdef cython.double polywave(mus_any gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double polywave_unmodulated(mus_any gen)
 * cpdef bint is_polywave(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_polywave {
  int __pyx_n;
  double fm;
};

/* "clm.pxd":189
 * cpdef bint is_polywave(mus_any gen)
 * 
 * cpdef cython.double polyshape(mus_any gen, cython.double index=*, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double polyshape_unmodulated(mus_any gen, cython.double index=*)
 * cpdef bint is_polyshape(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_polyshape {
  int __pyx_n;
  double index;
  double fm;
};

/* "clm.pxd":190
 * 
 * cpdef cython.double polyshape(mus_any gen, cython.double index=*, cython.double fm=*)
 * cpdef cython.double polyshape_unmodulated(mus_any gen, cython.double index=*)             # <<<<<<<<<<<<<<
 * cpdef bint is_polyshape(mus_any gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_polyshape_unmodulated {
  int __pyx_n;
  double index;
};

/* "clm.pxd":193
 * cpdef bint is_polyshape(mus_any gen)
 * 
 * cpdef cython.double triangle_wave(mus_any gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double triangle_wave_unmodulated(mus_any gen)
 * cpdef bint is_triangle_wave(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_triangle_wave {
  int __pyx_n;
  double fm;
};

/* "clm.pxd":197
 * cpdef bint is_triangle_wave(mus_any gen)
 * 
 * cpdef cython.double square_wave(mus_any gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef bint is_square_wave(mus_any gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_square_wave {
  int __pyx_n;
  double fm;
};

/* "clm.pxd":200
 * cpdef bint is_square_wave(mus_any gen)
 * 
 * cpdef mus_any make_sawtooth_wave(cython.double frequency, cython.double amplitude=*, cython.double phase=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double sawtooth_wave(mus_any gen, cython.double fm =*)
 * cpdef bint is_sawtooth_wave(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_make_sawtooth_wave {
  int __pyx_n;
  double amplitude;
  double phase;
};

/* "clm.pxd":201
 * 
 * cpdef mus_any make_sawtooth_wave(cython.double frequency, cython.double amplitude=*, cython.double phase=*)
 * cpdef cython.double sawtooth_wave(mus_any gen, cython.double fm =*)             # <<<<<<<<<<<<<<
 * cpdef bint is_sawtooth_wave(mus_any gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_sawtooth_wave {
  int __pyx_n;
  double fm;
};

/* "clm.pxd":204
 * cpdef bint is_sawtooth_wave(mus_any gen)
 * 
 * cpdef cython.double pulse_train(mus_any gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double pulse_train_unmodulated(mus_any gen)
 * cpdef bint is_pulse_train(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_pulse_train {
  int __pyx_n;
  double fm;
};

/* "clm.pxd":208
 * cpdef bint is_pulse_train(mus_any gen)
 * 
 * cpdef cython.double ncos(mus_any gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef bint is_ncos(mus_any gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_ncos {
  int __pyx_n;
  double fm;
};

/* "clm.pxd":211
 * cpdef bint is_ncos(mus_any gen)
 * 
 * cpdef cython.double nsin(mus_any gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef bint is_nsin(mus_any gen )
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_nsin {
  int __pyx_n;
  double fm;
};

/* "clm.pxd":214
 * cpdef bint is_nsin(mus_any gen )
 * 
 * cpdef cython.double nrxysin(mus_any gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef bint is_nrxysin(mus_any gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_nrxysin {
  int __pyx_n;
  double fm;
};

/* "clm.pxd":217
 * cpdef bint is_nrxysin(mus_any gen)
 * 
 * cpdef cython.double nrxycos(mus_any gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef bint is_nrxycos(mus_any gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_nrxycos {
  int __pyx_n;
  double fm;
};

/* "clm.pxd":220
 * cpdef bint is_nrxycos(mus_any gen)
 * 
 * cpdef cython.double rxykcos(mus_any gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef bint is_rxykcos(mus_any gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_rxykcos {
  int __pyx_n;
  double fm;
};

/* "clm.pxd":223
 * cpdef bint is_rxykcos(mus_any gen)
 * 
 * cpdef cython.double rxyksin(mus_any gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef bint is_rxyksin(mus_any gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_rxyksin {
  int __pyx_n;
  double fm;
};

/* "clm.pxd":226
 * cpdef bint is_rxyksin(mus_any gen)
 * 
 * cpdef cython.double ssb_am(mus_any gen, cython.double insig, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double ssb_am_unmodulated(mus_any gen, cython.double insig)
 * cpdef bint is_ssb_am(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_ssb_am {
  int __pyx_n;
  double fm;
};

/* "clm.pxd":230
 * cpdef bint is_ssb_am(mus_any gen)
 * 
 * cpdef cython.double wave_train(mus_any gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double wave_train_unmodulated(mus_any gen)
 * cpdef bint is_wave_train(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_wave_train {
  int __pyx_n;
  double fm;
};

/* "clm.pxd":234
 * cpdef bint is_wave_train(mus_any gen)
 * 
 * cpdef cython.double rand(mus_any gen, cython.double sweep=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double rand_unmodulated(mus_any gen)
 * cpdef bint is_rand(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_rand {
  int __pyx_n;
  double sweep;
};

/* "clm.pxd":238
 * cpdef bint is_rand(mus_any gen)
 * 
 * cpdef rand_interp(mus_any gen, cython.double sweep=*)             # <<<<<<<<<<<<<<
 * cpdef rand_interp_unmodulated(mus_any gen)
 * cpdef bint is_rand_interp(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_rand_interp {
  int __pyx_n;
  double sweep;
};

/* "clm.pxd":254
 * cpdef bint is_two_zero(mus_any gen)
 * 
 * cpdef cython.double formant(mus_any gen, cython.double insig, cython.double radians=*)             # <<<<<<<<<<<<<<
 * cpdef bint is_formant(mus_any gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_formant {
  int __pyx_n;
  double radians;
};

/* "clm.pxd":260
 * cpdef bint is_formant_bank(mus_any gen)
 * 
 * cpdef firmant(mus_any gen, cython.double insig, cython.double radians=*)             # <<<<<<<<<<<<<<
 * cpdef is_firmant(mus_any gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_firmant {
  int __pyx_n;
  double radians;
};

/* "clm.pxd":272
 * cpdef bint is_iir_filter(mus_any gen)
 * 
 * cpdef cython.double delay(mus_any gen, cython.double insig, cython.double pm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double delay_unmodulated(mus_any gen, cython.double insig)
 * cpdef bint is_delay(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_delay {
  int __pyx_n;
  double pm;
};

/* "clm.pxd":275
 * cpdef cython.double delay_unmodulated(mus_any gen, cython.double insig)
 * cpdef bint is_delay(mus_any gen)
 * cpdef cython.double tap(mus_any gen, offset=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double tap_unmodulated(mus_any gen)
 * cpdef bint is_tap(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_tap {
  int __pyx_n;
  PyObject *offset;
};

/* "clm.pxd":280
 * cpdef cython.double delay_tick(mus_any gen, cython.double insig )
 * 
 * cpdef cython.double comb(mus_any gen, cython.double insig, pm=*)             # <<<<<<<<<<<<<<
 * cpdef bint is_comb(mus_any gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_comb {
  int __pyx_n;
  PyObject *pm;
};

/* "clm.pxd":286
 * cpdef bint is_comb_bank(mus_any gen)
 * 
 * cpdef cython.double filtered_comb(mus_any gen, cython.double insig, cython.double pm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double filtered_comb_unmodulated(mus_any gen, cython.double insig)
 * cpdef bint is_filtered_comb(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_filtered_comb {
  int __pyx_n;
  double pm;
};

/* "clm.pxd":293
 * cpdef bint is_filtered_comb_bank(mus_any gen)
 * 
 * cpdef cython.double notch(mus_any gen, cython.double insig, cython.double pm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double notch_unmodulated(mus_any gen, cython.double insig)
 * cpdef bint is_notch(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_notch {
  int __pyx_n;
  double pm;
};

/* "clm.pxd":297
 * cpdef bint is_notch(mus_any gen)
 * 
 * cpdef cython.double all_pass(mus_any gen, cython.double insig, pm=*)             # <<<<<<<<<<<<<<
 * cpdef is_all_pass(mus_any gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_all_pass {
  int __pyx_n;
  PyObject *pm;
};

/* "clm.pxd":315
 * cpdef is_moving_norm(mus_any gen)
 * 
 * cpdef cython.double asymmetric_fm(mus_any gen, cython.double index, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double asymmetric_fm_unmodulated(mus_any gen, cython.double index)
 * cpdef bint is_asymmetric_fm(mus_any gen)
 */
struct __pyx_opt_args_8pysndlib_3clm_asymmetric_fm {
  int __pyx_n;
  double fm;
};

/* "clm.pxd":319
 * cpdef bint is_asymmetric_fm(mus_any gen)
 * 
 * cpdef cython.double file2sample(mus_any gen, cython.long loc, cython.int chan=*)             # <<<<<<<<<<<<<<
 * cpdef bint is_file2sample(mus_any gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_file2sample {
  int __pyx_n;
  int chan;
};

/* "clm.pxd":335
 * cpdef is_readin(mus_any gen)
 * 
 * cpdef cython.double src(mus_any gen, cython.double sr_change=*)             # <<<<<<<<<<<<<<
 * cpdef bint is_src(mus_any gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_src {
  int __pyx_n;
  double sr_change;
};

/* "clm.pxd":352
 * cpdef np.ndarray phase_vocoder_phase_increments(mus_any gen)
 * 
 * cpdef out_any(cython.long loc, cython.double data, cython.int channel, output=*)             # <<<<<<<<<<<<<<
 * cpdef outa(cython.long loc, cython.double data, output=*)
 * cpdef outb(cython.long loc, cython.double data, output=*)
 */
struct __pyx_opt_args_8pysndlib_3clm_out_any {
  int __pyx_n;
  PyObject *output;
};

/* "clm.pxd":353
 * 
 * cpdef out_any(cython.long loc, cython.double data, cython.int channel, output=*)
 * cpdef outa(cython.long loc, cython.double data, output=*)             # <<<<<<<<<<<<<<
 * cpdef outb(cython.long loc, cython.double data, output=*)
 * cpdef outc(cython.long loc, cython.double data, output=*)
 */
struct __pyx_opt_args_8pysndlib_3clm_outa {
  int __pyx_n;
  PyObject *output;
};

/* "clm.pxd":354
 * cpdef out_any(cython.long loc, cython.double data, cython.int channel, output=*)
 * cpdef outa(cython.long loc, cython.double data, output=*)
 * cpdef outb(cython.long loc, cython.double data, output=*)             # <<<<<<<<<<<<<<
 * cpdef outc(cython.long loc, cython.double data, output=*)
 * cpdef outd(cython.long loc, cython.double data, output=*)
 */
struct __pyx_opt_args_8pysndlib_3clm_outb {
  int __pyx_n;
  PyObject *output;
};

/* "clm.pxd":355
 * cpdef outa(cython.long loc, cython.double data, output=*)
 * cpdef outb(cython.long loc, cython.double data, output=*)
 * cpdef outc(cython.long loc, cython.double data, output=*)             # <<<<<<<<<<<<<<
 * cpdef outd(cython.long loc, cython.double data, output=*)
 * cpdef in_any(cython.long loc, cython.int channel, inp)
 */
struct __pyx_opt_args_8pysndlib_3clm_outc {
  int __pyx_n;
  PyObject *output;
};

/* "clm.pxd":356
 * cpdef outb(cython.long loc, cython.double data, output=*)
 * cpdef outc(cython.long loc, cython.double data, output=*)
 * cpdef outd(cython.long loc, cython.double data, output=*)             # <<<<<<<<<<<<<<
 * cpdef in_any(cython.long loc, cython.int channel, inp)
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_outd {
  int __pyx_n;
  PyObject *output;
};

/* "clm.pxd":372
 * cpdef move_locsig(mus_any gen,cython.double degree, cython.double distance)
 * 
 * cpdef convolve_files(str file1, str file2, cython.double maxamp=*, str outputfile=*, sample_type=*, header_type=*)             # <<<<<<<<<<<<<<
 * 
 * 
 */
struct __pyx_opt_args_8pysndlib_3clm_convolve_files {
  int __pyx_n;
  double maxamp;
  PyObject *outputfile;
  PyObject *sample_type;
  PyObject *header_type;
};

/* "clm.pxd":380
 * cpdef cython.double biquad(mus_any gen, cython.double insig)
 * cpdef bint is_biquad(mus_any gen)
 * cpdef biquad_set_resonance(mus_any gen, cython.double fc, cython.double radius, bint normalize=*)             # <<<<<<<<<<<<<<
 * cpdef biquad_set_equal_gain_zeroes(mus_any gen)
 * cpdef cython.double bes_j0(cython.double x)
 */
struct __pyx_opt_args_8pysndlib_3clm_biquad_set_resonance {
  int __pyx_n;
  int normalize;
};

/* "clm.pxd":390
 * 
 * 
 * cpdef cython.doublecomplex edot_product(cython.doublecomplex freq, np.ndarray data, size=*)             # <<<<<<<<<<<<<<
 * 
 * #cpdef mus_any make_oscil(cython.double frequency=*,cython.double  initial_phase = *)
 */
struct __pyx_opt_args_8pysndlib_3clm_edot_product {
  int __pyx_n;
  PyObject *size;
};
struct __pyx_opt_args_8pysndlib_10generators_12CLMGenerator_next;
struct __pyx_opt_args_8pysndlib_10generators_nssb;
struct __pyx_opt_args_8pysndlib_10generators_nxysin;
struct __pyx_opt_args_8pysndlib_10generators_nxycos;
struct __pyx_opt_args_8pysndlib_10generators_nxy1cos;
struct __pyx_opt_args_8pysndlib_10generators_nxy1sin;
struct __pyx_opt_args_8pysndlib_10generators_noddcos;
struct __pyx_opt_args_8pysndlib_10generators_noddssb;
struct __pyx_opt_args_8pysndlib_10generators_ncos2;
struct __pyx_opt_args_8pysndlib_10generators_ncos4;
struct __pyx_opt_args_8pysndlib_10generators_npcos;
struct __pyx_opt_args_8pysndlib_10generators_ncos5;
struct __pyx_opt_args_8pysndlib_10generators_nsin5;
struct __pyx_opt_args_8pysndlib_10generators_nrsin;
struct __pyx_opt_args_8pysndlib_10generators_nrcos;
struct __pyx_opt_args_8pysndlib_10generators_5Nrssb_call_interp;
struct __pyx_opt_args_8pysndlib_10generators_nrssb;
struct __pyx_opt_args_8pysndlib_10generators_nrssb_interp;
struct __pyx_opt_args_8pysndlib_10generators_5Nkssb_call_interp;
struct __pyx_opt_args_8pysndlib_10generators_nkssb;
struct __pyx_opt_args_8pysndlib_10generators_nkssb_interp;
struct __pyx_opt_args_8pysndlib_10generators_nsincos;
struct __pyx_opt_args_8pysndlib_10generators_n1cos;
struct __pyx_opt_args_8pysndlib_10generators_npos1cos;
struct __pyx_opt_args_8pysndlib_10generators_npos3cos;
struct __pyx_opt_args_8pysndlib_10generators_rcos;
struct __pyx_opt_args_8pysndlib_10generators_4Rssb_call_interp;
struct __pyx_opt_args_8pysndlib_10generators_rssb;
struct __pyx_opt_args_8pysndlib_10generators_rssb_interp;
struct __pyx_opt_args_8pysndlib_10generators_rxysin;
struct __pyx_opt_args_8pysndlib_10generators_rxycos;
struct __pyx_opt_args_8pysndlib_10generators_10SafeRxycos_clamp_rxycos_r;
struct __pyx_opt_args_8pysndlib_10generators_safe_rxycos;
struct __pyx_opt_args_8pysndlib_10generators_ercos;
struct __pyx_opt_args_8pysndlib_10generators_erssb;
struct __pyx_opt_args_8pysndlib_10generators_r2cos;
struct __pyx_opt_args_8pysndlib_10generators_r2ssb;
struct __pyx_opt_args_8pysndlib_10generators_eoddcos;
struct __pyx_opt_args_8pysndlib_10generators_rkcos;
struct __pyx_opt_args_8pysndlib_10generators_rksin;
struct __pyx_opt_args_8pysndlib_10generators_rkfcos;
struct __pyx_opt_args_8pysndlib_10generators_rkfssb;
struct __pyx_opt_args_8pysndlib_10generators_r2kfcos;
struct __pyx_opt_args_8pysndlib_10generators_k2sin;
struct __pyx_opt_args_8pysndlib_10generators_k2cos;
struct __pyx_opt_args_8pysndlib_10generators_k2ssb;
struct __pyx_opt_args_8pysndlib_10generators_dblsum;
struct __pyx_opt_args_8pysndlib_10generators_rkoddssb;
struct __pyx_opt_args_8pysndlib_10generators_krksin;
struct __pyx_opt_args_8pysndlib_10generators_absin;
struct __pyx_opt_args_8pysndlib_10generators_abcos;
struct __pyx_opt_args_8pysndlib_10generators_r2k2cos;
struct __pyx_opt_args_8pysndlib_10generators_blsaw;
struct __pyx_opt_args_8pysndlib_10generators_5Asyfm_next_i;
struct __pyx_opt_args_8pysndlib_10generators_5Asyfm_next_j;
struct __pyx_opt_args_8pysndlib_10generators_asyfm;
struct __pyx_opt_args_8pysndlib_10generators_asyfm_i;
struct __pyx_opt_args_8pysndlib_10generators_asyfm_j;
struct __pyx_opt_args_8pysndlib_10generators_bess;
struct __pyx_opt_args_8pysndlib_10generators_jjcos;
struct __pyx_opt_args_8pysndlib_10generators_j0evencos;
struct __pyx_opt_args_8pysndlib_10generators_j2cos;
struct __pyx_opt_args_8pysndlib_10generators_jpcos;
struct __pyx_opt_args_8pysndlib_10generators_jncos;
struct __pyx_opt_args_8pysndlib_10generators_j0j1cos;
struct __pyx_opt_args_8pysndlib_10generators_jycos;
struct __pyx_opt_args_8pysndlib_10generators_jcos;
struct __pyx_opt_args_8pysndlib_10generators_blackman;
struct __pyx_opt_args_8pysndlib_10generators_fmssb;
struct __pyx_opt_args_8pysndlib_10generators_k3sin;
struct __pyx_opt_args_8pysndlib_10generators_izcos;
struct __pyx_opt_args_8pysndlib_10generators_adjustable_square_wave;
struct __pyx_opt_args_8pysndlib_10generators_adjustable_triangle_wave;
struct __pyx_opt_args_8pysndlib_10generators_adjustable_sawtooth_wave;
struct __pyx_opt_args_8pysndlib_10generators_adjustable_oscil;
struct __pyx_opt_args_8pysndlib_10generators_make_table_lookup_with_env;
struct __pyx_opt_args_8pysndlib_10generators_make_wave_train_with_env;
struct __pyx_opt_args_8pysndlib_10generators_round_interp;
struct __pyx_opt_args_8pysndlib_10generators_nchoosekcos;
struct __pyx_opt_args_8pysndlib_10generators_sinc_train;
struct __pyx_opt_args_8pysndlib_10generators_pink_noise;
struct __pyx_opt_args_8pysndlib_10generators_brown_noise;
struct __pyx_opt_args_8pysndlib_10generators_moving_sum;
struct __pyx_opt_args_8pysndlib_10generators_moving_variance;
struct __pyx_opt_args_8pysndlib_10generators_moving_rms;
struct __pyx_opt_args_8pysndlib_10generators_moving_length;
struct __pyx_opt_args_8pysndlib_10generators_weighted_moving_average;
struct __pyx_opt_args_8pysndlib_10generators_exponentially_weighted_moving_average;
struct __pyx_opt_args_8pysndlib_10generators_polyoid;
struct __pyx_opt_args_8pysndlib_10generators_tanhsin;

/* "generators.pxd":27
 * 
 * cdef class CLMGenerator:
 *     cpdef cython.double next(self, cython.double  fm=*)             # <<<<<<<<<<<<<<
 * 
 * # --------------- nssb ---------------- #
 */
struct __pyx_opt_args_8pysndlib_10generators_12CLMGenerator_next {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":37
 * 
 * 
 * cpdef cython.double nssb (Nssb gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_nssb(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_nssb {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":48
 *     cdef public cython.double norm
 * 
 * cpdef cython.double nxysin(Nxysin gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_nxysin(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_nxysin {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":58
 *     cdef public cython.double angle
 * 
 * cpdef cython.double nxycos(Nxycos gen, cython.double fm =*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_nxycos(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_nxycos {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":68
 *     cdef public cython.double angle
 * 
 * cpdef cython.double nxy1cos(Nxy1cos gen, cython.double fm =*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_nxy1cos(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_nxy1cos {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":78
 *     cdef public cython.double angle
 * 
 * cpdef cython.double nxy1sin(Nxy1sin gen, cython.double fm =*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_nxy1sin(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_nxy1sin {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":89
 * 
 * 
 * cpdef cython.double noddcos(Noddcos gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_noddcos(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_noddcos {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":99
 *     cdef public cython.double angle
 * 
 * cpdef cython.double noddssb(Noddssb gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_noddssb(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_noddssb {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":108
 *     cdef public cython.double angle
 * 
 * cpdef cython.double ncos2(Ncos2 gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_ncos2(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_ncos2 {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":115
 *     pass
 * 
 * cpdef cython.double ncos4(Ncos4 gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_ncos4(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_ncos4 {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":124
 *     cdef public cython.double angle
 * 
 * cpdef cython.double npcos(Npcos gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_npcos(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_npcos {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":133
 *     cdef public cython.double angle
 * 
 * cpdef cython.double ncos5(Ncos5 gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_ncos5(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_ncos5 {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":145
 *     cdef public cython.double norm
 * 
 * cpdef cython.double nsin5(Nsin5 gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_nsin5(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_nsin5 {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":150
 * # --------------- nrsin ---------------- #
 * 
 * cpdef cython.double nrsin(clm.mus_any gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_nrsin(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_nrsin {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":167
 *     cdef public cython.bint trouble
 * 
 * cpdef cython.double nrcos(Nrcos gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_nrcos(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_nrcos {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":181
 *     cdef public cython.double rn1
 *     cdef public cython.double norm
 *     cpdef call_interp(self, cython.double fm =*, cython.double interp=*)             # <<<<<<<<<<<<<<
 * 
 * cpdef cython.double nrssb(Nrssb gen, cython.double fm=*)
 */
struct __pyx_opt_args_8pysndlib_10generators_5Nrssb_call_interp {
  int __pyx_n;
  double fm;
  double interp;
};

/* "generators.pxd":183
 *     cpdef call_interp(self, cython.double fm =*, cython.double interp=*)
 * 
 * cpdef cython.double nrssb(Nrssb gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double nrssb_interp(Nrssb gen, cython.double fm=*,cython.double interp =* )
 * cpdef cython.bint is_nrssb(gen)
 */
struct __pyx_opt_args_8pysndlib_10generators_nrssb {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":184
 * 
 * cpdef cython.double nrssb(Nrssb gen, cython.double fm=*)
 * cpdef cython.double nrssb_interp(Nrssb gen, cython.double fm=*,cython.double interp =* )             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_nrssb(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_nrssb_interp {
  int __pyx_n;
  double fm;
  double interp;
};

/* "generators.pxd":195
 *     cdef public cython.double norm
 *     cdef public cython.double interp
 *     cpdef cython.double call_interp(self, cython.double fm=*, cython.double interp=*)             # <<<<<<<<<<<<<<
 * 
 * cpdef cython.double nkssb(Nkssb gen, cython.double fm=*)
 */
struct __pyx_opt_args_8pysndlib_10generators_5Nkssb_call_interp {
  int __pyx_n;
  double fm;
  double interp;
};

/* "generators.pxd":197
 *     cpdef cython.double call_interp(self, cython.double fm=*, cython.double interp=*)
 * 
 * cpdef cython.double nkssb(Nkssb gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double nkssb_interp(Nkssb gen, cython.double fm=*, cython.double interp=*)
 * cpdef cython.bint is_nkssb(gen)
 */
struct __pyx_opt_args_8pysndlib_10generators_nkssb {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":198
 * 
 * cpdef cython.double nkssb(Nkssb gen, cython.double fm=*)
 * cpdef cython.double nkssb_interp(Nkssb gen, cython.double fm=*, cython.double interp=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_nkssb(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_nkssb_interp {
  int __pyx_n;
  double fm;
  double interp;
};

/* "generators.pxd":210
 *     cdef public cython.double norm
 * 
 * cpdef cython.double nsincos(Nsincos gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_nsincos(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_nsincos {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":220
 *     cdef public cython.double angle
 * 
 * cpdef cython.double n1cos(N1cos gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_n1cos(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_n1cos {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":230
 *     cdef public cython.double angle
 * 
 * cpdef cython.double npos1cos(Npos1cos gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_npos1cos(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_npos1cos {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":238
 *     cdef public cython.double n
 *     cdef public cython.double angle
 * cpdef cython.double npos3cos(Npos3cos gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_npos3cos(gen)
 * # --------------- rcos ---------------- #
 */
struct __pyx_opt_args_8pysndlib_10generators_npos3cos {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":251
 *     cdef public cython.double norm
 * 
 * cpdef cython.double rcos(Rcos gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_rcos(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_rcos {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":264
 * 
 * 
 *     cpdef cython.double call_interp(self, cython.double fm=*, cython.double interp=*)             # <<<<<<<<<<<<<<
 * 
 * cpdef cython.double rssb(Rssb gen, cython.double fm=*)
 */
struct __pyx_opt_args_8pysndlib_10generators_4Rssb_call_interp {
  int __pyx_n;
  double fm;
  double interp;
};

/* "generators.pxd":266
 *     cpdef cython.double call_interp(self, cython.double fm=*, cython.double interp=*)
 * 
 * cpdef cython.double rssb(Rssb gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double rssb_interp(Rssb gen, cython.double fm=*, cython.double interp=*)
 * cpdef cython.bint is_rssb(gen)
 */
struct __pyx_opt_args_8pysndlib_10generators_rssb {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":267
 * 
 * cpdef cython.double rssb(Rssb gen, cython.double fm=*)
 * cpdef cython.double rssb_interp(Rssb gen, cython.double fm=*, cython.double interp=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_rssb(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_rssb_interp {
  int __pyx_n;
  double fm;
  double interp;
};

/* "generators.pxd":280
 *     cdef public cython.double r2
 * 
 * cpdef cython.double rxysin(Rxysin gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_rxysin(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_rxysin {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":294
 *     cdef public cython.double norm
 * 
 * cpdef cython.double rxycos(Rxycos gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_rxycos(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_rxycos {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":306
 *     cdef public cython.double angle
 *     cdef public cython.double cutoff
 *     cpdef clamp_rxycos_r(self, cython.double fm=*)             # <<<<<<<<<<<<<<
 * 
 * cpdef cython.double safe_rxycos(SafeRxycos gen, cython.double fm=*)
 */
struct __pyx_opt_args_8pysndlib_10generators_10SafeRxycos_clamp_rxycos_r {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":308
 *     cpdef clamp_rxycos_r(self, cython.double fm=*)
 * 
 * cpdef cython.double safe_rxycos(SafeRxycos gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_safe_rxycos(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_safe_rxycos {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":321
 *     cdef public clm.mus_any osc
 * 
 * cpdef cython.double ercos(Ercos gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_ercos(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_ercos {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":331
 *     cdef public cython.double angle
 * 
 * cpdef cython.double erssb(Erssb gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_erssb(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_erssb {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":345
 *     cdef public cython.double angle
 * 
 * cpdef cython.double r2cos(R2cos gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_r2cos(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_r2cos {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":357
 *     cdef public clm.mus_any osc
 * 
 * cpdef cython.double r2ssb(R2ssb gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_r2ssb(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_r2ssb {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":367
 *     cdef public clm.mus_any osc
 * 
 * cpdef cython.double eoddcos(Eoddcos gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_eoddcos(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_eoddcos {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":379
 *     cdef public clm.mus_any osc
 * 
 * cpdef cython.double rkcos(Rkcos gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_rkcos(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_rkcos {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":390
 *     cdef public cython.double angle
 * 
 * cpdef cython.double rksin(Rksin gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_rksin(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_rksin {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":413
 *     cdef public cython.double norm
 * 
 * cpdef cython.double rkfcos(Rkfcos gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_rkfcos(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_rkfcos {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":425
 *     cdef public cython.double angle
 * 
 * cpdef cython.double rkfssb(Rkfssb gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_rkfssb(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_rkfssb {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":445
 *     cdef public cython.double norm
 * 
 * cpdef cython.double r2kfcos(R2kfcos gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_r2kfcos(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_r2kfcos {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":454
 *     cdef public cython.double angle
 * 
 * cpdef cython.double k2sin(K2sin gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_k2sin(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_k2sin {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":463
 *     cdef public cython.double angle
 * 
 * cpdef cython.double k2cos(K2cos gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_k2cos(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_k2cos {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":473
 *     cdef public cython.double angle
 * 
 * cpdef cython.double k2ssb(K2ssb gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_k2ssb(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_k2ssb {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":483
 *     cdef public cython.double angle
 * 
 * cpdef cython.double dblsum(Dblsum gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_dblsum(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_dblsum {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":496
 *     cdef public cython.double norm
 * 
 * cpdef cython.double rkoddssb(Rkoddssb gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_rkoddssb(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_rkoddssb {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":506
 *     cdef public cython.double angle
 * 
 * cpdef cython.double krksin(Krksin gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_krksin(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_krksin {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":515
 *     cdef public clm.mus_any osc
 * 
 * cpdef cython.double absin(Absin gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_absin(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_absin {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":528
 *     cdef public cython.double norm
 * 
 * cpdef cython.double abcos(Abcos gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_abcos(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_abcos {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":539
 *     cdef cython.double r2k2cos_norm(self, cython.double a)
 * 
 * cpdef cython.double r2k2cos(R2k2cos gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_r2k2cos(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_r2k2cos {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":550
 *     cdef public cython.double angle
 * 
 * cpdef cython.double blsaw(Blsaw gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_blsaw(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_blsaw {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":562
 *     cdef public cython.double index
 *     cdef public cython.double angle
 *     cpdef cython.double next_i(self, cython.double fm =*)             # <<<<<<<<<<<<<<
 *     cpdef cython.double next_j(self, cython.double fm =*)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_5Asyfm_next_i {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":563
 *     cdef public cython.double angle
 *     cpdef cython.double next_i(self, cython.double fm =*)
 *     cpdef cython.double next_j(self, cython.double fm =*)             # <<<<<<<<<<<<<<
 * 
 * cpdef cython.double asyfm(Asyfm gen, cython.double fm=*)
 */
struct __pyx_opt_args_8pysndlib_10generators_5Asyfm_next_j {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":565
 *     cpdef cython.double next_j(self, cython.double fm =*)
 * 
 * cpdef cython.double asyfm(Asyfm gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double asyfm_i(Asyfm gen, cython.double fm=*)
 * cpdef cython.double asyfm_j(Asyfm gen, cython.double fm=*)
 */
struct __pyx_opt_args_8pysndlib_10generators_asyfm {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":566
 * 
 * cpdef cython.double asyfm(Asyfm gen, cython.double fm=*)
 * cpdef cython.double asyfm_i(Asyfm gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.double asyfm_j(Asyfm gen, cython.double fm=*)
 * cpdef cython.bint is_asyfm(gen)
 */
struct __pyx_opt_args_8pysndlib_10generators_asyfm_i {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":567
 * cpdef cython.double asyfm(Asyfm gen, cython.double fm=*)
 * cpdef cython.double asyfm_i(Asyfm gen, cython.double fm=*)
 * cpdef cython.double asyfm_j(Asyfm gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_asyfm(gen)
 * #
 */
struct __pyx_opt_args_8pysndlib_10generators_asyfm_j {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":578
 *     cdef public cython.double norm
 * 
 * cpdef cython.double bess(Bess gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_bess(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_bess {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":590
 *     cdef public cython.double angle
 * 
 * cpdef cython.double jjcos(Jjcos gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_jjcos(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_jjcos {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":599
 *     cdef public cython.double angle
 * 
 * cpdef cython.double j0evencos(J0evencos gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_j0evencos(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_j0evencos {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":612
 *     cdef public cython.double norm
 * 
 * cpdef cython.double j2cos(J2cos gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_j2cos(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_j2cos {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":625
 *     cdef public cython.double angle
 * 
 * cpdef cython.double jpcos(Jpcos gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_jpcos(gen)
 * #
 */
struct __pyx_opt_args_8pysndlib_10generators_jpcos {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":639
 *     cdef public cython.double ra
 * 
 * cpdef cython.double jncos(Jncos gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_jncos(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_jncos {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":649
 *     cdef public cython.double angle
 * 
 * cpdef cython.double j0j1cos(J0j1cos gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_j0j1cos(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_j0j1cos {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":660
 *     cdef cython.double angle
 * 
 * cpdef cython.double jycos(Jycos gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_jycos(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_jycos {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":670
 *     cdef public cython.double a
 * 
 * cpdef cython.double jcos(Jcos gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_jcos(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_jcos {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":680
 *     cpdef mus_reset(self)
 * 
 * cpdef cython.double blackman(Blackman gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_blackman(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_blackman {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":691
 *     cdef public cython.double angle
 * 
 * cpdef cython.double fmssb(Fmssb gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_fmssb(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_fmssb {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":701
 *     cdef public cython.double angle
 * 
 * cpdef cython.double k3sin(K3sin gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_k3sin(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_k3sin {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":714
 *     cdef public cython.double inorm
 * 
 * cpdef cython.double izcos(Izcos gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_izcos(gen)
 * #
 */
struct __pyx_opt_args_8pysndlib_10generators_izcos {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":727
 *     cdef public clm.mus_any p2
 * 
 * cpdef cython.double adjustable_square_wave(AdjustableSquareWave gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_adjustable_square_wave(gen)
 * # #
 */
struct __pyx_opt_args_8pysndlib_10generators_adjustable_square_wave {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":742
 *     cdef public cython.double val
 * 
 * cpdef cython.double adjustable_triangle_wave(AdjustableTriangleWave gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_adjustable_triangle_wave(gen)
 * #
 */
struct __pyx_opt_args_8pysndlib_10generators_adjustable_triangle_wave {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":758
 *     cdef public cython.double val
 * 
 * cpdef cython.double adjustable_sawtooth_wave(AdjustableSawtoothWave gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_adjustable_sawtooth_wave(gen)
 * # #
 */
struct __pyx_opt_args_8pysndlib_10generators_adjustable_sawtooth_wave {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":773
 *     cdef public cython.double val
 * 
 * cpdef cython.double adjustable_oscil(AdjustableOscil gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_adjustable_oscil(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_adjustable_oscil {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":778
 * 
 * # --------------- make-table-lookup-with-env ---------------- #
 * cpdef make_table_lookup_with_env(frequency, pulse_env, size=*)             # <<<<<<<<<<<<<<
 * 
 * # --------------- make-wave-train-with-env ---------------- #
 */
struct __pyx_opt_args_8pysndlib_10generators_make_table_lookup_with_env {
  int __pyx_n;
  PyObject *size;
};

/* "generators.pxd":781
 * 
 * # --------------- make-wave-train-with-env ---------------- #
 * cpdef make_wave_train_with_env(frequency, pulse_env, size=*)             # <<<<<<<<<<<<<<
 * 
 * # --------------- round-interp ---------------- #
 */
struct __pyx_opt_args_8pysndlib_10generators_make_wave_train_with_env {
  int __pyx_n;
  PyObject *size;
};

/* "generators.pxd":791
 *     cdef public clm.mus_any flt
 * 
 * cpdef cython.double round_interp(RoundInterp gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_round_interp(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_round_interp {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":820
 *     cdef public cython.double angle
 * 
 * cpdef cython.double nchoosekcos(Nchoosekcos gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_nchoosekcos(gen)
 * #
 */
struct __pyx_opt_args_8pysndlib_10generators_nchoosekcos {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":834
 *     cdef public cython.double original_frequency
 * 
 * cpdef cython.double sinc_train(SincTrain gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_sinc_train(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_sinc_train {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":842
 *     cdef public np.ndarray data
 * 
 * cpdef cython.double pink_noise(PinkNoise gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_pink_noise(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_pink_noise {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":853
 *     cdef public clm.mus_any gr
 * 
 * cpdef cython.double brown_noise(BrownNoise gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_brown_noise(gen)
 * #
 */
struct __pyx_opt_args_8pysndlib_10generators_brown_noise {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":886
 *     cdef public clm.mus_any gen
 * 
 * cpdef cython.double moving_sum(MovingSum gen, cython.double insig=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_moving_sum(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_moving_sum {
  int __pyx_n;
  double insig;
};

/* "generators.pxd":896
 *     cdef public clm.mus_any gen2
 * 
 * cpdef cython.double moving_variance(MovingVariance gen, cython.double insig=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_moving_variance(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_moving_variance {
  int __pyx_n;
  double insig;
};

/* "generators.pxd":905
 *     cdef public clm.mus_any gen
 * 
 * cpdef cython.double moving_rms(MovingRMS gen, cython.double insig=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_moving_rms(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_moving_rms {
  int __pyx_n;
  double insig;
};

/* "generators.pxd":913
 *     pass
 * 
 * cpdef cython.double moving_length(MovingLength gen, cython.double insig=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_moving_length(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_moving_length {
  int __pyx_n;
  double insig;
};

/* "generators.pxd":926
 *     cdef public cython.double den
 * 
 * cpdef cython.double weighted_moving_average(WeightedMovingAverage gen, cython.double insig=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_weighted_moving_average(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_weighted_moving_average {
  int __pyx_n;
  double insig;
};

/* "generators.pxd":935
 *     cdef public clm.mus_any gen
 * 
 * cpdef cython.double exponentially_weighted_moving_average(ExponentiallyWeightedMovingAverage gen, cython.double insig=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_exponentially_weighted_moving_average(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_exponentially_weighted_moving_average {
  int __pyx_n;
  double insig;
};

/* "generators.pxd":940
 * # --------------- polyoid ---------------- #
 * 
 * cpdef cython.double polyoid(clm.mus_any gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_polyoid(gen)
 * 
 */
struct __pyx_opt_args_8pysndlib_10generators_polyoid {
  int __pyx_n;
  double fm;
};

/* "generators.pxd":961
 *     cdef public clm.mus_any osc
 * 
 * cpdef cython.double tanhsin(Tanhsin gen, cython.double fm=*)             # <<<<<<<<<<<<<<
 * cpdef cython.bint is_tanhsin(gen)
 * #
 */
struct __pyx_opt_args_8pysndlib_10generators_tanhsin {
  int __pyx_n;
  double fm;
};
struct __pyx_opt_args_8pysndlib_11instruments_6bouncy_bouncy;

/* "pysndlib/instruments/bouncy.py":20
 * 
 * # --------------- bouncy ---------------- #
 * @cython.ccall             # <<<<<<<<<<<<<<
 * def bouncy(start, dur, freq, amp, bounce_freq=5, bounce_amp=20):
 *     lng: cython.long = clm.seconds2samples(dur)
 */
struct __pyx_opt_args_8pysndlib_11instruments_6bouncy_bouncy {
  int __pyx_n;
  PyObject *bounce_freq;
  PyObject *bounce_amp;
};

/* "clm.pxd":40
 * 
 * 
 * cdef class mus_any:             # <<<<<<<<<<<<<<
 *     """
 *     a wrapper class for mus_any pointers in c
 */
struct __pyx_obj_8pysndlib_3clm_mus_any {
  PyObject_HEAD
  struct __pyx_vtabstruct_8pysndlib_3clm_mus_any *__pyx_vtab;
  mus_any *_ptr;
  int ptr_owner;
  input_cb _inputcallback;
  edit_cb _editcallback;
  analyze_cb _analyzecallback;
  synthesize_cb _synthesizecallback;
  PyObject *_cache;
  mus_float_t *_data_ptr;
  PyArrayObject *_data;
  mus_float_t *_xcoeffs_ptr;
  PyArrayObject *_xcoeffs;
  mus_float_t *_ycoeffs_ptr;
  PyArrayObject *_ycoeffs;
  mus_float_t *_pv_amp_increments_ptr;
  PyArrayObject *_pv_amp_increments;
  mus_float_t *_pv_amps_ptr;
  PyArrayObject *_pv_amps;
  mus_float_t *_pv_freqs_ptr;
  PyArrayObject *_pv_freqs;
  mus_float_t *_pv_phases_ptr;
  PyArrayObject *_pv_phases;
  mus_float_t *_pv_phase_increments_ptr;
  PyArrayObject *_pv_phase_increments;
};


/* "generators.pxd":26
 * np.import_array()
 * 
 * cdef class CLMGenerator:             # <<<<<<<<<<<<<<
 *     cpdef cython.double next(self, cython.double  fm=*)
 * 
 */
struct __pyx_obj_8pysndlib_10generators_CLMGenerator {
  PyObject_HEAD
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator *__pyx_vtab;
};


/* "generators.pxd":30
 * 
 * # --------------- nssb ---------------- #
 * cdef class Nssb(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */
struct __pyx_obj_8pysndlib_10generators_Nssb {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double ratio;
  int n;
  double angle;
};


/* "generators.pxd":41
 * 
 * # --------------- nxysin ---------------- #
 * cdef class Nxysin(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */
struct __pyx_obj_8pysndlib_10generators_Nxysin {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double ratio;
  int n;
  double angle;
  double norm;
};


/* "generators.pxd":52
 * 
 * # --------------- nxycos ---------------- #
 * cdef class Nxycos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */
struct __pyx_obj_8pysndlib_10generators_Nxycos {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double ratio;
  int n;
  double angle;
};


/* "generators.pxd":62
 * 
 * # --------------- nxy1cos ---------------- #
 * cdef class Nxy1cos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */
struct __pyx_obj_8pysndlib_10generators_Nxy1cos {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double ratio;
  int n;
  double angle;
};


/* "generators.pxd":72
 * 
 * # --------------- nxy1sin ---------------- #
 * cdef class Nxy1sin(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */
struct __pyx_obj_8pysndlib_10generators_Nxy1sin {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double ratio;
  int n;
  double angle;
};


/* "generators.pxd":82
 * 
 * # --------------- Noddcos ---------------- #
 * cdef class Noddcos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */
struct __pyx_obj_8pysndlib_10generators_Noddcos {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double ratio;
  int n;
  double angle;
};


/* "generators.pxd":93
 * 
 * # --------------- noddssb ---------------- #
 * cdef class Noddssb(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */
struct __pyx_obj_8pysndlib_10generators_Noddssb {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double ratio;
  double n;
  double angle;
};


/* "generators.pxd":103
 * 
 * # --------------- ncos2 ---------------- #
 * cdef class Ncos2(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.int n
 */
struct __pyx_obj_8pysndlib_10generators_Ncos2 {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  int n;
  double angle;
};


/* "generators.pxd":112
 * 
 * # --------------- ncos4 ---------------- #
 * cdef class Ncos4(Ncos2):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_8pysndlib_10generators_Ncos4 {
  struct __pyx_obj_8pysndlib_10generators_Ncos2 __pyx_base;
};


/* "generators.pxd":119
 * 
 * # --------------- npcos ---------------- #
 * cdef class Npcos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double n
 */
struct __pyx_obj_8pysndlib_10generators_Npcos {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double n;
  double angle;
};


/* "generators.pxd":128
 * 
 * # --------------- ncos5 ---------------- #
 * cdef class Ncos5(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double n
 */
struct __pyx_obj_8pysndlib_10generators_Ncos5 {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double n;
  double angle;
};


/* "generators.pxd":139
 * # --------------- nsin5 ---------------- #
 * 
 * cdef class Nsin5(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double n
 */
struct __pyx_obj_8pysndlib_10generators_Nsin5 {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double n;
  double angle;
  double norm;
};


/* "generators.pxd":155
 * #
 * # --------------- nrcos ---------------- #
 * cdef class Nrcos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double n
 */
struct __pyx_obj_8pysndlib_10generators_Nrcos {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double n;
  double r;
  double angle;
  double rr;
  double r1;
  double e1;
  double e2;
  double norm;
  int trouble;
};


/* "generators.pxd":171
 * 
 * # --------------- nrssb ---------------- #
 * cdef class Nrssb(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */
struct __pyx_obj_8pysndlib_10generators_Nrssb {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double ratio;
  double n;
  double r;
  double angle;
  double interp;
  double rn;
  double rn1;
  double norm;
};


/* "generators.pxd":188
 * 
 * # --------------- nkssb ---------------- #
 * cdef class Nkssb(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */
struct __pyx_obj_8pysndlib_10generators_Nkssb {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double ratio;
  double n;
  double angle;
  double norm;
  double interp;
};


/* "generators.pxd":202
 * 
 * # --------------- nsincos ---------------- #
 * cdef class Nsincos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double n
 */
struct __pyx_obj_8pysndlib_10generators_Nsincos {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double n;
  double angle;
  double n2;
  double cosn;
  double norm;
};


/* "generators.pxd":215
 * # --------------- n1cos ---------------- #
 * 
 * cdef class N1cos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double n
 */
struct __pyx_obj_8pysndlib_10generators_N1cos {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double n;
  double angle;
};


/* "generators.pxd":225
 * # TODO: --------------- npos1cos ---------------- #
 * 
 * cdef class Npos1cos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double n
 */
struct __pyx_obj_8pysndlib_10generators_Npos1cos {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double n;
  double angle;
};


/* "generators.pxd":234
 * 
 * # --------------- npos3cos ---------------- #
 * cdef class Npos3cos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double n
 */
struct __pyx_obj_8pysndlib_10generators_Npos3cos {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double n;
  double angle;
};


/* "generators.pxd":242
 * # --------------- rcos ---------------- #
 * 
 * cdef class Rcos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public clm.mus_any osc
 *     cdef public cython.double r
 */
struct __pyx_obj_8pysndlib_10generators_Rcos {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  struct __pyx_obj_8pysndlib_3clm_mus_any *osc;
  double r;
  double rr;
  double rrp1;
  double rrm1;
  double r2;
  double norm;
};


/* "generators.pxd":257
 * # --------------- rssb ---------------- #
 * 
 * cdef class Rssb(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */
struct __pyx_obj_8pysndlib_10generators_Rssb {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double ratio;
  double r;
  double angle;
};


/* "generators.pxd":272
 * # --------------- rxysin ---------------- #
 * 
 * cdef class Rxysin(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */
struct __pyx_obj_8pysndlib_10generators_Rxysin {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double ratio;
  double r;
  double angle;
  double rr;
  double r2;
};


/* "generators.pxd":285
 * # --------------- rxycos ---------------- #
 * 
 * cdef class Rxycos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */
struct __pyx_obj_8pysndlib_10generators_Rxycos {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double ratio;
  double r;
  double angle;
  double rr;
  double r2;
  double norm;
};


/* "generators.pxd":300
 * #  --------------- safe-rxycos ---------------- #
 * 
 * cdef class SafeRxycos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */
struct __pyx_obj_8pysndlib_10generators_SafeRxycos {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double ratio;
  double r;
  double angle;
  double cutoff;
};


/* "generators.pxd":312
 * 
 * # --------------- ercos ---------------- #
 * cdef class Ercos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double r
 */
struct __pyx_obj_8pysndlib_10generators_Ercos {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double r;
  double angle;
  double cosh_t;
  double offset;
  double scaler;
  struct __pyx_obj_8pysndlib_3clm_mus_any *osc;
};


/* "generators.pxd":325
 * 
 * # --------------- erssb ---------------- #
 * cdef class Erssb(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */
struct __pyx_obj_8pysndlib_10generators_Erssb {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double ratio;
  double r;
  double angle;
};


/* "generators.pxd":340
 * 
 * # --------------- r2cos ---------------- #
 * cdef class R2cos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double r
 */
struct __pyx_obj_8pysndlib_10generators_R2cos {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double r;
  double angle;
};


/* "generators.pxd":350
 * # --------------- r2ssb ---------------- #
 * 
 * cdef class R2ssb(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */
struct __pyx_obj_8pysndlib_10generators_R2ssb {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double ratio;
  double r;
  double angle;
  struct __pyx_obj_8pysndlib_3clm_mus_any *osc;
};


/* "generators.pxd":362
 * # --------------- eoddcos ---------------- #
 * 
 * cdef class Eoddcos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double r
 */
struct __pyx_obj_8pysndlib_10generators_Eoddcos {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double r;
  struct __pyx_obj_8pysndlib_3clm_mus_any *osc;
};


/* "generators.pxd":373
 * # --------------- rkcos ---------------- #
 * 
 * cdef class Rkcos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double r
 */
struct __pyx_obj_8pysndlib_10generators_Rkcos {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double r;
  double norm;
  struct __pyx_obj_8pysndlib_3clm_mus_any *osc;
};


/* "generators.pxd":384
 * # --------------- rksin ---------------- #
 * 
 * cdef class Rksin(CLMGenerator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public cython.double frequency
 */
struct __pyx_obj_8pysndlib_10generators_Rksin {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double r;
  double angle;
};


/* "generators.pxd":395
 * # --------------- rkssb ---------------- #
 * 
 * cdef class Rkssb(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double r
 */
struct __pyx_obj_8pysndlib_10generators_Rkssb {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double r;
  double norm;
  struct __pyx_obj_8pysndlib_3clm_mus_any *osc;
};


/* "generators.pxd":406
 * 
 * # --------------- rk!cos ---------------- #
 * cdef class Rkfcos(CLMGenerator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public cython.double frequency
 */
struct __pyx_obj_8pysndlib_10generators_Rkfcos {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double r;
  double angle;
  double norm;
};


/* "generators.pxd":419
 * # --------------- rk!ssb ---------------- #
 * 
 * cdef class Rkfssb(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */
struct __pyx_obj_8pysndlib_10generators_Rkfssb {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double ratio;
  double r;
  double angle;
};


/* "generators.pxd":436
 * # --------------- r2k!cos ---------------- #
 * 
 * cdef class R2kfcos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double r
 */
struct __pyx_obj_8pysndlib_10generators_R2kfcos {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double r;
  double k;
  double rr1;
  double r2;
  struct __pyx_obj_8pysndlib_3clm_mus_any *osc;
  double norm;
};


/* "generators.pxd":450
 * # --------------- k2sin ---------------- #
 * 
 * cdef class K2sin(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double angle
 */
struct __pyx_obj_8pysndlib_10generators_K2sin {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double angle;
};


/* "generators.pxd":459
 * # --------------- k2cos ---------------- #
 * 
 * cdef class K2cos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double angle
 */
struct __pyx_obj_8pysndlib_10generators_K2cos {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double angle;
};


/* "generators.pxd":468
 * # --------------- k2ssb ---------------- #
 * 
 * cdef class K2ssb(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */
struct __pyx_obj_8pysndlib_10generators_K2ssb {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double ratio;
  double angle;
};


/* "generators.pxd":478
 * # --------------- dblsum ---------------- #
 * 
 * cdef class Dblsum(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double r
 */
struct __pyx_obj_8pysndlib_10generators_Dblsum {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double r;
  double angle;
};


/* "generators.pxd":488
 * 
 * # --------------- rkoddssb ---------------- #
 * cdef class Rkoddssb(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */
struct __pyx_obj_8pysndlib_10generators_Rkoddssb {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double ratio;
  double r;
  double rr1;
  double angle;
  double norm;
};


/* "generators.pxd":501
 * # --------------- krksin ---------------- #
 * 
 * cdef class Krksin(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double r
 */
struct __pyx_obj_8pysndlib_10generators_Krksin {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double r;
  double angle;
};


/* "generators.pxd":511
 * # --------------- abssin ---------------- #
 * 
 * cdef class Absin(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public clm.mus_any osc
 */
struct __pyx_obj_8pysndlib_10generators_Absin {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  struct __pyx_obj_8pysndlib_3clm_mus_any *osc;
};


/* "generators.pxd":520
 * # --------------- abscos ---------------- #
 * 
 * cdef class Abcos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double a
 */
struct __pyx_obj_8pysndlib_10generators_Abcos {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double a;
  double b;
  double angle;
  double ab;
  double norm;
};


/* "generators.pxd":533
 * # --------------- r2k2cos ---------------- #
 * 
 * cdef class R2k2cos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double r
 */
struct __pyx_obj_8pysndlib_10generators_R2k2cos {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double r;
  double angle;
};


/* "generators.pxd":544
 * # --------------- blsaw ---------------- #
 * 
 * cdef class Blsaw(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double n
 */
struct __pyx_obj_8pysndlib_10generators_Blsaw {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double n;
  double r;
  double angle;
};


/* "generators.pxd":556
 * # TODO: add I variant--------------- asyfm ---------------- #
 * 
 * cdef class Asyfm(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */
struct __pyx_obj_8pysndlib_10generators_Asyfm {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double ratio;
  double r;
  double index;
  double angle;
};


/* "generators.pxd":572
 * # --------------- bess ---------------- #
 * 
 * cdef class Bess(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.int n
 */
struct __pyx_obj_8pysndlib_10generators_Bess {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  int n;
  double angle;
  double norm;
};


/* "generators.pxd":583
 * 
 * # --------------- jjcos ---------------- #
 * cdef class Jjcos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double r
 */
struct __pyx_obj_8pysndlib_10generators_Jjcos {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double r;
  double a;
  double k;
  double angle;
};


/* "generators.pxd":594
 * 
 * # --------------- j0evencos ---------------- #
 * cdef class J0evencos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double index
 */
struct __pyx_obj_8pysndlib_10generators_J0evencos {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double index;
  double angle;
};


/* "generators.pxd":605
 * 
 * 
 * cdef class J2cos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.int n
 */
struct __pyx_obj_8pysndlib_10generators_J2cos {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  int n;
  double r;
  double angle;
  double norm;
};


/* "generators.pxd":618
 * # --------------- jpcos ---------------- #
 * 
 * cdef class Jpcos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double r
 */
struct __pyx_obj_8pysndlib_10generators_Jpcos {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double r;
  double a;
  double k;
  double angle;
};


/* "generators.pxd":631
 * 
 * # --------------- jncos ---------------- #
 * cdef class Jncos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double r
 */
struct __pyx_obj_8pysndlib_10generators_Jncos {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double r;
  double a;
  int n;
  double angle;
  double ra;
};


/* "generators.pxd":644
 * # --------------- j0j1cos  ---------------- #
 * 
 * cdef class J0j1cos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double index
 */
struct __pyx_obj_8pysndlib_10generators_J0j1cos {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double index;
  double angle;
};


/* "generators.pxd":654
 * # --------------- jycos  ---------------- #
 * 
 * cdef class Jycos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double r
 */
struct __pyx_obj_8pysndlib_10generators_Jycos {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double r;
  double a;
  double angle;
};


/* "generators.pxd":664
 * 
 * # --------------- jcos  ---------------- #
 * cdef class Jcos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.int n
 */
struct __pyx_obj_8pysndlib_10generators_Jcos {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  int n;
  double r;
  double a;
};


/* "generators.pxd":674
 * 
 * # --------------- blackman  ---------------- #
 * cdef class Blackman(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.int n
 */
struct __pyx_obj_8pysndlib_10generators_Blackman {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  int n;
  struct __pyx_obj_8pysndlib_3clm_mus_any *osc;
};


/* "generators.pxd":685
 * # --------------- fmssb ---------------- #
 * 
 * cdef class Fmssb(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */
struct __pyx_obj_8pysndlib_10generators_Fmssb {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double ratio;
  double idx;
  double angle;
};


/* "generators.pxd":696
 * # --------------- k3sin  ---------------- #
 * 
 * cdef class K3sin(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public np.ndarray coeffs
 */
struct __pyx_obj_8pysndlib_10generators_K3sin {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  PyArrayObject *coeffs;
  double angle;
};


/* "generators.pxd":706
 * 
 * # --------------- izcos  ---------------- # #
 * cdef class Izcos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double r
 */
struct __pyx_obj_8pysndlib_10generators_Izcos {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double r;
  double angle;
  double dc;
  double norm;
  double inorm;
};


/* "generators.pxd":719
 * # --------------- adjustable-square-wave ---------------- #
 * # Note: modulating the duty factor will not work
 * cdef class AdjustableSquareWave(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double duty_factor
 */
struct __pyx_obj_8pysndlib_10generators_AdjustableSquareWave {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double duty_factor;
  double amplitude;
  double sum;
  struct __pyx_obj_8pysndlib_3clm_mus_any *p1;
  struct __pyx_obj_8pysndlib_3clm_mus_any *p2;
};


/* "generators.pxd":732
 * # --------------- adjustable-triangle-wave ---------------- #
 * #
 * cdef class AdjustableTriangleWave(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double duty_factor
 */
struct __pyx_obj_8pysndlib_10generators_AdjustableTriangleWave {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double duty_factor;
  double amplitude;
  struct __pyx_obj_8pysndlib_3clm_mus_any *gen;
  double top;
  double mtop;
  double scl;
  double val;
};


/* "generators.pxd":748
 * # --------------- adjustable-sawtooth-wave ---------------- #
 * 
 * cdef class AdjustableSawtoothWave(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double duty_factor
 */
struct __pyx_obj_8pysndlib_10generators_AdjustableSawtoothWave {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double duty_factor;
  double amplitude;
  struct __pyx_obj_8pysndlib_3clm_mus_any *gen;
  double top;
  double mtop;
  double scl;
  double val;
};


/* "generators.pxd":763
 * # TODO: --------------- adjustable-oscil-wave ---------------- #
 * 
 * cdef class AdjustableOscil(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double duty_factor
 */
struct __pyx_obj_8pysndlib_10generators_AdjustableOscil {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double duty_factor;
  double amplitude;
  struct __pyx_obj_8pysndlib_3clm_mus_any *gen;
  double top;
  double mtop;
  double scl;
  double val;
};


/* "generators.pxd":784
 * 
 * # --------------- round-interp ---------------- #
 * cdef class RoundInterp(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.int n
 */
struct __pyx_obj_8pysndlib_10generators_RoundInterp {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  int n;
  double amplitude;
  struct __pyx_obj_8pysndlib_3clm_mus_any *rnd;
  struct __pyx_obj_8pysndlib_3clm_mus_any *flt;
};


/* "generators.pxd":814
 * 
 * # TODO: This is not working correctly --------------- nchoosekcos ---------------- #
 * cdef class Nchoosekcos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */
struct __pyx_obj_8pysndlib_10generators_Nchoosekcos {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double ratio;
  int n;
  double angle;
};


/* "generators.pxd":827
 * # TODO: --------------- sinc-train ---------------- #
 * 
 * cdef class SincTrain(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.int n
 */
struct __pyx_obj_8pysndlib_10generators_SincTrain {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  int n;
  double angle;
  double original_n;
  double original_frequency;
};


/* "generators.pxd":838
 * 
 * # --------------- pink-noise ---------------- #
 * cdef class PinkNoise(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.int n
 *     cdef public np.ndarray data
 */
struct __pyx_obj_8pysndlib_10generators_PinkNoise {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  int n;
  PyArrayObject *data;
};


/* "generators.pxd":846
 * 
 * # --------------- brown-noise ---------------- #
 * cdef class BrownNoise(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double amplitude
 */
struct __pyx_obj_8pysndlib_10generators_BrownNoise {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double amplitude;
  double prev;
  double sum;
  struct __pyx_obj_8pysndlib_3clm_mus_any *gr;
};


/* "generators.pxd":858
 * #
 * # --------------- green-noise ---------------- #
 * cdef class GreenNoise(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double amplitude
 */
struct __pyx_obj_8pysndlib_10generators_GreenNoise {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double amplitude;
  double prev;
  double sum;
  struct __pyx_obj_8pysndlib_3clm_mus_any *gr;
  double low;
  double high;
};


/* "generators.pxd":868
 * #
 * # TODO: This is not working properly--------------- green-noise-interp ---------------- #
 * cdef class GreenNoiseInterp(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double amplitude
 */
struct __pyx_obj_8pysndlib_10generators_GreenNoiseInterp {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double amplitude;
  double sum;
  double low;
  double high;
  double dv;
  double incr;
  double angle;
};


/* "generators.pxd":882
 * # --------------- moving-sum ---------------- #
 * 
 * cdef class MovingSum(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.int n
 *     cdef public clm.mus_any gen
 */
struct __pyx_obj_8pysndlib_10generators_MovingSum {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  int n;
  struct __pyx_obj_8pysndlib_3clm_mus_any *gen;
};


/* "generators.pxd":891
 * # --------------- moving-variance ---------------- #
 * 
 * cdef class MovingVariance(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.int n
 *     cdef public clm.mus_any gen1
 */
struct __pyx_obj_8pysndlib_10generators_MovingVariance {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  int n;
  struct __pyx_obj_8pysndlib_3clm_mus_any *gen1;
  struct __pyx_obj_8pysndlib_3clm_mus_any *gen2;
};


/* "generators.pxd":901
 * # --------------- moving-rms ---------------- #
 * 
 * cdef class MovingRMS(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.int n
 *     cdef public clm.mus_any gen
 */
struct __pyx_obj_8pysndlib_10generators_MovingRMS {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  int n;
  struct __pyx_obj_8pysndlib_3clm_mus_any *gen;
};


/* "generators.pxd":910
 * # --------------- moving-length ---------------- #
 * 
 * cdef class MovingLength(MovingRMS):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
struct __pyx_obj_8pysndlib_10generators_MovingLength {
  struct __pyx_obj_8pysndlib_10generators_MovingRMS __pyx_base;
};


/* "generators.pxd":918
 * # --------------- weighted-moving-average---------------- #
 * 
 * cdef class WeightedMovingAverage(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.int n
 *     cdef public clm.mus_any dly
 */
struct __pyx_obj_8pysndlib_10generators_WeightedMovingAverage {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  int n;
  struct __pyx_obj_8pysndlib_3clm_mus_any *dly;
  double num;
  double sum;
  double y;
  double den;
};


/* "generators.pxd":931
 * # --------------- exponentially-weighted-moving-average---------------- #
 * 
 * cdef class ExponentiallyWeightedMovingAverage(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.int n
 *     cdef public clm.mus_any gen
 */
struct __pyx_obj_8pysndlib_10generators_ExponentiallyWeightedMovingAverage {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  int n;
  struct __pyx_obj_8pysndlib_3clm_mus_any *gen;
};


/* "generators.pxd":956
 * 
 * # --------------- tanhsin ---------------- #
 * cdef class Tanhsin(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double r
 */
struct __pyx_obj_8pysndlib_10generators_Tanhsin {
  struct __pyx_obj_8pysndlib_10generators_CLMGenerator __pyx_base;
  double frequency;
  double r;
  struct __pyx_obj_8pysndlib_3clm_mus_any *osc;
};



/* "clm.pxd":40
 * 
 * 
 * cdef class mus_any:             # <<<<<<<<<<<<<<
 *     """
 *     a wrapper class for mus_any pointers in c
 */

struct __pyx_vtabstruct_8pysndlib_3clm_mus_any {
  struct __pyx_obj_8pysndlib_3clm_mus_any *(*from_ptr)(mus_any *, struct __pyx_opt_args_8pysndlib_3clm_7mus_any_from_ptr *__pyx_optional_args);
  PyObject *(*cache_append)(struct __pyx_obj_8pysndlib_3clm_mus_any *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*cache_extend)(struct __pyx_obj_8pysndlib_3clm_mus_any *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*set_up_data)(struct __pyx_obj_8pysndlib_3clm_mus_any *, int __pyx_skip_dispatch, struct __pyx_opt_args_8pysndlib_3clm_7mus_any_set_up_data *__pyx_optional_args);
  PyObject *(*set_up_xcoeffs)(struct __pyx_obj_8pysndlib_3clm_mus_any *, int __pyx_skip_dispatch);
  PyObject *(*set_up_ycoeffs)(struct __pyx_obj_8pysndlib_3clm_mus_any *, int __pyx_skip_dispatch);
  PyObject *(*set_up_pv_data)(struct __pyx_obj_8pysndlib_3clm_mus_any *, int __pyx_skip_dispatch);
  PyObject *(*mus_reset)(struct __pyx_obj_8pysndlib_3clm_mus_any *, int __pyx_skip_dispatch);
  PyObject *(*next)(struct __pyx_obj_8pysndlib_3clm_mus_any *, int __pyx_skip_dispatch, struct __pyx_opt_args_8pysndlib_3clm_7mus_any_next *__pyx_optional_args);
};
static struct __pyx_vtabstruct_8pysndlib_3clm_mus_any *__pyx_vtabptr_8pysndlib_3clm_mus_any;


/* "generators.pxd":26
 * np.import_array()
 * 
 * cdef class CLMGenerator:             # <<<<<<<<<<<<<<
 *     cpdef cython.double next(self, cython.double  fm=*)
 * 
 */

struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator {
  double (*next)(struct __pyx_obj_8pysndlib_10generators_CLMGenerator *, int __pyx_skip_dispatch, struct __pyx_opt_args_8pysndlib_10generators_12CLMGenerator_next *__pyx_optional_args);
};
static struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator *__pyx_vtabptr_8pysndlib_10generators_CLMGenerator;


/* "generators.pxd":30
 * 
 * # --------------- nssb ---------------- #
 * cdef class Nssb(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Nssb {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Nssb *__pyx_vtabptr_8pysndlib_10generators_Nssb;


/* "generators.pxd":41
 * 
 * # --------------- nxysin ---------------- #
 * cdef class Nxysin(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Nxysin {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Nxysin *__pyx_vtabptr_8pysndlib_10generators_Nxysin;


/* "generators.pxd":52
 * 
 * # --------------- nxycos ---------------- #
 * cdef class Nxycos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Nxycos {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Nxycos *__pyx_vtabptr_8pysndlib_10generators_Nxycos;


/* "generators.pxd":62
 * 
 * # --------------- nxy1cos ---------------- #
 * cdef class Nxy1cos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Nxy1cos {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Nxy1cos *__pyx_vtabptr_8pysndlib_10generators_Nxy1cos;


/* "generators.pxd":72
 * 
 * # --------------- nxy1sin ---------------- #
 * cdef class Nxy1sin(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Nxy1sin {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Nxy1sin *__pyx_vtabptr_8pysndlib_10generators_Nxy1sin;


/* "generators.pxd":82
 * 
 * # --------------- Noddcos ---------------- #
 * cdef class Noddcos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Noddcos {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Noddcos *__pyx_vtabptr_8pysndlib_10generators_Noddcos;


/* "generators.pxd":93
 * 
 * # --------------- noddssb ---------------- #
 * cdef class Noddssb(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Noddssb {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Noddssb *__pyx_vtabptr_8pysndlib_10generators_Noddssb;


/* "generators.pxd":103
 * 
 * # --------------- ncos2 ---------------- #
 * cdef class Ncos2(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.int n
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Ncos2 {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Ncos2 *__pyx_vtabptr_8pysndlib_10generators_Ncos2;


/* "generators.pxd":112
 * 
 * # --------------- ncos4 ---------------- #
 * cdef class Ncos4(Ncos2):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Ncos4 {
  struct __pyx_vtabstruct_8pysndlib_10generators_Ncos2 __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Ncos4 *__pyx_vtabptr_8pysndlib_10generators_Ncos4;


/* "generators.pxd":119
 * 
 * # --------------- npcos ---------------- #
 * cdef class Npcos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double n
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Npcos {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Npcos *__pyx_vtabptr_8pysndlib_10generators_Npcos;


/* "generators.pxd":128
 * 
 * # --------------- ncos5 ---------------- #
 * cdef class Ncos5(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double n
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Ncos5 {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Ncos5 *__pyx_vtabptr_8pysndlib_10generators_Ncos5;


/* "generators.pxd":139
 * # --------------- nsin5 ---------------- #
 * 
 * cdef class Nsin5(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double n
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Nsin5 {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Nsin5 *__pyx_vtabptr_8pysndlib_10generators_Nsin5;


/* "generators.pxd":155
 * #
 * # --------------- nrcos ---------------- #
 * cdef class Nrcos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double n
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Nrcos {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Nrcos *__pyx_vtabptr_8pysndlib_10generators_Nrcos;


/* "generators.pxd":171
 * 
 * # --------------- nrssb ---------------- #
 * cdef class Nrssb(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Nrssb {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
  PyObject *(*call_interp)(struct __pyx_obj_8pysndlib_10generators_Nrssb *, int __pyx_skip_dispatch, struct __pyx_opt_args_8pysndlib_10generators_5Nrssb_call_interp *__pyx_optional_args);
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Nrssb *__pyx_vtabptr_8pysndlib_10generators_Nrssb;


/* "generators.pxd":188
 * 
 * # --------------- nkssb ---------------- #
 * cdef class Nkssb(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Nkssb {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
  double (*call_interp)(struct __pyx_obj_8pysndlib_10generators_Nkssb *, int __pyx_skip_dispatch, struct __pyx_opt_args_8pysndlib_10generators_5Nkssb_call_interp *__pyx_optional_args);
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Nkssb *__pyx_vtabptr_8pysndlib_10generators_Nkssb;


/* "generators.pxd":202
 * 
 * # --------------- nsincos ---------------- #
 * cdef class Nsincos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double n
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Nsincos {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Nsincos *__pyx_vtabptr_8pysndlib_10generators_Nsincos;


/* "generators.pxd":215
 * # --------------- n1cos ---------------- #
 * 
 * cdef class N1cos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double n
 */

struct __pyx_vtabstruct_8pysndlib_10generators_N1cos {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_N1cos *__pyx_vtabptr_8pysndlib_10generators_N1cos;


/* "generators.pxd":225
 * # TODO: --------------- npos1cos ---------------- #
 * 
 * cdef class Npos1cos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double n
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Npos1cos {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Npos1cos *__pyx_vtabptr_8pysndlib_10generators_Npos1cos;


/* "generators.pxd":234
 * 
 * # --------------- npos3cos ---------------- #
 * cdef class Npos3cos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double n
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Npos3cos {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Npos3cos *__pyx_vtabptr_8pysndlib_10generators_Npos3cos;


/* "generators.pxd":242
 * # --------------- rcos ---------------- #
 * 
 * cdef class Rcos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public clm.mus_any osc
 *     cdef public cython.double r
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Rcos {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Rcos *__pyx_vtabptr_8pysndlib_10generators_Rcos;


/* "generators.pxd":257
 * # --------------- rssb ---------------- #
 * 
 * cdef class Rssb(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Rssb {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
  double (*call_interp)(struct __pyx_obj_8pysndlib_10generators_Rssb *, int __pyx_skip_dispatch, struct __pyx_opt_args_8pysndlib_10generators_4Rssb_call_interp *__pyx_optional_args);
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Rssb *__pyx_vtabptr_8pysndlib_10generators_Rssb;


/* "generators.pxd":272
 * # --------------- rxysin ---------------- #
 * 
 * cdef class Rxysin(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Rxysin {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Rxysin *__pyx_vtabptr_8pysndlib_10generators_Rxysin;


/* "generators.pxd":285
 * # --------------- rxycos ---------------- #
 * 
 * cdef class Rxycos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Rxycos {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Rxycos *__pyx_vtabptr_8pysndlib_10generators_Rxycos;


/* "generators.pxd":300
 * #  --------------- safe-rxycos ---------------- #
 * 
 * cdef class SafeRxycos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */

struct __pyx_vtabstruct_8pysndlib_10generators_SafeRxycos {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
  PyObject *(*clamp_rxycos_r)(struct __pyx_obj_8pysndlib_10generators_SafeRxycos *, int __pyx_skip_dispatch, struct __pyx_opt_args_8pysndlib_10generators_10SafeRxycos_clamp_rxycos_r *__pyx_optional_args);
};
static struct __pyx_vtabstruct_8pysndlib_10generators_SafeRxycos *__pyx_vtabptr_8pysndlib_10generators_SafeRxycos;


/* "generators.pxd":312
 * 
 * # --------------- ercos ---------------- #
 * cdef class Ercos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double r
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Ercos {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Ercos *__pyx_vtabptr_8pysndlib_10generators_Ercos;


/* "generators.pxd":325
 * 
 * # --------------- erssb ---------------- #
 * cdef class Erssb(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Erssb {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Erssb *__pyx_vtabptr_8pysndlib_10generators_Erssb;


/* "generators.pxd":340
 * 
 * # --------------- r2cos ---------------- #
 * cdef class R2cos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double r
 */

struct __pyx_vtabstruct_8pysndlib_10generators_R2cos {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_R2cos *__pyx_vtabptr_8pysndlib_10generators_R2cos;


/* "generators.pxd":350
 * # --------------- r2ssb ---------------- #
 * 
 * cdef class R2ssb(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */

struct __pyx_vtabstruct_8pysndlib_10generators_R2ssb {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_R2ssb *__pyx_vtabptr_8pysndlib_10generators_R2ssb;


/* "generators.pxd":362
 * # --------------- eoddcos ---------------- #
 * 
 * cdef class Eoddcos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double r
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Eoddcos {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Eoddcos *__pyx_vtabptr_8pysndlib_10generators_Eoddcos;


/* "generators.pxd":373
 * # --------------- rkcos ---------------- #
 * 
 * cdef class Rkcos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double r
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Rkcos {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Rkcos *__pyx_vtabptr_8pysndlib_10generators_Rkcos;


/* "generators.pxd":384
 * # --------------- rksin ---------------- #
 * 
 * cdef class Rksin(CLMGenerator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public cython.double frequency
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Rksin {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Rksin *__pyx_vtabptr_8pysndlib_10generators_Rksin;


/* "generators.pxd":395
 * # --------------- rkssb ---------------- #
 * 
 * cdef class Rkssb(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double r
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Rkssb {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Rkssb *__pyx_vtabptr_8pysndlib_10generators_Rkssb;


/* "generators.pxd":406
 * 
 * # --------------- rk!cos ---------------- #
 * cdef class Rkfcos(CLMGenerator):             # <<<<<<<<<<<<<<
 * 
 *     cdef public cython.double frequency
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Rkfcos {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Rkfcos *__pyx_vtabptr_8pysndlib_10generators_Rkfcos;


/* "generators.pxd":419
 * # --------------- rk!ssb ---------------- #
 * 
 * cdef class Rkfssb(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Rkfssb {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Rkfssb *__pyx_vtabptr_8pysndlib_10generators_Rkfssb;


/* "generators.pxd":436
 * # --------------- r2k!cos ---------------- #
 * 
 * cdef class R2kfcos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double r
 */

struct __pyx_vtabstruct_8pysndlib_10generators_R2kfcos {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_R2kfcos *__pyx_vtabptr_8pysndlib_10generators_R2kfcos;


/* "generators.pxd":450
 * # --------------- k2sin ---------------- #
 * 
 * cdef class K2sin(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double angle
 */

struct __pyx_vtabstruct_8pysndlib_10generators_K2sin {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_K2sin *__pyx_vtabptr_8pysndlib_10generators_K2sin;


/* "generators.pxd":459
 * # --------------- k2cos ---------------- #
 * 
 * cdef class K2cos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double angle
 */

struct __pyx_vtabstruct_8pysndlib_10generators_K2cos {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_K2cos *__pyx_vtabptr_8pysndlib_10generators_K2cos;


/* "generators.pxd":468
 * # --------------- k2ssb ---------------- #
 * 
 * cdef class K2ssb(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */

struct __pyx_vtabstruct_8pysndlib_10generators_K2ssb {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_K2ssb *__pyx_vtabptr_8pysndlib_10generators_K2ssb;


/* "generators.pxd":478
 * # --------------- dblsum ---------------- #
 * 
 * cdef class Dblsum(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double r
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Dblsum {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Dblsum *__pyx_vtabptr_8pysndlib_10generators_Dblsum;


/* "generators.pxd":488
 * 
 * # --------------- rkoddssb ---------------- #
 * cdef class Rkoddssb(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Rkoddssb {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Rkoddssb *__pyx_vtabptr_8pysndlib_10generators_Rkoddssb;


/* "generators.pxd":501
 * # --------------- krksin ---------------- #
 * 
 * cdef class Krksin(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double r
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Krksin {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Krksin *__pyx_vtabptr_8pysndlib_10generators_Krksin;


/* "generators.pxd":511
 * # --------------- abssin ---------------- #
 * 
 * cdef class Absin(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public clm.mus_any osc
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Absin {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Absin *__pyx_vtabptr_8pysndlib_10generators_Absin;


/* "generators.pxd":520
 * # --------------- abscos ---------------- #
 * 
 * cdef class Abcos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double a
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Abcos {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Abcos *__pyx_vtabptr_8pysndlib_10generators_Abcos;


/* "generators.pxd":533
 * # --------------- r2k2cos ---------------- #
 * 
 * cdef class R2k2cos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double r
 */

struct __pyx_vtabstruct_8pysndlib_10generators_R2k2cos {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
  double (*r2k2cos_norm)(struct __pyx_obj_8pysndlib_10generators_R2k2cos *, double);
};
static struct __pyx_vtabstruct_8pysndlib_10generators_R2k2cos *__pyx_vtabptr_8pysndlib_10generators_R2k2cos;


/* "generators.pxd":544
 * # --------------- blsaw ---------------- #
 * 
 * cdef class Blsaw(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double n
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Blsaw {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Blsaw *__pyx_vtabptr_8pysndlib_10generators_Blsaw;


/* "generators.pxd":556
 * # TODO: add I variant--------------- asyfm ---------------- #
 * 
 * cdef class Asyfm(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Asyfm {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
  double (*next_i)(struct __pyx_obj_8pysndlib_10generators_Asyfm *, int __pyx_skip_dispatch, struct __pyx_opt_args_8pysndlib_10generators_5Asyfm_next_i *__pyx_optional_args);
  double (*next_j)(struct __pyx_obj_8pysndlib_10generators_Asyfm *, int __pyx_skip_dispatch, struct __pyx_opt_args_8pysndlib_10generators_5Asyfm_next_j *__pyx_optional_args);
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Asyfm *__pyx_vtabptr_8pysndlib_10generators_Asyfm;


/* "generators.pxd":572
 * # --------------- bess ---------------- #
 * 
 * cdef class Bess(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.int n
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Bess {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Bess *__pyx_vtabptr_8pysndlib_10generators_Bess;


/* "generators.pxd":583
 * 
 * # --------------- jjcos ---------------- #
 * cdef class Jjcos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double r
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Jjcos {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Jjcos *__pyx_vtabptr_8pysndlib_10generators_Jjcos;


/* "generators.pxd":594
 * 
 * # --------------- j0evencos ---------------- #
 * cdef class J0evencos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double index
 */

struct __pyx_vtabstruct_8pysndlib_10generators_J0evencos {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_J0evencos *__pyx_vtabptr_8pysndlib_10generators_J0evencos;


/* "generators.pxd":605
 * 
 * 
 * cdef class J2cos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.int n
 */

struct __pyx_vtabstruct_8pysndlib_10generators_J2cos {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_J2cos *__pyx_vtabptr_8pysndlib_10generators_J2cos;


/* "generators.pxd":618
 * # --------------- jpcos ---------------- #
 * 
 * cdef class Jpcos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double r
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Jpcos {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Jpcos *__pyx_vtabptr_8pysndlib_10generators_Jpcos;


/* "generators.pxd":631
 * 
 * # --------------- jncos ---------------- #
 * cdef class Jncos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double r
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Jncos {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Jncos *__pyx_vtabptr_8pysndlib_10generators_Jncos;


/* "generators.pxd":644
 * # --------------- j0j1cos  ---------------- #
 * 
 * cdef class J0j1cos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double index
 */

struct __pyx_vtabstruct_8pysndlib_10generators_J0j1cos {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_J0j1cos *__pyx_vtabptr_8pysndlib_10generators_J0j1cos;


/* "generators.pxd":654
 * # --------------- jycos  ---------------- #
 * 
 * cdef class Jycos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double r
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Jycos {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Jycos *__pyx_vtabptr_8pysndlib_10generators_Jycos;


/* "generators.pxd":664
 * 
 * # --------------- jcos  ---------------- #
 * cdef class Jcos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.int n
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Jcos {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Jcos *__pyx_vtabptr_8pysndlib_10generators_Jcos;


/* "generators.pxd":674
 * 
 * # --------------- blackman  ---------------- #
 * cdef class Blackman(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.int n
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Blackman {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
  PyObject *(*mus_reset)(struct __pyx_obj_8pysndlib_10generators_Blackman *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Blackman *__pyx_vtabptr_8pysndlib_10generators_Blackman;


/* "generators.pxd":685
 * # --------------- fmssb ---------------- #
 * 
 * cdef class Fmssb(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Fmssb {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Fmssb *__pyx_vtabptr_8pysndlib_10generators_Fmssb;


/* "generators.pxd":696
 * # --------------- k3sin  ---------------- #
 * 
 * cdef class K3sin(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public np.ndarray coeffs
 */

struct __pyx_vtabstruct_8pysndlib_10generators_K3sin {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_K3sin *__pyx_vtabptr_8pysndlib_10generators_K3sin;


/* "generators.pxd":706
 * 
 * # --------------- izcos  ---------------- # #
 * cdef class Izcos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double r
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Izcos {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Izcos *__pyx_vtabptr_8pysndlib_10generators_Izcos;


/* "generators.pxd":719
 * # --------------- adjustable-square-wave ---------------- #
 * # Note: modulating the duty factor will not work
 * cdef class AdjustableSquareWave(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double duty_factor
 */

struct __pyx_vtabstruct_8pysndlib_10generators_AdjustableSquareWave {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_AdjustableSquareWave *__pyx_vtabptr_8pysndlib_10generators_AdjustableSquareWave;


/* "generators.pxd":732
 * # --------------- adjustable-triangle-wave ---------------- #
 * #
 * cdef class AdjustableTriangleWave(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double duty_factor
 */

struct __pyx_vtabstruct_8pysndlib_10generators_AdjustableTriangleWave {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_AdjustableTriangleWave *__pyx_vtabptr_8pysndlib_10generators_AdjustableTriangleWave;


/* "generators.pxd":748
 * # --------------- adjustable-sawtooth-wave ---------------- #
 * 
 * cdef class AdjustableSawtoothWave(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double duty_factor
 */

struct __pyx_vtabstruct_8pysndlib_10generators_AdjustableSawtoothWave {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_AdjustableSawtoothWave *__pyx_vtabptr_8pysndlib_10generators_AdjustableSawtoothWave;


/* "generators.pxd":763
 * # TODO: --------------- adjustable-oscil-wave ---------------- #
 * 
 * cdef class AdjustableOscil(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double duty_factor
 */

struct __pyx_vtabstruct_8pysndlib_10generators_AdjustableOscil {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_AdjustableOscil *__pyx_vtabptr_8pysndlib_10generators_AdjustableOscil;


/* "generators.pxd":784
 * 
 * # --------------- round-interp ---------------- #
 * cdef class RoundInterp(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.int n
 */

struct __pyx_vtabstruct_8pysndlib_10generators_RoundInterp {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_RoundInterp *__pyx_vtabptr_8pysndlib_10generators_RoundInterp;


/* "generators.pxd":814
 * 
 * # TODO: This is not working correctly --------------- nchoosekcos ---------------- #
 * cdef class Nchoosekcos(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double ratio
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Nchoosekcos {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Nchoosekcos *__pyx_vtabptr_8pysndlib_10generators_Nchoosekcos;


/* "generators.pxd":827
 * # TODO: --------------- sinc-train ---------------- #
 * 
 * cdef class SincTrain(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.int n
 */

struct __pyx_vtabstruct_8pysndlib_10generators_SincTrain {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_SincTrain *__pyx_vtabptr_8pysndlib_10generators_SincTrain;


/* "generators.pxd":838
 * 
 * # --------------- pink-noise ---------------- #
 * cdef class PinkNoise(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.int n
 *     cdef public np.ndarray data
 */

struct __pyx_vtabstruct_8pysndlib_10generators_PinkNoise {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_PinkNoise *__pyx_vtabptr_8pysndlib_10generators_PinkNoise;


/* "generators.pxd":846
 * 
 * # --------------- brown-noise ---------------- #
 * cdef class BrownNoise(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double amplitude
 */

struct __pyx_vtabstruct_8pysndlib_10generators_BrownNoise {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_BrownNoise *__pyx_vtabptr_8pysndlib_10generators_BrownNoise;


/* "generators.pxd":858
 * #
 * # --------------- green-noise ---------------- #
 * cdef class GreenNoise(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double amplitude
 */

struct __pyx_vtabstruct_8pysndlib_10generators_GreenNoise {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_GreenNoise *__pyx_vtabptr_8pysndlib_10generators_GreenNoise;


/* "generators.pxd":868
 * #
 * # TODO: This is not working properly--------------- green-noise-interp ---------------- #
 * cdef class GreenNoiseInterp(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double amplitude
 */

struct __pyx_vtabstruct_8pysndlib_10generators_GreenNoiseInterp {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_GreenNoiseInterp *__pyx_vtabptr_8pysndlib_10generators_GreenNoiseInterp;


/* "generators.pxd":882
 * # --------------- moving-sum ---------------- #
 * 
 * cdef class MovingSum(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.int n
 *     cdef public clm.mus_any gen
 */

struct __pyx_vtabstruct_8pysndlib_10generators_MovingSum {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_MovingSum *__pyx_vtabptr_8pysndlib_10generators_MovingSum;


/* "generators.pxd":891
 * # --------------- moving-variance ---------------- #
 * 
 * cdef class MovingVariance(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.int n
 *     cdef public clm.mus_any gen1
 */

struct __pyx_vtabstruct_8pysndlib_10generators_MovingVariance {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_MovingVariance *__pyx_vtabptr_8pysndlib_10generators_MovingVariance;


/* "generators.pxd":901
 * # --------------- moving-rms ---------------- #
 * 
 * cdef class MovingRMS(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.int n
 *     cdef public clm.mus_any gen
 */

struct __pyx_vtabstruct_8pysndlib_10generators_MovingRMS {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_MovingRMS *__pyx_vtabptr_8pysndlib_10generators_MovingRMS;


/* "generators.pxd":910
 * # --------------- moving-length ---------------- #
 * 
 * cdef class MovingLength(MovingRMS):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */

struct __pyx_vtabstruct_8pysndlib_10generators_MovingLength {
  struct __pyx_vtabstruct_8pysndlib_10generators_MovingRMS __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_MovingLength *__pyx_vtabptr_8pysndlib_10generators_MovingLength;


/* "generators.pxd":918
 * # --------------- weighted-moving-average---------------- #
 * 
 * cdef class WeightedMovingAverage(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.int n
 *     cdef public clm.mus_any dly
 */

struct __pyx_vtabstruct_8pysndlib_10generators_WeightedMovingAverage {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_WeightedMovingAverage *__pyx_vtabptr_8pysndlib_10generators_WeightedMovingAverage;


/* "generators.pxd":931
 * # --------------- exponentially-weighted-moving-average---------------- #
 * 
 * cdef class ExponentiallyWeightedMovingAverage(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.int n
 *     cdef public clm.mus_any gen
 */

struct __pyx_vtabstruct_8pysndlib_10generators_ExponentiallyWeightedMovingAverage {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_ExponentiallyWeightedMovingAverage *__pyx_vtabptr_8pysndlib_10generators_ExponentiallyWeightedMovingAverage;


/* "generators.pxd":956
 * 
 * # --------------- tanhsin ---------------- #
 * cdef class Tanhsin(CLMGenerator):             # <<<<<<<<<<<<<<
 *     cdef public cython.double frequency
 *     cdef public cython.double r
 */

struct __pyx_vtabstruct_8pysndlib_10generators_Tanhsin {
  struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator __pyx_base;
};
static struct __pyx_vtabstruct_8pysndlib_10generators_Tanhsin *__pyx_vtabptr_8pysndlib_10generators_Tanhsin;
/* #### Code section: utility_code_proto ### */

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, Py_ssize_t);
    void (*DECREF)(void*, PyObject*, Py_ssize_t);
    void (*GOTREF)(void*, PyObject*, Py_ssize_t);
    void (*GIVEREF)(void*, PyObject*, Py_ssize_t);
    void* (*SetupContext)(const char*, Py_ssize_t, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
          }
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__))
  #define __Pyx_RefNannyFinishContextNogil() __Pyx_RefNannyFinishContext()
#endif
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_XINCREF(r)  do { if((r) == NULL); else {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) == NULL); else {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) == NULL); else {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) == NULL); else {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContextNogil()
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_Py_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; Py_XDECREF(tmp);\
    } while (0)
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#if PY_VERSION_HEX >= 0x030C00A6
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->current_exception != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->current_exception ? (PyObject*) Py_TYPE(__pyx_tstate->current_exception) : (PyObject*) NULL)
#else
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->curexc_type != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->curexc_type)
#endif
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  (PyErr_Occurred() != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A6
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK && CYTHON_FAST_THREAD_STATE
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  do {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  do {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_MultiplyObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_MultiplyObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceMultiply(op1, op2) : PyNumber_Multiply(op1, op2))
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#if !CYTHON_VECTORCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
#if !CYTHON_VECTORCALL
#if PY_VERSION_HEX >= 0x03080000
  #include "frameobject.h"
#if PY_VERSION_HEX >= 0x030b00a6 && !CYTHON_COMPILING_IN_LIMITED_API
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
  #define __Pxy_PyFrame_Initialize_Offsets()
  #define __Pyx_PyFrame_GetLocalsplus(frame)  ((frame)->f_localsplus)
#else
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif
#endif
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectFastCall.proto */
#define __Pyx_PyObject_FastCall(func, args, nargs)  __Pyx_PyObject_FastCallDict(func, args, (size_t)(nargs), NULL)
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject **args, size_t nargs, PyObject *kwargs);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* SetItemInt.proto */
#define __Pyx_SetItemInt(o, i, v, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_SetItemInt_Fast(o, (Py_ssize_t)i, v, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list assignment index out of range"), -1) :\
               __Pyx_SetItemInt_Generic(o, to_py_func(i), v)))
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v);
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v,
                                               int is_list, int wraparound, int boundscheck);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o, n, NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value);
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* TupleAndListFromArray.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n);
static CYTHON_INLINE PyObject* __Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n);
#endif

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* fastcall.proto */
#if CYTHON_AVOID_BORROWED_REFS
    #define __Pyx_Arg_VARARGS(args, i) PySequence_GetItem(args, i)
#elif CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_Arg_VARARGS(args, i) PyTuple_GET_ITEM(args, i)
#else
    #define __Pyx_Arg_VARARGS(args, i) PyTuple_GetItem(args, i)
#endif
#if CYTHON_AVOID_BORROWED_REFS
    #define __Pyx_Arg_NewRef_VARARGS(arg) __Pyx_NewRef(arg)
    #define __Pyx_Arg_XDECREF_VARARGS(arg) Py_XDECREF(arg)
#else
    #define __Pyx_Arg_NewRef_VARARGS(arg) arg
    #define __Pyx_Arg_XDECREF_VARARGS(arg)
#endif
#define __Pyx_NumKwargs_VARARGS(kwds) PyDict_Size(kwds)
#define __Pyx_KwValues_VARARGS(args, nargs) NULL
#define __Pyx_GetKwValue_VARARGS(kw, kwvalues, s) __Pyx_PyDict_GetItemStrWithError(kw, s)
#define __Pyx_KwargsAsDict_VARARGS(kw, kwvalues) PyDict_Copy(kw)
#if CYTHON_METH_FASTCALL
    #define __Pyx_Arg_FASTCALL(args, i) args[i]
    #define __Pyx_NumKwargs_FASTCALL(kwds) PyTuple_GET_SIZE(kwds)
    #define __Pyx_KwValues_FASTCALL(args, nargs) ((args) + (nargs))
    static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
    CYTHON_UNUSED static PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues);
  #else
    #define __Pyx_KwargsAsDict_FASTCALL(kw, kwvalues) _PyStack_AsDict(kwvalues, kw)
  #endif
    #define __Pyx_Arg_NewRef_FASTCALL(arg) arg  /* no-op, __Pyx_Arg_FASTCALL is direct and this needs
                                                   to have the same reference counting */
    #define __Pyx_Arg_XDECREF_FASTCALL(arg)
#else
    #define __Pyx_Arg_FASTCALL __Pyx_Arg_VARARGS
    #define __Pyx_NumKwargs_FASTCALL __Pyx_NumKwargs_VARARGS
    #define __Pyx_KwValues_FASTCALL __Pyx_KwValues_VARARGS
    #define __Pyx_GetKwValue_FASTCALL __Pyx_GetKwValue_VARARGS
    #define __Pyx_KwargsAsDict_FASTCALL __Pyx_KwargsAsDict_VARARGS
    #define __Pyx_Arg_NewRef_FASTCALL(arg) __Pyx_Arg_NewRef_VARARGS(arg)
    #define __Pyx_Arg_XDECREF_FASTCALL(arg) __Pyx_Arg_XDECREF_VARARGS(arg)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
#define __Pyx_ArgsSlice_VARARGS(args, start, stop) __Pyx_PyTuple_FromArray(&__Pyx_Arg_VARARGS(args, start), stop - start)
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) __Pyx_PyTuple_FromArray(&__Pyx_Arg_FASTCALL(args, start), stop - start)
#else
#define __Pyx_ArgsSlice_VARARGS(args, start, stop) PyTuple_GetSlice(args, start, stop)
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) PyTuple_GetSlice(args, start, stop)
#endif

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject *const *kwvalues,
    PyObject **argnames[],
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,
    const char* function_name);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto_3_0_8
#define __PYX_HAVE_RT_ImportType_proto_3_0_8
#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L
#include <stdalign.h>
#endif
#if (defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L) || __cplusplus >= 201103L
#define __PYX_GET_STRUCT_ALIGNMENT_3_0_8(s) alignof(s)
#else
#define __PYX_GET_STRUCT_ALIGNMENT_3_0_8(s) sizeof(void*)
#endif
enum __Pyx_ImportType_CheckSize_3_0_8 {
   __Pyx_ImportType_CheckSize_Error_3_0_8 = 0,
   __Pyx_ImportType_CheckSize_Warn_3_0_8 = 1,
   __Pyx_ImportType_CheckSize_Ignore_3_0_8 = 2
};
static PyTypeObject *__Pyx_ImportType_3_0_8(PyObject* module, const char *module_name, const char *class_name, size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_3_0_8 check_size);
#endif

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyTypeObject *type);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportDottedModule.proto */
static PyObject *__Pyx_ImportDottedModule(PyObject *name, PyObject *parts_tuple);
#if PY_MAJOR_VERSION >= 3
static PyObject *__Pyx_ImportDottedModule_WalkParts(PyObject *module, PyObject *name, PyObject *parts_tuple);
#endif

/* IncludeStructmemberH.proto */
#include <structmember.h>

/* FixUpExtensionType.proto */
#if CYTHON_USE_TYPE_SPECS
static int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type);
#endif

/* FetchSharedCythonModule.proto */
static PyObject *__Pyx_FetchSharedCythonABIModule(void);

/* FetchCommonType.proto */
#if !CYTHON_USE_TYPE_SPECS
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);
#else
static PyTypeObject* __Pyx_FetchCommonTypeFromSpec(PyObject *module, PyType_Spec *spec, PyObject *bases);
#endif

/* PyMethodNew.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ) {
    PyObject *typesModule=NULL, *methodType=NULL, *result=NULL;
    CYTHON_UNUSED_VAR(typ);
    if (!self)
        return __Pyx_NewRef(func);
    typesModule = PyImport_ImportModule("types");
    if (!typesModule) return NULL;
    methodType = PyObject_GetAttrString(typesModule, "MethodType");
    Py_DECREF(typesModule);
    if (!methodType) return NULL;
    result = PyObject_CallFunctionObjArgs(methodType, func, self, NULL);
    Py_DECREF(methodType);
    return result;
}
#elif PY_MAJOR_VERSION >= 3
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ) {
    CYTHON_UNUSED_VAR(typ);
    if (!self)
        return __Pyx_NewRef(func);
    return PyMethod_New(func, self);
}
#else
    #define __Pyx_PyMethod_New PyMethod_New
#endif

/* PyVectorcallFastCallDict.proto */
#if CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject *__Pyx_PyVectorcall_FastCallDict(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw);
#endif

/* CythonFunctionShared.proto */
#define __Pyx_CyFunction_USED
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CYFUNCTION_COROUTINE     0x08
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_CyFunction_GetClassObj(f)\
      (((__pyx_CyFunctionObject *) (f))->func_classobj)
#else
  #define __Pyx_CyFunction_GetClassObj(f)\
      ((PyObject*) ((PyCMethodObject *) (f))->mm_class)
#endif
#define __Pyx_CyFunction_SetClassObj(f, classobj)\
    __Pyx__CyFunction_SetClassObj((__pyx_CyFunctionObject *) (f), (classobj))
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject_HEAD
    PyObject *func;
#elif PY_VERSION_HEX < 0x030900B1
    PyCFunctionObject func;
#else
    PyCMethodObject func;
#endif
#if CYTHON_BACKPORT_VECTORCALL
    __pyx_vectorcallfunc func_vectorcall;
#endif
#if PY_VERSION_HEX < 0x030500A0 || CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_classobj;
#endif
    void *defaults;
    int defaults_pyobjects;
    size_t defaults_size;
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
    PyObject *func_is_coroutine;
} __pyx_CyFunctionObject;
#undef __Pyx_CyOrPyCFunction_Check
#define __Pyx_CyFunction_Check(obj)  __Pyx_TypeCheck(obj, __pyx_CyFunctionType)
#define __Pyx_CyOrPyCFunction_Check(obj)  __Pyx_TypeCheck2(obj, __pyx_CyFunctionType, &PyCFunction_Type)
#define __Pyx_CyFunction_CheckExact(obj)  __Pyx_IS_TYPE(obj, __pyx_CyFunctionType)
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void *cfunc);
#undef __Pyx_IsSameCFunction
#define __Pyx_IsSameCFunction(func, cfunc)   __Pyx__IsSameCyOrCFunction(func, cfunc)
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject* op, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void __Pyx__CyFunction_SetClassObj(__pyx_CyFunctionObject* f, PyObject* classobj);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(PyObject *module);
#if CYTHON_METH_FASTCALL
static PyObject * __Pyx_CyFunction_Vectorcall_NOARGS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_O(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
#if CYTHON_BACKPORT_VECTORCALL
#define __Pyx_CyFunction_func_vectorcall(f) (((__pyx_CyFunctionObject*)f)->func_vectorcall)
#else
#define __Pyx_CyFunction_func_vectorcall(f) (((PyCFunctionObject*)f)->vectorcall)
#endif
#endif

/* CythonFunction.proto */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);

/* PyObjectLookupSpecial.proto */
#if CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_LookupSpecialNoError(obj, attr_name)  __Pyx__PyObject_LookupSpecial(obj, attr_name, 0)
#define __Pyx_PyObject_LookupSpecial(obj, attr_name)  __Pyx__PyObject_LookupSpecial(obj, attr_name, 1)
static CYTHON_INLINE PyObject* __Pyx__PyObject_LookupSpecial(PyObject* obj, PyObject* attr_name, int with_error);
#else
#define __Pyx_PyObject_LookupSpecialNoError(o,n) __Pyx_PyObject_GetAttrStrNoError(o,n)
#define __Pyx_PyObject_LookupSpecial(o,n) __Pyx_PyObject_GetAttrStr(o,n)
#endif

/* PyObjectCallNoArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
#if !CYTHON_COMPILING_IN_LIMITED_API
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);
#endif

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* GCCDiagnostics.proto */
#if !defined(__INTEL_COMPILER) && defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* RealImag.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #define __Pyx_CREAL(z) ((z).real())
    #define __Pyx_CIMAG(z) ((z).imag())
  #else
    #define __Pyx_CREAL(z) (__real__(z))
    #define __Pyx_CIMAG(z) (__imag__(z))
  #endif
#else
    #define __Pyx_CREAL(z) ((z).real)
    #define __Pyx_CIMAG(z) ((z).imag)
#endif
#if defined(__cplusplus) && CYTHON_CCOMPLEX\
        && (defined(_WIN32) || defined(__clang__) || (defined(__GNUC__) && (__GNUC__ >= 5 || __GNUC__ == 4 && __GNUC_MINOR__ >= 4 )) || __cplusplus >= 201103)
    #define __Pyx_SET_CREAL(z,x) ((z).real(x))
    #define __Pyx_SET_CIMAG(z,y) ((z).imag(y))
#else
    #define __Pyx_SET_CREAL(z,x) __Pyx_CREAL(z) = (x)
    #define __Pyx_SET_CIMAG(z,y) __Pyx_CIMAG(z) = (y)
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
    #define __Pyx_c_eq_float(a, b)   ((a)==(b))
    #define __Pyx_c_sum_float(a, b)  ((a)+(b))
    #define __Pyx_c_diff_float(a, b) ((a)-(b))
    #define __Pyx_c_prod_float(a, b) ((a)*(b))
    #define __Pyx_c_quot_float(a, b) ((a)/(b))
    #define __Pyx_c_neg_float(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_float(z) ((z)==(float)0)
    #define __Pyx_c_conj_float(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (::std::abs(z))
        #define __Pyx_c_pow_float(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_float(z) ((z)==0)
    #define __Pyx_c_conj_float(z)    (conjf(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (cabsf(z))
        #define __Pyx_c_pow_float(a, b)  (cpowf(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex);
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex);
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex, __pyx_t_float_complex);
    #endif
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
    #define __Pyx_c_eq_double(a, b)   ((a)==(b))
    #define __Pyx_c_sum_double(a, b)  ((a)+(b))
    #define __Pyx_c_diff_double(a, b) ((a)-(b))
    #define __Pyx_c_prod_double(a, b) ((a)*(b))
    #define __Pyx_c_quot_double(a, b) ((a)/(b))
    #define __Pyx_c_neg_double(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_double(z) ((z)==(double)0)
    #define __Pyx_c_conj_double(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (::std::abs(z))
        #define __Pyx_c_pow_double(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_double(z) ((z)==0)
    #define __Pyx_c_conj_double(z)    (conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (cabs(z))
        #define __Pyx_c_pow_double(a, b)  (cpow(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex);
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex);
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex, __pyx_t_double_complex);
    #endif
#endif

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* FormatTypeName.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
typedef PyObject *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%U"
static __Pyx_TypeName __Pyx_PyType_GetName(PyTypeObject* tp);
#define __Pyx_DECREF_TypeName(obj) Py_XDECREF(obj)
#else
typedef const char *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%.200s"
#define __Pyx_PyType_GetName(tp) ((tp)->tp_name)
#define __Pyx_DECREF_TypeName(obj)
#endif

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) __Pyx_IsAnySubtype2(Py_TYPE(obj), (PyTypeObject *)type1, (PyTypeObject *)type2)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) (PyObject_TypeCheck(obj, (PyTypeObject *)type1) || PyObject_TypeCheck(obj, (PyTypeObject *)type2))
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyErr_ExceptionMatches2(err1, err2)  __Pyx_PyErr_GivenExceptionMatches2(__Pyx_PyErr_CurrentExceptionType(), err1, err2)
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static unsigned long __Pyx_get_runtime_version(void);
static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer);

/* FunctionImport.proto */
static int __Pyx_ImportFunction_3_0_8(PyObject *module, const char *funcname, void (**f)(void), const char *sig);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

/* #### Code section: module_declarations ### */
static CYTHON_INLINE PyObject *__pyx_f_5numpy_7ndarray_4base_base(PyArrayObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE PyArray_Descr *__pyx_f_5numpy_7ndarray_5descr_descr(PyArrayObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE int __pyx_f_5numpy_7ndarray_4ndim_ndim(PyArrayObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE npy_intp *__pyx_f_5numpy_7ndarray_5shape_shape(PyArrayObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE npy_intp *__pyx_f_5numpy_7ndarray_7strides_strides(PyArrayObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE npy_intp __pyx_f_5numpy_7ndarray_4size_size(PyArrayObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE char *__pyx_f_5numpy_7ndarray_4data_data(PyArrayObject *__pyx_v_self); /* proto*/

/* Module declarations from "cython" */

/* Module declarations from "pysndlib.csndlib" */

/* Module declarations from "pysndlib.cclm" */

/* Module declarations from "libc.string" */

/* Module declarations from "libc.stdio" */

/* Module declarations from "__builtin__" */

/* Module declarations from "cpython.type" */

/* Module declarations from "cpython" */

/* Module declarations from "cpython.object" */

/* Module declarations from "cpython.ref" */

/* Module declarations from "numpy" */

/* Module declarations from "numpy" */
static CYTHON_INLINE int __pyx_f_5numpy_import_array(void); /*proto*/

/* Module declarations from "pysndlib.sndlib" */

/* Module declarations from "pysndlib.clm" */
static long (*__pyx_f_8pysndlib_3clm_seconds2samples)(double, int __pyx_skip_dispatch); /*proto*/
static double (*__pyx_f_8pysndlib_3clm_oscil)(struct __pyx_obj_8pysndlib_3clm_mus_any *, int __pyx_skip_dispatch, struct __pyx_opt_args_8pysndlib_3clm_oscil *__pyx_optional_args); /*proto*/
static double (*__pyx_f_8pysndlib_3clm_env)(struct __pyx_obj_8pysndlib_3clm_mus_any *, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_8pysndlib_3clm_outa)(long, double, int __pyx_skip_dispatch, struct __pyx_opt_args_8pysndlib_3clm_outa *__pyx_optional_args); /*proto*/

/* Module declarations from "pysndlib" */

/* Module declarations from "pysndlib.generators" */
static double (*__pyx_f_8pysndlib_10generators_rkfssb)(struct __pyx_obj_8pysndlib_10generators_Rkfssb *, int __pyx_skip_dispatch, struct __pyx_opt_args_8pysndlib_10generators_rkfssb *__pyx_optional_args); /*proto*/

/* Module declarations from "pysndlib.instruments.bouncy" */
static PyObject *__pyx_f_8pysndlib_11instruments_6bouncy_bouncy(PyObject *, PyObject *, PyObject *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_8pysndlib_11instruments_6bouncy_bouncy *__pyx_optional_args); /*proto*/
/* #### Code section: typeinfo ### */
/* #### Code section: before_global_var ### */
#define __Pyx_MODULE_NAME "pysndlib.instruments.bouncy"
extern int __pyx_module_is_main_pysndlib__instruments__bouncy;
int __pyx_module_is_main_pysndlib__instruments__bouncy = 0;

/* Implementation of "pysndlib.instruments.bouncy" */
/* #### Code section: global_var ### */
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_ImportError;
/* #### Code section: string_decls ### */
static const char __pyx_k_r[] = "r";
static const char __pyx_k__3[] = "*";
static const char __pyx_k_np[] = "np";
static const char __pyx_k__10[] = "?";
static const char __pyx_k_amp[] = "amp";
static const char __pyx_k_clm[] = "clm";
static const char __pyx_k_dur[] = "dur";
static const char __pyx_k_base[] = "base";
static const char __pyx_k_exit[] = "__exit__";
static const char __pyx_k_fabs[] = "fabs";
static const char __pyx_k_freq[] = "freq";
static const char __pyx_k_gens[] = "gens";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_math[] = "math";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_play[] = "play";
static const char __pyx_k_spec[] = "__spec__";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_Sound[] = "Sound";
static const char __pyx_k_enter[] = "__enter__";
static const char __pyx_k_numpy[] = "numpy";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_start[] = "start";
static const char __pyx_k_zeros[] = "zeros";
static const char __pyx_k_bouncy[] = "bouncy";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_random[] = "random";
static const char __pyx_k_scaler[] = "scaler";
static const char __pyx_k_clipped[] = "clipped";
static const char __pyx_k_duration[] = "duration";
static const char __pyx_k_make_env[] = "make_env";
static const char __pyx_k_pysndlib[] = "pysndlib";
static const char __pyx_k_bounce_amp[] = "bounce_amp";
static const char __pyx_k_generators[] = "generators";
static const char __pyx_k_make_oscil[] = "make_oscil";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_statistics[] = "statistics";
static const char __pyx_k_ImportError[] = "ImportError";
static const char __pyx_k_bounce_freq[] = "bounce_freq";
static const char __pyx_k_make_rkfssb[] = "make_rkfssb";
static const char __pyx_k_initializing[] = "_initializing";
static const char __pyx_k_is_coroutine[] = "_is_coroutine";
static const char __pyx_k_pysndlib_clm[] = "pysndlib.clm";
static const char __pyx_k_asyncio_coroutines[] = "asyncio.coroutines";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_pysndlib_generators[] = "pysndlib.generators";
static const char __pyx_k_pysndlib_instruments_bouncy[] = "pysndlib.instruments.bouncy";
static const char __pyx_k_numpy_core_multiarray_failed_to[] = "numpy.core.multiarray failed to import";
static const char __pyx_k_src_pysndlib_instruments_bouncy[] = "src/pysndlib/instruments/bouncy.py";
static const char __pyx_k_numpy_core_umath_failed_to_impor[] = "numpy.core.umath failed to import";
/* #### Code section: decls ### */
static PyObject *__pyx_pf_8pysndlib_11instruments_6bouncy_bouncy(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_start, PyObject *__pyx_v_dur, PyObject *__pyx_v_freq, PyObject *__pyx_v_amp, PyObject *__pyx_v_bounce_freq, PyObject *__pyx_v_bounce_amp); /* proto */
/* #### Code section: late_includes ### */
/* #### Code section: module_state ### */
typedef struct {
  PyObject *__pyx_d;
  PyObject *__pyx_b;
  PyObject *__pyx_cython_runtime;
  PyObject *__pyx_empty_tuple;
  PyObject *__pyx_empty_bytes;
  PyObject *__pyx_empty_unicode;
  #ifdef __Pyx_CyFunction_USED
  PyTypeObject *__pyx_CyFunctionType;
  #endif
  #ifdef __Pyx_FusedFunction_USED
  PyTypeObject *__pyx_FusedFunctionType;
  #endif
  #ifdef __Pyx_Generator_USED
  PyTypeObject *__pyx_GeneratorType;
  #endif
  #ifdef __Pyx_IterableCoroutine_USED
  PyTypeObject *__pyx_IterableCoroutineType;
  #endif
  #ifdef __Pyx_Coroutine_USED
  PyTypeObject *__pyx_CoroutineAwaitType;
  #endif
  #ifdef __Pyx_Coroutine_USED
  PyTypeObject *__pyx_CoroutineType;
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  PyTypeObject *__pyx_ptype_7cpython_4type_type;
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  PyTypeObject *__pyx_ptype_5numpy_dtype;
  PyTypeObject *__pyx_ptype_5numpy_flatiter;
  PyTypeObject *__pyx_ptype_5numpy_broadcast;
  PyTypeObject *__pyx_ptype_5numpy_ndarray;
  PyTypeObject *__pyx_ptype_5numpy_generic;
  PyTypeObject *__pyx_ptype_5numpy_number;
  PyTypeObject *__pyx_ptype_5numpy_integer;
  PyTypeObject *__pyx_ptype_5numpy_signedinteger;
  PyTypeObject *__pyx_ptype_5numpy_unsignedinteger;
  PyTypeObject *__pyx_ptype_5numpy_inexact;
  PyTypeObject *__pyx_ptype_5numpy_floating;
  PyTypeObject *__pyx_ptype_5numpy_complexfloating;
  PyTypeObject *__pyx_ptype_5numpy_flexible;
  PyTypeObject *__pyx_ptype_5numpy_character;
  PyTypeObject *__pyx_ptype_5numpy_ufunc;
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  PyTypeObject *__pyx_ptype_8pysndlib_3clm_mus_any;
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_CLMGenerator;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Nssb;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Nxysin;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Nxycos;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Nxy1cos;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Nxy1sin;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Noddcos;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Noddssb;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Ncos2;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Ncos4;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Npcos;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Ncos5;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Nsin5;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Nrcos;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Nrssb;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Nkssb;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Nsincos;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_N1cos;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Npos1cos;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Npos3cos;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Rcos;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Rssb;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Rxysin;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Rxycos;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_SafeRxycos;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Ercos;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Erssb;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_R2cos;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_R2ssb;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Eoddcos;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Rkcos;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Rksin;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Rkssb;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Rkfcos;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Rkfssb;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_R2kfcos;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_K2sin;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_K2cos;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_K2ssb;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Dblsum;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Rkoddssb;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Krksin;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Absin;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Abcos;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_R2k2cos;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Blsaw;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Asyfm;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Bess;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Jjcos;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_J0evencos;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_J2cos;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Jpcos;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Jncos;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_J0j1cos;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Jycos;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Jcos;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Blackman;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Fmssb;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_K3sin;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Izcos;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_AdjustableSquareWave;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_AdjustableTriangleWave;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_AdjustableSawtoothWave;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_AdjustableOscil;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_RoundInterp;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Nchoosekcos;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_SincTrain;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_PinkNoise;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_BrownNoise;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_GreenNoise;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_GreenNoiseInterp;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_MovingSum;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_MovingVariance;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_MovingRMS;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_MovingLength;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_WeightedMovingAverage;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_ExponentiallyWeightedMovingAverage;
  PyTypeObject *__pyx_ptype_8pysndlib_10generators_Tanhsin;
  #if CYTHON_USE_MODULE_STATE
  #endif
  PyObject *__pyx_n_s_ImportError;
  PyObject *__pyx_n_s_Sound;
  PyObject *__pyx_n_s__10;
  PyObject *__pyx_n_s__3;
  PyObject *__pyx_n_s_amp;
  PyObject *__pyx_n_s_asyncio_coroutines;
  PyObject *__pyx_n_s_base;
  PyObject *__pyx_n_s_bounce_amp;
  PyObject *__pyx_n_s_bounce_freq;
  PyObject *__pyx_n_s_bouncy;
  PyObject *__pyx_n_s_cline_in_traceback;
  PyObject *__pyx_n_s_clipped;
  PyObject *__pyx_n_s_clm;
  PyObject *__pyx_n_s_dur;
  PyObject *__pyx_n_s_duration;
  PyObject *__pyx_n_s_enter;
  PyObject *__pyx_n_s_exit;
  PyObject *__pyx_n_s_fabs;
  PyObject *__pyx_n_s_freq;
  PyObject *__pyx_n_s_generators;
  PyObject *__pyx_n_s_gens;
  PyObject *__pyx_n_s_import;
  PyObject *__pyx_n_s_initializing;
  PyObject *__pyx_n_s_is_coroutine;
  PyObject *__pyx_n_s_main;
  PyObject *__pyx_n_u_main;
  PyObject *__pyx_n_s_make_env;
  PyObject *__pyx_n_s_make_oscil;
  PyObject *__pyx_n_s_make_rkfssb;
  PyObject *__pyx_n_s_math;
  PyObject *__pyx_n_s_name;
  PyObject *__pyx_n_s_np;
  PyObject *__pyx_n_s_numpy;
  PyObject *__pyx_kp_u_numpy_core_multiarray_failed_to;
  PyObject *__pyx_kp_u_numpy_core_umath_failed_to_impor;
  PyObject *__pyx_n_s_play;
  PyObject *__pyx_n_s_pysndlib;
  PyObject *__pyx_n_s_pysndlib_clm;
  PyObject *__pyx_n_s_pysndlib_generators;
  PyObject *__pyx_n_s_pysndlib_instruments_bouncy;
  PyObject *__pyx_n_s_pyx_vtable;
  PyObject *__pyx_n_s_r;
  PyObject *__pyx_n_s_random;
  PyObject *__pyx_n_s_range;
  PyObject *__pyx_n_s_scaler;
  PyObject *__pyx_n_s_spec;
  PyObject *__pyx_kp_s_src_pysndlib_instruments_bouncy;
  PyObject *__pyx_n_s_start;
  PyObject *__pyx_n_s_statistics;
  PyObject *__pyx_n_s_test;
  PyObject *__pyx_n_s_zeros;
  PyObject *__pyx_float__5;
  PyObject *__pyx_float_1_;
  PyObject *__pyx_float__01;
  PyObject *__pyx_float__03;
  PyObject *__pyx_float__15;
  PyObject *__pyx_float_0_0;
  PyObject *__pyx_float_1_0;
  PyObject *__pyx_float_2_5;
  PyObject *__pyx_int_0;
  PyObject *__pyx_int_1;
  PyObject *__pyx_int_2;
  PyObject *__pyx_int_3;
  PyObject *__pyx_int_4;
  PyObject *__pyx_int_5;
  PyObject *__pyx_int_10;
  PyObject *__pyx_int_20;
  PyObject *__pyx_int_32;
  PyObject *__pyx_int_200;
  PyObject *__pyx_int_300;
  PyObject *__pyx_tuple_;
  PyObject *__pyx_tuple__2;
  PyObject *__pyx_tuple__4;
  PyObject *__pyx_tuple__5;
  PyObject *__pyx_tuple__6;
  PyObject *__pyx_tuple__8;
  PyObject *__pyx_tuple__9;
  PyObject *__pyx_codeobj__7;
} __pyx_mstate;

#if CYTHON_USE_MODULE_STATE
#ifdef __cplusplus
namespace {
  extern struct PyModuleDef __pyx_moduledef;
} /* anonymous namespace */
#else
static struct PyModuleDef __pyx_moduledef;
#endif

#define __pyx_mstate(o) ((__pyx_mstate *)__Pyx_PyModule_GetState(o))

#define __pyx_mstate_global (__pyx_mstate(PyState_FindModule(&__pyx_moduledef)))

#define __pyx_m (PyState_FindModule(&__pyx_moduledef))
#else
static __pyx_mstate __pyx_mstate_global_static =
#ifdef __cplusplus
    {};
#else
    {0};
#endif
static __pyx_mstate *__pyx_mstate_global = &__pyx_mstate_global_static;
#endif
/* #### Code section: module_state_clear ### */
#if CYTHON_USE_MODULE_STATE
static int __pyx_m_clear(PyObject *m) {
  __pyx_mstate *clear_module_state = __pyx_mstate(m);
  if (!clear_module_state) return 0;
  Py_CLEAR(clear_module_state->__pyx_d);
  Py_CLEAR(clear_module_state->__pyx_b);
  Py_CLEAR(clear_module_state->__pyx_cython_runtime);
  Py_CLEAR(clear_module_state->__pyx_empty_tuple);
  Py_CLEAR(clear_module_state->__pyx_empty_bytes);
  Py_CLEAR(clear_module_state->__pyx_empty_unicode);
  #ifdef __Pyx_CyFunction_USED
  Py_CLEAR(clear_module_state->__pyx_CyFunctionType);
  #endif
  #ifdef __Pyx_FusedFunction_USED
  Py_CLEAR(clear_module_state->__pyx_FusedFunctionType);
  #endif
  Py_CLEAR(clear_module_state->__pyx_ptype_7cpython_4type_type);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_dtype);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_flatiter);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_broadcast);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_ndarray);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_generic);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_number);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_integer);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_signedinteger);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_unsignedinteger);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_inexact);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_floating);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_complexfloating);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_flexible);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_character);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_ufunc);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_3clm_mus_any);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_CLMGenerator);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Nssb);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Nxysin);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Nxycos);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Nxy1cos);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Nxy1sin);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Noddcos);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Noddssb);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Ncos2);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Ncos4);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Npcos);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Ncos5);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Nsin5);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Nrcos);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Nrssb);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Nkssb);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Nsincos);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_N1cos);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Npos1cos);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Npos3cos);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Rcos);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Rssb);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Rxysin);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Rxycos);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_SafeRxycos);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Ercos);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Erssb);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_R2cos);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_R2ssb);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Eoddcos);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Rkcos);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Rksin);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Rkssb);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Rkfcos);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Rkfssb);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_R2kfcos);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_K2sin);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_K2cos);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_K2ssb);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Dblsum);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Rkoddssb);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Krksin);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Absin);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Abcos);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_R2k2cos);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Blsaw);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Asyfm);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Bess);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Jjcos);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_J0evencos);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_J2cos);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Jpcos);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Jncos);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_J0j1cos);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Jycos);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Jcos);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Blackman);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Fmssb);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_K3sin);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Izcos);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_AdjustableSquareWave);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_AdjustableTriangleWave);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_AdjustableSawtoothWave);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_AdjustableOscil);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_RoundInterp);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Nchoosekcos);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_SincTrain);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_PinkNoise);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_BrownNoise);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_GreenNoise);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_GreenNoiseInterp);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_MovingSum);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_MovingVariance);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_MovingRMS);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_MovingLength);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_WeightedMovingAverage);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_ExponentiallyWeightedMovingAverage);
  Py_CLEAR(clear_module_state->__pyx_ptype_8pysndlib_10generators_Tanhsin);
  Py_CLEAR(clear_module_state->__pyx_n_s_ImportError);
  Py_CLEAR(clear_module_state->__pyx_n_s_Sound);
  Py_CLEAR(clear_module_state->__pyx_n_s__10);
  Py_CLEAR(clear_module_state->__pyx_n_s__3);
  Py_CLEAR(clear_module_state->__pyx_n_s_amp);
  Py_CLEAR(clear_module_state->__pyx_n_s_asyncio_coroutines);
  Py_CLEAR(clear_module_state->__pyx_n_s_base);
  Py_CLEAR(clear_module_state->__pyx_n_s_bounce_amp);
  Py_CLEAR(clear_module_state->__pyx_n_s_bounce_freq);
  Py_CLEAR(clear_module_state->__pyx_n_s_bouncy);
  Py_CLEAR(clear_module_state->__pyx_n_s_cline_in_traceback);
  Py_CLEAR(clear_module_state->__pyx_n_s_clipped);
  Py_CLEAR(clear_module_state->__pyx_n_s_clm);
  Py_CLEAR(clear_module_state->__pyx_n_s_dur);
  Py_CLEAR(clear_module_state->__pyx_n_s_duration);
  Py_CLEAR(clear_module_state->__pyx_n_s_enter);
  Py_CLEAR(clear_module_state->__pyx_n_s_exit);
  Py_CLEAR(clear_module_state->__pyx_n_s_fabs);
  Py_CLEAR(clear_module_state->__pyx_n_s_freq);
  Py_CLEAR(clear_module_state->__pyx_n_s_generators);
  Py_CLEAR(clear_module_state->__pyx_n_s_gens);
  Py_CLEAR(clear_module_state->__pyx_n_s_import);
  Py_CLEAR(clear_module_state->__pyx_n_s_initializing);
  Py_CLEAR(clear_module_state->__pyx_n_s_is_coroutine);
  Py_CLEAR(clear_module_state->__pyx_n_s_main);
  Py_CLEAR(clear_module_state->__pyx_n_u_main);
  Py_CLEAR(clear_module_state->__pyx_n_s_make_env);
  Py_CLEAR(clear_module_state->__pyx_n_s_make_oscil);
  Py_CLEAR(clear_module_state->__pyx_n_s_make_rkfssb);
  Py_CLEAR(clear_module_state->__pyx_n_s_math);
  Py_CLEAR(clear_module_state->__pyx_n_s_name);
  Py_CLEAR(clear_module_state->__pyx_n_s_np);
  Py_CLEAR(clear_module_state->__pyx_n_s_numpy);
  Py_CLEAR(clear_module_state->__pyx_kp_u_numpy_core_multiarray_failed_to);
  Py_CLEAR(clear_module_state->__pyx_kp_u_numpy_core_umath_failed_to_impor);
  Py_CLEAR(clear_module_state->__pyx_n_s_play);
  Py_CLEAR(clear_module_state->__pyx_n_s_pysndlib);
  Py_CLEAR(clear_module_state->__pyx_n_s_pysndlib_clm);
  Py_CLEAR(clear_module_state->__pyx_n_s_pysndlib_generators);
  Py_CLEAR(clear_module_state->__pyx_n_s_pysndlib_instruments_bouncy);
  Py_CLEAR(clear_module_state->__pyx_n_s_pyx_vtable);
  Py_CLEAR(clear_module_state->__pyx_n_s_r);
  Py_CLEAR(clear_module_state->__pyx_n_s_random);
  Py_CLEAR(clear_module_state->__pyx_n_s_range);
  Py_CLEAR(clear_module_state->__pyx_n_s_scaler);
  Py_CLEAR(clear_module_state->__pyx_n_s_spec);
  Py_CLEAR(clear_module_state->__pyx_kp_s_src_pysndlib_instruments_bouncy);
  Py_CLEAR(clear_module_state->__pyx_n_s_start);
  Py_CLEAR(clear_module_state->__pyx_n_s_statistics);
  Py_CLEAR(clear_module_state->__pyx_n_s_test);
  Py_CLEAR(clear_module_state->__pyx_n_s_zeros);
  Py_CLEAR(clear_module_state->__pyx_float__5);
  Py_CLEAR(clear_module_state->__pyx_float_1_);
  Py_CLEAR(clear_module_state->__pyx_float__01);
  Py_CLEAR(clear_module_state->__pyx_float__03);
  Py_CLEAR(clear_module_state->__pyx_float__15);
  Py_CLEAR(clear_module_state->__pyx_float_0_0);
  Py_CLEAR(clear_module_state->__pyx_float_1_0);
  Py_CLEAR(clear_module_state->__pyx_float_2_5);
  Py_CLEAR(clear_module_state->__pyx_int_0);
  Py_CLEAR(clear_module_state->__pyx_int_1);
  Py_CLEAR(clear_module_state->__pyx_int_2);
  Py_CLEAR(clear_module_state->__pyx_int_3);
  Py_CLEAR(clear_module_state->__pyx_int_4);
  Py_CLEAR(clear_module_state->__pyx_int_5);
  Py_CLEAR(clear_module_state->__pyx_int_10);
  Py_CLEAR(clear_module_state->__pyx_int_20);
  Py_CLEAR(clear_module_state->__pyx_int_32);
  Py_CLEAR(clear_module_state->__pyx_int_200);
  Py_CLEAR(clear_module_state->__pyx_int_300);
  Py_CLEAR(clear_module_state->__pyx_tuple_);
  Py_CLEAR(clear_module_state->__pyx_tuple__2);
  Py_CLEAR(clear_module_state->__pyx_tuple__4);
  Py_CLEAR(clear_module_state->__pyx_tuple__5);
  Py_CLEAR(clear_module_state->__pyx_tuple__6);
  Py_CLEAR(clear_module_state->__pyx_tuple__8);
  Py_CLEAR(clear_module_state->__pyx_tuple__9);
  Py_CLEAR(clear_module_state->__pyx_codeobj__7);
  return 0;
}
#endif
/* #### Code section: module_state_traverse ### */
#if CYTHON_USE_MODULE_STATE
static int __pyx_m_traverse(PyObject *m, visitproc visit, void *arg) {
  __pyx_mstate *traverse_module_state = __pyx_mstate(m);
  if (!traverse_module_state) return 0;
  Py_VISIT(traverse_module_state->__pyx_d);
  Py_VISIT(traverse_module_state->__pyx_b);
  Py_VISIT(traverse_module_state->__pyx_cython_runtime);
  Py_VISIT(traverse_module_state->__pyx_empty_tuple);
  Py_VISIT(traverse_module_state->__pyx_empty_bytes);
  Py_VISIT(traverse_module_state->__pyx_empty_unicode);
  #ifdef __Pyx_CyFunction_USED
  Py_VISIT(traverse_module_state->__pyx_CyFunctionType);
  #endif
  #ifdef __Pyx_FusedFunction_USED
  Py_VISIT(traverse_module_state->__pyx_FusedFunctionType);
  #endif
  Py_VISIT(traverse_module_state->__pyx_ptype_7cpython_4type_type);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_dtype);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_flatiter);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_broadcast);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_ndarray);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_generic);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_number);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_integer);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_signedinteger);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_unsignedinteger);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_inexact);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_floating);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_complexfloating);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_flexible);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_character);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_ufunc);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_3clm_mus_any);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_CLMGenerator);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Nssb);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Nxysin);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Nxycos);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Nxy1cos);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Nxy1sin);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Noddcos);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Noddssb);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Ncos2);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Ncos4);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Npcos);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Ncos5);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Nsin5);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Nrcos);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Nrssb);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Nkssb);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Nsincos);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_N1cos);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Npos1cos);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Npos3cos);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Rcos);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Rssb);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Rxysin);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Rxycos);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_SafeRxycos);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Ercos);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Erssb);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_R2cos);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_R2ssb);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Eoddcos);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Rkcos);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Rksin);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Rkssb);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Rkfcos);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Rkfssb);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_R2kfcos);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_K2sin);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_K2cos);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_K2ssb);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Dblsum);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Rkoddssb);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Krksin);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Absin);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Abcos);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_R2k2cos);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Blsaw);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Asyfm);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Bess);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Jjcos);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_J0evencos);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_J2cos);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Jpcos);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Jncos);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_J0j1cos);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Jycos);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Jcos);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Blackman);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Fmssb);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_K3sin);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Izcos);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_AdjustableSquareWave);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_AdjustableTriangleWave);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_AdjustableSawtoothWave);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_AdjustableOscil);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_RoundInterp);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Nchoosekcos);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_SincTrain);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_PinkNoise);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_BrownNoise);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_GreenNoise);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_GreenNoiseInterp);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_MovingSum);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_MovingVariance);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_MovingRMS);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_MovingLength);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_WeightedMovingAverage);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_ExponentiallyWeightedMovingAverage);
  Py_VISIT(traverse_module_state->__pyx_ptype_8pysndlib_10generators_Tanhsin);
  Py_VISIT(traverse_module_state->__pyx_n_s_ImportError);
  Py_VISIT(traverse_module_state->__pyx_n_s_Sound);
  Py_VISIT(traverse_module_state->__pyx_n_s__10);
  Py_VISIT(traverse_module_state->__pyx_n_s__3);
  Py_VISIT(traverse_module_state->__pyx_n_s_amp);
  Py_VISIT(traverse_module_state->__pyx_n_s_asyncio_coroutines);
  Py_VISIT(traverse_module_state->__pyx_n_s_base);
  Py_VISIT(traverse_module_state->__pyx_n_s_bounce_amp);
  Py_VISIT(traverse_module_state->__pyx_n_s_bounce_freq);
  Py_VISIT(traverse_module_state->__pyx_n_s_bouncy);
  Py_VISIT(traverse_module_state->__pyx_n_s_cline_in_traceback);
  Py_VISIT(traverse_module_state->__pyx_n_s_clipped);
  Py_VISIT(traverse_module_state->__pyx_n_s_clm);
  Py_VISIT(traverse_module_state->__pyx_n_s_dur);
  Py_VISIT(traverse_module_state->__pyx_n_s_duration);
  Py_VISIT(traverse_module_state->__pyx_n_s_enter);
  Py_VISIT(traverse_module_state->__pyx_n_s_exit);
  Py_VISIT(traverse_module_state->__pyx_n_s_fabs);
  Py_VISIT(traverse_module_state->__pyx_n_s_freq);
  Py_VISIT(traverse_module_state->__pyx_n_s_generators);
  Py_VISIT(traverse_module_state->__pyx_n_s_gens);
  Py_VISIT(traverse_module_state->__pyx_n_s_import);
  Py_VISIT(traverse_module_state->__pyx_n_s_initializing);
  Py_VISIT(traverse_module_state->__pyx_n_s_is_coroutine);
  Py_VISIT(traverse_module_state->__pyx_n_s_main);
  Py_VISIT(traverse_module_state->__pyx_n_u_main);
  Py_VISIT(traverse_module_state->__pyx_n_s_make_env);
  Py_VISIT(traverse_module_state->__pyx_n_s_make_oscil);
  Py_VISIT(traverse_module_state->__pyx_n_s_make_rkfssb);
  Py_VISIT(traverse_module_state->__pyx_n_s_math);
  Py_VISIT(traverse_module_state->__pyx_n_s_name);
  Py_VISIT(traverse_module_state->__pyx_n_s_np);
  Py_VISIT(traverse_module_state->__pyx_n_s_numpy);
  Py_VISIT(traverse_module_state->__pyx_kp_u_numpy_core_multiarray_failed_to);
  Py_VISIT(traverse_module_state->__pyx_kp_u_numpy_core_umath_failed_to_impor);
  Py_VISIT(traverse_module_state->__pyx_n_s_play);
  Py_VISIT(traverse_module_state->__pyx_n_s_pysndlib);
  Py_VISIT(traverse_module_state->__pyx_n_s_pysndlib_clm);
  Py_VISIT(traverse_module_state->__pyx_n_s_pysndlib_generators);
  Py_VISIT(traverse_module_state->__pyx_n_s_pysndlib_instruments_bouncy);
  Py_VISIT(traverse_module_state->__pyx_n_s_pyx_vtable);
  Py_VISIT(traverse_module_state->__pyx_n_s_r);
  Py_VISIT(traverse_module_state->__pyx_n_s_random);
  Py_VISIT(traverse_module_state->__pyx_n_s_range);
  Py_VISIT(traverse_module_state->__pyx_n_s_scaler);
  Py_VISIT(traverse_module_state->__pyx_n_s_spec);
  Py_VISIT(traverse_module_state->__pyx_kp_s_src_pysndlib_instruments_bouncy);
  Py_VISIT(traverse_module_state->__pyx_n_s_start);
  Py_VISIT(traverse_module_state->__pyx_n_s_statistics);
  Py_VISIT(traverse_module_state->__pyx_n_s_test);
  Py_VISIT(traverse_module_state->__pyx_n_s_zeros);
  Py_VISIT(traverse_module_state->__pyx_float__5);
  Py_VISIT(traverse_module_state->__pyx_float_1_);
  Py_VISIT(traverse_module_state->__pyx_float__01);
  Py_VISIT(traverse_module_state->__pyx_float__03);
  Py_VISIT(traverse_module_state->__pyx_float__15);
  Py_VISIT(traverse_module_state->__pyx_float_0_0);
  Py_VISIT(traverse_module_state->__pyx_float_1_0);
  Py_VISIT(traverse_module_state->__pyx_float_2_5);
  Py_VISIT(traverse_module_state->__pyx_int_0);
  Py_VISIT(traverse_module_state->__pyx_int_1);
  Py_VISIT(traverse_module_state->__pyx_int_2);
  Py_VISIT(traverse_module_state->__pyx_int_3);
  Py_VISIT(traverse_module_state->__pyx_int_4);
  Py_VISIT(traverse_module_state->__pyx_int_5);
  Py_VISIT(traverse_module_state->__pyx_int_10);
  Py_VISIT(traverse_module_state->__pyx_int_20);
  Py_VISIT(traverse_module_state->__pyx_int_32);
  Py_VISIT(traverse_module_state->__pyx_int_200);
  Py_VISIT(traverse_module_state->__pyx_int_300);
  Py_VISIT(traverse_module_state->__pyx_tuple_);
  Py_VISIT(traverse_module_state->__pyx_tuple__2);
  Py_VISIT(traverse_module_state->__pyx_tuple__4);
  Py_VISIT(traverse_module_state->__pyx_tuple__5);
  Py_VISIT(traverse_module_state->__pyx_tuple__6);
  Py_VISIT(traverse_module_state->__pyx_tuple__8);
  Py_VISIT(traverse_module_state->__pyx_tuple__9);
  Py_VISIT(traverse_module_state->__pyx_codeobj__7);
  return 0;
}
#endif
/* #### Code section: module_state_defines ### */
#define __pyx_d __pyx_mstate_global->__pyx_d
#define __pyx_b __pyx_mstate_global->__pyx_b
#define __pyx_cython_runtime __pyx_mstate_global->__pyx_cython_runtime
#define __pyx_empty_tuple __pyx_mstate_global->__pyx_empty_tuple
#define __pyx_empty_bytes __pyx_mstate_global->__pyx_empty_bytes
#define __pyx_empty_unicode __pyx_mstate_global->__pyx_empty_unicode
#ifdef __Pyx_CyFunction_USED
#define __pyx_CyFunctionType __pyx_mstate_global->__pyx_CyFunctionType
#endif
#ifdef __Pyx_FusedFunction_USED
#define __pyx_FusedFunctionType __pyx_mstate_global->__pyx_FusedFunctionType
#endif
#ifdef __Pyx_Generator_USED
#define __pyx_GeneratorType __pyx_mstate_global->__pyx_GeneratorType
#endif
#ifdef __Pyx_IterableCoroutine_USED
#define __pyx_IterableCoroutineType __pyx_mstate_global->__pyx_IterableCoroutineType
#endif
#ifdef __Pyx_Coroutine_USED
#define __pyx_CoroutineAwaitType __pyx_mstate_global->__pyx_CoroutineAwaitType
#endif
#ifdef __Pyx_Coroutine_USED
#define __pyx_CoroutineType __pyx_mstate_global->__pyx_CoroutineType
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#define __pyx_ptype_7cpython_4type_type __pyx_mstate_global->__pyx_ptype_7cpython_4type_type
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#define __pyx_ptype_5numpy_dtype __pyx_mstate_global->__pyx_ptype_5numpy_dtype
#define __pyx_ptype_5numpy_flatiter __pyx_mstate_global->__pyx_ptype_5numpy_flatiter
#define __pyx_ptype_5numpy_broadcast __pyx_mstate_global->__pyx_ptype_5numpy_broadcast
#define __pyx_ptype_5numpy_ndarray __pyx_mstate_global->__pyx_ptype_5numpy_ndarray
#define __pyx_ptype_5numpy_generic __pyx_mstate_global->__pyx_ptype_5numpy_generic
#define __pyx_ptype_5numpy_number __pyx_mstate_global->__pyx_ptype_5numpy_number
#define __pyx_ptype_5numpy_integer __pyx_mstate_global->__pyx_ptype_5numpy_integer
#define __pyx_ptype_5numpy_signedinteger __pyx_mstate_global->__pyx_ptype_5numpy_signedinteger
#define __pyx_ptype_5numpy_unsignedinteger __pyx_mstate_global->__pyx_ptype_5numpy_unsignedinteger
#define __pyx_ptype_5numpy_inexact __pyx_mstate_global->__pyx_ptype_5numpy_inexact
#define __pyx_ptype_5numpy_floating __pyx_mstate_global->__pyx_ptype_5numpy_floating
#define __pyx_ptype_5numpy_complexfloating __pyx_mstate_global->__pyx_ptype_5numpy_complexfloating
#define __pyx_ptype_5numpy_flexible __pyx_mstate_global->__pyx_ptype_5numpy_flexible
#define __pyx_ptype_5numpy_character __pyx_mstate_global->__pyx_ptype_5numpy_character
#define __pyx_ptype_5numpy_ufunc __pyx_mstate_global->__pyx_ptype_5numpy_ufunc
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#define __pyx_ptype_8pysndlib_3clm_mus_any __pyx_mstate_global->__pyx_ptype_8pysndlib_3clm_mus_any
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#define __pyx_ptype_8pysndlib_10generators_CLMGenerator __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_CLMGenerator
#define __pyx_ptype_8pysndlib_10generators_Nssb __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Nssb
#define __pyx_ptype_8pysndlib_10generators_Nxysin __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Nxysin
#define __pyx_ptype_8pysndlib_10generators_Nxycos __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Nxycos
#define __pyx_ptype_8pysndlib_10generators_Nxy1cos __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Nxy1cos
#define __pyx_ptype_8pysndlib_10generators_Nxy1sin __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Nxy1sin
#define __pyx_ptype_8pysndlib_10generators_Noddcos __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Noddcos
#define __pyx_ptype_8pysndlib_10generators_Noddssb __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Noddssb
#define __pyx_ptype_8pysndlib_10generators_Ncos2 __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Ncos2
#define __pyx_ptype_8pysndlib_10generators_Ncos4 __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Ncos4
#define __pyx_ptype_8pysndlib_10generators_Npcos __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Npcos
#define __pyx_ptype_8pysndlib_10generators_Ncos5 __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Ncos5
#define __pyx_ptype_8pysndlib_10generators_Nsin5 __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Nsin5
#define __pyx_ptype_8pysndlib_10generators_Nrcos __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Nrcos
#define __pyx_ptype_8pysndlib_10generators_Nrssb __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Nrssb
#define __pyx_ptype_8pysndlib_10generators_Nkssb __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Nkssb
#define __pyx_ptype_8pysndlib_10generators_Nsincos __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Nsincos
#define __pyx_ptype_8pysndlib_10generators_N1cos __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_N1cos
#define __pyx_ptype_8pysndlib_10generators_Npos1cos __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Npos1cos
#define __pyx_ptype_8pysndlib_10generators_Npos3cos __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Npos3cos
#define __pyx_ptype_8pysndlib_10generators_Rcos __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Rcos
#define __pyx_ptype_8pysndlib_10generators_Rssb __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Rssb
#define __pyx_ptype_8pysndlib_10generators_Rxysin __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Rxysin
#define __pyx_ptype_8pysndlib_10generators_Rxycos __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Rxycos
#define __pyx_ptype_8pysndlib_10generators_SafeRxycos __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_SafeRxycos
#define __pyx_ptype_8pysndlib_10generators_Ercos __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Ercos
#define __pyx_ptype_8pysndlib_10generators_Erssb __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Erssb
#define __pyx_ptype_8pysndlib_10generators_R2cos __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_R2cos
#define __pyx_ptype_8pysndlib_10generators_R2ssb __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_R2ssb
#define __pyx_ptype_8pysndlib_10generators_Eoddcos __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Eoddcos
#define __pyx_ptype_8pysndlib_10generators_Rkcos __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Rkcos
#define __pyx_ptype_8pysndlib_10generators_Rksin __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Rksin
#define __pyx_ptype_8pysndlib_10generators_Rkssb __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Rkssb
#define __pyx_ptype_8pysndlib_10generators_Rkfcos __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Rkfcos
#define __pyx_ptype_8pysndlib_10generators_Rkfssb __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Rkfssb
#define __pyx_ptype_8pysndlib_10generators_R2kfcos __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_R2kfcos
#define __pyx_ptype_8pysndlib_10generators_K2sin __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_K2sin
#define __pyx_ptype_8pysndlib_10generators_K2cos __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_K2cos
#define __pyx_ptype_8pysndlib_10generators_K2ssb __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_K2ssb
#define __pyx_ptype_8pysndlib_10generators_Dblsum __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Dblsum
#define __pyx_ptype_8pysndlib_10generators_Rkoddssb __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Rkoddssb
#define __pyx_ptype_8pysndlib_10generators_Krksin __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Krksin
#define __pyx_ptype_8pysndlib_10generators_Absin __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Absin
#define __pyx_ptype_8pysndlib_10generators_Abcos __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Abcos
#define __pyx_ptype_8pysndlib_10generators_R2k2cos __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_R2k2cos
#define __pyx_ptype_8pysndlib_10generators_Blsaw __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Blsaw
#define __pyx_ptype_8pysndlib_10generators_Asyfm __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Asyfm
#define __pyx_ptype_8pysndlib_10generators_Bess __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Bess
#define __pyx_ptype_8pysndlib_10generators_Jjcos __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Jjcos
#define __pyx_ptype_8pysndlib_10generators_J0evencos __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_J0evencos
#define __pyx_ptype_8pysndlib_10generators_J2cos __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_J2cos
#define __pyx_ptype_8pysndlib_10generators_Jpcos __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Jpcos
#define __pyx_ptype_8pysndlib_10generators_Jncos __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Jncos
#define __pyx_ptype_8pysndlib_10generators_J0j1cos __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_J0j1cos
#define __pyx_ptype_8pysndlib_10generators_Jycos __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Jycos
#define __pyx_ptype_8pysndlib_10generators_Jcos __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Jcos
#define __pyx_ptype_8pysndlib_10generators_Blackman __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Blackman
#define __pyx_ptype_8pysndlib_10generators_Fmssb __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Fmssb
#define __pyx_ptype_8pysndlib_10generators_K3sin __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_K3sin
#define __pyx_ptype_8pysndlib_10generators_Izcos __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Izcos
#define __pyx_ptype_8pysndlib_10generators_AdjustableSquareWave __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_AdjustableSquareWave
#define __pyx_ptype_8pysndlib_10generators_AdjustableTriangleWave __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_AdjustableTriangleWave
#define __pyx_ptype_8pysndlib_10generators_AdjustableSawtoothWave __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_AdjustableSawtoothWave
#define __pyx_ptype_8pysndlib_10generators_AdjustableOscil __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_AdjustableOscil
#define __pyx_ptype_8pysndlib_10generators_RoundInterp __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_RoundInterp
#define __pyx_ptype_8pysndlib_10generators_Nchoosekcos __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Nchoosekcos
#define __pyx_ptype_8pysndlib_10generators_SincTrain __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_SincTrain
#define __pyx_ptype_8pysndlib_10generators_PinkNoise __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_PinkNoise
#define __pyx_ptype_8pysndlib_10generators_BrownNoise __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_BrownNoise
#define __pyx_ptype_8pysndlib_10generators_GreenNoise __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_GreenNoise
#define __pyx_ptype_8pysndlib_10generators_GreenNoiseInterp __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_GreenNoiseInterp
#define __pyx_ptype_8pysndlib_10generators_MovingSum __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_MovingSum
#define __pyx_ptype_8pysndlib_10generators_MovingVariance __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_MovingVariance
#define __pyx_ptype_8pysndlib_10generators_MovingRMS __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_MovingRMS
#define __pyx_ptype_8pysndlib_10generators_MovingLength __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_MovingLength
#define __pyx_ptype_8pysndlib_10generators_WeightedMovingAverage __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_WeightedMovingAverage
#define __pyx_ptype_8pysndlib_10generators_ExponentiallyWeightedMovingAverage __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_ExponentiallyWeightedMovingAverage
#define __pyx_ptype_8pysndlib_10generators_Tanhsin __pyx_mstate_global->__pyx_ptype_8pysndlib_10generators_Tanhsin
#if CYTHON_USE_MODULE_STATE
#endif
#define __pyx_n_s_ImportError __pyx_mstate_global->__pyx_n_s_ImportError
#define __pyx_n_s_Sound __pyx_mstate_global->__pyx_n_s_Sound
#define __pyx_n_s__10 __pyx_mstate_global->__pyx_n_s__10
#define __pyx_n_s__3 __pyx_mstate_global->__pyx_n_s__3
#define __pyx_n_s_amp __pyx_mstate_global->__pyx_n_s_amp
#define __pyx_n_s_asyncio_coroutines __pyx_mstate_global->__pyx_n_s_asyncio_coroutines
#define __pyx_n_s_base __pyx_mstate_global->__pyx_n_s_base
#define __pyx_n_s_bounce_amp __pyx_mstate_global->__pyx_n_s_bounce_amp
#define __pyx_n_s_bounce_freq __pyx_mstate_global->__pyx_n_s_bounce_freq
#define __pyx_n_s_bouncy __pyx_mstate_global->__pyx_n_s_bouncy
#define __pyx_n_s_cline_in_traceback __pyx_mstate_global->__pyx_n_s_cline_in_traceback
#define __pyx_n_s_clipped __pyx_mstate_global->__pyx_n_s_clipped
#define __pyx_n_s_clm __pyx_mstate_global->__pyx_n_s_clm
#define __pyx_n_s_dur __pyx_mstate_global->__pyx_n_s_dur
#define __pyx_n_s_duration __pyx_mstate_global->__pyx_n_s_duration
#define __pyx_n_s_enter __pyx_mstate_global->__pyx_n_s_enter
#define __pyx_n_s_exit __pyx_mstate_global->__pyx_n_s_exit
#define __pyx_n_s_fabs __pyx_mstate_global->__pyx_n_s_fabs
#define __pyx_n_s_freq __pyx_mstate_global->__pyx_n_s_freq
#define __pyx_n_s_generators __pyx_mstate_global->__pyx_n_s_generators
#define __pyx_n_s_gens __pyx_mstate_global->__pyx_n_s_gens
#define __pyx_n_s_import __pyx_mstate_global->__pyx_n_s_import
#define __pyx_n_s_initializing __pyx_mstate_global->__pyx_n_s_initializing
#define __pyx_n_s_is_coroutine __pyx_mstate_global->__pyx_n_s_is_coroutine
#define __pyx_n_s_main __pyx_mstate_global->__pyx_n_s_main
#define __pyx_n_u_main __pyx_mstate_global->__pyx_n_u_main
#define __pyx_n_s_make_env __pyx_mstate_global->__pyx_n_s_make_env
#define __pyx_n_s_make_oscil __pyx_mstate_global->__pyx_n_s_make_oscil
#define __pyx_n_s_make_rkfssb __pyx_mstate_global->__pyx_n_s_make_rkfssb
#define __pyx_n_s_math __pyx_mstate_global->__pyx_n_s_math
#define __pyx_n_s_name __pyx_mstate_global->__pyx_n_s_name
#define __pyx_n_s_np __pyx_mstate_global->__pyx_n_s_np
#define __pyx_n_s_numpy __pyx_mstate_global->__pyx_n_s_numpy
#define __pyx_kp_u_numpy_core_multiarray_failed_to __pyx_mstate_global->__pyx_kp_u_numpy_core_multiarray_failed_to
#define __pyx_kp_u_numpy_core_umath_failed_to_impor __pyx_mstate_global->__pyx_kp_u_numpy_core_umath_failed_to_impor
#define __pyx_n_s_play __pyx_mstate_global->__pyx_n_s_play
#define __pyx_n_s_pysndlib __pyx_mstate_global->__pyx_n_s_pysndlib
#define __pyx_n_s_pysndlib_clm __pyx_mstate_global->__pyx_n_s_pysndlib_clm
#define __pyx_n_s_pysndlib_generators __pyx_mstate_global->__pyx_n_s_pysndlib_generators
#define __pyx_n_s_pysndlib_instruments_bouncy __pyx_mstate_global->__pyx_n_s_pysndlib_instruments_bouncy
#define __pyx_n_s_pyx_vtable __pyx_mstate_global->__pyx_n_s_pyx_vtable
#define __pyx_n_s_r __pyx_mstate_global->__pyx_n_s_r
#define __pyx_n_s_random __pyx_mstate_global->__pyx_n_s_random
#define __pyx_n_s_range __pyx_mstate_global->__pyx_n_s_range
#define __pyx_n_s_scaler __pyx_mstate_global->__pyx_n_s_scaler
#define __pyx_n_s_spec __pyx_mstate_global->__pyx_n_s_spec
#define __pyx_kp_s_src_pysndlib_instruments_bouncy __pyx_mstate_global->__pyx_kp_s_src_pysndlib_instruments_bouncy
#define __pyx_n_s_start __pyx_mstate_global->__pyx_n_s_start
#define __pyx_n_s_statistics __pyx_mstate_global->__pyx_n_s_statistics
#define __pyx_n_s_test __pyx_mstate_global->__pyx_n_s_test
#define __pyx_n_s_zeros __pyx_mstate_global->__pyx_n_s_zeros
#define __pyx_float__5 __pyx_mstate_global->__pyx_float__5
#define __pyx_float_1_ __pyx_mstate_global->__pyx_float_1_
#define __pyx_float__01 __pyx_mstate_global->__pyx_float__01
#define __pyx_float__03 __pyx_mstate_global->__pyx_float__03
#define __pyx_float__15 __pyx_mstate_global->__pyx_float__15
#define __pyx_float_0_0 __pyx_mstate_global->__pyx_float_0_0
#define __pyx_float_1_0 __pyx_mstate_global->__pyx_float_1_0
#define __pyx_float_2_5 __pyx_mstate_global->__pyx_float_2_5
#define __pyx_int_0 __pyx_mstate_global->__pyx_int_0
#define __pyx_int_1 __pyx_mstate_global->__pyx_int_1
#define __pyx_int_2 __pyx_mstate_global->__pyx_int_2
#define __pyx_int_3 __pyx_mstate_global->__pyx_int_3
#define __pyx_int_4 __pyx_mstate_global->__pyx_int_4
#define __pyx_int_5 __pyx_mstate_global->__pyx_int_5
#define __pyx_int_10 __pyx_mstate_global->__pyx_int_10
#define __pyx_int_20 __pyx_mstate_global->__pyx_int_20
#define __pyx_int_32 __pyx_mstate_global->__pyx_int_32
#define __pyx_int_200 __pyx_mstate_global->__pyx_int_200
#define __pyx_int_300 __pyx_mstate_global->__pyx_int_300
#define __pyx_tuple_ __pyx_mstate_global->__pyx_tuple_
#define __pyx_tuple__2 __pyx_mstate_global->__pyx_tuple__2
#define __pyx_tuple__4 __pyx_mstate_global->__pyx_tuple__4
#define __pyx_tuple__5 __pyx_mstate_global->__pyx_tuple__5
#define __pyx_tuple__6 __pyx_mstate_global->__pyx_tuple__6
#define __pyx_tuple__8 __pyx_mstate_global->__pyx_tuple__8
#define __pyx_tuple__9 __pyx_mstate_global->__pyx_tuple__9
#define __pyx_codeobj__7 __pyx_mstate_global->__pyx_codeobj__7
/* #### Code section: module_code ### */

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":245
 * 
 *         @property
 *         cdef inline PyObject* base(self) nogil:             # <<<<<<<<<<<<<<
 *             """Returns a borrowed reference to the object owning the data/memory.
 *             """
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_7ndarray_4base_base(PyArrayObject *__pyx_v_self) {
  PyObject *__pyx_r;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":248
 *             """Returns a borrowed reference to the object owning the data/memory.
 *             """
 *             return PyArray_BASE(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyArray_BASE(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":245
 * 
 *         @property
 *         cdef inline PyObject* base(self) nogil:             # <<<<<<<<<<<<<<
 *             """Returns a borrowed reference to the object owning the data/memory.
 *             """
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":251
 * 
 *         @property
 *         cdef inline dtype descr(self):             # <<<<<<<<<<<<<<
 *             """Returns an owned reference to the dtype of the array.
 *             """
 */

static CYTHON_INLINE PyArray_Descr *__pyx_f_5numpy_7ndarray_5descr_descr(PyArrayObject *__pyx_v_self) {
  PyArray_Descr *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyArray_Descr *__pyx_t_1;
  __Pyx_RefNannySetupContext("descr", 1);

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":254
 *             """Returns an owned reference to the dtype of the array.
 *             """
 *             return <dtype>PyArray_DESCR(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __pyx_t_1 = PyArray_DESCR(__pyx_v_self);
  __Pyx_INCREF((PyObject *)((PyArray_Descr *)__pyx_t_1));
  __pyx_r = ((PyArray_Descr *)__pyx_t_1);
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":251
 * 
 *         @property
 *         cdef inline dtype descr(self):             # <<<<<<<<<<<<<<
 *             """Returns an owned reference to the dtype of the array.
 *             """
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":257
 * 
 *         @property
 *         cdef inline int ndim(self) nogil:             # <<<<<<<<<<<<<<
 *             """Returns the number of dimensions in the array.
 *             """
 */

static CYTHON_INLINE int __pyx_f_5numpy_7ndarray_4ndim_ndim(PyArrayObject *__pyx_v_self) {
  int __pyx_r;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":260
 *             """Returns the number of dimensions in the array.
 *             """
 *             return PyArray_NDIM(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyArray_NDIM(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":257
 * 
 *         @property
 *         cdef inline int ndim(self) nogil:             # <<<<<<<<<<<<<<
 *             """Returns the number of dimensions in the array.
 *             """
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":263
 * 
 *         @property
 *         cdef inline npy_intp *shape(self) nogil:             # <<<<<<<<<<<<<<
 *             """Returns a pointer to the dimensions/shape of the array.
 *             The number of elements matches the number of dimensions of the array (ndim).
 */

static CYTHON_INLINE npy_intp *__pyx_f_5numpy_7ndarray_5shape_shape(PyArrayObject *__pyx_v_self) {
  npy_intp *__pyx_r;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":268
 *             Can return NULL for 0-dimensional arrays.
 *             """
 *             return PyArray_DIMS(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyArray_DIMS(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":263
 * 
 *         @property
 *         cdef inline npy_intp *shape(self) nogil:             # <<<<<<<<<<<<<<
 *             """Returns a pointer to the dimensions/shape of the array.
 *             The number of elements matches the number of dimensions of the array (ndim).
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":271
 * 
 *         @property
 *         cdef inline npy_intp *strides(self) nogil:             # <<<<<<<<<<<<<<
 *             """Returns a pointer to the strides of the array.
 *             The number of elements matches the number of dimensions of the array (ndim).
 */

static CYTHON_INLINE npy_intp *__pyx_f_5numpy_7ndarray_7strides_strides(PyArrayObject *__pyx_v_self) {
  npy_intp *__pyx_r;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":275
 *             The number of elements matches the number of dimensions of the array (ndim).
 *             """
 *             return PyArray_STRIDES(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyArray_STRIDES(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":271
 * 
 *         @property
 *         cdef inline npy_intp *strides(self) nogil:             # <<<<<<<<<<<<<<
 *             """Returns a pointer to the strides of the array.
 *             The number of elements matches the number of dimensions of the array (ndim).
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":278
 * 
 *         @property
 *         cdef inline npy_intp size(self) nogil:             # <<<<<<<<<<<<<<
 *             """Returns the total size (in number of elements) of the array.
 *             """
 */

static CYTHON_INLINE npy_intp __pyx_f_5numpy_7ndarray_4size_size(PyArrayObject *__pyx_v_self) {
  npy_intp __pyx_r;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":281
 *             """Returns the total size (in number of elements) of the array.
 *             """
 *             return PyArray_SIZE(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyArray_SIZE(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":278
 * 
 *         @property
 *         cdef inline npy_intp size(self) nogil:             # <<<<<<<<<<<<<<
 *             """Returns the total size (in number of elements) of the array.
 *             """
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":284
 * 
 *         @property
 *         cdef inline char* data(self) nogil:             # <<<<<<<<<<<<<<
 *             """The pointer to the data buffer as a char*.
 *             This is provided for legacy reasons to avoid direct struct field access.
 */

static CYTHON_INLINE char *__pyx_f_5numpy_7ndarray_4data_data(PyArrayObject *__pyx_v_self) {
  char *__pyx_r;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":290
 *             of `PyArray_DATA()` instead, which returns a 'void*'.
 *             """
 *             return PyArray_BYTES(self)             # <<<<<<<<<<<<<<
 * 
 *     ctypedef unsigned char      npy_bool
 */
  __pyx_r = PyArray_BYTES(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":284
 * 
 *         @property
 *         cdef inline char* data(self) nogil:             # <<<<<<<<<<<<<<
 *             """The pointer to the data buffer as a char*.
 *             This is provided for legacy reasons to avoid direct struct field access.
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":773
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew1(PyObject *__pyx_v_a) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew1", 1);

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":774
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 *     return PyArray_MultiIterNew(1, <void*>a)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(1, ((void *)__pyx_v_a)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 774, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":773
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":776
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew2(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew2", 1);

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":777
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(2, ((void *)__pyx_v_a), ((void *)__pyx_v_b)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 777, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":776
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew2", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":779
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew3(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew3", 1);

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":780
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(3, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 780, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":779
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew3", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":782
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew4(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew4", 1);

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":783
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(4, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 783, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":782
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew4", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":785
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew5(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d, PyObject *__pyx_v_e) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew5", 1);

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":786
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)             # <<<<<<<<<<<<<<
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(5, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d), ((void *)__pyx_v_e)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 786, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":785
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew5", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":788
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyDataType_SHAPE(PyArray_Descr *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("PyDataType_SHAPE", 1);

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":789
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */
  __pyx_t_1 = PyDataType_HASSUBARRAY(__pyx_v_d);
  if (__pyx_t_1) {

    /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":790
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape             # <<<<<<<<<<<<<<
 *     else:
 *         return ()
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject*)__pyx_v_d->subarray->shape));
    __pyx_r = ((PyObject*)__pyx_v_d->subarray->shape);
    goto __pyx_L0;

    /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":789
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */
  }

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":792
 *         return <tuple>d.subarray.shape
 *     else:
 *         return ()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_empty_tuple);
    __pyx_r = __pyx_empty_tuple;
    goto __pyx_L0;
  }

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":788
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":968
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

static CYTHON_INLINE void __pyx_f_5numpy_set_array_base(PyArrayObject *__pyx_v_arr, PyObject *__pyx_v_base) {
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":969
 * 
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!             # <<<<<<<<<<<<<<
 *     PyArray_SetBaseObject(arr, base)
 * 
 */
  Py_INCREF(__pyx_v_base);

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":970
 * cdef inline void set_array_base(ndarray arr, object base):
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object get_array_base(ndarray arr):
 */
  __pyx_t_1 = PyArray_SetBaseObject(__pyx_v_arr, __pyx_v_base); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(1, 970, __pyx_L1_error)

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":968
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("numpy.set_array_base", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":972
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_get_array_base(PyArrayObject *__pyx_v_arr) {
  PyObject *__pyx_v_base;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("get_array_base", 1);

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":973
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)             # <<<<<<<<<<<<<<
 *     if base is NULL:
 *         return None
 */
  __pyx_v_base = PyArray_BASE(__pyx_v_arr);

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":974
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */
  __pyx_t_1 = (__pyx_v_base == NULL);
  if (__pyx_t_1) {

    /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":975
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 *         return None             # <<<<<<<<<<<<<<
 *     return <object>base
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":974
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */
  }

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":976
 *     if base is NULL:
 *         return None
 *     return <object>base             # <<<<<<<<<<<<<<
 * 
 * # Versions of the import_* functions which are more suitable for
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_base));
  __pyx_r = ((PyObject *)__pyx_v_base);
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":972
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":980
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         __pyx_import_array()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_array(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_array", 1);

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":981
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":982
 * cdef inline int import_array() except -1:
 *     try:
 *         __pyx_import_array()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")
 */
      __pyx_t_4 = _import_array(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 982, __pyx_L3_error)

      /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":981
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":983
 *     try:
 *         __pyx_import_array()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(1, 983, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_7);

      /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":984
 *         __pyx_import_array()
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 984, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(1, 984, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;

    /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":981
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
    __pyx_L5_except_error:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":980
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         __pyx_import_array()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":986
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_umath(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_umath", 1);

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":987
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":988
 * cdef inline int import_umath() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")
 */
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 988, __pyx_L3_error)

      /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":987
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":989
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(1, 989, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_7);

      /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":990
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 990, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(1, 990, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;

    /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":987
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    __pyx_L5_except_error:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":986
 *         raise ImportError("numpy.core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":992
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_ufunc(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_ufunc", 1);

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":993
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":994
 * cdef inline int import_ufunc() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")
 */
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 994, __pyx_L3_error)

      /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":993
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":995
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(1, 995, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_7);

      /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":996
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 996, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(1, 996, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;

    /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":993
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    __pyx_L5_except_error:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":992
 *         raise ImportError("numpy.core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":999
 * 
 * 
 * cdef inline bint is_timedelta64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.timedelta64)`
 */

static CYTHON_INLINE int __pyx_f_5numpy_is_timedelta64_object(PyObject *__pyx_v_obj) {
  int __pyx_r;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1011
 *     bool
 *     """
 *     return PyObject_TypeCheck(obj, &PyTimedeltaArrType_Type)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = PyObject_TypeCheck(__pyx_v_obj, (&PyTimedeltaArrType_Type));
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":999
 * 
 * 
 * cdef inline bint is_timedelta64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.timedelta64)`
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1014
 * 
 * 
 * cdef inline bint is_datetime64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.datetime64)`
 */

static CYTHON_INLINE int __pyx_f_5numpy_is_datetime64_object(PyObject *__pyx_v_obj) {
  int __pyx_r;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1026
 *     bool
 *     """
 *     return PyObject_TypeCheck(obj, &PyDatetimeArrType_Type)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = PyObject_TypeCheck(__pyx_v_obj, (&PyDatetimeArrType_Type));
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1014
 * 
 * 
 * cdef inline bint is_datetime64_object(object obj):             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.datetime64)`
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1029
 * 
 * 
 * cdef inline npy_datetime get_datetime64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy datetime64 object
 */

static CYTHON_INLINE npy_datetime __pyx_f_5numpy_get_datetime64_value(PyObject *__pyx_v_obj) {
  npy_datetime __pyx_r;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1036
 *     also needed.  That can be found using `get_datetime64_unit`.
 *     """
 *     return (<PyDatetimeScalarObject*>obj).obval             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((PyDatetimeScalarObject *)__pyx_v_obj)->obval;
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1029
 * 
 * 
 * cdef inline npy_datetime get_datetime64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy datetime64 object
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1039
 * 
 * 
 * cdef inline npy_timedelta get_timedelta64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy timedelta64 object
 */

static CYTHON_INLINE npy_timedelta __pyx_f_5numpy_get_timedelta64_value(PyObject *__pyx_v_obj) {
  npy_timedelta __pyx_r;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1043
 *     returns the int64 value underlying scalar numpy timedelta64 object
 *     """
 *     return (<PyTimedeltaScalarObject*>obj).obval             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((PyTimedeltaScalarObject *)__pyx_v_obj)->obval;
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1039
 * 
 * 
 * cdef inline npy_timedelta get_timedelta64_value(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy timedelta64 object
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1046
 * 
 * 
 * cdef inline NPY_DATETIMEUNIT get_datetime64_unit(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the unit part of the dtype for a numpy datetime64 object.
 */

static CYTHON_INLINE NPY_DATETIMEUNIT __pyx_f_5numpy_get_datetime64_unit(PyObject *__pyx_v_obj) {
  NPY_DATETIMEUNIT __pyx_r;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1050
 *     returns the unit part of the dtype for a numpy datetime64 object.
 *     """
 *     return <NPY_DATETIMEUNIT>(<PyDatetimeScalarObject*>obj).obmeta.base             # <<<<<<<<<<<<<<
 */
  __pyx_r = ((NPY_DATETIMEUNIT)((PyDatetimeScalarObject *)__pyx_v_obj)->obmeta.base);
  goto __pyx_L0;

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1046
 * 
 * 
 * cdef inline NPY_DATETIMEUNIT get_datetime64_unit(object obj) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the unit part of the dtype for a numpy datetime64 object.
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "pysndlib/instruments/bouncy.py":20
 * 
 * # --------------- bouncy ---------------- #
 * @cython.ccall             # <<<<<<<<<<<<<<
 * def bouncy(start, dur, freq, amp, bounce_freq=5, bounce_amp=20):
 *     lng: cython.long = clm.seconds2samples(dur)
 */

static PyObject *__pyx_pw_8pysndlib_11instruments_6bouncy_1bouncy(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_8pysndlib_11instruments_6bouncy_bouncy(PyObject *__pyx_v_start, PyObject *__pyx_v_dur, PyObject *__pyx_v_freq, PyObject *__pyx_v_amp, CYTHON_UNUSED int __pyx_skip_dispatch, struct __pyx_opt_args_8pysndlib_11instruments_6bouncy_bouncy *__pyx_optional_args) {
  PyObject *__pyx_v_bounce_freq = ((PyObject *)__pyx_int_5);
  PyObject *__pyx_v_bounce_amp = ((PyObject *)__pyx_int_20);
  long __pyx_v_lng;
  PyObject *__pyx_v_gen = NULL;
  PyObject *__pyx_v_gen1 = NULL;
  PyObject *__pyx_v_bouncef = NULL;
  PyObject *__pyx_v_rf = NULL;
  PyObject *__pyx_v_ampf = NULL;
  long __pyx_v_stop;
  PyObject *__pyx_v_fv = NULL;
  long __pyx_v_i;
  long __pyx_v_j;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  long __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  long __pyx_t_10;
  long __pyx_t_11;
  double __pyx_t_12;
  long __pyx_t_13;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("bouncy", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_bounce_freq = __pyx_optional_args->bounce_freq;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_bounce_amp = __pyx_optional_args->bounce_amp;
      }
    }
  }
  __Pyx_INCREF(__pyx_v_start);

  /* "pysndlib/instruments/bouncy.py":22
 * @cython.ccall
 * def bouncy(start, dur, freq, amp, bounce_freq=5, bounce_amp=20):
 *     lng: cython.long = clm.seconds2samples(dur)             # <<<<<<<<<<<<<<
 *     start: cython.long = clm.seconds2samples(start)
 *     gen = gens.make_rkfssb(freq*4, 1/4, r=1.0)
 */
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_dur); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 22, __pyx_L1_error)
  __pyx_t_2 = __pyx_f_8pysndlib_3clm_seconds2samples(__pyx_t_1, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 22, __pyx_L1_error)
  __pyx_v_lng = __pyx_t_2;

  /* "pysndlib/instruments/bouncy.py":23
 * def bouncy(start, dur, freq, amp, bounce_freq=5, bounce_amp=20):
 *     lng: cython.long = clm.seconds2samples(dur)
 *     start: cython.long = clm.seconds2samples(start)             # <<<<<<<<<<<<<<
 *     gen = gens.make_rkfssb(freq*4, 1/4, r=1.0)
 *     gen1 = clm.make_oscil(bounce_freq)
 */
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_start); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 23, __pyx_L1_error)
  __pyx_t_2 = __pyx_f_8pysndlib_3clm_seconds2samples(__pyx_t_1, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 23, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_From_long(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF_SET(__pyx_v_start, __pyx_t_3);
  __pyx_t_3 = 0;

  /* "pysndlib/instruments/bouncy.py":24
 *     lng: cython.long = clm.seconds2samples(dur)
 *     start: cython.long = clm.seconds2samples(start)
 *     gen = gens.make_rkfssb(freq*4, 1/4, r=1.0)             # <<<<<<<<<<<<<<
 *     gen1 = clm.make_oscil(bounce_freq)
 *     bouncef = clm.make_env([0,1,1,0], base=32, scaler=bounce_amp, duration=1.)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_gens); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_make_rkfssb); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyInt_MultiplyObjC(__pyx_v_freq, __pyx_int_4, 4, 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = PyFloat_FromDouble((1.0 / 4.0)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_3);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3)) __PYX_ERR(0, 24, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_5);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_5)) __PYX_ERR(0, 24, __pyx_L1_error);
  __pyx_t_3 = 0;
  __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_r, __pyx_float_1_0) < 0) __PYX_ERR(0, 24, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_6, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_gen = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pysndlib/instruments/bouncy.py":25
 *     start: cython.long = clm.seconds2samples(start)
 *     gen = gens.make_rkfssb(freq*4, 1/4, r=1.0)
 *     gen1 = clm.make_oscil(bounce_freq)             # <<<<<<<<<<<<<<
 *     bouncef = clm.make_env([0,1,1,0], base=32, scaler=bounce_amp, duration=1.)
 *     rf = clm.make_env([0,0,1,1,max(2.0, dur), 0], base=32, scaler=3, duration=dur)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_clm); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_make_oscil); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  __pyx_t_7 = 0;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
      __pyx_t_7 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_v_bounce_freq};
    __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_6, __pyx_callargs+1-__pyx_t_7, 1+__pyx_t_7);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 25, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __pyx_v_gen1 = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pysndlib/instruments/bouncy.py":26
 *     gen = gens.make_rkfssb(freq*4, 1/4, r=1.0)
 *     gen1 = clm.make_oscil(bounce_freq)
 *     bouncef = clm.make_env([0,1,1,0], base=32, scaler=bounce_amp, duration=1.)             # <<<<<<<<<<<<<<
 *     rf = clm.make_env([0,0,1,1,max(2.0, dur), 0], base=32, scaler=3, duration=dur)
 *     ampf = clm.make_env([0,0,.01,1,.03,1,1,.15, max(2.,dur), 0.0], base=32, scaler=amp, duration=dur)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_clm); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_make_env); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyList_New(4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 0, __pyx_int_0)) __PYX_ERR(0, 26, __pyx_L1_error);
  __Pyx_INCREF(__pyx_int_1);
  __Pyx_GIVEREF(__pyx_int_1);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 1, __pyx_int_1)) __PYX_ERR(0, 26, __pyx_L1_error);
  __Pyx_INCREF(__pyx_int_1);
  __Pyx_GIVEREF(__pyx_int_1);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 2, __pyx_int_1)) __PYX_ERR(0, 26, __pyx_L1_error);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 3, __pyx_int_0)) __PYX_ERR(0, 26, __pyx_L1_error);
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_3);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3)) __PYX_ERR(0, 26, __pyx_L1_error);
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_base, __pyx_int_32) < 0) __PYX_ERR(0, 26, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_scaler, __pyx_v_bounce_amp) < 0) __PYX_ERR(0, 26, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_duration, __pyx_float_1_) < 0) __PYX_ERR(0, 26, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_5, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_bouncef = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "pysndlib/instruments/bouncy.py":27
 *     gen1 = clm.make_oscil(bounce_freq)
 *     bouncef = clm.make_env([0,1,1,0], base=32, scaler=bounce_amp, duration=1.)
 *     rf = clm.make_env([0,0,1,1,max(2.0, dur), 0], base=32, scaler=3, duration=dur)             # <<<<<<<<<<<<<<
 *     ampf = clm.make_env([0,0,.01,1,.03,1,1,.15, max(2.,dur), 0.0], base=32, scaler=amp, duration=dur)
 *     stop: cython.long = start+ lng
 */
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_clm); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_make_env); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_INCREF(__pyx_v_dur);
  __pyx_t_4 = __pyx_v_dur;
  __pyx_t_1 = 2.0;
  __pyx_t_6 = PyFloat_FromDouble(__pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_8 = PyObject_RichCompare(__pyx_t_4, __pyx_t_6, Py_GT); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely((__pyx_t_9 < 0))) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (__pyx_t_9) {
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_5 = __pyx_t_4;
  } else {
    __pyx_t_8 = PyFloat_FromDouble(__pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 27, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_5 = __pyx_t_8;
    __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyList_New(6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 0, __pyx_int_0)) __PYX_ERR(0, 27, __pyx_L1_error);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 1, __pyx_int_0)) __PYX_ERR(0, 27, __pyx_L1_error);
  __Pyx_INCREF(__pyx_int_1);
  __Pyx_GIVEREF(__pyx_int_1);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 2, __pyx_int_1)) __PYX_ERR(0, 27, __pyx_L1_error);
  __Pyx_INCREF(__pyx_int_1);
  __Pyx_GIVEREF(__pyx_int_1);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 3, __pyx_int_1)) __PYX_ERR(0, 27, __pyx_L1_error);
  __Pyx_INCREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_5);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 4, __pyx_t_5)) __PYX_ERR(0, 27, __pyx_L1_error);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 5, __pyx_int_0)) __PYX_ERR(0, 27, __pyx_L1_error);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_4);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4)) __PYX_ERR(0, 27, __pyx_L1_error);
  __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_base, __pyx_int_32) < 0) __PYX_ERR(0, 27, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_scaler, __pyx_int_3) < 0) __PYX_ERR(0, 27, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_duration, __pyx_v_dur) < 0) __PYX_ERR(0, 27, __pyx_L1_error)
  __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, __pyx_t_4); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_rf = __pyx_t_8;
  __pyx_t_8 = 0;

  /* "pysndlib/instruments/bouncy.py":28
 *     bouncef = clm.make_env([0,1,1,0], base=32, scaler=bounce_amp, duration=1.)
 *     rf = clm.make_env([0,0,1,1,max(2.0, dur), 0], base=32, scaler=3, duration=dur)
 *     ampf = clm.make_env([0,0,.01,1,.03,1,1,.15, max(2.,dur), 0.0], base=32, scaler=amp, duration=dur)             # <<<<<<<<<<<<<<
 *     stop: cython.long = start+ lng
 *     fv = np.zeros(lng)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_n_s_clm); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_make_env); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_INCREF(__pyx_v_dur);
  __pyx_t_8 = __pyx_v_dur;
  __pyx_t_1 = 2.;
  __pyx_t_3 = PyFloat_FromDouble(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = PyObject_RichCompare(__pyx_t_8, __pyx_t_3, Py_GT); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely((__pyx_t_9 < 0))) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (__pyx_t_9) {
    __Pyx_INCREF(__pyx_t_8);
    __pyx_t_5 = __pyx_t_8;
  } else {
    __pyx_t_6 = PyFloat_FromDouble(__pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 28, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_5 = __pyx_t_6;
    __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = PyList_New(10); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 0, __pyx_int_0)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 1, __pyx_int_0)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__01);
  __Pyx_GIVEREF(__pyx_float__01);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 2, __pyx_float__01)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_int_1);
  __Pyx_GIVEREF(__pyx_int_1);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 3, __pyx_int_1)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__03);
  __Pyx_GIVEREF(__pyx_float__03);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 4, __pyx_float__03)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_int_1);
  __Pyx_GIVEREF(__pyx_int_1);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 5, __pyx_int_1)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_int_1);
  __Pyx_GIVEREF(__pyx_int_1);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 6, __pyx_int_1)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float__15);
  __Pyx_GIVEREF(__pyx_float__15);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 7, __pyx_float__15)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_5);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 8, __pyx_t_5)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_INCREF(__pyx_float_0_0);
  __Pyx_GIVEREF(__pyx_float_0_0);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_8, 9, __pyx_float_0_0)) __PYX_ERR(0, 28, __pyx_L1_error);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_8);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_8)) __PYX_ERR(0, 28, __pyx_L1_error);
  __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  if (PyDict_SetItem(__pyx_t_8, __pyx_n_s_base, __pyx_int_32) < 0) __PYX_ERR(0, 28, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_8, __pyx_n_s_scaler, __pyx_v_amp) < 0) __PYX_ERR(0, 28, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_8, __pyx_n_s_duration, __pyx_v_dur) < 0) __PYX_ERR(0, 28, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, __pyx_t_8); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_v_ampf = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "pysndlib/instruments/bouncy.py":29
 *     rf = clm.make_env([0,0,1,1,max(2.0, dur), 0], base=32, scaler=3, duration=dur)
 *     ampf = clm.make_env([0,0,.01,1,.03,1,1,.15, max(2.,dur), 0.0], base=32, scaler=amp, duration=dur)
 *     stop: cython.long = start+ lng             # <<<<<<<<<<<<<<
 *     fv = np.zeros(lng)
 * 
 */
  __pyx_t_6 = __Pyx_PyInt_From_long(__pyx_v_lng); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_8 = PyNumber_Add(__pyx_v_start, __pyx_t_6); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_2 = __Pyx_PyInt_As_long(__pyx_t_8); if (unlikely((__pyx_t_2 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 29, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_v_stop = __pyx_t_2;

  /* "pysndlib/instruments/bouncy.py":30
 *     ampf = clm.make_env([0,0,.01,1,.03,1,1,.15, max(2.,dur), 0.0], base=32, scaler=amp, duration=dur)
 *     stop: cython.long = start+ lng
 *     fv = np.zeros(lng)             # <<<<<<<<<<<<<<
 * 
 *     i: cython.long = 0
 */
  __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_np); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_zeros); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_PyInt_From_long(__pyx_v_lng); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = NULL;
  __pyx_t_7 = 0;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
      __pyx_t_7 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_6};
    __pyx_t_8 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+1-__pyx_t_7, 1+__pyx_t_7);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 30, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __pyx_v_fv = __pyx_t_8;
  __pyx_t_8 = 0;

  /* "pysndlib/instruments/bouncy.py":32
 *     fv = np.zeros(lng)
 * 
 *     i: cython.long = 0             # <<<<<<<<<<<<<<
 *     for i in range(lng):
 *         fv[i] = clm.env(rf) + math.fabs(clm.env(bouncef) * clm.oscil(gen1))
 */
  __pyx_v_i = 0;

  /* "pysndlib/instruments/bouncy.py":33
 * 
 *     i: cython.long = 0
 *     for i in range(lng):             # <<<<<<<<<<<<<<
 *         fv[i] = clm.env(rf) + math.fabs(clm.env(bouncef) * clm.oscil(gen1))
 * 
 */
  __pyx_t_2 = __pyx_v_lng;
  __pyx_t_10 = __pyx_t_2;
  for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_10; __pyx_t_11+=1) {
    __pyx_v_i = __pyx_t_11;

    /* "pysndlib/instruments/bouncy.py":34
 *     i: cython.long = 0
 *     for i in range(lng):
 *         fv[i] = clm.env(rf) + math.fabs(clm.env(bouncef) * clm.oscil(gen1))             # <<<<<<<<<<<<<<
 * 
 *     j: cython.long = 0
 */
    if (!(likely(((__pyx_v_rf) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_rf, __pyx_ptype_8pysndlib_3clm_mus_any))))) __PYX_ERR(0, 34, __pyx_L1_error)
    __pyx_t_1 = __pyx_f_8pysndlib_3clm_env(((struct __pyx_obj_8pysndlib_3clm_mus_any *)__pyx_v_rf), 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 34, __pyx_L1_error)
    __pyx_t_8 = PyFloat_FromDouble(__pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 34, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_math); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 34, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_fabs); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 34, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (!(likely(((__pyx_v_bouncef) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_bouncef, __pyx_ptype_8pysndlib_3clm_mus_any))))) __PYX_ERR(0, 34, __pyx_L1_error)
    __pyx_t_1 = __pyx_f_8pysndlib_3clm_env(((struct __pyx_obj_8pysndlib_3clm_mus_any *)__pyx_v_bouncef), 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 34, __pyx_L1_error)
    if (!(likely(((__pyx_v_gen1) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_gen1, __pyx_ptype_8pysndlib_3clm_mus_any))))) __PYX_ERR(0, 34, __pyx_L1_error)
    __pyx_t_12 = __pyx_f_8pysndlib_3clm_oscil(((struct __pyx_obj_8pysndlib_3clm_mus_any *)__pyx_v_gen1), 0, NULL); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 34, __pyx_L1_error)
    __pyx_t_6 = PyFloat_FromDouble((__pyx_t_1 * __pyx_t_12)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 34, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_3 = NULL;
    __pyx_t_7 = 0;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_7 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_6};
      __pyx_t_5 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+1-__pyx_t_7, 1+__pyx_t_7);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 34, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __pyx_t_4 = PyNumber_Add(__pyx_t_8, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 34, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely((__Pyx_SetItemInt(__pyx_v_fv, __pyx_v_i, __pyx_t_4, long, 1, __Pyx_PyInt_From_long, 0, 1, 1) < 0))) __PYX_ERR(0, 34, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }

  /* "pysndlib/instruments/bouncy.py":36
 *         fv[i] = clm.env(rf) + math.fabs(clm.env(bouncef) * clm.oscil(gen1))
 * 
 *     j: cython.long = 0             # <<<<<<<<<<<<<<
 * 
 *     for i in range(start, stop):
 */
  __pyx_v_j = 0;

  /* "pysndlib/instruments/bouncy.py":38
 *     j: cython.long = 0
 * 
 *     for i in range(start, stop):             # <<<<<<<<<<<<<<
 *         gen.r = fv[j]
 *         clm.outa(i, clm.env(ampf) * gens.rkfssb(gen))
 */
  __pyx_t_2 = __pyx_v_stop;
  __pyx_t_10 = __Pyx_PyInt_As_long(__pyx_v_start); if (unlikely((__pyx_t_10 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 38, __pyx_L1_error)
  __pyx_t_11 = __pyx_t_2;
  for (__pyx_t_13 = __pyx_t_10; __pyx_t_13 < __pyx_t_11; __pyx_t_13+=1) {
    __pyx_v_i = __pyx_t_13;

    /* "pysndlib/instruments/bouncy.py":39
 * 
 *     for i in range(start, stop):
 *         gen.r = fv[j]             # <<<<<<<<<<<<<<
 *         clm.outa(i, clm.env(ampf) * gens.rkfssb(gen))
 *         j+=1
 */
    __pyx_t_4 = __Pyx_GetItemInt(__pyx_v_fv, __pyx_v_j, long, 1, __Pyx_PyInt_From_long, 0, 1, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 39, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (__Pyx_PyObject_SetAttrStr(__pyx_v_gen, __pyx_n_s_r, __pyx_t_4) < 0) __PYX_ERR(0, 39, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "pysndlib/instruments/bouncy.py":40
 *     for i in range(start, stop):
 *         gen.r = fv[j]
 *         clm.outa(i, clm.env(ampf) * gens.rkfssb(gen))             # <<<<<<<<<<<<<<
 *         j+=1
 * 
 */
    if (!(likely(((__pyx_v_ampf) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_ampf, __pyx_ptype_8pysndlib_3clm_mus_any))))) __PYX_ERR(0, 40, __pyx_L1_error)
    __pyx_t_12 = __pyx_f_8pysndlib_3clm_env(((struct __pyx_obj_8pysndlib_3clm_mus_any *)__pyx_v_ampf), 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 40, __pyx_L1_error)
    if (!(likely(((__pyx_v_gen) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_gen, __pyx_ptype_8pysndlib_10generators_Rkfssb))))) __PYX_ERR(0, 40, __pyx_L1_error)
    __pyx_t_1 = __pyx_f_8pysndlib_10generators_rkfssb(((struct __pyx_obj_8pysndlib_10generators_Rkfssb *)__pyx_v_gen), 0, NULL); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 40, __pyx_L1_error)
    __pyx_t_4 = __pyx_f_8pysndlib_3clm_outa(__pyx_v_i, (__pyx_t_12 * __pyx_t_1), 0, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 40, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "pysndlib/instruments/bouncy.py":41
 *         gen.r = fv[j]
 *         clm.outa(i, clm.env(ampf) * gens.rkfssb(gen))
 *         j+=1             # <<<<<<<<<<<<<<
 * 
 * if __name__ == '__main__':
 */
    __pyx_v_j = (__pyx_v_j + 1);
  }

  /* "pysndlib/instruments/bouncy.py":20
 * 
 * # --------------- bouncy ---------------- #
 * @cython.ccall             # <<<<<<<<<<<<<<
 * def bouncy(start, dur, freq, amp, bounce_freq=5, bounce_amp=20):
 *     lng: cython.long = clm.seconds2samples(dur)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("pysndlib.instruments.bouncy.bouncy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_gen);
  __Pyx_XDECREF(__pyx_v_gen1);
  __Pyx_XDECREF(__pyx_v_bouncef);
  __Pyx_XDECREF(__pyx_v_rf);
  __Pyx_XDECREF(__pyx_v_ampf);
  __Pyx_XDECREF(__pyx_v_fv);
  __Pyx_XDECREF(__pyx_v_start);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_8pysndlib_11instruments_6bouncy_1bouncy(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_8pysndlib_11instruments_6bouncy_1bouncy = {"bouncy", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_8pysndlib_11instruments_6bouncy_1bouncy, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_8pysndlib_11instruments_6bouncy_1bouncy(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_start = 0;
  PyObject *__pyx_v_dur = 0;
  PyObject *__pyx_v_freq = 0;
  PyObject *__pyx_v_amp = 0;
  PyObject *__pyx_v_bounce_freq = 0;
  PyObject *__pyx_v_bounce_amp = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[6] = {0,0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("bouncy (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_start,&__pyx_n_s_dur,&__pyx_n_s_freq,&__pyx_n_s_amp,&__pyx_n_s_bounce_freq,&__pyx_n_s_bounce_amp,0};
    values[4] = __Pyx_Arg_NewRef_FASTCALL(((PyObject *)__pyx_int_5));
    values[5] = __Pyx_Arg_NewRef_FASTCALL(((PyObject *)__pyx_int_20));
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  6: values[5] = __Pyx_Arg_FASTCALL(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = __Pyx_Arg_FASTCALL(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = __Pyx_Arg_FASTCALL(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_start)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 20, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_dur)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[1]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 20, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("bouncy", 0, 4, 6, 1); __PYX_ERR(0, 20, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_freq)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[2]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 20, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("bouncy", 0, 4, 6, 2); __PYX_ERR(0, 20, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_amp)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[3]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 20, __pyx_L3_error)
        else {
          __Pyx_RaiseArgtupleInvalid("bouncy", 0, 4, 6, 3); __PYX_ERR(0, 20, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_bounce_freq);
          if (value) { values[4] = __Pyx_Arg_NewRef_FASTCALL(value); kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 20, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (kw_args > 0) {
          PyObject* value = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_bounce_amp);
          if (value) { values[5] = __Pyx_Arg_NewRef_FASTCALL(value); kw_args--; }
          else if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 20, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "bouncy") < 0)) __PYX_ERR(0, 20, __pyx_L3_error)
      }
    } else {
      switch (__pyx_nargs) {
        case  6: values[5] = __Pyx_Arg_FASTCALL(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = __Pyx_Arg_FASTCALL(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = __Pyx_Arg_FASTCALL(__pyx_args, 3);
        values[2] = __Pyx_Arg_FASTCALL(__pyx_args, 2);
        values[1] = __Pyx_Arg_FASTCALL(__pyx_args, 1);
        values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_start = values[0];
    __pyx_v_dur = values[1];
    __pyx_v_freq = values[2];
    __pyx_v_amp = values[3];
    __pyx_v_bounce_freq = values[4];
    __pyx_v_bounce_amp = values[5];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("bouncy", 0, 4, 6, __pyx_nargs); __PYX_ERR(0, 20, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("pysndlib.instruments.bouncy.bouncy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_8pysndlib_11instruments_6bouncy_bouncy(__pyx_self, __pyx_v_start, __pyx_v_dur, __pyx_v_freq, __pyx_v_amp, __pyx_v_bounce_freq, __pyx_v_bounce_amp);

  /* function exit code */
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pysndlib_11instruments_6bouncy_bouncy(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_start, PyObject *__pyx_v_dur, PyObject *__pyx_v_freq, PyObject *__pyx_v_amp, PyObject *__pyx_v_bounce_freq, PyObject *__pyx_v_bounce_amp) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_8pysndlib_11instruments_6bouncy_bouncy __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("bouncy", 1);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 2;
  __pyx_t_2.bounce_freq = __pyx_v_bounce_freq;
  __pyx_t_2.bounce_amp = __pyx_v_bounce_amp;
  __pyx_t_1 = __pyx_f_8pysndlib_11instruments_6bouncy_bouncy(__pyx_v_start, __pyx_v_dur, __pyx_v_freq, __pyx_v_amp, 0, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 20, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pysndlib.instruments.bouncy.bouncy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif
/* #### Code section: pystring_table ### */

static int __Pyx_CreateStringTabAndInitStrings(void) {
  __Pyx_StringTabEntry __pyx_string_tab[] = {
    {&__pyx_n_s_ImportError, __pyx_k_ImportError, sizeof(__pyx_k_ImportError), 0, 0, 1, 1},
    {&__pyx_n_s_Sound, __pyx_k_Sound, sizeof(__pyx_k_Sound), 0, 0, 1, 1},
    {&__pyx_n_s__10, __pyx_k__10, sizeof(__pyx_k__10), 0, 0, 1, 1},
    {&__pyx_n_s__3, __pyx_k__3, sizeof(__pyx_k__3), 0, 0, 1, 1},
    {&__pyx_n_s_amp, __pyx_k_amp, sizeof(__pyx_k_amp), 0, 0, 1, 1},
    {&__pyx_n_s_asyncio_coroutines, __pyx_k_asyncio_coroutines, sizeof(__pyx_k_asyncio_coroutines), 0, 0, 1, 1},
    {&__pyx_n_s_base, __pyx_k_base, sizeof(__pyx_k_base), 0, 0, 1, 1},
    {&__pyx_n_s_bounce_amp, __pyx_k_bounce_amp, sizeof(__pyx_k_bounce_amp), 0, 0, 1, 1},
    {&__pyx_n_s_bounce_freq, __pyx_k_bounce_freq, sizeof(__pyx_k_bounce_freq), 0, 0, 1, 1},
    {&__pyx_n_s_bouncy, __pyx_k_bouncy, sizeof(__pyx_k_bouncy), 0, 0, 1, 1},
    {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
    {&__pyx_n_s_clipped, __pyx_k_clipped, sizeof(__pyx_k_clipped), 0, 0, 1, 1},
    {&__pyx_n_s_clm, __pyx_k_clm, sizeof(__pyx_k_clm), 0, 0, 1, 1},
    {&__pyx_n_s_dur, __pyx_k_dur, sizeof(__pyx_k_dur), 0, 0, 1, 1},
    {&__pyx_n_s_duration, __pyx_k_duration, sizeof(__pyx_k_duration), 0, 0, 1, 1},
    {&__pyx_n_s_enter, __pyx_k_enter, sizeof(__pyx_k_enter), 0, 0, 1, 1},
    {&__pyx_n_s_exit, __pyx_k_exit, sizeof(__pyx_k_exit), 0, 0, 1, 1},
    {&__pyx_n_s_fabs, __pyx_k_fabs, sizeof(__pyx_k_fabs), 0, 0, 1, 1},
    {&__pyx_n_s_freq, __pyx_k_freq, sizeof(__pyx_k_freq), 0, 0, 1, 1},
    {&__pyx_n_s_generators, __pyx_k_generators, sizeof(__pyx_k_generators), 0, 0, 1, 1},
    {&__pyx_n_s_gens, __pyx_k_gens, sizeof(__pyx_k_gens), 0, 0, 1, 1},
    {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
    {&__pyx_n_s_initializing, __pyx_k_initializing, sizeof(__pyx_k_initializing), 0, 0, 1, 1},
    {&__pyx_n_s_is_coroutine, __pyx_k_is_coroutine, sizeof(__pyx_k_is_coroutine), 0, 0, 1, 1},
    {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
    {&__pyx_n_u_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 1, 0, 1},
    {&__pyx_n_s_make_env, __pyx_k_make_env, sizeof(__pyx_k_make_env), 0, 0, 1, 1},
    {&__pyx_n_s_make_oscil, __pyx_k_make_oscil, sizeof(__pyx_k_make_oscil), 0, 0, 1, 1},
    {&__pyx_n_s_make_rkfssb, __pyx_k_make_rkfssb, sizeof(__pyx_k_make_rkfssb), 0, 0, 1, 1},
    {&__pyx_n_s_math, __pyx_k_math, sizeof(__pyx_k_math), 0, 0, 1, 1},
    {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
    {&__pyx_n_s_np, __pyx_k_np, sizeof(__pyx_k_np), 0, 0, 1, 1},
    {&__pyx_n_s_numpy, __pyx_k_numpy, sizeof(__pyx_k_numpy), 0, 0, 1, 1},
    {&__pyx_kp_u_numpy_core_multiarray_failed_to, __pyx_k_numpy_core_multiarray_failed_to, sizeof(__pyx_k_numpy_core_multiarray_failed_to), 0, 1, 0, 0},
    {&__pyx_kp_u_numpy_core_umath_failed_to_impor, __pyx_k_numpy_core_umath_failed_to_impor, sizeof(__pyx_k_numpy_core_umath_failed_to_impor), 0, 1, 0, 0},
    {&__pyx_n_s_play, __pyx_k_play, sizeof(__pyx_k_play), 0, 0, 1, 1},
    {&__pyx_n_s_pysndlib, __pyx_k_pysndlib, sizeof(__pyx_k_pysndlib), 0, 0, 1, 1},
    {&__pyx_n_s_pysndlib_clm, __pyx_k_pysndlib_clm, sizeof(__pyx_k_pysndlib_clm), 0, 0, 1, 1},
    {&__pyx_n_s_pysndlib_generators, __pyx_k_pysndlib_generators, sizeof(__pyx_k_pysndlib_generators), 0, 0, 1, 1},
    {&__pyx_n_s_pysndlib_instruments_bouncy, __pyx_k_pysndlib_instruments_bouncy, sizeof(__pyx_k_pysndlib_instruments_bouncy), 0, 0, 1, 1},
    {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
    {&__pyx_n_s_r, __pyx_k_r, sizeof(__pyx_k_r), 0, 0, 1, 1},
    {&__pyx_n_s_random, __pyx_k_random, sizeof(__pyx_k_random), 0, 0, 1, 1},
    {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
    {&__pyx_n_s_scaler, __pyx_k_scaler, sizeof(__pyx_k_scaler), 0, 0, 1, 1},
    {&__pyx_n_s_spec, __pyx_k_spec, sizeof(__pyx_k_spec), 0, 0, 1, 1},
    {&__pyx_kp_s_src_pysndlib_instruments_bouncy, __pyx_k_src_pysndlib_instruments_bouncy, sizeof(__pyx_k_src_pysndlib_instruments_bouncy), 0, 0, 1, 0},
    {&__pyx_n_s_start, __pyx_k_start, sizeof(__pyx_k_start), 0, 0, 1, 1},
    {&__pyx_n_s_statistics, __pyx_k_statistics, sizeof(__pyx_k_statistics), 0, 0, 1, 1},
    {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
    {&__pyx_n_s_zeros, __pyx_k_zeros, sizeof(__pyx_k_zeros), 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 0, 0}
  };
  return __Pyx_InitStrings(__pyx_string_tab);
}
/* #### Code section: cached_builtins ### */
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 33, __pyx_L1_error)
  __pyx_builtin_ImportError = __Pyx_GetBuiltinName(__pyx_n_s_ImportError); if (!__pyx_builtin_ImportError) __PYX_ERR(1, 984, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: cached_constants ### */

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":984
 *         __pyx_import_array()
 *     except Exception:
 *         raise ImportError("numpy.core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_u_numpy_core_multiarray_failed_to); if (unlikely(!__pyx_tuple_)) __PYX_ERR(1, 984, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "../../../../../private/var/folders/kx/gw6dssyn19d9qjs9mvh4hkz80000gn/T/build-env-ztoci14v/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":990
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy.core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_u_numpy_core_umath_failed_to_impor); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(1, 990, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "pysndlib/instruments/bouncy.py":10
 * import cython
 * import numpy as np
 * import pysndlib.clm as clm             # <<<<<<<<<<<<<<
 * import pysndlib.generators as gens
 * if cython.compiled:
 */
  __pyx_tuple__4 = PyTuple_Pack(2, __pyx_n_s_pysndlib, __pyx_n_s_clm); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(0, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "pysndlib/instruments/bouncy.py":11
 * import numpy as np
 * import pysndlib.clm as clm
 * import pysndlib.generators as gens             # <<<<<<<<<<<<<<
 * if cython.compiled:
 *     from cython.cimports.pysndlib import clm
 */
  __pyx_tuple__5 = PyTuple_Pack(2, __pyx_n_s_pysndlib, __pyx_n_s_generators); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(0, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "pysndlib/instruments/bouncy.py":20
 * 
 * # --------------- bouncy ---------------- #
 * @cython.ccall             # <<<<<<<<<<<<<<
 * def bouncy(start, dur, freq, amp, bounce_freq=5, bounce_amp=20):
 *     lng: cython.long = clm.seconds2samples(dur)
 */
  __pyx_tuple__6 = PyTuple_Pack(6, __pyx_n_s_start, __pyx_n_s_dur, __pyx_n_s_freq, __pyx_n_s_amp, __pyx_n_s_bounce_freq, __pyx_n_s_bounce_amp); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 20, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);
  __pyx_codeobj__7 = (PyObject*)__Pyx_PyCode_New(6, 0, 0, 6, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__6, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_pysndlib_instruments_bouncy, __pyx_n_s_bouncy, 20, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__7)) __PYX_ERR(0, 20, __pyx_L1_error)
  __pyx_tuple__8 = PyTuple_Pack(2, __pyx_int_5, __pyx_int_20); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(0, 20, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);

  /* "pysndlib/instruments/bouncy.py":45
 * if __name__ == '__main__':
 * 
 *     with clm.Sound(clipped=False, statistics=True, play=True):             # <<<<<<<<<<<<<<
 *         bouncy(0,2,300,.5, 5, 10)
 *         bouncy(2.5,2,200,.5, 3, 2)
 */
  __pyx_tuple__9 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(0, 45, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}
/* #### Code section: init_constants ### */

static CYTHON_SMALL_CODE int __Pyx_InitConstants(void) {
  if (__Pyx_CreateStringTabAndInitStrings() < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_float__5 = PyFloat_FromDouble(.5); if (unlikely(!__pyx_float__5)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_1_ = PyFloat_FromDouble(1.); if (unlikely(!__pyx_float_1_)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__01 = PyFloat_FromDouble(.01); if (unlikely(!__pyx_float__01)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__03 = PyFloat_FromDouble(.03); if (unlikely(!__pyx_float__03)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float__15 = PyFloat_FromDouble(.15); if (unlikely(!__pyx_float__15)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_0_0 = PyFloat_FromDouble(0.0); if (unlikely(!__pyx_float_0_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_1_0 = PyFloat_FromDouble(1.0); if (unlikely(!__pyx_float_1_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_2_5 = PyFloat_FromDouble(2.5); if (unlikely(!__pyx_float_2_5)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_2 = PyInt_FromLong(2); if (unlikely(!__pyx_int_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_3 = PyInt_FromLong(3); if (unlikely(!__pyx_int_3)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_4 = PyInt_FromLong(4); if (unlikely(!__pyx_int_4)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_5 = PyInt_FromLong(5); if (unlikely(!__pyx_int_5)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_10 = PyInt_FromLong(10); if (unlikely(!__pyx_int_10)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_20 = PyInt_FromLong(20); if (unlikely(!__pyx_int_20)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_32 = PyInt_FromLong(32); if (unlikely(!__pyx_int_32)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_200 = PyInt_FromLong(200); if (unlikely(!__pyx_int_200)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_300 = PyInt_FromLong(300); if (unlikely(!__pyx_int_300)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: init_globals ### */

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  return 0;
}
/* #### Code section: init_module ### */

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType_3_0_8(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type", 
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyTypeObject),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(PyTypeObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_7cpython_4type_type) __PYX_ERR(2, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("numpy"); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 202, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_5numpy_dtype = __Pyx_ImportType_3_0_8(__pyx_t_1, "numpy", "dtype", sizeof(PyArray_Descr), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyArray_Descr),__Pyx_ImportType_CheckSize_Ignore_3_0_8); if (!__pyx_ptype_5numpy_dtype) __PYX_ERR(1, 202, __pyx_L1_error)
  __pyx_ptype_5numpy_flatiter = __Pyx_ImportType_3_0_8(__pyx_t_1, "numpy", "flatiter", sizeof(PyArrayIterObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyArrayIterObject),__Pyx_ImportType_CheckSize_Ignore_3_0_8); if (!__pyx_ptype_5numpy_flatiter) __PYX_ERR(1, 225, __pyx_L1_error)
  __pyx_ptype_5numpy_broadcast = __Pyx_ImportType_3_0_8(__pyx_t_1, "numpy", "broadcast", sizeof(PyArrayMultiIterObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyArrayMultiIterObject),__Pyx_ImportType_CheckSize_Ignore_3_0_8); if (!__pyx_ptype_5numpy_broadcast) __PYX_ERR(1, 229, __pyx_L1_error)
  __pyx_ptype_5numpy_ndarray = __Pyx_ImportType_3_0_8(__pyx_t_1, "numpy", "ndarray", sizeof(PyArrayObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyArrayObject),__Pyx_ImportType_CheckSize_Ignore_3_0_8); if (!__pyx_ptype_5numpy_ndarray) __PYX_ERR(1, 238, __pyx_L1_error)
  __pyx_ptype_5numpy_generic = __Pyx_ImportType_3_0_8(__pyx_t_1, "numpy", "generic", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyObject),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_5numpy_generic) __PYX_ERR(1, 809, __pyx_L1_error)
  __pyx_ptype_5numpy_number = __Pyx_ImportType_3_0_8(__pyx_t_1, "numpy", "number", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyObject),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_5numpy_number) __PYX_ERR(1, 811, __pyx_L1_error)
  __pyx_ptype_5numpy_integer = __Pyx_ImportType_3_0_8(__pyx_t_1, "numpy", "integer", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyObject),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_5numpy_integer) __PYX_ERR(1, 813, __pyx_L1_error)
  __pyx_ptype_5numpy_signedinteger = __Pyx_ImportType_3_0_8(__pyx_t_1, "numpy", "signedinteger", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyObject),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_5numpy_signedinteger) __PYX_ERR(1, 815, __pyx_L1_error)
  __pyx_ptype_5numpy_unsignedinteger = __Pyx_ImportType_3_0_8(__pyx_t_1, "numpy", "unsignedinteger", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyObject),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_5numpy_unsignedinteger) __PYX_ERR(1, 817, __pyx_L1_error)
  __pyx_ptype_5numpy_inexact = __Pyx_ImportType_3_0_8(__pyx_t_1, "numpy", "inexact", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyObject),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_5numpy_inexact) __PYX_ERR(1, 819, __pyx_L1_error)
  __pyx_ptype_5numpy_floating = __Pyx_ImportType_3_0_8(__pyx_t_1, "numpy", "floating", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyObject),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_5numpy_floating) __PYX_ERR(1, 821, __pyx_L1_error)
  __pyx_ptype_5numpy_complexfloating = __Pyx_ImportType_3_0_8(__pyx_t_1, "numpy", "complexfloating", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyObject),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_5numpy_complexfloating) __PYX_ERR(1, 823, __pyx_L1_error)
  __pyx_ptype_5numpy_flexible = __Pyx_ImportType_3_0_8(__pyx_t_1, "numpy", "flexible", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyObject),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_5numpy_flexible) __PYX_ERR(1, 825, __pyx_L1_error)
  __pyx_ptype_5numpy_character = __Pyx_ImportType_3_0_8(__pyx_t_1, "numpy", "character", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyObject),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_5numpy_character) __PYX_ERR(1, 827, __pyx_L1_error)
  __pyx_ptype_5numpy_ufunc = __Pyx_ImportType_3_0_8(__pyx_t_1, "numpy", "ufunc", sizeof(PyUFuncObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(PyUFuncObject),__Pyx_ImportType_CheckSize_Ignore_3_0_8); if (!__pyx_ptype_5numpy_ufunc) __PYX_ERR(1, 866, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("pysndlib.clm"); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_8pysndlib_3clm_mus_any = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.clm", "mus_any", sizeof(struct __pyx_obj_8pysndlib_3clm_mus_any), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_3clm_mus_any),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_3clm_mus_any) __PYX_ERR(3, 40, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_3clm_mus_any = (struct __pyx_vtabstruct_8pysndlib_3clm_mus_any*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_3clm_mus_any); if (unlikely(!__pyx_vtabptr_8pysndlib_3clm_mus_any)) __PYX_ERR(3, 40, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("pysndlib.generators"); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_8pysndlib_10generators_CLMGenerator = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "CLMGenerator", sizeof(struct __pyx_obj_8pysndlib_10generators_CLMGenerator), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_CLMGenerator),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_CLMGenerator) __PYX_ERR(4, 26, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_CLMGenerator = (struct __pyx_vtabstruct_8pysndlib_10generators_CLMGenerator*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_CLMGenerator); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_CLMGenerator)) __PYX_ERR(4, 26, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Nssb = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Nssb", sizeof(struct __pyx_obj_8pysndlib_10generators_Nssb), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Nssb),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Nssb) __PYX_ERR(4, 30, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Nssb = (struct __pyx_vtabstruct_8pysndlib_10generators_Nssb*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Nssb); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Nssb)) __PYX_ERR(4, 30, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Nxysin = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Nxysin", sizeof(struct __pyx_obj_8pysndlib_10generators_Nxysin), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Nxysin),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Nxysin) __PYX_ERR(4, 41, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Nxysin = (struct __pyx_vtabstruct_8pysndlib_10generators_Nxysin*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Nxysin); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Nxysin)) __PYX_ERR(4, 41, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Nxycos = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Nxycos", sizeof(struct __pyx_obj_8pysndlib_10generators_Nxycos), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Nxycos),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Nxycos) __PYX_ERR(4, 52, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Nxycos = (struct __pyx_vtabstruct_8pysndlib_10generators_Nxycos*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Nxycos); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Nxycos)) __PYX_ERR(4, 52, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Nxy1cos = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Nxy1cos", sizeof(struct __pyx_obj_8pysndlib_10generators_Nxy1cos), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Nxy1cos),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Nxy1cos) __PYX_ERR(4, 62, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Nxy1cos = (struct __pyx_vtabstruct_8pysndlib_10generators_Nxy1cos*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Nxy1cos); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Nxy1cos)) __PYX_ERR(4, 62, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Nxy1sin = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Nxy1sin", sizeof(struct __pyx_obj_8pysndlib_10generators_Nxy1sin), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Nxy1sin),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Nxy1sin) __PYX_ERR(4, 72, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Nxy1sin = (struct __pyx_vtabstruct_8pysndlib_10generators_Nxy1sin*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Nxy1sin); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Nxy1sin)) __PYX_ERR(4, 72, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Noddcos = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Noddcos", sizeof(struct __pyx_obj_8pysndlib_10generators_Noddcos), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Noddcos),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Noddcos) __PYX_ERR(4, 82, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Noddcos = (struct __pyx_vtabstruct_8pysndlib_10generators_Noddcos*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Noddcos); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Noddcos)) __PYX_ERR(4, 82, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Noddssb = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Noddssb", sizeof(struct __pyx_obj_8pysndlib_10generators_Noddssb), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Noddssb),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Noddssb) __PYX_ERR(4, 93, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Noddssb = (struct __pyx_vtabstruct_8pysndlib_10generators_Noddssb*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Noddssb); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Noddssb)) __PYX_ERR(4, 93, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Ncos2 = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Ncos2", sizeof(struct __pyx_obj_8pysndlib_10generators_Ncos2), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Ncos2),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Ncos2) __PYX_ERR(4, 103, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Ncos2 = (struct __pyx_vtabstruct_8pysndlib_10generators_Ncos2*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Ncos2); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Ncos2)) __PYX_ERR(4, 103, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Ncos4 = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Ncos4", sizeof(struct __pyx_obj_8pysndlib_10generators_Ncos4), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Ncos4),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Ncos4) __PYX_ERR(4, 112, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Ncos4 = (struct __pyx_vtabstruct_8pysndlib_10generators_Ncos4*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Ncos4); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Ncos4)) __PYX_ERR(4, 112, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Npcos = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Npcos", sizeof(struct __pyx_obj_8pysndlib_10generators_Npcos), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Npcos),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Npcos) __PYX_ERR(4, 119, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Npcos = (struct __pyx_vtabstruct_8pysndlib_10generators_Npcos*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Npcos); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Npcos)) __PYX_ERR(4, 119, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Ncos5 = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Ncos5", sizeof(struct __pyx_obj_8pysndlib_10generators_Ncos5), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Ncos5),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Ncos5) __PYX_ERR(4, 128, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Ncos5 = (struct __pyx_vtabstruct_8pysndlib_10generators_Ncos5*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Ncos5); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Ncos5)) __PYX_ERR(4, 128, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Nsin5 = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Nsin5", sizeof(struct __pyx_obj_8pysndlib_10generators_Nsin5), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Nsin5),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Nsin5) __PYX_ERR(4, 139, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Nsin5 = (struct __pyx_vtabstruct_8pysndlib_10generators_Nsin5*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Nsin5); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Nsin5)) __PYX_ERR(4, 139, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Nrcos = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Nrcos", sizeof(struct __pyx_obj_8pysndlib_10generators_Nrcos), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Nrcos),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Nrcos) __PYX_ERR(4, 155, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Nrcos = (struct __pyx_vtabstruct_8pysndlib_10generators_Nrcos*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Nrcos); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Nrcos)) __PYX_ERR(4, 155, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Nrssb = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Nrssb", sizeof(struct __pyx_obj_8pysndlib_10generators_Nrssb), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Nrssb),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Nrssb) __PYX_ERR(4, 171, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Nrssb = (struct __pyx_vtabstruct_8pysndlib_10generators_Nrssb*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Nrssb); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Nrssb)) __PYX_ERR(4, 171, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Nkssb = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Nkssb", sizeof(struct __pyx_obj_8pysndlib_10generators_Nkssb), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Nkssb),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Nkssb) __PYX_ERR(4, 188, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Nkssb = (struct __pyx_vtabstruct_8pysndlib_10generators_Nkssb*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Nkssb); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Nkssb)) __PYX_ERR(4, 188, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Nsincos = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Nsincos", sizeof(struct __pyx_obj_8pysndlib_10generators_Nsincos), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Nsincos),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Nsincos) __PYX_ERR(4, 202, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Nsincos = (struct __pyx_vtabstruct_8pysndlib_10generators_Nsincos*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Nsincos); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Nsincos)) __PYX_ERR(4, 202, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_N1cos = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "N1cos", sizeof(struct __pyx_obj_8pysndlib_10generators_N1cos), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_N1cos),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_N1cos) __PYX_ERR(4, 215, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_N1cos = (struct __pyx_vtabstruct_8pysndlib_10generators_N1cos*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_N1cos); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_N1cos)) __PYX_ERR(4, 215, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Npos1cos = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Npos1cos", sizeof(struct __pyx_obj_8pysndlib_10generators_Npos1cos), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Npos1cos),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Npos1cos) __PYX_ERR(4, 225, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Npos1cos = (struct __pyx_vtabstruct_8pysndlib_10generators_Npos1cos*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Npos1cos); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Npos1cos)) __PYX_ERR(4, 225, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Npos3cos = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Npos3cos", sizeof(struct __pyx_obj_8pysndlib_10generators_Npos3cos), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Npos3cos),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Npos3cos) __PYX_ERR(4, 234, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Npos3cos = (struct __pyx_vtabstruct_8pysndlib_10generators_Npos3cos*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Npos3cos); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Npos3cos)) __PYX_ERR(4, 234, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Rcos = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Rcos", sizeof(struct __pyx_obj_8pysndlib_10generators_Rcos), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Rcos),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Rcos) __PYX_ERR(4, 242, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Rcos = (struct __pyx_vtabstruct_8pysndlib_10generators_Rcos*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Rcos); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Rcos)) __PYX_ERR(4, 242, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Rssb = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Rssb", sizeof(struct __pyx_obj_8pysndlib_10generators_Rssb), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Rssb),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Rssb) __PYX_ERR(4, 257, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Rssb = (struct __pyx_vtabstruct_8pysndlib_10generators_Rssb*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Rssb); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Rssb)) __PYX_ERR(4, 257, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Rxysin = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Rxysin", sizeof(struct __pyx_obj_8pysndlib_10generators_Rxysin), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Rxysin),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Rxysin) __PYX_ERR(4, 272, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Rxysin = (struct __pyx_vtabstruct_8pysndlib_10generators_Rxysin*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Rxysin); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Rxysin)) __PYX_ERR(4, 272, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Rxycos = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Rxycos", sizeof(struct __pyx_obj_8pysndlib_10generators_Rxycos), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Rxycos),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Rxycos) __PYX_ERR(4, 285, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Rxycos = (struct __pyx_vtabstruct_8pysndlib_10generators_Rxycos*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Rxycos); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Rxycos)) __PYX_ERR(4, 285, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_SafeRxycos = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "SafeRxycos", sizeof(struct __pyx_obj_8pysndlib_10generators_SafeRxycos), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_SafeRxycos),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_SafeRxycos) __PYX_ERR(4, 300, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_SafeRxycos = (struct __pyx_vtabstruct_8pysndlib_10generators_SafeRxycos*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_SafeRxycos); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_SafeRxycos)) __PYX_ERR(4, 300, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Ercos = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Ercos", sizeof(struct __pyx_obj_8pysndlib_10generators_Ercos), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Ercos),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Ercos) __PYX_ERR(4, 312, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Ercos = (struct __pyx_vtabstruct_8pysndlib_10generators_Ercos*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Ercos); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Ercos)) __PYX_ERR(4, 312, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Erssb = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Erssb", sizeof(struct __pyx_obj_8pysndlib_10generators_Erssb), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Erssb),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Erssb) __PYX_ERR(4, 325, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Erssb = (struct __pyx_vtabstruct_8pysndlib_10generators_Erssb*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Erssb); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Erssb)) __PYX_ERR(4, 325, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_R2cos = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "R2cos", sizeof(struct __pyx_obj_8pysndlib_10generators_R2cos), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_R2cos),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_R2cos) __PYX_ERR(4, 340, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_R2cos = (struct __pyx_vtabstruct_8pysndlib_10generators_R2cos*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_R2cos); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_R2cos)) __PYX_ERR(4, 340, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_R2ssb = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "R2ssb", sizeof(struct __pyx_obj_8pysndlib_10generators_R2ssb), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_R2ssb),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_R2ssb) __PYX_ERR(4, 350, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_R2ssb = (struct __pyx_vtabstruct_8pysndlib_10generators_R2ssb*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_R2ssb); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_R2ssb)) __PYX_ERR(4, 350, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Eoddcos = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Eoddcos", sizeof(struct __pyx_obj_8pysndlib_10generators_Eoddcos), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Eoddcos),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Eoddcos) __PYX_ERR(4, 362, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Eoddcos = (struct __pyx_vtabstruct_8pysndlib_10generators_Eoddcos*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Eoddcos); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Eoddcos)) __PYX_ERR(4, 362, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Rkcos = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Rkcos", sizeof(struct __pyx_obj_8pysndlib_10generators_Rkcos), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Rkcos),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Rkcos) __PYX_ERR(4, 373, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Rkcos = (struct __pyx_vtabstruct_8pysndlib_10generators_Rkcos*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Rkcos); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Rkcos)) __PYX_ERR(4, 373, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Rksin = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Rksin", sizeof(struct __pyx_obj_8pysndlib_10generators_Rksin), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Rksin),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Rksin) __PYX_ERR(4, 384, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Rksin = (struct __pyx_vtabstruct_8pysndlib_10generators_Rksin*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Rksin); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Rksin)) __PYX_ERR(4, 384, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Rkssb = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Rkssb", sizeof(struct __pyx_obj_8pysndlib_10generators_Rkssb), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Rkssb),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Rkssb) __PYX_ERR(4, 395, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Rkssb = (struct __pyx_vtabstruct_8pysndlib_10generators_Rkssb*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Rkssb); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Rkssb)) __PYX_ERR(4, 395, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Rkfcos = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Rkfcos", sizeof(struct __pyx_obj_8pysndlib_10generators_Rkfcos), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Rkfcos),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Rkfcos) __PYX_ERR(4, 406, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Rkfcos = (struct __pyx_vtabstruct_8pysndlib_10generators_Rkfcos*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Rkfcos); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Rkfcos)) __PYX_ERR(4, 406, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Rkfssb = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Rkfssb", sizeof(struct __pyx_obj_8pysndlib_10generators_Rkfssb), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Rkfssb),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Rkfssb) __PYX_ERR(4, 419, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Rkfssb = (struct __pyx_vtabstruct_8pysndlib_10generators_Rkfssb*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Rkfssb); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Rkfssb)) __PYX_ERR(4, 419, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_R2kfcos = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "R2kfcos", sizeof(struct __pyx_obj_8pysndlib_10generators_R2kfcos), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_R2kfcos),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_R2kfcos) __PYX_ERR(4, 436, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_R2kfcos = (struct __pyx_vtabstruct_8pysndlib_10generators_R2kfcos*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_R2kfcos); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_R2kfcos)) __PYX_ERR(4, 436, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_K2sin = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "K2sin", sizeof(struct __pyx_obj_8pysndlib_10generators_K2sin), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_K2sin),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_K2sin) __PYX_ERR(4, 450, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_K2sin = (struct __pyx_vtabstruct_8pysndlib_10generators_K2sin*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_K2sin); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_K2sin)) __PYX_ERR(4, 450, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_K2cos = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "K2cos", sizeof(struct __pyx_obj_8pysndlib_10generators_K2cos), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_K2cos),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_K2cos) __PYX_ERR(4, 459, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_K2cos = (struct __pyx_vtabstruct_8pysndlib_10generators_K2cos*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_K2cos); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_K2cos)) __PYX_ERR(4, 459, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_K2ssb = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "K2ssb", sizeof(struct __pyx_obj_8pysndlib_10generators_K2ssb), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_K2ssb),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_K2ssb) __PYX_ERR(4, 468, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_K2ssb = (struct __pyx_vtabstruct_8pysndlib_10generators_K2ssb*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_K2ssb); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_K2ssb)) __PYX_ERR(4, 468, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Dblsum = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Dblsum", sizeof(struct __pyx_obj_8pysndlib_10generators_Dblsum), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Dblsum),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Dblsum) __PYX_ERR(4, 478, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Dblsum = (struct __pyx_vtabstruct_8pysndlib_10generators_Dblsum*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Dblsum); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Dblsum)) __PYX_ERR(4, 478, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Rkoddssb = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Rkoddssb", sizeof(struct __pyx_obj_8pysndlib_10generators_Rkoddssb), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Rkoddssb),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Rkoddssb) __PYX_ERR(4, 488, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Rkoddssb = (struct __pyx_vtabstruct_8pysndlib_10generators_Rkoddssb*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Rkoddssb); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Rkoddssb)) __PYX_ERR(4, 488, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Krksin = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Krksin", sizeof(struct __pyx_obj_8pysndlib_10generators_Krksin), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Krksin),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Krksin) __PYX_ERR(4, 501, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Krksin = (struct __pyx_vtabstruct_8pysndlib_10generators_Krksin*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Krksin); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Krksin)) __PYX_ERR(4, 501, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Absin = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Absin", sizeof(struct __pyx_obj_8pysndlib_10generators_Absin), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Absin),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Absin) __PYX_ERR(4, 511, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Absin = (struct __pyx_vtabstruct_8pysndlib_10generators_Absin*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Absin); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Absin)) __PYX_ERR(4, 511, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Abcos = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Abcos", sizeof(struct __pyx_obj_8pysndlib_10generators_Abcos), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Abcos),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Abcos) __PYX_ERR(4, 520, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Abcos = (struct __pyx_vtabstruct_8pysndlib_10generators_Abcos*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Abcos); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Abcos)) __PYX_ERR(4, 520, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_R2k2cos = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "R2k2cos", sizeof(struct __pyx_obj_8pysndlib_10generators_R2k2cos), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_R2k2cos),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_R2k2cos) __PYX_ERR(4, 533, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_R2k2cos = (struct __pyx_vtabstruct_8pysndlib_10generators_R2k2cos*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_R2k2cos); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_R2k2cos)) __PYX_ERR(4, 533, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Blsaw = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Blsaw", sizeof(struct __pyx_obj_8pysndlib_10generators_Blsaw), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Blsaw),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Blsaw) __PYX_ERR(4, 544, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Blsaw = (struct __pyx_vtabstruct_8pysndlib_10generators_Blsaw*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Blsaw); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Blsaw)) __PYX_ERR(4, 544, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Asyfm = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Asyfm", sizeof(struct __pyx_obj_8pysndlib_10generators_Asyfm), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Asyfm),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Asyfm) __PYX_ERR(4, 556, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Asyfm = (struct __pyx_vtabstruct_8pysndlib_10generators_Asyfm*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Asyfm); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Asyfm)) __PYX_ERR(4, 556, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Bess = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Bess", sizeof(struct __pyx_obj_8pysndlib_10generators_Bess), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Bess),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Bess) __PYX_ERR(4, 572, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Bess = (struct __pyx_vtabstruct_8pysndlib_10generators_Bess*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Bess); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Bess)) __PYX_ERR(4, 572, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Jjcos = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Jjcos", sizeof(struct __pyx_obj_8pysndlib_10generators_Jjcos), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Jjcos),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Jjcos) __PYX_ERR(4, 583, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Jjcos = (struct __pyx_vtabstruct_8pysndlib_10generators_Jjcos*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Jjcos); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Jjcos)) __PYX_ERR(4, 583, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_J0evencos = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "J0evencos", sizeof(struct __pyx_obj_8pysndlib_10generators_J0evencos), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_J0evencos),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_J0evencos) __PYX_ERR(4, 594, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_J0evencos = (struct __pyx_vtabstruct_8pysndlib_10generators_J0evencos*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_J0evencos); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_J0evencos)) __PYX_ERR(4, 594, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_J2cos = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "J2cos", sizeof(struct __pyx_obj_8pysndlib_10generators_J2cos), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_J2cos),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_J2cos) __PYX_ERR(4, 605, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_J2cos = (struct __pyx_vtabstruct_8pysndlib_10generators_J2cos*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_J2cos); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_J2cos)) __PYX_ERR(4, 605, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Jpcos = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Jpcos", sizeof(struct __pyx_obj_8pysndlib_10generators_Jpcos), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Jpcos),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Jpcos) __PYX_ERR(4, 618, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Jpcos = (struct __pyx_vtabstruct_8pysndlib_10generators_Jpcos*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Jpcos); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Jpcos)) __PYX_ERR(4, 618, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Jncos = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Jncos", sizeof(struct __pyx_obj_8pysndlib_10generators_Jncos), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Jncos),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Jncos) __PYX_ERR(4, 631, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Jncos = (struct __pyx_vtabstruct_8pysndlib_10generators_Jncos*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Jncos); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Jncos)) __PYX_ERR(4, 631, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_J0j1cos = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "J0j1cos", sizeof(struct __pyx_obj_8pysndlib_10generators_J0j1cos), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_J0j1cos),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_J0j1cos) __PYX_ERR(4, 644, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_J0j1cos = (struct __pyx_vtabstruct_8pysndlib_10generators_J0j1cos*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_J0j1cos); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_J0j1cos)) __PYX_ERR(4, 644, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Jycos = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Jycos", sizeof(struct __pyx_obj_8pysndlib_10generators_Jycos), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Jycos),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Jycos) __PYX_ERR(4, 654, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Jycos = (struct __pyx_vtabstruct_8pysndlib_10generators_Jycos*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Jycos); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Jycos)) __PYX_ERR(4, 654, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Jcos = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Jcos", sizeof(struct __pyx_obj_8pysndlib_10generators_Jcos), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Jcos),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Jcos) __PYX_ERR(4, 664, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Jcos = (struct __pyx_vtabstruct_8pysndlib_10generators_Jcos*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Jcos); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Jcos)) __PYX_ERR(4, 664, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Blackman = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Blackman", sizeof(struct __pyx_obj_8pysndlib_10generators_Blackman), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Blackman),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Blackman) __PYX_ERR(4, 674, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Blackman = (struct __pyx_vtabstruct_8pysndlib_10generators_Blackman*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Blackman); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Blackman)) __PYX_ERR(4, 674, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Fmssb = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Fmssb", sizeof(struct __pyx_obj_8pysndlib_10generators_Fmssb), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Fmssb),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Fmssb) __PYX_ERR(4, 685, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Fmssb = (struct __pyx_vtabstruct_8pysndlib_10generators_Fmssb*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Fmssb); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Fmssb)) __PYX_ERR(4, 685, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_K3sin = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "K3sin", sizeof(struct __pyx_obj_8pysndlib_10generators_K3sin), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_K3sin),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_K3sin) __PYX_ERR(4, 696, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_K3sin = (struct __pyx_vtabstruct_8pysndlib_10generators_K3sin*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_K3sin); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_K3sin)) __PYX_ERR(4, 696, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Izcos = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Izcos", sizeof(struct __pyx_obj_8pysndlib_10generators_Izcos), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Izcos),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Izcos) __PYX_ERR(4, 706, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Izcos = (struct __pyx_vtabstruct_8pysndlib_10generators_Izcos*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Izcos); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Izcos)) __PYX_ERR(4, 706, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_AdjustableSquareWave = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "AdjustableSquareWave", sizeof(struct __pyx_obj_8pysndlib_10generators_AdjustableSquareWave), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_AdjustableSquareWave),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_AdjustableSquareWave) __PYX_ERR(4, 719, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_AdjustableSquareWave = (struct __pyx_vtabstruct_8pysndlib_10generators_AdjustableSquareWave*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_AdjustableSquareWave); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_AdjustableSquareWave)) __PYX_ERR(4, 719, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_AdjustableTriangleWave = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "AdjustableTriangleWave", sizeof(struct __pyx_obj_8pysndlib_10generators_AdjustableTriangleWave), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_AdjustableTriangleWave),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_AdjustableTriangleWave) __PYX_ERR(4, 732, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_AdjustableTriangleWave = (struct __pyx_vtabstruct_8pysndlib_10generators_AdjustableTriangleWave*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_AdjustableTriangleWave); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_AdjustableTriangleWave)) __PYX_ERR(4, 732, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_AdjustableSawtoothWave = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "AdjustableSawtoothWave", sizeof(struct __pyx_obj_8pysndlib_10generators_AdjustableSawtoothWave), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_AdjustableSawtoothWave),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_AdjustableSawtoothWave) __PYX_ERR(4, 748, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_AdjustableSawtoothWave = (struct __pyx_vtabstruct_8pysndlib_10generators_AdjustableSawtoothWave*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_AdjustableSawtoothWave); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_AdjustableSawtoothWave)) __PYX_ERR(4, 748, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_AdjustableOscil = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "AdjustableOscil", sizeof(struct __pyx_obj_8pysndlib_10generators_AdjustableOscil), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_AdjustableOscil),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_AdjustableOscil) __PYX_ERR(4, 763, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_AdjustableOscil = (struct __pyx_vtabstruct_8pysndlib_10generators_AdjustableOscil*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_AdjustableOscil); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_AdjustableOscil)) __PYX_ERR(4, 763, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_RoundInterp = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "RoundInterp", sizeof(struct __pyx_obj_8pysndlib_10generators_RoundInterp), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_RoundInterp),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_RoundInterp) __PYX_ERR(4, 784, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_RoundInterp = (struct __pyx_vtabstruct_8pysndlib_10generators_RoundInterp*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_RoundInterp); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_RoundInterp)) __PYX_ERR(4, 784, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Nchoosekcos = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Nchoosekcos", sizeof(struct __pyx_obj_8pysndlib_10generators_Nchoosekcos), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Nchoosekcos),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Nchoosekcos) __PYX_ERR(4, 814, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Nchoosekcos = (struct __pyx_vtabstruct_8pysndlib_10generators_Nchoosekcos*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Nchoosekcos); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Nchoosekcos)) __PYX_ERR(4, 814, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_SincTrain = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "SincTrain", sizeof(struct __pyx_obj_8pysndlib_10generators_SincTrain), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_SincTrain),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_SincTrain) __PYX_ERR(4, 827, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_SincTrain = (struct __pyx_vtabstruct_8pysndlib_10generators_SincTrain*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_SincTrain); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_SincTrain)) __PYX_ERR(4, 827, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_PinkNoise = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "PinkNoise", sizeof(struct __pyx_obj_8pysndlib_10generators_PinkNoise), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_PinkNoise),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_PinkNoise) __PYX_ERR(4, 838, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_PinkNoise = (struct __pyx_vtabstruct_8pysndlib_10generators_PinkNoise*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_PinkNoise); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_PinkNoise)) __PYX_ERR(4, 838, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_BrownNoise = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "BrownNoise", sizeof(struct __pyx_obj_8pysndlib_10generators_BrownNoise), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_BrownNoise),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_BrownNoise) __PYX_ERR(4, 846, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_BrownNoise = (struct __pyx_vtabstruct_8pysndlib_10generators_BrownNoise*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_BrownNoise); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_BrownNoise)) __PYX_ERR(4, 846, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_GreenNoise = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "GreenNoise", sizeof(struct __pyx_obj_8pysndlib_10generators_GreenNoise), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_GreenNoise),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_GreenNoise) __PYX_ERR(4, 858, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_GreenNoise = (struct __pyx_vtabstruct_8pysndlib_10generators_GreenNoise*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_GreenNoise); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_GreenNoise)) __PYX_ERR(4, 858, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_GreenNoiseInterp = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "GreenNoiseInterp", sizeof(struct __pyx_obj_8pysndlib_10generators_GreenNoiseInterp), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_GreenNoiseInterp),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_GreenNoiseInterp) __PYX_ERR(4, 868, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_GreenNoiseInterp = (struct __pyx_vtabstruct_8pysndlib_10generators_GreenNoiseInterp*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_GreenNoiseInterp); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_GreenNoiseInterp)) __PYX_ERR(4, 868, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_MovingSum = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "MovingSum", sizeof(struct __pyx_obj_8pysndlib_10generators_MovingSum), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_MovingSum),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_MovingSum) __PYX_ERR(4, 882, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_MovingSum = (struct __pyx_vtabstruct_8pysndlib_10generators_MovingSum*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_MovingSum); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_MovingSum)) __PYX_ERR(4, 882, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_MovingVariance = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "MovingVariance", sizeof(struct __pyx_obj_8pysndlib_10generators_MovingVariance), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_MovingVariance),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_MovingVariance) __PYX_ERR(4, 891, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_MovingVariance = (struct __pyx_vtabstruct_8pysndlib_10generators_MovingVariance*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_MovingVariance); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_MovingVariance)) __PYX_ERR(4, 891, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_MovingRMS = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "MovingRMS", sizeof(struct __pyx_obj_8pysndlib_10generators_MovingRMS), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_MovingRMS),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_MovingRMS) __PYX_ERR(4, 901, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_MovingRMS = (struct __pyx_vtabstruct_8pysndlib_10generators_MovingRMS*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_MovingRMS); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_MovingRMS)) __PYX_ERR(4, 901, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_MovingLength = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "MovingLength", sizeof(struct __pyx_obj_8pysndlib_10generators_MovingLength), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_MovingLength),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_MovingLength) __PYX_ERR(4, 910, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_MovingLength = (struct __pyx_vtabstruct_8pysndlib_10generators_MovingLength*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_MovingLength); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_MovingLength)) __PYX_ERR(4, 910, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_WeightedMovingAverage = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "WeightedMovingAverage", sizeof(struct __pyx_obj_8pysndlib_10generators_WeightedMovingAverage), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_WeightedMovingAverage),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_WeightedMovingAverage) __PYX_ERR(4, 918, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_WeightedMovingAverage = (struct __pyx_vtabstruct_8pysndlib_10generators_WeightedMovingAverage*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_WeightedMovingAverage); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_WeightedMovingAverage)) __PYX_ERR(4, 918, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_ExponentiallyWeightedMovingAverage = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "ExponentiallyWeightedMovingAverage", sizeof(struct __pyx_obj_8pysndlib_10generators_ExponentiallyWeightedMovingAverage), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_ExponentiallyWeightedMovingAverage),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_ExponentiallyWeightedMovingAverage) __PYX_ERR(4, 931, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_ExponentiallyWeightedMovingAverage = (struct __pyx_vtabstruct_8pysndlib_10generators_ExponentiallyWeightedMovingAverage*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_ExponentiallyWeightedMovingAverage); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_ExponentiallyWeightedMovingAverage)) __PYX_ERR(4, 931, __pyx_L1_error)
  __pyx_ptype_8pysndlib_10generators_Tanhsin = __Pyx_ImportType_3_0_8(__pyx_t_1, "pysndlib.generators", "Tanhsin", sizeof(struct __pyx_obj_8pysndlib_10generators_Tanhsin), __PYX_GET_STRUCT_ALIGNMENT_3_0_8(struct __pyx_obj_8pysndlib_10generators_Tanhsin),__Pyx_ImportType_CheckSize_Warn_3_0_8); if (!__pyx_ptype_8pysndlib_10generators_Tanhsin) __PYX_ERR(4, 956, __pyx_L1_error)
  __pyx_vtabptr_8pysndlib_10generators_Tanhsin = (struct __pyx_vtabstruct_8pysndlib_10generators_Tanhsin*)__Pyx_GetVtable(__pyx_ptype_8pysndlib_10generators_Tanhsin); if (unlikely(!__pyx_vtabptr_8pysndlib_10generators_Tanhsin)) __PYX_ERR(4, 956, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __pyx_t_1 = PyImport_ImportModule("pysndlib.clm"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportFunction_3_0_8(__pyx_t_1, "seconds2samples", (void (**)(void))&__pyx_f_8pysndlib_3clm_seconds2samples, "long (double, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction_3_0_8(__pyx_t_1, "oscil", (void (**)(void))&__pyx_f_8pysndlib_3clm_oscil, "double (struct __pyx_obj_8pysndlib_3clm_mus_any *, int __pyx_skip_dispatch, struct __pyx_opt_args_8pysndlib_3clm_oscil *__pyx_optional_args)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction_3_0_8(__pyx_t_1, "env", (void (**)(void))&__pyx_f_8pysndlib_3clm_env, "double (struct __pyx_obj_8pysndlib_3clm_mus_any *, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction_3_0_8(__pyx_t_1, "outa", (void (**)(void))&__pyx_f_8pysndlib_3clm_outa, "PyObject *(long, double, int __pyx_skip_dispatch, struct __pyx_opt_args_8pysndlib_3clm_outa *__pyx_optional_args)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("pysndlib.generators"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportFunction_3_0_8(__pyx_t_1, "rkfssb", (void (**)(void))&__pyx_f_8pysndlib_10generators_rkfssb, "double (struct __pyx_obj_8pysndlib_10generators_Rkfssb *, int __pyx_skip_dispatch, struct __pyx_opt_args_8pysndlib_10generators_rkfssb *__pyx_optional_args)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}


#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_bouncy(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_bouncy},
  {0, NULL}
};
#endif

#ifdef __cplusplus
namespace {
  struct PyModuleDef __pyx_moduledef =
  #else
  static struct PyModuleDef __pyx_moduledef =
  #endif
  {
      PyModuleDef_HEAD_INIT,
      "bouncy",
      0, /* m_doc */
    #if CYTHON_PEP489_MULTI_PHASE_INIT
      0, /* m_size */
    #elif CYTHON_USE_MODULE_STATE
      sizeof(__pyx_mstate), /* m_size */
    #else
      -1, /* m_size */
    #endif
      __pyx_methods /* m_methods */,
    #if CYTHON_PEP489_MULTI_PHASE_INIT
      __pyx_moduledef_slots, /* m_slots */
    #else
      NULL, /* m_reload */
    #endif
    #if CYTHON_USE_MODULE_STATE
      __pyx_m_traverse, /* m_traverse */
      __pyx_m_clear, /* m_clear */
      NULL /* m_free */
    #else
      NULL, /* m_traverse */
      NULL, /* m_clear */
      NULL /* m_free */
    #endif
  };
  #ifdef __cplusplus
} /* anonymous namespace */
#endif
#endif

#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initbouncy(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initbouncy(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_bouncy(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_bouncy(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *module, const char* from_name, const char* to_name, int allow_none)
#else
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none)
#endif
{
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
#if CYTHON_COMPILING_IN_LIMITED_API
            result = PyModule_AddObject(module, to_name, value);
#else
            result = PyDict_SetItemString(moddict, to_name, value);
#endif
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    CYTHON_UNUSED_VAR(def);
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
#if CYTHON_COMPILING_IN_LIMITED_API
    moddict = module;
#else
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
#endif
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_bouncy(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  int stringtab_initialized = 0;
  #if CYTHON_USE_MODULE_STATE
  int pystate_addmodule_run = 0;
  #endif
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  struct __pyx_opt_args_8pysndlib_11instruments_6bouncy_bouncy __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'bouncy' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("bouncy", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #elif CYTHON_USE_MODULE_STATE
  __pyx_t_1 = PyModule_Create(&__pyx_moduledef); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  {
    int add_module_result = PyState_AddModule(__pyx_t_1, &__pyx_moduledef);
    __pyx_t_1 = 0; /* transfer ownership from __pyx_t_1 to "bouncy" pseudovariable */
    if (unlikely((add_module_result < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
    pystate_addmodule_run = 1;
  }
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #endif
  CYTHON_UNUSED_VAR(__pyx_t_1);
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = __Pyx_PyImport_AddModuleRef(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_cython_runtime = __Pyx_PyImport_AddModuleRef((const char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_bouncy(void)", 0);
  if (__Pyx_check_binary_version(__PYX_LIMITED_VERSION_HEX, __Pyx_get_runtime_version(), CYTHON_COMPILING_IN_LIMITED_API) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init(__pyx_m) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(WITH_THREAD) && PY_VERSION_HEX < 0x030700F0 && defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  PyEval_InitThreads();
  #endif
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  stringtab_initialized = 1;
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_pysndlib__instruments__bouncy) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "pysndlib.instruments.bouncy")) {
      if (unlikely((PyDict_SetItemString(modules, "pysndlib.instruments.bouncy", __pyx_m) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  (void)__Pyx_modinit_type_init_code();
  if (unlikely((__Pyx_modinit_type_import_code() < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_variable_import_code();
  if (unlikely((__Pyx_modinit_function_import_code() < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "pysndlib/instruments/bouncy.py":6
 * #==================================================================================
 * 
 * import math             # <<<<<<<<<<<<<<
 * import random
 * import cython
 */
  __pyx_t_2 = __Pyx_ImportDottedModule(__pyx_n_s_math, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_math, __pyx_t_2) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pysndlib/instruments/bouncy.py":7
 * 
 * import math
 * import random             # <<<<<<<<<<<<<<
 * import cython
 * import numpy as np
 */
  __pyx_t_2 = __Pyx_ImportDottedModule(__pyx_n_s_random, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_random, __pyx_t_2) < 0) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pysndlib/instruments/bouncy.py":9
 * import random
 * import cython
 * import numpy as np             # <<<<<<<<<<<<<<
 * import pysndlib.clm as clm
 * import pysndlib.generators as gens
 */
  __pyx_t_2 = __Pyx_ImportDottedModule(__pyx_n_s_numpy, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_np, __pyx_t_2) < 0) __PYX_ERR(0, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pysndlib/instruments/bouncy.py":10
 * import cython
 * import numpy as np
 * import pysndlib.clm as clm             # <<<<<<<<<<<<<<
 * import pysndlib.generators as gens
 * if cython.compiled:
 */
  __pyx_t_2 = __Pyx_ImportDottedModule(__pyx_n_s_pysndlib_clm, __pyx_tuple__4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_clm, __pyx_t_2) < 0) __PYX_ERR(0, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pysndlib/instruments/bouncy.py":11
 * import numpy as np
 * import pysndlib.clm as clm
 * import pysndlib.generators as gens             # <<<<<<<<<<<<<<
 * if cython.compiled:
 *     from cython.cimports.pysndlib import clm
 */
  __pyx_t_2 = __Pyx_ImportDottedModule(__pyx_n_s_pysndlib_generators, __pyx_tuple__5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gens, __pyx_t_2) < 0) __PYX_ERR(0, 11, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pysndlib/instruments/bouncy.py":20
 * 
 * # --------------- bouncy ---------------- #
 * @cython.ccall             # <<<<<<<<<<<<<<
 * def bouncy(start, dur, freq, amp, bounce_freq=5, bounce_amp=20):
 *     lng: cython.long = clm.seconds2samples(dur)
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_8pysndlib_11instruments_6bouncy_1bouncy, 0, __pyx_n_s_bouncy, NULL, __pyx_n_s_pysndlib_instruments_bouncy, __pyx_d, ((PyObject *)__pyx_codeobj__7)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 20, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_tuple__8);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_bouncy, __pyx_t_2) < 0) __PYX_ERR(0, 20, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pysndlib/instruments/bouncy.py":43
 *         j+=1
 * 
 * if __name__ == '__main__':             # <<<<<<<<<<<<<<
 * 
 *     with clm.Sound(clipped=False, statistics=True, play=True):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_name); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_t_2, __pyx_n_u_main, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {

    /* "pysndlib/instruments/bouncy.py":45
 * if __name__ == '__main__':
 * 
 *     with clm.Sound(clipped=False, statistics=True, play=True):             # <<<<<<<<<<<<<<
 *         bouncy(0,2,300,.5, 5, 10)
 *         bouncy(2.5,2,200,.5, 3, 2)
 */
    /*with:*/ {
      __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_clm); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 45, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_Sound); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 45, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 45, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_clipped, Py_False) < 0) __PYX_ERR(0, 45, __pyx_L1_error)
      if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_statistics, Py_True) < 0) __PYX_ERR(0, 45, __pyx_L1_error)
      if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_play, Py_True) < 0) __PYX_ERR(0, 45, __pyx_L1_error)
      __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 45, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_1 = __Pyx_PyObject_LookupSpecial(__pyx_t_5, __pyx_n_s_exit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 45, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_2 = __Pyx_PyObject_LookupSpecial(__pyx_t_5, __pyx_n_s_enter); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 45, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 45, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      /*try:*/ {
        {
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __Pyx_ExceptionSave(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
          __Pyx_XGOTREF(__pyx_t_6);
          __Pyx_XGOTREF(__pyx_t_7);
          __Pyx_XGOTREF(__pyx_t_8);
          /*try:*/ {

            /* "pysndlib/instruments/bouncy.py":46
 * 
 *     with clm.Sound(clipped=False, statistics=True, play=True):
 *         bouncy(0,2,300,.5, 5, 10)             # <<<<<<<<<<<<<<
 *         bouncy(2.5,2,200,.5, 3, 2)
 * 
 */
            __pyx_t_9.__pyx_n = 2;
            __pyx_t_9.bounce_freq = __pyx_int_5;
            __pyx_t_9.bounce_amp = __pyx_int_10;
            __pyx_t_5 = __pyx_f_8pysndlib_11instruments_6bouncy_bouncy(__pyx_int_0, __pyx_int_2, __pyx_int_300, __pyx_float__5, 0, &__pyx_t_9); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 46, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_5);
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

            /* "pysndlib/instruments/bouncy.py":47
 *     with clm.Sound(clipped=False, statistics=True, play=True):
 *         bouncy(0,2,300,.5, 5, 10)
 *         bouncy(2.5,2,200,.5, 3, 2)             # <<<<<<<<<<<<<<
 * 
 */
            __pyx_t_9.__pyx_n = 2;
            __pyx_t_9.bounce_freq = __pyx_int_3;
            __pyx_t_9.bounce_amp = __pyx_int_2;
            __pyx_t_5 = __pyx_f_8pysndlib_11instruments_6bouncy_bouncy(__pyx_float_2_5, __pyx_int_2, __pyx_int_200, __pyx_float__5, 0, &__pyx_t_9); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 47, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_5);
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

            /* "pysndlib/instruments/bouncy.py":45
 * if __name__ == '__main__':
 * 
 *     with clm.Sound(clipped=False, statistics=True, play=True):             # <<<<<<<<<<<<<<
 *         bouncy(0,2,300,.5, 5, 10)
 *         bouncy(2.5,2,200,.5, 3, 2)
 */
          }
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          goto __pyx_L12_try_end;
          __pyx_L7_error:;
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          /*except:*/ {
            __Pyx_AddTraceback("pysndlib.instruments.bouncy", __pyx_clineno, __pyx_lineno, __pyx_filename);
            if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_4, &__pyx_t_2) < 0) __PYX_ERR(0, 45, __pyx_L9_except_error)
            __Pyx_XGOTREF(__pyx_t_5);
            __Pyx_XGOTREF(__pyx_t_4);
            __Pyx_XGOTREF(__pyx_t_2);
            __pyx_t_10 = PyTuple_Pack(3, __pyx_t_5, __pyx_t_4, __pyx_t_2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 45, __pyx_L9_except_error)
            __Pyx_GOTREF(__pyx_t_10);
            __pyx_t_11 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_10, NULL);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 45, __pyx_L9_except_error)
            __Pyx_GOTREF(__pyx_t_11);
            __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_11);
            __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
            if (__pyx_t_3 < 0) __PYX_ERR(0, 45, __pyx_L9_except_error)
            __pyx_t_12 = (!__pyx_t_3);
            if (unlikely(__pyx_t_12)) {
              __Pyx_GIVEREF(__pyx_t_5);
              __Pyx_GIVEREF(__pyx_t_4);
              __Pyx_XGIVEREF(__pyx_t_2);
              __Pyx_ErrRestoreWithState(__pyx_t_5, __pyx_t_4, __pyx_t_2);
              __pyx_t_5 = 0; __pyx_t_4 = 0; __pyx_t_2 = 0; 
              __PYX_ERR(0, 45, __pyx_L9_except_error)
            }
            __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
            __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
            goto __pyx_L8_exception_handled;
          }
          __pyx_L9_except_error:;
          __Pyx_XGIVEREF(__pyx_t_6);
          __Pyx_XGIVEREF(__pyx_t_7);
          __Pyx_XGIVEREF(__pyx_t_8);
          __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
          goto __pyx_L1_error;
          __pyx_L8_exception_handled:;
          __Pyx_XGIVEREF(__pyx_t_6);
          __Pyx_XGIVEREF(__pyx_t_7);
          __Pyx_XGIVEREF(__pyx_t_8);
          __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
          __pyx_L12_try_end:;
        }
      }
      /*finally:*/ {
        /*normal exit:*/{
          if (__pyx_t_1) {
            __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__9, NULL);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 45, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_8);
            __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          }
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
      goto __pyx_L16;
      __pyx_L3_error:;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L1_error;
      __pyx_L16:;
    }

    /* "pysndlib/instruments/bouncy.py":43
 *         j+=1
 * 
 * if __name__ == '__main__':             # <<<<<<<<<<<<<<
 * 
 *     with clm.Sound(clipped=False, statistics=True, play=True):
 */
  }

  /* "pysndlib/instruments/bouncy.py":1
 * #==================================================================================             # <<<<<<<<<<<<<<
 * # The code is ported from Bill Schottstedaet's 'generators.scm'
 * # file  available at https://ccrma.stanford.edu/software/snd/sndlib/
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_10);
  if (__pyx_m) {
    if (__pyx_d && stringtab_initialized) {
      __Pyx_AddTraceback("init pysndlib.instruments.bouncy", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    #if !CYTHON_USE_MODULE_STATE
    Py_CLEAR(__pyx_m);
    #else
    Py_DECREF(__pyx_m);
    if (pystate_addmodule_run) {
      PyObject *tp, *value, *tb;
      PyErr_Fetch(&tp, &value, &tb);
      PyState_RemoveModule(&__pyx_moduledef);
      PyErr_Restore(tp, value, tb);
    }
    #endif
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init pysndlib.instruments.bouncy");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}
/* #### Code section: cleanup_globals ### */
/* #### Code section: cleanup_module ### */
/* #### Code section: main_method ### */
/* #### Code section: utility_code_pragmas ### */
#ifdef _MSC_VER
#pragma warning( push )
/* Warning 4127: conditional expression is constant
 * Cython uses constant conditional expressions to allow in inline functions to be optimized at
 * compile-time, so this warning is not useful
 */
#pragma warning( disable : 4127 )
#endif



/* #### Code section: utility_code_def ### */

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    int result;
    PyObject *exc_type;
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *current_exception = tstate->current_exception;
    if (unlikely(!current_exception)) return 0;
    exc_type = (PyObject*) Py_TYPE(current_exception);
    if (exc_type == err) return 1;
#else
    exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
#endif
    #if CYTHON_AVOID_BORROWED_REFS
    Py_INCREF(exc_type);
    #endif
    if (unlikely(PyTuple_Check(err))) {
        result = __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    } else {
        result = __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
    }
    #if CYTHON_AVOID_BORROWED_REFS
    Py_DECREF(exc_type);
    #endif
    return result;
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *tmp_value;
    assert(type == NULL || (value != NULL && type == (PyObject*) Py_TYPE(value)));
    if (value) {
        #if CYTHON_COMPILING_IN_CPYTHON
        if (unlikely(((PyBaseExceptionObject*) value)->traceback != tb))
        #endif
            PyException_SetTraceback(value, tb);
    }
    tmp_value = tstate->current_exception;
    tstate->current_exception = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
#else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject* exc_value;
    exc_value = tstate->current_exception;
    tstate->current_exception = 0;
    *value = exc_value;
    *type = NULL;
    *tb = NULL;
    if (exc_value) {
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        #if CYTHON_COMPILING_IN_CPYTHON
        *tb = ((PyBaseExceptionObject*) exc_value)->traceback;
        Py_XINCREF(*tb);
        #else
        *tb = PyException_GetTraceback(exc_value);
        #endif
    }
#else
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#endif
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* PyObjectGetAttrStrNoError */
#if __PYX_LIMITED_VERSION_HEX < 0x030d00A1
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if __PYX_LIMITED_VERSION_HEX >= 0x030d00A1
    (void) PyObject_GetOptionalAttr(obj, attr_name, &result);
    return result;
#else
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS && PY_VERSION_HEX >= 0x030700B1
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
#endif
}

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStrNoError(__pyx_b, name);
    if (unlikely(!result) && !PyErr_Occurred()) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK && CYTHON_FAST_THREAD_STATE
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_value == NULL || exc_info->exc_value == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    PyObject *exc_value = exc_info->exc_value;
    if (exc_value == NULL || exc_value == Py_None) {
        *value = NULL;
        *type = NULL;
        *tb = NULL;
    } else {
        *value = exc_value;
        Py_INCREF(*value);
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        *tb = PyException_GetTraceback(exc_value);
    }
  #elif CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
  #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
  #endif
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = tstate->exc_info;
    PyObject *tmp_value = exc_info->exc_value;
    exc_info->exc_value = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
  #else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
  #endif
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type = NULL, *local_value, *local_tb = NULL;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
  #if PY_VERSION_HEX >= 0x030C00A6
    local_value = tstate->current_exception;
    tstate->current_exception = 0;
    if (likely(local_value)) {
        local_type = (PyObject*) Py_TYPE(local_value);
        Py_INCREF(local_type);
        local_tb = PyException_GetTraceback(local_value);
    }
  #else
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
  #endif
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE && PY_VERSION_HEX >= 0x030C00A6
    if (unlikely(tstate->current_exception))
#elif CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
      #if PY_VERSION_HEX >= 0x030B00a4
        tmp_value = exc_info->exc_value;
        exc_info->exc_value = local_value;
        tmp_type = NULL;
        tmp_tb = NULL;
        Py_XDECREF(local_type);
        Py_XDECREF(local_tb);
      #else
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
      #endif
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    #if PY_MAJOR_VERSION < 3
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    #else
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    #endif
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    __Pyx_PyThreadState_declare
    CYTHON_UNUSED_VAR(cause);
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
      #if PY_VERSION_HEX >= 0x030C00A6
        PyException_SetTraceback(value, tb);
      #elif CYTHON_FAST_THREAD_STATE
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#else
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && PY_VERSION_HEX < 0x030d0000
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#elif CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(!__pyx_m)) {
        return NULL;
    }
    result = PyObject_GetAttr(__pyx_m, name);
    if (likely(result)) {
        return result;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* PyIntBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_MultiplyObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check) {
    CYTHON_MAYBE_UNUSED_VAR(intval);
    CYTHON_MAYBE_UNUSED_VAR(inplace);
    CYTHON_UNUSED_VAR(zerodivision_check);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        
#ifdef HAVE_LONG_LONG
            if (sizeof(PY_LONG_LONG) > sizeof(long)) {
                PY_LONG_LONG result = (PY_LONG_LONG)a * (PY_LONG_LONG)b;
                return (result >= LONG_MIN && result <= LONG_MAX) ?
                    PyInt_FromLong((long)result) : PyLong_FromLongLong(result);
            }
#endif
#if CYTHON_USE_TYPE_SLOTS
            return PyInt_Type.tp_as_number->nb_multiply(op1, op2);
#else
            return PyNumber_Multiply(op1, op2);
#endif
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        if (unlikely(__Pyx_PyLong_IsZero(op1))) {
            return __Pyx_NewRef(op1);
        }
        if (likely(__Pyx_PyLong_IsCompact(op1))) {
            a = __Pyx_PyLong_CompactValue(op1);
        } else {
            const digit* digits = __Pyx_PyLong_Digits(op1);
            const Py_ssize_t size = __Pyx_PyLong_SignedDigitCount(op1);
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT+30) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT+30) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT+30) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT+30) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT+30) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT+30) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT+30) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT+30) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT+30) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT+30) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT+30) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    #ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT+30) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    #endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_multiply(op1, op2);
            }
        }
                CYTHON_UNUSED_VAR(a);
                CYTHON_UNUSED_VAR(b);
                #ifdef HAVE_LONG_LONG
                lla = a;
                goto long_long;
                #else
                return PyLong_Type.tp_as_number->nb_multiply(op1, op2);
                #endif
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla * llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
#if CYTHON_COMPILING_IN_LIMITED_API
        double a = __pyx_PyFloat_AsDouble(op1);
#else
        double a = PyFloat_AS_DOUBLE(op1);
#endif
            double result;
            
            PyFPE_START_PROTECT("multiply", return NULL)
            result = ((double)a) * (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceMultiply : PyNumber_Multiply)(op1, op2);
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL && !CYTHON_VECTORCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    #if PY_MAJOR_VERSION < 3
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object"))) {
        return NULL;
    }
    #else
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object"))) {
        return NULL;
    }
    #endif
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = __Pyx_CyOrPyCFunction_GET_FUNCTION(func);
    self = __Pyx_CyOrPyCFunction_GET_SELF(func);
    #if PY_MAJOR_VERSION < 3
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    #else
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    #endif
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectFastCall */
#if PY_VERSION_HEX < 0x03090000 || CYTHON_COMPILING_IN_LIMITED_API
static PyObject* __Pyx_PyObject_FastCall_fallback(PyObject *func, PyObject **args, size_t nargs, PyObject *kwargs) {
    PyObject *argstuple;
    PyObject *result = 0;
    size_t i;
    argstuple = PyTuple_New((Py_ssize_t)nargs);
    if (unlikely(!argstuple)) return NULL;
    for (i = 0; i < nargs; i++) {
        Py_INCREF(args[i]);
        if (__Pyx_PyTuple_SET_ITEM(argstuple, (Py_ssize_t)i, args[i]) < 0) goto bad;
    }
    result = __Pyx_PyObject_Call(func, argstuple, kwargs);
  bad:
    Py_DECREF(argstuple);
    return result;
}
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject **args, size_t _nargs, PyObject *kwargs) {
    Py_ssize_t nargs = __Pyx_PyVectorcall_NARGS(_nargs);
#if CYTHON_COMPILING_IN_CPYTHON
    if (nargs == 0 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_NOARGS))
            return __Pyx_PyObject_CallMethO(func, NULL);
    }
    else if (nargs == 1 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_O))
            return __Pyx_PyObject_CallMethO(func, args[0]);
    }
#endif
    #if PY_VERSION_HEX < 0x030800B1
    #if CYTHON_FAST_PYCCALL
    if (PyCFunction_Check(func)) {
        if (kwargs) {
            return _PyCFunction_FastCallDict(func, args, nargs, kwargs);
        } else {
            return _PyCFunction_FastCallKeywords(func, args, nargs, NULL);
        }
    }
    #if PY_VERSION_HEX >= 0x030700A1
    if (!kwargs && __Pyx_IS_TYPE(func, &PyMethodDescr_Type)) {
        return _PyMethodDescr_FastCallKeywords(func, args, nargs, NULL);
    }
    #endif
    #endif
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs);
    }
    #endif
    #endif
    if (kwargs == NULL) {
        #if CYTHON_VECTORCALL
        #if PY_VERSION_HEX < 0x03090000
        vectorcallfunc f = _PyVectorcall_Function(func);
        #else
        vectorcallfunc f = PyVectorcall_Function(func);
        #endif
        if (f) {
            return f(func, args, (size_t)nargs, NULL);
        }
        #elif defined(__Pyx_CyFunction_USED) && CYTHON_BACKPORT_VECTORCALL
        if (__Pyx_CyFunction_CheckExact(func)) {
            __pyx_vectorcallfunc f = __Pyx_CyFunction_func_vectorcall(func);
            if (f) return f(func, args, (size_t)nargs, NULL);
        }
        #endif
    }
    if (nargs == 0) {
        return __Pyx_PyObject_Call(func, __pyx_empty_tuple, kwargs);
    }
    #if PY_VERSION_HEX >= 0x03090000 && !CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_VectorcallDict(func, args, (size_t)nargs, kwargs);
    #else
    return __Pyx_PyObject_FastCall_fallback(func, args, (size_t)nargs, kwargs);
    #endif
}

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    __Pyx_TypeName obj_type_name;
    __Pyx_TypeName type_name;
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    obj_type_name = __Pyx_PyType_GetName(Py_TYPE(obj));
    type_name = __Pyx_PyType_GetName(type);
    PyErr_Format(PyExc_TypeError,
                 "Cannot convert " __Pyx_FMT_TYPENAME " to " __Pyx_FMT_TYPENAME,
                 obj_type_name, type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    __Pyx_DECREF_TypeName(type_name);
    return 0;
}

/* SetItemInt */
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v) {
    int r;
    if (unlikely(!j)) return -1;
    r = PyObject_SetItem(o, j, v);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v, int is_list,
                                               CYTHON_NCP_UNUSED int wraparound, CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = (!wraparound) ? i : ((likely(i >= 0)) ? i : i + PyList_GET_SIZE(o));
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o)))) {
            PyObject* old = PyList_GET_ITEM(o, n);
            Py_INCREF(v);
            PyList_SET_ITEM(o, n, v);
            Py_DECREF(old);
            return 1;
        }
    } else {
        PyMappingMethods *mm = Py_TYPE(o)->tp_as_mapping;
        PySequenceMethods *sm = Py_TYPE(o)->tp_as_sequence;
        if (mm && mm->mp_ass_subscript) {
            int r;
            PyObject *key = PyInt_FromSsize_t(i);
            if (unlikely(!key)) return -1;
            r = mm->mp_ass_subscript(o, key, v);
            Py_DECREF(key);
            return r;
        }
        if (likely(sm && sm->sq_ass_item)) {
            if (wraparound && unlikely(i < 0) && likely(sm->sq_length)) {
                Py_ssize_t l = sm->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return -1;
                    PyErr_Clear();
                }
            }
            return sm->sq_ass_item(o, i, v);
        }
    }
#else
    if (is_list || !PyMapping_Check(o))
    {
        return PySequence_SetItem(o, i, v);
    }
#endif
    return __Pyx_SetItemInt_Generic(o, PyInt_FromSsize_t(i), v);
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (unlikely(!j)) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PyMappingMethods *mm = Py_TYPE(o)->tp_as_mapping;
        PySequenceMethods *sm = Py_TYPE(o)->tp_as_sequence;
        if (mm && mm->mp_subscript) {
            PyObject *r, *key = PyInt_FromSsize_t(i);
            if (unlikely(!key)) return NULL;
            r = mm->mp_subscript(o, key);
            Py_DECREF(key);
            return r;
        }
        if (likely(sm && sm->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(sm->sq_length)) {
                Py_ssize_t l = sm->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return sm->sq_item(o, i);
        }
    }
#else
    if (is_list || !PyMapping_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* PyObjectSetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_setattr))
        return tp->tp_setattr(obj, PyString_AS_STRING(attr_name), value);
#endif
    return PyObject_SetAttr(obj, attr_name, value);
}
#endif

/* TupleAndListFromArray */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE void __Pyx_copy_object_array(PyObject *const *CYTHON_RESTRICT src, PyObject** CYTHON_RESTRICT dest, Py_ssize_t length) {
    PyObject *v;
    Py_ssize_t i;
    for (i = 0; i < length; i++) {
        v = dest[i] = src[i];
        Py_INCREF(v);
    }
}
static CYTHON_INLINE PyObject *
__Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        Py_INCREF(__pyx_empty_tuple);
        return __pyx_empty_tuple;
    }
    res = PyTuple_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyTupleObject*)res)->ob_item, n);
    return res;
}
static CYTHON_INLINE PyObject *
__Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        return PyList_New(0);
    }
    res = PyList_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyListObject*)res)->ob_item, n);
    return res;
}
#endif

/* BytesEquals */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS && (PY_VERSION_HEX < 0x030B0000)
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        #if PY_MAJOR_VERSION < 3
        Py_XDECREF(owned_ref);
        #endif
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* fastcall */
#if CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s)
{
    Py_ssize_t i, n = PyTuple_GET_SIZE(kwnames);
    for (i = 0; i < n; i++)
    {
        if (s == PyTuple_GET_ITEM(kwnames, i)) return kwvalues[i];
    }
    for (i = 0; i < n; i++)
    {
        int eq = __Pyx_PyUnicode_Equals(s, PyTuple_GET_ITEM(kwnames, i), Py_EQ);
        if (unlikely(eq != 0)) {
            if (unlikely(eq < 0)) return NULL;
            return kwvalues[i];
        }
    }
    return NULL;
}
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
CYTHON_UNUSED static PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues) {
    Py_ssize_t i, nkwargs = PyTuple_GET_SIZE(kwnames);
    PyObject *dict;
    dict = PyDict_New();
    if (unlikely(!dict))
        return NULL;
    for (i=0; i<nkwargs; i++) {
        PyObject *key = PyTuple_GET_ITEM(kwnames, i);
        if (unlikely(PyDict_SetItem(dict, key, kwvalues[i]) < 0))
            goto bad;
    }
    return dict;
bad:
    Py_DECREF(dict);
    return NULL;
}
#endif
#endif

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject *const *kwvalues,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    int kwds_is_tuple = CYTHON_METH_FASTCALL && likely(PyTuple_Check(kwds));
    while (1) {
        Py_XDECREF(key); key = NULL;
        Py_XDECREF(value); value = NULL;
        if (kwds_is_tuple) {
            Py_ssize_t size;
#if CYTHON_ASSUME_SAFE_MACROS
            size = PyTuple_GET_SIZE(kwds);
#else
            size = PyTuple_Size(kwds);
            if (size < 0) goto bad;
#endif
            if (pos >= size) break;
#if CYTHON_AVOID_BORROWED_REFS
            key = __Pyx_PySequence_ITEM(kwds, pos);
            if (!key) goto bad;
#elif CYTHON_ASSUME_SAFE_MACROS
            key = PyTuple_GET_ITEM(kwds, pos);
#else
            key = PyTuple_GetItem(kwds, pos);
            if (!key) goto bad;
#endif
            value = kwvalues[pos];
            pos++;
        }
        else
        {
            if (!PyDict_Next(kwds, &pos, &key, &value)) break;
#if CYTHON_AVOID_BORROWED_REFS
            Py_INCREF(key);
#endif
        }
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
#if CYTHON_AVOID_BORROWED_REFS
            Py_INCREF(value);
            Py_DECREF(key);
#endif
            key = NULL;
            value = NULL;
            continue;
        }
#if !CYTHON_AVOID_BORROWED_REFS
        Py_INCREF(key);
#endif
        Py_INCREF(value);
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
#if CYTHON_AVOID_BORROWED_REFS
                    value = NULL;
#endif
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key)
                );
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
#if CYTHON_AVOID_BORROWED_REFS
                    value = NULL;
#endif
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    Py_XDECREF(key);
    Py_XDECREF(value);
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    #if PY_MAJOR_VERSION < 3
    PyErr_Format(PyExc_TypeError,
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
    PyErr_Format(PyExc_TypeError,
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    Py_XDECREF(key);
    Py_XDECREF(value);
    return -1;
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType_3_0_8
#define __PYX_HAVE_RT_ImportType_3_0_8
static PyTypeObject *__Pyx_ImportType_3_0_8(PyObject *module, const char *module_name, const char *class_name,
    size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_3_0_8 check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
    Py_ssize_t itemsize;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_basicsize;
    PyObject *py_itemsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#if !CYTHON_COMPILING_IN_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
    itemsize = ((PyTypeObject *)result)->tp_itemsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
    py_itemsize = PyObject_GetAttrString(result, "__itemsize__");
    if (!py_itemsize)
        goto bad;
    itemsize = PyLong_AsSsize_t(py_itemsize);
    Py_DECREF(py_itemsize);
    py_itemsize = 0;
    if (itemsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (itemsize) {
        if (size % alignment) {
            alignment = size % alignment;
        }
        if (itemsize < (Py_ssize_t)alignment)
            itemsize = (Py_ssize_t)alignment;
    }
    if ((size_t)(basicsize + itemsize) < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize+itemsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error_3_0_8 &&
            ((size_t)basicsize > size || (size_t)(basicsize + itemsize) < size)) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd-%zd from PyObject",
            module_name, class_name, size, basicsize, basicsize+itemsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn_3_0_8 && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* GetVTable */
static void* __Pyx_GetVtable(PyTypeObject *type) {
    void* ptr;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *ob = PyObject_GetAttr((PyObject *)type, __pyx_n_s_pyx_vtable);
#else
    PyObject *ob = PyObject_GetItem(type->tp_dict, __pyx_n_s_pyx_vtable);
#endif
    if (!ob)
        goto bad;
    ptr = PyCapsule_GetPointer(ob, 0);
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *module = 0;
    PyObject *empty_dict = 0;
    PyObject *empty_list = 0;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (unlikely(!py_import))
        goto bad;
    if (!from_list) {
        empty_list = PyList_New(0);
        if (unlikely(!empty_list))
            goto bad;
        from_list = empty_list;
    }
    #endif
    empty_dict = PyDict_New();
    if (unlikely(!empty_dict))
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.') != NULL) {
                module = PyImport_ImportModuleLevelObject(
                    name, __pyx_d, empty_dict, from_list, 1);
                if (unlikely(!module)) {
                    if (unlikely(!PyErr_ExceptionMatches(PyExc_ImportError)))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (unlikely(!py_level))
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, __pyx_d, empty_dict, from_list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, __pyx_d, empty_dict, from_list, level);
            #endif
        }
    }
bad:
    Py_XDECREF(empty_dict);
    Py_XDECREF(empty_list);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    return module;
}

/* ImportDottedModule */
#if PY_MAJOR_VERSION >= 3
static PyObject *__Pyx__ImportDottedModule_Error(PyObject *name, PyObject *parts_tuple, Py_ssize_t count) {
    PyObject *partial_name = NULL, *slice = NULL, *sep = NULL;
    if (unlikely(PyErr_Occurred())) {
        PyErr_Clear();
    }
    if (likely(PyTuple_GET_SIZE(parts_tuple) == count)) {
        partial_name = name;
    } else {
        slice = PySequence_GetSlice(parts_tuple, 0, count);
        if (unlikely(!slice))
            goto bad;
        sep = PyUnicode_FromStringAndSize(".", 1);
        if (unlikely(!sep))
            goto bad;
        partial_name = PyUnicode_Join(sep, slice);
    }
    PyErr_Format(
#if PY_MAJOR_VERSION < 3
        PyExc_ImportError,
        "No module named '%s'", PyString_AS_STRING(partial_name));
#else
#if PY_VERSION_HEX >= 0x030600B1
        PyExc_ModuleNotFoundError,
#else
        PyExc_ImportError,
#endif
        "No module named '%U'", partial_name);
#endif
bad:
    Py_XDECREF(sep);
    Py_XDECREF(slice);
    Py_XDECREF(partial_name);
    return NULL;
}
#endif
#if PY_MAJOR_VERSION >= 3
static PyObject *__Pyx__ImportDottedModule_Lookup(PyObject *name) {
    PyObject *imported_module;
#if PY_VERSION_HEX < 0x030700A1 || (CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM  < 0x07030400)
    PyObject *modules = PyImport_GetModuleDict();
    if (unlikely(!modules))
        return NULL;
    imported_module = __Pyx_PyDict_GetItemStr(modules, name);
    Py_XINCREF(imported_module);
#else
    imported_module = PyImport_GetModule(name);
#endif
    return imported_module;
}
#endif
#if PY_MAJOR_VERSION >= 3
static PyObject *__Pyx_ImportDottedModule_WalkParts(PyObject *module, PyObject *name, PyObject *parts_tuple) {
    Py_ssize_t i, nparts;
    nparts = PyTuple_GET_SIZE(parts_tuple);
    for (i=1; i < nparts && module; i++) {
        PyObject *part, *submodule;
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        part = PyTuple_GET_ITEM(parts_tuple, i);
#else
        part = PySequence_ITEM(parts_tuple, i);
#endif
        submodule = __Pyx_PyObject_GetAttrStrNoError(module, part);
#if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
        Py_DECREF(part);
#endif
        Py_DECREF(module);
        module = submodule;
    }
    if (unlikely(!module)) {
        return __Pyx__ImportDottedModule_Error(name, parts_tuple, i);
    }
    return module;
}
#endif
static PyObject *__Pyx__ImportDottedModule(PyObject *name, PyObject *parts_tuple) {
#if PY_MAJOR_VERSION < 3
    PyObject *module, *from_list, *star = __pyx_n_s__3;
    CYTHON_UNUSED_VAR(parts_tuple);
    from_list = PyList_New(1);
    if (unlikely(!from_list))
        return NULL;
    Py_INCREF(star);
    PyList_SET_ITEM(from_list, 0, star);
    module = __Pyx_Import(name, from_list, 0);
    Py_DECREF(from_list);
    return module;
#else
    PyObject *imported_module;
    PyObject *module = __Pyx_Import(name, NULL, 0);
    if (!parts_tuple || unlikely(!module))
        return module;
    imported_module = __Pyx__ImportDottedModule_Lookup(name);
    if (likely(imported_module)) {
        Py_DECREF(module);
        return imported_module;
    }
    PyErr_Clear();
    return __Pyx_ImportDottedModule_WalkParts(module, name, parts_tuple);
#endif
}
static PyObject *__Pyx_ImportDottedModule(PyObject *name, PyObject *parts_tuple) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030400B1
    PyObject *module = __Pyx__ImportDottedModule_Lookup(name);
    if (likely(module)) {
        PyObject *spec = __Pyx_PyObject_GetAttrStrNoError(module, __pyx_n_s_spec);
        if (likely(spec)) {
            PyObject *unsafe = __Pyx_PyObject_GetAttrStrNoError(spec, __pyx_n_s_initializing);
            if (likely(!unsafe || !__Pyx_PyObject_IsTrue(unsafe))) {
                Py_DECREF(spec);
                spec = NULL;
            }
            Py_XDECREF(unsafe);
        }
        if (likely(!spec)) {
            PyErr_Clear();
            return module;
        }
        Py_DECREF(spec);
        Py_DECREF(module);
    } else if (PyErr_Occurred()) {
        PyErr_Clear();
    }
#endif
    return __Pyx__ImportDottedModule(name, parts_tuple);
}

/* FixUpExtensionType */
#if CYTHON_USE_TYPE_SPECS
static int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type) {
#if PY_VERSION_HEX > 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    CYTHON_UNUSED_VAR(spec);
    CYTHON_UNUSED_VAR(type);
#else
    const PyType_Slot *slot = spec->slots;
    while (slot && slot->slot && slot->slot != Py_tp_members)
        slot++;
    if (slot && slot->slot == Py_tp_members) {
        int changed = 0;
#if !(PY_VERSION_HEX <= 0x030900b1 && CYTHON_COMPILING_IN_CPYTHON)
        const
#endif
            PyMemberDef *memb = (PyMemberDef*) slot->pfunc;
        while (memb && memb->name) {
            if (memb->name[0] == '_' && memb->name[1] == '_') {
#if PY_VERSION_HEX < 0x030900b1
                if (strcmp(memb->name, "__weaklistoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_weaklistoffset = memb->offset;
                    changed = 1;
                }
                else if (strcmp(memb->name, "__dictoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_dictoffset = memb->offset;
                    changed = 1;
                }
#if CYTHON_METH_FASTCALL
                else if (strcmp(memb->name, "__vectorcalloffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
#if PY_VERSION_HEX >= 0x030800b4
                    type->tp_vectorcall_offset = memb->offset;
#else
                    type->tp_print = (printfunc) memb->offset;
#endif
                    changed = 1;
                }
#endif
#else
                if ((0));
#endif
#if PY_VERSION_HEX <= 0x030900b1 && CYTHON_COMPILING_IN_CPYTHON
                else if (strcmp(memb->name, "__module__") == 0) {
                    PyObject *descr;
                    assert(memb->type == T_OBJECT);
                    assert(memb->flags == 0 || memb->flags == READONLY);
                    descr = PyDescr_NewMember(type, memb);
                    if (unlikely(!descr))
                        return -1;
                    if (unlikely(PyDict_SetItem(type->tp_dict, PyDescr_NAME(descr), descr) < 0)) {
                        Py_DECREF(descr);
                        return -1;
                    }
                    Py_DECREF(descr);
                    changed = 1;
                }
#endif
            }
            memb++;
        }
        if (changed)
            PyType_Modified(type);
    }
#endif
    return 0;
}
#endif

/* FetchSharedCythonModule */
static PyObject *__Pyx_FetchSharedCythonABIModule(void) {
    return __Pyx_PyImport_AddModuleRef((char*) __PYX_ABI_MODULE_NAME);
}

/* FetchCommonType */
static int __Pyx_VerifyCachedType(PyObject *cached_type,
                               const char *name,
                               Py_ssize_t basicsize,
                               Py_ssize_t expected_basicsize) {
    if (!PyType_Check(cached_type)) {
        PyErr_Format(PyExc_TypeError,
            "Shared Cython type %.200s is not a type object", name);
        return -1;
    }
    if (basicsize != expected_basicsize) {
        PyErr_Format(PyExc_TypeError,
            "Shared Cython type %.200s has the wrong size, try recompiling",
            name);
        return -1;
    }
    return 0;
}
#if !CYTHON_USE_TYPE_SPECS
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* abi_module;
    const char* object_name;
    PyTypeObject *cached_type = NULL;
    abi_module = __Pyx_FetchSharedCythonABIModule();
    if (!abi_module) return NULL;
    object_name = strrchr(type->tp_name, '.');
    object_name = object_name ? object_name+1 : type->tp_name;
    cached_type = (PyTypeObject*) PyObject_GetAttrString(abi_module, object_name);
    if (cached_type) {
        if (__Pyx_VerifyCachedType(
              (PyObject *)cached_type,
              object_name,
              cached_type->tp_basicsize,
              type->tp_basicsize) < 0) {
            goto bad;
        }
        goto done;
    }
    if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
    PyErr_Clear();
    if (PyType_Ready(type) < 0) goto bad;
    if (PyObject_SetAttrString(abi_module, object_name, (PyObject *)type) < 0)
        goto bad;
    Py_INCREF(type);
    cached_type = type;
done:
    Py_DECREF(abi_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}
#else
static PyTypeObject *__Pyx_FetchCommonTypeFromSpec(PyObject *module, PyType_Spec *spec, PyObject *bases) {
    PyObject *abi_module, *cached_type = NULL;
    const char* object_name = strrchr(spec->name, '.');
    object_name = object_name ? object_name+1 : spec->name;
    abi_module = __Pyx_FetchSharedCythonABIModule();
    if (!abi_module) return NULL;
    cached_type = PyObject_GetAttrString(abi_module, object_name);
    if (cached_type) {
        Py_ssize_t basicsize;
#if CYTHON_COMPILING_IN_LIMITED_API
        PyObject *py_basicsize;
        py_basicsize = PyObject_GetAttrString(cached_type, "__basicsize__");
        if (unlikely(!py_basicsize)) goto bad;
        basicsize = PyLong_AsSsize_t(py_basicsize);
        Py_DECREF(py_basicsize);
        py_basicsize = 0;
        if (unlikely(basicsize == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
#else
        basicsize = likely(PyType_Check(cached_type)) ? ((PyTypeObject*) cached_type)->tp_basicsize : -1;
#endif
        if (__Pyx_VerifyCachedType(
              cached_type,
              object_name,
              basicsize,
              spec->basicsize) < 0) {
            goto bad;
        }
        goto done;
    }
    if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
    PyErr_Clear();
    CYTHON_UNUSED_VAR(module);
    cached_type = __Pyx_PyType_FromModuleAndSpec(abi_module, spec, bases);
    if (unlikely(!cached_type)) goto bad;
    if (unlikely(__Pyx_fix_up_extension_type_from_spec(spec, (PyTypeObject *) cached_type) < 0)) goto bad;
    if (PyObject_SetAttrString(abi_module, object_name, cached_type) < 0) goto bad;
done:
    Py_DECREF(abi_module);
    assert(cached_type == NULL || PyType_Check(cached_type));
    return (PyTypeObject *) cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}
#endif

/* PyVectorcallFastCallDict */
#if CYTHON_METH_FASTCALL
static PyObject *__Pyx_PyVectorcall_FastCallDict_kw(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw)
{
    PyObject *res = NULL;
    PyObject *kwnames;
    PyObject **newargs;
    PyObject **kwvalues;
    Py_ssize_t i, pos;
    size_t j;
    PyObject *key, *value;
    unsigned long keys_are_strings;
    Py_ssize_t nkw = PyDict_GET_SIZE(kw);
    newargs = (PyObject **)PyMem_Malloc((nargs + (size_t)nkw) * sizeof(args[0]));
    if (unlikely(newargs == NULL)) {
        PyErr_NoMemory();
        return NULL;
    }
    for (j = 0; j < nargs; j++) newargs[j] = args[j];
    kwnames = PyTuple_New(nkw);
    if (unlikely(kwnames == NULL)) {
        PyMem_Free(newargs);
        return NULL;
    }
    kwvalues = newargs + nargs;
    pos = i = 0;
    keys_are_strings = Py_TPFLAGS_UNICODE_SUBCLASS;
    while (PyDict_Next(kw, &pos, &key, &value)) {
        keys_are_strings &= Py_TYPE(key)->tp_flags;
        Py_INCREF(key);
        Py_INCREF(value);
        PyTuple_SET_ITEM(kwnames, i, key);
        kwvalues[i] = value;
        i++;
    }
    if (unlikely(!keys_are_strings)) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        goto cleanup;
    }
    res = vc(func, newargs, nargs, kwnames);
cleanup:
    Py_DECREF(kwnames);
    for (i = 0; i < nkw; i++)
        Py_DECREF(kwvalues[i]);
    PyMem_Free(newargs);
    return res;
}
static CYTHON_INLINE PyObject *__Pyx_PyVectorcall_FastCallDict(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw)
{
    if (likely(kw == NULL) || PyDict_GET_SIZE(kw) == 0) {
        return vc(func, args, nargs, NULL);
    }
    return __Pyx_PyVectorcall_FastCallDict_kw(func, vc, args, nargs, kw);
}
#endif

/* CythonFunctionShared */
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void *cfunc) {
    if (__Pyx_CyFunction_Check(func)) {
        return PyCFunction_GetFunction(((__pyx_CyFunctionObject*)func)->func) == (PyCFunction) cfunc;
    } else if (PyCFunction_Check(func)) {
        return PyCFunction_GetFunction(func) == (PyCFunction) cfunc;
    }
    return 0;
}
#else
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void *cfunc) {
    return __Pyx_CyOrPyCFunction_Check(func) && __Pyx_CyOrPyCFunction_GET_FUNCTION(func) == (PyCFunction) cfunc;
}
#endif
static CYTHON_INLINE void __Pyx__CyFunction_SetClassObj(__pyx_CyFunctionObject* f, PyObject* classobj) {
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    __Pyx_Py_XDECREF_SET(
        __Pyx_CyFunction_GetClassObj(f),
            ((classobj) ? __Pyx_NewRef(classobj) : NULL));
#else
    __Pyx_Py_XDECREF_SET(
        ((PyCMethodObject *) (f))->mm_class,
        (PyTypeObject*)((classobj) ? __Pyx_NewRef(classobj) : NULL));
#endif
}
static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, void *closure)
{
    CYTHON_UNUSED_VAR(closure);
    if (unlikely(op->func_doc == NULL)) {
#if CYTHON_COMPILING_IN_LIMITED_API
        op->func_doc = PyObject_GetAttrString(op->func, "__doc__");
        if (unlikely(!op->func_doc)) return NULL;
#else
        if (((PyCFunctionObject*)op)->m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(((PyCFunctionObject*)op)->m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(((PyCFunctionObject*)op)->m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
#endif
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_doc, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(op->func_name == NULL)) {
#if CYTHON_COMPILING_IN_LIMITED_API
        op->func_name = PyObject_GetAttrString(op->func, "__name__");
#elif PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(((PyCFunctionObject*)op)->m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(((PyCFunctionObject*)op)->m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_name, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_qualname, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_dict, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(op);
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op, void *context)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = __Pyx_PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = __Pyx_PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value) {
        value = Py_None;
    } else if (unlikely(value != Py_None && !PyTuple_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    PyErr_WarnEx(PyExc_RuntimeWarning, "changes to cyfunction.__defaults__ will not "
                 "currently affect the values used in function calls", 1);
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->defaults_tuple, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result = op->defaults_tuple;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (unlikely(__Pyx_CyFunction_init_defaults(op) < 0)) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value) {
        value = Py_None;
    } else if (unlikely(value != Py_None && !PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    PyErr_WarnEx(PyExc_RuntimeWarning, "changes to cyfunction.__kwdefaults__ will not "
                 "currently affect the values used in function calls", 1);
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->defaults_kwdict, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result = op->defaults_kwdict;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (unlikely(__Pyx_CyFunction_init_defaults(op) < 0)) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value || value == Py_None) {
        value = NULL;
    } else if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_annotations, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result = op->func_annotations;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_is_coroutine(__pyx_CyFunctionObject *op, void *context) {
    int is_coroutine;
    CYTHON_UNUSED_VAR(context);
    if (op->func_is_coroutine) {
        return __Pyx_NewRef(op->func_is_coroutine);
    }
    is_coroutine = op->flags & __Pyx_CYFUNCTION_COROUTINE;
#if PY_VERSION_HEX >= 0x03050000
    if (is_coroutine) {
        PyObject *module, *fromlist, *marker = __pyx_n_s_is_coroutine;
        fromlist = PyList_New(1);
        if (unlikely(!fromlist)) return NULL;
        Py_INCREF(marker);
#if CYTHON_ASSUME_SAFE_MACROS
        PyList_SET_ITEM(fromlist, 0, marker);
#else
        if (unlikely(PyList_SetItem(fromlist, 0, marker) < 0)) {
            Py_DECREF(marker);
            Py_DECREF(fromlist);
            return NULL;
        }
#endif
        module = PyImport_ImportModuleLevelObject(__pyx_n_s_asyncio_coroutines, NULL, NULL, fromlist, 0);
        Py_DECREF(fromlist);
        if (unlikely(!module)) goto ignore;
        op->func_is_coroutine = __Pyx_PyObject_GetAttrStr(module, marker);
        Py_DECREF(module);
        if (likely(op->func_is_coroutine)) {
            return __Pyx_NewRef(op->func_is_coroutine);
        }
ignore:
        PyErr_Clear();
    }
#endif
    op->func_is_coroutine = __Pyx_PyBool_FromLong(is_coroutine);
    return __Pyx_NewRef(op->func_is_coroutine);
}
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *
__Pyx_CyFunction_get_module(__pyx_CyFunctionObject *op, void *context) {
    CYTHON_UNUSED_VAR(context);
    return PyObject_GetAttrString(op->func, "__module__");
}
static int
__Pyx_CyFunction_set_module(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    return PyObject_SetAttrString(op->func, "__module__", value);
}
#endif
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {(char *) "_is_coroutine", (getter)__Pyx_CyFunction_get_is_coroutine, 0, 0, 0},
#if CYTHON_COMPILING_IN_LIMITED_API
    {"__module__", (getter)__Pyx_CyFunction_get_module, (setter)__Pyx_CyFunction_set_module, 0, 0},
#endif
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
#if !CYTHON_COMPILING_IN_LIMITED_API
    {(char *) "__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), 0, 0},
#endif
#if CYTHON_USE_TYPE_SPECS
    {(char *) "__dictoffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_dict), READONLY, 0},
#if CYTHON_METH_FASTCALL
#if CYTHON_BACKPORT_VECTORCALL
    {(char *) "__vectorcalloffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_vectorcall), READONLY, 0},
#else
#if !CYTHON_COMPILING_IN_LIMITED_API
    {(char *) "__vectorcalloffset__", T_PYSSIZET, offsetof(PyCFunctionObject, vectorcall), READONLY, 0},
#endif
#endif
#endif
#if PY_VERSION_HEX < 0x030500A0 || CYTHON_COMPILING_IN_LIMITED_API
    {(char *) "__weaklistoffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_weakreflist), READONLY, 0},
#else
    {(char *) "__weaklistoffset__", T_PYSSIZET, offsetof(PyCFunctionObject, m_weakreflist), READONLY, 0},
#endif
#endif
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, PyObject *args)
{
    CYTHON_UNUSED_VAR(args);
#if PY_MAJOR_VERSION >= 3
    Py_INCREF(m->func_qualname);
    return m->func_qualname;
#else
    return PyString_FromString(((PyCFunctionObject*)m)->m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0 || CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) (((PyCFunctionObject*)cyfunc)->m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject *op, PyMethodDef *ml, int flags, PyObject* qualname,
                                       PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
#if !CYTHON_COMPILING_IN_LIMITED_API
    PyCFunctionObject *cf = (PyCFunctionObject*) op;
#endif
    if (unlikely(op == NULL))
        return NULL;
#if CYTHON_COMPILING_IN_LIMITED_API
    op->func = PyCFunction_NewEx(ml, (PyObject*)op, module);
    if (unlikely(!op->func)) return NULL;
#endif
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
#if !CYTHON_COMPILING_IN_LIMITED_API
    cf->m_ml = ml;
    cf->m_self = (PyObject *) op;
#endif
    Py_XINCREF(closure);
    op->func_closure = closure;
#if !CYTHON_COMPILING_IN_LIMITED_API
    Py_XINCREF(module);
    cf->m_module = module;
#endif
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    op->func_classobj = NULL;
#else
    ((PyCMethodObject*)op)->mm_class = NULL;
#endif
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults_size = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    op->func_is_coroutine = NULL;
#if CYTHON_METH_FASTCALL
    switch (ml->ml_flags & (METH_VARARGS | METH_FASTCALL | METH_NOARGS | METH_O | METH_KEYWORDS | METH_METHOD)) {
    case METH_NOARGS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_NOARGS;
        break;
    case METH_O:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_O;
        break;
    case METH_METHOD | METH_FASTCALL | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD;
        break;
    case METH_FASTCALL | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS;
        break;
    case METH_VARARGS | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = NULL;
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags for CyFunction");
        Py_DECREF(op);
        return NULL;
    }
#endif
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
#if CYTHON_COMPILING_IN_LIMITED_API
    Py_CLEAR(m->func);
#else
    Py_CLEAR(((PyCFunctionObject*)m)->m_module);
#endif
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
#if !CYTHON_COMPILING_IN_LIMITED_API
#if PY_VERSION_HEX < 0x030900B1
    Py_CLEAR(__Pyx_CyFunction_GetClassObj(m));
#else
    {
        PyObject *cls = (PyObject*) ((PyCMethodObject *) (m))->mm_class;
        ((PyCMethodObject *) (m))->mm_class = NULL;
        Py_XDECREF(cls);
    }
#endif
#endif
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    Py_CLEAR(m->func_is_coroutine);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyObject_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    __Pyx_PyHeapTypeObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
#if CYTHON_COMPILING_IN_LIMITED_API
    Py_VISIT(m->func);
#else
    Py_VISIT(((PyCFunctionObject*)m)->m_module);
#endif
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
#if !CYTHON_COMPILING_IN_LIMITED_API
    Py_VISIT(__Pyx_CyFunction_GetClassObj(m));
#endif
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    Py_VISIT(m->func_is_coroutine);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *f = ((__pyx_CyFunctionObject*)func)->func;
    PyObject *py_name = NULL;
    PyCFunction meth;
    int flags;
    meth = PyCFunction_GetFunction(f);
    if (unlikely(!meth)) return NULL;
    flags = PyCFunction_GetFlags(f);
    if (unlikely(flags < 0)) return NULL;
#else
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    int flags = f->m_ml->ml_flags;
#endif
    Py_ssize_t size;
    switch (flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)(void*)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
#if CYTHON_ASSUME_SAFE_MACROS
            size = PyTuple_GET_SIZE(arg);
#else
            size = PyTuple_Size(arg);
            if (unlikely(size < 0)) return NULL;
#endif
            if (likely(size == 0))
                return (*meth)(self, NULL);
#if CYTHON_COMPILING_IN_LIMITED_API
            py_name = __Pyx_CyFunction_get_name((__pyx_CyFunctionObject*)func, NULL);
            if (!py_name) return NULL;
            PyErr_Format(PyExc_TypeError,
                "%.200S() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                py_name, size);
            Py_DECREF(py_name);
#else
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
#endif
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
#if CYTHON_ASSUME_SAFE_MACROS
            size = PyTuple_GET_SIZE(arg);
#else
            size = PyTuple_Size(arg);
            if (unlikely(size < 0)) return NULL;
#endif
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = __Pyx_PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
#if CYTHON_COMPILING_IN_LIMITED_API
            py_name = __Pyx_CyFunction_get_name((__pyx_CyFunctionObject*)func, NULL);
            if (!py_name) return NULL;
            PyErr_Format(PyExc_TypeError,
                "%.200S() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                py_name, size);
            Py_DECREF(py_name);
#else
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
#endif
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags for CyFunction");
        return NULL;
    }
#if CYTHON_COMPILING_IN_LIMITED_API
    py_name = __Pyx_CyFunction_get_name((__pyx_CyFunctionObject*)func, NULL);
    if (!py_name) return NULL;
    PyErr_Format(PyExc_TypeError, "%.200S() takes no keyword arguments",
                 py_name);
    Py_DECREF(py_name);
#else
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
#endif
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *self, *result;
#if CYTHON_COMPILING_IN_LIMITED_API
    self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)func)->func);
    if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
    self = ((PyCFunctionObject*)func)->m_self;
#endif
    result = __Pyx_CyFunction_CallMethod(func, self, arg, kw);
    return result;
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
#if CYTHON_METH_FASTCALL
     __pyx_vectorcallfunc vc = __Pyx_CyFunction_func_vectorcall(cyfunc);
    if (vc) {
#if CYTHON_ASSUME_SAFE_MACROS
        return __Pyx_PyVectorcall_FastCallDict(func, vc, &PyTuple_GET_ITEM(args, 0), (size_t)PyTuple_GET_SIZE(args), kw);
#else
        (void) &__Pyx_PyVectorcall_FastCallDict;
        return PyVectorcall_Call(func, args, kw);
#endif
    }
#endif
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
#if CYTHON_ASSUME_SAFE_MACROS
        argc = PyTuple_GET_SIZE(args);
#else
        argc = PyTuple_Size(args);
        if (unlikely(!argc) < 0) return NULL;
#endif
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
#if PY_MAJOR_VERSION > 2
            PyErr_Format(PyExc_TypeError,
                         "unbound method %.200S() needs an argument",
                         cyfunc->func_qualname);
#else
            PyErr_SetString(PyExc_TypeError,
                            "unbound method needs an argument");
#endif
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
#if CYTHON_METH_FASTCALL
static CYTHON_INLINE int __Pyx_CyFunction_Vectorcall_CheckArgs(__pyx_CyFunctionObject *cyfunc, Py_ssize_t nargs, PyObject *kwnames)
{
    int ret = 0;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        if (unlikely(nargs < 1)) {
            PyErr_Format(PyExc_TypeError, "%.200s() needs an argument",
                         ((PyCFunctionObject*)cyfunc)->m_ml->ml_name);
            return -1;
        }
        ret = 1;
    }
    if (unlikely(kwnames) && unlikely(PyTuple_GET_SIZE(kwnames))) {
        PyErr_Format(PyExc_TypeError,
                     "%.200s() takes no keyword arguments", ((PyCFunctionObject*)cyfunc)->m_ml->ml_name);
        return -1;
    }
    return ret;
}
static PyObject * __Pyx_CyFunction_Vectorcall_NOARGS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyMethodDef* def = ((PyCFunctionObject*)cyfunc)->m_ml;
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, kwnames)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
        self = ((PyCFunctionObject*)cyfunc)->m_self;
        break;
    default:
        return NULL;
    }
    if (unlikely(nargs != 0)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
            def->ml_name, nargs);
        return NULL;
    }
    return def->ml_meth(self, NULL);
}
static PyObject * __Pyx_CyFunction_Vectorcall_O(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyMethodDef* def = ((PyCFunctionObject*)cyfunc)->m_ml;
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, kwnames)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
        self = ((PyCFunctionObject*)cyfunc)->m_self;
        break;
    default:
        return NULL;
    }
    if (unlikely(nargs != 1)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
            def->ml_name, nargs);
        return NULL;
    }
    return def->ml_meth(self, args[0]);
}
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyMethodDef* def = ((PyCFunctionObject*)cyfunc)->m_ml;
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, NULL)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
        self = ((PyCFunctionObject*)cyfunc)->m_self;
        break;
    default:
        return NULL;
    }
    return ((_PyCFunctionFastWithKeywords)(void(*)(void))def->ml_meth)(self, args, nargs, kwnames);
}
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyMethodDef* def = ((PyCFunctionObject*)cyfunc)->m_ml;
    PyTypeObject *cls = (PyTypeObject *) __Pyx_CyFunction_GetClassObj(cyfunc);
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, NULL)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
        self = ((PyCFunctionObject*)cyfunc)->m_self;
        break;
    default:
        return NULL;
    }
    return ((__Pyx_PyCMethod)(void(*)(void))def->ml_meth)(self, cls, args, (size_t)nargs, kwnames);
}
#endif
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_CyFunctionType_slots[] = {
    {Py_tp_dealloc, (void *)__Pyx_CyFunction_dealloc},
    {Py_tp_repr, (void *)__Pyx_CyFunction_repr},
    {Py_tp_call, (void *)__Pyx_CyFunction_CallAsMethod},
    {Py_tp_traverse, (void *)__Pyx_CyFunction_traverse},
    {Py_tp_clear, (void *)__Pyx_CyFunction_clear},
    {Py_tp_methods, (void *)__pyx_CyFunction_methods},
    {Py_tp_members, (void *)__pyx_CyFunction_members},
    {Py_tp_getset, (void *)__pyx_CyFunction_getsets},
    {Py_tp_descr_get, (void *)__Pyx_PyMethod_New},
    {0, 0},
};
static PyType_Spec __pyx_CyFunctionType_spec = {
    __PYX_TYPE_MODULE_PREFIX "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
#ifdef Py_TPFLAGS_METHOD_DESCRIPTOR
    Py_TPFLAGS_METHOD_DESCRIPTOR |
#endif
#if (defined(_Py_TPFLAGS_HAVE_VECTORCALL) && CYTHON_METH_FASTCALL)
    _Py_TPFLAGS_HAVE_VECTORCALL |
#endif
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_BASETYPE,
    __pyx_CyFunctionType_slots
};
#else
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    __PYX_TYPE_MODULE_PREFIX "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
#if !CYTHON_METH_FASTCALL
    0,
#elif CYTHON_BACKPORT_VECTORCALL
    (printfunc)offsetof(__pyx_CyFunctionObject, func_vectorcall),
#else
    offsetof(PyCFunctionObject, vectorcall),
#endif
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_CallAsMethod,
    0,
    0,
    0,
    0,
#ifdef Py_TPFLAGS_METHOD_DESCRIPTOR
    Py_TPFLAGS_METHOD_DESCRIPTOR |
#endif
#if defined(_Py_TPFLAGS_HAVE_VECTORCALL) && CYTHON_METH_FASTCALL
    _Py_TPFLAGS_HAVE_VECTORCALL |
#endif
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_BASETYPE,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_PyMethod_New,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
    0,
#endif
#if __PYX_NEED_TP_PRINT_SLOT
    0,
#endif
#if PY_VERSION_HEX >= 0x030C0000
    0,
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
    0,
#endif
};
#endif
static int __pyx_CyFunction_init(PyObject *module) {
#if CYTHON_USE_TYPE_SPECS
    __pyx_CyFunctionType = __Pyx_FetchCommonTypeFromSpec(module, &__pyx_CyFunctionType_spec, NULL);
#else
    CYTHON_UNUSED_VAR(module);
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
#endif
    if (unlikely(__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_Malloc(size);
    if (unlikely(!m->defaults))
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    m->defaults_size = size;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* CythonFunction */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    PyObject *op = __Pyx_CyFunction_Init(
        PyObject_GC_New(__pyx_CyFunctionObject, __pyx_CyFunctionType),
        ml, flags, qualname, closure, module, globals, code
    );
    if (likely(op)) {
        PyObject_GC_Track(op);
    }
    return op;
}

/* PyObjectLookupSpecial */
#if CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx__PyObject_LookupSpecial(PyObject* obj, PyObject* attr_name, int with_error) {
    PyObject *res;
    PyTypeObject *tp = Py_TYPE(obj);
#if PY_MAJOR_VERSION < 3
    if (unlikely(PyInstance_Check(obj)))
        return with_error ? __Pyx_PyObject_GetAttrStr(obj, attr_name) : __Pyx_PyObject_GetAttrStrNoError(obj, attr_name);
#endif
    res = _PyType_Lookup(tp, attr_name);
    if (likely(res)) {
        descrgetfunc f = Py_TYPE(res)->tp_descr_get;
        if (!f) {
            Py_INCREF(res);
        } else {
            res = f(res, obj, (PyObject *)tp);
        }
    } else if (with_error) {
        PyErr_SetObject(PyExc_AttributeError, attr_name);
    }
    return res;
}
#endif

/* PyObjectCallNoArg */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
    PyObject *arg[2] = {NULL, NULL};
    return __Pyx_PyObject_FastCall(func, arg + 1, 0 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    CYTHON_MAYBE_UNUSED_VAR(tstate);
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStrNoError(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        (void) PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
#if !CYTHON_COMPILING_IN_LIMITED_API
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}
#endif

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
#if PY_VERSION_HEX >= 0x030b00a6 && !CYTHON_COMPILING_IN_LIMITED_API
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_PyCode_Replace_For_AddTraceback(PyObject *code, PyObject *scratch_dict,
                                                       PyObject *firstlineno, PyObject *name) {
    PyObject *replace = NULL;
    if (unlikely(PyDict_SetItemString(scratch_dict, "co_firstlineno", firstlineno))) return NULL;
    if (unlikely(PyDict_SetItemString(scratch_dict, "co_name", name))) return NULL;
    replace = PyObject_GetAttrString(code, "replace");
    if (likely(replace)) {
        PyObject *result;
        result = PyObject_Call(replace, __pyx_empty_tuple, scratch_dict);
        Py_DECREF(replace);
        return result;
    }
    PyErr_Clear();
    #if __PYX_LIMITED_VERSION_HEX < 0x030780000
    {
        PyObject *compiled = NULL, *result = NULL;
        if (unlikely(PyDict_SetItemString(scratch_dict, "code", code))) return NULL;
        if (unlikely(PyDict_SetItemString(scratch_dict, "type", (PyObject*)(&PyType_Type)))) return NULL;
        compiled = Py_CompileString(
            "out = type(code)(\n"
            "  code.co_argcount, code.co_kwonlyargcount, code.co_nlocals, code.co_stacksize,\n"
            "  code.co_flags, code.co_code, code.co_consts, code.co_names,\n"
            "  code.co_varnames, code.co_filename, co_name, co_firstlineno,\n"
            "  code.co_lnotab)\n", "<dummy>", Py_file_input);
        if (!compiled) return NULL;
        result = PyEval_EvalCode(compiled, scratch_dict, scratch_dict);
        Py_DECREF(compiled);
        if (!result) PyErr_Print();
        Py_DECREF(result);
        result = PyDict_GetItemString(scratch_dict, "out");
        if (result) Py_INCREF(result);
        return result;
    }
    #else
    return NULL;
    #endif
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyObject *code_object = NULL, *py_py_line = NULL, *py_funcname = NULL, *dict = NULL;
    PyObject *replace = NULL, *getframe = NULL, *frame = NULL;
    PyObject *exc_type, *exc_value, *exc_traceback;
    int success = 0;
    if (c_line) {
        (void) __pyx_cfilenm;
        (void) __Pyx_CLineForTraceback(__Pyx_PyThreadState_Current, c_line);
    }
    PyErr_Fetch(&exc_type, &exc_value, &exc_traceback);
    code_object = Py_CompileString("_getframe()", filename, Py_eval_input);
    if (unlikely(!code_object)) goto bad;
    py_py_line = PyLong_FromLong(py_line);
    if (unlikely(!py_py_line)) goto bad;
    py_funcname = PyUnicode_FromString(funcname);
    if (unlikely(!py_funcname)) goto bad;
    dict = PyDict_New();
    if (unlikely(!dict)) goto bad;
    {
        PyObject *old_code_object = code_object;
        code_object = __Pyx_PyCode_Replace_For_AddTraceback(code_object, dict, py_py_line, py_funcname);
        Py_DECREF(old_code_object);
    }
    if (unlikely(!code_object)) goto bad;
    getframe = PySys_GetObject("_getframe");
    if (unlikely(!getframe)) goto bad;
    if (unlikely(PyDict_SetItemString(dict, "_getframe", getframe))) goto bad;
    frame = PyEval_EvalCode(code_object, dict, dict);
    if (unlikely(!frame) || frame == Py_None) goto bad;
    success = 1;
  bad:
    PyErr_Restore(exc_type, exc_value, exc_traceback);
    Py_XDECREF(code_object);
    Py_XDECREF(py_py_line);
    Py_XDECREF(py_funcname);
    Py_XDECREF(dict);
    Py_XDECREF(replace);
    if (success) {
        PyTraceBack_Here(
            (struct _frame*)frame);
    }
    Py_XDECREF(frame);
}
#else
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_srcfile = NULL;
    py_srcfile = PyString_FromString(filename);
    if (!py_srcfile) goto bad;
    #endif
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        if (!py_funcname) goto bad;
        #endif
    }
    #if PY_MAJOR_VERSION < 3
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    #else
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    #endif
    Py_XDECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_srcfile);
    #endif
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject *ptype, *pvalue, *ptraceback;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) {
            /* If the code object creation fails, then we should clear the
               fetched exception references and propagate the new exception */
            Py_XDECREF(ptype);
            Py_XDECREF(pvalue);
            Py_XDECREF(ptraceback);
            goto bad;
        }
        __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}
#endif

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* Declarations */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return ::std::complex< float >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return x + y*(__pyx_t_float_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      __pyx_t_float_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabsf(b.real) >= fabsf(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                float r = b.imag / b.real;
                float s = (float)(1.0) / (b.real + b.imag * r);
                return __pyx_t_float_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            float r = b.real / b.imag;
            float s = (float)(1.0) / (b.imag + b.real * r);
            return __pyx_t_float_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            float denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_float_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrtf(z.real*z.real + z.imag*z.imag);
          #else
            return hypotf(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
            __pyx_t_float_complex z;
            float r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    float denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_float(a, a);
                    case 3:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, a);
                    case 4:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if ((b.imag == 0) && (a.real >= 0)) {
                    z.real = powf(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2f(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_float(a);
                theta = atan2f(a.imag, a.real);
            }
            lnr = logf(r);
            z_r = expf(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cosf(z_theta);
            z.imag = z_r * sinf(z_theta);
            return z;
        }
    #endif
#endif

/* Declarations */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return ::std::complex< double >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return x + y*(__pyx_t_double_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      __pyx_t_double_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabs(b.real) >= fabs(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                double r = b.imag / b.real;
                double s = (double)(1.0) / (b.real + b.imag * r);
                return __pyx_t_double_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            double r = b.real / b.imag;
            double s = (double)(1.0) / (b.imag + b.real * r);
            return __pyx_t_double_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            double denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_double_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrt(z.real*z.real + z.imag*z.imag);
          #else
            return hypot(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
            __pyx_t_double_complex z;
            double r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    double denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_double(a, a);
                    case 3:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, a);
                    case 4:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if ((b.imag == 0) && (a.real >= 0)) {
                    z.real = pow(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_double(a);
                theta = atan2(a.imag, a.real);
            }
            lnr = log(r);
            z_r = exp(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cos(z_theta);
            z.imag = z_r * sin(z_theta);
            return z;
        }
    #endif
#endif

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
#else
        PyObject *from_bytes, *result = NULL;
        PyObject *py_bytes = NULL, *arg_tuple = NULL, *kwds = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(long));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        arg_tuple = PyTuple_Pack(2, py_bytes, order_str);
        if (!arg_tuple) goto limited_bad;
        if (!is_unsigned) {
            kwds = PyDict_New();
            if (!kwds) goto limited_bad;
            if (PyDict_SetItemString(kwds, "signed", __Pyx_NewRef(Py_True))) goto limited_bad;
        }
        result = PyObject_Call(from_bytes, arg_tuple, kwds);
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(arg_tuple);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(long) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            if (unlikely(__Pyx_PyLong_IsNeg(x))) {
                goto raise_neg_overflow;
            } else if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_DigitCount(x)) {
                    case 2:
                        if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(long) >= 2 * PyLong_SHIFT)) {
                                return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(long) >= 3 * PyLong_SHIFT)) {
                                return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(long) >= 4 * PyLong_SHIFT)) {
                                return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                            }
                        }
                        break;
                }
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if ((sizeof(long) <= sizeof(unsigned long))) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(long) <= sizeof(unsigned PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_SignedDigitCount(x)) {
                    case -2:
                        if ((8 * sizeof(long) - 1 > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                                return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                            }
                        }
                        break;
                    case 2:
                        if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                                return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                            }
                        }
                        break;
                    case -3:
                        if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                                return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                                return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                            }
                        }
                        break;
                    case -4:
                        if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                                return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                                return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                            }
                        }
                        break;
                }
            }
#endif
            if ((sizeof(long) <= sizeof(long))) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(long) <= sizeof(PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
#if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
#endif
            if (likely(v)) {
                int ret = -1;
#if PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                ret = _PyLong_AsByteArray((PyLongObject *)v,
                                           bytes, sizeof(val),
                                           is_little, !is_unsigned);
#else
                PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
                int bits, remaining_bits, is_negative = 0;
                long idigit;
                int chunk_size = (sizeof(long) < 8) ? 30 : 62;
                if (unlikely(!PyLong_CheckExact(v))) {
                    PyObject *tmp = v;
                    v = PyNumber_Long(v);
                    assert(PyLong_CheckExact(v));
                    Py_DECREF(tmp);
                    if (unlikely(!v)) return (long) -1;
                }
#if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
                if (Py_SIZE(x) == 0)
                    return (long) 0;
                is_negative = Py_SIZE(x) < 0;
#else
                {
                    int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                    if (unlikely(result < 0))
                        return (long) -1;
                    is_negative = result == 1;
                }
#endif
                if (is_unsigned && unlikely(is_negative)) {
                    goto raise_neg_overflow;
                } else if (is_negative) {
                    stepval = PyNumber_Invert(v);
                    if (unlikely(!stepval))
                        return (long) -1;
                } else {
                    stepval = __Pyx_NewRef(v);
                }
                val = (long) 0;
                mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
                shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
                for (bits = 0; bits < (int) sizeof(long) * 8 - chunk_size; bits += chunk_size) {
                    PyObject *tmp, *digit;
                    digit = PyNumber_And(stepval, mask);
                    if (unlikely(!digit)) goto done;
                    idigit = PyLong_AsLong(digit);
                    Py_DECREF(digit);
                    if (unlikely(idigit < 0)) goto done;
                    tmp = PyNumber_Rshift(stepval, shift);
                    if (unlikely(!tmp)) goto done;
                    Py_DECREF(stepval); stepval = tmp;
                    val |= ((long) idigit) << bits;
                    #if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
                    if (Py_SIZE(stepval) == 0)
                        goto unpacking_done;
                    #endif
                }
                idigit = PyLong_AsLong(stepval);
                if (unlikely(idigit < 0)) goto done;
                remaining_bits = ((int) sizeof(long) * 8) - bits - (is_unsigned ? 0 : 1);
                if (unlikely(idigit >= (1L << remaining_bits)))
                    goto raise_overflow;
                val |= ((long) idigit) << bits;
            #if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
            unpacking_done:
            #endif
                if (!is_unsigned) {
                    if (unlikely(val & (((long) 1) << (sizeof(long) * 8 - 1))))
                        goto raise_overflow;
                    if (is_negative)
                        val = ~val;
                }
                ret = 0;
            done:
                Py_XDECREF(shift);
                Py_XDECREF(mask);
                Py_XDECREF(stepval);
#endif
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* FormatTypeName */
#if CYTHON_COMPILING_IN_LIMITED_API
static __Pyx_TypeName
__Pyx_PyType_GetName(PyTypeObject* tp)
{
    PyObject *name = __Pyx_PyObject_GetAttrStr((PyObject *)tp,
                                               __pyx_n_s_name);
    if (unlikely(name == NULL) || unlikely(!PyUnicode_Check(name))) {
        PyErr_Clear();
        Py_XDECREF(name);
        name = __Pyx_NewRef(__pyx_n_s__10);
    }
    return name;
}
#endif

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(int) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            if (unlikely(__Pyx_PyLong_IsNeg(x))) {
                goto raise_neg_overflow;
            } else if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_DigitCount(x)) {
                    case 2:
                        if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(int) >= 2 * PyLong_SHIFT)) {
                                return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(int) >= 3 * PyLong_SHIFT)) {
                                return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(int) >= 4 * PyLong_SHIFT)) {
                                return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                            }
                        }
                        break;
                }
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if ((sizeof(int) <= sizeof(unsigned long))) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(int) <= sizeof(unsigned PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            if (__Pyx_PyLong_IsCompact(x)) {
                __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
            } else {
                const digit* digits = __Pyx_PyLong_Digits(x);
                assert(__Pyx_PyLong_DigitCount(x) > 1);
                switch (__Pyx_PyLong_SignedDigitCount(x)) {
                    case -2:
                        if ((8 * sizeof(int) - 1 > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                                return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                            }
                        }
                        break;
                    case 2:
                        if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                                return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                            }
                        }
                        break;
                    case -3:
                        if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                                return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                            }
                        }
                        break;
                    case 3:
                        if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                                return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                            }
                        }
                        break;
                    case -4:
                        if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                                return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                            }
                        }
                        break;
                    case 4:
                        if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                            if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                                __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                            } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                                return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                            }
                        }
                        break;
                }
            }
#endif
            if ((sizeof(int) <= sizeof(long))) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if ((sizeof(int) <= sizeof(PY_LONG_LONG))) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
#if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
#endif
            if (likely(v)) {
                int ret = -1;
#if PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                ret = _PyLong_AsByteArray((PyLongObject *)v,
                                           bytes, sizeof(val),
                                           is_little, !is_unsigned);
#else
                PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
                int bits, remaining_bits, is_negative = 0;
                long idigit;
                int chunk_size = (sizeof(long) < 8) ? 30 : 62;
                if (unlikely(!PyLong_CheckExact(v))) {
                    PyObject *tmp = v;
                    v = PyNumber_Long(v);
                    assert(PyLong_CheckExact(v));
                    Py_DECREF(tmp);
                    if (unlikely(!v)) return (int) -1;
                }
#if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
                if (Py_SIZE(x) == 0)
                    return (int) 0;
                is_negative = Py_SIZE(x) < 0;
#else
                {
                    int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                    if (unlikely(result < 0))
                        return (int) -1;
                    is_negative = result == 1;
                }
#endif
                if (is_unsigned && unlikely(is_negative)) {
                    goto raise_neg_overflow;
                } else if (is_negative) {
                    stepval = PyNumber_Invert(v);
                    if (unlikely(!stepval))
                        return (int) -1;
                } else {
                    stepval = __Pyx_NewRef(v);
                }
                val = (int) 0;
                mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
                shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
                for (bits = 0; bits < (int) sizeof(int) * 8 - chunk_size; bits += chunk_size) {
                    PyObject *tmp, *digit;
                    digit = PyNumber_And(stepval, mask);
                    if (unlikely(!digit)) goto done;
                    idigit = PyLong_AsLong(digit);
                    Py_DECREF(digit);
                    if (unlikely(idigit < 0)) goto done;
                    tmp = PyNumber_Rshift(stepval, shift);
                    if (unlikely(!tmp)) goto done;
                    Py_DECREF(stepval); stepval = tmp;
                    val |= ((int) idigit) << bits;
                    #if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
                    if (Py_SIZE(stepval) == 0)
                        goto unpacking_done;
                    #endif
                }
                idigit = PyLong_AsLong(stepval);
                if (unlikely(idigit < 0)) goto done;
                remaining_bits = ((int) sizeof(int) * 8) - bits - (is_unsigned ? 0 : 1);
                if (unlikely(idigit >= (1L << remaining_bits)))
                    goto raise_overflow;
                val |= ((int) idigit) << bits;
            #if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030B0000
            unpacking_done:
            #endif
                if (!is_unsigned) {
                    if (unlikely(val & (((int) 1) << (sizeof(int) * 8 - 1))))
                        goto raise_overflow;
                    if (is_negative)
                        val = ~val;
                }
                ret = 0;
            done:
                Py_XDECREF(shift);
                Py_XDECREF(mask);
                Py_XDECREF(stepval);
#endif
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = __Pyx_PyType_GetSlot(a, tp_base, PyTypeObject*);
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (cls == a || cls == b) return 1;
    mro = cls->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            PyObject *base = PyTuple_GET_ITEM(mro, i);
            if (base == (PyObject *)a || base == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(cls, a) || __Pyx_InBases(cls, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    if (exc_type1) {
        return __Pyx_IsAnySubtype2((PyTypeObject*)err, (PyTypeObject*)exc_type1, (PyTypeObject*)exc_type2);
    } else {
        return __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
static unsigned long __Pyx_get_runtime_version(void) {
#if __PYX_LIMITED_VERSION_HEX >= 0x030B00A4
    return Py_Version & ~0xFFUL;
#else
    const char* rt_version = Py_GetVersion();
    unsigned long version = 0;
    unsigned long factor = 0x01000000UL;
    unsigned int digit = 0;
    int i = 0;
    while (factor) {
        while ('0' <= rt_version[i] && rt_version[i] <= '9') {
            digit = digit * 10 + (unsigned int) (rt_version[i] - '0');
            ++i;
        }
        version += factor * digit;
        if (rt_version[i] != '.')
            break;
        digit = 0;
        factor >>= 8;
        ++i;
    }
    return version;
#endif
}
static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer) {
    const unsigned long MAJOR_MINOR = 0xFFFF0000UL;
    if ((rt_version & MAJOR_MINOR) == (ct_version & MAJOR_MINOR))
        return 0;
    if (likely(allow_newer && (rt_version & MAJOR_MINOR) > (ct_version & MAJOR_MINOR)))
        return 1;
    {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compile time Python version %d.%d "
                      "of module '%.100s' "
                      "%s "
                      "runtime version %d.%d",
                       (int) (ct_version >> 24), (int) ((ct_version >> 16) & 0xFF),
                       __Pyx_MODULE_NAME,
                       (allow_newer) ? "was newer than" : "does not match",
                       (int) (rt_version >> 24), (int) ((rt_version >> 16) & 0xFF)
       );
        return PyErr_WarnEx(NULL, message, 1);
    }
}

/* FunctionImport */
#ifndef __PYX_HAVE_RT_ImportFunction_3_0_8
#define __PYX_HAVE_RT_ImportFunction_3_0_8
static int __Pyx_ImportFunction_3_0_8(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, funcname);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C function %.200s",
                PyModule_GetName(module), funcname);
        goto bad;
    }
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    tmp.p = PyCapsule_GetPointer(cobj, sig);
    *f = tmp.fp;
    if (!(*f))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* InitStrings */
#if PY_MAJOR_VERSION >= 3
static int __Pyx_InitString(__Pyx_StringTabEntry t, PyObject **str) {
    if (t.is_unicode | t.is_str) {
        if (t.intern) {
            *str = PyUnicode_InternFromString(t.s);
        } else if (t.encoding) {
            *str = PyUnicode_Decode(t.s, t.n - 1, t.encoding, NULL);
        } else {
            *str = PyUnicode_FromStringAndSize(t.s, t.n - 1);
        }
    } else {
        *str = PyBytes_FromStringAndSize(t.s, t.n - 1);
    }
    if (!*str)
        return -1;
    if (PyObject_Hash(*str) == -1)
        return -1;
    return 0;
}
#endif
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION >= 3
        __Pyx_InitString(*t, t->p);
        #else
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        #endif
        ++t;
    }
    return 0;
}

#include <string.h>
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s) {
    size_t len = strlen(s);
    if (unlikely(len > (size_t) PY_SSIZE_T_MAX)) {
        PyErr_SetString(PyExc_OverflowError, "byte string is too long");
        return -1;
    }
    return (Py_ssize_t) len;
}
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return __Pyx_PyUnicode_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return PyByteArray_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY && !CYTHON_COMPILING_IN_LIMITED_API) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
    __Pyx_TypeName result_type_name = __Pyx_PyType_GetName(Py_TYPE(result));
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type " __Pyx_FMT_TYPENAME ").  "
                "The ability to return an instance of a strict subclass of int is deprecated, "
                "and may be removed in a future version of Python.",
                result_type_name)) {
            __Pyx_DECREF_TypeName(result_type_name);
            Py_DECREF(result);
            return NULL;
        }
        __Pyx_DECREF_TypeName(result_type_name);
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type " __Pyx_FMT_TYPENAME ")",
                 type_name, type_name, result_type_name);
    __Pyx_DECREF_TypeName(result_type_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(__Pyx_PyLong_IsCompact(b))) {
        return __Pyx_PyLong_CompactValue(b);
    } else {
      const digit* digits = __Pyx_PyLong_Digits(b);
      const Py_ssize_t size = __Pyx_PyLong_SignedDigitCount(b);
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
#if PY_MAJOR_VERSION < 3
  } else if (likely(PyInt_CheckExact(o))) {
    return PyInt_AS_LONG(o);
#endif
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyInt_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


/* #### Code section: utility_code_pragmas_end ### */
#ifdef _MSC_VER
#pragma warning( pop )
#endif



/* #### Code section: end ### */
#endif /* Py_PYTHON_H */
